from setuptools import setup
from setuptools.command.install import install
import os, shutil, platform, sys

# package name
package_name_0 = os.path.join("package_name.txt")
with open(package_name_0, "r", encoding="utf-8") as fileObj:
    package = fileObj.read()
package_name_1 = os.path.join(package, "package_name.txt") # package readme
shutil.copy(package_name_0, package_name_1)

# delete old shortcut files
apps = {
    "uniquebible": ("UniqueBible", "UniqueBible App"),
}
appName, appFullName = apps[package]
shortcutFiles = (f"{appName}.bat", f"{appName}.command", f"{appName}.desktop")
for shortcutFile in shortcutFiles:
    shortcut = os.path.join(package, shortcutFile)
    if os.path.isfile(shortcut):
        os.remove(shortcut)

# update package readme
latest_readme = "README.md" # github repository readme
package_readme = os.path.join(package, "README.md") # package readme
shutil.copy(latest_readme, package_readme)
with open(package_readme, "r", encoding="utf-8") as fileObj:
    long_description = fileObj.read()

# get required packages
install_requires = []
with open(os.path.join(package, "requirements.txt"), "r") as fileObj:
    for line in fileObj.readlines():
        mod = line.strip()
        if mod:
            install_requires.append(mod)

# make sure config.py is empty
open(os.path.join(package, "config.py"), "w").close()

# https://packaging.python.org/en/latest/guides/distributing-packages-using-setuptools/
setup(
    name=package,
    version="0.1.13",
    python_requires=">=3.8, <3.13",
    description=f"UniqueBible App is a cross-platform & offline bible application, integrated with high-quality resources and unique features. Developers: Eliran Wong and Oliver Tseng",
    long_description=long_description,
    author="Eliran Wong",
    author_email="support@marvel.bible",
    packages=[
        package,
        f"{package}.video",
        #f"{package}.nltk_data", # size too large
        #f"{package}.nltk_data.corpora", # size too large
        f"{package}.gui",
        f"{package}.db",
        f"{package}.terminal_history",
        f"{package}.terminal_mode",
        f"{package}.terminal_mode.developers",
        f"{package}.terminal_mode.how_to",
        #f"{package}.screenshots",
        f"{package}.install",
        #f"{package}.testing",
        #f"{package}.tests",
        f"{package}.webstorage",
        f"{package}.webstorage.Cache",
        f"{package}.webstorage.Downloads",
        f"{package}.webstorage.PersistentStorage",
        f"{package}.htmlResources",
        f"{package}.htmlResources.material",
        f"{package}.htmlResources.js",
        f"{package}.htmlResources.icons",
        f"{package}.htmlResources.css",
        f"{package}.htmlResources.lib",
        f"{package}.htmlResources.buttons",
        f"{package}.htmlResources.fonts",
        f"{package}.htmlResources.images",
        #f"{package}.htmlResources.images.exlbl", # size too large
        #f"{package}.htmlResources.images.exlbl_largeHD", # size too large
        #f"{package}.htmlResources.images.exlbl_large", # size too large
        f"{package}.lang",
        f"{package}.workspace",
        #f"{package}.installation",
        f"{package}.selectedVerses",
        #f"{package}.llama_index", # size too large
        #f"{package}.llama_index.KJV_md_index", # size too large
        #f"{package}.llama_index.NET_md_index", # size too large
        f"{package}.notes",
        f"{package}.marvelData",
        f"{package}.marvelData.docx",
        f"{package}.marvelData.statistics",
        f"{package}.marvelData.pdf",
        f"{package}.marvelData.books",
        f"{package}.marvelData.indexes",
        f"{package}.marvelData.indexes.bible",
        f"{package}.marvelData.bibles",
        f"{package}.marvelData.lexicons",
        f"{package}.marvelData.chats",
        f"{package}.marvelData.xref",
        f"{package}.marvelData.data",
        f"{package}.marvelData.epub",
        f"{package}.marvelData.commentaries",
        f"{package}.marvelData.devotionals",
        f"{package}.util",
        f"{package}.macros",
        f"{package}.music",
        f"{package}.audio",
        f"{package}.temp",
        f"{package}.startup",
        f"{package}.thirdParty",
        f"{package}.thirdParty.dictionaries",
        f"{package}.import",
        f"{package}.tools",
        f"{package}.plugins",
        f"{package}.plugins.layout",
        f"{package}.plugins.menu",
        f"{package}.plugins.menu.ePubViewer",
        f"{package}.plugins.menu.ePubViewer.polyfills",
        f"{package}.plugins.menu.ePubViewer.libs",
        f"{package}.plugins.menu.ToDo",
        f"{package}.plugins.menu.Bible_Data",
        f"{package}.plugins.menu.GoogleDriveUtility",
        f"{package}.plugins.text_editor",
        f"{package}.plugins.terminal",
        f"{package}.plugins.language",
        f"{package}.plugins.shutdown",
        f"{package}.plugins.event",
        f"{package}.plugins.config",
        f"{package}.plugins.chatGPT",
        f"{package}.plugins.context",
        f"{package}.plugins.context.Strongs2csv",
        f"{package}.plugins.startup",
        f"{package}.htmlResources.material.social.travel_explore.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.file.folder_open.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.video_camera_front.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.checklist_rtl.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.file.folder.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.play_circle_outline.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.maps.local_parking.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.auto_fix_high.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.queue_music.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.maps.local_printshop.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.fact_check.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_clear.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.file.grid_view.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.communication.swap_calls.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.speed.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.maps.local_library.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.volume_up.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_size.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.communication.comment.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.view_day.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.navigation.unfold_more.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.ads_click.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.drag_indicator.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.stop_circle.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.superscript.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.notification.more.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.description.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.flash_off.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.expand.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.playlist_add_circle.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.schema.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.align_horizontal_right.materialicons.48dp.2x",
        f"{package}.htmlResources.material.hardware.smart_toy.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.toggle.toggle_off.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.places.all_inclusive.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.edit_note.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.toggle.toggle_on.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.content.bolt.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.device.wallpaper.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_align_center.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.assignment.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.open_in_full.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.content.stacked_bar_chart.materialiconsround.18dp.2x",
        f"{package}.htmlResources.material.editor.format_color_fill.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.skip_next.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.question_answer.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.social.male.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.maps.layers.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.notification.sync_disabled.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.translate.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.zoom_in.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.home.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.tune.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.horizontal_split.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.hardware.browser_updated.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.social.groups.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.account_balance.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.find_in_page.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.communication.list_alt.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.device.widgets.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.communication.stay_current_portrait.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.volume_off.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.device.sim_card_download.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.playlist_add_check.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.social.share.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.help_outline.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.insert_photo.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_color_text.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.work.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_underlined.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.content.add_circle_outline.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.insert_link.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.file.drive_file_move.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.skip_previous.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.content.inventory_2.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.device.reviews.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.content.remove_circle_outline.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.notification.sync.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_italic.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.file.workspaces.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.view_column.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_align_left.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.add_photo_alternate.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.record_voice_over.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.maps.pin_drop.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.open_in_new.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.done_all.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.play_circle.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.content.save_as.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.picture_as_pdf.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.remove_done.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.pause_circle.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.file.file_open.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.flare.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.maps.rate_review.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.social.female.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.playlist_add.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.device.note_alt.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.auto_awesome_motion.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.auto_fix_off.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_align_right.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.view_timeline.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.communication.stay_current_landscape.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.interpreter_mode.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.file.drive_file_rename_outline.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.post_add.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.communication.import_export.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.space_dashboard.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.switch_access_shortcut_add.materialiconssharp.48dp.2x",
        f"{package}.htmlResources.material.action.preview.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.auto_stories.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.content.save.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.looks_3.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.auto_fix_normal.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.edit.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.navigation.more_vert.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.text_increase.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.hardware.keyboard_return.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.maps.layers_clear.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.token.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.subscript.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.flash_auto.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.highlight_alt.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.title.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.history.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.compare.materialicons.48dp.2x",
        f"{package}.htmlResources.material.action.abc.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.contrast.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.navigate_before.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.hardware.gamepad.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.table_rows.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.note_add.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.text_decrease.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.navigation.arrow_back_ios.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_list_numbered.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.settings.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.communication.cancel_presentation.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.communication.email.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.switch_access_shortcut.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.content.add_link.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.auto_awesome.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.thumb_up_off_alt.materialicons.48dp.2x",
        f"{package}.htmlResources.material.action.hide_source.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.navigation.arrow_forward_ios.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.fit_screen.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.device.mobiledata_off.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.file_present.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.filter_alt.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_bold.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.subtitles.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.content.dynamic_feed.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.subtitles_off.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.compare_arrows.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.language.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.format_align_justify.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.navigate_next.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.book.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.calendar_month.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.aspect_ratio.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.navigation.refresh.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.terminal.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.maps.diamond.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.search.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.av.video_settings.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.action.addchart.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.image.wb_sunny.materialiconsoutlined.48dp.2x",
        f"{package}.htmlResources.material.editor.highlight.materialiconsoutlined.48dp.2x",
    ],
    package_data={
        package: ["*.*"],
        f"{package}.video": ["*.*"],
        #f"{package}.nltk_data": ["*.*"], # size too large
        #f"{package}.nltk_data.corpora": ["*.*"], # size too large
        f"{package}.gui": ["*.*"],
        f"{package}.db": ["*.*"],
        f"{package}.terminal_history": ["*.*"],
        f"{package}.terminal_mode": ["*.*"],
        f"{package}.terminal_mode.developers": ["*.*"],
        f"{package}.terminal_mode.how_to": ["*.*"],
        #f"{package}.screenshots": ["*.*"],
        f"{package}.install": ["*.*"],
        #f"{package}.testing": ["*.*"],
        #f"{package}.tests": ["*.*"],
        f"{package}.webstorage": ["*.*"],
        f"{package}.webstorage.Cache": ["*.*"],
        f"{package}.webstorage.Downloads": ["*.*"],
        f"{package}.webstorage.PersistentStorage": ["*.*"],
        f"{package}.htmlResources": ["*.*"],
        f"{package}.htmlResources.material": ["*.*"],
        f"{package}.htmlResources.js": ["*.*"],
        f"{package}.htmlResources.icons": ["*.*"],
        f"{package}.htmlResources.css": ["*.*"],
        f"{package}.htmlResources.lib": ["*.*"],
        f"{package}.htmlResources.buttons": ["*.*"],
        f"{package}.htmlResources.fonts": ["*.*"],
        f"{package}.htmlResources.images": ["*.*"],
        #f"{package}.htmlResources.images.exlbl": ["*.*"], # size too large
        #f"{package}.htmlResources.images.exlbl_largeHD": ["*.*"], # size too large
        #f"{package}.htmlResources.images.exlbl_large": ["*.*"], # size too large
        f"{package}.lang": ["*.*"],
        f"{package}.workspace": ["*.*"],
        #f"{package}.installation": ["*.*"],
        f"{package}.selectedVerses": ["*.*"],
        #f"{package}.llama_index": ["*.*"], # size too large
        #f"{package}.llama_index.KJV_md_index": ["*.*"], # size too large
        #f"{package}.llama_index.NET_md_index": ["*.*"], # size too large
        f"{package}.notes": ["*.*"],
        f"{package}.marvelData": ["*.*"],
        f"{package}.marvelData.docx": ["*.*"],
        f"{package}.marvelData.statistics": ["*.*"],
        f"{package}.marvelData.pdf": ["*.*"],
        f"{package}.marvelData.books": ["*.*"],
        f"{package}.marvelData.indexes": ["*.*"],
        f"{package}.marvelData.indexes.bible": ["*.*"],
        f"{package}.marvelData.bibles": ["*.*"],
        f"{package}.marvelData.lexicons": ["*.*"],
        f"{package}.marvelData.chats": ["*.*"],
        f"{package}.marvelData.xref": ["*.*"],
        f"{package}.marvelData.data": ["*.*"],
        f"{package}.marvelData.epub": ["*.*"],
        f"{package}.marvelData.commentaries": ["*.*"],
        f"{package}.marvelData.devotionals": ["*.*"],
        f"{package}.util": ["*.*"],
        f"{package}.macros": ["*.*"],
        f"{package}.music": ["*.*"],
        f"{package}.audio": ["*.*"],
        f"{package}.temp": ["*.*"],
        f"{package}.startup": ["*.*"],
        f"{package}.thirdParty": ["*.*"],
        f"{package}.thirdParty.dictionaries": ["*.*"],
        f"{package}.import": ["*.*"],
        f"{package}.tools": ["*.*"],
        f"{package}.plugins": ["*.*"],
        f"{package}.plugins.layout": ["*.*"],
        f"{package}.plugins.menu": ["*.*"],
        f"{package}.plugins.menu.ePubViewer": ["*.*"],
        f"{package}.plugins.menu.ePubViewer.polyfills": ["*.*"],
        f"{package}.plugins.menu.ePubViewer.libs": ["*.*"],
        f"{package}.plugins.menu.ToDo": ["*.*"],
        f"{package}.plugins.menu.Bible_Data": ["*.*"],
        f"{package}.plugins.menu.GoogleDriveUtility": ["*.*"],
        f"{package}.plugins.text_editor": ["*.*"],
        f"{package}.plugins.terminal": ["*.*"],
        f"{package}.plugins.language": ["*.*"],
        f"{package}.plugins.shutdown": ["*.*"],
        f"{package}.plugins.event": ["*.*"],
        f"{package}.plugins.config": ["*.*"],
        f"{package}.plugins.chatGPT": ["*.*"],
        f"{package}.plugins.context": ["*.*"],
        f"{package}.plugins.context.Strongs2csv": ["*.*"],
        f"{package}.plugins.startup": ["*.*"],
        f"{package}.htmlResources.material.social.travel_explore.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.file.folder_open.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.video_camera_front.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.checklist_rtl.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.file.folder.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.play_circle_outline.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.maps.local_parking.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.auto_fix_high.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.queue_music.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.maps.local_printshop.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.fact_check.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_clear.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.file.grid_view.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.communication.swap_calls.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.speed.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.maps.local_library.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.volume_up.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_size.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.communication.comment.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.view_day.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.navigation.unfold_more.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.ads_click.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.drag_indicator.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.stop_circle.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.superscript.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.notification.more.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.description.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.flash_off.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.expand.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.playlist_add_circle.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.schema.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.align_horizontal_right.materialicons.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.hardware.smart_toy.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.toggle.toggle_off.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.places.all_inclusive.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.edit_note.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.toggle.toggle_on.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.content.bolt.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.device.wallpaper.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_align_center.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.assignment.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.open_in_full.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.content.stacked_bar_chart.materialiconsround.18dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_color_fill.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.skip_next.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.question_answer.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.social.male.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.maps.layers.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.notification.sync_disabled.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.translate.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.zoom_in.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.home.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.tune.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.horizontal_split.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.hardware.browser_updated.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.social.groups.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.account_balance.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.find_in_page.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.communication.list_alt.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.device.widgets.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.communication.stay_current_portrait.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.volume_off.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.device.sim_card_download.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.playlist_add_check.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.social.share.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.help_outline.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.insert_photo.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_color_text.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.work.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_underlined.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.content.add_circle_outline.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.insert_link.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.file.drive_file_move.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.skip_previous.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.content.inventory_2.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.device.reviews.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.content.remove_circle_outline.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.notification.sync.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_italic.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.file.workspaces.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.view_column.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_align_left.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.add_photo_alternate.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.record_voice_over.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.maps.pin_drop.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.open_in_new.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.done_all.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.play_circle.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.content.save_as.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.picture_as_pdf.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.remove_done.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.pause_circle.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.file.file_open.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.flare.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.maps.rate_review.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.social.female.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.playlist_add.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.device.note_alt.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.auto_awesome_motion.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.auto_fix_off.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_align_right.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.view_timeline.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.communication.stay_current_landscape.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.interpreter_mode.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.file.drive_file_rename_outline.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.post_add.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.communication.import_export.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.space_dashboard.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.switch_access_shortcut_add.materialiconssharp.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.preview.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.auto_stories.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.content.save.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.looks_3.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.auto_fix_normal.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.edit.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.navigation.more_vert.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.text_increase.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.hardware.keyboard_return.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.maps.layers_clear.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.token.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.subscript.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.flash_auto.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.highlight_alt.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.title.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.history.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.compare.materialicons.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.abc.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.contrast.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.navigate_before.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.hardware.gamepad.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.table_rows.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.note_add.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.text_decrease.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.navigation.arrow_back_ios.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_list_numbered.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.settings.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.communication.cancel_presentation.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.communication.email.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.switch_access_shortcut.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.content.add_link.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.auto_awesome.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.thumb_up_off_alt.materialicons.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.hide_source.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.navigation.arrow_forward_ios.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.fit_screen.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.device.mobiledata_off.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.file_present.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.filter_alt.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_bold.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.subtitles.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.content.dynamic_feed.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.subtitles_off.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.compare_arrows.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.language.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.format_align_justify.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.navigate_next.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.book.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.calendar_month.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.aspect_ratio.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.navigation.refresh.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.terminal.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.maps.diamond.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.search.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.av.video_settings.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.action.addchart.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.image.wb_sunny.materialiconsoutlined.48dp.2x": ["*.*"],
        f"{package}.htmlResources.material.editor.highlight.materialiconsoutlined.48dp.2x": ["*.*"],

    },
    license="GNU General Public License (GPL)",
    install_requires=install_requires,
    extras_require={
        'gui': ["PySide6"],  # Dependencies for the gui module
    },
    entry_points={
        "console_scripts": [
            f"{package}={package}.uba:main",
        ],
    },
    keywords="bible scripture na28 bsha hebrew greek ai marvelbible biblebento uba uniquebible",
    url="https://www.uniquebible.app/",
    project_urls={
        "Source": "https://github.com/eliranwong/UniqueBible",
        "Tracker": "https://github.com/eliranwong/UniqueBible/issues",
        "Documentation": "https://github.com/eliranwong/UniqueBible/wiki",
        "Funding": "https://www.paypal.me/MarvelBible",
    },
    classifiers=[
        # Reference: https://pypi.org/classifiers/

        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: End Users/Desktop',
        'Topic :: Utilities',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
)
