import os
import re

class LexicalData:

    data = {
        'E70001': ('בְּ', 'prep', 'in', "b'"),
        'E70002': ('רֵאשִׁית', 'subs', 'beginning', 'reishit'),
        'E70003': ('ברא', 'verb', 'create', 'vr'),
        'E70004': ('אֱלֹהִים', 'subs', 'god(s)', 'elohim'),
        'E70005': ('אֵת', 'prep', '[object marker]', 'eit'),
        'E70006': ('הַ', 'art', 'the', 'ah'),
        'E70007': ('שָׁמַיִם', 'subs', 'heavens', 'shamayim'),
        'E70008': ('וְ', 'conj', 'and', "v'"),
        'E70009': ('אֶרֶץ', 'subs', 'earth', 'erets'),
        'E70010': ('היה', 'verb', 'be', 'hyh'),
        'E70011': ('תֹּהוּ', 'subs', 'emptiness', 'tohu'),
        'E70012': ('בֹּהוּ', 'subs', 'emptiness', 'bohu'),
        'E70013': ('חֹשֶׁךְ', 'subs', 'darkness', 'choshech'),
        'E70014': ('עַל', 'prep', 'upon', 'al'),
        'E70015': ('פָּנֶה', 'subs', 'face', 'paneh'),
        'E70016': ('תְּהֹום', 'subs', 'primeval ocean', "t'hovm"),
        'E70017': ('רוּחַ', 'subs', 'wind', 'ruach'),
        'E70018': ('רחף', 'verb', 'shake', 'rchf'),
        'E70019': ('מַיִם', 'subs', 'water', 'mayim'),
        'E70020': ('אמר', 'verb', 'say', 'mr'),
        'E70021': ('אֹור', 'subs', 'light', 'ovr'),
        'E70022': ('ראה', 'verb', 'see', 'rh'),
        'E70023': ('כִּי', 'conj', 'that', 'ki'),
        'E70024': ('טוב', 'verb', 'be good', 'tvv'),
        'E70025': ('בדל', 'verb', 'separate', 'vdl'),
        'E70026': ('בַּיִן', 'subs', 'interval', 'bayin'),
        'E70027': ('קרא', 'verb', 'call', 'kr'),
        'E70028': ('לְ', 'prep', 'to', "l'"),
        'E70029': ('יֹום', 'subs', 'day', 'yovm'),
        'E70030': ('לַיְלָה', 'subs', 'night', 'layla'),
        'E70031': ('עֶרֶב', 'subs', 'evening', 'erev'),
        'E70032': ('בֹּקֶר', 'subs', 'morning', 'boker'),
        'E70033': ('אֶחָד', 'subs', 'one', 'echad'),
        'E70034': ('רָקִיעַ', 'subs', 'firmament', 'rakia'),
        'E70035': ('תָּוֶךְ', 'subs', 'midst', 'tavech'),
        'E70036': ('עשׂה', 'verb', 'make', "s'h"),
        'E70037': ('אֲשֶׁר', 'conj', '[relative]', 'asher'),
        'E70038': ('מִן', 'prep', 'from', 'min'),
        'E70039': ('תַּחַת', 'subs', 'under part', 'tachat'),
        'E70040': ('כֵּן', 'advb', 'thus', 'kein'),
        'E70041': ('שֵׁנִי', 'adjv', 'second', 'sheini'),
        'E70042': ('קוה', 'verb', 'collect', 'kvh'),
        'E70043': ('אֶל', 'prep', 'to', 'el'),
        'E70044': ('מָקֹום', 'subs', 'place', 'makovm'),
        'E70045': ('יַבָּשָׁה', 'subs', 'dry land', 'yabasha'),
        'E70046': ('מִקְוֶה', 'subs', 'collection', 'mikveh'),
        'E70047': ('יָם', 'subs', 'sea', 'yam'),
        'E70048': ('דשׁא', 'verb', 'grow green', 'dsh'),
        'E70049': ('דֶּשֶׁא', 'subs', 'young grass', 'deshe'),
        'E70050': ('עֵשֶׂב', 'subs', 'herb', 'eisev'),
        'E70051': ('זרע', 'verb', 'sow', 'zr'),
        'E70052': ('זֶרַע', 'subs', 'seed', 'zera'),
        'E70053': ('עֵץ', 'subs', 'tree', 'eits'),
        'E70054': ('פְּרִי', 'subs', 'fruit', "p'ri"),
        'E70055': ('מִין', 'subs', 'kind', 'min'),
        'E70056': ('יצא', 'verb', 'go out', 'yts'),
        'E70057': ('שְׁלִישִׁי', 'adjv', 'third', "sh'lishi"),
        'E70058': ('מָאֹור', 'subs', 'lamp', 'maovr'),
        'E70059': ('אֹות', 'subs', 'sign', 'ovt'),
        'E70060': ('מֹועֵד', 'subs', 'appointment', 'moveid'),
        'E70061': ('שָׁנָה', 'subs', 'year', 'shana'),
        'E70062': ('אור', 'verb', 'be light', 'vr'),
        'E70063': ('שְׁנַיִם', 'subs', 'two', "sh'nayim"),
        'E70064': ('גָּדֹול', 'adjv', 'great', 'gadovl'),
        'E70065': ('מֶמְשֶׁלֶת', 'subs', 'dominion', 'memshelet'),
        'E70066': ('קָטֹן', 'adjv', 'small', 'katon'),
        'E70067': ('כֹּוכָב', 'subs', 'star', 'kovchov'),
        'E70068': ('נתן', 'verb', 'give', 'ntn'),
        'E70069': ('משׁל', 'verb', 'rule', 'mshl'),
        'E70070': ('רְבִיעִי', 'adjv', 'fourth', "r'vi'i"),
        'E70071': ('שׁרץ', 'verb', 'swarm', 'shrts'),
        'E70072': ('שֶׁרֶץ', 'subs', 'swarming creatures', 'sherets'),
        'E70073': ('נֶפֶשׁ', 'subs', 'soul', 'nefesh'),
        'E70074': ('חַי', 'adjv', 'alive', 'chay'),
        'E70075': ('עֹוף', 'subs', 'birds', 'ovf'),
        'E70076': ('עוף', 'verb', 'fly', 'vf'),
        'E70077': ('תַּנִּין', 'subs', 'sea-monster', 'tanin'),
        'E70078': ('כֹּל', 'subs', 'whole', 'kol'),
        'E70079': ('רמשׂ', 'verb', 'creep', 'rms'),
        'E70080': ('כָּנָף', 'subs', 'wing', 'kanaf'),
        'E70081': ('ברך', 'verb', 'bless', 'vrch'),
        'E70082': ('פרה', 'verb', 'be fertile', 'frh'),
        'E70083': ('רבה', 'verb', 'be many', 'rvh'),
        'E70084': ('מלא', 'verb', 'be full', 'ml'),
        'E70085': ('חֲמִישִׁי', 'adjv', 'fifth', 'chamishi'),
        'E70086': ('בְּהֵמָה', 'subs', 'cattle', "b'heima"),
        'E70087': ('רֶמֶשׂ', 'subs', 'creeping animals', 'remes'),
        'E70088': ('חַיָּה', 'subs', 'wild animal', 'chaya'),
        'E70089': ('אֲדָמָה', 'subs', 'soil', 'adama'),
        'E70090': ('אָדָם', 'subs', 'human, mankind', 'adam'),
        'E70091': ('צֶלֶם', 'subs', 'image', 'tselem'),
        'E70092': ('כְּ', 'prep', 'as', "k'"),
        'E70093': ('דְּמוּת', 'subs', 'likeness', "d'mut"),
        'E70094': ('רדה', 'verb', 'tread, to rule', 'rdh'),
        'E70095': ('דָּגָה', 'subs', 'fish', 'daga'),
        'E70096': ('זָכָר', 'adjv', 'male', 'zachor'),
        'E70097': ('נְקֵבָה', 'adjv', 'female', "n'keiva"),
        'E70098': ('כבשׁ', 'verb', 'subdue', 'chvsh'),
        'E70099': ('הִנֵּה', 'intj', 'behold', 'hinei'),
        'E70100': ('אָכְלָה', 'subs', 'food', 'achla'),
        'E70101': ('יֶרֶק', 'subs', 'greens', 'yerek'),
        'E70102': ('טֹוב', 'adjv', 'good', 'tovv'),
        'E70103': ('מְאֹד', 'subs', 'might', "m'od"),
        'E70104': ('שִׁשִּׁי', 'adjv', 'sixth', 'shishi'),
        'E70105': ('כלה', 'verb', 'be complete', 'chlh'),
        'E70106': ('צָבָא', 'subs', 'service', 'tsava'),
        'E70107': ('שְׁבִיעִי', 'adjv', 'seventh', "sh'vi'i"),
        'E70108': ('מְלֶאכֶת', 'subs', 'work', "m'lechet"),
        'E70109': ('שׁבת', 'verb', 'cease', 'shvt'),
        'E70110': ('קדשׁ', 'verb', 'be holy', 'kdsh'),
        'E70111': ('אֵלֶּה', 'prde', 'these', 'eileh'),
        'E70112': ('תֹּולֵדֹות', 'subs', 'generations', 'tovleidovt'),
        'E70113': ('יְהוָה', 'nmpr', 'YHWH', "y'hva"),
        'E70114': ('שִׂיחַ', 'subs', 'shrub', 'siach'),
        'E70115': ('שָׂדֶה', 'subs', 'open field', 'sadeh'),
        'E70116': ('טֶרֶם', 'subs', 'beginning', 'terem'),
        'E70117': ('צמח', 'verb', 'sprout', 'tsmch'),
        'E70118': ('לֹא', 'nega', 'not', 'lo'),
        'E70119': ('מטר', 'verb', 'rain', 'mtr'),
        'E70120': ('אַיִן', 'subs', '[NEG]', 'ayin'),
        'E70121': ('עבד', 'verb', 'work, serve', 'vd'),
        'E70122': ('אֵד', 'subs', '[uncertain]', 'eid'),
        'E70123': ('עלה', 'verb', 'ascend', 'lh'),
        'E70124': ('שׁקה', 'verb', 'give drink', 'shkh'),
        'E70125': ('יצר', 'verb', 'shape', 'ytsr'),
        'E70126': ('עָפָר', 'subs', 'dust', 'afar'),
        'E70127': ('נפח', 'verb', 'blow', 'nfch'),
        'E70128': ('אַף', 'subs', 'nose', 'af'),
        'E70129': ('נְשָׁמָה', 'subs', 'breath', "n'shama"),
        'E70130': ('חַיִּים', 'subs', 'life', 'chayim'),
        'E70131': ('נטע', 'verb', 'plant', 'nt'),
        'E70132': ('גַּן', 'subs', 'garden', 'gan'),
        'E70133': ('עֵדֶן', 'nmpr', 'Eden', 'eiden'),
        'E70134': ('קֶדֶם', 'subs', 'front', 'kedem'),
        'E70135': ('שׂים', 'verb', 'put', 'sym'),
        'E70136': ('שָׁם', 'advb', 'there', 'sham'),
        'E70137': ('חמד', 'verb', 'desire', 'chmd'),
        'E70138': ('מַרְאֶה', 'subs', 'sight', 'mareh'),
        'E70139': ('מַאֲכָל', 'subs', 'food', "ma'achol"),
        'E70140': ('דַּעַת', 'subs', 'knowledge', "da'at"),
        'E70141': ('רַע', 'adjv', 'evil', 'ra'),
        'E70142': ('נָהָר', 'subs', 'stream', 'nahar'),
        'E70143': ('פרד', 'verb', 'divide', 'frd'),
        'E70144': ('אַרְבַּע', 'subs', 'four', 'arba'),
        'E70145': ('רֹאשׁ', 'subs', 'head', 'rosh'),
        'E70146': ('שֵׁם', 'subs', 'name', 'sheim'),
        'E70147': ('פִּישֹׁון', 'nmpr', 'Pishon', 'pishovn'),
        'E70148': ('הוּא', 'prps', 'he', 'hu'),
        'E70149': ('סבב', 'verb', 'turn', "sv'v"),
        'E70150': ('חֲוִילָה', 'nmpr', 'Havilah', 'chavila'),
        'E70151': ('זָהָב', 'subs', 'gold', 'zahav'),
        'E70152': ('הִיא', 'prps', 'she', 'hi'),
        'E70153': ('בְּדֹלַח', 'subs', 'bdellium-gum', "b'dolach"),
        'E70154': ('אֶבֶן', 'subs', 'stone', 'even'),
        'E70155': ('שֹׁהַם', 'subs', 'carnelian', 'shoham'),
        'E70156': ('גִּיחֹון', 'nmpr', 'Gihon', 'gichovn'),
        'E70157': ('כּוּשׁ', 'nmpr', 'Cush', 'kush'),
        'E70158': ('חִדֶּקֶל', 'nmpr', 'Tigris', 'chidekel'),
        'E70159': ('הלך', 'verb', 'walk', 'hlch'),
        'E70160': ('קִדְמָה', 'subs', 'front', 'kidma'),
        'E70161': ('אַשּׁוּר', 'nmpr', 'Asshur', 'ashur'),
        'E70162': ('פְּרָת', 'nmpr', 'Euphrates', "p'rat"),
        'E70163': ('לקח', 'verb', 'take', 'lkch'),
        'E70164': ('נוח', 'verb', 'settle', 'nvch'),
        'E70165': ('שׁמר', 'verb', 'keep', 'shmr'),
        'E70166': ('צוה', 'verb', 'command', 'tsvh'),
        'E70167': ('אכל', 'verb', 'eat', 'chl'),
        'E70168': ('מות', 'verb', 'die', 'mvt'),
        'E70169': ('בַּד', 'subs', 'linen, part, stave', 'bad'),
        'E70170': ('עֵזֶר', 'subs', 'help, helper', 'eizer'),
        'E70171': ('נֶגֶד', 'subs', 'counterpart', 'neged'),
        'E70172': ('בוא', 'verb', 'come', 'vv'),
        'E70173': ('מָה', 'prin', 'what', 'ma'),
        'E70174': ('מצא', 'verb', 'find', 'mts'),
        'E70175': ('נפל', 'verb', 'fall', 'nfl'),
        'E70176': ('תַּרְדֵּמָה', 'subs', 'sleep', 'tardeima'),
        'E70177': ('ישׁן', 'verb', 'sleep', 'yshn'),
        'E70178': ('צֵלָע', 'subs', 'side', 'tseila'),
        'E70179': ('סגר', 'verb', 'close', 'sgr'),
        'E70180': ('בָּשָׂר', 'subs', 'flesh', 'basar'),
        'E70181': ('בנה', 'verb', 'build', 'vnh'),
        'E70182': ('אִשָּׁה', 'subs', 'woman', 'isha'),
        'E70183': ('זֹאת', 'prde', 'this', 'zot'),
        'E70184': ('פַּעַם', 'subs', 'foot', "pa'am"),
        'E70185': ('עֶצֶם', 'subs', 'bone', 'etsem'),
        'E70186': ('אִישׁ', 'subs', 'man', 'ish'),
        'E70187': ('עזב', 'verb', 'leave', 'zv'),
        'E70188': ('אָב', 'subs', 'father', 'av'),
        'E70189': ('אֵם', 'subs', 'mother', 'eim'),
        'E70190': ('דבק', 'verb', 'cling, cleave to', 'dvk'),
        'E70191': ('עָרֹום', 'adjv', 'naked', 'arovm'),
        'E70192': ('בושׁ', 'verb', 'be ashamed', 'vvsh'),
        'E70193': ('נָחָשׁ', 'subs', 'serpent', 'nachash'),
        'E70194': ('עָרוּם', 'adjv', 'shrewd', 'arum'),
        'E70195': ('אַף', 'advb', 'even', 'af'),
        'E70196': ('נגע', 'verb', 'touch', 'ng'),
        'E70197': ('פֶּן', 'conj', 'lest', 'pen'),
        'E70198': ('ידע', 'verb', 'know', 'yd'),
        'E70199': ('פקח', 'verb', 'open', 'fkch'),
        'E70200': ('עַיִן', 'subs', 'eye', 'ayin'),
        'E70201': ('תַּאֲוָה', 'subs', 'desire', "ta'ava"),
        'E70202': ('שׂכל', 'verb', 'prosper', 'schl'),
        'E70203': ('גַּם', 'advb', 'even', 'gam'),
        'E70204': ('עִם', 'prep', 'with', 'im'),
        'E70205': ('עֵירֹם', 'adjv', 'naked', 'eirom'),
        'E70206': ('הֵם', 'prps', 'they', 'heim'),
        'E70207': ('תפר', 'verb', 'sew', 'tfr'),
        'E70208': ('עָלֶה', 'subs', 'leafage', 'aleh'),
        'E70209': ('תְּאֵנָה', 'subs', 'fig', "t'eina"),
        'E70210': ('חֲגֹורָה', 'subs', 'girdle', 'chagovra'),
        'E70211': ('שׁמע', 'verb', 'hear', 'shm'),
        'E70212': ('קֹול', 'subs', 'sound', 'kovl'),
        'E70213': ('חבא', 'verb', 'hide', 'chv'),
        'E70214': ('אֵי', 'inrg', 'where', 'ei'),
        'E70215': ('ירא', 'verb', 'fear', 'yr'),
        'E70216': ('אָנֹכִי', 'prps', 'i', 'anochi'),
        'E70217': ('מִי', 'prin', 'who', 'mi'),
        'E70218': ('נגד', 'verb', 'report', 'ngd'),
        'E70219': ('אַתָּה', 'prps', 'you', 'ata'),
        'E70220': ('הֲ', 'inrg', '[interrogative]', 'ah'),
        'E70221': ('בֵּלֶת', 'subs', 'failure', 'beilet'),
        'E70222': ('עִמָּד', 'subs', 'company', 'imad'),
        'E70223': ('נשׁא', 'verb', 'beguile', 'nsh'),
        'E70224': ('ארר', 'verb', 'curse', "r'r"),
        'E70225': ('גָּחֹון', 'subs', 'belly', 'gachovn'),
        'E70226': ('אֵיבָה', 'subs', 'enmity', 'eiva'),
        'E70227': ('שׁית', 'verb', 'put', 'shyt'),
        'E70228': ('שׁוף', 'verb', 'bruise', 'shvf'),
        'E70229': ('שׁוף', 'verb', 'snatch', 'shvf'),
        'E70230': ('עָקֵב', 'subs', 'heel', 'akeiv'),
        'E70231': ('עִצָּבֹון', 'subs', 'pain', 'itsavovn'),
        'E70232': ('הֵרֹון', 'subs', 'pregnancy', 'heirovn'),
        'E70233': ('עֶצֶב', 'subs', 'pain', 'etsev'),
        'E70234': ('ילד', 'verb', 'bear', 'yld'),
        'E70235': ('בֵּן', 'subs', 'son', 'bein'),
        'E70236': ('תְּשׁוּקָה', 'subs', 'impulse', "t'shuka"),
        'E70237': ('עֲבוּר', 'subs', 'way', 'avur'),
        'E70238': ('קֹוץ', 'subs', 'thorn-bush', 'kovts'),
        'E70239': ('דַּרְדַּר', 'subs', 'thistles', 'dardar'),
        'E70240': ('זֵעָה', 'subs', 'sweat', 'zeia'),
        'E70241': ('לֶחֶם', 'subs', 'bread', 'lechem'),
        'E70242': ('עַד', 'prep', 'unto', 'ad'),
        'E70243': ('שׁוב', 'verb', 'return', 'shvv'),
        'E70244': ('חַוָּה', 'nmpr', 'Eve', 'chaua'),
        'E70245': ('כֻּתֹּנֶת', 'subs', 'tunic', 'kutonet'),
        'E70246': ('עֹור', 'subs', 'skin', 'ovr'),
        'E70247': ('לבשׁ', 'verb', 'cloth', 'lvsh'),
        'E70248': ('הֵן', 'intj', 'behold', 'hein'),
        'E70249': ('עַתָּה', 'advb', 'now', 'ata'),
        'E70250': ('שׁלח', 'verb', 'send', 'shlch'),
        'E70251': ('יָד', 'subs', 'hand', 'yad'),
        'E70252': ('חיה', 'verb', 'be alive', 'chyh'),
        'E70253': ('עֹולָם', 'subs', 'eternity', 'ovlam'),
        'E70254': ('גרשׁ', 'verb', 'drive out', 'grsh'),
        'E70255': ('שׁכן', 'verb', 'dwell', 'shchn'),
        'E70256': ('כְּרוּב', 'subs', 'cherub', "k'ruv"),
        'E70257': ('לַהַט', 'subs', 'flame', 'lahat'),
        'E70258': ('חֶרֶב', 'subs', 'dagger', 'cherev'),
        'E70259': ('הפך', 'verb', 'turn', 'hfch'),
        'E70260': ('דֶּרֶךְ', 'subs', 'way', 'derech'),
        'E70261': ('הרה', 'verb', 'be pregnant', 'hrh'),
        'E70262': ('קַיִן', 'nmpr', 'Kain', 'kayin'),
        'E70263': ('קנה', 'verb', 'create', 'knh'),
        'E70264': ('אֵת', 'prep', 'together with', 'eit'),
        'E70265': ('יסף', 'verb', 'add', 'ysf'),
        'E70266': ('אָח', 'subs', 'brother', 'ach'),
        'E70267': ('הֶבֶל', 'nmpr', 'Abel', 'hevel'),
        'E70268': ('רעה', 'verb', 'pasture', 'rh'),
        'E70269': ('צֹאן', 'subs', 'cattle', 'tson'),
        'E70270': ('קֵץ', 'subs', 'end', 'keits'),
        'E70271': ('מִנְחָה', 'subs', 'present', 'mincha'),
        'E70272': ('בְּכֹר', 'subs', 'first-born', "b'chor"),
        'E70273': ('חֵלֶב', 'subs', 'fat', 'cheilev'),
        'E70274': ('שׁעה', 'verb', 'look', 'shh'),
        'E70275': ('חרה', 'verb', 'be hot', 'chrh'),
        'E70276': ('לָמָה', 'inrg', 'why', 'lama'),
        'E70277': ('אִם', 'conj', 'if', 'im'),
        'E70278': ('יטב', 'verb', 'be good', 'ytv'),
        'E70279': ('שְׂאֵת', 'subs', 'uprising', "s'eit"),
        'E70280': ('פֶּתַח', 'subs', 'opening', 'petach'),
        'E70281': ('חַטָּאת', 'subs', 'sin', 'chatat'),
        'E70282': ('רבץ', 'verb', 'lie down', 'rvts'),
        'E70283': ('קום', 'verb', 'arise', 'kvm'),
        'E70284': ('הרג', 'verb', 'kill', 'hrg'),
        'E70285': ('דָּם', 'subs', 'blood', 'dam'),
        'E70286': ('צעק', 'verb', 'cry', 'tsk'),
        'E70287': ('פצה', 'verb', 'open', "fts'h"),
        'E70288': ('פֶּה', 'subs', 'mouth', 'peh'),
        'E70289': ('כֹּחַ', 'subs', 'strength', 'koach'),
        'E70290': ('נוע', 'verb', 'quiver', 'nv'),
        'E70291': ('נוד', 'verb', 'waver', 'nvd'),
        'E70292': ('עָוֹן', 'subs', 'sin', 'aon'),
        'E70293': ('נשׂא', 'verb', 'lift', 'ns'),
        'E70294': ('סתר', 'verb', 'hide', 'str'),
        'E70295': ('לָכֵן', 'advb', 'therefore', 'lachein'),
        'E70296': ('שֶׁבַע', 'subs', 'seven', 'sheva'),
        'E70297': ('נקם', 'verb', 'avenge', 'nkm'),
        'E70298': ('נכה', 'verb', 'strike', 'nchh'),
        'E70299': ('ישׁב', 'verb', 'sit', 'yshv'),
        'E70300': ('נֹוד', 'nmpr', 'Nod', 'novd'),
        'E70301': ('חֲנֹוךְ', 'nmpr', 'Henoch', 'chanovch'),
        'E70302': ('עִיר', 'subs', 'town', 'ir'),
        'E70303': ('חֲנֹוךְ', 'nmpr', 'Enoch', 'chanovch'),
        'E70304': ('עִירָד', 'nmpr', 'Irad', 'irad'),
        'E70305': ('מְחוּיָאֵל', 'nmpr', 'Mehujael', "m'chuyaeil"),
        'E70306': ('מְחִיּיָאֵל', 'nmpr', 'Mehujael', "m'chiaeil"),
        'E70307': ('מְתוּשָׁאֵל', 'nmpr', 'Methushael', "m'tushaeil"),
        'E70308': ('לֶמֶךְ', 'nmpr', 'Lamech', 'lemech'),
        'E70309': ('עָדָה', 'nmpr', 'Adah', 'ada'),
        'E70310': ('צִלָּה', 'nmpr', 'Zillah', 'tsila'),
        'E70311': ('יָבָל', 'nmpr', 'Jabal', 'yaval'),
        'E70312': ('אֹהֶל', 'subs', 'tent', 'ohel'),
        'E70313': ('מִקְנֶה', 'subs', 'purchase', 'mikneh'),
        'E70314': ('יוּבָל', 'nmpr', 'Jubal', 'yuval'),
        'E70315': ('תפשׂ', 'verb', 'seize', 'tfs'),
        'E70316': ('כִּנֹּור', 'subs', 'cither', 'kinovr'),
        'E70317': ('עוּגָב', 'subs', 'flute', 'ugav'),
        'E70318': ('תּוּבַל קַיִן', 'nmpr', 'Tubal-Cain', 'tuval kayin'),
        'E70319': ('לטשׁ', 'verb', 'sharpen', 'ltsh'),
        'E70320': ('חרשׁ', 'verb', 'plough', 'chrsh'),
        'E70321': ('נְחֹשֶׁת', 'subs', 'bronze', "n'choshet"),
        'E70322': ('בַּרְזֶל', 'subs', 'iron', 'barzel'),
        'E70323': ('אָחֹות', 'subs', 'sister', 'achovt'),
        'E70324': ('נַעֲמָה', 'nmpr', 'Naamah', "na'ama"),
        'E70325': ('אזן', 'verb', 'listen', 'zn'),
        'E70326': ('אִמְרָה', 'subs', 'word', 'imra'),
        'E70327': ('פֶּצַע', 'subs', 'bruise', 'petsa'),
        'E70328': ('יֶלֶד', 'subs', 'boy', 'yeled'),
        'E70329': ('חַבּוּרָה', 'subs', 'bruise', 'chabura'),
        'E70330': ('אָדָם', 'nmpr', 'Adam', 'adam'),
        'E70331': ('עֹוד', 'subs', 'duration', 'ovd'),
        'E70332': ('שֵׁת', 'nmpr', 'Seth', 'sheit'),
        'E70333': ('אַחֵר', 'adjv', 'other', 'acheir'),
        'E70334': ('אֱנֹושׁ', 'nmpr', 'Enosh', 'enovsh'),
        'E70335': ('אָז', 'advb', 'then', 'az'),
        'E70336': ('חלל', 'verb', 'defile', "chl'l"),
        'E70337': ('זֶה', 'prde', 'this', 'zeh'),
        'E70338': ('סֵפֶר', 'subs', 'letter', 'seifer'),
        'E70339': ('שָׁלֹשׁ', 'subs', 'three', 'shalosh'),
        'E70340': ('מֵאָה', 'subs', 'hundred', 'meia'),
        'E70341': ('אַחַר', 'subs', 'after', 'achar'),
        'E70342': ('שְׁמֹנֶה', 'subs', 'eight', "sh'moneh"),
        'E70343': ('בַּת', 'subs', 'daughter', 'bat'),
        'E70344': ('תֵּשַׁע', 'subs', 'nine', 'teisha'),
        'E70345': ('חָמֵשׁ', 'subs', 'five', 'chameish'),
        'E70346': ('עֶשְׂרֵה', 'subs', '-teen', 'esrei'),
        'E70347': ('קֵינָן', 'nmpr', 'Kenan', 'keinan'),
        'E70348': ('מַהֲלַלְאֵל', 'nmpr', 'Mahalalel', 'mahalaleil'),
        'E70349': ('עֶשֶׂר', 'subs', 'ten', 'eser'),
        'E70350': ('שֵׁשׁ', 'subs', 'six', 'sheish'),
        'E70351': ('יֶרֶד', 'nmpr', 'Jered', 'yered'),
        'E70352': ('מְתוּשֶׁלַח', 'nmpr', 'Methuselah', "m'tushelach"),
        'E70353': ('נֹחַ', 'nmpr', 'Noah', 'noach'),
        'E70354': ('נחם', 'verb', 'repent, console', 'nchm'),
        'E70355': ('מַעֲשֶׂה', 'subs', 'deed', "ma'aseh"),
        'E70356': ('שֵׁם', 'nmpr', 'Shem', 'sheim'),
        'E70357': ('חָם', 'nmpr', 'Ham', 'cham'),
        'E70358': ('יֶפֶת', 'nmpr', 'Japheth', 'yefet'),
        'E70359': ('רבב', 'verb', 'be much', "rv'v"),
        'E70360': ('הֵנָּה', 'prps', 'they', 'heina'),
        'E70361': ('בחר', 'verb', 'examine', 'vchr'),
        'E70362': ('דון', 'verb', '[uncertain]', 'dvn'),
        'E70363': ('שַׁ', 'conj', '[relative]', 'sha'),
        'E70364': ('עֶשְׂרִים', 'subs', 'twenty', 'esrim'),
        'E70365': ('נְפִילִים', 'subs', 'giants', "n'filim"),
        'E70366': ('הֵמָּה', 'prps', 'they', 'heima'),
        'E70367': ('גִּבֹּור', 'subs', 'vigorous', 'gibovr'),
        'E70368': ('רַב', 'adjv', 'much', 'rav'),
        'E70369': ('רָעָה', 'subs', 'evil', "ra'a"),
        'E70370': ('יֵצֶר', 'subs', 'form', 'yeitser'),
        'E70371': ('מַחֲשָׁבָה', 'subs', 'thought', 'machashava'),
        'E70372': ('לֵב', 'subs', 'heart', 'leiv'),
        'E70373': ('רַק', 'advb', 'only', 'rak'),
        'E70374': ('עצב', 'verb', 'hurt', 'tsv'),
        'E70375': ('מחה', 'verb', 'wipe', 'mchh'),
        'E70376': ('חֵן', 'subs', 'grace', 'chein'),
        'E70377': ('צַדִּיק', 'adjv', 'just', 'tsadik'),
        'E70378': ('תָּמִים', 'adjv', 'complete', 'tamim'),
        'E70379': ('דֹּור', 'subs', 'generation', 'dovr'),
        'E70380': ('שׁחת', 'verb', 'destroy', 'shcht'),
        'E70381': ('חָמָס', 'subs', 'violence', 'chamas'),
        'E70382': ('תֵּבָה', 'subs', 'ark', 'teiva'),
        'E70383': ('גֹּפֶר', 'subs', 'gopher', 'gofer'),
        'E70384': ('קֵן', 'subs', 'nest', 'kein'),
        'E70385': ('כפר', 'verb', 'cover', 'chfr'),
        'E70386': ('בַּיִת', 'subs', 'house', 'bayit'),
        'E70387': ('חוּץ', 'subs', 'outside', 'chuts'),
        'E70388': ('כֹּפֶר', 'subs', 'cover', 'kofer'),
        'E70389': ('אַמָּה', 'subs', 'cubit', 'ama'),
        'E70390': ('אֹרֶךְ', 'subs', 'length', 'orech'),
        'E70391': ('רֹחַב', 'subs', 'breadth', 'rochav'),
        'E70392': ('קֹומָה', 'subs', 'height', 'kovma'),
        'E70393': ('צֹהַר', 'subs', 'roof', 'tsohar'),
        'E70394': ('מַעַל', 'subs', 'top', "ma'al"),
        'E70395': ('צַד', 'subs', 'side', 'tsad'),
        'E70396': ('תַּחְתִּי', 'adjv', 'lower', 'tachti'),
        'E70397': ('אֲנִי', 'prps', 'i', 'ani'),
        'E70398': ('מַבּוּל', 'subs', 'deluge', 'mabul'),
        'E70399': ('גוע', 'verb', 'expire', 'gv'),
        'E70400': ('בְּרִית', 'subs', 'covenant', "b'rit"),
        'E70401': ('אסף', 'verb', 'gather', 'sf'),
        'E70402': ('טָהֹר', 'adjv', 'pure', 'tahor'),
        'E70403': ('יְקוּם', 'subs', 'substance', "y'kum"),
        'E70404': ('חֹדֶשׁ', 'subs', 'month', 'chodesh'),
        'E70405': ('עָשָׂר', 'subs', '-teen', 'asar'),
        'E70406': ('בקע', 'verb', 'split', 'vk'),
        'E70407': ('מַעְיָן', 'subs', 'well', 'mayan'),
        'E70408': ('אֲרֻבָּה', 'subs', 'hole', 'aruba'),
        'E70409': ('פתח', 'verb', 'open', "ft'ch"),
        'E70410': ('גֶּשֶׁם', 'subs', 'rain', 'geshem'),
        'E70411': ('צִפֹּור', 'subs', 'bird', 'tsipovr'),
        'E70412': ('בַּעַד', 'subs', 'distance', "ba'ad"),
        'E70413': ('רום', 'verb', 'be high', 'rvm'),
        'E70414': ('גבר', 'verb', 'be superior', 'gvr'),
        'E70415': ('כסה', 'verb', 'cover', "chs'h"),
        'E70416': ('הַר', 'subs', 'mountain', 'har'),
        'E70417': ('גָּבֹהַּ', 'adjv', 'high', 'gavoha'),
        'E70418': ('חָרָבָה', 'subs', 'dry ground', 'charava'),
        'E70419': ('שׁאר', 'verb', 'remain', 'shr'),
        'E70420': ('אַךְ', 'advb', 'only', 'ach'),
        'E70421': ('זכר', 'verb', 'remember', 'zchr'),
        'E70422': ('עבר', 'verb', 'pass', 'vr'),
        'E70423': ('שׁכך', 'verb', 'go down', "shch'ch"),
        'E70424': ('סכר', 'verb', 'stop', "s'chr"),
        'E70425': ('כלא', 'verb', 'restrain', 'chl'),
        'E70426': ('חסר', 'verb', 'diminish', 'chsr'),
        'E70427': ('קָצֶה', 'subs', 'end', 'katseh'),
        'E70428': ('אֲרָרַט', 'nmpr', 'Ararat', 'ararat'),
        'E70429': ('עֲשִׂירִי', 'adjv', 'tenth', 'asiri'),
        'E70430': ('חַלֹּון', 'subs', 'window', 'chalovn'),
        'E70431': ('עֹרֵב', 'subs', 'raven', 'oreiv'),
        'E70432': ('יבשׁ', 'verb', 'be dry', 'yvsh'),
        'E70433': ('יֹונָה', 'subs', 'dove', 'yovna'),
        'E70434': ('קלל', 'verb', 'be slight', "kl'l"),
        'E70435': ('מָנֹוחַ', 'subs', 'resting place', 'manovach'),
        'E70436': ('כַּף', 'subs', 'palm', 'kaf'),
        'E70437': ('רֶגֶל', 'subs', 'foot', 'regel'),
        'E70438': ('חיל', 'verb', 'wait', 'chyl'),
        'E70439': ('עֵת', 'subs', 'time', 'eit'),
        'E70440': ('זַיִת', 'subs', 'olive', 'zayit'),
        'E70441': ('טָרָף', 'adjv', 'fresh plucked', 'taraf'),
        'E70442': ('יחל', 'verb', 'wait, to hope', 'ychl'),
        'E70443': ('רִאשֹׁון', 'adjv', 'first', 'rishovn'),
        'E70444': ('חרב', 'verb', 'be dry', 'chrv'),
        'E70445': ('סור', 'verb', 'turn aside', 'svr'),
        'E70446': ('מִכְסֶה', 'subs', 'covering', 'michseh'),
        'E70447': ('דבר', 'verb', 'speak', 'dvr'),
        'E70448': ('מִשְׁפָּחָה', 'subs', 'clan', 'mishpacha'),
        'E70449': ('מִזְבֵּחַ', 'subs', 'altar', 'mizbeiach'),
        'E70450': ('עֹלָה', 'subs', 'burnt-offering', 'ola'),
        'E70451': ('רוח', 'verb', 'be spacious', 'rvch'),
        'E70452': ('רֵיחַ', 'subs', 'scent', 'reiach'),
        'E70453': ('נִיחֹחַ', 'subs', 'smell of appeasement', 'nichoach'),
        'E70454': ('נְעוּרִים', 'subs', 'youth', "n'urim"),
        'E70455': ('קָצִיר', 'subs', 'harvest', 'katsir'),
        'E70456': ('קֹר', 'subs', 'cold', 'kor'),
        'E70457': ('חֹם', 'subs', 'heat', 'chom'),
        'E70458': ('קַיִץ', 'subs', 'summer', 'kayits'),
        'E70459': ('חֹרֶף', 'subs', 'autumn', 'choref'),
        'E70460': ('מֹורָא', 'subs', 'fear', 'movra'),
        'E70461': ('חַת', 'subs', 'terror', 'chat'),
        'E70462': ('דָּג', 'subs', 'fish', 'dag'),
        'E70463': ('דרשׁ', 'verb', 'inquire', 'drsh'),
        'E70464': ('שׁפך', 'verb', 'pour', 'shfch'),
        'E70465': ('אַתֶּם', 'prps', 'you', 'atem'),
        'E70466': ('כרת', 'verb', 'cut', 'chrt'),
        'E70467': ('קֶשֶׁת', 'subs', 'bow', 'keshet'),
        'E70468': ('עָנָן', 'subs', 'cloud', 'anan'),
        'E70469': ('ענן', 'verb', 'appear', "n'n"),
        'E70470': ('כְּנַעַן', 'nmpr', 'Canaan', "k'na'an"),
        'E70471': ('נפץ', 'verb', 'shatter', 'nfts'),
        'E70472': ('כֶּרֶם', 'subs', 'vineyard', 'kerem'),
        'E70473': ('שׁתה', 'verb', 'drink', 'shth'),
        'E70474': ('יַיִן', 'subs', 'wine', 'yayin'),
        'E70475': ('שׁכר', 'verb', 'be drunk', 'shchr'),
        'E70476': ('גלה', 'verb', 'uncover', 'glh'),
        'E70477': ('עֶרְוָה', 'subs', 'nakedness', 'erva'),
        'E70478': ('שִׂמְלָה', 'subs', 'mantle', 'simla'),
        'E70479': ('שְׁכֶם', 'subs', 'shoulder', "sh'chem"),
        'E70480': ('אֲחֹרַנִּית', 'advb', 'backwards', 'achoranit'),
        'E70481': ('יקץ', 'verb', 'awake', 'ykts'),
        'E70482': ('קָטָן', 'adjv', 'small', 'katan'),
        'E70483': ('עֶבֶד', 'subs', 'servant', 'eved'),
        'E70484': ('פתה', 'verb', 'be spacious', 'fth'),
        'E70485': ('גֹּמֶר', 'nmpr', 'Gomer', 'gomer'),
        'E70486': ('מָגֹוג', 'nmpr', 'Magog', 'magovg'),
        'E70487': ('מָדַי', 'nmpr', 'Media', 'maday'),
        'E70488': ('יָוָן', 'nmpr', '[jawan, Greece]', 'yavan'),
        'E70489': ('תֻּבַל', 'nmpr', 'Tubal', 'tuval'),
        'E70490': ('מֶשֶׁךְ', 'nmpr', 'Meshech', 'meshech'),
        'E70491': ('תִּירָס', 'nmpr', 'Tiras', 'tiras'),
        'E70492': ('אַשְׁכֲּנַז', 'nmpr', 'Ashkenaz', 'ashkanaz'),
        'E70493': ('רִיפַת', 'nmpr', 'Riphath', 'rifat'),
        'E70494': ('תֹּוגַרְמָה', 'nmpr', 'Togarmah', 'tovgarma'),
        'E70495': ('אֱלִישָׁה', 'nmpr', 'Elishah', 'elisha'),
        'E70496': ('תַּרְשִׁישׁ', 'nmpr', 'Tarshish', 'tarshish'),
        'E70497': ('כִּתִּיִּים', 'nmpr', 'Kittim', "kiti'im"),
        'E70498': ('דֹּדָנִים', 'nmpr', 'Rodanim', 'dodanim'),
        'E70499': ('אִי', 'subs', 'coast, island', 'i'),
        'E70500': ('גֹּוי', 'subs', 'people', 'govy'),
        'E70501': ('לָשֹׁון', 'subs', 'tongue', 'lashovn'),
        'E70502': ('מִצְרַיִם', 'nmpr', 'Egypt', 'mitsrayim'),
        'E70503': ('פּוּט', 'nmpr', 'Put', 'put'),
        'E70504': ('כּוּשׁ', 'nmpr', 'Cush', 'kush'),
        'E70505': ('סְבָא', 'nmpr', 'Seba', "s'va"),
        'E70506': ('סַבְתָּא', 'nmpr', 'Sabta', 'savta'),
        'E70507': ('רַעְמָה', 'nmpr', 'Raamah', 'rama'),
        'E70508': ('סַבְתְּכָא', 'nmpr', 'Sabteca', "savt'cho"),
        'E70509': ('שְׁבָא', 'nmpr', 'Sheba', "sh'va"),
        'E70510': ('דְּדָן', 'nmpr', 'Dedan', "d'dan"),
        'E70511': ('נִמְרֹד', 'nmpr', 'Nimrod', 'nimrod'),
        'E70512': ('צַיִד', 'subs', 'hunting', 'tsayid'),
        'E70513': ('מַמְלֶכֶת', 'subs', 'kingdom', 'mamlechet'),
        'E70514': ('בָּבֶל', 'nmpr', 'Babel', 'bavel'),
        'E70515': ('אֶרֶךְ', 'nmpr', 'Erech', 'erech'),
        'E70516': ('אַכַּד', 'nmpr', 'Akkad', 'akad'),
        'E70517': ('כַּלְנֵה', 'nmpr', 'Calneh', 'kalnei'),
        'E70518': ('שִׁנְעָר', 'nmpr', 'Shinar', 'shinar'),
        'E70519': ('נִינְוֵה', 'nmpr', 'Nineveh', 'ninvei'),
        'E70520': ('רְחֹבֹת עִיר', 'nmpr', 'Rehoboth Ir', "r'chovot ir"),
        'E70521': ('כֶּלַח', 'nmpr', 'Calah', 'kelach'),
        'E70522': ('רֶסֶן', 'nmpr', 'Resen', 'resen'),
        'E70523': ('לוּדִים', 'nmpr', 'Ludites', 'ludim'),
        'E70524': ('עֲנָמִים', 'nmpr', 'Anamites', 'anamim'),
        'E70525': ('לְהָבִים', 'nmpr', 'Lehabites', "l'havim"),
        'E70526': ('נַפְתֻּחִים', 'nmpr', 'Naphtuhites', 'naftuchim'),
        'E70527': ('פַּתְרֻסִים', 'nmpr', 'Pathrusites', 'patrusim'),
        'E70528': ('כַּסְלֻחִים', 'nmpr', 'Casluhites', 'kasluchim'),
        'E70529': ('פְּלִשְׁתִּי', 'adjv', 'Philistine', "p'lishti"),
        'E70530': ('כַּפְתֹּרִי', 'adjv', 'Caphtorite', 'kaftori'),
        'E70531': ('צִידֹון', 'nmpr', 'Sidon', 'tsidovn'),
        'E70532': ('חֵת', 'nmpr', 'Heth', 'cheit'),
        'E70533': ('יְבוּסִי', 'adjv', 'Jebusite', "y'vusi"),
        'E70534': ('אֱמֹרִי', 'adjv', 'Amorite', 'emori'),
        'E70535': ('גִּרְגָּשִׁי', 'adjv', 'Girgashite', 'girgashi'),
        'E70536': ('חִוִּי', 'adjv', 'Hivite', 'chiui'),
        'E70537': ('עַרְקִי', 'adjv', 'Arkite', 'arki'),
        'E70538': ('סִינִי', 'adjv', 'Sinite', 'sini'),
        'E70539': ('אַרְוָדִי', 'adjv', 'Arvadite', 'arvadi'),
        'E70540': ('צְמָרִי', 'adjv', 'Zemarite', "ts'mari"),
        'E70541': ('חֲמָתִי', 'adjv', 'Hamathite', 'chamati'),
        'E70542': ('פוץ', 'verb', 'disperse', 'fvts'),
        'E70543': ('כְּנַעֲנִי', 'adjv', 'Canaanite', "k'na'ani"),
        'E70544': ('גְּבוּל', 'subs', 'boundary', "g'vul"),
        'E70545': ('גְּרָר', 'nmpr', 'Gerar', "g'rar"),
        'E70546': ('עַזָּה', 'nmpr', 'Gaza', 'aza'),
        'E70547': ('סְדֹם', 'nmpr', 'Sodom', "s'dom"),
        'E70548': ('עֲמֹרָה', 'nmpr', 'Gomorrah', 'amora'),
        'E70549': ('אַדְמָה', 'nmpr', 'Admah', 'adma'),
        'E70550': ('צְבֹאיִם', 'nmpr', 'Zeboiim', "ts'voyim"),
        'E70551': ('לֶשַׁע', 'nmpr', 'Lasha', 'lesha'),
        'E70552': ('עֵבֶר', 'nmpr', 'Eber', 'eiver'),
        'E70553': ('עֵילָם', 'nmpr', 'Elam', 'eilam'),
        'E70554': ('אַרְפַּכְשָׁד', 'nmpr', 'Arpachshad', 'arpachshod'),
        'E70555': ('לוּד', 'nmpr', 'Lud', 'lud'),
        'E70556': ('אֲרָם', 'nmpr', 'Aram', 'aram'),
        'E70557': ('עוּץ', 'nmpr', 'Uz', 'uts'),
        'E70558': ('חוּל', 'nmpr', 'Hul', 'chul'),
        'E70559': ('גֶּתֶר', 'nmpr', 'Gether', 'geter'),
        'E70560': ('מַשׁ', 'nmpr', 'Meshech', 'mash'),
        'E70561': ('שֶׁלַח', 'nmpr', 'Shelah', 'shelach'),
        'E70562': ('פֶּלֶג', 'nmpr', 'Peleg', 'peleg'),
        'E70563': ('פלג', 'verb', 'divide', 'flg'),
        'E70564': ('יָקְטָן', 'nmpr', 'Joktan', 'yaktan'),
        'E70565': ('אַלְמֹודָד', 'nmpr', 'Almodad', 'almovdad'),
        'E70566': ('שֶׁלֶף', 'nmpr', 'Sheleph', 'shelef'),
        'E70567': ('חֲצַרְמָוֶת', 'nmpr', 'Hazarmaveth', 'chatsarmavet'),
        'E70568': ('יֶרַח', 'nmpr', 'Jerah', 'yerach'),
        'E70569': ('הֲדֹורָם', 'nmpr', 'Hadoram', 'hadovram'),
        'E70570': ('אוּזָל', 'nmpr', 'Uzal', 'uzal'),
        'E70571': ('דִּקְלָה', 'nmpr', 'Diklah', 'dikla'),
        'E70572': ('עֹובָל', 'nmpr', 'Obal', 'ovval'),
        'E70573': ('אֲבִימָאֵל', 'nmpr', 'Abimael', 'avimaeil'),
        'E70574': ('אֹופִיר', 'nmpr', '[land of gold]', 'ovfir'),
        'E70575': ('יֹובָב', 'nmpr', 'Jobab', 'yovvav'),
        'E70576': ('מֹושָׁב', 'subs', 'seat', 'movshav'),
        'E70577': ('מֵשָׁא', 'nmpr', 'Mesha', 'meisha'),
        'E70578': ('סְפָר', 'nmpr', 'Sephar', "s'far"),
        'E70579': ('שָׂפָה', 'subs', 'lip', 'safa'),
        'E70580': ('דָּבָר', 'subs', 'word', 'davar'),
        'E70581': ('נסע', 'verb', 'pull out', 'ns'),
        'E70582': ('בִּקְעָה', 'subs', 'valley', 'bika'),
        'E70583': ('רֵעַ', 'subs', 'fellow', 'reia'),
        'E70584': ('יהב', 'verb', 'give', 'yhv'),
        'E70585': ('לבן', 'verb', 'make brick', 'lvn'),
        'E70586': ('לְבֵנָה', 'subs', 'brick', "l'veina"),
        'E70587': ('שׂרף', 'verb', 'burn', 'srf'),
        'E70588': ('שְׂרֵפָה', 'subs', 'burning', "s'reifa"),
        'E70589': ('חֵמָר', 'subs', 'bitumen', 'cheimar'),
        'E70590': ('חֹמֶר', 'subs', 'clay', 'chomer'),
        'E70591': ('מִגְדָּל', 'subs', 'tower', 'migdal'),
        'E70592': ('ירד', 'verb', 'descend', 'yrd'),
        'E70593': ('עַם', 'subs', 'people', 'am'),
        'E70594': ('בצר', 'verb', 'be impossible', 'vtsr'),
        'E70595': ('זמם', 'verb', 'ponder', "zm'm"),
        'E70596': ('בלל', 'verb', 'moisten, confound', "vl'l"),
        'E70597': ('חדל', 'verb', 'cease', 'chdl'),
        'E70598': ('רְעוּ', 'nmpr', 'Reu', "r'u"),
        'E70599': ('שְׂרוּג', 'nmpr', 'Serug', "s'rug"),
        'E70600': ('נָחֹור', 'nmpr', 'Nahor', 'nachovr'),
        'E70601': ('תֶּרַח', 'nmpr', 'Terah', 'terach'),
        'E70602': ('אַבְרָם', 'nmpr', 'Abram', 'avram'),
        'E70603': ('הָרָן', 'nmpr', 'Haran', 'haran'),
        'E70604': ('לֹוט', 'nmpr', 'Lot', 'lovt'),
        'E70605': ('מֹולֶדֶת', 'subs', 'offspring', 'movledet'),
        'E70606': ('אוּר', 'nmpr', '[ur of chaldeans]', 'ur'),
        'E70607': ('כַּשְׂדִּים', 'nmpr', 'Chaldeans', 'kasdim'),
        'E70608': ('שָׂרַי', 'nmpr', 'Sarai', 'saray'),
        'E70609': ('מִלְכָּה', 'nmpr', 'Milcah', 'milka'),
        'E70610': ('יִסְכָּה', 'nmpr', 'Iscah', 'yiska'),
        'E70611': ('עָקָר', 'adjv', 'barren', 'akar'),
        'E70612': ('וָלָד', 'subs', 'child', 'valad'),
        'E70613': ('כַּלָּה', 'subs', 'bride', 'kala'),
        'E70614': ('חָרָן', 'nmpr', '[town]', 'charan'),
        'E70615': ('גדל', 'verb', 'be strong', 'gdl'),
        'E70616': ('בְּרָכָה', 'subs', 'blessing', "b'racho"),
        'E70617': ('רְכוּשׁ', 'subs', 'property', "r'chush"),
        'E70618': ('רכשׁ', 'verb', 'gather', 'rchsh'),
        'E70619': ('שְׁכֶם', 'nmpr', 'Shechem', "sh'chem"),
        'E70620': ('אֵלֹון', 'subs', 'big tree', 'eilovn'),
        'E70621': ('מֹורֶה', 'nmpr', 'Moreh', 'movreh'),
        'E70622': ('עתק', 'verb', 'advance', 'tk'),
        'E70623': ('בֵּית אֵל', 'nmpr', 'Bethel', 'beit eil'),
        'E70624': ('נטה', 'verb', 'extend', 'nth'),
        'E70625': ('עַי', 'nmpr', 'Ai', 'ay'),
        'E70626': ('נֶגֶב', 'subs', 'south', 'negev'),
        'E70627': ('רָעָב', 'subs', 'hunger', "ra'av"),
        'E70628': ('גור', 'verb', 'dwell', 'gvr'),
        'E70629': ('כָּבֵד', 'adjv', 'heavy', 'kaveid'),
        'E70630': ('קרב', 'verb', 'approach', 'krv'),
        'E70631': ('נָא', 'intj', 'yeah', 'na'),
        'E70632': ('יָפֶה', 'adjv', 'beautiful', 'yafeh'),
        'E70633': ('אַתְּ', 'prps', 'you', 'at'),
        'E70634': ('מִצְרִי', 'adjv', 'Egyptian', 'mitsri'),
        'E70635': ('לְמַעַן', 'prep', 'because of', "l'ma'an"),
        'E70636': ('גָּלָל', 'subs', 'matter', 'galal'),
        'E70637': ('שַׂר', 'subs', 'chief', 'sar'),
        'E70638': ('פַּרְעֹה', 'subs', 'pharaoh', 'paro'),
        'E70639': ('הלל', 'verb', 'praise', "hl'l"),
        'E70640': ('בָּקָר', 'subs', 'cattle', 'bakar'),
        'E70641': ('חֲמֹור', 'subs', 'he-ass', 'chamovr'),
        'E70642': ('שִׁפְחָה', 'subs', 'maidservant', 'shifcha'),
        'E70643': ('אָתֹון', 'subs', 'she-ass', 'atovn'),
        'E70644': ('גָּמָל', 'subs', 'camel', 'gamal'),
        'E70645': ('נֶגַע', 'subs', 'stroke', 'nega'),
        'E70646': ('כֶּסֶף', 'subs', 'silver', 'kesef'),
        'E70647': ('מַסַּע', 'subs', 'breaking', 'masa'),
        'E70648': ('תְּחִלָּה', 'subs', 'beginning', "t'chila"),
        'E70649': ('יַחְדָּו', 'advb', 'together', 'yachdav'),
        'E70650': ('יכל', 'verb', 'be able', 'ychl'),
        'E70651': ('רִיב', 'subs', 'law-case', 'riv'),
        'E70652': ('פְּרִזִּי', 'adjv', 'Perizzite', "p'rizi"),
        'E70653': ('אַל', 'nega', 'not', 'al'),
        'E70654': ('מְרִיבָה', 'subs', 'quarrel', "m'riva"),
        'E70655': ('אֲנַחְנוּ', 'prps', 'we', 'anachnu'),
        'E70656': ('שְׂמֹאל', 'subs', 'lefthand side', "s'mol"),
        'E70657': ('ימן', 'verb', 'go to right', 'ymn'),
        'E70658': ('יָמִין', 'subs', 'right-hand side', 'yamin'),
        'E70659': ('שׂמאל', 'verb', 'turn left', 'sml'),
        'E70660': ('כִּכָּר', 'subs', 'disk', 'kikar'),
        'E70661': ('יַרְדֵּן', 'nmpr', 'Jordan', 'yardein'),
        'E70662': ('מַשְׁקֶה', 'subs', 'butler, irrigated, drink', 'mashkeh'),
        'E70663': ('צֹעַר', 'nmpr', 'Zoar', 'tsoar'),
        'E70664': ('אהל', 'verb', 'pitch a tent', 'hl'),
        'E70665': ('חַטָּא', 'adjv', 'sinful', 'chata'),
        'E70666': ('צָפֹון', 'subs', 'north', 'tsafovn'),
        'E70667': ('מנה', 'verb', 'count', 'mnh'),
        'E70668': ('מַמְרֵא', 'nmpr', 'Mamre', 'mamrei'),
        'E70669': ('חֶבְרֹון', 'nmpr', 'Hebron', 'chevrovn'),
        'E70670': ('אַמְרָפֶל', 'nmpr', 'Amraphel', 'amrafel'),
        'E70671': ('מֶלֶךְ', 'subs', 'king', 'melech'),
        'E70672': ('אַרְיֹוךְ', 'nmpr', 'Arioch', 'aryovch'),
        'E70673': ('אֶלָּסָר', 'nmpr', 'Ellasar', 'elasar'),
        'E70674': ('כְּדָרְלָעֹמֶר', 'nmpr', 'Kedorlaomer', "k'darlaomer"),
        'E70675': ('תִּדְעָל', 'nmpr', 'Tidal', 'tidal'),
        'E70676': ('גֹּויִם', 'nmpr', 'Goyim', 'govyim'),
        'E70677': ('מִלְחָמָה', 'subs', 'war', 'milchama'),
        'E70678': ('בֶּרַע', 'nmpr', 'Bera', 'bera'),
        'E70679': ('בִּרְשַׁע', 'nmpr', 'Birsha', 'birsha'),
        'E70680': ('שִׁנְאָב', 'nmpr', 'Shinab', 'shinav'),
        'E70681': ('שֶׁמְאֵבֶר', 'nmpr', 'Shemeber', 'shemeiver'),
        'E70682': ('בֶּלַע', 'nmpr', 'Bela', 'bela'),
        'E70683': ('חבר', 'verb', 'be united', 'chvr'),
        'E70684': ('עֵמֶק', 'subs', 'valley', 'eimek'),
        'E70685': ('שִׂדִּים', 'nmpr', 'Siddim', 'sidim'),
        'E70686': ('מֶלַח', 'subs', 'salt', 'melach'),
        'E70687': ('מרד', 'verb', 'rebel', 'mrd'),
        'E70688': ('רְפָאִים', 'nmpr', 'Rephaim', "r'faim"),
        'E70689': ('עַשְׁתְּרֹת קַרְנַיִם',
                   'nmpr',
                   'Ashteroth Karnaim',
                   'ashtrot karnayim'),
        'E70690': ('זוּזִים', 'nmpr', 'Zuzim', 'zuzim'),
        'E70691': ('הָם', 'nmpr', 'Ham', 'ham'),
        'E70692': ('אֵימִים', 'nmpr', 'Emim', 'eimim'),
        'E70693': ('שָׁוֵה', 'subs', 'plain', 'shavei'),
        'E70694': ('קִרְיָתָיִם', 'nmpr', 'Kiriathaim', 'kiratayim'),
        'E70695': ('חֹרִי', 'adjv', 'Horite', 'chori'),
        'E70696': ('שֵׂעִיר', 'nmpr', 'Seir', 'seiir'),
        'E70697': ('אֵיל פָּארָן', 'nmpr', 'El Paran', 'eil paran'),
        'E70698': ('מִדְבָּר', 'subs', 'desert', 'midbar'),
        'E70699': ('עֵין מִשְׁפָּט', 'nmpr', 'En Mishpat', 'ein mishpat'),
        'E70700': ('קָדֵשׁ', 'nmpr', 'Kadesh', 'kadeish'),
        'E70701': ('עֲמָלֵקִי', 'adjv', 'Amalekite', 'amaleiki'),
        'E70702': ('חַצְצֹן תָּמָר', 'nmpr', 'Hazazon Tamar', "chats'tson tamar"),
        'E70703': ('ערך', 'verb', 'arrange', 'rch'),
        'E70704': ('בְּאֵר', 'subs', 'well', "b'eir"),
        'E70705': ('נוס', 'verb', 'flee', 'nvs'),
        'E70706': ('אֹכֶל', 'subs', 'food', 'ochel'),
        'E70707': ('פָּלִיט', 'subs', 'escaped one', 'palit'),
        'E70708': ('עִבְרִי', 'adjv', 'Hebrew', 'ivri'),
        'E70709': ('מַמְרֵא', 'nmpr', 'Mamre', 'mamrei'),
        'E70710': ('אֶשְׁכֹּל', 'nmpr', 'Eshcol', 'eshkol'),
        'E70711': ('עָנֵר', 'nmpr', 'Aner', 'aneir'),
        'E70712': ('בַּעַל', 'subs', 'lord, baal', "ba'al"),
        'E70713': ('שׁבה', 'verb', 'take captive', 'shvh'),
        'E70714': ('ריק', 'verb', 'be empty', 'ryk'),
        'E70715': ('חָנִיךְ', 'subs', 'follower', 'chanich'),
        'E70716': ('יָלִיד', 'subs', 'son', 'yalid'),
        'E70717': ('רדף', 'verb', 'pursue', 'rdf'),
        'E70718': ('דָּן', 'nmpr', 'Dan', 'dan'),
        'E70719': ('חלק', 'verb', 'divide', 'chlk'),
        'E70720': ('חֹובָה', 'nmpr', 'Hobah', 'chovva'),
        'E70721': ('דַּמֶּשֶׂק', 'nmpr', 'Damascus', 'damesek'),
        'E70722': ('קרא', 'verb', 'encounter', 'kr'),
        'E70723': ('שָׁוֵה', 'nmpr', 'Shaveh', 'shavei'),
        'E70724': ('מַלְכִּי צֶדֶק', 'nmpr', 'Melchizedek', 'malki tsedek'),
        'E70725': ('שָׁלֵם', 'nmpr', 'Salem', 'shaleim'),
        'E70726': ('כֹּהֵן', 'subs', 'priest', 'kohein'),
        'E70727': ('אֵל', 'subs', 'god', 'eil'),
        'E70728': ('עֶלְיֹון', 'adjv', 'upper', 'elyovn'),
        'E70729': ('מגן', 'verb', 'deliver', 'mgn'),
        'E70730': ('צַר', 'subs', 'adversary', 'tsar'),
        'E70731': ('מַעֲשֵׂר', 'subs', 'tenth', "ma'aseir"),
        'E70732': ('חוּט', 'subs', 'thread', 'chut'),
        'E70733': ('שְׂרֹוךְ', 'subs', 'sandal-thong', "s'rovch"),
        'E70734': ('נַעַל', 'subs', 'sandal', "na'al"),
        'E70735': ('עשׁר', 'verb', 'become rich', 'shr'),
        'E70736': ('בִּלְעֲדֵי', 'prep', 'without', 'biladei'),
        'E70737': ('נַעַר', 'subs', 'boy', "na'ar"),
        'E70738': ('חֵלֶק', 'subs', 'share', 'cheilek'),
        'E70739': ('מַחֲזֶה', 'subs', 'vision', 'machazeh'),
        'E70740': ('מָגֵן', 'subs', 'shield', 'magein'),
        'E70741': ('שָׂכָר', 'subs', 'hire', 'sachor'),
        'E70742': ('אֲדֹנָי', 'nmpr', 'Lord', 'adonay'),
        'E70743': ('עֲרִירִי', 'adjv', 'childless', 'ariri'),
        'E70744': ('מֶשֶׁק', 'subs', '[uncertain]', 'meshek'),
        'E70745': ('אֱלִיעֶזֶר', 'nmpr', 'Eliezer', 'eliezer'),
        'E70746': ('ירשׁ', 'verb', 'trample down', 'yrsh'),
        'E70747': ('מֵעִים', 'subs', 'bowels', 'meiim'),
        'E70748': ('נבט', 'verb', 'look at', 'nvt'),
        'E70749': ('ספר', 'verb', 'count', 'sfr'),
        'E70750': ('כֹּה', 'advb', 'thus', 'ko'),
        'E70751': ('אמן', 'verb', 'be firm', 'mn'),
        'E70752': ('חשׁב', 'verb', 'account', 'chshv'),
        'E70753': ('צְדָקָה', 'subs', 'justice', "ts'daka"),
        'E70754': ('עֶגְלָה', 'subs', 'cow', 'egla'),
        'E70755': ('שׁלשׁ', 'verb', 'be three', 'shlsh'),
        'E70756': ('עֵז', 'subs', 'goat', 'eiz'),
        'E70757': ('אַיִל', 'subs', 'ram, despot', 'ayil'),
        'E70758': ('תֹּור', 'subs', 'dove', 'tovr'),
        'E70759': ('גֹּוזָל', 'subs', 'young of birds', 'govzal'),
        'E70760': ('בתר', 'verb', 'cut', 'vtr'),
        'E70761': ('בֶּתֶר', 'subs', 'piece', 'beter'),
        'E70762': ('עַיִט', 'subs', 'bird of prey', 'ayit'),
        'E70763': ('פֶּגֶר', 'subs', 'corpse', 'peger'),
        'E70764': ('נשׁב', 'verb', 'blow', 'nshv'),
        'E70765': ('שֶׁמֶשׁ', 'subs', 'sun', 'shemesh'),
        'E70766': ('אֵימָה', 'subs', 'fright', 'eima'),
        'E70767': ('חֲשֵׁכָה', 'subs', 'darkness', 'chasheicho'),
        'E70768': ('גֵּר', 'subs', 'sojourner', 'geir'),
        'E70769': ('ענה', 'verb', 'be lowly', 'nh'),
        'E70770': ('דין', 'verb', 'judge', 'dyn'),
        'E70771': ('שָׁלֹום', 'subs', 'peace', 'shalovm'),
        'E70772': ('קבר', 'verb', 'bury', 'kvr'),
        'E70773': ('שֵׂיבָה', 'subs', 'age', 'seiva'),
        'E70774': ('הֵנָּה', 'advb', 'here', 'heina'),
        'E70775': ('שָׁלֵם', 'adjv', 'complete', 'shaleim'),
        'E70776': ('עֲלָטָה', 'subs', 'darkness', 'alata'),
        'E70777': ('תַּנּוּר', 'subs', 'furnace', 'tanur'),
        'E70778': ('עָשָׁן', 'subs', 'smoke', 'ashan'),
        'E70779': ('לַפִּיד', 'subs', 'torch', 'lapid'),
        'E70780': ('אֵשׁ', 'subs', 'fire', 'eish'),
        'E70781': ('גֶּזֶר', 'subs', 'piece', 'gezer'),
        'E70782': ('קֵינִי', 'adjv', 'Kenite', 'keini'),
        'E70783': ('קְנִזִּי', 'adjv', 'Kenizzite', "k'nizi"),
        'E70784': ('קַדְמֹנִי', 'adjv', 'easterner', 'kadmoni'),
        'E70785': ('חִתִּי', 'adjv', 'Hittite', 'chiti'),
        'E70786': ('הָגָר', 'nmpr', 'Hagar', 'hagar'),
        'E70787': ('עצר', 'verb', 'restrain', 'tsr'),
        'E70788': ('אוּלַי', 'advb', 'perhaps', 'ulay'),
        'E70789': ('גְּבֶרֶת', 'subs', 'lady', "g'veret"),
        'E70790': ('חֵיק', 'subs', 'lap', 'cheik'),
        'E70791': ('שׁפט', 'verb', 'judge', 'shft'),
        'E70792': ('ברח', 'verb', 'run away', 'vrch'),
        'E70793': ('מַלְאָךְ', 'subs', 'messenger', 'malach'),
        'E70794': ('שׁוּר', 'nmpr', 'Shur', 'shur'),
        'E70795': ('אָן', 'inrg', 'whither', 'an'),
        'E70796': ('רֹב', 'subs', 'multitude', 'rov'),
        'E70797': ('הָרָה', 'adjv', 'pregnant', 'hara'),
        'E70798': ('יִשְׁמָעֵאל', 'nmpr', 'Ishmael', 'yishmaeil'),
        'E70799': ('עֳנִי', 'subs', 'poverty', 'ani'),
        'E70800': ('פֶּרֶא', 'subs', 'zebra', 'pere'),
        'E70801': ('רֳאִי', 'subs', 'looking', 'rai'),
        'E70802': ('הֲלֹם', 'advb', 'hither', 'halom'),
        'E70803': ('לַחַי רֹאִי', 'nmpr', 'Beer Lahai Roi', 'lachay roi'),
        'E70804': ('בֶּרֶד', 'nmpr', 'Bered', 'bered'),
        'E70805': ('שַׁדַּי', 'nmpr', 'Almighty', 'shaday'),
        'E70806': ('הָמֹון', 'subs', 'commotion', 'hamovn'),
        'E70807': ('אַבְרָהָם', 'nmpr', 'Abraham', 'avraham'),
        'E70808': ('מְגוּרִים', 'subs', 'neighbourhood', "m'gurim"),
        'E70809': ('אֲחֻזָּה', 'subs', 'land property', 'achuza'),
        'E70810': ('מול', 'verb', 'circumcise', 'mvl'),
        'E70811': ('מלל', 'verb', 'circumcise', "ml'l"),
        'E70812': ('עָרְלָה', 'subs', 'foreskin', 'arla'),
        'E70813': ('מִקְנָה', 'subs', 'purchase', 'mikna'),
        'E70814': ('נֵכָר', 'subs', 'foreigner', 'neichor'),
        'E70815': ('עָרֵל', 'adjv', 'uncircumcised', 'areil'),
        'E70816': ('פרר', 'verb', 'break', "fr'r"),
        'E70817': ('שָׂרָה', 'nmpr', 'Sarah', 'sara'),
        'E70818': ('צחק', 'verb', 'laugh', 'tschk'),
        'E70819': ('לוּ', 'conj', 'if only', 'lu'),
        'E70820': ('אֲבָל', 'intj', 'verily', 'aval'),
        'E70821': ('יִצְחָק', 'nmpr', 'Isaac', 'yitschak'),
        'E70822': ('נָשִׂיא', 'subs', 'chief', 'nasi'),
        'E70823': ('חמם', 'verb', 'be hot', "chm'm"),
        'E70824': ('נצב', 'verb', 'stand', 'ntsv'),
        'E70825': ('רוץ', 'verb', 'run', 'rvts'),
        'E70826': ('חוה', 'verb', 'bow down', 'chvh'),
        'E70827': ('מְעַט', 'subs', 'little', "m'at"),
        'E70828': ('רחץ', 'verb', 'wash', 'rchts'),
        'E70829': ('שׁען', 'verb', 'lean', 'shn'),
        'E70830': ('פַּת', 'subs', 'bit', 'pat'),
        'E70831': ('סעד', 'verb', 'support', 'sd'),
        'E70832': ('מהר', 'verb', 'hasten', 'mhr'),
        'E70833': ('סְאָה', 'subs', 'seah', "s'a"),
        'E70834': ('קֶמַח', 'subs', 'flour', 'kemach'),
        'E70835': ('סֹלֶת', 'subs', 'wheat groat', 'solet'),
        'E70836': ('לושׁ', 'verb', 'knead', 'lvsh'),
        'E70837': ('עֻגָה', 'subs', 'cake', 'uga'),
        'E70838': ('רַךְ', 'adjv', 'tender', 'rach'),
        'E70839': ('חֶמְאָה', 'subs', 'butter', 'chema'),
        'E70840': ('חָלָב', 'subs', 'milk', 'chalav'),
        'E70841': ('עמד', 'verb', 'stand', 'md'),
        'E70842': ('אַיֵּה', 'inrg', 'where', 'ayei'),
        'E70843': ('זָקֵן', 'adjv', 'old', 'zakein'),
        'E70844': ('אֹרַח', 'subs', 'path', 'orach'),
        'E70845': ('קֶרֶב', 'subs', 'interior', 'kerev'),
        'E70846': ('בלה', 'verb', 'be worn out', 'vlh'),
        'E70847': ('עֶדְנָה', 'subs', 'delight', 'edna'),
        'E70848': ('אָדֹון', 'subs', 'lord', 'adovn'),
        'E70849': ('זקן', 'verb', 'be old', 'zkn'),
        'E70850': ('אֻמְנָם', 'advb', 'really', 'umnam'),
        'E70851': ('פלא', 'verb', 'be miraculous', 'fl'),
        'E70852': ('כחשׁ', 'verb', 'grow lean', 'chchsh'),
        'E70853': ('שׁקף', 'verb', 'look', 'shkf'),
        'E70854': ('עָצוּם', 'adjv', 'mighty', 'atsum'),
        'E70855': ('מִשְׁפָּט', 'subs', 'justice', 'mishpat'),
        'E70856': ('זְעָקָה', 'subs', 'cry', "z'aka"),
        'E70857': ('כבד', 'verb', 'be heavy', 'chvd'),
        'E70858': ('צְעָקָה', 'subs', 'cry', "ts'aka"),
        'E70859': ('כָּלָה', 'subs', 'destruction', 'kala'),
        'E70860': ('פנה', 'verb', 'turn', 'fnh'),
        'E70861': ('נגשׁ', 'verb', 'approach', 'ngsh'),
        'E70862': ('ספה', 'verb', 'sweep away', 'sfh'),
        'E70863': ('רָשָׁע', 'adjv', 'guilty', 'rasha'),
        'E70864': ('יֵשׁ', 'subs', 'existence', 'yeish'),
        'E70865': ('חָלִילָה', 'intj', 'be it far', 'chalila'),
        'E70866': ('ענה', 'verb', 'answer', 'nh'),
        'E70867': ('יאל', 'verb', 'begin', 'yl'),
        'E70868': ('אֵפֶר', 'subs', 'dust', 'eifer'),
        'E70869': ('עֲשָׂרָה', 'subs', 'ten', 'asara'),
        'E70870': ('שַׁעַר', 'subs', 'gate', "sha'ar"),
        'E70871': ('לין', 'verb', 'lodge', 'lyn'),
        'E70872': ('שׁכם', 'verb', 'rise early', 'shchm'),
        'E70873': ('רְחֹב', 'subs', 'open place', "r'chov"),
        'E70874': ('פצר', 'verb', 'entreat', 'ftsr'),
        'E70875': ('מִשְׁתֶּה', 'subs', 'drinking', 'mishteh'),
        'E70876': ('מַצָּה', 'subs', 'matzah', 'matsa'),
        'E70877': ('אפה', 'verb', 'bake', 'fh'),
        'E70878': ('שׁכב', 'verb', 'lie down', 'shchv'),
        'E70879': ('דֶּלֶת', 'subs', 'door', 'delet'),
        'E70880': ('רעע', 'verb', 'be evil', 'r'),
        'E70881': ('אֵל', 'prde', 'these', 'eil'),
        'E70882': ('צֵל', 'subs', 'shadow', 'tseil'),
        'E70883': ('קֹורָה', 'subs', 'beam', 'kovra'),
        'E70884': ('הָלְאָה', 'advb', 'further', 'hala'),
        'E70885': ('שׁבר', 'verb', 'break', 'shvr'),
        'E70886': ('סַנְוֵרִים', 'subs', 'blindness', 'sanveirim'),
        'E70887': ('לאה', 'verb', 'be weary', 'lh'),
        'E70888': ('פֹּה', 'advb', 'here', 'po'),
        'E70889': ('חָתָן', 'subs', 'son-in-law', 'chatan'),
        'E70890': ('כְּמֹו', 'prep', 'like', "k'mov"),
        'E70891': ('שַׁחַר', 'subs', 'dawn', 'shachar'),
        'E70892': ('אוץ', 'verb', 'urge', 'vts'),
        'E70893': ('מָהַהּ', 'verb', 'tarry', 'mahah'),
        'E70894': ('חזק', 'verb', 'be strong', 'chzk'),
        'E70895': ('חֶמְלָה', 'subs', 'compassion', 'chemla'),
        'E70896': ('מלט', 'verb', 'escape', 'mlt'),
        'E70897': ('חֶסֶד', 'subs', 'loyalty', 'chesed'),
        'E70898': ('קָרֹוב', 'adjv', 'near', 'karovv'),
        'E70899': ('מִצְעָר', 'subs', 'little', 'mitsar'),
        'E70900': ('גָּפְרִית', 'subs', 'sulphur', 'gafrit'),
        'E70901': ('צֶמַח', 'subs', 'sprout', 'tsemach'),
        'E70902': ('נְצִיב', 'subs', 'pillar', "n'tsiv"),
        'E70903': ('קִיטֹור', 'subs', 'smoke', 'kitovr'),
        'E70904': ('כִּבְשָׁן', 'subs', 'kiln', 'kivshan'),
        'E70905': ('הֲפֵכָה', 'subs', 'overthrow', 'hafeicho'),
        'E70906': ('מְעָרָה', 'subs', 'cave', "m'ara"),
        'E70907': ('בְּכִירָה', 'subs', 'first-born woman', "b'chira"),
        'E70908': ('צָעִיר', 'adjv', 'little', 'tsair'),
        'E70909': ('מָחֳרָת', 'subs', 'next day', 'macharat'),
        'E70910': ('אֶמֶשׁ', 'subs', 'yesterday evening', 'emesh'),
        'E70911': ('מֹואָב', 'nmpr', 'Moab', 'movav'),
        'E70912': ('בֶּן עַמִּי', 'nmpr', 'Ben-Ammi', 'ben ami'),
        'E70913': ('עַמֹּון', 'nmpr', 'Ammon', 'amovn'),
        'E70914': ('אֲבִימֶלֶךְ', 'nmpr', 'Abimelech', 'avimelech'),
        'E70915': ('חֲלֹום', 'subs', 'dream', 'chalovm'),
        'E70916': ('בעל', 'verb', 'own', 'vl'),
        'E70917': ('תֹּם', 'subs', 'completeness', 'tom'),
        'E70918': ('לֵבָב', 'subs', 'heart', 'leivav'),
        'E70919': ('נִקָּיֹון', 'subs', 'innocence', 'nikayovn'),
        'E70920': ('חשׂך', 'verb', 'withhold', 'chsch'),
        'E70921': ('חטא', 'verb', 'miss', 'cht'),
        'E70922': ('נָבִיא', 'subs', 'prophet', 'navi'),
        'E70923': ('פלל', 'verb', 'pray', "fl'l"),
        'E70924': ('אֹזֶן', 'subs', 'ear', 'ozen'),
        'E70925': ('חֲטָאָה', 'subs', 'sin', "chata'a"),
        'E70926': ('יִרְאָה', 'subs', 'fear', 'yira'),
        'E70927': ('אָמְנָה', 'advb', 'indeed', 'amna'),
        'E70928': ('תעה', 'verb', 'err', 'th'),
        'E70929': ('אֶלֶף', 'subs', 'thousand', 'elef'),
        'E70930': ('כְּסוּת', 'subs', 'covering', "k'sut"),
        'E70931': ('יכח', 'verb', 'reprove', 'ychch'),
        'E70932': ('רפא', 'verb', 'heal', 'rf'),
        'E70933': ('אָמָה', 'subs', 'handmaid', 'ama'),
        'E70934': ('רֶחֶם', 'subs', 'womb', 'rechem'),
        'E70935': ('פקד', 'verb', 'miss', 'fkd'),
        'E70936': ('זְקֻנִים', 'subs', 'old age', "z'kunim"),
        'E70937': ('צְחֹק', 'subs', 'laughter', "ts'chok"),
        'E70938': ('מלל', 'verb', 'make signs', "ml'l"),
        'E70939': ('ינק', 'verb', 'suck', 'ynk'),
        'E70940': ('גמל', 'verb', 'deal fully', 'gml'),
        'E70941': ('אֹודֹות', 'subs', 'inducement', 'ovdovt'),
        'E70942': ('חֵמֶת', 'subs', 'water-skin', 'cheimet'),
        'E70943': ('שֶׁבַע', 'nmpr', 'Sheba', 'sheva'),
        'E70944': ('שׁלך', 'verb', 'throw', 'shlch'),
        'E70945': ('רחק', 'verb', 'be far', 'rchk'),
        'E70946': ('טחה', 'verb', 'shoot', "t'chh"),
        'E70947': ('מָוֶת', 'subs', 'death', 'mavet'),
        'E70948': ('בכה', 'verb', 'weep', 'vchh'),
        'E70949': ('רבה', 'verb', 'shoot', 'rvh'),
        'E70950': ('קַשָּׁת', 'subs', 'bowman', 'kashat'),
        'E70951': ('פָּארָן', 'nmpr', 'Paran', 'paran'),
        'E70952': ('פִּיכֹל', 'nmpr', 'Phicol', 'pichol'),
        'E70953': ('שׁבע', 'verb', 'swear', 'shv'),
        'E70954': ('שׁקר', 'verb', 'do falsely', 'shkr'),
        'E70955': ('נִין', 'subs', 'offspring', 'nin'),
        'E70956': ('נֶכֶד', 'subs', 'progeny', 'neched'),
        'E70957': ('גזל', 'verb', 'tear away', 'gzl'),
        'E70958': ('כִּבְשָׂה', 'subs', 'young ewe-lamb', 'kivsa'),
        'E70959': ('עֵדָה', 'subs', 'witness', 'eida'),
        'E70960': ('חפר', 'verb', 'dig', 'chfr'),
        'E70961': ('אֵשֶׁל', 'subs', 'tamarisk', 'eishel'),
        'E70962': ('נסה', 'verb', 'try', "ns'h"),
        'E70963': ('יָחִיד', 'adjv', 'only one', 'yachid'),
        'E70964': ('אהב', 'verb', 'love', 'hv'),
        'E70965': ('מֹרִיָּה', 'nmpr', 'Moriah', 'moria'),
        'E70966': ('חבשׁ', 'verb', 'saddle', 'chvsh'),
        'E70967': ('רָחֹוק', 'adjv', 'remote', 'rachovk'),
        'E70968': ('מַאֲכֶלֶת', 'subs', 'knife', "ma'achelet"),
        'E70969': ('שֶׂה', 'subs', 'lamb', 'seh'),
        'E70970': ('עקד', 'verb', 'bind', 'kd'),
        'E70971': ('שׁחט', 'verb', 'slaughter', 'shcht'),
        'E70972': ('מְאוּמָה', 'subs', 'something', "m'uma"),
        'E70973': ('יָרֵא', 'adjv', 'afraid', 'yarei'),
        'E70974': ('אחז', 'verb', 'seize', 'chz'),
        'E70975': ('סְבַךְ', 'subs', 'thicket', "s'vach"),
        'E70976': ('קֶרֶן', 'subs', 'horn', 'keren'),
        'E70977': ('נְאֻם', 'subs', 'speech', "n'um"),
        'E70978': ('יַעַן', 'subs', 'motive', "ya'an"),
        'E70979': ('חֹול', 'subs', 'sand', 'chovl'),
        'E70980': ('איב', 'verb', 'be hostile', 'yv'),
        'E70981': ('עֵקֶב', 'subs', 'end', 'eikev'),
        'E70982': ('בּוּז', 'nmpr', 'Buz', 'buz'),
        'E70983': ('קְמוּאֵל', 'nmpr', 'Kemuel', "k'mueil"),
        'E70984': ('כֶּשֶׂד', 'nmpr', 'Kesed', 'kesed'),
        'E70985': ('חֲזֹו', 'nmpr', 'Hazo', 'chazov'),
        'E70986': ('פִּלְדָּשׁ', 'nmpr', 'Pildash', 'pildash'),
        'E70987': ('יִדְלָף', 'nmpr', 'Jidlaph', 'yidlaf'),
        'E70988': ('בְּתוּאֵל', 'nmpr', '[father of Laban]', "b'tueil"),
        'E70989': ('רִבְקָה', 'nmpr', 'Rebekah', 'rivka'),
        'E70990': ('פִּלֶגֶשׁ', 'subs', 'concubine', 'pilegesh'),
        'E70991': ('רְאוּמָה', 'nmpr', 'Reumah', "r'uma"),
        'E70992': ('טֶבַח', 'nmpr', 'Tebah', 'tevach'),
        'E70993': ('גַּחַם', 'nmpr', 'Gaham', 'gacham'),
        'E70994': ('תַּחַשׁ', 'nmpr', 'Tahash', 'tachash'),
        'E70995': ('מַעֲכָה', 'nmpr', 'Maacah', "ma'acho"),
        'E70996': ('קִרְיַת אַרְבַּע', 'nmpr', 'Kiriath Arba', 'kirat arba'),
        'E70997': ('ספד', 'verb', 'lament', 'sfd'),
        'E70998': ('תֹּושָׁב', 'subs', 'sojourner', 'tovshav'),
        'E70999': ('קֶבֶר', 'subs', 'grave', 'kever'),
        'E71000': ('מִבְחָר', 'subs', 'choice', 'mivchar'),
        'E71001': ('פגע', 'verb', 'meet', 'fg'),
        'E71002': ('עֶפְרֹון', 'nmpr', 'Ephron', 'efrovn'),
        'E71003': ('צֹחַר', 'nmpr', 'Zohar', 'tsochar'),
        'E71004': ('מַכְפֵּלָה', 'nmpr', 'Machpelah', 'machpeila'),
        'E71005': ('מָלֵא', 'adjv', 'full', 'malei'),
        'E71006': ('שֶׁקֶל', 'subs', 'shekel', 'shekel'),
        'E71007': ('שׁקל', 'verb', 'weigh', 'shkl'),
        'E71008': ('סחר', 'verb', 'go about', "s'chr"),
        'E71009': ('סָבִיב', 'subs', 'surrounding', 'saviv'),
        'E71010': ('יָרֵךְ', 'subs', 'upper thigh', 'yareich'),
        'E71011': ('אבה', 'verb', 'want', 'vh'),
        'E71012': ('נקה', 'verb', 'be clean', 'nkh'),
        'E71013': ('שְׁבוּעָה', 'subs', 'oath', "sh'vua"),
        'E71014': ('טוּב', 'subs', 'best', 'tuv'),
        'E71015': ('אֲרַם נַהֲרַיִם', 'nmpr', 'Mesopotamia', 'aram naharayim'),
        'E71016': ('ברך', 'verb', 'kneel', 'vrch'),
        'E71017': ('שׁאב', 'verb', 'draw water', 'shv'),
        'E71018': ('קרה', 'verb', 'meet', 'krh'),
        'E71019': ('נַעֲרָה', 'subs', 'girl', "na'ara"),
        'E71020': ('כַּד', 'subs', 'pitcher', 'kad'),
        'E71021': ('בְּתוּלָה', 'subs', 'virgin', "b'tula"),
        'E71022': ('גמא', 'verb', 'swallow', 'gm'),
        'E71023': ('ערה', 'verb', 'pour out', 'rh'),
        'E71024': ('שֹׁקֶת', 'subs', 'watering trough', 'shoket'),
        'E71025': ('שׁאה', 'verb', 'gaze', 'shh'),
        'E71026': ('חרשׁ', 'verb', 'be deaf', 'chrsh'),
        'E71027': ('צלח', 'verb', 'be strong', 'tslch'),
        'E71028': ('נֶזֶם', 'subs', 'nose-ring', 'nezem'),
        'E71029': ('בֶּקַע', 'subs', 'half-shekel', 'beka'),
        'E71030': ('מִשְׁקָל', 'subs', 'weight', 'mishkal'),
        'E71031': ('צָמִיד', 'subs', 'bracelet', 'tsamid'),
        'E71032': ('תֶּבֶן', 'subs', 'straw', 'teven'),
        'E71033': ('מִסְפֹּוא', 'subs', 'fodder', 'mispov'),
        'E71034': ('קדד', 'verb', 'kneel down', "kd'd"),
        'E71035': ('אֶמֶת', 'subs', 'trustworthiness', 'emet'),
        'E71036': ('נחה', 'verb', 'lead', 'nchh'),
        'E71037': ('לָבָן', 'nmpr', 'Laban', 'lavan'),
        'E71038': ('זִקְנָה', 'subs', 'old age', 'zikna'),
        'E71039': ('מִשְׁפַּחַת', 'subs', 'clan', 'mishpachat'),
        'E71040': ('אָלָה', 'subs', 'curse', 'ala'),
        'E71041': ('נָקִי', 'adjv', 'innocent', 'naki'),
        'E71042': ('עַלְמָה', 'subs', 'young woman', 'alma'),
        'E71043': ('שׁאל', 'verb', 'ask', 'shl'),
        'E71044': ('אֹו', 'conj', 'or', 'ov'),
        'E71045': ('כְּלִי', 'subs', 'tool', "k'li"),
        'E71046': ('בֶּגֶד', 'subs', 'garment', 'beged'),
        'E71047': ('מִגְדָּנֹות', 'subs', 'presents', 'migdanovt'),
        'E71048': ('עָשֹׂור', 'subs', 'a ten', 'asovr'),
        'E71049': ('אחר', 'verb', 'be behind', 'chr'),
        'E71050': ('מֵינֶקֶת', 'subs', 'nursing woman', 'meineket'),
        'E71051': ('רְבָבָה', 'subs', 'multitude', "r'vava"),
        'E71052': ('שׂנא', 'verb', 'hate', 'sn'),
        'E71053': ('רכב', 'verb', 'ride', 'rchv'),
        'E71054': ('שׂוח', 'verb', '[uncertain]', 'svch'),
        'E71055': ('לָזֶה', 'prde', 'this there', 'lazeh'),
        'E71056': ('צָעִיף', 'subs', 'shawl', 'tsaif'),
        'E71057': ('קְטוּרָה', 'nmpr', 'Keturah', "k'tura"),
        'E71058': ('זִמְרָן', 'nmpr', 'Zimran', 'zimran'),
        'E71059': ('יָקְשָׁן', 'nmpr', 'Jokshan', 'yakshan'),
        'E71060': ('מְדָן', 'nmpr', 'Medan', "m'dan"),
        'E71061': ('מִדְיָן', 'nmpr', 'Midian', 'midan'),
        'E71062': ('יִשְׁבָּק', 'nmpr', 'Ishbak', 'yishbak'),
        'E71063': ('שׁוּחַ', 'nmpr', 'Shuah', 'shuach'),
        'E71064': ('אַשּׁוּרִים', 'nmpr', 'Asshurites', 'ashurim'),
        'E71065': ('לְטוּשִׁים', 'nmpr', 'Letushites', "l'tushim"),
        'E71066': ('לְאֻמִּים', 'nmpr', 'Leummites', "l'umim"),
        'E71067': ('עֵיפָה', 'nmpr', 'Ephah', 'eifa'),
        'E71068': ('עֵפֶר', 'nmpr', 'Epher', 'eifer'),
        'E71069': ('אֲבִידָע', 'nmpr', 'Abida', 'avida'),
        'E71070': ('אֶלְדָּעָה', 'nmpr', 'Eldaah', "elda'a"),
        'E71071': ('מַתָּנָה', 'subs', 'present', 'matana'),
        'E71072': ('שָׂבֵעַ', 'adjv', 'sated', 'saveia'),
        'E71073': ('קנה', 'verb', 'buy', 'knh'),
        'E71074': ('נְבָיֹות', 'nmpr', 'Nebaioth', "n'vayovt"),
        'E71075': ('קֵדָר', 'nmpr', 'Kedar', 'keidar'),
        'E71076': ('אַדְבְּאֵל', 'nmpr', 'Abdeel', 'adbeil'),
        'E71077': ('מִבְשָׂם', 'nmpr', 'Mibsam', 'mivsam'),
        'E71078': ('מִשְׁמָע', 'nmpr', 'Mishma', 'mishma'),
        'E71079': ('דּוּמָה', 'nmpr', 'Dumah', 'duma'),
        'E71080': ('מַשָּׂא', 'nmpr', 'Massa', 'masa'),
        'E71081': ('חֲדַד', 'nmpr', 'Hadad', 'chadad'),
        'E71082': ('תֵּימָא', 'nmpr', 'Tema', 'teima'),
        'E71083': ('יְטוּר', 'nmpr', 'Jetur', "y'tur"),
        'E71084': ('נָפִישׁ', 'nmpr', 'Naphish', 'nafish'),
        'E71085': ('קֵדְמָה', 'nmpr', 'Kedemah', 'keidma'),
        'E71086': ('חָצֵר', 'subs', 'court', 'chatseir'),
        'E71087': ('טִירָה', 'subs', 'encampment', 'tira'),
        'E71088': ('אֻמָּה', 'subs', 'clan', 'uma'),
        'E71089': ('אֲרַמִּי', 'adjv', 'Aramean', 'arami'),
        'E71090': ('פַּדָּן', 'subs', '[paddan], field?', 'padan'),
        'E71091': ('עתר', 'verb', 'entreat', 'tr'),
        'E71092': ('נֹכַח', 'subs', 'straightness', 'nochach'),
        'E71093': ('רצץ', 'verb', 'crush', "rts'ts"),
        'E71094': ('בֶּטֶן', 'subs', 'belly', 'beten'),
        'E71095': ('לְאֹם', 'subs', 'people', "l'om"),
        'E71096': ('אמץ', 'verb', 'be strong', 'mts'),
        'E71097': ('תֹּואֲמִם', 'subs', 'twins', 'tovamim'),
        'E71098': ('אַדְמֹנִי', 'adjv', 'red', 'admoni'),
        'E71099': ('אַדֶּרֶת', 'subs', 'splendour', 'aderet'),
        'E71100': ('שֵׂעָר', 'subs', 'hair', 'seiar'),
        'E71101': ('עֵשָׂו', 'nmpr', 'Esau', 'eisav'),
        'E71102': ('יַעֲקֹב', 'nmpr', 'Jacob', "ya'akov"),
        'E71103': ('תָּם', 'adjv', 'complete', 'tam'),
        'E71104': ('זיד', 'verb', 'be presumptuous', 'zyd'),
        'E71105': ('נָזִיד', 'subs', 'boiled food', 'nazid'),
        'E71106': ('עָיֵף', 'adjv', 'faint', 'ayeif'),
        'E71107': ('לעט', 'verb', 'swallow', 'lt'),
        'E71108': ('אָדֹם', 'adjv', 'red', 'adom'),
        'E71109': ('אֱדֹום', 'nmpr', 'Edom', 'edovm'),
        'E71110': ('מכר', 'verb', 'sell', 'mchr'),
        'E71111': ('בְּכֹרָה', 'subs', 'right of first-born', "b'chora"),
        'E71112': ('עֲדָשָׁה', 'subs', 'lentil', 'adasha'),
        'E71113': ('בזה', 'verb', 'despise', 'vzh'),
        'E71114': ('מִשְׁמֶרֶת', 'subs', 'guard-post', 'mishmeret'),
        'E71115': ('מִצְוָה', 'subs', 'commandment', 'mitsva'),
        'E71116': ('חֻקָּה', 'subs', 'regulation', 'chuka'),
        'E71117': ('תֹּורָה', 'subs', 'instruction', 'tovra'),
        'E71118': ('ארך', 'verb', 'be long', 'rch'),
        'E71119': ('אֵיךְ', 'inrg', 'how', 'eich'),
        'E71120': ('אָשָׁם', 'subs', 'guilt', 'asham'),
        'E71121': ('שַׁעַר', 'subs', 'measure', "sha'ar"),
        'E71122': ('גָּדֵל', 'adjv', 'growing', 'gadeil'),
        'E71123': ('עֲבֻדָּה', 'subs', 'slaves', 'avuda'),
        'E71124': ('קנא', 'verb', 'be jealous', 'kn'),
        'E71125': ('סתם', 'verb', 'stop up', 'stm'),
        'E71126': ('עצם', 'verb', 'be mighty', 'tsm'),
        'E71127': ('חנה', 'verb', 'encamp', 'chnh'),
        'E71128': ('נַחַל', 'subs', 'wadi', 'nachal'),
        'E71129': ('ריב', 'verb', 'contend', 'ryv'),
        'E71130': ('עֵשֶׂק', 'nmpr', 'Esek', 'eisek'),
        'E71131': ('עשׂק', 'verb', 'contend', 'sk'),
        'E71132': ('שִׂטְנָה', 'nmpr', 'Sitnah', 'sitna'),
        'E71133': ('רְחֹבֹות', 'nmpr', 'Rehoboth', "r'chovovt"),
        'E71134': ('רחב', 'verb', 'be wide', 'rchv'),
        'E71135': ('כרה', 'verb', 'dig', 'chrh'),
        'E71136': ('אֲחֻזַּת', 'nmpr', 'Ahuzzath', 'achuzat'),
        'E71137': ('מֵרֵעַ', 'subs', 'friend', 'meireia'),
        'E71138': ('מַדּוּעַ', 'inrg', 'why', 'madua'),
        'E71139': ('שִׁבְעָה', 'nmpr', 'Shibah', 'shiva'),
        'E71140': ('יְהוּדִית', 'nmpr', 'Judith', "y'hudit"),
        'E71141': ('בְּאֵרִי', 'nmpr', 'Beeri', "b'eiri"),
        'E71142': ('בָּשְׂמַת', 'nmpr', 'Basemath', 'basmat'),
        'E71143': ('אֵילֹון', 'nmpr', '[judge]', 'eilovn'),
        'E71144': ('מֹרָה', 'subs', 'bitterness', 'mora'),
        'E71145': ('כהה', 'verb', 'grow dim', "chh'h"),
        'E71146': ('תְּלִי', 'subs', 'quiverbelt', "t'li"),
        'E71147': ('צוד', 'verb', 'hunt', 'tsvd'),
        'E71148': ('מַטְעַמִּים', 'subs', 'savoury food', 'matamim'),
        'E71149': ('גְּדִי', 'subs', 'goat', "g'di"),
        'E71150': ('שָׂעִיר', 'adjv', 'hairy', 'sair'),
        'E71151': ('חָלָק', 'adjv', 'smooth', 'chalak'),
        'E71152': ('משׁשׁ', 'verb', 'grope', "msh'sh"),
        'E71153': ('תעע', 'verb', 'mock', 't'),
        'E71154': ('קְלָלָה', 'subs', 'curse', "k'lala"),
        'E71155': ('חֲמֻדֹות', 'subs', 'desirables', 'chamudovt'),
        'E71156': ('חֶלְקָה', 'subs', 'smoothness', 'chelka'),
        'E71157': ('צַוָּאר', 'subs', 'neck', 'tsauar'),
        'E71158': ('מושׁ', 'verb', 'touch', 'mvsh'),
        'E71159': ('נכר', 'verb', 'recognise', 'nchr'),
        'E71160': ('נשׁק', 'verb', 'kiss', 'nshk'),
        'E71161': ('טַל', 'subs', 'dew', 'tal'),
        'E71162': ('שָׁמָן', 'adjv', 'fat', 'shaman'),
        'E71163': ('דָּגָן', 'subs', 'corn', 'dagan'),
        'E71164': ('תִּירֹושׁ', 'subs', 'wine', 'tirovsh'),
        'E71165': ('הוה', 'verb', 'become', 'hvh'),
        'E71166': ('גְּבִיר', 'subs', 'lord', "g'vir"),
        'E71167': ('חרד', 'verb', 'tremble', 'chrd'),
        'E71168': ('חֲרָדָה', 'subs', 'trembling', 'charada'),
        'E71169': ('אֵפֹו', 'intj', 'then', 'eifov'),
        'E71170': ('מַר', 'adjv', 'bitter', 'mar'),
        'E71171': ('מִרְמָה', 'subs', 'deceit', 'mirma'),
        'E71172': ('עקב', 'verb', 'seize at heel', 'kv'),
        'E71173': ('אצל', 'verb', 'set aside', 'tsl'),
        'E71174': ('סמך', 'verb', 'support', 'smch'),
        'E71175': ('עַל', 'subs', 'height', 'al'),
        'E71176': ('רדד', 'verb', 'subdue', "rd'd"),
        'E71177': ('פרק', 'verb', 'tear away', 'frk'),
        'E71178': ('עֹל', 'subs', 'yoke', 'ol'),
        'E71179': ('שׂטם', 'verb', 'bear grudge', 'stm'),
        'E71180': ('אֵבֶל', 'subs', 'mourning rites', 'eivel'),
        'E71181': ('חֵמָה', 'subs', 'heat', 'cheima'),
        'E71182': ('שׁכח', 'verb', 'forget', 'shchch'),
        'E71183': ('שׁכל', 'verb', 'be bereaved of children', 'shchl'),
        'E71184': ('קוץ', 'verb', 'loath', 'kvts'),
        'E71185': ('קָהָל', 'subs', 'assembly', 'kahal'),
        'E71186': ('מַחֲלַת', 'nmpr', 'Mahalath', 'machalat'),
        'E71187': ('מְרַאֲשֹׁות', 'subs', 'head-place', "m'ra'ashovt"),
        'E71188': ('חלם', 'verb', 'dream', 'chlm'),
        'E71189': ('סֻלָּם', 'subs', 'stair', 'sulam'),
        'E71190': ('פרץ', 'verb', 'break', 'frts'),
        'E71191': ('שֵׁנָה', 'subs', 'sleep', 'sheina'),
        'E71192': ('אָכֵן', 'intj', 'surely', 'achein'),
        'E71193': ('מַצֵּבָה', 'subs', 'massebe', 'matseiva'),
        'E71194': ('יצק', 'verb', 'pour', 'ytsk'),
        'E71195': ('שֶׁמֶן', 'subs', 'oil', 'shemen'),
        'E71196': ('אוּלָם', 'advb', 'but', 'ulam'),
        'E71197': ('לוּז', 'nmpr', 'Luz', 'luz'),
        'E71198': ('נדר', 'verb', 'vow', 'ndr'),
        'E71199': ('נֶדֶר', 'subs', 'vow', 'neder'),
        'E71200': ('עשׂר', 'verb', 'take tenth', 'sr'),
        'E71201': ('עֵדֶר', 'subs', 'flock', 'eider'),
        'E71202': ('גלל', 'verb', 'roll', "gl'l"),
        'E71203': ('אַיִן', 'inrg', 'whence', 'ayin'),
        'E71204': ('רָחֵל', 'nmpr', 'Rachel', 'racheil'),
        'E71205': ('שֵׁמַע', 'subs', 'hearsay', 'sheima'),
        'E71206': ('חבק', 'verb', 'embrace', 'chvk'),
        'E71207': ('חִנָּם', 'advb', 'in vain', 'chinam'),
        'E71208': ('מַשְׂכֹּרֶת', 'subs', 'wage', 'maskoret'),
        'E71209': ('לֵאָה', 'nmpr', 'Leah', 'leia'),
        'E71210': ('תֹּאַר', 'subs', 'form', 'toar'),
        'E71211': ('אֲהָבָה', 'subs', 'love', 'ahava'),
        'E71212': ('זִלְפָּה', 'nmpr', 'Zilpah', 'zilpa'),
        'E71213': ('רמה', 'verb', 'deceive', 'rmh'),
        'E71214': ('שָׁבוּעַ', 'subs', 'week', 'shavua'),
        'E71215': ('עֲבֹדָה', 'subs', 'work', 'avoda'),
        'E71216': ('בִּלְהָה', 'nmpr', '[mother of Dan Naphtali]', 'bilha'),
        'E71217': ('רְאוּבֵן', 'nmpr', 'Reuben', "r'uvein"),
        'E71218': ('שִׁמְעֹון', 'nmpr', 'Simeon', 'shimovn'),
        'E71219': ('לוה', 'verb', 'accompany', 'lvh'),
        'E71220': ('לֵוִי', 'nmpr', 'Levi', 'leivi'),
        'E71221': ('ידה', 'verb', 'praise', 'ydh'),
        'E71222': ('יְהוּדָה', 'nmpr', 'Judah', "y'huda"),
        'E71223': ('מנע', 'verb', 'withhold', 'mn'),
        'E71224': ('בֶּרֶךְ', 'subs', 'knee', 'berech'),
        'E71225': ('נַפְתּוּלִים', 'subs', 'wrestling', 'naftulim'),
        'E71226': ('פתל', 'verb', 'twist', 'ftl'),
        'E71227': ('נַפְתָּלִי', 'nmpr', 'Naphtali', 'naftali'),
        'E71228': ('גַּד', 'subs', 'fortune', 'gad'),
        'E71229': ('גָּד', 'nmpr', 'Gad', 'gad'),
        'E71230': ('אֹשֶׁר', 'subs', 'happiness', 'osher'),
        'E71231': ('אשׁר', 'verb', 'be happy', 'shr'),
        'E71232': ('אָשֵׁר', 'nmpr', 'Asher', 'asheir'),
        'E71233': ('חִטָּה', 'subs', 'wheat', 'chita'),
        'E71234': ('דּוּדַי', 'subs', 'mandrake', 'duday'),
        'E71235': ('שׂכר', 'verb', 'hire', 'schr'),
        'E71236': ('יִשָּׂשׂכָר', 'nmpr', 'Issachar', 'yisaschor'),
        'E71237': ('זבד', 'verb', 'bestow', 'zvd'),
        'E71238': ('זֵבֶד', 'subs', 'endowment', 'zeived'),
        'E71239': ('זבל', 'verb', 'tolerate', 'zvl'),
        'E71240': ('זְבוּלֻן', 'nmpr', 'Zebulun', "z'vulun"),
        'E71241': ('דִּינָה', 'nmpr', 'Dinah', 'dina'),
        'E71242': ('חֶרְפָּה', 'subs', 'reproach', 'cherpa'),
        'E71243': ('יֹוסֵף', 'nmpr', 'Joseph', 'yovseif'),
        'E71244': ('נחשׁ', 'verb', 'divine', 'nchsh'),
        'E71245': ('נקב', 'verb', 'bore', 'nkv'),
        'E71246': ('מָתַי', 'inrg', 'when', 'matay'),
        'E71247': ('נָקֹד', 'adjv', 'speckled', 'nakod'),
        'E71248': ('טלא', 'verb', 'patch', 'tl'),
        'E71249': ('חוּם', 'adjv', 'ruttish', 'chum'),
        'E71250': ('כֶּשֶׂב', 'subs', 'young ram', 'kesev'),
        'E71251': ('מָחָר', 'subs', 'next day', 'machar'),
        'E71252': ('גנב', 'verb', 'steal', 'gnv'),
        'E71253': ('תַּיִשׁ', 'subs', 'he-goat', 'tayish'),
        'E71254': ('עָקֹד', 'adjv', 'twisted', 'akod'),
        'E71255': ('לָבָן', 'adjv', 'white', 'lavan'),
        'E71256': ('יתר', 'verb', 'remain', 'ytr'),
        'E71257': ('מַקֵּל', 'subs', 'rod', 'makeil'),
        'E71258': ('לִבְנֶה', 'subs', 'storax tree', 'livneh'),
        'E71259': ('לַח', 'adjv', 'fresh', 'lach'),
        'E71260': ('לוּז', 'subs', 'almond tree', 'luz'),
        'E71261': ('עֶרְמֹון', 'subs', 'plane-tree', 'ermovn'),
        'E71262': ('פצל', 'verb', 'peel', 'ftsl'),
        'E71263': ('פְּצָלָה', 'subs', 'peeled stripe', "p'tsala"),
        'E71264': ('מַחְשֹׂף', 'subs', 'stripping', 'machsof'),
        'E71265': ('יצג', 'verb', 'set', 'ytsg'),
        'E71266': ('רַהַט', 'subs', 'trough', 'rahat'),
        'E71267': ('יחם', 'verb', 'be hot', 'ychm'),
        'E71268': ('קשׁר', 'verb', 'tie', 'kshr'),
        'E71269': ('עטף', 'verb', 'faint', 'tf'),
        'E71270': ('עָטוּף', 'adjv', 'feeble', 'atuf'),
        'E71271': ('כָּבֹוד', 'subs', 'weight', 'kavovd'),
        'E71272': ('תְּמֹול', 'subs', 'yesterday', "t'movl"),
        'E71273': ('שִׁלְשֹׁום', 'subs', 'day before yesterday', 'shilshovm'),
        'E71274': ('אַתֵּנָה', 'prps', 'you', 'ateina'),
        'E71275': ('תלל', 'verb', 'mock', "tl'l"),
        'E71276': ('חלף', 'verb', 'come after', 'chlf'),
        'E71277': ('מֹנֶה', 'subs', 'portion', 'moneh'),
        'E71278': ('נצל', 'verb', 'deliver', 'ntsl'),
        'E71279': ('עַתּוּד', 'subs', 'ram', 'atud'),
        'E71280': ('בָּרֹד', 'adjv', 'speckled', 'barod'),
        'E71281': ('משׁח', 'verb', 'smear', 'mshch'),
        'E71282': ('נַחֲלָה', 'subs', 'heritage', 'nachala'),
        'E71283': ('נָכְרִי', 'adjv', 'foreign', 'nachri'),
        'E71284': ('עֹשֶׁר', 'subs', 'riches', 'osher'),
        'E71285': ('נהג', 'verb', 'drive', 'nhg'),
        'E71286': ('קִנְיָן', 'subs', 'property', 'kinan'),
        'E71287': ('גזז', 'verb', 'shear', "gz'z"),
        'E71288': ('תְּרָפִים', 'subs', 'teraphim', "t'rafim"),
        'E71289': ('בְּלִי', 'subs', 'destruction', "b'li"),
        'E71290': ('גִּלְעָד', 'nmpr', 'Gilead', 'gilad'),
        'E71291': ('נשׂג', 'verb', 'overtake', 'nsg'),
        'E71292': ('תקע', 'verb', 'blow', 'tk'),
        'E71293': ('שִׂמְחָה', 'subs', 'joy', 'simcha'),
        'E71294': ('שִׁיר', 'subs', 'song', 'shir'),
        'E71295': ('תֹּף', 'subs', 'tambourine', 'tof'),
        'E71296': ('נטשׁ', 'verb', 'abandon', 'ntsh'),
        'E71297': ('סכל', 'verb', 'be foolish', "s'chl"),
        'E71298': ('אֵל', 'subs', 'power', 'eil'),
        'E71299': ('כסף', 'verb', 'long', 'chsf'),
        'E71300': ('כַּר', 'subs', 'basket-saddle', 'kar'),
        'E71301': ('חפשׂ', 'verb', 'search', 'chfs'),
        'E71302': ('פֶּשַׁע', 'subs', 'rebellion', 'pesha'),
        'E71303': ('דלק', 'verb', 'set ablaze', 'dlk'),
        'E71304': ('רָחֵל', 'subs', 'ewe', 'racheil'),
        'E71305': ('טְרֵיפָה', 'subs', 'prey', "t'reifa"),
        'E71306': ('בקשׁ', 'verb', 'seek', 'vksh'),
        'E71307': ('חֹרֶב', 'subs', 'dryness', 'chorev'),
        'E71308': ('קֶרַח', 'subs', 'frost', 'kerach'),
        'E71309': ('נדד', 'verb', 'flee', "nd'd"),
        'E71310': ('לוּלֵא', 'conj', 'unless', 'lulei'),
        'E71311': ('פַּחַד', 'subs', 'trembling', 'pachad'),
        'E71312': ('רֵיקָם', 'advb', 'with empty hands', 'reikam'),
        'E71313': ('יְגִיעַ', 'subs', 'toil', "y'gia"),
        'E71314': ('עֵד', 'subs', 'witness', 'eid'),
        'E71315': ('לקט', 'verb', 'gather', 'lkt'),
        'E71316': ('גַּל', 'subs', 'heap', 'gal'),
        'E71317': ('יְגַר', 'subs', 'heap of stones', "y'gar"),
        'E71318': ('שָׂהֲדוּ', 'subs', 'testimony', 'sahadu'),
        'E71319': ('גַּלְעֵד', 'nmpr', 'Galeed', 'galeid'),
        'E71320': ('מִצְפָּה', 'nmpr', 'Mizpah', 'mitspa'),
        'E71321': ('צפה', 'verb', 'look out', 'tsfh'),
        'E71322': ('ירה', 'verb', 'cast', 'yrh'),
        'E71323': ('זבח', 'verb', 'slaughter', 'zvch'),
        'E71324': ('זֶבַח', 'subs', 'sacrifice', 'zevach'),
        'E71325': ('מַחֲנֶה', 'subs', 'camp', 'machaneh'),
        'E71326': ('מַחֲנַיִם', 'nmpr', 'Mahanaim', 'machanayim'),
        'E71327': ('שֹׁור', 'subs', 'bullock', 'shovr'),
        'E71328': ('צרר', 'verb', 'wrap, be narrow', "tsr'r"),
        'E71329': ('חצה', 'verb', 'divide', "chts'h"),
        'E71330': ('פְּלֵיטָה', 'subs', 'escape', "p'leita"),
        'E71331': ('קטן', 'verb', 'be small', 'ktn'),
        'E71332': ('פָּרָה', 'subs', 'cow', 'para'),
        'E71333': ('פַּר', 'subs', 'young bull', 'par'),
        'E71334': ('עַיִר', 'subs', 'steed', 'ayir'),
        'E71335': ('רֶוַח', 'subs', 'space', 'revach'),
        'E71336': ('פגשׁ', 'verb', 'meet', 'fgsh'),
        'E71337': ('מַעֲבָר', 'subs', 'sweep', "ma'avar"),
        'E71338': ('יַבֹּק', 'nmpr', 'Jabbok', 'yabok'),
        'E71339': ('אבק', 'verb', 'wrestle', 'vk'),
        'E71340': ('יקע', 'verb', 'turn', 'yk'),
        'E71341': ('יִשְׂרָאֵל', 'nmpr', 'Israel', 'yisraeil'),
        'E71342': ('שׂרה', 'verb', 'contend', 'srh'),
        'E71343': ('פְּנִיאֵל', 'nmpr', 'Peniel', "p'nieil"),
        'E71344': ('זרח', 'verb', 'flash up', 'zrch'),
        'E71345': ('פְּנוּאֵל', 'nmpr', '[locality]', "p'nueil"),
        'E71346': ('צלע', 'verb', 'limp', 'tsl'),
        'E71347': ('גִּיד', 'subs', 'sinew', 'gid'),
        'E71348': ('נָשֶׁה', 'subs', 'ischias nerve', 'nasheh'),
        'E71349': ('אַחֲרֹון', 'adjv', 'at the back', 'acharovn'),
        'E71350': ('חנן', 'verb', 'favour', "chn'n"),
        'E71351': ('רצה', 'verb', 'like', "rts'h"),
        'E71352': ('עול', 'verb', 'nurse', 'vl'),
        'E71353': ('דפק', 'verb', 'drive', 'dfk'),
        'E71354': ('נהל', 'verb', 'lead', 'nhl'),
        'E71355': ('אַט', 'subs', 'gentle', 'at'),
        'E71356': ('מְלָאכָה', 'subs', 'work', "m'lacho"),
        'E71357': ('סֻכֹּות', 'nmpr', 'Succoth', 'sukovt'),
        'E71358': ('סֻכָּה', 'subs', 'cover of foliage', 'suka'),
        'E71359': ('חֶלְקָה', 'subs', 'plot of land', 'chelka'),
        'E71360': ('חֲמֹור', 'nmpr', 'Hamor', 'chamovr'),
        'E71361': ('קְשִׂיטָה', 'subs', 'qesitha', "k'sita"),
        'E71362': ('שְׁכֶם', 'nmpr', 'Shechem', "sh'chem"),
        'E71363': ('יַלְדָּה', 'subs', 'girl', 'yalda'),
        'E71364': ('טמא', 'verb', 'be unclean', 'tm'),
        'E71365': ('נְבָלָה', 'subs', 'stupidity', "n'vala"),
        'E71366': ('חשׁק', 'verb', 'love', 'chshk'),
        'E71367': ('חתן', 'verb', 'be father-in-law', 'chtn'),
        'E71368': ('מֹהַר', 'subs', 'marriage-money', 'mohar'),
        'E71369': ('מַתָּן', 'subs', 'present', 'matan'),
        'E71370': ('אות', 'verb', '[uncertain]', 'vt'),
        'E71371': ('חפץ', 'verb', 'desire', 'chfts'),
        'E71372': ('רָחָב', 'adjv', 'wide', 'rachav'),
        'E71373': ('כאב', 'verb', 'be in pain', 'chv'),
        'E71374': ('בֶּטַח', 'subs', 'trust', 'betach'),
        'E71375': ('חָלָל', 'adjv', 'pierced', 'chalal'),
        'E71376': ('בזז', 'verb', 'spoil', "vz'z"),
        'E71377': ('חַיִל', 'subs', 'power', 'chayil'),
        'E71378': ('טַף', 'subs', '[those unable to march]', 'taf'),
        'E71379': ('עכר', 'verb', 'taboo', 'chr'),
        'E71380': ('באשׁ', 'verb', 'stink', 'vsh'),
        'E71381': ('מַת', 'subs', 'man', 'mat'),
        'E71382': ('מִסְפָּר', 'subs', 'number', 'mispar'),
        'E71383': ('שׁמד', 'verb', 'destroy', 'shmd'),
        'E71384': ('זנה', 'verb', 'fornicate', 'znh'),
        'E71385': ('טהר', 'verb', 'be clean', 'thr'),
        'E71386': ('צָרָה', 'subs', 'distress', 'tsara'),
        'E71387': ('טמן', 'verb', 'hide', 'tmn'),
        'E71388': ('אֵלָה', 'subs', 'big tree', 'eila'),
        'E71389': ('חִתָּה', 'subs', 'terror', 'chita'),
        'E71390': ('דְּבֹורָה', 'nmpr', 'Deborah', "d'vovra"),
        'E71391': ('אַלֹּון', 'subs', 'big tree', 'alovn'),
        'E71392': ('בָּכוּת', 'subs', 'weeping', 'bachut'),
        'E71393': ('חֶלֶץ', 'subs', 'loins', 'chelets'),
        'E71394': ('מַצֶּבֶת', 'subs', 'stump', 'matsevet'),
        'E71395': ('נסך', 'verb', 'pour', "ns'ch"),
        'E71396': ('נֵסֶךְ', 'subs', 'libation', 'neisech'),
        'E71397': ('כְּבָרָה', 'subs', 'stretch', "k'vara"),
        'E71398': ('אֶפְרָת', 'nmpr', 'Ephrath', 'efrat'),
        'E71399': ('קשׁה', 'verb', 'be hard', 'kshh'),
        'E71400': ('בֶּן אֹונִי', 'nmpr', 'Ben-Oni', 'ben ovni'),
        'E71401': ('בִּנְיָמִן', 'nmpr', 'Benjamin', 'binamin'),
        'E71402': ('בֵּית לֶחֶם', 'nmpr', 'Bethlehem', 'beit lechem'),
        'E71403': ('קְבוּרָה', 'subs', 'grave', "k'vura"),
        'E71404': ('מִגְדַּל עֵדֶר', 'nmpr', 'Migdal Eder', 'migdal eider'),
        'E71405': ('אָהֳלִיבָמָה', 'nmpr', '[wife of Esau]', 'ahalivama'),
        'E71406': ('עֲנָה', 'nmpr', 'Anah', 'ana'),
        'E71407': ('צִבְעֹון', 'nmpr', 'Zibeon', 'tsivovn'),
        'E71408': ('אֱלִיפַז', 'nmpr', 'Eliphaz', 'elifaz'),
        'E71409': ('רְעוּאֵל', 'nmpr', 'Reuel', "r'ueil"),
        'E71410': ('יְעוּשׁ', 'nmpr', 'Jeush', "y'ush"),
        'E71411': ('יַעְלָם', 'nmpr', 'Jalam', 'yalam'),
        'E71412': ('קֹרַח', 'nmpr', 'Korah', 'korach'),
        'E71413': ('תֵּימָן', 'nmpr', 'Teman', 'teiman'),
        'E71414': ('אֹומָר', 'nmpr', 'Omar', 'ovmar'),
        'E71415': ('צְפֹו', 'nmpr', 'Zepho', "ts'fov"),
        'E71416': ('גַּעְתָּם', 'nmpr', 'Gatam', 'gatam'),
        'E71417': ('קְנַז', 'nmpr', 'Kenaz', "k'naz"),
        'E71418': ('תִּמְנָע', 'nmpr', 'Timna', 'timna'),
        'E71419': ('עֲמָלֵק', 'nmpr', 'Amalek', 'amaleik'),
        'E71420': ('נַחַת', 'nmpr', 'Nahath', 'nachat'),
        'E71421': ('זֶרַח', 'nmpr', 'Zerah', 'zerach'),
        'E71422': ('שַׁמָּה', 'nmpr', 'Shammah', 'shama'),
        'E71423': ('מִזָּה', 'nmpr', 'Mizzah', 'miza'),
        'E71424': ('אַלּוּף', 'subs', 'chief', 'aluf'),
        'E71425': ('לֹוטָן', 'nmpr', 'Lotan', 'lovtan'),
        'E71426': ('שֹׁובָל', 'nmpr', 'Shobal', 'shovval'),
        'E71427': ('דִּישֹׁון', 'nmpr', 'Dishon', 'dishovn'),
        'E71428': ('אֵצֶר', 'nmpr', 'Ezer', 'eitser'),
        'E71429': ('דִּישָׁן', 'nmpr', 'Dishan', 'dishan'),
        'E71430': ('חֹרִי', 'nmpr', 'Hori', 'chori'),
        'E71431': ('הֵימָם', 'nmpr', 'Homam', 'heimam'),
        'E71432': ('עַלְוָן', 'nmpr', 'Alvan', 'alvan'),
        'E71433': ('מָנַחַת', 'nmpr', 'Manahath', 'manachat'),
        'E71434': ('עֵיבָל', 'nmpr', 'Ebal', 'eival'),
        'E71435': ('שְׁפֹו', 'nmpr', 'Shepho', "sh'fov"),
        'E71436': ('אֹונָם', 'nmpr', 'Onam', 'ovnam'),
        'E71437': ('אַיָּה', 'nmpr', 'Ayyah', 'aya'),
        'E71438': ('יֵמִם', 'subs', 'adders', 'yeimim'),
        'E71439': ('חֶמְדָּן', 'nmpr', 'Hemdan', 'chemdan'),
        'E71440': ('אֶשְׁבָּן', 'nmpr', 'Eshban', 'eshban'),
        'E71441': ('יִתְרָן', 'nmpr', 'Ithran', 'yitran'),
        'E71442': ('כְּרָן', 'nmpr', 'Keran', "k'ran"),
        'E71443': ('בִּלְהָן', 'nmpr', 'Bilhan', 'bilhan'),
        'E71444': ('זַעֲוָן', 'nmpr', 'Zaavan', "za'avan"),
        'E71445': ('עֲקָן', 'nmpr', 'Akan', 'akan'),
        'E71446': ('אֲרָן', 'nmpr', 'Aran', 'aran'),
        'E71447': ('מלך', 'verb', 'be king', 'mlch'),
        'E71448': ('בֶּלַע', 'nmpr', '[king of edom]', 'bela'),
        'E71449': ('בְּעֹור', 'nmpr', 'Beor', "b'ovr"),
        'E71450': ('דִּנְהָבָה', 'nmpr', 'Dinhabah', 'dinhava'),
        'E71451': ('יֹובָב', 'nmpr', '[king]', 'yovvav'),
        'E71452': ('בָּצְרָה', 'nmpr', 'Bozrah', 'batsra'),
        'E71453': ('חֻשָׁם', 'nmpr', 'Husham', 'chusham'),
        'E71454': ('תֵּימָנִי', 'adjv', 'Temanite', 'teimani'),
        'E71455': ('הֲדַד', 'nmpr', 'Hadad', 'hadad'),
        'E71456': ('בְּדַד', 'nmpr', 'Bedad', "b'dad"),
        'E71457': ('עֲוִית', 'nmpr', 'Avith', 'avit'),
        'E71458': ('שַׂמְלָה', 'nmpr', 'Samlah', 'samla'),
        'E71459': ('מַשְׂרֵקָה', 'nmpr', 'Masrekah', 'masreika'),
        'E71460': ('שָׁאוּל', 'nmpr', 'Saul', 'shaul'),
        'E71461': ('בַּעַל חָנָן', 'nmpr', 'Baal-Hanan', "ba'al chanan"),
        'E71462': ('עַכְבֹּור', 'nmpr', 'Acbor', 'achbovr'),
        'E71463': ('הֲדַר', 'nmpr', 'Hadad', 'hadar'),
        'E71464': ('פָּעוּ', 'nmpr', 'Pau', 'pau'),
        'E71465': ('מְהֵיטַבְאֵל', 'nmpr', 'Mehetabel', "m'heitaveil"),
        'E71466': ('מַטְרֵד', 'nmpr', 'Matred', 'matreid'),
        'E71467': ('מֵי זָהָב', 'nmpr', 'Me-Zahab', 'mei zahav'),
        'E71468': ('עַלְוָה', 'nmpr', 'Alvah', 'alva'),
        'E71469': ('יְתֵת', 'nmpr', 'Jetheth', "y'teit"),
        'E71470': ('אֵלָה', 'nmpr', 'Elah', 'eila'),
        'E71471': ('פִּינֹן', 'nmpr', 'Pinon', 'pinon'),
        'E71472': ('מִבְצָר', 'nmpr', 'Mibzar', 'mivtsar'),
        'E71473': ('מַגְדִּיאֵל', 'nmpr', 'Magdiel', 'magdieil'),
        'E71474': ('עִירָם', 'nmpr', 'Iram', 'iram'),
        'E71475': ('דִּבָּה', 'subs', 'evil report', 'diba'),
        'E71476': ('פַּס', 'subs', 'sole?', 'pas'),
        'E71477': ('אלם', 'verb', 'bind', 'lm'),
        'E71478': ('אֲלֻמָּה', 'subs', 'sheaf', 'aluma'),
        'E71479': ('יָרֵחַ', 'subs', 'moon', 'yareiach'),
        'E71480': ('גער', 'verb', 'rebuke', 'gr'),
        'E71481': ('אֵיפֹה', 'inrg', 'where', 'eifo'),
        'E71482': ('דֹּתָן', 'nmpr', 'Dothan', 'dotan'),
        'E71483': ('נכל', 'verb', 'act cunningly', 'nchl'),
        'E71484': ('בֹּור', 'subs', 'cistern', 'bovr'),
        'E71485': ('פשׁט', 'verb', 'strip off', 'fsht'),
        'E71486': ('רֵיק', 'adjv', 'empty', 'reik'),
        'E71487': ('אֹרְחָה', 'subs', 'caravan', 'orcha'),
        'E71488': ('יִשְׁמְעֵאלִי', 'adjv', 'Ishmaelite', 'yishmeili'),
        'E71489': ('נְכֹאת', 'subs', 'ladanum', "n'chot"),
        'E71490': ('צֳרִי', 'subs', 'mastic', 'tsari'),
        'E71491': ('לֹט', 'subs', 'mastix', 'lot'),
        'E71492': ('בֶּצַע', 'subs', 'profit', 'betsa'),
        'E71493': ('מִדְיָנִי', 'adjv', 'Midianite', 'midani'),
        'E71494': ('משׁך', 'verb', 'draw', 'mshch'),
        'E71495': ('קרע', 'verb', 'tear', 'kr'),
        'E71496': ('שָׂעִיר', 'subs', 'he-goat', 'sair'),
        'E71497': ('טבל', 'verb', 'dip', 'tvl'),
        'E71498': ('טרף', 'verb', 'tear', 'trf'),
        'E71499': ('שַׂק', 'subs', 'sack', 'sak'),
        'E71500': ('מָתְנַיִם', 'subs', 'hips', 'matnayim'),
        'E71501': ('אבל', 'verb', 'mourn', 'vl'),
        'E71502': ('מאן', 'verb', 'refuse', 'mn'),
        'E71503': ('אָבֵל', 'adjv', 'mourning', 'aveil'),
        'E71504': ('שְׁאֹול', 'subs', 'nether world', "sh'ovl"),
        'E71505': ('מְדָנִי', 'adjv', 'Medanite', "m'dani"),
        'E71506': ('פֹּוטִיפַר', 'nmpr', 'Potiphar', 'povtifar'),
        'E71507': ('סָרִיס', 'subs', 'official', 'saris'),
        'E71508': ('טַבָּח', 'subs', 'butcher', 'tabach'),
        'E71509': ('עֲדֻלָּמִי', 'adjv', 'Adullamite', 'adulami'),
        'E71510': ('חִירָה', 'nmpr', 'Hirah', 'chira'),
        'E71511': ('שׁוּעַ', 'nmpr', 'Shua', 'shua'),
        'E71512': ('עֵר', 'nmpr', 'Er', 'eir'),
        'E71513': ('אֹונָן', 'nmpr', 'Onan', 'ovnan'),
        'E71514': ('שֵׁלָה', 'nmpr', 'Shelah', 'sheila'),
        'E71515': ('כְּזִיב', 'nmpr', 'Kezib', "k'ziv"),
        'E71516': ('תָּמָר', 'nmpr', 'Tamar', 'tamar'),
        'E71517': ('יבם', 'verb', 'marry', 'yvm'),
        'E71518': ('אַלְמָנָה', 'subs', 'widow', 'almana'),
        'E71519': ('תִּמְנָה', 'nmpr', 'Timnah', 'timna'),
        'E71520': ('חָם', 'subs', 'father-in-law', 'cham'),
        'E71521': ('אַלְמָנוּת', 'subs', 'widowhood', 'almanut'),
        'E71522': ('עלף', 'verb', 'cover', 'lf'),
        'E71523': ('עֵינַיִם', 'nmpr', 'Enaim', 'einayim'),
        'E71524': ('עֵרָבֹון', 'subs', 'pledge', 'eiravovn'),
        'E71525': ('חֹותָם', 'subs', 'seal', 'chovtam'),
        'E71526': ('פָּתִיל', 'subs', 'cord', 'patil'),
        'E71527': ('מַטֶּה', 'subs', 'staff', 'mateh'),
        'E71528': ('קָדֵשׁ', 'adjv', 'temple prostitute', 'kadeish'),
        'E71529': ('בּוּז', 'subs', 'contempt', 'buz'),
        'E71530': ('זְנוּנִים', 'subs', 'fornication', "z'nunim"),
        'E71531': ('חֹתֶמֶת', 'subs', 'sealing ring', 'chotemet'),
        'E71532': ('צדק', 'verb', 'be just', 'tsdk'),
        'E71533': ('שָׁנִי', 'subs', 'scarlet', 'shani'),
        'E71534': ('פֶּרֶץ', 'subs', 'breach', 'perets'),
        'E71535': ('פֶּרֶץ', 'nmpr', 'Perez', 'perets'),
        'E71536': ('שׁרת', 'verb', 'serve', 'shrt'),
        'E71537': ('אֵצֶל', 'subs', 'side', 'eitsel'),
        'E71538': ('סֹהַר', 'subs', 'prison', 'sohar'),
        'E71539': ('אָסִיר', 'subs', 'prisoner', 'asir'),
        'E71540': ('אסר', 'verb', 'bind', 'sr'),
        'E71541': ('אֹפֶה', 'subs', 'baker', 'ofeh'),
        'E71542': ('קצף', 'verb', 'be angry', 'ktsf'),
        'E71543': ('מִשְׁמָר', 'subs', 'guard', 'mishmar'),
        'E71544': ('פִּתָּרֹון', 'subs', 'interpretation', 'pitarovn'),
        'E71545': ('זעף', 'verb', 'be embittered', 'zf'),
        'E71546': ('פתר', 'verb', 'interpret', 'ftr'),
        'E71547': ('גֶּפֶן', 'subs', 'vine', 'gefen'),
        'E71548': ('שָׂרִיג', 'subs', 'tendril', 'sarig'),
        'E71549': ('פרח', 'verb', 'sprout', 'frch'),
        'E71550': ('נֵץ', 'subs', 'blossom', 'neits'),
        'E71551': ('בשׁל', 'verb', 'boil', 'vshl'),
        'E71552': ('אֶשְׁכֹּול', 'subs', 'grape', 'eshkovl'),
        'E71553': ('עֵנָב', 'subs', 'grape', 'einav'),
        'E71554': ('כֹּוס', 'subs', 'cup', 'kovs'),
        'E71555': ('שׂחט', 'verb', 'squeeze out', 'scht'),
        'E71556': ('כֵּן', 'subs', 'base', 'kein'),
        'E71557': ('סַל', 'subs', 'basket', 'sal'),
        'E71558': ('חֹרִי', 'subs', 'cake', 'chori'),
        'E71559': ('תלה', 'verb', 'hang', 'tlh'),
        'E71560': ('יְאֹר', 'nmpr', 'Nile', "y'or"),
        'E71561': ('בָּרִיא', 'adjv', 'fat', 'bari'),
        'E71562': ('אָחוּ', 'subs', 'reed', 'achu'),
        'E71563': ('דַּק', 'adjv', 'thin', 'dak'),
        'E71564': ('שִׁבֹּלֶת', 'subs', 'grain', 'shibolet'),
        'E71565': ('קָנֶה', 'subs', 'reed', 'kaneh'),
        'E71566': ('שׁדף', 'verb', 'scorch', 'shdf'),
        'E71567': ('קָדִים', 'subs', 'east', 'kadim'),
        'E71568': ('בלע', 'verb', 'swallow', 'vl'),
        'E71569': ('פעם', 'verb', 'move', 'fm'),
        'E71570': ('חַרְטֹם', 'subs', 'magician', 'chartom'),
        'E71571': ('חָכָם', 'adjv', 'wise', 'chachom'),
        'E71572': ('חֵטְא', 'subs', 'offence', 'cheit'),
        'E71573': ('גלח', 'verb', 'shave', 'glch'),
        'E71574': ('דַּל', 'adjv', 'poor', 'dal'),
        'E71575': ('רַק', 'adjv', 'thin', 'rak'),
        'E71576': ('רֹעַ', 'subs', 'wickedness', 'roa'),
        'E71577': ('צָנֻם', 'adjv', 'barren', 'tsanum'),
        'E71578': ('שָׂבָע', 'subs', 'satiety', 'sava'),
        'E71579': ('שׁנה', 'verb', 'change', 'shnh'),
        'E71580': ('כון', 'verb', 'be firm', 'chvn'),
        'E71581': ('בין', 'verb', 'understand', 'vyn'),
        'E71582': ('פָּקִיד', 'subs', 'officer', 'pakid'),
        'E71583': ('חמשׁ', 'verb', 'array', 'chmsh'),
        'E71584': ('קבץ', 'verb', 'collect', 'kvts'),
        'E71585': ('צבר', 'verb', 'heap', 'tsvr'),
        'E71586': ('בַּר', 'subs', 'grain', 'bar'),
        'E71587': ('פִּקָּדֹון', 'subs', 'deposit', 'pikadovn'),
        'E71588': ('כִּסֵּא', 'subs', 'seat', 'kisei'),
        'E71589': ('טַבַּעַת', 'subs', 'sealing ring', "taba'at"),
        'E71590': ('שֵׁשׁ', 'subs', 'linen', 'sheish'),
        'E71591': ('רָבִיד', 'subs', 'necklace', 'ravid'),
        'E71592': ('מִרְכֶּבֶת', 'subs', 'chariot', 'mirkevet'),
        'E71593': ('מִשְׁנֶה', 'subs', 'second', 'mishneh'),
        'E71594': ('אַבְרֵךְ', 'intj', '[uncertain]', 'avreich'),
        'E71595': ('צָפְנַת פַּעְנֵחַ',
                   'nmpr',
                   'Zaphenath-Paneah',
                   'tsafnat paneiach'),
        'E71596': ('אָסְנַת', 'nmpr', 'Asenath', 'asnat'),
        'E71597': ('פֹּוטִי פֶרַע', 'nmpr', 'Potiphera', 'povti fera'),
        'E71598': ('אֹן', 'nmpr', '[Heliopolis]', 'on'),
        'E71599': ('קֹמֶץ', 'subs', 'handful', 'komets'),
        'E71600': ('מְנַשֶּׁה', 'nmpr', 'Manasseh', "m'nasheh"),
        'E71601': ('נשׁה', 'verb', 'forget', 'nshh'),
        'E71602': ('עָמָל', 'subs', 'labour', 'amal'),
        'E71603': ('אֶפְרַיִם', 'nmpr', 'Ephraim', 'efrayim'),
        'E71604': ('רעב', 'verb', 'be hungry', 'rv'),
        'E71605': ('שׁבר', 'verb', 'buy grain', 'shvr'),
        'E71606': ('שֶׁבֶר', 'subs', 'corn', 'shever'),
        'E71607': ('אָסֹון', 'subs', 'accident', 'asovn'),
        'E71608': ('שַׁלִּיט', 'adjv', 'tyrant', 'shalit'),
        'E71609': ('קָשֶׁה', 'adjv', 'hard', 'kasheh'),
        'E71610': ('רגל', 'verb', 'slander', 'rgl'),
        'E71611': ('נַחְנוּ', 'prps', 'we', 'nachnu'),
        'E71612': ('כֵּן', 'adjv', 'correct', 'kein'),
        'E71613': ('בחן', 'verb', 'examine', 'vchn'),
        'E71614': ('רְעָבֹון', 'subs', 'hunger', "r'avovn"),
        'E71615': ('אָשֵׁם', 'adjv', 'guilty', 'asheim'),
        'E71616': ('ליץ', 'verb', 'boast', 'lyts'),
        'E71617': ('צֵידָה', 'subs', 'provision', 'tseida'),
        'E71618': ('מָלֹון', 'subs', 'sleeping place', 'malovn'),
        'E71619': ('אַמְתַּחַת', 'subs', 'sack', 'amtachat'),
        'E71620': ('צְרֹור', 'subs', 'bag', "ts'rovr"),
        'E71621': ('יָגֹון', 'subs', 'grief', 'yagovn'),
        'E71622': ('עוד', 'verb', 'warn, to witness', 'vd'),
        'E71623': ('ערב', 'verb', 'stand bail', 'rv'),
        'E71624': ('זִמְרָה', 'subs', 'strength', 'zimra'),
        'E71625': ('דְּבַשׁ', 'subs', 'honey', "d'vash"),
        'E71626': ('בָּטְנָה', 'subs', 'pistachio', 'batna'),
        'E71627': ('שָׁקֵד', 'subs', 'almond tree', 'shakeid'),
        'E71628': ('מִשְׁגֶּה', 'subs', 'mistake', 'mishgeh'),
        'E71629': ('רַחֲמִים', 'subs', 'compassion', 'rachamim'),
        'E71630': ('טבח', 'verb', 'slaughter', 'tvch'),
        'E71631': ('טֶבַח', 'subs', 'slaughtering', 'tevach'),
        'E71632': ('צָהֳרַיִם', 'subs', 'noon', 'tsaharayim'),
        'E71633': ('בִּי', 'intj', 'upon me', 'bi'),
        'E71634': ('מַטְמֹן', 'subs', 'treasure', 'matmon'),
        'E71635': ('כמר', 'verb', 'grow hot', 'chmr'),
        'E71636': ('חֶדֶר', 'subs', 'room', 'cheder'),
        'E71637': ('אפק', 'verb', 'be strong', 'fk'),
        'E71638': ('תֹּועֵבָה', 'subs', 'abomination', 'toveiva'),
        'E71639': ('צְעִירָה', 'subs', 'youth', "ts'ira"),
        'E71640': ('תָּמַהּ', 'verb', 'be astounded', 'tamah'),
        'E71641': ('מַשְׂאֵת', 'subs', 'lifting up', 'maseit'),
        'E71642': ('גָּבִיעַ', 'subs', 'cup', 'gavia'),
        'E71643': ('שׁלם', 'verb', 'be complete', 'shlm'),
        'E71644': ('טֹובָה', 'subs', 'what is good', 'tovva'),
        'E71645': ('עמס', 'verb', 'load', 'ms'),
        'E71646': ('בְּכִי', 'subs', 'weeping', "b'chi"),
        'E71647': ('בהל', 'verb', 'disturb', 'vhl'),
        'E71648': ('מִחְיָה', 'subs', 'life preservation', 'micha'),
        'E71649': ('חָרִישׁ', 'subs', 'ploughing', 'charish'),
        'E71650': ('שְׁאֵרִית', 'subs', 'rest', "sh'eirit"),
        'E71651': ('גֹּשֶׁן', 'nmpr', 'Goshen', 'goshen'),
        'E71652': ('כול', 'verb', 'comprehend', 'chvl'),
        'E71653': ('טען', 'verb', 'load', 'tn'),
        'E71654': ('בְּעִיר', 'subs', 'cattle', "b'ir"),
        'E71655': ('עֲגָלָה', 'subs', 'chariot', 'agala'),
        'E71656': ('חוס', 'verb', 'pity', 'chvs'),
        'E71657': ('חֲלִיפָה', 'subs', 'substitute', 'chalifa'),
        'E71658': ('מָזֹון', 'subs', 'food', 'mazovn'),
        'E71659': ('רגז', 'verb', 'quake', 'rgz'),
        'E71660': ('פוג', 'verb', 'grow numb', 'fvg'),
        'E71661': ('מַרְאָה', 'subs', 'vision', 'mara'),
        'E71662': ('פַּלּוּא', 'nmpr', 'Pallu', 'palu'),
        'E71663': ('חֶצְרֹון', 'nmpr', 'Hezron', 'chetsrovn'),
        'E71664': ('כַּרְמִי', 'nmpr', 'Carmi', 'karmi'),
        'E71665': ('יְמוּאֵל', 'nmpr', 'Jemuel', "y'mueil"),
        'E71666': ('יָמִין', 'nmpr', 'Jamin', 'yamin'),
        'E71667': ('אֹהַד', 'nmpr', 'Ohad', 'ohad'),
        'E71668': ('יָכִין', 'nmpr', 'Jakin', 'yachin'),
        'E71669': ('גֵּרְשֹׁון', 'nmpr', 'Gershon', 'geirshovn'),
        'E71670': ('קְהָת', 'nmpr', 'Kohath', "k'hat"),
        'E71671': ('מְרָרִי', 'nmpr', 'Merari', "m'rari"),
        'E71672': ('חָמוּל', 'nmpr', 'Hamul', 'chamul'),
        'E71673': ('תֹּולָע', 'nmpr', 'Tola', 'tovla'),
        'E71674': ('פֻּוָּה', 'nmpr', 'Puah', "pu'ua"),
        'E71675': ('יֹוב', 'nmpr', 'Jashub', 'yovv'),
        'E71676': ('שִׁמְרֹון', 'nmpr', 'Shimron', 'shimrovn'),
        'E71677': ('סֶרֶד', 'nmpr', 'Sered', 'sered'),
        'E71678': ('אֵלֹון', 'nmpr', 'Elon', 'eilovn'),
        'E71679': ('יַחְלְאֵל', 'nmpr', 'Jahleel', 'yachleil'),
        'E71680': ('צִפְיֹון', 'nmpr', 'Zephon', 'tsifovn'),
        'E71681': ('חַגִּי', 'nmpr', 'Haggi', 'chagi'),
        'E71682': ('שׁוּנִי', 'nmpr', 'Shuni', 'shuni'),
        'E71683': ('אֶצְבֹּון', 'nmpr', 'Ezbon', 'etsbovn'),
        'E71684': ('עֵרִי', 'nmpr', 'Eri', 'eiri'),
        'E71685': ('אֲרֹודִי', 'nmpr', 'Arodi', 'arovdi'),
        'E71686': ('אַרְאֵלִי', 'nmpr', 'Areli', 'areili'),
        'E71687': ('יִמְנָה', 'nmpr', 'Imnah', 'yimna'),
        'E71688': ('יִשְׁוָה', 'nmpr', 'Ishvah', 'yishva'),
        'E71689': ('יִשְׁוִי', 'nmpr', 'Ishvi', 'yishvi'),
        'E71690': ('בְּרִיעָה', 'nmpr', 'Beriah', "b'ria"),
        'E71691': ('שֶׂרַח', 'nmpr', 'Serah', 'serach'),
        'E71692': ('חֶבֶר', 'nmpr', 'Heber', 'chever'),
        'E71693': ('מַלְכִּיאֵל', 'nmpr', 'Malkiel', 'malkieil'),
        'E71694': ('בֶּכֶר', 'nmpr', 'Beker', 'becher'),
        'E71695': ('אַשְׁבֵּל', 'nmpr', 'Ashbel', 'ashbeil'),
        'E71696': ('גֵּרָא', 'nmpr', 'Gera', 'geira'),
        'E71697': ('נַעֲמָן', 'nmpr', 'Naaman', "na'aman"),
        'E71698': ('אֵחִי', 'nmpr', 'Ehi', 'eichi'),
        'E71699': ('רֹאשׁ', 'nmpr', 'Rosh', 'rosh'),
        'E71700': ('מֻפִּים', 'nmpr', 'Muppim', 'mupim'),
        'E71701': ('חֻפִּים', 'nmpr', 'Huppim', 'chupim'),
        'E71702': ('אַרְדְּ', 'nmpr', 'Ard', 'ard'),
        'E71703': ('חוּשִׁים', 'nmpr', 'Hushim', 'chushim'),
        'E71704': ('יַחְצְאֵל', 'nmpr', 'Jahzeel', 'yachtseil'),
        'E71705': ('גּוּנִי', 'nmpr', 'Guni', 'guni'),
        'E71706': ('יֵצֶר', 'nmpr', 'Jezer', 'yeitser'),
        'E71707': ('שִׁלֵּם', 'nmpr', 'Shillem', 'shileim'),
        'E71708': ('ירה', 'verb', 'teach', 'yrh'),
        'E71709': ('מִרְעֶה', 'subs', 'pasture', 'mireh'),
        'E71710': ('מֵיטָב', 'subs', 'best', 'meitav'),
        'E71711': ('רַעְמְסֵס', 'nmpr', 'Rameses', 'ramseis'),
        'E71712': ('יָלַהּ', 'verb', 'be confused', 'yalah'),
        'E71713': ('תמם', 'verb', 'be complete', "tm'm"),
        'E71714': ('אפס', 'verb', 'end', 'fs'),
        'E71715': ('סוּס', 'subs', 'horse', 'sus'),
        'E71716': ('כחד', 'verb', 'hide', 'chchd'),
        'E71717': ('גְּוִיָּה', 'subs', 'body', "g'uia"),
        'E71718': ('ישׁם', 'verb', 'be desolate', 'yshm'),
        'E71719': ('חֹק', 'subs', 'portion', 'chok'),
        'E71720': ('הֵא', 'intj', 'behold', 'hei'),
        'E71721': ('תְּבוּאָה', 'subs', 'yield', "t'vua"),
        'E71722': ('חֲמִישִׁית', 'subs', 'fifth part', 'chamishit'),
        'E71723': ('חֹמֶשׁ', 'subs', '[uncertain]', 'chomesh'),
        'E71724': ('מִטָּה', 'subs', 'couch', 'mita'),
        'E71725': ('חלה', 'verb', 'become weak', 'chlh'),
        'E71726': ('זֹקֶן', 'subs', 'old age', 'zoken'),
        'E71727': ('פלל', 'verb', 'arbitrate', "fl'l"),
        'E71728': ('שׂכל', 'verb', 'exchange', 'schl'),
        'E71729': ('גאל', 'verb', 'redeem', 'gl'),
        'E71730': ('דגה', 'verb', 'multiply', 'dgh'),
        'E71731': ('תמך', 'verb', 'grasp', 'tmch'),
        'E71732': ('מְלֹא', 'subs', 'fullness', "m'lo"),
        'E71733': ('אַחֲרִית', 'subs', 'end', 'acharit'),
        'E71734': ('אֹון', 'subs', 'generative power', 'ovn'),
        'E71735': ('יֶתֶר', 'subs', 'remainder', 'yeter'),
        'E71736': ('עָז', 'subs', 'strength', 'az'),
        'E71737': ('פַּחַז', 'subs', '[uncertain]', 'pachaz'),
        'E71738': ('מִשְׁכָּב', 'subs', 'couch', 'mishkav'),
        'E71739': ('יָצוּעַ', 'subs', 'couch', 'yatsua'),
        'E71740': ('מְכֵרָה', 'subs', '[uncertain]', "m'cheira"),
        'E71741': ('סֹוד', 'subs', 'confidential talk', 'sovd'),
        'E71742': ('יחד', 'verb', 'be united', 'ychd'),
        'E71743': ('רָצֹון', 'subs', 'pleasure', 'ratsovn'),
        'E71744': ('עקר', 'verb', 'root up', 'kr'),
        'E71745': ('עַז', 'adjv', 'strong', 'az'),
        'E71746': ('עֶבְרָה', 'subs', 'anger', 'evra'),
        'E71747': ('עֹרֶף', 'subs', 'neck', 'oref'),
        'E71748': ('גּוּר', 'subs', 'lion', 'gur'),
        'E71749': ('אַרְיֵה', 'subs', 'lion', 'aryei'),
        'E71750': ('טֶרֶף', 'subs', 'prey', 'teref'),
        'E71751': ('כרע', 'verb', 'kneel', 'chr'),
        'E71752': ('לָבִיא', 'subs', 'lion', 'lavi'),
        'E71753': ('שֵׁבֶט', 'subs', 'rod', 'sheivet'),
        'E71754': ('חקק', 'verb', 'engrave', "chk'k"),
        'E71755': ('שִׁלֹו', 'nmpr', 'Shiloh', 'shilov'),
        'E71756': ('יְקָהָה', 'subs', 'obedience', "y'kaha"),
        'E71757': ('עִיר', 'subs', 'ass', 'ir'),
        'E71758': ('שְׂרֵקָה', 'subs', 'vine', "s'reika"),
        'E71759': ('כבס', 'verb', 'wash', 'chvs'),
        'E71760': ('לְבוּשׁ', 'subs', 'clothing', "l'vush"),
        'E71761': ('סוּת', 'subs', 'garment', 'sut'),
        'E71762': ('חַכְלִילִי', 'adjv', 'dull', 'chachlili'),
        'E71763': ('שֵׁן', 'subs', 'tooth', 'shein'),
        'E71764': ('חֹוף', 'subs', 'shore', 'chovf'),
        'E71765': ('אֳנִיָּה', 'subs', 'ship', 'ania'),
        'E71766': ('יַרְכָּה', 'subs', 'backside', 'yarka'),
        'E71767': ('גֶּרֶם', 'subs', 'bone', 'gerem'),
        'E71768': ('מִשְׁפְּתַיִם', 'subs', 'saddle-bags', 'mishptayim'),
        'E71769': ('מְנוּחָה', 'subs', 'resting place', "m'nucha"),
        'E71770': ('נעם', 'verb', 'be pleasant', 'nm'),
        'E71771': ('סבל', 'verb', 'bear', 'svl'),
        'E71772': ('מַס', 'subs', 'forced labour', 'mas'),
        'E71773': ('שְׁפִיפֹן', 'subs', 'snake', "sh'fifon"),
        'E71774': ('נשׁך', 'verb', 'bite', 'nshch'),
        'E71775': ('אָחֹור', 'subs', 'back(wards)', 'achovr'),
        'E71776': ('יְשׁוּעָה', 'subs', 'salvation', "y'shua"),
        'E71777': ('קוה', 'verb', 'wait for', 'kvh'),
        'E71778': ('גְּדוּד', 'subs', 'band', "g'dud"),
        'E71779': ('גוד', 'verb', 'attack', 'gvd'),
        'E71780': ('שָׁמֵן', 'adjv', 'fat', 'shamein'),
        'E71781': ('מַעֲדָן', 'subs', 'dainty', "ma'adan"),
        'E71782': ('אַיָּלָה', 'subs', 'doe', 'ayala'),
        'E71783': ('אֵמֶר', 'subs', 'antler', 'eimer'),
        'E71784': ('שֶׁפֶר', 'subs', 'antlers', 'shefer'),
        'E71785': ('פֹּרָת', 'subs', 'tamarisk?', 'porat'),
        'E71786': ('צעד', 'verb', 'march', 'tsd'),
        'E71787': ('שׁוּר', 'subs', 'wall', 'shur'),
        'E71788': ('מרר', 'verb', 'be bitter', "mr'r"),
        'E71789': ('רבב', 'verb', 'shoot', "rv'v"),
        'E71790': ('חֵץ', 'subs', 'arrow', 'cheits'),
        'E71791': ('אֵיתָן', 'adjv', 'ever-flowing', 'eitan'),
        'E71792': ('פזז', 'verb', 'be agile', "fz'z"),
        'E71793': ('זְרֹועַ', 'subs', 'arm', "z'rova"),
        'E71794': ('אָבִיר', 'adjv', 'strong', 'avir'),
        'E71795': ('עזר', 'verb', 'help', 'zr'),
        'E71796': ('שַׁד', 'subs', 'breast', 'shad'),
        'E71797': ('גִּבְעָה', 'subs', 'hill', 'giva'),
        'E71798': ('קָדְקֹד', 'subs', 'scalp', 'kadkod'),
        'E71799': ('נָזִיר', 'subs', 'singled out', 'nazir'),
        'E71800': ('זְאֵב', 'subs', 'wolf', "z'eiv"),
        'E71801': ('עַד', 'subs', 'prey', 'ad'),
        'E71802': ('שָׁלָל', 'subs', 'plunder', 'shalal'),
        'E71803': ('חנט', 'verb', 'embalm', 'chnt'),
        'E71804': ('חֲנֻטִים', 'subs', 'embalming', 'chanutim'),
        'E71805': ('בְּכִית', 'subs', 'weeping', "b'chit"),
        'E71806': ('רֶכֶב', 'subs', 'chariot', 'rechev'),
        'E71807': ('פָּרָשׁ', 'subs', 'horseman', 'parash'),
        'E71808': ('גֹּרֶן', 'subs', 'threshing-floor', 'goren'),
        'E71809': ('אָטָד', 'subs', 'thorny bush', 'atad'),
        'E71810': ('עֵבֶר', 'subs', 'opposite', 'eiver'),
        'E71811': ('מִסְפֵּד', 'subs', 'wailing', 'mispeid'),
        'E71812': ('אָבֵל מִצְרַיִם', 'nmpr', 'Abel Mizraim', 'aveil mitsrayim'),
        'E71813': ('אָנָּא', 'intj', 'pray', 'ana'),
        'E71814': ('שִׁלֵּשִׁים', 'subs', 'grandsons', 'shileishim'),
        'E71815': ('מָכִיר', 'nmpr', 'Makir', 'machir'),
        'E71816': ('ישׂם', 'verb', '[uncertain]', 'ysm'),
        'E71817': ('אֲרֹון', 'subs', 'ark', 'arovn'),
        'E71818': ('חָדָשׁ', 'adjv', 'new', 'chadash'),
        'E71819': ('חכם', 'verb', 'be wise', 'chchm'),
        'E71820': ('לחם', 'verb', 'fight', 'lchm'),
        'E71821': ('סִבְלֹות', 'subs', 'loads', 'sivlovt'),
        'E71822': ('מִסְכְּנֹות', 'subs', 'storages', 'misknovt'),
        'E71823': ('פִּתֹם', 'nmpr', 'Pithom', 'pitom'),
        'E71824': ('פֶּרֶךְ', 'subs', 'act of violence', 'perech'),
        'E71825': ('שִׁפְרָה', 'nmpr', 'Shiphrah', 'shifra'),
        'E71826': ('פּוּעָה', 'nmpr', 'Puah', 'pua'),
        'E71827': ('אֹבֶן', 'subs', "potter's wheel", 'oven'),
        'E71828': ('חָיֶה', 'adjv', 'bearing easily', 'chayeh'),
        'E71829': ('יִלֹּוד', 'adjv', 'new-born', 'yilovd'),
        'E71830': ('צפן', 'verb', 'hide', 'tsfn'),
        'E71831': ('יֶרַח', 'subs', 'month', 'yerach'),
        'E71832': ('תֵּבָה', 'subs', 'chest', 'teiva'),
        'E71833': ('גֹּמֶא', 'subs', 'papyrus', 'gome'),
        'E71834': ('חמר', 'verb', 'cover', 'chmr'),
        'E71835': ('זֶפֶת', 'subs', 'pitch', 'zefet'),
        'E71836': ('סוּף', 'subs', 'rush', 'suf'),
        'E71837': ('יצב', 'verb', 'stand', 'ytsv'),
        'E71838': ('חמל', 'verb', 'have compassion', 'chml'),
        'E71839': ('מֹשֶׁה', 'nmpr', 'Moses', 'mosheh'),
        'E71840': ('משׁה', 'verb', 'draw out', 'mshh'),
        'E71841': ('נצה', 'verb', 'decay', "nts'h"),
        'E71842': ('דלה', 'verb', 'draw water', 'dlh'),
        'E71843': ('ישׁע', 'verb', 'help', 'ysh'),
        'E71844': ('צִפֹּרָה', 'nmpr', 'Zipporah', 'tsipora'),
        'E71845': ('גֵּרְשֹׁם', 'nmpr', 'Gershom', 'geirshom'),
        'E71846': ('אנח', 'verb', 'gasp', 'nch'),
        'E71847': ('זעק', 'verb', 'cry', 'zk'),
        'E71848': ('שַׁוְעָה', 'subs', 'cry', 'shava'),
        'E71849': ('נְאָקָה', 'subs', 'groaning', "n'aka"),
        'E71850': ('יִתְרֹו', 'nmpr', 'Jethro', 'yitrov'),
        'E71851': ('חֹרֵב', 'nmpr', 'Horeb', 'choreiv'),
        'E71852': ('לַבָּה', 'subs', '[uncertain]', 'laba'),
        'E71853': ('סְנֶה', 'subs', 'blackberry bush', "s'neh"),
        'E71854': ('בער', 'verb', 'burn', 'vr'),
        'E71855': ('נשׁל', 'verb', 'draw off', 'nshl'),
        'E71856': ('קֹדֶשׁ', 'subs', 'holiness', 'kodesh'),
        'E71857': ('נגשׂ', 'verb', 'drive', 'ngs'),
        'E71858': ('מַכְאֹוב', 'subs', 'pain', 'machovv'),
        'E71859': ('זוב', 'verb', 'flow', 'zvv'),
        'E71860': ('לַחַץ', 'subs', 'oppression', 'lachats'),
        'E71861': ('לחץ', 'verb', 'press', 'lchts'),
        'E71862': ('זֵכֶר', 'subs', 'mention', 'zeicher'),
        'E71863': ('חָזָק', 'adjv', 'strong', 'chazak'),
        'E71864': ('שָׁכֵן', 'subs', 'inhabitant', 'shachein'),
        'E71865': ('זָנָב', 'subs', 'tail', 'zanav'),
        'E71866': ('צרע', 'verb', 'have skin-disease', 'tsr'),
        'E71867': ('שֶׁלֶג', 'subs', 'snow', 'sheleg'),
        'E71868': ('יַבֶּשֶׁת', 'subs', 'dry land', 'yabeshet'),
        'E71869': ('אִלֵּם', 'adjv', 'speechless', 'ileim'),
        'E71870': ('חֵרֵשׁ', 'adjv', 'deaf', 'cheireish'),
        'E71871': ('פִּקֵּחַ', 'adjv', 'clear-sighted', 'pikeiach'),
        'E71872': ('עִוֵּר', 'adjv', 'blind', 'iueir'),
        'E71873': ('אַהֲרֹן', 'nmpr', 'Aaron', 'aharon'),
        'E71874': ('לֵוִי', 'adjv', 'Levite', 'leivi'),
        'E71875': ('שׂמח', 'verb', 'rejoice', 'smch'),
        'E71876': ('יֶתֶר', 'nmpr', 'Jether', 'yeter'),
        'E71877': ('מֹופֵת', 'subs', 'sign', 'movfeit'),
        'E71878': ('צֹר', 'subs', 'pebble', 'tsor'),
        'E71879': ('רפה', 'verb', 'be slack', 'rfh'),
        'E71880': ('מוּלָה', 'subs', 'circumcision', 'mula'),
        'E71881': ('חגג', 'verb', 'jump', "chg'g"),
        'E71882': ('דֶּבֶר', 'subs', 'pest', 'dever'),
        'E71883': ('פרע', 'verb', 'let loose', 'fr'),
        'E71884': ('שׁטר', 'verb', 'register', 'shtr'),
        'E71885': ('קשׁשׁ', 'verb', 'collect', "ksh'sh"),
        'E71886': ('מַתְכֹּנֶת', 'subs', 'proportion', 'matkonet'),
        'E71887': ('גרע', 'verb', 'clip', 'gr'),
        'E71888': ('שֶׁקֶר', 'subs', 'lie', 'sheker'),
        'E71889': ('קַשׁ', 'subs', 'stubble', 'kash'),
        'E71890': ('תֹּכֶן', 'subs', 'quantity', 'tochen'),
        'E71891': ('שֶׁפֶט', 'subs', 'judgment', 'shefet'),
        'E71892': ('מֹורָשָׁה', 'subs', 'possession', 'movrasha'),
        'E71893': ('קֹצֶר', 'subs', 'shortness', 'kotser'),
        'E71894': ('לִבְנִי', 'nmpr', 'Libni', 'livni'),
        'E71895': ('שִׁמְעִי', 'nmpr', 'Shimei', 'shimi'),
        'E71896': ('עַמְרָם', 'nmpr', 'Amram', 'amram'),
        'E71897': ('יִצְהָר', 'nmpr', 'Izhar', "yits'har"),
        'E71898': ('חֶבְרֹון', 'nmpr', '[levite]', 'chevrovn'),
        'E71899': ('עֻזִּיאֵל', 'nmpr', 'Uzziel', 'uzieil'),
        'E71900': ('מַחְלִי', 'nmpr', 'Mahli', 'machli'),
        'E71901': ('מוּשִׁי', 'nmpr', 'Mushi', 'mushi'),
        'E71902': ('יֹוכֶבֶד', 'nmpr', 'Jochebed', 'yovcheved'),
        'E71903': ('דֹּודָה', 'subs', "father's sister", 'dovda'),
        'E71904': ('נֶפֶג', 'nmpr', 'Nepheg', 'nefeg'),
        'E71905': ('זִכְרִי', 'nmpr', 'Zicri', 'zichri'),
        'E71906': ('מִישָׁאֵל', 'nmpr', 'Mishael', 'mishaeil'),
        'E71907': ('אֶלְצָפָן', 'nmpr', 'Elizaphan', 'eltsafan'),
        'E71908': ('סִתְרִי', 'nmpr', 'Sithri', 'sitri'),
        'E71909': ('אֱלִישֶׁבַע', 'nmpr', 'Elisheba', 'elisheva'),
        'E71910': ('עַמִּינָדָב', 'nmpr', 'Amminadab', 'aminadav'),
        'E71911': ('נַחְשֹׁון', 'nmpr', 'Nahshon', 'nachshovn'),
        'E71912': ('נָדָב', 'nmpr', 'Nadab', 'nadav'),
        'E71913': ('אֲבִיהוּא', 'nmpr', 'Abihu', 'avihu'),
        'E71914': ('אֶלְעָזָר', 'nmpr', 'Eleazar', 'elazar'),
        'E71915': ('אִיתָמָר', 'nmpr', 'Ithamar', 'itamar'),
        'E71916': ('אַסִּיר', 'nmpr', 'Assir', 'asir'),
        'E71917': ('אֶלְקָנָה', 'nmpr', 'Elkanah', 'elkana'),
        'E71918': ('אֲבִיאָסָף', 'nmpr', 'Abiasaph', 'aviasaf'),
        'E71919': ('קָרְחִי', 'adjv', 'Korahite', 'karchi'),
        'E71920': ('פּוּטִיאֵל', 'nmpr', 'Putiel', 'putieil'),
        'E71921': ('פִּינְחָס', 'nmpr', 'Phinehas', 'pinchas'),
        'E71922': ('כשׁף', 'verb', 'practice sorcery', 'chshf'),
        'E71923': ('לְהָטִים', 'subs', 'enchantments', "l'hatim"),
        'E71924': ('יְאֹר', 'subs', 'stream', "y'or"),
        'E71925': ('אֲגַם', 'subs', 'reedy pool', 'agam'),
        'E71926': ('לָט', 'subs', 'secrecy', 'lat'),
        'E71927': ('נגף', 'verb', 'hurt', 'ngf'),
        'E71928': ('צְפַרְדֵּעַ', 'subs', 'frog', "ts'fardeia"),
        'E71929': ('מִשְׁאֶרֶת', 'subs', 'kneading-trough', 'misheret'),
        'E71930': ('פאר', 'verb', 'glorify', 'fr'),
        'E71931': ('חֹמֶר', 'subs', 'homer', 'chomer'),
        'E71932': ('רְוָחָה', 'subs', 'relief', "r'vacha"),
        'E71933': ('כֵּן', 'subs', 'gnat', 'kein'),
        'E71934': ('כִּנָּם', 'subs', 'gnat', 'kinam'),
        'E71935': ('אֶצְבַּע', 'subs', 'finger', 'etsba'),
        'E71936': ('עָרֹב', 'subs', 'swarm', 'arov'),
        'E71937': ('פלה', 'verb', 'be special', 'flh'),
        'E71938': ('פְּדוּת', 'subs', 'redemption', "p'dut"),
        'E71939': ('סקל', 'verb', 'stone', 'skl'),
        'E71940': ('חֹפֶן', 'subs', 'hollow', 'chofen'),
        'E71941': ('פִּיחַ', 'subs', 'soot', 'piach'),
        'E71942': ('זרק', 'verb', 'toss', 'zrk'),
        'E71943': ('אָבָק', 'subs', 'dust', 'avak'),
        'E71944': ('שְׁחִין', 'subs', 'boil', "sh'chin"),
        'E71945': ('אֲבַעְבֻּעֹת', 'subs', 'blisters', 'avabuot'),
        'E71946': ('מַגֵּפָה', 'subs', 'blow', 'mageifa'),
        'E71947': ('סלל', 'verb', 'build', "sl'l"),
        'E71948': ('בָּרָד', 'subs', 'hail', 'barad'),
        'E71949': ('יסד', 'verb', 'found', 'ysd'),
        'E71950': ('עוז', 'verb', 'take refuge', 'vz'),
        'E71951': ('פרשׂ', 'verb', 'spread out', 'frs'),
        'E71952': ('פִּשְׁתָּה', 'subs', 'flax', 'pishta'),
        'E71953': ('שְׂעֹרָה', 'subs', 'barley', "s'ora"),
        'E71954': ('אָבִיב', 'subs', 'ears', 'aviv'),
        'E71955': ('גִּבְעֹל', 'subs', 'flower-bud', 'givol'),
        'E71956': ('כֻּסֶּמֶת', 'subs', 'spelt', 'kusemet'),
        'E71957': ('אָפִיל', 'adjv', 'late', 'afil'),
        'E71958': ('מָטָר', 'subs', 'rain', 'matar'),
        'E71959': ('נתך', 'verb', 'pour', "nt'ch"),
        'E71960': ('עלל', 'verb', 'deal with', "l'l"),
        'E71961': ('אַרְבֶּה', 'subs', 'locust', 'arbeh'),
        'E71962': ('מֹוקֵשׁ', 'subs', 'bait', 'movkeish'),
        'E71963': ('אבד', 'verb', 'perish', 'vd'),
        'E71964': ('חַג', 'subs', 'festival', 'chag'),
        'E71965': ('גֶּבֶר', 'subs', 'vigorous man', 'gever'),
        'E71966': ('חשׁך', 'verb', 'be dark', 'chshch'),
        'E71967': ('אֲפֵלָה', 'subs', 'darkness', 'afeila'),
        'E71968': ('פַּרְסָה', 'subs', 'hoof', 'parsa'),
        'E71969': ('רְעוּת', 'subs', 'female companion', "r'ut"),
        'E71970': ('חֲצֹות', 'subs', 'middle', 'chatsovt'),
        'E71971': ('רֵחַיִם', 'subs', 'hand-mill', 'reichayim'),
        'E71972': ('חרץ', 'verb', 'cut off', 'chrts'),
        'E71973': ('כֶּלֶב', 'subs', 'dog', 'kelev'),
        'E71974': ('חֳרִי', 'subs', 'heat', 'chari'),
        'E71975': ('עֵדָה', 'subs', 'gathering', 'eida'),
        'E71976': ('מעט', 'verb', 'be little', 'mt'),
        'E71977': ('מִכְסָה', 'subs', 'amount', 'michsa'),
        'E71978': ('כסס', 'verb', 'compute', "chs's"),
        'E71979': ('כֶּבֶשׂ', 'subs', 'young ram', 'keves'),
        'E71980': ('מְזוּזָה', 'subs', 'door-post', "m'zuza"),
        'E71981': ('מַשְׁקֹוף', 'subs', 'lintel', 'mashkovf'),
        'E71982': ('צָלִי', 'subs', 'roast', 'tsali'),
        'E71983': ('מָרֹר', 'adjv', 'bitter', 'maror'),
        'E71984': ('נָא', 'adjv', 'raw', 'na'),
        'E71985': ('בָּשֵׁל', 'adjv', 'cooked', 'basheil'),
        'E71986': ('כְּרָעַיִם', 'subs', 'shank', "k'ra'ayim"),
        'E71987': ('כָּכָה', 'advb', 'thus', 'kacho'),
        'E71988': ('חגר', 'verb', 'gird', 'chgr'),
        'E71989': ('חִפָּזֹון', 'subs', 'haste', 'chipazovn'),
        'E71990': ('פֶּסַח', 'subs', 'Passover', 'pesach'),
        'E71991': ('פסח', 'verb', 'be lame', "fs'ch"),
        'E71992': ('נֶגֶף', 'subs', 'plague', 'negef'),
        'E71993': ('מַשְׁחִית', 'subs', 'destruction', 'mashchit'),
        'E71994': ('זִכָּרֹון', 'subs', 'remembrance', 'zikarovn'),
        'E71995': ('שְׂאֹר', 'subs', 'leaven', "s'or"),
        'E71996': ('חָמֵץ', 'subs', 'leaven', 'chameits'),
        'E71997': ('מִקְרָא', 'subs', 'convocation', 'mikra'),
        'E71998': ('חמץ', 'verb', 'be leavened', 'chmts'),
        'E71999': ('אֶזְרָח', 'subs', 'native', 'ezrach'),
        'E72000': ('אֲגֻדָּה', 'subs', 'bundle', 'aguda'),
        'E72001': ('אֵזֹוב', 'subs', 'hyssop', 'eizovv'),
        'E72002': ('סַף', 'subs', 'basin', 'saf'),
        'E72003': ('חֲצִי', 'subs', 'half', 'chatsi'),
        'E72004': ('שְׁבִי', 'subs', 'captive', "sh'vi"),
        'E72005': ('בָּצֵק', 'subs', 'dough', 'batseik'),
        'E72006': ('רַגְלִי', 'adjv', 'on foot', 'ragli'),
        'E72007': ('עֵרֶב', 'subs', 'mixture', 'eirev'),
        'E72008': ('לַיִל', 'subs', 'night', 'layil'),
        'E72009': ('שִׁמֻּרִים', 'subs', 'vigil', 'shimurim'),
        'E72010': ('שָׂכִיר', 'adjv', 'hired', 'sachir'),
        'E72011': ('פֶּטֶר', 'subs', 'first-born', 'peter'),
        'E72012': ('חֹזֶק', 'subs', 'strength', 'chozek'),
        'E72013': ('שֶׁגֶר', 'subs', 'litter', 'sheger'),
        'E72014': ('פדה', 'verb', 'buy off', 'fdh'),
        'E72015': ('ערף', 'verb', 'break', 'rf'),
        'E72016': ('טֹוטָפֹת', 'subs', 'appendage', 'tovtafot'),
        'E72017': ('אֵתָם', 'nmpr', 'Etham', 'eitam'),
        'E72018': ('יֹומָם', 'advb', 'by day', 'yovmam'),
        'E72019': ('עַמּוּד', 'subs', 'pillar', 'amud'),
        'E72020': ('מושׁ', 'verb', 'depart', 'mvsh'),
        'E72021': ('פִּי הַחִירֹות', 'nmpr', 'Pi Hahiroth', 'pi hachirovt'),
        'E72022': ('מִגְדֹּל', 'nmpr', 'Migdol', 'migdol'),
        'E72023': ('בַּעַל צְפֹן', 'nmpr', 'Baal Zephon', "ba'al ts'fon"),
        'E72024': ('בוך', 'verb', 'confuse', 'vvch'),
        'E72025': ('שָׁלִישׁ', 'subs', 'adjutant', 'shalish'),
        'E72026': ('חֹומָה', 'subs', 'wall', 'chovma'),
        'E72027': ('אַשְׁמֹרֶת', 'subs', 'night watch', 'ashmoret'),
        'E72028': ('המם', 'verb', 'confuse', "hm'm"),
        'E72029': ('אֹופַן', 'subs', 'wheel', 'ovfan'),
        'E72030': ('מֶרְכָּבָה', 'subs', 'chariot', 'merkava'),
        'E72031': ('כְּבֵדֻת', 'subs', 'heaviness', "k'veidut"),
        'E72032': ('נער', 'verb', 'shake off', 'nr'),
        'E72033': ('שׁיר', 'verb', 'sing', 'shyr'),
        'E72034': ('שִׁירָה', 'subs', 'song', 'shira'),
        'E72035': ('גאה', 'verb', 'be high', 'gh'),
        'E72036': ('רמה', 'verb', 'shoot', 'rmh'),
        'E72037': ('עֹז', 'subs', 'protection', 'oz'),
        'E72038': ('יָהּ', 'nmpr', 'the Lord', 'yah'),
        'E72039': ('נוה', 'verb', 'praise', 'nvh'),
        'E72040': ('טבע', 'verb', 'sink', 'tv'),
        'E72041': ('מְצֹולָה', 'subs', 'depth', "m'tsovla"),
        'E72042': ('אדר', 'verb', 'be powerful', 'dr'),
        'E72043': ('רעץ', 'verb', 'break', 'rts'),
        'E72044': ('גָּאֹון', 'subs', 'height', 'gaovn'),
        'E72045': ('הרס', 'verb', 'tear down', 'hrs'),
        'E72046': ('חָרֹון', 'subs', 'anger', 'charovn'),
        'E72047': ('ערם', 'verb', 'heap', 'rm'),
        'E72048': ('נֵד', 'subs', 'barrier', 'neid'),
        'E72049': ('נזל', 'verb', 'flow', 'nzl'),
        'E72050': ('קפא', 'verb', 'condense', 'kf'),
        'E72051': ('נשׁף', 'verb', 'blow', 'nshf'),
        'E72052': ('צלל', 'verb', 'sink', "tsl'l"),
        'E72053': ('עֹופֶרֶת', 'subs', 'lead', 'ovferet'),
        'E72054': ('אַדִּיר', 'adjv', 'mighty', 'adir'),
        'E72055': ('תְּהִלָּה', 'subs', 'praise', "t'hila"),
        'E72056': ('פֶּלֶא', 'subs', 'miracle', 'pele'),
        'E72057': ('זוּ', 'conj', '[relative]', 'zu'),
        'E72058': ('עֹז', 'subs', 'power', 'oz'),
        'E72059': ('נָוֶה', 'subs', 'pasture', 'naveh'),
        'E72060': ('חִיל', 'subs', 'labour pains', 'chil'),
        'E72061': ('פְּלֶשֶׁת', 'nmpr', 'Philistia', "p'leshet"),
        'E72062': ('רַעַד', 'subs', 'trembling', "ra'ad"),
        'E72063': ('מוג', 'verb', 'faint', 'mvg'),
        'E72064': ('דמם', 'verb', 'rest', "dm'm"),
        'E72065': ('מָכֹון', 'subs', 'site', 'machovn'),
        'E72066': ('פעל', 'verb', 'make', 'fl'),
        'E72067': ('מִקְדָּשׁ', 'subs', 'sanctuary', 'mikdash'),
        'E72068': ('עַד', 'subs', 'future', 'ad'),
        'E72069': ('מִרְיָם', 'nmpr', 'Miriam', 'miram'),
        'E72070': ('נְבִיאָה', 'subs', 'prophetess', "n'via"),
        'E72071': ('מְחֹלָה', 'subs', 'round dance', "m'chola"),
        'E72072': ('ענה', 'verb', 'sing', 'nh'),
        'E72073': ('מָרָה', 'nmpr', 'Marah', 'mara'),
        'E72074': ('לון', 'verb', 'murmur', 'lvn'),
        'E72075': ('מתק', 'verb', 'be sweet', 'mtk'),
        'E72076': ('יָשָׁר', 'adjv', 'right', 'yashar'),
        'E72077': ('מַחֲלָה', 'subs', 'disease', 'machala'),
        'E72078': ('אֵילִם', 'nmpr', 'Elim', 'eilim'),
        'E72079': ('תָּמָר', 'subs', 'date-palm', 'tamar'),
        'E72080': ('סִין', 'nmpr', 'Sin', 'sin'),
        'E72081': ('סִינַי', 'nmpr', 'Sinai', 'sinay'),
        'E72082': ('סִיר', 'subs', 'pot', 'sir'),
        'E72083': ('שֹׂבַע', 'subs', 'satiety', 'sova'),
        'E72084': ('תְּלֻנֹּות', 'subs', 'murmuring', "t'lunovt"),
        'E72085': ('שׂבע', 'verb', 'be sated', 'sv'),
        'E72086': ('שְׂלָו', 'subs', 'quail', "s'lav"),
        'E72087': ('שִׁכְבָה', 'subs', 'lying down', 'shichva'),
        'E72088': ('חספס', 'verb', 'crackle', 'chsfs'),
        'E72089': ('כְּפֹור', 'subs', 'hoar frost', "k'fovr"),
        'E72090': ('מָן', 'prin', 'what', 'man'),
        'E72091': ('עֹמֶר', 'subs', 'omer', 'omer'),
        'E72092': ('גֻּלְגֹּלֶת', 'subs', 'skull', 'gulgolet'),
        'E72093': ('מדד', 'verb', 'measure', "md'd"),
        'E72094': ('עדף', 'verb', 'remain', 'df'),
        'E72095': ('רמם', 'verb', 'be wormy', "rm'm"),
        'E72096': ('תֹּולָע', 'subs', 'scarlet worm', 'tovla'),
        'E72097': ('מסס', 'verb', 'melt', "ms's"),
        'E72098': ('שַׁבָּתֹון', 'subs', 'sabbatical feast', 'shabatovn'),
        'E72099': ('שַׁבָּת', 'subs', 'sabbath', 'shabat'),
        'E72100': ('רִמָּה', 'subs', 'maggot', 'rima'),
        'E72101': ('מָן', 'subs', 'manna', 'man'),
        'E72102': ('גַּד', 'subs', 'coriander', 'gad'),
        'E72103': ('טַעַם', 'subs', 'taste', "ta'am"),
        'E72104': ('צַפִּיחִת', 'subs', 'flat cake', 'tsapichit'),
        'E72105': ('צִנְצֶנֶת', 'subs', 'receptacle', 'tsintsenet'),
        'E72106': ('עֵדוּת', 'subs', 'reminder', 'eidut'),
        'E72107': ('אֵיפָה', 'subs', 'ephah', 'eifa'),
        'E72108': ('רְפִידִים', 'nmpr', 'Rephidim', "r'fidim"),
        'E72109': ('צמא', 'verb', 'be thirsty', 'tsm'),
        'E72110': ('צָמָא', 'subs', 'thirst', 'tsama'),
        'E72111': ('צוּר', 'subs', 'rock', 'tsur'),
        'E72112': ('מַסָּה', 'nmpr', 'Massah', 'masa'),
        'E72113': ('מְרִיבָה', 'nmpr', 'Meribah', "m'riva"),
        'E72114': ('יְהֹושֻׁעַ', 'nmpr', 'Joshua', "y'hovshua"),
        'E72115': ('חוּר', 'nmpr', 'Hur', 'chur'),
        'E72116': ('אֱמוּנָה', 'subs', 'steadiness', 'emuna'),
        'E72117': ('חלשׁ', 'verb', 'defeat', 'chlsh'),
        'E72118': ('כתב', 'verb', 'write', 'chtv'),
        'E72119': ('נֵס', 'subs', 'signal', 'neis'),
        'E72120': ('שִׁלּוּחִים', 'subs', 'act of sending', 'shiluchim'),
        'E72121': ('תְּלָאָה', 'subs', 'weariness', "t'la'a"),
        'E72122': ('חדה', 'verb', 'rejoice', 'chdh'),
        'E72123': ('נבל', 'verb', 'wither', 'nvl'),
        'E72124': ('יעץ', 'verb', 'advise', 'yts'),
        'E72125': ('מוּל', 'subs', 'front', 'mul'),
        'E72126': ('זהר', 'verb', 'warn', 'zhr'),
        'E72127': ('חזה', 'verb', 'see', 'chzh'),
        'E72128': ('נֶשֶׁר', 'subs', 'eagle', 'nesher'),
        'E72129': ('סְגֻלָּה', 'subs', 'property', "s'gula"),
        'E72130': ('קָדֹושׁ', 'adjv', 'holy', 'kadovsh'),
        'E72131': ('עָב', 'subs', 'cloud', 'av'),
        'E72132': ('גבל', 'verb', 'set border', 'gvl'),
        'E72133': ('יֹובֵל', 'subs', 'ram', 'yovveil'),
        'E72134': ('בָּרָק', 'subs', 'lightning', 'barak'),
        'E72135': ('שֹׁופָר', 'subs', 'horn', 'shovfar'),
        'E72136': ('עשׁן', 'verb', 'smoke', 'shn'),
        'E72137': ('חָזֵק', 'adjv', 'strong', 'chazeik'),
        'E72138': ('פֶּסֶל', 'subs', 'idol', 'pesel'),
        'E72139': ('תְּמוּנָה', 'subs', 'form', "t'muna"),
        'E72140': ('קַנָּא', 'adjv', 'jealous', 'kana'),
        'E72141': ('רִבֵּעַ', 'subs', 'of fourth generation', 'ribeia'),
        'E72142': ('שָׁוְא', 'subs', 'vanity', 'shav'),
        'E72143': ('רצח', 'verb', 'kill', 'rtsch'),
        'E72144': ('נאף', 'verb', 'commit adultery', 'nf'),
        'E72145': ('עָשֵׁן', 'adjv', 'smoking', 'ashein'),
        'E72146': ('עֲרָפֶל', 'subs', 'darkness', 'arafel'),
        'E72147': ('שֶׁלֶם', 'subs', 'final offer', 'shelem'),
        'E72148': ('גָּזִית', 'subs', 'hewn stone', 'gazit'),
        'E72149': ('נוף', 'verb', 'swing', 'nvf'),
        'E72150': ('מַעֲלָה', 'subs', 'ascent', "ma'ala"),
        'E72151': ('חָפְשִׁי', 'adjv', 'released', 'chafshi'),
        'E72152': ('גַּף', 'subs', 'body', 'gaf'),
        'E72153': ('רצע', 'verb', 'pierce', 'rts'),
        'E72154': ('מַרְצֵעַ', 'subs', 'awl', 'martseia'),
        'E72155': ('יעד', 'verb', 'appoint', 'yd'),
        'E72156': ('בגד', 'verb', 'deal treacherously', 'vgd'),
        'E72157': ('שְׁאֵר', 'subs', 'body', "sh'eir"),
        'E72158': ('עֹנָה', 'subs', 'intercourse', 'ona'),
        'E72159': ('צדה', 'verb', 'be after', 'tsdh'),
        'E72160': ('אנה', 'verb', 'occur', 'nh'),
        'E72161': ('עָרְמָה', 'subs', 'prudence', 'arma'),
        'E72162': ('אֶגְרֹף', 'subs', 'broom', 'egrof'),
        'E72163': ('מִשְׁעֶנֶת', 'subs', 'support', 'mishenet'),
        'E72164': ('שֶׁבֶת', 'subs', 'inaction', 'shevet'),
        'E72165': ('ענשׁ', 'verb', 'fine', 'nsh'),
        'E72166': ('פָּלִיל', 'subs', 'judge', 'palil'),
        'E72167': ('כְּוִיָּה', 'subs', 'branding', "k'uia"),
        'E72168': ('נגח', 'verb', 'gore', 'ngch'),
        'E72169': ('נַגָּח', 'adjv', 'goring', 'nagach'),
        'E72170': ('כֹּפֶר', 'subs', 'ransom', 'kofer'),
        'E72171': ('פִּדְיֹון', 'subs', 'ransom', 'pidovn'),
        'E72172': ('מַחְתֶּרֶת', 'subs', 'breaking in', 'machteret'),
        'E72173': ('גַּנָּב', 'subs', 'thief', 'ganav'),
        'E72174': ('גְּנֵבָה', 'subs', 'what is stolen', "g'neiva"),
        'E72175': ('גָּדִישׁ', 'subs', 'heap', 'gadish'),
        'E72176': ('קָמָה', 'subs', 'standing grain', 'kama'),
        'E72177': ('בְּעֵרָה', 'subs', 'burning', "b'eira"),
        'E72178': ('שַׂלְמָה', 'subs', 'wrapper', 'salma'),
        'E72179': ('אֲבֵדָה', 'subs', 'what is lost', 'aveida'),
        'E72180': ('רשׁע', 'verb', 'be guilty', 'rsh'),
        'E72181': ('פתה', 'verb', 'seduce', 'fth'),
        'E72182': ('ארשׂ', 'verb', 'betroth', 'rs'),
        'E72183': ('מהר', 'verb', 'obtain', 'mhr'),
        'E72184': ('חרם', 'verb', 'consecrate', 'chrm'),
        'E72185': ('ינה', 'verb', 'oppress', 'ynh'),
        'E72186': ('יָתֹום', 'subs', 'orphan', 'yatovm'),
        'E72187': ('לוה', 'verb', 'borrow', 'lvh'),
        'E72188': ('עָנִי', 'adjv', 'humble', 'ani'),
        'E72189': ('נשׁא', 'verb', 'give loan', 'nsh'),
        'E72190': ('נֶשֶׁךְ', 'subs', 'interest', 'neshech'),
        'E72191': ('חבל', 'verb', 'take a pledge', 'chvl'),
        'E72192': ('חַנּוּן', 'adjv', 'gracious', 'chanun'),
        'E72193': ('מְלֵאָה', 'subs', 'full produce', "m'leia"),
        'E72194': ('דֶּמַע', 'subs', 'abundance', 'dema'),
        'E72195': ('שְׁמִינִי', 'adjv', 'eighth', "sh'mini"),
        'E72196': ('הדר', 'verb', 'honour', 'hdr'),
        'E72197': ('מַשָּׂא', 'subs', 'burden', 'masa'),
        'E72198': ('אֶבְיֹון', 'adjv', 'poor', 'evyovn'),
        'E72199': ('שֹׁחַד', 'subs', 'present', 'shochad'),
        'E72200': ('עור', 'verb', 'be blind', 'vr'),
        'E72201': ('סלף', 'verb', 'distort', 'slf'),
        'E72202': ('שׁמט', 'verb', 'let loose', 'shmt'),
        'E72203': ('נפשׁ', 'verb', 'take breath', 'nfsh'),
        'E72204': ('בִּכּוּרִים', 'subs', 'first fruits', 'bikurim'),
        'E72205': ('אָסִיף', 'subs', 'harvesting', 'asif'),
        'E72206': ('זְכוּר', 'subs', 'male', "z'chur"),
        'E72207': ('מרה', 'verb', 'rebel', 'mrh'),
        'E72208': ('צרר', 'verb', 'be hostile', "tsr'r"),
        'E72209': ('צִרְעָה', 'subs', 'depression', 'tsira'),
        'E72210': ('שְׁמָמָה', 'subs', 'desolation', "sh'mama"),
        'E72211': ('נחל', 'verb', 'take possession', 'nchl'),
        'E72212': ('אַגָּן', 'subs', 'bowl', 'agan'),
        'E72213': ('סַפִּיר', 'subs', 'lapis lazuli', 'sapir'),
        'E72214': ('טֹהַר', 'subs', 'purity', 'tohar'),
        'E72215': ('אָצִיל', 'adjv', 'noble', 'atsil'),
        'E72216': ('לוּחַ', 'subs', 'tablet', 'luach'),
        'E72217': ('תְּרוּמָה', 'subs', 'contribution', "t'ruma"),
        'E72218': ('נדב', 'verb', 'incite', 'ndv'),
        'E72219': ('תְּכֵלֶת', 'subs', 'purple wool', "t'cheilet"),
        'E72220': ('אַרְגָּמָן', 'subs', 'purple-wool', 'argaman'),
        'E72221': ('תֹּולַעַת', 'subs', 'worm', "tovla'at"),
        'E72222': ('אדם', 'verb', 'be ruddy', 'dm'),
        'E72223': ('תַּחַשׁ', 'subs', 'tachash-skin', 'tachash'),
        'E72224': ('שִׁטָּה', 'subs', 'acacia', 'shita'),
        'E72225': ('בֹּשֶׂם', 'subs', 'balsam-tree', 'bosem'),
        'E72226': ('מִשְׁחָה', 'subs', 'anointment', 'mishcha'),
        'E72227': ('קְטֹרֶת', 'subs', 'smoke of sacrifice', "k'toret"),
        'E72228': ('סַם', 'subs', 'perfume', 'sam'),
        'E72229': ('מִלֻּאִים', 'subs', 'installation', 'miluim'),
        'E72230': ('אֵפֹד', 'subs', 'ephod', 'eifod'),
        'E72231': ('חֹשֶׁן', 'subs', 'breast-piece', 'choshen'),
        'E72232': ('תַּבְנִית', 'subs', 'model', 'tavnit'),
        'E72233': ('מִשְׁכָּן', 'subs', 'dwelling-place', 'mishkan'),
        'E72234': ('צפה', 'verb', 'arrange', 'tsfh'),
        'E72235': ('זֵר', 'subs', 'border', 'zeir'),
        'E72236': ('כַּפֹּרֶת', 'subs', 'cover', 'kaporet'),
        'E72237': ('מִקְשָׁה', 'subs', 'hammered work', 'miksha'),
        'E72238': ('קָצָה', 'subs', 'end', 'katsa'),
        'E72239': ('סכך', 'verb', 'block', "s'ch'ch"),
        'E72240': ('שֻׁלְחָן', 'subs', 'table', 'shulchan'),
        'E72241': ('מִסְגֶּרֶת', 'subs', 'bulwark', 'misgeret'),
        'E72242': ('טֹפַח', 'subs', 'span', 'tofach'),
        'E72243': ('פֵּאָה', 'subs', 'corner', 'peia'),
        'E72244': ('עֻמָּה', 'subs', 'side', 'uma'),
        'E72245': ('קְעָרָה', 'subs', 'platter', "k'ara"),
        'E72246': ('קַשְׂוָה', 'subs', 'jar', 'kasva'),
        'E72247': ('מְנַקִּית', 'subs', 'bowl', "m'nakit"),
        'E72248': ('תָּמִיד', 'subs', 'continuity', 'tamid'),
        'E72249': ('מְנֹורָה', 'subs', 'lamp stand', "m'novra"),
        'E72250': ('כַּפְתֹּור', 'subs', 'bulb', 'kaftovr'),
        'E72251': ('פֶּרַח', 'subs', 'bud', 'perach'),
        'E72252': ('מְשֻׁקָּד', 'adjv', 'almond-like', "m'shukad"),
        'E72253': ('נֵר', 'subs', 'lamp', 'neir'),
        'E72254': ('מֶלְקָחַיִם', 'subs', 'snuffers', 'melkachayim'),
        'E72255': ('מַחְתָּה', 'subs', 'fire holder', 'machta'),
        'E72256': ('יְרִיעָה', 'subs', 'curtain', "y'ria"),
        'E72257': ('שׁזר', 'verb', 'twist', 'shzr'),
        'E72258': ('מִדָּה', 'subs', 'measured stretch', 'mida'),
        'E72259': ('לֻלָאֹת', 'subs', 'loops', 'lulaot'),
        'E72260': ('חֹבֶרֶת', 'subs', 'drapery', 'choveret'),
        'E72261': ('קִיצֹון', 'adjv', 'outermost', 'kitsovn'),
        'E72262': ('מַחְבֶּרֶת', 'subs', 'junction', 'machberet'),
        'E72263': ('קבל', 'verb', 'take', 'kvl'),
        'E72264': ('קֶרֶס', 'subs', 'hook', 'keres'),
        'E72265': ('עַשְׁתֵּי', 'subs', 'eleven', 'ashtei'),
        'E72266': ('כפל', 'verb', 'fold double', 'chfl'),
        'E72267': ('סֶרַח', 'subs', 'overhanging', 'serach'),
        'E72268': ('סרח', 'verb', 'overhang', 'srch'),
        'E72269': ('סָרוּחַ', 'adjv', 'overhanging', 'saruach'),
        'E72270': ('קֶרֶשׁ', 'subs', 'board', 'keresh'),
        'E72271': ('שׁלב', 'verb', 'bind', 'shlv'),
        'E72272': ('תֵּימָן', 'subs', 'south', 'teiman'),
        'E72273': ('אֶדֶן', 'subs', 'pedestal', 'eden'),
        'E72274': ('קצע', 'verb', 'cut off', 'kts'),
        'E72275': ('מַטָּה', 'subs', 'low place', 'mata'),
        'E72276': ('מִקְצֹועַ', 'subs', 'corner post', 'miktsova'),
        'E72277': ('בְּרִיחַ', 'subs', 'bar', "b'riach"),
        'E72278': ('תִּיכֹון', 'adjv', 'middle', 'tichovn'),
        'E72279': ('פָּרֹכֶת', 'subs', 'curtain', 'parochet'),
        'E72280': ('וָו', 'subs', 'hook', 'vav'),
        'E72281': ('מָסָךְ', 'subs', 'covering', 'masach'),
        'E72282': ('רקם', 'verb', 'weave', 'rkm'),
        'E72283': ('רבע', 'verb', 'be square', 'rv'),
        'E72284': ('פִּנָּה', 'subs', 'corner', 'pina'),
        'E72285': ('דשׁן', 'verb', 'grow fat', 'dshn'),
        'E72286': ('יָעֶה', 'subs', 'shovel', 'yaeh'),
        'E72287': ('מִזְרָק', 'subs', 'bowl', 'mizrak'),
        'E72288': ('מִזְלָג', 'subs', 'fork', 'mizlag'),
        'E72289': ('מִכְבָּר', 'subs', 'lattice', 'michbor'),
        'E72290': ('רֶשֶׁת', 'subs', 'net', 'reshet'),
        'E72291': ('כַּרְכֹּב', 'subs', 'rim', 'karkov'),
        'E72292': ('נבב', 'verb', 'be hollow', "nv'v"),
        'E72293': ('קֶלַע', 'subs', 'curtain', 'kela'),
        'E72294': ('חָשׁוּק', 'subs', 'joint', 'chashuk'),
        'E72295': ('מִזְרָח', 'subs', 'sunrise', 'mizrach'),
        'E72296': ('כָּתֵף', 'subs', 'shoulder', 'kateif'),
        'E72297': ('יָתֵד', 'subs', 'peg', 'yateid'),
        'E72298': ('זַךְ', 'adjv', 'pure', 'zach'),
        'E72299': ('כָּתִית', 'adjv', 'beaten, pure', 'katit'),
        'E72300': ('כהן', 'verb', 'act as priest', 'chhn'),
        'E72301': ('תִּפְאֶרֶת', 'subs', 'splendour', 'tiferet'),
        'E72302': ('חָכְמָה', 'subs', 'wisdom', 'chachma'),
        'E72303': ('מְעִיל', 'subs', 'coat', "m'il"),
        'E72304': ('תַּשְׁבֵּץ', 'subs', 'chequered work', 'tashbeits'),
        'E72305': ('מִצְנֶפֶת', 'subs', 'turban', 'mitsnefet'),
        'E72306': ('אַבְנֵט', 'subs', 'scarf', 'avneit'),
        'E72307': ('חֵשֶׁב', 'subs', 'girdle', 'cheishev'),
        'E72308': ('אֲפֻדָּה', 'subs', 'covering', 'afuda'),
        'E72309': ('פתח', 'verb', 'engrave', "ft'ch"),
        'E72310': ('חָרָשׁ', 'subs', 'artisan', 'charash'),
        'E72311': ('פִּתּוּחַ', 'subs', 'engraving', 'pituach'),
        'E72312': ('מִשְׁבְּצֹות', 'subs', 'settings', 'mishbtsovt'),
        'E72313': ('שַׁרְשְׁרָה', 'subs', 'chain', 'sharshra'),
        'E72314': ('מִגְבָּלֹת', 'subs', 'cords', 'migbalot'),
        'E72315': ('עֲבֹת', 'subs', 'rope', 'avot'),
        'E72316': ('זֶרֶת', 'subs', 'span', 'zeret'),
        'E72317': ('מִלֻּאָה', 'subs', 'setting', 'milua'),
        'E72318': ('טוּר', 'subs', 'row', 'tur'),
        'E72319': ('אֹדֶם', 'subs', 'ruby', 'odem'),
        'E72320': ('פִּטְדָּה', 'subs', 'chrysolyte', 'pitda'),
        'E72321': ('בָּרֶקֶת', 'subs', 'beryl', 'bareket'),
        'E72322': ('נֹפֶךְ', 'subs', 'turquoise', 'nofech'),
        'E72323': ('יַהֲלֹם', 'subs', 'precious stone', 'yahalom'),
        'E72324': ('לֶשֶׁם', 'subs', '[gem]', 'leshem'),
        'E72325': ('שְׁבֹו', 'subs', '[precious stone]', "sh'vov"),
        'E72326': ('אַחְלָמָה', 'subs', 'jaspis', 'achlama'),
        'E72327': ('תַּרְשִׁישׁ', 'subs', 'precious stone', 'tarshish'),
        'E72328': ('יָשְׁפֵה', 'subs', 'jade', 'yashfei'),
        'E72329': ('שׁבץ', 'verb', 'weave', 'shvts'),
        'E72330': ('גַּבְלֻת', 'subs', '[uncertain]', 'gavlut'),
        'E72331': ('רכס', 'verb', 'bind', 'rchs'),
        'E72332': ('זחח', 'verb', 'displace', "zch'ch"),
        'E72333': ('אוּר', 'subs', '[oracle device]', 'ur'),
        'E72334': ('תֻּמִּים', 'subs', 'tummim', 'tumim'),
        'E72335': ('כָּלִיל', 'adjv', 'entire', 'kalil'),
        'E72336': ('ארג', 'verb', 'weave', 'rg'),
        'E72337': ('תַּחְרָא', 'subs', 'cuirass', 'tachra'),
        'E72338': ('שׁוּל', 'subs', 'skirt', 'shul'),
        'E72339': ('רִמֹּון', 'subs', 'pomegranate', 'rimovn'),
        'E72340': ('פַּעֲמֹון', 'subs', 'bell', "pa'amovn"),
        'E72341': ('צִיץ', 'subs', 'blossom', 'tsits'),
        'E72342': ('מֵצַח', 'subs', 'forehead', 'meitsach'),
        'E72343': ('מִגְבָּעָה', 'subs', 'head-gear', "migba'a"),
        'E72344': ('מִכְנָס', 'subs', 'drawer', 'michnas'),
        'E72345': ('חַלָּה', 'subs', 'bread', 'chala'),
        'E72346': ('רָקִיק', 'subs', 'wafer', 'rakik'),
        'E72347': ('אפד', 'verb', 'dress', 'fd'),
        'E72348': ('נֵזֶר', 'subs', 'consecration', 'neizer'),
        'E72349': ('כְּהֻנָּה', 'subs', 'priesthood', "k'huna"),
        'E72350': ('יְסֹוד', 'subs', 'foundation', "y'sovd"),
        'E72351': ('יֹתֶרֶת', 'subs', 'appendage of liver', 'yoteret'),
        'E72352': ('כָּבֵד', 'subs', 'liver', 'kaveid'),
        'E72353': ('כִּלְיָה', 'subs', 'kidney', 'kila'),
        'E72354': ('קטר', 'verb', 'smoke', 'ktr'),
        'E72355': ('פֶּרֶשׁ', 'subs', 'contents of stomach', 'peresh'),
        'E72356': ('נתח', 'verb', 'cut', "nt'ch"),
        'E72357': ('נֵתַח', 'subs', 'piece', 'neitach'),
        'E72358': ('אִשֶּׁה', 'subs', 'fire offering', 'isheh'),
        'E72359': ('תְּנוּךְ', 'subs', 'earlobe', "t'nuch"),
        'E72360': ('יְמָנִי', 'adjv', 'right-hand', "y'mani"),
        'E72361': ('בֹּהֶן', 'subs', 'thumb', 'bohen'),
        'E72362': ('נזה', 'verb', 'spatter', 'nzh'),
        'E72363': ('אַלְיָה', 'subs', 'fat tail', 'alya'),
        'E72364': ('שֹׁוק', 'subs', 'splintbone', 'shovk'),
        'E72365': ('תְּנוּפָה', 'subs', 'offering', "t'nufa"),
        'E72366': ('חָזֶה', 'subs', 'breast', 'chazeh'),
        'E72367': ('מָנָה', 'subs', 'portion', 'mana'),
        'E72368': ('מָשְׁחָה', 'subs', 'anointment', 'mashcha'),
        'E72369': ('זָר', 'adjv', 'strange', 'zar'),
        'E72370': ('כִּפֻּרִים', 'subs', 'atonement', 'kipurim'),
        'E72371': ('עִשָּׂרֹון', 'subs', 'tenth part', 'isarovn'),
        'E72372': ('רֶבַע', 'subs', 'fourth part', 'reva'),
        'E72373': ('הִין', 'subs', 'hin', 'hin'),
        'E72374': ('מִקְטָר', 'subs', 'burning place', 'miktar'),
        'E72375': ('גָּג', 'subs', 'roof', 'gag'),
        'E72376': ('קִיר', 'subs', 'wall', 'kir'),
        'E72377': ('מַחֲצִית', 'subs', 'half', 'machatsit'),
        'E72378': ('גֵּרָה', 'subs', 'gera', 'geira'),
        'E72379': ('עָשִׁיר', 'adjv', 'rich', 'ashir'),
        'E72380': ('כִּיֹּור', 'subs', 'basin', 'kiovr'),
        'E72381': ('מֹר', 'subs', 'myrrh', 'mor'),
        'E72382': ('דְּרֹור', 'subs', 'drop', "d'rovr"),
        'E72383': ('קִנָּמֹון', 'subs', 'cinnamon', 'kinamovn'),
        'E72384': ('קִדָּה', 'subs', 'cassia-bud', 'kida'),
        'E72385': ('רֹקַח', 'subs', 'spice', 'rokach'),
        'E72386': ('מִרְקַחַת', 'subs', 'ointment mixture', 'mirkachat'),
        'E72387': ('רקח', 'verb', 'mix', 'rkch'),
        'E72388': ('יסך', 'verb', '[uncertain]', "ys'ch"),
        'E72389': ('נָטָף', 'subs', 'drops', 'nataf'),
        'E72390': ('שְׁחֵלֶת', 'subs', 'incense', "sh'cheilet"),
        'E72391': ('חֶלְבָּנָה', 'subs', 'galbanum', 'chelbana'),
        'E72392': ('לְבֹנָה', 'subs', 'incense', "l'vona"),
        'E72393': ('מלח', 'verb', 'salt', 'mlch'),
        'E72394': ('שׁחק', 'verb', 'pulverise', 'shchk'),
        'E72395': ('דקק', 'verb', 'crush', "dk'k"),
        'E72396': ('בְּצַלְאֵל', 'nmpr', 'Bezalel', "b'tsaleil"),
        'E72397': ('אוּרִי', 'nmpr', 'Uri', 'uri'),
        'E72398': ('תְּבוּנָה', 'subs', 'understanding', "t'vuna"),
        'E72399': ('חֲרֹשֶׁת', 'subs', 'work', 'charoshet'),
        'E72400': ('אָהֳלִיאָב', 'nmpr', 'Oholiab', 'ahaliav'),
        'E72401': ('אֲחִיסָמָךְ', 'nmpr', 'Ahisamach', 'achisamach'),
        'E72402': ('שְׂרָד', 'subs', 'cord', "s'rad"),
        'E72403': ('קהל', 'verb', 'assemble', 'khl'),
        'E72404': ('צור', 'verb', 'shape', 'tsvr'),
        'E72405': ('חֶרֶט', 'subs', 'stylus', 'cheret'),
        'E72406': ('עֵגֶל', 'subs', 'bull', 'eigel'),
        'E72407': ('מַסֵּכָה', 'subs', 'molten image', 'maseicho'),
        'E72408': ('מִכְתָּב', 'subs', 'writing', 'michtav'),
        'E72409': ('חרת', 'verb', 'engrave', 'chrt'),
        'E72410': ('רֵעַ', 'subs', 'shouting', 'reia'),
        'E72411': ('גְּבוּרָה', 'subs', 'strength', "g'vura"),
        'E72412': ('חֲלוּשָׁה', 'subs', 'defeat', 'chalusha'),
        'E72413': ('טחן', 'verb', 'grind', "t'chn"),
        'E72414': ('זרה', 'verb', 'scatter', 'zrh'),
        'E72415': ('שִׁמְצָה', 'subs', '[uncertain]', 'shimtsa'),
        'E72416': ('עֲדִי', 'subs', 'ornament', 'adi'),
        'E72417': ('רֶגַע', 'subs', 'moment', 'rega'),
        'E72418': ('נוּן', 'nmpr', 'Nun', 'nun'),
        'E72419': ('רחם', 'verb', 'have compassion', 'rchm'),
        'E72420': ('נְקָרָה', 'subs', 'hole', "n'kara"),
        'E72421': ('שׂכך', 'verb', 'weave', "sch'ch"),
        'E72422': ('פסל', 'verb', 'cut', 'fsl'),
        'E72423': ('רַחוּם', 'adjv', 'compassionate', 'rachum'),
        'E72424': ('אָרֵךְ', 'adjv', 'long', 'areich'),
        'E72425': ('נצר', 'verb', 'watch', 'ntsr'),
        'E72426': ('חַטָּאָה', 'subs', 'sin', "chata'a"),
        'E72427': ('סלח', 'verb', 'forgive', 'slch'),
        'E72428': ('נתץ', 'verb', 'break', 'ntts'),
        'E72429': ('אֲשֵׁרָה', 'subs', 'asherah', 'asheira'),
        'E72430': ('תְּקוּפָה', 'subs', 'turning point', "t'kufa"),
        'E72431': ('קרן', 'verb', 'shine', 'krn'),
        'E72432': ('מַסְוֶה', 'subs', 'mask', 'masveh'),
        'E72433': ('נָדִיב', 'adjv', 'willing', 'nadiv'),
        'E72434': ('מֵיתָר', 'subs', 'string', 'meitar'),
        'E72435': ('חָח', 'subs', 'thorn', 'chach'),
        'E72436': ('כּוּמָז', 'subs', 'breastplate', 'kumaz'),
        'E72437': ('טוה', 'verb', 'spin', 'tvh'),
        'E72438': ('מַטְוֶה', 'subs', 'yarn', 'matveh'),
        'E72439': ('נְדָבָה', 'subs', 'free will', "n'dava"),
        'E72440': ('מַחֲשֶׁבֶת', 'subs', 'plan', 'machashevet'),
        'E72441': ('דַּי', 'subs', 'sufficiency', 'day'),
        'E72442': ('קְצָת', 'subs', 'end', "k'tsat"),
        'E72443': ('צבא', 'verb', 'serve', 'tsv'),
        'E72444': ('צִפּוּי', 'subs', 'plating', 'tsipuy'),
        'E72445': ('רקע', 'verb', 'stamp', 'rk'),
        'E72446': ('פַּח', 'subs', 'plate', 'pach'),
        'E72447': ('קצץ', 'verb', 'cut', "kts'ts"),
        'E72448': ('פְּאֵר', 'subs', 'headdress', "p'eir"),
        'E72449': ('מַעֲרָכָה', 'subs', 'row', "ma'aracho"),
        'E72450': ('עֵרֶךְ', 'subs', 'arrangement', 'eirech'),
        'E72451': ('קָרְבָּן', 'subs', 'offering', 'karban'),
        'E72452': ('פֶּדֶר', 'subs', 'kidney-fat', 'peder'),
        'E72453': ('מלק', 'verb', 'nip off', 'mlk'),
        'E72454': ('מצה', 'verb', 'drain', "mts'h"),
        'E72455': ('מֻרְאָה', 'subs', 'crop', 'mura'),
        'E72456': ('נֹצָה', 'subs', 'filth', 'notsa'),
        'E72457': ('דֶּשֶׁן', 'subs', 'fatness', 'deshen'),
        'E72458': ('שׁסע', 'verb', 'cleave', 'shs'),
        'E72459': ('קמץ', 'verb', 'take handful', 'kmts'),
        'E72460': ('אַזְכָּרָה', 'subs', 'memorial offering', 'azkara'),
        'E72461': ('מַאֲפֶה', 'subs', 'cake', "ma'afeh"),
        'E72462': ('מַחֲבַת', 'subs', 'griddle', 'machavat'),
        'E72463': ('פתת', 'verb', 'crumble', "ft't"),
        'E72464': ('מַרְחֶשֶׁת', 'subs', 'cooking pot', 'marcheshet'),
        'E72465': ('קלה', 'verb', 'roast', 'klh'),
        'E72466': ('גֶּרֶשׂ', 'subs', 'groats', 'geres'),
        'E72467': ('כַּרְמֶל', 'subs', 'new corn', 'karmel'),
        'E72468': ('כֶּסֶל', 'subs', 'loin', 'kesel'),
        'E72469': ('עָצֶה', 'subs', 'spine', 'atseh'),
        'E72470': ('שְׁגָגָה', 'subs', 'error', "sh'gaga"),
        'E72471': ('מָשִׁיחַ', 'subs', 'anointed', 'mashiach'),
        'E72472': ('אשׁם', 'verb', 'do wrong', 'shm'),
        'E72473': ('שֶׁפֶךְ', 'subs', 'rubbish-heap', 'shefech'),
        'E72474': ('שׁגה', 'verb', 'err', 'shgh'),
        'E72475': ('עלם', 'verb', 'hide', 'lm'),
        'E72476': ('שְׂעִירָה', 'subs', 'goat', "s'ira"),
        'E72477': ('טָמֵא', 'adjv', 'unclean', 'tamei'),
        'E72478': ('נְבֵלָה', 'subs', 'corpse', "n'veila"),
        'E72479': ('טֻמְאָה', 'subs', 'uncleanness', 'tuma'),
        'E72480': ('בטא', 'verb', 'speak thoughtlessly', 'vt'),
        'E72481': ('כִּשְׂבָּה', 'subs', 'ewe-lamb', 'kisba'),
        'E72482': ('מעל', 'verb', 'be unfaithful', 'ml'),
        'E72483': ('מַעַל', 'subs', 'unfaithfulness', "ma'al"),
        'E72484': ('שׁגג', 'verb', 'commit error', "shg'g"),
        'E72485': ('עָמִית', 'subs', 'fellow', 'amit'),
        'E72486': ('תְּשׂוּמֶת', 'subs', 'deposit', "t'sumet"),
        'E72487': ('גָּזֵל', 'subs', 'robbery', 'gazeil'),
        'E72488': ('עשׁק', 'verb', 'oppress', 'shk'),
        'E72489': ('גְּזֵלָה', 'subs', 'what is robbed', "g'zeila"),
        'E72490': ('עֹשֶׁק', 'subs', 'oppression', 'oshek'),
        'E72491': ('אַשְׁמָה', 'subs', 'guiltiness', 'ashma'),
        'E72492': ('מֹוקְדָה', 'subs', '[uncertain]', 'movkda'),
        'E72493': ('יקד', 'verb', 'burn', 'ykd'),
        'E72494': ('מַד', 'subs', 'cloth', 'mad'),
        'E72495': ('כבה', 'verb', 'go out', 'chvh'),
        'E72496': ('רבך', 'verb', 'mix', 'rvch'),
        'E72497': ('תֻּפִינִים', 'subs', '[uncertain]', 'tufinim'),
        'E72498': ('חֶרֶשׂ', 'subs', 'clay', 'cheres'),
        'E72499': ('מרק', 'verb', 'polish', 'mrk'),
        'E72500': ('שׁטף', 'verb', 'wash off', 'shtf'),
        'E72501': ('חָרֵב', 'adjv', 'dry', 'chareiv'),
        'E72502': ('תֹּודָה', 'subs', 'thanksgiving', 'tovda'),
        'E72503': ('פִּגּוּל', 'subs', 'unclean meat', 'pigul'),
        'E72504': ('שֶׁקֶץ', 'subs', 'something detestable', 'shekets'),
        'E72505': ('מִשְׁחָה', 'subs', 'portion', 'mishcha'),
        'E72506': ('מְכַסֶּה', 'subs', 'covering', "m'chaseh"),
        'E72507': ('רנן', 'verb', 'cry of joy', "rn'n"),
        'E72508': ('דֹּוד', 'subs', 'beloved one', 'dovd'),
        'E72509': ('פרם', 'verb', 'rent', 'frm'),
        'E72510': ('שֵׁכָר', 'subs', 'strong drink', 'sheichor'),
        'E72511': ('חֹל', 'adjv', 'profane', 'chol'),
        'E72512': ('פְּנִימָה', 'advb', 'within', "p'nima"),
        'E72513': ('פרס', 'verb', 'break', 'frs'),
        'E72514': ('שֶׁסַע', 'subs', 'cleft', 'shesa'),
        'E72515': ('גֵּרָה', 'subs', 'cud', 'geira'),
        'E72516': ('שָׁפָן', 'subs', 'rockbadger', 'shafan'),
        'E72517': ('אַרְנֶבֶת', 'subs', 'hare', 'arnevet'),
        'E72518': ('חֲזִיר', 'subs', 'swine', 'chazir'),
        'E72519': ('גרר', 'verb', 'drag away', "gr'r"),
        'E72520': ('סְנַפִּיר', 'subs', 'fin', "s'napir"),
        'E72521': ('קַשְׂקֶשֶׂת', 'subs', 'scale', 'kaskeset'),
        'E72522': ('שׁקץ', 'verb', 'detest', 'shkts'),
        'E72523': ('פֶּרֶס', 'subs', 'vulture', 'peres'),
        'E72524': ('עָזְנִיָּה', 'subs', 'vulture', 'aznia'),
        'E72525': ('דָּאָה', 'subs', 'red bird', "da'a"),
        'E72526': ('אַיָּה', 'subs', 'black bird', 'aya'),
        'E72527': ('יַעֲנָה', 'subs', '[uncertain]', "ya'ana"),
        'E72528': ('תַּחְמָס', 'subs', 'owl', 'tachmas'),
        'E72529': ('שַׁחַף', 'subs', '[animal]', 'shachaf'),
        'E72530': ('נֵץ', 'subs', 'falcon', 'neits'),
        'E72531': ('כֹּוס', 'subs', 'owlet', 'kovs'),
        'E72532': ('שָׁלָךְ', 'subs', 'cormorant', 'shalach'),
        'E72533': ('יַנְשֹׁוף', 'subs', 'owl', 'yanshovf'),
        'E72534': ('תִּנְשֶׁמֶת', 'subs', 'owl', 'tinshemet'),
        'E72535': ('קָאַת', 'subs', 'owl?', "ka'at"),
        'E72536': ('רָחָם', 'subs', 'vulture', 'racham'),
        'E72537': ('חֲסִידָה', 'subs', 'heron', 'chasida'),
        'E72538': ('אֲנָפָה', 'subs', "[bird's name]", 'anafa'),
        'E72539': ('דּוּכִיפַת', 'subs', 'hoopoe', 'duchifat'),
        'E72540': ('עֲטַלֵּף', 'subs', 'bat', 'ataleif'),
        'E72541': ('נתר', 'verb', 'run', 'ntr'),
        'E72542': ('סָלְעָם', 'subs', 'locust', 'salam'),
        'E72543': ('חַרְגֹּל', 'subs', 'locust', 'chargol'),
        'E72544': ('חָגָב', 'subs', '[uncertain]', 'chagav'),
        'E72545': ('חֹלֶד', 'subs', 'mole-rat', 'choled'),
        'E72546': ('עַכְבָּר', 'subs', 'jerboa', 'achbor'),
        'E72547': ('צָב', 'subs', 'lizard', 'tsav'),
        'E72548': ('אֲנָקָה', 'subs', 'gecko', 'anaka'),
        'E72549': ('כֹּחַ', 'subs', 'lizard', 'koach'),
        'E72550': ('לְטָאָה', 'subs', 'gecko', "l'ta'a"),
        'E72551': ('חֹמֶט', 'subs', 'reptile', 'chomet'),
        'E72552': ('תִּנְשֶׁמֶת', 'subs', 'chameleon', 'tinshemet'),
        'E72553': ('כִּיר', 'subs', 'hearth', 'kir'),
        'E72554': ('זֵרוּעַ', 'subs', 'plant', 'zeirua'),
        'E72555': ('נִדָּה', 'subs', 'menstruation', 'nida'),
        'E72556': ('דוה', 'verb', 'menstruate', 'dvh'),
        'E72557': ('טָהֳרָה', 'subs', 'purification', 'tahara'),
        'E72558': ('מָקֹור', 'subs', 'well', 'makovr'),
        'E72559': ('שְׂאֵת', 'subs', 'swelling', "s'eit"),
        'E72560': ('סַפַּחַת', 'subs', 'scab', 'sapachat'),
        'E72561': ('בַּהֶרֶת', 'subs', '[spot on skin]', 'baheret'),
        'E72562': ('צָרַעַת', 'subs', 'skin-disease', "tsara'at"),
        'E72563': ('עָמֹק', 'adjv', 'deep', 'amok'),
        'E72564': ('פשׂה', 'verb', 'spread', "fs'h"),
        'E72565': ('מִסְפַּחַת', 'subs', 'scab', 'mispachat'),
        'E72566': ('ישׁן', 'verb', 'be old', 'yshn'),
        'E72567': ('אֲדַמְדָּם', 'adjv', 'reddish', 'adamdam'),
        'E72568': ('שָׁפָל', 'adjv', 'low', 'shafal'),
        'E72569': ('כֵּהֶה', 'adjv', 'dim', 'keiheh'),
        'E72570': ('צָרֶבֶת', 'subs', 'scar', 'tsarevet'),
        'E72571': ('מִכְוָה', 'subs', 'scar', 'michva'),
        'E72572': ('זָקָן', 'subs', 'beard', 'zakan'),
        'E72573': ('צָהֹב', 'adjv', 'gleaming', 'tsahov'),
        'E72574': ('נֶתֶק', 'subs', 'scab', 'netek'),
        'E72575': ('שָׁחֹר', 'adjv', 'black', 'shachor'),
        'E72576': ('בקר', 'verb', 'attend to', 'vkr'),
        'E72577': ('בֹּהַק', 'subs', 'skin-disease', 'bohak'),
        'E72578': ('מרט', 'verb', 'pull off', 'mrt'),
        'E72579': ('קֵרֵחַ', 'adjv', 'bald', 'keireiach'),
        'E72580': ('גִּבֵּחַ', 'adjv', 'bald', 'gibeiach'),
        'E72581': ('קָרַחַת', 'subs', 'bald spot', 'karachat'),
        'E72582': ('גַּבַּחַת', 'subs', 'bareness', 'gabachat'),
        'E72583': ('שָׂפָם', 'subs', 'moustache', 'safam'),
        'E72584': ('עטה', 'verb', 'cover', 'th'),
        'E72585': ('בָּדָד', 'subs', 'alone', 'badad'),
        'E72586': ('צֶמֶר', 'subs', 'wool', 'tsemer'),
        'E72587': ('פֵּשֶׁת', 'subs', 'flax', 'peishet'),
        'E72588': ('שְׁתִי', 'subs', 'texture', "sh'ti"),
        'E72589': ('עֵרֶב', 'subs', 'woof', 'eirev'),
        'E72590': ('יְרַקְרַק', 'adjv', 'pale-green', "y'rakrak"),
        'E72591': ('מאר', 'verb', 'hurt', 'mr'),
        'E72592': ('פְּחֶתֶת', 'subs', 'boring', "p'chetet"),
        'E72593': ('אֶרֶז', 'subs', 'cedar', 'erez'),
        'E72594': ('גַּב', 'subs', 'curve', 'gav'),
        'E72595': ('לֹג', 'subs', 'log', 'log'),
        'E72596': ('שְׂמָאלִי', 'adjv', 'lefthand', "s'mali"),
        'E72597': ('שְׁקַעֲרוּרָה', 'subs', 'hollow spot', "sh'ka'arura"),
        'E72598': ('חלץ', 'verb', 'draw off', 'chlts'),
        'E72599': ('קצע', 'verb', 'scrape off', 'kts'),
        'E72600': ('קצה', 'verb', 'cut off', "kts'h"),
        'E72601': ('טוח', 'verb', 'plaster', 'tvch'),
        'E72602': ('זֹוב', 'subs', 'flux', 'zovv'),
        'E72603': ('ריר', 'verb', 'spittle', 'ryr'),
        'E72604': ('חתם', 'verb', 'seal', 'chtm'),
        'E72605': ('רקק', 'verb', 'spit', "rk'k"),
        'E72606': ('מֶרְכָּב', 'subs', 'chariot', 'merkav'),
        'E72607': ('נזר', 'verb', 'dedicate', 'nzr'),
        'E72608': ('דָּוֶה', 'adjv', 'unwell', 'daveh'),
        'E72609': ('צנף', 'verb', 'wrap', 'tsnf'),
        'E72610': ('גֹּורָל', 'subs', 'lot', 'govral'),
        'E72611': ('עֲזָאזֵל', 'nmpr', 'Azazel', 'azazeil'),
        'E72612': ('גַּחַל', 'subs', 'charcoals', 'gachal'),
        'E72613': ('עִתִּי', 'adjv', 'ready', 'iti'),
        'E72614': ('גְּזֵרָה', 'subs', 'dry land', "g'zeira"),
        'E72615': ('שָׂעִיר', 'subs', 'demon', 'sair'),
        'E72616': ('שַׁאֲרָה', 'subs', '[uncertain]', "sha'ara"),
        'E72617': ('זִמָּה', 'subs', 'loose conduct', 'zima'),
        'E72618': ('שְׁכֹבֶת', 'subs', 'copulation', "sh'chovet"),
        'E72619': ('מֹלֶךְ', 'nmpr', 'Moloch', 'molech'),
        'E72620': ('רבע', 'verb', 'lie down', 'rv'),
        'E72621': ('תֶּבֶל', 'subs', 'abomination', 'tevel'),
        'E72622': ('קיא', 'verb', 'vomit', 'ky'),
        'E72623': ('אֱלִיל', 'subs', 'god', 'elil'),
        'E72624': ('קצר', 'verb', 'harvest', 'ktsr'),
        'E72625': ('לֶקֶט', 'subs', 'gleaning', 'leket'),
        'E72626': ('פֶּרֶט', 'subs', 'berries', 'peret'),
        'E72627': ('פְּעֻלָּה', 'subs', 'work', "p'ula"),
        'E72628': ('מִכְשֹׁול', 'subs', 'stumbling block', 'michshovl'),
        'E72629': ('עָוֶל', 'subs', 'injustice', 'avel'),
        'E72630': ('צֶדֶק', 'subs', 'justice', 'tsedek'),
        'E72631': ('רָכִיל', 'subs', 'slanderer', 'rachil'),
        'E72632': ('נטר', 'verb', 'be angry', 'ntr'),
        'E72633': ('כִּלְאַיִם', 'subs', 'doubleness', 'kilayim'),
        'E72634': ('שַׁעַטְנֵז', 'subs', 'cloth', "sha'atneiz"),
        'E72635': ('חרף', 'verb', 'spend autumn', 'chrf'),
        'E72636': ('חֻפְשָׁה', 'subs', 'freedom', 'chufsha'),
        'E72637': ('בִּקֹּרֶת', 'subs', 'punishment', 'bikoret'),
        'E72638': ('חפשׁ', 'verb', 'be free', 'chfsh'),
        'E72639': ('ערל', 'verb', 'leave unharvested', 'rl'),
        'E72640': ('הִלּוּלִים', 'subs', 'exultation', 'hilulim'),
        'E72641': ('נקף', 'verb', 'go around', 'nkf'),
        'E72642': ('שֶׂרֶט', 'subs', 'tattoo', 'seret'),
        'E72643': ('כְּתֹבֶת', 'subs', 'tattooing', "k'tovet"),
        'E72644': ('קַעֲקַע', 'subs', 'tattoo', "ka'aka"),
        'E72645': ('אֹוב', 'subs', 'spirit of dead', 'ovv'),
        'E72646': ('יִדְּעֹנִי', 'subs', 'soothsayer', 'yidoni'),
        'E72647': ('מְשׂוּרָה', 'subs', 'mesura', "m'sura"),
        'E72648': ('מֹאזְנַיִם', 'subs', 'balances', 'moznayim'),
        'E72649': ('רגם', 'verb', 'stone', 'rgm'),
        'E72650': ('חֶסֶד', 'subs', 'shame', 'chesed'),
        'E72651': ('קרח', 'verb', 'make bald', 'krch'),
        'E72652': ('קָרְחָה', 'subs', 'baldness', 'karcha'),
        'E72653': ('שׂרט', 'verb', 'tattoo', 'srt'),
        'E72654': ('שָׂרֶטֶת', 'subs', 'tattoo', 'saretet'),
        'E72655': ('בְּתוּלִים', 'subs', 'virginity', "b'tulim"),
        'E72656': ('מאוּם', 'subs', 'blemish', 'mum'),
        'E72657': ('פִּסֵּחַ', 'adjv', 'lame', 'piseiach'),
        'E72658': ('חרם', 'verb', 'mutilate', 'chrm'),
        'E72659': ('שׂרע', 'verb', 'deform', 'sr'),
        'E72660': ('שֶׁבֶר', 'subs', 'breaking', 'shever'),
        'E72661': ('גִּבֵּן', 'adjv', 'hunched', 'gibein'),
        'E72662': ('תְּבַלֻּל', 'subs', 'spot', "t'valul"),
        'E72663': ('גָּרָב', 'subs', 'mange', 'garav'),
        'E72664': ('יַלֶּפֶת', 'subs', 'ringworm', 'yalefet'),
        'E72665': ('מָרֹוחַ', 'adjv', 'crushed', 'marovach'),
        'E72666': ('אֶשֶׁךְ', 'subs', 'testicle', 'eshech'),
        'E72667': ('עַוֶּרֶת', 'subs', 'blindness', 'aueret'),
        'E72668': ('שָׁבוּר', 'subs', 'fracture', 'shavur'),
        'E72669': ('חָרוּץ', 'adjv', 'cut', 'charuts'),
        'E72670': ('יַבֶּלֶת', 'subs', 'wart', 'yabelet'),
        'E72671': ('קלט', 'verb', 'stunt', 'klt'),
        'E72672': ('מעך', 'verb', 'squeeze', 'mch'),
        'E72673': ('כתת', 'verb', 'crush', "cht't"),
        'E72674': ('נתק', 'verb', 'pull off', 'ntk'),
        'E72675': ('מָשְׁחָת', 'subs', 'corruption', 'mashchat'),
        'E72676': ('עֹמֶר', 'subs', 'cut grain', 'omer'),
        'E72677': ('קָלִי', 'subs', 'parched grain', 'kali'),
        'E72678': ('תְּרוּעָה', 'subs', 'shouting', "t'rua"),
        'E72679': ('עֲצֶרֶת', 'subs', 'assembly', 'atseret'),
        'E72680': ('הָדָר', 'subs', 'ornament', 'hadar'),
        'E72681': ('כִּפָּה', 'subs', 'shoot', 'kipa'),
        'E72682': ('עָנָף', 'subs', 'branches', 'anaf'),
        'E72683': ('עָבֹות', 'adjv', 'branchy', 'avovt'),
        'E72684': ('עֲרָבָה', 'subs', 'poplar', 'arava'),
        'E72685': ('מַעֲרֶכֶת', 'subs', 'row', "ma'arechet"),
        'E72686': ('יִשְׂרְאֵלִי', 'adjv', 'Israelite', 'yisreili'),
        'E72687': ('שְׁלֹמִית', 'nmpr', 'Shelomith', "sh'lomit"),
        'E72688': ('דִּבְרִי', 'nmpr', 'Dibri', 'divri'),
        'E72689': ('פרשׁ', 'verb', 'explain', 'frsh'),
        'E72690': ('זמר', 'verb', 'prune', 'zmr'),
        'E72691': ('סָפִיחַ', 'subs', 'growth', 'safiach'),
        'E72692': ('בצר', 'verb', 'gather grapes', 'vtsr'),
        'E72693': ('דְּרֹור', 'subs', 'release', "d'rovr"),
        'E72694': ('מִמְכָּר', 'subs', 'what is sold', 'mimkar'),
        'E72695': ('יָשָׁן', 'adjv', 'old', 'yashan'),
        'E72696': ('תְּשִׁיעִי', 'adjv', 'ninth', "t'shi'i"),
        'E72697': ('צְמִתֻת', 'subs', 'forfeiture', "ts'mitut"),
        'E72698': ('גְּאֻלָּה', 'subs', 'right of buying back', "g'ula"),
        'E72699': ('מוך', 'verb', 'grow poor', 'mvch'),
        'E72700': ('מִגְרָשׁ', 'subs', 'pasture', 'migrash'),
        'E72701': ('מוט', 'verb', 'totter', 'mvt'),
        'E72702': ('תַּרְבִּית', 'subs', 'interest', 'tarbit'),
        'E72703': ('מַרְבִּית', 'subs', 'great number', 'marbit'),
        'E72704': ('מִמְכֶּרֶת', 'subs', 'sale', 'mimkeret'),
        'E72705': ('עֵקֶר', 'subs', 'offspring', 'eiker'),
        'E72706': ('מַשְׂכִּית', 'subs', 'figure', 'maskit'),
        'E72707': ('יְבוּל', 'subs', 'produce', "y'vul"),
        'E72708': ('דַּיִשׁ', 'subs', 'threshing', 'dayish'),
        'E72709': ('בָּצִיר', 'subs', 'vintage', 'batsir'),
        'E72710': ('געל', 'verb', 'abhor', 'gl'),
        'E72711': ('מֹוטָה', 'subs', 'yoke-bar', 'movta'),
        'E72712': ('קֹומְמִיּוּת', 'subs', 'uprightness', "kovm'miut"),
        'E72713': ('מאס', 'verb', 'retract', 'ms'),
        'E72714': ('בֶּהָלָה', 'subs', 'terror', 'behala'),
        'E72715': ('שַׁחֶפֶת', 'subs', 'consumption', 'shachefet'),
        'E72716': ('קַדַּחַת', 'subs', 'inflammation', 'kadachat'),
        'E72717': ('דוב', 'verb', 'pine away', 'dvv'),
        'E72718': ('רִיק', 'subs', 'emptiness', 'rik'),
        'E72719': ('יסר', 'verb', 'admonish', 'ysr'),
        'E72720': ('נְחוּשָׁה', 'subs', 'bronze', "n'chusha"),
        'E72721': ('קְרִי', 'subs', 'opposition', "k'ri"),
        'E72722': ('מַכָּה', 'subs', 'blow', 'maka'),
        'E72723': ('שׁמם', 'verb', 'be desolate', "shm'm"),
        'E72724': ('נָקָם', 'subs', 'vengeance', 'nakam'),
        'E72725': ('בָּמָה', 'subs', 'high place', 'bama'),
        'E72726': ('חַמָּן', 'subs', 'incense-stand', 'chaman'),
        'E72727': ('גִּלּוּלִים', 'subs', 'idols', 'gilulim'),
        'E72728': ('חָרְבָּה', 'subs', 'ruin', 'charba'),
        'E72729': ('רצה', 'verb', 'pay off', "rts'h"),
        'E72730': ('מֹרֶךְ', 'subs', 'timidity', 'morech'),
        'E72731': ('נדף', 'verb', 'scatter', 'ndf'),
        'E72732': ('מְנוּסָה', 'subs', 'flight', "m'nusa"),
        'E72733': ('כשׁל', 'verb', 'stumble', 'chshl'),
        'E72734': ('תְּקוּמָה', 'subs', 'power to stand', "t'kuma"),
        'E72735': ('מקק', 'verb', 'putrefy', "mk'k"),
        'E72736': ('כנע', 'verb', 'be humble', 'chn'),
        'E72737': ('מור', 'verb', 'exchange', 'mvr'),
        'E72738': ('תְּמוּרָה', 'subs', 'exchange', "t'mura"),
        'E72739': ('חֵרֶם', 'subs', 'ban', 'cheirem'),
        'E72740': ('בכר', 'verb', 'be first-born', 'vchr'),
        'E72741': ('אֱלִיצוּר', 'nmpr', 'Elizur', 'elitsur'),
        'E72742': ('שְׁדֵיאוּר', 'nmpr', 'Shedeur', "sh'deiur"),
        'E72743': ('שְׁלֻמִיאֵל', 'nmpr', 'Shelumiel', "sh'lumieil"),
        'E72744': ('צוּרִישַׁדַּי', 'nmpr', 'Zurishaddai', 'tsurishaday'),
        'E72745': ('נְתַנְאֵל', 'nmpr', 'Nethanel', "n'taneil"),
        'E72746': ('צוּעָר', 'nmpr', 'Zuar', 'tsuar'),
        'E72747': ('אֱלִיאָב', 'nmpr', 'Eliab', 'eliav'),
        'E72748': ('חֵלֹן', 'nmpr', 'Helon', 'cheilon'),
        'E72749': ('אֱלִישָׁמָע', 'nmpr', 'Elishama', 'elishama'),
        'E72750': ('עַמִּיהוּד', 'nmpr', 'Ammihud', 'amihud'),
        'E72751': ('גַּמְלִיאֵל', 'nmpr', 'Gamaliel', 'gamlieil'),
        'E72752': ('פְּדָהצוּר', 'nmpr', 'Pedahzur', "p'dahtsur"),
        'E72753': ('אֲבִידָן', 'nmpr', 'Abidan', 'avidan'),
        'E72754': ('גִּדְעֹנִי', 'nmpr', 'Gideoni', 'gidoni'),
        'E72755': ('אֲחִיעֶזֶר', 'nmpr', 'Ahiezer', 'achiezer'),
        'E72756': ('עַמִּישַׁדָּי', 'nmpr', 'Ammishaddai', 'amishaday'),
        'E72757': ('פַּגְעִיאֵל', 'nmpr', 'Pagiel', 'pagieil'),
        'E72758': ('עָכְרָן', 'nmpr', 'Ocran', 'achran'),
        'E72759': ('אֶלְיָסָף', 'nmpr', 'Eliasaph', 'elyasaf'),
        'E72760': ('דְּעוּאֵל', 'nmpr', 'Deuel', "d'ueil"),
        'E72761': ('אֲחִירַע', 'nmpr', 'Ahira', 'achira'),
        'E72762': ('עֵינָן', 'nmpr', 'Enan', 'einan'),
        'E72763': ('אֶלֶף', 'subs', 'group of thousand', 'elef'),
        'E72764': ('קָרֵב', 'adjv', 'approaching', 'kareiv'),
        'E72765': ('דֶּגֶל', 'subs', 'banner', 'degel'),
        'E72766': ('קֶצֶף', 'subs', 'anger', 'ketsef'),
        'E72767': ('לִבְנִי', 'adjv', 'Libnite', 'livni'),
        'E72768': ('שִׁמְעִי', 'adjv', 'Shimeite', 'shimi'),
        'E72769': ('גֵּרְשֻׁנִּי', 'adjv', 'Gershonite', 'geirshuni'),
        'E72770': ('לָאֵל', 'nmpr', 'Lael', 'laeil'),
        'E72771': ('עַמְרָמִי', 'adjv', 'Amramite', 'amrami'),
        'E72772': ('יִצְהָרִי', 'adjv', 'Izharite', "yits'hari"),
        'E72773': ('חֶבְרֹנִי', 'adjv', 'Hebronite', 'chevroni'),
        'E72774': ('עָזִּיאֵלִי', 'adjv', 'Uzzielite', 'azieili'),
        'E72775': ('קְהָתִי', 'adjv', 'Kohathite', "k'hati"),
        'E72776': ('אֱלִיצָפָן', 'nmpr', 'Elizaphan', 'elitsafan'),
        'E72777': ('פְּקֻדָּה', 'subs', 'commission', "p'kuda"),
        'E72778': ('מַחְלִי', 'adjv', 'Mahlite', 'machli'),
        'E72779': ('מוּשִׁי', 'adjv', 'Mushite', 'mushi'),
        'E72780': ('צוּרִיאֵל', 'nmpr', 'Zuriel', 'tsurieil'),
        'E72781': ('אֲבִיחַיִל', 'nmpr', 'Abihail', 'avichayil'),
        'E72782': ('פְּדוּיִם', 'subs', 'ransom', "p'duyim"),
        'E72783': ('פִּדְיֹום', 'subs', 'ransom', 'pidovm'),
        'E72784': ('כָּסוּי', 'subs', 'covering', 'kasuy'),
        'E72785': ('מֹוט', 'subs', 'bar', 'movt'),
        'E72786': ('שָׁרֵת', 'subs', 'cultic service', 'shareit'),
        'E72787': ('שׂטה', 'verb', 'turn aside', 'sth'),
        'E72788': ('קִנְאָה', 'subs', 'jealousy', 'kina'),
        'E72789': ('קַרְקַע', 'subs', 'floor', 'karka'),
        'E72790': ('צָבֶה', 'adjv', 'swollen', 'tsaveh'),
        'E72791': ('צבה', 'verb', 'swell', 'tsvh'),
        'E72792': ('אָמֵן', 'intj', 'surely', 'amein'),
        'E72793': ('חֹמֶץ', 'subs', 'vinegar', 'chomets'),
        'E72794': ('מִשְׁרָה', 'subs', 'liquid', 'mishra'),
        'E72795': ('יָבֵשׁ', 'adjv', 'dry', 'yaveish'),
        'E72796': ('חַרְצָן', 'subs', 'grape', 'chartsan'),
        'E72797': ('זָג', 'subs', 'skin of grape', 'zag'),
        'E72798': ('תַּעַר', 'subs', 'knife', "ta'ar"),
        'E72799': ('פֶּרַע', 'subs', 'loose hair', 'pera'),
        'E72800': ('פֶּתַע', 'subs', 'instant', 'peta'),
        'E72801': ('פִּתְאֹם', 'advb', 'suddenly', 'pitom'),
        'E72802': ('צָב', 'subs', 'wagon', 'tsav'),
        'E72803': ('חֲנֻכָּה', 'subs', 'dedication', 'chanuka'),
        'E72804': ('פִּטְרָה', 'subs', 'first-born', 'pitra'),
        'E72805': ('חֲצֹצְרָה', 'subs', 'clarion', 'chatsotsra'),
        'E72806': ('רוע', 'verb', 'shout', 'rv'),
        'E72807': ('חֹבָב', 'nmpr', 'Hobab', 'chovav'),
        'E72808': ('תור', 'verb', 'spy', 'tvr'),
        'E72809': ('אנן', 'verb', 'complain', "n'n"),
        'E72810': ('שׁקע', 'verb', 'collapse', 'shk'),
        'E72811': ('תַּבְעֵרָה', 'nmpr', 'Taberah', 'taveira'),
        'E72812': ('אֲסַפְסֻף', 'subs', 'cluster', 'asafsuf'),
        'E72813': ('אוה', 'verb', 'wish', 'vh'),
        'E72814': ('קִשֻּׁאָה', 'subs', 'cucumber', 'kishua'),
        'E72815': ('אֲבַטִּיחַ', 'subs', 'water-melon', 'avatiach'),
        'E72816': ('חָצִיר', 'subs', 'leek', 'chatsir'),
        'E72817': ('בָּצָל', 'subs', 'onion', 'batsal'),
        'E72818': ('שׁוּמִים', 'subs', 'garlic', 'shumim'),
        'E72819': ('שׁוט', 'verb', 'rove about', 'shvt'),
        'E72820': ('דוך', 'verb', 'pound', 'dvch'),
        'E72821': ('מְדֹכָה', 'subs', 'mortar', "m'docho"),
        'E72822': ('פָּרוּר', 'subs', 'cooking pot', 'parur'),
        'E72823': ('לָשָׁד', 'subs', 'dainty', 'lashad'),
        'E72824': ('זָרָא', 'subs', 'nausea', 'zara'),
        'E72825': ('קצר', 'verb', 'be short', 'ktsr'),
        'E72826': ('נבא', 'verb', 'speak as prophet', 'nv'),
        'E72827': ('אֶלְדָּד', 'nmpr', 'Eldad', 'eldad'),
        'E72828': ('מֵידָד', 'nmpr', 'Medad', 'meidad'),
        'E72829': ('בְּחֻרִים', 'subs', 'youth', "b'churim"),
        'E72830': ('גוז', 'verb', 'pass along', 'gvz'),
        'E72831': ('שׁטח', 'verb', 'spread', "sht'ch"),
        'E72832': ('קִבְרֹות הַתַּאֲוָה',
                   'nmpr',
                   'Kibroth Hattaavah',
                   "kivrovt hata'ava"),
        'E72833': ('חֲצֵרֹות', 'nmpr', 'Hazeroth', 'chatseirovt'),
        'E72834': ('כֻּשִׁי', 'adjv', 'Ethiopian', 'kushi'),
        'E72835': ('עָנָו', 'adjv', 'humble', 'anav'),
        'E72836': ('חִידָה', 'subs', 'riddle', 'chida'),
        'E72837': ('יאל', 'verb', 'be foolish', 'yl'),
        'E72838': ('ירק', 'verb', 'spit', 'yrk'),
        'E72839': ('כלם', 'verb', 'humiliate', 'chlm'),
        'E72840': ('שַׁמּוּעַ', 'nmpr', 'Shammua', 'shamua'),
        'E72841': ('זַכּוּר', 'nmpr', 'Zaccur', 'zakur'),
        'E72842': ('שָׁפָט', 'nmpr', 'Shaphat', 'shafat'),
        'E72843': ('כָּלֵב', 'nmpr', 'Caleb', 'kaleiv'),
        'E72844': ('יְפֻנֶּה', 'nmpr', 'Jephunneh', "y'funeh"),
        'E72845': ('יִגְאָל', 'nmpr', 'Igal', 'yigal'),
        'E72846': ('הֹושֵׁעַ', 'nmpr', 'Hosea', 'hovsheia'),
        'E72847': ('פַּלְטִי', 'nmpr', 'Palti', 'palti'),
        'E72848': ('רָפוּא', 'nmpr', 'Raphu', 'rafu'),
        'E72849': ('גַּדִּיאֵל', 'nmpr', 'Gaddiel', 'gadieil'),
        'E72850': ('סֹודִי', 'nmpr', 'Sodi', 'sovdi'),
        'E72851': ('גַּדִּי', 'nmpr', 'Gaddi', 'gadi'),
        'E72852': ('סוּסִי', 'nmpr', 'Susi', 'susi'),
        'E72853': ('עַמִּיאֵל', 'nmpr', 'Ammiel', 'amieil'),
        'E72854': ('גְּמַלִּי', 'nmpr', 'Gemalli', "g'mali"),
        'E72855': ('סְתוּר', 'nmpr', 'Sethur', "s'tur"),
        'E72856': ('מִיכָאֵל', 'nmpr', 'Michael', 'michoeil'),
        'E72857': ('נַחְבִּי', 'nmpr', 'Nahbi', 'nachbi'),
        'E72858': ('וָפְסִי', 'nmpr', 'Vophsi', 'vafsi'),
        'E72859': ('גְּאוּאֵל', 'nmpr', 'Geuel', "g'ueil"),
        'E72860': ('מָכִי', 'nmpr', 'Maki', 'machi'),
        'E72861': ('רָפֶה', 'adjv', 'slack', 'rafeh'),
        'E72862': ('מִבְצָר', 'subs', 'fortification', 'mivtsar'),
        'E72863': ('רָזֶה', 'adjv', 'lean', 'razeh'),
        'E72864': ('צִן', 'nmpr', 'Zin', 'tsin'),
        'E72865': ('רְחֹב', 'nmpr', 'Rehob', "r'chov"),
        'E72866': ('חֲמָת', 'nmpr', 'Hamath', 'chamat'),
        'E72867': ('אֲחִימַן', 'nmpr', 'Ahiman', 'achiman'),
        'E72868': ('שֵׁשַׁי', 'nmpr', 'Sheshai', 'sheishay'),
        'E72869': ('תַּלְמַי', 'nmpr', 'Talmai', 'talmay'),
        'E72870': ('עֲנָק', 'subs', 'neck', 'anak'),
        'E72871': ('צֹעַן', 'nmpr', 'Zoan', 'tsoan'),
        'E72872': ('אֶשְׁכֹּל', 'nmpr', '[valley of grapes]', 'eshkol'),
        'E72873': ('זְמֹורָה', 'subs', 'shoot', "z'movra"),
        'E72874': ('אֶפֶס', 'subs', 'end', 'efes'),
        'E72875': ('בָּצוּר', 'adjv', 'fortified', 'batsur'),
        'E72876': ('הסה', 'verb', 'be still', "hs'h"),
        'E72877': ('בַּז', 'subs', 'spoiling', 'baz'),
        'E72878': ('נאץ', 'verb', 'contemn', 'nts'),
        'E72879': ('גֹּדֶל', 'subs', 'greatness', 'godel'),
        'E72880': ('זְנוּת', 'subs', 'fornication', "z'nut"),
        'E72881': ('תְּנוּאָה', 'subs', 'opposition', "t'nua"),
        'E72882': ('עפל', 'verb', 'be heedless', 'fl'),
        'E72883': ('חָרְמָה', 'nmpr', 'Hormah', 'charma'),
        'E72884': ('שְׁלִישִׁית', 'subs', 'third part', "sh'lishit"),
        'E72885': ('עֲרִיסָה', 'subs', 'dough', 'arisa'),
        'E72886': ('חֶטְאָה', 'subs', 'error', 'cheta'),
        'E72887': ('גדף', 'verb', 'blaspheme', 'gdf'),
        'E72888': ('צִיצִת', 'subs', 'tuft', 'tsitsit'),
        'E72889': ('דָּתָן', 'nmpr', 'Dathan', 'datan'),
        'E72890': ('אֲבִירָם', 'nmpr', 'Abiram', 'aviram'),
        'E72891': ('אֹון', 'nmpr', 'On', 'ovn'),
        'E72892': ('פֶּלֶת', 'nmpr', 'Peleth', 'pelet'),
        'E72893': ('קָרִיא', 'adjv', 'summoned', 'kari'),
        'E72894': ('שׂרר', 'verb', 'rule', "sr'r"),
        'E72895': ('נקר', 'verb', 'bore out', 'nkr'),
        'E72896': ('בְּרִיאָה', 'subs', 'what is created', "b'ria"),
        'E72897': ('רִקֻּעַ', 'adjv', 'beaten', 'rikua'),
        'E72898': ('רמם', 'verb', 'rise', "rm'm"),
        'E72899': ('מְהֵרָה', 'subs', 'haste', "m'heira"),
        'E72900': ('צוץ', 'verb', 'blossom', 'tsvts'),
        'E72901': ('מְרִי', 'subs', 'rebellion', "m'ri"),
        'E72902': ('מָשְׁחָה', 'subs', 'portion', 'mashcha'),
        'E72903': ('יִצְהָר', 'subs', 'oil', "yits'har"),
        'E72904': ('חֵלֶף', 'subs', 'return', 'cheilef'),
        'E72905': ('יֶקֶב', 'subs', 'pit', 'yekev'),
        'E72906': ('צָמִיד', 'subs', 'cover', 'tsamid'),
        'E72907': ('סֶלַע', 'subs', 'rock', 'sela'),
        'E72908': ('מְסִלָּה', 'subs', 'highway', "m'sila"),
        'E72909': ('מֶכֶר', 'subs', 'price', 'mecher'),
        'E72910': ('הֹר', 'nmpr', 'Mount Hor', 'hor'),
        'E72911': ('עֲרָד', 'nmpr', 'Arad', 'arad'),
        'E72912': ('אֲתָרִים', 'nmpr', 'Atharim', 'atarim'),
        'E72913': ('קְלֹקֵל', 'subs', 'starvation-bread', "k'lokeil"),
        'E72914': ('שָׂרָף', 'subs', 'serpent', 'saraf'),
        'E72915': ('אֹבֹת', 'nmpr', 'Oboth', 'ovot'),
        'E72916': ('עִיֵּי הָעֲבָרִים', 'nmpr', 'Iye Abarim', "iei ha'avarim"),
        'E72917': ('זֶרֶד', 'nmpr', 'Zered', 'zered'),
        'E72918': ('אַרְנֹון', 'nmpr', 'Arnon', 'arnovn'),
        'E72919': ('וָהֵב', 'nmpr', 'Waheb', 'vaheiv'),
        'E72920': ('סוּפָה', 'nmpr', 'Suphah', 'sufa'),
        'E72921': ('אָשֵׁד', 'subs', 'slope', 'asheid'),
        'E72922': ('עָר', 'nmpr', 'Ar', 'ar'),
        'E72923': ('בְּאֵר', 'nmpr', 'Beer', "b'eir"),
        'E72924': ('מַתָּנָה', 'nmpr', 'Mattanah', 'matana'),
        'E72925': ('נַחֲלִיאֵל', 'nmpr', 'Nahaliel', 'nachalieil'),
        'E72926': ('בָּמֹות', 'nmpr', 'Bamoth', 'bamovt'),
        'E72927': ('גַּיְא', 'subs', 'valley', 'gay'),
        'E72928': ('פִּסְגָּה', 'nmpr', 'Pisgah', 'pisga'),
        'E72929': ('יְשִׁימֹן', 'nmpr', 'Jeshimon', "y'shimon"),
        'E72930': ('סִיחֹון', 'nmpr', 'Sihon', 'sichovn'),
        'E72931': ('יָהְצָה', 'nmpr', 'Jahzah', 'yahtsa'),
        'E72932': ('חֶשְׁבֹּון', 'nmpr', 'Heshbon', 'cheshbovn'),
        'E72933': ('משׁל', 'verb', 'say proverb', 'mshl'),
        'E72934': ('לֶהָבָה', 'subs', 'flame', 'lehava'),
        'E72935': ('קִרְיָה', 'subs', 'town', 'kira'),
        'E72936': ('אֹוי', 'intj', 'woe', 'ovy'),
        'E72937': ('כְּמֹושׁ', 'nmpr', 'Chemosh', "k'movsh"),
        'E72938': ('פָּלֵיט', 'subs', 'escaped one', 'paleit'),
        'E72939': ('שְׁבִית', 'subs', 'captivity', "sh'vit"),
        'E72940': ('דִּיבֹן', 'nmpr', 'Dibon', 'divon'),
        'E72941': ('נֹפַח', 'nmpr', 'Nophah', 'nofach'),
        'E72942': ('מֵידְבָא', 'nmpr', 'Medeba', 'meidva'),
        'E72943': ('יַעְזֵר', 'nmpr', 'Jazer', 'yazeir'),
        'E72944': ('לכד', 'verb', 'seize', 'lchd'),
        'E72945': ('בָּשָׁן', 'nmpr', 'Bashan', 'bashan'),
        'E72946': ('עֹוג', 'nmpr', 'Og', 'ovg'),
        'E72947': ('אֶדְרֶעִי', 'nmpr', 'Edrei', 'edrei'),
        'E72948': ('שָׂרִיד', 'subs', 'survivor', 'sarid'),
        'E72949': ('עֲרָבָה', 'subs', 'desert', 'arava'),
        'E72950': ('יְרִיחֹוֿ יְרִחֹו', 'nmpr', 'Jericho', "y'richov y'richov"),
        'E72951': ('בָּלָק', 'nmpr', 'Balak', 'balak'),
        'E72952': ('צִפֹּור', 'nmpr', 'Zippor', 'tsipovr'),
        'E72953': ('גור', 'verb', 'be afraid', 'gvr'),
        'E72954': ('לחך', 'verb', 'eat', 'lchch'),
        'E72955': ('בִּלְעָם', 'nmpr', '[prophet]', 'bilam'),
        'E72956': ('פְּתֹור', 'nmpr', 'Pethor', "p'tovr"),
        'E72957': ('קֶסֶם', 'subs', 'divination', 'kesem'),
        'E72958': ('קבב', 'verb', 'curse', "kv'v"),
        'E72959': ('שָׂטָן', 'subs', 'adversary', 'satan'),
        'E72960': ('שׁלף', 'verb', 'draw', 'shlf'),
        'E72961': ('מִשְׁעֹול', 'subs', 'hollow way', 'mishovl'),
        'E72962': ('גָּדֵר', 'subs', 'wall', 'gadeir'),
        'E72963': ('צַר', 'adjv', 'narrow', 'tsar'),
        'E72964': ('סכן', 'verb', 'serve', "s'chn"),
        'E72965': ('ירט', 'verb', 'throw', 'yrt'),
        'E72966': ('קִרְיַת חֻצֹות', 'nmpr', 'Kiriath Huzoth', 'kirat chutsovt'),
        'E72967': ('בָּמֹות בַּעַל', 'nmpr', 'Bamoth Baal', "bamovt ba'al"),
        'E72968': ('שְׁפִי', 'subs', 'track', "sh'fi"),
        'E72969': ('מָשָׁל', 'subs', 'proverb', 'mashal'),
        'E72970': ('זעם', 'verb', 'curse', 'zm'),
        'E72971': ('שׁור', 'verb', 'regard', 'shvr'),
        'E72972': ('רֹבַע', 'subs', 'dust', 'rova'),
        'E72973': ('כזב', 'verb', 'lie', 'chzv'),
        'E72974': ('אָוֶן', 'subs', 'wickedness', 'aven'),
        'E72975': ('תֹּועָפֹות', 'subs', 'horns', 'tovafovt'),
        'E72976': ('רְאֵם', 'subs', 'wild oxen', "r'eim"),
        'E72977': ('נַחַשׁ', 'subs', 'spell', 'nachash'),
        'E72978': ('אֲרִי', 'subs', 'lion', 'ari'),
        'E72979': ('ישׁר', 'verb', 'be right', 'yshr'),
        'E72980': ('פְּעֹור', 'nmpr', 'Peor', "p'ovr"),
        'E72981': ('שְׁתֻם', 'subs', '[uncertain]', "sh'tum"),
        'E72982': ('אֵמֶר', 'subs', 'word', 'eimer'),
        'E72983': ('גַּנָּה', 'subs', 'garden', 'gana'),
        'E72984': ('אָהָל', 'subs', 'aloe', 'ahal'),
        'E72985': ('דְּלִי', 'subs', 'bucket', "d'li"),
        'E72986': ('אֲגַג', 'nmpr', 'Agag', 'agag'),
        'E72987': ('מַלְכוּת', 'subs', 'kingship', 'malchut'),
        'E72988': ('גרם', 'verb', 'gnaw', 'grm'),
        'E72989': ('מחץ', 'verb', 'break', 'mchts'),
        'E72990': ('ספק', 'verb', 'clap hands', 'sfk'),
        'E72991': ('דרך', 'verb', 'tread', 'drch'),
        'E72992': ('קַרְקַר', 'subs', '[uncertain]', 'karkar'),
        'E72993': ('שֵׁת', 'subs', 'defiance', 'sheit'),
        'E72994': ('יְרֵשָׁה', 'subs', 'possession', "y'reisha"),
        'E72995': ('קַיִן', 'nmpr', 'Kain', 'kayin'),
        'E72996': ('צִי', 'subs', 'ship', 'tsi'),
        'E72997': ('שִׁטִּים', 'nmpr', 'Shittim', 'shitim'),
        'E72998': ('צמד', 'verb', 'attach', 'tsmd'),
        'E72999': ('בַּעַל פְּעֹור', 'nmpr', 'Baal Peor', "ba'al p'ovr"),
        'E73000': ('רֹמַח', 'subs', 'lance', 'romach'),
        'E73001': ('קֻבָּה', 'subs', "women's room", 'kuba'),
        'E73002': ('דקר', 'verb', 'pierce', 'dkr'),
        'E73003': ('קֵבָה', 'subs', 'rennet-bag', 'keiva'),
        'E73004': ('זִמְרִי', 'nmpr', 'Zimri', 'zimri'),
        'E73005': ('סָלוּא', 'nmpr', 'Salu', 'salu'),
        'E73006': ('שִׁמְעֹנִי', 'adjv', 'Simeonite', 'shimoni'),
        'E73007': ('כָּזְבִּי', 'nmpr', 'Cozbi', 'kazbi'),
        'E73008': ('צוּר', 'nmpr', 'Zur', 'tsur'),
        'E73009': ('נֵכֶל', 'subs', 'cunning', 'neichel'),
        'E73010': ('חֲנֹכִי', 'adjv', 'Hanochite', 'chanochi'),
        'E73011': ('פַּלֻּאִי', 'adjv', 'Palluite', 'palui'),
        'E73012': ('חֶצְרֹונִי', 'adjv', 'Hezronite', 'chetsrovni'),
        'E73013': ('כַּרְמִי', 'adjv', 'Carmite', 'karmi'),
        'E73014': ('רְאוּבֵנִי', 'adjv', 'Reubenite', "r'uveini"),
        'E73015': ('נְמוּאֵל', 'nmpr', 'Nemuel', "n'mueil"),
        'E73016': ('נְמוּאֵלִי', 'adjv', 'Nemuelite', "n'mueili"),
        'E73017': ('יָמִינִי', 'adjv', 'Jaminite', 'yamini'),
        'E73018': ('יָכִינִי', 'adjv', 'Jakinite', 'yachini'),
        'E73019': ('זַרְחִי', 'adjv', 'Zerahite', 'zarchi'),
        'E73020': ('שָׁאוּלִי', 'adjv', 'Shaulite', 'shauli'),
        'E73021': ('צְפֹון', 'nmpr', 'Zephon', "ts'fovn"),
        'E73022': ('צְפֹונִי', 'adjv', 'Zephonite', "ts'fovni"),
        'E73023': ('חַגִּי', 'adjv', 'Haggite', 'chagi'),
        'E73024': ('שׁוּנִי', 'adjv', 'Shunite', 'shuni'),
        'E73025': ('אָזְנִי', 'nmpr', 'Ozni', 'azni'),
        'E73026': ('אָזְנִי', 'adjv', 'Oznite', 'azni'),
        'E73027': ('עֵרִי', 'adjv', 'Erite', 'eiri'),
        'E73028': ('אֲרֹוד', 'nmpr', 'Arodi', 'arovd'),
        'E73029': ('אֲרֹודִי', 'adjv', 'Arodite', 'arovdi'),
        'E73030': ('אַרְאֵלִי', 'adjv', 'Arelite', 'areili'),
        'E73031': ('שֵׁלָנִי', 'adjv', 'Shelanite', 'sheilani'),
        'E73032': ('פַּרְצִי', 'adjv', 'Perezite', 'partsi'),
        'E73033': ('חָמוּלִי', 'adjv', 'Hamulite', 'chamuli'),
        'E73034': ('תֹּולָעִי', 'adjv', 'Tolaite', 'tovlai'),
        'E73035': ('פּוּנִי', 'adjv', 'Puite', 'puni'),
        'E73036': ('יָשׁוּב', 'nmpr', 'Jashub', 'yashuv'),
        'E73037': ('יָשֻׁבִי', 'adjv', 'Jashubite', 'yashuvi'),
        'E73038': ('שִׁמְרֹנִי', 'adjv', 'Shimronite', 'shimroni'),
        'E73039': ('סַרְדִּי', 'adjv', 'Seredite', 'sardi'),
        'E73040': ('אֵלֹנִי', 'adjv', 'Elonite', 'eiloni'),
        'E73041': ('יַחְלְאֵלִי', 'adjv', 'Jahleelite', 'yachleili'),
        'E73042': ('זְבוּלֹנִי', 'adjv', 'Zebulunite', "z'vuloni"),
        'E73043': ('מָכִירִי', 'adjv', 'Makirite', 'machiri'),
        'E73044': ('גִּלְעָדִי', 'adjv', 'Gileadite', 'giladi'),
        'E73045': ('אִיעֶזֶר', 'nmpr', 'Iezer', 'iezer'),
        'E73046': ('אִיעֶזְרִי', 'adjv', 'Iezerite', 'iezri'),
        'E73047': ('חֵלֶק', 'nmpr', 'Helek', 'cheilek'),
        'E73048': ('חֶלְקִי', 'adjv', 'Helekite', 'chelki'),
        'E73049': ('אַשְׂרִיאֵל', 'nmpr', 'Asriel', 'asrieil'),
        'E73050': ('אַשְׂרִיאֵלִי', 'adjv', 'Asrielite', 'asrieili'),
        'E73051': ('שֶׁכֶם', 'nmpr', 'Shechem', 'shechem'),
        'E73052': ('שִׁכְמִי', 'adjv', 'Shechemite', 'shichmi'),
        'E73053': ('שְׁמִידָע', 'nmpr', 'Shemida', "sh'mida"),
        'E73054': ('שְׁמִידָעִי', 'adjv', 'Shemidaite', "sh'midai"),
        'E73055': ('חֵפֶר', 'nmpr', '[son of Gilead]', 'cheifer'),
        'E73056': ('חֶפְרִי', 'adjv', 'Hepherite', 'chefri'),
        'E73057': ('צְלָפְחָד', 'nmpr', 'Zelophehad', "ts'lafchad"),
        'E73058': ('מַחְלָה', 'nmpr', 'Mahlah', 'machla'),
        'E73059': ('נֹעָה', 'nmpr', 'Noah', 'noa'),
        'E73060': ('חָגְלָה', 'nmpr', 'Hoglah', 'chagla'),
        'E73061': ('תִּרְצָה', 'nmpr', 'Tirzah', 'tirtsa'),
        'E73062': ('שׁוּתֶלַח', 'nmpr', 'Shuthelah', 'shutelach'),
        'E73063': ('שֻׁתַלְחִי', 'adjv', 'Shuthelahite', 'shutalchi'),
        'E73064': ('בַּכְרִי', 'adjv', 'Bekerite', 'bachri'),
        'E73065': ('תַּחַן', 'nmpr', 'Tahan', 'tachan'),
        'E73066': ('תַּחֲנִי', 'adjv', 'Tahanite', 'tachani'),
        'E73067': ('עֵרָן', 'nmpr', 'Eran', 'eiran'),
        'E73068': ('עֵרָנִי', 'adjv', 'Eranite', 'eirani'),
        'E73069': ('בַּלְעִי', 'adjv', 'Belaite', 'bali'),
        'E73070': ('אַשְׁבֵּלִי', 'adjv', 'Ashbelite', 'ashbeili'),
        'E73071': ('אֲחִירָם', 'nmpr', 'Ahiram', 'achiram'),
        'E73072': ('אֲחִירָמִי', 'adjv', 'Ahiramite', 'achirami'),
        'E73073': ('שְׁפוּפָם', 'nmpr', 'Shupham', "sh'fufam"),
        'E73074': ('שׁוּפָמִי', 'adjv', 'Shuphamite', 'shufami'),
        'E73075': ('חוּפָם', 'nmpr', 'Hupham', 'chufam'),
        'E73076': ('חוּפָמִי', 'adjv', 'Huphamite', 'chufami'),
        'E73077': ('אַרְדִּי', 'adjv', 'Ardite', 'ardi'),
        'E73078': ('נַעֲמִי', 'adjv', 'Naamite', "na'ami"),
        'E73079': ('שׁוּחָם', 'nmpr', 'Shuham', 'shucham'),
        'E73080': ('שׁוּחָמִי', 'adjv', 'Shuhamite', 'shuchami'),
        'E73081': ('יִשְׁוִי', 'adjv', 'Ishvite', 'yishvi'),
        'E73082': ('בְּרִיעִי', 'adjv', 'Beriite', "b'ri'i"),
        'E73083': ('חֶבְרִי', 'adjv', 'Heberite', 'chevri'),
        'E73084': ('מַלְכִּיאֵלִי', 'adjv', 'Malkielite', 'malkieili'),
        'E73085': ('יַחְצְאֵלִי', 'adjv', 'Jahzeelite', 'yachtseili'),
        'E73086': ('גּוּנִי', 'adjv', 'Gunite', 'guni'),
        'E73087': ('יִצְרִי', 'adjv', 'Jezerite', 'yitsri'),
        'E73088': ('שִׁלֵּמִי', 'adjv', 'Shillemite', 'shileimi'),
        'E73089': ('מְרָרִי', 'adjv', 'Merarite', "m'rari"),
        'E73090': ('עֲבָרִים', 'nmpr', 'Abarim', 'avarim'),
        'E73091': ('הֹוד', 'subs', 'splendour', 'hovd'),
        'E73092': ('אִסָּר', 'subs', 'obligation', 'isar'),
        'E73093': ('נוא', 'verb', 'forbid', 'nv'),
        'E73094': ('מִבְטָא', 'subs', 'promise', 'mivta'),
        'E73095': ('מֹוצָא', 'subs', 'issue', 'movtsa'),
        'E73096': ('נְקָמָה', 'subs', 'vengeance', "n'kama"),
        'E73097': ('מסר', 'verb', 'assign', 'msr'),
        'E73098': ('אֱוִי', 'nmpr', 'Evi', 'evi'),
        'E73099': ('רֶקֶם', 'nmpr', 'Rekem', 'rekem'),
        'E73100': ('רֶבַע', 'nmpr', 'Reba', 'reva'),
        'E73101': ('מַלְקֹוחַ', 'subs', 'what is taken', 'malkovach'),
        'E73102': ('בְּדִיל', 'subs', 'tin', "b'dil"),
        'E73103': ('מֶכֶס', 'subs', 'tax', 'meches'),
        'E73104': ('מֶחֱצָה', 'subs', 'half', 'mechetsa'),
        'E73105': ('אֶצְעָדָה', 'subs', 'bracelet', 'etsada'),
        'E73106': ('עָגִיל', 'subs', 'adornment', 'agil'),
        'E73107': ('עֲטָרֹות', 'nmpr', 'Ataroth', 'atarovt'),
        'E73108': ('נִמְרָה', 'nmpr', 'Nimrah', 'nimra'),
        'E73109': ('אֶלְעָלֵה', 'nmpr', 'Elealeh', 'elalei'),
        'E73110': ('שְׂבָם', 'nmpr', 'Sebam', "s'vam"),
        'E73111': ('נְבֹו', 'nmpr', '[mountain]', "n'vov"),
        'E73112': ('בְּעֹן', 'nmpr', 'Beon', "b'on"),
        'E73113': ('קָדֵשׁ בַּרְנֵעַ', 'nmpr', 'Kadesh Barnea', 'kadeish barneia'),
        'E73114': ('תַּרְבּוּת', 'subs', 'increase', 'tarbut'),
        'E73115': ('גְּדֵרָה', 'subs', 'heap of stones', "g'deira"),
        'E73116': ('חושׁ', 'verb', 'make haste', 'chvsh'),
        'E73117': ('צֹנָא', 'subs', '[uncertain]', 'tsona'),
        'E73118': ('גְּבוּלָה', 'subs', 'boundary', "g'vula"),
        'E73119': ('עֲרֹועֵר', 'nmpr', 'Aroer', 'aroveir'),
        'E73120': ('עַטְרֹות שֹׁופָן', 'nmpr', 'Atroth Shophan', 'atrovt shovfan'),
        'E73121': ('יָגְבֳּהָה', 'nmpr', 'Jogbehah', 'yagbaha'),
        'E73122': ('בֵּית נִמְרָה', 'nmpr', 'Beth Nimrah', 'beit nimra'),
        'E73123': ('בֵּית הָרָן', 'nmpr', 'Beth Haran', 'beit haran'),
        'E73124': ('בַּעַל מְעֹון', 'nmpr', 'Baal Meon', "ba'al m'ovn"),
        'E73125': ('שִׂבְמָה', 'nmpr', 'Sibmah', 'sivma'),
        'E73126': ('יָאִיר', 'nmpr', 'Jair', 'yair'),
        'E73127': ('חַוָּה', 'subs', 'tent camp', 'chaua'),
        'E73128': ('נֹבַח', 'nmpr', 'Nobah', 'novach'),
        'E73129': ('קְנָת', 'nmpr', 'Kenath', "k'nat"),
        'E73130': ('נֹבַח', 'nmpr', 'Nobah', 'novach'),
        'E73131': ('הַחִירֹות', 'nmpr', 'Pi Hahiroth', 'hachirovt'),
        'E73132': ('דָּפְקָה', 'nmpr', 'Dophkah', 'dafka'),
        'E73133': ('אָלוּשׁ', 'nmpr', 'Alush', 'alush'),
        'E73134': ('רִתְמָה', 'nmpr', 'Rithmah', 'ritma'),
        'E73135': ('רִמֹּון פֶּרֶץ', 'nmpr', 'Rimmon Perez', 'rimovn perets'),
        'E73136': ('לִבְנָה', 'nmpr', 'Libnah', 'livna'),
        'E73137': ('רִסָּה', 'nmpr', 'Rissah', 'risa'),
        'E73138': ('קְהֵלָתָה', 'nmpr', 'Kehelathah', "k'heilata"),
        'E73139': ('שֶׁפֶר', 'nmpr', 'Shepher', 'shefer'),
        'E73140': ('חֲרָדָה', 'nmpr', 'Haradah', 'charada'),
        'E73141': ('מַקְהֵלֹת', 'nmpr', 'Makheloth', 'makheilot'),
        'E73142': ('תַּחַת', 'nmpr', 'Tahath', 'tachat'),
        'E73143': ('תָּרַח', 'nmpr', 'Terah', 'tarach'),
        'E73144': ('מִתְקָה', 'nmpr', 'Mithcah', 'mitka'),
        'E73145': ('חַשְׁמֹנָה', 'nmpr', 'Hashmonah', 'chashmona'),
        'E73146': ('מֹסֵרֹות', 'nmpr', 'Moseroth', 'moseirovt'),
        'E73147': ('בְּנֵי יַעֲקָן', 'nmpr', 'Bene Jaakan', "b'nei ya'akan"),
        'E73148': ('חֹר הַגִּדְגָּד', 'nmpr', 'Hor Haggidgad', 'chor hagidgad'),
        'E73149': ('יָטְבָתָה', 'nmpr', 'Jotbathah', 'yatvata'),
        'E73150': ('עַבְרֹנָה', 'nmpr', 'Abronah', 'avrona'),
        'E73151': ('עֶצְיֹון גֶּבֶר', 'nmpr', 'Ezion Geber', 'etsyovn gever'),
        'E73152': ('צַלְמֹנָה', 'nmpr', 'Zalmonah', 'tsalmona'),
        'E73153': ('פּוּנֹן', 'nmpr', 'Punon', 'punon'),
        'E73154': ('עִיִּים', 'nmpr', 'Iyim', "i'im"),
        'E73155': ('עַלְמֹון דִּבְלָתָיְמָה',
                   'nmpr',
                   'Almon Diblathaim',
                   'almovn divlatayma'),
        'E73156': ('בֵּית הַיְשִׁמֹות', 'nmpr', 'Beth Jeshimoth', 'beit hayshimovt'),
        'E73157': ('אָבֵל הַשִּׁטִּים', 'nmpr', 'Abel Shittim', 'aveil hashitim'),
        'E73158': ('שֵׂךְ', 'subs', 'thorn', 'seich'),
        'E73159': ('צְנִינִים', 'subs', 'pricks', "ts'ninim"),
        'E73160': ('דמה', 'verb', 'be like', 'dmh'),
        'E73161': ('מַעֲלֵה עַקְרַבִּים', 'nmpr', 'Scorpion Pass', "ma'alei akrabim"),
        'E73162': ('תֹּוצָאֹות', 'subs', 'outlets', 'tovtsaovt'),
        'E73163': ('חֲצַר אַדָּר', 'nmpr', 'Hazar Addar', 'chatsar adar'),
        'E73164': ('עַצְמֹון', 'nmpr', 'Azmon', 'atsmovn'),
        'E73165': ('תאה', 'verb', 'draw', 'th'),
        'E73166': ('צְדָד', 'nmpr', 'Zedad', "ts'dad"),
        'E73167': ('זִפְרֹון', 'nmpr', 'Ziphron', 'zifrovn'),
        'E73168': ('חֲצַר עֵינָן', 'nmpr', 'Hazar Enan', 'chatsar einan'),
        'E73169': ('שְׁפָם', 'nmpr', 'Shepham', "sh'fam"),
        'E73170': ('רִבְלָה', 'nmpr', 'Riblah', 'rivla'),
        'E73171': ('עַיִן', 'nmpr', 'Ain', 'ayin'),
        'E73172': ('מחה', 'verb', 'strike', 'mchh'),
        'E73173': ('כִּנֶּרֶת', 'nmpr', 'Kinnereth', 'kineret'),
        'E73174': ('גָּדִי', 'adjv', 'Gadite', 'gadi'),
        'E73175': ('שְׁמוּאֵל', 'nmpr', 'Samuel', "sh'mueil"),
        'E73176': ('אֱלִידָד', 'nmpr', 'Elidad', 'elidad'),
        'E73177': ('כִּסְלֹון', 'nmpr', 'Kislon', 'kislovn'),
        'E73178': ('בֻּקִּי', 'nmpr', 'Bukki', 'buki'),
        'E73179': ('יָגְלִי', 'nmpr', 'Jogli', 'yagli'),
        'E73180': ('חַנִּיאֵל', 'nmpr', 'Hanniel', 'chanieil'),
        'E73181': ('אֵפֹד', 'nmpr', 'Ephod', 'eifod'),
        'E73182': ('שִׁפְטָן', 'nmpr', 'Shiphtan', 'shiftan'),
        'E73183': ('פַּרְנָךְ', 'nmpr', 'Parnach', 'parnach'),
        'E73184': ('פַּלְטִיאֵל', 'nmpr', 'Paltiel', 'paltieil'),
        'E73185': ('עַזָּן', 'nmpr', 'Azzan', 'azan'),
        'E73186': ('אֲחִיהוּד', 'nmpr', 'Ahihud', 'achihud'),
        'E73187': ('שְׁלֹמִי', 'nmpr', 'Shelomi', "sh'lomi"),
        'E73188': ('פְּדַהְאֵל', 'nmpr', 'Pedahel', "p'daheil"),
        'E73189': ('מִקְלָט', 'subs', 'refuge', 'miklat'),
        'E73190': ('שִׂנְאָה', 'subs', 'hatred', 'sina'),
        'E73191': ('הדף', 'verb', 'push', 'hdf'),
        'E73192': ('צְדִיָּה', 'subs', 'malicious intent', "ts'dia"),
        'E73193': ('חנף', 'verb', 'alienate', 'chnf'),
        'E73194': ('סוּף', 'nmpr', 'Suph', 'suf'),
        'E73195': ('תֹּפֶל', 'nmpr', 'Tophel', 'tofel'),
        'E73196': ('לָבָן', 'nmpr', 'Laban', 'lavan'),
        'E73197': ('דִּי זָהָב', 'nmpr', 'Dizahab', 'di zahav'),
        'E73198': ('עַשְׁתָּרֹות', 'nmpr', 'Ashtaroth', 'ashtarovt'),
        'E73199': ('באר', 'verb', 'explain', 'vr'),
        'E73200': ('שְׁפֵלָה', 'subs', 'low land', "sh'feila"),
        'E73201': ('לְבָנֹון', 'nmpr', 'Lebanon', "l'vanovn"),
        'E73202': ('אֵיכָה', 'inrg', 'how', 'eicho'),
        'E73203': ('טֹרַח', 'subs', 'burden', 'torach'),
        'E73204': ('חתת', 'verb', 'be terrified', "cht't"),
        'E73205': ('רגן', 'verb', 'grumble', 'rgn'),
        'E73206': ('ערץ', 'verb', 'tremble', 'rts'),
        'E73207': ('זוּלָה', 'subs', 'except', 'zula'),
        'E73208': ('אנף', 'verb', 'be angry', 'nf'),
        'E73209': ('מִלְחֶמֶת', 'subs', 'war', 'milchemet'),
        'E73210': ('הון', 'verb', 'be easy', 'hvn'),
        'E73211': ('דְּבֹורָה', 'subs', 'bee', "d'vovra"),
        'E73212': ('גרה', 'verb', 'stir', 'grh'),
        'E73213': ('מִדְרָךְ', 'subs', 'footstep', 'midrach'),
        'E73214': ('יְרֻשָּׁה', 'subs', 'possession', "y'rusha"),
        'E73215': ('כרה', 'verb', 'purchase', 'chrh'),
        'E73216': ('אֵילַת', 'nmpr', 'Elath', 'eilat'),
        'E73217': ('צור', 'verb', 'damage', 'tsvr'),
        'E73218': ('מֹואָבִי', 'adjv', 'Moabite', 'movavi'),
        'E73219': ('עַמֹּונִי', 'adjv', 'Ammonite', 'amovni'),
        'E73220': ('זַמְזֻמִּים', 'nmpr', 'Zamzummites', 'zamzumim'),
        'E73221': ('עַוִּים', 'nmpr', 'Avvim', 'auim'),
        'E73222': ('כַּפְתֹּור', 'nmpr', 'Caphtor', 'kaftovr'),
        'E73223': ('חיל', 'verb', 'have labour pain, to cry', 'chyl'),
        'E73224': ('קְדֵמֹות', 'nmpr', 'Kedemoth', "k'deimovt"),
        'E73225': ('שׂגב', 'verb', 'be high', 'sgv'),
        'E73226': ('חֶבֶל', 'subs', 'cord', 'chevel'),
        'E73227': ('אַרְגֹּב', 'nmpr', 'Argob', 'argov'),
        'E73228': ('פְּרָזִי', 'subs', 'open country', "p'razi"),
        'E73229': ('חֶרְמֹון', 'nmpr', 'Hermon', 'chermovn'),
        'E73230': ('צִידֹנִי', 'adjv', 'Sidonian', 'tsidoni'),
        'E73231': ('שִׂרְיֹן', 'nmpr', 'Sirion', 'siron'),
        'E73232': ('שְׂנִיר', 'nmpr', 'Senir', "s'nir"),
        'E73233': ('מִישֹׁור', 'subs', 'fairness', 'mishovr'),
        'E73234': ('סַלְכָה', 'nmpr', 'Salecah', 'salcho'),
        'E73235': ('עֶרֶשׂ', 'subs', 'couch', 'eres'),
        'E73236': ('גְּשׁוּרִי', 'adjv', 'Geshurite', "g'shuri"),
        'E73237': ('מַעֲכָתִי', 'adjv', 'Maacathite', "ma'achoti"),
        'E73238': ('מַמְלָכָה', 'subs', 'kingdom', 'mamlacho'),
        'E73239': ('עבר', 'verb', 'be angry', 'vr'),
        'E73240': ('בֵּית פְּעֹור', 'nmpr', 'Beth Peor', "beit p'ovr"),
        'E73241': ('למד', 'verb', 'learn', 'lmd'),
        'E73242': ('דָּבֵק', 'adjv', 'clinging, cleaving to', 'daveik'),
        'E73243': ('בִּינָה', 'subs', 'understanding', 'bina'),
        'E73244': ('סֶמֶל', 'subs', 'image', 'semel'),
        'E73245': ('נדח', 'verb', 'wield', 'ndch'),
        'E73246': ('כּוּר', 'subs', 'furnace', 'kur'),
        'E73247': ('כעס', 'verb', 'be discontent', 'chs'),
        'E73248': ('מַסָּה', 'subs', 'proving', 'masa'),
        'E73249': ('בֶּצֶר', 'nmpr', '[place of refuge]', 'betser'),
        'E73250': ('רָמֹת', 'nmpr', 'Ramoth', 'ramot'),
        'E73251': ('גֹּולָן', 'nmpr', 'Golan', 'govlan'),
        'E73252': ('מְנַשִּׁי', 'adjv', 'Manassite', "m'nashi"),
        'E73253': ('שִׂיאֹן', 'nmpr', 'Siyon', 'sion'),
        'E73254': ('שׁנן', 'verb', 'repeat', "shn'n"),
        'E73255': ('חצב', 'verb', 'hew', 'chtsv'),
        'E73256': ('גדע', 'verb', 'cut off', 'gd'),
        'E73257': ('פָּסִיל', 'subs', 'idol', 'pasil'),
        'E73258': ('אֶלֶף', 'subs', 'cattle', 'elef'),
        'E73259': ('עַשְׁתֶּרֶת', 'subs', 'ewe', 'ashteret'),
        'E73260': ('חֳלִי', 'subs', 'sickness', 'chali'),
        'E73261': ('מַדְוֶה', 'subs', 'sickness', 'madveh'),
        'E73262': ('הום', 'verb', 'stir', 'hvm'),
        'E73263': ('מְהוּמָה', 'subs', 'discomfiture', "m'huma"),
        'E73264': ('יקשׁ', 'verb', 'lay snares', 'yksh'),
        'E73265': ('תעב', 'verb', 'be abhorrent', 'tv'),
        'E73266': ('בצק', 'verb', 'swell', 'vtsk'),
        'E73267': ('מִסְכֵּנֻת', 'subs', 'poverty', 'miskeinut'),
        'E73268': ('עַקְרָב', 'subs', 'scorpion', 'akrav'),
        'E73269': ('צִמָּאֹון', 'subs', 'thirsty ground', 'tsimaovn'),
        'E73270': ('חַלָּמִישׁ', 'subs', 'flint', 'chalamish'),
        'E73271': ('עֹצֶם', 'subs', 'might', 'otsem'),
        'E73272': ('רִשְׁעָה', 'subs', 'guilt', 'risha'),
        'E73273': ('יֹשֶׁר', 'subs', 'uprightness', 'yosher'),
        'E73274': ('יגר', 'verb', 'be afraid', 'ygr'),
        'E73275': ('קְשִׁי', 'subs', 'stubbornness', "k'shi"),
        'E73276': ('רֶשַׁע', 'subs', 'guilt', 'resha'),
        'E73277': ('בְּאֵרֹת בְּנֵי יַעֲקָן',
                   'nmpr',
                   'the wells of the Jaakanites',
                   "b'eirot b'nei ya'akan"),
        'E73278': ('מֹוסֵרָה', 'nmpr', 'Moserah', 'movseira'),
        'E73279': ('גִּדְגָּד', 'nmpr', 'Hor Haggidgad', 'gidgad'),
        'E73280': ('מוּסָר', 'subs', 'chastening', 'musar'),
        'E73281': ('צוף', 'verb', 'flow', 'tsvf'),
        'E73282': ('יָרָק', 'subs', 'greens', 'yarak'),
        'E73283': ('יֹורֶה', 'subs', 'early rain', 'yovreh'),
        'E73284': ('מַלְקֹושׁ', 'subs', 'spring-rain', 'malkovsh'),
        'E73285': ('גְּרִזִים', 'nmpr', 'Gerizim', "g'rizim"),
        'E73286': ('עֵיבָל', 'nmpr', 'Ebal', 'eival'),
        'E73287': ('מָבֹוא', 'subs', 'entrance', 'mavov'),
        'E73288': ('גִּלְגָּל', 'nmpr', 'Gilgal', 'gilgal'),
        'E73289': ('רַעֲנָן', 'adjv', 'luxuriant', "ra'anan"),
        'E73290': ('מִשְׁלָח', 'subs', 'undertaking', 'mishlach'),
        'E73291': ('אַוָּה', 'subs', 'desire', 'aua'),
        'E73292': ('צְבִי', 'subs', 'gazelle', "ts'vi"),
        'E73293': ('אַיָּל', 'subs', 'fallow-buck', 'ayal'),
        'E73294': ('נקשׁ', 'verb', 'ensnare', 'nksh'),
        'E73295': ('סָרָה', 'subs', 'rebellion', 'sara'),
        'E73296': ('סות', 'verb', 'incite', 'svt'),
        'E73297': ('סֵתֶר', 'subs', 'hiding place', 'seiter'),
        'E73298': ('בְּלִיַּעַל', 'subs', 'wickedness', "b'lia'al"),
        'E73299': ('חקר', 'verb', 'explore', 'chkr'),
        'E73300': ('תֵּל', 'subs', 'mound', 'teil'),
        'E73301': ('גדד', 'verb', 'gather', "gd'd"),
        'E73302': ('יַחְמוּר', 'subs', 'reebok', 'yachmur'),
        'E73303': ('אַקֹּו', 'subs', 'wildgoat', 'akov'),
        'E73304': ('דִּישֹׁון', 'subs', '[uncertain]', 'dishovn'),
        'E73305': ('תְּאֹו', 'subs', 'wild sheep', "t'ov"),
        'E73306': ('זֶמֶר', 'subs', 'gazelle', 'zemer'),
        'E73307': ('רָאָה', 'subs', 'red bird', "ra'a"),
        'E73308': ('דַּיָּה', 'subs', '[uncertain]', 'daya'),
        'E73309': ('רָחָמָה', 'subs', 'vulture', 'rachama'),
        'E73310': ('צור', 'verb', 'bind', 'tsvr'),
        'E73311': ('שְׁמִטָּה', 'subs', 'remitting', "sh'mita"),
        'E73312': ('מַשֶּׁה', 'subs', 'loan', 'masheh'),
        'E73313': ('עבט', 'verb', 'borrow', 'vt'),
        'E73314': ('קפץ', 'verb', 'draw together', 'kfts'),
        'E73315': ('מַחְסֹור', 'subs', 'need', 'machsovr'),
        'E73316': ('ענק', 'verb', 'put around neck', 'nk'),
        'E73317': ('חֶרְמֵשׁ', 'subs', 'sickle', 'chermeish'),
        'E73318': ('מִסָּה', 'subs', 'measure', 'misa'),
        'E73319': ('שָׂמֵחַ', 'adjv', 'joyful', 'sameiach'),
        'E73320': ('דִּין', 'subs', 'claim', 'din'),
        'E73321': ('רִיבָה', 'subs', 'case', 'riva'),
        'E73322': ('זָדֹון', 'subs', 'insolence', 'zadovn'),
        'E73323': ('לְחִי', 'subs', 'jaw', "l'chi"),
        'E73324': ('גֵּז', 'subs', 'grass', 'geiz'),
        'E73325': ('קסם', 'verb', 'practice divination', 'ksm'),
        'E73326': ('חֶבֶר', 'subs', 'company', 'chever'),
        'E73327': ('יַעַר', 'subs', 'wood', "ya'ar"),
        'E73328': ('חטב', 'verb', 'gather wood', 'chtv'),
        'E73329': ('גַּרְזֶן', 'subs', 'axe', 'garzen'),
        'E73330': ('ארב', 'verb', 'lie in ambush', 'rv'),
        'E73331': ('סוג', 'verb', 'turn', 'svg'),
        'E73332': ('רכך', 'verb', 'be tender', "rch'ch"),
        'E73333': ('חפז', 'verb', 'hurry', 'chfz'),
        'E73334': ('חנך', 'verb', 'dedicate', 'chnch'),
        'E73335': ('מָצֹור', 'subs', 'siege', 'matsovr'),
        'E73336': ('שִׁבְיָה', 'subs', 'captives', 'shiva'),
        'E73337': ('צִפֹּרֶן', 'subs', 'nail', 'tsiporen'),
        'E73338': ('עמר', 'verb', 'deal violently', 'mr'),
        'E73339': ('שָׂנִיא', 'adjv', 'disdained', 'sani'),
        'E73340': ('סרר', 'verb', 'rebel', "sr'r"),
        'E73341': ('זלל', 'verb', 'be lavish', "zl'l"),
        'E73342': ('סבא', 'verb', 'drink', 'sv'),
        'E73343': ('אֶפְרֹחַ', 'subs', 'young one', 'efroach'),
        'E73344': ('בֵּיצָה', 'subs', 'egg', 'beitsa'),
        'E73345': ('מַעֲקֶה', 'subs', 'parapet', "ma'akeh"),
        'E73346': ('גָּדִל', 'subs', 'tassel', 'gadil'),
        'E73347': ('עֲלִילָה', 'subs', 'deed', 'alila'),
        'E73348': ('פצע', 'verb', 'squeeze', 'fts'),
        'E73349': ('דַּכָּה', 'subs', 'crushing', 'daka'),
        'E73350': ('שָׁפְכָה', 'subs', 'penis', 'shafcho'),
        'E73351': ('מַמְזֵר', 'subs', 'bastard', 'mamzeir'),
        'E73352': ('קדם', 'verb', 'be in front', 'kdm'),
        'E73353': ('אֲדֹמִי', 'adjv', 'Edomite', 'adomi'),
        'E73354': ('קָרֶה', 'subs', 'pollution', 'kareh'),
        'E73355': ('אָזֵן', 'subs', 'tool', 'azein'),
        'E73356': ('צֵאָה', 'subs', 'excrement', 'tseia'),
        'E73357': ('אֶתְנַן', 'subs', 'gift', 'etnan'),
        'E73358': ('מְחִיר', 'subs', 'price', "m'chir"),
        'E73359': ('נשׁך', 'verb', 'claim interest', 'nshch'),
        'E73360': ('קטף', 'verb', 'pluck off', 'ktf'),
        'E73361': ('מְלִילָה', 'subs', 'rubbed wheat', "m'lila"),
        'E73362': ('כְּרִיתוּת', 'subs', 'divorce', "k'ritut"),
        'E73363': ('מַשָּׁאָה', 'subs', 'loan', "masha'a"),
        'E73364': ('עֲבֹוט', 'subs', 'pledge', 'avovt'),
        'E73365': ('חבט', 'verb', 'beat off', 'chvt'),
        'E73366': ('פאר', 'verb', 'knock down', 'fr'),
        'E73367': ('קלה', 'verb', 'contempt', 'klh'),
        'E73368': ('חסם', 'verb', 'muzzle', 'chsm'),
        'E73369': ('דושׁ', 'verb', 'tread on', 'dvsh'),
        'E73370': ('יָבָם', 'subs', "husband's brother", 'yavam'),
        'E73371': ('יְבֶמֶת', 'subs', "brother's widow", "y'vemet"),
        'E73372': ('מְבוּשִׁים', 'subs', 'private parts', "m'vushim"),
        'E73373': ('כִּיס', 'subs', 'bag', 'kis'),
        'E73374': ('זנב', 'verb', 'cut off', 'znv'),
        'E73375': ('חשׁל', 'verb', 'wear out', 'chshl'),
        'E73376': ('יָגֵעַ', 'adjv', 'weary', 'yageia'),
        'E73377': ('טֶנֶא', 'subs', 'basket', 'tene'),
        'E73378': ('אֹנֶה', 'subs', 'time of mourning', 'oneh'),
        'E73379': ('מָעֹון', 'subs', 'dwelling', 'maovn'),
        'E73380': ('שׂיד', 'verb', 'whitewash', 'syd'),
        'E73381': ('שִׂיד', 'subs', 'lime', 'sid'),
        'E73382': ('סכת', 'verb', 'be silent', "s'cht"),
        'E73383': ('אָסָם', 'subs', 'store', 'asam'),
        'E73384': ('אֹוצָר', 'subs', 'supply', 'ovtsar'),
        'E73385': ('מְאֵרָה', 'subs', 'curse', "m'eira"),
        'E73386': ('מִגְעֶרֶת', 'subs', 'rebuke', 'migeret'),
        'E73387': ('מַעֲלָל', 'subs', 'deed', "ma'alal"),
        'E73388': ('דַּלֶּקֶת', 'subs', 'inflammation', 'daleket'),
        'E73389': ('חַרְחֻר', 'subs', 'heat', 'charchur'),
        'E73390': ('שִׁדָּפֹון', 'subs', 'scorching', 'shidafovn'),
        'E73391': ('יֵרָקֹון', 'subs', 'paleness', 'yeirakovn'),
        'E73392': ('זַעֲוָה', 'subs', 'terror', "za'ava"),
        'E73393': ('טְחֹרִים', 'subs', 'tumours', "t'chorim"),
        'E73394': ('חֶרֶס', 'subs', 'scabies', 'cheres'),
        'E73395': ('שִׁגָּעֹון', 'subs', 'madness', 'shigaovn'),
        'E73396': ('עִוָּרֹון', 'subs', 'blindness', 'iuarovn'),
        'E73397': ('תִּמָּהֹון', 'subs', 'bewilderment', 'timahovn'),
        'E73398': ('כָּלֶה', 'adjv', 'failing', 'kaleh'),
        'E73399': ('שׁגע', 'verb', 'be mad', 'shg'),
        'E73400': ('שַׁמָּה', 'subs', 'destruction', 'shama'),
        'E73401': ('שְׁנִינָה', 'subs', 'taunt', "sh'nina"),
        'E73402': ('חסל', 'verb', 'consume', 'chsl'),
        'E73403': ('אגר', 'verb', 'gather', 'gr'),
        'E73404': ('סוך', 'verb', 'anoint', 'svch'),
        'E73405': ('צְלָצַל', 'subs', 'cricket', "ts'latsal"),
        'E73406': ('חֹסֶר', 'subs', 'want', 'choser'),
        'E73407': ('דאה', 'verb', 'pounce', 'dh'),
        'E73408': ('בטח', 'verb', 'trust', "vt'ch"),
        'E73409': ('מָצֹור', 'subs', 'stress', 'matsovr'),
        'E73410': ('מָצֹוק', 'subs', 'stress', 'matsovk'),
        'E73411': ('צוק', 'verb', 'oppress', 'tsvk'),
        'E73412': ('עָנֹג', 'adjv', 'dainty', 'anog'),
        'E73413': ('ענג', 'verb', 'be dainty', 'ng'),
        'E73414': ('רֹךְ', 'subs', 'tenderness', 'roch'),
        'E73415': ('שִׁלְיָה', 'subs', 'afterbirth', 'shila'),
        'E73416': ('שׂושׂ', 'verb', 'rejoice', 'svs'),
        'E73417': ('נסח', 'verb', 'tear', "ns'ch"),
        'E73418': ('רגע', 'verb', 'stir', 'rg'),
        'E73419': ('רַגָּז', 'adjv', 'excited', 'ragaz'),
        'E73420': ('כִּלָּיֹון', 'subs', 'annihilation', 'kilayovn'),
        'E73421': ('דְּאָבֹון', 'subs', 'despair', "d'avovn"),
        'E73422': ('תלא', 'verb', 'hang', 'tl'),
        'E73423': ('פחד', 'verb', 'tremble', 'fchd'),
        'E73424': ('שִׁקּוּץ', 'subs', 'idol', 'shikuts'),
        'E73425': ('שֹׁרֶשׁ', 'subs', 'root', 'shoresh'),
        'E73426': ('רֹאשׁ', 'subs', 'poison', 'rosh'),
        'E73427': ('לַעֲנָה', 'subs', 'wormwood', "la'ana"),
        'E73428': ('שְׁרִרוּת', 'subs', 'stubbornness', "sh'rirut"),
        'E73429': ('רָוֶה', 'adjv', 'watered', 'raveh'),
        'E73430': ('צָמֵא', 'adjv', 'thirsty', 'tsamei'),
        'E73431': ('תַּחֲלֻאִים', 'subs', 'diseases', 'tachaluim'),
        'E73432': ('מַהְפֵּכָה', 'subs', 'overthrow', 'mahpeicho'),
        'E73433': ('נתשׁ', 'verb', 'root out', 'ntsh'),
        'E73434': ('שׁוב', 'verb', 'gather', 'shvv'),
        'E73435': ('שְׁבוּת', 'subs', 'captivity', "sh'vut"),
        'E73436': ('ערף', 'verb', 'drip', 'rf'),
        'E73437': ('לֶקַח', 'subs', 'teaching', 'lekach'),
        'E73438': ('שָׂעִיר', 'subs', 'shower of rain', 'sair'),
        'E73439': ('רְבִיבִים', 'subs', 'rain', "r'vivim"),
        'E73440': ('פֹּעַל', 'subs', 'doing', 'poal'),
        'E73441': ('עִקֵּשׁ', 'adjv', 'crooked', 'ikeish'),
        'E73442': ('פְּתַלְתֹּל', 'adjv', 'queer', "p'taltol"),
        'E73443': ('נָבָל', 'adjv', 'stupid', 'naval'),
        'E73444': ('יְלֵל', 'subs', 'howling', "y'leil"),
        'E73445': ('יְשִׁימֹון', 'subs', 'wilderness', "y'shimovn"),
        'E73446': ('אִישֹׁון', 'subs', 'pupil of eye', 'ishovn'),
        'E73447': ('עור', 'verb', 'be awake', 'vr'),
        'E73448': ('אֶבְרָה', 'subs', 'wing', 'evra'),
        'E73449': ('תְּנוּבָה', 'subs', 'produce', "t'nuva"),
        'E73450': ('שָׂדַי', 'subs', 'open field', 'saday'),
        'E73451': ('כַּר', 'subs', 'ram', 'kar'),
        'E73452': ('חֶמֶר', 'subs', 'wine', 'chemer'),
        'E73453': ('שׁמן', 'verb', 'be fat', 'shmn'),
        'E73454': ('יְשֻׁרוּן', 'nmpr', 'Jeshurun', "y'shurun"),
        'E73455': ('בעט', 'verb', 'kick', 'vt'),
        'E73456': ('עבה', 'verb', 'be thick', 'vh'),
        'E73457': ('כשׂה', 'verb', 'grow fat', "chs'h"),
        'E73458': ('אֱלֹוהַּ', 'subs', 'god', 'elovha'),
        'E73459': ('שֵׁד', 'subs', 'demon', 'sheid'),
        'E73460': ('שׂער', 'verb', 'know', 'sr'),
        'E73461': ('שׁיה', 'verb', '[uncertain]', 'shyh'),
        'E73462': ('כַּעַס', 'subs', 'grief', "ka'as"),
        'E73463': ('תַּהְפֻּכֹות', 'subs', 'perversity', 'tahpuchovt'),
        'E73464': ('אֵמֻן', 'subs', 'faith', 'eimun'),
        'E73465': ('הֶבֶל', 'subs', 'breath', 'hevel'),
        'E73466': ('קדח', 'verb', 'be kindled', 'kdch'),
        'E73467': ('להט', 'verb', 'devour', 'lht'),
        'E73468': ('מֹוסָד', 'subs', 'foundation', 'movsad'),
        'E73469': ('מָזֶה', 'adjv', 'hungry', 'mazeh'),
        'E73470': ('לחם', 'verb', 'feed', 'lchm'),
        'E73471': ('רֶשֶׁף', 'subs', 'flame', 'reshef'),
        'E73472': ('קֶטֶב', 'subs', 'sting', 'ketev'),
        'E73473': ('מְרִירִי', 'adjv', 'bitter', "m'riri"),
        'E73474': ('זחל', 'verb', 'glide away', 'zchl'),
        'E73475': ('בָּחוּר', 'subs', 'young man', 'bachur'),
        'E73476': ('פאה', 'verb', 'split', 'fh'),
        'E73477': ('אֱנֹושׁ', 'subs', 'man', 'enovsh'),
        'E73478': ('עֵצָה', 'subs', 'counsel', 'eitsa'),
        'E73479': ('שְׁדֵמָה', 'subs', 'terrace', "sh'deima"),
        'E73480': ('פֶּתֶן', 'subs', 'cobra', 'peten'),
        'E73481': ('אַכְזָר', 'adjv', 'cruel', 'achzar'),
        'E73482': ('כמס', 'verb', 'store', 'chms'),
        'E73483': ('שִׁלֵּם', 'subs', '[uncertain]', 'shileim'),
        'E73484': ('אֵיד', 'subs', 'calamity', 'eid'),
        'E73485': ('עָתִיד', 'adjv', 'ready', 'atid'),
        'E73486': ('אזל', 'verb', 'go away', 'zl'),
        'E73487': ('חסה', 'verb', 'seek refuge', "chs'h"),
        'E73488': ('נָסִיךְ', 'subs', 'libation', 'nasich'),
        'E73489': ('סִתְרָה', 'subs', 'hiding place', 'sitra'),
        'E73490': ('שׁנן', 'verb', 'sharpen', "shn'n"),
        'E73491': ('יפע', 'verb', 'shine', 'yf'),
        'E73492': ('אתה', 'verb', 'come', 'th'),
        'E73493': ('דָּת', 'subs', 'orders', 'dat'),
        'E73494': ('חבב', 'verb', 'love', "chv'v"),
        'E73495': ('תכה', 'verb', '[uncertain]', "t'chh"),
        'E73496': ('דַּבֶּרֶת', 'subs', 'word', 'daberet'),
        'E73497': ('קְהִלָּה', 'subs', 'assembly', "k'hila"),
        'E73498': ('יַחַד', 'subs', 'gathering', 'yachad'),
        'E73499': ('חָסִיד', 'adjv', 'loyal', 'chasid'),
        'E73500': ('קְטֹורָה', 'subs', 'smoke sacrifice', "k'tovra"),
        'E73501': ('יָדִיד', 'adjv', 'beloved', 'yadid'),
        'E73502': ('חפף', 'verb', 'shelter', "chf'f"),
        'E73503': ('מֶגֶד', 'subs', 'present', 'meged'),
        'E73504': ('גֶּרֶשׁ', 'subs', 'produce', 'geresh'),
        'E73505': ('שֶׁפַע', 'subs', 'abundance', 'shefa'),
        'E73506': ('שׂפן', 'verb', 'cover', 'sfn'),
        'E73507': ('ספן', 'verb', 'cover', 'sfn'),
        'E73508': ('זנק', 'verb', 'leap', 'znk'),
        'E73509': ('דָּרֹום', 'subs', 'south', 'darovm'),
        'E73510': ('מִנְעָל', 'subs', 'lock', 'minal'),
        'E73511': ('דֹּבֶא', 'subs', 'walk', 'dove'),
        'E73512': ('גַּאֲוָה', 'subs', 'uproar', "ga'ava"),
        'E73513': ('שַׁחַק', 'subs', 'dust', 'shachak'),
        'E73514': ('מְעֹנָה', 'subs', 'hiding place', "m'ona"),
        'E73515': ('אֶשֶׁר', 'subs', 'happiness', 'esher'),
        'E73516': ('לֵחַ', 'subs', 'sap', 'leiach'),
        'E73517': ('הגה', 'verb', 'mutter', 'hgh'),
        'E73518': ('חֶרֶשׁ', 'subs', 'silence', 'cheresh'),
        'E73519': ('רָחָב', 'nmpr', 'Rahab', 'rachav'),
        'E73520': ('מַעְבָּרָה', 'subs', 'ford', 'mabara'),
        'E73521': ('תִּקְוָה', 'subs', 'cord', 'tikva'),
        'E73522': ('גִּדְיָה', 'subs', 'bank', 'gida'),
        'E73523': ('אָדָם', 'nmpr', 'Adam', 'adam'),
        'E73524': ('צָרְתָן', 'nmpr', 'Zarethan', 'tsartan'),
        'E73525': ('מַצָּב', 'subs', 'standing place', 'matsav'),
        'E73526': ('עֲבוּר', 'subs', 'produce', 'avur'),
        'E73527': ('שֹׁמַע', 'subs', 'rumour', 'shoma'),
        'E73528': ('עָכָן', 'nmpr', 'Achan', 'achon'),
        'E73529': ('זַבְדִּי', 'nmpr', 'Zabdi', 'zavdi'),
        'E73530': ('בֵּית אָוֶן', 'nmpr', 'Beth Aven', 'beit aven'),
        'E73531': ('יגע', 'verb', 'be weary', 'yg'),
        'E73532': ('שְׁבָרִים', 'nmpr', 'Shebarim', "sh'varim"),
        'E73533': ('מֹורָד', 'subs', 'descent', 'movrad'),
        'E73534': ('אֲהָהּ', 'intj', 'alas', 'ahah'),
        'E73535': ('עָכֹור', 'nmpr', 'Achor', 'achovr'),
        'E73536': ('יצת', 'verb', 'kindle', 'ytst'),
        'E73537': ('מַאְרָב', 'subs', 'ambush', 'marav'),
        'E73538': ('כִּידֹון', 'subs', 'dart', 'kidovn'),
        'E73539': ('גִּבְעֹון', 'nmpr', 'Gibeon', 'givovn'),
        'E73540': ('ציר', 'verb', '[uncertain]', 'tsyr'),
        'E73541': ('בָּלֶה', 'adjv', 'worn out', 'baleh'),
        'E73542': ('נֹאד', 'subs', 'skin-bottle', 'nod'),
        'E73543': ('צַיִד', 'subs', 'provision', 'tsayid'),
        'E73544': ('נִקֻּדִים', 'subs', 'crumbs', 'nikudim'),
        'E73545': ('חָם', 'adjv', 'hot', 'cham'),
        'E73546': ('ציד', 'verb', 'feed', 'tsyd'),
        'E73547': ('כְּפִירָה', 'nmpr', 'Kephirah', "k'fira"),
        'E73548': ('בְּאֵרֹות', 'nmpr', 'Beeroth', "b'eirovt"),
        'E73549': ('קִרְיַת יְעָרִים', 'nmpr', 'Kiriath Jearim', "kirat y'arim"),
        'E73550': ('אֲדֹנִי צֶדֶק', 'nmpr', 'Adoni-Zedek', 'adoni tsedek'),
        'E73551': ('יְרוּשָׁלִַם', 'nmpr', 'Jerusalem', "y'rushalaim"),
        'E73552': ('הֹוהָם', 'nmpr', 'Hoham', 'hovham'),
        'E73553': ('פִּרְאָם', 'nmpr', 'Piram', 'piram'),
        'E73554': ('יַרְמוּת', 'nmpr', 'Jarmuth', 'yarmut'),
        'E73555': ('יָפִיעַ', 'nmpr', 'Japhia', 'yafia'),
        'E73556': ('לָכִישׁ', 'nmpr', 'Lachish', 'lachish'),
        'E73557': ('דְּבִיר', 'nmpr', 'Debir', "d'vir"),
        'E73558': ('עֶגְלֹון', 'nmpr', 'Eglon', 'eglovn'),
        'E73559': ('מַעֲלֶה', 'subs', 'ascent', "ma'aleh"),
        'E73560': ('בֵּית חֹורֹון', 'nmpr', 'Beth Horon', 'beit chovrovn'),
        'E73561': ('עֲזֵקָה', 'nmpr', 'Azekah', 'azeika'),
        'E73562': ('מַקֵּדָה', 'nmpr', 'Makkedah', 'makeida'),
        'E73563': ('אַיָּלֹון', 'nmpr', 'Aijalon', 'ayalovn'),
        'E73564': ('שׂרד', 'verb', 'run away', 'srd'),
        'E73565': ('קָצִין', 'subs', 'chief', 'katsin'),
        'E73566': ('הֹרָם', 'nmpr', 'Horam', 'horam'),
        'E73567': ('גֶּזֶר', 'nmpr', 'Gezer', 'gezer'),
        'E73568': ('דְּבִר', 'nmpr', 'Debir', "d'vir"),
        'E73569': ('יָבִין', 'nmpr', 'Jabin', 'yavin'),
        'E73570': ('חָצֹור', 'nmpr', 'Hazor', 'chatsovr'),
        'E73571': ('מָדֹון', 'nmpr', 'Madon', 'madovn'),
        'E73572': ('שִׁמְרֹון', 'nmpr', 'Shimron', 'shimrovn'),
        'E73573': ('אַכְשָׁף', 'nmpr', 'Acshaph', 'achshof'),
        'E73574': ('נָפָה', 'subs', 'yoke', 'nafa'),
        'E73575': ('דֹּור', 'nmpr', 'Dor', 'dovr'),
        'E73576': ('מֵרֹום', 'nmpr', 'Merom', 'meirovm'),
        'E73577': ('מִשְׂרְפֹות מַיִם', 'nmpr', 'Misrephoth Maim', 'misrfovt mayim'),
        'E73578': ('מִצְפֶּה', 'nmpr', 'Mizpeh', 'mitspeh'),
        'E73579': ('בַּעַל גָּד', 'nmpr', 'Baal Gad', "ba'al gad"),
        'E73580': ('תְּחִנָּה', 'subs', 'supplication', "t'china"),
        'E73581': ('עֲנָב', 'nmpr', 'Anab', 'anav'),
        'E73582': ('גַּת', 'nmpr', 'Gath', 'gat'),
        'E73583': ('אַשְׁדֹּוד', 'nmpr', 'Ashdod', 'ashdovd'),
        'E73584': ('מַחֲלֹקֶת', 'subs', 'division', 'machaloket'),
        'E73585': ('שׁקט', 'verb', 'be at peace', 'shkt'),
        'E73586': ('גֶּדֶר', 'nmpr', 'Geder', 'geder'),
        'E73587': ('עֲדֻלָּם', 'nmpr', 'Adullam', 'adulam'),
        'E73588': ('תַּפּוּחַ', 'nmpr', 'Tappuah', 'tapuach'),
        'E73589': ('חֵפֶר', 'nmpr', 'Hepher', 'cheifer'),
        'E73590': ('אֲפֵק', 'nmpr', 'Aphek', 'afeik'),
        'E73591': ('שָׁרֹון', 'nmpr', 'Sharon', 'sharovn'),
        'E73592': ('שִׁמְרֹון מְראֹון', 'nmpr', 'Shimron Meron', "shimrovn m'rovn"),
        'E73593': ('תַּעֲנַךְ', 'nmpr', 'Taanach', "ta'anach"),
        'E73594': ('מְגִדֹּו', 'nmpr', 'Megiddo', "m'gidov"),
        'E73595': ('קֶדֶשׁ', 'nmpr', 'Kedesh', 'kedesh'),
        'E73596': ('יָקְנְעָם', 'nmpr', 'Jokneam', 'yaknam'),
        'E73597': ('כַּרְמֶל', 'nmpr', 'Carmel', 'karmel'),
        'E73598': ('תִּרְצָה', 'nmpr', 'Tirzah', 'tirtsa'),
        'E73599': ('גְּלִילָה', 'subs', 'region', "g'lila"),
        'E73600': ('שִׁיחֹור', 'nmpr', 'Shihor', 'shichovr'),
        'E73601': ('עֶקְרֹון', 'nmpr', 'Ekron', 'ekrovn'),
        'E73602': ('סְרָנִים', 'subs', 'lords', "s'ranim"),
        'E73603': ('עַזָּתִי', 'adjv', 'from Gaza', 'azati'),
        'E73604': ('אַשְׁדֹּודִי', 'adjv', 'from Ashdod', 'ashdovdi'),
        'E73605': ('אֶשְׁקְלֹונִי', 'adjv', 'Ashkelonite', 'eshklovni'),
        'E73606': ('גִּתִּי', 'adjv', 'Gittite', 'giti'),
        'E73607': ('עֶקְרֹונִי', 'adjv', 'from Ekron', 'ekrovni'),
        'E73608': ('מְעָרָה', 'nmpr', 'Mearah', "m'ara"),
        'E73609': ('גִּבְלִי', 'adjv', 'Gebalite', 'givli'),
        'E73610': ('מַמְלָכוּת', 'subs', 'kingdom', 'mamlachut'),
        'E73611': ('גְּשׁוּר', 'nmpr', 'Geshur', "g'shur"),
        'E73612': ('מַעֲכָת', 'nmpr', 'Maacath', "ma'achot"),
        'E73613': ('בֵּית בַּעַל מְעֹון', 'nmpr', 'Baal Meon', "beit ba'al m'ovn"),
        'E73614': ('מֵיפַעַת', 'nmpr', 'Mephaath', "meifa'at"),
        'E73615': ('צֶרֶת הַשַּׁחַר', 'nmpr', 'Zereth Shahar', 'tseret hashachar'),
        'E73616': ('נָסִיךְ', 'subs', 'leader', 'nasich'),
        'E73617': ('רַבָּה', 'nmpr', 'Rabbah', 'raba'),
        'E73618': ('רָמָה', 'subs', 'high place', 'rama'),
        'E73619': ('מִצְפֶּה', 'subs', 'watch-tower', 'mitspeh'),
        'E73620': ('בְּטֹנִים', 'nmpr', 'Betonim', "b'tonim"),
        'E73621': ('לִדְבִר', 'nmpr', 'Debir', 'lidvir'),
        'E73622': ('בֵּית הָרָם', 'nmpr', 'Beth Haram', 'beit haram'),
        'E73623': ('צָפֹון', 'nmpr', 'Zaphon', 'tsafovn'),
        'E73624': ('מסה', 'verb', 'melt', "ms'h"),
        'E73625': ('חֶצְרֹון', 'nmpr', 'Hezron', 'chetsrovn'),
        'E73626': ('אַדָּר', 'nmpr', '[town]', 'adar'),
        'E73627': ('קַרְקַע', 'nmpr', 'Karka', 'karka'),
        'E73628': ('בֵּית חָגְלָה', 'nmpr', 'Beth Hoglah', 'beit chagla'),
        'E73629': ('בֵּית הָעֲרָבָה', 'nmpr', 'Beth Arabah', "beit ha'arava"),
        'E73630': ('בֹּהַן', 'nmpr', 'Bohan', 'bohan'),
        'E73631': ('מַעֲלֵה אֲדֻמִּים',
                   'nmpr',
                   'the Pass of Adummim',
                   "ma'alei adumim"),
        'E73632': ('עֵין שֶׁמֶשׁ', 'nmpr', 'En Shemesh', 'ein shemesh'),
        'E73633': ('עֵין רֹגֵל', 'nmpr', 'En Rogel', 'ein rogeil'),
        'E73634': ('בֶּן הִנֹּם', 'nmpr', 'the Valley of Ben Hinnom', 'ben hinom'),
        'E73635': ('הִנֹּם', 'nmpr', 'Hinnom', 'hinom'),
        'E73636': ('רְפָאִים', 'nmpr', '[valley]', "r'faim"),
        'E73637': ('תאר', 'verb', 'turn', 'tr'),
        'E73638': ('נֶפְתֹּוחַ', 'nmpr', 'Nephtoah', 'neftovach'),
        'E73639': ('עֶפְרֹון', 'nmpr', 'Ephron', 'efrovn'),
        'E73640': ('בַּעֲלָה', 'nmpr', 'Baalah', "ba'ala"),
        'E73641': ('יְעָרִים', 'nmpr', 'Jearim', "y'arim"),
        'E73642': ('כְּסָלֹון', 'nmpr', 'Kesalon', "k'salovn"),
        'E73643': ('בֵּית שֶׁמֶשׁ', 'nmpr', 'Beth Shemesh', 'beit shemesh'),
        'E73644': ('שִׁכְּרֹון', 'nmpr', 'Shikkeron', 'shikrovn'),
        'E73645': ('יַבְנְאֵל', 'nmpr', 'Jabneel', 'yavneil'),
        'E73646': ('קִרְיַת סֵפֶר', 'nmpr', 'Kiriath Sepher', 'kirat seifer'),
        'E73647': ('עַכְסָה', 'nmpr', 'Acsah', 'achsa'),
        'E73648': ('עָתְנִיאֵל', 'nmpr', 'Othniel', 'atnieil'),
        'E73649': ('צנח', 'verb', "clap one's hand", 'tsnch'),
        'E73650': ('גֻּלָּה', 'subs', 'basin', 'gula'),
        'E73651': ('עִלִּי', 'adjv', 'upper', 'ili'),
        'E73652': ('קַבְצְאֵל', 'nmpr', 'Kabzeel', 'kavtseil'),
        'E73653': ('עֵדֶר', 'nmpr', 'Eder', 'eider'),
        'E73654': ('יָגוּר', 'nmpr', 'Jagur', 'yagur'),
        'E73655': ('קִינָה', 'nmpr', 'Kinah', 'kina'),
        'E73656': ('דִּימֹונָה', 'nmpr', 'Dimonah', 'dimovna'),
        'E73657': ('עַדְעָדָה', 'nmpr', 'Adadah', 'adada'),
        'E73658': ('יִתְנָן', 'nmpr', 'Ithnan', 'yitnan'),
        'E73659': ('זִיף', 'nmpr', 'Ziph', 'zif'),
        'E73660': ('טֶלֶם', 'nmpr', 'Telem', 'telem'),
        'E73661': ('בְּעָלֹות', 'nmpr', 'Bealoth', "b'alovt"),
        'E73662': ('חָצֹור חֲדַתָּה', 'nmpr', 'Hazor Hadattah', 'chatsovr chadata'),
        'E73663': ('קְרִיֹּות', 'nmpr', 'Kerioth', "k'riovt"),
        'E73664': ('אֲמָם', 'nmpr', 'Amam', 'amam'),
        'E73665': ('שְׁמַע', 'nmpr', 'Shema', "sh'ma"),
        'E73666': ('מֹולָדָה', 'nmpr', 'Moladah', 'movlada'),
        'E73667': ('חֲצַר גַּדָּה', 'nmpr', 'Hazar Gaddah', 'chatsar gada'),
        'E73668': ('חֶשְׁמֹון', 'nmpr', 'Heshmon', 'cheshmovn'),
        'E73669': ('בֵּית פֶּלֶט', 'nmpr', 'Beth Pelet', 'beit pelet'),
        'E73670': ('חֲצַר שׁוּעָל', 'nmpr', 'Hazar Shual', 'chatsar shual'),
        'E73671': ('בִּזְיֹותְיָה', 'nmpr', 'Biziothiah', 'bizovtya'),
        'E73672': ('עֶצֶם', 'nmpr', 'Ezem', 'etsem'),
        'E73673': ('אֶלְתֹּולַד', 'nmpr', 'Eltolad', 'eltovlad'),
        'E73674': ('כְּסִיל', 'nmpr', 'Kesil', "k'sil"),
        'E73675': ('צִקְלַג', 'nmpr', 'Ziklag', 'tsiklag'),
        'E73676': ('מַדְמַנָּה', 'nmpr', 'Madmannah', 'madmana'),
        'E73677': ('סַנְסַנָּה', 'nmpr', 'Sansannah', 'sansana'),
        'E73678': ('לְבָאֹות', 'nmpr', 'Lebaoth', "l'vaovt"),
        'E73679': ('שִׁלְחִים', 'nmpr', 'Shilhim', 'shilchim'),
        'E73680': ('עֵין רִמֹּון', 'nmpr', 'En Rimmon', 'ein rimovn'),
        'E73681': ('אֶשְׁתָּאֹל', 'nmpr', 'Eshtaol', 'eshtaol'),
        'E73682': ('צָרְעָה', 'nmpr', 'Zorah', 'tsara'),
        'E73683': ('אַשְׁנָה', 'nmpr', 'Ashnah', 'ashna'),
        'E73684': ('זָנֹוחַ', 'nmpr', 'Zanoah', 'zanovach'),
        'E73685': ('עֵין גַּנִּים', 'nmpr', 'En Gannim', 'ein ganim'),
        'E73686': ('עֵינָם', 'nmpr', 'Enam', 'einam'),
        'E73687': ('שֹׂוכֹה', 'nmpr', 'Socoh', 'sovcho'),
        'E73688': ('שַׁעֲרַיִם', 'nmpr', 'Shaaraim', "sha'arayim"),
        'E73689': ('עֲדִיתַיִם', 'nmpr', 'Adithaim', 'aditayim'),
        'E73690': ('גְּדֵרָה', 'nmpr', 'Gederah', "g'deira"),
        'E73691': ('גְּדֵרֹתָיִם', 'nmpr', 'Gederothaim', "g'deirotayim"),
        'E73692': ('צְנָן', 'nmpr', 'Zenan', "ts'nan"),
        'E73693': ('חֲדָשָׁה', 'nmpr', 'Hadashah', 'chadasha'),
        'E73694': ('מִגְדַּל גָּד', 'nmpr', 'Migdal Gad', 'migdal gad'),
        'E73695': ('דִּלְעָן', 'nmpr', 'Dilean', 'dilan'),
        'E73696': ('יָקְתְאֵל', 'nmpr', 'Joktheel', 'yakteil'),
        'E73697': ('בָּצְקַת', 'nmpr', 'Bozkath', 'batskat'),
        'E73698': ('כַּבֹּון', 'nmpr', 'Cabbon', 'kabovn'),
        'E73699': ('לַחְמָס', 'nmpr', 'Lahmas', 'lachmas'),
        'E73700': ('כִּתְלִישׁ', 'nmpr', 'Kitlish', 'kitlish'),
        'E73701': ('גְּדֵרֹות', 'nmpr', 'Gederoth', "g'deirovt"),
        'E73702': ('בֵּית דָּגֹון', 'nmpr', 'Beth Dagon', 'beit dagovn'),
        'E73703': ('נַעֲמָה', 'nmpr', 'Naamah', "na'ama"),
        'E73704': ('עֶתֶר', 'nmpr', 'Ether', 'eter'),
        'E73705': ('עָשָׁן', 'nmpr', 'Ashan', 'ashan'),
        'E73706': ('יִפְתָּח', 'nmpr', 'Iphtah', 'yiftach'),
        'E73707': ('נְצִיב', 'nmpr', 'Nezib', "n'tsiv"),
        'E73708': ('קְעִילָה', 'nmpr', 'Keilah', "k'ila"),
        'E73709': ('אַכְזִיב', 'nmpr', 'Aczib', 'achziv'),
        'E73710': ('מָרֵשָׁה', 'nmpr', 'Mareshah', 'mareisha'),
        'E73711': ('שָׁמִיר', 'nmpr', 'Shamir', 'shamir'),
        'E73712': ('יַתִּר', 'nmpr', 'Jattir', 'yatir'),
        'E73713': ('דַּנָּה', 'nmpr', 'Dannah', 'dana'),
        'E73714': ('קִרְיַת סַנָּה', 'nmpr', 'Kiriath Sannah', 'kirat sana'),
        'E73715': ('אֶשְׁתְּמֹעַ', 'nmpr', 'Eshtemoa', 'eshtmoa'),
        'E73716': ('עָנִים', 'nmpr', 'Anim', 'anim'),
        'E73717': ('חֹלֹן', 'nmpr', 'Holon', 'cholon'),
        'E73718': ('גִּלֹה', 'nmpr', 'Giloh', 'gilo'),
        'E73719': ('אֲרַב', 'nmpr', 'Arab', 'arav'),
        'E73720': ('רוּמָה', 'nmpr', 'Rumah', 'ruma'),
        'E73721': ('אֶשְׁעָן', 'nmpr', 'Eshan', 'eshan'),
        'E73722': ('יָנוּם', 'nmpr', 'Janim', 'yanum'),
        'E73723': ('בֵּית תַּפּוּחַ', 'nmpr', 'Beth Tappuah', 'beit tapuach'),
        'E73724': ('אֲפֵקָה', 'nmpr', 'Aphekah', 'afeika'),
        'E73725': ('חֻמְטָה', 'nmpr', 'Humtah', 'chumta'),
        'E73726': ('צִיעֹר', 'nmpr', 'Zior', 'tsior'),
        'E73727': ('מָעֹון', 'nmpr', 'Maon', 'maovn'),
        'E73728': ('יוּטָּה', 'nmpr', 'Juttah', 'yuta'),
        'E73729': ('יִזְרְעֶאל', 'nmpr', '[town]', 'yizrel'),
        'E73730': ('יָקְדְעָם', 'nmpr', 'Jokdeam', 'yakdam'),
        'E73731': ('קַיִן', 'nmpr', 'Kain', 'kayin'),
        'E73732': ('גִּבְעָה', 'nmpr', 'Gibeah', 'giva'),
        'E73733': ('חַלְחוּל', 'nmpr', 'Halhul', 'chalchul'),
        'E73734': ('בֵּית צוּר', 'nmpr', 'Beth Zur', 'beit tsur'),
        'E73735': ('גְּדֹר', 'nmpr', 'Gedor', "g'dor"),
        'E73736': ('מַעֲרָת', 'nmpr', 'Maarath', "ma'arat"),
        'E73737': ('בֵּית עֲנֹות', 'nmpr', 'Beth Anoth', 'beit anovt'),
        'E73738': ('אֶלְתְּקֹן', 'nmpr', 'Eltekon', 'eltkon'),
        'E73739': ('קִרְיַת בַּעַל', 'nmpr', 'Kiriath Baal', "kirat ba'al"),
        'E73740': ('מִדִּין', 'nmpr', 'Middin', 'midin'),
        'E73741': ('סְכָכָה', 'nmpr', 'Secacah', "s'chocho"),
        'E73742': ('נִבְשָׁן', 'nmpr', 'Nibshan', 'nivshan'),
        'E73743': ('עִיר הַמֶּלַח', 'nmpr', 'City of Salt', 'ir hamelach'),
        'E73744': ('עֵין גֶּדִי', 'nmpr', 'En Gedi', 'ein gedi'),
        'E73745': ('אַרְכִּי', 'adjv', 'Arkite', 'arki'),
        'E73746': ('יַפְלֵטִי', 'adjv', 'Japhletite', 'yafleiti'),
        'E73747': ('תַּחְתֹּון', 'adjv', 'lower', 'tachtovn'),
        'E73748': ('עַטְרֹות אַדָּר', 'nmpr', 'Ataroth Addar', 'atrovt adar'),
        'E73749': ('מִכְמְתָת', 'nmpr', 'Micmethath', 'michmtat'),
        'E73750': ('תַּאֲנַת שִׁלֹה', 'nmpr', 'Taanath Shiloh', "ta'anat shilo"),
        'E73751': ('יָנֹוחָה', 'nmpr', 'Janoah', 'yanovcha'),
        'E73752': ('נַעֲרָה', 'nmpr', '[locality]', "na'ara"),
        'E73753': ('קָנָה', 'nmpr', 'Kanah', 'kana'),
        'E73754': ('מִבְדָּלָה', 'subs', 'enclave', 'mivdala'),
        'E73755': ('אֲבִיעֶזֶר', 'nmpr', 'Abiezer', 'aviezer'),
        'E73756': ('עֵין תַּפּוּחַ', 'nmpr', 'En Tappuah', 'ein tapuach'),
        'E73757': ('בֵּית שְׁאָן', 'nmpr', 'Beth Shan', "beit sh'an"),
        'E73758': ('יִבְלְעָם', 'nmpr', 'Ibleam', 'yivlam'),
        'E73759': ('דֹּאר', 'nmpr', 'Dor', 'dor'),
        'E73760': ('עֵין דֹּור', 'nmpr', 'Endor', 'ein dovr'),
        'E73761': ('נֶפֶת', 'subs', 'hill', 'nefet'),
        'E73762': ('ברא', 'verb', 'cut down', 'vr'),
        'E73763': ('שִׁלֹה', 'nmpr', 'Shiloh', 'shilo'),
        'E73764': ('גְּלִילֹות', 'nmpr', 'Geliloth', "g'lilovt"),
        'E73765': ('קְצִיץ', 'nmpr', 'Emek Keziz', "k'tsits"),
        'E73766': ('צְמָרַיִם', 'nmpr', '[mountain]', "ts'marayim"),
        'E73767': ('פָּרָה', 'nmpr', 'Parah', 'para'),
        'E73768': ('עָפְרָה', 'nmpr', 'Ophrah', 'afra'),
        'E73769': ('כָּפָר', 'subs', 'village', 'kafar'),
        'E73770': ('עַמֹּנָה', 'nmpr', 'Ammoni', 'amona'),
        'E73771': ('עָפְנִי', 'nmpr', 'Ophni', 'afni'),
        'E73772': ('גֶּבַע', 'nmpr', 'Geba', 'geva'),
        'E73773': ('רָמָה', 'nmpr', 'Ramah', 'rama'),
        'E73774': ('מֹצָה', 'nmpr', 'Mozah', 'motsa'),
        'E73775': ('רֶקֶם', 'nmpr', 'Rekem', 'rekem'),
        'E73776': ('יִרְפְּאֵל', 'nmpr', 'Irpeel', 'yirpeil'),
        'E73777': ('תַּרְאֲלָה', 'nmpr', 'Taralah', 'tarala'),
        'E73778': ('צֵלָע', 'nmpr', 'Zela', 'tseila'),
        'E73779': ('אֶלֶף', 'nmpr', 'Eleph', 'elef'),
        'E73780': ('גִּבְעַת', 'nmpr', 'Gibeath', 'givat'),
        'E73781': ('קִרְיַת', 'nmpr', 'Kiriath', 'kirat'),
        'E73782': ('בָּלָה', 'nmpr', 'Balah', 'bala'),
        'E73783': ('בְּתוּאֵל', 'nmpr', 'Bethuel', "b'tueil"),
        'E73784': ('בֵּית הַמַּרְכָּבֹות',
                   'nmpr',
                   'Beth Marcaboth',
                   'beit hamarkavovt'),
        'E73785': ('חֲצַר סוּסָה', 'nmpr', 'Hazar Susah', 'chatsar susa'),
        'E73786': ('בֵּית לְבָאֹות', 'nmpr', 'Beth Lebaoth', "beit l'vaovt"),
        'E73787': ('שָׁרוּחֶן', 'nmpr', 'Sharuhen', 'sharuchen'),
        'E73788': ('רִמֹּון', 'nmpr', 'Rimmon', 'rimovn'),
        'E73789': ('בַּעֲלַת בְּאֵר', 'nmpr', 'Baalath Beer', "ba'alat b'eir"),
        'E73790': ('שָׂרִיד', 'nmpr', 'Sarid', 'sarid'),
        'E73791': ('מַרְעֲלָה', 'nmpr', 'Maralah', 'marala'),
        'E73792': ('דַּבֶּשֶׁת', 'nmpr', 'Dabbesheth', 'dabeshet'),
        'E73793': ('כִּסְלֹת תָּבֹר', 'nmpr', 'Kisloth Tabor', 'kislot tavor'),
        'E73794': ('דָּבְרַת', 'nmpr', 'Daberath', 'davrat'),
        'E73795': ('יָפִיעַ', 'nmpr', 'Japhia', 'yafia'),
        'E73796': ('גַּת', 'subs', 'wine-press', 'gat'),
        'E73797': ('נֵעָה', 'nmpr', 'Neah', 'neia'),
        'E73798': ('חַנָּתֹן', 'nmpr', 'Hannathon', 'chanaton'),
        'E73799': ('יִפְתַּח אֵל', 'nmpr', 'Iphtah El', 'yiftach eil'),
        'E73800': ('קַטָּת', 'nmpr', 'Kattath', 'katat'),
        'E73801': ('נַהֲלָל', 'nmpr', 'Nahalal', 'nahalal'),
        'E73802': ('יִדְאֲלָה', 'nmpr', 'Idalah', 'yidala'),
        'E73803': ('כְּסוּלֹת', 'nmpr', 'Kesulloth', "k'sulot"),
        'E73804': ('שׁוּנֵם', 'nmpr', 'Shunem', 'shuneim'),
        'E73805': ('חֲפָרַיִם', 'nmpr', 'Hapharaim', 'chafarayim'),
        'E73806': ('שִׁיאֹן', 'nmpr', 'Shion', 'shion'),
        'E73807': ('אֲנָחֲרַת', 'nmpr', 'Anaharath', 'anacharat'),
        'E73808': ('רַבִּית', 'nmpr', 'Rabbith', 'rabit'),
        'E73809': ('קִשְׁיֹון', 'nmpr', 'Kishion', 'kishovn'),
        'E73810': ('אָבֶץ', 'nmpr', 'Ebez', 'avets'),
        'E73811': ('רֶמֶת', 'nmpr', 'Remeth', 'remet'),
        'E73812': ('עֵין חַדָּה', 'nmpr', 'En Haddah', 'ein chada'),
        'E73813': ('בֵּית פַּצֵּץ', 'nmpr', 'Beth Pazzez', 'beit patseits'),
        'E73814': ('תָּבֹור', 'nmpr', 'Tabor', 'tavovr'),
        'E73815': ('שַׁחֲצִימָה', 'nmpr', 'Shahazumah', 'shachatsima'),
        'E73816': ('חֶלְקַת', 'nmpr', 'Helkath', 'chelkat'),
        'E73817': ('חֲלִי', 'nmpr', 'Hali', 'chali'),
        'E73818': ('בֶּטֶן', 'nmpr', 'Beten', 'beten'),
        'E73819': ('אַלַּמֶּלֶךְ', 'nmpr', 'Allammelech', 'alamelech'),
        'E73820': ('עַמְעָד', 'nmpr', 'Amad', 'amad'),
        'E73821': ('מִשְׁאָל', 'nmpr', 'Mishal', 'mishal'),
        'E73822': ('שִׁיחֹור לִבְנָת', 'nmpr', 'Shihor Libnath', 'shichovr livnat'),
        'E73823': ('בֵּית הָעֵמֶק', 'nmpr', 'Beth Emek', 'beit haeimek'),
        'E73824': ('נְעִיאֵל', 'nmpr', 'Neiel', "n'ieil"),
        'E73825': ('כָּבוּל', 'nmpr', 'Cabul', 'kavul'),
        'E73826': ('עֶבְרֹן', 'nmpr', 'Abdon', 'evron'),
        'E73827': ('חַמֹּון', 'nmpr', 'Hammon', 'chamovn'),
        'E73828': ('צֹר', 'nmpr', 'Tyrus', 'tsor'),
        'E73829': ('חֹסָה', 'nmpr', 'Hosah', 'chosa'),
        'E73830': ('עֻמָה', 'nmpr', 'Ummah', 'uma'),
        'E73831': ('חֵלֶף', 'nmpr', 'Heleph', 'cheilef'),
        'E73832': ('צַעֲנַנִּים', 'nmpr', 'Zaanannim', "tsa'ananim"),
        'E73833': ('אֲדָמִי הַנֶּקֶב', 'nmpr', 'Adami-nekeb', 'adami hanekev'),
        'E73834': ('לַקּוּם', 'nmpr', 'Lakkum', 'lakum'),
        'E73835': ('אַזְנֹות תָּבֹור', 'nmpr', 'Aznoth Tabor', 'aznovt tavovr'),
        'E73836': ('חֻקֹּק', 'nmpr', 'Hukkok', 'chukok'),
        'E73837': ('צִדִּים', 'nmpr', 'Ziddim', 'tsidim'),
        'E73838': ('צֵר', 'nmpr', 'Zer', 'tseir'),
        'E73839': ('חַמַּת', 'nmpr', 'Hammath', 'chamat'),
        'E73840': ('רַקַּת', 'nmpr', 'Rakkath', 'rakat'),
        'E73841': ('אֲדָמָה', 'nmpr', 'Adamah', 'adama'),
        'E73842': ('עֵין חָצֹור', 'nmpr', 'En Hazor', 'ein chatsovr'),
        'E73843': ('יִרְאֹון', 'nmpr', 'Iron', 'yirovn'),
        'E73844': ('מִגְדַּל אֵל', 'nmpr', 'Migdal El', 'migdal eil'),
        'E73845': ('חֳרֵם', 'nmpr', 'Horem', 'chareim'),
        'E73846': ('בֵּית עֲנָת', 'nmpr', 'Beth Anath', 'beit anat'),
        'E73847': ('עִיר שֶׁמֶשׁ', 'nmpr', 'Ir Shemesh', 'ir shemesh'),
        'E73848': ('שַׁעֲלַבִּין', 'nmpr', 'Shaalabbin', "sha'alabin"),
        'E73849': ('יִתְלָה', 'nmpr', 'Ithlah', 'yitla'),
        'E73850': ('אֵילֹון', 'nmpr', 'Elon', 'eilovn'),
        'E73851': ('אֶלְתְּקֵא', 'nmpr', 'Eltekeh', 'eltkei'),
        'E73852': ('גִּבְּתֹון', 'nmpr', 'Gibbethon', 'gibtovn'),
        'E73853': ('בַּעֲלַת', 'nmpr', 'Baalath', "ba'alat"),
        'E73854': ('יְהוּד', 'nmpr', 'Jehud', "y'hud"),
        'E73855': ('בְּנֵי בְרַק', 'nmpr', 'Bene Berak', "b'nei v'rak"),
        'E73856': ('גַּת רִמֹּון', 'nmpr', 'Gath Rimmon', 'gat rimovn'),
        'E73857': ('יַרְקֹון', 'nmpr', 'Jarkon', 'yarkovn'),
        'E73858': ('רַקֹּון', 'nmpr', 'Rakkon', 'rakovn'),
        'E73859': ('יָפֹו', 'nmpr', 'Joppa', 'yafov'),
        'E73860': ('לֶשֶׁם', 'nmpr', 'Leshem', 'leshem'),
        'E73861': ('תִּמְנַת סֶרַח', 'nmpr', 'Timnath Serah', 'timnat serach'),
        'E73862': ('גָּלִיל', 'nmpr', 'Galilee', 'galil'),
        'E73863': ('מוּעָדָה', 'subs', 'appointing', 'muada'),
        'E73864': ('עֲנֹוק', 'nmpr', 'Anak', 'anovk'),
        'E73865': ('עֲנָתֹות', 'nmpr', '[town]', 'anatovt'),
        'E73866': ('עַלְמֹון', 'nmpr', 'Almon', 'almovn'),
        'E73867': ('קִבְצַיִם', 'nmpr', 'Kibzaim', 'kivtsayim'),
        'E73868': ('בְּעֶשְׁתְּרָה', 'nmpr', 'Be Eshtarah', "b'eshtra"),
        'E73869': ('עַבְדֹּון', 'nmpr', 'Abdon', 'avdovn'),
        'E73870': ('חַמֹּת דֹּאר', 'nmpr', 'Hammoth Dor', 'chamot dor'),
        'E73871': ('קַרְתָּן', 'nmpr', 'Kartan', 'kartan'),
        'E73872': ('קַרְתָּה', 'nmpr', 'Kartah', 'karta'),
        'E73873': ('דִּמְנָה', 'nmpr', 'Dimnah', 'dimna'),
        'E73874': ('נְכָסִים', 'subs', 'riches', "n'chosim"),
        'E73875': ('מֶרֶד', 'subs', 'rebellion', 'mered'),
        'E73876': ('דְּאָגָה', 'subs', 'care', "d'aga"),
        'E73877': ('פַּח', 'subs', 'bird-trap', 'pach'),
        'E73878': ('שֹׁטֵט', 'subs', '[uncertain]', 'shoteit'),
        'E73879': ('מַאֲפֵל', 'subs', 'darkness', "ma'afeil"),
        'E73880': ('קַנֹּוא', 'adjv', 'jealous', 'kanov'),
        'E73881': ('אַלָּה', 'subs', 'big tree', 'ala'),
        'E73882': ('גַּעַשׁ', 'nmpr', 'Gaash', "ga'ash"),
        'E73883': ('בֶּזֶק', 'nmpr', 'Bezek', 'bezek'),
        'E73884': ('אֲדֹנִי בֶזֶק', 'nmpr', 'Adoni-Bezek', 'adoni vezek'),
        'E73885': ('בְּהֹון', 'subs', 'thumb', "b'hovn"),
        'E73886': ('צְפַת', 'nmpr', 'Zephath', "ts'fat"),
        'E73887': ('אַשְׁקְלֹון', 'nmpr', 'Ashkelon', 'ashklovn'),
        'E73888': ('קִטְרֹון', 'nmpr', 'Kitron', 'kitrovn'),
        'E73889': ('נַהֲלֹל', 'nmpr', 'Nahalol', 'nahalol'),
        'E73890': ('עַכֹּו', 'nmpr', 'Acco', 'akov'),
        'E73891': ('אַחְלָב', 'nmpr', 'Ahlab', 'achlav'),
        'E73892': ('חֶלְבָּה', 'nmpr', 'Helbah', 'chelba'),
        'E73893': ('אֲפִיק', 'nmpr', 'Aphek', 'afik'),
        'E73894': ('אָשֵׁרִי', 'adjv', 'from Asher', 'asheiri'),
        'E73895': ('הַר חֶרֶס', 'nmpr', 'Mount Heres', 'har cheres'),
        'E73896': ('שַׁעַלְבִים', 'nmpr', 'Shaalbim', "sha'alvim"),
        'E73897': ('סֶלַע', 'nmpr', 'Sela', 'sela'),
        'E73898': ('בֹּכִים', 'nmpr', 'Bokim', 'bochim'),
        'E73899': ('צַד', 'subs', 'snare', 'tsad'),
        'E73900': ('תִּמְנַת חֶרֶס', 'nmpr', 'Timnath Heres', 'timnat cheres'),
        'E73901': ('עַשְׁתֹּרֶת', 'subs', 'idol', 'ashtoret'),
        'E73902': ('שׁסה', 'verb', 'spoil', "shs'h"),
        'E73903': ('שׁסס', 'verb', 'plunder', "shs's"),
        'E73904': ('דחק', 'verb', 'urge', 'dchk'),
        'E73905': ('בַּעַל חֶרְמֹון', 'nmpr', 'Baal Hermon', "ba'al chermovn"),
        'E73906': ('כּוּשַׁן רִשְׁעָתָיִם',
                   'nmpr',
                   'Cushan-Rishathaim',
                   'kushan rishatayim'),
        'E73907': ('עזז', 'verb', 'be strong', "z'z"),
        'E73908': ('עֶגְלֹון', 'nmpr', 'Eglon', 'eglovn'),
        'E73909': ('אֵהוּד', 'nmpr', 'Ehud', 'eihud'),
        'E73910': ('יְמִינִי', 'adjv', 'Benjaminite', "y'mini"),
        'E73911': ('אִטֵּר', 'adjv', 'left-handed', 'iteir'),
        'E73912': ('גֹּמֶד', 'subs', 'cubit', 'gomed'),
        'E73913': ('עֲלִיָּה', 'subs', 'upper room', 'alia'),
        'E73914': ('מְקֵרָה', 'subs', 'coolness', "m'keira"),
        'E73915': ('נִצָּב', 'subs', 'handle', 'nitsav'),
        'E73916': ('לַהַב', 'subs', 'flame', 'lahav'),
        'E73917': ('פַּרְשְׁדֹן', 'subs', 'hole', 'parshdon'),
        'E73918': ('מִסְדְּרֹון', 'subs', '[uncertain]', 'misdrovn'),
        'E73919': ('נעל', 'verb', 'lock', 'nl'),
        'E73920': ('מַפְתֵּחַ', 'subs', 'key', 'mafteiach'),
        'E73921': ('שְׂעִירָה', 'nmpr', 'Seirah', "s'ira"),
        'E73922': ('שַׁמְגַּר', 'nmpr', 'Shamgar', 'shamgar'),
        'E73923': ('עֲנָת', 'nmpr', 'Anath', 'anat'),
        'E73924': ('מַלְמָד', 'subs', 'ox-goad', 'malmad'),
        'E73925': ('סִיסְרָא', 'nmpr', 'Sisera', 'sisra'),
        'E73926': ('חֲרֹשֶׁת הַגֹּויִם',
                   'nmpr',
                   'Harosheth Haggoyim',
                   'charoshet hagovyim'),
        'E73927': ('חָזְקָה', 'subs', 'strength', 'chazka'),
        'E73928': ('לַפִּידֹות', 'nmpr', 'Lappidoth', 'lapidovt'),
        'E73929': ('תֹּמֶר', 'subs', 'date-palm', 'tomer'),
        'E73930': ('בָּרָק', 'nmpr', 'Barak', 'barak'),
        'E73931': ('אֲבִינֹעַם', 'nmpr', 'Abinoam', 'avinoam'),
        'E73932': ('קִישֹׁון', 'nmpr', 'Kishon', 'kishovn'),
        'E73933': ('יָעֵל', 'nmpr', 'Jael', 'yaeil'),
        'E73934': ('שְׂמִיכָה', 'subs', 'curtain', "s'micho"),
        'E73935': ('מַקֶּבֶת', 'subs', 'hammer', 'makevet'),
        'E73936': ('רַקָּה', 'subs', 'temple', 'raka'),
        'E73937': ('רדם', 'verb', 'sleep', 'rdm'),
        'E73938': ('עיף', 'verb', 'be faint', 'yf'),
        'E73939': ('רזן', 'verb', 'be weighty', 'rzn'),
        'E73940': ('זמר', 'verb', 'sing', 'zmr'),
        'E73941': ('רעשׁ', 'verb', 'quake', 'rsh'),
        'E73942': ('נטף', 'verb', 'drop', 'ntf'),
        'E73943': ('נְתִיבָה', 'subs', 'path', "n'tiva"),
        'E73944': ('עֲקַלְקָל', 'adjv', 'crooked', 'akalkal'),
        'E73945': ('פְּרָזֹון', 'subs', 'countrymen', "p'razovn"),
        'E73946': ('לָחֶם', 'subs', '[uncertain]', 'lachem'),
        'E73947': ('צָחֹר', 'adjv', 'yellowish red', 'tsachor'),
        'E73948': ('שׂיח', 'verb', 'be concerned with', 'sych'),
        'E73949': ('חצץ', 'verb', 'divide', "chts'ts"),
        'E73950': ('מַשְׁאָב', 'subs', 'watering-channel', 'mashav'),
        'E73951': ('תנה', 'verb', 'recount', 'tnh'),
        'E73952': ('סֹפֵר', 'subs', 'scribe', 'sofeir'),
        'E73953': ('פְּלַגָּה', 'subs', 'division', "p'laga"),
        'E73954': ('שְׁרֵקָה', 'subs', 'whistling', "sh'reika"),
        'E73955': ('חֵקֶר', 'subs', 'exploration', 'cheiker'),
        'E73956': ('מִפְרָץ', 'subs', 'landing place', 'mifrats'),
        'E73957': ('חרף', 'verb', 'reproach', 'chrf'),
        'E73958': ('מָרֹום', 'subs', 'high place', 'marovm'),
        'E73959': ('גרף', 'verb', 'wash away', 'grf'),
        'E73960': ('קְדוּמִים', 'subs', '[uncertain]', "k'dumim"),
        'E73961': ('הלם', 'verb', 'strike', 'hlm'),
        'E73962': ('דַּהֲרָה', 'subs', 'dashing', 'dahara'),
        'E73963': ('אַבִּיר', 'adjv', 'strong', 'abir'),
        'E73964': ('מֵרֹוז', 'nmpr', 'Meroz', 'meirovz'),
        'E73965': ('עֶזְרָה', 'subs', 'help', 'ezra'),
        'E73966': ('סֵפֶל', 'subs', 'bowl', 'seifel'),
        'E73967': ('הַלְמוּת', 'subs', 'hammer', 'halmut'),
        'E73968': ('עָמֵל', 'adjv', 'labouring', 'ameil'),
        'E73969': ('מחק', 'verb', 'skin', 'mchk'),
        'E73970': ('חלף', 'verb', 'cut', 'chlf'),
        'E73971': ('שׁדד', 'verb', 'despoil', "shd'd"),
        'E73972': ('יבב', 'verb', 'lament', "yv'v"),
        'E73973': ('אֶשְׁנָב', 'subs', 'window', 'eshnav'),
        'E73974': ('שָׂרָה', 'subs', 'mistress', 'sara'),
        'E73975': ('רַחְמָה', 'subs', 'womb', 'rachma'),
        'E73976': ('צֶבַע', 'subs', 'stuff', 'tseva'),
        'E73977': ('רִקְמָה', 'subs', 'woven stuff', 'rikma'),
        'E73978': ('מִנְהָרָה', 'subs', 'store', 'minhara'),
        'E73979': ('מְצָד', 'subs', 'unapproachable', "m'tsad"),
        'E73980': ('דלל', 'verb', 'belittle', "dl'l"),
        'E73981': ('יֹואָשׁ', 'nmpr', 'Joash', 'yovash'),
        'E73982': ('עֶזְרִי', 'adjv', '-ezrite', 'ezri'),
        'E73983': ('גִּדְעֹון', 'nmpr', 'Gideon', 'gidovn'),
        'E73984': ('מָרָק', 'subs', 'juice', 'marak'),
        'E73985': ('לָז', 'prde', 'this there', 'laz'),
        'E73986': ('מָעֹוז', 'subs', 'fort', 'maovz'),
        'E73987': ('יְרֻבַּעַל', 'nmpr', 'Jerub-Baal', "y'ruba'al"),
        'E73988': ('גִּזָּה', 'subs', 'fleece', 'giza'),
        'E73989': ('זור', 'verb', 'press', 'zvr'),
        'E73990': ('עֵין חֲרֹד', 'nmpr', 'the spring of Harod', 'ein charod'),
        'E73991': ('חָרֵד', 'adjv', 'trembling', 'chareid'),
        'E73992': ('צפר', 'verb', '[uncertain]', 'tsfr'),
        'E73993': ('צרף', 'verb', 'melt', 'tsrf'),
        'E73994': ('לקק', 'verb', 'lick', "lk'k"),
        'E73995': ('פֻּרָה', 'nmpr', 'Purah', 'pura'),
        'E73996': ('צְלִיל', 'subs', '[uncertain]', "ts'lil"),
        'E73997': ('שֵׁבֶר', 'subs', 'interpretation', 'sheiver'),
        'E73998': ('בֵּית הַשִּׁטָּה', 'nmpr', 'Beth Shittah', 'beit hashita'),
        'E73999': ('צְרֵרָה', 'nmpr', 'Zererah', "ts'reira"),
        'E74000': ('אָבֵל מְחֹולָה', 'nmpr', 'Abel Meholah', "aveil m'chovla"),
        'E74001': ('טַבָּת', 'nmpr', 'Tabbath', 'tabat'),
        'E74002': ('בֵּית בָּרָה', 'nmpr', 'Beth Barah', 'beit bara'),
        'E74003': ('עֹרֵב', 'nmpr', 'Oreb', 'oreiv'),
        'E74004': ('זְאֵב', 'nmpr', 'Zeeb', "z'eiv"),
        'E74005': ('עֹלֵלֹות', 'subs', 'gleaning', 'oleilovt'),
        'E74006': ('זֶבַח', 'nmpr', 'Zebah', 'zevach'),
        'E74007': ('צַלְמֻנָּע', 'nmpr', 'Zalmunna', 'tsalmuna'),
        'E74008': ('בַּרְקֳנִים', 'subs', '[uncertain]', 'barkanim'),
        'E74009': ('קַרְקֹר', 'nmpr', 'Karkor', 'karkor'),
        'E74010': ('מַעֲלֵה הֶחָרֶס', 'nmpr', 'the Pass of Heres', "ma'alei hechares"),
        'E74011': ('יָעֵף', 'adjv', 'weary', 'yaeif'),
        'E74012': ('שַׂהֲרֹנִים', 'subs', '[ornament]', 'saharonim'),
        'E74013': ('שְׁאֵלָה', 'subs', 'request', "sh'eila"),
        'E74014': ('נְטִפָה', 'subs', 'eardrops', "n'tifa"),
        'E74015': ('עֲנָק', 'subs', 'necklace', 'anak'),
        'E74016': ('בַּעַל בְּרִית', 'nmpr', 'Baal-Berith', "ba'al b'rit"),
        'E74017': ('פחז', 'verb', 'be insolent', 'fchz'),
        'E74018': ('יֹותָם', 'nmpr', 'Jotham', 'yovtam'),
        'E74019': ('מִלֹּוא', 'subs', 'mound', 'milov'),
        'E74020': ('מֻצָּב', 'subs', 'post', 'mutsav'),
        'E74021': ('מֹתֶק', 'subs', 'sweetness', 'motek'),
        'E74022': ('גְּמוּל', 'subs', 'deed', "g'mul"),
        'E74023': ('גַּעַל', 'nmpr', 'Gaal', "ga'al"),
        'E74024': ('עֶבֶד', 'nmpr', 'Ebed', 'eved'),
        'E74025': ('זְבֻל', 'nmpr', 'Zebul', "z'vul"),
        'E74026': ('תָּרְמָה', 'nmpr', 'Tormah', 'tarma'),
        'E74027': ('טַבּוּר', 'subs', 'navel', 'tabur'),
        'E74028': ('אֲרוּמָה', 'nmpr', 'Arumah', 'aruma'),
        'E74029': ('צְרִיחַ', 'subs', 'cellar', "ts'riach"),
        'E74030': ('אֵל בְּרִית', 'nmpr', 'El-Berith', "eil b'rit"),
        'E74031': ('צַלְמֹון', 'nmpr', 'Zalmon', 'tsalmovn'),
        'E74032': ('קַרְדֹּם', 'subs', 'axe', 'kardom'),
        'E74033': ('שֹׂוכָה', 'subs', 'brushwood', 'sovcho'),
        'E74034': ('שֹׂוךְ', 'subs', '[uncertain]', 'sovch'),
        'E74035': ('תֵּבֵץ', 'nmpr', 'Thebez', 'teiveits'),
        'E74036': ('פֶּלַח', 'subs', 'slice', 'pelach'),
        'E74037': ('פּוּאָה', 'nmpr', 'Puah', 'pua'),
        'E74038': ('דֹּודֹו', 'nmpr', 'Dodo', 'dovdov'),
        'E74039': ('קָמֹון', 'nmpr', 'Kamon', 'kamovn'),
        'E74040': ('יִפְתָּח', 'nmpr', '[judge]', 'yiftach'),
        'E74041': ('טֹוב', 'nmpr', 'Tob', 'tovv'),
        'E74042': ('עַרְעֹור', 'nmpr', 'Aroer', 'arovr'),
        'E74043': ('מִנִּית', 'nmpr', 'Minnith', 'minit'),
        'E74044': ('אָבֵל כְּרָמִים', 'nmpr', 'Abel Keramim', "aveil k'ramim"),
        'E74045': ('רֵעָה', 'subs', 'female companion', 'reia'),
        'E74046': ('אֶפְרָתִי', 'adjv', 'Ephrathite', 'efrati'),
        'E74047': ('סִבֹּלֶת', 'subs', 'ear of grain', 'sibolet'),
        'E74048': ('אִבְצָן', 'nmpr', 'Ibzan', 'ivtsan'),
        'E74049': ('עַבְדֹּון', 'nmpr', 'Abdon', 'avdovn'),
        'E74050': ('הִלֵּל', 'nmpr', 'Hillel', 'hileil'),
        'E74051': ('פִּרְעָתֹונִי', 'adjv', 'Pirathonite', 'piratovni'),
        'E74052': ('פִּרְעָתֹון', 'nmpr', 'Pirathon', 'piratovn'),
        'E74053': ('דָּנִי', 'adjv', 'Danite', 'dani'),
        'E74054': ('מָנֹוחַ', 'nmpr', 'Manoah', 'manovach'),
        'E74055': ('מֹורָה', 'subs', 'razor', 'movra'),
        'E74056': ('פֶּלִאי', 'adjv', 'miraculous', 'peli'),
        'E74057': ('שִׁמְשֹׁון', 'nmpr', 'Samson', 'shimshovn'),
        'E74058': ('מַחֲנֵה דָן', 'nmpr', 'Mahaneh Dan', 'machanei dan'),
        'E74059': ('תֹּאֲנָה', 'subs', 'occasion', 'toana'),
        'E74060': ('כְּפִיר', 'subs', 'young lion', "k'fir"),
        'E74061': ('שׁאג', 'verb', 'roar', 'shg'),
        'E74062': ('מַפֶּלֶת', 'subs', 'carcass', 'mapelet'),
        'E74063': ('רדה', 'verb', 'scrape out', 'rdh'),
        'E74064': ('חוד', 'verb', 'propound a riddle', 'chvd'),
        'E74065': ('סָדִין', 'subs', 'wrapper', 'sadin'),
        'E74066': ('מָתֹוק', 'adjv', 'sweet', 'matovk'),
        'E74067': ('חֶרֶס', 'subs', 'sun', 'cheres'),
        'E74068': ('חֲלִיצָה', 'subs', 'stripped', 'chalitsa'),
        'E74069': ('רעה', 'verb', 'deal with', 'rh'),
        'E74070': ('שׁוּעָל', 'subs', 'fox', 'shual'),
        'E74071': ('תִּמְנִי', 'adjv', 'Timnite', 'timni'),
        'E74072': ('סָעִיף', 'subs', 'cleft', 'saif'),
        'E74073': ('עֵיטָם', 'nmpr', 'Etam', 'eitam'),
        'E74074': ('לְחִי', 'nmpr', 'Lehi', "l'chi"),
        'E74075': ('אֵסוּר', 'subs', 'fetter', 'eisur'),
        'E74076': ('טָרִי', 'adjv', 'fresh', 'tari'),
        'E74077': ('תְּשׁוּעָה', 'subs', 'salvation', "t'shua"),
        'E74078': ('מַכְתֵּשׁ', 'subs', 'grinder', 'machteish'),
        'E74079': ('שֹׂרֵק', 'nmpr', 'Sorek', 'soreik'),
        'E74080': ('דְּלִילָה', 'nmpr', 'Delilah', "d'lila"),
        'E74081': ('יֶתֶר', 'subs', 'sinew', 'yeter'),
        'E74082': ('נְעֹרֶת', 'subs', 'tow', "n'oret"),
        'E74083': ('כָּזָב', 'subs', 'lie', 'kazav'),
        'E74084': ('מַחְלָפֹות', 'subs', 'plaits', 'machlafovt'),
        'E74085': ('מַסֶּכֶת', 'subs', 'warp-threads', 'masechet'),
        'E74086': ('אֶרֶג', 'subs', "weaver's bobbin", 'ereg'),
        'E74087': ('אלץ', 'verb', 'press hard', 'lts'),
        'E74088': ('דָּגֹון', 'nmpr', 'Dagon', 'dagovn'),
        'E74089': ('שׂחק', 'verb', 'laugh', 'schk'),
        'E74090': ('לפת', 'verb', 'grasp', 'lft'),
        'E74091': ('מִיכָיְהוּ', 'nmpr', '[prophet]', 'michoyhu'),
        'E74092': ('אלה', 'verb', 'swear', 'lh'),
        'E74093': ('מִיכָה', 'nmpr', 'Micah', 'micho'),
        'E74094': ('קְצֹות', 'subs', 'end', "k'tsovt"),
        'E74095': ('זֹה', 'prde', 'this', 'zo'),
        'E74096': ('לַיִשׁ', 'nmpr', '[locality]', 'layish'),
        'E74097': ('עֶצֶר', 'subs', 'oppression', 'etser'),
        'E74098': ('חשׁה', 'verb', 'be silent', 'chshh'),
        'E74099': ('עצל', 'verb', 'be sluggish', 'tsl'),
        'E74100': ('כְּבוּדָּה', 'subs', 'what is valuable', "k'vuda"),
        'E74101': ('בֵּית רְחֹוב', 'nmpr', 'Beth Rehob', "beit r'chovv"),
        'E74102': ('יְהֹונָתָן', 'nmpr', 'Jehonathan', "y'hovnatan"),
        'E74103': ('זנה', 'verb', 'feel repugnance', 'znh'),
        'E74104': ('צֶמֶד', 'subs', 'span', 'tsemed'),
        'E74105': ('ערב', 'verb', 'become evening', 'rv'),
        'E74106': ('יְבוּס', 'nmpr', 'Jebus', "y'vus"),
        'E74107': ('ארח', 'verb', 'wander', 'rch'),
        'E74108': ('סַף', 'subs', 'threshold', 'saf'),
        'E74109': ('עוץ', 'verb', 'plan', 'vts'),
        'E74110': ('חָבֵר', 'subs', 'companion', 'chaveir'),
        'E74111': ('קלע', 'verb', 'sling forth', 'kl'),
        'E74112': ('שַׂעֲרָה', 'subs', 'single hair', "sa'ara"),
        'E74113': ('צום', 'verb', 'fast', 'tsvm'),
        'E74114': ('בַּעַל תָּמָר', 'nmpr', 'Baal Tamar', "ba'al tamar"),
        'E74115': ('גיח', 'verb', 'burst forth', 'gych'),
        'E74116': ('מַעֲרֶה', 'subs', 'vicinity', "ma'areh"),
        'E74117': ('כתר', 'verb', 'surround', 'chtr'),
        'E74118': ('גִּדְעֹם', 'nmpr', 'Gidom', 'gidom'),
        'E74119': ('מְתֹם', 'subs', 'sound spot', "m'tom"),
        'E74120': ('יָבֵישׁ', 'nmpr', 'Jabesh', 'yaveish'),
        'E74121': ('לְבֹונָה', 'nmpr', 'Lebonah', "l'vovna"),
        'E74122': ('חול', 'verb', 'dance', 'chvl'),
        'E74123': ('חטף', 'verb', 'seize', 'chtf'),
        'E74124': ('יְרֹחָם', 'nmpr', 'Jeroham', "y'rocham"),
        'E74125': ('אֱלִיהוּא', 'nmpr', 'Elihu', 'elihu'),
        'E74126': ('תֹּחוּ', 'nmpr', 'Tohu', 'tochu'),
        'E74127': ('צוּף', 'nmpr', 'Zuph', 'tsuf'),
        'E74128': ('חַנָּה', 'nmpr', 'Hannah', 'chana'),
        'E74129': ('פְּנִנָּה', 'nmpr', 'Peninnah', "p'nina"),
        'E74130': ('עֵלִי', 'nmpr', 'Eli', 'eili'),
        'E74131': ('חָפְנִי', 'nmpr', 'Hophni', 'chafni'),
        'E74132': ('צָרָה', 'subs', 'rival wife', 'tsara'),
        'E74133': ('רעם', 'verb', 'be disconcerted', 'rm'),
        'E74134': ('הֵיכָל', 'subs', 'palace', 'heichol'),
        'E74135': ('שִׁכֹּור', 'adjv', 'drunk', 'shikovr'),
        'E74136': ('שִׂיחַ', 'subs', 'concern', 'siach'),
        'E74137': ('נֵבֶל', 'subs', 'jar', 'neivel'),
        'E74138': ('עלץ', 'verb', 'rejoice', 'lts'),
        'E74139': ('עָתָק', 'adjv', 'unrestrained', 'atak'),
        'E74140': ('דֵּעָה', 'subs', 'knowledge', 'deia'),
        'E74141': ('תכן', 'verb', 'estimate', "t'chn"),
        'E74142': ('חַת', 'adjv', 'terror-filled', 'chat'),
        'E74143': ('אזר', 'verb', 'put on', 'zr'),
        'E74144': ('רָעֵב', 'adjv', 'hungry', 'raeiv'),
        'E74145': ('אמל', 'verb', 'wither', 'ml'),
        'E74146': ('שׁפל', 'verb', 'be low', 'shfl'),
        'E74147': ('אַשְׁפֹּת', 'subs', 'ash pit', 'ashpot'),
        'E74148': ('מָצוּק', 'subs', 'support', 'matsuk'),
        'E74149': ('תֵּבֵל', 'subs', 'world', 'teiveil'),
        'E74150': ('רעם', 'verb', 'thunder', 'rm'),
        'E74151': ('מַזְלֵג', 'subs', 'fork', 'mazleig'),
        'E74152': ('דּוּד', 'subs', 'cooking pot', 'dud'),
        'E74153': ('קַלַּחַת', 'subs', 'cauldron', 'kalachat'),
        'E74154': ('צלה', 'verb', 'roast', 'tslh'),
        'E74155': ('שְׁמוּעָה', 'subs', 'report', "sh'mua"),
        'E74156': ('ברא', 'verb', 'fatten', 'vr'),
        'E74157': ('אדב', 'verb', 'invite', 'dv'),
        'E74158': ('אֲגֹורָה', 'subs', 'payment', 'agovra'),
        'E74159': ('ספח', 'verb', 'attach', 'sfch'),
        'E74160': ('יָקָר', 'adjv', 'rare', 'yakar'),
        'E74161': ('חָזֹון', 'subs', 'vision', 'chazovn'),
        'E74162': ('צלל', 'verb', 'tingle', "tsl'l"),
        'E74163': ('כהה', 'verb', 'rebuke', "chh'h"),
        'E74164': ('אֶבֶן הָעֵזֶר', 'nmpr', 'Ebenezer', 'even haeizer'),
        'E74165': ('אֶתְמֹול', 'subs', 'yesterday', 'etmovl'),
        'E74166': ('בשׂר', 'verb', 'announce', 'vsr'),
        'E74167': ('מַפְרֶקֶת', 'subs', 'neck', 'mafreket'),
        'E74168': ('צִיר', 'subs', 'convulsion', 'tsir'),
        'E74169': ('אִיכָבֹוד', 'nmpr', 'Ichabod', 'ichovovd'),
        'E74170': ('מִפְתָּן', 'subs', 'podium', 'miftan'),
        'E74171': ('שׂתר', 'verb', 'break out', 'str'),
        'E74172': ('אַרְגַּז', 'subs', 'saddle-bag', 'argaz'),
        'E74173': ('מִקְרֶה', 'subs', 'accident', 'mikreh'),
        'E74174': ('געה', 'verb', 'low', 'gh'),
        'E74175': ('שִׁמְשִׁי', 'adjv', '-shemeshite', 'shimshi'),
        'E74176': ('כֹּפֶר', 'subs', 'village', 'kofer'),
        'E74177': ('אָבֵל', 'subs', 'water-course', 'aveil'),
        'E74178': ('אֲבִינָדָב', 'nmpr', 'Abinadab', 'avinadav'),
        'E74179': ('נהה', 'verb', 'hold', "nh'h"),
        'E74180': ('טָלֶה', 'subs', 'lamb', 'taleh'),
        'E74181': ('בֵּית כָּר', 'nmpr', 'Beth Car', 'beit kar'),
        'E74182': ('שֵׁן', 'nmpr', 'Shen', 'shein'),
        'E74183': ('תְּשׁוּבָה', 'subs', 'return', "t'shuva"),
        'E74184': ('יֹואֵל', 'nmpr', 'Joel', 'yoveil'),
        'E74185': ('אֲבִיָּה', 'nmpr', 'Abijah', 'abia'),
        'E74186': ('רַקָּח', 'subs', 'ointment mixer', 'rakach'),
        'E74187': ('טַבָּחָה', 'subs', 'female cook', 'tabacha'),
        'E74188': ('קִישׁ', 'nmpr', 'Kish', 'kish'),
        'E74189': ('אֲבִיאֵל', 'nmpr', 'Abiel', 'avieil'),
        'E74190': ('צְרֹור', 'nmpr', 'Zeror', "ts'rovr"),
        'E74191': ('בְּכֹורַת', 'nmpr', 'Becorath', "b'chovrat"),
        'E74192': ('אֲפִיחַ', 'nmpr', 'Aphiah', 'afiach'),
        'E74193': ('שָׁלִשָׁה', 'nmpr', 'Shalisha', 'shalisha'),
        'E74194': ('שַׁעֲלִים', 'nmpr', 'Shaalim', "sha'alim"),
        'E74195': ('דאג', 'verb', 'be afraid', 'dg'),
        'E74196': ('תְּשׁוּרָה', 'subs', 'gift', "t'shura"),
        'E74197': ('רֹאֶה', 'subs', 'seer', 'roeh'),
        'E74198': ('נָגִיד', 'subs', 'chief', 'nagid'),
        'E74199': ('חֶמְדָּה', 'subs', 'what is desirable', 'chemda'),
        'E74200': ('לִשְׁכָּה', 'subs', 'hall', 'lishka'),
        'E74201': ('פַּךְ', 'subs', 'flask', 'pach'),
        'E74202': ('צֶלְצַח', 'nmpr', 'Zelzah', 'tseltsach'),
        'E74203': ('נֵבֶל', 'subs', 'harp', 'neivel'),
        'E74204': ('חָלִיל', 'subs', 'flute', 'chalil'),
        'E74205': ('מְלוּכָה', 'subs', 'kingship', "m'lucho"),
        'E74206': ('מַטְרִי', 'adjv', 'Matrite', 'matri'),
        'E74207': ('גָּבַהּ', 'verb', 'be high', 'gavah'),
        'E74208': ('נָחָשׁ', 'nmpr', 'Nahash', 'nachash'),
        'E74209': ('חדשׁ', 'verb', 'be new', 'chdsh'),
        'E74210': ('שׂיב', 'verb', 'be old', 'syv'),
        'E74211': ('בְּדָן', 'nmpr', 'Bedan', "b'dan"),
        'E74212': ('יעל', 'verb', 'profit', 'yl'),
        'E74213': ('מִכְמָשׂ', 'nmpr', 'Micmash', 'michmas'),
        'E74214': ('יֹונָתָן', 'nmpr', 'Jonathan', 'yovnatan'),
        'E74215': ('חֹוחַ', 'subs', 'recess', 'chovach'),
        'E74216': ('שׁוּעָל', 'nmpr', 'Shual', 'shual'),
        'E74217': ('צְבֹעִים', 'nmpr', 'Zeboim', "ts'voim"),
        'E74218': ('חֲנִית', 'subs', 'spear', 'chanit'),
        'E74219': ('מַחֲרֶשֶׁת', 'subs', 'ploughshare', 'machareshet'),
        'E74220': ('אֵת', 'subs', 'ploughshare', 'eit'),
        'E74221': ('מַחֲרֵשָׁה', 'subs', 'ploughshare', 'machareisha'),
        'E74222': ('פְּצִירָה', 'subs', 'sharpening', "p'tsira"),
        'E74223': ('פִּים', 'subs', 'pim', 'pim'),
        'E74224': ('קִלְּשֹׁון', 'subs', 'trident', 'kilshovn'),
        'E74225': ('דָּרְבָן', 'subs', 'point', 'darvan'),
        'E74226': ('מִגְרֹון', 'nmpr', 'Migron', 'migrovn'),
        'E74227': ('אֲחִיָּה', 'nmpr', 'Ahijah', 'achia'),
        'E74228': ('אֲחִיטוּב', 'nmpr', 'Ahitub', 'achituv'),
        'E74229': ('בֹּוצֵץ', 'nmpr', 'Bozez', 'bovtseits'),
        'E74230': ('סֶנֶּה', 'nmpr', 'Seneh', 'seneh'),
        'E74231': ('מַעְצֹור', 'subs', 'hindrance', 'matsovr'),
        'E74232': ('חֹר', 'subs', 'hole', 'chor'),
        'E74233': ('מַצָּבָה', 'subs', 'guard', 'matsava'),
        'E74234': ('מַעֲנָה', 'subs', 'ploughing line', "ma'ana"),
        'E74235': ('טעם', 'verb', 'taste', 'tm'),
        'E74236': ('יַעַר', 'subs', 'honeycomb', "ya'ar"),
        'E74237': ('הֵלֶךְ', 'subs', 'going', 'heilech'),
        'E74238': ('יַעֲרָה', 'subs', 'honeycomb', "ya'ara"),
        'E74239': ('עיט', 'verb', 'rebuke', 'yt'),
        'E74240': ('צֹובָא', 'nmpr', 'Zobah', 'tsovva'),
        'E74241': ('מַלְכִּי שׁוּעַ', 'nmpr', 'Malki-Shua', 'malki shua'),
        'E74242': ('מֵרַב', 'nmpr', 'Merab', 'meirav'),
        'E74243': ('מִיכַל', 'nmpr', 'Michal', 'michal'),
        'E74244': ('אֲחִינֹעַם', 'nmpr', 'Ahinoam', 'achinoam'),
        'E74245': ('אֲחִימַעַץ', 'nmpr', 'Ahimaaz', "achima'ats"),
        'E74246': ('אַבְנֵר', 'nmpr', 'Abner', 'avneir'),
        'E74247': ('נֵר', 'nmpr', 'Ner', 'neir'),
        'E74248': ('עֹולֵל', 'subs', 'child', 'ovleil'),
        'E74249': ('טְלָאִים', 'nmpr', 'Telaim', "t'laim"),
        'E74250': ('יֹותֵר', 'subs', 'rest', 'yovteir'),
        'E74251': ('חֵפֶץ', 'subs', 'pleasure', 'cheifets'),
        'E74252': ('קשׁב', 'verb', 'give attention', 'kshv'),
        'E74253': ('נֵצַח', 'subs', 'glory', 'neitsach'),
        'E74254': ('מַעֲדַנֹּת', 'subs', 'reluctance', "ma'adanot"),
        'E74255': ('שׁסף', 'verb', 'cut to pieces?', 'shsf'),
        'E74256': ('יִשַׁי', 'nmpr', 'Jesse', 'yishay'),
        'E74257': ('לַחְמִי', 'adjv', '-lehemite', 'lachmi'),
        'E74258': ('דָּוִד', 'nmpr', 'David', 'david'),
        'E74259': ('בעת', 'verb', 'terrify', 'vt'),
        'E74260': ('נגן', 'verb', 'play harp', 'ngn'),
        'E74261': ('אֶפֶס דַּמִּים', 'nmpr', 'Ephes Dammim', 'efes damim'),
        'E74262': ('בֵּנַיִם', 'subs', '[space between]', 'beinayim'),
        'E74263': ('גָּלְיָת', 'nmpr', 'Goliath', 'galyat'),
        'E74264': ('גֹּבַהּ', 'subs', 'height', 'govah'),
        'E74265': ('כֹּובַע', 'subs', 'helmet', 'kovva'),
        'E74266': ('שִׁרְיֹון', 'subs', 'scaly mail', 'shirovn'),
        'E74267': ('מִצְחָה', 'subs', 'front side', 'mitscha'),
        'E74268': ('מָנֹור', 'subs', "weaver's beam", 'manovr'),
        'E74269': ('לַהֶבֶת', 'subs', 'flame', 'lahevet'),
        'E74270': ('צִנָּה', 'subs', 'shield', 'tsina'),
        'E74271': ('ברה', 'verb', '[uncertain]', 'vrh'),
        'E74272': ('חָרִיץ', 'subs', 'slice', 'charits'),
        'E74273': ('עֲרֻבָּה', 'subs', 'pledge', 'aruba'),
        'E74274': ('מַעְגָּל', 'subs', 'camp circle', 'magal'),
        'E74275': ('דֹּב', 'subs', 'bear', 'dov'),
        'E74276': ('קֹובַע', 'subs', 'helmet', 'kovva'),
        'E74277': ('אלה', 'verb', 'be unable', 'lh'),
        'E74278': ('חַלֻּק', 'adjv', 'smooth', 'chaluk'),
        'E74279': ('יַלְקוּט', 'subs', 'bag', 'yalkut'),
        'E74280': ('קֶלַע', 'subs', 'sling', 'kela'),
        'E74281': ('עֶלֶם', 'subs', 'young man', 'elem'),
        'E74282': ('חֲגֹור', 'subs', 'girdle', 'chagovr'),
        'E74283': ('שָׁלִישׁ', 'subs', 'triangle', 'shalish'),
        'E74284': ('עין', 'verb', 'look at', 'yn'),
        'E74285': ('טול', 'verb', 'cast', 'tvl'),
        'E74286': ('חַי', 'subs', 'family', 'chay'),
        'E74287': ('עַדְרִיאֵל', 'nmpr', 'Adriel', 'adrieil'),
        'E74288': ('מְחֹלָתִי', 'adjv', 'Meholathite', "m'cholati"),
        'E74289': ('רושׁ', 'verb', 'be poor', 'rvsh'),
        'E74290': ('יקר', 'verb', 'be precious', 'ykr'),
        'E74291': ('פטר', 'verb', 'escape', 'ftr'),
        'E74292': ('כָּבִיר', 'subs', 'quilt', 'kavir'),
        'E74293': ('נָיֹות', 'nmpr', 'Naioth', 'nayovt'),
        'E74294': ('לַהֲקָה', 'subs', 'seniors', 'lahaka'),
        'E74295': ('שֶׂכוּ', 'nmpr', 'Secu', 'sechu'),
        'E74296': ('פֶּשַׂע', 'subs', 'step', 'pesa'),
        'E74297': ('אֶזֶל', 'nmpr', '[uncertain]', 'ezel'),
        'E74298': ('מַטָּרָה', 'subs', 'target', 'matara'),
        'E74299': ('עוה', 'verb', 'do wrong', 'vh'),
        'E74300': ('מַרְדּוּת', 'subs', 'rebellion', 'mardut'),
        'E74301': ('בֹּשֶׁת', 'subs', 'shame', 'boshet'),
        'E74302': ('חֵצִי', 'subs', 'arrow', 'cheitsi'),
        'E74303': ('נֹב', 'nmpr', 'Nob', 'nov'),
        'E74304': ('אֲחִימֶלֶךְ', 'nmpr', 'Ahimelech', 'achimelech'),
        'E74305': ('פְּלֹנִי', 'subs', 'a certain', "p'loni"),
        'E74306': ('אַלְמֹנִי', 'subs', 'stranger', 'almoni'),
        'E74307': ('דֹּאֵג', 'nmpr', 'Doeg', 'doeig'),
        'E74308': ('אִין', 'conj', 'if', 'in'),
        'E74309': ('נחץ', 'verb', 'urge', 'nchts'),
        'E74310': ('לוט', 'verb', 'enwrap', 'lvt'),
        'E74311': ('אָכִישׁ', 'nmpr', 'Achish', 'achish'),
        'E74312': ('הלל', 'verb', 'be infatuated', "hl'l"),
        'E74313': ('תוה', 'verb', 'mark', 'tvh'),
        'E74314': ('רִיר', 'subs', 'spittle', 'rir'),
        'E74315': ('חָסֵר', 'adjv', 'lacking', 'chaseir'),
        'E74316': ('מְצוּדָה', 'subs', 'fortification', "m'tsuda"),
        'E74317': ('חֶרֶת', 'nmpr', 'Hereth', 'cheret'),
        'E74318': ('מִשְׁמַעַת', 'subs', 'subject', "mishma'at"),
        'E74319': ('אֶבְיָתָר', 'nmpr', 'Abiathar', 'evyatar'),
        'E74320': ('חֹרְשָׁה', 'nmpr', 'Horesh', 'chorsha'),
        'E74321': ('זִיפִי', 'adjv', 'Ziphite', 'zifi'),
        'E74322': ('חֲכִילָה', 'nmpr', 'Hakilah', 'chachila'),
        'E74323': ('ערם', 'verb', 'be cunning', 'rm'),
        'E74324': ('מַחֲבֹאִים', 'subs', 'hiding-places', 'machavoim'),
        'E74325': ('עטר', 'verb', 'surround', 'tr'),
        'E74326': ('יָעֵל', 'subs', 'goat', 'yaeil'),
        'E74327': ('קַדְמֹנִי', 'adjv', 'eastern', 'kadmoni'),
        'E74328': ('פַּרְעֹשׁ', 'subs', 'flea', 'parosh'),
        'E74329': ('דַּיָּן', 'subs', 'judge', 'dayan'),
        'E74330': ('נָבָל', 'nmpr', 'Nabal', 'naval'),
        'E74331': ('אֲבִיגַיִל', 'nmpr', 'Abigail', 'avigayil'),
        'E74332': ('שֶׂכֶל', 'subs', 'insight', 'sechel'),
        'E74333': ('כָּלִבִּי', 'adjv', 'Calebite', 'kalibi'),
        'E74334': ('טִבְחָה', 'subs', 'slaughtered meat', 'tivcha'),
        'E74335': ('צִמּוּקִים', 'subs', 'cakes', 'tsimukim'),
        'E74336': ('דְּבֵלָה', 'subs', 'fig cake', "d'veila"),
        'E74337': ('שׁתן', 'verb', 'piss', 'shtn'),
        'E74338': ('פּוּקָה', 'subs', 'staggering', 'puka'),
        'E74339': ('לַיִשׁ', 'nmpr', 'Laish', 'layish'),
        'E74340': ('גַּלִּים', 'nmpr', 'Gallim', 'galim'),
        'E74341': ('אֲבִישַׁי', 'nmpr', 'Abishai', 'avishay'),
        'E74342': ('צְרוּיָה', 'nmpr', 'Zeruiah', "ts'ruya"),
        'E74343': ('יֹואָב', 'nmpr', 'Joab', 'yovav'),
        'E74344': ('יָשֵׁן', 'adjv', 'sleeping', 'yashein'),
        'E74345': ('צַפַּחַת', 'subs', 'jar', 'tsapachat'),
        'E74346': ('קיץ', 'verb', 'pass summer', 'kyts'),
        'E74347': ('קֹרֵא', 'subs', 'partridge', 'korei'),
        'E74348': ('יאשׁ', 'verb', 'despair', 'ysh'),
        'E74349': ('מָעֹוךְ', 'nmpr', 'Maoch', 'maovch'),
        'E74350': ('יִזְרְעֵאלִי', 'adjv', 'Jezreelite', 'yizreili'),
        'E74351': ('כַּרְמְלִי', 'adjv', 'Carmelite', 'karmli'),
        'E74352': ('גִּזְרִי', 'adjv', 'Girzite', 'gizri'),
        'E74353': ('אַל', 'inrg', 'where', 'al'),
        'E74354': ('יְרַחְמְאֵלִי', 'adjv', 'Jerahmeelite', "y'rachmeili"),
        'E74355': ('גִּלְבֹּעַ', 'nmpr', 'Gilboa', 'gilboa'),
        'E74356': ('בַּעֲלָה', 'subs', 'mistress', "ba'ala"),
        'E74357': ('עָר', 'subs', '[uncertain]', 'ar'),
        'E74358': ('מַרְבֵּק', 'subs', 'fatted calf', 'marbeik'),
        'E74359': ('בְּשֹׂור', 'nmpr', 'Besor', "b'sovr"),
        'E74360': ('פגר', 'verb', 'be faint', 'fgr'),
        'E74361': ('כְּרֵתִי', 'adjv', 'Cherethite', "k'reiti"),
        'E74362': ('נֶשֶׁף', 'subs', 'breeze', 'neshef'),
        'E74363': ('עדר', 'verb', 'be missing', 'dr'),
        'E74364': ('שִׂפְמֹות', 'nmpr', 'Siphmoth', 'sifmovt'),
        'E74365': ('רָכָל', 'nmpr', 'Racal', 'rachol'),
        'E74366': ('בֹּור עָשָׁן', 'nmpr', 'Bor Ashan', 'bovr ashan'),
        'E74367': ('עֲתָךְ', 'nmpr', 'Athach', 'atach'),
        'E74368': ('עָצָב', 'subs', 'image', 'atsav'),
        'E74369': ('שָׁבָץ', 'subs', 'cramp', 'shavats'),
        'E74370': ('קין', 'verb', 'chant elegy', 'kyn'),
        'E74371': ('קִינָה', 'subs', 'elegy', 'kina'),
        'E74372': ('צְבִי', 'subs', 'beauty', "ts'vi"),
        'E74373': ('עלז', 'verb', 'rejoice', 'lz'),
        'E74374': ('נָעִים', 'adjv', 'pleasant', 'naim'),
        'E74375': ('עֵדֶן', 'subs', 'delight', 'eiden'),
        'E74376': ('אִישׁ בֹּשֶׁת', 'nmpr', 'Ish-Bosheth', 'ish boshet'),
        'E74377': ('אֲשׁוּרִי', 'adjv', 'Ashurite', 'ashuri'),
        'E74378': ('בְּרֵכָה', 'subs', 'pool', "b'reicho"),
        'E74379': ('חֶלְקַת הַצּוּרִים',
                   'nmpr',
                   'Helkath Hazzurim',
                   'chelkat hatsurim'),
        'E74380': ('עֲשָׂהאֵל', 'nmpr', 'Asahel', 'asaheil'),
        'E74381': ('קַל', 'adjv', 'light', 'kal'),
        'E74382': ('חֹמֶשׁ', 'subs', 'belly', 'chomesh'),
        'E74383': ('אַמָּה', 'nmpr', 'Ammah', 'ama'),
        'E74384': ('גִּיחַ', 'nmpr', 'Giah', 'giach'),
        'E74385': ('בִּתְרֹון', 'subs', 'gully', 'bitrovn'),
        'E74386': ('אָרֹךְ', 'adjv', 'long', 'aroch'),
        'E74387': ('אַמְנֹון', 'nmpr', 'Amnon', 'amnovn'),
        'E74388': ('כִּלְאָב', 'nmpr', 'Kileab', 'kilav'),
        'E74389': ('אַבְשָׁלֹום', 'nmpr', 'Absalom', 'avshalovm'),
        'E74390': ('אֲדֹנִיָּה', 'nmpr', 'Adonijah', 'adonia'),
        'E74391': ('חַגִּית', 'nmpr', 'Haggith', 'chagit'),
        'E74392': ('שְׁפַטְיָה', 'nmpr', 'Shephatiah', "sh'fatya"),
        'E74393': ('אֲבִיטָל', 'nmpr', 'Abital', 'avital'),
        'E74394': ('יִתְרְעָם', 'nmpr', 'Ithream', 'yitram'),
        'E74395': ('עֶגְלָה', 'nmpr', 'Eglah', 'egla'),
        'E74396': ('רִצְפָּה', 'nmpr', 'Rizpah', 'ritspa'),
        'E74397': ('בַּחֻרִים', 'nmpr', 'Bahurim', 'bachurim'),
        'E74398': ('מֹובָא', 'subs', 'coming', 'movva'),
        'E74399': ('סִרָה', 'nmpr', 'Sirah', 'sira'),
        'E74400': ('שְׁלִי', 'subs', 'quiet', "sh'li"),
        'E74401': ('פֶּלֶךְ', 'subs', 'whirl of spindle', 'pelech'),
        'E74402': ('עַוְלָה', 'subs', 'wickedness', 'avla'),
        'E74403': ('ברה', 'verb', 'eat', 'vrh'),
        'E74404': ('בַּעֲנָה', 'nmpr', 'Baanah', "ba'ana"),
        'E74405': ('רֵכָב', 'nmpr', 'Recab', 'reichov'),
        'E74406': ('רִמֹּון', 'nmpr', 'Rimmon', 'rimovn'),
        'E74407': ('בְּאֵרֹתִי', 'adjv', 'Beerothite', "b'eiroti"),
        'E74408': ('גִּתַּיִם', 'nmpr', 'Gittaim', 'gitayim'),
        'E74409': ('נָכֵה', 'adjv', 'smitten', 'nachei'),
        'E74410': ('מְפִיבֹשֶׁת', 'nmpr', 'Mephibosheth', "m'fivoshet"),
        'E74411': ('בְּשֹׂרָה', 'subs', 'report', "b'sora"),
        'E74412': ('צִיֹּון', 'nmpr', 'Zion', 'tsiovn'),
        'E74413': ('צִנֹּור', 'subs', 'water-tunnel', 'tsinovr'),
        'E74414': ('חִירָם', 'nmpr', 'Hiram', 'chiram'),
        'E74415': ('שֹׁובָב', 'nmpr', 'Shobab', 'shovvav'),
        'E74416': ('נָתָן', 'nmpr', 'Nathan', 'natan'),
        'E74417': ('שְׁלֹמֹה', 'nmpr', 'Solomon', "sh'lomo"),
        'E74418': ('יִבְחָר', 'nmpr', 'Ibhar', 'yivchar'),
        'E74419': ('אֱלִישׁוּעַ', 'nmpr', 'Elishua', 'elishua'),
        'E74420': ('אֶלְיָדָע', 'nmpr', 'Eliada', 'elyada'),
        'E74421': ('אֱלִיפֶלֶט', 'nmpr', 'Eliphelet', 'elifelet'),
        'E74422': ('בַּעַל פְּרָצִים', 'nmpr', 'Baal Perazim', "ba'al p'ratsim"),
        'E74423': ('בָּכָא', 'subs', 'baka-shrub', 'bacho'),
        'E74424': ('צְעָדָה', 'subs', 'marching', "ts'ada"),
        'E74425': ('חרץ', 'verb', 'be eager', 'chrts'),
        'E74426': ('בַּעֲלֵי יְהוּדָה', 'nmpr', 'Baalah of Judah', "ba'alei y'huda"),
        'E74427': ('עֻזָּה', 'nmpr', 'Uzzah', 'uza'),
        'E74428': ('אַחְיֹו', 'nmpr', 'Ahio', 'achyov'),
        'E74429': ('בְּרֹושׁ', 'subs', 'juniper', "b'rovsh"),
        'E74430': ('מְנַעַנְעִים', 'subs', 'sistrum', "m'na'anim"),
        'E74431': ('צֶלְצֶלִים', 'subs', 'cymbals', 'tseltselim'),
        'E74432': ('נָכֹון', 'nmpr', 'Nacon', 'nachovn'),
        'E74433': ('שַׁל', 'subs', '[uncertain]', 'shal'),
        'E74434': ('פֶּרֶץ עֻזָּה', 'nmpr', 'Perez Uzzah', 'perets uza'),
        'E74435': ('עֹבֵד אֱדֹום', 'nmpr', 'Obed-edom', 'oveid edovm'),
        'E74436': ('צַעַד', 'subs', 'marching', "tsa'ad"),
        'E74437': ('מְרִיא', 'subs', 'fatling', "m'ri"),
        'E74438': ('כרר', 'verb', 'dance', "chr'r"),
        'E74439': ('אֶשְׁפָּר', 'subs', '[type of cake]', 'eshpar'),
        'E74440': ('אֲשִׁישָׁה', 'subs', 'raisin cake', 'ashisha'),
        'E74441': ('חִזָּיֹון', 'subs', 'vision', 'chizayovn'),
        'E74442': ('גְּדוּלָּה', 'subs', 'greatness', "g'dula"),
        'E74443': ('תְּפִלָּה', 'subs', 'prayer', "t'fila"),
        'E74444': ('מֶתֶג', 'subs', 'bridle', 'meteg'),
        'E74445': ('אַמָּה', 'subs', '[uncertain]', 'ama'),
        'E74446': ('הֲדַדְעֶזֶר', 'nmpr', 'Hadadezer', 'hadadezer'),
        'E74447': ('רְחֹב', 'nmpr', 'Rehob', "r'chov"),
        'E74448': ('שֶׁלֶט', 'subs', 'shield', 'shelet'),
        'E74449': ('בֶּטַח', 'nmpr', 'Tebah', 'betach'),
        'E74450': ('בֵּרֹתַי', 'nmpr', 'Berothai', 'beirotay'),
        'E74451': ('תֹּעוּ', 'nmpr', 'Tou', 'tou'),
        'E74452': ('יֹורָם', 'nmpr', 'Joram', 'yovram'),
        'E74453': ('גֵּיא מֶלַח', 'nmpr', 'the Valley of Salt', 'gei melach'),
        'E74454': ('יְהֹושָׁפָט', 'nmpr', 'Jehoshaphat', "y'hovshafat"),
        'E74455': ('אֲחִילוּד', 'nmpr', 'Ahilud', 'achilud'),
        'E74456': ('צָדֹוק', 'nmpr', 'Zadok', 'tsadovk'),
        'E74457': ('שְׂרָיָה', 'nmpr', 'Seraiah', "s'raya"),
        'E74458': ('בְּנָיָהוּ', 'nmpr', 'Benaiah', "b'nayahu"),
        'E74459': ('יְהֹויָדָע', 'nmpr', 'Jehoiada', "y'hovyada"),
        'E74460': ('פְּלֵתִי', 'adjv', 'Pelethite', "p'leiti"),
        'E74461': ('צִיבָא', 'nmpr', 'Ziba', 'tsiva'),
        'E74462': ('לֹא דְבָר', 'nmpr', 'Lo Debar', "lo d'var"),
        'E74463': ('מִיכָא', 'nmpr', 'Mica', 'micho'),
        'E74464': ('חָנוּן', 'nmpr', 'Hanun', 'chanun'),
        'E74465': ('מָדוּ', 'subs', 'garment', 'madu'),
        'E74466': ('שֵׁת', 'subs', 'posterior', 'sheit'),
        'E74467': ('מַעֲכָה', 'nmpr', 'Maacah', "ma'acho"),
        'E74468': ('אַבְשַׁי', 'nmpr', 'Abishai', 'avshay'),
        'E74469': ('חֵילָם', 'nmpr', 'Helam', 'cheilam'),
        'E74470': ('שֹׁובַךְ', 'nmpr', 'Shobach', 'shovvach'),
        'E74471': ('בַּת שֶׁבַע', 'nmpr', 'Bathsheba', 'bat sheva'),
        'E74472': ('אֱלִיעָם', 'nmpr', 'Eliam', 'eliam'),
        'E74473': ('אוּרִיָּה', 'nmpr', 'Uriah', 'uria'),
        'E74474': ('יְרֻבֶּשֶׁת', 'nmpr', 'Jerub-Besheth', "y'rubeshet"),
        'E74475': ('אנשׁ', 'verb', 'be sickly', 'nsh'),
        'E74476': ('צֹום', 'subs', 'fasting', 'tsovm'),
        'E74477': ('לחשׁ', 'verb', 'whisper', 'lchsh'),
        'E74478': ('יְדִידְיָהּ', 'nmpr', 'Jedidiah', "y'didah"),
        'E74479': ('עֲטֶרֶת', 'subs', 'wreath', 'ateret'),
        'E74480': ('מְגֵרָה', 'subs', 'stone-saw', "m'geira"),
        'E74481': ('חָרִיץ', 'subs', 'axe', 'charits'),
        'E74482': ('מַגְזֵרָה', 'subs', 'axe', 'magzeira'),
        'E74483': ('מַלְבֵּן', 'subs', 'brick', 'malbein'),
        'E74484': ('יֹונָדָב', 'nmpr', 'Jonadab', 'yovnadav'),
        'E74485': ('שִׁמְעָה', 'nmpr', 'Shimeah', 'shima'),
        'E74486': ('יְהֹונָדָב', 'nmpr', 'Jehonadab', "y'hovnadav"),
        'E74487': ('בִּרְיָה', 'subs', 'bread of consolation', 'bira'),
        'E74488': ('לבב', 'verb', 'bake', "lv'v"),
        'E74489': ('לְבִבָה', 'subs', 'cooky', "l'viva"),
        'E74490': ('מַשְׂרֵת', 'subs', 'pan', 'masreit'),
        'E74491': ('בַּעַל חָצֹור', 'nmpr', 'Baal Hazor', "ba'al chatsovr"),
        'E74492': ('פֶּרֶד', 'subs', 'mule', 'pered'),
        'E74493': ('תְּקֹועַ', 'nmpr', 'Tekoa', "t'kova"),
        'E74494': ('תְּקֹועִי', 'adjv', 'Tekoite', "t'kovi"),
        'E74495': ('גַּחֶלֶת', 'subs', 'glow', 'gachelet'),
        'E74496': ('הַרְבָּה', 'subs', '[uncertain]', 'harba'),
        'E74497': ('נגר', 'verb', 'run', 'ngr'),
        'E74498': ('דחח', 'verb', 'thrust', "dch'ch"),
        'E74499': ('אִשׁ', 'subs', 'existence', 'ish'),
        'E74500': ('נָכֹחַ', 'adjv', 'straight', 'nachoach'),
        'E74501': ('אֲחִיתֹפֶל', 'nmpr', 'Ahithophel', 'achitofel'),
        'E74502': ('גִּילֹנִי', 'adjv', 'Gilonite', 'giloni'),
        'E74503': ('קֶשֶׁר', 'subs', 'conspiracy', 'kesher'),
        'E74504': ('אַמִּיץ', 'adjv', 'strong', 'amits'),
        'E74505': ('מֶרְחָק', 'subs', 'distance', 'merchak'),
        'E74506': ('אִתַּי', 'nmpr', 'Ittai', 'itay'),
        'E74507': ('קִדְרֹון', 'nmpr', 'Kidron', 'kidrovn'),
        'E74508': ('חפה', 'verb', 'cover', 'chfh'),
        'E74509': ('יָחֵף', 'adjv', 'barefoot', 'yacheif'),
        'E74510': ('חוּשַׁי', 'nmpr', 'Hushai', 'chushay'),
        'E74511': ('רֵעֶה', 'subs', 'fellow', 'reieh'),
        'E74512': ('עפר', 'verb', 'throw', 'fr'),
        'E74513': ('שַׁכּוּל', 'adjv', 'bereaved of children', 'shakul'),
        'E74514': ('פַּחַת', 'subs', 'pit', 'pachat'),
        'E74515': ('קְרָב', 'subs', 'fight', "k'rav"),
        'E74516': ('סחב', 'verb', 'drag', "s'chv"),
        'E74517': ('צְרֹור', 'subs', 'pebble', "ts'rovr"),
        'E74518': ('בלע', 'verb', 'communicate', 'vl'),
        'E74519': ('רִיפֹות', 'subs', '[uncertain]', 'rifovt'),
        'E74520': ('מִיכָל', 'subs', '[uncertain]', 'michol'),
        'E74521': ('חנק', 'verb', 'strangle', 'chnk'),
        'E74522': ('עֲמָשָׂא', 'nmpr', 'Amasa', 'amasa'),
        'E74523': ('יִתְרָא', 'nmpr', 'Jether', 'yitra'),
        'E74524': ('שֹׁבִי', 'nmpr', 'Shobi', 'shovi'),
        'E74525': ('בַּרְזִלַּי', 'nmpr', 'Barzillai', 'barzilay'),
        'E74526': ('רֹגְלִים', 'nmpr', 'Rogelim', 'roglim'),
        'E74527': ('יֹוצֵר', 'subs', 'potter', 'yovtseir'),
        'E74528': ('פֹּול', 'subs', 'beans', 'povl'),
        'E74529': ('שְׁפֹות', 'subs', 'yoghurt', "sh'fovt"),
        'E74530': ('שֹׂובֶךְ', 'subs', 'network of boughs', 'sovvech'),
        'E74531': ('שֹׁועֵר', 'subs', 'porter', 'shoveir'),
        'E74532': ('מְרוּצָה', 'subs', 'running', "m'rutsa"),
        'E74533': ('עֲבָרָה', 'subs', 'crossing', 'avara'),
        'E74534': ('שִׁיבָה', 'subs', '[uncertain]', 'shiva'),
        'E74535': ('גְּמוּלָה', 'subs', 'recompense', "g'mula"),
        'E74536': ('כִּמְהָם', 'nmpr', 'Kimham', 'kimham'),
        'E74537': ('כִּמְהָן', 'nmpr', 'Kimham', 'kimhan'),
        'E74538': ('שֶׁבַע', 'nmpr', 'Sheba', 'sheva'),
        'E74539': ('בִּכְרִי', 'nmpr', 'Bicri', 'bichri'),
        'E74540': ('חַיּוּת', 'subs', 'lifetime', 'chayut'),
        'E74541': ('יגה', 'verb', 'thrust away', 'ygh'),
        'E74542': ('אָבֵל', 'nmpr', 'Abel', 'aveil'),
        'E74543': ('בֵּרִים', 'subs', '[uncertain]', 'beirim'),
        'E74544': ('סֹלֲלָה', 'subs', 'rampart', 'solala'),
        'E74545': ('חֵיל', 'subs', 'rampart', 'cheil'),
        'E74546': ('אֱמוּן', 'adjv', 'faithful', 'emun'),
        'E74547': ('בְּנָיָה', 'nmpr', 'Benaiah', "b'naya"),
        'E74548': ('אֲדֹרָם', 'nmpr', 'Adoniram', 'adoram'),
        'E74549': ('שְׁוָא', 'nmpr', 'Sheva', "sh'va"),
        'E74550': ('עִירָא', 'nmpr', 'Ira', 'ira'),
        'E74551': ('יָאִירִי', 'adjv', 'Jairite', 'yairi'),
        'E74552': ('גִּבְעֹונִי', 'adjv', 'Gibeonite', 'givovni'),
        'E74553': ('בָּחִיר', 'adjv', 'chosen', 'bachir'),
        'E74554': ('אַרְמֹנִי', 'nmpr', 'Armoni', 'armoni'),
        'E74555': ('יִשְׁבִּי', 'nmpr', 'Ishbi-Benob', 'yishbi'),
        'E74556': ('רָפָה', 'nmpr', 'Rapha', 'rafa'),
        'E74557': ('קַיִן', 'subs', 'spear', 'kayin'),
        'E74558': ('גֹּוב', 'nmpr', 'Gob', 'govv'),
        'E74559': ('סִבְּכַי', 'nmpr', 'Sibbecai', 'sibchay'),
        'E74560': ('חֻשָׁתִי', 'adjv', 'Hushathite', 'chushati'),
        'E74561': ('סַף', 'nmpr', 'Saph', 'saf'),
        'E74562': ('אֶלְחָנָן', 'nmpr', 'Elhanan', 'elchanan'),
        'E74563': ('יַעֲרֵי', 'nmpr', 'Jaare-oregim', "ya'arei"),
        'E74564': ('מָדֹון', 'subs', 'contention', 'madovn'),
        'E74565': ('פלט', 'verb', 'escape', 'flt'),
        'E74566': ('יֵשַׁע', 'subs', 'help', 'yeisha'),
        'E74567': ('מִשְׂגָּב', 'subs', 'secure height', 'misgav'),
        'E74568': ('מָנֹוס', 'subs', 'refuge', 'manovs'),
        'E74569': ('אפף', 'verb', 'encompass', "f'f"),
        'E74570': ('מִשְׁבָּר', 'subs', 'breaker', 'mishbar'),
        'E74571': ('געשׁ', 'verb', 'shake', 'gsh'),
        'E74572': ('מֹוסָדָה', 'subs', 'foundation', 'movsada'),
        'E74573': ('גֶּחָל', 'subs', 'charcoals', 'gechal'),
        'E74574': ('חַשְׁרָה', 'subs', '[uncertain]', 'chashra'),
        'E74575': ('נֹגַהּ', 'subs', 'brightness', 'nogah'),
        'E74576': ('אָפִיק', 'subs', 'stream', 'afik'),
        'E74577': ('גְּעָרָה', 'subs', 'rebuke', "g'ara"),
        'E74578': ('מִשְׁעָן', 'subs', 'support', 'mishan'),
        'E74579': ('מֶרְחָב', 'subs', 'wide place', 'merchav'),
        'E74580': ('בֹּר', 'subs', 'cleanness', 'bor'),
        'E74581': ('חסד', 'verb', 'act piously', 'chsd'),
        'E74582': ('ברר', 'verb', 'purge', "vr'r"),
        'E74583': ('תבר', 'verb', '[uncertain]', 'tvr'),
        'E74584': ('תפל', 'verb', 'be silly', 'tfl'),
        'E74585': ('נָגַהּ', 'verb', 'shine', 'nagah'),
        'E74586': ('דלג', 'verb', 'ascend', 'dlg'),
        'E74587': ('שׁוה', 'verb', 'be like', 'shvh'),
        'E74588': ('נחת', 'verb', 'descend', 'ncht'),
        'E74589': ('מעד', 'verb', 'shake', 'md'),
        'E74590': ('קַרְסֹל', 'subs', 'ankle', 'karsol'),
        'E74591': ('צמת', 'verb', 'be silent', 'tsmt'),
        'E74592': ('טִיט', 'subs', 'clay', 'tit'),
        'E74593': ('מִגְדֹּול', 'subs', '[uncertain]', 'migdovl'),
        'E74594': ('זָמִיר', 'subs', 'song', 'zamir'),
        'E74595': ('מִלָּה', 'subs', 'word', 'mila'),
        'E74596': ('קֹוץ', 'subs', 'snuff', 'kovts'),
        'E74597': ('תַּחְכְּמֹנִי', 'adjv', 'Tahkemonite', 'tachkmoni'),
        'E74598': ('עֲדִינֹו', 'nmpr', 'Adino', 'adinov'),
        'E74599': ('עֶצְנִי', 'subs', '[uncertain]', 'etsni'),
        'E74600': ('אֲחֹוחִי', 'adjv', 'Ahohite', 'achovchi'),
        'E74601': ('שַׁמָּא', 'nmpr', 'Shamma', 'shama'),
        'E74602': ('אָגֵא', 'nmpr', 'Agee', 'agei'),
        'E74603': ('הֲרָרִי', 'adjv', 'Hararite', 'harari'),
        'E74604': ('אֲרִיאֵל', 'nmpr', 'Ariel', 'arieil'),
        'E74605': ('חֲרֹדִי', 'adjv', 'Harodite', 'charodi'),
        'E74606': ('אֱלִיקָא', 'nmpr', 'Elika', 'elika'),
        'E74607': ('חֶלֶץ', 'nmpr', 'Helez', 'chelets'),
        'E74608': ('פַּלְטִי', 'adjv', 'Paltite', 'palti'),
        'E74609': ('עִקֵּשׁ', 'nmpr', 'Ikkesh', 'ikeish'),
        'E74610': ('עַנְּתֹתִי', 'adjv', 'Anathothite', 'antoti'),
        'E74611': ('מְבֻנַּי', 'nmpr', 'Mebunnai', "m'vunay"),
        'E74612': ('צַלְמֹון', 'nmpr', 'Zalmon', 'tsalmovn'),
        'E74613': ('מַהְרַי', 'nmpr', 'Maharai', 'mahray'),
        'E74614': ('נְטֹופָתִי', 'adjv', 'Netophathite', "n'tovfati"),
        'E74615': ('חֵלֶב', 'nmpr', 'Heled', 'cheilev'),
        'E74616': ('רִיבַי', 'nmpr', 'Ribai', 'rivay'),
        'E74617': ('הִדַּי', 'nmpr', 'Hiddai', 'hiday'),
        'E74618': ('אֲבִי עַלְבֹון', 'nmpr', 'Abi-albon', 'avi alvovn'),
        'E74619': ('עַרְבָתִי', 'adjv', 'Arbathite', 'arvati'),
        'E74620': ('עַזְמָוֶת', 'nmpr', 'Azmaveth', 'azmavet'),
        'E74621': ('בַּרְחֻמִי', 'adjv', 'Barhumite', 'barchumi'),
        'E74622': ('אֶלְיַחְבָּא', 'nmpr', 'Eliahba', 'elyachba'),
        'E74623': ('שַׁעַלְבֹנִי', 'adjv', 'Shaalbonite', "sha'alvoni"),
        'E74624': ('יָשֵׁן', 'nmpr', 'Jashen', 'yashein'),
        'E74625': ('אֲחִיאָם', 'nmpr', 'Ahiam', 'achiam'),
        'E74626': ('שָׁרָר', 'nmpr', 'Sharar', 'sharar'),
        'E74627': ('אֲרָרִי', 'adjv', '[uncertain]', 'arari'),
        'E74628': ('אֲחַסְבַּי', 'nmpr', 'Ahasbai', 'achasbay'),
        'E74629': ('חֶצְרַי', 'nmpr', 'Hezro', 'chetsray'),
        'E74630': ('פַּעֲרַי', 'nmpr', 'Paarai', "pa'aray"),
        'E74631': ('אַרְבִּי', 'adjv', 'Arbite', 'arbi'),
        'E74632': ('בָּנִי', 'nmpr', 'Bani', 'bani'),
        'E74633': ('צֶלֶק', 'nmpr', 'Zelek', 'tselek'),
        'E74634': ('נַחְרַי', 'nmpr', 'Naharai', 'nachray'),
        'E74635': ('יִתְרִי', 'adjv', 'Ithrite', 'yitri'),
        'E74636': ('גָּרֵב', 'nmpr', 'Gareb', 'gareiv'),
        'E74637': ('תַּחְתִּים חָדְשִׁי', 'nmpr', 'Tahtim Hodshi', 'tachtim chadshi'),
        'E74638': ('מִפְקָד', 'subs', 'order', 'mifkad'),
        'E74639': ('חֹזֶה', 'subs', 'seer', 'chozeh'),
        'E74640': ('נטל', 'verb', 'lay upon', 'ntl'),
        'E74641': ('אֲרַוְנָה', 'nmpr', 'Araunah', 'aravna'),
        'E74642': ('מֹורַג', 'subs', 'threshing-sledge', 'movrag'),
        'E74643': ('אֲבִישַׁג', 'nmpr', 'Abishag', 'avishag'),
        'E74644': ('שׁוּנַמִּי', 'adjv', 'Shunammite', 'shunami'),
        'E74645': ('רֵעִי', 'nmpr', 'Rei', 'reii'),
        'E74646': ('אֲדֹנִיָּהוּ', 'nmpr', 'Adonijah', 'adoniahu'),
        'E74647': ('זֹחֶלֶת', 'nmpr', 'Zoheleth', 'zochelet'),
        'E74648': ('פִּרְדָּה', 'subs', 'she-mule', 'pirda'),
        'E74649': ('חלל', 'verb', 'play flute', "chl'l"),
        'E74650': ('המה', 'verb', 'make noise', 'hmh'),
        'E74651': ('מרץ', 'verb', 'hurt', 'mrts'),
        'E74652': ('יִשְׁרָה', 'subs', 'uprightness', 'yishra'),
        'E74653': ('גזר', 'verb', 'cut', 'gzr'),
        'E74654': ('עֲזַרְיָהוּ', 'nmpr', 'Azariahu', 'azaryahu'),
        'E74655': ('אֱלִיחֹרֶף', 'nmpr', 'Elihoreph', 'elichoref'),
        'E74656': ('שִׁישָׁא', 'nmpr', 'Shisha', 'shisha'),
        'E74657': ('זָבוּד', 'nmpr', 'Zabud', 'zavud'),
        'E74658': ('אֲחִישָׁר', 'nmpr', 'Ahishar', 'achishar'),
        'E74659': ('אֲדֹנִירָם', 'nmpr', 'Adoniram', 'adoniram'),
        'E74660': ('עַבְדָּא', 'nmpr', 'Abda', 'avda'),
        'E74661': ('דֶּקֶר', 'nmpr', 'Deker', 'deker'),
        'E74662': ('מָקַץ', 'nmpr', 'Makaz', 'makats'),
        'E74663': ('בֵּית חָנָן', 'nmpr', 'Bethhanan', 'beit chanan'),
        'E74664': ('חֶסֶד', 'nmpr', 'Ben-Hesed', 'chesed'),
        'E74665': ('אֲרֻבֹּות', 'nmpr', 'Arubboth', 'arubovt'),
        'E74666': ('טָפַת', 'nmpr', 'Taphath', 'tafat'),
        'E74667': ('בַּעֲנָא', 'nmpr', 'Baana', "ba'ana"),
        'E74668': ('יָקְמְעָם', 'nmpr', 'Jokmeam', 'yakmam'),
        'E74669': ('גֶּבֶר', 'nmpr', 'Geber', 'gever'),
        'E74670': ('אֲחִינָדָב', 'nmpr', 'Ahinadab', 'achinadav'),
        'E74671': ('עִדֹּוא', 'nmpr', 'Iddo', 'idov'),
        'E74672': ('פָּרוּחַ', 'nmpr', 'Paruah', 'paruach'),
        'E74673': ('אֵלָא', 'nmpr', 'Ela', 'eila'),
        'E74674': ('אֻרִי', 'nmpr', 'Uri', 'uri'),
        'E74675': ('כֹּר', 'subs', 'kor', 'kor'),
        'E74676': ('רְעִי', 'subs', 'pasture', "r'i"),
        'E74677': ('בַּרְבֻּר', 'subs', 'cuckoo', 'barbur'),
        'E74678': ('אבס', 'verb', 'fatten', 'vs'),
        'E74679': ('תִּפְסַח', 'nmpr', 'Tiphsah', 'tifsach'),
        'E74680': ('אֻרְוָה', 'subs', 'stall', 'urva'),
        'E74681': ('רֶכֶשׁ', 'subs', 'span', 'rechesh'),
        'E74682': ('אֵיתָן', 'nmpr', 'Ethan', 'eitan'),
        'E74683': ('אֶזְרָחִי', 'adjv', 'Ezrahite', 'ezrachi'),
        'E74684': ('הֵימָן', 'nmpr', 'Heman', 'heiman'),
        'E74685': ('כַּלְכֹּל', 'nmpr', 'Calcol', 'kalkol'),
        'E74686': ('דַּרְדַּע', 'nmpr', 'Darda', 'darda'),
        'E74687': ('מָחֹול', 'nmpr', 'Mahol', 'machovl'),
        'E74688': ('פֶּגַע', 'subs', 'occurrence', 'pega'),
        'E74689': ('דֹּבְרֹות', 'subs', 'raft', 'dovrovt'),
        'E74690': ('מַכֹּלֶת', 'subs', 'food', 'makolet'),
        'E74691': ('סַבָּל', 'subs', 'bearer', 'sabal'),
        'E74692': ('זִו', 'nmpr', 'Siw', 'ziv'),
        'E74693': ('אֵילָם', 'subs', 'porch', 'eilam'),
        'E74694': ('שְׁקֻפִים', 'subs', 'windows', "sh'kufim"),
        'E74695': ('אטם', 'verb', 'plug', 'tm'),
        'E74696': ('יָצִיעַ', 'subs', 'outhouse', 'yatsia'),
        'E74697': ('דְּבִיר', 'subs', 'backroom', "d'vir"),
        'E74698': ('מִגְרָעֹות', 'subs', 'recesses', 'migraovt'),
        'E74699': ('אחז', 'verb', 'cover', 'chz'),
        'E74700': ('מַסָּע', 'subs', 'quarry-stone', 'masa'),
        'E74701': ('לוּל', 'subs', '[uncertain]', 'lul'),
        'E74702': ('גֵּב', 'subs', 'pit', 'geiv'),
        'E74703': ('שְׂדֵרָה', 'subs', '[uncertain]', "s'deira"),
        'E74704': ('סִפֻּן', 'subs', 'ceiling', 'sipun'),
        'E74705': ('מִקְלַעַת', 'subs', 'carving', "mikla'at"),
        'E74706': ('פְּקָעִים', 'subs', 'gourd-ornament', "p'kaim"),
        'E74707': ('סָגוּר', 'subs', 'hammered gold', 'sagur'),
        'E74708': ('רַתֹּוק', 'subs', '[uncertain]', 'ratovk'),
        'E74709': ('קֶצֶב', 'subs', 'cut', 'ketsev'),
        'E74710': ('פְּנִימִי', 'adjv', 'inner', "p'nimi"),
        'E74711': ('מֵסַב', 'subs', 'surrounding', 'meisav'),
        'E74712': ('קלע', 'verb', 'carve', 'kl'),
        'E74713': ('תִּמֹרָה', 'subs', 'palm-ornament', 'timora'),
        'E74714': ('חִיצֹון', 'adjv', 'external', 'chitsovn'),
        'E74715': ('אַיִל', 'subs', 'pillar', 'ayil'),
        'E74716': ('גָּלִיל', 'subs', 'pivot', 'galil'),
        'E74717': ('חקה', 'verb', 'carve', 'chkh'),
        'E74718': ('כְּרוּתָה', 'subs', 'beam', "k'ruta"),
        'E74719': ('בּוּל', 'nmpr', 'Bul', 'bul'),
        'E74720': ('מֶחֱזָה', 'subs', 'opening', 'mecheza'),
        'E74721': ('שָׁקֶף', 'subs', '[uncertain]', 'shakef'),
        'E74722': ('עָב', 'subs', '[uncertain]', 'av'),
        'E74723': ('מַסַּד', 'subs', 'foundation', 'masad'),
        'E74724': ('טַפְחָה', 'subs', 'span', 'tafcha'),
        'E74725': ('צֹרִי', 'adjv', 'Tyrian', 'tsori'),
        'E74726': ('כֹּתֶרֶת', 'subs', 'capital', 'koteret'),
        'E74727': ('שְׂבָכָה', 'subs', 'net', "s'vacho"),
        'E74728': ('שׁוּשַׁן', 'subs', 'lily', 'shushan'),
        'E74729': ('בֹּעַז', 'nmpr', 'Boaz', 'boaz'),
        'E74730': ('עָגֹל', 'adjv', 'round', 'agol'),
        'E74731': ('קָו', 'subs', 'line', 'kav'),
        'E74732': ('יְצֻקָה', 'subs', 'casting of metal', "y'tsuka"),
        'E74733': ('עֲבִי', 'subs', 'thickness', 'avi'),
        'E74734': ('טֶפַח', 'subs', 'span', 'tefach'),
        'E74735': ('בַּת', 'subs', '[measure]', 'bat'),
        'E74736': ('מְכֹונָה', 'subs', 'place', "m'chovna"),
        'E74737': ('שָׁלָב', 'subs', 'cross-bar', 'shalav'),
        'E74738': ('לֹיָה', 'subs', 'wreath', 'loya'),
        'E74739': ('סֶרֶן', 'subs', 'axletree', 'seren'),
        'E74740': ('חִשֻּׁק', 'subs', 'spoke', 'chishuk'),
        'E74741': ('חִשֻּׁר', 'subs', 'nave', 'chishur'),
        'E74742': ('מַעַר', 'subs', 'nakedness', "ma'ar"),
        'E74743': ('מוּצָק', 'subs', 'casting', 'mutsak'),
        'E74744': ('מַעֲבֵה', 'subs', 'foundry', "ma'avei"),
        'E74745': ('מְזַמֶּרֶת', 'subs', 'snuffer', "m'zameret"),
        'E74746': ('פֹּת', 'subs', 'forehead', 'pot'),
        'E74747': ('זְבֻל', 'subs', 'dominion', "z'vul"),
        'E74748': ('רִנָּה', 'subs', 'cry of joy', 'rina'),
        'E74749': ('חָסִיל', 'subs', 'cockroach', 'chasil'),
        'E74750': ('פשׁע', 'verb', 'rebel', 'fsh'),
        'E74751': ('חֵשֶׁק', 'subs', 'desire', 'cheishek'),
        'E74752': ('שׁרק', 'verb', 'whistle', 'shrk'),
        'E74753': ('תַּדְמֹר', 'nmpr', 'Tadmor', 'tadmor'),
        'E74754': ('אֳנִי', 'subs', 'ship', 'ani'),
        'E74755': ('מַלְכָּה', 'subs', 'queen', 'malka'),
        'E74756': ('מַעֲמָד', 'subs', 'standing', "ma'amad"),
        'E74757': ('מַלְבּוּשׁ', 'subs', 'garment', 'malbush'),
        'E74758': ('אַלְמֻגִּים', 'subs', 'almug wood', 'almugim'),
        'E74759': ('מִסְעָד', 'subs', '[uncertain]', 'misad'),
        'E74760': ('מִסְחַר', 'subs', '[uncertain]', "mis'char"),
        'E74761': ('רכל', 'verb', 'trade', 'rchl'),
        'E74762': ('עֶרֶב', 'nmpr', 'Arabia', 'erev'),
        'E74763': ('פֶּחָה', 'subs', 'governor', 'pecha'),
        'E74764': ('שׁחט', 'verb', 'alloy', 'shcht'),
        'E74765': ('מָנֶה', 'subs', 'mineh', 'maneh'),
        'E74766': ('פזז', 'verb', 'set with topaz', "fz'z"),
        'E74767': ('שֶׁנְהַבִּים', 'subs', 'ivory', 'shenhabim'),
        'E74768': ('קֹוף', 'subs', 'ape', 'kovf'),
        'E74769': ('תֻּכִּיִּים', 'subs', 'poultry', "tuki'im"),
        'E74770': ('נֵשֶׁק', 'subs', 'equipment', 'neishek'),
        'E74771': ('שִׁקְמָה', 'subs', 'sycomore', 'shikma'),
        'E74772': ('עַשְׁתֹּרֶת', 'nmpr', 'Astarte', 'ashtoret'),
        'E74773': ('מִלְכֹּם', 'nmpr', 'Milcom', 'milkom'),
        'E74774': ('אֲדַד', 'nmpr', 'Hadad', 'adad'),
        'E74775': ('תַּחְפְּנֵיס', 'nmpr', 'Tahpenes', 'tachpneis'),
        'E74776': ('גְּבִירָה', 'subs', 'mistress', "g'vira"),
        'E74777': ('גְּנֻבַת', 'nmpr', 'Genubath', "g'nuvat"),
        'E74778': ('רְזֹון', 'nmpr', 'Rezon', "r'zovn"),
        'E74779': ('יָרָבְעָם', 'nmpr', 'Jeroboam', 'yaravam'),
        'E74780': ('נְבָט', 'nmpr', 'Nebat', "n'vat"),
        'E74781': ('צְרֵדָה', 'nmpr', 'Zeredah', "ts'reida"),
        'E74782': ('צְרוּעָה', 'nmpr', 'Zeruah', "ts'rua"),
        'E74783': ('סֵבֶל', 'subs', 'load', 'seivel'),
        'E74784': ('שִׁילֹנִי', 'adjv', 'Shilonite', 'shiloni'),
        'E74785': ('קְרָעִים', 'subs', 'rags', "k'raim"),
        'E74786': ('נִיר', 'subs', 'lamp', 'nir'),
        'E74787': ('שִׁישָׁק', 'nmpr', 'Shishak', 'shishak'),
        'E74788': ('רְחַבְעָם', 'nmpr', 'Rehoboam', "r'chavam"),
        'E74789': ('קֹטֶן', 'subs', 'the little one', 'koten'),
        'E74790': ('שֹׁוט', 'subs', 'whip', 'shovt'),
        'E74791': ('סִבָּה', 'subs', 'turn', 'siba'),
        'E74792': ('שְׁמַעְיָה', 'nmpr', 'Shemaiah', "sh'maya"),
        'E74793': ('בדא', 'verb', 'devise', 'vd'),
        'E74794': ('יֹאשִׁיָּהוּ', 'nmpr', 'Josiah', 'yoshiahu'),
        'E74795': ('מַתַּת', 'subs', 'gift', 'matat'),
        'E74796': ('הֹוי', 'intj', 'alas', 'hovy'),
        'E74797': ('שֹׁמְרֹון', 'nmpr', 'Samaria', 'shomrovn'),
        'E74798': ('חָפֵץ', 'adjv', 'delighting', 'chafeits'),
        'E74799': ('בַּקְבֻּק', 'subs', 'bottle', 'bakbuk'),
        'E74800': ('אֲחִיָּהוּ', 'nmpr', 'Ahijah', 'achiahu'),
        'E74801': ('שֵׂיב', 'subs', 'age', 'seiv'),
        'E74802': ('גַּו', 'subs', 'back', 'gav'),
        'E74803': ('גָּלָל', 'subs', 'dung', 'galal'),
        'E74804': ('תָּא', 'subs', 'chamber', 'ta'),
        'E74805': ('אֲבִיָּם', 'nmpr', 'Abijah', 'abiam'),
        'E74806': ('אֲבִישָׁלֹום', 'nmpr', 'Abishalom', 'avishalovm'),
        'E74807': ('אָסָא', 'nmpr', 'Asa', 'asa'),
        'E74808': ('מִפְלֶצֶת', 'subs', 'idol', 'mifletset'),
        'E74809': ('בַּעְשָׁא', 'nmpr', 'Baasha', 'basha'),
        'E74810': ('בֶּן הֲדַד', 'nmpr', 'Ben-Hadad', 'ben hadad'),
        'E74811': ('טַבְרִמֹּן', 'nmpr', 'Tabrimmon', 'tavrimon'),
        'E74812': ('חֶזְיֹון', 'nmpr', 'Hezion', 'chezyovn'),
        'E74813': ('עִיֹּון', 'nmpr', 'Ijon', 'iovn'),
        'E74814': ('אָבֵל בֵּית מַעֲכָה',
                   'nmpr',
                   'Abel-beth-maacah',
                   "aveil beit ma'acho"),
        'E74815': ('יֵהוּא', 'nmpr', 'Jehu', 'yeihu'),
        'E74816': ('חֲנָנִי', 'nmpr', 'Hanani', 'chanani'),
        'E74817': ('אַרְצָא', 'nmpr', 'Arza', 'artsa'),
        'E74818': ('עָמְרִי', 'nmpr', 'Omri', 'amri'),
        'E74819': ('אַרְמֹון', 'subs', 'dwelling tower', 'armovn'),
        'E74820': ('תִּבְנִי', 'nmpr', 'Tibni', 'tivni'),
        'E74821': ('גִּינַת', 'nmpr', 'Ginath', 'ginat'),
        'E74822': ('שֶׁמֶר', 'nmpr', 'Shemer', 'shemer'),
        'E74823': ('אַחְאָב', 'nmpr', 'Ahab', 'achav'),
        'E74824': ('אִיזֶבֶל', 'nmpr', 'Jezebel', 'izevel'),
        'E74825': ('אֶתְבַּעַל', 'nmpr', 'Ethbaal', "etba'al"),
        'E74826': ('חִיאֵל', 'nmpr', 'Hiel', 'chieil'),
        'E74827': ('אֱלִי', 'adjv', '-elite', 'eli'),
        'E74828': ('שְׂגוּב', 'nmpr', 'Segub', "s'guv"),
        'E74829': ('אֵלִיָּהוּ', 'nmpr', 'Elijah', 'eiliahu'),
        'E74830': ('תִּשְׁבִּי', 'adjv', 'Tishbite', 'tishbi'),
        'E74831': ('כְּרִית', 'nmpr', 'Kerith', "k'rit"),
        'E74832': ('צָרְפַת', 'nmpr', 'Zarephath', 'tsarfat'),
        'E74833': ('מָעֹוג', 'subs', 'venue', 'maovg'),
        'E74834': ('עֹבַדְיָהוּ', 'nmpr', 'Obadiah', 'ovadyahu'),
        'E74835': ('חָצִיר', 'subs', 'grass', 'chatsir'),
        'E74836': ('סְעִפִּים', 'subs', 'crutches', "s'ipim"),
        'E74837': ('התל', 'verb', 'mock', 'htl'),
        'E74838': ('שִׂיג', 'subs', 'excrement', 'sig'),
        'E74839': ('קֶשֶׁב', 'subs', 'attentiveness', 'keshev'),
        'E74840': ('תְּעָלָה', 'subs', 'channel', "t'ala"),
        'E74841': ('גהר', 'verb', 'crouch', 'ghr'),
        'E74842': ('קדר', 'verb', 'be dark', 'kdr'),
        'E74843': ('שׁנס', 'verb', 'gird', 'shns'),
        'E74844': ('רֹתֶם', 'subs', 'broom', 'rotem'),
        'E74845': ('רֶצֶף', 'subs', 'coal', 'retsef'),
        'E74846': ('אֲכִילָה', 'subs', 'food', 'achila'),
        'E74847': ('רַעַשׁ', 'subs', 'quaking', "ra'ash"),
        'E74848': ('דְּמָמָה', 'subs', 'calm', "d'mama"),
        'E74849': ('חֲזָאֵל', 'nmpr', 'Hazael', 'chazaeil'),
        'E74850': ('נִמְשִׁי', 'nmpr', 'Nimshi', 'nimshi'),
        'E74851': ('אֱלִישָׁע', 'nmpr', 'Elisha', 'elisha'),
        'E74852': ('מַחְמָד', 'subs', 'desire', 'machmad'),
        'E74853': ('שׂפק', 'verb', 'suffice', 'sfk'),
        'E74854': ('שֹׁעַל', 'subs', 'handful', 'shoal'),
        'E74855': ('מְדִינָה', 'subs', 'district', "m'dina"),
        'E74856': ('חָשִׂף', 'subs', 'handful', 'chasif'),
        'E74857': ('חלט', 'verb', 'decide', 'chlt'),
        'E74858': ('אֲפֵר', 'subs', 'band', 'afeir'),
        'E74859': ('סַר', 'adjv', 'sullen', 'sar'),
        'E74860': ('זָעֵף', 'adjv', 'raging', 'zaeif'),
        'E74861': ('נָבֹות', 'nmpr', 'Naboth', 'navovt'),
        'E74862': ('חֹר', 'subs', 'noble one', 'chor'),
        'E74863': ('יִמְלָא', 'nmpr', 'Imlah', 'yimla'),
        'E74864': ('צִדְקִיָּה', 'nmpr', 'Zedekiah', 'tsidkia'),
        'E74865': ('כְּנַעֲנָה', 'nmpr', 'Kenaanah', "k'na'ana"),
        'E74866': ('צִדְקִיָּהוּ', 'nmpr', 'Zedekiah', 'tsidkiahu'),
        'E74867': ('חבה', 'verb', 'hide', 'chvh'),
        'E74868': ('אָמֹון', 'nmpr', 'Amon', 'amovn'),
        'E74869': ('כֶּלֶא', 'subs', 'prison', 'kele'),
        'E74870': ('דֶּבֶק', 'subs', 'glue', 'devek'),
        'E74871': ('רַכָּב', 'subs', 'charioteer', 'rakav'),
        'E74872': ('אֲחַזְיָהוּ', 'nmpr', 'Ahaziah', 'achazyahu'),
        'E74873': ('עֲזוּבָה', 'nmpr', 'Azubah', 'azuva'),
        'E74874': ('שִׁלְחִי', 'nmpr', 'Shilhi', 'shilchi'),
        'E74875': ('יְהֹורָם', 'nmpr', 'Jehoram', "y'hovram"),
        'E74876': ('אֲחַזְיָה', 'nmpr', 'Ahaziah', 'achazya'),
        'E74877': ('זְבוּב', 'subs', 'fly', "z'vuv"),
        'E74878': ('אֵלִיָּה', 'nmpr', 'Elijah', 'eilia'),
        'E74879': ('אֵזֹור', 'subs', 'loin-cloth', 'eizovr'),
        'E74880': ('סְעָרָה', 'subs', 'storm', "s'ara"),
        'E74881': ('גלם', 'verb', 'wrap', 'glm'),
        'E74882': ('צְלֹחִית', 'subs', 'pan', "ts'lochit"),
        'E74883': ('קלס', 'verb', 'disdain', 'kls'),
        'E74884': ('מֵישַׁע', 'nmpr', '[king of moab]', 'meisha'),
        'E74885': ('נֹקֵד', 'subs', 'sheep-raiser', 'nokeid'),
        'E74886': ('מִבְחֹור', 'subs', 'choice', 'mivchovr'),
        'E74887': ('חרב', 'verb', 'slaughter', 'chrv'),
        'E74888': ('קִיר חֲרָשֶׂת', 'nmpr', 'Kir Hareseth', 'kir charaset'),
        'E74889': ('קַלָּע', 'subs', 'slinger', 'kala'),
        'E74890': ('אָסוּךְ', 'subs', 'jar', 'asuch'),
        'E74891': ('נְשִׁי', 'subs', 'debt', "n'shi"),
        'E74892': ('גֵּיחֲזִי', 'nmpr', 'Gehazi', 'geichazi'),
        'E74893': ('שׁלה', 'verb', 'be easy', 'shlh'),
        'E74894': ('זרר', 'verb', 'squeeze', "zr'r"),
        'E74895': ('שׁפת', 'verb', 'set', 'shft'),
        'E74896': ('אֹרֹת', 'subs', 'mallow [flower]', 'orot'),
        'E74897': ('פַּקֻּעֹת', 'subs', 'gourd', 'pakuot'),
        'E74898': ('פלח', 'verb', 'cleave', 'flch'),
        'E74899': ('בַּעַל שָׁלִשָׁה', 'nmpr', 'Baal Shalishah', "ba'al shalisha"),
        'E74900': ('צִקָּלֹון', 'subs', '[uncertain]', 'tsikalovn'),
        'E74901': ('אַחֲלַי', 'intj', 'alas', 'achalay'),
        'E74902': ('אֲמָנָה', 'nmpr', 'Amana', 'amana'),
        'E74903': ('פַּרְפַּר', 'nmpr', 'Pharpar', 'parpar'),
        'E74904': ('רִמֹּון', 'nmpr', 'Rimmon', 'rimovn'),
        'E74905': ('חָרִיט', 'subs', 'purse', 'charit'),
        'E74906': ('עֹפֶל', 'nmpr', 'Ophel', 'ofel'),
        'E74907': ('קצב', 'verb', 'cut off', 'ktsv'),
        'E74908': ('תַּחֲנֹתִי', 'subs', '[uncertain]', 'tachanoti'),
        'E74909': ('נָחֵת', 'subs', '[uncertain]', 'nacheit'),
        'E74910': ('סער', 'verb', 'be stormy', 'sr'),
        'E74911': ('אֵיכֹה', 'inrg', 'how', 'eicho'),
        'E74912': ('כרה', 'verb', 'feast', 'chrh'),
        'E74913': ('כֵּרָה', 'subs', 'feast', 'keira'),
        'E74914': ('רֹבַע', 'subs', 'fourth part', 'rova'),
        'E74915': ('קַב', 'subs', 'kab', 'kav'),
        'E74916': ('דִּבְיֹונִים', 'subs', 'excretion', 'divovnim'),
        'E74917': ('חכה', 'verb', 'wait', 'chchh'),
        'E74918': ('רמס', 'verb', 'trample', 'rms'),
        'E74919': ('רטשׁ', 'verb', 'dash', 'rtsh'),
        'E74920': ('מַכְבֵּר', 'subs', 'cover', 'machbeir'),
        'E74921': ('צָעִיר', 'nmpr', 'Zair', 'tsair'),
        'E74922': ('עֲתַלְיָהוּ', 'nmpr', 'Athaliah', 'atalyahu'),
        'E74923': ('שִׁפְעָה', 'subs', 'mass', 'shifa'),
        'E74924': ('מִנְהָג', 'subs', 'driving', 'minhag'),
        'E74925': ('כֶּשֶׁף', 'subs', 'sorcery', 'keshef'),
        'E74926': ('בִּדְקַר', 'nmpr', 'Bidkar', 'bidkar'),
        'E74927': ('מַשָּׂא', 'subs', 'utterance', 'masa'),
        'E74928': ('בֵּית הַגָּן', 'nmpr', 'Beth Haggan', 'beit hagan'),
        'E74929': ('מַעֲלֵה גוּר', 'nmpr', 'the way up to Gur', "ma'alei gur"),
        'E74930': ('סמם', 'verb', 'apply paste', "sm'm"),
        'E74931': ('פּוּךְ', 'subs', 'mascara', 'puch'),
        'E74932': ('דֹּמֶן', 'subs', 'dung', 'domen'),
        'E74933': ('צִבֻּר', 'subs', 'heap', 'tsibur'),
        'E74934': ('בֵּית עֵקֶד הָרֹעִים',
                   'nmpr',
                   'Beth Eked of the Shepherds',
                   'beit eiked haroim'),
        'E74935': ('בֵּית עֵקֶד', 'nmpr', 'Beth Eked', 'beit eiked'),
        'E74936': ('עָקְבָּה', 'subs', 'insidiousness', 'akba'),
        'E74937': ('עֲצָרָה', 'subs', 'assembly', 'atsara'),
        'E74938': ('מֶלְתָּחָה', 'subs', 'ward-robe', 'meltacha'),
        'E74939': ('מֹוצָאָה', 'subs', 'origin', "movtsa'a"),
        'E74940': ('יְהֹואָחָז', 'nmpr', 'Jehoahaz', "y'hovachaz"),
        'E74941': ('עֲתַלְיָה', 'nmpr', 'Athaliah', 'atalya'),
        'E74942': ('יְהֹושֶׁבַע', 'nmpr', 'Jehosheba', "y'hovsheva"),
        'E74943': ('כָּרִי', 'nmpr', 'Carites', 'kari'),
        'E74944': ('סוּר', 'nmpr', 'Sur', 'sur'),
        'E74945': ('מַסָּח', 'advb', 'alternatively', 'masach'),
        'E74946': ('מַתָּן', 'nmpr', 'Mattan', 'matan'),
        'E74947': ('יְהֹואָשׁ', 'nmpr', 'Jehoash', "y'hovash"),
        'E74948': ('צִבְיָה', 'nmpr', 'Zibiah', 'tsiva'),
        'E74949': ('מַכָּר', 'subs', 'client', 'makar'),
        'E74950': ('בֶּדֶק', 'subs', 'breach', 'bedek'),
        'E74951': ('גדר', 'verb', 'heap stones', 'gdr'),
        'E74952': ('מַחְצֵב', 'subs', 'hewn stone', 'machtseiv'),
        'E74953': ('סִלָּא', 'nmpr', 'Silla', 'sila'),
        'E74954': ('יֹוזָבָד', 'nmpr', 'Jozabad', 'yovzavad'),
        'E74955': ('שִׁמְעָת', 'nmpr', 'Shimeath', 'shimat'),
        'E74956': ('יְהֹוזָבָד', 'nmpr', 'Jehozabad', "y'hovzavad"),
        'E74957': ('שֹׁמֵר', 'nmpr', 'Shomer', 'shomeir'),
        'E74958': ('אֲמַצְיָה', 'nmpr', 'Amaziah', 'amatsya'),
        'E74959': ('יֹואָחָז', 'nmpr', 'Joahaz', 'yovachaz'),
        'E74960': ('אֲמַצְיָהוּ', 'nmpr', 'Amaziah', 'amatsyahu'),
        'E74961': ('יְהֹועַדָּן', 'nmpr', 'Jehoaddin', "y'hovadan"),
        'E74962': ('חֹוחַ', 'subs', 'thorn', 'chovach'),
        'E74963': ('תַּעֲרוּבֹות', 'subs', 'hostages', "ta'aruvovt"),
        'E74964': ('עֲזַרְיָה', 'nmpr', 'Azariah', 'azarya'),
        'E74965': ('יֹונָה', 'nmpr', 'Jonah', 'yovna'),
        'E74966': ('אֲמִתַּי', 'nmpr', 'Amittai', 'amitay'),
        'E74967': ('גַּת הַחֵפֶר', 'nmpr', 'Gath Hepher', 'gat hacheifer'),
        'E74968': ('זְכַרְיָה', 'nmpr', 'Zechariah', "z'charya"),
        'E74969': ('יְכָלְיָהוּ', 'nmpr', 'Jecoliah', "y'cholyahu"),
        'E74970': ('חָפְשִׁית', 'subs', 'exempt', 'chafshit'),
        'E74971': ('זְכַרְיָהוּ', 'nmpr', 'Zechariah', "z'charyahu"),
        'E74972': ('שַׁלּוּם', 'nmpr', 'Shallum', 'shalum'),
        'E74973': ('יָבֵישׁ', 'nmpr', 'Jabesh', 'yaveish'),
        'E74974': ('קָבָלְ־', 'subs', '[uncertain]', 'kaval'),
        'E74975': ('עֻזִּיָּה', 'nmpr', 'Uzziah', 'uzia'),
        'E74976': ('מְנַחֵם', 'nmpr', 'Menahem', "m'nacheim"),
        'E74977': ('גָּדִי', 'nmpr', 'Gadi', 'gadi'),
        'E74978': ('פּוּל', 'nmpr', 'Pul', 'pul'),
        'E74979': ('פְּקַחְיָה', 'nmpr', 'Pekahiah', "p'kachya"),
        'E74980': ('פֶּקַח', 'nmpr', 'Pekah', 'pekach'),
        'E74981': ('רְמַלְיָהוּ', 'nmpr', 'Remaliah', "r'malyahu"),
        'E74982': ('אַרְיֵה', 'nmpr', 'Arieh', 'aryei'),
        'E74983': ('תִּגְלַת פִּלְאֶסֶר', 'nmpr', 'Tiglath-Pileser', 'tiglat pileser'),
        'E74984': ('יָנֹוחַ', 'nmpr', 'Janoah', 'yanovach'),
        'E74985': ('עֻזִּיָּהוּ', 'nmpr', 'Uzziah', 'uziahu'),
        'E74986': ('יְרוּשָׁא', 'nmpr', 'Jerusha', "y'rusha"),
        'E74987': ('רְצִין', 'nmpr', 'Rezin', "r'tsin"),
        'E74988': ('אָחָז', 'nmpr', 'Ahaz', 'achaz'),
        'E74989': ('יְהוּדִי', 'adjv', 'Jewish', "y'hudi"),
        'E74990': ('קִיר', 'nmpr', 'Kir', 'kir'),
        'E74991': ('דּוּמֶּשֶׂק', 'nmpr', 'Damascus', 'dumesek'),
        'E74992': ('מַרְצֶפֶת', 'subs', 'pavement', 'martsefet'),
        'E74993': ('מוּסָךְ', 'subs', '[uncertain]', 'musach'),
        'E74994': ('חִזְקִיָּהוּ', 'nmpr', 'Hizkiah', 'chizkiahu'),
        'E74995': ('שַׁלְמַנְאֶסֶר', 'nmpr', 'Shalmaneser', 'shalmaneser'),
        'E74996': ('סֹוא', 'nmpr', 'So', 'sov'),
        'E74997': ('חֲלַח', 'nmpr', 'Halah', 'chalach'),
        'E74998': ('חָבֹור', 'nmpr', 'Habor', 'chavovr'),
        'E74999': ('גֹּוזָן', 'nmpr', 'Gozan', 'govzan'),
        'E75000': ('חפא', 'verb', '[uncertain]', 'chf'),
        'E75001': ('הבל', 'verb', 'be vain', 'hvl'),
        'E75002': ('כּוּת', 'nmpr', 'Cuthah', 'kut'),
        'E75003': ('עַוָּא', 'nmpr', 'Avva', 'aua'),
        'E75004': ('סְפַרְוַיִם', 'nmpr', 'Sepharvaim', "s'farvayim"),
        'E75005': ('שֹׁמְרֹונִי', 'adjv', 'from Samaria', 'shomrovni'),
        'E75006': ('סֻכֹּות בְּנֹות', 'nmpr', 'Succoth Benoth', "sukovt b'novt"),
        'E75007': ('נֵרְגַל', 'nmpr', 'Nergal', 'neirgal'),
        'E75008': ('אֲשִׁימָא', 'nmpr', 'Ashima', 'ashima'),
        'E75009': ('נִבְחַז', 'nmpr', 'Nibhaz', 'nivchaz'),
        'E75010': ('תַּרְתָּק', 'nmpr', 'Tartak', 'tartak'),
        'E75011': ('סְפַרְוִי', 'adjv', 'Sepharvite', "s'farvi"),
        'E75012': ('אַדְרַמֶּלֶךְ', 'nmpr', '[Assyrian god]', 'adramelech'),
        'E75013': ('עֲנַמֶּלֶךְ', 'nmpr', 'Anammelech', 'anamelech'),
        'E75014': ('חִזְקִיָּה', 'nmpr', 'Hizkiah', 'chizkia'),
        'E75015': ('אֲבִי', 'nmpr', 'Abi', 'avi'),
        'E75016': ('נְחֻשְׁתָּן', 'nmpr', 'Nehushtan', "n'chushtan"),
        'E75017': ('סַנְחֵרִיב', 'nmpr', 'Sennacherib', 'sancheiriv'),
        'E75018': ('אֹמְנָה', 'subs', 'door-post', 'omna'),
        'E75019': ('תַּרְתָּן', 'subs', 'commander', 'tartan'),
        'E75020': ('רַב', 'subs', 'chief', 'rav'),
        'E75021': ('רַב שָׁקֵה', 'subs', 'rabshake', 'rav shakei'),
        'E75022': ('אֶלְיָקִים', 'nmpr', 'Eliakim', 'elyakim'),
        'E75023': ('חִלְקִיָּהוּ', 'nmpr', 'Hilkiah', 'chilkiahu'),
        'E75024': ('שֶׁבְנָה', 'nmpr', 'Shebna', 'shevna'),
        'E75025': ('יֹואָח', 'nmpr', 'Joah', 'yovach'),
        'E75026': ('אָסָף', 'nmpr', 'Asaph', 'asaf'),
        'E75027': ('בִּטָּחֹון', 'subs', 'confidence', 'bitachovn'),
        'E75028': ('אֲרָמִי', 'adjv', 'in Aramaic', 'arami'),
        'E75029': ('צֹאָה', 'subs', 'vomit', 'tsoa'),
        'E75030': ('אַרְפַּד', 'nmpr', 'Arpad', 'arpad'),
        'E75031': ('הֵנַע', 'nmpr', 'Hena', 'heina'),
        'E75032': ('עַוָּה', 'nmpr', 'Ivvah', 'aua'),
        'E75033': ('חִלְקִיָּה', 'nmpr', 'Hilkiah', 'chilkia'),
        'E75034': ('שֶׁבְנָא', 'nmpr', 'Shebna', 'shevna'),
        'E75035': ('יְשַׁעְיָהוּ', 'nmpr', 'Isaiah', "y'shayahu"),
        'E75036': ('אָמֹוץ', 'nmpr', 'Amoz', 'amovts'),
        'E75037': ('תֹּוכֵחָה', 'subs', 'rebuke', 'tovcheicha'),
        'E75038': ('נְאָצָה', 'subs', 'contumely', "n'atsa"),
        'E75039': ('מַשְׁבֵּר', 'subs', 'mouth of womb', 'mashbeir'),
        'E75040': ('לֵדָה', 'subs', 'birth', 'leida'),
        'E75041': ('תִּרְהָקָה', 'nmpr', 'Tirhakah', 'tirhaka'),
        'E75042': ('רֶצֶף', 'nmpr', 'Rezeph', 'retsef'),
        'E75043': ('עֶדֶן', 'nmpr', 'Eden', 'eden'),
        'E75044': ('תְּלַאשָּׂר', 'nmpr', 'Telassar', "t'lasar"),
        'E75045': ('אָמְנָם', 'advb', 'really', 'amnam'),
        'E75046': ('בוז', 'verb', 'despise', 'vvz'),
        'E75047': ('לעג', 'verb', 'mock', 'lg'),
        'E75048': ('כַּרְמֶל', 'subs', 'orchard', 'karmel'),
        'E75049': ('קור', 'verb', 'dig', 'kvr'),
        'E75050': ('מָצֹור', 'nmpr', 'Egypt', 'matsovr'),
        'E75051': ('שׁאה', 'verb', 'be waste', 'shh'),
        'E75052': ('קָצֵר', 'adjv', 'short', 'katseir'),
        'E75053': ('שְׁדֵפָה', 'subs', 'scorching', "sh'deifa"),
        'E75054': ('שַׁאֲנָן', 'adjv', 'at ease', "sha'anan"),
        'E75055': ('סָחִישׁ', 'subs', 'shoot', 'sachish'),
        'E75056': ('גנן', 'verb', 'enclose', "gn'n"),
        'E75057': ('נִסְרֹךְ', 'nmpr', 'Nisroch', 'nisroch'),
        'E75058': ('אַדְרַמֶּלֶךְ', 'nmpr', 'Adrammelech', 'adramelech'),
        'E75059': ('שַׂרְאֶצֶר', 'nmpr', 'Sharezer', 'saretser'),
        'E75060': ('אֵסַר חַדֹּן', 'nmpr', 'Esarhaddon', 'eisar chadon'),
        'E75061': ('דִּמְעָה', 'subs', 'tear', 'dima'),
        'E75062': ('דְּבֶלֶת', 'subs', 'fig cake', "d'velet"),
        'E75063': ('יְחִזְקִיָּהוּ', 'nmpr', 'Hezekiah', "y'chizkiahu"),
        'E75064': ('בְּרֹאדַךְ בַּלְאֲדָן',
                   'nmpr',
                   'Merodach-Baladan',
                   "b'rodach baladan"),
        'E75065': ('בַּלְאֲדָן', 'nmpr', 'Baladan', 'baladan'),
        'E75066': ('נְכֹת', 'subs', 'treasure', "n'chot"),
        'E75067': ('טֹוב', 'subs', 'perfume', 'tovv'),
        'E75068': ('אצר', 'verb', 'store', 'tsr'),
        'E75069': ('חֶפְצִי בָהּ', 'nmpr', 'Hephzibah', 'cheftsi vah'),
        'E75070': ('מִשְׁקֶלֶת', 'subs', 'leveller', 'mishkelet'),
        'E75071': ('צַלַּחַת', 'subs', 'pan', 'tsalachat'),
        'E75072': ('מְשִׁסָּה', 'subs', 'plunder', "m'shisa"),
        'E75073': ('עֻזָּא', 'nmpr', 'Uzza', 'uza'),
        'E75074': ('מְשֻׁלֶּמֶת', 'nmpr', 'Meshullemeth', "m'shulemet"),
        'E75075': ('חָרוּץ', 'nmpr', 'Haruz', 'charuts'),
        'E75076': ('יָטְבָה', 'nmpr', 'Jotbah', 'yatva'),
        'E75077': ('יְדִידָה', 'nmpr', 'Jedidah', "y'dida"),
        'E75078': ('עֲדָיָה', 'nmpr', 'Adaiah', 'adaya'),
        'E75079': ('שָׁפָן', 'nmpr', 'Shaphan', 'shafan'),
        'E75080': ('אֲצַלְיָהוּ', 'nmpr', 'Azaliah', 'atsalyahu'),
        'E75081': ('מְשֻׁלָּם', 'nmpr', 'Meshullam', "m'shulam"),
        'E75082': ('אֲחִיקָם', 'nmpr', 'Ahikam', 'achikam'),
        'E75083': ('מִיכָיָה', 'nmpr', 'Micaiah', 'michoya'),
        'E75084': ('עֲשָׂיָה', 'nmpr', 'Asaiah', 'asaya'),
        'E75085': ('חֻלְדָּה', 'nmpr', 'Huldah', 'chulda'),
        'E75086': ('תִּקְוָה', 'nmpr', 'Tikvah', 'tikva'),
        'E75087': ('חַרְחַס', 'nmpr', 'Harhas', 'charchas'),
        'E75088': ('כֹּמֶר', 'subs', 'idol priest', 'komer'),
        'E75089': ('מַזָּלֹות', 'subs', 'constellation', 'mazalovt'),
        'E75090': ('תֹּפֶת', 'nmpr', 'Topheth', 'tofet'),
        'E75091': ('נְתַן מֶלֶךְ', 'nmpr', 'Nathan-Melech', "n'tan melech"),
        'E75092': ('פַּרְבָּר', 'subs', 'court', 'parbar'),
        'E75093': ('צִיּוּן', 'subs', 'signpost', 'tsiun'),
        'E75094': ('נְכֹה', 'nmpr', 'Neco', "n'cho"),
        'E75095': ('חֲמוּטַל', 'nmpr', 'Hamutal', 'chamutal'),
        'E75096': ('יִרְמְיָהוּ', 'nmpr', 'Jeremiah', 'yirmahu'),
        'E75097': ('עֹנֶשׁ', 'subs', 'fine', 'onesh'),
        'E75098': ('יְהֹויָקִים', 'nmpr', 'Jehoiakim', "y'hovyakim"),
        'E75099': ('זְבוּדָּה', 'nmpr', 'Zebidah', "z'vuda"),
        'E75100': ('פְּדָיָה', 'nmpr', 'Pedaiah', "p'daya"),
        'E75101': ('נְבוּכַדְנֶאצַּר', 'nmpr', 'Nebuchadnezzar', "n'vuchadnetsar"),
        'E75102': ('יְהֹויָכִין', 'nmpr', 'Jehoiachin', "y'hovyachin"),
        'E75103': ('נְחֻשְׁתָּא', 'nmpr', 'Nehushta', "n'chushta"),
        'E75104': ('אֶלְנָתָן', 'nmpr', 'Elnathan', 'elnatan'),
        'E75105': ('מַסְגֵּר', 'subs', 'bulwark-maker', 'masgeir'),
        'E75106': ('דַּלָּה', 'subs', 'poor', 'dala'),
        'E75107': ('גֹּולָה', 'subs', 'exile', 'govla'),
        'E75108': ('מַתַּנְיָה', 'nmpr', 'Mattaniah', 'matanya'),
        'E75109': ('דָּיֵק', 'subs', 'bulwark', 'dayeik'),
        'E75110': ('נְבוּזַרְאֲדָן', 'nmpr', 'Nebuzaradan', "n'vuzaradan"),
        'E75111': ('כֹּרֵם', 'subs', 'vinedresser', 'koreim'),
        'E75112': ('יגב', 'verb', 'labour', 'ygv'),
        'E75113': ('צְפַנְיָהוּ', 'nmpr', 'Zephaniah', "ts'fanyahu"),
        'E75114': ('גְּדַלְיָהוּ', 'nmpr', 'Gedaliah', "g'dalyahu"),
        'E75115': ('נְתַנְיָה', 'nmpr', 'Nethaniah', "n'tanya"),
        'E75116': ('יֹוחָנָן', 'nmpr', 'Johanan', 'yovchanan'),
        'E75117': ('קָרֵחַ', 'nmpr', 'Kareah', 'kareiach'),
        'E75118': ('תַּנְחֻמֶת', 'nmpr', 'Tanhumeth', 'tanchumet'),
        'E75119': ('יַאֲזַנְיָהוּ', 'nmpr', 'Jaazaniah', "ya'azanyahu"),
        'E75120': ('גָּלוּת', 'subs', 'exile', 'galut'),
        'E75121': ('אֱוִיל מְרֹדַךְ', 'nmpr', 'Evil-merodach', "evil m'rodach"),
        'E75122': ('אֲרֻחָה', 'subs', 'sustenance', 'arucha'),
        'E75123': ('אֵבוּס', 'subs', 'manger', 'eivus'),
        'E75124': ('זור', 'verb', 'turn aside', 'zvr'),
        'E75125': ('דַּוָּי', 'adjv', 'ill', 'dauay'),
        'E75126': ('מְלוּנָה', 'subs', 'watching-place', "m'luna"),
        'E75127': ('מִקְשָׁה', 'subs', 'cucumber field', 'miksha'),
        'E75128': ('זכה', 'verb', 'be clean', 'zchh'),
        'E75129': ('אשׁר', 'verb', 'walk straight', 'shr'),
        'E75130': ('חָמֹוץ', 'subs', 'oppressor', 'chamovts'),
        'E75131': ('לבן', 'verb', 'be white', 'lvn'),
        'E75132': ('סִיג', 'subs', 'lead oxide', 'sig'),
        'E75133': ('סֹבֶא', 'subs', 'drink', 'sove'),
        'E75134': ('מהל', 'verb', 'vitiate', 'mhl'),
        'E75135': ('שַׁלְמֹנִים', 'subs', 'gifts', 'shalmonim'),
        'E75136': ('בֹּר', 'subs', 'potash', 'bor'),
        'E75137': ('בָּדִיל', 'subs', 'dross of melting', 'badil'),
        'E75138': ('אַיִל', 'subs', 'mighty tree', 'ayil'),
        'E75139': ('חפר', 'verb', 'be ashamed', 'chfr'),
        'E75140': ('חָסֹן', 'adjv', 'strong', 'chason'),
        'E75141': ('נִיצֹוץ', 'subs', 'spark', 'nitsovts'),
        'E75142': ('נהר', 'verb', 'stream', 'nhr'),
        'E75143': ('מַזְמֵרָה', 'subs', 'pruning-knife', 'mazmeira'),
        'E75144': ('שׂפק', 'verb', 'clap hands', 'sfk'),
        'E75145': ('קֵצֶה', 'subs', 'end', 'keitseh'),
        'E75146': ('שׁחח', 'verb', 'bow down', "shch'ch"),
        'E75147': ('גַּבְהוּת', 'subs', 'haughtiness', 'gavhut'),
        'E75148': ('רוּם', 'subs', 'height', 'rum'),
        'E75149': ('גֵּאֶה', 'adjv', 'haughty', 'geieh'),
        'E75150': ('שָׁפֵל', 'adjv', 'low', 'shafeil'),
        'E75151': ('שְׂכִיָּה', 'subs', 'ship', "s'kia"),
        'E75152': ('מְחִלָּה', 'subs', 'hole', "m'chila"),
        'E75153': ('פֵּרֹות', 'subs', '[uncertain]', 'peirovt'),
        'E75154': ('מַשְׁעֵן', 'subs', 'support', 'mashein'),
        'E75155': ('מַשְׁעֵנָה', 'subs', 'support', 'masheina'),
        'E75156': ('חֶרֶשׁ', 'subs', '[uncertain]', 'cheresh'),
        'E75157': ('לַחַשׁ', 'subs', 'whispering', 'lachash'),
        'E75158': ('תַּעֲלוּלִים', 'subs', 'ill-treatment', "ta'alulim"),
        'E75159': ('רהב', 'verb', 'storm against', 'rhv'),
        'E75160': ('מַכְשֵׁלָה', 'subs', 'decay', 'machsheila'),
        'E75161': ('הַכָּרָה', 'subs', 'expression', 'hakara'),
        'E75162': ('בלע', 'verb', 'confuse', 'vl'),
        'E75163': ('דכא', 'verb', 'oppress', 'dch'),
        'E75164': ('גָּרֹון', 'subs', 'throat', 'garovn'),
        'E75165': ('שׂקר', 'verb', 'seduce', 'skr'),
        'E75166': ('טפף', 'verb', 'trip', "tf'f"),
        'E75167': ('עכס', 'verb', 'fetter', 'chs'),
        'E75168': ('שׂפח', 'verb', 'be scabby', 'sfch'),
        'E75169': ('עֶכֶס', 'subs', 'anklet', 'eches'),
        'E75170': ('שָׁבִיס', 'subs', 'headband', 'shavis'),
        'E75171': ('שֵׁר', 'subs', 'bracelet', 'sheir'),
        'E75172': ('רְעָלָה', 'subs', 'veil', "r'ala"),
        'E75173': ('קִשֻּׁרִים', 'subs', 'bands', 'kishurim'),
        'E75174': ('מַחֲלָצֹות', 'subs', 'robes', 'machalatsovt'),
        'E75175': ('מַעֲטֶפֶת', 'subs', 'overtunic', "ma'atefet"),
        'E75176': ('מִטְפַּחַת', 'subs', 'cloak', 'mitpachat'),
        'E75177': ('גִּלָּיֹון', 'subs', 'tablet', 'gilayovn'),
        'E75178': ('צְנִיפָה', 'subs', 'turban', "ts'nifa"),
        'E75179': ('רְדִיד', 'subs', 'wrapper', "r'did"),
        'E75180': ('מַק', 'subs', 'musty smell', 'mak'),
        'E75181': ('נִקְפָּה', 'subs', 'rope', 'nikpa'),
        'E75182': ('מִקְשֶׁה', 'subs', 'hair setting', 'miksheh'),
        'E75183': ('פְּתִיגִיל', 'subs', 'garment', "p'tigil"),
        'E75184': ('מַחֲגֹרֶת', 'subs', 'girding', 'machagoret'),
        'E75185': ('כִּי', 'subs', 'branding', 'ki'),
        'E75186': ('יֳפִי', 'subs', 'beauty', 'yafi'),
        'E75187': ('אנה', 'verb', 'mourn', 'nh'),
        'E75188': ('דוח', 'verb', 'rinse', 'dvch'),
        'E75189': ('חֻפָּה', 'subs', 'balcony', 'chupa'),
        'E75190': ('מַחְסֶה', 'subs', 'refuge', 'machseh'),
        'E75191': ('מִסְתֹּור', 'subs', 'shelter', 'mistovr'),
        'E75192': ('זֶרֶם', 'subs', 'rain', 'zerem'),
        'E75193': ('עזק', 'verb', 'dig', 'zk'),
        'E75194': ('שֹׂרֵק', 'subs', 'grape', 'soreik'),
        'E75195': ('בָּאֻשׁ', 'subs', 'rotten berry', 'baush'),
        'E75196': ('מְשׂוּכָּה', 'subs', 'hedge', "m'suka"),
        'E75197': ('מִרְמָס', 'subs', 'trampled land', 'mirmas'),
        'E75198': ('בָּתָה', 'subs', '[uncertain]', 'bata'),
        'E75199': ('עדר', 'verb', 'weed', 'dr'),
        'E75200': ('שָׁמִיר', 'subs', 'thornbush', 'shamir'),
        'E75201': ('שַׁיִת', 'subs', 'weed', 'shayit'),
        'E75202': ('נֶטַע', 'subs', 'plantation', 'neta'),
        'E75203': ('שַׁעֲשׁוּעִים', 'subs', 'delight', "sha'ashuim"),
        'E75204': ('מִשְׂפָּח', 'subs', 'deviation', 'mispach'),
        'E75205': ('צִחֶה', 'adjv', 'parched', 'tsicheh'),
        'E75206': ('פער', 'verb', 'open', 'fr'),
        'E75207': ('שָׁאֹון', 'subs', 'roar', 'shaovn'),
        'E75208': ('עָלֵז', 'adjv', 'rejoicing', 'aleiz'),
        'E75209': ('דֹּבֶר', 'subs', 'pasture', 'dover'),
        'E75210': ('מֵחַ', 'subs', 'fatling', 'meiach'),
        'E75211': ('מסך', 'verb', 'mix', "ms'ch"),
        'E75212': ('חֲשַׁשׁ', 'subs', 'dried grass', 'chashash'),
        'E75213': ('סוּחָה', 'subs', 'offal', 'sucha'),
        'E75214': ('נום', 'verb', 'slumber', 'nvm'),
        'E75215': ('צַר', 'subs', '[uncertain]', 'tsar'),
        'E75216': ('גַּלְגַּל', 'subs', 'wheel', 'galgal'),
        'E75217': ('סוּפָה', 'subs', 'storm', 'sufa'),
        'E75218': ('שְׁאָגָה', 'subs', 'roaring', "sh'aga"),
        'E75219': ('נהם', 'verb', 'groan', 'nhm'),
        'E75220': ('נְהָמָה', 'subs', 'growling', "n'hama"),
        'E75221': ('עֲרִיפִים', 'subs', 'drops', 'arifim'),
        'E75222': ('דמה', 'verb', 'be silent', 'dmh'),
        'E75223': ('רִצְפָּה', 'subs', 'coal', 'ritspa'),
        'E75224': ('שׁעע', 'verb', 'paste', 'sh'),
        'E75225': ('שַׁלֶּכֶת', 'subs', 'cutting down', 'shalechet'),
        'E75226': ('נחה', 'verb', 'lean', 'nchh'),
        'E75227': ('שְׁאָר יָשׁוּב', 'nmpr', 'Shear-Jashub', "sh'ar yashuv"),
        'E75228': ('אוּד', 'subs', 'log', 'ud'),
        'E75229': ('קוץ', 'verb', 'be asunder', 'kvts'),
        'E75230': ('טָבְאֵל', 'nmpr', 'Tabeel', 'taveil'),
        'E75231': ('עמק', 'verb', 'be deep', 'mk'),
        'E75232': ('שְׁאָלָה', 'subs', '[uncertain]', "sh'ala"),
        'E75233': ('עִמָּנוּ אֵל', 'nmpr', 'Immanuel', 'imanu eil'),
        'E75234': ('בַּתָּה', 'subs', 'gully', 'bata'),
        'E75235': ('נָקִיק', 'subs', 'cleft', 'nakik'),
        'E75236': ('נַעֲצוּץ', 'subs', 'thorn-bush', "na'atsuts"),
        'E75237': ('נַהֲלֹל', 'subs', 'watering place', 'nahalol'),
        'E75238': ('מַעְדֵּר', 'subs', 'hoe', 'madeir'),
        'E75239': ('מַהֵר שָׁלָל חָשׁ בַּז',
                   'nmpr',
                   'Maher-Shalal-Hash-Baz',
                   'maheir shalal chash baz'),
        'E75240': ('יְבֶרֶכְיָהוּ', 'nmpr', 'Jeberekiah', "y'verechyahu"),
        'E75241': ('שִׁלֹחַ', 'nmpr', 'Shiloah', 'shiloach'),
        'E75242': ('מָשֹׂושׂ', 'subs', 'joy', 'masovs'),
        'E75243': ('מֻטָּה', 'subs', 'spreading', 'muta'),
        'E75244': ('חֶזְקָה', 'subs', 'strength', 'chezka'),
        'E75245': ('תְּעוּדָה', 'subs', 'attestation', "t'uda"),
        'E75246': ('לִמֻּד', 'subs', 'disciple', 'limud'),
        'E75247': ('צפף', 'verb', 'whisper', "tsf'f"),
        'E75248': ('מְעוּף', 'subs', 'darkness', "m'uf"),
        'E75249': ('צוּקָה', 'subs', 'distress', 'tsuka'),
        'E75250': ('מוּעָף', 'subs', 'darkness', 'muaf'),
        'E75251': ('מוּצָק', 'subs', 'constraint', 'mutsak'),
        'E75252': ('צַלְמָוֶת', 'subs', 'darkness', 'tsalmavet'),
        'E75253': ('גיל', 'verb', 'rejoice', 'gyl'),
        'E75254': ('סֹבֶל', 'subs', 'load', 'sovel'),
        'E75255': ('סְאֹון', 'subs', 'boot', "s'ovn"),
        'E75256': ('סאן', 'verb', 'tramp', 'sn'),
        'E75257': ('מַאֲכֹלֶת', 'subs', 'food', "ma'acholet"),
        'E75258': ('מִשְׂרָה', 'subs', 'dominion', 'misra'),
        'E75259': ('מַרְבֶּה', 'subs', 'abundance', 'marbeh'),
        'E75260': ('סוך', 'verb', 'incite', 'svch'),
        'E75261': ('אַגְמֹון', 'subs', 'rush', 'agmovn'),
        'E75262': ('חָנֵף', 'adjv', 'alienated', 'chaneif'),
        'E75263': ('אבך', 'verb', 'bear along', 'vch'),
        'E75264': ('גֵּאוּת', 'subs', 'rise', 'geiut'),
        'E75265': ('עתם', 'verb', '[uncertain]', 'tm'),
        'E75266': ('גזר', 'verb', 'eat', 'gzr'),
        'E75267': ('שֹׁואָה', 'subs', 'trouble', 'shova'),
        'E75268': ('אַסִּיר', 'subs', 'prisoner', 'asir'),
        'E75269': ('זַעַם', 'subs', 'curse', "za'am"),
        'E75270': ('שׁלל', 'verb', 'plunder', "shl'l"),
        'E75271': ('כַּרְכְּמִישׁ', 'nmpr', 'Carchemish', 'karkmish'),
        'E75272': ('כַּלְנֹו', 'nmpr', 'Calno', 'kalnov'),
        'E75273': ('בצע', 'verb', 'cut off', 'vts'),
        'E75274': ('עָתוּד', 'subs', 'ready', 'atud'),
        'E75275': ('שׁשׂה', 'verb', '[uncertain]', "shs'h"),
        'E75276': ('כַּבִּיר', 'adjv', 'great', 'kabir'),
        'E75277': ('מַשֹּׂור', 'subs', 'saw', 'masovr'),
        'E75278': ('מִשְׁמָן', 'subs', 'fatness', 'mishman'),
        'E75279': ('רָזֹון', 'subs', 'emaciation', 'razovn'),
        'E75280': ('יְקֹד', 'subs', 'burning', "y'kod"),
        'E75281': ('נסס', 'verb', 'falter', "ns's"),
        'E75282': ('שְׁאָר', 'subs', 'rest', "sh'ar"),
        'E75283': ('מִזְעָר', 'subs', 'trifle', 'mizar'),
        'E75284': ('תַּבְלִית', 'subs', 'destruction', 'tavlit'),
        'E75285': ('חבל', 'verb', 'be corrupt', 'chvl'),
        'E75286': ('עַיַּת', 'nmpr', 'Aiath', 'ayat'),
        'E75287': ('צהל', 'verb', 'cry', "ts'hl"),
        'E75288': ('לַיְשָׁה', 'nmpr', 'Laishah', 'laysha'),
        'E75289': ('מַדְמֵנָה', 'nmpr', 'Madmenah', 'madmeina'),
        'E75290': ('גֵּבִים', 'nmpr', 'Gebim', 'geivim'),
        'E75291': ('סעף', 'verb', 'cut off', 'sf'),
        'E75292': ('פֻּארָה', 'subs', 'bough', 'pura'),
        'E75293': ('מַעֲרָצָה', 'subs', 'terror', "ma'aratsa"),
        'E75294': ('נקף', 'verb', 'strike', 'nkf'),
        'E75295': ('חֹטֶר', 'subs', 'shoot', 'choter'),
        'E75296': ('גֶּזַע', 'subs', 'stump', 'geza'),
        'E75297': ('נֵצֶר', 'subs', 'shoot', 'neitser'),
        'E75298': ('מִשְׁמָע', 'subs', 'rumour', 'mishma'),
        'E75299': ('נָמֵר', 'subs', 'leopard', 'nameir'),
        'E75300': ('שׁעע', 'verb', 'take delight', 'sh'),
        'E75301': ('חֻר', 'subs', 'hiding-place', 'chur'),
        'E75302': ('מְאוּרָה', 'subs', 'young one', "m'ura"),
        'E75303': ('צִפְעֹנִי', 'subs', 'viper', 'tsifoni'),
        'E75304': ('הדה', 'verb', 'put', 'hdh'),
        'E75305': ('פַּתְרֹוס', 'nmpr', 'Upper Egypt', 'patrovs'),
        'E75306': ('מִשְׁלֹוחַ', 'subs', 'sending', 'mishlovach'),
        'E75307': ('בַּעְיָם', 'subs', '[uncertain]', 'bayam'),
        'E75308': ('שָׂשֹׂון', 'subs', 'rejoicing', 'sasovn'),
        'E75309': ('שׁפה', 'verb', 'sweep bare', 'shfh'),
        'E75310': ('עַלִּיז', 'adjv', 'rejoicing', 'aliz'),
        'E75311': ('ילל', 'verb', 'howl', "yl'l"),
        'E75312': ('שֹׁד', 'subs', 'violence', 'shod'),
        'E75313': ('חֵבֶל', 'subs', 'labour pains', 'cheivel'),
        'E75314': ('אַכְזָרִי', 'adjv', 'cruel', 'achzari'),
        'E75315': ('כְּסִיל', 'subs', 'orion', "k'sil"),
        'E75316': ('הלל', 'verb', 'light', "hl'l"),
        'E75317': ('זֵד', 'adjv', 'insolent', 'zeid'),
        'E75318': ('עָרִיץ', 'adjv', 'ruthless', 'arits'),
        'E75319': ('פַּז', 'subs', 'topaz', 'paz'),
        'E75320': ('כֶּתֶם', 'subs', 'gold', 'ketem'),
        'E75321': ('עֲרָבִי', 'adjv', 'Arab', 'aravi'),
        'E75322': ('צִי', 'subs', 'demon', 'tsi'),
        'E75323': ('אֹחַ', 'subs', 'eagle owl', 'oach'),
        'E75324': ('רקד', 'verb', 'skip', 'rkd'),
        'E75325': ('אִי', 'subs', 'islander', 'i'),
        'E75326': ('תַּן', 'subs', 'jackal', 'tan'),
        'E75327': ('עֹנֶג', 'subs', 'delight', 'oneg'),
        'E75328': ('עֹצֶב', 'subs', 'pain', 'otsev'),
        'E75329': ('רֹגֶז', 'subs', 'excitement', 'rogez'),
        'E75330': ('מַדְהֵבָה', 'subs', '[uncertain]', 'madheiva'),
        'E75331': ('סָרָה', 'subs', 'ceasing', 'sara'),
        'E75332': ('מֻרְדָּף', 'subs', '[uncertain]', 'murdaf'),
        'E75333': ('פצח', 'verb', 'be serene', 'ftsch'),
        'E75334': ('רְפָאִים', 'subs', 'ghosts', "r'faim"),
        'E75335': ('הֶמְיָה', 'subs', 'sound', 'hemya'),
        'E75336': ('יצע', 'verb', 'spread out', 'yts'),
        'E75337': ('תֹּולֵעָה', 'subs', 'maggot', 'tovleia'),
        'E75338': ('הֵילֵל', 'subs', 'morning-star', 'heileil'),
        'E75339': ('שׁגח', 'verb', 'gaze', 'shgch'),
        'E75340': ('טען', 'verb', 'pierce', 'tn'),
        'E75341': ('בוס', 'verb', 'tread down', 'vvs'),
        'E75342': ('מַטְבֵּחַ', 'subs', 'slaughtering place', 'matbeiach'),
        'E75343': ('בַּל', 'nega', 'not', 'bal'),
        'E75344': ('מֹורָשׁ', 'subs', 'possession', 'movrash'),
        'E75345': ('קִפֹּד', 'subs', 'hedgehog', 'kipod'),
        'E75346': ('טאטא', 'verb', 'sweep', "t't"),
        'E75347': ('מַטְאֲטֵא', 'subs', 'broom', 'matatei'),
        'E75348': ('צֶפַע', 'subs', 'serpent', 'tsefa'),
        'E75349': ('בדד', 'verb', 'be alone', "vd'd"),
        'E75350': ('מֹועָד', 'subs', 'place of assemblage', 'movad'),
        'E75351': ('קִיר מֹואָב', 'nmpr', 'Kir in Moab', 'kir movav'),
        'E75352': ('יַהַץ', 'nmpr', 'Jahaz', 'yahats'),
        'E75353': ('ירע', 'verb', 'quiver', 'yr'),
        'E75354': ('עֶגְלַת שְׁלִשִׁיָּה',
                   'nmpr',
                   'Eglath Shelishiyah',
                   "eglat sh'lishia"),
        'E75355': ('לוּחִית', 'nmpr', 'Luhith', 'luchit'),
        'E75356': ('חֹרֹונָיִם', 'nmpr', 'Horonaim', 'chorovnayim'),
        'E75357': ('נִמְרִים', 'nmpr', 'Nimrim', 'nimrim'),
        'E75358': ('מְשַׁמָּה', 'subs', 'devastation', "m'shama"),
        'E75359': ('יִתְרָה', 'subs', 'remainder', 'yitra'),
        'E75360': ('אֶגְלַיִם', 'nmpr', 'Eglaim', 'eglayim'),
        'E75361': ('יְלָלָה', 'subs', 'howling', "y'lala"),
        'E75362': ('בְּאֵר אֵילִים', 'nmpr', 'Beer Elim', "b'eir eilim"),
        'E75363': ('דִּימֹון', 'nmpr', 'Dimon', 'dimovn'),
        'E75364': ('פְּלִילָה', 'subs', 'decision', "p'lila"),
        'E75365': ('מֵץ', 'subs', '[uncertain]', 'meits'),
        'E75366': ('מָהִיר', 'adjv', 'quick', 'mahir'),
        'E75367': ('גֵּא', 'adjv', 'haughty', 'gei'),
        'E75368': ('בַּד', 'subs', 'idle talk', 'bad'),
        'E75369': ('נָכָא', 'adjv', 'smitten', 'nacho'),
        'E75370': ('שָׂרֹק', 'adjv', 'bright red', 'sarok'),
        'E75371': ('שְׁלֻחֹות', 'subs', 'shoots', "sh'luchovt"),
        'E75372': ('רוה', 'verb', 'drink', 'rvh'),
        'E75373': ('הֵידָד', 'subs', 'shouting', 'heidad'),
        'E75374': ('גִּיל', 'subs', 'rejoicing', 'gil'),
        'E75375': ('קִיר חֶרֶשׂ', 'nmpr', 'Kir Hareseth', 'kir cheres'),
        'E75376': ('מְעִי', 'subs', '[uncertain]', "m'i"),
        'E75377': ('מַפָּלָה', 'subs', 'decay', 'mapala'),
        'E75378': ('רזה', 'verb', 'diminish', 'rzh'),
        'E75379': ('נֹקֶף', 'subs', 'strike', 'nokef'),
        'E75380': ('גַּרְגַּר', 'subs', 'olive', 'gargar'),
        'E75381': ('אָמִיר', 'subs', 'branch', 'amir'),
        'E75382': ('סָעִיף', 'subs', 'bough', 'saif'),
        'E75383': ('חֹרֶשׁ', 'subs', 'wood', 'choresh'),
        'E75384': ('נַעֲמָנִים', 'subs', 'pleasant places', "na'amanim"),
        'E75385': ('שׂוג', 'verb', '[uncertain]', 'svg'),
        'E75386': ('כְּאֵב', 'subs', 'pain', "k'eiv"),
        'E75387': ('אָנוּשׁ', 'adjv', 'incurable', 'anush'),
        'E75388': ('שׁאה', 'verb', 'roar', 'shh'),
        'E75389': ('מֹץ', 'subs', 'chaff', 'mots'),
        'E75390': ('גַּלְגַּל', 'subs', 'wheel', 'galgal'),
        'E75391': ('בַּלָּהָה', 'subs', 'terror', 'balaha'),
        'E75392': ('צִיר', 'subs', 'messenger', 'tsir'),
        'E75393': ('קַו', 'subs', '[uncertain]', 'kav'),
        'E75394': ('מְבוּסָה', 'subs', 'subjugation', "m'vusa"),
        'E75395': ('בזא', 'verb', 'wash away', 'vz'),
        'E75396': ('צַח', 'adjv', 'clear', 'tsach'),
        'E75397': ('בֹּסֶר', 'subs', 'grape', 'boser'),
        'E75398': ('נִצָּה', 'subs', 'blossom', 'nitsa'),
        'E75399': ('זַלְזַלִּים', 'subs', 'sprigs', 'zalzalim'),
        'E75400': ('נְטִישֹׁות', 'subs', 'shoots', "n'tishovt"),
        'E75401': ('תזז', 'verb', 'strike', "tz'z"),
        'E75402': ('יבל', 'verb', 'bring', 'yvl'),
        'E75403': ('שַׁי', 'subs', 'gift', 'shay'),
        'E75404': ('בקק', 'verb', 'lay waste', "vk'k"),
        'E75405': ('אִטִּים', 'subs', 'charmers', 'itim'),
        'E75406': ('נשׁת', 'verb', 'dry', 'nsht'),
        'E75407': ('זנח', 'verb', 'stink', 'znch'),
        'E75408': ('קמל', 'verb', 'be lousy', 'kml'),
        'E75409': ('עָרָה', 'subs', 'rush', 'ara'),
        'E75410': ('מִזְרָע', 'subs', 'seed-land', 'mizra'),
        'E75411': ('דַּיָּג', 'subs', 'fisher', 'dayag'),
        'E75412': ('חַכָּה', 'subs', 'fish-hook', 'chaka'),
        'E75413': ('מִכְמֶרֶת', 'subs', 'net', 'michmeret'),
        'E75414': ('שָׂרִיק', 'adjv', 'carded', 'sarik'),
        'E75415': ('חֹורָי', 'subs', '[uncertain]', 'chovray'),
        'E75416': ('שָׁתֹת', 'subs', '[uncertain]', 'shatot'),
        'E75417': ('שֶׂכֶר', 'subs', 'hire', 'secher'),
        'E75418': ('אָגֵם', 'adjv', 'grieved', 'ageim'),
        'E75419': ('אֱוִיל', 'adjv', 'foolish', 'evil'),
        'E75420': ('בער', 'verb', 'be stupid', 'vr'),
        'E75421': ('נֹף', 'nmpr', 'Memphis', 'nof'),
        'E75422': ('עִוְעִים', 'subs', 'wavering', 'ivim'),
        'E75423': ('קִיא', 'subs', 'vomit', 'ki'),
        'E75424': ('חָגָּא', 'subs', 'shame', 'chaga'),
        'E75425': ('הֶרֶס', 'subs', 'destruction', 'heres'),
        'E75426': ('סַרְגֹון', 'nmpr', 'Sargon', 'sargovn'),
        'E75427': ('חֲשׂוּפַי', 'subs', '[uncertain]', 'chasufay'),
        'E75428': ('מַבָּט', 'subs', 'hope', 'mabat'),
        'E75429': ('חָזוּת', 'subs', 'vision', 'chazut'),
        'E75430': ('אֲנָחָה', 'subs', 'sigh', 'anacha'),
        'E75431': ('חַלְחָלָה', 'subs', 'anguish', 'chalchala'),
        'E75432': ('פַּלָּצוּת', 'subs', 'shuddering', 'palatsut'),
        'E75433': ('צָפִית', 'subs', 'seat order', 'tsafit'),
        'E75434': ('מְדֻשָׁה', 'subs', 'trample', "m'dusha"),
        'E75435': ('בעה', 'verb', 'inquire', 'vh'),
        'E75436': ('עֲרַב', 'nmpr', 'Arabs', 'arav'),
        'E75437': ('דְּדָנִי', 'adjv', 'Dedanites', "d'dani"),
        'E75438': ('כֹּבֶד', 'subs', 'heaviness', 'koved'),
        'E75439': ('תְּשֻׁאָה', 'subs', 'shouting', "t'shua"),
        'E75440': ('מְבוּכָה', 'subs', 'confusion', "m'vucho"),
        'E75441': ('קרר', 'verb', 'be cool', "kr'r"),
        'E75442': ('שֹׁועַ', 'nmpr', 'Shoa', 'shova'),
        'E75443': ('אַשְׁפָּה', 'subs', 'quiver', 'ashpa'),
        'E75444': ('בָּקִיעַ', 'subs', 'breach', 'bakia'),
        'E75445': ('מִקְוָה', 'subs', 'reservoir', 'mikva'),
        'E75446': ('טַלְטֵלָה', 'subs', 'hurling', 'talteila'),
        'E75447': ('עטה', 'verb', 'grasp', 'th'),
        'E75448': ('צְנֵפָה', 'subs', 'winding', "ts'neifa"),
        'E75449': ('דּוּר', 'subs', 'circle', 'dur'),
        'E75450': ('קָלֹון', 'subs', 'dishonour', 'kalovn'),
        'E75451': ('צֶאֱצָאִים', 'subs', 'offspring', "tse'etsaim"),
        'E75452': ('צְפִיעָה', 'subs', 'leaf', "ts'fia"),
        'E75453': ('סַחַר', 'subs', 'gain', 'sachar'),
        'E75454': ('קַדְמָה', 'subs', 'beginning', 'kadma'),
        'E75455': ('כִּנְעָנִי', 'subs', 'tradesman', 'kinani'),
        'E75456': ('מֵזַח', 'subs', 'shipyard', 'meizach'),
        'E75457': ('מָעֹזֶן', 'subs', '[uncertain]', 'maozen'),
        'E75458': ('בַּחוּן', 'subs', '[uncertain]', 'bachun'),
        'E75459': ('ערר', 'verb', 'strip', "r'r"),
        'E75460': ('חסן', 'verb', 'store', 'chsn'),
        'E75461': ('שִׂבְעָה', 'subs', 'satiety', 'siva'),
        'E75462': ('עָתִיק', 'adjv', 'choice', 'atik'),
        'E75463': ('בלק', 'verb', 'waste', 'vlk'),
        'E75464': ('חרה', 'verb', 'diminish', 'chrh'),
        'E75465': ('צְוָחָה', 'subs', 'cry', "ts'vacha"),
        'E75466': ('שְׁאִיָּה', 'subs', 'desolation', "sh'ia"),
        'E75467': ('אוּר', 'subs', 'light', 'ur'),
        'E75468': ('רָזִי', 'subs', '[uncertain]', 'razi'),
        'E75469': ('בֶּגֶד', 'subs', 'treachery', 'beged'),
        'E75470': ('רעע', 'verb', 'break', 'r'),
        'E75471': ('פרר', 'verb', 'stir', "fr'r"),
        'E75472': ('אֲסֵפָה', 'subs', '[uncertain]', 'aseifa'),
        'E75473': ('מַסְגֵּר', 'subs', 'dungeon', 'masgeir'),
        'E75474': ('לְבָנָה', 'subs', 'full moon', "l'vana"),
        'E75475': ('חַמָּה', 'subs', 'heat', 'chama'),
        'E75476': ('אֹמֶן', 'subs', 'faithfulness', 'omen'),
        'E75477': ('צָיֹון', 'subs', 'waterless country', 'tsayovn'),
        'E75478': ('שֶׁמֶר', 'subs', 'dregs', 'shemer'),
        'E75479': ('מחה', 'verb', 'fill with marrow', 'mchh'),
        'E75480': ('זקק', 'verb', 'filter', "zk'k"),
        'E75481': ('מַסֵּכָה', 'subs', 'covering', 'maseicho'),
        'E75482': ('נסך', 'verb', 'interweave', "ns'ch"),
        'E75483': ('מַתְבֵּן', 'subs', 'straw-heap', 'matbein'),
        'E75484': ('בְּמֹו', 'prep', 'in', "b'mov"),
        'E75485': ('מַדְמֵנָה', 'subs', 'dung-place', 'madmeina'),
        'E75486': ('שׂחה', 'verb', 'swim', 'schh'),
        'E75487': ('אָרְבָּה', 'subs', '[uncertain]', 'arba'),
        'E75488': ('בָּטוּחַ', 'subs', 'confident', 'batuach'),
        'E75489': ('מֵישָׁרִים', 'subs', 'uprightness', 'meisharim'),
        'E75490': ('מַעְגָּל', 'subs', 'course', 'magal'),
        'E75491': ('פלס', 'verb', 'level', 'fls'),
        'E75492': ('שׁחר', 'verb', 'look for', 'shchr'),
        'E75493': ('עול', 'verb', 'do wrong', 'vl'),
        'E75494': ('קָצוּ', 'subs', 'end', 'katsu'),
        'E75495': ('לִוְיָתָן', 'subs', 'leviathan', 'livatan'),
        'E75496': ('בָּרִחַ', 'adjv', 'fleeing', 'bariach'),
        'E75497': ('עֲקַלָּתֹון', 'adjv', 'crooked', 'akalatovn'),
        'E75498': ('חֶמֶד', 'subs', 'splendour', 'chemed'),
        'E75499': ('פשׂע', 'verb', 'step', 'fs'),
        'E75500': ('צות', 'verb', 'kindle', 'tsvt'),
        'E75501': ('שׁרשׁ', 'verb', 'root', 'shrsh'),
        'E75502': ('הֶרֶג', 'subs', 'slaughter', 'hereg'),
        'E75503': ('סאסא', 'verb', 'cry', "s's"),
        'E75504': ('הגה', 'verb', 'expel', 'hgh'),
        'E75505': ('גִּר', 'subs', 'chalk', 'gir'),
        'E75506': ('קָצִיר', 'subs', 'bough', 'katsir'),
        'E75507': ('שִׁבֹּלֶת', 'subs', 'stream', 'shibolet'),
        'E75508': ('שַׂעַר', 'subs', 'storm', "sa'ar"),
        'E75509': ('צִיצָה', 'subs', 'flower', 'tsitsa'),
        'E75510': ('בִּכּוּרָה', 'subs', 'early fig', 'bikura'),
        'E75511': ('צְפִירָה', 'subs', '[uncertain]', "ts'fira"),
        'E75512': ('תִּפְאָרָה', 'subs', 'splendour', 'tifara'),
        'E75513': ('רֹאֶה', 'subs', 'vision', 'roeh'),
        'E75514': ('פוק', 'verb', 'totter', 'fvk'),
        'E75515': ('פְּלִילִיָּה', 'subs', 'decision', "p'lilia"),
        'E75516': ('עַתִּיק', 'adjv', 'removed', 'atik'),
        'E75517': ('צַו', 'subs', '[untranslatable]', 'tsav'),
        'E75518': ('קַו', 'subs', '[uncertain]', 'kav'),
        'E75519': ('זְעֵיר', 'subs', 'little', "z'eir"),
        'E75520': ('לָעֵג', 'subs', 'stammering', 'laeig'),
        'E75521': ('מַרְגֵּעָה', 'subs', 'resting place', 'margeia'),
        'E75522': ('לָצֹון', 'subs', 'boasting', 'latsovn'),
        'E75523': ('שֹׁוט', 'subs', 'spate', 'shovt'),
        'E75524': ('בֹּחַן', 'subs', '[type of stone]', 'bochan'),
        'E75525': ('יִקְרָה', 'subs', 'meeting', 'yikra'),
        'E75526': ('מוּסָד', 'subs', 'foundation', 'musad'),
        'E75527': ('יעה', 'verb', 'sweep', 'yh'),
        'E75528': ('זְוָעָה', 'subs', 'trembling', "z'va'a"),
        'E75529': ('מַצָּע', 'subs', 'couch', 'matsa'),
        'E75530': ('כנס', 'verb', 'gather', 'chns'),
        'E75531': ('פְּרָצִים', 'nmpr', 'Perazim', "p'ratsim"),
        'E75532': ('מֹוסֵר', 'subs', 'band', 'movseir'),
        'E75533': ('שׂדד', 'verb', 'draw', "sd'd"),
        'E75534': ('קֶצַח', 'subs', 'black cumin', 'ketsach'),
        'E75535': ('כַּמֹּן', 'subs', 'cumin', 'kamon'),
        'E75536': ('שֹׂורָה', 'subs', '[animal]', 'sovra'),
        'E75537': ('סמן', 'verb', '[uncertain]', 'smn'),
        'E75538': ('חָרוּץ', 'subs', 'threshing instrument', 'charuts'),
        'E75539': ('אדשׁ', 'verb', 'thresh?', 'dsh'),
        'E75540': ('גִּלְגָּל', 'subs', 'wheel', 'gilgal'),
        'E75541': ('תּוּשִׁיָּה', 'subs', 'effect', 'tushia'),
        'E75542': ('אֲרִיאֵל', 'nmpr', '[part of Jerusalem]', 'arieil'),
        'E75543': ('תַּאֲנִיָּה', 'subs', 'mourning', "ta'ania"),
        'E75544': ('אֲנִיָּה', 'subs', 'mourning', 'ania'),
        'E75545': ('מְצוּרָה', 'subs', 'fortification', "m'tsura"),
        'E75546': ('רַעַם', 'subs', 'thunder', "ra'am"),
        'E75547': ('צֹבֶה', 'subs', '[uncertain]', 'tsoveh'),
        'E75548': ('מְצֹודָה', 'subs', 'net', "m'tsovda"),
        'E75549': ('שֹׁוקֵק', 'adjv', 'narrow', 'shovkeik'),
        'E75550': ('עצם', 'verb', 'close', 'tsm'),
        'E75551': ('מַחְשָׁךְ', 'subs', 'dark place', 'machshach'),
        'E75552': ('הֵפֶךְ', 'subs', 'opposite', 'heifech'),
        'E75553': ('אֹפֶל', 'subs', 'darkness', 'ofel'),
        'E75554': ('לֵץ', 'adjv', 'scorner', 'leits'),
        'E75555': ('שׁקד', 'verb', 'be wakeful', 'shkd'),
        'E75556': ('קושׁ', 'verb', 'lay bait', 'kvsh'),
        'E75557': ('חור', 'verb', 'grow pale', 'chvr'),
        'E75558': ('מַסֵּכָה', 'subs', 'alliance', 'maseicho'),
        'E75559': ('חָסוּת', 'subs', 'refuge', 'chasut'),
        'E75560': ('כְּלִמָּה', 'subs', 'insult', "k'lima"),
        'E75561': ('חָנֵס', 'nmpr', 'Hanes', 'chaneis'),
        'E75562': ('לַיִשׁ', 'subs', 'lion', 'layish'),
        'E75563': ('אֶפְעֶה', 'subs', 'snake', 'efeh'),
        'E75564': ('דַּבֶּשֶׁת', 'subs', 'hump', 'dabeshet'),
        'E75565': ('רַהַב', 'nmpr', '[Egypt]', 'rahav'),
        'E75566': ('כֶּחָשׁ', 'adjv', 'mendacious', 'kechash'),
        'E75567': ('מַהֲתַלֹּות', 'subs', 'deceit', 'mahatalovt'),
        'E75568': ('לוז', 'verb', 'turn aside', 'lvz'),
        'E75569': ('מְכִתָּה', 'subs', 'crushing', "m'chita"),
        'E75570': ('חתה', 'verb', 'rake together', 'chth'),
        'E75571': ('יָקוּד', 'subs', 'fire-place', 'yakud'),
        'E75572': ('חשׂף', 'verb', 'strip', 'chsf'),
        'E75573': ('גֶּבֶא', 'subs', 'ditch', 'geve'),
        'E75574': ('שׁוּבָה', 'subs', 'coming back', 'shuva'),
        'E75575': ('נַחַת', 'subs', 'rest', 'nachat'),
        'E75576': ('בִּטְחָה', 'subs', 'confidence', "bit'cha"),
        'E75577': ('תֹּרֶן', 'subs', 'mast', 'toren'),
        'E75578': ('כנף', 'verb', 'hide', 'chnf'),
        'E75579': ('מֹורֶה', 'subs', 'teacher', 'movreh'),
        'E75580': ('צֵא', 'subs', 'dirt', 'tsei'),
        'E75581': ('דָּשֵׁן', 'adjv', 'fat', 'dashein'),
        'E75582': ('כַּר', 'subs', 'pasture', 'kar'),
        'E75583': ('בְּלִיל', 'subs', 'mash', "b'lil"),
        'E75584': ('חָמִיץ', 'subs', 'sorrelfodder', 'chamits'),
        'E75585': ('רַחַת', 'subs', 'winnowing shovel', 'rachat'),
        'E75586': ('מִזְרֶה', 'subs', 'pitch-fork', 'mizreh'),
        'E75587': ('פֶּלֶג', 'subs', 'division', 'peleg'),
        'E75588': ('יָבָל', 'subs', 'water-ditch', 'yaval'),
        'E75589': ('מַחַץ', 'subs', 'breaking', 'machats'),
        'E75590': ('מַשָּׂאָה', 'subs', 'lifting up', "masa'a"),
        'E75591': ('הֲנָפָה', 'subs', 'swing', 'hanafa'),
        'E75592': ('רֶסֶן', 'subs', 'bridle', 'resen'),
        'E75593': ('נַחַת', 'subs', 'descent', 'nachat'),
        'E75594': ('זַעַף', 'subs', 'rage', "za'af"),
        'E75595': ('נֶפֶץ', 'subs', 'pattering', 'nefets'),
        'E75596': ('מוּסָדָה', 'subs', 'foundation', 'musada'),
        'E75597': ('תָּפְתֶּה', 'subs', '[uncertain]', 'tafteh'),
        'E75598': ('מְדוּרָה', 'subs', 'pile', "m'dura"),
        'E75599': ('מָגֹור', 'subs', 'horror', 'magovr'),
        'E75600': ('מַחֲבֵא', 'subs', 'hiding-place', 'machavei'),
        'E75601': ('עִלֵּג', 'adjv', 'stammerer', 'ileig'),
        'E75602': ('כִּילַי', 'subs', '[uncertain]', 'kilay'),
        'E75603': ('שֹׁועַ', 'subs', 'noble', 'shova'),
        'E75604': ('חֹנֶף', 'subs', 'alienation', 'chonef'),
        'E75605': ('תֹּועָה', 'subs', 'confusion', 'tova'),
        'E75606': ('נְדִיבָה', 'subs', 'willingness', "n'diva"),
        'E75607': ('אֹסֶף', 'subs', 'gathering', 'osef'),
        'E75608': ('בַּחַן', 'subs', 'watch-tower', 'bachan'),
        'E75609': ('מְעָרָה', 'subs', 'barren field', "m'ara"),
        'E75610': ('מִבְטָח', 'subs', 'trust', 'mivtach'),
        'E75611': ('ברד', 'verb', 'hail', 'vrd'),
        'E75612': ('שִׁפְלָה', 'subs', 'low state', 'shifla'),
        'E75613': ('נלה', 'verb', '[uncertain]', 'nlh'),
        'E75614': ('רֹומֵמֻת', 'subs', 'arising', 'rovmeimut'),
        'E75615': ('מַשָּׁק', 'subs', 'rushing', 'mashak'),
        'E75616': ('גֵּבָה', 'subs', 'swarm', 'geiva'),
        'E75617': ('שׁקק', 'verb', 'rush', "shk'k"),
        'E75618': ('חֹסֶן', 'subs', 'store', 'chosen'),
        'E75619': ('אַרְאֵל', 'subs', '[uncertain]', 'areil'),
        'E75620': ('מִשְׂרְפֹות', 'subs', 'burning', 'misrfovt'),
        'E75621': ('כסח', 'verb', 'cut off', "chs'ch"),
        'E75622': ('רְעָדָה', 'subs', 'trembling', "r'ada"),
        'E75623': ('מֹוקֵד', 'subs', 'hearth', 'movkeid'),
        'E75624': ('מַעֲשַׁקֹּות', 'subs', 'extortions', "ma'ashakovt"),
        'E75625': ('יעז', 'verb', 'be insolent', 'yz'),
        'E75626': ('עָמֵק', 'adjv', 'unintelligible', 'ameik'),
        'E75627': ('צען', 'verb', 'pack up', 'tsn'),
        'E75628': ('שַׁיִט', 'subs', 'oar', 'shayit'),
        'E75629': ('בְּאֹשׁ', 'subs', 'stench', "b'osh"),
        'E75630': ('שִׁלּוּם', 'subs', 'reward', 'shilum'),
        'E75631': ('סִירָה', 'subs', 'hook', 'sira'),
        'E75632': ('קִמֹּושׂ', 'subs', 'weed', 'kimovs'),
        'E75633': ('לִילִית', 'subs', 'night-demon', 'lilit'),
        'E75634': ('קנן', 'verb', 'make nest', "kn'n"),
        'E75635': ('קִפֹּוז', 'subs', '[animal]', 'kipovz'),
        'E75636': ('דגר', 'verb', '[uncertain]', 'dgr'),
        'E75637': ('צִיָּה', 'subs', 'dry country', 'tsia'),
        'E75638': ('חֲבַצֶּלֶת', 'subs', 'asphodel', 'chavatselet'),
        'E75639': ('גִּילָה', 'subs', 'rejoicing', 'gila'),
        'E75640': ('שָׁרָב', 'subs', 'heat', 'sharav'),
        'E75641': ('מַבּוּעַ', 'subs', 'spring', 'mabua'),
        'E75642': ('רֵבֶץ', 'subs', 'resting place', 'reivets'),
        'E75643': ('חָצִיר', 'subs', 'reed', 'chatsir'),
        'E75644': ('מַסְלוּל', 'subs', 'highway', 'maslul'),
        'E75645': ('פָּרִיץ', 'subs', 'burglar', 'parits'),
        'E75646': ('שָׁחִיס', 'subs', '[uncertain]', 'shachis'),
        'E75647': ('דְּמִי', 'subs', 'likeness', "d'mi"),
        'E75648': ('חֶדֶל', 'subs', '[uncertain]', 'chedel'),
        'E75649': ('דֹּור', 'subs', 'tent camp', 'dovr'),
        'E75650': ('קפד', 'verb', 'roll up', 'kfd'),
        'E75651': ('דַּלָּה', 'subs', 'thread', 'dala'),
        'E75652': ('עָגוּר', 'subs', 'bulbul-bird', 'agur'),
        'E75653': ('עָשְׁקָה', 'subs', 'oppression', 'ashka'),
        'E75654': ('דדה', 'verb', '[uncertain]', "d'dh"),
        'E75655': ('שַׁחַת', 'subs', 'pit', 'shachat'),
        'E75656': ('גֵּו', 'subs', 'back', 'geiv'),
        'E75657': ('שׂבר', 'verb', 'examine', 'svr'),
        'E75658': ('נְגִינָה', 'subs', 'music', "n'gina"),
        'E75659': ('מרח', 'verb', 'rub', 'mrch'),
        'E75660': ('מְרֹדַךְ בַּלְאֲדָן',
                   'nmpr',
                   'Merodach-Baladan',
                   "m'rodach baladan"),
        'E75661': ('כֶּפֶל', 'subs', 'double', 'kefel'),
        'E75662': ('עָקֹב', 'subs', 'something knobby', 'akov'),
        'E75663': ('רֶכֶס', 'subs', 'rugged ground', 'reches'),
        'E75664': ('טְלִי', 'subs', 'lamb', "t'li"),
        'E75665': ('שָׁלִישׁ', 'subs', 'third part', 'shalish'),
        'E75666': ('פֶּלֶס', 'subs', 'indicator', 'peles'),
        'E75667': ('מַר', 'subs', 'drop', 'mar'),
        'E75668': ('רְתֻקֹות', 'subs', 'chains', "r'tukovt"),
        'E75669': ('סכן', 'verb', 'be poor?', "s'chn"),
        'E75670': ('רקב', 'verb', 'rot', 'rkv'),
        'E75671': ('חוּג', 'subs', 'circle', 'chug'),
        'E75672': ('דֹּק', 'subs', 'something thin', 'dok'),
        'E75673': ('מתח', 'verb', 'spread out', "mt'ch"),
        'E75674': ('יעף', 'verb', 'be weary', 'yf'),
        'E75675': ('עָצְמָה', 'subs', 'might', 'atsma'),
        'E75676': ('קֹוי', 'subs', '[uncertain]', 'kovy'),
        'E75677': ('אֵבֶר', 'subs', 'wing', 'eiver'),
        'E75678': ('חלק', 'verb', 'be smooth', 'chlk'),
        'E75679': ('פַּטִּישׁ', 'subs', 'hammer', 'patish'),
        'E75680': ('מַסְמֵר', 'subs', 'nail', 'masmeir'),
        'E75681': ('אָצִיל', 'subs', 'remote part', 'atsil'),
        'E75682': ('מַצֻּת', 'subs', 'strife', 'matsut'),
        'E75683': ('פִּיפִיֹּות', 'adjv', 'double-edged', 'pipiovt'),
        'E75684': ('הֲדַס', 'subs', 'myrtle', 'hadas'),
        'E75685': ('תִּדְהָר', 'subs', 'box tree', 'tidhar'),
        'E75686': ('תְּאַשּׁוּר', 'subs', 'cypress', "t'ashur"),
        'E75687': ('עֲצֻמֹות', 'subs', 'arguments', 'atsumovt'),
        'E75688': ('אָפַע', 'subs', '[uncertain]', 'afa'),
        'E75689': ('סָגָן', 'subs', 'prefect', 'sagan'),
        'E75690': ('נֵסֶךְ', 'subs', 'molten image', 'neisech'),
        'E75691': ('צוח', 'verb', 'cry', 'tsvch'),
        'E75692': ('צרח', 'verb', 'cry', 'tsrch'),
        'E75693': ('פעה', 'verb', 'groan', 'fh'),
        'E75694': ('נשׁם', 'verb', 'pant', 'nshm'),
        'E75695': ('שׁאף', 'verb', 'gasp', 'shf'),
        'E75696': ('מַעֲקַשִּׁים', 'subs', 'rugged country', "ma'akashim"),
        'E75697': ('פחח', 'verb', 'be trapped', "fch'ch"),
        'E75698': ('עֱזוּז', 'subs', 'power', 'ezuz'),
        'E75699': ('כוה', 'verb', 'burn', 'chvh'),
        'E75700': ('מַעֲרָב', 'subs', 'sunset', "ma'arav"),
        'E75701': ('עִזּוּז', 'adjv', 'powerful', 'izuz'),
        'E75702': ('דעך', 'verb', 'be extinguished', 'dch'),
        'E75703': ('גִּדּוּף', 'subs', 'reviling word', 'giduf'),
        'E75704': ('כנה', 'verb', 'honour', 'chnh'),
        'E75705': ('ירה', 'verb', 'be stupefied', 'yrh'),
        'E75706': ('מַעֲצָד', 'subs', 'billhook', "ma'atsad"),
        'E75707': ('פֶּחָם', 'subs', 'charcoal', 'pecham'),
        'E75708': ('שֶׂרֶד', 'subs', 'red chalk', 'sered'),
        'E75709': ('מַקְצֻעָה', 'subs', 'wood-scraper', 'maktsua'),
        'E75710': ('מְחוּגָה', 'subs', '[uncertain]', "m'chuga"),
        'E75711': ('תִּרְזָה', 'subs', 'tree', 'tirza'),
        'E75712': ('אֹרֶן', 'subs', 'laurel', 'oren'),
        'E75713': ('נשׂק', 'verb', 'kindle', 'nsk'),
        'E75714': ('סגד', 'verb', 'prostrate', 'sgd'),
        'E75715': ('הֶאָח', 'intj', 'aha', 'heach'),
        'E75716': ('טחח', 'verb', 'be besmeared', "t'ch'ch"),
        'E75717': ('בּוּל', 'subs', 'dry wood', 'bul'),
        'E75718': ('צוּלָה', 'subs', 'deep', 'tsula'),
        'E75719': ('כֹּורֶשׁ', 'nmpr', 'Cyrus', 'kovresh'),
        'E75720': ('הֲדוּרִים', 'subs', '[uncertain]', 'hadurim'),
        'E75721': ('מִסְתָּר', 'subs', 'hiding-place', 'mistar'),
        'E75722': ('מַעֲרָבָה', 'subs', '[uncertain]', "ma'arava"),
        'E75723': ('רעף', 'verb', 'trickle', 'rf'),
        'E75724': ('סְבָאִים', 'nmpr', 'Sabeans', "s'vaim"),
        'E75725': ('זִקִּים', 'subs', 'fetters', 'zikim'),
        'E75726': ('צִיר', 'subs', '[uncertain]', 'tsir'),
        'E75727': ('בֵּל', 'nmpr', 'Bel', 'beil'),
        'E75728': ('קרס', 'verb', 'bend down', 'krs'),
        'E75729': ('נְבֹו', 'nmpr', 'Nabu', "n'vov"),
        'E75730': ('נְשׂוּאָה', 'subs', 'load', "n'sua"),
        'E75731': ('זול', 'verb', 'lavish', 'zvl'),
        'E75732': ('אשׁשׁ', 'verb', '[uncertain]', "sh'sh"),
        'E75733': ('צַמָּה', 'subs', 'veil', 'tsama'),
        'E75734': ('שֹׁבֶל', 'subs', 'train', 'shovel'),
        'E75735': ('דּוּמָם', 'advb', 'silently', 'dumam'),
        'E75736': ('עָדִין', 'adjv', 'voluptuous', 'adin'),
        'E75737': ('שְׁכֹול', 'subs', 'loss of children', "sh'chovl"),
        'E75738': ('אַלְמֹן', 'subs', 'widowhood', 'almon'),
        'E75739': ('הֹוָה', 'subs', 'disaster', 'hova'),
        'E75740': ('הבר', 'verb', 'divide', 'hvr'),
        'E75741': ('עֹצֶב', 'subs', 'idol', 'otsev'),
        'E75742': ('חטם', 'verb', 'restrain', 'chtm'),
        'E75743': ('טפח', 'verb', 'spread', 'tfch'),
        'E75744': ('גַּל', 'subs', 'wave', 'gal'),
        'E75745': ('מָעָה', 'subs', 'grain', "ma'a"),
        'E75746': ('חַד', 'adjv', 'sharp', 'chad'),
        'E75747': ('מַרְעִית', 'subs', 'pasturage', 'marit'),
        'E75748': ('סִינִים', 'nmpr', 'Aswan', 'sinim'),
        'E75749': ('עוּל', 'subs', 'suckling', 'ul'),
        'E75750': ('הֲרִיסֻת', 'subs', 'ruin', 'harisut'),
        'E75751': ('שִׁכֻּלִים', 'subs', 'childlessness', 'shikulim'),
        'E75752': ('שָׁכוּל', 'adjv', 'bereaved of children', 'shachul'),
        'E75753': ('גַּלְמוּד', 'adjv', 'barren', 'galmud'),
        'E75754': ('סוּר', 'adjv', 'backsliding', 'sur'),
        'E75755': ('חֹצֶן', 'subs', 'bosom', 'chotsen'),
        'E75756': ('יָרִיב', 'subs', 'opponent', 'yariv'),
        'E75757': ('עָסִיס', 'subs', 'juice', 'asis'),
        'E75758': ('קַדְרוּת', 'subs', 'darkness', 'kadrut'),
        'E75759': ('עות', 'verb', '[uncertain]', 'vt'),
        'E75760': ('רֹק', 'subs', 'spittle', 'rok'),
        'E75761': ('עָשׁ', 'subs', 'moth', 'ash'),
        'E75762': ('זִיקָה', 'subs', 'fire-arrow', 'zika'),
        'E75763': ('מַעֲצֵבָה', 'subs', 'pain', "ma'atseiva"),
        'E75764': ('מַקֶּבֶת', 'subs', 'hole', 'makevet'),
        'E75765': ('זִמְרָה', 'subs', 'melody', 'zimra'),
        'E75766': ('מלח', 'verb', 'disperse', 'mlch'),
        'E75767': ('גִּדּוּפָה', 'subs', 'reviling word', 'gidufa'),
        'E75768': ('סָס', 'subs', 'moth', 'sas'),
        'E75769': ('חלל', 'verb', 'pierce', "chl'l"),
        'E75770': ('מַעֲמַקִּים', 'subs', 'depths', "ma'amakim"),
        'E75771': ('צעה', 'verb', 'put in chains', "ts'h"),
        'E75772': ('קֻבַּעַת', 'subs', 'cup', "kuba'at"),
        'E75773': ('תַּרְעֵלָה', 'subs', 'reeling', 'tareila'),
        'E75774': ('מִכְמָר', 'subs', 'net', 'michmar'),
        'E75775': ('שָׁכֻר', 'adjv', 'drunk', 'shachur'),
        'E75776': ('יגה', 'verb', 'grieve', 'ygh'),
        'E75777': ('שׁחה', 'verb', 'bow down', 'shchh'),
        'E75778': ('שְׁבִיָּה', 'subs', 'captive', "sh'bia"),
        'E75779': ('נאה', 'verb', 'be lovely', 'nh'),
        'E75780': ('מִשְׁחָת', 'subs', 'destruction', 'mishchat'),
        'E75781': ('יֹונֵק', 'subs', 'sapling', 'yovneik'),
        'E75782': ('חָדֵל', 'adjv', 'ceasing', 'chadeil'),
        'E75783': ('מַסְתֵּר', 'subs', 'making hide face', 'masteir'),
        'E75784': ('עֹצֶר', 'subs', 'restraint', 'otser'),
        'E75785': ('עֲלוּמִים', 'subs', 'youth', 'alumim'),
        'E75786': ('שֶׁצֶף', 'subs', 'flowing', 'shetsef'),
        'E75787': ('כַּדְכֹּד', 'subs', 'ruby', 'kadkod'),
        'E75788': ('אֶקְדָּח', 'subs', 'beryl', 'ekdach'),
        'E75789': ('מְחִתָּה', 'subs', 'terror', "m'chita"),
        'E75790': ('גור', 'verb', 'attack', 'gvr'),
        'E75791': ('מחא', 'verb', 'clap', 'mch'),
        'E75792': ('סִרְפַּד', 'subs', 'nettle', 'sirpad'),
        'E75793': ('נבח', 'verb', 'bark', 'nvch'),
        'E75794': ('הזה', 'verb', 'pant', 'hzh'),
        'E75795': ('שׁור', 'verb', 'descend', 'shvr'),
        'E75796': ('רִקֻּחַ', 'subs', 'ointment', 'rikuach'),
        'E75797': ('חַיָּה', 'subs', 'life', 'chaya'),
        'E75798': ('קִבּוּץ', 'subs', '[uncertain]', 'kibuts'),
        'E75799': ('דַּכָּא', 'adjv', 'crushed', 'daka'),
        'E75800': ('שֹׁובָב', 'adjv', 'apostate', 'shovvav'),
        'E75801': ('נִחֻמִים', 'subs', 'comfort', 'nichumim'),
        'E75802': ('נִיב', 'subs', 'fruit', 'niv'),
        'E75803': ('רֶפֶשׁ', 'subs', 'sea-weed', 'refesh'),
        'E75804': ('קִרְבָה', 'subs', 'approach', 'kirva'),
        'E75805': ('עַצָּב', 'subs', '[uncertain]', 'atsav'),
        'E75806': ('מַצָּה', 'subs', 'contention', 'matsa'),
        'E75807': ('כפף', 'verb', 'bend', "chf'f"),
        'E75808': ('חַרְצֹב', 'subs', 'fedders', 'chartsov'),
        'E75809': ('מָרוּד', 'subs', 'homelessness', 'marud'),
        'E75810': ('אֲרוּכָה', 'subs', 'healing', 'arucho'),
        'E75811': ('שׁוע', 'verb', 'cry', 'shv'),
        'E75812': ('צַחְצָחָה', 'subs', 'scorched land', 'tsachtsacha'),
        'E75813': ('גאל', 'verb', 'pollute', 'gl'),
        'E75814': ('קוּר', 'subs', 'thread', 'kur'),
        'E75815': ('עַכָּבִישׁ', 'subs', 'spider', 'akavish'),
        'E75816': ('עקשׁ', 'verb', 'crook', 'ksh'),
        'E75817': ('נְגֹהָה', 'subs', 'brightness', "n'goha"),
        'E75818': ('גשׁשׁ', 'verb', 'grope', "gsh'sh"),
        'E75819': ('אַשְׁמַנִּים', 'subs', '[uncertain]', 'ashmanim'),
        'E75820': ('תִּלְבֹּשֶׁת', 'subs', 'clothing', 'tilboshet'),
        'E75821': ('זֶרַח', 'subs', 'flashing up', 'zerach'),
        'E75822': ('נהר', 'verb', 'beam', 'nhr'),
        'E75823': ('בֵּכֶר', 'subs', 'young male camel', 'beicher'),
        'E75824': ('עֵיפָה', 'nmpr', 'Ephah', 'eifa'),
        'E75825': ('שֹׁד', 'subs', 'breast', 'shod'),
        'E75826': ('מַטָּע', 'subs', 'planting', 'mata'),
        'E75827': ('פְּקַח קֹוחַ', 'subs', 'opening', "p'kach kovach"),
        'E75828': ('מַעֲטֶה', 'subs', 'wrap', "ma'ateh"),
        'E75829': ('אִכָּר', 'subs', 'farmer', 'ikar'),
        'E75830': ('ימר', 'verb', '[uncertain]', 'ymr'),
        'E75831': ('יעט', 'verb', '[uncertain]', 'yt'),
        'E75832': ('עדה', 'verb', 'adorn', 'dh'),
        'E75833': ('צָנִיף', 'subs', 'turban', 'tsanif'),
        'E75834': ('דֳּמִי', 'subs', 'rest', 'dami'),
        'E75835': ('חָמוּץ', 'adjv', 'coloured', 'chamuts'),
        'E75836': ('פּוּרָה', 'subs', 'trough', 'pura'),
        'E75837': ('נֵצַח', 'subs', 'blood-spout', 'neitsach'),
        'E75838': ('גְּאוּלִים', 'subs', 'avenge', "g'ulim"),
        'E75839': ('קשׁח', 'verb', 'be hard', 'kshch'),
        'E75840': ('הֶמֶס', 'subs', 'brush-wood', 'hemes'),
        'E75841': ('עִדָּה', 'subs', 'menstruation', 'ida'),
        'E75842': ('נְצוּרִים', 'subs', 'watch-huts', "n'tsurim"),
        'E75843': ('גַּד', 'nmpr', '[god of fortune]', 'gad'),
        'E75844': ('מְנִי', 'nmpr', 'destiny', "m'ni"),
        'E75845': ('מִמְסָךְ', 'subs', 'mixed wine', 'mimsach'),
        'E75846': ('הֲדֹם', 'subs', 'foot-stool', 'hadom'),
        'E75847': ('מְגֹורָה', 'subs', 'horror', "m'govra"),
        'E75848': ('נדה', 'verb', 'exclude', 'ndh'),
        'E75849': ('תַּנְחוּמִים', 'subs', 'consolation', 'tanchumim'),
        'E75850': ('מצץ', 'verb', 'sip', "mts'ts"),
        'E75851': ('זִיז', 'subs', 'udder', 'ziz'),
        'E75852': ('סוף', 'verb', 'end', 'svf'),
        'E75853': ('פּוּל', 'nmpr', 'Libyans', 'pul'),
        'E75854': ('כִּרְכָּרָה', 'subs', 'dromedary', 'kirkara'),
        'E75855': ('דֵּרָאֹון', 'subs', 'abhorrence', 'deiraovn'),
        'E75856': ('כְּלוּלֹת', 'subs', 'bridehood', "k'lulot"),
        'E75857': ('שׁוּחָה', 'subs', 'pit', 'shucha'),
        'E75858': ('שׂער', 'verb', 'bristle', 'sr'),
        'E75859': ('תַּחְפַּנְחֵס', 'nmpr', 'Tahpanhes', 'tachpancheis'),
        'E75860': ('מְשׁוּבָה', 'subs', 'apostasy', "m'shuva"),
        'E75861': ('פַּחְדָּה', 'subs', '[uncertain]', 'pachda'),
        'E75862': ('מֹוסֵרָה', 'subs', 'band', 'movseira'),
        'E75863': ('נֶתֶר', 'subs', 'natron', 'neter'),
        'E75864': ('בֹּרִית', 'subs', 'alkaline salt', 'borit'),
        'E75865': ('כתם', 'verb', 'defile', 'chtm'),
        'E75866': ('בִּכְרָה', 'subs', 'young female camel', 'bichra'),
        'E75867': ('שׂרך', 'verb', 'run aimlessly', 'srch'),
        'E75868': ('תַּאֲנָה', 'subs', 'rutting', "ta'ana"),
        'E75869': ('צִמְאָה', 'subs', 'thirst', 'tsima'),
        'E75870': ('מַאְפֵּלְיָה', 'subs', '[uncertain]', 'mapeila'),
        'E75871': ('רוד', 'verb', 'roam', 'rvd'),
        'E75872': ('אַלּוּף', 'adjv', 'familiar', 'aluf'),
        'E75873': ('בָּגֹוד', 'adjv', 'treacherous', 'bagovd'),
        'E75874': ('קֹל', 'subs', 'lightness', 'kol'),
        'E75875': ('פזר', 'verb', 'scatter', 'fzr'),
        'E75876': ('תַּחֲנוּן', 'subs', 'supplication', 'tachanun'),
        'E75877': ('ניר', 'verb', 'till', 'nyr'),
        'E75878': ('נִיר', 'subs', 'tilled ground', 'nir'),
        'E75879': ('סְבֹךְ', 'subs', 'thicket', "s'voch"),
        'E75880': ('עֲנָנָה', 'subs', 'rain cloud', 'anana'),
        'E75881': ('סָכָל', 'adjv', 'foolish', 'sachol'),
        'E75882': ('עָב', 'subs', 'thicket', 'av'),
        'E75883': ('כֵּף', 'subs', 'rock', 'keif'),
        'E75884': ('יפה', 'verb', 'be beautiful', 'yfh'),
        'E75885': ('עגב', 'verb', 'desire', 'gv'),
        'E75886': ('יפח', 'verb', 'gasp', 'yfch'),
        'E75887': ('יזן', 'verb', 'be ruttish', 'yzn'),
        'E75888': ('שׁכה', 'verb', 'be potent', 'shchh'),
        'E75889': ('שָׁרָה', 'subs', '[uncertain]', 'shara'),
        'E75890': ('רשׁשׁ', 'verb', 'shatter', "rsh'sh"),
        'E75891': ('יָקוּשׁ', 'subs', 'fowler', 'yakush'),
        'E75892': ('כְּלוּב', 'subs', 'basket', "k'luv"),
        'E75893': ('עשׁת', 'verb', 'be smooth?', 'sht'),
        'E75894': ('שַׁעֲרוּר', 'adjv', 'horrible', "sha'arur"),
        'E75895': ('בֵּית הַכֶּרֶם', 'nmpr', 'Beth Hakkerem', 'beit hakerem'),
        'E75896': ('נָוֶה', 'adjv', '[uncertain]', 'naveh'),
        'E75897': ('עֵצָה', 'subs', 'wood', 'eitsa'),
        'E75898': ('בַּיִר', 'subs', '[uncertain]', 'bayir'),
        'E75899': ('סַלְסִלָּה', 'subs', 'shoot', 'salsila'),
        'E75900': ('עֹולָל', 'subs', 'child', 'ovlal'),
        'E75901': ('מַרְגֹּועַ', 'subs', 'rest', 'margova'),
        'E75902': ('ערב', 'verb', 'be pleasing', 'rv'),
        'E75903': ('פלשׁ', 'verb', 'roll', 'flsh'),
        'E75904': ('תַּמְרוּרִים', 'subs', 'bitterness', 'tamrurim'),
        'E75905': ('בָּחֹון', 'subs', 'examiner', 'bachovn'),
        'E75906': ('חרר', 'verb', 'be hot', "chr'r"),
        'E75907': ('מַפֻּחַ', 'subs', 'bellow', 'mapuach'),
        'E75908': ('פָּרִיץ', 'subs', 'robber', 'parits'),
        'E75909': ('כַּוָּן', 'subs', 'cake', 'kauan'),
        'E75910': ('מְלֶכֶת', 'subs', 'queen of heaven', "m'lechet"),
        'E75911': ('מֹועֵצָה', 'subs', 'counsel', 'moveitsa'),
        'E75912': ('הֲרֵגָה', 'subs', 'slaughter', 'hareiga'),
        'E75913': ('שֹׁובֵב', 'adjv', 'apostate', 'shovveiv'),
        'E75914': ('נצח', 'verb', 'prevail', 'ntsch'),
        'E75915': ('תַּרְמִית', 'subs', 'deceitfulness', 'tarmit'),
        'E75916': ('סִיס', 'subs', 'swift-bird', 'sis'),
        'E75917': ('עֵט', 'subs', 'stylus', 'eit'),
        'E75918': ('מַרְפֵּא', 'subs', 'healing', 'marpei'),
        'E75919': ('בְּעָתָה', 'subs', 'terror', "b'ata"),
        'E75920': ('נַחֲרָה', 'subs', 'snorting', 'nachara'),
        'E75921': ('מִצְהָלֹות', 'subs', 'neighing', "mits'halovt"),
        'E75922': ('מַבְלִיגִית', 'subs', '[uncertain]', 'mavligit'),
        'E75923': ('אֹרֶב', 'subs', 'ambush', 'orev'),
        'E75924': ('נְהִי', 'subs', 'lamentation', "n'hi"),
        'E75925': ('נָוָה', 'subs', 'pasture', 'nava'),
        'E75926': ('עַפְעַפִּים', 'subs', 'beaming eyes', 'afapim'),
        'E75927': ('עָמִיר', 'subs', 'grass', 'amir'),
        'E75928': ('מַשְׂמֵרָה', 'subs', 'nail', 'masmeira'),
        'E75929': ('תֹּמֶר', 'subs', 'scare-crow', 'tomer'),
        'E75930': ('יאה', 'verb', 'be fitting', 'yh'),
        'E75931': ('כסל', 'verb', 'be insolent', 'chsl'),
        'E75932': ('אוּפָז', 'nmpr', 'Uphaz', 'ufaz'),
        'E75933': ('כְּ', 'prep', 'like', "k'"),
        'E75934': ('דְּנָה', 'prde', 'this', "d'na"),
        'E75935': ('אמר', 'verb', 'say', 'mr'),
        'E75936': ('לְ', 'prep', 'to', "l'"),
        'E75937': ('אֱלָהּ', 'subs', 'god', 'elah'),
        'E75938': ('דִּי', 'prep', '[relative]', 'di'),
        'E75939': ('שְׁמַיִן', 'subs', 'heaven', "sh'mayin"),
        'E75940': ('וְ', 'conj', 'and', "v'"),
        'E75941': ('אֲרַק', 'subs', 'earth', 'arak'),
        'E75942': ('לָא', 'nega', 'not', 'la'),
        'E75943': ('עבד', 'verb', 'do', 'vd'),
        'E75944': ('אבד', 'verb', 'perish', 'vd'),
        'E75945': ('מִן', 'prep', 'from', 'min'),
        'E75946': ('אֲרַע', 'subs', 'earth', 'ara'),
        'E75947': ('תְּחֹות', 'subs', 'underneath', "t'chovt"),
        'E75948': ('אֵלֶּה', 'prde', 'these', 'eileh'),
        'E75949': ('נָשִׂיא', 'subs', 'fog', 'nasi'),
        'E75950': ('תַּעְתֻּעִים', 'subs', 'mockery', 'tatuim'),
        'E75951': ('כִּנְעָה', 'subs', 'load', 'kina'),
        'E75952': ('מְזִמָּה', 'subs', 'purpose', "m'zima"),
        'E75953': ('הֲמֻלָּה', 'subs', 'sound', 'hamula'),
        'E75954': ('דָּלִית', 'subs', 'foliage', 'dalit'),
        'E75955': ('אבל', 'verb', 'dry up', 'vl'),
        'E75956': ('יְדִדוּת', 'subs', 'beloved one', "y'didut"),
        'E75957': ('צָבוּעַ', 'subs', 'hyena', 'tsavua'),
        'E75958': ('שָׁמֵם', 'adjv', 'desolate', 'shameim'),
        'E75959': ('שִׁכָּרֹון', 'subs', 'drunkenness', 'shikarovn'),
        'E75960': ('גֵּוָה', 'subs', 'pride', 'geiva'),
        'E75961': ('דמע', 'verb', 'shed tears', 'dm'),
        'E75962': ('חמס', 'verb', 'treat violently', 'chms'),
        'E75963': ('חֲבַרְבֻּרֹת', 'subs', 'skin-specks', 'chavarburot'),
        'E75964': ('מְנָת', 'subs', 'portion', "m'nat"),
        'E75965': ('נִאֻפִים', 'subs', 'adultery', 'niufim'),
        'E75966': ('בַּצֹּרֶת', 'subs', 'drought', 'batsoret'),
        'E75967': ('אַיֶּלֶת', 'subs', 'doe', 'ayelet'),
        'E75968': ('מִקְוֶה', 'subs', 'hope', 'mikveh'),
        'E75969': ('דהם', 'verb', 'surprise', 'dhm'),
        'E75970': ('גשׁם', 'verb', 'rain', 'gshm'),
        'E75971': ('עִיר', 'subs', 'excitement', 'ir'),
        'E75972': ('שׁרה', 'verb', 'let loose', 'shrh'),
        'E75973': ('אַכְזָב', 'adjv', 'deceitful', 'achzav'),
        'E75974': ('מְמֹותִים', 'subs', 'death', "m'movtim"),
        'E75975': ('מַרְזֵחַ', 'subs', 'banquet', 'marzeiach'),
        'E75976': ('חֲנִינָה', 'subs', 'kindness', 'chanina'),
        'E75977': ('דיג', 'verb', 'fish', 'dyg'),
        'E75978': ('צַיָּד', 'subs', 'hunter', 'tsayad'),
        'E75979': ('שָׁמִיר', 'subs', 'diamond', 'shamir'),
        'E75980': ('עַרְעָר', 'subs', 'juniper', 'arar'),
        'E75981': ('חֲרֵרִים', 'subs', 'lava desert', 'chareirim'),
        'E75982': ('מְלֵחָה', 'subs', 'salt country', "m'leicha"),
        'E75983': ('שׁתל', 'verb', 'transplant', 'shtl'),
        'E75984': ('יוּבַל', 'subs', 'canal', 'yuval'),
        'E75985': ('שִׁבָּרֹון', 'subs', 'breakdown', 'shibarovn'),
        'E75986': ('שַׁעֲרוּרִי', 'adjv', 'horrible', "sha'aruri"),
        'E75987': ('קַר', 'adjv', 'cool', 'kar'),
        'E75988': ('שְׁבִיל', 'subs', 'path', "sh'vil"),
        'E75989': ('חַרְסִית', 'subs', 'sherd', 'charsit'),
        'E75990': ('שְׁרִקָה', 'subs', 'whistling', "sh'rika"),
        'E75991': ('פַּשְׁחוּר', 'nmpr', 'Pashhur', 'pashchur'),
        'E75992': ('אִמֵּר', 'nmpr', 'Immer', 'imeir'),
        'E75993': ('מַהְפֶּכֶת', 'subs', 'stock', 'mahpechet'),
        'E75994': ('יְקָר', 'subs', 'preciousness', "y'kar"),
        'E75995': ('שְׂחֹוק', 'subs', 'laughter', "s'chovk"),
        'E75996': ('קֶלֶס', 'subs', 'derision', 'keles'),
        'E75997': ('צֶלַע', 'subs', 'stumbling', 'tsela'),
        'E75998': ('מַלְכִּיָּה', 'nmpr', 'Malkijah', 'malkia'),
        'E75999': ('צְפַנְיָה', 'nmpr', 'Zephaniah', "ts'fanya"),
        'E76000': ('מַעֲשֵׂיָה', 'nmpr', 'Maaseiah', "ma'aseia"),
        'E76001': ('נְבוּכַדְרֶאצַּר', 'nmpr', 'Nebuchadnezzar', "n'vuchadretsar"),
        'E76002': ('עָשֹׁוק', 'subs', 'oppressor', 'ashovk'),
        'E76003': ('שָׁשַׁר', 'subs', 'minium', 'shashar'),
        'E76004': ('מְרוּצָה', 'subs', 'extortion', "m'rutsa"),
        'E76005': ('שַׁלְוָה', 'subs', 'ease', 'shalva'),
        'E76006': ('כָּנְיָהוּ', 'nmpr', 'Jehoiachin', 'kanyahu'),
        'E76007': ('יָגֹור', 'adjv', 'fearing', 'yagovr'),
        'E76008': ('עֶצֶב', 'subs', 'form', 'etsev'),
        'E76009': ('חֲלַקְלַקֹּות', 'subs', 'smoothness', 'chalaklakovt'),
        'E76010': ('תִּפְלָה', 'subs', 'unseemliness', 'tifla'),
        'E76011': ('חֲנֻפָּה', 'subs', 'alienation', 'chanupa'),
        'E76012': ('סַעַר', 'subs', 'storm', "sa'ar"),
        'E76013': ('פצץ', 'verb', 'shatter', "fts'ts"),
        'E76014': ('נאם', 'verb', 'speak', 'nm'),
        'E76015': ('פַּחֲזוּת', 'subs', 'boasting', 'pachazut'),
        'E76016': ('כְּלִמּוּת', 'subs', 'insult', "k'limut"),
        'E76017': ('יְכָנְיָהוּ', 'nmpr', 'Jehoiachin', "y'chonyahu"),
        'E76018': ('רִאשֹׁונִי', 'adjv', 'first', 'rishovni'),
        'E76019': ('עוּץ', 'nmpr', '[land of hiob]', 'uts'),
        'E76020': ('בּוּז', 'nmpr', 'Buz', 'buz'),
        'E76021': ('זִמְרִי', 'nmpr', 'Zimri', 'zimri'),
        'E76022': ('שֵׁשַׁךְ', 'nmpr', 'Sheshach', 'sheishach'),
        'E76023': ('קיה', 'verb', 'vomit', 'kyh'),
        'E76024': ('סֹךְ', 'subs', 'cover', 'soch'),
        'E76025': ('מֹורַשְׁתִּי', 'adjv', 'from Moresheth', 'movrashti'),
        'E76026': ('עִי', 'subs', 'heap', 'i'),
        'E76027': ('אוּרִיָּהוּ', 'nmpr', 'Uriah', 'uriahu'),
        'E76028': ('שְׁמַעְיָהוּ', 'nmpr', 'Shemaiah', "sh'mayahu"),
        'E76029': ('יִרְמְיָה', 'nmpr', 'Jeremiah', 'yirma'),
        'E76030': ('כַּשָּׁף', 'subs', 'sorcerer', 'kashaf'),
        'E76031': ('יְכֹונְיָה', 'nmpr', 'Jehoiachin', "y'chovnya"),
        'E76032': ('חֲנַנְיָה', 'nmpr', 'Hananiah', 'chananya'),
        'E76033': ('עַזּוּר', 'nmpr', 'Azzur', 'azur'),
        'E76034': ('יְכָנְיָה', 'nmpr', 'Jehoiachin', "y'chonya"),
        'E76035': ('אֶלְעָשָׂה', 'nmpr', 'Elasah', 'elasa'),
        'E76036': ('גְּמַרְיָה', 'nmpr', 'Gemariah', "g'marya"),
        'E76037': ('תִּקְוָה', 'subs', 'hope', 'tikva'),
        'E76038': ('שֹׁעָר', 'adjv', 'burst', 'shoar'),
        'E76039': ('קֹולָיָה', 'nmpr', 'Kolaiah', 'kovlaya'),
        'E76040': ('נֶחֱלָמִי', 'adjv', 'Nehelamite', 'nechelami'),
        'E76041': ('צִינֹק', 'subs', 'collar', 'tsinok'),
        'E76042': ('שׁאן', 'verb', 'be at ease', 'shn'),
        'E76043': ('מָזֹור', 'subs', 'ulcer', 'mazovr'),
        'E76044': ('רְפוּאָה', 'subs', 'healing', "r'fua"),
        'E76045': ('תְּעָלָה', 'subs', 'healing', "t'ala"),
        'E76046': ('שׁאס', 'verb', '[uncertain]', 'shs'),
        'E76047': ('צער', 'verb', 'be small', 'tsr'),
        'E76048': ('מָחֹול', 'subs', 'round dance', 'machovl'),
        'E76049': ('דאב', 'verb', 'languish', 'dv'),
        'E76050': ('יַקִּיר', 'adjv', 'precious', 'yakir'),
        'E76051': ('תַּמְרוּרִים', 'subs', 'sign-post', 'tamrurim'),
        'E76052': ('חמק', 'verb', 'turn away', 'chmk'),
        'E76053': ('קהה', 'verb', 'be dull', "kh'h"),
        'E76054': ('חֲנַנְאֵל', 'nmpr', 'Hananel', 'chananeil'),
        'E76055': ('גֹּעָה', 'nmpr', 'Goah', 'goa'),
        'E76056': ('חֲנַמְאֵל', 'nmpr', 'Hanamel', 'chanameil'),
        'E76057': ('בָּרוּךְ', 'nmpr', 'Baruch', 'baruch'),
        'E76058': ('נֵרִיָּה', 'nmpr', 'Neriah', 'neiria'),
        'E76059': ('מַחְסֵיָה', 'nmpr', 'Mahseiah', 'machseia'),
        'E76060': ('עֲלִילִיָּה', 'subs', 'deed', 'alilia'),
        'E76061': ('אֶזְרֹעַ', 'subs', 'arm', 'ezroa'),
        'E76062': ('נְעֻרֹות', 'subs', 'youth', "n'urovt"),
        'E76063': ('עֲתֶרֶת', 'subs', '[uncertain]', 'ateret'),
        'E76064': ('יִשְׂחָק', 'nmpr', 'Isaac', 'yischak'),
        'E76065': ('רֵכָבִי', 'adjv', 'Recabite', 'reichovi'),
        'E76066': ('יַאֲזַנְיָה', 'nmpr', 'Jaazaniah', "ya'azanya"),
        'E76067': ('חֲבַצִּנְיָה', 'nmpr', 'Habazziniah', 'chavatsina'),
        'E76068': ('חָנָן', 'nmpr', 'Hanan', 'chanan'),
        'E76069': ('יִגְדַּלְיָהוּ', 'nmpr', 'Igdaliah', 'yigdalyahu'),
        'E76070': ('מַעֲשֵׂיָהוּ', 'nmpr', 'Maaseiah', "ma'aseiahu"),
        'E76071': ('מְגִלָּה', 'subs', 'roll', "m'gila"),
        'E76072': ('גְּמַרְיָהוּ', 'nmpr', 'Gemariah', "g'maryahu"),
        'E76073': ('דְּלָיָהוּ', 'nmpr', 'Delaiah', "d'layahu"),
        'E76074': ('חֲנַנְיָהוּ', 'nmpr', 'Hananiah', 'chananyahu'),
        'E76075': ('יְהוּדִי', 'nmpr', 'Jehudi', "y'hudi"),
        'E76076': ('נְתַנְיָהוּ', 'nmpr', 'Nethaniah', "n'tanyahu"),
        'E76077': ('שֶׁלֶמְיָהוּ', 'nmpr', 'Shelemiah', 'shelemyahu'),
        'E76078': ('כּוּשִׁי', 'nmpr', 'Cushi', 'kushi'),
        'E76079': ('נֵרִיָּהוּ', 'nmpr', 'Neriah', 'neiriahu'),
        'E76080': ('דְּיֹו', 'subs', 'ink', "d'yov"),
        'E76081': ('אָח', 'subs', 'brazier', 'ach'),
        'E76082': ('יְרַחְמְאֵל', 'nmpr', 'Jerahmeel', "y'rachmeil"),
        'E76083': ('שְׂרָיָהוּ', 'nmpr', 'Seraiah', "s'rayahu"),
        'E76084': ('עַזְרִיאֵל', 'nmpr', 'Azriel', 'azrieil'),
        'E76085': ('עַבְדְּאֵל', 'nmpr', 'Abdeel', 'avdeil'),
        'E76086': ('יְהוּכַל', 'nmpr', 'Jehucal', "y'huchal"),
        'E76087': ('שֶׁלֶמְיָה', 'nmpr', 'Shelemiah', 'shelemya'),
        'E76088': ('כְּלוּא', 'subs', 'prison', "k'lu"),
        'E76089': ('פְּקִדֻת', 'subs', 'oversight', "p'kidut"),
        'E76090': ('יִרְאִיָּיה', 'nmpr', 'Irijah', 'yiriay'),
        'E76091': ('חָנוּת', 'subs', 'room', 'chanut'),
        'E76092': ('יוּכַל', 'nmpr', 'Jehucal', 'yuchal'),
        'E76093': ('מַלְכִּיָּהוּ', 'nmpr', 'Malkijah', 'malkiahu'),
        'E76094': ('עֶבֶד מֶלֶךְ', 'nmpr', 'Ebed-Melech', 'eved melech'),
        'E76095': ('בְּלֹוי', 'subs', 'waste', "b'lovy"),
        'E76096': ('סְחָבֹות', 'subs', 'rags', "s'chavovt"),
        'E76097': ('מֶלַח', 'subs', 'rag', 'melach'),
        'E76098': ('אַצִּיל', 'subs', 'armpit', 'atsil'),
        'E76099': ('בֹּץ', 'subs', 'silt', 'bots'),
        'E76100': ('נֵרְגַל שַׂר אֶצֶר',
                   'nmpr',
                   'Nergal-Sharezer',
                   'neirgal sar etser'),
        'E76101': ('סַמְגַּר נְבוּ', 'nmpr', 'Samgar-Nebo', "samgar n'vu"),
        'E76102': ('שַׂר סְכִים', 'nmpr', 'Sarsekim', "sar s'chim"),
        'E76103': ('מָג', 'subs', 'official', 'mag'),
        'E76104': ('יָגֵב', 'subs', 'compulsory labour', 'yageiv'),
        'E76105': ('נְבוּשַׁזְבָּן', 'nmpr', 'Nebushazban', "n'vushazban"),
        'E76106': ('אֲזִקִּים', 'subs', 'handcuffs', 'azikim'),
        'E76107': ('גְּדַלְיָה', 'nmpr', 'Gedaliah', "g'dalya"),
        'E76108': ('עֵיפַי', 'nmpr', 'Ephai', 'eifay'),
        'E76109': ('יְזַנְיָהוּ', 'nmpr', 'Jezaniah', "y'zanyahu"),
        'E76110': ('בַּעֲלִיס', 'nmpr', 'Baalis', "ba'alis"),
        'E76111': ('גֵּרוּת', 'subs', 'lodging-place', 'geirut'),
        'E76112': ('יְזַנְיָה', 'nmpr', 'Jezaniah', "y'zanya"),
        'E76113': ('הֹושַׁעְיָה', 'nmpr', 'Hoshaiah', 'hovshaya'),
        'E76114': ('מֶלֶט', 'subs', 'mortar', 'melet'),
        'E76115': ('שַׁפְרִיר', 'subs', 'state tent', 'shafrir'),
        'E76116': ('עצב', 'verb', 'shape', 'tsv'),
        'E76117': ('קִטֵּר', 'subs', '[uncertain]', 'kiteir'),
        'E76118': ('חָפְרַע', 'nmpr', 'Hophra', 'chafra'),
        'E76119': ('נְכֹו', 'nmpr', 'Neco', "n'chov"),
        'E76120': ('סִרְיֹון', 'subs', 'scaly coat', 'sirovn'),
        'E76121': ('סחף', 'verb', 'wash away', "s'chf"),
        'E76122': ('פִּיָּה', 'subs', '[uncertain]', 'pia'),
        'E76123': ('קֶרֶץ', 'subs', 'mosquito', 'kerets'),
        'E76124': ('אָמֹון', 'nmpr', 'Amon', 'amovn'),
        'E76125': ('נֹא', 'nmpr', 'Thebes', 'no'),
        'E76126': ('שְׁעָטָה', 'subs', 'stamping', "sh'ata"),
        'E76127': ('רִפָּיֹון', 'subs', 'despair', 'ripayovn'),
        'E76128': ('מַדְמֵן', 'nmpr', 'Madmen', 'madmein'),
        'E76129': ('עֲרֹועֵר', 'subs', '[uncertain]', 'aroveir'),
        'E76130': ('נצא', 'verb', '[uncertain]', 'nts'),
        'E76131': ('רְמִיָּה', 'subs', 'looseness', "r'mia"),
        'E76132': ('בֵּית דִּבְלָתָיִם', 'nmpr', 'Beth Diblathaim', 'beit divlatayim'),
        'E76133': ('בֵּית גָּמוּל', 'nmpr', 'Beth Gamul', 'beit gamul'),
        'E76134': ('בֵּית מְעֹון', 'nmpr', 'Beth Meon', "beit m'ovn"),
        'E76135': ('גְּדוּדָה', 'subs', 'incision', "g'duda"),
        'E76136': ('מַלְכָּם', 'nmpr', 'Malcam', 'malkam'),
        'E76137': ('סרח', 'verb', 'be corrupt', 'srch'),
        'E76138': ('תִּפְלֶצֶת', 'subs', 'horror', 'tifletset'),
        'E76139': ('חָגוּ', 'subs', 'abode', 'chagu'),
        'E76140': ('רֶטֶט', 'subs', 'panic', 'retet'),
        'E76141': ('שָׁלֵיו', 'adjv', 'quiet', 'shaleiv'),
        'E76142': ('מְרֹדָךְ', 'nmpr', 'Marduk', "m'rodach"),
        'E76143': ('פושׁ', 'verb', 'paw the ground', 'fvsh'),
        'E76144': ('ידה', 'verb', 'shoot', 'ydh'),
        'E76145': ('אָשְׁיָה', 'subs', 'pillar', 'ashya'),
        'E76146': ('מַגָּל', 'subs', 'sickle', 'magal'),
        'E76147': ('מְרָתַיִם', 'nmpr', 'Merathaim', "m'ratayim"),
        'E76148': ('פְּקֹוד', 'nmpr', 'Pekod', "p'kovd"),
        'E76149': ('מַאֲבוּס', 'subs', 'granary', "ma'avus"),
        'E76150': ('עֲרֵמָה', 'subs', 'heap', 'areima'),
        'E76151': ('רַב', 'subs', 'dart', 'rav'),
        'E76152': ('אַלְמָן', 'subs', 'widower', 'alman'),
        'E76153': ('יֶלֶק', 'subs', 'locust', 'yelek'),
        'E76154': ('מַפֵּץ', 'subs', 'club', 'mapeits'),
        'E76155': ('מִנִּי', 'nmpr', 'Minni', 'mini'),
        'E76156': ('טִפְסָר', 'subs', 'official', 'tifsar'),
        'E76157': ('סָמָר', 'adjv', 'bristly', 'samar'),
        'E76158': ('כָּרֵשׂ', 'subs', 'belly', 'kareis'),
        'E76159': ('נער', 'verb', 'growl', 'nr'),
        'E76160': ('גֹּור', 'subs', "lion's whelp", 'govr'),
        'E76161': ('בֶּלַע', 'subs', 'what is swallowed', 'bela'),
        'E76162': ('אנק', 'verb', 'sigh', 'nk'),
        'E76163': ('אָמֹון', 'subs', 'fondling', 'amovn'),
        'E76164': ('כְּבָר', 'nmpr', 'Kebar', "k'var"),
        'E76165': ('יֹויָכִין', 'nmpr', 'Jehoiachin', 'yovyachin'),
        'E76166': ('יְחֶזְקֵאל', 'nmpr', 'Ezekiel', "y'chezkeil"),
        'E76167': ('בּוּזִי', 'nmpr', 'Buzi', 'buzi'),
        'E76168': ('חַשְׁמַל', 'subs', '[kind of metal]', 'chashmal'),
        'E76169': ('נצץ', 'verb', 'sparkle', "nts'ts"),
        'E76170': ('קָלָל', 'adjv', 'burnished', 'kalal'),
        'E76171': ('רצא', 'verb', '[uncertain]', 'rts'),
        'E76172': ('בָּזָק', 'subs', '[uncertain]', 'bazak'),
        'E76173': ('סָרָב', 'adjv', 'obstinate', 'sarav'),
        'E76174': ('סִלֹּון', 'subs', 'prickle', 'silovn'),
        'E76175': ('הֶגֶה', 'subs', 'whisper', 'hegeh'),
        'E76176': ('הִי', 'subs', 'wailing', 'hi'),
        'E76177': ('תֵּל אָבִיב', 'nmpr', 'Tel Abib', 'teil aviv'),
        'E76178': ('חֵךְ', 'subs', 'palate', 'cheich'),
        'E76179': ('דֹּחַן', 'subs', 'millet', 'dochan'),
        'E76180': ('כֻּסְּמִים', 'subs', 'spelt', 'kusmim'),
        'E76181': ('מִשְׁקֹול', 'subs', 'weight', 'mishkovl'),
        'E76182': ('גֵּל', 'subs', 'dung-cake', 'geil'),
        'E76183': ('עוג', 'verb', 'bake', 'vg'),
        'E76184': ('צָפִיעַ', 'subs', 'dung', 'tsafia'),
        'E76185': ('שִׁמָּמֹון', 'subs', 'horror', 'shimamovn'),
        'E76186': ('גַּלָּב', 'subs', 'barber', 'galav'),
        'E76187': ('הַמָן', 'subs', '[uncertain]', 'haman'),
        'E76188': ('תֹּוכַחַת', 'subs', 'rebuke', 'tovchachat'),
        'E76189': ('קוט', 'verb', 'loath', 'kvt'),
        'E76190': ('אָח', 'intj', 'alas', 'ach'),
        'E76191': ('דִּבְלָה', 'nmpr', 'Riblah', 'divla'),
        'E76192': ('הֵד', 'subs', '[uncertain]', 'heid'),
        'E76193': ('הֵמֶה', 'subs', '[uncertain]', 'heimeh'),
        'E76194': ('נֹהַּ', 'subs', '[uncertain]', 'noha'),
        'E76195': ('תָּקֹועַ', 'subs', '[uncertain]', 'takova'),
        'E76196': ('בּוּשָׁה', 'subs', 'shame', 'busha'),
        'E76197': ('קְפָדָה', 'subs', 'anguish', "k'fada"),
        'E76198': ('זֹהַר', 'subs', 'shining', 'zohar'),
        'E76199': ('בִּאָה', 'subs', 'entrance', 'bia'),
        'E76200': ('חתר', 'verb', 'dig', 'chtr'),
        'E76201': ('מִקְטֶרֶת', 'subs', 'censer', 'mikteret'),
        'E76202': ('עָתָר', 'subs', 'perfume', 'atar'),
        'E76203': ('תַּמּוּז', 'nmpr', 'Tammuz', 'tamuz'),
        'E76204': ('מַשְׁחֵת', 'subs', 'destruction', 'mashcheit'),
        'E76205': ('מַפָּץ', 'subs', 'shattering', 'mapats'),
        'E76206': ('לָבוּשׁ', 'adjv', 'clad', 'lavush'),
        'E76207': ('קֶסֶת', 'subs', 'writing-case', 'keset'),
        'E76208': ('תָּו', 'subs', 'mark', 'tav'),
        'E76209': ('מֻטֶּה', 'subs', 'crookedness', 'muteh'),
        'E76210': ('פְּלַטְיָהוּ', 'nmpr', 'Pelatiah', "p'latyahu"),
        'E76211': ('מְצוּדָה', 'subs', 'net', "m'tsuda"),
        'E76212': ('אֲגַף', 'subs', 'troop', 'agaf'),
        'E76213': ('רָגְזָה', 'subs', 'agitation', 'ragza'),
        'E76214': ('מִקְסָם', 'subs', 'divination', 'miksam'),
        'E76215': ('כְּתָב', 'subs', 'writing', "k'tav"),
        'E76216': ('טעה', 'verb', 'wander', 'th'),
        'E76217': ('חַיִץ', 'subs', 'wall', 'chayits'),
        'E76218': ('תָּפֵל', 'subs', 'whitewash', 'tafeil'),
        'E76219': ('אֶלְגָּבִישׁ', 'subs', 'ice-crystal', 'elgavish'),
        'E76220': ('טִיחַ', 'subs', 'coating', 'tiach'),
        'E76221': ('כֶּסֶת', 'subs', 'band', 'keset'),
        'E76222': ('מִסְפָּחֹות', 'subs', 'veils', 'mispachovt'),
        'E76223': ('פְּתֹות', 'subs', 'fragment', "p'tovt"),
        'E76224': ('פרח', 'verb', 'fly?', 'frch'),
        'E76225': ('כאה', 'verb', 'dishearten', 'chh'),
        'E76226': ('דָּנִיֵּאל', 'nmpr', 'Daniel', 'danieil'),
        'E76227': ('אִיֹּוב', 'nmpr', 'Job', 'iovv'),
        'E76228': ('מְכוּרָה', 'subs', 'origin', "m'chura"),
        'E76229': ('שֹׁר', 'subs', 'navel', 'shor'),
        'E76230': ('מִשְׁעִי', 'subs', '[uncertain]', 'mishi'),
        'E76231': ('חתל', 'verb', 'swaddle', 'chtl'),
        'E76232': ('חֻמְלָה', 'subs', 'compassion', 'chumla'),
        'E76233': ('גֹּעַל', 'subs', 'loathing', 'goal'),
        'E76234': ('עֶרְיָה', 'subs', 'nakedness', 'erya'),
        'E76235': ('מֶשִׁי', 'subs', 'textile', 'meshi'),
        'E76236': ('תַּזְנוּת', 'subs', 'fornication', 'taznut'),
        'E76237': ('פשׂק', 'verb', 'open', 'fsk'),
        'E76238': ('אֲמֻלָה', 'subs', '[uncertain]', 'amula'),
        'E76239': ('לִבָּה', 'subs', 'rage', 'liba'),
        'E76240': ('שַׁלֶּטֶת', 'adjv', 'domineering', 'shaletet'),
        'E76241': ('נֵדֶה', 'subs', 'gift', 'neideh'),
        'E76242': ('נָדָן', 'subs', 'gift', 'nadan'),
        'E76243': ('שׁחד', 'verb', 'give present', 'shchd'),
        'E76244': ('נְחֹשֶׁת', 'subs', 'menstruation', "n'choshet"),
        'E76245': ('בתק', 'verb', 'slaughter', 'vtk'),
        'E76246': ('קָט', 'subs', '[uncertain]', 'kat'),
        'E76247': ('שָׂבְעָה', 'subs', 'satiety', 'sava'),
        'E76248': ('שׁוט', 'verb', 'despise', 'shvt'),
        'E76249': ('פִּתָּחֹון', 'subs', 'inducement', 'pitachovn'),
        'E76250': ('נֹוצָה', 'subs', 'plumage', 'novtsa'),
        'E76251': ('צַמֶּרֶת', 'subs', 'tree-top', 'tsameret'),
        'E76252': ('יְנִיקָה', 'subs', 'shoot', "y'nika"),
        'E76253': ('קָח', 'subs', 'willow', 'kach'),
        'E76254': ('צַפְצָפָה', 'subs', 'willow', 'tsaftsafa'),
        'E76255': ('פֹּארָה', 'subs', 'shoot', 'pora'),
        'E76256': ('כפן', 'verb', 'desire', 'chfn'),
        'E76257': ('עֲרוּגָה', 'subs', 'garden bed', 'aruga'),
        'E76258': ('קסס', 'verb', 'be scaly', "ks's"),
        'E76259': ('פור', 'verb', 'destroy', 'fvr'),
        'E76260': ('מִבְרָח', 'subs', '[uncertain]', 'mivrach'),
        'E76261': ('יֹונֶקֶת', 'subs', 'shoot', 'yovneket'),
        'E76262': ('תָּלוּל', 'adjv', 'lofty', 'talul'),
        'E76263': ('חֲבֹלָה', 'subs', 'pledge', 'chavola'),
        'E76264': ('חֹוב', 'subs', 'guilt', 'chovv'),
        'E76265': ('חֲבֹל', 'subs', 'pledge', 'chavol'),
        'E76266': ('גֵּזֶל', 'subs', 'robbing', 'geizel'),
        'E76267': ('לְבִיָּא', 'subs', 'lion', "l'bia"),
        'E76268': ('סוּגַר', 'subs', 'cage', 'sugar'),
        'E76269': ('עָנֵף', 'adjv', 'branchy', 'aneif'),
        'E76270': ('עֲבֹות', 'subs', 'branch', 'avovt'),
        'E76271': ('מָסֹרֶת', 'subs', 'obligation', 'masoret'),
        'E76272': ('שַׁלְהֶבֶת', 'subs', 'flame', 'shalhevet'),
        'E76273': ('צרב', 'verb', 'burn', 'tsrv'),
        'E76274': ('מְרִירוּת', 'subs', 'bitterness', "m'rirut"),
        'E76275': ('חדד', 'verb', 'be sharp', "chd'd"),
        'E76276': ('מגר', 'verb', 'hurl', 'mgr'),
        'E76277': ('חדר', 'verb', 'surround', 'chdr'),
        'E76278': ('אִבְחָה', 'subs', '[uncertain]', 'ivcha'),
        'E76279': ('מְעֻטָּה', 'subs', '[uncertain]', "m'uta"),
        'E76280': ('אחד', 'verb', '[uncertain]', 'chd'),
        'E76281': ('רֶצַח', 'subs', 'killing', 'retsach'),
        'E76282': ('עֲטָרָה', 'subs', 'wreath', 'atara'),
        'E76283': ('עַוָּה', 'subs', 'ruin', 'aua'),
        'E76284': ('פוח', 'verb', 'wheeze', 'fvch'),
        'E76285': ('קַלָּסָה', 'subs', 'derision', 'kalasa'),
        'E76286': ('קְבֻצָה', 'subs', 'gathering', "k'vutsa"),
        'E76287': ('הִתּוּךְ', 'subs', 'melting', 'hituch'),
        'E76288': ('גֹּשֶׁם', 'subs', '[uncertain]', 'goshem'),
        'E76289': ('דַּד', 'subs', 'breast', 'dad'),
        'E76290': ('אָהֳלָה', 'nmpr', 'Oholah', 'ahala'),
        'E76291': ('אָהֳלִיבָה', 'nmpr', 'Oholibah', 'ahaliva'),
        'E76292': ('שְׁפֹוט', 'subs', 'judgment', "sh'fovt"),
        'E76293': ('עֲגָבָה', 'subs', 'desire', 'agava'),
        'E76294': ('מִכְלֹול', 'subs', 'perfection', 'michlovl'),
        'E76295': ('חָגֹור', 'adjv', 'girded', 'chagovr'),
        'E76296': ('טְבוּלִים', 'subs', 'turban', "t'vulim"),
        'E76297': ('נקע', 'verb', 'be separate', 'nk'),
        'E76298': ('זִרְמָה', 'subs', 'penis', 'zirma'),
        'E76299': ('קֹועַ', 'nmpr', 'Koa', 'kova'),
        'E76300': ('הֹצֶן', 'subs', '[uncertain]', 'hotsen'),
        'E76301': ('לַעַג', 'subs', 'derision', "la'ag"),
        'E76302': ('מִרְבָּה', 'subs', '[uncertain]', 'mirba'),
        'E76303': ('כחל', 'verb', 'paint', 'chchl'),
        'E76304': ('סָבָא', 'subs', 'weed', 'sava'),
        'E76305': ('דור', 'verb', 'circulate', 'dvr'),
        'E76306': ('רתח', 'verb', 'boil', "rt'ch"),
        'E76307': ('רֶתַח', 'subs', '[uncertain]', 'retach'),
        'E76308': ('חֶלְאָה', 'subs', 'rust', 'chela'),
        'E76309': ('צָחִיחַ', 'subs', 'surface', 'tsachiach'),
        'E76310': ('מֶרְקָחָה', 'subs', 'ointment pot', 'merkacha'),
        'E76311': ('תְּאֻנִים', 'subs', '[uncertain]', "t'unim"),
        'E76312': ('מַחְמַל', 'subs', '[uncertain]', 'machmal'),
        'E76313': ('הַשְׁמָעוּת', 'subs', 'information', 'hashmaut'),
        'E76314': ('מִרְבָּץ', 'subs', 'resting place', 'mirbats'),
        'E76315': ('שְׁאָט', 'subs', 'scorn', "sh'at"),
        'E76316': ('סחה', 'verb', 'scrap', "s'chh"),
        'E76317': ('מִשְׁטָח', 'subs', 'spreading-place', 'mishtach'),
        'E76318': ('חֵרֶם', 'subs', 'net', 'cheirem'),
        'E76319': ('מְחִי', 'subs', 'stroke', "m'chi"),
        'E76320': ('קְבֹל', 'subs', 'battering-ram', "k'vol"),
        'E76321': ('רְכֻלָּה', 'subs', 'traffic', "r'chula"),
        'E76322': ('חִתִּית', 'subs', 'terror', 'chitit'),
        'E76323': ('כלל', 'verb', 'perfect', "chl'l"),
        'E76324': ('מָשֹׁוט', 'subs', 'oar', 'mashovt'),
        'E76325': ('אָשׁוּר', 'subs', 'step', 'ashur'),
        'E76326': ('מִפְרָשׂ', 'subs', 'sail', 'mifras'),
        'E76327': ('אַרְוַד', 'nmpr', 'Arvad', 'arvad'),
        'E76328': ('חֹבֵל', 'subs', 'sailor', 'choveil'),
        'E76329': ('גְּבַל', 'nmpr', '[mountain]', "g'val"),
        'E76330': ('מַלָּח', 'subs', 'mariner', 'malach'),
        'E76331': ('מַעֲרָב', 'subs', 'exchange', "ma'arav"),
        'E76332': ('פָּרַס', 'nmpr', 'Persia', 'paras'),
        'E76333': ('חֵילֵךְ', 'nmpr', 'Helech', 'cheileich'),
        'E76334': ('גַּמָּדִים', 'nmpr', 'Gammad', 'gamadim'),
        'E76335': ('הֹון', 'subs', 'abundance', 'hovn'),
        'E76336': ('עִזְבֹונִים', 'subs', 'stores', 'izvovnim'),
        'E76337': ('סְחֹרָה', 'subs', '[uncertain]', "s'chora"),
        'E76338': ('הָבְנִים', 'subs', 'ebony', 'havnim'),
        'E76339': ('אֶשְׁכָּר', 'subs', 'tribute', 'eshkar'),
        'E76340': ('בּוּץ', 'subs', 'byssus', 'buts'),
        'E76341': ('רָאמֹות', 'subs', '[uncertain]', 'ramovt'),
        'E76342': ('פַּנַּג', 'subs', '[uncertain]', 'panag'),
        'E76343': ('חֶלְבֹּון', 'nmpr', 'Helbon', 'chelbovn'),
        'E76344': ('צָחַר', 'nmpr', 'Zahar', 'tsachar'),
        'E76345': ('וְדָן', 'nmpr', 'Wedan', "v'dan"),
        'E76346': ('עָשֹׁות', 'adjv', 'wrought', 'ashovt'),
        'E76347': ('חֹפֶשׁ', 'subs', 'cloth', 'chofesh'),
        'E76348': ('רִכְבָּה', 'subs', 'driving', 'richbo'),
        'E76349': ('כַּנֵּה', 'nmpr', 'Canneh', 'kanei'),
        'E76350': ('כִּלְמַד', 'nmpr', 'Kilmad', 'kilmad'),
        'E76351': ('מַכְלֻלִים', 'subs', 'garments', 'machlulim'),
        'E76352': ('גְּלֹום', 'subs', 'garment', "g'lovm"),
        'E76353': ('גֶּנֶז', 'subs', 'treasury', 'genez'),
        'E76354': ('בְּרֹמִים', 'subs', 'two-coloured stuff', "b'romim"),
        'E76355': ('אָרוּז', 'adjv', 'solid', 'aruz'),
        'E76356': ('מַרְכֹּלֶת', 'subs', '[uncertain]', 'markolet'),
        'E76357': ('מִגְרְשֹׁות', 'subs', '[uncertain]', 'migrshovt'),
        'E76358': ('נִי', 'subs', 'wailing', 'ni'),
        'E76359': ('דֻּמָה', 'subs', '[uncertain]', 'duma'),
        'E76360': ('שַׂעַר', 'subs', 'bristling', "sa'ar"),
        'E76361': ('עמם', 'verb', 'ally', "m'm"),
        'E76362': ('יִפְעָה', 'subs', 'splendour', 'yifa'),
        'E76363': ('תָּכְנִית', 'subs', 'example', 'tachnit'),
        'E76364': ('מְסֻכָה', 'subs', 'covering', "m'sucho"),
        'E76365': ('בָּרְקַת', 'subs', 'beryl', 'barkat'),
        'E76366': ('נֶקֶב', 'subs', 'passage', 'nekev'),
        'E76367': ('מִמְשַׁח', 'subs', '[uncertain]', 'mimshach'),
        'E76368': ('רַאֲוָה', 'subs', 'seeing', "ra'ava"),
        'E76369': ('תַּנִּים', 'subs', 'sea-monster', 'tanim'),
        'E76370': ('מִגְדֹּל סְוֵנֵה',
                   'nmpr',
                   'from Migdol to Aswan',
                   "migdol s'veinei"),
        'E76371': ('הָהּ', 'intj', 'alas', 'hah'),
        'E76372': ('יְסֹדֹות', 'subs', 'foundation', "y'sodovt"),
        'E76373': ('כּוּב', 'nmpr', 'Libya', 'kuv'),
        'E76374': ('סִין', 'nmpr', 'Sin', 'sin'),
        'E76375': ('אָוֶן', 'nmpr', '[town]', 'aven'),
        'E76376': ('פִּי בֶסֶת', 'nmpr', 'Bubastis', 'pi veset'),
        'E76377': ('חִתּוּל', 'subs', 'bandage', 'chitul'),
        'E76378': ('נאק', 'verb', 'groan', 'nk'),
        'E76379': ('צלל', 'verb', 'grow dark', "tsl'l"),
        'E76380': ('גָּבֵהַּ', 'adjv', 'tall', 'gaveiha'),
        'E76381': ('סַרְעַפָּה', 'subs', 'bough', 'sarapa'),
        'E76382': ('סְעַפָּה', 'subs', 'bough', "s'apa"),
        'E76383': ('עֻלְפֶּה', 'subs', '[uncertain]', 'ulpeh'),
        'E76384': ('דלח', 'verb', 'make turbid', 'dlch'),
        'E76385': ('רפס', 'verb', '[uncertain]', 'rfs'),
        'E76386': ('רָמוּת', 'subs', 'height', 'ramut'),
        'E76387': ('צָפָה', 'subs', '[uncertain]', 'tsafa'),
        'E76388': ('עפף', 'verb', 'double', "f'f"),
        'E76389': ('נהה', 'verb', 'lament', "nh'h"),
        'E76390': ('חַד', 'subs', '[uncertain]', 'chad'),
        'E76391': ('עֲגָבִים', 'subs', '[uncertain]', 'agavim'),
        'E76392': ('בַּקָּרָה', 'subs', 'care', 'bakara'),
        'E76393': ('מִשְׁקָע', 'subs', 'clear water', 'mishka'),
        'E76394': ('רפשׂ', 'verb', 'foul', 'rfs'),
        'E76395': ('מִרְפָּשׂ', 'subs', 'troubled water', 'mirpas'),
        'E76396': ('אַתֵּן', 'prps', 'you', 'atein'),
        'E76397': ('שִׁמֲמָה', 'subs', '[uncertain]', 'shimama'),
        'E76398': ('נֶאָצָה', 'subs', 'contumely', 'neatsa'),
        'E76399': ('רִאשָׁה', 'subs', 'early state', 'risha'),
        'E76400': ('לֵזוּ', 'prde', 'this there', 'leizu'),
        'E76401': ('קרם', 'verb', 'spread', 'krm'),
        'E76402': ('גֹּוג', 'nmpr', 'Gog', 'govg'),
        'E76403': ('פְּרָזָה', 'subs', 'village', "p'raza"),
        'E76404': ('מַדְרֵגָה', 'subs', 'steep way', 'madreiga'),
        'E76405': ('שׁשׁא', 'verb', 'walk', "sh'sh"),
        'E76406': ('הֲמֹונָה', 'nmpr', 'Hamonah', 'hamovna'),
        'E76407': ('מִבְנֶה', 'subs', 'structure', 'mivneh'),
        'E76408': ('בִּנְיָן', 'subs', 'building', 'binan'),
        'E76409': ('אִיתֹון', 'adjv', '[uncertain]', 'itovn'),
        'E76410': ('רִצְפָה', 'subs', 'pavement', 'ritsfa'),
        'E76411': ('שְׁפַתַּיִם', 'subs', 'packsaddles', "sh'fatayim"),
        'E76412': ('מוּסָב', 'subs', '[uncertain]', 'musav'),
        'E76413': ('גִּזְרָה', 'subs', 'room', 'gizra'),
        'E76414': ('בִּנְיָה', 'subs', 'building', 'bina'),
        'E76415': ('אַתִּיק', 'subs', 'gallery', 'atik'),
        'E76416': ('שָׂחִיף', 'subs', '[uncertain]', 'sachif'),
        'E76417': ('מַהֲלָךְ', 'subs', 'passage', 'mahalach'),
        'E76418': ('קָצוּר', 'adjv', 'shorter', 'katsur'),
        'E76419': ('גְּדֶרֶת', 'subs', 'wall', "g'deret"),
        'E76420': ('הֲגִינָה', 'subs', '[uncertain]', 'hagina'),
        'E76421': ('צוּרָה', 'subs', 'fashion', 'tsura'),
        'E76422': ('תְּכוּנָה', 'subs', 'fixed place', "t'chuna"),
        'E76423': ('עֲזָרָה', 'subs', 'barrier', 'azara'),
        'E76424': ('הַרְאֵל', 'subs', '[uncertain]', 'hareil'),
        'E76425': ('אֲרִיאֵל', 'subs', 'fire-place', 'arieil'),
        'E76426': ('יֶזַע', 'subs', 'sweat', 'yeza'),
        'E76427': ('כסם', 'verb', 'clip', 'chsm'),
        'E76428': ('גְּרֻשָׁה', 'subs', 'expropriation', "g'rusha"),
        'E76429': ('שׁשׁה', 'verb', '[uncertain]', "sh'shh"),
        'E76430': ('פֶּתִי', 'subs', 'young man', 'peti'),
        'E76431': ('רסס', 'verb', 'sprinkle', "rs's"),
        'E76432': ('קְטֻרֹות', 'subs', '[uncertain]', "k'turovt"),
        'E76433': ('מְבַשְּׁלֹות', 'subs', 'cooking places', "m'bashlovt"),
        'E76434': ('פכה', 'verb', 'trickle', 'fchh'),
        'E76435': ('אָפְסַיִם', 'subs', 'ankles', 'afsayim'),
        'E76436': ('שָׂחוּ', 'subs', 'swimming', 'sachu'),
        'E76437': ('קַדְמֹון', 'adjv', 'eastern', 'kadmovn'),
        'E76438': ('דַּוָּג', 'subs', 'fisher', 'dauag'),
        'E76439': ('עֵין עֶגְלַיִם', 'nmpr', 'En Eglaim', 'ein eglayim'),
        'E76440': ('מִשְׁטֹוחַ', 'subs', 'scattering', 'mishtovach'),
        'E76441': ('בִּצָּה', 'subs', 'marshland', 'bitsa'),
        'E76442': ('תְּרוּפָה', 'subs', 'healing', "t'rufa"),
        'E76443': ('חֶתְלֹן', 'nmpr', 'Hethlon', 'chetlon'),
        'E76444': ('בֵּרֹותָה', 'nmpr', 'Berothah', 'beirovta'),
        'E76445': ('סִבְרַיִם', 'nmpr', 'Sibraim', 'sivrayim'),
        'E76446': ('חַוְרָן', 'nmpr', 'Hauran', 'chavran'),
        'E76447': ('חֲצַר עֵינֹון', 'nmpr', 'Hazar Enan', 'chatsar einovn'),
        'E76448': ('תָּמָר', 'nmpr', 'Tamar', 'tamar'),
        'E76449': ('תְּרוּמִיָּה', 'subs', 'contribution', "t'rumia"),
        'E76450': ('יְחִזְקִיָּה', 'nmpr', 'Hezekiah', "y'chizkia"),
        'E76451': ('גֹּמֶר', 'nmpr', 'Gomer', 'gomer'),
        'E76452': ('דִּבְלָיִם', 'nmpr', 'Diblaim', 'divlayim'),
        'E76453': ('יִזְרְעֶאל', 'nmpr', 'Jezreel', 'yizrel'),
        'E76454': ('לֹא רֻחָמָה', 'nmpr', 'Lo-Ruhamah', 'lo ruchama'),
        'E76455': ('לֹא עַמִּי', 'nmpr', 'Lo-Ammi', 'lo ami'),
        'E76456': ('עַמִּי', 'nmpr', 'Ammi', 'ami'),
        'E76457': ('רֻחָמָה', 'nmpr', 'Ruhamah', 'ruchama'),
        'E76458': ('נַאֲפוּפִים', 'subs', 'adultery', "na'afufim"),
        'E76459': ('שִׁקּוּי', 'subs', 'drink', 'shikuy'),
        'E76460': ('שׂוך', 'verb', 'fence', 'svch'),
        'E76461': ('נַבְלוּת', 'subs', 'vagina', 'navlut'),
        'E76462': ('אֶתְנָה', 'subs', 'reward', 'etna'),
        'E76463': ('חֶלְיָה', 'subs', 'ornament', 'chelya'),
        'E76464': ('לֵתֶךְ', 'subs', 'letek', 'leitech'),
        'E76465': ('לבט', 'verb', 'trample', 'lvt'),
        'E76466': ('מָגֵן', 'adjv', 'insolent', 'magein'),
        'E76467': ('שַׁחֲטָה', 'subs', 'oppression', 'shachata'),
        'E76468': ('שֵׂט', 'subs', '[uncertain]', 'seit'),
        'E76469': ('עָשׁ', 'subs', 'putrefaction', 'ash'),
        'E76470': ('רָקָב', 'subs', 'rottenness', 'rakav'),
        'E76471': ('יָרֵב', 'nmpr', 'the great king', 'yareiv'),
        'E76472': ('גהה', 'verb', 'heal', "gh'h"),
        'E76473': ('שַׁחַל', 'subs', 'young lion', 'shachal'),
        'E76474': ('ירה', 'verb', '[uncertain]', 'yrh'),
        'E76475': ('כַּחַשׁ', 'subs', 'leanness', 'kachash'),
        'E76476': ('זרק', 'verb', 'be white', 'zrk'),
        'E76477': ('זֹו', 'prde', 'this', 'zov'),
        'E76478': ('זנח', 'verb', 'reject', 'znch'),
        'E76479': ('שׂור', 'verb', '[uncertain]', 'svr'),
        'E76480': ('שָׁבָב', 'subs', 'splinter', 'shavav'),
        'E76481': ('אַהַב', 'subs', 'love', 'ahav'),
        'E76482': ('הַבְהָבַי', 'subs', '[uncertain]', 'havhavay'),
        'E76483': ('מֹף', 'nmpr', 'Memphis', 'mof'),
        'E76484': ('מַשְׂטֵמָה', 'subs', 'animosity', 'masteima'),
        'E76485': ('אֹהַב', 'subs', 'display of love', 'ohav'),
        'E76486': ('הֵרָיֹון', 'subs', 'conception', 'heirayovn'),
        'E76487': ('צמק', 'verb', 'shrivel', 'tsmk'),
        'E76488': ('שׁוה', 'verb', 'place', 'shvh'),
        'E76489': ('תֶּלֶם', 'subs', 'furrow', 'telem'),
        'E76490': ('בָּשְׁנָה', 'subs', 'shame', 'bashna'),
        'E76491': ('קֶצֶף', 'subs', 'bough', 'ketsef'),
        'E76492': ('עַלְוָה', 'subs', 'disobedience', 'alva'),
        'E76493': ('שַׁלְמַן', 'nmpr', 'Shalman', 'shalman'),
        'E76494': ('בֵּית אַרְבֵאל', 'nmpr', 'Beth Arbel', 'beit arveil'),
        'E76495': ('רְתֵת', 'subs', 'trembling', "r'teit"),
        'E76496': ('תַּלְאֻבֹות', 'subs', 'fevers', 'taluvovt'),
        'E76497': ('סְגֹור', 'subs', 'enclosure', "s'govr"),
        'E76498': ('אֱהִי', 'inrg', 'where', 'ehi'),
        'E76499': ('דֶּבֶר', 'subs', 'sting', 'dever'),
        'E76500': ('נֹחַם', 'subs', 'compassion', 'nocham'),
        'E76501': ('פרא', 'verb', 'be fertile', 'fr'),
        'E76502': ('פְּתוּאֵל', 'nmpr', 'Pethuel', "p'tueil"),
        'E76503': ('גָּזָם', 'subs', 'locust', 'gazam'),
        'E76504': ('מְתַלְּעֹות', 'subs', 'jaw-bones', "m'talovt"),
        'E76505': ('קְצָפָה', 'subs', 'stump', "k'tsafa"),
        'E76506': ('אלה', 'verb', 'wail', 'lh'),
        'E76507': ('תַּפּוּחַ', 'subs', 'apple-tree', 'tapuach'),
        'E76508': ('עבשׁ', 'verb', 'shrivel', 'vsh'),
        'E76509': ('פְּרֻדֹות', 'subs', 'dried figs', "p'rudovt"),
        'E76510': ('מֶגְרָפָה', 'subs', 'shovel', 'megrafa'),
        'E76511': ('מַמְּגֻרָה', 'subs', 'pond', 'mamgura'),
        'E76512': ('ערג', 'verb', 'long for', 'rg'),
        'E76513': ('פָּארוּר', 'subs', 'glow', 'parur'),
        'E76514': ('עבט', 'verb', 'change', 'vt'),
        'E76515': ('שֶׁלַח', 'subs', 'missile', 'shelach'),
        'E76516': ('צְפֹונִי', 'adjv', 'northern', "ts'fovni"),
        'E76517': ('סֹוף', 'subs', 'end', 'sovf'),
        'E76518': ('צַחֲנָה', 'subs', 'smell of putrefaction', 'tsachana'),
        'E76519': ('שׁוק', 'verb', 'be narrow', 'shvk'),
        'E76520': ('תִּימָרָה', 'subs', 'column', 'timara'),
        'E76521': ('ידד', 'verb', 'cast lot', "yd'd"),
        'E76522': ('יְוָנִי', 'adjv', 'Greek', "y'vani"),
        'E76523': ('שְׁבָאִים', 'nmpr', 'Sabeans', "sh'vaim"),
        'E76524': ('חַלָּשׁ', 'subs', 'weak man', 'chalash'),
        'E76525': ('עושׁ', 'verb', '[uncertain]', 'vsh'),
        'E76526': ('עָמֹוס', 'nmpr', 'Amos', 'amovs'),
        'E76527': ('בִּקְעַת אָוֶן', 'nmpr', 'the Valley of Aven', 'bikat aven'),
        'E76528': ('בֵּית עֶדֶן', 'nmpr', 'Beth Eden', 'beit eden'),
        'E76529': ('עֲנוּשִׁים', 'subs', 'fines', 'anushim'),
        'E76530': ('עוק', 'verb', 'totter', 'vk'),
        'E76531': ('עֲשׁוּקִים', 'subs', 'oppression', 'ashukim'),
        'E76532': ('בָּדָל', 'subs', 'piece', 'badal'),
        'E76533': ('פֵּאָה', 'subs', 'splendour', 'peia'),
        'E76534': ('דְּמֶשֶׁק', 'subs', '[uncertain]', "d'meshek"),
        'E76535': ('צֵן', 'subs', '[uncertain]', 'tsein'),
        'E76536': ('דּוּגָה', 'subs', 'fishery', 'duga'),
        'E76537': ('הַרְמֹון', 'nmpr', 'Harmon', 'harmovn'),
        'E76538': ('שֵׂחַ', 'subs', 'thought', 'seiach'),
        'E76539': ('עֵיפָה', 'subs', 'darkness', 'eifa'),
        'E76540': ('כִּימָה', 'subs', 'pleiads', 'kima'),
        'E76541': ('בלג', 'verb', 'gleam', 'vlg'),
        'E76542': ('בשׁס', 'verb', 'draw farm-rent', 'vshs'),
        'E76543': ('הֹו', 'intj', 'ah', 'hov'),
        'E76544': ('אָפֵל', 'subs', 'darkness', 'afeil'),
        'E76545': ('סִכּוּת', 'nmpr', 'Sakkuth', 'sikut'),
        'E76546': ('כִּיּוּן', 'nmpr', 'Kaiwan', 'kiun'),
        'E76547': ('פרט', 'verb', 'improvise', 'frt'),
        'E76548': ('תאב', 'verb', '[uncertain]', 'tv'),
        'E76549': ('סרף', 'verb', 'burn', 'srf'),
        'E76550': ('רָסִיס', 'subs', 'fragment', 'rasis'),
        'E76551': ('קַרְנָיִם', 'nmpr', 'Karnaim', 'karnayim'),
        'E76552': ('גֹּבַי', 'subs', 'swarm', 'govay'),
        'E76553': ('לֶקֶשׁ', 'subs', 'late sowing', 'lekesh'),
        'E76554': ('אֲנָךְ', 'subs', 'plummet', 'anach'),
        'E76555': ('בֹּוקֵר', 'subs', 'herdsman', 'bovkeir'),
        'E76556': ('בלס', 'verb', 'nip', 'vls'),
        'E76557': ('עות', 'verb', 'be crooked', 'vt'),
        'E76558': ('מַפָּל', 'subs', 'refuse', 'mapal'),
        'E76559': ('כְּבָרָה', 'subs', 'sieve', "k'vara"),
        'E76560': ('הֲרִיסָה', 'subs', 'ruin', 'harisa'),
        'E76561': ('עֹבַדְיָה', 'nmpr', 'Obadiah', 'ovadya'),
        'E76562': ('מַצְפּוּנִים', 'subs', 'hidden treasures', 'matspunim'),
        'E76563': ('מָזֹור', 'subs', '[uncertain]', 'mazovr'),
        'E76564': ('קֶטֶל', 'subs', 'slaughter', 'ketel'),
        'E76565': ('נֵכֶר', 'subs', 'misfortune', 'neicher'),
        'E76566': ('פֶּרֶק', 'subs', 'plunder', 'perek'),
        'E76567': ('לעע', 'verb', 'sip', 'l'),
        'E76568': ('סְפָרַד', 'nmpr', 'Sepharad', "s'farad"),
        'E76569': ('סְפִינָה', 'subs', 'vessel', "s'fina"),
        'E76570': ('עשׁת', 'verb', 'think', 'sht'),
        'E76571': ('שׁתק', 'verb', 'be silent', 'shtk'),
        'E76572': ('קְרִיאָה', 'subs', 'proclamation', "k'ria"),
        'E76573': ('קִיקָיֹון', 'subs', 'qiqayon', 'kikayovn'),
        'E76574': ('חֲרִישִׁי', 'adjv', '[uncertain]', 'charishi'),
        'E76575': ('עמל', 'verb', 'labour', 'ml'),
        'E76576': ('רִבֹּוא', 'subs', 'ten thousand', 'ribov'),
        'E76577': ('דֹּונַג', 'subs', 'wax', 'dovnag'),
        'E76578': ('שֹׁולָל', 'adjv', 'barefoot', 'shovlal'),
        'E76579': ('עַפְרָה', 'nmpr', 'Aphrah', 'afra'),
        'E76580': ('שָׁפִיר', 'nmpr', 'Shaphir', 'shafir'),
        'E76581': ('צַאֲנָן', 'nmpr', 'Zaanan', "tsa'anan"),
        'E76582': ('בֵּית הָאֵצֶל', 'nmpr', 'Beth Ezel', 'beit haeitsel'),
        'E76583': ('עֶמְדָּה', 'subs', 'standing place', 'emda'),
        'E76584': ('מָרֹות', 'nmpr', 'Maroth', 'marovt'),
        'E76585': ('רתם', 'verb', 'bind', 'rtm'),
        'E76586': ('מֹורֶשֶׁת גַּת', 'nmpr', 'Moresheth Gath', 'movreshet gat'),
        'E76587': ('תַּעֲנוּג', 'subs', 'comfort', "ta'anug"),
        'E76588': ('רֹומָה', 'subs', 'haughtiness', 'rovma'),
        'E76589': ('אֶדֶר', 'subs', 'might', 'eder'),
        'E76590': ('חֶבֶל', 'subs', 'destruction', 'chevel'),
        'E76591': ('בָּצְרָה', 'subs', '[uncertain]', 'batsra'),
        'E76592': ('פצח', 'verb', 'scatter', 'ftsch'),
        'E76593': ('מַעֲנֶה', 'subs', 'answer', "ma'aneh"),
        'E76594': ('הלא', 'verb', '[uncertain]', 'hl'),
        'E76595': ('מֶמְשָׁלָה', 'subs', 'dominion', 'memshala'),
        'E76596': ('אֶפְרָתָה', 'nmpr', 'Ephrathah', 'efrata'),
        'E76597': ('צנע', 'verb', 'be cautious', 'tsn'),
        'E76598': ('רְמִיָּה', 'subs', 'deceit', "r'mia"),
        'E76599': ('יֶשַׁח', 'subs', '[uncertain]', 'yeshach'),
        'E76600': ('אַלְלַי', 'intj', 'alas', "al'lay"),
        'E76601': ('הַוָּה', 'subs', 'desire', 'haua'),
        'E76602': ('עבת', 'verb', 'pervert', 'vt'),
        'E76603': ('חֵדֶק', 'subs', 'nightshade', 'cheidek'),
        'E76604': ('מְסוּכָה', 'subs', 'hedge', "m'sucho"),
        'E76605': ('חָמֹות', 'subs', 'mother-in-law', 'chamovt'),
        'E76606': ('נַחוּם', 'nmpr', '[prophet]', 'nachum'),
        'E76607': ('אֶלְקֹשִׁי', 'adjv', 'Elkoshite', 'elkoshi'),
        'E76608': ('שְׂעָרָה', 'subs', 'storm', "s'ara"),
        'E76609': ('שֶׁטֶף', 'subs', 'flood', 'shetef'),
        'E76610': ('סבך', 'verb', 'be interwoven', 'svch'),
        'E76611': ('תלע', 'verb', 'wrap in scarlet', 'tl'),
        'E76612': ('פְּלָדֹת', 'subs', '[uncertain]', "p'ladot"),
        'E76613': ('רעל', 'verb', 'quiver', 'rl'),
        'E76614': ('הֲלִיכָה', 'subs', 'walk', 'halicho'),
        'E76615': ('סֹכֵךְ', 'subs', 'barricade', 'socheich'),
        'E76616': ('נהג', 'verb', 'moan', 'nhg'),
        'E76617': ('תפף', 'verb', 'beat', "tf'f"),
        'E76618': ('בּוּקָה', 'subs', 'emptiness', 'buka'),
        'E76619': ('מְבוּקָה', 'subs', 'emptiness', "m'vuka"),
        'E76620': ('פִּק', 'subs', 'tottering', 'pik'),
        'E76621': ('לְבָאָה', 'subs', 'lionness', "l'va'a"),
        'E76622': ('דהר', 'verb', 'dash', 'dhr'),
        'E76623': ('נֹא אָמֹון', 'nmpr', 'Thebes', 'no amovn'),
        'E76624': ('לוּבִי', 'adjv', 'Libyan', 'luvi'),
        'E76625': ('רתק', 'verb', 'bind', 'rtk'),
        'E76626': ('מִנְּזָרִים', 'subs', 'guardsmen', 'minzarim'),
        'E76627': ('גֹּוב', 'subs', '[uncertain]', 'govv'),
        'E76628': ('קָרָה', 'subs', 'cold', 'kara'),
        'E76629': ('כֵּהָה', 'subs', 'effacement', 'keiha'),
        'E76630': ('חֲבַקּוּק', 'nmpr', 'Habakkuk', 'chavakuk'),
        'E76631': ('עקל', 'verb', 'crook', 'kl'),
        'E76632': ('אָיֹם', 'adjv', 'frightful', 'ayom'),
        'E76633': ('מְגַמַּת', 'subs', '[uncertain]', "m'gamat"),
        'E76634': ('מִשְׂחָק', 'subs', 'laughter', 'mischak'),
        'E76635': ('יָהִיר', 'adjv', 'haughty', 'yahir'),
        'E76636': ('נוה', 'verb', 'reach', 'nvh'),
        'E76637': ('מְלִיצָה', 'subs', 'saying', "m'litsa"),
        'E76638': ('עַבְטִיט', 'subs', 'debt', 'avtit'),
        'E76639': ('זוע', 'verb', 'tremble', 'zv'),
        'E76640': ('כָּפִיס', 'subs', 'stucco-work', 'kafis'),
        'E76641': ('מָעֹור', 'subs', 'nakedness', 'maovr'),
        'E76642': ('קִיקָלֹון', 'subs', 'disgrace', 'kikalovn'),
        'E76643': ('שִׁגָּיֹון', 'subs', 'dirge', 'shigayovn'),
        'E76644': ('סֶלָה', 'intj', 'sela', 'sela'),
        'E76645': ('חֶבְיֹון', 'subs', 'veil', 'chevyovn'),
        'E76646': ('כּוּשָׁן', 'nmpr', 'Cushan', 'kushan'),
        'E76647': ('עור', 'verb', 'be naked', 'vr'),
        'E76648': ('אֹמֶר', 'subs', 'saying', 'omer'),
        'E76649': ('רֹום', 'subs', '[uncertain]', 'rovm'),
        'E76650': ('פָּרָז', 'subs', '[uncertain]', 'paraz'),
        'E76651': ('עֲלִיצֻת', 'subs', 'presumption', 'alitsut'),
        'E76652': ('נוח', 'verb', 'lament', 'nvch'),
        'E76653': ('מִכְלָא', 'subs', 'enclosure', 'michla'),
        'E76654': ('רֶפֶת', 'subs', 'enclosure', 'refet'),
        'E76655': ('אֲמַרְיָה', 'nmpr', 'Amariah', 'amarya'),
        'E76656': ('נָטִיל', 'adjv', 'weighing', 'natil'),
        'E76657': ('מְצוּקָה', 'subs', 'stress', "m'tsuka"),
        'E76658': ('מְשֹׁואָה', 'subs', 'desolation', "m'shova"),
        'E76659': ('לְחוּם', 'subs', '[uncertain]', "l'chum"),
        'E76660': ('עֲנָוָה', 'subs', 'humility', 'anava'),
        'E76661': ('כָּרָה', 'subs', '[uncertain]', 'kara'),
        'E76662': ('מִמְשָׁק', 'subs', '[uncertain]', 'mimshak'),
        'E76663': ('חָרוּל', 'subs', 'chickling', 'charul'),
        'E76664': ('מִכְרֶה', 'subs', 'pit', 'michreh'),
        'E76665': ('אַרְזָה', 'subs', 'wainscot', 'arza'),
        'E76666': ('מַרְבֵּץ', 'subs', 'resting place', 'marbeits'),
        'E76667': ('מרא', 'verb', 'be rebellious', 'mr'),
        'E76668': ('בֹּגְדֹות', 'subs', 'treachery', 'bogdovt'),
        'E76669': ('עַוָּל', 'subs', 'evildoer', 'aual'),
        'E76670': ('צדה', 'verb', 'waste', 'tsdh'),
        'E76671': ('בָּרוּר', 'adjv', 'pure', 'barur'),
        'E76672': ('פּוּצַי', 'subs', '[uncertain]', 'putsay'),
        'E76673': ('דָּרְיָוֶשׁ', 'nmpr', 'Darius', 'daryavesh'),
        'E76674': ('חַגַּי', 'nmpr', 'Haggai', 'chagay'),
        'E76675': ('זְרֻבָּבֶל', 'nmpr', 'Zerubbabel', "z'rubavel"),
        'E76676': ('שְׁאַלְתִּיאֵל', 'nmpr', 'Shealtiel', "sh'altieil"),
        'E76677': ('יְהֹוצָדָק', 'nmpr', 'Jehozadak', "y'hovtsadak"),
        'E76678': ('מַלְאָכוּת', 'subs', 'commission', 'malachut'),
        'E76679': ('מְגוּרָה', 'subs', 'corn-pit', "m'gura"),
        'E76680': ('בֶּרֶכְיָה', 'nmpr', 'Berekiah', 'berechya'),
        'E76681': ('עִדֹּו', 'nmpr', 'Iddo', 'idov'),
        'E76682': ('שְׁבָט', 'nmpr', 'Shebat', "sh'vat"),
        'E76683': ('בֶּרֶכְיָהוּ', 'nmpr', 'Berekiah', 'berechyahu'),
        'E76684': ('בָּבָה', 'subs', 'eyeball', 'bava'),
        'E76685': ('שׂטן', 'verb', 'bear grudge', 'stn'),
        'E76686': ('צֹאִי', 'adjv', 'filthy', 'tsoi'),
        'E76687': ('מוּצֶקֶת', 'subs', 'casting', 'mutseket'),
        'E76688': ('רֹאשָׁה', 'adjv', 'upper', 'rosha'),
        'E76689': ('צַנְתְּרֹות', 'subs', 'pipes', 'tsantrovt'),
        'E76690': ('אָמֹץ', 'adjv', 'piebald', 'amots'),
        'E76691': ('חֶלְדַּי', 'nmpr', 'Heldai', 'chelday'),
        'E76692': ('טֹובִיָּה', 'nmpr', 'Tobijah', 'tovbia'),
        'E76693': ('יְדַעְיָה', 'nmpr', 'Jedaiah', "y'daya"),
        'E76694': ('יֹאשִׁיָּה', 'nmpr', 'Josiah', 'yoshia'),
        'E76695': ('חֵלֶם', 'nmpr', 'Heldai', 'cheilem'),
        'E76696': ('חֵן', 'nmpr', 'Hen', 'chein'),
        'E76697': ('כִּסְלֵו', 'nmpr', 'Kislev', 'kisleiv'),
        'E76698': ('רֶגֶם מֶלֶךְ', 'nmpr', 'Regem-Melech', 'regem melech'),
        'E76699': ('חַדְרָךְ', 'nmpr', 'Hadrach', 'chadrach'),
        'E76700': ('חָרוּץ', 'subs', 'gold', 'charuts'),
        'E76701': ('מֹשֶׁל', 'subs', 'dominion', 'moshel'),
        'E76702': ('בִּצָּרֹון', 'subs', '[uncertain]', 'bitsarovn'),
        'E76703': ('זָוִית', 'subs', 'corner', 'zavit'),
        'E76704': ('נוב', 'verb', 'thrive', 'nvv'),
        'E76705': ('חֲזִיז', 'subs', 'storm-cloud', 'chaziz'),
        'E76706': ('בָּצִיר', 'subs', 'forest', 'batsir'),
        'E76707': ('נֹעַם', 'subs', 'kindness', 'noam'),
        'E76708': ('חֹבְלִים', 'subs', 'union', 'chovlim'),
        'E76709': ('בחל', 'verb', '[uncertain]', 'vchl'),
        'E76710': ('אַחֲוָה', 'subs', 'brotherhood', 'achava'),
        'E76711': ('אֱוִלִי', 'adjv', 'awkward', 'evili'),
        'E76712': ('רַעַל', 'subs', 'reeling', "ra'al"),
        'E76713': ('מַעֲמָסָה', 'subs', 'heavy stone', "ma'amasa"),
        'E76714': ('אַמְצָה', 'subs', 'strength', 'amtsa'),
        'E76715': ('הֲדַד רִמֹּון', 'nmpr', 'Hadad Rimmon', 'hadad rimovn'),
        'E76716': ('אָצַל', 'nmpr', 'Azel', 'atsal'),
        'E76717': ('קִפָּאֹון', 'subs', 'frost', 'kipaovn'),
        'E76718': ('ראם', 'verb', 'rise', 'rm'),
        'E76719': ('מְצִלָּה', 'subs', 'bell', "m'tsila"),
        'E76720': ('מַלְאָכִי', 'nmpr', 'Malachi', 'malachi'),
        'E76721': ('אֲנָקָה', 'subs', 'sigh', 'anaka'),
        'E76722': ('חֲבֶרֶת', 'subs', 'companion', 'chaveret'),
        'E76723': ('קבע', 'verb', 'deceive', 'kv'),
        'E76724': ('טֶרֶף', 'subs', 'food', 'teref'),
        'E76725': ('קְדֹרַנִּית', 'advb', 'mourningly', "k'doranit"),
        'E76726': ('עסס', 'verb', 'tread down', "s's"),
        'E76727': ('רגשׁ', 'verb', 'be in turmoil', 'rgsh'),
        'E76728': ('יסד', 'verb', 'close', 'ysd'),
        'E76729': ('בַּר', 'subs', 'son', 'bar'),
        'E76730': ('מִזְמֹור', 'subs', 'psalm', 'mizmovr'),
        'E76731': ('נְחִילֹות', 'subs', '[uncertain]', "n'chilovt"),
        'E76732': ('הָגִיג', 'subs', 'groaning', 'hagig'),
        'E76733': ('שֶׁוַע', 'subs', 'cry', 'sheva'),
        'E76734': ('שֹׁורֵר', 'subs', 'foe', 'shovreir'),
        'E76735': ('הַוָּה', 'subs', 'destruction', 'haua'),
        'E76736': ('אֻמְלַל', 'adjv', 'fading away', 'umlal'),
        'E76737': ('עשׁשׁ', 'verb', 'become weak', "sh'sh"),
        'E76738': ('גמר', 'verb', 'end', 'gmr'),
        'E76739': ('חבל', 'verb', 'be pregnant', 'chvl'),
        'E76740': ('גִּתִּית', 'subs', '[uncertain]', 'gitit'),
        'E76741': ('צֹנֶה', 'subs', 'small cattle', 'tsoneh'),
        'E76742': ('עַלְמוּת', 'subs', '[uncertain]', 'almut'),
        'E76743': ('דַּךְ', 'adjv', 'oppressed', 'dach'),
        'E76744': ('הִגָּיֹון', 'subs', 'groaning', 'higayovn'),
        'E76745': ('מֹורָה', 'subs', '[uncertain]', 'movra'),
        'E76746': ('חיל', 'verb', 'endure', 'chyl'),
        'E76747': ('תֹּךְ', 'subs', 'oppression', 'toch'),
        'E76748': ('חֵלְכָה', 'subs', '[uncertain]', 'cheilcho'),
        'E76749': ('דכה', 'verb', 'crush', 'dchh'),
        'E76750': ('כָּאֶה', 'subs', '[uncertain]', 'kaeh'),
        'E76751': ('זַלְעָפָה', 'subs', 'irritation', 'zalafa'),
        'E76752': ('פסס', 'verb', '[uncertain]', "fs's"),
        'E76753': ('עֲלִיל', 'subs', 'entrance', 'alil'),
        'E76754': ('זֻלֻּת', 'subs', 'vileness', 'zulut'),
        'E76755': ('עֵצָה', 'subs', 'rebellion', 'eitsa'),
        'E76756': ('אלח', 'verb', 'be corrupt', 'lch'),
        'E76757': ('מִכְתָּם', 'subs', '[uncertain]', 'michtam'),
        'E76758': ('עַצָּבָה', 'subs', 'pain', 'atsava'),
        'E76759': ('שׁפר', 'verb', 'be pleasing', 'shfr'),
        'E76760': ('אַשּׁוּר', 'subs', 'step', 'ashur'),
        'E76761': ('דִּמְיֹן', 'subs', 'likeness', 'dimon'),
        'E76762': ('חֶלֶד', 'subs', 'duration of life', 'cheled'),
        'E76763': ('חֵזֶק', 'subs', 'strength', 'cheizek'),
        'E76764': ('חרג', 'verb', 'come out', 'chrg'),
        'E76765': ('דבר', 'verb', 'push back', 'dvr'),
        'E76766': ('נבע', 'verb', 'bubble', 'nv'),
        'E76767': ('חוה', 'verb', 'make known', 'chvh'),
        'E76768': ('פִּקּוּדִים', 'subs', 'orders', 'pikudim'),
        'E76769': ('בַּר', 'adjv', 'pure', 'bar'),
        'E76770': ('נֹפֶת', 'subs', 'honey', 'nofet'),
        'E76771': ('צוּף', 'subs', 'honey-comb', 'tsuf'),
        'E76772': ('שְׁגִיאָה', 'subs', 'sin', "sh'gia"),
        'E76773': ('דגל', 'verb', 'lift banner', 'dgl'),
        'E76774': ('מִשְׁאָלָה', 'subs', 'request', 'mishala'),
        'E76775': ('אֲרֶשֶׁת', 'subs', 'desire', 'areshet'),
        'E76776': ('דּוּמִיָּה', 'subs', 'silence', 'dumia'),
        'E76777': ('גחה', 'verb', '[uncertain]', 'gchh'),
        'E76778': ('מַלְקֹחַיִם', 'subs', 'jaw', 'malkochayim'),
        'E76779': ('עַצְמָה', 'subs', 'evil deed, pain', 'atsma'),
        'E76780': ('אֱיָלוּת', 'subs', 'power', 'eyalut'),
        'E76781': ('עֱנוּת', 'subs', 'affliction', 'enut'),
        'E76782': ('רְוָיָה', 'subs', 'saturation', "r'vaya"),
        'E76783': ('מַקְהֵל', 'subs', 'assembly', 'makheil'),
        'E76784': ('יָפֵחַ', 'subs', '[uncertain]', 'yafeiach'),
        'E76785': ('הֲדָרָה', 'subs', 'adornment', 'hadara'),
        'E76786': ('חשׂף', 'verb', 'accelerate', 'chsf'),
        'E76787': ('יַעֲרָה', 'subs', 'kid', "ya'ara"),
        'E76788': ('שָׁלוּ', 'subs', 'quiet', 'shalu'),
        'E76789': ('רֹכֶס', 'subs', '[uncertain]', 'roches'),
        'E76790': ('גרז', 'verb', 'exterminate', 'grz'),
        'E76791': ('מַשְׂכִּיל', 'subs', '[uncertain]', 'maskil'),
        'E76792': ('חֲרָבֹון', 'subs', 'heat', 'charavovn'),
        'E76793': ('רֹן', 'subs', '[uncertain]', 'ron'),
        'E76794': ('בלם', 'verb', 'curb?', 'vlm'),
        'E76795': ('נָאוֶה', 'adjv', 'lovely', 'naveh'),
        'E76796': ('דחה', 'verb', 'push', 'dchh'),
        'E76797': ('נֵכֶה', 'subs', '[uncertain]', 'neicheh'),
        'E76798': ('חרק', 'verb', 'grind', 'chrk'),
        'E76799': ('שֹׁוא', 'subs', '[uncertain]', 'shov'),
        'E76800': ('קרץ', 'verb', 'twinkle', 'krts'),
        'E76801': ('רָגֵעַ', 'adjv', 'quiet', 'rageia'),
        'E76802': ('מלל', 'verb', 'wither', "ml'l"),
        'E76803': ('מִצְעָד', 'subs', 'step', 'mitsad'),
        'E76804': ('אִוֶּלֶת', 'subs', 'foolishness', 'iuelet'),
        'E76805': ('יְדוּתוּן', 'subs', '[uncertain]', "y'dutun"),
        'E76806': ('מַחְסֹום', 'subs', 'muzzle', 'machsovm'),
        'E76807': ('תֹּוחֶלֶת', 'subs', 'hope', 'tovchelet'),
        'E76808': ('תִּגְרָה', 'subs', 'irritation', 'tigra'),
        'E76809': ('שָׁאֹון', 'subs', 'waste', 'shaovn'),
        'E76810': ('יָוֵן', 'subs', 'sediment', 'yavein'),
        'E76811': ('רַהַב', 'subs', 'stormer', 'rahav'),
        'E76812': ('שׂוט', 'verb', 'fall away', 'svt'),
        'E76813': ('דְּוַי', 'subs', 'illness', "d'vay"),
        'E76814': ('סָךְ', 'subs', '[uncertain]', 'sach'),
        'E76815': ('שׁיח', 'verb', 'melt away', 'shych'),
        'E76816': ('חֶרְמֹונִים', 'subs', 'sacred places', 'chermovnim'),
        'E76817': ('מָנֹוד', 'subs', 'head-shaking', 'manovd'),
        'E76818': ('תַּעֲלֻמָה', 'subs', 'secret', "ta'aluma"),
        'E76819': ('יְדִידֹת', 'subs', 'love-song', "y'didot"),
        'E76820': ('רחשׁ', 'verb', 'be astir', 'rchsh'),
        'E76821': ('עַנְוָה', 'subs', '[uncertain]', 'anva'),
        'E76822': ('קְצִיעָה', 'subs', 'cassia', "k'tsia"),
        'E76823': ('שֵׁגַל', 'subs', 'queen', 'sheigal'),
        'E76824': ('מִפְעָל', 'subs', 'work', 'mifal'),
        'E76825': ('נֹוף', 'subs', 'height', 'novf'),
        'E76826': ('פסג', 'verb', 'pass between?', 'fsg'),
        'E76827': ('חָכְמֹות', 'subs', 'wisdom', 'chachmovt'),
        'E76828': ('הָגוּת', 'subs', 'meditation', 'hagut'),
        'E76829': ('כְּסִיל', 'adjv', 'insolent', "k'sil"),
        'E76830': ('בַּעַר', 'adjv', 'stupid', "ba'ar"),
        'E76831': ('כֶּסֶל', 'subs', 'imperturbability', 'kesel'),
        'E76832': ('מִכְלָל', 'subs', 'perfection', 'michlal'),
        'E76833': ('שׂער', 'verb', 'whirl', 'sr'),
        'E76834': ('זִיז', 'subs', 'swarm', 'ziz'),
        'E76835': ('דֳּפִי', 'subs', 'blemish', 'dafi'),
        'E76836': ('טֻחֹות', 'subs', '[uncertain]', 'tuchovt'),
        'E76837': ('בֶּלַע', 'subs', 'confusion', 'bela'),
        'E76838': ('חתה', 'verb', 'knock down', 'chth'),
        'E76839': ('מָחֲלַת', 'subs', '[uncertain]', 'machalat'),
        'E76840': ('עָקָה', 'subs', 'pressure', 'aka'),
        'E76841': ('מִפְלָט', 'subs', 'escape', 'miflat'),
        'E76842': ('סעה', 'verb', 'calumniate', "s'h"),
        'E76843': ('רֶגֶשׁ', 'subs', 'turmoil', 'regesh'),
        'E76844': ('מְגוּרָם', 'subs', '[uncertain]', "m'guram"),
        'E76845': ('מַחֲמָאֹת', 'subs', '[uncertain]', 'machamaot'),
        'E76846': ('פְּתִיחָה', 'subs', 'dagger', "p'ticha"),
        'E76847': ('יְהָב', 'subs', '[uncertain]', "y'hav"),
        'E76848': ('אֵלֶם', 'subs', '[uncertain]', 'eilem'),
        'E76849': ('נֹדִי', 'subs', '[uncertain]', 'nodi'),
        'E76850': ('סִפְרָה', 'subs', 'scroll', 'sifra'),
        'E76851': ('דְּחִי', 'subs', 'stumbling', "d'chi"),
        'E76852': ('חרף', 'verb', 'confuse', 'chrf'),
        'E76853': ('לָבָא', 'subs', 'lion', 'lava'),
        'E76854': ('שִׁיחָה', 'subs', 'pit', 'shicha'),
        'E76855': ('מאס', 'verb', 'dissolve', 'ms'),
        'E76856': ('שַׁבְּלוּל', 'subs', 'miscarriage', 'shablul'),
        'E76857': ('תֶּמֶס', 'subs', 'melting', 'temes'),
        'E76858': ('נֵפֶל', 'subs', 'miscarriage', 'neifel'),
        'E76859': ('פצם', 'verb', 'split', 'ftsm'),
        'E76860': ('קֹשֶׁט', 'subs', 'bow', 'koshet'),
        'E76861': ('רַחַץ', 'subs', 'washing', 'rachats'),
        'E76862': ('התת', 'verb', 'reprove', "ht't"),
        'E76863': ('כָּמַהּ', 'verb', 'faint', 'kamah'),
        'E76864': ('שׁבח', 'verb', 'praise', 'shvch'),
        'E76865': ('רְנָנָה', 'subs', 'cry of joy', "r'nana"),
        'E76866': ('אַשְׁמוּרָה', 'subs', 'night watch', 'ashmura'),
        'E76867': ('רִגְשָׁה', 'subs', 'turmoil', 'rigsha'),
        'E76868': ('חֵפֶשׂ', 'subs', '[uncertain]', 'cheifes'),
        'E76869': ('שׁבח', 'verb', 'be calm', 'shvch'),
        'E76870': ('גְּדוּד', 'subs', 'clod', "g'dud"),
        'E76871': ('עטף', 'verb', 'turn aside', 'tf'),
        'E76872': ('מוּעָקָה', 'subs', '[uncertain]', 'muaka'),
        'E76873': ('רֹומַם', 'subs', 'praise', 'rovmam'),
        'E76874': ('כֹּושָׁרָה', 'subs', 'prosperity', 'kovshara'),
        'E76875': ('צְחִיחָה', 'subs', 'naked land', "ts'chicha"),
        'E76876': ('נוף', 'verb', 'besprinkle', 'nvf'),
        'E76877': ('שׁלג', 'verb', 'snow', 'shlg'),
        'E76878': ('גַּבְנֹן', 'subs', 'knoll', 'gavnon'),
        'E76879': ('רצד', 'verb', 'watch', 'rtsd'),
        'E76880': ('שִׁנְאָן', 'subs', 'highness', 'shinan'),
        'E76881': ('מֹושָׁעֹות', 'subs', 'saving acts', 'movshaovt'),
        'E76882': ('מֵן', 'subs', 'portion', 'mein'),
        'E76883': ('רִגְמָה', 'subs', 'shouting crowd', 'rigma'),
        'E76884': ('רַץ', 'subs', '[uncertain]', 'rats'),
        'E76885': ('בזר', 'verb', 'scatter', 'vzr'),
        'E76886': ('חַשְׁמַנִּים', 'subs', 'bronze ware', 'chashmanim'),
        'E76887': ('תַּעֲצֻמֹות', 'subs', 'might', "ta'atsumovt"),
        'E76888': ('מָעֳמָד', 'subs', 'position', "ma'amad"),
        'E76889': ('חרר', 'verb', 'be hoarse', "chr'r"),
        'E76890': ('אטר', 'verb', 'close', 'tr'),
        'E76891': ('נושׁ', 'verb', '[uncertain]', 'nvsh'),
        'E76892': ('בָּרוּת', 'subs', 'bread of consolation', 'barut'),
        'E76893': ('חמץ', 'verb', 'oppress', 'chmts'),
        'E76894': ('גזה', 'verb', '[uncertain]', 'gzh'),
        'E76895': ('סְפֹרֹות', 'subs', '[uncertain]', "s'forovt"),
        'E76896': ('זַרְזִיף', 'subs', '[uncertain]', 'zarzif'),
        'E76897': ('פִּסָּה', 'subs', '[uncertain]', 'pisa'),
        'E76898': ('רעשׁ', 'verb', 'be abundant', 'rsh'),
        'E76899': ('נין', 'verb', 'sprout forth', 'nyn'),
        'E76900': ('אוּל', 'subs', '[uncertain]', 'ul'),
        'E76901': ('שִׁית', 'subs', 'garment', 'shit'),
        'E76902': ('מוק', 'verb', '[uncertain]', 'mvk'),
        'E76903': ('שׂגה', 'verb', 'be great', 'sgh'),
        'E76904': ('מַשּׁוּאֹות', 'subs', 'deceptions', 'mashuovt'),
        'E76905': ('רָחֵק', 'adjv', 'far', 'racheik'),
        'E76906': ('כַּשִּׁיל', 'subs', 'axe', 'kashil'),
        'E76907': ('כֵּילָף', 'subs', 'crowbar', 'keilaf'),
        'E76908': ('מֶסֶךְ', 'subs', 'mixture', 'mesech'),
        'E76909': ('בצר', 'verb', 'humble', 'vtsr'),
        'E76910': ('שְׁמֻרָה', 'subs', 'eyelid', "sh'mura"),
        'E76911': ('חנה', 'verb', 'be gracious', 'chnh'),
        'E76912': ('זרם', 'verb', 'flood thunderingly?', 'zrm'),
        'E76913': ('נשׁק', 'verb', 'be equipped', 'nshk'),
        'E76914': ('תוה', 'verb', 'hurt', 'tvh'),
        'E76915': ('חֲנָמַל', 'subs', 'flood', 'chanamal'),
        'E76916': ('מִשְׁלַחַת', 'subs', 'discharge', 'mishlachat'),
        'E76917': ('נָתִיב', 'subs', 'path', 'nativ'),
        'E76918': ('רון', 'verb', 'be sober', 'rvn'),
        'E76919': ('תְּמוּתָה', 'subs', 'death', "t'muta"),
        'E76920': ('ארה', 'verb', 'pluck', 'rh'),
        'E76921': ('כרסם', 'verb', 'eat away', 'chrsm'),
        'E76922': ('כַּנָּה', 'subs', '[uncertain]', 'kana'),
        'E76923': ('כֶּסֶא', 'subs', 'full moon', 'kese'),
        'E76924': ('יְהֹוסֵף', 'nmpr', 'Joseph', "y'hovseif"),
        'E76925': ('הַגְרִי', 'adjv', 'Hagrite', 'hagri'),
        'E76926': ('גְּבָל', 'nmpr', 'Gebal', "g'val"),
        'E76927': ('דְּרֹור', 'subs', 'swallow', "d'rovr"),
        'E76928': ('ספף', 'verb', 'stand at threshold', "sf'f"),
        'E76929': ('כִּסְלָה', 'subs', 'confidence', 'kisla'),
        'E76930': ('סַלָּח', 'adjv', 'forgiving', 'salach'),
        'E76931': ('יְסוּדָה', 'subs', 'foundation', "y'suda"),
        'E76932': ('אֱיָל', 'subs', 'strength', 'eyal'),
        'E76933': ('אֲבַדֹּון', 'subs', 'destruction', 'avadovn'),
        'E76934': ('נְשִׁיָּה', 'subs', 'oblivion', "n'shia"),
        'E76935': ('נֹעַר', 'subs', 'youth', 'noar'),
        'E76936': ('פון', 'verb', '[uncertain]', 'fvn'),
        'E76937': ('בִּעוּת', 'subs', 'terror', 'biut'),
        'E76938': ('חֲסִין', 'subs', '[uncertain]', 'chasin'),
        'E76939': ('נאר', 'verb', 'abandon', 'nr'),
        'E76940': ('רֹהַב', 'subs', 'insistence', 'rohav'),
        'E76941': ('חִישׁ', 'subs', 'haste', 'chish'),
        'E76942': ('סֹחֵרָה', 'subs', 'bulwark', 'socheira'),
        'E76943': ('שִׁלֻּמָה', 'subs', 'retribution', 'shiluma'),
        'E76944': ('דֳּכִי', 'subs', 'dashing', 'dachi'),
        'E76945': ('אמר', 'verb', 'boast', 'mr'),
        'E76946': ('דּוּמָה', 'subs', 'silence', 'duma'),
        'E76947': ('שַׂרְעַפִּים', 'subs', 'disquieting thoughts', 'sarapim'),
        'E76948': ('מֶחְקָר', 'subs', '[uncertain]', 'mechkar'),
        'E76949': ('נוט', 'verb', '[uncertain]', 'nvt'),
        'E76950': ('סֵטִים', 'subs', '[uncertain]', 'seitim'),
        'E76951': ('לשׁן', 'verb', 'slander', 'lshn'),
        'E76952': ('קֵד', 'subs', '[uncertain]', 'keid'),
        'E76953': ('שׁקד', 'verb', 'be emaciated', 'shkd'),
        'E76954': ('זָכוּר', 'adjv', 'remembering', 'zachur'),
        'E76955': ('רְכוּב', 'subs', 'chariot', "r'chuv"),
        'E76956': ('עֳפִי', 'subs', 'foliage', 'afi'),
        'E76957': ('צהל', 'verb', 'shine', "ts'hl"),
        'E76958': ('רעד', 'verb', 'tremble', 'rd'),
        'E76959': ('כֶּבֶל', 'subs', 'fetter', 'kevel'),
        'E76960': ('ערב', 'verb', 'mix', 'rv'),
        'E76961': ('מכך', 'verb', 'sink', "mch'ch"),
        'E76962': ('שְׁחִית', 'subs', 'pit', "sh'chit"),
        'E76963': ('מָחֹוז', 'subs', 'shipyard', 'machovz'),
        'E76964': ('מֵזַח', 'subs', 'girdle', 'meizach'),
        'E76965': ('מִשְׁחָר', 'subs', '[uncertain]', 'mishchar'),
        'E76966': ('יַלְדוּת', 'subs', 'youth', 'yaldut'),
        'E76967': ('דִּבְרָה', 'subs', 'cause', 'divra'),
        'E76968': ('לעז', 'verb', 'speak unintelligibly', 'lz'),
        'E76969': ('מֵצַר', 'subs', 'distress', 'meitsar'),
        'E76970': ('כֹּל', 'subs', 'whole', 'kol'),
        'E76971': ('תַּגְמוּל', 'subs', 'benefit', 'tagmul'),
        'E76972': ('מול', 'verb', 'be off', 'mvl'),
        'E76973': ('גרס', 'verb', 'be crushed', 'grs'),
        'E76974': ('תַּאֲבָה', 'subs', 'longing', "ta'ava"),
        'E76975': ('דלף', 'verb', 'be sleepless', 'dlf'),
        'E76976': ('תּוּגָה', 'subs', 'grief', 'tuga'),
        'E76977': ('תאב', 'verb', 'long for', 'tv'),
        'E76978': ('נֶחָמָה', 'subs', 'comfort', 'nechama'),
        'E76979': ('טפל', 'verb', 'soil', 'tfl'),
        'E76980': ('טפשׁ', 'verb', 'be unfeeling', 'tfsh'),
        'E76981': ('תִּכְלָה', 'subs', 'perfection', 'tichla'),
        'E76982': ('שִׂיחָה', 'subs', 'concern', 'sicha'),
        'E76983': ('מלץ', 'verb', 'be smooth', 'mlts'),
        'E76984': ('סֵעֵף', 'adjv', 'despicable', "sei'eif"),
        'E76985': ('שֵׂבֶר', 'subs', 'hope', 'seiver'),
        'E76986': ('סלה', 'verb', 'reject', 'slh'),
        'E76987': ('סמר', 'verb', 'bristle', 'smr'),
        'E76988': ('פֵּתַח', 'subs', 'disclosure', 'peitach'),
        'E76989': ('יאב', 'verb', 'long', 'yv'),
        'E76990': ('שׁלט', 'verb', 'dominate', 'shlt'),
        'E76991': ('אֹויָה', 'intj', 'woe', 'ovya'),
        'E76992': ('אֲזַי', 'advb', 'then', 'azay'),
        'E76993': ('זֵידֹון', 'adjv', 'running', 'zeidovn'),
        'E76994': ('מֶשֶׁךְ', 'subs', 'bag', 'meshech'),
        'E76995': ('שֵׁנָא', 'subs', 'sleep', 'sheina'),
        'E76996': ('שָׁתִיל', 'subs', 'layer', 'shatil'),
        'E76997': ('מַעֲנִית', 'subs', '[uncertain]', "ma'anit"),
        'E76998': ('עמר', 'verb', 'gather ears', 'mr'),
        'E76999': ('קַשֻּׁב', 'adjv', 'attentive', 'kashuv'),
        'E77000': ('סְלִיחָה', 'subs', 'forgiveness', "s'licha"),
        'E77001': ('תְּנוּמָה', 'subs', 'slumber', "t'numa"),
        'E77002': ('יַעַר', 'nmpr', 'Jaar', "ya'ar"),
        'E77003': ('שֵׁפֶל', 'subs', 'low state', 'sheifel'),
        'E77004': ('תֹּולָל', 'subs', '[uncertain]', 'tovlal'),
        'E77005': ('רֵעַ', 'subs', 'will', 'reia'),
        'E77006': ('זרה', 'verb', 'measure off', 'zrh'),
        'E77007': ('סלק', 'verb', 'ascend', 'slk'),
        'E77008': ('אֹורָה', 'subs', 'light', 'ovra'),
        'E77009': ('סכך', 'verb', 'weave', "s'ch'ch"),
        'E77010': ('עֹצֶם', 'subs', 'bones', 'otsem'),
        'E77011': ('גֹּלֶם', 'subs', 'embryo', 'golem'),
        'E77012': ('קטל', 'verb', 'kill', 'ktl'),
        'E77013': ('תַּכְלִית', 'subs', 'completeness', 'tachlit'),
        'E77014': ('עַכְשׁוּב', 'subs', 'cerastes', 'achshuv'),
        'E77015': ('מַאֲוַיִּים', 'subs', 'desire', "ma'avayim"),
        'E77016': ('זְמָם', 'subs', 'plan', "z'mam"),
        'E77017': ('מַהֲמֹרֹות', 'subs', 'rain-pits', 'mahamorovt'),
        'E77018': ('מַדְחֵפָה', 'subs', 'thrust', 'madcheifa'),
        'E77019': ('שָׁמְרָה', 'subs', 'guard', 'shamra'),
        'E77020': ('דַּל', 'subs', 'door', 'dal'),
        'E77021': ('מַנְעַמִּים', 'subs', 'dainties', 'manamim'),
        'E77022': ('ברק', 'verb', 'lighten', 'vrk'),
        'E77023': ('נָטִיעַ', 'subs', 'plantlet', 'natia'),
        'E77024': ('מָזוּ', 'subs', '[uncertain]', 'mazu'),
        'E77025': ('זַן', 'subs', 'kind', 'zan'),
        'E77026': ('אלף', 'verb', 'produce in abundance', 'lf'),
        'E77027': ('יֹוצֵאת', 'subs', 'abortion', 'yovtseit'),
        'E77028': ('זקף', 'verb', 'raise up', 'zkf'),
        'E77029': ('עֶשְׁתֹּנֶת', 'subs', 'thought', 'eshtonet'),
        'E77030': ('תֵּקַע', 'subs', 'blast', 'teika'),
        'E77031': ('מֵן', 'subs', 'string', 'mein'),
        'E77032': ('שֶׁמַע', 'subs', 'sound', 'shema'),
        'E77033': ('תֻּמָּה', 'subs', 'integrity', 'tuma'),
        'E77034': ('גרד', 'verb', 'scrap', 'grd'),
        'E77035': ('בִּלְדַּד', 'nmpr', 'Bildad', 'bildad'),
        'E77036': ('שׁוּחִי', 'adjv', 'Shuhite', 'shuchi'),
        'E77037': ('צֹפַר', 'nmpr', 'Zophar', 'tsofar'),
        'E77038': ('נַעֲמָתִי', 'adjv', 'Naamathite', "na'amati"),
        'E77039': ('נְהָרָה', 'subs', 'light', "n'hara"),
        'E77040': ('כַּמְרִיר', 'subs', 'gloom', 'kamrir'),
        'E77041': ('יָגִיעַ', 'adjv', 'weary', 'yagia'),
        'E77042': ('נתע', 'verb', 'break', 'nt'),
        'E77043': ('שֵׁמֶץ', 'subs', '[uncertain]', 'sheimets'),
        'E77044': ('שְׂעִפִּים', 'subs', 'disquieting thoughts', "s'ipim"),
        'E77045': ('תָּהֳלָה', 'subs', 'error', 'tahala'),
        'E77046': ('צַמִּים', 'subs', '[uncertain]', 'tsamim'),
        'E77047': ('כָּפָן', 'subs', 'hunger', 'kafan'),
        'E77048': ('כֶּלַח', 'subs', 'vigour', 'kelach'),
        'E77049': ('לעע', 'verb', 'talk inconsiderately', 'l'),
        'E77050': ('נהק', 'verb', 'cry', 'nhk'),
        'E77051': ('תָּפֵל', 'subs', 'something tasteless', 'tafeil'),
        'E77052': ('חַלָּמוּת', 'subs', 'marshmallow', 'chalamut'),
        'E77053': ('סלד', 'verb', 'jump', 'sld'),
        'E77054': ('חִילָה', 'subs', 'labour pains', 'chila'),
        'E77055': ('נָחוּשׁ', 'adjv', 'bronze', 'nachush'),
        'E77056': ('מָס', 'subs', '[uncertain]', 'mas'),
        'E77057': ('זרב', 'verb', 'press', 'zrv'),
        'E77058': ('חֲתַת', 'subs', 'terror', 'chatat'),
        'E77059': ('נְדֻדִים', 'subs', 'restlessness', "n'dudim"),
        'E77060': ('גּוּשׁ', 'subs', 'crust', 'gush'),
        'E77061': ('מַחֲנָק', 'subs', 'suffocation', 'machanak'),
        'E77062': ('מִפְגָּע', 'subs', 'target', 'mifga'),
        'E77063': ('אֵב', 'subs', 'bud', 'eiv'),
        'E77064': ('יָקֹוט', 'subs', '[uncertain]', 'yakovt'),
        'E77065': ('רָטֹב', 'adjv', 'sapful', 'ratov'),
        'E77066': ('פלץ', 'verb', 'shudder', 'flts'),
        'E77067': ('עָשׁ', 'subs', 'constellation of lion', 'ash'),
        'E77068': ('חתף', 'verb', 'snatch away', 'chtf'),
        'E77069': ('מַמְּרֹורִים', 'subs', 'bitterness', 'mamrovrim'),
        'E77070': ('מַסָּה', 'subs', 'despair', 'masa'),
        'E77071': ('אֵבֶה', 'subs', 'reed', 'eiveh'),
        'E77072': ('טושׂ', 'verb', 'flutter', 'tvs'),
        'E77073': ('שֶׁלֶג', 'subs', 'soap', 'sheleg'),
        'E77074': ('זכך', 'verb', 'be clean', "zch'ch"),
        'E77075': ('גְּבִנָּה', 'subs', 'cheese', "g'vina"),
        'E77076': ('סֵדֶר', 'subs', 'order', 'seider'),
        'E77077': ('לבב', 'verb', 'stir', "lv'v"),
        'E77078': ('תְּעוּפָה', 'subs', 'darkness', "t'ufa"),
        'E77079': ('מַפָּח', 'subs', 'expiring', 'mapach'),
        'E77080': ('פִּיד', 'subs', 'decay', 'pid'),
        'E77081': ('עַשְׁתּוּת', 'subs', 'thought', 'ashtut'),
        'E77082': ('נָכֹון', 'subs', 'push', 'nachovn'),
        'E77083': ('בַּטֻּחֹות', 'subs', 'safety', 'batuchovt'),
        'E77084': ('יָשִׁישׁ', 'adjv', 'aged', 'yashish'),
        'E77085': ('מְזִיחַ', 'subs', 'girdle', "m'ziach"),
        'E77086': ('אָפִיק', 'adjv', 'strong', 'afik'),
        'E77087': ('שׂגא', 'verb', 'be great', 'sg'),
        'E77088': ('אַחְוָה', 'subs', '[uncertain]', 'achva'),
        'E77089': ('סַד', 'subs', 'stock', 'sad'),
        'E77090': ('סָפִיחַ', 'subs', 'outpouring', 'safiach'),
        'E77091': ('תקף', 'verb', 'overpower', 'tkf'),
        'E77092': ('אלף', 'verb', 'be familiar', 'lf'),
        'E77093': ('תַּנְחוּמֹות', 'subs', 'consolation', 'tanchumovt'),
        'E77094': ('רזם', 'verb', 'wink', 'rzm'),
        'E77095': ('כִּידֹור', 'subs', 'onset', 'kidovr'),
        'E77096': ('פִּימָה', 'subs', 'fat', 'pima'),
        'E77097': ('עתד', 'verb', 'prepare', 'td'),
        'E77098': ('מִנְלָם', 'subs', '[uncertain]', 'minlam'),
        'E77099': ('רען', 'verb', 'be luxuriant', 'rn'),
        'E77100': ('חבר', 'verb', 'be brilliant', 'chvr'),
        'E77101': ('נִיד', 'subs', 'condolence', 'nid'),
        'E77102': ('קמט', 'verb', 'seize', 'kmt'),
        'E77103': ('עֲוִיל', 'subs', 'boy', 'avil'),
        'E77104': ('מְרֵרָה', 'subs', 'gall-bladder', "m'reira"),
        'E77105': ('גֵּלֶד', 'subs', 'skin', 'geiled'),
        'E77106': ('עלל', 'verb', 'insert', "l'l"),
        'E77107': ('חמר', 'verb', 'be red', 'chmr'),
        'E77108': ('שָׂהֵד', 'subs', 'witness', 'saheid'),
        'E77109': ('זעך', 'verb', 'extinguish', 'zch'),
        'E77110': ('הֲתֻלִים', 'subs', 'mockery', 'hatulim'),
        'E77111': ('תֹּפֶת', 'subs', 'spitting', 'tofet'),
        'E77112': ('יְצֻרִים', 'subs', 'members', "y'tsurim"),
        'E77113': ('אֹמֶץ', 'subs', 'strength', 'omets'),
        'E77114': ('מֹורָשׁ', 'subs', 'desire', 'movrash'),
        'E77115': ('רפד', 'verb', 'spread out', 'rfd'),
        'E77116': ('קֶנֶץ', 'subs', '[uncertain]', 'kenets'),
        'E77117': ('טמה', 'verb', 'be unclean', 'tmh'),
        'E77118': ('שָׁבִיב', 'subs', 'spark', 'shaviv'),
        'E77119': ('מַלְכֹּדֶת', 'subs', 'snare', 'malkodet'),
        'E77120': ('הכר', 'verb', 'do wrong', 'hchr'),
        'E77121': ('מְשׁוּגָה', 'subs', 'error', "m'shuga"),
        'E77122': ('מָצֹוד', 'subs', 'net', 'matsovd'),
        'E77123': ('זור', 'verb', 'be loathsome', 'zvr'),
        'E77124': ('חנן', 'verb', 'be loathsome', "chn'n"),
        'E77125': ('מלט', 'verb', 'be bald', 'mlt'),
        'E77126': ('שַׁדּוּן', 'subs', '[uncertain]', 'shadun'),
        'E77127': ('שִׂיא', 'subs', 'excellency', 'si'),
        'E77128': ('שׁזף', 'verb', 'catch sight', 'shzf'),
        'E77129': ('מְרֹרָה', 'subs', 'gall', "m'rora"),
        'E77130': ('יָגָע', 'subs', 'product', 'yaga'),
        'E77131': ('עלס', 'verb', 'enjoy', 'ls'),
        'E77132': ('שֵׂפֶק', 'subs', 'plenty', 'seifek'),
        'E77133': ('גֵּוָה', 'subs', '[uncertain]', 'geiva'),
        'E77134': ('כִּיד', 'subs', 'decay', 'kid'),
        'E77135': ('שַׁלְאֲנַן', 'subs', '[uncertain]', 'shalanan'),
        'E77136': ('עֲטִין', 'subs', '[uncertain]', 'atin'),
        'E77137': ('מֹחַ', 'subs', 'marrow', 'moach'),
        'E77138': ('רֶגֶב', 'subs', 'clod', 'regev'),
        'E77139': ('קִים', 'subs', 'adversary', 'kim'),
        'E77140': ('בֶּצֶר', 'subs', 'ore', 'betser'),
        'E77141': ('צוּר', 'subs', 'pebble', 'tsur'),
        'E77142': ('שַׁח', 'adjv', 'downcast', 'shach'),
        'E77143': ('אִי', 'nega', 'not', 'i'),
        'E77144': ('לקשׁ', 'verb', 'despoil', 'lksh'),
        'E77145': ('רטב', 'verb', 'be moist', 'rtv'),
        'E77146': ('שׁוּרָה', 'subs', 'wall', 'shura'),
        'E77147': ('צהר', 'verb', 'press out', "ts'hr"),
        'E77148': ('אַל', 'subs', 'nothingness', 'al'),
        'E77149': ('אהל', 'verb', 'be bright', 'hl'),
        'E77150': ('פַּרְשֵׂז', 'subs', '[uncertain]', 'parseiz'),
        'E77151': ('חוג', 'verb', '[uncertain]', 'chvg'),
        'E77152': ('רפף', 'verb', 'shake', "rf'f"),
        'E77153': ('שִׁפְרָה', 'subs', '[uncertain]', 'shifra'),
        'E77154': ('לְמֹו', 'prep', 'to', "l'mov"),
        'E77155': ('שַׁחַץ', 'subs', 'greatness', 'shachats'),
        'E77156': ('עדה', 'verb', 'stride', 'dh'),
        'E77157': ('תַּעֲלֻם', 'subs', '[uncertain]', "ta'alum"),
        'E77158': ('סלה', 'verb', 'pay', 'slh'),
        'E77159': ('זְכֹוכִית', 'subs', 'transparent', "z'chovchit"),
        'E77160': ('גָּבִישׁ', 'subs', 'rock-crystal', 'gavish'),
        'E77161': ('פְּנִינִים', 'subs', 'corals', "p'ninim"),
        'E77162': ('חֹרֶף', 'subs', 'youth', 'choref'),
        'E77163': ('הָלִיךְ', 'subs', 'step', 'halich'),
        'E77164': ('קֶרֶת', 'subs', 'town', 'keret'),
        'E77165': ('חֹול', 'subs', 'phoenix', 'chovl'),
        'E77166': ('חֶסֶר', 'subs', 'lack', 'cheser'),
        'E77167': ('ערק', 'verb', 'gnaw', 'rk'),
        'E77168': ('מַלּוּחַ', 'subs', 'salt-herb', 'maluach'),
        'E77169': ('גֵּו', 'subs', 'community', 'geiv'),
        'E77170': ('עָרוּץ', 'subs', 'slope', 'aruts'),
        'E77171': ('נכא', 'verb', 'scourge', 'nch'),
        'E77172': ('פִּרְחַח', 'subs', '[uncertain]', 'pirchach'),
        'E77173': ('נתס', 'verb', 'tear', 'nts'),
        'E77174': ('שׁוּעַ', 'subs', '[uncertain]', 'shua'),
        'E77175': ('עגם', 'verb', 'be grieved', 'gm'),
        'E77176': ('שׁחר', 'verb', 'be black', 'shchr'),
        'E77177': ('פְּלִילִי', 'adjv', 'calling for judgment', "p'lili"),
        'E77178': ('חֹב', 'subs', 'pocket', 'chov'),
        'E77179': ('ענד', 'verb', 'bind', 'nd'),
        'E77180': ('בָּאְשָׁה', 'subs', '[malodorous plant]', 'basha'),
        'E77181': ('בַּרַכְאֵל', 'nmpr', 'Barakel', 'baracheil'),
        'E77182': ('בּוּזִי', 'adjv', 'Buzite', 'buzi'),
        'E77183': ('רָם', 'nmpr', 'Ram', 'ram'),
        'E77184': ('דֵּעַ', 'subs', 'knowledge', 'deia'),
        'E77185': ('אֹוב', 'subs', 'bag', 'ovv'),
        'E77186': ('אֶכֶף', 'subs', 'pressure', 'echef'),
        'E77187': ('חַף', 'adjv', 'clean', 'chaf'),
        'E77188': ('זהם', 'verb', 'loath', 'zhm'),
        'E77189': ('פדע', 'verb', '[uncertain]', 'fd'),
        'E77190': ('רטפשׁ', 'verb', '[uncertain]', 'rtfsh'),
        'E77191': ('חֶבְרָה', 'subs', 'company', 'chevra'),
        'E77192': ('מַעֲבָד', 'subs', 'work', "ma'avad"),
        'E77193': ('פַּשׁ', 'subs', '[uncertain]', 'pash'),
        'E77194': ('כבר', 'verb', 'multiply', 'chvr'),
        'E77195': ('רַחַב', 'subs', 'expanse', 'rachav'),
        'E77196': ('שֶׂפֶק', 'subs', '[uncertain]', 'sefek'),
        'E77197': ('מַאֲמָץ', 'subs', 'effort', "ma'amats"),
        'E77198': ('שַׂגִּיא', 'adjv', 'exalted', 'sagi'),
        'E77199': ('נֶטֶף', 'subs', 'drop', 'netef'),
        'E77200': ('הוה', 'verb', 'fall', 'hvh'),
        'E77201': ('אֶרֶב', 'subs', 'ambush', 'erev'),
        'E77202': ('מְזָרִים', 'subs', 'north-winds', "m'zarim"),
        'E77203': ('רִי', 'subs', 'moisture', 'ri'),
        'E77204': ('טרח', 'verb', 'burden', 'trch'),
        'E77205': ('מְסִבָּה', 'subs', 'circle', "m'siba"),
        'E77206': ('תַּחְבֻּלֹות', 'subs', 'steering', 'tachbulovt'),
        'E77207': ('מִפְלָשׂ', 'subs', 'layer', 'miflas'),
        'E77208': ('מִפְלְאֹות', 'subs', 'wonders', 'miflovt'),
        'E77209': ('רְאִי', 'subs', 'mirror', "r'i"),
        'E77210': ('בָּהִיר', 'adjv', 'brilliant', 'bahir'),
        'E77211': ('מְמָד', 'subs', 'measurement', "m'mad"),
        'E77212': ('חֲתֻלָּה', 'subs', 'swaddling band', 'chatula'),
        'E77213': ('נֵבֶךְ', 'subs', 'sandy ground', 'neivech'),
        'E77214': ('אֵגֶל', 'subs', 'drop', 'eigel'),
        'E77215': ('מֹשְׁכֹות', 'subs', 'cords', 'moshchovt'),
        'E77216': ('מַזָּרֹות', 'subs', 'constellations', 'mazarovt'),
        'E77217': ('עַיִשׁ', 'subs', 'lioness', 'ayish'),
        'E77218': ('מִשְׁטָר', 'subs', 'writing', 'mishtar'),
        'E77219': ('שֶׂכְוִי', 'subs', 'cock', 'sechvi'),
        'E77220': ('חֵבֶל', 'subs', 'foetus', 'cheivel'),
        'E77221': ('בַּר', 'subs', 'open field', 'bar'),
        'E77222': ('עָרֹוד', 'subs', 'wild ass', 'arovd'),
        'E77223': ('יָרֹוק', 'subs', 'green plants', 'yarovk'),
        'E77224': ('רְנָנִים', 'subs', 'she-ostriches', "r'nanim"),
        'E77225': ('מרא', 'verb', 'toss', 'mr'),
        'E77226': ('רַעְמָה', 'subs', 'horse mane', 'rama'),
        'E77227': ('נַחַר', 'subs', 'snorting', 'nachar'),
        'E77228': ('רנה', 'verb', 'rattle', 'rnh'),
        'E77229': ('אבר', 'verb', 'soar up', 'vr'),
        'E77230': ('עלע', 'verb', '[uncertain]', 'l'),
        'E77231': ('יִסֹּור', 'subs', 'reprover', 'yisovr'),
        'E77232': ('הדך', 'verb', 'tread down', 'hdch'),
        'E77233': ('בְּהֵמֹות', 'subs', 'hippopotamus', "b'heimovt"),
        'E77234': ('שָׁרִיר', 'subs', 'sinew', 'sharir'),
        'E77235': ('חפץ', 'verb', 'bend down', 'chfts'),
        'E77236': ('פַּחַד', 'subs', 'thigh', 'pachad'),
        'E77237': ('שׂרג', 'verb', 'intertwine', 'srg'),
        'E77238': ('מָטִיל', 'subs', 'rod', 'matil'),
        'E77239': ('צֶאֱלִים', 'subs', 'lotus', "tse'elim"),
        'E77240': ('חַבָּר', 'subs', 'partner', 'chabar'),
        'E77241': ('שֻׂכָּה', 'subs', 'harpoon', 'suka'),
        'E77242': ('צִלְצָל', 'subs', 'harpoon', 'tsiltsal'),
        'E77243': ('חִין', 'subs', '[uncertain]', 'chin'),
        'E77244': ('עֲטִישָׁה', 'subs', 'sneezing', 'atisha'),
        'E77245': ('כִּידֹוד', 'subs', 'spark', 'kidovd'),
        'E77246': ('נְחִירַיִם', 'subs', 'nostrils', "n'chirayim"),
        'E77247': ('דוץ', 'verb', 'leap', 'dvts'),
        'E77248': ('דְּאָבָה', 'subs', 'despair', "d'ava"),
        'E77249': ('שִׁרְיָה', 'subs', 'arrow-head', 'shira'),
        'E77250': ('רִקָּבֹון', 'subs', 'rottenness', 'rikavovn'),
        'E77251': ('תֹּותָח', 'subs', 'club', 'tovtach'),
        'E77252': ('חַדּוּדִים', 'subs', 'points', 'chadudim'),
        'E77253': ('מֹשֶׁל', 'subs', 'likeness', 'moshel'),
        'E77254': ('שִׁבְעָנָה', 'subs', 'twice seven', 'shivana'),
        'E77255': ('יְמִימָה', 'nmpr', 'Jemimah', "y'mima"),
        'E77256': ('קְצִיעָה', 'nmpr', 'Keziah', "k'tsia"),
        'E77257': ('קֶרֶן הַפּוּךְ', 'nmpr', 'Keren-Happuch', 'keren hapuch'),
        'E77258': ('לִוְיָה', 'subs', 'wreath', 'liva'),
        'E77259': ('גַּרְגְּרֹת', 'subs', 'throat', 'gargrot'),
        'E77260': ('פֶּתִי', 'subs', 'simplicity', 'peti'),
        'E77261': ('שׁוח', 'verb', 'run down', 'shvch'),
        'E77262': ('רִפְאוּת', 'subs', 'healing', 'rifut'),
        'E77263': ('לֶכֶד', 'subs', 'capture', 'leched'),
        'E77264': ('עִקְּשׁוּת', 'subs', 'crookedness', 'ikshut'),
        'E77265': ('לָזוּת', 'subs', 'crookedness', 'lazut'),
        'E77266': ('פלס', 'verb', 'watch', 'fls'),
        'E77267': ('יַעֲלָה', 'subs', 'goat', "ya'ala"),
        'E77268': ('נְמָלָה', 'subs', 'ant', "n'mala"),
        'E77269': ('עָצֵל', 'adjv', 'sluggish', 'atseil'),
        'E77270': ('חִבֻּק', 'subs', 'folding', 'chibuk'),
        'E77271': ('רֵישׁ', 'subs', 'poverty', 'reish'),
        'E77272': ('מִדְיָן', 'subs', 'contention', 'midan'),
        'E77273': ('מֹדָע', 'subs', 'kinsman', 'moda'),
        'E77274': ('שׁוּק', 'subs', 'street', 'shuk'),
        'E77275': ('מַרְבַדִּים', 'subs', 'coverlets', 'marvadim'),
        'E77276': ('רבד', 'verb', 'prepare', 'rvd'),
        'E77277': ('חטב', 'verb', 'be coloured', 'chtv'),
        'E77278': ('אֵטוּן', 'subs', 'linen', 'eitun'),
        'E77279': ('חֵלֶק', 'subs', 'smoothness', 'cheilek'),
        'E77280': ('מִפְתָּח', 'subs', 'opening', 'miftach'),
        'E77281': ('גֵּאָה', 'subs', 'arrogance', 'geia'),
        'E77282': ('עָתֵק', 'adjv', 'hereditary', 'ateik'),
        'E77283': ('גַּף', 'subs', 'back?', 'gaf'),
        'E77284': ('כְּסִילוּת', 'subs', 'insolence', "k'silut"),
        'E77285': ('פְּתַיּוּת', 'subs', '[uncertain]', "p'tayut"),
        'E77286': ('עֹמֶק', 'subs', 'depth', 'omek'),
        'E77287': ('חָרוּץ', 'adjv', 'assiduous', 'charuts'),
        'E77288': ('עַצֶּבֶת', 'subs', 'pain', 'atsevet'),
        'E77289': ('צָנוּעַ', 'adjv', 'humble', 'tsanua'),
        'E77290': ('סֶלֶף', 'subs', 'crookedness', 'selef'),
        'E77291': ('תֹּוקְעִים', 'subs', 'handshakes', 'tovkim'),
        'E77292': ('מַדְקֵרָה', 'subs', 'stab', 'madkeira'),
        'E77293': ('חרך', 'verb', 'start?', 'chrch'),
        'E77294': ('מֹותָר', 'subs', 'advantage', 'movtar'),
        'E77295': ('רָזֹון', 'subs', '[official]', 'razovn'),
        'E77296': ('מַרְפֵּא', 'subs', 'calmness', 'marpei'),
        'E77297': ('נָכֵא', 'adjv', 'smitten', 'nachei'),
        'E77298': ('מְשֻׂכָה', 'subs', 'hedge', "m'sucho"),
        'E77299': ('מַעֲרָךְ', 'subs', 'arrangement', "ma'arach"),
        'E77300': ('מַעֲנֶה', 'subs', 'purpose', "ma'aneh"),
        'E77301': ('כִּשָּׁלֹון', 'subs', 'stumbling', 'kishalovn'),
        'E77302': ('מָתֵק', 'adjv', 'sweet', 'mateik'),
        'E77303': ('אכף', 'verb', 'press hard', 'chf'),
        'E77304': ('צָרָב', 'adjv', 'burning', 'tsarav'),
        'E77305': ('עצה', 'verb', 'blink', "ts'h"),
        'E77306': ('מַצְרֵף', 'subs', 'melting pot', 'matsreif'),
        'E77307': ('גלע', 'verb', 'burst out', 'gl'),
        'E77308': ('גֵּהָה', 'subs', 'healing', 'geiha'),
        'E77309': ('מֶמֶר', 'subs', 'bitterness', 'memer'),
        'E77310': ('מַהֲלֻמֹות', 'subs', 'strokes', 'mahalumovt'),
        'E77311': ('להם', 'verb', 'swallow', 'lhm'),
        'E77312': ('מַחֲלֶה', 'subs', 'disease', 'machaleh'),
        'E77313': ('נַהַם', 'subs', 'groaning', 'naham'),
        'E77314': ('דֶּלֶף', 'subs', 'roof', 'delef'),
        'E77315': ('טרד', 'verb', 'drive away', 'trd'),
        'E77316': ('עַצְלָה', 'subs', 'sluggishness', 'atsla'),
        'E77317': ('עָרֵב', 'adjv', 'pleasant', 'areiv'),
        'E77318': ('חָצָץ', 'subs', 'pebble', 'chatsats'),
        'E77319': ('אֱשׁוּן', 'subs', 'approach', 'eshun'),
        'E77320': ('תַּמְרוּק', 'subs', 'massage', 'tamruk'),
        'E77321': ('הֲפַכְפַּךְ', 'adjv', 'crooked', 'hafachpach'),
        'E77322': ('וָזָר', 'adjv', 'guilty', 'vazar'),
        'E77323': ('כפה', 'verb', 'subdue', 'chfh'),
        'E77324': ('קֹשְׁטְ', 'subs', 'truth', 'kosht'),
        'E77325': ('חָשֹׁךְ', 'adjv', 'dark', 'chashoch'),
        'E77326': ('שַׂכִּין', 'subs', 'knife', 'sakin'),
        'E77327': ('לֹעַ', 'subs', 'throat', 'loa'),
        'E77328': ('שׁער', 'verb', 'estimate', 'shr'),
        'E77329': ('נוּמָה', 'subs', 'somnolence', 'numa'),
        'E77330': ('חֶתֶף', 'subs', 'robber', 'chetef'),
        'E77331': ('אֲבֹוי', 'subs', 'desire', 'avovy'),
        'E77332': ('חַכְלִלוּת', 'subs', 'dullness', 'chachlilut'),
        'E77333': ('חִבֵּל', 'subs', '[uncertain]', 'chibeil'),
        'E77334': ('מָטֶה', 'subs', 'coming', 'mateh'),
        'E77335': ('קִמָּשֹׂון', 'subs', 'weed', 'kimasovn'),
        'E77336': ('חסד', 'verb', 'insult', 'chsd'),
        'E77337': ('אֹפֶן', 'subs', 'right time', 'ofen'),
        'E77338': ('חֲלִי', 'subs', 'ornament', 'chali'),
        'E77339': ('צִנָּה', 'subs', 'cold', 'tsina'),
        'E77340': ('מֵפִיץ', 'subs', '[uncertain]', 'meifits'),
        'E77341': ('מַרְגֵּמָה', 'subs', 'heap of stones', 'margeima'),
        'E77342': ('קֵא', 'subs', 'vomit', 'kei'),
        'E77343': ('צִיר', 'subs', 'pivot', 'tsir'),
        'E77344': ('להה', 'verb', 'be crazy', "lh'h"),
        'E77345': ('זֵק', 'subs', 'fire-arrow', 'zeik'),
        'E77346': ('מַשָּׁאֹון', 'subs', 'deceiving', 'mashaovn'),
        'E77347': ('מִדְחֶה', 'subs', 'overthrow', 'midcheh'),
        'E77348': ('נֵטֶל', 'subs', 'load', 'neitel'),
        'E77349': ('אַכְזְרִיּוּת', 'subs', 'cruelty', 'achzriut'),
        'E77350': ('נְשִׁיקָה', 'subs', 'kiss', "n'shika"),
        'E77351': ('סַגְרִיר', 'subs', 'heavy rain', 'sagrir'),
        'E77352': ('אֲבַדֹּה', 'subs', 'destruction', 'avado'),
        'E77353': ('מַהֲלָל', 'subs', 'fame', 'mahalal'),
        'E77354': ('כתשׁ', 'verb', 'pound', 'chtsh'),
        'E77355': ('עֱלִי', 'subs', 'pestle', 'eli'),
        'E77356': ('שְׁחוּת', 'subs', 'pit', "sh'chut"),
        'E77357': ('פנק', 'verb', 'pamper', 'fnk'),
        'E77358': ('מָנֹון', 'subs', '[uncertain]', 'manovn'),
        'E77359': ('אָגוּר', 'nmpr', 'Agur', 'agur'),
        'E77360': ('יָקֶה', 'nmpr', 'Jakeh', 'yakeh'),
        'E77361': ('אִיתִיאֵל', 'nmpr', 'Ithiel', 'itieil'),
        'E77362': ('אֻכָל', 'subs', '[uncertain]', 'uchol'),
        'E77363': ('עֲלוּקָה', 'subs', 'leech', 'aluka'),
        'E77364': ('שְׂמָמִית', 'subs', 'gecko', "s'mamit"),
        'E77365': ('זַרְזִיר', 'subs', '[uncertain]', 'zarzir'),
        'E77366': ('אַלְקוּם', 'subs', '[uncertain]', 'alkum'),
        'E77367': ('מִיץ', 'subs', 'squeezing', 'mits'),
        'E77368': ('לְמוּאֵל', 'nmpr', 'Lemuel', "l'mueil"),
        'E77369': ('חֲלֹוף', 'subs', '[uncertain]', 'chalovf'),
        'E77370': ('כִּישֹׁור', 'subs', 'whorl', 'kishovr'),
        'E77371': ('עַצְלוּת', 'subs', 'sluggishness', 'atslut'),
        'E77372': ('אֱלִימֶלֶךְ', 'nmpr', 'Elimelech', 'elimelech'),
        'E77373': ('נָעֳמִי', 'nmpr', 'Naomi', "na'ami"),
        'E77374': ('מַחְלֹון', 'nmpr', 'Mahlon', 'machlovn'),
        'E77375': ('כִּלְיֹון', 'nmpr', 'Kilion', 'kilovn'),
        'E77376': ('עָרְפָּה', 'nmpr', 'Orpah', 'arpa'),
        'E77377': ('רוּת', 'nmpr', 'Ruth', 'rut'),
        'E77378': ('לָהֵן', 'advb', 'therefore', 'lahein'),
        'E77379': ('עגן', 'verb', 'withdraw', 'gn'),
        'E77380': ('בֹּעַז', 'nmpr', 'Boaz', 'boaz'),
        'E77381': ('צבט', 'verb', 'hold out', 'tsvt'),
        'E77382': ('שׁלל', 'verb', 'draw out', "shl'l"),
        'E77383': ('צֶבֶת', 'subs', 'bundle', 'tsevet'),
        'E77384': ('מֹדַעַת', 'subs', 'kinsman', "moda'at"),
        'E77385': ('מַרְגְּלֹת', 'subs', 'feet-place', 'marglot'),
        'E77386': ('עֹובֵד', 'nmpr', 'Obed', 'ovveid'),
        'E77387': ('שַׂלְמָה', 'nmpr', 'Salmon', 'salma'),
        'E77388': ('שַׂלְמֹון', 'nmpr', 'Salmon', 'salmovn'),
        'E77389': ('שְׁחַרְחֹר', 'adjv', 'blackish', "sh'charchor"),
        'E77390': ('נחר', 'verb', 'snort', 'nchr'),
        'E77391': ('נטר', 'verb', 'keep', 'ntr'),
        'E77392': ('גְּדִיָּה', 'subs', 'young goat', "g'dia"),
        'E77393': ('סוּסָה', 'subs', 'mare', 'susa'),
        'E77394': ('רַעְיָה', 'subs', 'girl friend', 'raya'),
        'E77395': ('תֹּור', 'subs', 'turn', 'tovr'),
        'E77396': ('חֲרוּזִים', 'subs', 'string of shells', 'charuzim'),
        'E77397': ('נְקֻדָּה', 'subs', 'glass-bead', "n'kuda"),
        'E77398': ('נֵרְדְּ', 'subs', 'nard', 'neird'),
        'E77399': ('כֹּפֶר', 'subs', 'hennah', 'kofer'),
        'E77400': ('רָהִיט', 'subs', 'rafter', 'rahit'),
        'E77401': ('בְּרֹות', 'subs', 'juniper', "b'rovt"),
        'E77402': ('צְבָאָה', 'subs', 'she-gazelle', "ts'va'a"),
        'E77403': ('עֹפֶר', 'subs', 'gazelle', 'ofer'),
        'E77404': ('כֹּתֶל', 'subs', 'wall', 'kotel'),
        'E77405': ('צוץ', 'verb', 'glance', 'tsvts'),
        'E77406': ('חֲרַכִּים', 'subs', 'lattice', 'charakim'),
        'E77407': ('סְתָו', 'subs', 'winter', "s'tav"),
        'E77408': ('נִצָּן', 'subs', 'blossom', 'nitsan'),
        'E77409': ('זָמִיר', 'subs', 'pruning', 'zamir'),
        'E77410': ('פַּג', 'subs', 'unripe fig', 'pag'),
        'E77411': ('סְמָדַר', 'subs', 'perule', "s'madar"),
        'E77412': ('פוח', 'verb', 'breath', 'fvch'),
        'E77413': ('בֶּתֶר', 'subs', '[uncertain]', 'beter'),
        'E77414': ('אֲבָקָה', 'subs', 'scent-powder', 'avaka'),
        'E77415': ('אָחֻז', 'adjv', 'holding', 'achuz'),
        'E77416': ('אַפִּרְיֹון', 'subs', 'litter', 'apirovn'),
        'E77417': ('רְפִידָה', 'subs', 'saddle-cloth', "r'fida"),
        'E77418': ('רצף', 'verb', 'fit', 'rtsf'),
        'E77419': ('חֲתֻנָּה', 'subs', 'marriage', 'chatuna'),
        'E77420': ('גלשׁ', 'verb', 'go down', 'glsh'),
        'E77421': ('רַחְצָה', 'subs', 'washing', 'rachtsa'),
        'E77422': ('תאם', 'verb', 'bear twins', 'tm'),
        'E77423': ('מִדְבָּר', 'subs', 'speaking', 'midbar'),
        'E77424': ('תַּלְפִּיֹּות', 'subs', 'stone courses', 'talpiovt'),
        'E77425': ('צְבִיָּה', 'subs', 'she-gazelle', "ts'bia"),
        'E77426': ('צַוְּרֹנִים', 'subs', 'necklace', 'tsauronim'),
        'E77427': ('שְׁלָחִים', 'subs', 'channel', "sh'lachim"),
        'E77428': ('פַּרְדֵּס', 'subs', 'park', 'pardeis'),
        'E77429': ('כַּרְכֹּום', 'subs', 'curcuma', 'karkovm'),
        'E77430': ('קְוֻצֹּות', 'subs', 'hairlocks', "k'vutsovt"),
        'E77431': ('רָסִיס', 'subs', 'drop', 'rasis'),
        'E77432': ('אֵיכָכָה', 'inrg', 'how', 'eichocho'),
        'E77433': ('טנף', 'verb', 'soil', 'tnf'),
        'E77434': ('מַנְעוּל', 'subs', 'lock', 'manul'),
        'E77435': ('תַּלְתָּל', 'subs', 'date-panicle', 'taltal'),
        'E77436': ('מִלֵּאת', 'subs', '[uncertain]', 'mileit'),
        'E77437': ('מֶרְקָח', 'subs', 'herb', 'merkach'),
        'E77438': ('עֶשֶׁת', 'subs', 'plate', 'eshet'),
        'E77439': ('שֵׁשׁ', 'subs', 'alabaster', 'sheish'),
        'E77440': ('מַמְתַקִּים', 'subs', 'sweetness', 'mamtakim'),
        'E77441': ('אֱגֹוז', 'subs', 'nut', 'egovz'),
        'E77442': ('שׁוּלַמִּי', 'adjv', 'Shulammite', 'shulami'),
        'E77443': ('חַמּוּק', 'subs', 'turning', 'chamuk'),
        'E77444': ('אָמָּן', 'subs', 'artisan', 'aman'),
        'E77445': ('סַהַר', 'subs', 'enclosure', 'sahar'),
        'E77446': ('מֶזֶג', 'subs', 'mixed wine', 'mezeg'),
        'E77447': ('סוג', 'verb', 'fence', 'svg'),
        'E77448': ('בַּת רַבִּים', 'nmpr', 'Bath Rabbim', 'bat rabim'),
        'E77449': ('סַנְסִנָּה', 'subs', 'date-fruit', 'sansina'),
        'E77450': ('דבב', 'verb', 'flow', "dv'v"),
        'E77451': ('רֶקַח', 'subs', 'spice', 'rekach'),
        'E77452': ('רפק', 'verb', 'lean', 'rfk'),
        'E77453': ('שַׁלְהֶבֶתְיָה', 'subs', 'big flame', 'shalhevetya'),
        'E77454': ('בַּעַל הָמֹון', 'nmpr', 'Baal Hamon', "ba'al hamovn"),
        'E77455': ('קֹהֶלֶת', 'subs', 'speaker', 'kohelet'),
        'E77456': ('יִתְרֹון', 'subs', 'profit', 'yitrovn'),
        'E77457': ('כְּבָר', 'subs', 'long time', "k'var"),
        'E77458': ('עִנְיָן', 'subs', 'occupation', 'inan'),
        'E77459': ('ענה', 'verb', 'be worried', 'nh'),
        'E77460': ('רְעוּת', 'subs', 'longing', "r'ut"),
        'E77461': ('תקן', 'verb', 'be straight', 'tkn'),
        'E77462': ('חֶסְרֹון', 'subs', 'want', 'chesrovn'),
        'E77463': ('הֹולֵלָה', 'subs', 'madness', 'hovleila'),
        'E77464': ('שִׂכְלוּת', 'subs', 'folly', 'sichlut'),
        'E77465': ('רַעְיֹון', 'subs', 'striving', 'rayovn'),
        'E77466': ('סִכְלוּת', 'subs', 'folly', 'sichlut'),
        'E77467': ('שִׁדָּה', 'subs', 'lady', 'shida'),
        'E77468': ('כִּשְׁרֹון', 'subs', 'advantage', 'kishrovn'),
        'E77469': ('חושׁ', 'verb', 'feel', 'chvsh'),
        'E77470': ('זְמָן', 'subs', 'season', "z'man"),
        'E77471': ('עֲדֶנָה', 'advb', 'hitherto', 'adena'),
        'E77472': ('עֲדֶן', 'advb', 'hitherto', 'aden'),
        'E77473': ('אִי', 'inrg', 'how', 'i'),
        'E77474': ('מִסְכֵּן', 'adjv', 'poor', 'miskein'),
        'E77475': ('רְאוּת', 'subs', 'look', "r'ut"),
        'E77476': ('אִלּוּ', 'conj', 'if', 'ilu'),
        'E77477': ('תַּקִּיף', 'adjv', 'mighty', 'takif'),
        'E77478': ('חֶשְׁבֹּון', 'subs', 'account', 'cheshbovn'),
        'E77479': ('חִשָּׁבֹון', 'subs', 'invention', 'chishavovn'),
        'E77480': ('פֵּשֶׁר', 'subs', 'interpretation', 'peisher'),
        'E77481': ('שִׁלְטֹון', 'subs', 'mastery', 'shiltovn'),
        'E77482': ('פִּתְגָם', 'subs', 'decision', 'pitgam'),
        'E77483': ('עֲבָד', 'subs', 'work', 'avad'),
        'E77484': ('מֵרֹוץ', 'subs', 'running', 'meirovts'),
        'E77485': ('סֶכֶל', 'subs', 'folly', 'sechel'),
        'E77486': ('גּוּמָּץ', 'subs', 'pit', 'gumats'),
        'E77487': ('כשׁר', 'verb', 'succeed', 'chshr'),
        'E77488': ('הֹולֵלוּת', 'subs', 'madness', 'hovleilut'),
        'E77489': ('שְׁתִי', 'subs', 'drinking', "sh'ti"),
        'E77490': ('עֲצַלְתַּיִם', 'subs', 'great sluggishness', 'atsaltayim'),
        'E77491': ('מְקָרֶה', 'subs', 'beam-work', "m'kareh"),
        'E77492': ('שִׁפְלוּת', 'subs', 'inactivity', 'shiflut'),
        'E77493': ('דלף', 'verb', 'drip', 'dlf'),
        'E77494': ('מַדָּע', 'subs', 'knowledge', 'mada'),
        'E77495': ('בְּחוּרֹות', 'subs', 'youth', "b'churovt"),
        'E77496': ('שַׁחֲרוּת', 'subs', 'black hair', 'shacharut'),
        'E77497': ('בטל', 'verb', 'cease working', 'vtl'),
        'E77498': ('טֹחֲנֹות', 'subs', 'molar teeth', 'tochanovt'),
        'E77499': ('טַחֲנָה', 'subs', 'grinding mill', 'tachana'),
        'E77500': ('חַתְחַתִּים', 'subs', 'terror', "chat'chatim"),
        'E77501': ('אֲבִיֹּונָה', 'subs', 'caper-fruit', 'abiovna'),
        'E77502': ('אזן', 'verb', 'ponder', 'zn'),
        'E77503': ('דָּרְבֹן', 'subs', '[uncertain]', 'darvon'),
        'E77504': ('אֲסֻפָּה', 'subs', 'collection', 'asupa'),
        'E77505': ('לַהַג', 'subs', 'devotion', 'lahag'),
        'E77506': ('יְגִעָה', 'subs', 'wearying', "y'gia"),
        'E77507': ('מַחְמֹד', 'subs', 'something precious', 'machmod'),
        'E77508': ('מִשְׁבָּת', 'subs', 'ruin', 'mishbat'),
        'E77509': ('שׂקד', 'verb', '[uncertain]', 'skd'),
        'E77510': ('עוב', 'verb', 'becloud', 'vv'),
        'E77511': ('שֹׂךְ', 'subs', '[uncertain]', 'soch'),
        'E77512': ('מַדּוּחִים', 'subs', 'enticement', 'maduchim'),
        'E77513': ('פּוּגָה', 'subs', 'benumbing', 'puga'),
        'E77514': ('טִפֻּחִים', 'subs', 'healthy ones', 'tipuchim'),
        'E77515': ('טפח', 'verb', 'bring forth', 'tfch'),
        'E77516': ('שׂתם', 'verb', 'stop', 'stm'),
        'E77517': ('פשׁח', 'verb', 'be untilled', 'fshch'),
        'E77518': ('כפשׁ', 'verb', 'cower', 'chfsh'),
        'E77519': ('יָחִיל', 'subs', '[uncertain]', 'yachil'),
        'E77520': ('סְחִי', 'subs', 'refuse', "s'chi"),
        'E77521': ('שֵׁאת', 'subs', 'desolation', 'sheit'),
        'E77522': ('הֲפֻגָה', 'subs', 'stop', 'hafuga'),
        'E77523': ('עַוָּתָה', 'subs', 'oppression', 'auata'),
        'E77524': ('קִימָה', 'subs', 'rising up', 'kima'),
        'E77525': ('מַנְגִּינָה', 'subs', 'mocking song', 'mangina'),
        'E77526': ('מָגִנָּה', 'subs', 'insolence', 'magina'),
        'E77527': ('תַּאְלָה', 'subs', 'curse', 'tala'),
        'E77528': ('עמם', 'verb', 'be dark', "m'm"),
        'E77529': ('שׁנא', 'verb', 'shine', 'shn'),
        'E77530': ('יָעֵן', 'subs', '[uncertain]', 'yaein'),
        'E77531': ('צחה', 'verb', 'be white', 'tschh'),
        'E77532': ('שְׁחֹור', 'subs', 'blackness', "sh'chovr"),
        'E77533': ('צפד', 'verb', 'shrivel', 'tsfd'),
        'E77534': ('רַחֲמָנִי', 'adjv', 'compassionate', 'rachamani'),
        'E77535': ('נוץ', 'verb', 'depart', 'nvts'),
        'E77536': ('צִפִּיָּה', 'subs', 'outlook', 'tsipia'),
        'E77537': ('טְחֹון', 'subs', 'hand-mill', "t'chovn"),
        'E77538': ('אֲחַשְׁוֵרֹושׁ', 'nmpr', 'Ahasuerus', 'achashveirovsh'),
        'E77539': ('הֹדּוּ', 'nmpr', 'India', 'hodu'),
        'E77540': ('שׁוּשַׁן', 'nmpr', 'Susa', 'shushan'),
        'E77541': ('בִּירָה', 'subs', 'citadel', 'bira'),
        'E77542': ('פַּרְתְּמִים', 'subs', 'nobles', 'partmim'),
        'E77543': ('בִּיתָן', 'subs', 'palace', 'bitan'),
        'E77544': ('חוּר', 'subs', 'linen', 'chur'),
        'E77545': ('כַּרְפַּס', 'subs', 'linen', 'karpas'),
        'E77546': ('בַּהַט', 'subs', '[uncertain]', 'bahat'),
        'E77547': ('דַּר', 'subs', 'pavement', 'dar'),
        'E77548': ('סֹחֶרֶת', 'subs', 'pavement', 'socheret'),
        'E77549': ('שְׁתִיָּה', 'subs', 'drinking', "sh'tia"),
        'E77550': ('אנס', 'verb', 'compel', 'ns'),
        'E77551': ('וַשְׁתִּי', 'nmpr', 'Vashti', 'vashti'),
        'E77552': ('מְהוּמָן', 'nmpr', 'Mehuman', "m'human"),
        'E77553': ('בִּזְּתָא', 'nmpr', 'Biztha', 'bizta'),
        'E77554': ('חַרְבֹונָא', 'nmpr', 'Harbona', 'charvovna'),
        'E77555': ('בִּגְתָא', 'nmpr', 'Bigtha', 'bigta'),
        'E77556': ('אֲבַגְתָא', 'nmpr', 'Abagtha', 'avagta'),
        'E77557': ('זֵתַר', 'nmpr', 'Zethar', 'zeitar'),
        'E77558': ('כַּרְכַּס', 'nmpr', 'Carcas', 'karkas'),
        'E77559': ('כֶּתֶר', 'subs', 'headdress', 'keter'),
        'E77560': ('כַּרְשְׁנָא', 'nmpr', 'Carshena', 'karshna'),
        'E77561': ('שֵׁתָר', 'nmpr', 'Shethar', 'sheitar'),
        'E77562': ('אַדְמָתָא', 'nmpr', 'Admatha', 'admata'),
        'E77563': ('תַּרְשִׁישׁ', 'nmpr', 'Tarshish', 'tarshish'),
        'E77564': ('מֶרֶס', 'nmpr', 'Meres', 'meres'),
        'E77565': ('מַרְסְנָא', 'nmpr', 'Marsena', 'marsna'),
        'E77566': ('מְמוּכָן', 'nmpr', 'Memucan', "m'muchon"),
        'E77567': ('מַאֲמָר', 'subs', 'word', "ma'amar"),
        'E77568': ('בִּזָּיֹון', 'subs', 'contempt', 'bizayovn'),
        'E77569': ('הֵגַי', 'nmpr', 'Hegai', 'heigay'),
        'E77570': ('מָרְדְּכַי', 'nmpr', 'Mordecai', 'mardchay'),
        'E77571': ('הֲדַסָּה', 'nmpr', 'Hadassah', 'hadasa'),
        'E77572': ('אֶסְתֵּר', 'nmpr', 'Esther', 'esteir'),
        'E77573': ('מְרוּקִים', 'subs', 'massage', "m'rukim"),
        'E77574': ('שַׁעֲשְׁגַז', 'nmpr', 'Shaashgaz', "sha'ashgaz"),
        'E77575': ('טֵבֵת', 'nmpr', 'Tebeth', 'teiveit'),
        'E77576': ('הֲנָחָה', 'subs', 'release', 'hanacha'),
        'E77577': ('אָמְנָה', 'subs', 'care', 'amna'),
        'E77578': ('בִּגְתָן', 'nmpr', 'Bigthana', 'bigtan'),
        'E77579': ('תֶּרֶשׁ', 'nmpr', 'Teresh', 'teresh'),
        'E77580': ('הָמָן', 'nmpr', 'Haman', 'haman'),
        'E77581': ('הַמְּדָתָא', 'nmpr', 'Hammedatha', 'hamdata'),
        'E77582': ('אֲגָגִי', 'adjv', 'Agagite', 'agagi'),
        'E77583': ('נִיסָן', 'nmpr', 'Nissan', 'nisan'),
        'E77584': ('פּוּר', 'subs', 'lot', 'pur'),
        'E77585': ('אֲדָר', 'nmpr', 'Adar', 'adar'),
        'E77586': ('אֲחַשְׁדַּרְפָּן', 'subs', 'satrap', 'achashdarpan'),
        'E77587': ('פַּתְשֶׁגֶן', 'subs', 'copy', 'patshegen'),
        'E77588': ('דחף', 'verb', 'hurry', 'dchf'),
        'E77589': ('הֲתָךְ', 'nmpr', 'Hathach', 'hatach'),
        'E77590': ('פָּרָשָׁה', 'subs', 'statement', 'parasha'),
        'E77591': ('ישׁט', 'verb', 'hold', 'ysht'),
        'E77592': ('שַׁרְבִיט', 'subs', 'staff', 'sharvit'),
        'E77593': ('הַצָּלָה', 'subs', 'deliverance', 'hatsala'),
        'E77594': ('בַּקָּשָׁה', 'subs', 'request', 'bakasha'),
        'E77595': ('זֶרֶשׁ', 'nmpr', 'Zeresh', 'zeresh'),
        'E77596': ('נֵזֶק', 'subs', 'damage', 'neizek'),
        'E77597': ('חַרְבֹונָה', 'nmpr', 'Harbona', 'charvovna'),
        'E77598': ('אָבְדָן', 'subs', 'destruction', 'avdan'),
        'E77599': ('סִיוָן', 'nmpr', 'Siwan', 'sivan'),
        'E77600': ('אֲחַשְׁתְּרָן', 'adjv', 'royal', 'achashtran'),
        'E77601': ('רַמָּכָה', 'subs', 'mare', 'ramacho'),
        'E77602': ('תַּכְרִיךְ', 'subs', 'robe', 'tachrich'),
        'E77603': ('יהד', 'verb', 'be jew', 'yhd'),
        'E77604': ('אַבְדָן', 'subs', 'destruction', 'avdan'),
        'E77605': ('פַּרְשַׁנְדָּתָא', 'nmpr', 'Parshandatha', 'parshandata'),
        'E77606': ('דַּלְפֹון', 'nmpr', 'Dalphon', 'dalfovn'),
        'E77607': ('אַסְפָּתָא', 'nmpr', 'Aspatha', 'aspata'),
        'E77608': ('פֹּורָתָא', 'nmpr', 'Poratha', 'povrata'),
        'E77609': ('אֲדַלְיָא', 'nmpr', 'Adalia', 'adalya'),
        'E77610': ('אֲרִידָתָא', 'nmpr', 'Aridatha', 'aridata'),
        'E77611': ('פַּרְמַשְׁתָּא', 'nmpr', 'Parmashta', 'parmashta'),
        'E77612': ('אֲרִיסַי', 'nmpr', 'Arisai', 'arisay'),
        'E77613': ('אֲרִדַי', 'nmpr', 'Aridai', 'ariday'),
        'E77614': ('וַיְזָתָא', 'nmpr', 'Vaizatha', 'vayzata'),
        'E77615': ('בִּזָּה', 'subs', 'spoil', 'biza'),
        'E77616': ('אִגֶּרֶת', 'subs', 'letter', 'igeret'),
        'E77617': ('תֹּקֶף', 'subs', 'power', 'tokef'),
        'E77618': ('אַשְׁפְּנַז', 'nmpr', 'Ashpenaz', 'ashpnaz'),
        'E77619': ('פַּתְבַּג', 'subs', 'table', 'patbag'),
        'E77620': ('בֵּלְטְשַׁאצַּר', 'nmpr', 'Belteshazzar', 'beiltshatsar'),
        'E77621': ('שַׁדְרַךְ', 'nmpr', 'Shadrach', 'shadrach'),
        'E77622': ('מֵישַׁךְ', 'nmpr', 'Meshach', 'meishach'),
        'E77623': ('עֲבֵד נְגֹו', 'nmpr', 'Abednego', "aveid n'gov"),
        'E77624': ('גִּיל', 'subs', 'age', 'gil'),
        'E77625': ('חוב', 'verb', 'be guilty', 'chvv'),
        'E77626': ('מֶלְצַר', 'subs', 'guardian', 'meltsar'),
        'E77627': ('זֵרֹעִים', 'subs', 'herbs', 'zeiroim'),
        'E77628': ('זֵרְעֹנִים', 'subs', 'herbs', 'zeironim'),
        'E77629': ('אַשָּׁף', 'subs', 'conjurer', 'ashaf'),
        'E77630': ('מֶלֶךְ', 'subs', 'king', 'melech'),
        'E77631': ('עָלַם', 'subs', 'eternity', 'alam'),
        'E77632': ('חיה', 'verb', 'live', 'chyh'),
        'E77633': ('חֵלֶם', 'subs', 'dream', 'cheilem'),
        'E77634': ('עֲבֵד', 'subs', 'servant', 'aveid'),
        'E77635': ('פְּשַׁר', 'subs', 'interpretation', "p'shar"),
        'E77636': ('חוה', 'verb', 'know', 'chvh'),
        'E77637': ('ענה', 'verb', 'answer', 'nh'),
        'E77638': ('כַּשְׂדָּי', 'adjv', 'Chaldaean', 'kasday'),
        'E77639': ('מִלָּה', 'subs', 'word', 'mila'),
        'E77640': ('אַזְדָּא', 'advb', 'publicly known', 'azda'),
        'E77641': ('הֵן', 'conj', 'if', 'hein'),
        'E77642': ('ידע', 'verb', 'know', 'yd'),
        'E77643': ('הַדָּם', 'subs', 'limb', 'hadam'),
        'E77644': ('בַּיִת', 'subs', 'house', 'bayit'),
        'E77645': ('נְוָלוּ', 'subs', 'dunghill', "n'valu"),
        'E77646': ('שׂים', 'verb', 'place', 'sym'),
        'E77647': ('מַתְּנָה', 'subs', 'gift', 'matna'),
        'E77648': ('נְבִזְבָּה', 'subs', 'present', "n'vizba"),
        'E77649': ('יְקָר', 'subs', 'dignity', "y'kar"),
        'E77650': ('שַׂגִּיא', 'adjv', 'much', 'sagi'),
        'E77651': ('קבל', 'verb', 'receive', 'kvl'),
        'E77652': ('קֳדָם', 'subs', 'before', 'kadam'),
        'E77653': ('לָהֵן', 'advb', 'therefore', 'lahein'),
        'E77654': ('תִּנְיָנוּת', 'advb', 'again', 'tinanut'),
        'E77655': ('יַצִּיב', 'adjv', 'reliable', 'yatsiv'),
        'E77656': ('אֲנָה', 'prps', 'I', 'ana'),
        'E77657': ('עִדָּן', 'subs', 'time', 'idan'),
        'E77658': ('אַנְתּוּן', 'prps', 'you', 'antun'),
        'E77659': ('זבן', 'verb', 'buy', 'zvn'),
        'E77660': ('קֳבֵל', 'subs', 'opposite', 'kaveil'),
        'E77661': ('חזה', 'verb', 'see', 'chzh'),
        'E77662': ('חַד', 'subs', 'one', 'chad'),
        'E77663': ('הִיא', 'prps', 'she', 'hi'),
        'E77664': ('דָּת', 'subs', 'law', 'dat'),
        'E77665': ('כִּדְבָה', 'subs', 'lie', 'kidva'),
        'E77666': ('שׁחת', 'verb', 'spoil', 'shcht'),
        'E77667': ('זמן', 'verb', 'arrange', 'zmn'),
        'E77668': ('עַד', 'prep', 'until', 'ad'),
        'E77669': ('שׁנה', 'verb', 'be different', 'shnh'),
        'E77670': ('אִיתַי', 'subs', 'existence', 'itay'),
        'E77671': ('אֱנָשׁ', 'subs', 'mankind', 'enash'),
        'E77672': ('עַל', 'prep', 'upon', 'al'),
        'E77673': ('יַבְּשָׁה', 'subs', 'land', 'yabsha'),
        'E77674': ('יכל', 'verb', 'be able', 'ychl'),
        'E77675': ('רַב', 'adjv', 'great', 'rav'),
        'E77676': ('שַׁלִּיט', 'adjv', 'mighty', 'shalit'),
        'E77677': ('שׁאל', 'verb', 'ask', 'shl'),
        'E77678': ('חַרְטֹּם', 'subs', 'magician', 'chartom'),
        'E77679': ('אָשַׁף', 'subs', 'enchanter', 'ashaf'),
        'E77680': ('יַקִּיר', 'adjv', 'difficult', 'yakir'),
        'E77681': ('אָחֳרָן', 'adjv', 'another', 'acharan'),
        'E77682': ('לָהֵן', 'conj', 'but', 'lahein'),
        'E77683': ('מְדֹור', 'subs', 'dwelling', "m'dovr"),
        'E77684': ('עִם', 'prep', 'with', 'im'),
        'E77685': ('בְּשַׂר', 'subs', 'flesh', "b'sar"),
        'E77686': ('בנס', 'verb', 'become angry', 'vns'),
        'E77687': ('קצף', 'verb', 'get angry', 'ktsf'),
        'E77688': ('חַכִּים', 'adjv', 'wise', 'chakim'),
        'E77689': ('בָּבֶל', 'nmpr', 'Babylon', 'bavel'),
        'E77690': ('נפק', 'verb', 'go out', 'nfk'),
        'E77691': ('קטל', 'verb', 'kill', 'ktl'),
        'E77692': ('בעה', 'verb', 'seek', 'vh'),
        'E77693': ('דָּנִיֵּאל', 'nmpr', 'Daniel', 'danieil'),
        'E77694': ('חֲבַר', 'subs', 'colleague', 'chavar'),
        'E77695': ('בְּ', 'prep', 'in', "b'"),
        'E77696': ('אֱדַיִן', 'advb', 'then', 'edayin'),
        'E77697': ('תוב', 'verb', 'return', 'tvv'),
        'E77698': ('עֵטָה', 'subs', 'advice', 'eita'),
        'E77699': ('טְעֵם', 'subs', 'sense', "t'eim"),
        'E77700': ('אַרְיֹוךְ', 'nmpr', 'Arioch', 'aryovch'),
        'E77701': ('רַב', 'subs', 'chief', 'rav'),
        'E77702': ('טַבַּח', 'subs', 'executioner', 'tabach'),
        'E77703': ('מָה', 'prin', 'what', 'ma'),
        'E77704': ('חצף', 'verb', 'be bare', 'chtsf'),
        'E77705': ('עלל', 'verb', 'enter', "l'l"),
        'E77706': ('זְמָן', 'subs', 'time', "z'man"),
        'E77707': ('נתן', 'verb', 'give', 'ntn'),
        'E77708': ('אזל', 'verb', 'go', 'zl'),
        'E77709': ('חֲנַנְיָה', 'nmpr', 'Hananiah', 'chananya'),
        'E77710': ('מִישָׁאֵל', 'nmpr', 'Mishael', 'mishaeil'),
        'E77711': ('עֲזַרְיָה', 'nmpr', 'Azariah', 'azarya'),
        'E77712': ('רַחֲמִין', 'subs', 'mercy', 'rachamin'),
        'E77713': ('רָז', 'subs', 'secret', 'raz'),
        'E77714': ('שְׁאָר', 'subs', 'rest', "sh'ar"),
        'E77715': ('חֱזוּ', 'subs', 'vision', 'chezu'),
        'E77716': ('לֵילֵי', 'subs', 'night', 'leilei'),
        'E77717': ('גלה', 'verb', 'reveal', 'glh'),
        'E77718': ('ברך', 'verb', 'bless', 'vrch'),
        'E77719': ('הוה', 'verb', 'be', 'hvh'),
        'E77720': ('שֻׁם', 'subs', 'name', 'shum'),
        'E77721': ('חָכְמָה', 'subs', 'wisdom', 'chachma'),
        'E77722': ('גְּבוּרָה', 'subs', 'power', "g'vura"),
        'E77723': ('הוּא', 'prps', 'he', 'hu'),
        'E77724': ('עדה', 'verb', 'go', 'dh'),
        'E77725': ('קום', 'verb', 'stand', 'kvm'),
        'E77726': ('יהב', 'verb', 'give', 'yhv'),
        'E77727': ('מַנְדַּע', 'subs', 'knowledge', 'manda'),
        'E77728': ('בִּינָה', 'subs', 'discernment', 'bina'),
        'E77729': ('עַמִּיק', 'adjv', 'deep', 'amik'),
        'E77730': ('סתר', 'verb', 'hide', 'str'),
        'E77731': ('חֲשֹׁוךְ', 'subs', 'darkness', 'chashovch'),
        'E77732': ('נְהֹור', 'subs', 'light', "n'hovr"),
        'E77733': ('שׁרה', 'verb', 'loosen', 'shrh'),
        'E77734': ('אַב', 'subs', 'father', 'av'),
        'E77735': ('ידה', 'verb', 'praise', 'ydh'),
        'E77736': ('שׁבח', 'verb', 'praise', 'shvch'),
        'E77737': ('כְּעַן', 'advb', 'now', "k'an"),
        'E77738': ('מנה', 'verb', 'count', 'mnh'),
        'E77739': ('כֵּן', 'advb', 'thus', 'kein'),
        'E77740': ('אַל', 'nega', 'not', 'al'),
        'E77741': ('בהל', 'verb', 'disturb', 'vhl'),
        'E77742': ('שׁכח', 'verb', 'find', 'shchch'),
        'E77743': ('גְּבַר', 'subs', 'man', "g'var"),
        'E77744': ('בַּר', 'subs', 'son', 'bar'),
        'E77745': ('גָּלוּ', 'subs', 'exile', 'galu'),
        'E77746': ('יְהוּד', 'nmpr', 'Judaea', "y'hud"),
        'E77747': ('בֵּלְטְשַׁאצַּר', 'nmpr', 'Belteshazzar', 'beiltshatsar'),
        'E77748': ('הֲ', 'inrg', '[interrogative]', 'ah'),
        'E77749': ('כהל', 'verb', 'be able', 'chhl'),
        'E77750': ('גזר', 'verb', 'cut', 'gzr'),
        'E77751': ('בְּרַם', 'advb', 'but', "b'ram"),
        'E77752': ('נְבוּכַדְנֶצַּר', 'nmpr', 'Nebuchadnezzar', "n'vuchadnetsar"),
        'E77753': ('אַחֲרִי', 'subs', 'end', 'achari'),
        'E77754': ('יֹום', 'subs', 'day', 'yovm'),
        'E77755': ('רֵאשׁ', 'subs', 'head', 'reish'),
        'E77756': ('מִשְׁכַּב', 'subs', 'bed', 'mishkav'),
        'E77757': ('אַנְתָּה', 'prps', 'you', 'anta'),
        'E77758': ('רַעְיֹון', 'subs', 'thought', 'rayovn'),
        'E77759': ('סלק', 'verb', 'go up', 'slk'),
        'E77760': ('אַחַר', 'subs', 'after', 'achar'),
        'E77761': ('חַי', 'adjv', 'living', 'chay'),
        'E77762': ('דִּבְרָה', 'subs', 'matter', 'divra'),
        'E77763': ('לְבַב', 'subs', 'heart', "l'vav"),
        'E77764': ('אֲלוּ', 'intj', 'behold', 'alu'),
        'E77765': ('צְלֵם', 'subs', 'statue', "ts'leim"),
        'E77766': ('דִּכֵּן', 'prde', 'that', 'dikein'),
        'E77767': ('זִיו', 'subs', 'brightness', 'ziv'),
        'E77768': ('יַתִּיר', 'adjv', 'excessive', 'yatir'),
        'E77769': ('רֵו', 'subs', 'appearance', 'reiv'),
        'E77770': ('דחל', 'verb', 'fear', 'dchl'),
        'E77771': ('דְּהַב', 'subs', 'gold', "d'hav"),
        'E77772': ('טָב', 'adjv', 'good', 'tav'),
        'E77773': ('חֲדֵה', 'subs', 'breast', 'chadei'),
        'E77774': ('דְּרָע', 'subs', 'arm', "d'ra"),
        'E77775': ('כְּסַף', 'subs', 'silver', "k'saf"),
        'E77776': ('מְעֵה', 'subs', 'belly', "m'ei"),
        'E77777': ('יַרְכָה', 'subs', 'thigh', 'yarcho'),
        'E77778': ('נְחָשׁ', 'subs', 'copper', "n'chash"),
        'E77779': ('שָׁק', 'subs', 'leg', 'shak'),
        'E77780': ('פַּרְזֶל', 'subs', 'iron', 'parzel'),
        'E77781': ('רְגַל', 'subs', 'foot', "r'gal"),
        'E77782': ('חֲסַף', 'subs', 'clay', 'chasaf'),
        'E77783': ('אֶבֶן', 'subs', 'stone', 'even'),
        'E77784': ('יַד', 'subs', 'hand', 'yad'),
        'E77785': ('מחא', 'verb', 'hit', 'mch'),
        'E77786': ('דקק', 'verb', 'crush', "dk'k"),
        'E77787': ('הִמֹּון', 'prps', 'they', 'himovn'),
        'E77788': ('דוק', 'verb', 'pulverise', 'dvk'),
        'E77789': ('עוּר', 'subs', 'chaff', 'ur'),
        'E77790': ('אִדַּר', 'subs', 'threshing-floor', 'idar'),
        'E77791': ('קַיִט', 'subs', 'summer', 'kayit'),
        'E77792': ('נשׂא', 'verb', 'take', 'ns'),
        'E77793': ('רוּחַ', 'subs', 'wind', 'ruach'),
        'E77794': ('אֲתַר', 'subs', 'place', 'atar'),
        'E77795': ('טוּר', 'subs', 'mountain', 'tur'),
        'E77796': ('מלא', 'verb', 'fill', 'ml'),
        'E77797': ('מַלְכוּ', 'subs', 'kingdom', 'malchu'),
        'E77798': ('חֱסֵן', 'subs', 'power', 'chesein'),
        'E77799': ('תְּקֹף', 'subs', 'strength', "t'kof"),
        'E77800': ('דור', 'verb', 'dwell', 'dvr'),
        'E77801': ('חֵיוָה', 'subs', 'animal', 'cheiva'),
        'E77802': ('בַּר', 'subs', 'field', 'bar'),
        'E77803': ('עֹוף', 'subs', 'bird', 'ovf'),
        'E77804': ('שׁלט', 'verb', 'rule', 'shlt'),
        'E77805': ('בָּאתַר', 'subs', 'after', 'batar'),
        'E77806': ('אָחֳרִי', 'adjv', 'another', 'achari'),
        'E77807': ('תְּלִיתָי', 'adjv', 'third', "t'litay"),
        'E77808': ('רְבִיעָי', 'adjv', 'fourth', "r'viay"),
        'E77809': ('תַּקִּיף', 'adjv', 'strong', 'takif'),
        'E77810': ('חשׁל', 'verb', 'hammer', 'chshl'),
        'E77811': ('רעע', 'verb', 'break', 'r'),
        'E77812': ('אִלֵּין', 'prde', 'these', 'ilein'),
        'E77813': ('אֶצְבַּע', 'subs', 'finger', 'etsba'),
        'E77814': ('פֶּחָר', 'subs', 'potter', 'pechar'),
        'E77815': ('פלג', 'verb', 'divide', 'flg'),
        'E77816': ('נִצְבָּה', 'subs', 'planting', 'nitsba'),
        'E77817': ('ערב', 'verb', 'mix', 'rv'),
        'E77818': ('טִין', 'subs', 'clay', 'tin'),
        'E77819': ('קְצָת', 'subs', 'end', "k'tsat"),
        'E77820': ('תבר', 'verb', 'break', 'tvr'),
        'E77821': ('זְרַע', 'subs', 'seed', "z'ra"),
        'E77822': ('דבק', 'verb', 'adhere', 'dvk'),
        'E77823': ('הֵא', 'intj', '[idiomatic prefix]', 'hei'),
        'E77824': ('אִנּוּן', 'prps', 'they', 'inun'),
        'E77825': ('חבל', 'verb', 'be harmful', 'chvl'),
        'E77826': ('עַם', 'subs', 'people', 'am'),
        'E77827': ('שׁבק', 'verb', 'leave', 'shvk'),
        'E77828': ('סוף', 'verb', 'end', 'svf'),
        'E77829': ('אמן', 'verb', 'be steady', 'mn'),
        'E77830': ('נפל', 'verb', 'fall', 'nfl'),
        'E77831': ('אֲנַף', 'subs', 'face', 'anaf'),
        'E77832': ('סגד', 'verb', 'bow', 'sgd'),
        'E77833': ('מִנְחָה', 'subs', 'offering', 'mincha'),
        'E77834': ('נִיחֹוחַ', 'subs', 'incense', 'nichovach'),
        'E77835': ('נסך', 'verb', 'poor out', "ns'ch"),
        'E77836': ('קְשֹׁט', 'subs', 'truth', "k'shot"),
        'E77837': ('מָרֵא', 'subs', 'lord', 'marei'),
        'E77838': ('רבה', 'verb', 'grow', 'rvh'),
        'E77839': ('מְדִינָה', 'subs', 'province', "m'dina"),
        'E77840': ('סְגַן', 'subs', 'prefect', "s'gan"),
        'E77841': ('עֲבִידַה', 'subs', 'work', 'avida'),
        'E77842': ('שַׁדְרַךְ', 'nmpr', 'Shadrach', 'shadrach'),
        'E77843': ('מֵישַׁךְ', 'nmpr', 'Meshach', 'meishach'),
        'E77844': ('עֲבֵד נְגֹו', 'nmpr', 'Abed-Nego', "aveid n'gov"),
        'E77845': ('תְּרַע', 'subs', 'door', "t'ra"),
        'E77846': ('רוּם', 'subs', 'height', 'rum'),
        'E77847': ('אַמָּה', 'subs', 'cubit', 'ama'),
        'E77848': ('שִׁתִּין', 'subs', 'sixty', 'shitin'),
        'E77849': ('פְּתָי', 'subs', 'breadth', "p'tay"),
        'E77850': ('שֵׁת', 'subs', 'six', 'sheit'),
        'E77851': ('בִּקְעָה', 'subs', 'plain', 'bika'),
        'E77852': ('דּוּרָא', 'nmpr', 'Dura', 'dura'),
        'E77853': ('שׁלח', 'verb', 'send', 'shlch'),
        'E77854': ('כנשׁ', 'verb', 'assemble', 'chnsh'),
        'E77855': ('אֲחַשְׁדַּרְפַּן', 'subs', 'satrap', 'achashdarpan'),
        'E77856': ('פֶּחָה', 'subs', 'governor', 'pecha'),
        'E77857': ('אֲדַרְגָּזַר', 'subs', 'counselor', 'adargazar'),
        'E77858': ('גְּדָבַר', 'subs', 'treasurer', "g'davar"),
        'E77859': ('דְּתָבַר', 'subs', 'law official', "d'tavar"),
        'E77860': ('תִּפְתָּי', 'subs', 'police officer', 'tiftay'),
        'E77861': ('שִׁלְטֹן', 'subs', 'dominion', 'shilton'),
        'E77862': ('אתה', 'verb', 'come', 'th'),
        'E77863': ('חֲנֻכָּה', 'subs', 'inauguration', 'chanuka'),
        'E77864': ('כָּרֹוז', 'subs', 'herald', 'karovz'),
        'E77865': ('קרא', 'verb', 'call', 'kr'),
        'E77866': ('חַיִל', 'subs', 'strength', 'chayil'),
        'E77867': ('אֻמָּה', 'subs', 'nation', 'uma'),
        'E77868': ('לִשָּׁן', 'subs', 'tongue', 'lishan'),
        'E77869': ('שׁמע', 'verb', 'hear', 'shm'),
        'E77870': ('קָל', 'subs', 'voice', 'kal'),
        'E77871': ('קֶרֶן', 'subs', 'horn', 'keren'),
        'E77872': ('מַשְׁרֹוקִי', 'subs', 'pipe', 'mashrovki'),
        'E77873': ('קַתְרֹס', 'subs', 'zither', 'katros'),
        'E77874': ('שַׂבְּכָא', 'subs', 'sambuca', 'sabcho'),
        'E77875': ('פְּסַנְתֵּרִין', 'subs', 'psaltery', "p'santeirin"),
        'E77876': ('סוּמְפֹּנְיָה', 'subs', 'symphony', 'sumponya'),
        'E77877': ('זַן', 'subs', 'sort', 'zan'),
        'E77878': ('זְמָר', 'subs', 'music for strings', "z'mar"),
        'E77879': ('מַן', 'prin', 'who', 'man'),
        'E77880': ('שָׁעָה', 'subs', 'hour', "sha'a"),
        'E77881': ('רמה', 'verb', 'throw', 'rmh'),
        'E77882': ('גֹּוא', 'subs', 'interior', 'gov'),
        'E77883': ('אַתּוּן', 'subs', 'furnace', 'atun'),
        'E77884': ('נוּר', 'subs', 'fire', 'nur'),
        'E77885': ('יקד', 'verb', 'burn', 'ykd'),
        'E77886': ('קרב', 'verb', 'approach', 'krv'),
        'E77887': ('אכל', 'verb', 'eat', 'chl'),
        'E77888': ('קְרַץ', 'subs', 'piece', "k'rats"),
        'E77889': ('יְהוּדָי', 'adjv', 'Jew', "y'huday"),
        'E77890': ('יָת', 'prep', '[nota accusativi]', 'yat'),
        'E77891': ('אִלֵּךְ', 'prde', 'these', 'ileich'),
        'E77892': ('פלח', 'verb', 'serve', 'flch'),
        'E77893': ('רְגַז', 'subs', 'anger', "r'gaz"),
        'E77894': ('חֲמָה', 'subs', 'wrath', 'chama'),
        'E77895': ('צְדָא', 'advb', 'really', "ts'da"),
        'E77896': ('עֲתִיד', 'adjv', 'ready', 'atid'),
        'E77897': ('יזב', 'verb', 'save', 'yzv'),
        'E77898': ('חשׁח', 'verb', 'need', 'chshch'),
        'E77899': ('אֲנַחְנָא', 'prps', 'we', 'anachna'),
        'E77900': ('פִּתְגָם', 'subs', 'word', 'pitgam'),
        'E77901': ('אזה', 'verb', 'heat', 'zh'),
        'E77902': ('שְׁבַע', 'subs', 'seven', "sh'va"),
        'E77903': ('גִּבָּר', 'subs', 'strong man', 'gibar'),
        'E77904': ('כפת', 'verb', 'bind', 'chft'),
        'E77905': ('סַרְבָּל', 'subs', 'trousers', 'sarbal'),
        'E77906': ('פְּטַשׁ', 'subs', 'legging', "p'tash"),
        'E77907': ('כַּרְבְּלָה', 'subs', 'hat', 'karbla'),
        'E77908': ('לְבוּשׁ', 'subs', 'garment', "l'vush"),
        'E77909': ('שְׁבִיב', 'subs', 'flame', "sh'viv"),
        'E77910': ('תְּלָת', 'subs', 'three', "t'lat"),
        'E77911': ('תְּוַהּ', 'verb', 'be amazed', "t'vah"),
        'E77912': ('הַדָּבַר', 'subs', 'companion', 'hadavar'),
        'E77913': ('הָא', 'intj', 'behold', 'ha'),
        'E77914': ('אַרְבַּע', 'subs', 'four', 'arba'),
        'E77915': ('הלך', 'verb', 'go', 'hlch'),
        'E77916': ('חֲבָל', 'subs', 'damage', 'chaval'),
        'E77917': ('דמה', 'verb', 'resemble', 'dmh'),
        'E77918': ('עִלָּי', 'adjv', 'highest', 'ilay'),
        'E77919': ('גְּשֵׁם', 'subs', 'body', "g'sheim"),
        'E77920': ('שְׂעַר', 'subs', 'hair', "s'ar"),
        'E77921': ('חרך', 'verb', 'burn', 'chrch'),
        'E77922': ('רֵיחַ', 'subs', 'smell', 'reiach'),
        'E77923': ('מַלְאַךְ', 'subs', 'angel', 'malach'),
        'E77924': ('רחץ', 'verb', 'trust', 'rchts'),
        'E77925': ('שָׁלוּ', 'subs', 'negligence', 'shalu'),
        'E77926': ('שׁוה', 'verb', 'be equal', 'shvh'),
        'E77927': ('נצל', 'verb', 'save', 'ntsl'),
        'E77928': ('צלח', 'verb', 'prosper', 'tslch'),
        'E77929': ('שְׁלָם', 'subs', 'well-being', "sh'lam"),
        'E77930': ('שׂגא', 'verb', 'be much', 'sg'),
        'E77931': ('אָת', 'subs', 'sign', 'at'),
        'E77932': ('תְּמַהּ', 'subs', 'wonder', "t'mah"),
        'E77933': ('שׁפר', 'verb', 'please', 'shfr'),
        'E77934': ('שָׁלְטָן', 'subs', 'official', 'shaltan'),
        'E77935': ('דָּר', 'subs', 'generation', 'dar'),
        'E77936': ('שְׁלֵה', 'adjv', 'carefree', "sh'lei"),
        'E77937': ('רַעְנַן', 'adjv', 'flourishing', 'ranan'),
        'E77938': ('הֵיכַל', 'subs', 'palace', 'heichal'),
        'E77939': ('הַרְהֹר', 'subs', 'imagining', 'harhor'),
        'E77940': ('אָחֳרֵין', 'advb', 'at last', 'acharein'),
        'E77941': ('קַדִּישׁ', 'adjv', 'holy', 'kadish'),
        'E77942': ('אנס', 'verb', 'subdue', 'ns'),
        'E77943': ('אִילָן', 'subs', 'tree', 'ilan'),
        'E77944': ('תקף', 'verb', 'be strong', 'tkf'),
        'E77945': ('מטא', 'verb', 'come', 'mt'),
        'E77946': ('חֲזֹות', 'subs', 'appearance', 'chazovt'),
        'E77947': ('סֹוף', 'subs', 'end', 'sovf'),
        'E77948': ('עֳפִי', 'subs', 'foliage', 'afi'),
        'E77949': ('שַׁפִּיר', 'adjv', 'beautiful', 'shapir'),
        'E77950': ('אֵב', 'subs', 'fruit', 'eiv'),
        'E77951': ('מָזֹון', 'subs', 'food', 'mazovn'),
        'E77952': ('טלל', 'verb', 'shelter', "tl'l"),
        'E77953': ('עֲנַף', 'subs', 'branch', 'anaf'),
        'E77954': ('צִפַּר', 'subs', 'bird', 'tsipar'),
        'E77955': ('זון', 'verb', 'feed', 'zvn'),
        'E77956': ('עִיר', 'subs', 'angel', 'ir'),
        'E77957': ('נחת', 'verb', 'come down', 'ncht'),
        'E77958': ('גדד', 'verb', 'cut down', "gd'd"),
        'E77959': ('קצץ', 'verb', 'cut off', "kts'ts"),
        'E77960': ('נתר', 'verb', 'drop', 'ntr'),
        'E77961': ('בדר', 'verb', 'scatter', 'vdr'),
        'E77962': ('נוד', 'verb', 'flee', 'nvd'),
        'E77963': ('עִקַּר', 'subs', 'offshoot', 'ikar'),
        'E77964': ('שְׁרֹשׁ', 'subs', 'root', "sh'rosh"),
        'E77965': ('אֱסוּר', 'subs', 'bond', 'esur'),
        'E77966': ('דֶּתֶא', 'subs', 'vegetation', 'dete'),
        'E77967': ('טַל', 'subs', 'dew', 'tal'),
        'E77968': ('צבע', 'verb', 'wet', 'tsv'),
        'E77969': ('חֲלָק', 'subs', 'share', 'chalak'),
        'E77970': ('עֲשַׂב', 'subs', 'grass', 'asav'),
        'E77971': ('חלף', 'verb', 'pass by', 'chlf'),
        'E77972': ('גְּזֵרָה', 'subs', 'decision', "g'zeira"),
        'E77973': ('מֵאמַר', 'subs', 'word', 'meimar'),
        'E77974': ('שְׁאֵלָה', 'subs', 'question', "sh'eila"),
        'E77975': ('צבה', 'verb', 'desire', 'tsvh'),
        'E77976': ('שְׁפַל', 'adjv', 'low', "sh'fal"),
        'E77977': ('שׁמם', 'verb', 'be astonished', "shm'm"),
        'E77978': ('שׂנא', 'verb', 'hate', 'sn'),
        'E77979': ('עָר', 'subs', 'adversary', 'ar'),
        'E77980': ('שׁכן', 'verb', 'dwell', 'shchn'),
        'E77981': ('רְבוּ', 'subs', 'greatness', "r'vu"),
        'E77982': ('טרד', 'verb', 'drive out', 'trd'),
        'E77983': ('תֹּור', 'subs', 'ox', 'tovr'),
        'E77984': ('טעם', 'verb', 'eat', 'tm'),
        'E77985': ('קַיָּם', 'adjv', 'during', 'kayam'),
        'E77986': ('מְלַךְ', 'subs', 'advise', "m'lach"),
        'E77987': ('חֲטָא', 'subs', 'sin', 'chata'),
        'E77988': ('צִדְקָה', 'subs', 'charity', 'tsidka'),
        'E77989': ('פרק', 'verb', 'atone', 'frk'),
        'E77990': ('עֲוָיָה', 'subs', 'iniquity', 'avaya'),
        'E77991': ('חנן', 'verb', 'show mercy', "chn'n"),
        'E77992': ('עֲנֵה', 'adjv', 'humble', 'anei'),
        'E77993': ('אַרְכָה', 'subs', 'length', 'archo'),
        'E77994': ('שְׁלֵוָה', 'subs', 'happiness', "sh'leiva"),
        'E77995': ('יְרַח', 'subs', 'month', "y'rach"),
        'E77996': ('תְּרֵין', 'subs', 'two', "t'rein"),
        'E77997': ('עֲשַׂר', 'subs', 'ten', 'asar'),
        'E77998': ('דָּא', 'prde', 'this', 'da'),
        'E77999': ('בנה', 'verb', 'build', 'vnh'),
        'E78000': ('תְּקַף', 'subs', 'strength', "t'kaf"),
        'E78001': ('הֲדַר', 'subs', 'glory', 'hadar'),
        'E78002': ('עֹוד', 'subs', 'still', 'ovd'),
        'E78003': ('פֻּם', 'subs', 'mouth', 'pum'),
        'E78004': ('נְשַׁר', 'subs', 'eagle', "n'shar"),
        'E78005': ('טְפַר', 'subs', 'nail', "t'far"),
        'E78006': ('עַיִן', 'subs', 'eye', 'ayin'),
        'E78007': ('נטל', 'verb', 'lift up', 'ntl'),
        'E78008': ('הדר', 'verb', 'honour', 'hdr'),
        'E78009': ('חשׁב', 'verb', 'consider', 'chshv'),
        'E78010': ('רַבְרְבָנִין', 'subs', 'lords', 'ravrvanin'),
        'E78011': ('תקן', 'verb', 'be stable', 'tkn'),
        'E78012': ('יסף', 'verb', 'add', 'ysf'),
        'E78013': ('רום', 'verb', 'be high', 'rvm'),
        'E78014': ('מַעֲבָד', 'subs', 'work', "ma'avad"),
        'E78015': ('אֲרַח', 'subs', 'way', 'arach'),
        'E78016': ('דִּין', 'subs', 'judgement', 'din'),
        'E78017': ('גֵּוָה', 'subs', 'arrogance', 'geiva'),
        'E78018': ('שׁפל', 'verb', 'be low', 'shfl'),
        'E78019': ('בֵּלְשַׁאצַּר', 'nmpr', 'Belshazzar', 'beilshatsar'),
        'E78020': ('לְחֶם', 'subs', 'meal', "l'chem"),
        'E78021': ('אֲלַף', 'subs', 'thousand', 'alaf'),
        'E78022': ('חֲמַר', 'subs', 'wine', 'chamar'),
        'E78023': ('שׁתה', 'verb', 'drink', 'shth'),
        'E78024': ('מָאן', 'subs', 'vessel', 'man'),
        'E78025': ('יְרוּשְׁלֶם', 'nmpr', 'Jerusalem', "y'rushlem"),
        'E78026': ('שֵׁגַל', 'subs', 'concubine', 'sheigal'),
        'E78027': ('לְחֵנָה', 'subs', 'maid', "l'cheina"),
        'E78028': ('אָע', 'subs', 'wood', 'a'),
        'E78029': ('כתב', 'verb', 'write', 'chtv'),
        'E78030': ('נֶבְרְשָׁה', 'subs', 'lamp', 'nevrsha'),
        'E78031': ('גִּיר', 'subs', 'plaster', 'gir'),
        'E78032': ('כְּתַל', 'subs', 'wall', "k'tal"),
        'E78033': ('פַּס', 'subs', 'palm', 'pas'),
        'E78034': ('קְטַר', 'subs', 'knot', "k'tar"),
        'E78035': ('חֲרַץ', 'subs', 'hip', 'charats'),
        'E78036': ('אַרְכֻבָּה', 'subs', 'knee', 'archubo'),
        'E78037': ('נקשׁ', 'verb', 'knock', 'nksh'),
        'E78038': ('כְּתָב', 'subs', 'writing', "k'tav"),
        'E78039': ('אַרְגְּוָן', 'subs', 'purple', 'argvan'),
        'E78040': ('לבשׁ', 'verb', 'wear', 'lvsh'),
        'E78041': ('הַמְנִיךְ', 'subs', 'necklace', 'hamnich'),
        'E78042': ('צַוַּאר', 'subs', 'neck', 'tsauar'),
        'E78043': ('תְּלָת', 'subs', 'triumvir', "t'lat"),
        'E78044': ('שׁבשׁ', 'verb', 'branch off', 'shvsh'),
        'E78045': ('מַלְכָּה', 'subs', 'queen', 'malka'),
        'E78046': ('מִשְׁתֵּא', 'subs', 'banquet', 'mishtei'),
        'E78047': ('נַהִירוּ', 'subs', 'brilliance', 'nahiru'),
        'E78048': ('שָׂכְלְתָנוּ', 'subs', 'insight', 'sachltanu'),
        'E78049': ('פשׁר', 'verb', 'interpret', 'fshr'),
        'E78050': ('אֲחִידָה', 'subs', 'riddle', 'achida'),
        'E78051': ('זוע', 'verb', 'tremble', 'zv'),
        'E78052': ('זיד', 'verb', 'flow over', 'zyd'),
        'E78053': ('כָּרְסֵא', 'subs', 'throne', 'karsei'),
        'E78054': ('עֲרָד', 'subs', 'wild ass', 'arad'),
        'E78055': ('נִשְׁמָה', 'subs', 'breath', 'nishma'),
        'E78056': ('רשׁם', 'verb', 'write', 'rshm'),
        'E78057': ('מְנֵא', 'subs', 'mina', "m'nei"),
        'E78058': ('תְּקֵל', 'subs', 'shekel', "t'keil"),
        'E78059': ('פְּרֵס', 'subs', 'half-shekel', "p'reis"),
        'E78060': ('שׁלם', 'verb', 'be finished', 'shlm'),
        'E78061': ('תקל', 'verb', 'weigh', 'tkl'),
        'E78062': ('מֹאזְנֵא', 'subs', 'balance', 'moznei'),
        'E78063': ('חַסִּיר', 'adjv', 'wanting', 'chasir'),
        'E78064': ('פרס', 'verb', 'divide', 'frs'),
        'E78065': ('מָדַי', 'nmpr', 'Media', 'maday'),
        'E78066': ('פָּרַס', 'nmpr', 'Persia', 'paras'),
        'E78067': ('כרז', 'verb', 'proclaim', 'chrz'),
        'E78068': ('דָּרְיָוֶשׁ', 'nmpr', 'Darius', 'daryavesh'),
        'E78069': ('מָדַי', 'adjv', 'Median', 'maday'),
        'E78070': ('שְׁנָה', 'subs', 'year', "sh'na"),
        'E78071': ('מְאָה', 'subs', 'hundred', "m'a"),
        'E78072': ('עֶשְׂרִין', 'subs', 'twenty', 'esrin'),
        'E78073': ('עֵלָּא', 'advb', 'above', 'eila'),
        'E78074': ('סָרַךְ', 'subs', 'high official', 'sarach'),
        'E78075': ('נזק', 'verb', 'suffer', 'nzk'),
        'E78076': ('נצח', 'verb', 'shine', 'ntsch'),
        'E78077': ('עשׁת', 'verb', 'intend', 'sht'),
        'E78078': ('עִלָּה', 'subs', 'pretext', 'ila'),
        'E78079': ('צַד', 'subs', 'side', 'tsad'),
        'E78080': ('רגשׁ', 'verb', 'tremble', 'rgsh'),
        'E78081': ('יעט', 'verb', 'advise', 'yt'),
        'E78082': ('קְיָם', 'subs', 'statute', "k'yam"),
        'E78083': ('אֱסָר', 'subs', 'inhibition', 'esar'),
        'E78084': ('בָּעוּ', 'subs', 'request', 'bau'),
        'E78085': ('תְּלָתִין', 'subs', 'thirty', "t'latin"),
        'E78086': ('גֹּב', 'subs', 'pit', 'gov'),
        'E78087': ('אַרְיֵה', 'subs', 'lion', 'aryei'),
        'E78088': ('כַּוָּה', 'subs', 'window', 'kaua'),
        'E78089': ('פתח', 'verb', 'open', "ft'ch"),
        'E78090': ('עִלִּי', 'subs', 'upper room', 'ili'),
        'E78091': ('נֶגֶד', 'subs', 'toward', 'neged'),
        'E78092': ('ברך', 'verb', 'kneel', 'vrch'),
        'E78093': ('בְּרַךְ', 'subs', 'knee', "b'rach"),
        'E78094': ('צלה', 'verb', 'pray', 'tslh'),
        'E78095': ('קַדְמָה', 'subs', 'former time', 'kadma'),
        'E78096': ('באשׁ', 'verb', 'be evil', 'vsh'),
        'E78097': ('בָּל', 'subs', 'mind', 'bal'),
        'E78098': ('מֶעַל', 'subs', 'entrance', 'meal'),
        'E78099': ('שְׁמַשׁ', 'subs', 'sun', "sh'mash"),
        'E78100': ('שׁדר', 'verb', 'throw', 'shdr'),
        'E78101': ('תְּדִיר', 'subs', 'constantly', "t'dir"),
        'E78102': ('חתם', 'verb', 'seal', 'chtm'),
        'E78103': ('עִזְקָה', 'subs', 'seal ring', 'izka'),
        'E78104': ('צְבוּ', 'subs', 'thing', "ts'vu"),
        'E78105': ('בית', 'verb', 'spend the night', 'vyt'),
        'E78106': ('טְוָת', 'advb', 'fastingly', "t'vat"),
        'E78107': ('דַּחֲוָה', 'subs', '[uncertain]', 'dachava'),
        'E78108': ('שְׁנָה', 'subs', 'sleep', "sh'na"),
        'E78109': ('נדד', 'verb', 'flee', "nd'd"),
        'E78110': ('שְׁפַרְפָּר', 'subs', 'dawn', "sh'farpar"),
        'E78111': ('נְגַהּ', 'subs', 'dawn', "n'gah"),
        'E78112': ('עֲצִיב', 'adjv', 'sad', 'atsiv'),
        'E78113': ('זעק', 'verb', 'cry', 'zk'),
        'E78114': ('מלל', 'verb', 'speak', "ml'l"),
        'E78115': ('סגר', 'verb', 'lock', 'sgr'),
        'E78116': ('זָכוּ', 'subs', 'innocence', 'zachu'),
        'E78117': ('אַף', 'advb', 'also', 'af'),
        'E78118': ('חֲבוּלָה', 'subs', 'harm', 'chavula'),
        'E78119': ('טאב', 'verb', 'be good', 'tv'),
        'E78120': ('נְשִׁין', 'subs', 'women', "n'shin"),
        'E78121': ('אַרְעִי', 'subs', 'bottom', 'ari'),
        'E78122': ('גְּרַם', 'subs', 'bone', "g'ram"),
        'E78123': ('כֹּורֶשׁ', 'nmpr', 'Cyrus', 'kovresh'),
        'E78124': ('פַּרְסָי', 'adjv', 'Persian', 'parsay'),
        'E78125': ('אֲרוּ', 'intj', 'behold', 'aru'),
        'E78126': ('גוח', 'verb', 'stir up', 'gvch'),
        'E78127': ('יַם', 'subs', 'sea', 'yam'),
        'E78128': ('קַדְמָי', 'adjv', 'first', 'kadmay'),
        'E78129': ('גַּף', 'subs', 'wing', 'gaf'),
        'E78130': ('מרט', 'verb', 'pluck out', 'mrt'),
        'E78131': ('תִּנְיָן', 'adjv', 'second', 'tinan'),
        'E78132': ('דֹּב', 'subs', 'bear', 'dov'),
        'E78133': ('שְׂטַר', 'subs', 'side', "s'tar"),
        'E78134': ('עֲלַע', 'subs', 'rib', 'ala'),
        'E78135': ('בֵּין', 'subs', 'between', 'bein'),
        'E78136': ('שֵׁן', 'subs', 'tooth', 'shein'),
        'E78137': ('נְמַר', 'subs', 'panther', "n'mar"),
        'E78138': ('גַּב', 'subs', 'side', 'gav'),
        'E78139': ('אֵימְתָנִי', 'adjv', 'terrible', 'eimtani'),
        'E78140': ('רפס', 'verb', 'trample upon', 'rfs'),
        'E78141': ('שׂכל', 'verb', 'consider', 'schl'),
        'E78142': ('זְעֵיר', 'adjv', 'small', "z'eir"),
        'E78143': ('עקר', 'verb', 'pluck out', 'kr'),
        'E78144': ('עַתִּיק', 'adjv', 'old', 'atik'),
        'E78145': ('יתב', 'verb', 'sit', 'ytv'),
        'E78146': ('תְּלַג', 'subs', 'snow', "t'lag"),
        'E78147': ('חִוָּר', 'adjv', 'white', 'chiuar'),
        'E78148': ('עֲמַר', 'subs', 'wool', 'amar'),
        'E78149': ('נְקֵא', 'adjv', 'pure', "n'kei"),
        'E78150': ('גַּלְגַּל', 'subs', 'wheel', 'galgal'),
        'E78151': ('דלק', 'verb', 'burn', 'dlk'),
        'E78152': ('נְהַר', 'subs', 'river', "n'har"),
        'E78153': ('נגד', 'verb', 'flow', 'ngd'),
        'E78154': ('שׁמשׁ', 'verb', 'serve', 'shmsh'),
        'E78155': ('רִבֹּו', 'subs', 'myriad', 'ribov'),
        'E78156': ('סְפַר', 'subs', 'book', "s'far"),
        'E78157': ('יְקֵדָה', 'subs', 'firebrand', "y'keida"),
        'E78158': ('אֶשָּׁא', 'subs', 'fire', 'esha'),
        'E78159': ('חַיִּין', 'subs', 'life', 'chayin'),
        'E78160': ('עֲנָן', 'subs', 'cloud', 'anan'),
        'E78161': ('כרה', 'verb', 'be sick', 'chrh'),
        'E78162': ('נִדְנֶה', 'subs', 'sheath', 'nidneh'),
        'E78163': ('אִנִּין', 'prps', 'they', 'inin'),
        'E78164': ('עֶלְיֹון', 'adjv', 'most high', 'elyovn'),
        'E78165': ('חסן', 'verb', 'possess', 'chsn'),
        'E78166': ('יצב', 'verb', 'stand', 'ytsv'),
        'E78167': ('חַבְרָה', 'subs', 'colleague', 'chavra'),
        'E78168': ('קְרָב', 'subs', 'war', "k'rav"),
        'E78169': ('דושׁ', 'verb', 'trample upon', 'dvsh'),
        'E78170': ('בלה', 'verb', 'wear out', 'vlh'),
        'E78171': ('סבר', 'verb', 'intend', 'svr'),
        'E78172': ('פְּלַג', 'subs', 'half', "p'lag"),
        'E78173': ('שׁמד', 'verb', 'exterminate', 'shmd'),
        'E78174': ('כָּה', 'advb', 'here', 'ka'),
        'E78175': ('לֵב', 'subs', 'heart', 'leiv'),
        'E78176': ('נטר', 'verb', 'guard', 'ntr'),
        'E78177': ('בֵּלְאשַׁצַּר', 'nmpr', 'Belshazzar', 'beilshatsar'),
        'E78178': ('אֻבָל', 'subs', 'canal', 'uval'),
        'E78179': ('אוּלַי', 'nmpr', 'Ulai', 'ulay'),
        'E78180': ('צָפִיר', 'subs', 'he-goat', 'tsafir'),
        'E78181': ('פַּלְמֹנִי', 'subs', 'a certain', 'palmoni'),
        'E78182': ('גַּבְרִיאֵל', 'nmpr', 'Gabriel', 'gavrieil'),
        'E78183': ('עֹמֶד', 'subs', 'place', 'omed'),
        'E78184': ('יְעָף', 'subs', '[uncertain]', "y'af"),
        'E78185': ('חתך', 'verb', 'be determined', "cht'ch"),
        'E78186': ('חָרוּץ', 'subs', 'moat', 'charuts'),
        'E78187': ('צֹוק', 'subs', 'oppression', 'tsovk'),
        'E78188': ('הֵיךְ', 'inrg', 'how', 'heich'),
        'E78189': ('רשׁם', 'verb', 'inscribe', 'rshm'),
        'E78190': ('מָדִי', 'adjv', 'Mede', 'madi'),
        'E78191': ('מִמְשָׁל', 'subs', 'dominion', 'mimshal'),
        'E78192': ('הֶדֶר', 'subs', 'dignity', 'heder'),
        'E78193': ('מֵרַע', 'subs', 'evil deed', 'meira'),
        'E78194': ('מִכְמַנִּים', 'subs', 'hidden treasures', 'michmanim'),
        'E78195': ('אַפֶּדֶן', 'subs', 'royal tent', 'apeden'),
        'E78196': ('זהר', 'verb', 'shine', 'zhr'),
        'E78197': ('מִתְרְדָת', 'nmpr', 'Mithredath', 'mitrdat'),
        'E78198': ('גִּזְבָּר', 'subs', 'treasurer', 'gizbar'),
        'E78199': ('שֵׁשְׁבַּצַּר', 'nmpr', 'Sheshbazzar', 'sheishbatsar'),
        'E78200': ('אֲגַרְטָל', 'subs', 'basket', 'agartal'),
        'E78201': ('מַחֲלָף', 'subs', 'disease', 'machalaf'),
        'E78202': ('כְּפֹור', 'subs', 'bowl', "k'fovr"),
        'E78203': ('יֵשׁוּעַ', 'nmpr', 'Jeshua', 'yeishua'),
        'E78204': ('נְחֶמְיָה', 'nmpr', 'Nehemiah', "n'chemya"),
        'E78205': ('רְעֵלָיָה', 'nmpr', 'Reelaiah', "r'eilaya"),
        'E78206': ('בִּלְשָׁן', 'nmpr', 'Bilshan', 'bilshan'),
        'E78207': ('מִסְפָּר', 'nmpr', 'Mispar', 'mispar'),
        'E78208': ('בִּגְוַי', 'nmpr', 'Bigvai', 'bigvay'),
        'E78209': ('רְחוּם', 'nmpr', 'Rehum', "r'chum"),
        'E78210': ('פַּרְעֹשׁ', 'nmpr', 'Parosh', 'parosh'),
        'E78211': ('אָרַח', 'nmpr', 'Arah', 'arach'),
        'E78212': ('פַּחַת מֹואָב', 'nmpr', 'Pahath-Moab', 'pachat movav'),
        'E78213': ('זַתּוּא', 'nmpr', 'Zattu', 'zatu'),
        'E78214': ('זַכָּי', 'nmpr', 'Zaccai', 'zakay'),
        'E78215': ('בֵּבַי', 'nmpr', 'Bebai', 'beivay'),
        'E78216': ('עַזְגָּד', 'nmpr', 'Azgad', 'azgad'),
        'E78217': ('אֲדֹנִיקָם', 'nmpr', 'Adonikam', 'adonikam'),
        'E78218': ('עָדִין', 'nmpr', 'Adin', 'adin'),
        'E78219': ('אָטֵר', 'nmpr', 'Ater', 'ateir'),
        'E78220': ('בֵּצָי', 'nmpr', 'Bezai', 'beitsay'),
        'E78221': ('יֹורָה', 'nmpr', 'Jorah', 'yovra'),
        'E78222': ('חָשֻׁם', 'nmpr', 'Hashum', 'chashum'),
        'E78223': ('גִּבָּר', 'nmpr', 'Gibbar', 'gibar'),
        'E78224': ('נְטֹפָה', 'nmpr', 'Netophah', "n'tofa"),
        'E78225': ('מִכְמָס', 'nmpr', 'Micmash', 'michmas'),
        'E78226': ('מַגְבִּישׁ', 'nmpr', 'Magbish', 'magbish'),
        'E78227': ('חָרִם', 'nmpr', 'Harim', 'charim'),
        'E78228': ('לֹד', 'nmpr', 'Lod', 'lod'),
        'E78229': ('חָדִיד', 'nmpr', 'Hadid', 'chadid'),
        'E78230': ('אֹונֹו', 'nmpr', 'Ono', 'ovnov'),
        'E78231': ('סְנָאָה', 'nmpr', 'Senaah', "s'na'a"),
        'E78232': ('קַדְמִיאֵל', 'nmpr', 'Kadmiel', 'kadmieil'),
        'E78233': ('הֹודַוְיָה', 'nmpr', 'Hodaviah', 'hovdavya'),
        'E78234': ('טַלְמֹון', 'nmpr', 'Talmon', 'talmovn'),
        'E78235': ('עַקּוּב', 'nmpr', 'Akkub', 'akuv'),
        'E78236': ('חֲטִיטָא', 'nmpr', 'Hatita', 'chatita'),
        'E78237': ('שֹׁבָי', 'nmpr', 'Shobai', 'shovay'),
        'E78238': ('נָתִין', 'subs', 'temple slave', 'natin'),
        'E78239': ('צִיחָא', 'nmpr', 'Ziha', 'tsicha'),
        'E78240': ('חֲשׂוּפָא', 'nmpr', 'Hasupha', 'chasufa'),
        'E78241': ('טַבָּעֹות', 'nmpr', 'Tabbaoth', 'tabaovt'),
        'E78242': ('קֵירֹס', 'nmpr', 'Keros', 'keiros'),
        'E78243': ('סִיעָא', 'nmpr', 'Sia', 'sia'),
        'E78244': ('פָּדֹון', 'nmpr', 'Padon', 'padovn'),
        'E78245': ('לְבָנָה', 'nmpr', 'Lebanah', "l'vana"),
        'E78246': ('חֲגָבָה', 'nmpr', 'Hagabah', 'chagava'),
        'E78247': ('חָגָב', 'nmpr', 'Hagab', 'chagav'),
        'E78248': ('שַׁלְמָי', 'nmpr', 'Shalmai', 'shalmay'),
        'E78249': ('גִּדֵּל', 'nmpr', 'Giddel', 'gideil'),
        'E78250': ('גַּחַר', 'nmpr', 'Gahar', 'gachar'),
        'E78251': ('רְאָיָה', 'nmpr', 'Reaiah', "r'aya"),
        'E78252': ('נְקֹודָא', 'nmpr', 'Nekoda', "n'kovda"),
        'E78253': ('גַּזָּם', 'nmpr', 'Gazzam', 'gazam'),
        'E78254': ('פָּסֵחַ', 'nmpr', 'Paseah', 'paseiach'),
        'E78255': ('בֵּסַי', 'nmpr', 'Besai', 'beisay'),
        'E78256': ('אַסְנָה', 'nmpr', 'Asnah', 'asna'),
        'E78257': ('מְעוּנִים', 'nmpr', 'Meunites', "m'unim"),
        'E78258': ('נְפוּסִים', 'nmpr', 'Nephussim', "n'fusim"),
        'E78259': ('בַּקְבּוּק', 'nmpr', 'Bakbuk', 'bakbuk'),
        'E78260': ('חֲקוּפָא', 'nmpr', 'Hakupha', 'chakufa'),
        'E78261': ('חַרְחוּר', 'nmpr', 'Harhur', 'charchur'),
        'E78262': ('בַּצְלוּת', 'nmpr', 'Bazluth', 'batslut'),
        'E78263': ('מְחִידָא', 'nmpr', 'Mehida', "m'chida"),
        'E78264': ('חַרְשָׁא', 'nmpr', 'Harsha', 'charsha'),
        'E78265': ('בַּרְקֹוס', 'nmpr', 'Barkos', 'barkovs'),
        'E78266': ('תָּמַח', 'nmpr', 'Temah', 'tamach'),
        'E78267': ('נְצִיחַ', 'nmpr', 'Neziah', "n'tsiach"),
        'E78268': ('חֲטִיפָא', 'nmpr', 'Hatipha', 'chatifa'),
        'E78269': ('סֹוטַי', 'nmpr', 'Sotai', 'sovtay'),
        'E78270': ('סֹפֶרֶת', 'subs', 'function of scribe', 'soferet'),
        'E78271': ('פְּרוּדָא', 'nmpr', 'Peruda', "p'ruda"),
        'E78272': ('יַעְלָה', 'nmpr', 'Jaala', 'yala'),
        'E78273': ('דַּרְקֹון', 'nmpr', 'Darkon', 'darkovn'),
        'E78274': ('חַטִּיל', 'nmpr', 'Hattil', 'chatil'),
        'E78275': ('פֹּכֶרֶת הַצְּבָיִים',
                   'nmpr',
                   'Pokereth-Hazzebaim',
                   'pocheret hatsvayim'),
        'E78276': ('אָמִי', 'nmpr', 'Ami', 'ami'),
        'E78277': ('תֵּל מֶלַח', 'nmpr', 'Tel Melah', 'teil melach'),
        'E78278': ('תֵּל חַרְשָׁא', 'nmpr', 'Tel Harsha', 'teil charsha'),
        'E78279': ('כְּרוּב', 'nmpr', 'Kerub', "k'ruv"),
        'E78280': ('אַדָּן', 'nmpr', 'Addon', 'adan'),
        'E78281': ('דְּלָיָה', 'nmpr', 'Delaiah', "d'laya"),
        'E78282': ('חֳבַיָּה', 'nmpr', 'Hobaiah', 'chavaya'),
        'E78283': ('קֹוץ', 'nmpr', 'Koz', 'kovts'),
        'E78284': ('יחשׂ', 'verb', 'register', 'ychs'),
        'E78285': ('תִּרְשָׁתָא', 'subs', 'governor', 'tirshata'),
        'E78286': ('דַּרְכְּמֹנִים', 'subs', 'drachmas', 'darkmonim'),
        'E78287': ('יֹוצָדָק', 'nmpr', 'Jozadak', 'yovtsadak'),
        'E78288': ('רִשְׁיֹון', 'subs', 'permission', 'rishovn'),
        'E78289': ('חֵנָדָד', 'nmpr', 'Henadad', 'cheinadad'),
        'E78290': ('מְצִלְתַּיִם', 'subs', 'cymbals', "m'tsiltayim"),
        'E78291': ('סכר', 'verb', 'hire', "s'chr"),
        'E78292': ('שִׂטְנָה', 'subs', 'accusation', 'sitna'),
        'E78293': ('אַרְתַּחְשַׁשְׂתָּא', 'nmpr', 'Artaxerxes', 'artachshasta'),
        'E78294': ('בִּשְׁלָם', 'nmpr', 'Bishlam', 'bishlam'),
        'E78295': ('כְּנָת', 'subs', 'companion', "k'nat"),
        'E78296': ('נִשְׁתְּוָן', 'subs', 'letter', 'nishtvan'),
        'E78297': ('תרגם', 'verb', 'interpret', 'trgm'),
        'E78298': ('רְחוּם', 'nmpr', 'Rehum', "r'chum"),
        'E78299': ('בְּעֵל', 'subs', 'master', "b'eil"),
        'E78300': ('שִׁמְשַׁי', 'nmpr', 'Shimshai', 'shimshay'),
        'E78301': ('סָפַר', 'subs', 'scribe', 'safar'),
        'E78302': ('אִגְּרָה', 'subs', 'letter', 'igra'),
        'E78303': ('אַרְתַּחְשַׁשְׂתְּא', 'nmpr', 'Artaxerxes', 'artachshast'),
        'E78304': ('כְּנֵמָא', 'advb', 'thus', "k'neima"),
        'E78305': ('כְּנָת', 'subs', 'colleague', "k'nat"),
        'E78306': ('דִּינָי', 'adjv', '[uncertain]', 'dinay'),
        'E78307': ('אֲפַרְסַתְכָי', 'subs', '[uncertain]', "afarsat'choy"),
        'E78308': ('טַרְפְּלָי', 'subs', '[unknown]', 'tarplay'),
        'E78309': ('אֲפָרְסָי', 'subs', '[uncertain]', 'afarsay'),
        'E78310': ('אַרְכְּוָי', 'adjv', 'Erechite', 'arkvay'),
        'E78311': ('בָּבְלָי', 'adjv', 'Babylonian', 'bavlay'),
        'E78312': ('שׁוּשַׁנְכָי', 'adjv', 'man from Susa', 'shushanchoy'),
        'E78313': ('דֶּהָי', 'adjv', 'Dehite', 'dehay'),
        'E78314': ('עֵלָמִי', 'adjv', 'Elamite', 'eilami'),
        'E78315': ('אָסְנַפַּר', 'nmpr', 'Assurbanipal', 'asnapar'),
        'E78316': ('הִמֹּו', 'prps', 'they', 'himov'),
        'E78317': ('קִרְיָה', 'subs', 'city', 'kira'),
        'E78318': ('שָׁמְרָיִן', 'nmpr', 'Samaria', 'shamrayin'),
        'E78319': ('עֲבַר', 'subs', 'opposite bank', 'avar'),
        'E78320': ('כְּעֶנֶת', 'advb', 'now', "k'enet"),
        'E78321': ('פַּרְשֶׁגֶן', 'subs', 'copy', 'parshegen'),
        'E78322': ('לְוָת', 'prep', 'with', "l'vat"),
        'E78323': ('מָרָד', 'adjv', 'rebellious', 'marad'),
        'E78324': ('בִּאישׁ', 'adjv', 'bad', 'bish'),
        'E78325': ('שׁוּר', 'subs', 'wall', 'shur'),
        'E78326': ('כלל', 'verb', 'finish', "chl'l"),
        'E78327': ('אֹשׁ', 'subs', 'foundation', 'osh'),
        'E78328': ('יחט', 'verb', 'dig', 'ycht'),
        'E78329': ('דָּךְ', 'prde', 'that', 'dach'),
        'E78330': ('מִדָּה', 'subs', 'tax', 'mida'),
        'E78331': ('בְּלֹו', 'subs', 'tribute', "b'lov"),
        'E78332': ('הֲלָךְ', 'subs', 'tax', 'halach'),
        'E78333': ('אַפְּתֹם', 'subs', '[uncertain]', 'aptom'),
        'E78334': ('מְלַח', 'subs', 'salt', "m'lach"),
        'E78335': ('מלח', 'verb', 'eat salt', 'mlch'),
        'E78336': ('עַרְוָה', 'subs', 'nakedness', 'arva'),
        'E78337': ('אֲרִיךְ', 'adjv', 'fitting', 'arich'),
        'E78338': ('בקר', 'verb', 'search', 'vkr'),
        'E78339': ('דָּכְרָן', 'subs', 'memorandum', 'dachran'),
        'E78340': ('אֶשְׁתַּדּוּר', 'subs', 'revolt', 'eshtadur'),
        'E78341': ('חרב', 'verb', 'destroy', 'chrv'),
        'E78342': ('כְּעֶת', 'advb', 'now', "k'et"),
        'E78343': ('נִשְׁתְּוָן', 'subs', 'decree', 'nishtvan'),
        'E78344': ('פרשׁ', 'verb', 'interpret', 'frsh'),
        'E78345': ('מְרַד', 'subs', 'rebellion', "m'rad"),
        'E78346': ('בטל', 'verb', 'be idle', 'vtl'),
        'E78347': ('זְהִיר', 'adjv', 'cautious', "z'hir"),
        'E78348': ('בְּהִילוּ', 'subs', 'haste', "b'hilu"),
        'E78349': ('אֶדְרָע', 'subs', 'arm', 'edra'),
        'E78350': ('נבא', 'verb', 'prophesy', 'nv'),
        'E78351': ('חַגַּי', 'nmpr', 'Haggai', 'chagay'),
        'E78352': ('נְבִיא', 'subs', 'prophet', "n'vi"),
        'E78353': ('זְכַרְיָה', 'nmpr', 'Zechariah', "z'charya"),
        'E78354': ('עִדֹּוא', 'nmpr', 'Iddo', 'idov'),
        'E78355': ('יִשְׂרָאֵל', 'nmpr', 'Israel', 'yisraeil'),
        'E78356': ('זְרֻבָּבֶל', 'nmpr', 'Zerubbabel', "z'rubavel"),
        'E78357': ('שְׁאַלְתִּיאֵל', 'nmpr', 'Shealtiel', "sh'altieil"),
        'E78358': ('יֵשׁוּעַ', 'nmpr', 'Jeshua', 'yeishua'),
        'E78359': ('יֹוצָדָק', 'nmpr', 'Jozadak', 'yovtsadak'),
        'E78360': ('סעד', 'verb', 'support', 'sd'),
        'E78361': ('תַּתְּנַי', 'nmpr', 'Tattenai', 'tatnay'),
        'E78362': ('שְׁתַר בֹּוזְנַי', 'nmpr', 'Shethar-Bozenai', "sh'tar bovznay"),
        'E78363': ('אשׁרן', 'subs', '[uncertain]', 'shrn'),
        'E78364': ('בִּנְיָן', 'subs', 'building', 'binan'),
        'E78365': ('שָׂב', 'subs', 'elder', 'sav'),
        'E78366': ('אֲפַרְסְכָי', 'subs', '[uncertain]', "afars'choy"),
        'E78367': ('גְּלָל', 'subs', '[uncertain]', "g'lal"),
        'E78368': ('אָסְפַּרְנָא', 'advb', 'exactly', 'asparna'),
        'E78369': ('רגז', 'verb', 'be angry', 'rgz'),
        'E78370': ('סתר', 'verb', 'demolish', 'str'),
        'E78371': ('יבל', 'verb', 'bring', 'yvl'),
        'E78372': ('שֵׁשְׁבַּצַּר', 'nmpr', 'Sheshbazzar', 'sheishbatsar'),
        'E78373': ('אֵל', 'prde', 'these', 'eil'),
        'E78374': ('דֵּךְ', 'prde', 'that', 'deich'),
        'E78375': ('גְּנַז', 'subs', 'treasure', "g'naz"),
        'E78376': ('תַּמָּה', 'advb', 'there', 'tama'),
        'E78377': ('רְעוּ', 'subs', 'will', "r'u"),
        'E78378': ('אַחְמְתָא', 'nmpr', 'Ecbatana', 'achmta'),
        'E78379': ('בִּירָה', 'subs', 'fortified place', 'bira'),
        'E78380': ('מְגִלָּה', 'subs', 'scroll', "m'gila"),
        'E78381': ('דִּכְרֹון', 'subs', 'memorandum', 'dichrovn'),
        'E78382': ('דבח', 'verb', 'sacrifice', 'dvch'),
        'E78383': ('דְּבַח', 'subs', 'sacrifice', "d'vach"),
        'E78384': ('סבל', 'verb', 'carry', 'svl'),
        'E78385': ('נִדְבָּךְ', 'subs', 'layer', 'nidbach'),
        'E78386': ('חֲדַת', 'adjv', 'new', 'chadat'),
        'E78387': ('נִפְקָה', 'subs', 'expenses', 'nifka'),
        'E78388': ('רַחִיק', 'adjv', 'far', 'rachik'),
        'E78389': ('נְכַס', 'subs', 'possession', "n'chas"),
        'E78390': ('חַשְׁחָה', 'subs', 'need', 'chashcha'),
        'E78391': ('דְּכַר', 'subs', 'ram', "d'char"),
        'E78392': ('אִמַּר', 'subs', 'lamb', 'imar'),
        'E78393': ('עַלְוָה', 'subs', 'sacrifice', 'alva'),
        'E78394': ('חִנְטָה', 'subs', 'wheat', 'chinta'),
        'E78395': ('מְשַׁח', 'subs', 'oil', "m'shach"),
        'E78396': ('כָּהֵן', 'subs', 'priest', 'kahein'),
        'E78397': ('נסח', 'verb', 'pull out', "ns'ch"),
        'E78398': ('זקף', 'verb', 'erect', 'zkf'),
        'E78399': ('מגר', 'verb', 'fall', 'mgr'),
        'E78400': ('נְבוּאָה', 'subs', 'prophecy', "n'vua"),
        'E78401': ('יצא', 'verb', 'go out', 'yts'),
        'E78402': ('אֲדָר', 'nmpr', 'Adar', 'adar'),
        'E78403': ('לֵוָי', 'adjv', 'Levite', 'leivay'),
        'E78404': ('חֶדְוָה', 'subs', 'joy', 'chedva'),
        'E78405': ('צְפִיר', 'subs', 'he-goat', "ts'fir"),
        'E78406': ('עֵז', 'subs', 'goat', 'eiz'),
        'E78407': ('חַטָּאָה', 'subs', 'sin-offering', "chata'a"),
        'E78408': ('מִנְיָן', 'subs', 'number', 'minan'),
        'E78409': ('שְׁבַט', 'subs', 'tribe', "sh'vat"),
        'E78410': ('פְּלֻגָּה', 'subs', 'division', "p'luga"),
        'E78411': ('מַחְלְקָה', 'subs', 'division', 'machlka'),
        'E78412': ('מֹשֶׁה', 'nmpr', 'Moses', 'mosheh'),
        'E78413': ('אַרְתַּחְשַׁסְתְּא', 'nmpr', 'Artaxerxes', 'artachshast'),
        'E78414': ('עֶזְרָא', 'nmpr', 'Ezra', 'ezra'),
        'E78415': ('מְרָיֹות', 'nmpr', 'Meraioth', "m'rayovt"),
        'E78416': ('זְרַחְיָה', 'nmpr', 'Zerahiah', "z'rachya"),
        'E78417': ('עֻזִּי', 'nmpr', 'Uzzi', 'uzi'),
        'E78418': ('אֲבִישׁוּעַ', 'nmpr', 'Abishua', 'avishua'),
        'E78419': ('יְסֻד', 'subs', 'establishment', "y'sud"),
        'E78420': ('פַּרְשֶׁגֶן', 'subs', 'copy', 'parshegen'),
        'E78421': ('עֶזְרָא', 'nmpr', 'Ezra', 'ezra'),
        'E78422': ('גמר', 'verb', 'bring to an end', 'gmr'),
        'E78423': ('נדב', 'verb', 'be willing', 'ndv'),
        'E78424': ('מִשְׁכַּן', 'subs', 'abode', 'mishkan'),
        'E78425': ('קנה', 'verb', 'buy', 'knh'),
        'E78426': ('נְסַךְ', 'subs', 'libation', "n'sach"),
        'E78427': ('מַדְבַּח', 'subs', 'altar', 'madbach'),
        'E78428': ('אַח', 'subs', 'brother', 'ach'),
        'E78429': ('יטב', 'verb', 'be good', 'ytv'),
        'E78430': ('פָּלְחָן', 'subs', 'service', 'palchan'),
        'E78431': ('חַשְׁחוּ', 'subs', 'need', 'chashchu'),
        'E78432': ('גִּזְבַּר', 'subs', 'treasurer', 'gizbar'),
        'E78433': ('כַּכַּר', 'subs', 'talent', 'kakar'),
        'E78434': ('כֹּר', 'subs', 'kor', 'kor'),
        'E78435': ('בַּת', 'subs', 'bath', 'bat'),
        'E78436': ('אַדְרַזְדָּא', 'advb', 'with zeal', 'adrazda'),
        'E78437': ('קְצַף', 'subs', 'wrath', "k'tsaf"),
        'E78438': ('זַמָּר', 'subs', 'musician', 'zamar'),
        'E78439': ('תָּרָע', 'subs', 'door-keeper', 'tara'),
        'E78440': ('נְתִין', 'subs', 'temple servant', "n'tin"),
        'E78441': ('שׁפט', 'verb', 'judge', 'shft'),
        'E78442': ('דַּיָּן', 'subs', 'judge', 'dayan'),
        'E78443': ('דין', 'verb', 'judge', 'dyn'),
        'E78444': ('מֹות', 'subs', 'death', 'movt'),
        'E78445': ('שְׁרֹשִׁי', 'subs', 'banishment', "sh'roshi"),
        'E78446': ('עֲנָשׁ', 'subs', 'fine', 'anash'),
        'E78447': ('חַטּוּשׁ', 'nmpr', 'Hattush', 'chatush'),
        'E78448': ('שְׁכַנְיָה', 'nmpr', 'Shecaniah', "sh'chanya"),
        'E78449': ('אֶלְיְהֹועֵינַי', 'nmpr', 'Eliehoenai', 'elyhoveinay'),
        'E78450': ('יַחֲזִיאֵל', 'nmpr', 'Jahaziel', 'yachazieil'),
        'E78451': ('יְשַׁעְיָה', 'nmpr', 'Jeshaiah', "y'shaya"),
        'E78452': ('זְבַדְיָה', 'nmpr', 'Zebadiah', "z'vadya"),
        'E78453': ('יְחִיאֵל', 'nmpr', 'Jehiel', "y'chieil"),
        'E78454': ('יֹוסִפְיָה', 'nmpr', 'Josiphiah', 'yovsifa'),
        'E78455': ('קָטָן', 'nmpr', 'Hakkatan', 'katan'),
        'E78456': ('יְעִיאֵל', 'nmpr', 'Jeiel', "y'ieil"),
        'E78457': ('עוּתַי', 'nmpr', 'Uthai', 'utay'),
        'E78458': ('אַהֲוָא', 'nmpr', 'Ahava', 'ahava'),
        'E78459': ('יָרִיב', 'nmpr', 'Jarib', 'yariv'),
        'E78460': ('יֹויָרִיב', 'nmpr', 'Joiarib', 'yovyariv'),
        'E78461': ('אִדֹּו', 'nmpr', 'Iddo', 'idov'),
        'E78462': ('כָּסִפְיָא', 'nmpr', 'Casiphia', 'kasifa'),
        'E78463': ('שֵׁרֵבְיָה', 'nmpr', 'Sherebiah', 'sheireiva'),
        'E78464': ('חֲשַׁבְיָה', 'nmpr', 'Hashabiah', 'chashavya'),
        'E78465': ('אֲדַרְכֹּון', 'subs', '[currency unit]', 'adarkovn'),
        'E78466': ('צהב', 'verb', 'gleam', "ts'hv"),
        'E78467': ('מְרֵמֹות', 'nmpr', 'Meremoth', "m'reimovt"),
        'E78468': ('נֹועַדְיָה', 'nmpr', 'Noadiah', 'novadya'),
        'E78469': ('בִּנּוּי', 'nmpr', 'Binnui', 'binuy'),
        'E78470': ('תַּעֲנִית', 'subs', 'humiliation', "ta'anit"),
        'E78471': ('עַבְדֻת', 'subs', 'servitude', 'avdut'),
        'E78472': ('בֶּכֶה', 'subs', 'weeping', 'becheh'),
        'E78473': ('יְהֹוחָנָן', 'nmpr', 'Jehohanan', "y'hovchanan"),
        'E78474': ('אֶלְיָשִׁיב', 'nmpr', 'Eliashib', 'elyashiv'),
        'E78475': ('זמן', 'verb', 'appoint', 'zmn'),
        'E78476': ('יַחְזְיָה', 'nmpr', 'Jahzeiah', 'yachzya'),
        'E78477': ('שַׁבְּתַי', 'nmpr', 'Shabbethai', 'shabtay'),
        'E78478': ('אֶלְיֹועֵינַי', 'nmpr', 'Elioenai', 'elyoveinay'),
        'E78479': ('קֵלָיָה', 'nmpr', 'Kelaiah', 'keilaya'),
        'E78480': ('קְלִיטָא', 'nmpr', 'Kelita', "k'lita"),
        'E78481': ('פְּתַחְיָה', 'nmpr', 'Pethahiah', "p'tachya"),
        'E78482': ('טֶלֶם', 'nmpr', 'Telem', 'telem'),
        'E78483': ('רַמְיָה', 'nmpr', 'Ramiah', 'ramya'),
        'E78484': ('יִזִּיָּה', 'nmpr', 'Izziah', 'yizia'),
        'E78485': ('מִיָּמִן', 'nmpr', 'Mijamin', 'miamin'),
        'E78486': ('עַבְדִּי', 'nmpr', 'Abdi', 'avdi'),
        'E78487': ('יְרֵמֹות', 'nmpr', 'Jeremoth', "y'reimovt"),
        'E78488': ('זָבָד', 'nmpr', 'Zabad', 'zavad'),
        'E78489': ('עֲזִיזָא', 'nmpr', 'Aziza', 'aziza'),
        'E78490': ('זַבַּי', 'nmpr', 'Zabbai', 'zabay'),
        'E78491': ('עַתְלָי', 'nmpr', 'Athlai', 'atlay'),
        'E78492': ('מַלּוּךְ', 'nmpr', 'Malluch', 'maluch'),
        'E78493': ('שְׁאָל', 'nmpr', 'Sheal', "sh'al"),
        'E78494': ('רָמֹות', 'nmpr', 'Jeremoth', 'ramovt'),
        'E78495': ('עַדְנָא', 'nmpr', 'Adna', 'adna'),
        'E78496': ('כְּלָל', 'nmpr', 'Kelal', "k'lal"),
        'E78497': ('יִשִּׁיָּה', 'nmpr', 'Ishijah', 'yishia'),
        'E78498': ('שְׁמַרְיָה', 'nmpr', 'Shemariah', "sh'marya"),
        'E78499': ('מַתְּנַי', 'nmpr', 'Mattenai', 'matnay'),
        'E78500': ('מַתַּתָּה', 'nmpr', 'Mattattah', 'matata'),
        'E78501': ('יְרֵמַי', 'nmpr', 'Jeremai', "y'reimay"),
        'E78502': ('מַעֲדַי', 'nmpr', 'Maadai', "ma'aday"),
        'E78503': ('אוּאֵל', 'nmpr', 'Uel', 'ueil'),
        'E78504': ('בֵּדְיָה', 'nmpr', 'Bedeiah', 'beida'),
        'E78505': ('כְּלֻהוּ', 'nmpr', 'Keluhi', "k'luhu"),
        'E78506': ('וַנְיָה', 'nmpr', 'Vaniah', 'vanya'),
        'E78507': ('יַעֲשַׂי', 'nmpr', 'Jaasu', "ya'asay"),
        'E78508': ('מַכְנַדְבַי', 'nmpr', 'Macnadebai', 'machnadvay'),
        'E78509': ('שָׁשַׁי', 'nmpr', 'Shashai', 'shashay'),
        'E78510': ('שָׁרַי', 'nmpr', 'Sharai', 'sharay'),
        'E78511': ('עֲזַרְאֵל', 'nmpr', 'Azarel', 'azareil'),
        'E78512': ('מַתִּתְיָה', 'nmpr', 'Mattithiah', 'matita'),
        'E78513': ('זְבִינָא', 'nmpr', 'Zebina', "z'vina"),
        'E78514': ('יַדַּי', 'nmpr', 'Jaddai', 'yaday'),
        'E78515': ('חֲכַלְיָה', 'nmpr', 'Hacaliah', 'chachalya'),
        'E78516': ('קַשָּׁב', 'adjv', 'attentive', 'kashav'),
        'E78517': ('סַנְבַלַּט', 'nmpr', 'Sanballat', 'sanvalat'),
        'E78518': ('חֹרֹנִי', 'adjv', 'Horonite', 'choroni'),
        'E78519': ('עֵין הַתַּנִּין', 'nmpr', 'the Jackal Well', 'ein hatanin'),
        'E78520': ('גֶּשֶׁם', 'nmpr', 'Geshem', 'geshem'),
        'E78521': ('עַרְבִי', 'adjv', 'Arab', 'arvi'),
        'E78522': ('מֵאָה', 'nmpr', 'the Hundred', 'meia'),
        'E78523': ('אִמְרִי', 'nmpr', 'Imri', 'imri'),
        'E78524': ('מְשֵׁיזַבְאֵל', 'nmpr', 'Meshezabel', "m'sheizaveil"),
        'E78525': ('יֹויָדָע', 'nmpr', 'Joiada', 'yovyada'),
        'E78526': ('בְּסֹודְיָה', 'nmpr', 'Besodeiah', "b'sovdya"),
        'E78527': ('מְלַטְיָה', 'nmpr', 'Melatiah', "m'latya"),
        'E78528': ('יָדֹון', 'nmpr', 'Jadon', 'yadovn'),
        'E78529': ('מֵרֹנֹתִי', 'adjv', 'Meronothite', 'meironoti'),
        'E78530': ('חַרְהֲיָה', 'nmpr', 'Harhaiah', 'charhaya'),
        'E78531': ('עזב', 'verb', 'flag', 'zv'),
        'E78532': ('רְפָיָה', 'nmpr', 'Rephaiah', "r'faya"),
        'E78533': ('פֶּלֶךְ', 'subs', 'district', 'pelech'),
        'E78534': ('יְדָיָה', 'nmpr', 'Jedaiah', "y'daya"),
        'E78535': ('חֲרוּמַף', 'nmpr', 'Harumaph', 'charumaf'),
        'E78536': ('חֲשַׁבְנְיָה', 'nmpr', 'Hashabneiah', 'chashavnya'),
        'E78537': ('חַשּׁוּב', 'nmpr', 'Hasshub', 'chashuv'),
        'E78538': ('לֹוחֵשׁ', 'nmpr', 'Hallohesh', 'lovcheish'),
        'E78539': ('שַׁלּוּן', 'nmpr', 'Shallun', 'shalun'),
        'E78540': ('כָּל חֹזֶה', 'nmpr', 'Col-Hozeh', 'kal chozeh'),
        'E78541': ('טלל', 'verb', 'cover', "tl'l"),
        'E78542': ('שֶׁלַח', 'subs', 'water-conduit', 'shelach'),
        'E78543': ('עַזְבּוּק', 'nmpr', 'Azbuk', 'azbuk'),
        'E78544': ('בַּוַּי', 'nmpr', 'Binnui', 'bauay'),
        'E78545': ('עֵזֶר', 'nmpr', 'Ezer', 'eizer'),
        'E78546': ('עֲנָנְיָה', 'nmpr', 'Ananiah', 'ananya'),
        'E78547': ('פָּלָל', 'nmpr', 'Palal', 'palal'),
        'E78548': ('אוּזַי', 'nmpr', 'Uzai', 'uzay'),
        'E78549': ('צָלָף', 'nmpr', 'Zalaph', 'tsalaf'),
        'E78550': ('נִשְׁכָּה', 'subs', 'hall', 'nishka'),
        'E78551': ('צֹרְפִי', 'subs', 'guild of goldsmiths', 'tsorfi'),
        'E78552': ('אֲמֵלָל', 'adjv', 'decaying', 'ameilal'),
        'E78553': ('בּוּזָה', 'subs', 'contempt', 'buza'),
        'E78554': ('מִדָּה', 'subs', 'tribute', 'mida'),
        'E78555': ('מלך', 'verb', 'counsel', 'mlch'),
        'E78556': ('מַשָּׁא', 'subs', 'claim', 'masha'),
        'E78557': ('כְּפִירִים', 'nmpr', 'Kephirim', "k'firim"),
        'E78558': ('גַּשְׁמוּ', 'nmpr', 'Geshem', 'gashmu'),
        'E78559': ('נְבוּאָה', 'subs', 'prophecy', "n'vua"),
        'E78560': ('אֱלוּל', 'nmpr', 'Elul', 'elul'),
        'E78561': ('גוף', 'verb', 'shut', 'gvf'),
        'E78562': ('יַחַשׂ', 'subs', 'book', 'yachas'),
        'E78563': ('רַעַמְיָה', 'nmpr', 'Raamiah', "ra'amya"),
        'E78564': ('נַחֲמָנִי', 'nmpr', 'Nahamani', 'nachamani'),
        'E78565': ('מִסְפֶּרֶת', 'nmpr', 'Mispereth', 'misperet'),
        'E78566': ('נְחוּם', 'nmpr', 'Nehum', "n'chum"),
        'E78567': ('חָרִיף', 'nmpr', 'Hariph', 'charif'),
        'E78568': ('בֵּית עַזְמָוֶת', 'nmpr', 'Azmaveth', 'beit azmavet'),
        'E78569': ('הֹודְוָה', 'nmpr', 'Hodaviah', 'hovdva'),
        'E78570': ('נְפוּשְׁסִים', 'nmpr', 'Nephussim', "n'fushsim"),
        'E78571': ('פְּרִידָא', 'nmpr', 'Perida', "p'rida"),
        'E78572': ('אַדֹּון', 'nmpr', 'Addon', 'adovn'),
        'E78573': ('שֶׁמַע', 'nmpr', 'Shema', 'shema'),
        'E78574': ('עֲנָיָה', 'nmpr', 'Anaiah', 'anaya'),
        'E78575': ('חַשְׁבַּדָּנָה', 'nmpr', 'Hashbaddanah', 'chashbadana'),
        'E78576': ('מֹעַל', 'subs', 'lifting', 'moal'),
        'E78577': ('הֹודִיָּה', 'nmpr', 'Hodiah', 'hovdia'),
        'E78578': ('פְּלָאיָה', 'nmpr', 'Pelaiah', "p'laya"),
        'E78579': ('מַשְׁמַנִּים', 'subs', 'dishes', 'mashmanim'),
        'E78580': ('חֶדְוָה', 'subs', 'joy', 'chedva'),
        'E78581': ('שְׁבַנְיָה', 'nmpr', 'Shebaniah', "sh'vanya"),
        'E78582': ('בֻּנִּי', 'nmpr', 'Bunni', 'buni'),
        'E78583': ('כְּנָנִי', 'nmpr', 'Kenani', "k'nani"),
        'E78584': ('פֵּאָה', 'subs', 'piece', 'peia'),
        'E78585': ('עדן', 'verb', 'be luxurious', 'dn'),
        'E78586': ('אֲמָנָה', 'subs', 'trustworthy arrangement', 'amana'),
        'E78587': ('גִּנְּתֹון', 'nmpr', 'Ginnethon', 'gintovn'),
        'E78588': ('מַעַזְיָה', 'nmpr', 'Maaziah', "ma'azya"),
        'E78589': ('בִּלְגַּי', 'nmpr', 'Bilgai', 'bilgay'),
        'E78590': ('אֲזַנְיָה', 'nmpr', 'Azaniah', 'azanya'),
        'E78591': ('בְּנִינוּ', 'nmpr', 'Beninu', "b'ninu"),
        'E78592': ('עֲנָתֹות', 'nmpr', 'Anathoth', 'anatovt'),
        'E78593': ('נֵיבַי', 'nmpr', 'Nebai', 'neivay'),
        'E78594': ('מַגְפִּיעָשׁ', 'nmpr', 'Magpiash', 'magpiash'),
        'E78595': ('חֵזִיר', 'nmpr', 'Hezir', 'cheizir'),
        'E78596': ('יַדּוּעַ', 'nmpr', 'Jaddua', 'yadua'),
        'E78597': ('פְּלַטְיָה', 'nmpr', 'Pelatiah', "p'latya"),
        'E78598': ('פִּלְחָא', 'nmpr', 'Pilha', 'pilcha'),
        'E78599': ('שֹׁובֵק', 'nmpr', 'Shobek', 'shovveik'),
        'E78600': ('חֲשַׁבְנָה', 'nmpr', 'Hashabnah', 'chashavna'),
        'E78601': ('עָנָן', 'nmpr', 'Anan', 'anan'),
        'E78602': ('מַקָּחֹות', 'subs', 'wares', 'makachovt'),
        'E78603': ('קֻרְבָּן', 'subs', 'supply', 'kurban'),
        'E78604': ('עֲתָיָה', 'nmpr', 'Athaiah', 'ataya'),
        'E78605': ('חֲזָיָה', 'nmpr', 'Hazaiah', 'chazaya'),
        'E78606': ('סַלּוּא', 'nmpr', 'Sallu', 'salu'),
        'E78607': ('יֹועֵד', 'nmpr', 'Joed', 'yoveid'),
        'E78608': ('גַּבַּי', 'nmpr', 'Gabbai', 'gabay'),
        'E78609': ('סַלַּי', 'nmpr', 'Sallai', 'salay'),
        'E78610': ('סְנוּאָה', 'nmpr', 'Hassenuah', "s'nua"),
        'E78611': ('פְּלַלְיָה', 'nmpr', 'Pelaliah', "p'lalya"),
        'E78612': ('אַמְצִי', 'nmpr', 'Amzi', 'amtsi'),
        'E78613': ('עֲמַשְׁסַי', 'nmpr', 'Amashsai', 'amashsay'),
        'E78614': ('אַחְזַי', 'nmpr', 'Ahzai', 'achzay'),
        'E78615': ('מְשִׁלֵּמֹות', 'nmpr', 'Meshillemoth', "m'shileimovt"),
        'E78616': ('זַבְדִּיאֵל', 'nmpr', 'Zabdiel', 'zavdieil'),
        'E78617': ('עַזְרִיקָם', 'nmpr', 'Azrikam', 'azrikam'),
        'E78618': ('בּוּנִּי', 'nmpr', 'Bunni', 'buni'),
        'E78619': ('בַּקְבֻּקְיָה', 'nmpr', 'Bakbukiah', 'bakbukya'),
        'E78620': ('גָּלָל', 'nmpr', 'Galal', 'galal'),
        'E78621': ('יְדוּתוּן', 'nmpr', 'Jeduthun', "y'dutun"),
        'E78622': ('גִּשְׁפָּא', 'nmpr', 'Gishpa', 'gishpa'),
        'E78623': ('יְקַבְצְאֵל', 'nmpr', 'Jekabzeel', "y'kavtseil"),
        'E78624': ('יֵשׁוּעַ', 'nmpr', 'Jeshua', 'yeishua'),
        'E78625': ('מְכֹנָה', 'nmpr', 'Meconah', "m'chona"),
        'E78626': ('עַיָּה', 'nmpr', 'Aiah', 'aya'),
        'E78627': ('עֲנָנְיָה', 'nmpr', 'Ananiah', 'ananya'),
        'E78628': ('נְבַלָּט', 'nmpr', 'Neballat', "n'valat"),
        'E78629': ('חָרָשׁ', 'subs', 'craftsman', 'charash'),
        'E78630': ('גִּנְּתֹוי', 'nmpr', 'Ginnethoi', 'gintovy'),
        'E78631': ('מַעַדְיָה', 'nmpr', 'Moadiah', "ma'adya"),
        'E78632': ('בִּלְגָּה', 'nmpr', 'Bilgah', 'bilga'),
        'E78633': ('עָמֹוק', 'nmpr', 'Amok', 'amovk'),
        'E78634': ('הֻיְּדֹות', 'subs', '[uncertain]', 'huydovt'),
        'E78635': ('עֻנִּי', 'nmpr', 'Unni', 'uni'),
        'E78636': ('יֹויָקִים', 'nmpr', 'Joiakim', 'yovyakim'),
        'E78637': ('מְרָיָה', 'nmpr', 'Meraiah', "m'raya"),
        'E78638': ('מְלִיכוּ', 'nmpr', 'Malluch', "m'lichu"),
        'E78639': ('חֶלְקַי', 'nmpr', 'Helkai', 'chelkay'),
        'E78640': ('מִנְיָמִין', 'nmpr', 'Miniamin', 'minamin'),
        'E78641': ('מֹועַדְיָה', 'nmpr', 'Moadiah', 'movadya'),
        'E78642': ('פִּלְטָי', 'nmpr', 'Piltai', 'piltay'),
        'E78643': ('קַלָּי', 'nmpr', 'Kallai', 'kalay'),
        'E78644': ('פָּרְסִי', 'adjv', 'Persian', 'parsi'),
        'E78645': ('אָסֹף', 'subs', 'store', 'asof'),
        'E78646': ('בֵּית הַגִּלְגָּל', 'nmpr', 'Beth Gilgal', 'beit hagilgal'),
        'E78647': ('תַּהֲלֻכֹת', 'subs', 'procession', 'tahaluchot'),
        'E78648': ('מִלֲלַי', 'nmpr', 'Milalai', 'milalay'),
        'E78649': ('גִּלֲלַי', 'nmpr', 'Gilalai', 'gilalay'),
        'E78650': ('מָעַי', 'nmpr', 'Maai', "ma'ay"),
        'E78651': ('יִזְרַחְיָה', 'nmpr', 'Izrahiah', 'yizrachya'),
        'E78652': ('לֵן', 'adjv', 'spending night', 'lein'),
        'E78653': ('גֹּאֲלִים', 'subs', 'defilement', 'goalim'),
        'E78654': ('דִּיפַת', 'nmpr', 'Riphath', 'difat'),
        'E78655': ('רֹודָנִים', 'nmpr', 'Rodanim', 'rovdanim'),
        'E78656': ('אֹופִיר', 'nmpr', 'Ophir', 'ovfir'),
        'E78657': ('צְפִי', 'nmpr', 'Zephi', "ts'fi"),
        'E78658': ('הֹומָם', 'nmpr', 'Homam', 'hovmam'),
        'E78659': ('עַלְיָן', 'nmpr', 'Alvan', 'alyan'),
        'E78660': ('שְׁפִי', 'nmpr', 'Shephi', "sh'fi"),
        'E78661': ('חַמְרָן', 'nmpr', 'Hemdan', 'chamran'),
        'E78662': ('יַעֲקָן', 'nmpr', 'Jaakan', "ya'akan"),
        'E78663': ('פָּעִי', 'nmpr', 'Pai', 'pai'),
        'E78664': ('אָהֳלִיבָמָה', 'nmpr', '[tribe]', 'ahalivama'),
        'E78665': ('דָּרַע', 'nmpr', 'Darda', 'dara'),
        'E78666': ('עָכָר', 'nmpr', 'Achar', 'achor'),
        'E78667': ('כְּלוּבָי', 'nmpr', 'Caleb', "k'luvay"),
        'E78668': ('שַׂלְמָא', 'nmpr', 'Salma', 'salma'),
        'E78669': ('שִׁמְעָא', 'nmpr', 'Shimea', 'shima'),
        'E78670': ('רַדַּי', 'nmpr', 'Raddai', 'raday'),
        'E78671': ('אֹצֶם', 'nmpr', 'Ozem', 'otsem'),
        'E78672': ('יְרִיעֹות', 'nmpr', 'Jerioth', "y'riovt"),
        'E78673': ('יֵשֶׁר', 'nmpr', 'Jesher', 'yeisher'),
        'E78674': ('אַרְדֹּון', 'nmpr', 'Ardon', 'ardovn'),
        'E78675': ('אַשְׁחוּר', 'nmpr', 'Ashhur', 'ashchur'),
        'E78676': ('בּוּנָה', 'nmpr', 'Bunah', 'buna'),
        'E78677': ('אֹרֶן', 'nmpr', 'Oren', 'oren'),
        'E78678': ('עֲטָרָה', 'nmpr', 'Atarah', 'atara'),
        'E78679': ('מַעַץ', 'nmpr', 'Maaz', "ma'ats"),
        'E78680': ('עֵקֶר', 'nmpr', 'Eker', 'eiker'),
        'E78681': ('שַׁמַּי', 'nmpr', 'Shammai', 'shamay'),
        'E78682': ('יָדָע', 'nmpr', 'Jada', 'yada'),
        'E78683': ('אֲבִישׁוּר', 'nmpr', 'Abishur', 'avishur'),
        'E78684': ('אֲבִיהַיִל', 'nmpr', 'Abihail', 'avihayil'),
        'E78685': ('אַחְבָּן', 'nmpr', 'Ahban', 'achban'),
        'E78686': ('מֹולִיד', 'nmpr', 'Molid', 'movlid'),
        'E78687': ('סֶלֶד', 'nmpr', 'Seled', 'seled'),
        'E78688': ('אַפָּיִם', 'nmpr', 'Appaim', 'apayim'),
        'E78689': ('יִשְׁעִי', 'nmpr', 'Ishi', 'yishi'),
        'E78690': ('שֵׁשָׁן', 'nmpr', 'Sheshan', 'sheishan'),
        'E78691': ('אַחְלָי', 'nmpr', 'Ahlai', 'achlay'),
        'E78692': ('זָזָא', 'nmpr', 'Zaza', 'zaza'),
        'E78693': ('יַרְחָע', 'nmpr', 'Jarha', 'yarcha'),
        'E78694': ('עַתָּי', 'nmpr', 'Attai', 'atay'),
        'E78695': ('אֶפְלָל', 'nmpr', 'Ephlal', 'eflal'),
        'E78696': ('סִסְמַי', 'nmpr', 'Sismai', 'sismay'),
        'E78697': ('יְקַמְיָה', 'nmpr', 'Jekamiah', "y'kamya"),
        'E78698': ('מֵישָׁע', 'nmpr', 'Mesha', 'meisha'),
        'E78699': ('מָרֵשָׁה', 'nmpr', 'Mareshah', 'mareisha'),
        'E78700': ('תַּפֻּחַ', 'nmpr', 'Tappuah', 'tapuach'),
        'E78701': ('רַחַם', 'nmpr', 'Raham', 'racham'),
        'E78702': ('יָרְקֳעָם', 'nmpr', 'Jorkeam', "yarka'am"),
        'E78703': ('מָעֹון', 'nmpr', 'Maon', 'maovn'),
        'E78704': ('חָרָן', 'nmpr', 'Haran', 'charan'),
        'E78705': ('מֹוצָא', 'nmpr', 'Moza', 'movtsa'),
        'E78706': ('גָּזֵז', 'nmpr', 'Gazez', 'gazeiz'),
        'E78707': ('יָהְדָּי', 'nmpr', 'Jahdai', 'yahday'),
        'E78708': ('רֶגֶם', 'nmpr', 'Regem', 'regem'),
        'E78709': ('גֵּישָׁן', 'nmpr', 'Geshan', 'geishan'),
        'E78710': ('פֶּלֶט', 'nmpr', 'Pelet', 'pelet'),
        'E78711': ('שָׁעַף', 'nmpr', 'Shaaph', "sha'af"),
        'E78712': ('שֶׁבֶר', 'nmpr', 'Sheber', 'shever'),
        'E78713': ('תִּרְחֲנָה', 'nmpr', 'Tirhanah', 'tirchana'),
        'E78714': ('מַדְמַנָּה', 'nmpr', 'Madmannah', 'madmana'),
        'E78715': ('מַכְבֵּנָה', 'nmpr', 'Macbenah', 'machbeina'),
        'E78716': ('גִּבְעָא', 'nmpr', 'Gibea', 'giva'),
        'E78717': ('חָרֵף', 'nmpr', 'Hareph', 'chareif'),
        'E78718': ('בֵּית גָּדֵר', 'nmpr', 'Beth Gader', 'beit gadeir'),
        'E78719': ('הָרֹאֶה', 'nmpr', 'Haroeh', 'haroeh'),
        'E78720': ('חֲצִי הַמְּנֻחֹות',
                   'nmpr',
                   'half the Manahathites',
                   'chatsi hamnuchovt'),
        'E78721': ('פּוּתִי', 'adjv', 'Puthite', 'puti'),
        'E78722': ('שֻׁמָתִי', 'adjv', 'Shumathite', 'shumati'),
        'E78723': ('מִשְׁרָעִי', 'adjv', 'Mishraite', 'mishrai'),
        'E78724': ('צָרְעָתִי', 'adjv', 'Zorathite', 'tsarati'),
        'E78725': ('אֶשְׁתָּאֻלִי', 'adjv', 'Eshtaolite', 'eshtauli'),
        'E78726': ('עַטְרֹות בֵּית יֹואָב',
                   'nmpr',
                   'Atroth Beth Joab',
                   'atrovt beit yovav'),
        'E78727': ('מָנַחְתִּי', 'adjv', 'Manahathite', 'manachti'),
        'E78728': ('צָרְעִי', 'adjv', 'Zorite', 'tsari'),
        'E78729': ('יַעְבֵּץ', 'nmpr', 'Jabez', 'yabeits'),
        'E78730': ('תִּרְעָתִי', 'adjv', 'Tirathite', 'tirati'),
        'E78731': ('שִׁמְעָתִי', 'adjv', 'Shimeathite', 'shimati'),
        'E78732': ('שׂוּכָתִי', 'adjv', 'Sucathite', 'suchoti'),
        'E78733': ('חַמַּת', 'nmpr', 'Hammath', 'chamat'),
        'E78734': ('בַּת שׁוּעַ', 'nmpr', 'daughter of Shua', 'bat shua'),
        'E78735': ('נֹגַהּ', 'nmpr', 'Nogah', 'nogah'),
        'E78736': ('אַסִּר', 'nmpr', 'Assir', 'asir'),
        'E78737': ('מַלְכִּירָם', 'nmpr', 'Malkiram', 'malkiram'),
        'E78738': ('שֶׁנְאַצַּר', 'nmpr', 'Shenazzar', 'shenatsar'),
        'E78739': ('הֹושָׁמָע', 'nmpr', 'Hoshama', 'hovshama'),
        'E78740': ('נְדַבְיָה', 'nmpr', 'Nedabiah', "n'davya"),
        'E78741': ('חֲשֻׁבָה', 'nmpr', 'Hashubah', 'chashuva'),
        'E78742': ('אֹהֶל', 'nmpr', 'Ohel', 'ohel'),
        'E78743': ('חֲסַדְיָה', 'nmpr', 'Hasadiah', 'chasadya'),
        'E78744': ('יוּשַׁב חֶסֶד', 'nmpr', 'Jushab-Hesed', 'yushav chesed'),
        'E78745': ('אַרְנָן', 'nmpr', 'Arnan', 'arnan'),
        'E78746': ('בָּרִיחַ', 'nmpr', 'Bariah', 'bariach'),
        'E78747': ('נְעַרְיָה', 'nmpr', 'Neariah', "n'arya"),
        'E78748': ('הֹודַוְיָהוּ', 'nmpr', 'Hodaviah', 'hovdavyahu'),
        'E78749': ('פְּלָיָה', 'nmpr', 'Pelaiah', "p'laya"),
        'E78750': ('עֲנָנִי', 'nmpr', 'Anani', 'anani'),
        'E78751': ('יַחַת', 'nmpr', 'Jahath', 'yachat'),
        'E78752': ('אֲחוּמַי', 'nmpr', 'Ahumai', 'achumay'),
        'E78753': ('לָהַד', 'nmpr', 'Lahad', 'lahad'),
        'E78754': ('יִשְׁמָא', 'nmpr', 'Ishma', 'yishma'),
        'E78755': ('יִדְבָּשׁ', 'nmpr', 'Idbash', 'yidbash'),
        'E78756': ('הַצְלֶלְפֹּונִי', 'nmpr', 'Hazzelelponi', 'hatslelpovni'),
        'E78757': ('פְּנוּאֵל', 'nmpr', 'Penuel', "p'nueil"),
        'E78758': ('חוּשָׁה', 'nmpr', 'Hushah', 'chusha'),
        'E78759': ('חֶלְאָה', 'nmpr', 'Helah', 'chela'),
        'E78760': ('נַעֲרָה', 'nmpr', 'Naarah', "na'ara"),
        'E78761': ('אֲחֻזָּם', 'nmpr', 'Ahuzzam', 'achuzam'),
        'E78762': ('תֵּימְנִי', 'adjv', 'Temenite', 'teimni'),
        'E78763': ('אֲחַשְׁתָּרִי', 'adjv', 'Ahashtarite', 'achashtari'),
        'E78764': ('צֶרֶת', 'nmpr', 'Zereth', 'tseret'),
        'E78765': ('אֶתְנָן', 'nmpr', 'Ethnan', 'etnan'),
        'E78766': ('עָנוּב', 'nmpr', 'Anub', 'anuv'),
        'E78767': ('הַצֹּבֵבָה', 'nmpr', 'Hazzobebah', 'hatsoveiva'),
        'E78768': ('אֲחַרְחֵל', 'nmpr', 'Aharhel', 'acharcheil'),
        'E78769': ('הָרוּם', 'nmpr', 'Harum', 'harum'),
        'E78770': ('כְּלוּב', 'nmpr', 'Kelub', "k'luv"),
        'E78771': ('שׁוּחָה', 'nmpr', 'Shuhah', 'shucha'),
        'E78772': ('מְחִיר', 'nmpr', 'Mehir', "m'chir"),
        'E78773': ('אֶשְׁתֹּון', 'nmpr', 'Eshton', 'eshtovn'),
        'E78774': ('רָפָא', 'nmpr', 'Rapha', 'rafa'),
        'E78775': ('תְּחִנָּה', 'nmpr', 'Tehinnah', "t'china"),
        'E78776': ('עִיר נָחָשׁ', 'nmpr', 'Ir Nahash', 'ir nachash'),
        'E78777': ('רֵכָה', 'nmpr', 'Recah', 'reicho'),
        'E78778': ('חֲתַת', 'nmpr', 'Hathath', 'chatat'),
        'E78779': ('מְעֹונֹתַי', 'nmpr', 'Meonothai', "m'ovnotay"),
        'E78780': ('עָפְרָה', 'nmpr', 'Ophrah', 'afra'),
        'E78781': ('עִירוּ', 'nmpr', 'Iru', 'iru'),
        'E78782': ('נַעַם', 'nmpr', 'Naam', "na'am"),
        'E78783': ('יְהַלֶּלְאֵל', 'nmpr', 'Jehallelel', "y'haleleil"),
        'E78784': ('זִיף', 'nmpr', 'Ziph', 'zif'),
        'E78785': ('זִיפָה', 'nmpr', 'Ziphah', 'zifa'),
        'E78786': ('תִּירְיָא', 'nmpr', 'Tiria', 'tira'),
        'E78787': ('אֲשַׂרְאֵל', 'nmpr', 'Asarel', 'asareil'),
        'E78788': ('עֶזְרָה', 'nmpr', 'Ezrah', 'ezra'),
        'E78789': ('מֶרֶד', 'nmpr', 'Mered', 'mered'),
        'E78790': ('יָלֹון', 'nmpr', 'Jalon', 'yalovn'),
        'E78791': ('יִשְׁבָּח', 'nmpr', 'Ishbah', 'yishbach'),
        'E78792': ('גְּדֹור', 'nmpr', 'Gedor', "g'dovr"),
        'E78793': ('יְקוּתִיאֵל', 'nmpr', 'Jekuthiel', "y'kutieil"),
        'E78794': ('זָנֹוחַ', 'nmpr', 'Zanoah', 'zanovach'),
        'E78795': ('בִּתְיָה', 'nmpr', 'Bithiah', 'bita'),
        'E78796': ('נַחַם', 'nmpr', 'Naham', 'nacham'),
        'E78797': ('גַּרְמִי', 'adjv', 'Garmite', 'garmi'),
        'E78798': ('שִׁימֹון', 'nmpr', 'Shimon', 'shimovn'),
        'E78799': ('רִנָּה', 'nmpr', 'Rinnah', 'rina'),
        'E78800': ('בֶּן חָנָן', 'nmpr', 'Ben-Hanan', 'ben chanan'),
        'E78801': ('תִּילֹון', 'nmpr', 'Tilon', 'tilovn'),
        'E78802': ('זֹוחֵת', 'nmpr', 'Zoheth', 'zovcheit'),
        'E78803': ('בֶּן זֹוחֵת', 'nmpr', 'Ben-Zoheth', 'ben zovcheit'),
        'E78804': ('לֵכָה', 'nmpr', 'Lecah', 'leicho'),
        'E78805': ('לַעְדָּה', 'nmpr', 'Laadah', 'lada'),
        'E78806': ('אַשְׁבֵּעַ', 'nmpr', 'Ashbea', 'ashbeia'),
        'E78807': ('יֹוקִים', 'nmpr', 'Jokim', 'yovkim'),
        'E78808': ('כֹּזֵבָא', 'nmpr', 'Cozeba', 'kozeiva'),
        'E78809': ('שָׂרָף', 'nmpr', 'Saraph', 'saraf'),
        'E78810': ('יָשֻׁבִי לָחֶם', 'nmpr', 'Jashubi Lehem', 'yashuvi lachem'),
        'E78811': ('נְטָעִים', 'nmpr', 'Netaim', "n'taim"),
        'E78812': ('חַמּוּאֵל', 'nmpr', 'Hammuel', 'chamueil'),
        'E78813': ('בִּלְהָה', 'nmpr', 'Bilhah', 'bilha'),
        'E78814': ('תֹּולָד', 'nmpr', 'Tolad', 'tovlad'),
        'E78815': ('בֵּית מַרְכָּבֹות', 'nmpr', 'Beth Marcaboth', 'beit markavovt'),
        'E78816': ('חֲצַר סוּסִים', 'nmpr', 'Hazar Susim', 'chatsar susim'),
        'E78817': ('בֵּית בִּרְאִי', 'nmpr', 'Beth Biri', 'beit biri'),
        'E78818': ('תֹּכֶן', 'nmpr', 'Token', 'tochen'),
        'E78819': ('בָּעַל', 'nmpr', 'Baalath', "ba'al"),
        'E78820': ('מְשֹׁובָב', 'nmpr', 'Meshobab', "m'shovvav"),
        'E78821': ('יַמְלֵךְ', 'nmpr', 'Jamlech', 'yamleich'),
        'E78822': ('יֹושָׁה', 'nmpr', 'Joshah', 'yovsha'),
        'E78823': ('יֹושִׁבְיָה', 'nmpr', 'Joshibiah', 'yovshiva'),
        'E78824': ('עֲשִׂיאֵל', 'nmpr', 'Asiel', 'asieil'),
        'E78825': ('יַעֲקֹבָה', 'nmpr', 'Jaakobah', "ya'akova"),
        'E78826': ('יְשֹׁוחָיָה', 'nmpr', 'Jeshohaiah', "y'shovchaya"),
        'E78827': ('עֲדִיאֵל', 'nmpr', 'Adiel', 'adieil'),
        'E78828': ('יְשִׂימִאֵל', 'nmpr', 'Jesimiel', "y'simieil"),
        'E78829': ('זִיזָא', 'nmpr', 'Ziza', 'ziza'),
        'E78830': ('שִׁפְעִי', 'nmpr', 'Shiphi', 'shifi'),
        'E78831': ('אַלֹּון', 'nmpr', 'Allon', 'alovn'),
        'E78832': ('שִׁמְרִי', 'nmpr', 'Shimri', 'shimri'),
        'E78833': ('בַּעַל', 'nmpr', 'Baal', "ba'al"),
        'E78834': ('בְּאֵרָה', 'nmpr', 'Beerah', "b'eira"),
        'E78835': ('תִּלְּגַת פִּלְנְאֶסֶר',
                   'nmpr',
                   'Tiglath-Pileser',
                   'tilgat pilneser'),
        'E78836': ('עָזָז', 'nmpr', 'Azaz', 'azaz'),
        'E78837': ('שָׁפָם', 'nmpr', 'Shapham', 'shafam'),
        'E78838': ('יַעְנַי', 'nmpr', 'Janai', 'yanay'),
        'E78839': ('יֹורַי', 'nmpr', 'Jorai', 'yovray'),
        'E78840': ('יַעְכָּן', 'nmpr', 'Jacan', 'yakan'),
        'E78841': ('זִיעַ', 'nmpr', 'Zia', 'zia'),
        'E78842': ('חוּרִי', 'nmpr', 'Huri', 'churi'),
        'E78843': ('יָרֹוחַ', 'nmpr', 'Jaroah', 'yarovach'),
        'E78844': ('יְשִׁישַׁי', 'nmpr', 'Jeshishai', "y'shishay"),
        'E78845': ('יַחְדֹּו', 'nmpr', 'Jahdo', 'yachdov'),
        'E78846': ('אֲחִי', 'nmpr', 'Ahi', 'achi'),
        'E78847': ('עַבְדִּיאֵל', 'nmpr', 'Abdi-el', 'avdieil'),
        'E78848': ('נֹודָב', 'nmpr', 'Nodab', 'novdav'),
        'E78849': ('אֱלִיאֵל', 'nmpr', 'Eliel', 'elieil'),
        'E78850': ('יַחְדִּיאֵל', 'nmpr', 'Jahdiel', 'yachdieil'),
        'E78851': ('הָרָא', 'nmpr', 'Hara', 'hara'),
        'E78852': ('זִמָּה', 'nmpr', 'Zimmah', 'zima'),
        'E78853': ('יְאָתְרַי', 'nmpr', 'Jeatherai', "y'atray"),
        'E78854': ('אֶבְיָסָף', 'nmpr', 'Ebiasaph', 'evyasaf'),
        'E78855': ('תַּחַת', 'nmpr', 'Tahath', 'tachat'),
        'E78856': ('אוּרִיאֵל', 'nmpr', 'Uriel', 'urieil'),
        'E78857': ('עֲמָשַׂי', 'nmpr', 'Amasai', 'amasay'),
        'E78858': ('אֲחִימֹות', 'nmpr', 'Ahimoth', 'achimovt'),
        'E78859': ('צֹופַי', 'nmpr', 'Zophai', 'tsovfay'),
        'E78860': ('ושׁני֜', 'nmpr', 'Vashni', 'vshny'),
        'E78861': ('חַגִּיָּה', 'nmpr', 'Haggiah', 'chagia'),
        'E78862': ('תֹּוחַ', 'nmpr', 'Toah', 'tovach'),
        'E78863': ('מַחַת', 'nmpr', 'Mahath', 'machat'),
        'E78864': ('בַּעֲשֵׂיָה', 'nmpr', 'Baaseiah', "ba'aseia"),
        'E78865': ('אֶתְנִי', 'nmpr', 'Ethni', 'etni'),
        'E78866': ('קִישִׁי', 'nmpr', 'Kishi', 'kishi'),
        'E78867': ('חִילֵז', 'nmpr', 'Hilez', 'chileiz'),
        'E78868': ('עָלֶמֶת', 'nmpr', 'Alemeth', 'alemet'),
        'E78869': ('עָנֵר', 'nmpr', 'Aner', 'aneir'),
        'E78870': ('בִּלְעָם', 'nmpr', 'Bileam', 'bilam'),
        'E78871': ('עָנֵם', 'nmpr', 'Anem', 'aneim'),
        'E78872': ('מָשָׁל', 'nmpr', 'Mashal', 'mashal'),
        'E78873': ('חוּקֹק', 'nmpr', 'Hukok', 'chukok'),
        'E78874': ('רִמֹּונֹו', 'nmpr', 'Rimmono', 'rimovnov'),
        'E78875': ('יְרִיאֵל', 'nmpr', 'Jeriel', "y'rieil"),
        'E78876': ('יַחְמַי', 'nmpr', 'Jahmai', 'yachmay'),
        'E78877': ('יִבְשָׂם', 'nmpr', 'Ibsam', 'yivsam'),
        'E78878': ('יְדִיעֲאֵל', 'nmpr', 'Jediael', "y'diaeil"),
        'E78879': ('יְרִימֹות', 'nmpr', 'Jerimoth', "y'rimovt"),
        'E78880': ('עִירִי', 'nmpr', 'Iri', 'iri'),
        'E78881': ('זְמִירָה', 'nmpr', 'Zemirah', "z'mira"),
        'E78882': ('יֹועָשׁ', 'nmpr', 'Joash', 'yovash'),
        'E78883': ('עָלֶמֶת', 'nmpr', 'Alemeth', 'alemet'),
        'E78884': ('זֵיתָן', 'nmpr', 'Zethan', 'zeitan'),
        'E78885': ('אֲחִישָׁחַר', 'nmpr', 'Ahishahar', 'achishachar'),
        'E78886': ('שֻׁפִּים', 'nmpr', 'Shuppim', 'shupim'),
        'E78887': ('עִיר', 'nmpr', 'Ir', 'ir'),
        'E78888': ('חֻשִׁם', 'nmpr', 'Hushim', 'chushim'),
        'E78889': ('אַחֵר', 'nmpr', 'Aher', 'acheir'),
        'E78890': ('יַחֲצִיאֵל', 'nmpr', 'Jahziel', 'yachatsieil'),
        'E78891': ('פֶּרֶשׁ', 'nmpr', 'Peresh', 'peresh'),
        'E78892': ('שֶׁרֶשׁ', 'nmpr', 'Sheresh', 'sheresh'),
        'E78893': ('אוּלָם', 'nmpr', 'Ulam', 'ulam'),
        'E78894': ('מֹלֶכֶת', 'nmpr', 'Hammoleketh', 'molechet'),
        'E78895': ('אִישְׁהֹוד', 'nmpr', 'Ishhod', 'ishhovd'),
        'E78896': ('אַחְיָן', 'nmpr', 'Ahian', 'achyan'),
        'E78897': ('לִקְחִי', 'nmpr', 'Likhi', 'likchi'),
        'E78898': ('אֲנִיעָם', 'nmpr', 'Aniam', 'aniam'),
        'E78899': ('בֶּרֶד', 'nmpr', 'Bered', 'bered'),
        'E78900': ('אֶלְעָדָה', 'nmpr', 'Eleadah', 'elada'),
        'E78901': ('אֶלְעָד', 'nmpr', 'Elead', 'elad'),
        'E78902': ('שֶׁאֱרָה', 'nmpr', 'Sheerah', "she'era"),
        'E78903': ('אֻזֵּן שֶׁאֱרָה', 'nmpr', 'Uzzen Sheerah', "uzein she'era"),
        'E78904': ('רֶפַח', 'nmpr', 'Rephah', 'refach'),
        'E78905': ('רֶשֶׁף', 'nmpr', '[god]', 'reshef'),
        'E78906': ('תֶּלַח', 'nmpr', 'Telah', 'telach'),
        'E78907': ('לַעְדָּן', 'nmpr', 'Ladan', 'ladan'),
        'E78908': ('נַעֲרָן', 'nmpr', 'Naaran', "na'aran"),
        'E78909': ('בִּרְזַיִת', 'nmpr', 'Birzaith', 'birzayit'),
        'E78910': ('יַפְלֵט', 'nmpr', 'Japhlet', 'yafleit'),
        'E78911': ('שֹׁומֵר', 'nmpr', 'Shomer', 'shovmeir'),
        'E78912': ('חֹותָם', 'nmpr', 'Hotham', 'chovtam'),
        'E78913': ('שׁוּעָא', 'nmpr', 'Shua', 'shua'),
        'E78914': ('פָּסַךְ', 'nmpr', 'Pasach', 'pasach'),
        'E78915': ('בִּמְהָל', 'nmpr', 'Bimhal', 'bimhal'),
        'E78916': ('עַשְׁוָת', 'nmpr', 'Ashvath', 'ashvat'),
        'E78917': ('רָהְגָּה', 'nmpr', 'Rohgah', 'rahga'),
        'E78918': ('חֻבָּה', 'nmpr', 'Hubbah', 'chuba'),
        'E78919': ('הֵלֶם', 'nmpr', 'Helem', 'heilem'),
        'E78920': ('צֹופַח', 'nmpr', 'Zophah', 'tsovfach'),
        'E78921': ('יִמְנָע', 'nmpr', 'Imna', 'yimna'),
        'E78922': ('שֵׁלֶשׁ', 'nmpr', 'Shelesh', 'sheilesh'),
        'E78923': ('עָמָל', 'nmpr', 'Amal', 'amal'),
        'E78924': ('סוּחַ', 'nmpr', 'Suah', 'suach'),
        'E78925': ('חַרְנֶפֶר', 'nmpr', 'Harnepher', 'charnefer'),
        'E78926': ('שׁוּעָל', 'nmpr', 'Shual', 'shual'),
        'E78927': ('בֵּרִי', 'nmpr', 'Beri', 'beiri'),
        'E78928': ('יִמְרָה', 'nmpr', 'Imrah', 'yimra'),
        'E78929': ('בֶּצֶר', 'nmpr', 'Bezer', 'betser'),
        'E78930': ('הֹוד', 'nmpr', 'Hod', 'hovd'),
        'E78931': ('שִׁלְשָׁה', 'nmpr', 'Shilshah', 'shilsha'),
        'E78932': ('בְּאֵרָא', 'nmpr', 'Beera', "b'eira"),
        'E78933': ('פִּסְפָּה', 'nmpr', 'Pispah', 'pispa'),
        'E78934': ('אֲרָא', 'nmpr', 'Ara', 'ara'),
        'E78935': ('עֻלָּא', 'nmpr', 'Ulla', 'ula'),
        'E78936': ('רִצְיָא', 'nmpr', 'Rizia', 'ritsa'),
        'E78937': ('אַחְרַח', 'nmpr', 'Aharah', 'achrach'),
        'E78938': ('נֹוחָה', 'nmpr', 'Nohah', 'novcha'),
        'E78939': ('אַדָּר', 'nmpr', 'Addar', 'adar'),
        'E78940': ('אֲבִיהוּד', 'nmpr', 'Abihud', 'avihud'),
        'E78941': ('אֲחֹוחַ', 'nmpr', 'Ahoah', 'achovach'),
        'E78942': ('שְׁפוּפָן', 'nmpr', 'Shephuphan', "sh'fufan"),
        'E78943': ('חוּרָם', 'nmpr', 'Huram', 'churam'),
        'E78944': ('אֵחוּד', 'nmpr', 'Ehud', 'eichud'),
        'E78945': ('מָנָחַת', 'nmpr', 'Manahath', 'manachat'),
        'E78946': ('אֲחִיחֻד', 'nmpr', 'Ahihud', 'achichud'),
        'E78947': ('שַׁחֲרַיִם', 'nmpr', 'Shaharaim', 'shacharayim'),
        'E78948': ('חוּשִׁים', 'nmpr', 'Hushim', 'chushim'),
        'E78949': ('בַּעֲרָא', 'nmpr', 'Baara', "ba'ara"),
        'E78950': ('חֹדֶשׁ', 'nmpr', 'Hodesh', 'chodesh'),
        'E78951': ('צִבְיָא', 'nmpr', 'Zibia', 'tsiva'),
        'E78952': ('מֵישָׁא', 'nmpr', 'Mesha', 'meisha'),
        'E78953': ('יְעוּץ', 'nmpr', 'Jeuz', "y'uts"),
        'E78954': ('שָׂכְיָה', 'nmpr', 'Sakia', 'sachya'),
        'E78955': ('מִרְמָה', 'nmpr', 'Mirmah', 'mirma'),
        'E78956': ('אֲבִיטוּב', 'nmpr', 'Abitub', 'avituv'),
        'E78957': ('אֶלְפָּעַל', 'nmpr', 'Elpaal', "elpa'al"),
        'E78958': ('מִשְׁעָם', 'nmpr', 'Misham', 'misham'),
        'E78959': ('שֶׁמֶד', 'nmpr', 'Shemed', 'shemed'),
        'E78960': ('שָׁשָׁק', 'nmpr', 'Shashak', 'shashak'),
        'E78961': ('עֲרָד', 'nmpr', 'Arad', 'arad'),
        'E78962': ('עֶדֶר', 'nmpr', 'Eder', 'eder'),
        'E78963': ('יִשְׁפָּה', 'nmpr', 'Ishpah', 'yishpa'),
        'E78964': ('יֹוחָא', 'nmpr', 'Joha', 'yovcha'),
        'E78965': ('חִזְקִי', 'nmpr', 'Hizki', 'chizki'),
        'E78966': ('יִשְׁמְרַי', 'nmpr', 'Ishmerai', 'yishmray'),
        'E78967': ('יִזְלִיאָה', 'nmpr', 'Izliah', 'yizlia'),
        'E78968': ('יָקִים', 'nmpr', 'Jakim', 'yakim'),
        'E78969': ('אֱלִיעֵנַי', 'nmpr', 'Elienai', 'elieinay'),
        'E78970': ('צִלְּתַי', 'nmpr', 'Zillethai', 'tsiltay'),
        'E78971': ('בְּרָאיָה', 'nmpr', 'Beraiah', "b'raya"),
        'E78972': ('שִׁמְרָת', 'nmpr', 'Shimrath', 'shimrat'),
        'E78973': ('יִשְׁפָּן', 'nmpr', 'Ishpan', 'yishpan'),
        'E78974': ('עַנְתֹתִיָּה', 'nmpr', 'Anthothijah', 'antotia'),
        'E78975': ('יִפְדְיָה', 'nmpr', 'Iphdeiah', 'yifda'),
        'E78976': ('שַׁמְשְׁרַי', 'nmpr', 'Shamsherai', 'shamshray'),
        'E78977': ('שְׁחַרְיָה', 'nmpr', 'Shehariah', "sh'charya"),
        'E78978': ('יַעֲרֶשְׁיָה', 'nmpr', 'Jaareshiah', "ya'areshya"),
        'E78979': ('זֵכֶר', 'nmpr', 'Zeker', 'zeicher'),
        'E78980': ('מִקְלֹות', 'nmpr', 'Mikloth', 'miklovt'),
        'E78981': ('שִׁמְאָה', 'nmpr', 'Shimeah', 'shima'),
        'E78982': ('אֶשְׁבַּעַל', 'nmpr', 'Esh-Baal', "eshba'al"),
        'E78983': ('מְרִיב בַּעַל', 'nmpr', 'Merib-Baal', "m'riv ba'al"),
        'E78984': ('פִּיתֹון', 'nmpr', 'Pithon', 'pitovn'),
        'E78985': ('מֶלֶךְ', 'nmpr', 'Melech', 'melech'),
        'E78986': ('תַּאְרֵעַ', 'nmpr', 'Tarea', 'tareia'),
        'E78987': ('יְהֹועַדָּה', 'nmpr', 'Jehoaddah', "y'hovada"),
        'E78988': ('בִּנְעָא', 'nmpr', 'Binea', 'bina'),
        'E78989': ('רָפָה', 'nmpr', 'Raphah', 'rafa'),
        'E78990': ('אָצֵל', 'nmpr', 'Azel', 'atseil'),
        'E78991': ('בֹּכְרוּ', 'nmpr', 'Bokeru', 'bochru'),
        'E78992': ('שְׁעַרְיָה', 'nmpr', 'Sheariah', "sh'arya"),
        'E78993': ('עֵשֶׁק', 'nmpr', 'Eshek', 'eishek'),
        'E78994': ('יְעוּאֵל', 'nmpr', 'Jeuel', "y'ueil"),
        'E78995': ('יִבְנְיָה', 'nmpr', 'Ibneiah', 'yivna'),
        'E78996': ('מִכְרִי', 'nmpr', 'Micri', 'michri'),
        'E78997': ('יִבְנִיָּה', 'nmpr', 'Ibnijah', 'yivnia'),
        'E78998': ('יְהֹויָרִיב', 'nmpr', 'Jehoiarib', "y'hovyariv"),
        'E78999': ('מַעְשַׂי', 'nmpr', 'Maasai', 'masay'),
        'E79000': ('יַחְזֵרָה', 'nmpr', 'Jahzerah', 'yachzeira'),
        'E79001': ('מְשִׁלֵּמִית', 'nmpr', 'Meshillemith', "m'shileimit"),
        'E79002': ('בַּקְבַּקַּר', 'nmpr', 'Bakbuk', 'bakbakar'),
        'E79003': ('חֶרֶשׁ', 'nmpr', 'Heresh', 'cheresh'),
        'E79004': ('קֹורֵא', 'nmpr', 'Kore', 'kovrei'),
        'E79005': ('מְשֶׁלֶמְיָה', 'nmpr', 'Meshelemiah', "m'shelemya"),
        'E79006': ('חֲבִתִּים', 'subs', 'pans', 'chavitim'),
        'E79007': ('שִׁמְאָם', 'nmpr', 'Shimeam', 'shimam'),
        'E79008': ('תַּחְרֵעַ', 'nmpr', 'Tahrea', 'tachreia'),
        'E79009': ('יַעְרָה', 'nmpr', 'Jadah', 'yara'),
        'E79010': ('גּוּפָה', 'subs', 'corpse', 'gufa'),
        'E79011': ('יָשָׁבְעָם', 'nmpr', 'Jashobeam', 'yashavam'),
        'E79012': ('חַכְמֹונִי', 'nmpr', 'Hacmoni', 'chachmovni'),
        'E79013': ('שַׁמֹּות', 'nmpr', 'Shammoth', 'shamovt'),
        'E79014': ('הֲרֹורִי', 'adjv', 'Harorite', 'harovri'),
        'E79015': ('עִילַי', 'nmpr', 'Ilai', 'ilay'),
        'E79016': ('חֵלֶד', 'nmpr', 'Heled', 'cheiled'),
        'E79017': ('חוּרַי', 'nmpr', 'Hurai', 'churay'),
        'E79018': ('בַּחֻרִמִי', 'adjv', 'Baharumite', 'bachurimi'),
        'E79019': ('הָשֵׁם', 'nmpr', 'Hashem', 'hasheim'),
        'E79020': ('גִּזֹונִי', 'adjv', 'Gizonite', 'gizovni'),
        'E79021': ('שָׁגֵה', 'nmpr', 'Shagee', 'shagei'),
        'E79022': ('שָׂכָר', 'nmpr', 'Sacar', 'sachor'),
        'E79023': ('אֱלִיפַל', 'nmpr', 'Eliphal', 'elifal'),
        'E79024': ('אוּר', 'nmpr', 'Ur', 'ur'),
        'E79025': ('מְכֵרָתִי', 'adjv', 'Mekerathite', "m'cheirati"),
        'E79026': ('חֶצְרֹו', 'nmpr', 'Hezro', 'chetsrov'),
        'E79027': ('נַעֲרַי', 'nmpr', 'Naarai', "na'aray"),
        'E79028': ('אֶזְבָּי', 'nmpr', 'Ezbai', 'ezbay'),
        'E79029': ('מִבְחָר', 'nmpr', 'Mibhar', 'mivchar'),
        'E79030': ('הַגְרִי', 'nmpr', 'Hagri', 'hagri'),
        'E79031': ('עֲדִינָא', 'nmpr', 'Adina', 'adina'),
        'E79032': ('שִׁיזָא', 'nmpr', 'Shiza', 'shiza'),
        'E79033': ('יֹושָׁפָט', 'nmpr', 'Joshaphat', 'yovshafat'),
        'E79034': ('מִתְנִי', 'adjv', 'Mithnite', 'mitni'),
        'E79035': ('עֻזִיָּא', 'nmpr', 'Uzzia', 'uzia'),
        'E79036': ('עֲשְׁתְּרָתִי', 'adjv', 'Ashterathite', 'ashtrati'),
        'E79037': ('שָׁמָע', 'nmpr', 'Shama', 'shama'),
        'E79038': ('עֲרֹעֵרִי', 'adjv', 'Aroerite', 'aroeiri'),
        'E79039': ('תִּיצִי', 'adjv', 'Tizite', 'titsi'),
        'E79040': ('מַחֲוִים', 'subs', '[uncertain]', 'machavim'),
        'E79041': ('יְרִיבַי', 'nmpr', 'Jeribai', "y'rivay"),
        'E79042': ('יֹושַׁוְיָה', 'nmpr', 'Joshaviah', 'yovshavya'),
        'E79043': ('אֶלְנָעַם', 'nmpr', 'Elnaam', "elna'am"),
        'E79044': ('יִתְמָה', 'nmpr', 'Ithmah', 'yitma'),
        'E79045': ('יַעֲשִׂיאֵל', 'nmpr', 'Jaasiel', "ya'asieil"),
        'E79046': ('מְצֹבָיָה', 'nmpr', 'Mezobaite', "m'tsovaya"),
        'E79047': ('שְׁמָעָה', 'nmpr', 'Shemaah', "sh'ma'a"),
        'E79048': ('גִּבְעָתִי', 'adjv', 'Gibeathite', 'givati'),
        'E79049': ('יְזִיאֵל', 'nmpr', 'Jeziel', "y'zieil"),
        'E79050': ('בְּרָכָה', 'nmpr', 'Beracah', "b'racho"),
        'E79051': ('יִשְׁמַעְיָה', 'nmpr', 'Ishmaiah', 'yishmaya'),
        'E79052': ('גְּדֵרָתִי', 'adjv', 'Gederathite', "g'deirati"),
        'E79053': ('אֶלְעוּזַי', 'nmpr', 'Eluzai', 'eluzay'),
        'E79054': ('בְּעַלְיָה', 'nmpr', 'Bealiah', "b'alya"),
        'E79055': ('שְׁמַרְיָהוּ', 'nmpr', 'Shemariah', "sh'maryahu"),
        'E79056': ('שְׁפַטְיָהוּ', 'nmpr', 'Shephatiah', "sh'fatyahu"),
        'E79057': ('חֲרוּפִי', 'adjv', 'Haruphite', 'charufi'),
        'E79058': ('יִשִּׁיָּהוּ', 'nmpr', 'Isshiah', 'yishiahu'),
        'E79059': ('יֹועֶזֶר', 'nmpr', 'Joezer', 'yovezer'),
        'E79060': ('יֹועֵאלָה', 'nmpr', 'Joelah', 'yoveila'),
        'E79061': ('צָבָא', 'subs', 'gazelle', 'tsava'),
        'E79062': ('מִשְׁמַנָּה', 'nmpr', 'Mishmannah', 'mishmana'),
        'E79063': ('אֶלְזָבָד', 'nmpr', 'Elzabad', 'elzavad'),
        'E79064': ('מַכְבַּנַּי', 'nmpr', 'Macbannai', 'machbanay'),
        'E79065': ('עַדְנַח', 'nmpr', 'Adnah', 'adnach'),
        'E79066': ('עדר', 'verb', 'flock together', 'dr'),
        'E79067': ('כִּידֹן', 'nmpr', 'Kidon', 'kidon'),
        'E79068': ('אֶלְפֶּלֶט', 'nmpr', 'Elpelet', 'elpelet'),
        'E79069': ('בְּעֶלְיָדָע', 'nmpr', 'Beeliada', "b'elyada"),
        'E79070': ('קוּשָׁיָהוּ', 'nmpr', 'Kushaiah', 'kushayahu'),
        'E79071': ('בֵּן', 'nmpr', 'Ben', 'bein'),
        'E79072': ('יַעֲזִיאֵל', 'nmpr', 'Jaaziel', "ya'azieil"),
        'E79073': ('שְׁמִירָמֹות', 'nmpr', 'Shemiramoth', "sh'miramovt"),
        'E79074': ('מַתִּתְיָהוּ', 'nmpr', 'Mattithiah', 'matitahu'),
        'E79075': ('אֱלִיפְלֵהוּ', 'nmpr', 'Eliphelehu', 'elifleihu'),
        'E79076': ('מִקְנֵיָהוּ', 'nmpr', 'Mikneiah', 'mikneiahu'),
        'E79077': ('עֲזִיאֵל', 'nmpr', 'Aziel', 'azieil'),
        'E79078': ('עֲזַזְיָהוּ', 'nmpr', 'Azaziah', 'azazyahu'),
        'E79079': ('כְּנַנְיָהוּ', 'nmpr', 'Chenaniah', "k'nanyahu"),
        'E79080': ('שְׁבַנְיָהוּ', 'nmpr', 'Shebaniah', "sh'vanyahu"),
        'E79081': ('חצר', 'verb', 'sound clarion', 'chtsr'),
        'E79082': ('יְחִיָּה', 'nmpr', 'Jehiah', "y'chia"),
        'E79083': ('כרבל', 'verb', 'wrap', 'chrvl'),
        'E79084': ('כְּנַנְיָה', 'nmpr', 'Kenaniah', "k'nanya"),
        'E79085': ('חֹסָה', 'nmpr', 'Hosah', 'chosa'),
        'E79086': ('דַּרְמֶשֶׂק', 'nmpr', 'Damascus', 'darmesek'),
        'E79087': ('טִבְחַת', 'nmpr', 'Tibhath', 'tivchat'),
        'E79088': ('כּוּן', 'nmpr', 'Cun', 'kun'),
        'E79089': ('שַׁוְשָׁא', 'nmpr', 'Shavsha', 'shavsha'),
        'E79090': ('מִפְשָׂעָה', 'subs', 'posterior', "mifsa'a"),
        'E79091': ('אֲרַם מַעֲכָה', 'nmpr', 'Aram Maacah', "aram ma'acho"),
        'E79092': ('שֹׁופַךְ', 'nmpr', 'Shophach', 'shovfach'),
        'E79093': ('סִפַּי', 'nmpr', 'Sippai', 'sipay'),
        'E79094': ('יָעִיר', 'nmpr', 'Jair', 'yair'),
        'E79095': ('לַחְמִי', 'nmpr', 'Lahmi', 'lachmi'),
        'E79096': ('אָרְנָן', 'nmpr', 'Ornan', 'arnan'),
        'E79097': ('נָדָן', 'subs', 'sheath', 'nadan'),
        'E79098': ('מְחַבְּרֹות', 'subs', 'clamps', "m'chabrovt"),
        'E79099': ('שֶׁקֶט', 'subs', 'quiet', 'sheket'),
        'E79100': ('זֵתָם', 'nmpr', 'Zetham', 'zeitam'),
        'E79101': ('חֲזִיאֵל', 'nmpr', 'Haziel', 'chazieil'),
        'E79102': ('זִינָא', 'nmpr', 'Zina', 'zina'),
        'E79103': ('זִיזָה', 'nmpr', 'Ziza', 'ziza'),
        'E79104': ('שְׁבוּאֵל', 'nmpr', 'Shubael', "sh'vueil"),
        'E79105': ('רְחַבְיָה', 'nmpr', 'Rehabiah', "r'chavya"),
        'E79106': ('יְרִיָּהוּ', 'nmpr', 'Jeriah', "y'riahu"),
        'E79107': ('יְקַמְעָם', 'nmpr', 'Jekameam', "y'kamam"),
        'E79108': ('עֵדֶר', 'nmpr', 'Eder', 'eider'),
        'E79109': ('שְׂעֹרִים', 'nmpr', 'Seorim', "s'orim"),
        'E79110': ('שְׁכַנְיָהוּ', 'nmpr', 'Shecaniah', "sh'chanyahu"),
        'E79111': ('חֻפָּה', 'nmpr', 'Huppah', 'chupa'),
        'E79112': ('יֶשֶׁבְאָב', 'nmpr', 'Jeshebeab', 'yeshevav'),
        'E79113': ('הַפִּצֵּץ', 'nmpr', 'Happizzez', 'hapitseits'),
        'E79114': ('גָּמוּל', 'nmpr', 'Gamul', 'gamul'),
        'E79115': ('מַעַזְיָהוּ', 'nmpr', 'Maaziah', "ma'azyahu"),
        'E79116': ('שׁוּבָאֵל', 'nmpr', 'Shubael', 'shuvaeil'),
        'E79117': ('יֶחְדְּיָהוּ', 'nmpr', 'Jehdeiah', 'yechdyahu'),
        'E79118': ('רְחַבְיָהוּ', 'nmpr', 'Rehabiah', "r'chavyahu"),
        'E79119': ('שְׁלֹמֹות', 'nmpr', 'Shelomoth', "sh'lomovt"),
        'E79120': ('אֲמַרְיָהוּ', 'nmpr', 'Amariah', 'amaryahu'),
        'E79121': ('שָׁמִיר', 'nmpr', 'Shamir', 'shamir'),
        'E79122': ('יַעֲזִיָּהוּ', 'nmpr', 'Jaaziah', "ya'aziahu"),
        'E79123': ('שֹׁהַם', 'nmpr', 'Shoham', 'shoham'),
        'E79124': ('עִבְרִי', 'nmpr', 'Ibri', 'ivri'),
        'E79125': ('אֲשַׂרְאֵלָה', 'nmpr', 'Asarelah', 'asareila'),
        'E79126': ('צְרִי', 'nmpr', 'Zeri', "ts'ri"),
        'E79127': ('חֲשַׁבְיָהוּ', 'nmpr', 'Hashabiah', 'chashavyahu'),
        'E79128': ('בֻּקִּיָּהוּ', 'nmpr', 'Bukkiah', 'bukiahu'),
        'E79129': ('מַתַּנְיָהוּ', 'nmpr', 'Mattaniah', 'matanyahu'),
        'E79130': ('אֱלִיאָתָה', 'nmpr', 'Eliathah', 'eliata'),
        'E79131': ('גִּדַּלְתִּי', 'nmpr', 'Giddalti', 'gidalti'),
        'E79132': ('רֹמַמְתִּי עֶזֶר', 'nmpr', 'Romamti-Ezer', 'romamti ezer'),
        'E79133': ('יָשְׁבְּקָשָׁה', 'nmpr', 'Joshbekashah', 'yashbkasha'),
        'E79134': ('מַלֹּותִי', 'nmpr', 'Mallothi', 'malovti'),
        'E79135': ('הֹותִיר', 'nmpr', 'Hothir', 'hovtir'),
        'E79136': ('מַחֲזִיאֹות', 'nmpr', 'Mahazioth', 'machaziovt'),
        'E79137': ('תַּלְמִיד', 'subs', 'scholar', 'talmid'),
        'E79138': ('יְשַׂרְאֵלָה', 'nmpr', 'Jesarelah', "y'sareila"),
        'E79139': ('אֱלִיָּתָה', 'nmpr', 'Eliathah', 'eliata'),
        'E79140': ('מְשֶׁלֶמְיָהוּ', 'nmpr', 'Meshelemiah', "m'shelemyahu"),
        'E79141': ('זְבַדְיָהוּ', 'nmpr', 'Zebadiah', "z'vadyahu"),
        'E79142': ('יַתְנִיאֵל', 'nmpr', 'Jathniel', 'yatnieil'),
        'E79143': ('פְּעֻלְּתַי', 'nmpr', 'Peullethai', "p'ultay"),
        'E79144': ('עָתְנִי', 'nmpr', 'Othni', 'atni'),
        'E79145': ('רְפָאֵל', 'nmpr', 'Rephael', "r'faeil"),
        'E79146': ('אֱלִיהוּ', 'nmpr', 'Elihu', 'elihu'),
        'E79147': ('סְמַכְיָהוּ', 'nmpr', 'Semakiah', "s'machyahu"),
        'E79148': ('טְבַלְיָהוּ', 'nmpr', 'Tabaliah', "t'valyahu"),
        'E79149': ('שַׁלֶּכֶת', 'nmpr', 'Shalleketh', 'shalechet'),
        'E79150': ('יְחִיאֵלִי', 'adjv', 'Jehieli', "y'chieili"),
        'E79151': ('יְרִיָּה', 'nmpr', 'Jeriah', "y'ria"),
        'E79152': ('דֹּודַי', 'nmpr', 'Dodai', 'dovday'),
        'E79153': ('עַמִּיזָבָד', 'nmpr', 'Ammizabad', 'amizavad'),
        'E79154': ('שַׁמְהוּת', 'nmpr', 'Shamhuth', 'shamhut'),
        'E79155': ('יִזְרָח', 'subs', '[uncertain]', 'yizrach'),
        'E79156': ('יִשְׁמַעְיָהוּ', 'nmpr', 'Ishmaiah', 'yishmayahu'),
        'E79157': ('פְּדָיָהוּ', 'nmpr', 'Pedaiah', "p'dayahu"),
        'E79158': ('יִדֹּו', 'nmpr', 'Iddo', 'yidov'),
        'E79159': ('עֶזְרִי', 'nmpr', 'Ezri', 'ezri'),
        'E79160': ('רָמָתִי', 'adjv', 'Ramathite', 'ramati'),
        'E79161': ('שִׁפְמִי', 'adjv', 'Shiphmite', 'shifmi'),
        'E79162': ('גְּדֵרִי', 'adjv', 'Gederite', "g'deiri"),
        'E79163': ('שִׁרְטַי', 'nmpr', 'Shitrai', 'shirtay'),
        'E79164': ('שָׁרֹונִי', 'adjv', 'Sharonite', 'sharovni'),
        'E79165': ('עַדְלָי', 'nmpr', 'Adlai', 'adlay'),
        'E79166': ('אֹובִיל', 'nmpr', 'Obil', 'ovvil'),
        'E79167': ('יָזִיז', 'nmpr', 'Jaziz', 'yaziz'),
        'E79168': ('גַּנְזַךְ', 'subs', 'treasury', 'ganzach'),
        'E79169': ('שַׁיִשׁ', 'subs', 'alabaster', 'shayish'),
        'E79170': ('קְוֵא', 'nmpr', 'Kue', "k'vei"),
        'E79171': ('אַרְגְּוָן', 'subs', '[uncertain]', 'argvan'),
        'E79172': ('כַּרְמִיל', 'subs', 'carmesine', 'karmil'),
        'E79173': ('צֹרֶךְ', 'subs', 'need', 'tsorech'),
        'E79174': ('רַפְסֹדֹות', 'subs', 'rafts', 'rafsodovt'),
        'E79175': ('סְפָר', 'subs', 'census', "s'far"),
        'E79176': ('פַּרְוָיִם', 'nmpr', 'Parvaim', 'parvayim'),
        'E79177': ('צַעֲצֻעִים', 'subs', 'cast images', "tsa'atsuim"),
        'E79178': ('צֶפֶת', 'subs', 'capital of pillar', 'tsefet'),
        'E79179': ('מִכְלֹות', 'subs', 'perfection', 'michlovt'),
        'E79180': ('נֹוחַ', 'subs', 'rest', 'novach'),
        'E79181': ('חֲמָת צֹובָא', 'nmpr', 'Hamath Zobah', 'chamat tsovva'),
        'E79182': ('כֶּבֶשׁ', 'subs', 'footstool', 'kevesh'),
        'E79183': ('חֲזֹות', 'subs', 'vision', 'chazovt'),
        'E79184': ('יֶעְדֹּו', 'nmpr', 'Iddo', 'yedov'),
        'E79185': ('נְסִבָּה', 'subs', 'dispensation', "n'siba"),
        'E79186': ('אֲדֹורַיִם', 'nmpr', 'Adoraim', 'adovrayim'),
        'E79187': ('זָהַם', 'nmpr', 'Zaham', 'zaham'),
        'E79188': ('סֻכִּי', 'adjv', 'Sukkite', 'suki'),
        'E79189': ('מִיכָיָהוּ', 'nmpr', 'Micaiah', 'michoyahu'),
        'E79190': ('יְשָׁנָה', 'nmpr', 'Jeshanah', "y'shana"),
        'E79191': ('עֶפְרַיִן', 'nmpr', 'Ephron', 'efrayin'),
        'E79192': ('אֲבִיָּהוּ', 'nmpr', 'Abijah', 'abiahu'),
        'E79193': ('מִדְרָשׁ', 'subs', 'exposition', 'midrash'),
        'E79194': ('צְפַתָה', 'nmpr', 'Zephathah', "ts'fata"),
        'E79195': ('עֹדֵד', 'nmpr', 'Oded', 'odeid'),
        'E79196': ('אָבֵל מָיִם', 'nmpr', 'Abel Maim', 'aveil mayim'),
        'E79197': ('חלא', 'verb', 'fall ill', 'chl'),
        'E79198': ('בֶּן חַיִל', 'nmpr', 'Ben-Hail', 'ben chayil'),
        'E79199': ('טֹובִיָּהוּ', 'nmpr', 'Tobijah', 'tovbiahu'),
        'E79200': ('טֹוב אֲדֹנִיָּה', 'nmpr', 'Tob-Adonijah', 'tovv adonia'),
        'E79201': ('בִּירָנִיָּה', 'subs', 'fortified place', 'birania'),
        'E79202': ('עַדְנָה', 'nmpr', 'Adnah', 'adna'),
        'E79203': ('עֲמַסְיָה', 'nmpr', 'Amasiah', 'amasya'),
        'E79204': ('מַשֹּׂא', 'subs', 'partiality', 'maso'),
        'E79205': ('מִקָּח', 'subs', 'taking', 'mikach'),
        'E79206': ('צִיץ', 'nmpr', 'Ziz', 'tsits'),
        'E79207': ('יְרוּאֵל', 'nmpr', 'Jeruel', "y'rueil"),
        'E79208': ('דֹּדָוָהוּ', 'nmpr', 'Dodavahu', 'dodavahu'),
        'E79209': ('תְּבוּסָה', 'subs', 'downtreading', "t'vusa"),
        'E79210': ('יְהֹושַׁבְעַת', 'nmpr', 'Jehosheba', "y'hovshavat"),
        'E79211': ('עֲדָיָהוּ', 'nmpr', 'Adaiah', 'adayahu'),
        'E79212': ('אֱלִישָׁפָט', 'nmpr', 'Elishaphat', 'elishafat'),
        'E79213': ('מִרְשַׁעַת', 'subs', 'wickedness', "mirsha'at"),
        'E79214': ('מַחֲלוּיִם', 'subs', 'diseases', 'machaluyim'),
        'E79215': ('שִׁמְרִית', 'nmpr', 'Shimrith', 'shimrit'),
        'E79216': ('יַבְנֵה', 'nmpr', 'Jabneh', 'yavnei'),
        'E79217': ('גּוּר בָּעַל', 'nmpr', 'Gur Baal', "gur ba'al"),
        'E79218': ('חַדְלָי', 'nmpr', 'Hadlai', 'chadlay'),
        'E79219': ('מַעֲרֻמִּים', 'subs', 'naked persons', "ma'arumim"),
        'E79220': ('גִּמְזֹו', 'nmpr', 'Gimzo', 'gimzov'),
        'E79221': ('עֵדֶן', 'nmpr', 'Eden', 'eiden'),
        'E79222': ('מְקַטְּרֹות', 'subs', 'incense altars', "m'katrovt"),
        'E79223': ('שְׁחִיטָה', 'subs', 'slaying', "sh'chita"),
        'E79224': ('כָּנַנְיָהוּ', 'nmpr', 'Conaniah', 'kananyahu'),
        'E79225': ('יִסְמַכְיָהוּ', 'nmpr', 'Ismakiah', 'yismachyahu'),
        'E79226': ('מִיצִיאִים', 'subs', '[uncertain]', "mitsi'im"),
        'E79227': ('אֲוֵרֹות', 'subs', '[uncertain]', 'aveirovt'),
        'E79228': ('עֵילֹום', 'subs', '[uncertain]', 'eilovm'),
        'E79229': ('בדק', 'verb', 'repair', 'vdk'),
        'E79230': ('תָּקְהַת', 'nmpr', 'Tokhath', 'takhat'),
        'E79231': ('חַסְרָה', 'nmpr', 'Hasrah', 'chasra'),
        'E79232': ('פְּלֻגָּה', 'subs', 'division', "p'luga"),
        'E79233': ('חֲלֻקָּה', 'subs', 'portion', 'chaluka'),
        'E79234': ('מִפְלַגָּה', 'subs', 'division', 'miflaga'),
        'E79235': ('לעב', 'verb', 'delight', 'lv'),
        'E79236': ('יָשֵׁשׁ', 'adjv', 'aged', 'yasheish'),
        "H1": ("אָב", "H:N-M", "father", "av"),
        "H2": ("אַב", "A:N-M", "father", "av"),
        "H3": ("אֵב", "H:N-M", "greenery", "ev"),
        "H4": ("אֵב", "A:N-M", "fruit", "ev"),
        "H5": ("אֲבַגְתָא", "N:N-M-P", "Abagtha", "a.vag.ta"),
        "H6": ("אָבַד", "H:V", "to perish", "a.vad"),
        "H7": ("אֲבַד", "A:V", "to destroy", "a.vad"),
        "H8": ("אֹבֵד", "H:N-M", "destruction", "o.ved"),
        "H9": ("אֲבֵדָה", "H:N-F", "something lost", "a.ve.dah"),
        "H10": ("אֲבַדּוֹן", "H:N-F", "destruction", "a.vad.don"),
        "H11": ("אֲבַדּוֹן", "N:N--L", "Abaddon", "a.vad.don"),
        "H12": ("אַבְדָ֑ן", "H:N-M", "destruction", "av.dan"),
        "H13": ("אׇבְדַן", "H:N-M", "destruction", "ov.dan"),
        "H14": ("אָבָה", "H:V", "be willing", "a.vah"),
        "H15": ("אָבִי ", "H:N-F", "oh that!", "a.veh"),
        "H16": ("אֵ֫בֶה", "H:N-M", "papyrus", "e.veh"),
        "H17": ("אֲבוֹי", "H:Intj", "pain!", "a.voy"),
        "H18": ("אֵבוּס", "H:N-M", "crib", "e.vus"),
        "H19": ("אִבְחָה", "H:N-F", "slaughter", "iv.chah"),
        "H20": ("אֲבַטִּחִים", "H:N-M", "melon", "a.vat.ti.ach"),
        "H21": ("אֲבִי", "N:N-F-P", "Abi", "a.vi"),
        "H22": ("אֲבִיאֵל", "N:N-M-P", "Abiel", "a.vi.el"),
        "H23": ("אֲבִיאָסָף", "N:N-M-P", "Abiasaph", "a.vi.a.saph"),
        "H24": ("אָבִיב", "H:N-M / N:N--T", "Abib", "a.viv"),
        "H25": ("אֲבִי גִבְעוֹן", "N:N-M-P / N:N--L", "Abi-gibeon", "a.vi giv.on"),
        "H26": ("אֲבִיגַ֫יִל, אֲבִיגַ֫ל", "N:N-F-P", "Abigail", "a.vi.ga.yil"),
        "H27": ("אֲבִידָן", "N:N-M-P", "Abidan", "a.vi.dan"),
        "H28": ("אֲבִידָע", "N:N-M-P", "Abida", "a.vi.da"),
        "H29": ("אֲבִיָּ֫הוּ, אֲבִיָּה", "N:N-M-P", "Abijah", "a.viy.yah"),
        "H30": ("אֲבִיהוּא", "N:N-M-P", "Abihu", "a.vi.hu"),
        "H31": ("אֲבִיהוּד", "N:N-M-P", "Abihud", "a.vi.hud"),
        "H32": ("אֲבִיחַ֫יִל", "N:N-M-P", "Abihail", "a.vi.ha.yil"),
        "H33": ("אֲבִי הָעֶזְרִי", "N:N--PG", "Abi-ezrite", "a.vi ha.ez.ri"),
        "H34": ("אֶבְיוֹן", "H:A-M", "needy", "ev.yon"),
        "H35": ("אֲבִיּוֹנָה", "H:N-F", "desire", "a.viy.yo.nah"),
        "H36": ("אֲבִיטוּב", "N:N-M-P", "Abitub", "a.vi.tuv"),
        "H37": ("אֲבִיטָ֑ל", "N:N-F-P", "Abital", "a.vi.tal"),
        "H38": ("אֲבִיָּם", "N:N-M-P", "Abijam", "a.viy.yam"),
        "H39": ("אֲבִימָאֵל", "N:N-M-P", "Abimael", "a.vi.ma.el"),
        "H40": ("אֲבִימֶ֫לֶךְ", "N:N-M-P", "Abimelech", "a.vi.me.lekh"),
        "H41": ("אֲבִינָדָב", "N:N-M-P", "Abinadab", "a.vi.na.dav"),
        "H42": ("אֲבִינֹ֫עַם", "N:N-M-P", "Abinoam", "a.vi.no.am"),
        "H43": ("אֶבְיָסָף", "N:N-M-P", "Ebiasaph", "ev.ya.saph"),
        "H44": ("אֲבִיעֶ֫זֶר", "N:N-M-P", "Abiezer", "a.vi.e.zer"),
        "H45": ("אֲבִי־עַלְבוֹן", "N:N-M-P", "Abi-albon", "a.vi-al.von"),
        "H46": ("אָבִיר", "H:A-M", "mighty", "a.vir"),
        "H47": ("אַבִּיר", "H:A-M", "mighty", "ab.bir"),
        "H48": ("אֲבִירָם", "N:N-M-P", "Abiram", "a.vi.ram"),
        "H49": ("אֲבִישַׁג", "N:N-F-P", "Abishag", "a.vi.shag"),
        "H50": ("אֲבִישׁוּעַ", "N:N-M-P", "Abishua", "a.vi.shu.a"),
        "H51": ("אֲבִישׁוּר", "N:N-M-P", "Abishur", "a.vi.shur"),
        "H52": ("אֲבִישַׁי", "N:N-M-P", "Abishai", "a.vi.shay"),
        "H53": ("אֲבִישָׁלוֹם", "N:N-M-P", "Absalom", "a.vi.sha.lom"),
        "H54": ("אֶבְיָתָר", "N:N-M-P", "Abiathar", "ev.ya.tar"),
        "H55": ("אָבַךְ", "H:V", "to roll up", "a.vakh"),
        "H56": ("אָבַל", "H:V", "to mourn", "a.val"),
        "H57": ("אָבֵל", "H:A", "mourning", "a.vel"),
        "H58": ("אָבֵל", "H:N-F", "meadow", "a.vel"),
        "H59": ("אָבֵל", "N:N--L", "Abel", "a.vel"),
        "H60": ("אֵ֫בֶל", "H:N-M", "mourning", "e.vel"),
        "H61": ("אֲבָל", "H:Adv", "truly", "a.val"),
        "H62": ("אָבֵל בֵּית־מֲעַכָה", "N:N--L", "Abel-beth-maachah", "a.vel bet-ma.a.khah"),
        "H63": ("אָבֵל הַשִּׁטִּים", "N:N--L", "Abel-shittim", "a.vel hash.shit.tim"),
        "H64": ("אָבֵל כְּרָמִים", "N:N--L", "Abel-keramim", "a.vel ke.ra.mim"),
        "H65": ("אָבֵל מְחוֹלָה", "N:N--L", "Abel-meholah", "a.vel me.cho.lah"),
        "H66": ("אַבֵל מַיִם", "N:N--L", "Abel-maim", "a.vel ma.yim"),
        "H67": ("אָבֵל מִצְרַיִם", "N:N--L", "Abel-mizraim", "a.vel mits.ra.yim"),
        "H68": ("אֶ֫בֶן", "H:N-F", "stone", "e.ven"),
        "H69": ("אֶ֫בֶן", "A:N-F", "stone", "e.ven"),
        "H70": ("אֹבֶן", "H:N-M", "wheel", "o.ven"),
        "H71": ("אֲבָנָה", "N:N--L", "Abana", "a.va.nah"),
        "H72": ("אֶבֶן הָעֵזֶר", "N:N--L", "Eben-ezer", "e.ven ha.e.zer"),
        "H73": ("אַבְנֵט", "H:N-M", "girdle", "av.net"),
        "H74": ("אֲבִינֵר", "N:N-M-P", "Abner", "av.ner"),
        "H75": ("אָבַס", "H:V", "to fatten", "a.vas"),
        "H76": ("אֲבַעְבֻּעֹת", "H:N-F", "boil", "a.va.bu.ah"),
        "H77": ("אֶ֫בֶץ", "N:N--L", "Abez", "e.vets"),
        "H78": ("אִבְצָן", "N:N-M-P", "Ibzan", "iv.tsan"),
        "H79": ("אָבַק", "H:V", "to wrestle", "a.vaq"),
        "H80": ("אָבָק", "H:N-M", "dust", "a.vaq"),
        "H81": ("אֲבָקָה", "H:N-F", "aromatic powder", "a.va.qah"),
        "H82": ("אָבַר", "H:V", "to fly", "a.var"),
        "H83": ("אֵ֫בֶר", "H:N-M", "wing", "e.ver"),
        "H84": ("אֶבְרָה", "H:N-F", "pinion", "ev.rah"),
        "H85": ("אַבְרָהָם", "N:N-M-P", "Abraham", "av.ra.ham"),
        "H86": ("אַבְרֵךְ", "H:V", "to bow", "av.rekh"),
        "H87": ("אַבְרָם", "N:N-M-P", "Abram", "av.ram"),
        "H88": ("אֹבֹת", "N:N--L", "Oboth", "o.vot"),
        "H89": ("אָגֵא", "N:N-M-P", "Agee", "a.ge"),
        "H90": ("אֲגָג", "N:N-M-P", "Agag", "a.gag"),
        "H91": ("אֲגָגִי", "N:N--PG", "Agagite", "a.ga.gi"),
        "H92": ("אֲגֻדָּה", "H:N-F", "band", "a.gud.dah"),
        "H93": ("אֱגוֹז", "H:N-M", "nut", "e.goz"),
        "H94": ("אָגוּר", "N:N-M-P", "Agur", "a.gur"),
        "H95": ("אֲגוֹרָה", "H:N-F", "piece", "a.go.rah"),
        "H96": ("אֵ֫גֶל", "H:N-M", "drop", "e.gel"),
        "H97": ("אֶגְלַ֫יִם", "N:N--L", "Eglaim", "eg.la.yim"),
        "H98": ("אֲגַם", "H:N-M", "pool", "a.gam"),
        "H99": ("אָגֵם", "H:A", "grieved", "a.gem"),
        "H100": ("אַגְמֹן", "H:N-M", "bulrush", "ag.mon"),
        "H101": ("אַגָּן", "H:N-M", "vessel", "ag.gan"),
        "H102": ("אֲגַף", "H:N-M", "band", "ag.gaph"),
        "H103": ("אָגַר", "H:V", "to gather", "a.gar"),
        "H104": ("אִגְּרָא", "A:N-F", "letter", "ig.g.ra"),
        "H105": ("אֲגַרְטָל", "H:N-M", "basin", "a.gar.tal"),
        "H106": ("אֶגְרֹף", "H:N-M", "fist", "eg.roph"),
        "H107": ("אִגֶּ֫רֶת", "H:N-F", "letter", "ig.ge.ret"),
        "H108": ("אֵד", "H:N-M", "mist", "ed"),
        "H109": ("אָדַב", "H:V", "to grieve", "a.dav"),
        "H110": ("אַדְבְּאֵל", "N:N-M-P", "Adbeel", "ad.be.el"),
        "H111": ("אֲדַד", "N:N-M-P", "Hadad", "a.dad"),
        "H112": ("אִדּוֹ", "N:N-M-P", "Iddo", "id.do"),
        "H113": ("אָדוֹן", "H:N-M", "lord", "a.don"),
        "H114": ("אַדּוֹן", "N:N--L", "Addon", "ad.don"),
        "H115": ("אֲדוֹרַ֫יִם", "N:N--L", "Adoraim", "a.do.ra.yim"),
        "H116": ("אֱדַ֫יִן", "A:Adv", "then", "e.da.yin"),
        "H117": ("אַדִּיר", "H:A", "great", "ad.dir"),
        "H118": ("אֲדַלְיָא", "N:N-M-P", "Adalia", "a.dal.ya"),
        "H119": ("אָדֹם", "H:V", "to redden", "a.dam"),
        "H120": ("אָדָם", "H:N-M", "man", "a.dam"),
        "H121": ("אָדָם", "N:N-M-P / N:N--L", "Adam", "a.dam"),
        "H122a": ("אָדֹם", "H:A", "red", "a.dom"),
        "H122b": ("אֱדֹם", "H:N-M", "red stuff", ""),
        "H123": ("אֱדוֹם", "N:N-M-P / N:N--L", "Edom", "e.dom"),
        "H124": ("אֹ֫דֶם", "H:N-F", "sardius", "o.dem"),
        "H125": ("אֲדַמְדָּם", "H:A", "reddish", "a.dam.dam"),
        "H126": ("אַדְמָה", "N:N--L", "Admah", "ad.mah"),
        "H127": ("אֲדָמָה", "H:N-F", "land", "a.da.mah"),
        "H128": ("אֲדָמָה", "N:N--L", "Adamah", "a.da.mah"),
        "H129": ("אֲדָמִי", "N:N--L", "Adami", "a.da.mi"),
        "H130": ("אֲדֹמִי", "N:N--PG", "Edomite", "e.do.mi"),
        "H131": ("אֲדֻמִּים", "N:N--L", "Adummim", "a.dum.mim"),
        "H132": ("אַדְמוֹנִי", "H:A", "red", "ad.mo.ni"),
        "H133": ("אַדְמָתָא", "N:N-M-P", "Admatha", "ad.ma.ta"),
        "H134": ("אֶ֫דֶן", "H:N-M", "socket", "e.den"),
        "H135": ("אַדָּן", "N:N--L", "Addan", "ad.dan"),
        "H136": ("אֲדֹנָי", "H:N-M", "Lord", "a.do.na"),
        "H137": ("אֲדֹנִי־בֶ֫זֶק", "N:N-M-P", "Adoni-bezek", "a.do.ni-ve.zeq"),
        "H138": ("אֲדֹנִיָּ֫הוּ", "N:N-M-P", "Adonijah", "a.do.niy.yah"),
        "H139": ("אֲדֹנִי־צֶ֫דֶק", "N:N-M-P", "Adoni-zedek", "a.do.ni-tse.deq"),
        "H140": ("אֲדֹנִיקָם", "N:N-M-P", "Adonikam", "a.do.ni.qam"),
        "H141": ("אֲדֹנִירָם", "N:N-M-P", "Adoniram", "a.do.ni.ram"),
        "H142": ("אָדַר", "H:V", "be glorious", "a.dar"),
        "H143": ("אֲדָר", "N:N--T", "Adar", "a.dar"),
        "H144": ("אֲדָר", "A:N--T", "Adar", "a.dar"),
        "H145": ("אֶ֫דֶר", "H:N", "robe", "e.der"),
        "H146": ("אַדָּר", "N:N-M-P / N:N--L", "Addar", "ad.dar"),
        "H147": ("אִדַּר", "A:N-F", "threshing floor", "id.dar"),
        "H148": ("אֲדַרְגָּזֵר", "A:N-M", "judge", "a.dar.ga.zer"),
        "H149": ("אַדְרַזְדָּא", "A:Adv", "diligently", "ad.raz.da"),
        "H150": ("אֲדַרְכֹּן", "H:N-M", "dram", "a.dar.kon"),
        "H151": ("אֲדֹרָם", "N:N-M-P", "Adoram", "a.do.ram"),
        "H152": ("אַדְרַמֶּ֫לֶךְ", "N:N-M-P / N:N-M-T", "Adrammelech", "a.de.ram.me.lekh"),
        "H153": ("אֶדְרַע", "A:N-F", "force", "ed.ra"),
        "H154": ("אֶדְרֶ֫עִי", "N:N--L", "Edrei", "ed.re.i"),
        "H155": ("אַדֶּ֫רֶת", "H:N-F", "clothing", "ad.de.ret"),
        "H156": ("אָדַשׁ", "H:V", "to thresh", "a.dash"),
        "H157": ("אָהֵב", "H:V", "to love", "a.hav"),
        "H158": ("אַ֫הַב", "H:N-M", "lover", "a.hav"),
        "H159": ("אֹ֫הַב", "H:N-M", "beloved", "o.hav"),
        "H160": ("אַהֲבָה", "H:N-F", "love", "a.ha.vah"),
        "H161": ("אֹ֫הַד", "N:N-M-P", "Ohad", "o.had"),
        "H162": ("אֲהָהּ", "H:Intj", "alas!", "a.hah"),
        "H163": ("אַהֲוָא", "N:N--L", "Ahava", "a.ha.va"),
        "H164": ("אֵהוּד", "N:N-M-P", "Ehud", "e.hud"),
        "H165": ("אֱהִי", "H:Adv", "where?", "e.hi"),
        "H166": ("אָהַל", "H:V", "to shine", "a.hal"),
        "H167": ("אָהַל", "H:V", "to pitch", "a.hal"),
        "H168": ("אֹ֫הֶל", "H:N-M", "tent", "o.hel"),
        "H169": ("אֹ֫הֶל", "N:N-M-P", "Ohel", "o.hel"),
        "H170": ("אׇהֳלָה", "N:N-F-P", "Aholah", "o.ho.lah"),
        "H171": ("אׇהֳלִיאָב", "N:N-M-P", "Aholiab", "o.ho.li.av"),
        "H172": ("אׇהֳלִיבָה", "N:N-F-P", "Aholibah", "o.ho.li.vah"),
        "H173": ("אׇהֳלִיבָמָה", "N:N-F-P", "Aholibamah", "o.ho.li.va.mah"),
        "H174": ("אָהָל", "H:N-F", "aloe", "a.ha.lim"),
        "H175": ("אַהֲרֹן", "N:N-M-P", "Aaron", "a.ha.ron"),
        "H176a": ("אוֹ", "H:Conj", "or", "o"),
        "H176b": ("אוֹ", "H:N-M", "desire", "o"),
        "H177": ("אוּאֵל", "N:N-M-P", "Uel", "u.el"),
        "H178": ("אוֹב", "H:N-M", "medium", "ov"),
        "H179": ("אוֹבִיל", "N:N-M-P", "Obil", "o.vil"),
        "H180": ("אוּבַל", "H:N-M", "river", "u.val"),
        "H181": ("אוּד", "H:N-M", "firebrand", "ud"),
        "H182": ("אוֹדָה", "H:N-F", "because", "o.dot"),
        "H183": ("אָוָה", "H:V", "to desire", "a.vah"),
        "H184": ("אָוָה", "H:V", "to mark", "a.vah"),
        "H185": ("אַוָּה", "H:N-F", "desire", "av.vah"),
        "H186": ("אוּזַי", "N:N-M-P", "Uzai", "u.zay"),
        "H187": ("אוּזָל", "N:N-M-P", "Uzal", "u.zal"),
        "H188": ("אוֹי", "H:Intj", "woe!", "oy"),
        "H189": ("אֱוִי", "N:N-M-P", "Evi", "e.vi"),
        "H190": ("אוֹיָה", "H:Intj", "woe!", "o.yah"),
        "H191": ("אֱוִיל", "H:A-M", "fool(ish)", "e.vil"),
        "H192": ("אֱוִיל מְרֹדַךְ", "N:N-M-P", "Evil-merodach", "e.vil me.ro.dakh"),
        "H193a": ("אוּל", "H:N-M", "strength", "ul"),
        "H193b": ("אוּל", "H:N-M", "mighty", "ul"),
        "H194": ("אוּלַי", "H:Neg", "perhaps", "u.lay"),
        "H195": ("אוּלַי", "N:N--L", "Ulai", "u.lay"),
        "H196": ("אֱוִלִי", "H:A", "foolish", "e.vi.li"),
        "H197": ("אוּלָם", "H:N-M", "porch", "u.lam"),
        "H198": ("אוּלָם", "N:N-M-P", "Ulam", "u.lam"),
        "H199": ("אוּלָם", "H:Adv", "but", "u.lam"),
        "H200": ("אִוֶּ֫לֶת", "H:N-F", "folly", "iv.ve.let"),
        "H201": ("אוֹמָר", "N:N-M-P", "Omar", "o.mar"),
        "H202": ("אוֹן", "H:N-M", "strength", "on"),
        "H203": ("אוֹן", "N:N-M-P", "On", "on"),
        "H204": ("אֹן", "N:N--L", "On", "on"),
        "H205": ("אָ֫וֶן", "H:N-M", "evil", "a.ven"),
        "H206": ("אָ֫וֶן", "N:N--L", "Aven", "a.ven"),
        "H207": ("אוֹנוֹ", "N:N--L", "Ono", "o.no"),
        "H208": ("אוֹנָם", "N:N-M-P", "Onam", "o.nam"),
        "H209": ("אוֹנָן", "N:N-M-P", "Onan", "o.nan"),
        "H210": ("אוּפָז", "N:N--L", "Uphaz", "u.phaz"),
        "H211": ("אוֹפִיר", "N:N--L / N:N-M-P", "Ophir", "o.phir"),
        "H212": ("אוֹפָן", "H:N-M", "wheel", "o.phan"),
        "H213": ("אוּץ", "H:V", "to hasten", "uts"),
        "H214": ("אוֹצָר", "H:N-M", "treasure", "o.tsar"),
        "H215": ("אוֹר", "H:V", "to light", "or"),
        "H216": ("אוֹר", "H:N-F", "light", "or"),
        "H217a": ("אוּר", "H:N-M", "flame", "ur"),
        "H217b": ("אֻרִים", "H:N-M", "flame", ""),
        "H218a": ("אוּר", "N:N-M-P", "Ur", "ur"),
        "H218b": ("אוּר", "N:N--L", "Ur", "ur"),
        "H219a": ("אוֹרָה", "H:N-F", "light", "o.rah"),
        "H219b": ("אוֹרָה", "H:N-F", "herb", "o.rah"),
        "H220": ("אֲוֵרָה", "H:N-F", "stall", "a.ve.rah"),
        "H221": ("אוּרִי", "N:N-M-P", "Uri", "u.ri"),
        "H222": ("אוּרִיאֵל", "N:N-M-P", "Uriel", "u.ri.el"),
        "H223a": ("אוּרִיָּה", "N:N-M-P", "Uriah", "u.riy.yah"),
        "H223b": ("אוּרִיָּ֫הוּ", "N:N-M-P", "Urijah", ""),
        "H224": ("אוּרִים", "N:N-M-T", "Urim", "u.rim"),
        "H225": ("אוּת", "H:V", "to consent", "ut"),
        "H226": ("אוֹת", "H:N-F", "sign", "ot"),
        "H227a": ("אָז", "H:Adv", "then", "az"),
        "H227b": ("מֵאָז", "H:Adv", "the past", ""),
        "H228": ("אֲזָא", "A:V", "to heat", "a.za"),
        "H229": ("אֶזְבָּ֑י", "N:N-M-P", "Ezbai", "ez.bay"),
        "H230": ("אַזְדָּא", "A:A", "gone", "a.zad"),
        "H231": ("אֵזוֹב", "H:N-M", "hyssop", "e.zov"),
        "H232": ("אֵזוֹר", "H:N-M", "girdle", "e.zor"),
        "H233": ("אֲזַי", "H:Adv", "then", "a.zay"),
        "H234": ("אַזְכָּרָה", "H:N-F", "memorial", "az.ka.rah"),
        "H235": ("אָזַל", "H:V", "be gone", "a.zal"),
        "H236": ("אֲזַל", "A:V", "to go", "a.zal"),
        "H237": ("אֶזֶל", "N:N--L", "Ezel", "e.zel"),
        "H238": ("אָזַן", "H:V", "to listen", "a.zan"),
        "H239": ("אָזַן", "H:V", "to ponder", "a.zan"),
        "H240": ("אָזֵן", "H:N-M", "weapon", "a.zen"),
        "H241": ("אֹ֫זֶן", "H:N-F", "ear", "o.zen"),
        "H242": ("אֻזֵּן שֶׁאֱרָה", "N:N--L", "Uzzen-sheerah", "uz.zen she.e.rah"),
        "H243": ("אַזְנוֹת תָּבוֹר", "N:N--L", "Aznoth-tabor", "az.not ta.vor"),
        "H244a": ("אׇזְנִי", "N:N-M-P", "Ozni", "oz.ni"),
        "H244b": ("אׇזְנִי", "N:N--PG", "Oznite", "oz.ni"),
        "H245": ("אֲזַנְיָ֫הוּ", "N:N-M-P", "Azaniah", "a.zan.yah"),
        "H246": ("אֲזֵק", "H:N-M", "chains", "a.ziq.qim"),
        "H247": ("אָזַר", "H:V", "to gird", "a.zar"),
        "H248": ("אֶזְרוֹעַ", "H:N-F", "arm", "ez.ro.a"),
        "H249": ("אֶזְרָח", "H:N-M", "born", "ez.rach"),
        "H250": ("אֶזְרָחִי", "N:N--PG", "Ezrahite", "ez.ra.chi"),
        "H251": ("אָח", "H:N-M", "brother", "ach"),
        "H252": ("אַח", "A:N-M", "brother", "ach"),
        "H253": ("אָח", "H:Intj", "ah!", "ach"),
        "H254": ("אָח", "H:N-F", "hearth", "ach"),
        "H255": ("אֹחַ", "H:N-M", "howling animal", "o.ach"),
        "H256": ("אַחְאָב", "N:N-M-P", "Ahab", "ach.av"),
        "H257": ("אַחְבָן", "N:N-M-P", "Ahban", "ach.van"),
        "H258": ("אָחַד", "H:V", "to go either way", "a.chad"),
        "H259": ("אֶחָד", "H:A", "one", "e.chad"),
        "H260": ("אָחוּ", "H:N-M", "meadow", "a.chu"),
        "H261": ("אֵחוּד", "N:N-M-P", "Ehud", "e.chud"),
        "H262": ("אַחְוָה", "H:N-F", "declaration", "ach.vah"),
        "H263": ("אַחֲוָיָה", "A:N-F", "explanation", "a.cha.vah"),
        "H264": ("אַחֲוָה", "H:N-M", "brotherhood", "a.cha.vah"),
        "H265": ("אֲחוֹחַ", "N:N-M-P", "Ahoah", "a.cho.ach"),
        "H266": ("אֲחוֹחִי", "N:N--PG", "Ahohite", "a.cho.chi"),
        "H267": ("אֲחוּמַי", "N:N-M-P", "Ahumai", "a.chu.may"),
        "H268": ("אָחוֹר", "H:N-M", "back", "a.chor"),
        "H269": ("אָחוֹת", "H:N-F", "sister", "a.chot"),
        "H270": ("אָחַז", "H:V", "to grasp", "a.chaz"),
        "H271": ("אָחָז", "N:N-M-P", "Ahaz", "a.chaz"),
        "H272": ("אֲחֻזָּה", "H:N-F", "possession", "a.chuz.zah"),
        "H273": ("אַחְזַי", "N:N-M-P", "Ahzai", "ach.zay"),
        "H274": ("אֲחַזְיָ֫הוּ", "N:N-M-P", "Ahaziah", "a.chaz.yah"),
        "H275": ("אֲחֻזָּם", "N:N-M-P", "Ahuzzam", "a.chuz.zam"),
        "H276": ("אֲחֻזַּת", "N:N-M-P", "Ahuzzath", "a.chuz.zat"),
        "H277": ("אֲחִי", "N:N-M-P", "Ahi", "a.chi"),
        "H278": ("אֵחִי", "N:N-M-P", "Ehi", "e.chi"),
        "H279": ("אֲחִיאָם", "N:N-M-P", "Ahiam", "a.chi.am"),
        "H280": ("אֲחִידָה", "A:N-F", "riddle", "a.chi.dah"),
        "H281": ("אֲחִיָּ֫הוּ", "N:N-M-P", "Ahiah", "a.chiy.yah"),
        "H282": ("אֲחִיהוּד", "N:N-M-P", "Ahihud", "a.chi.hud"),
        "H283": ("אַחְיוֹ", "N:N-M-P", "Ahio", "ach.yo"),
        "H284": ("אֲחִיחֻד", "N:N-M-P", "Ahihud", "a.chi.chud"),
        "H285": ("אֲחִיטוּב", "N:N-M-P", "Ahitub", "a.chi.tuv"),
        "H286": ("אֲחִילוּד", "N:N-M-P", "Ahilud", "a.chi.lud"),
        "H287": ("אֲחִימוֹת", "N:N-M-P", "Ahimoth", "a.chi.mot"),
        "H288": ("אֲחִימֶ֫לֶךְ", "N:N-M-P", "Ahimelech", "a.chi.me.lekh"),
        "H289": ("אֲחִימַן", "N:N-M-P", "Ahiman", "a.chi.man"),
        "H290": ("אֲחִימַ֫עַץ", "N:N-M-P", "Ahimaaz", "a.chi.ma.ats"),
        "H291": ("אַחְיָן", "N:N-M-P", "Ahian", "ach.yan"),
        "H292": ("אֲחִינָדָב", "N:N-M-P", "Ahinadab", "a.chi.na.dav"),
        "H293": ("אֲחִינֹ֫עַם", "N:N-F-P", "Ahinoam", "a.chi.no.am"),
        "H294": ("אֲחִיסָמָךְ", "N:N-M-P", "Ahisamach", "a.chi.sa.makh"),
        "H295": ("אֲחִיעֶ֫זֶר", "N:N-M-P", "Ahiezer", "a.chi.e.zer"),
        "H296": ("אֲחִיקָם", "N:N-M-P", "Ahikam", "a.chi.qam"),
        "H297": ("אֲחִירָם", "N:N-M-P", "Ahiram", "a.chi.ram"),
        "H298": ("אֲחִירָמִי", "N:N--PG", "Ahiramite", "a.chi.ra.mi"),
        "H299": ("אֲחִירַע", "N:N-M-P", "Ahira", "a.chi.ra"),
        "H300": ("אֲחִישָׁ֑חַר", "N:N-M-P", "Ahishahar", "a.chi.sha.char"),
        "H301": ("אֲחִישָׁר", "N:N-M-P", "Ahishar", "a.chi.shar"),
        "H302": ("אֲחִיתֹ֫פֶל", "N:N-M-P", "Ahithophel", "a.chi.to.phel"),
        "H303": ("אַחְלָב", "N:N--L", "Ahlab", "ach.lav"),
        "H304": ("אַחְלָ֑י", "N:N-M-P", "Ahlai", "ach.lay"),
        "H305": ("אַחֲלַי", "H:Intj", "O that!", "a.cha.lay"),
        "H306": ("אַחְלָ֑מָה", "H:N-F", "amethyst", "ach.la.mah"),
        "H307": ("אַחְמְתָא", "N:N--L", "Ecbatana", "ach.me.ta"),
        "H308": ("אֲחַסְבַּי", "N:N-M-P", "Ahasbai", "a.chas.bay"),
        "H309": ("אָחַר", "H:V", "to delay", "a.char"),
        "H310a": ("אַחַר", "H:A", "after", "a.char"),
        "H310b": ("אַחֲרַי", "H:Adv", "backwards", ""),
        "H311": ("אַחַר", "A:Adv", "after", "a.char"),
        "H312": ("אַחֵר", "H:A", "another", "a.cher"),
        "H313": ("אַחֵר", "N:N-M-P", "Aher", "a.cher"),
        "H314": ("אַחֲרוֹן", "H:A", "last", "a.cha.ron"),
        "H315": ("אַחְרַח", "N:N-M-P", "Aharah", "ach.rach"),
        "H316": ("אֲחַרְחֵל", "N:N-M-P", "Aharhel", "a.char.chel"),
        "H317": ("אׇחֳרִי", "A:A", "another", "o.cho.ri"),
        "H318": ("אׇחֳרֵין", "A:Adv", "finally", "o.cho.ren"),
        "H319": ("אַחֲרִית", "H:N-F", "end", "a.cha.rit"),
        "H320": ("אַחֲרִית", "A:N-F", "latter", "a.cha.rit"),
        "H321": ("אׇחֳרָן", "A:A", "another", "o.cho.ran"),
        "H322": ("אֲחֹרַנִּית", "H:Adv", "backwards", "a.cho.ran.nit"),
        "H323": ("אֲחַשְׁדַּרְפַּן", "H:N-M", "satrap", "a.chash.dar.pan"),
        "H324": ("אֲחַשְׁדַּרְפְּנִין", "A:N-M", "satrap", "a.chash.dar.pan"),
        "H325": ("אֲחַשְׁוֵרוֹשׁ", "N:N-M-P", "Ahasuerus", "a.chash.ve.rosh"),
        "H326": ("אֲחַשְׁתָּרִי", "N:N-M-P", "Haahashtari", "a.chash.ta.ri"),
        "H327": ("אֲחַשְׁתְּרָן", "H:A", "steed", "a.chash.ta.ran"),
        "H328a": ("אִטִּי", "H:N-M / H:Adv", "mutterer", "at"),
        "H328b": ("אַט", "H:N-M / H:Adv", "softly", "at"),
        "H329": ("אָטָד", "H:N-M", "bramble", "a.tad"),
        "H330": ("אֵטוּן", "H:N-M", "fine linen", "e.tun"),
        "H331": ("אָטַם", "H:V", "to shutter", "a.tam"),
        "H332": ("אָטַר", "H:V", "to shut", "a.tar"),
        "H333": ("אָטֵר", "N:N-M-P", "Ater", "a.ter"),
        "H334": ("אִטֵּר", "H:A", "lefthanded", "it.ter"),
        "H335": ("אַי", "H:Adv", "where?", "ay"),
        "H336": ("אִי", "H:Adv", "not", "i"),
        "H337": ("אִי", "H:Intj", "woe!", "i"),
        "H338": ("אִי", "H:N-M", "wild beast", "i"),
        "H339": ("אִי", "H:N-M", "coastland", "i"),
        "H340": ("אָיַב", "H:V", "be hostile", "a.yav"),
        "H341": ("אֹיֵב", "H:N-M", "enemy", "o.yev"),
        "H342": ("אֵיבָה", "H:N-F", "enmity", "e.vah"),
        "H343": ("אֵיד", "H:N-M", "calamity", "ed"),
        "H344": ("אַיָּה", "H:N-F", "falcon", "ay.yah"),
        "H345": ("אַיָּה", "N:N-M-P", "Aiah", "ay.yah"),
        "H346": ("אַיֵּה", "H:Adv", "where?", "ay.yeh"),
        "H347": ("אִיּוֹב", "N:N-M-P", "Job", "iy.yov"),
        "H348": ("אִיזֶבֶל", "N:N-F-P", "Jezebel", "i.ze.vel"),
        "H349a": ("אֵיךְ", "H:Adv", "how?", "ekh"),
        "H349b": ("אֵיכָה", "H:Adv", "how?", ""),
        "H349c": ("אֵיכָ֫כָה", "H:Adv", "how?", ""),
        "H350": ("אִי־כָבוֹד", "N:N-M-P", "Ichabod", "i-kha.vod"),
        "H351": ("אֵיכֹה", "H:Adv", "where?", "e.khoh"),
        "H352a": ("אַ֫יִל", "H:N-M", "ram", "a.yil"),
        "H352b": ("אַ֫יִל", "H:N-M", "pillar", "a.yil"),
        "H352c": ("אַ֫יִל", "H:N-M", "leader", "a.yil"),
        "H352d": ("אַ֫יִל", "H:N-M", "terebinth", "a.yil"),
        "H353": ("אֱיָל", "H:N-M", "strength", "e.yal"),
        "H354": ("אַיָּל", "H:N-M", "deer", "ay.yal"),
        "H355": ("אַיָּלָה", "H:N-F", "doe", "ay.ya.lah"),
        "H356": ("אֵלוֹן", "N:N-M-P / N:N--L", "Elon", "e.lon"),
        "H357": ("אַיָּלוֹן", "N:N--L", "Aijalon", "ay.ya.lon"),
        "H358": ("אֵילוֹן בֵּית חָנָן", "N:N--L", "Elonbeth-hanan", "e.lon bet cha.nan"),
        "H359a": ("אֵילַת, אֵילוֹת", "N:N--L", "Elath", "e.lot"),
        "H359b": ("אֵילוֹת", "N:N--L", "Eloth", "e.lot"),
        "H360": ("אֱיָלוּת", "H:N-F", "strength", "e.ya.lut"),
        "H361": ("אֵילָם", "H:N-M", "arches", "e.lam"),
        "H362": ("אֵילִם", "N:N--L", "Elim", "e.lim"),
        "H363": ("אִילָן", "A:N-M", "tree", "i.lan"),
        "H364": ("אֵיל פָּארָן", "N:N--L", "El-paran", "el pa.ran"),
        "H365": ("אַיֶּלֶת", "H:N-F", "doe", "ay.ye.let"),
        "H366": ("אָיֹם", "H:A", "terrible", "a.yom"),
        "H367": ("אֵימָה", "H:N-F", "terror", "e.mah"),
        "H368": ("אֵימִים", "N:N--L", "Emim", "e.mim"),
        "H369": ("אַ֫יִן", "H:Neg", "nothing", "a.yin"),
        "H370": ("אַ֫יִן", "H:Adv", "whence?", "a.yin"),
        "H371": ("אִין", "H:Adv", "isn't", "in"),
        "H372": ("אִיעֶזֶר", "N:N-M-P", "Iezer", "i.e.zer"),
        "H373": ("אִיעֶזְרִי", "N:N--PG", "Iezerite", "i.ez.ri"),
        "H374": ("אֵיפָה", "H:N-F", "ephah", "e.phah"),
        "H375": ("אֵיפֹה", "H:Adv", "where?", "e.phoh"),
        "H376": ("אִישׁ", "H:N-M", "man", "ish"),
        "H377": ("אשׁשׁ", "H:V", "be manly", "ish"),
        "H378": ("אִישׁ־בֹ֫שֶׁת", "N:N-M-P", "Ish-bosheth", "ish-bo.shet"),
        "H379": ("אִישְׁהוֹד", "N:N-M-P", "Ishhod", "ish.hod"),
        "H380": ("אִישׁוֹן", "H:N-M", "pupil", "i.shon"),
        "H381": ("אִישׁ־חַיִל", "H:N-M", "hero", "ish-cha.yil"),
        "H382": ("אִישׁ־טוֹב", "N:N--L", "Ish-tob", "ish-tov"),
        "H383": ("אִיתַי", "A:Part", "there is", "i.tay"),
        "H384": ("אִיתִיאֵל", "N:N-M-P", "Ithiel", "i.ti.el"),
        "H385": ("אִיתָמָר", "N:N-M-P", "Ithamar", "i.ta.mar"),
        "H386": ("אֵיתָן", "H:A", "strong", "e.tan"),
        "H387": ("אֵיתָן", "N:N-M-P", "Ethan", "e.tan"),
        "H388": ("אֵיתָנִים", "N:N-M-T", "Ethanim", "e.ta.nim"),
        "H389": ("אַךְ", "H:Adv", "surely", "akh"),
        "H390": ("אַכַּד", "N:N--L", "Accad", "ak.kad"),
        "H391": ("אַכְזָב", "H:A", "deceptive", "akh.zav"),
        "H392": ("אַכְזִיב", "N:N--L", "Achzib", "akh.ziv"),
        "H393": ("אַכְזָר", "H:A", "cruel", "akh.zar"),
        "H394": ("אַכְזָרִי", "H:A", "cruel", "akh.za.ri"),
        "H395": ("אַכְזְרִיּוּת", "H:N-F", "cruel", "akh.ze.riy.yut"),
        "H396": ("אֲכִילָה", "H:N-F", "food", "a.khi.lah"),
        "H397": ("אָכִישׁ", "N:N-M-P", "Achish", "a.khish"),
        "H398": ("אָכַל", "H:V", "to eat", "a.khal"),
        "H399": ("אֲכַל", "A:V", "to devour", "a.khal"),
        "H400": ("אֹ֫כֶל", "H:N-M", "food", "o.khel"),
        "H401": ("אֻכָל", "N:N-M-P", "Ucal", "u.khal"),
        "H402": ("אׇכְלָה", "H:N-F", "food", "okh.lah"),
        "H403": ("אָכֵן", "H:Adv", "surely", "a.khen"),
        "H404": ("אָכַף", "H:V", "to crave", "a.khaph"),
        "H405": ("אֶ֫כֶף", "H:N-M", "burden", "e.kheph"),
        "H406": ("אִכָּר", "H:N-M", "farmer", "ik.kar"),
        "H407": ("אַכְשָׁף", "N:N--L", "Achshaph", "akh.shaph"),
        "H408": ("אַל", "H:Adv", "not", "al"),
        "H409": ("אַל", "A:Adv", "not", "al"),
        "H410": ("אֵל", "H:N-M", "god", "el"),
        "H411": ("אֵל", "H:DemP", "these", "el"),
        "H412": ("אֵל", "A:DemP", "these", "el"),
        "H413": ("אֶל", "H:Prep", "to(wards)", "el"),
        "H414": ("אֵלָא", "N:N-M-P", "Ela", "e.la"),
        "H415": ("אֵל אֱלֹהֵי יִשְׂרָאֵל", "N:N-M-P", "El-Elohe-Israel", "el e.lo.he yis.ra.el"),
        "H416": ("אֵל בֵּית־אֵל", "N:N--L", "El-beth-el", "el bet-el"),
        "H417": ("אֶלְגָּבִישׁ", "H:N-M", "hail", "el.ga.vish"),
        "H418": ("אַלְגּוּמִּים", "H:N-M", "algum", "al.gum.mim"),
        "H419": ("אֶלְדָּד", "N:N-M-P", "Eldad", "el.dad"),
        "H420": ("אֶלְדָּעָה", "N:N-M-P", "Eldaah", "el.da.ah"),
        "H421": ("אָלָה", "H:V", "to wail", "a.lah"),
        "H422": ("אָלָה", "H:V", "to swear", "a.lah"),
        "H423": ("אָלָה", "H:N-F", "oath", "a.lah"),
        "H424": ("אֵלָה", "H:N-F", "oak", "e.lah"),
        "H425": ("אֵלָה", "N:N-M-P", "Elah", "e.lah"),
        "H426": ("אֱלָהּ", "H:N-M", "god", "e.lah"),
        "H427": ("אַלָּה", "H:N-M", "oak", "al.lah"),
        "H428": ("אֵ֫לֶּה", "H:DemP", "these", "el.leh"),
        "H429": ("אֵלֶּה", "A:DemP", "these", "el.leh"),
        "H430": ("אֱלֹהִים", "H:N-M", "God", "e.lo.him"),
        "H431": ("אֲלוּ", "A:Intj", "behold", "a.lu"),
        "H432": ("אִלּוּ", "H:Cond", "except", "il.lu"),
        "H433": ("אֱלֹהַּ", "H:N-M", "god", "e.lo.ah"),
        "H434": ("אֱלוּל", "H:A", "worthless", "e.lul"),
        "H435": ("אֱלוּל", "N:N--T", "Elul", "e.lul"),
        "H436": ("אֵלוֹן", "H:N-M", "terebinth", "e.lon"),
        "H437": ("אַלּוֹן", "H:N-M", "oak", "al.lon"),
        "H438": ("אַלּוֹן", "N:N-M-P / N:N--L", "Allon", "al.lon"),
        "H439": ("אַלּוֹן בָּכוּת", "N:N--L", "Allon-bacuth", "al.lon ba.khut"),
        "H440": ("אֵלֹנִי", "N:N--PG", "Elonite", "e.lo.ni"),
        "H441a": ("אַלּוּף", "H:A-M", "tame", "al.luph"),
        "H441b": ("אַלּוּף", "H:N-M", "chief", "al.luph"),
        "H442": ("אָלוּשׁ", "N:N--L", "Alush", "a.lush"),
        "H443": ("אֶלְזָבָד", "N:N-M-P", "Elzabad", "el.za.vad"),
        "H444": ("אָלַח", "H:V", "to corrupt", "a.lach"),
        "H445": ("אֶלְחָנָן", "N:N-M-P", "Elhanan", "el.cha.nan"),
        "H446": ("אֱלִיאָב", "N:N-M-P", "Eliab", "e.li.av"),
        "H447": ("אֱלִיאֵל", "N:N-M-P", "Eliel", "e.li.el"),
        "H448": ("אֳלִיאָ֫תָה", "N:N-M-P", "Eliathah", "e.li.a.tah"),
        "H449": ("אֱלִידָד", "N:N-M-P", "Elidad", "e.li.dad"),
        "H450": ("אֶלְיָדָע", "N:N-M-P", "Eliada", "el.ya.da"),
        "H451": ("אַלְיָה", "H:N-F", "fat tail", "al.yah"),
        "H452": ("אֵלִיָּה", "N:N-M-P", "Elijah", "e.liy.yah"),
        "H453": ("אֱלִיהוּ", "N:N-M-P", "Elihu", "e.li.hu"),
        "H454a": ("אֶלְיְהוֹעֵינַי", "N:N-M-P", "Elihoenai", "el.ye.ho.e.nay"),
        "H454b": ("אֶלְיוֹעֵינַי", "N:N-M-P", "Elioenai", ""),
        "H455": ("אֶלְיַחְבָּא", "N:N-M-P", "Eliahba", "el.yach.ba"),
        "H456": ("אֱלִיחֹ֫רֶף", "N:N-M-P", "Elihoreph", "e.li.cho.reph"),
        "H457": ("אֱלִיל", "H:A-M", "idol", "e.lil"),
        "H458": ("אֱלִימֶ֫לֶךְ", "N:N-M-P", "Elimelech", "e.li.me.lekh"),
        "H459": ("אִלֵּין", "A:DemP", "these", "il.len"),
        "H460": ("אֶלְיָסָף", "N:N-M-P", "Eliasaph", "el.ya.saph"),
        "H461": ("אֱלִיעֶ֫זֶר", "N:N-M-P", "Eliezer", "e.li.e.zer"),
        "H462": ("אֱלִיעֵינַי", "N:N-M-P", "Elienai", "e.li.e.nay"),
        "H463": ("אֱלִיעָם", "N:N-M-P", "Eliam", "e.li.am"),
        "H464": ("אֱלִיפַז", "N:N-M-P", "Eliphaz", "e.li.phaz"),
        "H465": ("אֱלִיפָל", "N:N-M-P", "Eliphal", "e.li.phal"),
        "H466": ("אֱלִיפְלֵ֫הוּ", "N:N-M-P", "Eliphelehu", "e.liph.le.hu"),
        "H467": ("אֲלִיפֶ֫לֶט", "N:N-M-P", "Eliphelet", "e.li.phe.let"),
        "H468": ("אֱלִיצוּר", "N:N-M-P", "Elizur", "e.li.tsur"),
        "H469": ("אֱלִיצָפָן", "N:N-M-P", "Elizaphan", "e.li.tsa.phan"),
        "H470": ("אֱלִיקָא", "N:N-M-P", "Elika", "e.li.qa"),
        "H471": ("אֶלְיָקִים", "N:N-M-P", "Eliakim", "el.ya.qim"),
        "H472": ("אֱלִישֶׁ֫בַע", "N:N-F-P", "Elisheba", "e.li.she.va"),
        "H473": ("אֱלִישָׁה", "N:N-M-P / N:N--L", "Elishah", "e.li.shah"),
        "H474": ("אֱלִישׁוּעַ", "N:N-M-P", "Elishua", "e.li.shu.a"),
        "H475": ("אֶלְיָשִׁיב", "N:N-M-P", "Eliashib", "el.ya.shiv"),
        "H476": ("אֱלִישָׁמָע", "N:N-M-P", "Elishama", "e.li.sha.ma"),
        "H477": ("אֱלִישָׁע", "N:N-M-P", "Elisha", "e.li.sah"),
        "H478": ("אֱלִישָׁפָט", "N:N-M-P", "Elishaphat", "e.li.sha.phat"),
        "H479": ("אִלֵּךְ", "A:DemP", "these", "il.lekh"),
        "H480": ("אַלְלַי", "H:Intj", "woe!", "al.lay"),
        "H481": ("אָלַם", "H:V", "be dumb", "a.lam"),
        "H482": ("אֵ֫לֶם", "H:A-M", "silence", "e.lem"),
        "H483": ("אִלֵּם", "H:A", "mute", "il.lem"),
        "H484": ("אַלְמֻגִּים", "H:N-M", "almug", "al.mug.gim"),
        "H485": ("אֲלֻמָה", "H:N-F", "sheaf", "a.lum.mah"),
        "H486": ("אַלְמוֹדָד", "N:N-M-P", "Almodad", "al.mo.dad"),
        "H487": ("אַלַּמֶּלֶךְ", "N:N--L", "Allammelech", "al.lam.me.lekh"),
        "H488": ("אַלְמָן", "H:A", "forsaken", "al.man"),
        "H489": ("אַלְמֹן", "H:N-M", "widower", "al.mon"),
        "H490": ("אַלְמָנָה", "H:N-F", "widow", "al.ma.nah"),
        "H491": ("אַלְמָנוּת", "H:N-F", "widowhood", "al.ma.nut"),
        "H492": ("אַלְמֹנִי", "H:A", "someone", "al.mo.ni"),
        "H493": ("אֶלְנָ֑עַם", "N:N-M-P", "Elnaam", "el.na.am"),
        "H494": ("אֶלְנָתָן", "N:N-M-P", "Elnathan", "el.na.tan"),
        "H495": ("אֶלָּסָר", "N:N--L", "Ellasar", "el.la.sar"),
        "H496": ("אֶלְעָד", "N:N-M-P", "Elead", "el.ad"),
        "H497": ("אֶלְעָדָה", "N:N-M-P", "Eleadah", "el.a.dah"),
        "H498": ("אֶלְעוּזַי", "N:N-M-P", "Eluzai", "el.u.zay"),
        "H499": ("אֶלְעָזָר", "N:N-M-P", "Eleazar", "el.a.zar"),
        "H500": ("אֶלְעָלֵא", "N:N--L", "Elealeh", "el.a.le"),
        "H501": ("אֶלְעָשָׂה", "N:N-M-P", "Elasah", "el.a.sah"),
        "H502": ("אָלַף", "H:V", "to teach/learn", "a.laph"),
        "H503": ("אָלַף", "H:V", "to produce thousands", "a.laph"),
        "H504": ("אֶ֫לֶף", "H:N-M", "cattle", "e.leph"),
        "H505": ("אֶ֫לֶף", "H:N-M", "thousand", "e.leph"),
        "H506": ("אֲלַ֑ף", "A:N-M", "thousand", "a.laph"),
        "H507": ("הָאֶ֫לֶף", "N:N--L", "Haeleph", "e.leph"),
        "H508": ("אֶלְפַּ֫עַל", "N:N-M-P", "Elpaal", "el.pa.al"),
        "H509": ("אָלַץ", "H:V", "to urge", "a.lats"),
        "H510": ("אַלְקוּם", "H:N-M", "army", "al.qum"),
        "H511": ("אֶלְקָנָה", "N:N-M-P", "Elkanah", "el.qa.nah"),
        "H512": ("אֶלְקֹשִׁי", "N:N--LG", "Elkoshite", "el.qo.shi"),
        "H513": ("אֶלְתּוֹלַד", "N:N--L", "Eltolad", "el.to.lad"),
        "H514": ("אֶלְתְּקֵא", "N:N--L", "Elteke", "el.te.qe"),
        "H515": ("אֶלְתְּקֹן", "N:N--L", "Eltekon", "el.te.qon"),
        "H516": ("אַל תַּשְׁחֵת", "H:N-M", "Al-taschith", "al tash.chet"),
        "H517": ("אֵם", "H:N-F", "mother", "em"),
        "H518a": ("אִם", "H:Cond", "if", "im"),
        "H518b": ("כִּי אם", "H:Cond", "except", ""),
        "H519": ("אָמָה", "H:N-F", "maidservant", "a.mah"),
        "H520a": ("אַמָּה", "H:N-F", "cubit", "am.mah"),
        "H520b": ("אַמָּה", "H:N-F", "foundation", "am.mah"),
        "H521": ("אַמָּה", "A:N", "cubit", "am.mah"),
        "H522": ("אַמָּה", "N:N--L", "Ammah", "am.mah"),
        "H523": ("אֻמָּה", "H:N-F", "people", "um.mah"),
        "H524": ("אֻמָּה", "A:N-F", "people", "um.mah"),
        "H525": ("אָמוֹן", "H:N-M", "artisan", "a.mon"),
        "H526": ("אָמוֹן", "N:N-M-P", "Amon", "a.mon"),
        "H527": ("אָמוֹן", "H:N-M", "multitude", "a.mon"),
        "H528": ("אָמוֹן", "N:N-M-T", "Amon", "a.mon"),
        "H529": ("אֵמֻן", "H:N-M", "faithful", "e.mun"),
        "H530": ("אֱמוּנָה", "H:N-F", "faithfulness", "e.mu.nah"),
        "H531": ("אָמוֹץ", "N:N-M-P", "Amoz", "a.mots"),
        "H532": ("אָמִי", "N:N-M-P", "Ami", "a.mi"),
        "H533": ("אַמִּיץ", "H:A", "strong", "am.mits"),
        "H534": ("אָמִיר", "H:N-M", "top", "a.mir"),
        "H535": ("אָמַל", "H:V", "to weaken", "a.mal"),
        "H536": ("אֻמְלַל", "H:A", "weak", "um.lal"),
        "H537": ("אֲמֵלָל", "H:A", "weak", "a.me.lal"),
        "H538": ("אֲמָם", "N:N--L", "Amam", "a.mam"),
        "H539": ("אָמַן", "H:V", "be faithful", "a.man"),
        "H540": ("אֲמַן", "A:V", "to trust", "a.man"),
        "H541": ("אָמַן", "H:V", "to turn right", "a.man"),
        "H542": ("אָמָּן", "H:N-M", "artisan", "a.man"),
        "H543": ("אָמֵן", "H:Adv", "amen", "a.men"),
        "H544": ("אֹ֫מֶן", "H:N-M", "faithfulness", "o.men"),
        "H545": ("אׇמְנָה", "H:N-F", "brought up", "om.nah"),
        "H546": ("אׇמְנָה", "H:Adv", "truly", "om.nah"),
        "H547": ("אֹמְנָה", "H:N-F", "pillar", "o.me.nah"),
        "H548": ("אֲמָנָה", "H:N-F", "sure", "a.ma.nah"),
        "H549": ("אֲמָנָה", "N:N--L", "Amana", "a.ma.nah"),
        "H550": ("אַמְנוֹן", "N:N-M-P", "Amnon", "am.non"),
        "H551": ("אׇמְנָם", "H:Adv", "truly", "om.nam"),
        "H552": ("אֻמְנָם", "H:Adv", "truly", "um.nam"),
        "H553": ("אָמֵץ", "H:V", "to strengthen", "a.mats"),
        "H554": ("אָמֹץ", "H:A", "strong", "a.mots"),
        "H555": ("אֹ֫מֶץ", "H:N-M", "strength", "o.mets"),
        "H556": ("אַמְצָה", "H:N-F", "strength", "am.tsah"),
        "H557": ("אַמְצִי", "N:N-M-P", "Amzi", "am.tsi"),
        "H558": ("אֲמַצְיָ֫הוּ", "N:N-M-P", "Amaziah", "a.mats.yah"),
        "H559": ("אָמַר", "H:V", "to say", "a.mar"),
        "H560": ("אֲמַר", "A:V", "to say", "a.mar"),
        "H561": ("אֵמֶר", "H:N-M", "word", "e.mer"),
        "H562": ("אֹמֶר", "H:N-M", "word", "o.mer"),
        "H563": ("אִמַּר", "A:N-M", "lamb", "im.mar"),
        "H564": ("אִמֵּר", "N:N-M-P / N:N--L", "Immer", "im.mer"),
        "H565a": ("אִמְרָה", "H:N-F", "word", "im.rah"),
        "H565b": ("אֶמְרָה", "H:N-F", "threat", ""),
        "H566": ("אִמְרִי", "N:N-M-P", "Imri", "im.ri"),
        "H567": ("אֱמֹרִי", "N:N--LG", "Amorites", "e.mo.ri"),
        "H568": ("אֲמַרְיָ֫הוּ", "N:N-M-P", "Amariah", "a.mar.yah"),
        "H569": ("אַמְרָפֶל", "N:N-M-P", "Amraphel", "am.ra.phel"),
        "H570": ("אֶ֫מֶשׁ", "H:Adv", "last night", "e.mesh"),
        "H571": ("אֱמֶת", "H:N-F", "truth", "e.met"),
        "H572": ("אַמְתַּ֫חַת", "H:N-F", "sack", "am.ta.chat"),
        "H573": ("אֲמִתַּי", "N:N-M-P", "Amittai", "a.mit.tay"),
        "H574": ("אֵימְתָן", "A:A", "terrible", "e.me.ta.ni"),
        "H575": ("אָן", "H:Adv", "where?", "an"),
        "H576a": ("אֲנָא", "A:PerP-CS", "me", "a.na"),
        "H576b": ("אֲנָה", "A:PerP-CS", "me", ""),
        "H577": ("אָֽנָּ֫א", "H:Intj", "Please!", "an.na"),
        "H578": ("אָנָה", "H:V", "to lament", "a.nah"),
        "H579": ("אָנָה", "H:V", "to meet", "a.nah"),
        "H580": ("אֲנוּ", "H:PerP-CP", "we", "a.nu"),
        "H581a": ("אִנּוּן", "A:DemP", "they", "in.nun"),
        "H581b": ("אִנִּין", "A:DemP", "they(fem.)", ""),
        "H582": ("אֱנוֹשׁ", "H:N-M", "human", "e.nosh"),
        "H583": ("אֱנוֹשׁ", "N:N-M-P", "Enosh", "e.nosh"),
        "H584": ("אָנַח", "H:V", "to sigh", "a.nach"),
        "H585": ("אֲנָחָה", "H:N-F", "sighing", "a.na.chah"),
        "H586": ("אֲנַ֫חְנָא", "A:PerP-CP", "we", "a.nach.na"),
        "H587": ("אֲנַ֫חְנוּ", "H:PerP-CP", "we", "a.nach.nu"),
        "H588": ("אֲנָֽחֲרָ֑ת", "N:N--L", "Anaharath", "a.na.cha.rat"),
        "H589": ("אֲנִי, אָֽנֹכִ֫י", "H:PerP-CS", "I", "a.ni"),
        "H590": ("אֳנִי", "H:N", "fleet", "o.ni"),
        "H591": ("אֳנִיָּה", "H:N-F", "fleet", "o.niy.yah"),
        "H592": ("אֲנִיָּה", "H:N-F", "lamentation", "a.niy.yah"),
        "H593": ("אֲנִיעָם", "N:N-M-P", "Aniam", "a.ni.am"),
        "H594": ("אֲנָךְ", "H:N-M", "plumbline", "a.nakh"),
        "H595": ("אָֽנֹכִ֫י", "H:PerP-CS", "I", "a.no.khi"),
        "H596": ("אָנַן", "H:V", "to complain", "a.nan"),
        "H597": ("אָנַס", "H:V", "to compel", "a.nas"),
        "H598": ("אֲנַס", "A:V", "to constrain", "a.nas"),
        "H599": ("אָנַף", "H:V", "be angry", "a.naph"),
        "H600": ("אֲנַף", "A:N-M", "face", "a.naph"),
        "H601": ("אֲנָפָה", "H:N-F", "heron", "a.na.phah"),
        "H602": ("אָנַק", "H:V", "to groan", "a.naq"),
        "H603": ("אֲנָקָה", "H:N-F", "groaning", "a.na.qah"),
        "H604": ("אֲנָקָה", "H:N-F", "gecko", "a.na.qah"),
        "H605": ("אָנַשׁ", "H:V", "be incurable", "a.nash"),
        "H606": ("אֱנָשׁ", "A:N-M", "man", "e.nash"),
        "H607": ("אַנְתָּה", "A:PerP-MS", "you", "an.tah"),
        "H608": ("אַנְתּוּן", "A:PerP-MP", "you", "an.tun"),
        "H609": ("אָסָא", "N:N-M-P", "Asa", "a.sa"),
        "H610": ("אָסוּךְ", "H:N-M", "flask", "a.sukh"),
        "H611": ("אָסוֹן", "H:N-M", "harm", "a.son"),
        "H612": ("אֵסוּר", "H:N-M", "bond", "e.sur"),
        "H613": ("אֱסוּר", "A:N-M", "bond", "e.sur"),
        "H614": ("אָסִיף", "H:N-M", "ingathering", "a.siph"),
        "H615": ("אָסִיר", "H:N-M", "prisoner", "a.sir"),
        "H616": ("אַסִּיר", "H:N-M", "prisoner", "as.sir"),
        "H617": ("אַסִּיר", "N:N-M-P", "Assir", "as.sir"),
        "H618": ("אָסָם", "H:N-M", "storehouse", "a.sam"),
        "H619": ("אַסְנָה", "N:N-M-P", "Asnah", "as.nah"),
        "H620": ("אָֽסְנַפַּר", "N:N-M-P", "Osnappar", "os.nap.par"),
        "H621": ("אָֽסְנַת", "N:N-F-P", "Asenath", "a.se.nat"),
        "H622": ("אָסַף", "H:V", "to gather", "a.saph"),
        "H623": ("אָסָף", "N:N-M-P", "Asaph", "a.saph"),
        "H624": ("אָסֹף", "H:N-M", "storehouse", "a.suph"),
        "H625": ("אֹ֫סֵף", "H:N-M", "gathering", "o.seph"),
        "H626": ("אֲסֵפָה", "H:N-F", "collecting", "a.se.phah"),
        "H627": ("אֲסֻפָּה", "H:N-F", "collection", "a.sup.pah"),
        "H628": ("אֲסַפְסֻף", "H:N-M", "rabble", "a.se.pe.suph"),
        "H629": ("אׇסְפַּרְנָא", "A:Adv", "diligently", "os.par.na"),
        "H630": ("אַסְפָּ֑תָא", "N:N-M-P", "Aspatha", "as.pa.ta"),
        "H631": ("אָסַר", "H:V", "to bind", "a.sar"),
        "H632": ("אִסָּר", "H:N-M", "injunction", "e.sar"),
        "H633": ("אֱסָר", "A:N-M", "injunction", "e.sar"),
        "H634": ("אֵסַר־חַדּוֹן", "N:N-M-P", "Esar-haddon", "e.sar-chad.don"),
        "H635": ("אֶסְתֵּר", "N:N-F-P", "Esther", "es.ter"),
        "H636": ("אָע", "A:N-M", "wood", "a"),
        "H637": ("אַף", "H:Part", "also", "aph"),
        "H638": ("אַף", "A:Conj", "also", "aph"),
        "H639": ("אַף", "H:N-M", "anger", "aph"),
        "H640": ("אָפַד", "H:V", "to gird", "a.phad"),
        "H641": ("אֵפֹד", "N:N-M-P", "Ephod", "e.phod"),
        "H642": ("אֲפֻדָּה", "H:N-F", "ephod", "e.phud.dah"),
        "H643": ("אַפֶּ֫דֶן", "H:N", "palace", "ap.pe.den"),
        "H644": ("אָפָה", "H:V", "to bake", "a.phah"),
        "H645": ("אֵפוֹ", "H:Part", "then", "e.pho"),
        "H646": ("אֵפוֹד", "H:N-M", "ephod", "e.phod"),
        "H647": ("אֲפִיחַ", "N:N-M-P", "Aphiah", "a.phi.ach"),
        "H648": ("אָפִיל", "H:A", "late", "a.phil"),
        "H649": ("אַפַּיִם", "N:N-M-P", "Appaim", "ap.pa.yim"),
        "H650": ("אָפִיק", "H:N-M", "channel", "a.phiq"),
        "H651": ("אָפֵל", "H:A", "gloom", "a.phel"),
        "H652": ("אֹ֫פֶל", "H:N-M", "darkness", "o.phel"),
        "H653": ("אֲפֵלָה", "H:N-F", "darkness", "a.phe.lah"),
        "H654": ("אֶפְלָל", "N:N-M-P", "Ephlal", "eph.lal"),
        "H655": ("אֹ֫פֶן", "H:N-M", "(right) time", "o.phen"),
        "H656": ("אָפֵס", "H:V", "to end", "a.phes"),
        "H657a": ("אֶ֫פֶס", "H:N-M", "end", "e.phes"),
        "H657b": ("אֹ֫פֶס", "H:N-M", "soles", ""),
        "H658": ("אֶ֫פֶס דַּמִּים", "N:N--L", "Ephes-dammim", "e.phes dam.mim"),
        "H659": ("אֶ֫פַע", "H:A", "worthless", "e.pah"),
        "H660": ("אֶפְעֶה", "H:N-M", "viper", "eph.eh"),
        "H661": ("אָפַף", "H:V", "to surround", "a.phaph"),
        "H662": ("אָפַק", "H:V", "to refrain", "a.phaq"),
        "H663": ("אֲפֵק", "N:N--L", "Aphek", "a.pheq"),
        "H664": ("אֲפֵקָה", "N:N--L", "Aphekah", "a.phe.qah"),
        "H665": ("אֵ֫פֶר", "H:N-M", "ashes", "e.pher"),
        "H666": ("אֲפֵר", "H:N-M", "bandage", "a.pher"),
        "H667": ("אֶפְרֹחַ", "H:N-M", "young", "eph.ro.ach"),
        "H668": ("אַפִּרְיוֹן", "H:N-M", "carriage", "ap.pir.yon"),
        "H669": ("אֶפְרַ֫יִם", "N:N-M-P / N:N-M-PG", "Ephraim", "eph.ra.yim"),
        "H670": ("אֲפָֽרְסָיֵא", "N:N--LG", "Apharsite", "a.pha.re.say"),
        "H671a": ("אֲפָֽרְסְכָיֵא", "N:N--LG", "Apharsachite", "a.phar.se.khay"),
        "H671b": ("אֲפָֽרְסַתְּכָיֵא", "N:N--TG", "Apharasthchite", ""),
        "H672": ("אֶפְרָ֫תָה", "N:N-F-P", "Ephrathah", "eph.rat"),
        "H673": ("אֶפְרָתִי", "N:N--PG", "Ephrathite", "eph.ra.ti"),
        "H674": ("אַפְּתֹם", "A:N-M", "revenue", "ap.p.tom"),
        "H675": ("אֶצְבּוֹן", "N:N-M-P", "Ezbon", "ets.bon"),
        "H676": ("אֶצְבַּע", "H:N-F", "finger", "ets.ba"),
        "H677": ("אֶצְבַּע", "A:N-F", "digit", "ets.ba"),
        "H678": ("אָצִיל", "H:N-M", "chief", "a.tsil"),
        "H679": ("אַצִּיל", "H:N-F", "joint", "ats.tsil"),
        "H680": ("אָצַל", "H:V", "to reserve", "a.tsal"),
        "H681": ("אֵ֫צֶל", "H:N-M", "beside", "e.tsel"),
        "H682a": ("אָצֵל", "N:N-M-P", "Azel", "a.tsel"),
        "H682b": ("אָצֵל", "N:N--L", "Azal", "a.tsel"),
        "H683": ("אֲצַלְיָ֫הוּ", "N:N-M-P", "Azaliah", "a.tsal.ya.hu"),
        "H684": ("אֹ֫צֶם", "N:N-M-P", "Ozem", "o.tsem"),
        "H685": ("אֶצְעָדָה", "H:N-F", "armlet", "ets.a.dah"),
        "H686": ("אָצַר", "H:V", "to store", "a.tsar"),
        "H687": ("אֵ֫צֶר", "N:N-M-P", "Ezer", "e.tser"),
        "H688": ("אֶקְדָּח", "H:N-M", "carbuncle", "eq.dach"),
        "H689": ("אַקּוֹ", "H:N-M", "wild goat", "aq.qo"),
        "H690": ("אֲרָא", "N:N-M-P", "Ara", "a.ra"),
        "H691": ("אֶרְאֵל", "H:N-M", "hero", "er.el"),
        "H692": ("אַרְאֵלִי", "N:N-M-P", "Areli", "ar.e.li"),
        "H693": ("אָרַב", "H:V", "to ambush", "a.rav"),
        "H694": ("אֲרָב", "N:N--L", "Arab", "a.rav"),
        "H695": ("אֶ֫רֶב", "H:N-M", "ambush", "e.rev"),
        "H696": ("אֹ֫רֶב", "H:N-M", "ambush", "o.rev"),
        "H697": ("אַרְבֶּה", "H:N-M", "locust", "ar.beh"),
        "H698": ("אָרְבָּה", "H:N-F", "skill", "o.ro.vah"),
        "H699": ("אֲרֻבָּה", "H:N-F", "window", "a.rub.bah"),
        "H700": ("אֲרֻבּוֹת", "N:N--L", "Arubboth", "a.rub.bot"),
        "H701": ("אַרְבִּי", "N:N--LG", "Arbite", "ar.bi"),
        "H702": ("אַרְבַּע", "H:N", "four", "ar.ba"),
        "H703": ("אַרְבַּע", "A:N", "four", "ar.ba"),
        "H704": ("אַרְבַּע", "N:N-M-P", "Arba", "ar.ba"),
        "H705": ("אַרְבָּעִים", "H:N", "forty", "ar.ba.im"),
        "H706": ("אַרְבַּעְתַּיִם", "H:Adv", "fourfold", "ar.ba.ta.yim"),
        "H707": ("אָרַג", "H:V", "to weave", "a.rag"),
        "H708": ("אֶ֫רֶג", "H:N-M", "shuttle", "e.reg"),
        "H709": ("אַרְגֹּב", "N:N--L / N:N-M-P", "Argob", "ar.gov"),
        "H710": ("אַרְגְּוָן", "H:N-M", "purple", "ar.ge.van"),
        "H711": ("אַרְגְּוָן", "A:N-M", "purple", "ar.ge.van"),
        "H712": ("אַרְגַּז", "H:N-M", "box", "ar.gaz"),
        "H713": ("אַרְגָּמָן", "H:N-M", "purple", "ar.ga.man"),
        "H714": ("אַרְדְּ", "N:N-M-P", "Ard", "ard"),
        "H715": ("אַרְדּוֹן", "N:N-M-P", "Ardon", "ar.don"),
        "H716": ("אַרְדִּי", "N:N--PG", "Ardite", "ar.di"),
        "H717": ("אָרָה", "H:V", "to pluck", "a.rah"),
        "H718": ("אֲרוּ", "A:Intj", "behold!", "a.ru"),
        "H719": ("אַרְוַד", "N:N--L", "Arvad", "ar.vad"),
        "H720": ("אֲרוֹד", "N:N-M-P", "Arod", "a.rod"),
        "H721": ("אַרְוָדִי", "N:N--LG", "Arvadite", "ar.va.di"),
        "H722": ("אֲרוֹדִי", "N:N--PG", "Arodi", "a.ro.di"),
        "H723": ("אֻרְיָה", "H:N-F", "stall", "ur.vah"),
        "H724": ("אֲרוּכָה", "H:N-F", "health", "a.ru.khah"),
        "H725": ("אֲרוּמָה", "N:N--L", "Arumah", "a.ru.mah"),
        "H726": ("אֲרוֹמִי", "N:N--PG", "Syrian", "a.ro.mi"),
        "H727": ("אֲרוֹן", "H:N-M", "ark", "a.ron"),
        "H728": ("אֲרַוְנָה", "N:N-M-P", "Araunah", "a.rav.nah"),
        "H729": ("אָרוּז", "H:A", "reinforced", "a.raz"),
        "H730": ("אֶ֫רֶז", "H:N-M", "cedar", "e.rez"),
        "H731": ("אַרְזָה", "H:N-F", "cedar", "ar.zah"),
        "H732": ("אָרַח", "H:V", "to journey", "a.rach"),
        "H733": ("אָרַח", "N:N-M-P", "Arah", "a.rach"),
        "H734": ("אֹ֫רַח", "H:N-M", "way", "o.rach"),
        "H735": ("אֹ֫רַח", "A:N-M", "way", "o.rach"),
        "H736": ("אֹרְחָה", "H:N-F", "caravan", "o.re.chah"),
        "H737": ("אֲרֻחָה", "H:N-F", "ration", "a.ru.chah"),
        "H738a": ("אֲרִי", "H:N-M", "lion", "a.ri"),
        "H738b": ("אַרְיֵה", "H:N-M", "lion", ""),
        "H739": ("אֲרִיאֵל", "N:N-M-P", "Ariel", "a.ri.el"),
        "H740": ("אֲרִיאֵל", "N:N-M-P / N:N--L", "Ariel", "a.ri.el"),
        "H741": ("אֲרִאֵיל", "H:N-M", "altar", "a.ri.el"),
        "H742": ("אֲרִידַי", "N:N-M-P", "Aridai", "a.ri.day"),
        "H743": ("אֲרִידָ֑תָא", "N:N-M-P", "Aridatha", "a.ri.da.ta"),
        "H744": ("אַרְיֵה", "A:N-M", "lion", "ar.yeh"),
        "H745": ("אַרְיֵה", "N:N-M-P", "Arieh", "ar.yeh"),
        "H746a": ("אַרְיוֹךְ", "N:N-M-P", "Arioch", "a.re.yokh"),
        "H746b": ("אֲרְיוֹךְ", "N:N-M-P", "Arioch", "a.re.yokh"),
        "H747": ("אֲרִיסַי", "N:N-M-P", "Arisai", "a.ri.say"),
        "H748": ("אָרַךְ", "H:V", "to prolong", "a.rakh"),
        "H749": ("אֲרִיךְ", "A:V", "be proper", "a.rakh"),
        "H750": ("אָרֵךְ", "H:A", "slow", "a.rekh"),
        "H751": ("אֶ֫רֶךְ", "N:N--L", "Erech", "e.rekh"),
        "H752": ("אָרֹךְ", "H:A", "long", "a.rokh"),
        "H753": ("אֹ֫רֶךְ", "H:N-M", "length", "o.rekh"),
        "H754": ("אַרְכָֿא", "A:N-F", "lengthening", "ar.ka"),
        "H755": ("אַרְכֻבָּה", "A:N-F", "knee", "ar.khu.vah"),
        "H756": ("אַרְכְּוָיֵא", "N:N--LG", "Archevite", "ar.ke.vay"),
        "H757": ("אַרְכִּי", "N:N--LG", "Archite", "ar.ki"),
        "H758": ("אֲרָם", "N:N-M-P / N:N-M-PG", "Aram", "a.ram"),
        "H759": ("אַרְמוֹן", "H:N-M", "citadel", "ar.mon"),
        "H760": ("אֲרַם צוֹבָה", "N:N--L", "Aram-zobah", "a.ram tso.vah"),
        "H761": ("אֲרַמִּי", "N:N--PG", "Aramean", "a.ram.mi"),
        "H762": ("אֲרָמִית", "H:Adv", "in Aramaic", "a.ra.mit"),
        "H763": ("אֲרַם נַהֲרַיִם", "N:N--L", "Mesopotamia", "a.ram na.ha.ra.yim"),
        "H764": ("אַרְמֹנִי", "N:N-M-P", "Armoni", "ar.mo.ni"),
        "H765": ("אֲרָן", "N:N-M-P", "Aran", "a.ran"),
        "H766": ("אֹ֫רֶן", "H:N-M", "fir", "o.ren"),
        "H767": ("אֹ֫רֶן", "N:N-M-P", "Oren", "o.ren"),
        "H768": ("אַרְנֶבֶת", "H:N-F", "hare", "ar.ne.vet"),
        "H769": ("אַרְנוֹן", "N:N--L", "Arnon", "ar.non"),
        "H770": ("אַרְנָן", "N:N-M-P", "Arnan", "ar.nan"),
        "H771": ("אׇרְנָן", "N:N-M-P", "Ornan", "or.nan"),
        "H772": ("אֲרַע", "A:N-M", "earth", "a.ra"),
        "H773": ("אַרְעִי", "A:N-F", "bottom", "ar.it"),
        "H774": ("אַרְפַּד", "N:N--L", "Arpad", "ar.pad"),
        "H775": ("אַרְפַּכְשַׁד", "N:N-M-P", "Arpachshad", "ar.pakh.shad"),
        "H776": ("אֶ֫רֶץ", "H:N-F", "land", "e.rets"),
        "H777": ("אַרְצָא", "N:N-M-P", "Arza", "ar.tsa"),
        "H778": ("אֲרַק", "A:N-F", "earth", "a.raq"),
        "H779": ("אָרַר", "H:V", "to curse", "a.rar"),
        "H780": ("אֲרָרַט", "N:N--L", "Ararat", "a.ra.rat"),
        "H781": ("אָרַשׂ", "H:V", "to betroth", "a.ra.s"),
        "H782": ("אֲרֶ֫שֶׁת", "H:N-F", "request", "a.re.shet"),
        "H783a": ("אַרְתַּחְשַׁשְׂתָּא", "N:N-M-P", "Artaxerxes", "ar.tach.shash.ta"),
        "H783b": ("אַרְתַּחְשַׁשְׂתְּא", "N:N-M-P", "Artaxerxes", "ar.tach.shash.ta"),
        "H784": ("אֵשׁ", "H:N-F", "fire", "esh"),
        "H785": ("אֶשָּׁא", "A:N-F", "fire", "esh"),
        "H786": ("אִשׁ", "H:Adv", "there is", "ish"),
        "H787": ("אֹשׁ", "A:N-M", "foundation", "osh"),
        "H788": ("אַשְׁבֵּל", "N:N-M-P", "Ashbel", "ash.bel"),
        "H789": ("אַשְׁבֵּלִי", "N:N--PG", "Ashbelite", "ash.be.li"),
        "H790": ("אֶשְׁבָּן", "N:N-M-P", "Eshban", "esh.ban"),
        "H791": ("אַשְׁבֵּעַ", "N:N--L", "Ashbea", "ash.be.a"),
        "H792": ("אֶשְׁבַּעַל", "N:N-M-P", "Eshbaal", "esh.ba.al"),
        "H793": ("אֶ֫שֶׁד", "H:N-M", "slope", "e.shed"),
        "H794": ("אֲשֵׁדָה", "H:N-F", "slope", "a.she.dah"),
        "H795": ("אַשְׁדּוֹד", "N:N--L", "Ashdod", "ash.dod"),
        "H796": ("אַשְׁדּוֹדִי", "N:N--LG", "Ashdodite", "ash.do.di"),
        "H797": ("אַשְׁדּוֹדִית", "H:Adv", "language of Ashdod", "ash.do.dit"),
        "H798": ("אַשְׁדּוֹת הַפִּסְגָּה", "N:N--L", "Ashdoth-pisgah", "ash.dot hap.pis.gah"),
        "H799": ("אֶשְׁדָּת", "H:N-F", "fiery", "esh.dat"),
        "H800": ("אֶשָּׁה", "H:N-F", "fire", "esh.shah"),
        "H801": ("אִשֶּׁה", "H:N-M", "food offering", "ish.shah"),
        "H802": ("אִשָּׁה", "H:N-F", "woman", "ish.shah"),
        "H803": ("אָשְׁיָה", "H:N-F", "foundation", "a.shu.yah"),
        "H804": ("אַשּׁוּר", "N:N--L / N:N--LG / N:N-M-P", "Asshur", "ash.shur"),
        "H805a": ("אַשּׁוּרִם", "N:N-M-PG", "Asshurim", "a.shu.ri"),
        "H805b": ("אֲשׁוּרִי", "N:N--LG", "Ashurite", "a.shu.ri"),
        "H806": ("אַשְׁחוּר", "N:N-M-P", "Ashhur", "ash.chur"),
        "H807": ("אֲשִׁימָא", "N:N-M-T", "Ashima", "a.shi.ma"),
        "H808": ("אָשִׁישׁ", "H:N-F", "foundation", "a.shish"),
        "H809": ("אֲשִׁישָׁה", "H:N-F", "raisin bun", "a.shi.shah"),
        "H810": ("אֶ֫שֶׁךְ", "H:N-M", "testicle", "e.shekh"),
        "H811": ("אֶשְׁכּוֹל", "H:N-M", "cluster", "esh.kol"),
        "H812": ("אֶשְׁכֹּל", "N:N--L / N:N-M-P", "Eshcol", "esh.kol"),
        "H813": ("אַשְׁכְּנַז", "N:N-M-P / N:N--L", "Ashkenaz", "ash.ke.naz"),
        "H814": ("אֶשְׁכָּר", "H:N-M", "gift", "esh.kar"),
        "H815": ("אֵ֫שֶׁל", "H:N-M", "tamarisk(s)", "e.shel"),
        "H816": ("אָשַׁם", "H:V", "be guilty", "a.sham"),
        "H817": ("אָשָׁם", "H:N-M", "guilt (offering)", "a.sham"),
        "H818": ("אָשֵׁם", "H:A", "guilty", "a.shem"),
        "H819": ("אַשְׁמָה", "H:N-F", "guiltiness", "ash.mah"),
        "H820": ("אַשְׁמַנִּים", "H:N-M", "strong", "ash.man"),
        "H821": ("אַשְׁמוּרָה", "H:N-F", "watch", "ash.mu.rah"),
        "H822": ("אֶשְׁנָב", "H:N-M", "lattice", "esh.nav"),
        "H823": ("אַשְׁנָה", "N:N--L", "Ashnah", "ash.nah"),
        "H824": ("אֶשְׁעָ֑ן", "N:N--L", "Eshan", "esh.an"),
        "H825": ("אַשָּׁף", "H:N-M", "enchanter", "ash.shaph"),
        "H826": ("אָשַׁף", "A:N-M", "enchanter", "ash.shaph"),
        "H827": ("אַשְׁפָּה", "H:N-F", "quiver", "ash.pah"),
        "H828": ("אַשְׁפְּנַז", "N:N-M-P", "Ashpenaz", "ash.pe.naz"),
        "H829": ("אֶשְׁפָּר", "H:N-M", "raisin bun", "esh.par"),
        "H830": ("אַשְׁפֹּת", "H:N-M", "refuse", "ash.pot"),
        "H831": ("אַשְׁקְלוֹן", "N:N--L", "Ashkelon", "ash.qe.lon"),
        "H832": ("אֶשְׁקְלוֹנִי", "N:N--LG", "Ashkelonite", "esh.qe.lo.ni"),
        "H833": ("אָשַׁר", "H:V", "to bless", "a.shar"),
        "H834a": ("אֲשֶׁר", "H:RelP", "which", "a.sher"),
        "H834b": ("בַאֲשֶׁר", "H:Prep+H:RelP", "in which", ""),
        "H834c": ("מֵאֲשֶׁר", "H:Prep+H:RelP", "from which", ""),
        "H834d": ("כַּאֲשֶׁר", "H:Prep+H:RelP", "as which", ""),
        "H835": ("אֶ֫שֶׁר", "H:N-M", "blessed", "e.sher"),
        "H836": ("אָשֵׁר", "N:N--L / N:N--LG / N:N-M-P", "Asher", "a.sher"),
        "H837": ("אֹ֫שֶׁר", "H:N-M", "happy", "o.sher"),
        "H838a": ("אֲשׁוּר", "H:N-F", "step", "a.shur"),
        "H838b": ("אַשֻּׁר", "H:N-F", "step", "a.shur"),
        "H839": ("אֲשֻׁרִים", "H:N-F", "boxwood", "a.shur"),
        "H840": ("אֲשַׂרְאֵל", "N:N-M-P", "Asarel", "a.sar.el"),
        "H841": ("אֲשַׂרְאֵ֫לָה", "N:N-M-P", "Asharelah", "a.sar.e.lah"),
        "H842": ("אֲשֵׁרָה", "N:N-F-T", "Asherah", "a.she.rah"),
        "H843": ("אָשֵׁרִי", "N:N--PG", "Asherite", "a.she.ri"),
        "H844": ("אַשְׂרִיאֵל", "N:N-M-P", "Asriel", "as.ri.el"),
        "H845": ("אַשְׂרִיאֵלִי", "N:N--PG", "Asrielite", "as.ri.e.li"),
        "H846": ("אֻשַּׁרְנָא", "A:N-M", "wall", "ush.shar.na"),
        "H847": ("אֶשְׁתָּאוֹל", "N:N--L", "Eshtaol", "esh.ta.ol"),
        "H848": ("אֶשְׁתָּאֻלִי", "N:N--LG", "Eshtaolite", "esh.ta.u.li"),
        "H849": ("אֶשְׁתַּדּוּר", "A:N-M", "sedition", "esh.tad.dur"),
        "H850": ("אֶשְׁתּוֹן", "N:N-M-P", "Eshton", "esh.ton"),
        "H851": ("אֶשְׁתְּמוֹעַ", "N:N--L / N:N-M-P", "Eshtemoa", "esh.te.mo.a"),
        "H852": ("אָת", "A:N-F", "sign", "at"),
        "H853": ("אֵת", "H:Part", "[Obj.]", "et"),
        "H854": ("אֵת", "H:Prep", "with", "et"),
        "H855": ("אֵת", "H:N-M", "plowshare", "et"),
        "H856": ("אֶתְבַּ֫עַל", "N:N-M-P", "Ethbaal", "et.ba.al"),
        "H857": ("אָתָה", "H:V", "to come", "a.tah"),
        "H858": ("אֲתָה", "A:V", "to come", "a.tah"),
        "H859a": ("אַתָּ֫ה", "H:PerP-MS", "you(m.s.)", "at.tah"),
        "H859b": ("אתִּי", "H:PerP-FS", "you(f.s.)", ""),
        "H859c": ("אַתְּ", "H:PerP-FS", "you(f.s.)", ""),
        "H859d": ("אַתֶּם", "H:PerP-MP", "you(m.p.)", ""),
        "H859e": ("אַתֵּן", "H:PerP-FP", "you(f.p.)", ""),
        "H860": ("אָתוֹן", "H:N-F", "she-ass", "a.ton"),
        "H861": ("אַתּוּן", "A:N-M", "furnace", "at.tun"),
        "H862a": ("אַתּוּק", "H:N-M", "gallery", "at.tuq"),
        "H862b": ("אַתִּיק", "H:N-M", "gallery", ""),
        "H863": ("אִתַּי", "N:N-M-P", "Ittai", "it.tay"),
        "H864": ("אֵתָם", "N:N--L", "Etham", "e.tam"),
        "H865": ("אֶתְמוֹל", "H:Adv", "previously", "et.mol"),
        "H866": ("אֶתְנָה", "H:N-F", "wages", "et.nah"),
        "H867": ("אֶתְנִי", "N:N-M-P", "Ethni", "et.ni"),
        "H868": ("אֶתְנַן", "H:N-M", "wages", "et.nan"),
        "H869": ("אֶתְנַן", "N:N-M-P", "Ethnan", "et.nan"),
        "H870": ("אֲתַר", "A:N-M", "place", "a.tar"),
        "H871": ("אֲתָרִים", "N:N--L", "Atharim", "a.ta.rim"),
        "H872": ("בִּאָה", "H:N-F", "entrance", "be.ah"),
        "H873": ("בִּאישׁ", "A:N-F", "bad", "bi.ush"),
        "H874": ("בָּאַר", "H:V", "to make plain", "ba.ar"),
        "H875": ("בְּאֵר", "H:N-F", "well", "be.er"),
        "H876": ("בְּאֵר", "N:N--L", "Beer", "be.er"),
        "H877": ("בֹּאר", "H:N-M", "cistern", "bor"),
        "H878": ("בְּאֵרָא", "N:N-M-P", "Beera", "be.e.ra"),
        "H879": ("בְּאֵר אֵילִים", "N:N--L", "Beer-elim", "be.er e.lim"),
        "H880": ("בְּאֵרָה", "N:N-M-P", "Beerah", "be.e.rah"),
        "H881": ("בְּאֵרוֹת", "N:N--L", "Beeroth", "be.e.rot"),
        "H882": ("בְּאֵרִי", "N:N-M-P", "Beeri", "be.e.ri"),
        "H883": ("בְּאֵר לַחַי רֹאִי", "N:N--L", "Beer-lahai-roi", "be.er la.chay ro.i"),
        "H884": ("בְּאֵר שֶׁ֫בַע", "N:N--L", "Beer-sheba", "be.er she.va"),
        "H885": ("בְּאֵרֹת בְּנֵי־יַעֲקָן", "N:N--L", "Beeroth-bene-jaakan", "be.e.rot be.ne-ya.a.qan"),
        "H886": ("בְּאֵרֹתִי", "N:N--LG", "Beerothite", "be.e.ro.ti"),
        "H887": ("בָּאַשׁ", "H:V", "to stink", "ba.ash"),
        "H888": ("בְּאֵשׁ", "A:V", "be displeased", "be.esh"),
        "H889": ("בְּאֹשׁ", "H:N-M", "stench", "be.osh"),
        "H890": ("בׇּאְשָׁה", "H:N-F", "foul weed", "bo.shah"),
        "H891": ("בְּאֻשִׁים", "H:N-M", "sour grapes", "be.u.shim"),
        "H892": ("בָּבָה", "H:N-F", "apple", "ba.vah"),
        "H893": ("בֵּבַי", "N:N-M-P", "Bebai", "be.vay"),
        "H894": ("בָּבֶ֫ל", "N:N--L", "Babel", "ba.vel"),
        "H895": ("בָּבֶל", "N:N--L", "Babylon", "ba.vel"),
        "H896": ("בָּֽבְלַי", "N:N--LG", "Babylonian", "bav.li"),
        "H897": ("בַּג", "H:N-M", "spoil", "bag"),
        "H898": ("בָּגַד", "H:V", "to act treacherously", "ba.gad"),
        "H899a": ("בֶּ֫גֶד", "H:N-M", "treachery", "be.ged"),
        "H899b": ("בֶּ֫גֶד", "H:N-M", "garment", "be.ged"),
        "H900": ("בֹּגְדוֹת", "H:N-F", "treachery", "bo.ge.dot"),
        "H901": ("בָּגוֹד", "H:A", "treacherous", "ba.god"),
        "H902": ("בִּגְוַי", "N:N-M-P", "Bigvai", "big.vay"),
        "H903": ("בִּגְתָא", "N:N-M-P", "Bigtha", "big.ta"),
        "H904": ("בִּגְתָן", "N:N-M-P", "Bigthan", "big.tan"),
        "H905": ("בַּד", "H:N-M", "alone", "bad"),
        "H906": ("בַּד", "H:N-M", "linen", "bad"),
        "H907": ("בַּד", "H:N-M", "bluster", "bad"),
        "H908": ("בָּדָא", "H:V", "to devise", "ba.da"),
        "H909": ("בָּדַד", "H:V", "be alone", "ba.dad"),
        "H910": ("בָּדָד", "H:N-M", "isolation", "ba.dad"),
        "H911": ("בְּדַד", "N:N-M-P", "Bedad", "be.dad"),
        "H912": ("בֵּֽדְיָה", "N:N-M-P", "Bedeiah", "be.de.yah"),
        "H913": ("בְּדִיל", "H:N-M", "tin", "be.dil"),
        "H914": ("בָּדַל", "H:V", "to separate", "ba.dal"),
        "H915": ("בָּדָל", "H:N-M", "piece", "ba.dal"),
        "H916": ("בְּדֹ֫לַח", "H:N-M", "bdellium", "be.do.lach"),
        "H917": ("בְּדָן", "N:N-M-P", "Bedan", "be.dan"),
        "H918": ("בָּדַק", "H:V", "to repair", "ba.daq"),
        "H919": ("בֶּ֫דֶק", "H:N-M", "breach", "be.deq"),
        "H920": ("בִּדְקַר", "N:N-M-P", "Bidkar", "bid.qar"),
        "H921": ("בְּדַר", "A:V", "to scatter", "be.dar"),
        "H922": ("בֹּהוּ", "H:N-M", "void", "bo.hu"),
        "H923": ("בַּ֫הַט", "H:N-M", "porphyry", "ba.hat"),
        "H924": ("בְּהִילוּ", "A:Adv", "hastely", "be.hi.lu"),
        "H925": ("בָּהִיר", "H:A", "bright", "ba.hir"),
        "H926": ("בָּהַל", "H:V", "to dismay", "ba.hal"),
        "H927": ("בְּהַל", "A:V", "to dismay", "be.hal"),
        "H928": ("בֶּֽהָלָה", "H:N-F", "dismay", "be.ha.lah"),
        "H929": ("בְּהֵמָה", "H:N-F", "animal", "be.he.mah"),
        "H930": ("בְּהֵמוֹת", "H:N-M", "Behemoth", "be.he.mot"),
        "H931": ("בֹּ֫הֶן", "H:N-F", "thumb/big toe", "bo.hen"),
        "H932": ("בֹּ֫הַן", "N:N-M-P", "Bohan", "bo.han"),
        "H933": ("בֹּ֫הַק", "H:N-M", "spot", "bo.haq"),
        "H934": ("בַּהֶ֫רֶת", "H:N-F", "bright spot", "bo.he.ret"),
        "H935": ("בּוֹא", "H:V", "to come (in)", "bo"),
        "H936": ("בּוּז", "H:V", "to despise", "buz"),
        "H937": ("בּוּז", "H:N-M", "contempt", "buz"),
        "H938": ("בּוּז", "N:N-M-P / N:N--L", "Buz", "buz"),
        "H939": ("בּוּזָה", "H:N-F", "contempt", "bu.zah"),
        "H940": ("בּוּזִי", "N:N--PG", "Buzite", "bu.zi"),
        "H941": ("בּוּזִי", "N:N-M-P", "Buzi", "bu.zi"),
        "H942": ("בַּוַּי", "N:N-M-P", "Bavvai", "bav.vay"),
        "H943": ("בּוּךְ", "H:V", "to perplex", "bukh"),
        "H944": ("בּוּל", "H:N-M", "produce", "bul"),
        "H945": ("בּוּל", "N:N-M-T", "Bul", "bul"),
        "H946": ("בּוּנָה", "N:N-M-P", "Bunah", "bu.nah"),
        "H947": ("בּוּס", "H:V", "to trample", "bus"),
        "H948": ("בּוּץ", "H:N-M", "fine linen", "buts"),
        "H949": ("בּוֹצֵץ", "N:N--L", "Bozez", "bo.tsets"),
        "H950": ("בּוּקָה", "H:N-F", "emptiness", "bu.qah"),
        "H951": ("בּוֹקֵר", "H:N-M", "herdsman", "bo.qer"),
        "H952": ("בּוּר", "H:V", "to explain", "bur"),
        "H953a": ("בּוֹר", "H:N-M", "pit", "bor"),
        "H953b": ("בּוֹר", "H:N-M", "well", ""),
        "H954": ("בּוֹשׁ", "H:V", "be ashamed", "bush"),
        "H955": ("בּוּשָׁה", "H:N-F", "shame", "bu.shah"),
        "H956": ("בִּית", "A:V", "to pass the night", "but"),
        "H957": ("בַּז", "H:N-M", "plunder", "baz"),
        "H958": ("בָּזָא", "H:V", "to divide", "ba.za"),
        "H959": ("בָּזָה", "H:V", "to despise", "ba.zah"),
        "H960": ("בָּזֹה", "H:V", "to despise", "ba.zoh"),
        "H961": ("בִּזָּה", "H:N-F", "plunder", "biz.zah"),
        "H962": ("בָּזַז", "H:V", "to plunder", "ba.zaz"),
        "H963": ("בִּזָּיוֹן", "H:N-M", "contempt", "biz.za.von"),
        "H964": ("בִּזְיוֹתְיָה", "N:N--L", "Biziothiah", "biz.yo.te.yah"),
        "H965": ("בָּזָק", "H:N-M", "lightning", "ba.zaq"),
        "H966": ("בֶּ֫זֶק", "N:N--L", "Bezek", "be.zeq"),
        "H967": ("בָּזַר", "H:V", "to scatter", "ba.zar"),
        "H968": ("בִּזְּתָא", "N:N-M-P", "Biztha", "biz.ta"),
        "H969": ("בָּחוֹן", "H:N-M", "assayer", "ba.chon"),
        "H970": ("בָּחוּר", "H:N-M", "youth", "ba.chur"),
        "H971": ("בַחוּן", "H:N-M", "tower", "ba.chin"),
        "H972": ("בָּחִיר", "H:N-M", "chosen", "ba.chir"),
        "H973a": ("בָּחֵל", "H:V", "to loath", "ba.chal"),
        "H973b": ("בָּחֵל", "H:V", "be greedy", "ba.chal"),
        "H974": ("בָּחַן", "H:V", "to test", "ba.chan"),
        "H975": ("בַּ֫חַן", "H:N-M", "watchtower", "ba.chan"),
        "H976": ("בֹּ֫חַן", "H:N-M", "testing", "bo.chan"),
        "H977": ("בָּחַר", "H:V", "to choose", "ba.char"),
        "H978": ("בַּחֲרוּמִי", "N:N--LG", "Baharumite", "ba.cha.ru.mi"),
        "H979": ("בְּחוּרִים", "H:N-F", "youth", "be.chu.rot"),
        "H980": ("בַּחֻרִים", "N:N--L", "Bahurim", "ba.chu.rim"),
        "H981": ("בָּטָה", "H:V", "to speak rashly", "ba.ta"),
        "H982": ("בָּטַח", "H:V", "to trust", "ba.tach"),
        "H983": ("בֶּ֫טַח", "H:N-M", "security", "be.tach"),
        "H984": ("בֶּ֫טַח", "N:N--L", "Betah", "be.tach"),
        "H985": ("בִּטְחָה", "H:N-F", "trust", "bit.chah"),
        "H986": ("בִּטָּחוֹן", "H:N-M", "trust", "bit.ta.chon"),
        "H987": ("בַּטֻּחוֹת", "H:N-F", "security", "bat.tu.chot"),
        "H988": ("בָּטַל", "H:V", "to cease", "ba.tel"),
        "H989": ("בְּטֵל", "A:V", "to cease", "be.tel"),
        "H990": ("בֶּ֫טֶן", "H:N-F", "belly", "be.ten"),
        "H991": ("בֶּ֫טֶן", "N:N--L", "Beten", "be.ten"),
        "H992": ("בָּטְנִים", "H:N-M", "pistachio", "bo.ten"),
        "H993": ("בְּטֹנִים", "N:N--L", "Betonim", "be.to.nim"),
        "H994": ("בִּי", "H:Part", "please", "bi"),
        "H995": ("בִּין", "H:V", "to understand", "bin"),
        "H996": ("בַּ֫יִן", "H:Prep", "between", "ben"),
        "H997": ("בֵּין", "A:Prep", "between", "ben"),
        "H998": ("בִּינָה", "H:N-F", "understanding", "bi.nah"),
        "H999": ("בִּינָה", "A:N-F", "understanding", "bi.nah"),
        "H1000": ("בֵּיצָה", "H:N-F", "egg", "be.tsah"),
        "H1001": ("בִּירְתָא", "A:N-F", "citadel", "bi.ra"),
        "H1002": ("בִּירָה", "H:N-F", "palace", "bi.rah"),
        "H1003": ("בִּירָנִית", "H:N-F", "fortress", "bi.ra.nit"),
        "H1004": ("בַּ֫יִת", "H:N-M", "house", "ba.yit"),
        "H1004a": ("בֵּית", "H:Prep", "between", "ba.yit"),
        "H1004b": ("בַּ֫יִת", "H:N-M", "house", "ba.yit"),
        "H1005": ("בַּ֫יִת", "A:N-M", "house", "ba.yit"),
        "H1006": ("בַּ֫יִת", "N:N--L", "Bajith", "ba.yit"),
        "H1007": ("בֵּית אָ֫וֶן", "N:N--L", "Beth-aven", "bet a.ven"),
        "H1008": ("בֵּֽיתְ־אֵל", "N:N--L", "Beth-el", "bet-el"),
        "H1009": ("בֵּית אַֽרְבֵֿאל", "N:N--L", "Beth-arbel", "bet ar.bel"),
        "H1010": ("בֵּית בַּ֫עַל מְעוֹן", "N:N--L", "Beth-baal-meon", "bet ba.al me.on"),
        "H1011": ("בֵּית בִּרְאִי", "N:N--L", "Beth-biri", "bet bir.i"),
        "H1012": ("בֵּית בָּרָה", "N:N--L", "Beth-barah", "bet ba.rah"),
        "H1013": ("בֵּית־גָּדֵר", "N:N--L", "Beth-gader", "bet-ga.der"),
        "H1014": ("בֵּית גָּמוּל", "N:N--L", "Beth-gamul", "bet ga.mul"),
        "H1015": ("בֵּית דִּבְלָתָ֑יִם", "N:N--L", "Beth-diblathaim", "bet div.la.ta.yim"),
        "H1016": ("בֵּית־דָּגוֹן", "N:N--L", "Beth-dagon", "bet-da.gon"),
        "H1017": ("בֵּית הָאֱלִי", "N:N--LG", "Beth-elite", "bet ha.e.li"),
        "H1018": ("בֵּית הָאֵצֶל", "N:N--L", "Beth-ezel", "bet ha.e.tsel"),
        "H1019": ("בֵּית הַגִּלְגָּל", "N:N--L", "Beth-gilgal", "bet hag.gil.gal"),
        "H1020": ("בֵּית הַיְשִׁימוֹת", "N:N--L", "Beth-jeshimoth", "bet hay.shi.mot"),
        "H1021": ("בֵּית הַכֶּ֫רֶם", "N:N--L", "Beth-haccerem", "bet hak.ke.rem"),
        "H1022": ("בֵּית הַלַּחְמִי", "N:N--LG", "Bethlehemite", "bet hal.lach.mi"),
        "H1023": ("בֵּית הַמֶּרְחָק", "N:N--L", "Beth-merchak", "bet ham.mer.chaq"),
        "H1024": ("בֵּית מַרְכָּבוֹת", "N:N--L", "Beth-marcaboth", "bet ham.mar.ka.vot"),
        "H1025": ("בֵּית הָעֵ֫מֶק", "N:N--L", "Beth-emek", "bet ha.e.meq"),
        "H1026": ("בֵּית הָֽעֲרָבָה", "N:N--L", "Beth-arabah", "bet ha.a.ra.vah"),
        "H1027": ("בֵּית הָרָם", "N:N--L", "Beth-haram", "bet ha.ram"),
        "H1028": ("בֵּית הָרָן", "N:N--L", "Beth-haran", "bet ha.ran"),
        "H1029": ("בֵּית הַשִּׁטָּה", "N:N--L", "Beth-shittah", "bet hash.shit.tah"),
        "H1030": ("בֵּית־הַשִּׁמְשִׁי", "N:N--LG", "Beth-shemite", "bet hash.shim.shi"),
        "H1031": ("בֵּית חׇגְלָה", "N:N--L", "Beth-hoglah", "bet chog.lah"),
        "H1032": ("בֵּית חוֹרֹן", "N:N--L", "Beth-horon", "bet cho.ron"),
        "H1033": ("בֵּית כָּ֑ר", "N:N--L", "Beth-car", "bet kar"),
        "H1034": ("בֵּית לְבָאוֹת", "N:N--L", "Beth-lebaoth", "bet le.va.ot"),
        "H1035": ("בֵּית לֶ֫חֶם", "N:N--L", "Beth-lehem", "bet le.chem"),
        "H1036": ("בֵּית לְעַפְרָה", "N:N--L", "Beth-le-aphrah", "bet le.aph.rah"),
        "H1037": ("בֵּית מִלּוֹא", "N:N--L", "Beth-millo", "bet mil.lo"),
        "H1038": ("בֵּית מַעֲכָה", "N:N--L", "Beth-maacah", "bet ma.a.khah"),
        "H1039": ("בֵּית נִמְרָה", "N:N--L", "Beth-nimrah", "bet nim.rah"),
        "H1040": ("בֵּית עֶ֑דֶן", "N:N--L", "Beth-eden", "bet e.den"),
        "H1041": ("בֵּית־עַזְמָ֫וֶת", "N:N--L", "Beth-azmaveth", "bet az.ma.vet"),
        "H1042": ("בֵּית־עֲנוֹת", "N:N--L", "Beth-anoth", "bet a.not"),
        "H1043": ("בֵּית־עֲנָת", "N:N--L", "Beth-anath", "bet a.nat"),
        "H1044": ("בֵּית־עֵ֫קֶד", "N:N--L", "Beth-eked", "bet e.qed"),
        "H1045": ("בֵּית עַשְׁתָּרוֹת", "N:N--L", "Beth-ashtaroth", "bet ash.ta.rot"),
        "H1046": ("בֵּית פָּ֑לֶט", "N:N--L", "Beth-pelet", "bet pe.let"),
        "H1047": ("בֵּית פְּעוֹר", "N:N--L", "Beth-peor", "bet pe.or"),
        "H1048": ("בֵּית פַּצֵּץ", "N:N--L", "Beth-pazzez", "bet pats.tsets"),
        "H1049": ("בֵּית־צוּר", "N:N--L / N:N-M-P", "Beth-zur", "bet tsur"),
        "H1050": ("בֵּית־רְחוֹב", "N:N--L", "Beth-rehob", "bet re.chov"),
        "H1051": ("בֵּית רָפָא", "N:N-M-P", "Beth-rapha", "bet ra.pah"),
        "H1052": ("בֵּית שְׁאָן", "N:N--L", "Beth-shean", "bet she.an"),
        "H1053": ("בֵּית שֶׁ֫מֶשׁ", "N:N--L", "Beth-shemesh", "bet she.mesh"),
        "H1054": ("בֵּית־תַּפּוּחַ", "N:N--L", "Beth-tappuah", "bet tap.pu.ach"),
        "H1055": ("בִּיתָן", "H:N-M", "palace", "bi.tan"),
        "H1056": ("בָּכָא", "N:N--L", "Baca", "ba.kah"),
        "H1057": ("בָּכָא", "H:N-M", "balsam", "ba.kah"),
        "H1058": ("בָּכָה", "H:V", "to weep", "ba.khah"),
        "H1059": ("בֶּ֫כֶה", "H:N-M", "bitterly", "be.kheh"),
        "H1060": ("בְּכוֹר", "H:N-M", "firstborn", "be.khor"),
        "H1061": ("בִּכּוּרִים", "H:N-M", "firstfruit", "bik.kur"),
        "H1062": ("בְּכֹרָה", "H:N-F", "birthright", "be.kho.rah"),
        "H1063": ("בַּכֻּרוֹת", "H:N-F", "early fig", "bik.ku.rah"),
        "H1064": ("בְּכוֹרַת", "N:N-M-P", "Becorath", "be.kho.rat"),
        "H1065": ("בְּכִי", "H:N-M", "weeping", "be.khi"),
        "H1066": ("בֹּכִים", "N:N--L", "Bochim", "bo.khim"),
        "H1067": ("בְּכִירָה", "H:N-F", "firstborn", "be.khi.rah"),
        "H1068": ("בְּכִית", "H:N-F", "weeping", "be.khit"),
        "H1069": ("בָּכַר", "H:V", "to be/bear firstborn", "ba.khar"),
        "H1070": ("בֶּכֶר", "H:N-M", "young camel", "be.kher"),
        "H1071": ("בֶּ֫כֶר", "N:N-M-P", "Becher", "be.kher"),
        "H1072": ("בִּכְרָה", "H:N-F", "young camel", "bikh.rah"),
        "H1073": ("בִּכּוּרָה", "H:N-F", "firstfruit", "bak.ku.rah"),
        "H1074": ("בֹּֽכְרוּ", "N:N-M-P", "Bocheru", "bo.khe.ru"),
        "H1075": ("בִּכְרִי", "N:N-M-P / N:A", "Bichri", "bikh.ri"),
        "H1076": ("בַּכְרִי", "N:N--PG", "Becherite", "bakh.ri"),
        "H1077": ("בַּל", "H:Adv", "not", "bal"),
        "H1078": ("בֵּל", "N:N-M-T", "Bel", "bel"),
        "H1079": ("בָּל", "A:N-M", "mind", "bal"),
        "H1080": ("בְּלָא", "A:V", "to wear out", "be.la"),
        "H1081": ("בַּלְאֲדָן", "N:N-M-P", "Baladan", "bal.a.dan"),
        "H1082": ("בָּלַג", "H:V", "be cheerful", "ba.lag"),
        "H1083": ("בִּלְגָּה", "N:N-M-P", "Bilgah", "bil.gah"),
        "H1084": ("בִּלְגַּי", "N:N-M-P", "Bilgai", "bil.gay"),
        "H1085": ("בִּלְדַּד", "N:N-M-P", "Bildad", "bil.dad"),
        "H1086": ("בָּלָה", "H:V", "to become old", "ba.lah"),
        "H1087": ("בָּלֶה", "H:A", "old", "ba.leh"),
        "H1088": ("בָּלָה", "N:N--L", "Balah", "ba.lah"),
        "H1089": ("בָּלַהּ", "H:V", "to trouble", "ba.lah"),
        "H1090a": ("בִּלְהָה", "N:N-F-P", "Bilhah", "bil.hah"),
        "H1090b": ("בִּלְהָה", "N:N--L", "Bilhah", "bil.hah"),
        "H1091": ("בַּלָּהָה", "H:N-F", "terror", "bal.la.hah"),
        "H1092": ("בִּלְהָן", "N:N-M-P", "Bilhan", "bil.han"),
        "H1093": ("בְּלוֹ", "A:N-M", "tribute", "be.lo"),
        "H1094": ("בְּלוֹא", "H:N-M", "old", "be.lo"),
        "H1095": ("בֵּלְטְשַׁאצַּר", "N:N-M-P", "Belteshazzar", "be.le.te.shats.tsar"),
        "H1096": ("בֵּלְטְשַׁאצַּר", "N:N-M-P", "Belteshazzar", "be.le.te.shats.tsar"),
        "H1097": ("בְּלִי", "H:Neg", "without", "be.li"),
        "H1098": ("בְּלִיל", "H:N-M", "fodder", "be.lil"),
        "H1099": ("בְּלִימָה", "H:N-M", "nothing", "be.li.mah"),
        "H1100": ("בְּלִיַּ֫עַל", "H:N-M", "Belial", "be.liy.ya.al"),
        "H1101a": ("בָּלַל", "H:V", "to mix", "ba.lal"),
        "H1101b": ("בָּלַל", "H:V", "to feed", "ba.lal"),
        "H1102": ("בָּלַם", "H:V", "to hold in", "ba.lam"),
        "H1103": ("בָּלַס", "H:V", "to gather/tend figs", "ba.las"),
        "H1104": ("בָּלַע", "H:V", "to swallow up", "ba.la"),
        "H1105": ("בֶ֫לַע", "H:N-M", "swallowing", "be.la"),
        "H1106a": ("בֶּ֫לַע", "N:N-M-P", "Bela", "be.la"),
        "H1106b": ("בֶּ֫לַע", "N:N--L", "Bela", "be.la"),
        "H1107": ("בִּלְעֲדֵי", "H:Part", "beside", "bil.a.de"),
        "H1108": ("בַּלְעִי", "N:N--PG", "Belaite", "bal.i"),
        "H1109a": ("בִּלְעָם", "N:N-M-P", "Balaam", "bil.am"),
        "H1109b": ("בִּלְעָם", "N:N--L", "Bileam", "bil.am"),
        "H1110": ("בָּלַק", "H:V", "to waste", "ba.laq"),
        "H1111": ("בָּלָק", "N:N-M-P", "Balak", "ba.laq"),
        "H1112": ("בֵּֽלְאשַׁצַּר, בֵּלְשַׁאצַּר", "N:N-M-P", "Belshazzar", "be.le.shats.tsar"),
        "H1113": ("בֵּלְשַׁאצַּר", "N:N-M-P", "Belshazzar", "be.le.shats.tsar"),
        "H1114": ("בִּלְשָׁן", "N:N-M-P", "Bilshan", "bil.shan"),
        "H1115": ("בִּלְתִּי", "H:Neg", "lest", "bil.ti"),
        "H1116": ("בָּמָה", "H:N-F", "high place", "ba.mah"),
        "H1117": ("בָּמָה", "N:N--L", "Bamah", "ba.mah"),
        "H1118": ("בִּמְהָל", "N:N-M-P", "Bimhal", "bim.hal"),
        "H1119": ("בְּמוֹ", "H:Prep", "in/at/by", "be.mo"),
        "H1120": ("בָּמוֹת", "N:N--L", "Bamoth", "ba.mot"),
        "H1121a": ("בֵּן", "H:N-M", "son", "ben"),
        "H1121b": ("בְּנוֹ", "N:N-M-P", "Beno", ""),
        "H1122": ("בֵּן", "N:N-M-P", "Ben", "ben"),
        "H1123": ("בֵּן", "A:N-M", "son", "ben"),
        "H1124": ("בְּנָה", "A:V", "to build", "be.na"),
        "H1125": ("בֶּן־אֲבִינָדָב", "N:N-M-P", "Ben-abinadab", "ben-a.vi.na.dav"),
        "H1126": ("בֶּן־אוֹנִ֑י", "N:N-M-P", "Ben-oni", "ben-o.ni"),
        "H1127": ("בֶּן־גֶּ֫בֶר", "N:N-M-P", "Ben-geber", "ben-ge.ver"),
        "H1128": ("בֶּן־דֶּ֫קֶר", "N:N-M-P", "Ben-deker", "ben-de.qer"),
        "H1129": ("בָּנָה", "H:V", "to build", "ba.nah"),
        "H1130": ("בֶּן־הֲדַד", "N:N-M-P", "Ben-hadad", "ben-ha.dad"),
        "H1131": ("בִּנּוּי", "N:N-M-P", "Binnui", "bin.nuy"),
        "H1132": ("בֶּן־זוֹחֵת", "N:N-M-P", "Ben-zoheth", "ben-zo.chet"),
        "H1133": ("בֶּן־חוּר", "N:N-M-P", "Ben-hur", "ben-chur"),
        "H1134": ("בֶּן־חַ֫יִל", "N:N-M-P", "Ben-hail", "ben-cha.yil"),
        "H1135": ("בֶּן־חָנָן", "N:N-M-P", "Ben-hanan", "ben-cha.nan"),
        "H1136": ("בֶּן־חֶ֫סֶד", "N:N-M-P", "Ben-hesed", "ben-che.sed"),
        "H1137": ("בָּנִי", "N:N-M-P", "Bani", "ba.ni"),
        "H1138": ("בֻּנִּי", "N:N-M-P", "Bunni", "bun.ni"),
        "H1139": ("בְּנֵי־בְרַק", "N:N--L", "Bene-berak", "be.ne-b.raq"),
        "H1140": ("בִּנְיָה", "H:N-F", "building", "bin.yah"),
        "H1141": ("בְּנָיָ֫הוּ", "N:N-M-P", "Benaiah", "be.na.yah"),
        "H1142": ("בְּנֵי יַעֲקָ֑ן", "N:N--L", "Bene-jaakan", "be.ne ya.a.qan"),
        "H1143": ("בֵּנַיִם", "H:N-M", "champion", "be.na.yim"),
        "H1144": ("בִּנְיָמִין", "N:N-M-P", "Benjamin", "bin.ya.min"),
        "H1145": ("בֶּן־יְמִינִי", "N:N--PG", "Benjamite", "ben-y.mi.ni"),
        "H1146": ("בִּנְיָן", "H:N-M", "building", "bin.yan"),
        "H1147": ("בִּנְיָן", "A:N-M", "building", "bin.yan"),
        "H1148": ("בְּנִ֑ינוּ", "N:N-M-P", "Beninu", "be.ni.nu"),
        "H1149": ("בְּנַס", "A:V", "be angry", "be.nas"),
        "H1150": ("בִּנְעָא", "N:N-M-P", "Binea", "bin.a"),
        "H1151": ("בֶּן־עַמִּי", "N:N-M-P", "Ben-ammi", "ben-am.mi"),
        "H1152": ("בְּסוֹדְיָ֑ה", "N:N-M-P", "Besodeiah", "be.so.de.yah"),
        "H1153": ("בֵּסַי", "N:N-M-P", "Besai", "be.say"),
        "H1154": ("בֶּסֶר", "H:N-M", "unripe grape", "be.ser"),
        "H1155": ("בֹּ֫סֶר", "H:N-M", "unripe grape", "bo.ser"),
        "H1156": ("בְּעָא", "A:V", "to ask", "be.a"),
        "H1157": ("בַּ֫עַד", "H:Prep", "about/through/for", "be.ad"),
        "H1158": ("בָּעָה", "H:V", "to enquire", "ba.ah"),
        "H1159": ("בָּעוּ", "A:N-F", "petition", "ba.u"),
        "H1160": ("בְּעוֹר", "N:N-M-P", "Beor", "be.or"),
        "H1161": ("בִּעוּתִים", "H:N-M", "terror", "bi.u.tim"),
        "H1162": ("בֹּ֫עַז", "N:N-M-P", "Boaz", "bo.az"),
        "H1163": ("בָּעַט", "H:V", "to kick", "ba.at"),
        "H1164": ("בְּעִי", "H:N-M", "ruins", "be.i"),
        "H1165": ("בְּעִיר", "H:N-M", "cattle", "be.ir"),
        "H1166": ("בָּעַל", "H:V", "to marry", "ba.al"),
        "H1167": ("בַּעַל", "H:N-M", "master", "ba.al"),
        "H1168a": ("בַּעַל", "N:N-M-T", "Baal", "ba.al"),
        "H1168b": ("בַּ֫עַל", "N:N-M-P / N:N--L", "Baal", "ba.al"),
        "H1169": ("בְּעֵל", "A:N-M", "commander", "be.el"),
        "H1170": ("בַּעַל בְּרִית", "N:N-M-T", "Baal-berith", "ba.al be.rit"),
        "H1171": ("בַּ֫עַל גָּד", "N:N--L", "Baal-gad", "ba.al gad"),
        "H1172": ("בַּעֲלָה", "H:N-F", "mistress", "ba.a.lah"),
        "H1173": ("בַּעֲלָה", "N:N--L", "Baalah", "ba.a.lah"),
        "H1174": ("בַּ֫עַל הָמוֹן", "N:N--L", "Baal-hamon", "ba.al ha.mon"),
        "H1175": ("בְּעָלוֹת", "N:N--L", "Bealoth", "be.a.lot"),
        "H1176": ("בַּעַל זְבוּב", "N:N-M-T", "Baal-zebub", "ba.al ze.vuv"),
        "H1177": ("בַּ֫עַל חָנָן", "N:N-M-P", "Baal-hanan", "ba.al cha.nan"),
        "H1178": ("בַּ֫עַל חָצוֹר", "N:N--L", "Baal-hazor", "ba.al cha.tsor"),
        "H1179": ("בַּ֫עַל חֶרְמוֹן", "N:N--L", "Baal-hermon", "ba.al cher.mon"),
        "H1180": ("בַּעֲלִי", "N:N-M-T", "Baali", "ba.a.li"),
        "H1181": ("בַּעֲלֵי בָּמוֹת", "N:N--LG", "Baal-bamoth", "ba.a.le ba.mot"),
        "H1182": ("בְּעֶלְיָדָע", "N:N-M-P", "Beeliada", "be.el.ya.da"),
        "H1183": ("בַּעַלְיָה", "N:N-M-P", "Bealiah", "be.al.yah"),
        "H1184": ("בַּעֲלֵי יְהוּדָה", "N:N--L", "Baale-judah", "ba.a.le ye.hu.dah"),
        "H1185": ("בַּעֲלִיס", "N:N-M-P", "Baalis", "ba.a.lis"),
        "H1186": ("בַּ֫עַל מְעוֹן", "N:N--L", "Baal-meon", "ba.al me.on"),
        "H1187": ("בַּ֫עַל פְּעוֹר", "N:N-M-T", "Baal-peor", "ba.al pe.or"),
        "H1188": ("בַּ֫עַל פְּרָצִים", "N:N--L", "Baal-perazim", "ba.al pe.ra.tsim"),
        "H1189": ("בַּ֫עַל צְפוֹן", "N:N--L", "Baal-zephon", "ba.al tse.phon"),
        "H1190": ("בַּ֫עַל שָׁלִשָׁה", "N:N--L", "Baal-shalishah", "ba.al sha.li.shah"),
        "H1191": ("בַּעֲלָת", "N:N--L", "Baalath", "ba.a.lat"),
        "H1192": ("בַּעֲלַת בְּאֵר", "N:N--L", "Baalath-beer", "ba.a.lat be.er"),
        "H1193": ("בַּ֫עַל תָּמָר", "N:N--L", "Baal-tamar", "ba.al ta.mar"),
        "H1194": ("בְּעֹן", "N:N--L", "Beon", "be.on"),
        "H1195": ("בַּעֲנָא", "N:N-M-P", "Baana", "ba.a.na"),
        "H1196": ("בַּעֲנָה", "N:N-M-P", "Baanah", "ba.a.nah"),
        "H1197a": ("בָּעַר", "H:V", "to burn", "ba.ar"),
        "H1197b": ("בָּעַר", "H:V", "be brutish", "ba.ar"),
        "H1198": ("בַּ֫עַר", "H:N-M", "stupid", "ba.ar"),
        "H1199": ("בַּעֲרָא", "N:N-F-P", "Baara", "ba.a.ra"),
        "H1200": ("בְּעֵרָה", "H:N-F", "fire", "be.e.rah"),
        "H1201": ("בַּעְשָׁא", "N:N-M-P", "Baasha", "ba.sah"),
        "H1202": ("בַּעֲשֵׂיָה", "N:N-M-P", "Baaseiah", "ba.a.se.yah"),
        "H1203": ("בְּעֶשְׁתְּרָה", "N:N--L", "Beeshterah", "be.esh.te.rah"),
        "H1204": ("בָּעַת", "H:V", "to terrify", "ba.at"),
        "H1205": ("בְּעָתָה", "H:N-F", "terror", "be.a.tah"),
        "H1206": ("בֹּץ", "H:N-M", "mire", "bots"),
        "H1207": ("בִּצָּה", "H:N-F", "swamp", "bits.tsah"),
        "H1208": ("בָּצוֹר", "H:N-M", "old-growth", "ba.tsor"),
        "H1209": ("בֵּצָ֑י", "N:N-M-P", "Bezai", "be.tsay"),
        "H1210": ("בָּצִיר", "H:N-M", "vintage", "ba.tsir"),
        "H1211": ("בָּצָל", "H:N-M", "onion", "be.tsel"),
        "H1212": ("בְּצַלְאֵל", "N:N-M-P", "Bezalel", "be.tsal.el"),
        "H1213": ("בַּצְלוּת", "N:N-M-P", "Bazluth", "bats.lut"),
        "H1214": ("בָּצַע", "H:V", "to cut off/covet", "ba.tsa"),
        "H1215": ("בֶּ֫צַע", "H:N-M", "unjust-gain", "be.tsa"),
        "H1216": ("בָּצֵק", "H:V", "to swell", "ba.tseq"),
        "H1217": ("בָּצֵק", "H:N-M", "dough", "ba.tseq"),
        "H1218": ("בׇּצְקַת", "N:N--L", "Bozkath", "bots.qat"),
        "H1219": ("בָּצַר", "H:V", "to gather/restrain/fortify", "ba.tsar"),
        "H1220": ("בֶּ֫צֶר", "H:N-M", "gold", "be.tser"),
        "H1221": ("בֶּ֫צֶר", "N:N-M-P / N:N--L", "Bezer", "be.tser"),
        "H1222": ("בְּצַר", "H:N-M", "gold", "be.tsar"),
        "H1223": ("בׇּצְרָה", "H:N-F", "enclosure", "bots.rah"),
        "H1224": ("בׇּצְרָה", "N:N--L", "Bozrah", "bots.rah"),
        "H1225": ("בִּצָּרוֹן", "H:N-M", "stronghold", "bits.tsa.ron"),
        "H1226": ("בַּצֹּרֶת", "H:N-F", "drought", "bats.tso.ret"),
        "H1227": ("בַּקְבּוּק", "N:N-M-P", "Bakbuk", "baq.buq"),
        "H1228": ("בַּקְבֻּק", "H:N-M", "flask", "baq.buq"),
        "H1229": ("בַּקְבֻּקְיָה", "N:N-M-P", "Bakbukiah", "baq.buq.yah"),
        "H1230": ("בַּקְבַּקַּר", "N:N-M-P", "Bakbakkar", "baq.baq.qar"),
        "H1231": ("בֻּקִּי", "N:N-M-P", "Bukki", "buq.qi"),
        "H1232": ("בֻּקִּיָּ֫הוּ", "N:N-M-P", "Bukkiah", "buq.qiy.yah"),
        "H1233": ("בְּקִיעַ", "H:N-M", "breach", "be.qi.a"),
        "H1234": ("בָּקַע", "H:V", "to break up/open", "ba.qa"),
        "H1235": ("בֶּ֫קַע", "H:N-M", "bekah", "be.qa"),
        "H1236": ("בִּקְעָא", "A:N-F", "plain", "biq.a"),
        "H1237": ("בִּקְעָה", "H:N-F", "valley", "biq.ah"),
        "H1238a": ("בָּקַק", "H:V", "be luxuriant", "ba.qaq"),
        "H1238b": ("בָּקַק", "H:V", "to empty", "ba.qaq"),
        "H1239": ("בָּקַר", "H:V", "to enquire", "ba.qar"),
        "H1240": ("בְּקַר", "A:V", "to enquire", "be.qar"),
        "H1241": ("בָּקָר", "H:N-M", "cattle", "ba.qar"),
        "H1242": ("בֹּ֫קֶר", "H:N-M", "morning", "bo.qer"),
        "H1243": ("בַּקָּרָה", "H:N-F", "seeking", "baq.qa.rah"),
        "H1244": ("בִּקֹּ֫רֶת", "H:N-F", "scourging", "biq.qo.ret"),
        "H1245": ("בָּקַשׁ", "H:V", "to seek", "ba.qash"),
        "H1246": ("בַּקָּשָׁה", "H:N-M", "request", "baq.qa.shah"),
        "H1247": ("בַּר", "A:N-M", "son", "bar"),
        "H1248": ("בַּר", "H:N-M", "son", "bar"),
        "H1249": ("בַּר", "H:A", "pure", "bar"),
        "H1250a": ("בַּר", "H:N-M", "grain", "bar"),
        "H1250b": ("בָּר", "H:N-M", "field", "bar"),
        "H1251": ("בַּר", "A:N-M", "field", "bar"),
        "H1252": ("בֹּר", "H:N-M", "cleanness", "bor"),
        "H1253": ("בֹּר", "H:N-M", "lye", "bor"),
        "H1254a": ("בָּרָא", "H:V", "to create", "ba.ra"),
        "H1254b": ("בָּרָא", "H:V", "to fatten", "ba.ra"),
        "H1255": ("בְּרֹאדַךְ בַּלְאֲדָן", "N:N-M-P", "Merodach-baladan", "be.ro.dakh bal.a.dan"),
        "H1256": ("בְּרָאיָה", "N:N-M-P", "Beraiah", "be.ra.yah"),
        "H1257": ("בַּרְבֻּרִים", "H:N-M", "fowl", "bar.bur"),
        "H1258": ("בָּרַד", "H:V", "to hail", "ba.rad"),
        "H1259": ("בָּרָד", "H:N-M", "hail", "ba.rad"),
        "H1260": ("בֶּ֫רֶד", "N:N-M-P / N:N--L", "Bered", "be.red"),
        "H1261": ("בָּרֹד", "H:A", "spotted", "ba.rod"),
        "H1262": ("בָּרָה", "H:V", "to eat", "ba.rah"),
        "H1263": ("בָּרוּךְ", "N:N-M-P", "Baruch", "ba.rukh"),
        "H1264": ("בְּרֹמִים", "H:N-M", "rich apparel", "be.rom"),
        "H1265": ("בְּרוֹשׁ", "H:N-M", "cypress", "be.rosh"),
        "H1266": ("בְּרוֹת", "H:N-M", "cypress", "be.rot"),
        "H1267": ("בָּרוּת", "H:N-F", "food", "ba.rut"),
        "H1268a": ("בֵּר֫וֹתָה", "N:N--L", "Berothah", "be.ro.tah"),
        "H1268b": ("בֵּרֹתַי", "N:N--L", "Berothai", ""),
        "H1269": ("ברזות", "N:N-M-P", "Birzavith", "bir.zot"),
        "H1270": ("בַּרְזֶל", "H:N-M", "iron", "bar.zel"),
        "H1271": ("בַּרְזִלַּי", "N:N-M-P", "Barzillai", "bar.zil.lay"),
        "H1272": ("בָּרַח", "H:V", "to flee", "ba.rach"),
        "H1273": ("בַּרְחֻמִי", "N:N--LG", "Barhumite", "bar.chu.mi"),
        "H1274": ("בִּרְיָה", "H:A", "fat", "be.ri"),
        "H1275": ("בֵּרִי", "N:N-M-P", "Beri", "be.ri"),
        "H1276": ("בֵּרִים", "N:N--LG", "Berite", "be.ri"),
        "H1277": ("בָּרִיא", "H:A", "fat", "ba.ri"),
        "H1278": ("בְּרִיאָה", "H:N-F", "creation", "be.ri.ah"),
        "H1279": ("בִּרְיָה", "H:N-F", "food", "bir.yah"),
        "H1280": ("בְּרִיחַ", "H:N-M", "bar", "be.ri.ach"),
        "H1281": ("בָּרִיחַ", "H:N-M", "fleeing", "ba.ri.ach"),
        "H1282": ("בָּרִיחַ", "N:N-M-P", "Bariah", "ba.ri.ach"),
        "H1283": ("בְּרִיעָה", "N:N-M-P", "Beriah", "be.ri.ah"),
        "H1284": ("בְּרִיעִי", "N:N--PG", "Beriite", "be.ri.i"),
        "H1285": ("בְּרִית", "H:N-F", "covenant", "be.rit"),
        "H1286": ("בְּרִית", "N:N-M-T", "Berith", "be.rit"),
        "H1287": ("בֹּרִית", "H:N-F", "lye", "bo.rit"),
        "H1288": ("בָּרַךְ", "H:V", "to bless", "ba.rakh"),
        "H1289": ("בְּרַךְ", "A:V", "to bless", "be.rakh"),
        "H1290": ("בֶּ֫רֶךְ", "H:N-F", "knee", "be.rekh"),
        "H1291": ("בְּרַךְ", "A:N-F", "knee", "be.rekh"),
        "H1292": ("בָּרַכְאֵל", "N:N-M-P", "Barachel", "ba.rakh.el"),
        "H1293": ("בְּרָכָה", "H:N-F", "blessing", "be.ra.khah"),
        "H1294": ("בְּרָכָה", "N:N-M-P / N:N--L", "Beracah", "be.ra.khah"),
        "H1295": ("בְּרֵכָה", "H:N-F", "pool", "be.re.khah"),
        "H1296": ("בֶּרֶכְיָה", "N:N-M-P", "Berechiah", "be.rekh.yah"),
        "H1297": ("בְּרַם", "A:Adv", "nevertheless", "be.ram"),
        "H1298": ("בֶּ֫רַע", "N:N-M-P", "Bera", "be.ra"),
        "H1299": ("בָּרַק", "H:V", "to flash", "ba.raq"),
        "H1300a": ("בָּרַק", "H:V", "to use lightning", "ba.raq"),
        "H1300b": ("בָּרָק", "H:N-M", "lightning", "ba.raq"),
        "H1301": ("בָּרָק", "N:N-M-P", "Barak", "ba.raq"),
        "H1302": ("בַּרְקוֹס", "N:N-M-P", "Barkos", "bar.qos"),
        "H1303": ("בַּרְקָנִים", "H:N-M", "briar", "bar.qan"),
        "H1304a": ("בָּרֶ֫קֶת", "H:N-F", "gem", "ba.re.qet"),
        "H1304b": ("בָּֽרְקַת", "H:N-F", "gem", ""),
        "H1305": ("בָּרַר", "H:V", "to purify", "ba.rar"),
        "H1306": ("בִּרְשַׁע", "N:N-M-P", "Birsha", "bir.sah"),
        "H1307": ("בֵּרֹתִי", "N:N--LG", "Berothite", "be.ro.ti"),
        "H1308": ("בְּשׂוֹר", "N:N--L", "Besor", "be.s.vor"),
        "H1309": ("בְּשֹׂרָה", "H:N-F", "good news", "be.s.vo.rah"),
        "H1310": ("בָּשַׁל", "H:V", "to boil", "ba.shal"),
        "H1311": ("בָּשֵׁל", "H:A", "cooked", "ba.shel"),
        "H1312": ("בִּשְׁלָם", "N:N-M-P", "Bishlam", "bish.lam"),
        "H1313": ("בֹּ֫שֶׂם", "H:N-M", "spice", "ba.sam"),
        "H1314": ("בֶּ֫שֶׂם", "H:N-M", "spice", "be.sem"),
        "H1315": ("בָּֽשְׂמַת", "N:N-F-P", "Basemath", "bos.mat"),
        "H1316": ("בָּשָׁן", "N:N--L", "Bashan", "ba.shan"),
        "H1317": ("בׇּשְׁנָה", "H:N-M", "shame", "bosh.nah"),
        "H1318": ("בָּשַׁס", "H:V", "to trample", "ba.shas"),
        "H1319": ("בָּשַׂר", "H:V", "to bear tidings", "ba.sar"),
        "H1320": ("בָּשָׂר", "H:N-M", "flesh", "ba.sar"),
        "H1321": ("בְּשַׁר", "A:N-F", "flesh", "be.shar"),
        "H1322": ("בֹּ֫שֶׁת", "H:N-F", "shame", "bo.shet"),
        "H1323": ("בַּת", "H:N-F", "daughter", "bat"),
        "H1324": ("בַּת", "H:N-M", "bath", "bat"),
        "H1325": ("בַּת", "A:N-M", "bath", "bat"),
        "H1326": ("בָּתָה", "H:N-F", "waste", "ba.tah"),
        "H1327": ("בַּתָּה", "H:N-F", "precipice", "bat.tah"),
        "H1328a": ("בְּתוּאֵל", "N:N-M-P", "Bethuel", "be.tu.el"),
        "H1328b": ("בְּתוּאֵל", "N:N--L", "Bethuel", "be.tu.el"),
        "H1329": ("בְּתוּל", "N:N--L", "Bethul", "be.tul"),
        "H1330": ("בְּתוּלָה", "H:N-F", "virgin", "be.tu.lah"),
        "H1331": ("בְּתוּלִים", "H:N-F", "virginity", "be.tu.lim"),
        "H1332": ("בִּתְיָה", "N:N-F-P", "Bithiah", "bit.yah"),
        "H1333": ("בָּתַק", "H:V", "to cut up", "ba.taq"),
        "H1334": ("בָּתַר", "H:V", "to cut up", "ba.tar"),
        "H1335": ("בֶּ֫תֶר", "H:N-M", "part", "be.ter"),
        "H1336": ("בֶּ֫תֶר", "N:N-M-P", "Bether", "be.ter"),
        "H1337": ("בַּת־רַבִּים", "N:N--L", "Bath-rabbim", "bat rab.bim"),
        "H1338": ("בִּתְרוֹן", "N:N--L", "Bithron", "bit.ron"),
        "H1339": ("בַּת־שֶׁ֫בַע", "N:N-F-P", "Bath-sheba", "bat-she.va"),
        "H1340": ("בַּת־שׁוּעַ", "N:N-F-P", "Bath-shua", "bat-shu.a"),
        "H1341": ("גֵּא", "H:A", "proud", "ge"),
        "H1342": ("גָּאָה", "H:V", "to rise up", "ga.ah"),
        "H1343": ("גֵּאֶה", "H:A", "proud", "ge.eh"),
        "H1344": ("גֵּאָה", "H:N-F", "pride", "ge.ah"),
        "H1345": ("גְּאוּאֵל", "N:N-M-P", "Geuel", "ge.u.el"),
        "H1346": ("גַּאֲוָה", "H:N-F", "pride", "ga.a.vah"),
        "H1347": ("גָּאוֹן", "H:N-M", "pride", "ga.on"),
        "H1348": ("גֵּאוּת", "H:N-F", "majesty", "ge.ut"),
        "H1349": ("גַּאֲיוֹן", "H:A", "proud", "ga.a.yon"),
        "H1350a": ("גָּאַל", "H:V", "to redeem", "ga.al"),
        "H1350b": ("גְּאוּלַי", "H:N", "redemption", ""),
        "H1351": ("גָּאַל", "H:V", "to defile", "ga.al"),
        "H1352": ("גֹּ֫אֶל", "H:N-M", "defilement", "go.el"),
        "H1353": ("גְּאֻלָּה", "H:N-F", "redemption", "ge.ul.lah"),
        "H1354": ("גַּב", "H:N", "back/rim/brow", "gav"),
        "H1355": ("גַּב", "A:N-M", "back", "gav"),
        "H1356a": ("גֵּב", "H:N-M", "cistern", "gev"),
        "H1356b": ("גֵּב", "H:N-M", "beam", "gev"),
        "H1357": ("גֵּב", "H:N-M", "locust", "gev"),
        "H1358": ("גֹּב", "A:N-M", "den", "gov"),
        "H1359": ("גֹּב", "N:N--L", "Gob", "gov"),
        "H1360": ("גֶּ֫בֶא", "H:N-M", "cistern", "ge.ve"),
        "H1361": ("גָּבַהּ", "H:V", "to exult", "ga.vah"),
        "H1362": ("גָּבֹהַּ", "H:A", "high", "ga.vah"),
        "H1363": ("גֹּ֫בַהּ", "H:N-M", "height", "go.vah"),
        "H1364": ("גָּבֹהַּ", "H:A", "high", "ga.vo.ah"),
        "H1365": ("גַּבְהוּת", "H:N-M", "haughtiness", "gav.hut"),
        "H1366": ("גְּבוּל", "H:N-M", "border", "ge.vul"),
        "H1367": ("גְּבוּלָה", "H:N-F", "border", "ge.vu.lah"),
        "H1368": ("גִּבּוֹר", "H:A", "mighty", "gib.bor"),
        "H1369": ("גְּבוּרָה", "H:N-F", "might", "ge.vu.rah"),
        "H1370": ("גְּבוּרָה", "A:N-F", "might", "ge.vu.rah"),
        "H1371": ("גִּבֵּחַ", "H:A", "bald", "gib.be.ach"),
        "H1372": ("גַּבַּ֫חַת", "H:N-F", "baldness", "gab.ba.chat"),
        "H1373": ("גַּבַּי", "N:N-M-P", "Gabbai", "gab.bay"),
        "H1374": ("גֵּבִים", "N:N--L", "Gebim", "ge.vim"),
        "H1375": ("גָּבִיעַ", "H:N-M", "cup", "ge.vi.a"),
        "H1376": ("גְּבִיר", "H:N-M", "lord", "ge.vir"),
        "H1377": ("גְּבִירָה", "H:N-F", "queen", "ge.vi.rah"),
        "H1378": ("גָּבִישׁ", "H:N-M", "crystal", "ga.vish"),
        "H1379": ("גָּבַל", "H:V", "to border", "ga.val"),
        "H1380": ("גְּבַל, גְּבָל", "N:N--L", "Gebal", "ge.val"),
        "H1381": ("גְּבָל", "N:N--L", "Gebal", "ge.val"),
        "H1382": ("גִּבְלִי", "N:N--LG", "Gebalite", "giv.li"),
        "H1383": ("גַּבְלֻת", "H:N-F", "twists", "gav.lut"),
        "H1384": ("גִּבֵּן", "H:A", "humpbacked", "gib.ben"),
        "H1385": ("גְּבִינָה", "H:N-F", "cheese", "ge.vi.nah"),
        "H1386": ("גַּבְנֹן", "H:N-M", "peak", "gav.non"),
        "H1387": ("גֶּ֫בַע", "N:N--L", "Geba", "ge.va"),
        "H1388": ("גִּבְעָא", "N:N-M-P", "Gibea", "giv.a"),
        "H1389": ("גִּבְעָה", "H:N-F", "hill", "giv.ah"),
        "H1390": ("גִּבְעָה", "N:N--L", "Gibeah", "giv.ah"),
        "H1391": ("גִּבְעוֹן", "N:N--L / N:N-M-P", "Gibeon", "giv.on"),
        "H1392": ("גִּבְעֹל", "H:N-M", "bud", "giv.ol"),
        "H1393": ("גִּבְעֹנִי", "N:N--LG", "Gibeonite", "giv.o.ni"),
        "H1394": ("גִּבְעַת", "N:N--L", "Gibeath", "giv.at"),
        "H1395": ("גִּבְעָתִי", "N:N--LG", "Gibeathite", "giv.a.ti"),
        "H1396": ("גָּבַר", "H:V", "to prevail", "ga.var"),
        "H1397": ("גֶּ֫בֶר", "H:N-M", "great man", "ge.ver"),
        "H1398": ("גֶּ֫בֶר", "N:N-M-P", "Geber", "ge.ver"),
        "H1399": ("גֶּ֫בֶר", "H:N-M", "great man", "ge.var"),
        "H1400": ("גְּבַר", "A:N-M", "man", "ge.var"),
        "H1401": ("גִּבָּר", "A:N-M", "mighty", "gib.bar"),
        "H1402": ("גִּבָּר", "N:N-M-P", "Gibbar", "gib.bar"),
        "H1403": ("גַּבְרִיאֵל", "N:N-M-P", "Gabriel", "gav.ri.el"),
        "H1404": ("גְּבֶ֫רֶת", "H:N-F", "lady", "ge.ve.ret"),
        "H1405": ("גִּבְּתוֹן", "N:N--L", "Gibbethon", "gib.b.ton"),
        "H1406": ("גָּג", "H:N-M", "roof", "gag"),
        "H1407": ("גַּד", "H:N-M", "coriander", "gad"),
        "H1408": ("גַּד", "N:N-M-T", "Fortune", "gad"),
        "H1409": ("גָּד", "H:N-M", "fortune", "gad"),
        "H1410": ("גָּד", "N:N-M-P", "Gad", "gad"),
        "H1411": ("גְּדָֽבְרַיָּא", "A:N-M", "treasurer", "ge.da.var"),
        "H1412": ("גֻּדְגֹּ֫דָה", "N:N--L", "Gudgodah", "gud.go.dah"),
        "H1413": ("גָּדַד", "H:V", "to cut", "ga.dad"),
        "H1414": ("גְּדַד", "A:V", "to chop", "ge.dad"),
        "H1415": ("גָּדָה", "H:N-F", "bank", "ga.dah"),
        "H1416": ("גְּדוּד", "H:N-M", "band", "ge.dud"),
        "H1417": ("גְּדוּד", "H:N", "furrow", "ge.dud"),
        "H1418": ("גְּדוּדָה", "H:N", "furrow", "ge.du.dah"),
        "H1419a": ("גָּדוֹל", "H:A", "great", "ga.dol"),
        "H1419b": ("הַגְּדוֹלִים", "N:N-M-P", "Haggedolim", ""),
        "H1420": ("גְּדוּלָּה", "H:N-F", "greatness", "ge.du.lah"),
        "H1421": ("גִּדּוּפִים", "H:N-M", "reviling", "gid.duph"),
        "H1422": ("גְּדוּפָה", "H:N-F", "taunt", "ge.du.phah"),
        "H1423": ("גְּדִי", "H:N-M", "kid", "ge.di"),
        "H1424": ("גָּדִי", "N:N-M-P", "Gadi", "ga.di"),
        "H1425": ("גָּדִי", "N:N--PG", "Gadite", "ga.di"),
        "H1426": ("גַּדִּי", "N:N-M-P", "Gaddi", "gad.di"),
        "H1427": ("גַּדִּיאֵל", "N:N-M-P", "Gaddiel", "gad.di.el"),
        "H1428": ("גִּדְיָה", "H:N-F", "bank", "gid.yah"),
        "H1429": ("גְּדִיָּה", "H:N-F", "kid", "ge.diy.yah"),
        "H1430a": ("גָּדִישׁ", "H:N-M", "stack", "ga.dish"),
        "H1430b": ("גָּדִישׁ", "H:N-M", "tomb", "ga.dish"),
        "H1431": ("גָּדַל", "H:V", "to magnify", "ga.dal"),
        "H1432": ("גָּדֵל", "H:A", "growing", "ga.del"),
        "H1433": ("גֹּ֫דֶל", "H:N-M", "greatness", "go.del"),
        "H1434": ("גְּדִלִים", "H:N-M", "tassel", "ge.dil"),
        "H1435": ("גִּדֵּל", "N:N-M-P", "Giddel", "gid.del"),
        "H1436a": ("גְּדַלְיָה", "N:N-M-P", "Gedaliah", "ge.dal.yah"),
        "H1436b": ("גְּדַלְיָ֫הוּ", "N:N-M-P", "Gedaliah", ""),
        "H1437": ("גִּדַּלְתִּי", "N:N-M-P", "Giddalti", "gid.dal.ti"),
        "H1438": ("גָּדַע", "H:V", "to cut down/off", "ga.da"),
        "H1439": ("גִּדְעוֹן", "N:N-M-P", "Gideon", "gid.on"),
        "H1440": ("גִּדְעֹם", "N:N--L", "Gidom", "gid.om"),
        "H1441": ("גִּדְעֹנִי", "N:N-M-P", "Gideoni", "gid.o.ni"),
        "H1442": ("גָּדַף", "H:V", "to blaspheme", "ga.daph"),
        "H1443": ("גָּדַר", "H:V", "to wall up/off", "ga.dar"),
        "H1444": ("גֶּדֶר", "H:N-M", "wall", "ge.der"),
        "H1445": ("גֶּ֫דֶר", "N:N--L", "Geder", "ge.der"),
        "H1446": ("גְּדוֹר", "N:N-M-P / N:N--L", "Gedor", "ge.dor"),
        "H1447": ("גָּדֵר", "H:N-M", "wall", "ga.der"),
        "H1448": ("גְּדֵרָה", "H:N-F", "wall", "ge.de.rah"),
        "H1449": ("גְּדֵרָה", "N:N--L", "Gederah", "ge.de.rah"),
        "H1450": ("גְּדֵרוֹת", "N:N--L", "Gederoth", "ge.de.rot"),
        "H1451": ("גְּדֵרִי", "N:N--LG", "Gederite", "ge.de.ri"),
        "H1452": ("גְּדֵרָתִי", "N:N--LG", "Gederathite", "ge.de.ra.ti"),
        "H1453": ("גְּדֵרֹתָ֑יִם", "N:N--L", "Gederothaim", "ge.de.ro.ta.yim"),
        "H1454": ("גֵּה", "H:DemP", "this", "geh"),
        "H1455": ("גָּהָה", "H:V", "to cure", "ga.hah"),
        "H1456": ("גֵּהָה", "H:N-F", "cure", "ge.hah"),
        "H1457": ("גָּהַר", "H:V", "to bend (down)", "ga.har"),
        "H1458": ("גַּו", "H:N-M", "the back", "gav"),
        "H1459": ("גַּו", "A:N-M", "midst", "gav"),
        "H1460a": ("גֵּו", "H:N-M", "back", "gev"),
        "H1460b": ("גֵּו", "H:N-M", "midst", "gev"),
        "H1461": ("גּוּב", "H:V", "to plough", "guv"),
        "H1462a": ("גּוֹב", "H:N-M", "locust", "gov"),
        "H1462b": ("גֹּבַי", "H:N-M", "locust", ""),
        "H1463": ("גּוֹג", "N:N-M-P / N:N--L", "Gog", "gog"),
        "H1464": ("גּוּד", "H:V", "to raid", "gud"),
        "H1465": ("גֵּוָה", "H:N-F", "back", "ge.vah"),
        "H1466": ("גֵּוָה", "H:N-F", "pride", "ge.vah"),
        "H1467": ("גֵּוָה", "A:N-F", "pride", "ge.vah"),
        "H1468": ("גּוּז", "H:V", "to cut off", "guz"),
        "H1469": ("גּוֹזָל", "H:N-M", "young bird", "go.zal"),
        "H1470": ("גּוֹזָן", "N:N--L", "Gozan", "go.zan"),
        "H1471a": ("גּוֹי", "H:N-M", "nation", "goy"),
        "H1471b": ("גּוֹיִם", "N:N--LG", "Goyim", ""),
        "H1472": ("גְּוִיָּה", "H:N-F", "body", "ge.viy.yah"),
        "H1473": ("גּוֹלָה", "H:N-F", "captivity", "go.lah"),
        "H1474": ("גּוֹלָן", "N:N--L", "Golan", "go.lan"),
        "H1475": ("גּוּמָּץ", "H:N-M", "pit", "gum.mats"),
        "H1476": ("גּוּנִי", "N:N-M-P", "Guni", "gu.ni"),
        "H1477": ("גּוּנִי", "N:N--PG", "Gunite", "gu.ni"),
        "H1478": ("גָּוַע", "H:V", "to die", "ga.va"),
        "H1479": ("גּוּף", "H:V", "to shut", "guph"),
        "H1480": ("גּוּפָה", "H:N-F", "body", "gu.phah"),
        "H1481a": ("גּוּר", "H:V", "to sojourn", "gur"),
        "H1481b": ("גּוּר", "H:V", "to quarrel", "gur"),
        "H1481c": ("גּוּר", "H:V", "to dread", "gur"),
        "H1482": ("גּוּר", "H:N-M", "whelp", "gur"),
        "H1483": ("גּוּר", "N:N--L", "Gur", "gur"),
        "H1484": ("גּוֹר", "H:N-M", "whelp", "gor"),
        "H1485": ("גּוּר־בָּ֑֫עַל", "N:N--L", "Gur-baal", "gur-ba.al"),
        "H1486": ("גּוֹרָל", "H:N-M", "allotted", "go.ral"),
        "H1487": ("גּוּשׁ", "H:N-M", "clod", "gush"),
        "H1488": ("גֵּז", "H:N-M", "fleece", "gez"),
        "H1489": ("גִּזְבָּר", "H:N-M", "treasurer", "giz.bar"),
        "H1490": ("גִּזְבָּר", "A:N-M", "treasurer", "giz.bar"),
        "H1491": ("גָּזָה", "H:V", "to cut", "ga.zah"),
        "H1492": ("גִּזָּה", "H:N-F", "fleece", "gaz.zah"),
        "H1493": ("גִּזוֹנִי", "N:N--LG", "Gizonite", "gi.zo.ni"),
        "H1494": ("גָּזַז", "H:V", "to shear", "ga.zaz"),
        "H1495": ("גָּזֵז", "N:N-M-P", "Gazez", "ga.zez"),
        "H1496": ("גָּזִית", "H:N-F", "cutting", "ga.zit"),
        "H1497": ("גָּזַל", "H:V", "to plunder", "ga.zal"),
        "H1498": ("גָּזֵל", "H:N-M", "robbery", "ga.zel"),
        "H1499": ("גֵּזֶל", "H:N-M", "violence", "ge.zel"),
        "H1500": ("גְּזֵלָה", "H:N-F", "violence", "ge.ze.lah"),
        "H1501": ("גָּזָם", "H:N-M", "locust", "ga.zam"),
        "H1502": ("גַּזָּם", "N:N-M-P", "Gazzam", "gaz.zam"),
        "H1503": ("גֶּ֫זַע", "H:N-M", "stock", "ge.za"),
        "H1504": ("גָּזַר", "H:V", "to cut", "ga.zar"),
        "H1505": ("גְּזַר", "A:V", "to determine", "ge.zar"),
        "H1506": ("גֶּ֫זֶר", "H:N-M", "piece", "ge.zer"),
        "H1507": ("גֶּ֫זֶר", "N:N--L", "Gezer", "ge.zer"),
        "H1508": ("גִּזְרָה", "H:N-F", "cutting/separation", "giz.rah"),
        "H1509": ("גְּזֵרָה", "H:N-F", "isolation", "ge.ze.rah"),
        "H1510": ("גְּזֵרָה", "A:N-F", "decree", "ge.ze.rah"),
        "H1511": ("גִּזְרִי", "N:N--LG", "Gezrite", "giz.ri"),
        "H1512": ("גָּחוֹן", "H:N-M", "belly", "ga.chon"),
        "H1513": ("גַּחֶ֫לֶת", "H:N-F", "coal", "ge.chel"),
        "H1514": ("גַּ֫חַם", "N:N-M-P", "Gaham", "ga.cham"),
        "H1515": ("גַּ֫חַר", "N:N-M-P", "Gahar", "ga.char"),
        "H1516": ("גַּיְא", "H:N", "valley", "gay"),
        "H1517": ("גִּיד", "H:N-M", "sinew", "gid"),
        "H1518": ("גִּיחַ", "H:V", "to burst/come out", "gi.ach"),
        "H1519": ("גִּיחַ", "A:V", "to strive", "gi.ach"),
        "H1520": ("גִּיחַ", "N:N--L", "Giah", "gi.ach"),
        "H1521": ("גִּיחוֹן", "N:N--L", "Gihon", "gi.chon"),
        "H1522": ("גֵּיחֲזִי", "N:N-M-P", "Gehazi", "ge.cha.zi"),
        "H1523": ("גִּיל", "H:V", "to rejoice", "gil"),
        "H1524a": ("גִּיל", "H:N-M", "rejoicing", "gil"),
        "H1524b": ("גִּיל", "H:N-M", "circle", "gil"),
        "H1525": ("גִּילָה", "H:N-F", "rejoicing", "gi.lah"),
        "H1526": ("גִּילֹנִי", "N:N--LG", "Gilonite", "gi.lo.ni"),
        "H1527": ("גִּינַ֑ת", "N:N-M-P", "Ginath", "gi.nat"),
        "H1528": ("גִּיר", "A:N-M", "plaster", "gir"),
        "H1529": ("גֵּישָׁן", "N:N-M-P", "Geshan", "ge.shan"),
        "H1530": ("גַּל", "H:N-M", "heap", "gal"),
        "H1531": ("גֹּל", "H:N-F", "bowl", "gol"),
        "H1532": ("גַּלָּב", "H:N-M", "barber", "gal.lav"),
        "H1533": ("גִּלְבֹּעַ", "N:N--L", "Gilboa", "gil.bo.a"),
        "H1534": ("גַּלְגַּל", "H:N-M", "wheel", "gal.gal"),
        "H1535": ("גַּלְגַּל", "A:N-M", "wheel", "gal.gal"),
        "H1536": ("גִּלְגָּל", "H:N-M", "wheel", "gil.gal"),
        "H1537": ("גִּלְגָּל", "N:N--L", "Gilgal", "gil.gal"),
        "H1538": ("גֻּלְגֹּ֫לֶת", "H:N-F", "head", "gul.go.let"),
        "H1539": ("גֶּ֫לֶד", "H:N-M", "skin", "ge.led"),
        "H1540": ("גָּלָה", "H:V", "to uncover", "ga.lah"),
        "H1541": ("גְּלָא", "A:V", "to reveal", "ge.lah"),
        "H1542": ("גִּלֹה", "N:N--L", "Giloh", "gi.loh"),
        "H1543": ("גֻּלָּה", "H:N-F", "bowl", "gul.lah"),
        "H1544": ("גִּלּוּל", "H:N-M", "idol", "gil.lul"),
        "H1545": ("גְּלוֹם", "H:N-M", "garment", "ge.lom"),
        "H1546": ("גָּלוּת", "H:N-F", "captivity", "ga.lut"),
        "H1547": ("גָּלוּ", "A:N-F", "captivity", "ga.lut"),
        "H1548": ("גָּלַח", "H:V", "to shave", "ga.lach"),
        "H1549": ("גִּלָּיוֹן", "H:N-M", "tablet", "gil.la.von"),
        "H1550a": ("גָּלִיל", "H:A", "turned", "ga.lil"),
        "H1550b": ("גָּלִיל", "H:N-M", "circuit", "ga.lil"),
        "H1551": ("גָּלִיל", "N:N--L", "Galilee", "ga.lil"),
        "H1552": ("גְלִילָה", "H:N-F", "border", "ge.li.lah"),
        "H1553": ("גְּלִילוֹת", "N:N--L", "Geliloth", "ge.li.lot"),
        "H1554": ("גַּלִּים", "N:N--L", "Gallim", "gal.lim"),
        "H1555": ("גׇּלְיַת", "N:N-M-P", "Goliath", "gol.yat"),
        "H1556": ("גָּלַל", "H:V", "to roll", "ga.lal"),
        "H1557": ("גָּלָל", "H:N-M", "dung", "ga.lal"),
        "H1558": ("גָּלָל", "H:N-M", "because of", "ga.lal"),
        "H1559": ("גָּלָל", "N:N-M-P", "Galal", "ga.lal"),
        "H1560": ("גְּלָל", "A:A", "great", "ge.lal"),
        "H1561": ("גֵּל", "H:N-M", "dung", "ge.lel"),
        "H1562": ("גִּלֲלַי", "N:N-M-P", "Gilalai", "gi.la.lay"),
        "H1563": ("גָּלַם", "H:V", "to fold", "ga.lam"),
        "H1564": ("גֹּ֫לֶם", "H:N-M", "embryo", "go.lem"),
        "H1565": ("גַּלְמוּד", "H:A", "solitary", "gal.mud"),
        "H1566": ("גָּלַע", "H:V", "to quarrel", "ga.la"),
        "H1567": ("גַּלְעֵד", "N:N--L", "Galeed", "gal.ed"),
        "H1568": ("גִּלְעָד", "N:N--L / N:N--LG / N:N-M-P", "Gilead", "gil.ad"),
        "H1569": ("גִּלְעָדִי", "N:N--PG", "Gileadite", "gil.a.di"),
        "H1570": ("גָּלַשׁ", "H:V", "to step down", "ga.lash"),
        "H1571": ("גַּם", "H:Adv", "also", "gam"),
        "H1572": ("גָּמָא", "H:V", "to swallow", "ga.ma"),
        "H1573": ("גֹּ֫מֶא", "H:N-M", "reed", "go.me"),
        "H1574": ("גֹּ֫מֶד", "H:N-M", "short cubit", "go.med"),
        "H1575": ("גַּמָּדִים", "H:N-M", "warrior", "gam.mad"),
        "H1576": ("גְּמוּל", "H:N-M", "recompense", "ge.mul"),
        "H1577": ("גָּמוּל", "N:N-M-P", "Gamul", "ga.mul"),
        "H1578": ("גְּמוּלָה", "H:N-F", "recompense", "ge.mu.lah"),
        "H1579": ("גִּמְזוֹ", "N:N--L", "Gimzo", "gim.zo"),
        "H1580": ("גָּמַל", "H:V", "to wean", "ga.mal"),
        "H1581": ("גָּמָל", "H:N", "camel", "ga.mal"),
        "H1582": ("גְּמַלִּי", "N:N-M-P", "Gemalli", "ge.mal.li"),
        "H1583": ("גַּמְלִיאֵל", "N:N-M-P", "Gamaliel", "gam.li.el"),
        "H1584": ("גָּמַר", "H:V", "to cease", "ga.mar"),
        "H1585": ("גְּמַר", "A:V", "to complete", "ge.mar"),
        "H1586": ("גֹּ֫מֶר", "N:N-M-P / N:N-F-P", "Gomer", "go.mer"),
        "H1587": ("גְּמַרְיָ֫הוּ", "N:N-M-P", "Gemariah", "ge.mar.yah"),
        "H1588": ("גַּן", "H:N", "garden", "gan"),
        "H1589": ("גָּנַב", "H:V", "to steal", "ga.nav"),
        "H1590": ("גַּנָּב", "H:N-M", "thief", "gan.nav"),
        "H1591": ("גְּנֵבָה", "H:N-F", "theft", "ge.ne.vah"),
        "H1592": ("גְּנֻבַת", "N:N-M-P", "Genubath", "ge.nu.vat"),
        "H1593": ("גַּנָּה", "H:N-F", "garden", "gan.nah"),
        "H1594": ("גִּנָּה", "H:N-F", "garden", "gin.nah"),
        "H1595": ("גְּנָזִים", "H:N-M", "treasury", "ge.nez"),
        "H1596": ("גְּנַז", "A:N-M", "treasure", "ge.naz"),
        "H1597": ("גַּנְזַךְ", "H:N-M", "treasury", "gin.zakh"),
        "H1598": ("גָּנַן", "H:V", "to defend", "ga.nan"),
        "H1599": ("גִּנְּתוֹן", "N:N-M-P", "Ginnethon", "gin.n.ton"),
        "H1600": ("גָּעָה", "H:V", "to low", "ga.ah"),
        "H1601": ("גֹּעָה", "N:N--L", "Goah", "go.ah"),
        "H1602": ("גָּעַל", "H:V", "to abhor", "ga.al"),
        "H1603": ("גַּ֫עַל", "N:N-M-P", "Gaal", "ga.al"),
        "H1604": ("גֹּ֫עַל", "H:N-M", "loathing", "go.al"),
        "H1605": ("גָּעַר", "H:V", "to rebuke", "ga.ar"),
        "H1606": ("גְּעָרָה", "H:N-F", "rebuke", "ge.a.rah"),
        "H1607": ("גָּעַשׁ", "H:V", "to shake", "ga.ash"),
        "H1608": ("גַּ֫עַשׁ", "N:N--L", "Gaash", "ga.ash"),
        "H1609": ("גַּעְתָּם", "N:N-M-P", "Gatam", "ga.tam"),
        "H1610": ("גַּף", "H:N-M", "single/height", "gaph"),
        "H1611": ("גַּף", "A:N-F", "wing", "gaph"),
        "H1612": ("גֶּ֫פֶן", "H:N-M", "vine", "ge.phen"),
        "H1613": ("גֹּ֫פֶר", "H:N-M", "gopher", "go.pher"),
        "H1614": ("גׇּפְרִית", "H:N-F", "brimstone", "goph.rit"),
        "H1615": ("גִּר", "H:N-M", "chalk", "gir"),
        "H1616": ("גֵּר", "H:N-M", "sojourner", "ger"),
        "H1617": ("גֵּרָא", "N:N-M-P", "Gera", "ge.ra"),
        "H1618": ("גָּרָב", "H:N-M", "scab", "ga.rav"),
        "H1619": ("גָּרֵב", "N:N-M-P / N:N--L", "Gareb", "ga.rev"),
        "H1620": ("גַּרְגַּר", "H:N-M", "berry", "gar.gar"),
        "H1621": ("גַּרְגְּרוֹת", "H:N-F", "neck", "gar.ge.rot"),
        "H1622": ("גִּרְגָּשִׁי", "N:N--LG", "Girgashite", "gir.ga.shi"),
        "H1623": ("גָּרַד", "H:V", "to scrape", "ga.rad"),
        "H1624": ("גָּרָה", "H:V", "to stir up", "ga.rah"),
        "H1625": ("גֵּרָה", "H:N-F", "cud", "ge.rah"),
        "H1626": ("גֵּרָה", "H:N-F", "gerah", "ge.rah"),
        "H1627": ("גָּרוֹן", "H:N-M", "throat", "ga.ron"),
        "H1628": ("גֵּרוּת", "N:N--L", "Gerth", "ge.rut"),
        "H1629": ("גָּרַז", "H:V", "to cut off", "ga.raz"),
        "H1630": ("גְּרִזִים", "N:N--L", "Gerizim", "ge.ri.zim"),
        "H1631": ("גַּרְזֶן", "H:N-M", "axe", "gar.zen"),
        "H1632": ("גָּרֹל", "H:A", "harsh", "ga.rol"),
        "H1633a": ("גָּרַם", "H:V", "to reserve", "ga.ram"),
        "H1633b": ("גָּרַם", "H:V", "to break bones", "ga.ram"),
        "H1634": ("גֶּ֫רֶם", "H:N-M", "bone", "ge.rem"),
        "H1635": ("גְּרַם", "A:N-M", "bone", "ge.rem"),
        "H1636": ("גַּרְמִי", "N:N--PG", "Garmite", "gar.mi"),
        "H1637": ("גֹּ֫רֶן", "H:N-M", "threshing floor", "go.ren"),
        "H1638": ("גָּרַס", "H:V", "to break", "ga.ras"),
        "H1639": ("גָּרַע", "H:V", "to dimish", "ga.ra"),
        "H1640": ("גָּרַף", "H:V", "to sweep away", "ga.raph"),
        "H1641": ("גָּרַר", "H:V", "to drag/chew/saw", "ga.rar"),
        "H1642": ("גְּרָר", "N:N--L", "Gerar", "ge.rar"),
        "H1643": ("גֶּ֫רֶשׂ", "H:N-M", "crushed grain", "ge.re.s"),
        "H1644": ("גָּרַשׁ", "H:V", "to drive out", "ga.rash"),
        "H1645": ("גֶּ֫רֶשׁ", "H:N-M", "produce", "ge.resh"),
        "H1646": ("גְרוּשָׁה", "H:N-F", "eviction", "ge.ru.shah"),
        "H1647": ("גֵּרְשֹׁם", "N:N-M-P", "Gershom", "ge.re.shom"),
        "H1648": ("גֵּרְשׁוֹן", "N:N-M-P", "Gershon", "ge.re.shon"),
        "H1649": ("גֵּרְשֻׁנִּי", "N:N--PG", "Gershonite", "ge.re.shun.ni"),
        "H1650": ("גְּשׁוּר", "N:N--L", "Geshur", "ge.shur"),
        "H1651": ("גְּשׁוּרִי", "N:N--LG", "Geshurite", "ge.shu.ri"),
        "H1652": ("גָּשַׁם", "H:V", "to rain", "ga.sham"),
        "H1653": ("גֶּ֫שֶׁם", "H:N-M", "rain", "ge.shem"),
        "H1654": ("גֶּ֫שֶׁם", "N:N-M-P", "Gashmu", "ge.shem"),
        "H1655": ("גְּשֵׁם", "A:N-M", "body", "ge.shem"),
        "H1656": ("גֹּ֫שֶׁם", "H:N-M", "rain", "go.shem"),
        "H1657": ("גֹּ֫שֶׁן", "N:N--L", "Goshen", "go.shen"),
        "H1658": ("גִּשְׁפָּא", "N:N-M-P", "Gishpa", "gish.pa"),
        "H1659": ("גָּשַׁשׁ", "H:V", "to grope", "ga.shash"),
        "H1660": ("גַּת", "H:N-F", "wine press", "gat"),
        "H1661": ("גַּת", "N:N--L", "Gath", "gat"),
        "H1662": ("גַּת הַחֵ֫פֶר", "N:N--L", "Gath-hepher", "gat-ha.che.pher"),
        "H1663": ("גִּתִּי", "N:N--LG", "Gittite", "git.ti"),
        "H1664": ("גִּתָּ֑יִם", "N:N--L", "Gittaim", "git.ta.yim"),
        "H1665": ("גִּתִּית", "N:N-F-T", "Gittith", "git.tit"),
        "H1666": ("גֶּ֫תֶר", "N:N-M-P", "Gether", "ge.ter"),
        "H1667": ("גַּת־רִמּוֹן", "N:N--L", "Gath-rimmon", "gat-rim.mon"),
        "H1668": ("דָּא", "A:DemP", "this", "da"),
        "H1669": ("דָּאֵב", "H:V", "to languish", "da.av"),
        "H1670": ("דְּאָבָה", "H:N-F", "dismay", "de.a.vah"),
        "H1671": ("דְּאָבוֹן", "H:N-M", "sorrow", "de.a.von"),
        "H1672": ("דָּאַג", "H:V", "be anxious", "da.ag"),
        "H1673": ("דֹּאֵג", "N:N-M-P", "Doeg", "do.eg"),
        "H1674": ("דְּאָגָה", "H:N-F", "anxiety", "de.a.gah"),
        "H1675": ("דָּאָה", "H:V", "to fly", "da.ah"),
        "H1676": ("דָּאָה", "H:N-F", "kite", "da.ah"),
        "H1677": ("דֹּב", "H:N", "bear", "dov"),
        "H1678": ("דֹּב", "A:N-M", "bear", "dov"),
        "H1679": ("דֹּ֫בֶא", "H:N-M", "strength", "do.ve"),
        "H1680": ("דָּבַב", "H:V", "to glide", "da.vav"),
        "H1681": ("דִּבָּה", "H:N-F", "slander", "dib.bah"),
        "H1682": ("דְּבוֹרָה", "H:N-F", "bee", "de.vo.rah"),
        "H1683": ("דְּבוֹרָה", "N:N-F-P", "Deborah", "de.bo.rah"),
        "H1684": ("דְּבַח", "A:V", "to sacrifice", "de.vach"),
        "H1685": ("דְּבַח", "A:N-M", "sacrifice", "de.vach"),
        "H1686": ("דִּבְיוֹנִים", "H:N-M", "quano", "div.yon"),
        "H1687": ("דְּבִיר", "H:N-M", "sanctuary", "de.vir"),
        "H1688a": ("דְּבִיר", "N:N-M-P / N:N--L", "Debir", "de.vir"),
        "H1688b": ("לִדְבִר", "N:N--L", "Debir", ""),
        "H1689": ("דִּבְלָה", "N:N--L", "Diblah", "div.lah"),
        "H1690": ("דְּבֵלָה", "H:N-F", "fig cake", "de.ve.lah"),
        "H1691": ("דִּבְלַ֫יִם", "N:N-M-P", "Diblaim", "div.la.yim"),
        "H1692": ("דָּבַק", "H:V", "to cleave", "da.vaq"),
        "H1693": ("דְּבֵק", "A:V", "to cleave", "de.vaq"),
        "H1694": ("דֶּ֫בֶק", "H:N-M", "joint", "de.veq"),
        "H1695": ("דָּבֵק", "H:A", "cleaving", "da.veq"),
        "H1696": ("דָבַר", "H:V", "to speak", "da.var"),
        "H1697": ("דָּבָר", "H:N-M", "word", "da.var"),
        "H1698": ("דֶּ֫בֶר", "H:N-M", "pestilence", "de.ver"),
        "H1699a": ("דֹּ֫בֶר", "H:N-M", "pasture", "do.ver"),
        "H1699b": ("דִּבֵּר", "H:N-M", "statement", ""),
        "H1700": ("דִּבְרָה", "H:N-M", "cause", "div.rah"),
        "H1701": ("דִּבְרָה", "A:N-F", "cause", "div.rah"),
        "H1702": ("דֹּבְרוֹת", "H:N-F", "raft", "do.ve.rah"),
        "H1703": ("דַּבֶּ֫רֶת", "H:N-F", "word", "dab.ba.rah"),
        "H1704": ("דִּבְרִי", "N:N-M-P", "Dibri", "div.ri"),
        "H1705": ("דָּֽבְרַת", "N:N--L", "Daberath", "da.ve.rat"),
        "H1706": ("דְּבַשׁ", "H:N-M", "honey", "de.vash"),
        "H1707": ("דַּבֶּ֫שֶׁת", "H:N-F", "hump", "dab.be.shet"),
        "H1708": ("דַּבֶּ֫שֶׁת", "N:N--L", "Dabbesheth", "dab.be.shet"),
        "H1709": ("דָּג", "H:N-M", "fish", "dag"),
        "H1710": ("דָּגָה", "H:N-F", "fish", "da.gah"),
        "H1711": ("דָּגָה", "H:V", "to multiply", "da.gah"),
        "H1712": ("דָּגוֹן", "N:N-M-T", "Dagon", "da.gon"),
        "H1713a": ("דָּגַל", "H:V", "to look", "da.gal"),
        "H1713b": ("דָּגַל", "H:V", "to set a banner", "da.gal"),
        "H1714": ("דֶּ֫גֶל", "H:N-M", "standard", "de.gel"),
        "H1715": ("דָּגָן", "H:N-M", "grain", "da.gan"),
        "H1716": ("דָּגַר", "H:V", "to gather", "da.gar"),
        "H1717": ("דַּד", "H:N-M", "breast", "dad"),
        "H1718": ("דָּדָה", "H:V", "to go slowly", "da.dah"),
        "H1719a": ("דְּדָן", "N:N-M-P / N:N--L", "Dedan", "de.dan"),
        "H1719b": ("רֹדָן", "N:N-M-P / N:N--L", "Rodan", ""),
        "H1719c": ("רֹדָן", "N:N-M-P / N:N--L", "Rodan", ""),
        "H1720": ("דְּדָנִי", "N:N--LG", "Dedanite", "de.da.nim"),
        "H1721": ("רוֹדָנִים", "N:N--PG", "Dodanim", "do.da.nim"),
        "H1722": ("דְּהַב", "A:N-M", "gold", "de.hav"),
        "H1723": ("דֵּהָיֵא", "N:N--LG", "Dehavite", "da.ha.va"),
        "H1724": ("דָּהַם", "H:V", "to astonish", "da.ham"),
        "H1725": ("דָּהַר", "H:V", "to rush", "da.har"),
        "H1726": ("דַּהֲרָה", "H:N-F", "rushing", "da.ha.har"),
        "H1727": ("דּוּב", "H:V", "to pine", "duv"),
        "H1728": ("דַּוָּג", "H:N-M", "fisher", "dav.vag"),
        "H1729": ("דּוּגָה", "H:N-F", "fishhook", "du.gah"),
        "H1730": ("דּוֹד", "H:N-M", "beloved", "dod"),
        "H1731": ("דּוּד", "H:N-M", "pot", "dud"),
        "H1732": ("דָּוִד", "N:N-M-P", "David", "da.vid"),
        "H1733": ("דֹּדָה", "H:N-F", "aunt", "do.dah"),
        "H1734": ("דּוֹדוֹ", "N:N-M-P", "Dodo", "do.do"),
        "H1735": ("דּוֹדָוָהוּ", "N:N-M-P", "Dodavahu", "do.da.va.hu"),
        "H1736": ("דּוּדַי", "H:N-M", "mandrake", "du.day"),
        "H1737": ("דּוֹדַי", "N:N-M-P", "Dodai", "do.day"),
        "H1738": ("דָּוָה", "H:V", "be sick", "da.vah"),
        "H1739": ("דָּוֶה", "H:A", "sick", "da.veh"),
        "H1740": ("דּוּחַ", "H:V", "to wash", "du.ach"),
        "H1741": ("דְּוַי", "H:N-M", "illness", "de.vay"),
        "H1742": ("דַּוָּי", "H:A", "faint", "dav.va"),
        "H1743": ("דּוּךְ", "H:V", "to beat", "dukh"),
        "H1744": ("דּוּכִיפַת", "H:N-F", "hoopoe", "du.khi.phat"),
        "H1745": ("דּוּמָה", "H:N-F", "silence", "du.mah"),
        "H1746": ("דּוּמָה", "N:N-M-P / N:N--L", "Dumah", "du.mah"),
        "H1747": ("דּוּמִיָּה", "H:N-F", "silence", "du.miy.yah"),
        "H1748": ("דּוּמָם", "H:N-M", "silence", "du.mam"),
        "H1749": ("דּוֹנַג", "H:N-M", "wax", "do.nag"),
        "H1750": ("דּוּץ", "H:V", "to dance", "duts"),
        "H1751": ("דּוּק", "A:V", "to shatter", "duq"),
        "H1752": ("דּוּר", "H:V", "to dwell", "dur"),
        "H1753": ("דּוּר", "A:V", "to dwell", "dur"),
        "H1754a": ("דּוּר", "H:N-M", "circle", "dur"),
        "H1754b": ("כַּדּוּר", "H:N-M", "ball", ""),
        "H1755": ("דּוֹר", "H:N-M", "generation", "dor"),
        "H1756": ("דֹּאר", "N:N--L", "Dor", "dor"),
        "H1757": ("דּוּרָא", "N:N--L", "Dura", "du.ra"),
        "H1758": ("דּוּשׁ", "H:V", "to tread", "dush"),
        "H1759": ("דּוּשׁ", "A:V", "to tread", "dush"),
        "H1760a": ("דָּחָה", "H:V", "to thrust", "da.chah"),
        "H1760b": ("דָּחַח", "H:V", "to thrust", ""),
        "H1761": ("דַּחֲוָה", "A:N-F", "diversion", "da.cha.vah"),
        "H1762": ("דְּחִי", "H:N-M", "falling", "de.chi"),
        "H1763": ("דְּחַל", "A:V", "to fear", "de.chal"),
        "H1764": ("דֹּ֫חַן", "H:N-M", "millet", "do.chan"),
        "H1765": ("דָּחַף", "H:V", "to hasten", "da.chaph"),
        "H1766": ("דָּחַק", "H:V", "to crowd", "da.chaq"),
        "H1767": ("דַּי", "H:N", "enough", "day"),
        "H1768": ("דִּי", "A:Part", "that", "di"),
        "H1769": ("דִּיבֹן", "N:N--L", "Dibon", "di.von"),
        "H1770": ("דִּיג", "H:V", "to fish", "dig"),
        "H1771": ("דַּיָּג", "H:N-M", "fisher", "day.yag"),
        "H1772": ("דַּיָּה", "H:N-F", "hawk", "day.yah"),
        "H1773": ("דְּיוֹ", "H:N-M", "ink", "de.yo"),
        "H1774": ("דִּי זָהָב", "N:N--L", "Dizahab", "di za.hav"),
        "H1775": ("דִּימוֹן", "N:N--L", "Dimon", "di.mon"),
        "H1776": ("דִּימוֹנָה", "N:N--L", "Dimonah", "di.mo.nah"),
        "H1777": ("דִּין", "H:V", "to judge", "din"),
        "H1778": ("דִּין", "A:V", "to judge", "din"),
        "H1779": ("דִּין", "H:N-M", "judgment", "din"),
        "H1780": ("דִּין", "A:N-M", "judgment", "din"),
        "H1781": ("דַּיָּן", "H:N-M", "judge", "day.yan"),
        "H1782": ("דַּיָּן", "A:N-M", "judge", "day.yan"),
        "H1783": ("דִּינָה", "N:N-F-P", "Dinah", "di.nah"),
        "H1784": ("דִּינָיֵא", "N:N--LG", "Dinaite", "di.nay"),
        "H1785": ("דָּיֵק", "H:N-M", "siegework", "da.yeq"),
        "H1786": ("דַּ֫יִשׁ", "H:N-M", "threshing", "da.yish"),
        "H1787": ("דִּישֹׁן", "N:N-M-P", "Dishon", "di.shon"),
        "H1788": ("דִּישֹׁן", "H:N-M", "ibex", "di.shon"),
        "H1789": ("דִּישָׁן", "N:N-M-P", "Dishan", "di.shan"),
        "H1790": ("דַּךְ", "H:A", "crushed", "dakh"),
        "H1791": ("דֵּךְ", "A:DemP", "this", "dekh"),
        "H1792": ("דָּכָא", "H:V", "to crush", "da.kah"),
        "H1793a": ("דַּכָּא", "H:A / H:N-M", "contrite", "dak.ka"),
        "H1793b": ("דַּכָּא", "H:N-M", "dust", "dak.ka"),
        "H1794": ("דָּכָה", "H:V", "to crush", "da.khah"),
        "H1795": ("דַּכָּה", "H:N-F", "crushing", "dak.kah"),
        "H1796": ("דֳּכִי", "H:N-M", "pounding", "do.khi"),
        "H1797": ("דִּכֵּן", "A:DemP", "this", "dik.ken"),
        "H1798": ("דְּכַר", "A:N-M", "ram", "de.khar"),
        "H1799a": ("דִּכְרוֹן", "A:N", "record", "dikh.ron"),
        "H1799b": ("דָּכְרָן", "A:N", "record", ""),
        "H1800": ("דַּל", "H:A", "poor", "dal"),
        "H1801": ("דָּלַג", "H:V", "to leap", "da.lag"),
        "H1802": ("דָּלָה", "H:V", "to draw (up/out)", "da.lah"),
        "H1803a": ("דַּלָּה", "H:N-F", "hair", "dal.lah"),
        "H1803b": ("דַּלָּה", "H:N-F", "poor", "dal.lah"),
        "H1804": ("דָּלַח", "H:V", "to trouble", "da.lach"),
        "H1805": ("דְּלִי", "H:N-M", "bucket", "de.li"),
        "H1806": ("דְּלָיָ֫הוּ", "N:N-M-P", "Delaiah", "de.la.yah"),
        "H1807": ("דְּלִילָה", "N:N-F-P", "Delilah", "de.li.lah"),
        "H1808": ("דָּלִיּוֹת", "H:N-F", "branch", "da.liy.yah"),
        "H1809": ("דָּלַל", "H:V", "to languish", "da.lal"),
        "H1810": ("דִּלְעָן", "N:N--L", "Dilean", "dil.an"),
        "H1811": ("דָּלַף", "H:V", "to drip", "da.laph"),
        "H1812": ("דֶּ֫לֶף", "H:N-M", "dripping", "de.leph"),
        "H1813": ("דַּלְפוֹן", "N:N-M-P", "Dalphon", "dal.phon"),
        "H1814": ("דָּלַק", "H:V", "to burn/pursue", "da.laq"),
        "H1815": ("דְּלַק", "A:V", "to burn", "de.laq"),
        "H1816": ("דַּלֶּ֫קֶת", "H:N-F", "inflammation", "dal.le.qet"),
        "H1817a": ("דָּל", "H:N-M", "door", "de.let"),
        "H1817b": ("דָּלָה", "H:N-F", "door", "de.let"),
        "H1817c": ("דֶּ֫לֶת", "H:N-M/F", "door", "de.let"),
        "H1818": ("דָּם", "H:N-M", "blood", "dam"),
        "H1819": ("דָּמָה", "H:V", "to resemble", "da.mah"),
        "H1820": ("דָּמָה", "H:V", "to cease", "da.mah"),
        "H1821": ("דְּמָה", "A:V", "be like", "de.mah"),
        "H1822": ("דֻּמָּה", "H:N-F", "destroyed", "dum.mah"),
        "H1823": ("דְּמוּת", "H:N-F", "likeness", "de.mut"),
        "H1824": ("דֳּמִי", "H:N-M", "quiet", "de.mi"),
        "H1825": ("דִּמְיֹן", "H:N-M", "likeness", "dim.yon"),
        "H1826a": ("דָּמַם", "H:V", "to silence", "da.mam"),
        "H1826b": ("דָּמַם", "H:V", "to wail", "da.mam"),
        "H1827": ("דְּמָמָה", "H:N-F", "silence", "de.ma.mah"),
        "H1828": ("דֹּ֫מֶן", "H:N-M", "dung", "do.men"),
        "H1829": ("דִּמְנָה", "N:N--L", "Dimnah", "dim.nah"),
        "H1830": ("דָּמַע", "H:V", "to weep", "da.ma"),
        "H1831": ("דֶּ֫מַע", "H:N-M", "juice", "de.ma"),
        "H1832": ("דִּמְעָה", "H:N-F", "tears", "dim.ah"),
        "H1833": ("דְּמֶשֶׁק", "H:N-M", "silk", "de.me.sheq"),
        "H1834": ("דַּמֶּ֫שֶׂק", "N:N--L", "Damascus", "dam.me.seq"),
        "H1835": ("דָּן", "N:N-M-P / N:N--L", "Dan", "dan"),
        "H1836": ("דְּנָה", "A:DemP", "this", "den"),
        "H1837": ("דַּנָּה", "N:N--L", "Dannah", "dan.nah"),
        "H1838": ("דִּנְהָ֑בָה", "N:N--L", "Dinhabah", "din.ha.vah"),
        "H1839": ("דָּנִי", "N:N--PG", "Danite", "da.ni"),
        "H1840": ("דָּנִאֵל", "N:N-M-P", "Daniel", "da.niy.yel"),
        "H1841": ("דָּנִיֵּאל, דָּנִאֵל", "N:N-M-P", "Daniel", "da.niy.yel"),
        "H1842": ("יַעַן", "N:N--L", "Jaan", "dan ya.an"),
        "H1843": ("דֵּעַ", "H:N-M", "knowledge", "de.a"),
        "H1844": ("דֵּעָה", "H:N-F", "knowledge", "de.ah"),
        "H1845": ("דְּעוּאֵל", "N:N-M-P", "Deuel", "de.u.el"),
        "H1846": ("דָּעַךְ", "H:V", "to put out", "da.akh"),
        "H1847": ("דַּ֫עַת", "H:N", "knowledge", "da.at"),
        "H1848": ("דֳּפִי", "H:N-M", "slander", "do.phi"),
        "H1849": ("דָּפַק", "H:V", "to beat", "da.phaq"),
        "H1850": ("דׇּפְקָה", "N:N--L", "Dophkah", "doph.qah"),
        "H1851": ("דַּק", "H:A", "thin", "daq"),
        "H1852": ("דֹּק", "H:N-M", "curtain", "doq"),
        "H1853": ("דִּקְלָה", "N:N-M-P / N:N--L", "Diklah", "diq.lah"),
        "H1854": ("דָּקַק", "H:V", "to crush", "da.qaq"),
        "H1855": ("דְּקַק", "A:V", "to break up", "de.qaq"),
        "H1856": ("דָּקַר", "H:V", "to pierce", "da.qar"),
        "H1857": ("דֶּ֫קֶר", "N:N-M-P", "Dekar", "de.qer"),
        "H1858": ("דַּר", "H:N-M", "mother-of-pearl", "dar"),
        "H1859": ("דָּר", "A:N-M", "generation", "dar"),
        "H1860": ("דֵּרָאוֹן", "H:N-M", "abhorrence", "de.ra.on"),
        "H1861a": ("דׇּרְבוֹן", "H:N", "goad", "dor.von"),
        "H1861b": ("דָּֽרְבֹנָה", "H:N", "goad", "dor.von"),
        "H1862": ("דַּרְדַּע", "N:N-M-P", "Darda", "dar.da"),
        "H1863": ("דַּרְדַּר", "H:N-M", "thistle", "dar.dar"),
        "H1864": ("דָּרוֹם", "H:N-M", "south", "da.rom"),
        "H1865": ("דְּרוֹר", "H:N-M", "liberty", "de.ror"),
        "H1866": ("דְּרוֹר", "H:N-F", "swallow", "de.ror"),
        "H1867": ("דָּֽרְיָ֫וֶשׁ", "N:N-M-P", "Darius", "dar.ya.vesh"),
        "H1868": ("דָּֽרְיָ֫וֶשׁ", "N:N-M-P", "Darius", "dar.ya.vesh"),
        "H1869": ("דָּרַךְ", "H:V", "to tread", "da.rakh"),
        "H1870": ("דֶּ֫רֶךְ", "H:N-M", "way", "de.rekh"),
        "H1871": ("דַּרְכְּמוֹן", "H:N-M", "drachma", "dar.ke.mon"),
        "H1872": ("דְּרַע", "A:N-F", "arm", "de.ra"),
        "H1873": ("דָּרַע", "N:N-M-P", "Dara", "da.ra"),
        "H1874": ("דַּרְקוֹן", "N:N-M-P", "Darkon", "dar.qon"),
        "H1875": ("דָּרַשׁ", "H:V", "to seek", "da.rash"),
        "H1876": ("דָּשָׁא", "H:V", "to sprout", "da.sah"),
        "H1877": ("דֶּ֫שֶׁא", "H:N-M", "grass", "de.she"),
        "H1878": ("דָּשֵׁן", "H:V", "to prosper", "da.shen"),
        "H1879": ("דָּשֵׁן", "H:A", "fat", "da.shen"),
        "H1880": ("דֶּ֫שֶׁן", "H:N-M", "ashes", "de.shen"),
        "H1881": ("דָּת", "H:N-F", "law", "dat"),
        "H1882": ("דָּת", "A:N-F", "law", "dat"),
        "H1883": ("דֶּ֫תֶא", "A:N-M", "grass", "de.te"),
        "H1884": ("דְּתָבָר", "A:N-M", "judge", "de.ta.var"),
        "H1885": ("דָּתָן", "N:N-M-P", "Dathan", "da.tan"),
        "H1886": ("דֹּתָן", "N:N--L", "Dothan", "do.tan"),
        "H1887": ("הֵא", "H:Intj", "behold", "he"),
        "H1888a": ("הָא", "A:Intj", "behold", "he"),
        "H1888b": ("הֵא", "A:Intj", "behold!", "he"),
        "H1889": ("הֶאָח", "H:Intj", "Aha!", "he.ach"),
        "H1890": ("הַבְהָב", "H:N-M", "gift", "hav.hav"),
        "H1891": ("הָבַל", "H:V", "to become vain", "ha.val"),
        "H1892": ("הֶ֫בֶל", "H:N-M", "vanity", "he.vel"),
        "H1893": ("הֶ֫בֶל", "N:N-M-P", "Abel", "he.vel"),
        "H1894": ("הָבְנִי", "H:N-M", "ebony", "ho.ven"),
        "H1895": ("הָבַר", "H:V", "to divide", "ha.var"),
        "H1896": ("הֵגֵא", "N:N-M-P", "Hegai", "he.ge"),
        "H1897": ("הָגָה", "H:V", "to mutter", "ha.gah"),
        "H1898": ("הָגָה", "H:V", "to remove", "ha.gah"),
        "H1899": ("הֶ֫גֶה", "H:N-M", "moaning", "he.geh"),
        "H1900": ("הָגוּת", "H:N-F", "meditation", "ha.gut"),
        "H1901": ("הָגִיג", "H:N-M", "meditation", "ha.gig"),
        "H1902": ("הִגָּיוֹן", "H:N-M", "meditation", "hig.ga.von"),
        "H1903": ("הָגִין", "H:A", "suitable", "ha.gin"),
        "H1904": ("הָגָר", "N:N-F-P", "Hagar", "ha.gar"),
        "H1905": ("הַגְרִי", "N:N--LG / N:N-M-P", "Hagrite", "hag.ri"),
        "H1906": ("הֵד", "H:N-M", "shout", "hed"),
        "H1907": ("הַדָּבָר", "A:N-M", "counselor", "had.da.var"),
        "H1908": ("הֲדַד", "N:N-M-P", "Hadad", "ha.dad"),
        "H1909": ("הֲדַדְעֶ֫זֶר, הֲדַרְעֶ֫זֶר", "N:N-M-P", "Hadadezer", "ha.dad.e.zer"),
        "H1910": ("הֲדַדְרִמּוֹן", "N:N--L", "Hadadrimmon", "ha.dad.rim.mon"),
        "H1911": ("הָדָה", "H:V", "to stretch out", "ha.dah"),
        "H1912": ("הֹ֫דּוּ", "N:N--L", "India", "ho.du"),
        "H1913a": ("הֲדוֹרָם", "N:N-M-P", "Hadoram", "ha.do.ram"),
        "H1913b": ("הֲדֹרָם", "N:N-M-P", "Hadoram", ""),
        "H1914": ("הִדַּי", "N:N-M-P", "Hiddai", "hid.day"),
        "H1915": ("הָדַךְ", "H:V", "to tread down", "ha.dakh"),
        "H1916": ("הֲדֹם", "H:N-M", "footstool", "ha.dom"),
        "H1917": ("הַדָּם", "A:N-M", "piece", "had.dam"),
        "H1918": ("הֲדַס", "H:N-M", "myrtle", "ha.das"),
        "H1919": ("הֲדַסָּה", "N:N-F-P", "Hadassah", "ha.das.sah"),
        "H1920": ("הָדַף", "H:V", "to thrust", "ha.daph"),
        "H1921": ("הָדַר", "H:V", "to honor", "ha.dar"),
        "H1922": ("הֲדַר", "A:V", "to honor", "ha.dar"),
        "H1923": ("הֲדַר", "A:N-M", "honor", "ha.dar"),
        "H1924": ("הֲדַר", "N:N-M-P", "Hadar", "ha.dar"),
        "H1925": ("הֶ֫דֶר", "H:N-M", "glory", "he.der"),
        "H1926": ("הָדָר", "H:N-M", "glory", "ha.dar"),
        "H1927": ("הֲדָרָה", "H:N-F", "adornment", "ha.da.rah"),
        "H1928": ("הֲדַרְעֶ֫זֶר", "N:N-M-P", "Hadarezer", "ha.dar.e.zer"),
        "H1929": ("הָהּ", "H:Intj", "alas!", "hah"),
        "H1930": ("הוֹ", "H:Intj", "Ah!", "ho"),
        "H1931": ("הוּא", "H:PerP-CS", "he/she/it", "hu"),
        "H1932": ("הוּא", "A:PerP-CS", "he/she/it", "hu"),
        "H1933a": ("הָוָא", "H:V", "to fall", "ha.va"),
        "H1933b": ("הָוָה", "H:V", "to be", ""),
        "H1934": ("הֲוָא", "A:V", "to be", "ha.va"),
        "H1935": ("הוֹד", "H:N-M", "splendor", "hod"),
        "H1936": ("הוֹד", "N:N-M-P", "Hod", "hod"),
        "H1937": ("הוֹדְיָה", "N:N-M-P", "Hodevah", "ho.de.vah"),
        "H1938": ("הוֹדַוְיָה", "N:N-M-P", "Hodaviah", "ho.dav.yah"),
        "H1939": ("הוֹדַיְוָהוּ", "N:N-M-P", "Hodaiah", "ho.day.va.hu"),
        "H1940": ("הוֹדִיָּה", "N:N-F-P", "Hodiah [fem]", "ho.diy.yah"),
        "H1941": ("הוֹדִיָּה", "N:N-M-P", "Hodiah", "ho.diy.yah"),
        "H1942": ("הַוָּה", "H:N-F", "desire", "hav.vah"),
        "H1943": ("הֹוָה", "H:N-F", "misfortune", "ho.vah"),
        "H1944": ("הוֹהָם", "N:N-M-P", "Hoham", "ho.ham"),
        "H1945": ("הוֹי", "H:Intj", "woe!", "hoy"),
        "H1946": ("הוּךְ", "A:V", "to go", "hukh"),
        "H1947": ("הוֹלֵלָה", "H:N-F", "madness", "ho.le.lah"),
        "H1948": ("הוֹלֵלוּת", "H:N-F", "madness", "ho.le.lut"),
        "H1949": ("הוּם", "H:V", "to make noise", "hum"),
        "H1950": ("הוֹמָ֑ם", "N:N-M-P", "Homam", "ho.mam"),
        "H1951": ("הוּן", "H:V", "be ready", "hun"),
        "H1952": ("הוֹן", "H:N-M", "substance", "hon"),
        "H1953": ("יְהוֹשָׁמָע", "N:N-M-P", "Hoshama", "ho.sha.ma"),
        "H1954": ("הוֹשֵׁעַ", "N:N-M-P", "Hoshea", "ho.she.a"),
        "H1955": ("הוֹשַׁעֲיָה", "N:N-M-P", "Hoshaiah", "ho.sha.yah"),
        "H1956": ("הוֹתִיר", "N:N-M-P", "Hothir", "ho.tir"),
        "H1957": ("הָזָה", "H:V", "to dream", "ha.zah"),
        "H1958": ("הִי", "H:N-M", "woe", "hi"),
        "H1959": ("הֵידָד", "H:N-M", "shout", "he.dad"),
        "H1960": ("הֻיְּדוֹת", "H:N-F", "praise", "huy.y.dah"),
        "H1961": ("הָיָה", "H:V", "to be", "ha.yah"),
        "H1962": ("הַיָּה", "H:N-F", "calamity", "hay.yah"),
        "H1963": ("הֵיךְ", "H:Adv", "how?", "hekh"),
        "H1964": ("הֵיכָל", "H:N-M", "temple", "he.khal"),
        "H1965": ("הֵיכַל", "A:N-M", "temple", "he.khal"),
        "H1966": ("הֵילֵל", "H:N-M", "star", "he.lel"),
        "H1967": ("הֵימָם", "N:N-M-P", "Hemam", "he.mam"),
        "H1968": ("הֵימָן", "N:N-M-P", "Heman", "he.man"),
        "H1969": ("הִין", "H:N-M", "hin", "hin"),
        "H1970": ("הָכַר", "H:V", "to wrong", "ha.khar"),
        "H1971": ("הַכָּרָה", "H:N-F", "look", "hak.ka.rah"),
        "H1972": ("הָלָא", "H:V", "to cast far off", "ha.la"),
        "H1973": ("הָֽ֫לְאָה", "H:Adv", "further", "ha.le.ah"),
        "H1974": ("הִלּוּל", "H:N-M", "praise", "hil.lul"),
        "H1975": ("הַלָּז", "H:DemP", "this", "hal.laz"),
        "H1976": ("הַלָּזֶה", "H:DemP", "this", "hal.la.zeh"),
        "H1977": ("הַלֵּ֫זוּ", "H:DemP", "this", "hal.le.zu"),
        "H1978": ("הָלִיךְ", "H:N-M", "step", "ha.likh"),
        "H1979": ("הֲלִיכָה", "H:N-F", "walk", "ha.li.khah"),
        "H1980": ("הָלַךְ", "H:V", "to go", "ha.lakh"),
        "H1981": ("הֲלַךְ", "A:V", "to go", "ha.lakh"),
        "H1982": ("הֵ֫לֶךְ", "H:N-M", "traveller", "he.lekh"),
        "H1983": ("הֲלָךְ", "A:N-M", "toll", "ha.lakh"),
        "H1984a": ("הָלַל", "H:V", "to shine", "ha.lal"),
        "H1984b": ("הָלַל", "H:V", "to boast", "ha.lal"),
        "H1984c": ("הָלַל", "H:V", "to be foolish", "ha.lal"),
        "H1985": ("הִלֵּל", "N:N-M-P", "Hillel", "hil.lel"),
        "H1986": ("הָלַם", "H:V", "to smite", "ha.lam"),
        "H1987": ("הֶ֫לֶם", "N:N-M-P", "Helem", "he.lem"),
        "H1988": ("הֲלֹם", "H:Adv", "here", "ha.lom"),
        "H1989": ("הַלְמוּת", "H:N-F", "hammer", "hal.mut"),
        "H1990": ("הָ֑ם", "N:N--L", "Ham", "ham"),
        "H1991": ("הֵם", "H:N-M", "wealth", "hem"),
        "H1992": ("הֵ֫מָּה", "H:DemP", "they(masc.)", "hem"),
        "H1993": ("הָמָה", "H:V", "to roar", "ha.mah"),
        "H1994": ("הִמּוֹ", "A:DemP", "they", "him.mo"),
        "H1995a": ("הָמוֹן", "H:N-M", "crowd", "ha.mon"),
        "H1995b": ("הָמַן", "H:V", "be turbulent", ""),
        "H1996": ("הֲמוֹן גּוֹג", "N:N--L", "Hamon-gog", "ha.mon gog"),
        "H1997": ("הֲמוֹנָהֿ", "N:N--L", "Hamonah", "ha.mo.nah"),
        "H1998": ("הֶמְיָה", "H:N-F", "sound", "hem.yah"),
        "H1999": ("הֲמֻלָּה", "H:N-F", "tumult", "ha.mul.lah"),
        "H2000": ("הָמַם", "H:V", "to confuse", "ha.mam"),
        "H2001": ("הָמָן", "N:N-M-P", "Haman", "ha.man"),
        "H2002": ("הַמְנִיכָא", "A:N-M", "chain", "ham.nikh"),
        "H2003": ("הֶ֫מֶס", "H:N-M", "brushwood", "ha.mas"),
        "H2004": ("הֵן", "H:PerP-FP", "they(fem.)", "hen"),
        "H2005": ("הֵן", "H:Intj", "look!", "hen"),
        "H2006a": ("הֵן", "A:Cond", "look!", "hen"),
        "H2006b": ("לָהֵן", "A:Conj", "therefore", ""),
        "H2007": ("הֵנָּה", "H:RelP", "they(fem.)", "hen.nah"),
        "H2008": ("הֵ֫נָּה", "H:Adv", "here/thus", "hen.nah"),
        "H2009": ("הִנֵּה", "H:Intj", "behold", "hin.neh"),
        "H2010": ("הֲנָחָה", "H:N-F", "holiday", "ha.na.chah"),
        "H2011": ("הִנֹּם", "N:N--L", "Hinnom", "hin.nom"),
        "H2012": ("הֵנַע", "N:N--L", "Hena", "he.na"),
        "H2013": ("הַס", "H:V", "to silence", "ha.sah"),
        "H2014": ("הֲפוּגָה", "H:N-F", "cessation", "ha.phu.gah"),
        "H2015": ("הָפַךְ", "H:V", "to overturn", "ha.phakh"),
        "H2016": ("הֶ֫פֶךְ", "H:N-M", "contrariness", "he.phekh"),
        "H2017": ("הֹפֶךְ", "H:N-M", "perversity", "ho.phekh"),
        "H2018": ("הֲפֵכָה", "H:N-F", "overthrow", "ha.phe.khah"),
        "H2019": ("הֲפַכְפַּךְ", "H:A", "crooked", "ha.phakh.pakh"),
        "H2020": ("הַצָּלָה", "H:N-F", "deliverance", "hats.tsa.lah"),
        "H2021": ("הֹ֫צֶן", "H:N-M", "weapon", "ho.tsen"),
        "H2022": ("הַר", "H:N-M", "mountain", "har"),
        "H2023": ("הֹר", "N:N--L", "Hor", "hor"),
        "H2024": ("הָרָא", "N:N--L", "Hara", "ha.ra"),
        "H2025": ("הַרְאֵל", "H:N-M", "altar", "har.el"),
        "H2026": ("הָרַג", "H:V", "to kill", "ha.rag"),
        "H2027": ("הֶ֫רֶג", "H:N-M", "slaughter", "he.reg"),
        "H2028": ("הֲרֵגָה", "H:N-F", "slaughter", "ha.re.gah"),
        "H2029": ("הָרָה", "H:V", "to conceive", "ha.rah"),
        "H2030a": ("הָרָה", "H:A-F", "pregnant", ""),
        "H2030b": ("הָרִיָּה", "H:A-F", "pregnant", ""),
        "H2031": ("הַרְהֹר", "A:N-M", "fantasies", "har.hor"),
        "H2032": ("הֵרָיוֹן", "H:N-M", "conception", "he.ron"),
        "H2033": ("הֲרוֹרִי", "N:N--L", "Harorite", "ha.ro.ri"),
        "H2034": ("הֲרִיסָה", "H:N-F", "ruins", "ha.ri.sah"),
        "H2035": ("הֲרִיסוּת", "H:N-F", "ruins", "ha.ri.sut"),
        "H2036": ("הֹרָם", "N:N-M-P", "Horam", "ho.ram"),
        "H2037": ("הָרֻם", "N:N-M-P", "Harum", "ha.rum"),
        "H2038": ("הַרְמוֹן", "N:N--L", "Harmon", "har.mon"),
        "H2039": ("הָרָן", "N:N-M-P / N:N--L", "Haran", "ha.ran"),
        "H2040": ("הָרַס", "H:V", "to overthrow", "ha.ras"),
        "H2041": ("הֶ֫רֶס", "H:N-M", "destruction", "he.res"),
        "H2042": ("הָרָר", "H:N-M", "mountain", "ha.rar"),
        "H2043": ("הֲרָרִי", "N:N--LG / N:N-M-P", "Hararite", "ha.ra.ri"),
        "H2044": ("הָשֵׁם", "N:N-M-P", "Hashem", "ha.shem"),
        "H2045": ("הַשְׁמָעוּת", "H:N-F", "report", "hash.ma.ut"),
        "H2046": ("הִתּוּךְ", "H:N-M", "melting", "hit.tukh"),
        "H2047": ("הֲתָךְ", "N:N-M-P", "Hathach", "ha.takh"),
        "H2048a": ("הָתַל", "H:V", "to mock", "ha.tal"),
        "H2048b": ("תָּלַל", "H:V", "to deceive", ""),
        "H2049": ("הֲתֻלִֿים", "H:N-M", "mockery", "ha.tol"),
        "H2050": ("הוּת", "H:V", "to plot", "ha.tat"),
        "H2051": ("וְדָן", "N:N--L", "Vedan", "ve.dan"),
        "H2052": ("וָהֵב", "N:N--L", "Waheb", "va.hev"),
        "H2053": ("וָו", "H:N-M", "hook", "vav"),
        "H2054": ("וָזָר", "H:N-M / H:A", "guilty", "va.zar"),
        "H2055": ("וַיְזָתָא", "N:N-M-P", "Vaizatha", "vay.za.ta"),
        "H2056": ("וָלָד", "H:N-M", "child", "va.lad"),
        "H2057": ("וַנְיָה", "N:N-M-P", "Vaniah", "van.yah"),
        "H2058": ("וׇפְסִי", "N:N-M-P", "Vophsi", "voph.si"),
        "H2059": ("וַשְׁנִי", "N:N-M-P", "Vashni", "vash.ni"),
        "H2060": ("וַשְׁתִּי", "N:N-F-P", "Vashti", "vash.ti"),
        "H2061": ("זְאֵב", "H:N-M", "wolf", "ze.ev"),
        "H2062": ("זְאֵב", "N:N-M-P", "Zeeb", "ze.ev"),
        "H2063": ("זֹאת", "H:DemP", "this", "zot"),
        "H2064": ("זָבַד", "H:V", "to endow", "za.vad"),
        "H2065": ("זֶ֫בֶד", "H:N-M", "dowry", "ze.ved"),
        "H2066": ("זָבָד", "N:N-M-P", "Zabad", "za.vad"),
        "H2067": ("זַבְדִּי", "N:N-M-P", "Zabdi", "zav.di"),
        "H2068": ("זַבְדִּיאֵל", "N:N-M-P", "Zabdiel", "zav.di.el"),
        "H2069": ("זְבַדְיָהוּ", "N:N-M-P", "Zebadiah", "ze.vad.yah"),
        "H2070": ("זְבוּב", "H:N-M", "fly", "ze.vuv"),
        "H2071": ("זָבוּד", "N:N-M-P", "Zabud", "za.vud"),
        "H2072": ("זבוד", "N:N-M-P", "Zabbud", "zab.bud"),
        "H2073": ("זְבֻל", "H:N-M", "elevation", "ze.vul"),
        "H2074": ("זְבוּלֻן", "N:N-M-P", "Zebulun", "ze.vu.lun"),
        "H2075": ("זְבוּלֹנִי", "N:N--PG", "Zebulunite", "ze.vu.lo.ni"),
        "H2076": ("זָבַח", "H:V", "to sacrifice", "za.vach"),
        "H2077": ("זֶ֫בַח", "H:N-M", "sacrifice", "ze.vach"),
        "H2078": ("זֶ֫בַח", "N:N-M-P", "Zebah", "ze.vach"),
        "H2079": ("זַבַּי", "N:N-M-P", "Zabbai", "zab.bay"),
        "H2080": ("זְבוּדָה", "N:N-F-P", "Zebudah", "ze.vi.dah"),
        "H2081": ("זְבִינָא", "N:N-M-P", "Zebina", "ze.vi.na"),
        "H2082": ("זָבַל", "H:V", "to honour", "za.val"),
        "H2083": ("זְבֻל", "N:N-M-P", "Zebul", "ze.vul"),
        "H2084": ("זְבַן", "A:V", "to buy", "ze.van"),
        "H2085": ("זָג", "H:N-M", "skin", "zag"),
        "H2086": ("זֵד", "H:N-M", "arrogant", "zed"),
        "H2087": ("זָדוֹן", "H:N-M", "arrogance", "za.don"),
        "H2088": ("זֶה", "H:DemP", "this", "zeh"),
        "H2089": ("זֶה", "H:N-M", "lamb", "zeh"),
        "H2090": ("זֹה", "H:DemP", "this", "zoh"),
        "H2091": ("זָהָב", "H:N-M", "gold", "za.hav"),
        "H2092": ("זָהַם", "H:V", "to loathe", "za.ham"),
        "H2093": ("זָ֑הַם", "N:N-M-P", "Zaham", "za.ham"),
        "H2094a": ("זָהַר", "H:V", "to shine", "za.har"),
        "H2094b": ("זָהַר", "H:V", "to warn", "za.har"),
        "H2095": ("זְהַר", "A:V", "to take heed", "ze.har"),
        "H2096": ("זֹ֫הַר", "H:N-M", "brightness", "zo.har"),
        "H2097": ("זוֹ", "H:DemP", "this", "zo"),
        "H2098": ("זוּ", "H:DemP", "this", "zu"),
        "H2099": ("זִו", "N:N-M-T", "Ziv", "ziv"),
        "H2100": ("זוּב", "H:V", "to flow", "zuv"),
        "H2101": ("זוֹב", "H:N-M", "discharge", "zov"),
        "H2102": ("זִיד", "H:V", "to boil", "zud"),
        "H2103": ("זוּד", "A:V", "be proud", "zud"),
        "H2104": ("זוּזִים", "N:N--LG", "Zuzim", "zu.zim"),
        "H2105": ("זוֹחֵת", "N:N-M-P", "Zoheth", "zo.chet"),
        "H2106": ("זָוִית", "H:N-F", "corner", "za.vit"),
        "H2107": ("זוּל", "H:V", "to lavish/despise", "zul"),
        "H2108": ("זוּלָה", "H:N-F", "exception", "zu.lah"),
        "H2109a": ("זוּן", "H:V", "to feed", "zun"),
        "H2109b": ("יָזַן", "H:V", "to feed", ""),
        "H2110": ("זוּן", "A:V", "to feed", "zun"),
        "H2111": ("זוּעַ", "H:V", "to tremble", "zu.a"),
        "H2112": ("זוּעַ", "A:V", "to tremble", "zu.a"),
        "H2113": ("זְוָעָה", "H:N-F", "trembling", "ze.va.ah"),
        "H2114a": ("זוּר", "H:V", "be a stranger", "zur"),
        "H2114b": ("זוּר", "H:V", "be loathsome", "zur"),
        "H2115": ("זוּר", "H:V", "to crush", "zur"),
        "H2116": ("זוּרֶה", "H:N-F", "crushed", "zu.reh"),
        "H2117": ("זָזָא", "N:N-M-P", "Zaza", "za.za"),
        "H2118": ("זָחַח", "H:V", "to remove", "za.chach"),
        "H2119a": ("זָחַל", "H:V", "to crawl", "za.chal"),
        "H2119b": ("זָחַל", "H:V", "to fear", "za.chal"),
        "H2120": ("זֹחֶ֫לֶת", "N:N--L", "Zoheleth", "zo.che.let"),
        "H2121": ("זֵידוֹן", "H:A", "raging", "ze.don"),
        "H2122": ("זִיו", "A:N-M", "splendor", "ziv"),
        "H2123a": ("זִיז", "H:N-M", "creature", "ziz"),
        "H2123b": ("זִיז", "H:N-M", "abundance", "ziz"),
        "H2124": ("זִיזָא", "N:N-M-P", "Ziza", "zi.za"),
        "H2125": ("זִיזָה", "N:N-M-P", "Zizah", "zi.zah"),
        "H2126": ("זִינָא", "N:N-M-P", "Zina", "zi.na"),
        "H2127": ("זִיעַ", "N:N-M-P", "Zia", "zi.a"),
        "H2128": ("זִיף", "N:N-M-P / N:N--L", "Ziph", "ziph"),
        "H2129": ("זִיפָה", "N:N-M-P", "Ziphah", "zi.phah"),
        "H2130": ("זִיפִי", "N:N--LG", "Ziphite", "zi.phi"),
        "H2131a": ("זֵק", "H:N-M", "missile", "zi.qah"),
        "H2131b": ("זֵק", "H:N-M", "fetter", "zi.qah"),
        "H2132": ("זַ֫יִת", "H:N-M", "olive", "za.yit"),
        "H2133": ("זֵיתָן", "N:N-M-P", "Zethan", "ze.tan"),
        "H2134": ("זַךְ", "H:A", "pure", "zakh"),
        "H2135": ("זָכָה", "H:V", "to clean", "za.khah"),
        "H2136": ("זָכוּ", "A:N-F", "innocence", "za.khu"),
        "H2137": ("זְכוּכִית", "H:N-F", "glass", "ze.khu.khit"),
        "H2138": ("זָכוּר", "H:N-M", "male", "za.khur"),
        "H2139": ("זַכּוּר", "N:N-M-P", "Zaccur", "zak.kur"),
        "H2140": ("זַכַּי", "N:N-M-P", "Zaccai", "zak.kay"),
        "H2141": ("זָכַךְ", "H:V", "be clean", "za.khakh"),
        "H2142": ("זָכַר", "H:V", "to remember", "za.khar"),
        "H2143": ("זֵ֫כֶר", "H:N-M", "memorial", "ze.kher"),
        "H2144": ("זֶ֫כֶר", "N:N-M-P", "Zecher", "ze.kher"),
        "H2145": ("זָכָר", "H:N-M", "male", "za.khar"),
        "H2146": ("זִכָּרוֹן", "H:N-M", "memorial", "zikh.ron"),
        "H2147": ("זִכְרִי", "N:N-M-P", "Zichri", "zikh.ri"),
        "H2148a": ("זְכַרְיָהוּ", "N:N-M-P", "Zechariah", "ze.khar.yah"),
        "H2148b": ("זְכַרְיָה", "N:N-M-P", "Zechariah", "ze.khar.yah"),
        "H2149": ("זֻלּוּת", "H:N-M", "vileness", "zul.lut"),
        "H2150": ("זַלְזַל", "H:N-M", "tendril", "zal.zal"),
        "H2151a": ("זָלַל", "H:V", "to shake", "za.lal"),
        "H2151b": ("זָלַל", "H:V", "be vile", "za.lal"),
        "H2152": ("זַלְעָפָה", "H:N-F", "scorching", "zal.a.phah"),
        "H2153": ("זִלְפָּה", "N:N-F-P", "Zilpah", "zil.pah"),
        "H2154": ("זִמָּה", "H:N-F", "wickedness", "zim.mah"),
        "H2155": ("זִמָּה", "N:N-M-P", "Zimmah", "zim.mah"),
        "H2156": ("זְמוֹרָה", "H:N", "branch", "ze.mo.rah"),
        "H2157": ("זַמְזֻמִּים", "N:N--LG", "Zamzummin", "zam.zom"),
        "H2158": ("זָמִיר", "H:N-M", "song", "za.mir"),
        "H2159": ("זָמִיר", "H:N-M", "branch", "za.mir"),
        "H2160": ("זְמִירָה", "N:N-M-P", "Zemirah", "ze.mi.rah"),
        "H2161": ("זָמַם", "H:V", "to plan", "za.mam"),
        "H2162": ("זָמָם", "H:N-M", "plan", "za.mam"),
        "H2163": ("זָמַן", "H:V", "to appoint", "za.man"),
        "H2164": ("זְמַן", "A:V", "to agree", "ze.man"),
        "H2165": ("זְמָ֑ן", "H:N-M", "time", "ze.man"),
        "H2166": ("זְמַן", "A:N-M", "time", "ze.man"),
        "H2167": ("זָמַר", "H:V", "to sing", "za.mar"),
        "H2168": ("זָמַר", "H:V", "to prune", "za.mar"),
        "H2169": ("זֶ֫מֶר", "H:N-M", "mountain goat", "ze.mer"),
        "H2170": ("זְמָר", "A:N-M", "music", "ze.mar"),
        "H2171": ("זַמָּר", "A:N-M", "singer", "zam.mar"),
        "H2172": ("זִמְרָה", "H:N-F", "melody", "zim.rah"),
        "H2173": ("זִמְרָה", "H:N-F", "best", "zim.rah"),
        "H2174a": ("זִמְרִי", "N:N-M-P", "Zimri", "zim.ri"),
        "H2174b": ("זִמְרִי", "N:N--LG / N:N-M-P", "Zimri", "zim.ri"),
        "H2175": ("זִמְרָן", "N:N-M-P", "Zimran", "zim.ran"),
        "H2176": ("זִמְרָת", "H:N-F", "song", "zim.rat"),
        "H2177": ("זַן", "H:N-M", "kind", "zan"),
        "H2178": ("זַן", "A:N-M", "kind", "zan"),
        "H2179": ("זָנַב", "H:V", "to cut off the tail", "za.nav"),
        "H2180": ("זָנָב", "H:N-M", "tail", "za.nav"),
        "H2181": ("זָנָה", "H:V", "to fornicate", "za.nah"),
        "H2182": ("זָנוֹחַ", "N:N--L", "Zanoah", "za.no.ach"),
        "H2183": ("זְנוּנִים", "H:N-M", "fornication", "za.nun"),
        "H2184": ("זְנוּת", "H:N-F", "fornication", "ze.nut"),
        "H2185": ("זֹנוֹת", "H:N-F", "prostitute", "zo.not"),
        "H2186a": ("זָנַח", "H:V", "to reject", "za.nach"),
        "H2186b": ("זָנַח", "H:V", "to stink", "za.nach"),
        "H2187": ("זָנַק", "H:V", "to leap", "za.naq"),
        "H2188": ("זֵעָה", "H:N-F", "sweat", "ze.ah"),
        "H2189": ("זַעֲוָה", "H:N-F", "horror", "za.a.vah"),
        "H2190": ("זַעֲוָן", "N:N-M-P", "Zaavan", "za.a.van"),
        "H2191": ("זְעֵיר", "H:N-M", "little", "ze.er"),
        "H2192": ("זְעֵיר", "A:A", "little", "ze.er"),
        "H2193": ("זָעַךְ", "H:V", "to extinguish", "za.akh"),
        "H2194": ("זָעַם", "H:V", "be indignant", "za.am"),
        "H2195": ("זַ֫עַם", "H:N-M", "indignation", "za.am"),
        "H2196": ("זָעַף", "H:V", "to enrage", "za.aph"),
        "H2197": ("זַ֫עַף", "H:N-M", "rage", "za.aph"),
        "H2198": ("זָעֵף", "H:A", "vexed", "za.eph"),
        "H2199": ("זָעַק", "H:V", "to cry out", "za.aq"),
        "H2200": ("זְעִ֑ק", "A:V", "to cry out", "ze.iq"),
        "H2201": ("זְעָקָה", "H:N-F", "outcry", "za.aq"),
        "H2202": ("זִפְרֹן", "N:N--L", "Ziphron", "ziph.ron"),
        "H2203": ("זֶ֫פֶת", "H:N-F", "pitch", "ze.phet"),
        "H2204": ("זָקֵן", "H:V", "be old", "za.qen"),
        "H2205": ("זָקֵן", "H:A", "old", "za.qen"),
        "H2206": ("זָקָן", "H:N-M", "beard", "za.qan"),
        "H2207": ("זֹ֫קֶן", "H:N-M", "old age", "zo.qen"),
        "H2208": ("זְקֻנִים", "H:N-M", "extreme age", "za.qun"),
        "H2209": ("זִקְנָה", "H:N-F", "old age", "ziq.nah"),
        "H2210": ("זָקַף", "H:V", "to raise", "za.qaph"),
        "H2211": ("זְקַף", "A:V", "to raise", "ze.qaph"),
        "H2212": ("זָקַק", "H:V", "to refine", "za.qaq"),
        "H2213": ("זֵר", "H:N-M", "border", "zer"),
        "H2214": ("זָרָא", "H:N-M", "loathsome", "za.ra"),
        "H2215": ("זָרַב", "H:V", "to burn", "za.rav"),
        "H2216": ("זְרֻבָּבֶ֫ל", "N:N-M-P", "Zerubbabel", "ze.rub.ba.vel"),
        "H2217": ("זְרֻבָּבֶ֫ל", "N:N-M-P", "Zerubbabel", "ze.rub.ba.vel"),
        "H2218": ("זֶ֫רֶד", "N:N--L", "Zered", "ze.red"),
        "H2219": ("זָרָה", "H:V", "to scatter", "za.rah"),
        "H2220": ("זְרוֹעַ", "H:N-F", "arm", "ze.ro.a"),
        "H2221": ("זֵרוּעַ", "H:N-M", "sowing", "ze.ru.a"),
        "H2222a": ("זָרַף", "H:V", "drip", "zar.ziph"),
        "H2222b": ("זַרְזִיף", "H:N-M", "drip", "zar.ziph"),
        "H2223": ("זַרְזִיר", "H:N-M", "greyhound", "zar.zir"),
        "H2224": ("זָרַח", "H:V", "to rise", "za.rach"),
        "H2225": ("זֶ֫רַח", "H:N-M", "dawning", "ze.rach"),
        "H2226": ("זֶ֫רַח", "N:N-M-P", "Zerah", "ze.rach"),
        "H2227": ("זַרְחִי", "N:N--PG", "Zarhites", "zar.chi"),
        "H2228": ("זְרַחְיָה", "N:N-M-P", "Zerahiah", "ze.rach.yah"),
        "H2229": ("זָרַם", "H:V", "to flood", "za.ram"),
        "H2230": ("זֶ֫רֶם", "H:N-M", "storm", "ze.rem"),
        "H2231": ("זִרְמָה", "H:N-F", "discharge", "zir.mah"),
        "H2232": ("זָרַע", "H:V", "to sow", "za.ra"),
        "H2233": ("זֶ֫רַע", "H:N-M", "seed", "ze.ra"),
        "H2234": ("זְרַע", "A:N-M", "seed", "ze.ra"),
        "H2235a": ("זֵרֹעַ", "H:N-M", "vegetable", "ze.ro.a"),
        "H2235b": ("זֵֽרְעֹן", "H:N-M", "vegetable", ""),
        "H2236": ("זָרַק", "H:V", "to scatter", "za.raq"),
        "H2237": ("זָרַר", "H:V", "to sneeze", "za.rar"),
        "H2238": ("זֶ֫רֶשׁ", "N:N-F-P", "Zeresh", "ze.resh"),
        "H2239": ("זֶ֫רֶת", "H:N-F", "span", "ze.ret"),
        "H2240": ("זַתּוּא", "N:N-M-P", "Zattu", "zat.tu"),
        "H2241": ("זֵתָם", "N:N-M-P", "Zetham", "ze.tam"),
        "H2242": ("זֵתַר", "N:N-M-P", "Zethar", "ze.tar"),
        "H2243": ("חֹב", "H:N-M", "breast", "chov"),
        "H2244": ("חָבָא", "H:V", "to hide", "cha.va"),
        "H2245": ("חָבַב", "H:V", "to love", "cha.vav"),
        "H2246": ("חֹבָב", "N:N-M-P", "Hobab", "cho.vav"),
        "H2247": ("חָבָה", "H:V", "to hide", "cha.vah"),
        "H2248": ("חֲבוּלָא", "A:N-M", "crime", "cha.vu.lah"),
        "H2249": ("חָבוֹר", "N:N--L", "Habor", "cha.vor"),
        "H2250": ("חַבּוּרָה", "H:N-F", "wound", "chab.bu.rah"),
        "H2251": ("חָבַט", "H:V", "to beat", "cha.vat"),
        "H2252": ("חֲבַיָה", "N:N-M-P", "Habaiah", "cha.vay.yah"),
        "H2253": ("חֶבְיוֹן", "H:N-M", "hiding", "chev.yon"),
        "H2254a": ("חָבַל", "H:V", "to pledge", "cha.val"),
        "H2254b": ("חָבַל", "H:V", "to destroy", "cha.val"),
        "H2254c": ("חָבַל", "H:V", "be in labour", "cha.val"),
        "H2255": ("חֲבַל", "A:V", "to destroy", "cha.val"),
        "H2256a": ("חֶ֫בֶל", "H:N-M", "cord", "che.vel"),
        "H2256b": ("חֵ֫בֶל", "H:N-M", "pain", ""),
        "H2256c": ("חֹבְלִים", "H:N-M", "union", ""),
        "H2256d": ("חֶ֫בֶל", "H:N-M", "destruction", "che.vel"),
        "H2257": ("חֲבָל", "A:N-M", "harm", "cha.val"),
        "H2258a": ("חֲבֹל", "H:N-M", "pledge", "cha.vol"),
        "H2258b": ("חֲבֹלָה", "H:N-M", "pledge", ""),
        "H2259": ("חֹבֵל", "H:N-M", "pilot", "cho.vel"),
        "H2260": ("חִבֵּל", "H:N-M", "mast", "chib.bel"),
        "H2261": ("חֲבַצֶּלֶת", "H:N-F", "crocus", "cha.vats.tse.let"),
        "H2262": ("חֲבַצִּנְיָה", "N:N-M-P", "Habazziniah", "cha.vats.tsan.yah"),
        "H2263": ("חָבַק", "H:V", "to embrace", "cha.vaq"),
        "H2264": ("חִבֻּק", "H:N-M", "folding", "chib.buq"),
        "H2265": ("חֲבַקּוּק", "N:N-M-P", "Habakkuk", "cha.vaq.quq"),
        "H2266": ("חָבַר", "H:V", "to unite", "cha.var"),
        "H2267": ("חֶ֫בֶר", "H:N-M", "spell", "che.ver"),
        "H2268": ("חֶ֫בֶר", "N:N-M-P", "Heber", "che.ver"),
        "H2269": ("חֲבַר", "A:N-M", "fellow", "cha.var"),
        "H2270": ("חָבֵר", "H:A / H:N-M", "companion", "cha.ver"),
        "H2271": ("חַבָּר", "H:N-M", "associate", "chab.bar"),
        "H2272": ("חֲבַרְבֻּרָה", "H:N-F", "spot", "cha.var.bu.rah"),
        "H2273": ("חַבְרָה", "A:N-F", "associate", "chav.rah"),
        "H2274": ("חֶבְרָה", "H:N-F", "company", "chev.rah"),
        "H2275a": ("חֶבְרוֹן", "N:N--L", "Hebron", "chev.ron"),
        "H2275b": ("חֶבְרוֹן", "N:N-M-P", "Hebron", "chev.ron"),
        "H2276": ("חֶבְרוֹנִי", "N:N--LG / N:N-M-P", "Hebronite", "chev.ro.ni"),
        "H2277": ("חֶבְרִי", "N:N--PG", "Heberites", "chev.ri"),
        "H2278": ("חֲבֶ֫רֶת", "H:N-F", "consort", "cha.ve.ret"),
        "H2279": ("חֹבֶ֫רֶת", "H:N-F", "set", "cho.ve.ret"),
        "H2280": ("חָבַשׁ", "H:V", "to saddle/tie", "cha.vash"),
        "H2281": ("חֲבִתִּים", "H:N-M", "flatbread", "cha.vet"),
        "H2282": ("חַג", "H:N-M", "feast", "chag"),
        "H2283": ("חָגָּא", "H:N-F", "terror", "cha.ga"),
        "H2284": ("חָגָב", "H:N-M", "locust", "cha.gav"),
        "H2285": ("חָגָב", "N:N-M-P", "Hagab", "cha.gav"),
        "H2286a": ("חֲגָבָא", "N:N-M-P", "Hagaba", "cha.ga.va"),
        "H2286b": ("חֲגָבָה", "N:N-M-P", "Hagabah", "cha.ga.va"),
        "H2287": ("חָגַג", "H:V", "to celebrate", "cha.gag"),
        "H2288": ("חֲגָוִים", "H:N-M", "cleft", "cha.gav"),
        "H2289": ("חָגוֹר", "H:A-M", "blelted", "cha.gor"),
        "H2290a": ("חֲגוֹר", "H:N-M/F", "belt", "cha.gor"),
        "H2290b": ("חֲגוֹרָה", "H:N-M/F", "belt", ""),
        "H2291": ("חַגִּי", "N:N-M-P / N:N-M-PG", "Haggi", "chag.gi"),
        "H2292a": ("חַגַּי", "N:N-M-P", "Haggai", "chag.gay"),
        "H2292b": ("חַגַּי", "N:N-M-P", "Haggai", "chag.gay"),
        "H2293": ("חַגִּיָּה", "N:N-M-P", "Haggiah", "chag.giy.yah"),
        "H2294": ("חַגִּית", "N:N-F-P", "Haggith", "chag.git"),
        "H2295": ("חׇגְלָה", "N:N-F-P", "Hoglah", "chog.lah"),
        "H2296": ("חָגַר", "H:V", "to gird", "cha.gar"),
        "H2297": ("חַד", "H:A", "one", "chad"),
        "H2298": ("חַד", "A:A", "one", "chad"),
        "H2299": ("חַד", "H:A", "sharp", "chad"),
        "H2300a": ("חָדַד", "H:V", "be sharp", "cha.dad"),
        "H2300b": ("חָדַה", "H:V", "to sharpen", ""),
        "H2301": ("חֲדַד", "N:N-M-P", "Hadad", "cha.dad"),
        "H2302": ("חָדָה", "H:V", "to rejoice", "cha.dah"),
        "H2303": ("חַדּוּד", "H:A", "sharp", "chad.dud"),
        "H2304": ("חֶדְוָה", "H:N-F", "joy", "ched.vah"),
        "H2305": ("חֶדְוָה", "A:N-F", "joy", "ched.vah"),
        "H2306": ("חֲדִי", "A:N-M", "breast", "cha.di"),
        "H2307": ("חָדִיד", "N:N--L", "Hadid", "cha.did"),
        "H2308": ("חָדַל", "H:V", "to cease", "cha.dal"),
        "H2309": ("חֶ֫דֶל", "H:N-M", "world", "che.del"),
        "H2310": ("חָדֵל", "H:A", "rejected/fleeting", "cha.del"),
        "H2311": ("חַדְלָ֑י", "N:N-M-P", "Hadlai", "chad.lay"),
        "H2312": ("חֵ֫דֶק", "H:N-M", "briar", "che.deq"),
        "H2313": ("חִדֶּ֫קֶל", "N:N--L", "Tigris", "chid.de.qel"),
        "H2314": ("חָדַר", "H:V", "to surround", "cha.dar"),
        "H2315": ("חֶ֫דֶר", "H:N-M", "chamber", "che.der"),
        "H2316": ("חֲדַר", "N:N-M-P", "Hadar", "cha.dar"),
        "H2317": ("חַדְרָ֔ךְ", "N:N--L", "Hadrach", "chad.rakh"),
        "H2318": ("חָדַשׁ", "H:V", "to renew", "cha.dash"),
        "H2319": ("חָדָשׁ", "H:A", "new", "cha.dash"),
        "H2320": ("חֹ֫דֶשׁ", "H:N-M", "month", "cho.desh"),
        "H2321": ("חֹ֫דֶשׁ", "N:N-F-P", "Hodesh", "cho.desh"),
        "H2322": ("חֲדָשָׁה", "N:N--L", "Hadashah", "cha.da.shah"),
        "H2323": ("חֲדַ֑ת", "A:A", "new", "cha.dat"),
        "H2324": ("חֲוָה", "A:V", "to show", "cha.va"),
        "H2325": ("חוּב", "H:V", "to endanger", "chuv"),
        "H2326": ("חוֹב", "H:N-M", "debtor", "chov"),
        "H2327": ("חוֹבָה", "N:N--L", "Hobah", "cho.vah"),
        "H2328": ("חוּג", "H:V", "to mark", "chug"),
        "H2329": ("חוּג", "H:N-M", "circle", "chug"),
        "H2330": ("חוּד", "H:V", "to riddle", "chud"),
        "H2331": ("חָוָה", "H:V", "to tell", "cha.vah"),
        "H2332": ("חַוָּה", "N:N-F-P", "Eve", "chav.vah"),
        "H2333": ("חַוָּה", "H:N-F", "village", "chav.vah"),
        "H2334": ("חַוּוֹת יָעִיר", "N:N--L", "Havvoth-jair", "chau.vot ya.ir"),
        "H2335": ("חוֹזַי", "N:N-M-P", "seer", "cho.zay"),
        "H2336": ("חוֹחַ", "H:N-M", "thistle", "cho.ach"),
        "H2337": ("חָוָח", "H:N-M", "thicket", "cha.vach"),
        "H2338": ("חוּט", "A:V", "to repair", "chut"),
        "H2339": ("חוּט", "H:N-M", "thread", "chut"),
        "H2340": ("חִוִּי", "N:N--LG", "Hivite", "chiv.vi"),
        "H2341": ("חֲוִילָה", "N:N--L / N:N-M-P", "Havilah", "cha.vi.lah"),
        "H2342a": ("חוּל", "H:V", "to twist", "chul"),
        "H2342b": ("חִיל", "H:V", "be firm", ""),
        "H2343": ("חוּל", "N:N-M-P", "Hul", "chul"),
        "H2344": ("חוֹל", "H:N-M", "sand", "chol"),
        "H2345": ("חוּם", "H:A", "darkened", "chum"),
        "H2346": ("חוֹמָה", "H:N-F", "wall", "cho.mah"),
        "H2347": ("חוּס", "H:V", "to pity", "chus"),
        "H2348": ("חוֹף", "H:N-M", "coast", "choph"),
        "H2349": ("חוּפָם", "N:N-M-P", "Hupham", "chu.pham"),
        "H2350": ("חוּפָמִי", "N:N--PG", "Huphamites", "chu.pha.mi"),
        "H2351": ("חוּץ", "H:N-M", "outside", "chuts"),
        "H2352": ("חֻר", "H:N-M", "cavern", "chur"),
        "H2353": ("חוּר", "H:N-M", "white", "chur"),
        "H2354": ("חוּר", "N:N-M-P", "Hur", "chur"),
        "H2355": ("חוֹרָ֑י", "H:N-M", "white cloth", "chor"),
        "H2356": ("חֹר", "H:N-M", "hole", "chor"),
        "H2357": ("חָוַר", "H:V", "to pale", "cha.var"),
        "H2358": ("חִוָּר", "A:A", "white", "chiv.var"),
        "H2359": ("חוּרִי", "N:N-M-P", "Huri", "chu.ri"),
        "H2360": ("חוּרַי", "N:N-M-P", "Hurai", "chu.ray"),
        "H2361": ("חוּרָם", "N:N-M-P", "Huram", "chu.ram"),
        "H2362": ("חַוְרָן", "N:N--L", "Hauran", "chav.ran"),
        "H2363a": ("חוּשׁ", "H:V", "to hasten", "chush"),
        "H2363b": ("חוּשׁ", "H:V", "to enjoy", "chush"),
        "H2364": ("חוּשָׁה", "N:N-M-P", "Hushah", "chu.shah"),
        "H2365": ("חוּשַׁי", "N:N-M-P", "Hushai", "chu.shay"),
        "H2366a": ("חוּשִׁים", "N:N-F-P", "Hushim", "chu.shim"),
        "H2366b": ("חֻשִׁים", "N:N-M-P", "Hushim", ""),
        "H2367": ("חֻשָׁם", "N:N-M-P", "Husham", "chu.sham"),
        "H2368": ("חֹתָם", "H:N-M", "signet", "cho.tam"),
        "H2369": ("חוֹתָם", "N:N-M-P", "Hotham", "cho.tam"),
        "H2370": ("חֲזָה", "A:V", "to see", "cha.za"),
        "H2371": ("חֲזָהאֵל", "N:N-M-P", "Hazael", "cha.za.el"),
        "H2372": ("חָזָה", "H:V", "to see", "cha.zah"),
        "H2373": ("חָזֶה", "H:N-M", "breast", "cha.zeh"),
        "H2374": ("חֹזֶה", "H:N-M", "seer", "cho.zeh"),
        "H2375": ("חֲזוֹ", "N:N-M-P", "Hazo", "cha.zo"),
        "H2376": ("חֱזוּ", "A:N-M", "vision", "che.zev"),
        "H2377": ("חָזוֹן", "H:N-M", "vision", "cha.zon"),
        "H2378": ("חָזוֹת", "H:N-F", "vision", "cha.zot"),
        "H2379": ("חֲזוֹת", "A:N-F", "visibility", "cha.zot"),
        "H2380": ("חָזוּת", "H:N-F", "vision", "cha.zut"),
        "H2381": ("חֲזִיאֵל", "N:N-M-P", "Haziel", "cha.zi.el"),
        "H2382": ("חֲזָיָה", "N:N-M-P", "Hazaiah", "cha.za.yah"),
        "H2383": ("חֶזְיוֹן", "N:N-M-P", "Hezion", "chez.yon"),
        "H2384": ("חִזָּיוֹן", "H:N-M", "vision", "chiz.za.von"),
        "H2385": ("חֲזִיז", "H:N-M", "lightning", "cha.ziz"),
        "H2386": ("חֲזִיר", "H:N-M", "swine", "cha.zir"),
        "H2387": ("חֵזִיר", "N:N-M-P", "Hezir", "che.zir"),
        "H2388": ("חָזַק", "H:V", "to strengthen", "cha.zaq"),
        "H2389": ("חָזָק", "H:A", "strong", "cha.zaq"),
        "H2390": ("חָזֵק", "H:A", "stronger", "cha.zeq"),
        "H2391": ("חֵ֫זֶק", "H:N-M", "strength", "che.zeq"),
        "H2392": ("חֹ֫זֶק", "H:N-M", "strength", "cho.zeq"),
        "H2393": ("חֶזְקָה", "H:N-F", "strength", "chez.qah"),
        "H2394": ("חׇזְקָה", "H:N-F", "force", "choz.qah"),
        "H2395": ("חִזְקִי", "N:N-M-P", "Hizki", "chiz.qi"),
        "H2396": ("חִזְקִיָּהוּ", "N:N-M-P", "Hezekiah", "chiz.qiy.yah"),
        "H2397": ("חַח", "H:N-M", "hook", "chach"),
        "H2398": ("חָטָא", "H:V", "to sin", "cha.ta"),
        "H2399": ("חֵטְא", "H:N-M", "sin", "chet"),
        "H2400": ("חַטָּא", "H:N-M / H:A", "sinner", "chat.ta"),
        "H2401": ("חֲטָאָה", "H:N-F", "sin", "cha.ta.ah"),
        "H2402": ("חַטָּאָה", "A:N-F", "sin offering", "chat.ta.ah"),
        "H2403a": ("חַטָּאָה", "H:N-F", "sin", "chat.ta.ah"),
        "H2403b": ("חַטָּאת", "H:N-F", "sin", ""),
        "H2404": ("חָטַב", "H:V", "to chop", "cha.tav"),
        "H2405": ("חֲטֻבוֹת", "H:N-F", "colored", "cha.tu.vah"),
        "H2406": ("חִטָּה", "H:N-F", "wheat", "chit.tah"),
        "H2407": ("חַטּוּשׁ", "N:N-M-P", "Hattush", "chat.tush"),
        "H2408": ("חֲטָי", "A:N-M", "sin", "cha.ti"),
        "H2409": ("חַטָּיָא", "A:N-F", "sin offering", "cha.t.ta.ya"),
        "H2410": ("חֲטִיטָא", "N:N-M-P", "Hatita", "cha.ti.ta"),
        "H2411": ("חַטִּיל", "N:N-M-P", "Hattil", "chat.til"),
        "H2412": ("חֲטִיפָא", "N:N-M-P", "Hatipha", "cha.ti.pah"),
        "H2413": ("חָטַם", "H:V", "to restrain", "cha.tam"),
        "H2414": ("חָטַף", "H:V", "to catch", "cha.taph"),
        "H2415": ("חֹטֵר", "H:N-M", "branch", "cho.ter"),
        "H2416a": ("חַי", "H:A", "alive", "chay"),
        "H2416b": ("חַי", "H:N-M", "kinsfolk", "chay"),
        "H2416c": ("חַיָּה", "H:N-F", "living thing", ""),
        "H2416d": ("חַיָּה", "H:N-F", "community", ""),
        "H2416e": ("חַיִּים", "H:N-M", "life", ""),
        "H2417": ("חַי", "A:A", "living", "chay"),
        "H2418": ("חֲיָא", "A:V", "to live", "cha.ya"),
        "H2419": ("חִיאֵל", "N:N-M-P", "Hiel", "chi.el"),
        "H2420": ("חִידָה", "H:N-F", "riddle", "chi.dah"),
        "H2421": ("חָיָה", "H:V", "to live", "cha.yah"),
        "H2422": ("חָיֶה", "H:A", "vigorous", "cha.yeh"),
        "H2423": ("חֵיוָא", "A:N-F", "beast", "che.va"),
        "H2424": ("חַיּוּת", "H:N-F", "living", "chay.yut"),
        "H2425": ("חָיַי", "H:V", "to live", "cha.yay"),
        "H2426": ("חֵל", "H:N-M", "rampart", "chel"),
        "H2427a": ("חִיל", "H:N-M", "agony", "chil"),
        "H2427b": ("חִילָה", "H:N-M", "agony", ""),
        "H2428": ("חַ֫יִל", "H:N-M", "strength", "cha.yil"),
        "H2428b": ("חֵילֵךְ", "N:N--L", "Helech", "che.lek"),
        "H2429": ("חַ֫יִל", "A:N-M", "strength", "cha.yil"),
        "H2430": ("חֵילָה", "H:N-F", "bulwark", "che.lah"),
        "H2431": ("חֵילָם", "N:N--L", "Helam", "che.lam"),
        "H2432": ("חִילֵן", "N:N--L", "Hilen", "chi.len"),
        "H2433": ("חִין", "H:N-M", "beauty", "chin"),
        "H2434": ("חַ֫יִץ", "H:N-M", "wall", "cha.yits"),
        "H2435": ("חִיצוֹן", "H:A", "outer", "chi.tson"),
        "H2436": ("חֵיק", "H:N-M", "bosom", "cheq"),
        "H2437": ("חִירָה", "N:N-M-P", "Hirah", "chi.rah"),
        "H2438": ("חִירָם, חוּרָם", "N:N-M-P", "Hiram", "chi.ram"),
        "H2439": ("חִישׁ", "H:V", "to haste", "chish"),
        "H2440": ("חִישׁ", "H:Adv", "quickly", "chish"),
        "H2441": ("חֵךְ", "H:N-M", "palate", "chekh"),
        "H2442": ("חָכָה", "H:V", "to wait", "cha.khah"),
        "H2443": ("חַכָּה", "H:N-F", "hook", "chak.kah"),
        "H2444": ("חֲכִילָה", "N:N--L", "Hachilah", "cha.khi.lah"),
        "H2445": ("חַכִּים", "A:N-M", "wise", "chak.kim"),
        "H2446": ("חֲכַלְיָה", "N:N-M-P", "Hacaliah", "cha.khal.yah"),
        "H2447": ("חַכְלִילִי", "H:A-M", "dull", "chakh.lil"),
        "H2448": ("חַכְלִלוּת", "H:N-F", "dullness", "chakh.li.lut"),
        "H2449": ("חָכַם", "H:V", "be wise", "cha.kham"),
        "H2450": ("חָכָם", "H:A", "wise", "cha.kham"),
        "H2451": ("חׇכְמָה", "H:N-F", "wisdom", "chokh.mah"),
        "H2452": ("חׇכְמָה", "A:N-F", "wisdom", "chokh.mah"),
        "H2453": ("חַכְמוֹנִי", "N:N-M-P", "Hachmoni", "chakh.mo.ni"),
        "H2454": ("חׇכְמוֹת", "H:N-F", "wisdom", "chokh.mot"),
        "H2455": ("חֹל", "H:N-M", "common", "chol"),
        "H2456": ("חָלָא", "H:V", "be sick", "cha.la"),
        "H2457": ("חֶלְאָה", "H:N-F", "rust", "chel.ah"),
        "H2458": ("חֶלְאָה", "N:N-F-P", "Helah", "chel.ah"),
        "H2459": ("חֵ֫לֶב", "H:N-M", "fat", "che.lev"),
        "H2460": ("חֵ֫לֶב", "N:N-M-P", "Heleb", "che.lev"),
        "H2461": ("חָלָב", "H:N-M", "milk", "cha.lav"),
        "H2462": ("חֶלְבָּה", "N:N--L", "Helbah", "chel.bah"),
        "H2463": ("חֶלְבּוֹן", "N:N--L", "Helbon", "chel.bon"),
        "H2464": ("חֶלְבְּנָה", "H:N-F", "galbanum", "chel.be.nah"),
        "H2465": ("חֶ֫לֶד", "H:N-M", "lifetime/world", "che.led"),
        "H2466": ("חֵ֫לֶד", "N:N-M-P", "Heled", "che.led"),
        "H2467": ("חֹ֫לֶד", "H:N-M", "weasel", "cho.led"),
        "H2468": ("חֻלְדָּה", "N:N-F-P", "Huldah", "chul.dah"),
        "H2469": ("חֶלְדַּי", "N:N-M-P", "Heldai", "chel.day"),
        "H2470a": ("חָלָה", "H:V", "be weak", "cha.lah"),
        "H2470b": ("חָלָה", "H:V", "to beg", "cha.lah"),
        "H2471": ("חַלָּה", "H:N-F", "bun", "chal.lah"),
        "H2472": ("חֲלוֹם", "H:N-M", "dream", "cha.lom"),
        "H2473": ("חֹלוֹן", "N:N--L", "Holon", "cho.lon"),
        "H2474": ("חַלּוֹן", "H:N", "window", "chal.lon"),
        "H2475": ("חֲלוֹף", "H:N-M", "destruction", "cha.loph"),
        "H2476": ("חֲלוּשָׁה", "H:N-F", "weakness", "cha.lu.shah"),
        "H2477": ("חֲלַח", "N:N--L", "Halah", "cha.lach"),
        "H2478": ("חַלְחוּל", "N:N--L", "Halhul", "chal.chul"),
        "H2479": ("חַלְחָלָה", "H:N-F", "anguish", "chal.cha.lah"),
        "H2480": ("חָלַט", "H:V", "to catch", "cha.lat"),
        "H2481": ("חֲלִי", "H:N-M", "ornament", "cha.li"),
        "H2482": ("חֲלִי", "N:N--L", "Hali", "cha.li"),
        "H2483": ("חֳלִי", "H:N-M", "sickness", "cho.li"),
        "H2484": ("חֶלְיָה", "H:N-F", "jewelry", "chel.yah"),
        "H2485": ("חָלִיל", "H:N-M", "flute", "cha.lil"),
        "H2486": ("חָלִ֫ילָה", "H:Intj", "forbid", "cha.li.lah"),
        "H2487": ("חֲלִיפָה", "H:N-F", "change", "cha.li.phah"),
        "H2488": ("חֲלִיצָה", "H:N-F", "spoil", "cha.li.tsah"),
        "H2489": ("חֵֽלְכָה", "H:A", "helpless", "che.le.kah"),
        "H2490a": ("חָלַל", "H:V", "to bore", "cha.lal"),
        "H2490b": ("חָלַל", "H:V", "to play flute", "cha.lal"),
        "H2490c": ("חָלַל", "H:V", "to profane/begin", "cha.lal"),
        "H2491a": ("חָלָל", "H:N-M", "slain", "cha.lal"),
        "H2491b": ("חָלָל", "H:A", "profaned", "cha.lal"),
        "H2492a": ("חָלַם", "H:V", "be healthy", "cha.lam"),
        "H2492b": ("חָלַם", "H:V", "to dream", "cha.lam"),
        "H2493": ("חֵ֫לֶם", "A:N-M", "dream", "che.lem"),
        "H2494": ("חֵ֫לֶם", "N:N-M-P", "Helem", "che.lem"),
        "H2495": ("חַלָּמוּת", "H:N-M", "mallow", "chal.la.mut"),
        "H2496": ("חַלָּמִישׁ", "H:N-M", "flint", "chal.la.mish"),
        "H2497": ("חֵלֹן", "N:N-M-P", "Helon", "che.lon"),
        "H2498": ("חָלַף", "H:V", "to pass", "cha.laph"),
        "H2499": ("חֲלַף", "A:V", "to pass", "cha.laph"),
        "H2500": ("חֵ֫לֶף", "H:N-M", "for", "che.leph"),
        "H2501": ("חֵ֫לֶף", "N:N--L", "Heleph", "che.leph"),
        "H2502a": ("חָלַץ", "H:V", "to rescue", "cha.lats"),
        "H2502b": ("חָלַץ", "H:V", "to arm", "cha.lats"),
        "H2503": ("חֶ֫לֶץ", "N:N-M-P", "Helez", "che.lets"),
        "H2504": ("חָלָץ", "H:N-F", "loin", "cha.lats"),
        "H2505a": ("חָלַק", "H:V", "to divide", "cha.laq"),
        "H2505b": ("חָלַק", "H:V", "to smooth", "cha.laq"),
        "H2506a": ("חֵ֫לֶק", "H:N-M", "portion", "che.leq"),
        "H2506b": ("חֵ֫לֶק", "H:N-M", "smoothness", "che.leq"),
        "H2507": ("חֵ֫לֶק", "N:N-M-P", "Helek", "che.leq"),
        "H2508": ("חֲלָק", "A:N-M", "portion", "cha.laq"),
        "H2509": ("חָלָק", "H:A", "smooth", "cha.laq"),
        "H2510": ("חָלָק", "N:N--L", "Halak", "cha.laq"),
        "H2511": ("חַלָּק", "H:N-M", "smooth", "chal.laq"),
        "H2512": ("חַלֻּק", "H:N-M", "smooth", "chal.luq"),
        "H2513a": ("חֶלְקָה", "H:N-F", "portion", "chel.qah"),
        "H2513b": ("חֶלְקָה", "H:N-F", "smoothness", "chel.qah"),
        "H2514": ("חֲלַקָּה", "H:N-F", "flattery", "cha.laq.qah"),
        "H2515": ("חֲלֻקָּה", "H:N-F", "division", "cha.luq.qah"),
        "H2516": ("חֶלְקִי", "N:N--PG", "Helekites", "chel.qi"),
        "H2517": ("חֶלְקָ֑י", "N:N-M-P", "Helkai", "chel.qay"),
        "H2518": ("חִלְקִיָּהוּ", "N:N-M-P", "Hilkiah", "chil.qiy.yah"),
        "H2519": ("חֲלַקְלַקּוֹת", "H:N-F", "smoothness", "cha.laq.laq.qah"),
        "H2520": ("חֶלְקָת֙", "N:N--L", "Helkath", "chel.qat"),
        "H2521": ("חֶלְקַת הַצֻּרִים", "N:N--L", "Helkath-hazzurim", "chel.qat hats.tsu.rim"),
        "H2522": ("חָלַשׁ", "H:V", "to weaken", "cha.lash"),
        "H2523": ("חַלָּשׁ", "H:A", "weak", "chal.lash"),
        "H2524": ("חָם", "H:N-M", "father-in-law", "cham"),
        "H2525": ("חָם", "H:A", "hot", "cham"),
        "H2526": ("חָם", "N:N-M-P / N:N-M-PG", "Ham", "cham"),
        "H2527": ("חֹם", "H:N-M", "heat", "chom"),
        "H2528": ("חֱמָא", "A:N-F", "rage", "che.ma"),
        "H2529a": ("חֶמְאָה", "H:N-F", "curd", "chem.ah"),
        "H2529b": ("חֵמָה", "H:N-F", "heat", ""),
        "H2530a": ("חָמַד", "H:V", "to desire", "cha.mad"),
        "H2530b": ("חֲמוּדָה", "H:N-F", "precious thing", ""),
        "H2531": ("חֶ֫מֶד", "H:N-M", "delight", "che.med"),
        "H2532a": ("חֶמְדָּה", "H:N-F", "desire", "chem.dah"),
        "H2532b": ("חֲמוּדָה", "H:N-F", "desirable thing", ""),
        "H2533": ("חֶמְדָּן", "N:N-M-P", "Hemdan", "chem.dan"),
        "H2534": ("חֵמָה", "H:N-F", "rage", "che.mah"),
        "H2535": ("חַמָּה", "H:N-F", "heat", "cham.mah"),
        "H2536": ("חַמּוּאֵל", "N:N-M-P", "Hammuel", "cham.mu.el"),
        "H2537": ("חֲמוּטַל", "N:N-F-P", "Hamutal", "cha.mu.tal"),
        "H2538": ("חָמוּל", "N:N-M-P", "Hamul", "cha.mul"),
        "H2539": ("חָמוּלִי", "N:N--PG", "Hamulites", "cha.mu.li"),
        "H2540": ("חַמּוֹן", "N:N--L", "Hammon", "cham.mon"),
        "H2541": ("חָמוֹץ", "H:N-M", "oppression", "cha.mots"),
        "H2542": ("חַמּוּק", "H:N-M", "curve", "cham.muq"),
        "H2543": ("חֲמוֹר", "H:N-M", "donkey", "cha.mor"),
        "H2544": ("חֲמוֹר", "N:N-M-P", "Hamor", "cha.mor"),
        "H2545": ("חֲמוֹת", "H:N-F", "mother-in-law", "cha.mot"),
        "H2546": ("חֹ֫מֶט", "H:N-M", "lizard", "cho.met"),
        "H2547": ("חֻמְטָה", "N:N--L", "Humtah", "chum.tah"),
        "H2548": ("חָמִיץ", "H:A", "salted", "cha.mits"),
        "H2549": ("חֲמִישִׁי", "H:A", "fifth", "cha.mi.shi"),
        "H2550": ("חָמַל", "H:V", "to spare", "cha.mal"),
        "H2551": ("חֶמְלָה", "H:N-F", "compassion", "chem.lah"),
        "H2552": ("חָמַם", "H:V", "to warm", "cha.mam"),
        "H2553": ("חַמָּן", "H:N-M", "pillar", "cham.man"),
        "H2554": ("חָמַס", "H:V", "to injure", "cha.mas"),
        "H2555": ("חָמָס", "H:N-M", "violence", "cha.mas"),
        "H2556a": ("חָמֵץ", "H:V", "to leaven", "cha.mets"),
        "H2556b": ("חָמַץ", "H:V", "be red", ""),
        "H2556c": ("חָמַץ", "H:V", "to oppress", ""),
        "H2557a": ("חָמֵץ", "H:N-M", "leaven", "cha.mets"),
        "H2557b": ("מַחְמֶ֫צֶת", "H:N-M", "leaven", ""),
        "H2558": ("חֹ֫מֶץ", "H:N-M", "vinegar", "cho.mets"),
        "H2559": ("חָמַק", "H:V", "to turn away", "cha.maq"),
        "H2560a": ("חָמַר", "H:V", "to trouble", "cha.mar"),
        "H2560b": ("חָמַר", "H:V", "to daub", "cha.mar"),
        "H2560c": ("חָמַר", "H:V", "be red", "cha.mar"),
        "H2561": ("חֶ֫מֶר", "H:N-M", "wine", "che.mer"),
        "H2562": ("חֲמַר", "A:N-M", "wine", "cha.mar"),
        "H2563a": ("חֹ֫מֶר", "H:N-M", "clay", "cho.mer"),
        "H2563b": ("חֹ֫מֶר", "H:N-M", "heap", "cho.mer"),
        "H2563c": ("חֹ֫מֶר", "H:N-M", "homer", "cho.mer"),
        "H2564": ("חֵמָר", "H:N-M", "bitumen", "che.mar"),
        "H2565": ("חֲמוֹר", "H:N-M", "heap", "cha.mo.rah"),
        "H2566": ("חַמְרָן", "N:N-M-P", "Hamran", "cham.ran"),
        "H2567": ("חָמַשׁ", "H:V", "to take the fifth part", "cha.mash"),
        "H2568": ("חָמֵשׁ", "H:N", "five", "cha.mesh"),
        "H2569": ("חֹ֫מֶשׁ", "H:N-M", "fifth", "cho.mesh"),
        "H2570": ("חֹ֫מֶשׁ", "H:N-M", "belly", "cho.mesh"),
        "H2571": ("חֲמֻשִׁים", "H:A", "armed", "cha.mush"),
        "H2572": ("חֲמִשִּׁים", "H:N", "fifty", "cha.mish.shim"),
        "H2573": ("חֵ֫מֶת", "H:N-M", "bottle", "che.met"),
        "H2574": ("חֲמָת", "N:N--L / N:N-M-P", "Hamath", "cha.mat"),
        "H2575a": ("חַמַּת", "N:N--L", "Hammath", "cham.mat"),
        "H2575b": ("חַמַּת", "N:N-M-P", "Hammath", "cham.mat"),
        "H2576": ("חַמֹּת דֹּאר", "N:N--L", "Hammoth-dor", "cham.mot dor"),
        "H2577": ("חֲמָתִי", "N:N--LG", "Hamathites", "cha.ma.ti"),
        "H2578": ("חֲמַת צוֹבָה", "N:N--L", "Hamath-zobah", "cha.mat tso.vah"),
        "H2579": ("חֲמַת רַבָּה", "N:N--L", "Hamath-rabbah", "cha.mat rab.bah"),
        "H2580": ("חֵן", "H:N-M", "favor", "chen"),
        "H2581": ("חֵן", "N:N-M-P", "Hen", "chen"),
        "H2582": ("חֵנָדָד", "N:N-M-P", "Henadad", "che.na.dad"),
        "H2583": ("חָנָה", "H:V", "to camp", "cha.nah"),
        "H2584": ("חַנָּה", "N:N-F-P", "Hannah", "chan.nah"),
        "H2585": ("חֲנוֹךְ", "N:N-M-P", "Enoch", "cha.nokh"),
        "H2586": ("חָנוּן", "N:N-M-P", "Hanun", "cha.nun"),
        "H2587": ("חַנּוּן", "H:A", "gracious", "chan.nun"),
        "H2588": ("חָנוּת", "H:N-F", "vault", "cha.nut"),
        "H2589": ("חַנּוֹת", "H:V", "to entreat", "chan.not"),
        "H2590a": ("חָנַט", "H:V", "to spice", "cha.nat"),
        "H2590b": ("חֲנֻטִים", "H:N-M", "embalming", ""),
        "H2591": ("חִנְטָה", "A:N-M", "wheat", "chin.ta"),
        "H2592": ("חַנִּיאֵל", "N:N-M-P", "Hanniel", "chan.ni.el"),
        "H2593": ("חָנִיךְ", "H:A", "trained", "cha.nikh"),
        "H2594": ("חֲנִינָה", "H:N-F", "favor", "cha.ni.nah"),
        "H2595": ("חֲנִית", "H:N-F", "spear", "cha.nit"),
        "H2596": ("חָנַךְ", "H:V", "to dedicate", "cha.nakh"),
        "H2597": ("חֲנֻכָּה", "A:N-F", "dedication", "cha.nuk.ka"),
        "H2598": ("חֲנֻכָּה", "H:N-F", "dedication", "cha.nuk.kah"),
        "H2599": ("חֲנֹכִי", "N:N--PG", "Hanochites", "cha.no.khi"),
        "H2600": ("חִנָּם", "H:Adv", "for nothing", "chin.nam"),
        "H2601": ("חֲנַמְאֵל", "N:N-M-P", "Hanamel", "cha.nam.el"),
        "H2602": ("חֲנָמֵל", "H:N-M", "frost", "cha.na.mal"),
        "H2603a": ("חָנַן", "H:V", "be gracious", "cha.nan"),
        "H2603b": ("חָנַן", "H:V", "be loathsome", "cha.nan"),
        "H2604": ("חֲנַן", "A:V", "to show favor", "cha.nan"),
        "H2605": ("חָנָן", "N:N-M-P", "Hanan", "cha.nan"),
        "H2606": ("חֲנַנְאֵל", "N:N-M-P", "Hananel", "cha.nan.el"),
        "H2607": ("חֲנָנִי", "N:N-M-P", "Hanani", "cha.na.ni"),
        "H2608a": ("חֲנַנְיָ֫הוּ", "N:N-M-P", "Hananiah", "cha.nan.yah"),
        "H2608b": ("חֲנַנְיָה", "N:N-M-P", "Hananiah", "cha.nan.yah"),
        "H2609": ("חָנֵס", "N:N--L", "Hanes", "cha.nes"),
        "H2610": ("חָנֵף", "H:V", "to pollute", "cha.neph"),
        "H2611": ("חָנֵף", "H:A", "profane", "cha.neph"),
        "H2612": ("חֹ֫נֶף", "H:N-M", "profaneness", "cho.neph"),
        "H2613": ("חֲנֻפָּה", "H:N-F", "profaneness", "cha.nu.phah"),
        "H2614": ("חָנַק", "H:V", "to strangle", "cha.naq"),
        "H2615": ("חַנָּתֹן", "N:N--L", "Hannathon", "chan.na.ton"),
        "H2616a": ("חָסַד", "H:V", "be kind", "cha.sad"),
        "H2616b": ("חָסַד", "H:V", "to shame", "cha.sad"),
        "H2617a": ("חֶ֫סֶד", "H:N-M", "kindness", "che.sed"),
        "H2617b": ("חֶ֫סֶד", "H:N-M", "shame", "che.sed"),
        "H2618": ("חֶסֶד", "N:N-M-P", "Hesed", "che.sed"),
        "H2619": ("חֲסַדְיָה", "N:N-M-P", "Hasadiah", "cha.sad.yah"),
        "H2620": ("חָסָה", "H:V", "to seek refuge", "cha.sah"),
        "H2621": ("חֹסָה", "N:N-M-P / N:N--L", "Hosah", "cho.sah"),
        "H2622": ("חָסוּת", "H:N-F", "refuge", "cha.sut"),
        "H2623": ("חָסִיד", "H:A", "pious", "cha.sid"),
        "H2624": ("חֲסִידָה", "H:N-F", "stork", "cha.si.dah"),
        "H2625": ("חָסִיל", "H:N-M", "locust", "cha.sil"),
        "H2626": ("חֲסִין", "H:A", "mighty", "cha.sin"),
        "H2627": ("חַסִּר", "A:A", "lacking", "chas.sir"),
        "H2628": ("חָסַל", "H:V", "to consume", "cha.sal"),
        "H2629": ("חָסַם", "H:V", "to muzzle", "cha.sam"),
        "H2630": ("חָסַן", "H:V", "to hoard", "cha.san"),
        "H2631": ("חֲסַן", "A:V", "to possess", "cha.san"),
        "H2632": ("חֵ֫סֶן", "A:N-M", "authority", "che.sen"),
        "H2633": ("חֹ֫סֶן", "H:N-M", "wealth", "cho.sen"),
        "H2634": ("חָסֹן", "H:A", "strong", "cha.son"),
        "H2635": ("חֲסַף", "A:N-M", "clay", "cha.saph"),
        "H2636": ("חַסְפַּס", "H:V", "to peel", "chas.pas"),
        "H2637": ("חָסֵר", "H:V", "to lack", "cha.ser"),
        "H2638": ("חָסֵר", "H:A", "lacking", "cha.ser"),
        "H2639": ("חֶ֫סֶר", "H:N-M", "poverty", "che.ser"),
        "H2640": ("חֹ֫סֶר", "H:N-M", "lack", "cho.ser"),
        "H2641": ("חַסְרָה", "N:N-M-P", "Hasrah", "chas.rah"),
        "H2642": ("חֶסְרוֹן", "H:N-M", "deficiency", "ches.ron"),
        "H2643": ("חַף", "H:A", "clean", "chaph"),
        "H2644": ("חָפָא", "H:V", "to do secretly", "cha.pah"),
        "H2645": ("חָפָה", "H:V", "to cover", "cha.phah"),
        "H2646": ("חֻפָּה", "H:N-F", "canopy", "chup.pah"),
        "H2647": ("חֻפָּה", "N:N-M-P", "Huppah", "chup.pah"),
        "H2648": ("חָפַז", "H:V", "to hurry", "cha.phaz"),
        "H2649": ("חִפָּזוֹן", "H:N-M", "haste", "chip.pa.zon"),
        "H2650": ("חֻפִּים", "N:N-M-P", "Huppim", "chup.pim"),
        "H2651": ("חֹ֫פֶן", "H:N-M", "palm", "cho.phen"),
        "H2652": ("חׇפְנִי", "N:N-M-P", "Hophni", "choph.ni"),
        "H2653": ("חׇפַף", "H:V", "to shield", "cho.phaph"),
        "H2654a": ("חָפֵץ", "H:V", "to delight in", "cha.phets"),
        "H2654b": ("חָפַץ", "H:V", "to sway", ""),
        "H2655": ("חָפֵץ", "H:A", "delighting", "cha.phets"),
        "H2656": ("חֵ֫פֶץ", "H:N-M", "pleasure", "che.phets"),
        "H2657": ("חֶפְצִי־בָהּ", "N:N-F-P", "Hephzi-bah", "cheph.tsi bah"),
        "H2658": ("חָפַר", "H:V", "to search", "cha.phar"),
        "H2659": ("חָפֵר", "H:V", "be ashamed", "cha.pher"),
        "H2660a": ("חֵ֫פֶר", "N:N-M-P", "Hepher", "che.pher"),
        "H2660b": ("חֵ֫פֶר", "N:N--L", "Hepher", "che.pher"),
        "H2661": ("חֲפֹר", "H:N-F", "mole", "cha.phor"),
        "H2662": ("חֶפְרִי", "N:N--PG", "Hepherites", "cheph.ri"),
        "H2663": ("חֲפָרַיִם", "N:N--L", "Hapharaim", "cha.pha.ra.yim"),
        "H2664": ("חָפַשׂ", "H:V", "to search", "cha.pha.s"),
        "H2665": ("חֵ֫פֶשׂ", "H:N-M", "plot", "che.phe.s"),
        "H2666": ("חָפַשׁ", "H:V", "be free", "cha.phash"),
        "H2667": ("חֹ֫פֶשׁ", "H:N-M", "spread", "cho.phesh"),
        "H2668": ("חֻפְֿשָׁה", "H:N-F", "freedom", "chuph.shah"),
        "H2669": ("חׇפְשִׁית", "H:N-F", "freedom", "choph.shut"),
        "H2670": ("חׇפְשִׁי", "H:A", "free", "choph.shi"),
        "H2671": ("חֵץ", "H:N-M", "arrow", "chets"),
        "H2672": ("חָצַב", "H:V", "to hew", "cha.tsav"),
        "H2673": ("חָצָה", "H:V", "to divide", "cha.tsah"),
        "H2674": ("חָצוֹר", "N:N--L", "Hazor", "cha.tsor"),
        "H2675": ("חָצוֹר חֲדַתָּה", "N:N--L", "Hazor-hadattah", "cha.tsor cha.dat.tah"),
        "H2676": ("חָצוֹת", "H:N-F", "middle", "cha.tsot"),
        "H2677": ("חֲצִי", "H:N-M", "half", "che.tsi"),
        "H2678": ("חֵ֫צִי", "H:N-M", "arrow", "chits.tsi"),
        "H2679": ("חֲצִי הַמְּנֻחוֹת", "N:N--PG", "Half-of-Manahathites", "cha.tsi ham.m.nu.chot"),
        "H2680": ("חֲצִי הַמְּנַחְתִּי מְנַשֶּׁה", "N:N--PG", "Chazi-manahethite", "cha.tsi ham.m.nach.ti"),
        "H2681": ("חָצִיר", "H:N-M", "abode", "cha.tsir"),
        "H2682a": ("חָצִיר", "H:N-M", "grass", "cha.tsir"),
        "H2682b": ("חָצִיר", "H:N-M", "leek", "cha.tsir"),
        "H2683": ("חֵ֫צֶן", "H:N-M", "bosom", "che.tsen"),
        "H2684": ("חֹ֫צֶן", "H:N-M", "bosom", "cho.tsen"),
        "H2685": ("חֲצַף", "A:V", "be hasty", "cha.tsaph"),
        "H2686a": ("חָצַץ", "H:V", "to divide", "cha.tsats"),
        "H2686b": ("חָצַץ", "H:V", "to shoot", "cha.tsats"),
        "H2687": ("חָצָץ", "H:N-M", "gravel", "cha.tsats"),
        "H2688": ("חַצֲצֹן תָּמָר", "N:N--L", "Hazazon-tamar", "chats.tson ta.mar"),
        "H2689": ("חֲצֹצְרָה", "H:N-F", "trumpet", "cha.tso.tse.rah"),
        "H2690": ("חָצַר", "H:V", "to blow", "cha.tsar"),
        "H2691a": ("חָצֵר", "H:N-M", "court", "cha.tser"),
        "H2691b": ("חָצֵר", "H:N-M", "village", "cha.tser"),
        "H2692": ("חֲצַר־אַדָּר", "N:N--L", "Hazar-addar", "cha.tsar ad.dar"),
        "H2693": ("חֲצַר גַּדָּה", "N:N--L", "Hazar-gaddah", "cha.tsar gad.dah"),
        "H2694": ("חֲצַר הַתִּיכוֹן", "N:N--L", "Hazer-hatticon", "cha.tsar hat.ti.khon"),
        "H2695": ("חֶצְרוֹ", "N:N-M-P", "Hezro", "chets.ro"),
        "H2696": ("חֶצְרוֹן", "N:N-M-P / N:N--L", "Hezron", "chets.ron"),
        "H2697": ("חֶצְרוֹנִי", "N:N--PG", "Hezronites", "chets.ro.ni"),
        "H2698": ("חֲצֵרוֹת", "N:N--L", "Hazeroth", "cha.tse.rot"),
        "H2699": ("חֲצֵרִים", "N:N--L", "Hazerim", "cha.tse.rim"),
        "H2700": ("חֲצַרְמָ֫וֶת", "N:N-M-P", "Hazar-maveth", "cha.tsar.ma.vet"),
        "H2701": ("חֲצַר סוּסָה", "N:N--L", "Hazar-susah", "cha.tsar su.sah"),
        "H2702": ("חֲצַר סוּסִים", "N:N--L", "Hazar-susim", "cha.tsar su.sim"),
        "H2703": ("חֲצַר עֵינָן", "N:N--L", "Hazar-enan", "cha.tsar e.non"),
        "H2704": ("חֲצַר עֵינוֹן", "N:N--L", "Hazar-enan", "cha.tsar e.nan"),
        "H2705": ("חֲצַר שׁוּעָל", "N:N--L", "Hazar-shual", "cha.tsar shu.al"),
        "H2706": ("חֹק", "H:N-M", "statute", "choq"),
        "H2707": ("חָקָה", "H:V", "to engrave", "cha.qah"),
        "H2708": ("חֻקָּה", "H:N-F", "statute", "chuq.qah"),
        "H2709": ("חֲקוּפָא", "N:N-M-P", "Hakupha", "cha.qu.pah"),
        "H2710": ("חָקַק", "H:V", "to decree", "cha.qaq"),
        "H2711": ("חֵקֶק", "H:N-M", "decree", "che.qeq"),
        "H2712a": ("חֻקֹֿק", "N:N--L", "Hukok", "chuq.qoq"),
        "H2712b": ("חוּקֹק", "N:N--L", "Hukok", ""),
        "H2713": ("חָקַר", "H:V", "to search", "cha.qar"),
        "H2714": ("חֵ֫קֶר", "H:N-M", "search", "che.qer"),
        "H2715": ("חֹר", "H:N-M", "noble", "chor"),
        "H2716": ("חֶרֶא", "H:N-M", "dung", "che.re"),
        "H2717a": ("חָרֵב", "H:V", "to dry", ""),
        "H2717b": ("חָרֵב", "H:V", "to destroy", ""),
        "H2717c": ("חָרַב", "H:V", "to slay", "cha.rav"),
        "H2718": ("חֲרַב", "A:V", "be wasted", "cha.rav"),
        "H2719": ("חֶ֫רֶב", "H:N-F", "sword", "che.rev"),
        "H2720a": ("חָרֵב", "H:A", "dry", "cha.rev"),
        "H2720b": ("חָרֵב", "H:A", "desolate", "cha.rev"),
        "H2721a": ("חֹ֫רֶב", "H:N-M", "drought", "cho.rev"),
        "H2721b": ("חֹ֫רֶב", "H:N-M", "desolation", "cho.rev"),
        "H2722": ("חֹרֵב", "N:N--L", "Horeb", "cho.rev"),
        "H2723": ("חׇרְבָּה", "H:N-F", "desolation", "chor.bah"),
        "H2724": ("חָֽרָבָה", "H:N-F", "dry ground", "cha.ra.vah"),
        "H2725": ("חֵרָבוֹן", "H:N-M", "drought", "cha.ra.von"),
        "H2726": ("חַרְבוֹנָא", "N:N-M-P", "Harbona", "char.vo.na"),
        "H2727": ("חָרַג", "H:V", "to quake", "cha.rag"),
        "H2728": ("חָרְגֹּל", "H:N-M", "locust", "cha.re.gol"),
        "H2729": ("חָרַד", "H:V", "to tremble", "cha.rad"),
        "H2730": ("חָרֵד", "H:A", "trembling", "cha.red"),
        "H2731": ("חֲרָדָה", "H:N-F", "trembling", "cha.ra.dah"),
        "H2732": ("חֲרָדָה", "N:N--L", "Haradah", "cha.ra.dah"),
        "H2733": ("חֲרֹדִי", "N:N--LG", "Harodite", "cha.ro.di"),
        "H2734": ("חָרָה", "H:V", "to be incensed", "cha.rah"),
        "H2735": ("חֹר הַגִּדְגָּד", "N:N--L", "Hor-haggidgad", "chor hag.gid.gad"),
        "H2736": ("חַרְחֲיָה", "N:N-M-P", "Harhaiah", "char.ha.yah"),
        "H2737": ("חָרוּז", "H:N-M", "string", "cha.ruz"),
        "H2738": ("חָרוּל", "H:N-M", "nettle", "cha.rul"),
        "H2739": ("חֲרוּמַף", "N:N-M-P", "Harumaph", "cha.ru.maph"),
        "H2740": ("חָרוֹן", "H:N-M", "burning anger", "cha.ron"),
        "H2741": ("חֲרִיפִי", "N:N--LG", "Haruphite", "cha.ru.phi"),
        "H2742a": ("חָרוּץ", "H:A", "sharp", "cha.ruts"),
        "H2742b": ("חָרוּץ", "H:N-M", "decision", "cha.ruts"),
        "H2742c": ("חָרוּץ", "H:N-M", "trench", "cha.ruts"),
        "H2742d": ("חָרוּץ", "H:N-M", "gold", "cha.ruts"),
        "H2743": ("חָרוּץ", "N:N-M-P", "Haruz", "cha.ruts"),
        "H2744": ("חַרְחוּר", "N:N-M-P", "Harhur", "char.chur"),
        "H2745": ("חַרְחַס", "N:N-M-P", "Harhas", "char.chas"),
        "H2746": ("חַרְחֻר", "H:N-M", "heat", "char.chur"),
        "H2747": ("חֶ֫רֶט", "H:N-M", "stylus", "che.ret"),
        "H2748": ("חַרְטֹם", "H:N-M", "magician", "char.tom"),
        "H2749": ("חַרְטֹם", "A:N-M", "magician", "char.tom"),
        "H2750": ("חֳרִי", "H:N-M", "burning", "cho.ri"),
        "H2751": ("חֹרִי", "H:N-M", "white", "cho.ri"),
        "H2752": ("חֹרִי", "N:N--LG", "Horite", "cho.ri"),
        "H2753": ("חֹרִי", "N:N-M-P", "Hori", "cho.ri"),
        "H2754": ("חָרִיט", "H:N-M", "purse", "cha.rit"),
        "H2755": ("חֲרֵי־יוֹנִים", "H:N-M", "dove-dung", "cha.re-yo.nim"),
        "H2756": ("חָרִיף", "N:N-M-P", "Hariph", "cha.riph"),
        "H2757": ("חָרִיץ", "H:N-M", "incision", "cha.rits"),
        "H2758": ("חָרִישׁ", "H:N-M", "plowing", "cha.rish"),
        "H2759": ("חֲרִישִׁי", "H:A", "scorching", "cha.ri.shi"),
        "H2760": ("חָרַךְ", "H:V", "to roast", "cha.rakh"),
        "H2761": ("חֲרַךְ", "A:V", "to singe", "cha.rakh"),
        "H2762": ("חֲרַכִּים", "H:N-M", "lattice", "che.rekh"),
        "H2763a": ("חָרַם", "H:V", "to devote/destroy", "cha.ram"),
        "H2763b": ("חָרַם", "H:V", "to slit", "cha.ram"),
        "H2764a": ("חֵ֫רֶם", "H:N-M", "devoted thing", "che.rem"),
        "H2764b": ("חֵ֫רֶם", "H:N-M", "net", "che.rem"),
        "H2765": ("חֳרֵם", "N:N--L", "Horem", "cho.rem"),
        "H2766": ("חָרִם", "N:N-M-P", "Harim", "cha.rim"),
        "H2767": ("חׇרְמָה", "N:N--L", "Hormah", "chor.mah"),
        "H2768": ("חֶרְמוֹן", "N:N--L", "Hermon", "cher.mon"),
        "H2769": ("חֶרְמוֹנִים", "N:N--L", "Hermon", "cher.mo.nim"),
        "H2770": ("חֶרְמֵשׁ", "H:N-M", "sickle", "cher.mesh"),
        "H2771a": ("חָרָן", "N:N--L", "Haran", "cha.ran"),
        "H2771b": ("חָרָן", "N:N-M-P", "Haran", "cha.ran"),
        "H2772": ("חֹרֹנִי", "N:N--LG", "Horonite", "cho.ro.ni"),
        "H2773": ("חֹרֹנַיִם", "N:N--L", "Horonaim", "cho.ro.na.yim"),
        "H2774": ("חַרְנֶ֫פֶר", "N:N-M-P", "Harnepher", "char.ne.pher"),
        "H2775a": ("חֶ֫רֶס", "H:N-M", "sun", "che.res"),
        "H2775b": ("חֶ֫רֶס", "H:N-M", "itch", "che.res"),
        "H2776": ("חֶ֫רֶס", "N:N--L", "Heres", "che.res"),
        "H2777": ("חַרְסִית", "H:N-F", "potsherd", "char.sut"),
        "H2778a": ("חָרַף", "H:V", "to taunt", "cha.raph"),
        "H2778b": ("חָרַף", "H:V", "to winter", "cha.raph"),
        "H2778c": ("חָרַף", "H:V", "to acquire", "cha.raph"),
        "H2779": ("חֹ֫רֶף", "H:N-M", "autumn", "cho.reph"),
        "H2780": ("חָרֵף", "N:N-M-P", "Hareph", "cha.reph"),
        "H2781": ("חֶרְפָּה", "H:N-F", "reproach", "cher.pah"),
        "H2782": ("חָרַץ", "H:V", "to decide", "cha.rats"),
        "H2783": ("חֲרַץ", "A:N-F", "loin", "cha.rats"),
        "H2784": ("חַרְצֻבָּה", "H:N-F", "bond", "char.tsub.bah"),
        "H2785": ("חַרְצַנִּים", "H:N-M", "pip", "char.tsan"),
        "H2786": ("חָרַק", "H:V", "to grind", "cha.raq"),
        "H2787": ("חָרַר", "H:V", "to scorch", "cha.rar"),
        "H2788": ("חָרֵר", "H:N-M", "parched", "cha.rer"),
        "H2789": ("חֶ֫רֶשׂ", "H:N-M", "earthenware", "che.re.s"),
        "H2790a": ("חָרַשׁ", "H:V", "to plow/plot", "cha.rash"),
        "H2790b": ("חָרֵשׁ", "H:V", "be quiet", ""),
        "H2791a": ("חֶ֫רֶשׁ", "H:Adv", "silently", "che.resh"),
        "H2791b": ("חֶ֫רֶשׁ", "H:N-M", "craftily", "che.resh"),
        "H2792": ("חֶ֫רֶשׁ", "N:N-M-P", "Heresh", "che.resh"),
        "H2793": ("חֹ֫רֶשׁ", "H:N-M / N:N--L", "wood", "cho.resh"),
        "H2794": ("חֹרֵשׁ", "H:N-M", "artificer", "cho.resh"),
        "H2795": ("חֵרֵשׁ", "H:A", "deaf", "che.resh"),
        "H2796": ("חָרָשׁ, חֲרָשִׁים", "H:N-M", "artificer", "cha.rash"),
        "H2797": ("חַרְשָׁא", "N:N-M-P", "Harsha", "char.sah"),
        "H2798": ("חֲרָשִׁים", "N:N--L", "Charashim", "cha.ra.shim"),
        "H2799": ("חֲרֹ֫שֶׁת", "H:N-F", "carving", "cha.ro.shet"),
        "H2800": ("חֲרֹ֫שֶׁת", "N:N--L", "Harosheth", "cha.ro.shet"),
        "H2801": ("חָרַת", "H:V", "to engrave", "cha.rat"),
        "H2802": ("חֶרֶת", "N:N--L", "Hereth", "che.ret"),
        "H2803": ("חָשַׁב", "H:V", "to devise", "cha.shav"),
        "H2804": ("חֲשַׁב", "A:V", "to account", "cha.shav"),
        "H2805": ("חֵ֫שֶׁב", "H:N-M", "artwork", "che.shev"),
        "H2806": ("חַשְׁבַּדָּנָה", "N:N-M-P", "Hashbaddanah", "chash.bad.da.nah"),
        "H2807": ("חֲשֻׁבָה", "N:N-M-P", "Hashubah", "cha.shu.vah"),
        "H2808": ("חֶשְׁבּוֹן", "H:N-M", "explanation", "chesh.bon"),
        "H2809": ("חֶשְׁבּוֹן", "N:N--L", "Heshbon", "chesh.bon"),
        "H2810": ("חִשָּׁבוֹן", "H:N-M", "invention", "chish.sha.von"),
        "H2811": ("חֲשַׁבְיָ֫הוּ", "N:N-M-P", "Hashabiah", "cha.shav.yah"),
        "H2812": ("חֲשַׁבְנָה", "N:N-M-P", "Hashabnah", "cha.shav.nah"),
        "H2813": ("חֲשַׁבְנְיָה", "N:N-M-P", "Hashabneiah", "cha.shav.ne.yah"),
        "H2814": ("חָשָׁה", "H:V", "be silent", "cha.shah"),
        "H2815": ("חַשּׁוּב", "N:N-M-P", "Hasshub", "chash.shuv"),
        "H2816": ("חֲשׁוֹךְ", "A:N-M", "darkness", "cha.shokh"),
        "H2817": ("חֲשׂוּפָא", "N:N-M-P", "Hasupha", "cha.su.pah"),
        "H2818a": ("חֲשַׁח", "A:V", "to need", "cha.shach"),
        "H2818b": ("חֲשַׁח", "A:N-F", "necessity", "cha.shach"),
        "H2819": ("חַשְׁחוּ", "A:N-F", "requirement", "chash.chut"),
        "H2820": ("חָשַׂךְ", "H:V", "to withhold", "cha.sakh"),
        "H2821": ("חָשַׁךְ", "H:V", "to darken", "cha.shakh"),
        "H2822": ("חֹ֫שֶׁךְ", "H:N-M", "darkness", "cho.shekh"),
        "H2823": ("חָשֹׁךְ", "H:A", "obscure", "cha.shokh"),
        "H2824": ("חֶשְׁכָה", "H:N-F", "dark", "chesh.khah"),
        "H2825": ("חֲשֵׁכָה", "H:N-F", "darkness", "cha.she.khah"),
        "H2826": ("חָשַׁל", "H:V", "be feeble", "cha.shal"),
        "H2827": ("חֲשַׁל", "A:V", "to shatter", "cha.shal"),
        "H2828": ("חָשֻׁם", "N:N-M-P", "Hashum", "cha.shum"),
        "H2829": ("חֶשְׁמוֹן", "N:N--L", "Heshmon", "chesh.mon"),
        "H2830": ("חַשְׁמַל", "H:N-M", "amber", "chash.mal"),
        "H2831": ("חַשְׁמַן", "H:N-M", "ambassador", "chash.man"),
        "H2832": ("חַשְׁמֹנָה", "N:N--L", "Hashmonah", "chash.mo.nah"),
        "H2833": ("חֹ֫שֶׁן", "H:N-M", "breastpiece", "cho.shen"),
        "H2834": ("חָשַׂף", "H:V", "to strip", "cha.saph"),
        "H2835": ("חָשִׂף", "H:N-M", "little flock", "cha.siph"),
        "H2836a": ("חָשַׁק", "H:V", "to desire", "cha.shaq"),
        "H2836b": ("חָשַׁק", "H:V", "to connect", "cha.shaq"),
        "H2837": ("חֵ֫שֶׁק", "H:N-M", "desire", "che.sheq"),
        "H2838": ("חָשׁוּק", "H:N-M", "ring", "cha.shuq"),
        "H2839": ("חִשֻּׁק", "H:N-M", "spoke", "chish.shuq"),
        "H2840": ("חִשֻּׁר", "H:N-M", "hub", "chish.shur"),
        "H2841": ("חַשְׁרָה", "H:N-F", "collection", "chash.rah"),
        "H2842": ("חֲשַׁשׁ", "H:N-M", "chaff", "cha.shash"),
        "H2843": ("חֻשָׁתִי", "N:N--PG", "Hushathite", "chu.sha.ti"),
        "H2844a": ("חַת", "H:N-M", "terror", "chat"),
        "H2844b": ("חַת", "H:A", "shattered", "chat"),
        "H2845": ("חֵת", "N:N-M-P", "Heth", "chet"),
        "H2846": ("חָתָה", "H:V", "to snatch up", "cha.tah"),
        "H2847": ("חִתָּה", "H:N-F", "terror", "chit.tah"),
        "H2848": ("חִתּוּל", "H:N-M", "bandage", "chit.tul"),
        "H2849": ("חַתְחַת", "H:N-M", "terror", "chat.chat"),
        "H2850": ("חִתִּי", "N:N--PG", "Hittite", "chit.ti"),
        "H2851": ("חִתִּית", "H:N-F", "terror", "chit.tit"),
        "H2852": ("חָתַךְ", "H:V", "to determine", "cha.takh"),
        "H2853": ("חָתַל", "H:V", "to entwine", "cha.tal"),
        "H2854": ("חֲתֻלָּה", "H:N-F", "swaddling", "cha.tul.lah"),
        "H2855": ("חֶתְלֹן", "N:N--L", "Hethlon", "chet.lon"),
        "H2856": ("חָתַם", "H:V", "to seal", "cha.tam"),
        "H2857": ("חֲתַם", "A:V", "to seal", "cha.tam"),
        "H2858": ("חֹתֶ֫מֶת", "H:N-F", "ring", "cho.te.met"),
        "H2859a": ("חֹתֵן", "H:N-M", "relative", "cha.tan"),
        "H2859b": ("חָתַן", "H:V", "be related", "cha.tan"),
        "H2860": ("חָתָן", "H:N-M", "son-in-law", "cha.tan"),
        "H2861": ("חֲתֻנָּה", "H:N-F", "marriage", "cha.tun.nah"),
        "H2862": ("חָתַף", "H:V", "to seize", "cha.taph"),
        "H2863": ("חֶ֫תֶף", "H:N-M", "robber", "che.teph"),
        "H2864": ("חָתַר", "H:V", "to dig", "cha.tar"),
        "H2865": ("חָתַת", "H:V", "to to be dismayed", "cha.tat"),
        "H2866": ("חֲתַת", "H:N-M", "terror", "cha.tat"),
        "H2867": ("חֲתַת", "N:N-M-P", "Hathath", "cha.tat"),
        "H2868": ("טְאֵב", "A:V", "be good", "te.ev"),
        "H2869": ("טָב", "A:A", "fine", "tav"),
        "H2870a": ("טָֽבְאֵל", "N:N-M-P", "Tabeel", "ta.ve.el"),
        "H2870b": ("טָֽבְאַ֑ל", "N:N-M-P", "Tabeal", ""),
        "H2871": ("טְבוּל", "H:N-M", "turban", "ta.vul"),
        "H2872": ("טַבּוּר", "H:N-M", "highest", "tab.bur"),
        "H2873": ("טָבַח", "H:V", "to slaughter", "ta.vach"),
        "H2874": ("טֶ֫בַח", "H:N-M", "slaughter", "te.vach"),
        "H2875": ("טֶ֫בַח", "N:N-M-P", "Tebah", "te.vach"),
        "H2876": ("טַבָּח", "H:N-M", "guard", "tab.bach"),
        "H2877": ("טַבָּח", "A:N-M", "guardsman", "tab.bach"),
        "H2878": ("טִבְחָה", "H:N-F", "slaughtered", "tiv.chah"),
        "H2879": ("טַבָּחָה", "H:N-F", "cook", "tab.ba.chah"),
        "H2880": ("טִבְחַת", "N:N--L", "Tibhath", "tiv.chat"),
        "H2881": ("טָבַל", "H:V", "to dip", "ta.val"),
        "H2882": ("טְבַלְיָ֫הוּ", "N:N-M-P", "Tebaliah", "te.val.ya.hu"),
        "H2883": ("טָבַע", "H:V", "to sink", "ta.va"),
        "H2884": ("טַבָּעוֹת", "N:N-M-P", "Tabbaoth", "tab.ba.ot"),
        "H2885": ("טַבַּ֫עַת", "H:N-M", "ring", "tab.ba.at"),
        "H2886": ("טַבְרִמּוֹן", "N:N-M-P", "Tabrimmon", "tav.rim.mon"),
        "H2887": ("טֵבֶ֑ת", "N:N--T", "Tebeth", "te.vet"),
        "H2888": ("טַבָּ֑ת", "N:N--L", "Tabbath", "tab.bat"),
        "H2889": ("טָהוֹר", "H:A", "pure", "ta.hor"),
        "H2890": ("טְהוֹר", "H:N-M / H:A", "pureness", "te.hor"),
        "H2891": ("טָהֵר", "H:V", "be pure", "ta.her"),
        "H2892a": ("טֹ֫הַר", "H:N-M", "purity", "to.har"),
        "H2892b": ("טֳהָר", "H:N-M", "clearness", ""),
        "H2893": ("טׇהֳרָה", "H:N-F", "purifying", "to.ho.rah"),
        "H2894": ("טֵאטֵא", "H:V", "to sweep", "tu"),
        "H2895": ("טוֹב", "H:V", "be pleasing", "tov"),
        "H2896a": ("טוֹב", "H:A", "pleasant", "tov"),
        "H2896b": ("טוֹב", "H:N-M", "good", "tov"),
        "H2896c": ("טוֹבָה", "H:N-F", "welfare", ""),
        "H2897": ("טוֹב", "N:N--L", "Tob", "tov"),
        "H2898": ("טוּב", "H:N-M", "goodness", "tuv"),
        "H2899": ("טוֹב אֲדֹנִיָּה", "N:N-M-P", "Tob-adonijah", "tov a.do.niy.ya.hu"),
        "H2900": ("טוֹבִיָּ֫הוּ", "N:N-M-P", "Tobiah", "to.viy.yah"),
        "H2901": ("טָוָה", "H:V", "to spin", "ta.vah"),
        "H2902a": ("טוּחַ", "H:V", "to overspread", "tu.ach"),
        "H2902b": ("טָחַח", "H:V", "to overspread", ""),
        "H2903": ("טוֹטָפוֹת", "H:N-F", "phylacteries", "to.pha.phah"),
        "H2904": ("טוּל", "H:V", "to cast", "tul"),
        "H2905": ("טוּר", "H:N-M", "row", "tur"),
        "H2906": ("טוּר", "A:N-M", "mountain", "tur"),
        "H2907": ("טוּשׂ", "H:V", "to dart", "tu.s"),
        "H2908": ("טְוָת", "A:Adv", "fasting", "te.vat"),
        "H2909": ("טָחָה", "H:V", "to shoot", "ta.chah"),
        "H2910": ("טֻחוֹת", "H:N-F", "inner parts", "tu.chah"),
        "H2911": ("טְחוֹן", "H:N-M", "grinding", "te.chon"),
        "H2912": ("טָחַן", "H:V", "to grind", "ta.chan"),
        "H2913": ("טַחֲנָה", "H:N-F", "mill", "ta.cha.nah"),
        "H2914": ("טְחֹר", "H:N-M", "tumor", "te.chor"),
        "H2915": ("טִיחַ", "H:N-M", "coating", "ti.ach"),
        "H2916": ("טִיט", "H:N-M", "mud", "tit"),
        "H2917": ("טִין", "A:N-M", "common", "tin"),
        "H2918": ("טִירָה", "H:N-F", "encampment", "ti.rah"),
        "H2919": ("טַל", "H:N-M", "dew", "tal"),
        "H2920": ("טַל", "A:N-M", "dew", "tal"),
        "H2921": ("טָלָא", "H:V", "to spot", "ta.la"),
        "H2922": ("טְלָא", "H:N-M", "lamb", "te.la"),
        "H2923": ("טְלָאִים", "N:N--L", "Telaim", "te.la.im"),
        "H2924": ("טָלֶה", "H:N-M", "lamb", "ta.leh"),
        "H2925": ("טַלְטֵלָה", "H:N-F", "captivity", "tal.te.lah"),
        "H2926": ("טָלַל", "H:V", "to cover", "ta.lal"),
        "H2927": ("טְלַל", "A:V", "to have shade", "te.lal"),
        "H2928": ("טֶ֫לֶם", "N:N-M-P / N:N--L", "Telem", "te.lem"),
        "H2929": ("טַלְמוֹן", "N:N-M-P", "Talmon", "tal.mon"),
        "H2930a": ("טָמֵא", "H:V", "to defile", "ta.me"),
        "H2930b": ("טָמְאָה", "H:N-F", "uncleanness", ""),
        "H2931": ("טָמֵא", "H:A", "unclean", "ta.me"),
        "H2932": ("טֻמְאָה", "H:N-F", "uncleanness", "tum.ah"),
        "H2933": ("טָמָה", "H:V", "to defile", "ta.mah"),
        "H2934": ("טָמַן", "H:V", "to hide", "ta.man"),
        "H2935": ("טֶ֫נֶא", "H:N-M", "basket", "te.ne"),
        "H2936": ("טָנַף", "H:V", "to soil", "ta.naph"),
        "H2937": ("טָעָה", "H:V", "to stray", "ta.ah"),
        "H2938": ("טָעַם", "H:V", "to taste", "ta.am"),
        "H2939": ("טְעֵם", "A:V", "to feed", "te.am"),
        "H2940": ("טַ֫עַם", "H:N-M", "taste", "ta.am"),
        "H2941": ("טְעֵם", "A:N-M", "account", "ta.am"),
        "H2942": ("טְעֵם", "A:N-M", "command", "te.em"),
        "H2943": ("טָעַן", "H:V", "to load", "ta.an"),
        "H2944": ("טָעַן", "H:V", "to pierce", "ta.an"),
        "H2945": ("טַף", "H:N-M", "child", "taph"),
        "H2946": ("טָפַח", "H:V", "to extend", "ta.phach"),
        "H2947": ("טֶ֫פַח", "H:N-M", "handbreadth", "te.phach"),
        "H2948": ("טֹ֫פַח", "H:N-M", "handbreadth", "to.phach"),
        "H2949": ("טִפֻּחִים", "H:N-M", "tender care", "tip.puch"),
        "H2950": ("טָפַל", "H:V", "to smear", "ta.phal"),
        "H2951": ("טִפְסָר", "H:N-M", "official", "tiph.sar"),
        "H2952": ("טָפַף", "H:V", "to mince", "ta.phaph"),
        "H2953": ("טְפַר", "A:N-M", "nail/claw", "te.phar"),
        "H2954": ("טָפַשׁ", "H:V", "be insensitive", "ta.phash"),
        "H2955": ("טָפַת", "N:N-F-P", "Taphath", "ta.phat"),
        "H2956": ("טָרַד", "H:V", "to pursue", "ta.rad"),
        "H2957": ("טְרַד", "A:V", "to chase away", "te.rad"),
        "H2958": ("טְרוֹם", "H:Adv", "before", "te.rom"),
        "H2959": ("טָרַח", "H:V", "to burden", "ta.rach"),
        "H2960": ("טֹ֫רַח", "H:N-M", "burden", "to.rach"),
        "H2961": ("טָרִי", "H:A-F", "fresh", "ta.ri"),
        "H2962": ("טֶ֫רֶם", "H:Prep", "before", "te.rem"),
        "H2963": ("טָרַף", "H:V", "to tear", "ta.raph"),
        "H2964": ("טֶ֫רֶף", "H:N-M", "prey", "te.reph"),
        "H2965": ("טָרָף", "H:A", "fresh-plucked", "ta.raph"),
        "H2966": ("טְרֵפָה", "H:N-F", "torn animal", "te.re.phah"),
        "H2967": ("טַרְפְּלָיֵא", "N:N--LG", "Tarpelite", "tar.pe.lay"),
        "H2968": ("יָאַב", "H:V", "to long", "ya.av"),
        "H2969": ("יָאָה", "H:V", "to befit", "ya.ah"),
        "H2970": ("יַאֲזַנְיָ֫הוּ", "N:N-M-P", "Jaazaniah", "ya.a.zan.yah"),
        "H2971": ("יָאִיר", "N:N-M-P / N:N--L", "Jair", "ya.ir"),
        "H2972": ("יָאִרִי", "N:N--PG", "Jairite", "ya.i.ri"),
        "H2973": ("יָאַל", "H:V", "be foolish", "ya.al"),
        "H2974": ("יָאַל", "H:V", "be willing", "ya.al"),
        "H2975": ("יְאֹר", "N:N--L", "stream", "ye.or"),
        "H2976": ("יָאַשׁ", "H:V", "to despair", "ya.ash"),
        "H2977": ("יֹאשִׁיָּ֫הוּ", "N:N-M-P", "Josiah", "yo.shiy.yah"),
        "H2978": ("אִיתוֹן", "H:N-M", "entrance", "ye.i.ton"),
        "H2979": ("יְאָֽתְרַי", "N:N-M-P", "Jeatherai", "ye.a.te.ray"),
        "H2980": ("יָבַב", "H:V", "to cry", "ya.vav"),
        "H2981": ("יְבוּל", "H:N-M", "crops", "ye.vul"),
        "H2982": ("יְבוּס", "N:N--L", "Jebus", "ye.vus"),
        "H2983": ("יְבוּסִי", "N:N--LG", "Jebusite", "ye.vu.si"),
        "H2984": ("יִבְחָר", "N:N-M-P", "Ibhar", "yiv.char"),
        "H2985": ("יָבִין", "N:N-M-P", "Jabin", "ya.vin"),
        "H2986": ("יָבַל", "H:V", "to conduct", "ya.val"),
        "H2987": ("יְבַל", "A:V", "to bring", "ye.val"),
        "H2988": ("יָבָל", "H:N-M", "stream", "ya.val"),
        "H2989": ("יָבָל", "N:N-M-P", "Jabal", "ya.val"),
        "H2990": ("יַבָּל", "H:N-M", "running", "yab.bel"),
        "H2991": ("יִבְלְעָם", "N:N--L", "Ibleam", "yiv.le.am"),
        "H2992": ("יָבַם", "H:V", "be brother-in-law", "ya.vam"),
        "H2993": ("יָבָם", "H:N-M", "brother-in-law", "ya.vam"),
        "H2994": ("יְבֵ֫מֶת", "H:N-F", "sister-in-law", "ye.ve.met"),
        "H2995": ("יַבְנְאֵל", "N:N--L", "Jabneel", "yav.ne.el"),
        "H2996": ("יַבְנֶה", "N:N--L", "Jabneh", "yav.neh"),
        "H2997": ("יִבְנְיָה", "N:N-M-P", "Ibneiah", "yiv.ne.yah"),
        "H2998": ("יִבְנִיָּה", "N:N-M-P", "Ibnijah", "yiv.niy.yah"),
        "H2999": ("יַבֹּק", "N:N--L", "Jabbok", "yab.boq"),
        "H3000": ("יְבֶרֶכְיָ֫הוּ", "N:N-M-P", "Jeberechiah", "ye.ve.rekh.ya.hu"),
        "H3001": ("יָבֵשׁ", "H:V", "to wither", "ya.vesh"),
        "H3002": ("יָבֵשׁ", "H:A", "dry", "ya.vesh"),
        "H3003": ("יָבֵשׁ", "N:N--L / N:N-M-P", "Jabesh", "ya.vesh"),
        "H3004": ("יַבָּשָׁה", "H:N-F", "dry land", "yab.ba.shah"),
        "H3005": ("יִבְשָׂם", "N:N-M-P", "Jibsam", "yiv.sam"),
        "H3006": ("יַבֶּ֫שֶׁת", "H:N-F", "dry land", "yab.be.shet"),
        "H3007": ("יַבֶּ֫שֶׁת", "A:N-F", "earth", "yab.be.shet"),
        "H3008": ("יִגְאָל", "N:N-M-P", "Igal", "yig.al"),
        "H3009": ("יָגַב", "H:V", "to till", "ya.gav"),
        "H3010": ("יָגֵב", "H:N-M", "field", "ya.gev"),
        "H3011": ("יׇגְבְּהָה", "N:N--L", "Jogbehah", "yog.be.hah"),
        "H3012": ("יִגְדַּלְיָ֫הוּ", "N:N-M-P", "Igdaliah", "yig.dal.ya.hu"),
        "H3013": ("יָגָה", "H:V", "to suffer", "ya.gah"),
        "H3014": ("יָגָה", "H:V", "to thrust away", "ya.gah"),
        "H3015": ("יָגוֹן", "H:N-M", "sorrow", "ya.gon"),
        "H3016": ("יָגוֹר", "H:A", "fearing", "ya.gor"),
        "H3017": ("יָגוּר", "N:N--L", "Jagur", "ya.gur"),
        "H3018": ("יְגִיעַ", "H:N-M", "toil", "ye.gi.a"),
        "H3019": ("יָגִיעַ", "H:A", "weary", "ya.gi.a"),
        "H3020": ("יׇגְלִי", "N:N-M-P", "Jogli", "yog.li"),
        "H3021": ("יָגֵעַ", "H:V", "be weary/toil", "ya.ga"),
        "H3022": ("יָגָע", "H:N-M", "toil", "ya.ga"),
        "H3023": ("יָגֵעַ", "H:A", "weary", "ya.ge.a"),
        "H3024": ("יְגִעָה", "H:N-F", "weariness", "ye.gi.ah"),
        "H3025": ("יָגֹר", "H:V", "to fear", "ya.gor"),
        "H3026a": ("יְגַר", "N:N--L", "Jegar-", "ye.gar"),
        "H3026b": ("שָֽׂהֲדוּ", "N:N--L", "-sahadutha", "sa.ha.du.ta"),
        "H3027": ("יָד", "H:N-F", "hand", "yad"),
        "H3028": ("יַד", "A:N-F", "hand", "yad"),
        "H3029": ("יְדָא", "A:V", "to praise", "ye.da"),
        "H3030": ("יִדְאֲלָה", "N:N--L", "Idalah", "yid.a.lah"),
        "H3031": ("יִדְבָּשׁ", "N:N-M-P", "Idbash", "yid.bash"),
        "H3032": ("יָדַד", "H:V", "to cast a lot", "ya.dad"),
        "H3033": ("יְדִדוּת", "H:N-F", "beloved", "ye.di.dut"),
        "H3034": ("יָדָה", "H:V", "to give thanks", "ya.dah"),
        "H3035": ("יִדּוֹ", "N:N-M-P", "Iddo", "yid.do"),
        "H3036": ("יָדוֹן", "N:N-M-P", "Jadon", "ya.don"),
        "H3037": ("יַדּוּעַ", "N:N-M-P", "Jaddua", "yad.du.a"),
        "H3038": ("יְדוּתוּן", "N:N-M-P", "Jeduthun", "ye.du.tun"),
        "H3039a": ("יָדִיד", "H:A", "beloved", "ye.did"),
        "H3039b": ("יָדִידָה", "H:N-F", "love", "ye.did"),
        "H3040": ("יְדִידָה", "N:N-F-P", "Jedidah", "ye.di.dah"),
        "H3041": ("יְדִידְיָה", "N:N-M-P", "Jedidiah", "ye.did.yah"),
        "H3042": ("יְדָיָה", "N:N-M-P", "Jedaiah", "ye.da.yah"),
        "H3043": ("יְדִיעֲאֵל", "N:N-M-P", "Jediael", "ye.di.a.el"),
        "H3044": ("יִדְלָ֑ף", "N:N-M-P", "Jidlaph", "yid.laph"),
        "H3045": ("יָדַע", "H:V", "to know", "ya.da"),
        "H3046": ("יְדַע", "A:V", "to know", "ye.da"),
        "H3047": ("יָדָע", "N:N-M-P", "Jada", "ya.da"),
        "H3048": ("יְדַעְיָה", "N:N-M-P", "Jedaiah", "ye.da.yah"),
        "H3049": ("יִדְּעֹנִי", "H:N-M", "spiritist", "yid.d.o.ni"),
        "H3050": ("יָהּ", "N:N-M-P", "YH--", "yah"),
        "H3051": ("יָהַב", "H:V", "to give", "ya.hav"),
        "H3052": ("יְהַב", "A:V", "to give", "ye.hav"),
        "H3053": ("יְהָב", "H:N-M", "burden", "ye.hav"),
        "H3054": ("יָהַד", "H:V", "to become a Jew", "ya.had"),
        "H3055": ("יְהֻד", "N:N--L", "Jehud", "ye.hud"),
        "H3056": ("יַהְדַּי", "N:N-M-P", "Jahdai", "yeh.day"),
        "H3057": ("יְהֻדִיָּה", "N:N-F-PG", "Judahite", "ye.hu.diy.yah"),
        "H3058": ("יֵהוּא", "N:N-M-P", "Jehu", "ye.hu"),
        "H3059": ("יְהוֹאָחָז", "N:N-M-P", "Jehoahaz", "ye.ho.a.chaz"),
        "H3060": ("יְהוֹאָשׁ", "N:N-M-P", "Jehoash", "ye.ho.ash"),
        "H3061": ("יְהוּד", "N:N--L", "Judea", "ye.hud"),
        "H3062": ("יְהוּדַי", "N:N--PG", "Jew", "ye.hu.da.i"),
        "H3063": ("יְהוּדָה", "N:N-M-P", "Judah", "ye.hu.dah"),
        "H3064": ("יְהוּדִי", "N:N--PG", "Jewish", "ye.hu.di"),
        "H3065": ("יְהוּדִי", "N:N-M-P", "Jehudi", "ye.hu.di"),
        "H3066": ("יְהוּדִית", "H:A-F", "Jewish language", "ye.hu.dit"),
        "H3067": ("יְהוּדִית", "N:N-F-P", "Judith", "ye.hu.dit"),
        "H3068": ("יְהֹוָה", "N:N--T", "YHWH/Lord", "ye.ho.vah"),
        "H3069": ("יְהֹוִה", "N:N-M-T", "YHWH/God", "ye.ho.vih"),
        "H3070": ("יְהֹוָה יִרְאֶה", "N:N--L", "YHWH/Jehovah-jireh", "ye.ho.vah yir.eh"),
        "H3071": ("יְהֹוָה נִסִּי", "N:N--L", "YHWH/Jehovah-nissi", "ye.ho.vah nis.si"),
        "H3072": ("יְהֹוָה צִדְקֵנוּ", "N:N-M-P", "YHWH/Yahweh-tsidkenu", "ye.ho.vah tsid.qe.nu"),
        "H3073": ("יְהֹוָה שָׁלוֹם", "N:N--L", "YHWH/Yahweh-shalom", "ye.ho.vah sha.lom"),
        "H3074": ("יְהֹוָה שָׁמָּה", "N:N--L", "YHWH/Yahweh-shammah", "ye.ho.vah sham.mah"),
        "H3075": ("יְהוֹזָבָד", "N:N-M-P", "Jehozabad", "ye.ho.za.vad"),
        "H3076": ("יְהוֹחָנָן", "N:N-M-P", "Jehohanan", "ye.ho.cha.nan"),
        "H3077": ("יְהוֹיָדָע", "N:N-M-P", "Jehoiada", "ye.ho.ya.da"),
        "H3078": ("יְהוֹיָכִין", "N:N-M-P", "Jehoiachin", "ye.ho.ya.khin"),
        "H3079": ("יְהוֹיָקִים", "N:N-M-P", "Jehoiakim", "ye.ho.ya.qim"),
        "H3080": ("יְהוֹיָרִיב", "N:N-M-P", "Jehoiarib", "ye.ho.ya.riv"),
        "H3081": ("יְהוּכַל", "N:N-M-P", "Jehucal", "ye.hu.khal"),
        "H3082": ("יְהוֹנָדָב", "N:N-M-P", "Jehonadab", "ye.ho.na.dav"),
        "H3083": ("יְהוֹנָתָן", "N:N-M-P", "Jehonathan", "ye.ho.na.tan"),
        "H3084": ("יְהוֹסֵף", "N:N-M-P", "Joseph", "ye.ho.seph"),
        "H3085": ("יְהוֹעַדָּה", "N:N-M-P", "Jehoaddah", "ye.ho.ad.dah"),
        "H3086": ("יְהוֹעַדָּן", "N:N-F-P", "Jehoaddan", "ye.ho.ad.din"),
        "H3087": ("יְהוֹצָדָק", "N:N-M-P", "Jehozadak", "ye.ho.tsa.daq"),
        "H3088": ("יְהוֹרָם", "N:N-M-P", "Jehoram", "ye.ho.ram"),
        "H3089": ("יְהוֹשֶׁבַע", "N:N-F-P", "Jehosheba", "ye.ho.she.va"),
        "H3090": ("יְהוֹשַׁבְעַת", "N:N-F-P", "Jehoshabeath", "ye.ho.shav.at"),
        "H3091": ("יְהוֹשׁוּעַ", "N:N-M-P", "Joshua", "ye.ho.shu.a"),
        "H3092": ("יְהוֹשָׁפָט", "N:N-M-P", "Jehoshaphat", "ye.ho.sha.phat"),
        "H3093": ("יָהִיר", "H:A", "proud", "ya.hir"),
        "H3094": ("יְהַלֶּלְאֵל", "N:N-M-P", "Jehallelel", "ye.hal.lel.el"),
        "H3095": ("יַהֲלֹם", "H:N-M", "jasper", "ya.ha.lom"),
        "H3096": ("יַ֫הַץ", "N:N--L", "Jahaz", "ya.hats"),
        "H3097": ("יוֹאָב", "N:N-M-P", "Joab", "yo.av"),
        "H3098": ("יוֹאָח", "N:N-M-P", "Joah", "yo.ach"),
        "H3099": ("יוֹאָחָז", "N:N-M-P", "Joahaz", "yo.a.chaz"),
        "H3100": ("יוֹאֵל", "N:N-M-P", "Joel", "yo.el"),
        "H3101": ("יוֹאָשׁ", "N:N-M-P", "Joash", "yo.ash"),
        "H3102": ("יוֹב", "N:N-M-P", "Job", "yov"),
        "H3103": ("יוֹבָב", "N:N-M-P", "Jobab", "yo.vav"),
        "H3104": ("יוֹבֵל", "H:N-M", "jubilee/horn", "yo.vel"),
        "H3105": ("יוּבַל", "H:N-M", "stream", "yu.val"),
        "H3106": ("יוּבָ֑ל", "N:N-M-P", "Jubal", "yu.val"),
        "H3107": ("יוֹזָבָד", "N:N-M-P", "Jozabad", "yo.za.vad"),
        "H3108": ("יוֹזָכָר", "N:N-M-P", "Jozachar", "yo.za.khar"),
        "H3109": ("יוֹחָא", "N:N-M-P", "Joha", "yo.ach"),
        "H3110": ("יוֹחָנָן", "N:N-M-P", "Johanan", "yo.cha.nan"),
        "H3111": ("יוֹיָדָע", "N:N-M-P", "Joiada", "yo.ya.da"),
        "H3112": ("יוֹיָכִין", "N:N-M-P", "Jehoiachin", "yo.ya.khin"),
        "H3113": ("יוֹיָקִים", "N:N-M-P", "Joiakim", "yo.ya.qim"),
        "H3114": ("יוֹיָרִיב", "N:N-M-P", "Joiarib", "yo.ya.riv"),
        "H3115": ("יוֹכֶ֫בֶד", "N:N-F-P", "Jochebed", "yo.khe.ved"),
        "H3116": ("יוּכַל", "N:N-M-P", "Jucal", "yu.khal"),
        "H3117": ("יוֹם", "H:N-M", "day", "yom"),
        "H3118": ("יוֹם", "A:N-M", "day", "yom"),
        "H3119": ("יוֹמָם", "H:Adv", "by day", "yo.mam"),
        "H3120": ("יָוָן", "N:N--L / N:N--LG / N:N-M-P", "Javan", "ya.van"),
        "H3121": ("יָוֵן", "H:N-M", "mire", "ya.ven"),
        "H3122": ("יוֹנָדָב", "N:N-M-P", "Jonadab", "yo.na.dav"),
        "H3123": ("יוֹנָה", "H:N-F", "dove", "yo.nah"),
        "H3124": ("יוֹנָה", "N:N-M-P", "Jonah", "yo.nah"),
        "H3125": ("יְוָנִי", "N:N--PG", "Javanite", "ye.va.ni"),
        "H3126": ("יוֹנֵק", "H:N-M", "sapling", "yo.neq"),
        "H3127": ("יוֹנֶ֫קֶת", "H:N-F", "shoot", "yo.ne.qet"),
        "H3128": ("יוֹנַת אֵלֶם רְחֹקִים", "H:N", "Jonath-elem-rechokim", "yo.nat e.lem re.cho.qim"),
        "H3129": ("יוֹנָתָן", "N:N-M-P", "Jonathan", "yo.na.tan"),
        "H3130": ("יוֹסֵף", "N:N-M-P", "Joseph", "yo.seph"),
        "H3131": ("יוֹסִפְיָה", "N:N-M-P", "Josiphiah", "yo.siph.yah"),
        "H3132": ("יוֹעֵאלָה", "N:N-M-P", "Joelah", "yo.e.lah"),
        "H3133": ("יוֹעֵד", "N:N-M-P", "Joed", "yo.ed"),
        "H3134": ("יוֹעֶ֫זֶר", "N:N-M-P", "Joezer", "yo.e.zer"),
        "H3135": ("יוֹעָשׁ", "N:N-M-P", "Joash", "yo.ash"),
        "H3136a": ("יוֹצָדָק", "N:N-M-P", "Jozadak", "yo.tsa.daq"),
        "H3136b": ("יוֹצָדָק", "N:N-M-P", "Jozadak", "yo.tsa.daq"),
        "H3137": ("יוֹקִים", "N:N-M-P", "Jokim", "yo.qim"),
        "H3138": ("יוֹרֶה", "H:N-M", "autumn rain", "yo.reh"),
        "H3139": ("יוֹרָה", "N:N-M-P", "Jorah", "yo.rah"),
        "H3140": ("יוֹרַי", "N:N-M-P", "Jorai", "yo.ray"),
        "H3141": ("יוֹרָם", "N:N-M-P", "Joram", "yo.ram"),
        "H3142": ("י֥וּשַׁב חֶ֖סֶד", "N:N-M-P", "Jushab-hesed", "yu.shav che.sed"),
        "H3143": ("יוֹשִׁבְיָה", "N:N-M-P", "Joshibiah", "yo.shiv.yah"),
        "H3144": ("יוֹשָׁה", "N:N-M-P", "Joshah", "yo.shah"),
        "H3145": ("יוֹשַׁוְיָה", "N:N-M-P", "Joshaviah", "yo.shav.yah"),
        "H3146": ("יוֹשָׁפָט", "N:N-M-P", "Joshaphat", "yo.sha.phat"),
        "H3147": ("יוֹתָם", "N:N-M-P", "Jotham", "yo.tam"),
        "H3148": ("יוֹתֵר", "H:N-M", "advantage", "yo.ter"),
        "H3149": ("יְזִיאֵל", "N:N-M-P", "Jeziel", "ye.zav.el"),
        "H3150": ("יִזִּיָּה", "N:N-M-P", "Izziah", "yiz.ziy.yah"),
        "H3151": ("יָזִיז", "N:N-M-P", "Jaziz", "ya.ziz"),
        "H3152": ("יִזְלִיאָה", "N:N-M-P", "Izliah", "yiz.li.ah"),
        "H3153": ("יְזַנְיָה", "N:N-M-P", "Jezaniah", "ye.zan.yah"),
        "H3154": ("יֶ֫זַע", "H:N-M", "sweat", "ye.za"),
        "H3155": ("יִזְרָ֑ח", "N:N--PG", "Izrahite", "yiz.rach"),
        "H3156": ("יִזְרַחְיָה", "N:N-M-P", "Izrahiah", "yiz.rach.yah"),
        "H3157": ("יִזְרְעֶאל", "N:N-M-P / N:N--L", "Jezreel", "yiz.re.el"),
        "H3158": ("יִזְרְעֵאלִי", "N:N--LG", "Jezreelite", "yiz.re.e.li"),
        "H3159": ("יִזְרְעֵאלִית", "N:N-F-L", "Jezreelitess", "yiz.re.e.lit"),
        "H3160": ("חֻבָּה", "N:N-M-P", "Jehubbah", "ye.chub.bah"),
        "H3161": ("יָחַד", "H:V", "be joined", "ya.chad"),
        "H3162a": ("יַ֫חַד", "H:N-M", "unitedness", "ya.chad"),
        "H3162b": ("יָחְדָּו", "H:Adv", "together", ""),
        "H3163": ("יַחְדּוֹ", "N:N-M-P", "Jahdo", "yach.do"),
        "H3164": ("יַחְדִּיאֵל", "N:N-M-P", "Jahdiel", "yach.di.el"),
        "H3165": ("יֶחְדְּיָ֫הוּ", "N:N-M-P", "Jehdeiah", "yech.diy.ya.hu"),
        "H3166": ("יַחֲזִיאֵל", "N:N-M-P", "Jahaziel", "ya.cha.zi.el"),
        "H3167": ("יַחְזְיָה", "N:N-M-P", "Jahzeiah", "yach.ze.yah"),
        "H3168": ("יְחֶזְקֵאל", "N:N-M-P", "Jehezkel", "ye.chez.qel"),
        "H3169": ("יְחִזְקִיָּה", "N:N-M-P", "Jehizkiah", "ye.chiz.qiy.yah"),
        "H3170": ("יַחְזֵרָה", "N:N-M-P", "Jahzerah", "yach.ze.rah"),
        "H3171": ("יְחִיאֵל", "N:N-M-P", "Jehiel", "ye.chi.el"),
        "H3172": ("יְחִיאֵלִי", "N:N-M-P", "Jehieli", "ye.chi.e.li"),
        "H3173": ("יָחִיד", "H:A", "only", "ya.chid"),
        "H3174": ("יְחִיָּה", "N:N-M-P", "Jehiah", "ye.chiy.yah"),
        "H3175": ("יָחִיל", "H:A", "waiting", "ya.chil"),
        "H3176": ("יָחַל", "H:V", "to wait", "ya.chal"),
        "H3177": ("יַחְלְאֵל", "N:N-M-P", "Jahleel", "yach.le.el"),
        "H3178": ("יַחְלְאֵלִי", "N:N--PG", "Jahleelite", "yach.le.e.li"),
        "H3179": ("יָחַם", "H:V", "to conceive", "ya.cham"),
        "H3180": ("יַחְמוּר", "H:N-M", "roebuck", "yach.mur"),
        "H3181": ("יַחְמַי", "N:N-M-P", "Jahmai", "yach.may"),
        "H3182": ("יָחֵף", "H:A", "barefoot", "ya.cheph"),
        "H3183": ("יַחְצְאֵל", "N:N-M-P", "Jahzeel", "yach.tse.el"),
        "H3184": ("יַחְצְאֵלִי", "N:N--PG", "Jahzeelite", "yach.tse.e.li"),
        "H3185": ("יַחְצִיאֵל", "N:N-M-P", "Jahziel", "yach.tsi.el"),
        "H3186": ("יָחַר", "H:V", "to delay", "ya.char"),
        "H3187": ("יָחַשׂ", "H:V", "to enroll", "ya.cha.s"),
        "H3188": ("יַ֫חַשׂ", "H:N-M", "genealogy", "ya.cha.s"),
        "H3189": ("יַ֫חַת", "N:N-M-P", "Jahath", "ya.chat"),
        "H3190": ("יָטַב", "H:V", "be good", "ya.tav"),
        "H3191": ("יְטַב", "A:V", "be good", "ye.tav"),
        "H3192": ("יׇטְבָה", "N:N--L", "Jotbah", "yot.vah"),
        "H3193": ("יׇטְבָ֫תָה", "N:N--L", "Jotbathah", "yot.va.tah"),
        "H3194": ("יֻטָּה", "N:N--L", "Juttah", "yut.tah"),
        "H3195": ("יְטוּר", "N:N-M-P / N:N--L", "Jetur", "ye.tur"),
        "H3196": ("יַ֫יִן", "H:N-M", "wine", "ya.yin"),
        "H3197": ("יַךְ", "H:N-M", "side", "yakh"),
        "H3198": ("יָכַח", "H:V", "to rebuke", "ya.khach"),
        "H3199": ("יָכִין", "N:N-M-P", "Jachin", "ya.khin"),
        "H3200": ("יָכִינִי", "N:N--PG", "Jachinite", "ya.khi.ni"),
        "H3201": ("יָכֹל", "H:V", "be able", "ya.khol"),
        "H3202": ("יְכִל", "A:V", "be able", "ye.khel"),
        "H3203": ("יְכָלְיָהוּ", "N:N-F-P", "Jecholiah", "ye.khol.yah"),
        "H3204": ("יְכוֹנְיָה", "N:N-M-P", "Jeconiah", "ye.khon.yah"),
        "H3205": ("יָלַד", "H:V", "to beget", "ya.lad"),
        "H3206": ("יֶ֫לֶד", "H:N-M", "youth", "ye.led"),
        "H3207": ("יַלְדָּה", "H:N-F", "maiden", "yal.dah"),
        "H3208": ("יַלְדוּת", "H:N-F", "youth", "yal.dut"),
        "H3209": ("יִלּוֹד", "H:A", "born", "yil.lod"),
        "H3210": ("יָלוֹן", "N:N-M-P", "Jalon", "ya.lon"),
        "H3211": ("יָלִיד", "H:A", "born", "ya.lid"),
        "H3212": ("יָלַךְ", "H:V", "to walk", "ya.lakh"),
        "H3213": ("יָלַל", "H:V", "to wail", "ya.lal"),
        "H3214": ("יְלֵל", "H:N-M", "howling", "ye.lel"),
        "H3215": ("יְלָלָה", "H:N-F", "wailing", "ye.la.lah"),
        "H3216": ("יָלַע", "H:V", "to blurt", "ya.la"),
        "H3217": ("יַלֶּ֫פֶת", "H:N-F", "scab", "yal.le.phet"),
        "H3218": ("יֶ֫לֶק", "H:N-M", "locust", "ye.leq"),
        "H3219": ("יַלְקוּט", "H:N-M", "pouch", "yal.qut"),
        "H3220": ("יָם", "H:N-M", "sea", "yam"),
        "H3221": ("יַם", "A:N-M", "sea", "yam"),
        "H3222": ("יֵמִם", "H:N-M", "hot spring", "yem"),
        "H3223": ("יְמוּאֵל", "N:N-M-P", "Jemuel", "ye.mu.el"),
        "H3224": ("יְמִימָה", "N:N-F-P", "Jemimah", "ye.mi.mah"),
        "H3225": ("יָמִין", "H:N-F", "right", "ya.min"),
        "H3226": ("יָמִין", "N:N-M-P", "Jamin", "ya.min"),
        "H3227a": ("יְמִינִי", "H:A", "right", "ye.mi.ni"),
        "H3227b": ("יְמִינִי", "N:N--PG", "Jaminite", "ye.mi.ni"),
        "H3228": ("יָֽמִינִי", "N:N--PG", "Jaminite", "ye.mi.ni"),
        "H3229": ("יִמְלָה", "N:N-M-P", "Imla", "yim.la"),
        "H3230": ("יַמְלֵךְ", "N:N-M-P", "Jamlech", "yam.lekh"),
        "H3231": ("יָמַן", "H:V", "to go right", "ya.man"),
        "H3232": ("יִמְנָה", "N:N-M-P", "Imnah", "yim.nah"),
        "H3233": ("יְמָנִי", "H:A", "right", "ye.ma.ni"),
        "H3234": ("יִמְנָע", "N:N-M-P", "Imna", "yim.na"),
        "H3235": ("יָמַר", "H:V", "to exchange", "ya.mar"),
        "H3236": ("יִמְרָה", "N:N-M-P", "Imrah", "yim.rah"),
        "H3237": ("יָמַשׁ", "H:V", "to feel", "ya.mash"),
        "H3238": ("יָנָה", "H:V", "to oppress", "ya.nah"),
        "H3239a": ("יָנוֹחַ", "N:N--L", "Janoah", "ya.no.ach"),
        "H3239b": ("יָנ֫וֹחָה", "N:N--L", "Janoah", ""),
        "H3240": ("יָנַח", "H:V", "to rest", "ya.nach"),
        "H3241": ("יָנוּם", "N:N--L", "Janum", "ya.nim"),
        "H3242": ("יְנִיקָה", "H:N-F", "shoot", "ye.ni.qah"),
        "H3243": ("יָנַק", "H:V", "to suck", "ya.naq"),
        "H3244": ("יַנְשׁוּף", "H:N-M", "owl", "yan.shuph"),
        "H3245": ("יָסַד", "H:V", "to found", "ya.sad"),
        "H3246": ("יְסֻד", "H:N-M", "beginning", "ye.sud"),
        "H3247": ("יְסוֹד", "H:N-F", "foundation", "ye.sod"),
        "H3248": ("יְסוּדָה", "H:N-F", "foundation", "ye.su.dah"),
        "H3249": ("יסור", "H:A", "departed", "ya.sur"),
        "H3250": ("יִסּוֹר", "H:N-M", "faultfinder", "yis.sor"),
        "H3251": ("יָסַךְ", "H:V", "to pour", "ya.sakh"),
        "H3252": ("יִסְכָּה", "N:N-F-P", "Iscah", "yis.kah"),
        "H3253": ("יִסְמַכְיָהוּ", "N:N-M-P", "Ismachiah", "yis.makh.ya.hu"),
        "H3254": ("יָסַף", "H:V", "to add", "ya.saph"),
        "H3255": ("יְסַף", "A:V", "to add", "ye.saph"),
        "H3256": ("יָסַר", "H:V", "to discipline", "ya.sar"),
        "H3257": ("יָע", "H:N-M", "shovel", "ya"),
        "H3258": ("יַעְבֵּץ", "N:N-M-P / N:N--L", "Jabez", "ya.bets"),
        "H3259": ("יָעַד", "H:V", "to appoint", "ya.ad"),
        "H3260": ("יֶעְדּוֹ", "N:N-M-P", "Iddo", "ye.di"),
        "H3261": ("יָעָה", "H:V", "to sweep away", "ya.ah"),
        "H3262": ("יְעוּאֵל", "N:N-M-P", "Jeuel", "ye.u.el"),
        "H3263": ("יְעוּץ", "N:N-M-P", "Jeuz", "ye.uts"),
        "H3264": ("יָעוֹר", "H:N-M", "wood", "ya.or"),
        "H3265": ("יָעִיר", "N:N-M-P", "Jair", "ya.ur"),
        "H3266": ("יְעוּשׁ", "N:N-M-P", "Jeush", "ye.ush"),
        "H3267": ("יָעַז", "H:V", "be fierce", "ya.az"),
        "H3268": ("יַעֲזִיאֵל", "N:N-M-P", "Jaaziel", "ya.a.zi.el"),
        "H3269": ("יַעֲזִיָּ֫הוּ", "N:N-M-P", "Jaaziah", "ya.a.ziy.ya.hu"),
        "H3270": ("יַעְזֵר", "N:N--L", "Jazer", "ya.a.zer"),
        "H3271": ("יָעַט", "H:V", "to cover", "ya.at"),
        "H3272a": ("יְעַט", "A:V", "to counsel", "ye.at"),
        "H3272b": ("יָעֵט", "A:N-M", "counsellor", ""),
        "H3273": ("יְעִיאֵל", "N:N-M-P", "Jeiel", "ye.i.el"),
        "H3274": ("יְעִישׁ", "N:N-M-P", "Jeush", "ye.ish"),
        "H3275": ("יַעְכָּן", "N:N-M-P", "Jacan", "ya.kan"),
        "H3276": ("יָעַל", "H:V", "to gain", "ya.al"),
        "H3277": ("יָעֵל", "H:N-M", "goat", "ya.el"),
        "H3278": ("יָעֵל", "N:N-F-P", "Jael", "ya.el"),
        "H3279": ("יַעֲלָא", "N:N-M-P", "Jaala", "ya.a.la"),
        "H3280": ("יַעֲלָה", "H:N-F", "doe", "ya.a.lah"),
        "H3281": ("יַעְלָם", "N:N-M-P", "Jalam", "ya.lam"),
        "H3282": ("יַעַן", "H:Conj", "because", "ya.an"),
        "H3283": ("יָעֵן", "H:N-F", "ostrich", "ya.en"),
        "H3284": ("יַעֲנָה", "H:N-F", "ostrich", "ya.a.nah"),
        "H3285": ("יַעֲנַי", "N:N-M-P", "Janai", "ya.a.nay"),
        "H3286": ("יָעֵף", "H:V", "to faint", "ya.aph"),
        "H3287": ("יָעֵף", "H:A", "weary", "ya.eph"),
        "H3288": ("יְעָף", "H:N-M", "weariness", "ye.aph"),
        "H3289": ("יָעַץ", "H:V", "to advise", "ya.ats"),
        "H3290": ("יַעֲקֹב", "N:N-M-P", "Jacob", "ya.a.qov"),
        "H3291": ("יַעֲקֹ֫בָה", "N:N-M-P", "Jaakobah", "ya.a.qo.vah"),
        "H3292": ("יַעֲקָן", "N:N-M-P", "Akan", "ya.a.qan"),
        "H3293a": ("יַ֫עַר", "H:N-M", "wood", "ya.ar"),
        "H3293b": ("יַ֫עַר", "H:N-M", "honeycomb", "ya.ar"),
        "H3294": ("יַעְרָה", "N:N-M-P", "Jarah", "ya.rah"),
        "H3295": ("יַעְרָה", "H:N-F", "honeycomb", "ya.a.rah"),
        "H3296": ("יַעֲרֵי אֹרְגִים", "N:N-M-P", "Jaare-oregim", "ya.a.re o.re.gim"),
        "H3297": ("יְעָרִים", "N:N--L", "Jearim", "ye.a.rim"),
        "H3298": ("יַעֲרֶשְׁיָה", "N:N-M-P", "Jaareshiah", "ya.a.resh.yah"),
        "H3299": ("וְיַעֲשׂוֹ", "N:N-M-P", "Jaasu", "ya.a.su"),
        "H3300": ("יַעֲשִׂיאֵל", "N:N-M-P", "Jaasiel", "ya.a.si.el"),
        "H3301": ("יִפְדְּיָה", "N:N-M-P", "Iphdeiah", "yiph.de.yah"),
        "H3302": ("יָפָה", "H:V", "be beautiful", "ya.phah"),
        "H3303": ("יָפֶה", "H:A", "beautiful", "ya.pheh"),
        "H3304": ("יְפֵה־פִיָּה", "H:A", "pretty", "ye.pheh-phiy.yah"),
        "H3305": ("יָפוֹ", "N:N--L", "Joppa", "ya.pho"),
        "H3306": ("יָפַח", "H:V", "to breathe", "ya.phach"),
        "H3307": ("יָפֵחַ", "H:A", "breathing", "ya.phe.ach"),
        "H3308": ("יֳפִי", "H:N-M", "beauty", "yo.phi"),
        "H3309": ("יָפִיעַ", "N:N--L / N:N-M-P", "Japhia", "ya.phi.a"),
        "H3310": ("יַפְלֵט", "N:N-M-P", "Japhlet", "yaph.let"),
        "H3311": ("יַפְלֵטִי", "N:N--PG", "Japhletite", "yaph.le.ti"),
        "H3312": ("יְפֻנֶּה", "N:N-M-P", "Jephunneh", "ye.phun.neh"),
        "H3313": ("יָפַע", "H:V", "to shine", "ya.pah"),
        "H3314": ("יִפְעָה", "H:N-F", "splendor", "yiph.ah"),
        "H3315": ("יֶ֫פֶת", "N:N-M-P", "Japheth", "ye.phet"),
        "H3316": ("יִפְתָּח", "N:N-M-P / N:N--L", "Jephthah", "yiph.tach"),
        "H3317": ("יִפְתַּח־אֵל", "N:N--L", "Iphtah-el", "yiph.tach-el"),
        "H3318": ("יָצָא", "H:V", "to come out", "ya.tsa"),
        "H3319": ("שֵׁיצִי", "A:V", "to finish", "ye.tsa"),
        "H3320": ("יָצַב", "H:V", "to stand", "ya.tsav"),
        "H3321": ("יְצַב", "A:V", "to know", "ye.tsav"),
        "H3322": ("יָצַג", "H:V", "to set", "ya.tsag"),
        "H3323": ("יִצְהָר", "H:N-M", "oil", "yits.har"),
        "H3324": ("יִצְהָר", "N:N-M-P", "Izhar", "yits.har"),
        "H3325": ("יִצְהָרִי", "N:N--PG", "Izharite", "yits.ha.ri"),
        "H3326a": ("יָצוּעַ", "H:N-M", "bed", "ya.tsu.a"),
        "H3326b": ("יָצִיעַ", "H:N-M", "floor", ""),
        "H3327": ("יִצְחָק", "N:N-M-P", "Isaac", "yits.chaq"),
        "H3328": ("יִצְחַר", "N:N-M-P", "Izhar", "yits.char"),
        "H3329": ("יָצִיא", "H:A", "offspring", "ya.tsi"),
        "H3330": ("יַצִּיב", "A:A", "certain", "yats.tsiv"),
        "H3331": ("יָצַע", "H:V", "to lay", "ya.tsa"),
        "H3332": ("יָצַק", "H:V", "to pour", "ya.tsaq"),
        "H3333": ("יְצֻקָה", "H:N-F", "casting", "ye.tsu.qah"),
        "H3334": ("יָצַר", "H:V", "be distressed", "ya.tsar"),
        "H3335": ("יָצַר", "H:V", "to form", "ya.tsar"),
        "H3336": ("יֵ֫צֶר", "H:N-M", "intention", "ye.tser"),
        "H3337": ("יֵ֫צֶר", "N:N-M-P", "Jezer", "ye.tser"),
        "H3338": ("יְצֻרִים", "H:N-M", "member", "ya.tsur"),
        "H3339": ("יִצְרִי", "N:N-M-P", "Izri", "yits.ri"),
        "H3340": ("יִצְרִי", "N:N--PG", "Jezerite", "yits.ri"),
        "H3341": ("יָצַת", "H:V", "to kindle", "ya.tsat"),
        "H3342": ("יֶ֫קֶב", "H:N-M", "wine", "ye.qev"),
        "H3343": ("יְקַבְצְאֵל", "N:N--L", "Jekabzeel", "ye.qav.tse.el"),
        "H3344": ("יָקַד", "H:V", "to burn", "ya.qad"),
        "H3345": ("יְקַד", "A:V", "to burn", "ye.qad"),
        "H3346": ("יְקֵדָא", "A:N-F", "burning", "ye.qe.da"),
        "H3347": ("יׇקְדְעָם", "N:N--L", "Jokdeam", "yoq.de.am"),
        "H3348": ("יָקֶה", "N:N-M-P", "Jakeh", "ya.qeh"),
        "H3349": ("יִקְּהָה", "H:N-F", "obedience", "yiq.qa.hah"),
        "H3350": ("יְקוֹד", "H:N-M", "burning", "ye.qod"),
        "H3351": ("יְקוּם", "H:N-M", "existence", "ye.qum"),
        "H3352": ("יָקוֹשׁ", "H:N-M", "fowler", "ya.qosh"),
        "H3353": ("יָקוּשׁ", "H:N-M", "fowler", "ya.qush"),
        "H3354": ("יְקוּתִיאֵל", "N:N-M-P", "Jekuthiel", "ye.qu.ti.el"),
        "H3355": ("יׇקְטָן", "N:N-M-P", "Joktan", "yoq.tan"),
        "H3356": ("יָקִים", "N:N-M-P", "Jakim", "ya.qim"),
        "H3357": ("יַקִּיר", "H:A", "precious", "yaq.qir"),
        "H3358": ("יַקִּיר", "A:A", "honorable", "yaq.qir"),
        "H3359": ("יְקַמְיָה", "N:N-M-P", "Jekamiah", "ye.qam.yah"),
        "H3360": ("יְקַמְעָם", "N:N-M-P", "Jekameam", "ye.qam.am"),
        "H3361": ("יׇקְמְעָם", "N:N--L", "Jokmeam", "yoq.me.am"),
        "H3362": ("יׇקְנְעָם", "N:N--L", "Jokneam", "yoq.ne.am"),
        "H3363": ("יָקַע", "H:V", "to dislocate/hang", "ya.qa"),
        "H3364": ("יָקַץ", "H:V", "to awake", "ya.qats"),
        "H3365": ("יָקַר", "H:V", "be precious", "ya.qar"),
        "H3366": ("יְקָר", "H:N-M", "preciousness", "ye.qar"),
        "H3367": ("יְקָר", "A:N-M", "honor", "ye.qar"),
        "H3368": ("יָקָר", "H:A", "precious", "ya.qar"),
        "H3369": ("יָקֹשׁ", "H:V", "to snare", "ya.qosh"),
        "H3370": ("יׇקְשָׁן", "N:N-M-P", "Jokshan", "yoq.shan"),
        "H3371": ("יׇקְתְאֵל", "N:N--L", "Joktheel", "yoq.te.el"),
        "H3372": ("יָרֵא", "H:V", "to fear", "ya.re"),
        "H3373": ("יָרֵא", "H:A", "afraid", "ya.re"),
        "H3374": ("יִרְאָה", "H:N-F", "fear", "yir.ah"),
        "H3375": ("יִרְאוֹן", "N:N--L", "Yiron", "yir.on"),
        "H3376": ("יִרְאִיָּיה", "N:N-M-P", "Irijah", "yir.iy.yah"),
        "H3377": ("יָרֵב", "N:N-M-P", "Jareb", "ya.rev"),
        "H3378": ("יְרֻבַּ֫עַל", "N:N-M-P", "Jerubbaal", "ye.rub.ba.al"),
        "H3379": ("יָֽרָבְעָם", "N:N-M-P", "Jeroboam", "ya.rov.am"),
        "H3380": ("יְרֻבֶּשֶׁת", "N:N-M-P", "Jerubbesheth", "ye.rub.be.shet"),
        "H3381": ("יָרַד", "H:V", "to go down", "ya.rad"),
        "H3382": ("יֶ֫רֶד", "N:N-M-P", "Jared", "ye.red"),
        "H3383": ("יַרְדֵּן", "N:N--L", "Jordan", "yar.den"),
        "H3384a": ("יָרָא", "H:V", "to shoot", "ya.rah"),
        "H3384b": ("יָרָה", "H:V", "to show", "ya.rah"),
        "H3385": ("יְרוּאֵל", "N:N--L", "Jeruel", "ye.ru.el"),
        "H3386": ("יָרוֹחַ", "N:N-M-P", "Jaroah", "ya.ro.ach"),
        "H3387": ("יָרוֹק", "H:N-M", "green", "ya.roq"),
        "H3388": ("יְרוּשָׁא, יְרוּשָׁה", "N:N-F-P", "Jerusha", "ye.ru.sah"),
        "H3389": ("יְרוּשָׁלִַ֫ם, יְרוּשְׁלֵם", "N:N--L", "Jerusalem", "ye.ru.sha.laim"),
        "H3390": ("יְרוּשְׁלֵם", "N:N--L", "Jerusalem", "ye.ru.sha.lem"),
        "H3391": ("יֶ֫רַח", "H:N-M", "month", "ye.rach"),
        "H3392": ("יֶ֫רַח", "N:N-M-P", "Jerah", "ye.rach"),
        "H3393": ("יְרַח", "A:N-M", "month", "ye.rach"),
        "H3394": ("יָרֵחַ", "H:N-M", "moon", "ya.re.ach"),
        "H3395": ("יְרֹחָם", "N:N-M-P", "Jeroham", "ye.ro.cham"),
        "H3396": ("יְרַחְמְאֵל", "N:N-M-P", "Jerahmeel", "ye.rach.me.el"),
        "H3397": ("יְרַחְמְאֵלִי", "N:N--PG", "Jerahmeelite", "ye.rach.me.e.li"),
        "H3398": ("יַרְחָע", "N:N-M-P / N:N-M-T", "Jarha", "yar.ach"),
        "H3399a": ("יָרַט", "H:V", "to push", "ya.rat"),
        "H3399b": ("רָטָה", "H:V", "to wring", ""),
        "H3400": ("יְרִיאֵל", "N:N-M-P", "Jeriel", "ye.ri.el"),
        "H3401": ("יָרִיב", "H:N-M", "opponent", "ya.riv"),
        "H3402": ("יָרִיב", "N:N-M-P", "Jarib", "ya.riv"),
        "H3403": ("יְרִיבַי", "N:N-M-P", "Jeribai", "ye.ri.vay"),
        "H3404": ("יְרִיָּהוּ", "N:N-M-P", "Jeriah", "ye.riy.yah"),
        "H3405": ("יְרִיחוֹ", "N:N--L", "Jericho", "ye.ri.cho"),
        "H3406": ("יְרֵמוֹת, יְרִימוֹת", "N:N-M-P", "Jeremoth", "ye.ri.mot"),
        "H3407": ("יְרִיעָה", "H:N-F", "curtain", "ye.ri.ah"),
        "H3408": ("יְרִיעוֹת", "N:N-F-P", "Jerioth", "ye.ri.ot"),
        "H3409": ("יָרֵךְ", "H:N-F", "thigh", "ya.rekh"),
        "H3410": ("יַרְכָא", "A:N-F", "thigh", "yar.kah"),
        "H3411": ("יַרְכָה", "H:N-F", "flank", "ye.re.khah"),
        "H3412": ("יַרְמוּת", "N:N--L", "Jarmuth", "yar.mut"),
        "H3413": ("יְרֵמַי", "N:N-M-P", "Jeremai", "ye.re.may"),
        "H3414": ("יִרְמְיָ֫הוּ", "N:N-M-P", "Jeremiah", "yir.me.yah"),
        "H3415": ("יָרַע", "H:V", "be ill", "ya.ra"),
        "H3416": ("יִרְפְּאֵל", "N:N--L", "Irpeel", "yir.pe.el"),
        "H3417": ("יָרַק", "H:V", "to spit", "ya.raq"),
        "H3418": ("יֶ֫רֶק", "H:N-M", "green", "ye.req"),
        "H3419": ("יָרָק", "H:N-M", "herb", "ya.raq"),
        "H3420": ("יֵרָקוֹן", "H:N-M", "mildew", "ye.ra.qon"),
        "H3421": ("יׇרְקְעָ֑ם", "N:N-M-P", "Jorkeam", "yor.qe.am"),
        "H3422": ("יְרַקְרַק", "H:A", "greenish", "ye.raq.raq"),
        "H3423": ("יָרַשׁ", "H:V", "to possess", "ya.rash"),
        "H3424": ("יְרֵשָׁה", "H:N-F", "possession", "ye.re.shah"),
        "H3425": ("יְרֻשָּׁה", "H:N-F", "possession", "ye.rush.shah"),
        "H3426": ("יֵשׁ", "H:Part", "be", "yesh"),
        "H3427": ("יָשַׁב", "H:V", "to dwell", "ya.shav"),
        "H3428": ("יֶֽשֶׁבְאָב", "N:N-M-P", "Jeshebeab", "ye.shev.av"),
        "H3429": ("יֹשֵׁב בַּשֶּׁ֫בֶת", "N:N-M-P", "Josheb-basshebet", "yo.shev bash.she.vet"),
        "H3430": ("יִשְׁבּוֹ בְּנֹב", "N:N-M-P", "Ishbi-benob", "yish.bo b.nov"),
        "H3431": ("יִשְׁבַּח", "N:N-M-P", "Ishbah", "yish.bach"),
        "H3432": ("יָשֻׁבִי", "N:N--PG", "Jashubite", "ya.shu.vi"),
        "H3433": ("יָשֻׁ֫בִי לָ֑חֶם", "N:N--L", "Jashubi-lehem", "ya.shu.vi le.chem"),
        "H3434": ("יָשׇׁבְעָם", "N:N-M-P", "Jashobeam", "ya.shov.am"),
        "H3435": ("יִשְׁבָּק", "N:N-M-P", "Ishbak", "yish.baq"),
        "H3436": ("יׇשְׁבְּקָ֫שָׁה", "N:N-M-P", "Joshbekashah", "yosh.be.qa.shah"),
        "H3437": ("יָשׁוּב", "N:N-M-P", "Jashub", "ya.shuv"),
        "H3438": ("יִשְׁוָה", "N:N-M-P", "Ishvah", "yish.vah"),
        "H3439": ("יְשׁוֹחָיָה", "N:N-M-P", "Jeshohaiah", "ye.sho.cha.yah"),
        "H3440": ("יִשְׁוִי", "N:N-M-P", "Ishvi", "yish.vi"),
        "H3441": ("יִשְׁוִי", "N:N--PG", "Ishvite", "yish.vi"),
        "H3442": ("יֵשׁוּעַ", "N:N-M-P / N:N--L", "Jeshua", "ye.shu.a"),
        "H3443": ("יֵשׁוּעַ", "N:N-M-P", "Jeshua", "ye.shu.a"),
        "H3444": ("יְשׁוּעָה", "H:N-F", "salvation", "ye.shu.ah"),
        "H3445": ("יֶ֫שַׁח", "H:N-M", "emptiness", "ye.shach"),
        "H3446": ("יִשְׂחָק", "N:N-M-P", "Isaac", "yis.chaq"),
        "H3447": ("יָשַׁט", "H:V", "to extend", "ya.shat"),
        "H3448": ("יִשַׁי", "N:N-M-P", "Jesse", "yi.shay"),
        "H3449": ("יִשִּׁיָּהוּ", "N:N-M-P", "Isshiah", "yish.shiy.yah"),
        "H3450": ("יְשִׁימִאֵל", "N:N-M-P", "Jesimiel", "ye.shi.ma.el"),
        "H3451": ("יְשִׁימָה", "H:N-F", "desolation", "ye.shi.mah"),
        "H3452": ("יְשִׁימוֹן", "H:N-M", "wilderness", "ye.shi.mon"),
        "H3453": ("יָשִׁישׁ", "H:A", "aged", "ya.shish"),
        "H3454": ("יְשִׁישַׁי", "N:N-M-P", "Jeshishai", "ye.shi.sah"),
        "H3455": ("יָשַׂם", "H:V", "to put", "ya.sam"),
        "H3456": ("יָשַׁם", "H:V", "be desolate", "ya.sham"),
        "H3457": ("יִשְׁמָא", "N:N-M-P", "Ishma", "yish.ma"),
        "H3458": ("יִשְׁמָעֵאל", "N:N-M-P", "Ishmael", "yish.ma.el"),
        "H3459": ("יִשְׁמְעֵאלִי", "N:N--PG", "Ishmaelite", "yish.ma.e.li"),
        "H3460": ("יִשְׁמַעְיָ֫הוּ", "N:N-M-P", "Ishmaiah", "yish.ma.yah"),
        "H3461": ("יִשְׁמְרַי", "N:N-M-P", "Ishmerai", "yish.me.ray"),
        "H3462": ("יָשֵׁן", "H:V", "to sleep", "ya.shen"),
        "H3463": ("יָשֵׁן", "H:A", "sleeping", "ya.shen"),
        "H3464": ("יָשֵׁן", "N:N-M-P", "Jashen", "ya.shen"),
        "H3465": ("יָשָׁן", "H:A", "old", "ya.shan"),
        "H3466": ("יְשָׁנָה", "N:N--L", "Jeshanah", "ye.sha.nah"),
        "H3467": ("יָשַׁע", "H:V", "to save", "ya.sah"),
        "H3468": ("יֵ֫שַׁע", "H:N-M", "salvation", "ye.sah"),
        "H3469": ("יִשְׁעִי", "N:N-M-P", "Ishi", "yish.i"),
        "H3470a": ("יְשַׁעְיָ֫הוּ", "N:N-M-P", "Isaiah", "ye.sha.yah"),
        "H3470b": ("יְשַׁעְיָה", "N:N-M-P", "Isaiah", "ye.sha.yah"),
        "H3471": ("יָֽשְׁפֶה", "H:N-M", "jasper", "ya.she.pheh"),
        "H3472": ("יִשְׁפָּה", "N:N-M-P", "Ishpah", "yish.pah"),
        "H3473": ("יִשְׁפָּן", "N:N-M-P", "Ishpan", "yish.pan"),
        "H3474": ("יָשַׁר", "H:V", "to smooth", "ya.shar"),
        "H3475": ("יֶ֫שֶׁר", "N:N-M-P", "Jesher", "ye.sher"),
        "H3476": ("יֹ֫שֶׁר", "H:N-M", "uprightness", "yo.sher"),
        "H3477": ("יָשָׁר", "H:A", "upright", "ya.shar"),
        "H3478": ("יִשְׂרָאֵל", "N:N-M-P / N:N-M-PG", "Israel", "yis.ra.el"),
        "H3479": ("יִשְׂרָאֵל", "N:N-M-P", "Israel", "yis.ra.el"),
        "H3480": ("יְשַׂרְאֵ֫לָה", "N:N-M-P", "Jesharelah", "ye.sar.e.lah"),
        "H3481": ("יִשְׂרְאֵלִי", "N:N--PG", "Israelite", "yis.r.e.li"),
        "H3482": ("יִשְׂרְאֵלִית", "N:N-F-PG", "Israelitess", "yis.r.e.lit"),
        "H3483": ("יְשָׁרָה", "H:N-F", "uprightness", "yish.rah"),
        "H3484": ("יְשֻׁרוּן", "N:N-M-P", "Jeshurun", "ye.shu.run"),
        "H3485": ("יִשָּׂשכָר", "N:N-M-P", "Issachar", "yis.sa.s.khar"),
        "H3486": ("יָשֵׁשׁ", "H:A", "decrepit", "ya.shesh"),
        "H3487": ("יָת", "A:Part", "whom", "yat"),
        "H3488": ("יְתִב", "A:V", "to dwell", "ye.tiv"),
        "H3489": ("יָתֵד", "H:N-F", "peg", "ya.ted"),
        "H3490": ("יָתוֹם", "H:N-M", "orphan", "ya.tom"),
        "H3491": ("יְתוּר", "H:N-M", "range", "ya.tur"),
        "H3492": ("יַתִּיר", "N:N--L", "Jattir", "yat.tir"),
        "H3493": ("יַתִּיר", "A:A", "preeminent", "yat.tir"),
        "H3494": ("יִתְלָה", "N:N--L", "Ithlah", "yit.lah"),
        "H3495": ("יִתְמָה", "N:N-M-P", "Ithmah", "yit.mah"),
        "H3496": ("יַתְנִיאֵל", "N:N-M-P", "Jathniel", "yat.ni.el"),
        "H3497": ("יִתְנָ֑ן", "N:N--L", "Ithnan", "yit.nan"),
        "H3498": ("יָתַר", "H:V", "to remain", "ya.tar"),
        "H3499a": ("יֶ֫תֶר", "H:N-M", "remainder", "ye.ter"),
        "H3499b": ("יֶ֫תֶר", "H:N-M", "cord", "ye.ter"),
        "H3500": ("יֶ֫תֶר", "N:N-M-P", "Jether", "ye.ter"),
        "H3501": ("יִתְרָא", "N:N-M-P", "Ithra", "yit.ra"),
        "H3502": ("יִתְרָה", "H:N-F", "abundance", "yit.rah"),
        "H3503": ("יִתְרוֹ", "N:N-M-P", "Jethro", "yit.ro"),
        "H3504": ("יִתְרוֹן", "H:N-M", "advantage", "yit.ron"),
        "H3505": ("יִתְרִי", "N:N--PG", "Ithrite", "yit.ri"),
        "H3506": ("יִתְרָן", "N:N-M-P", "Ithran", "yit.ran"),
        "H3507": ("יִתְרְעָם", "N:N-M-P", "Ithream", "yit.re.am"),
        "H3508": ("יֹתֶ֫רֶת", "H:N-F", "lobe", "yo.te.ret"),
        "H3509": ("יְתֵת", "N:N-M-P", "Jetheth", "ye.tet"),
        "H3510": ("כָּאַב", "H:V", "to pain", "ka.av"),
        "H3511": ("כְּאֵב", "H:N-M", "pain", "ke.ev"),
        "H3512a": ("כָּאָה", "H:V", "be disheartened", "ka.ah"),
        "H3512b": ("כָּאֶה", "H:A", "disheartened", ""),
        "H3513": ("כָּבֵד", "H:V", "to honor", "ka.vad"),
        "H3514": ("כֹּ֫בֶד", "H:N-M", "heaviness", "ko.ved"),
        "H3515": ("כָּבֵד", "H:A", "heavy", "ka.ved"),
        "H3516": ("כָּבֵד", "H:N-F", "liver", "ka.ved"),
        "H3517": ("כְּבֵדֻת", "H:N-F", "heaviness", "ke.ve.dut"),
        "H3518": ("כָּבָה", "H:V", "to quench", "ka.vah"),
        "H3519": ("כָּבוֹד", "H:N-M", "glory", "ka.vod"),
        "H3520a": ("כָּבוֹד", "H:A", "glorious", "ke.vud.dah"),
        "H3520b": ("כְּבוּדָּ֯ה", "H:N-F", "riches", "ke.vud.dah"),
        "H3521": ("כָּבוּל", "N:N--L", "Cabul", "ka.vul"),
        "H3522": ("כַּבּוֹן", "N:N--L", "Cabbon", "kab.bon"),
        "H3523": ("כָּבִיר", "H:N-M", "quilt", "ke.vir"),
        "H3524": ("כַּבִּיר", "H:A", "mighty", "kab.bir"),
        "H3525": ("כֶּ֫בֶל", "H:N-M", "fetter", "ke.vel"),
        "H3526": ("כָּבַס", "H:V", "to wash", "ka.vas"),
        "H3527": ("כָּבַר", "H:V", "to multiply", "ka.var"),
        "H3528": ("כְּבָר", "H:Adv", "already", "ke.var"),
        "H3529": ("כְּבָר", "N:N--L", "Chebar", "ke.var"),
        "H3530": ("כִּבְרָה", "H:N-F", "distance", "kiv.rah"),
        "H3531": ("כְּבָרָה", "H:N-F", "sieve", "ke.va.rah"),
        "H3532": ("כֶּ֫בֶשׂ", "H:N-M", "lamb", "ke.ve.s"),
        "H3533": ("כָּבַשׁ", "H:V", "to subdue", "ka.vash"),
        "H3534": ("כֶּ֫בֶשׁ", "H:N-M", "footstool", "ke.vesh"),
        "H3535": ("כִּבְשָׂה", "H:N-F", "ewe-lamb", "kiv.sah"),
        "H3536": ("כִּבְשָׁן", "H:N-M", "kiln", "kiv.shan"),
        "H3537": ("כַּד", "H:N-F", "jar", "kad"),
        "H3538": ("כְּדַב", "A:A", "false", "ke.dav"),
        "H3539": ("כַּדְכֹּד", "H:N-M", "ruby", "kad.kod"),
        "H3540": ("כְּדׇרְלָעֹמֶר", "N:N-M-P", "Chedorlaomer", "ke.dor.la.o.mer"),
        "H3541": ("כֹּה", "H:Adv", "thus", "koh"),
        "H3542": ("כָּה", "A:Adv", "thus", "kah"),
        "H3543a": ("כָּהָה", "H:V", "to grow dim", "ka.hah"),
        "H3543b": ("כָּהָה", "H:V", "to rebuke", "ka.hah"),
        "H3544": ("כֵּהֶה", "H:A", "faint", "ke.heh"),
        "H3545": ("כֵּהָה", "H:N-F", "easing", "ke.hah"),
        "H3546": ("כְּהַל", "A:V", "be able", "ke.hal"),
        "H3547": ("כָּהַן", "H:V", "to minister", "ka.han"),
        "H3548": ("כֹּהֵן", "H:N-M", "priest", "ko.hen"),
        "H3549": ("כָּהֵן", "A:N-M", "priest", "ka.hen"),
        "H3550": ("כְּהֻנָּה", "H:N-F", "priesthood", "ke.hun.nah"),
        "H3551": ("כַּוָּה", "A:N-F", "window", "kav"),
        "H3552": ("כּוּב", "N:N--L", "Chub", "kuv"),
        "H3553": ("כּוֹבַע", "H:N-M", "helmet", "ko.va"),
        "H3554": ("כָּוָה", "H:V", "to burn", "ka.vah"),
        "H3555": ("כְּוִיָּה", "H:N-F", "burning", "ke.viy.yah"),
        "H3556": ("כּוֹכָב", "H:N-M", "star", "ko.khav"),
        "H3557": ("כּוּל", "H:V", "to sustain", "kul"),
        "H3558": ("כּוּמָז", "H:N-M", "bracelet", "ku.maz"),
        "H3559a": ("כּוּן", "H:V", "to establish", "kun"),
        "H3559b": ("נָכוֹן", "H:N-M", "blow", ""),
        "H3560": ("כּוּן", "N:N--L", "Cun", "kun"),
        "H3561": ("כַּוָּן", "H:N-M", "bun", "kav.van"),
        "H3562": ("כּוֹנַנְיָ֫הוּ", "N:N-M-P", "Conaniah", "ko.nan.ya.hu"),
        "H3563a": ("כּוֹס", "H:N-F", "cup", "kos"),
        "H3563b": ("כּוֹס", "H:N-M", "owl", "kos"),
        "H3564": ("כּוּר", "H:N-M", "furnace", "kur"),
        "H3565": ("בּוֹר עָשָׁן", "N:N--L", "Chorashan", "kor a.shan"),
        "H3566": ("כּ֫וֹרֶשׁ", "N:N-M-P", "Cyrus", "ko.resh"),
        "H3567": ("כּ֫וֹרֶשׁ", "N:N-M-P", "Cyrus", "ko.resh"),
        "H3568a": ("כּוּשׁ", "N:N--L / N:N--LG / N:N-M-P", "Cush; Ethiopia", "kush"),
        "H3568b": ("כּוּשׁ", "N:N-M-P", "Cush", "kush"),
        "H3569": ("כּוּשִׁי", "N:N--PG", "Cushite", "ku.shi"),
        "H3570": ("כּוּשִׁי", "N:N-M-P", "Cushi", "ku.shi"),
        "H3571": ("כּוּשִׁית", "N:N-F-PG", "Cushitess", "ku.shit"),
        "H3572": ("כּוּשָׁן", "N:N--L", "Cushan", "ku.shan"),
        "H3573": ("כּוּשַׁן רִשְׁעָתַ֫יִם", "N:N-M-P", "Cushan-rishathaim", "ku.shan rish.a.ta.yim"),
        "H3574": ("כּוֹשָׁרָה", "H:N-F", "prosperity", "ko.sha.rah"),
        "H3575": ("כּוּתָה", "N:N--L", "Cuthah", "kut"),
        "H3576": ("כָּזַב", "H:V", "to lie", "ka.zav"),
        "H3577": ("כָּזָב", "H:N-M", "lie", "ka.zav"),
        "H3578": ("כֹּֽזֵבָ֗א", "N:N--L", "Cozeba", "ko.ze.va"),
        "H3579": ("כָּזְבִי", "N:N-F-P", "Cozbi", "ko.ze.vi"),
        "H3580": ("כְּזִיב", "N:N--L", "Chezib", "ke.ziv"),
        "H3581a": ("כֹּחַ", "H:N-M", "reptile", "ko.ach"),
        "H3581b": ("כֹּחַ", "H:N-M", "strength", "ko.ach"),
        "H3582": ("כָּחַד", "H:V", "to hide", "ka.chad"),
        "H3583": ("כָּחַל", "H:V", "to paint", "ka.chal"),
        "H3584": ("כָּחַשׁ", "H:V", "to deceive", "ka.chash"),
        "H3585": ("כַּ֫חַשׁ", "H:N-M", "lie", "ka.chash"),
        "H3586": ("כֶּחָשׁ", "H:A", "deceptive", "ke.chash"),
        "H3587": ("כִּי", "H:N-M", "branding", "ki"),
        "H3588a": ("כִּי", "H:Conj", "for", "ki"),
        "H3588b": ("כִּי אם", "H:Conj", "that", ""),
        "H3588c": ("כִּי [עַל כֵּן]", "H:Conj", "for since", ""),
        "H3589": ("כִּיד", "H:N-M", "ruin", "kid"),
        "H3590": ("כִּידוֹד", "H:N-M", "spark", "ki.dod"),
        "H3591": ("כִּידוֹן", "H:N-M", "javelin", "ki.don"),
        "H3592": ("כִּידֹן", "N:N-M-P", "Chidon", "ki.don"),
        "H3593": ("כִּידוֹר", "H:N-M", "battle", "ki.dor"),
        "H3594": ("כִּיּוּן", "N:N-M-T", "Kiyyun", "kiy.yun"),
        "H3595": ("כִּיֹּר", "H:N-M", "basin", "kiy.yor"),
        "H3596": ("כִּילַי", "H:N-M", "rogue", "ki.lay"),
        "H3597": ("כֵּילַפּוֹת", "H:N-F", "axe", "ke.laph"),
        "H3598": ("כִּימָה", "N:N-F-T", "Pleiades", "ki.mah"),
        "H3599": ("כִּיס", "H:N-M", "purse", "kis"),
        "H3600": ("כִּיר", "H:N-M", "stove", "kir"),
        "H3601": ("כִּישׁוֹר", "H:N-M", "distaff", "ki.shor"),
        "H3602": ("כָּ֫כָה", "H:Adv", "thus", "ka.khah"),
        "H3603": ("כִּכָּר", "H:N-F", "talent", "kik.kar"),
        "H3604": ("כִּכֵּר", "A:N-F", "talent", "kik.ker"),
        "H3605": ("כֹּל", "H:N-M", "all", "kol"),
        "H3606": ("כֹּל", "A:N-M", "all", "kol"),
        "H3607": ("כָּלָא", "H:V", "to restrain", "ka.la"),
        "H3608": ("כֶּ֫לֶא", "H:N-M", "prison", "ke.le"),
        "H3609": ("כִּלְאָב", "N:N-M-P", "Chileab", "kil.av"),
        "H3610": ("כִּלְאַיִם", "H:N-M", "two", "kil.a.yim"),
        "H3611": ("כֶּ֫לֶב", "H:N-M", "dog", "ke.lev"),
        "H3612": ("כָּלֵב", "N:N-M-P", "Caleb", "ka.lev"),
        "H3613": ("כָּלֵב אֶפְרָתָה", "N:N--L", "Caleb-ephrathah", "ka.lev eph.ra.tah"),
        "H3614": ("כָּלִבִּי", "N:N--PG", "Calebite", "ka.lib.bo"),
        "H3615": ("כָּלָה", "H:V", "to end", "ka.lah"),
        "H3616": ("כָּלֶה", "H:A", "longing", "ka.leh"),
        "H3617": ("כָּלָה", "H:N-F", "consumption", "ka.lah"),
        "H3618": ("כַּלָּה", "H:N-F", "daughter-in-law", "kal.lah"),
        "H3619": ("כְּלוּב", "H:N-M", "basket", "ke.luv"),
        "H3620": ("כְּלוּב", "N:N-M-P", "Chelub", "ke.luv"),
        "H3621": ("כְּלוּבָ֑י", "N:N-M-P", "Chelubai", "ke.lu.vay"),
        "H3622": ("כְּלוּהוּ", "N:N-M-P", "Chelluh", "ke.lu.hay"),
        "H3623": ("כְּלוּלָה", "H:N-F", "betrothal", "ke.lu.lah"),
        "H3624": ("כֶּ֫לַח", "H:N-M", "vigor", "ke.lach"),
        "H3625": ("כֶּ֫לַח", "N:N--L", "Calah", "ke.lach"),
        "H3626": ("כׇּל־חֹזֶה", "N:N-M-P", "Col-hozeh", "kol-cho.zeh"),
        "H3627": ("כְּלִי", "H:N-M", "article/utensil", "ke.li"),
        "H3628": ("כְּלִיא", "H:N-M", "prison", "ke.li"),
        "H3629": ("כִּלְיָה", "H:N-F", "kidney", "kil.yah"),
        "H3630": ("כִּלְיוֹן", "N:N-M-P", "Chilion", "kil.yon"),
        "H3631": ("כִּלָּיוֹן", "H:N-M", "failing", "kil.la.von"),
        "H3632": ("כָּלִיל", "H:A", "entire", "ka.lil"),
        "H3633": ("כַּלְכֹּל", "N:N-M-P", "Calcol", "kal.kol"),
        "H3634": ("כָּלַל", "H:V", "to perfect", "ka.lal"),
        "H3635a": ("כְּלַל", "A:V", "to complete", "ke.lal"),
        "H3635b": ("שַׁכְלֵל", "A:V", "to complete", ""),
        "H3636": ("כְּלָ֑ל", "N:N-M-P", "Chelal", "ke.lal"),
        "H3637": ("כָּלַם", "H:V", "be humiliated", "ka.lam"),
        "H3638": ("כִּלְמָד", "N:N--L", "Chilmad", "kil.mad"),
        "H3639": ("כְּלִמָּה", "H:N-F", "shame", "ke.lim.mah"),
        "H3640": ("כְּלִמּוּת", "H:N-F", "shame", "ke.lim.mut"),
        "H3641a": ("כַּלְנֵ֑ה", "N:N--L", "Calneh", "kal.neh"),
        "H3641b": ("כַּלְנֶה", "N:N--L", "Calno", "kal.neh"),
        "H3642": ("כָּמַהּ", "H:V", "to pine", "ka.mah"),
        "H3643": ("כִּמְהָם", "N:N-M-P / N:N--L", "Chimham", "kim.ham"),
        "H3644": ("כְּמוֹ", "H:Conj", "like", "ke.mo"),
        "H3645": ("כְּמוֹשׁ", "N:N-M-T", "Chemosh", "ke.mosh"),
        "H3646": ("כַּמֹּן", "H:N-M", "cummin", "kam.mon"),
        "H3647": ("כָּמַס", "H:V", "to store up", "ka.mas"),
        "H3648": ("כָּמַר", "H:V", "to grow warm", "ka.mar"),
        "H3649": ("כֹּ֫מֶר", "H:N-M", "pagan priest", "ka.mar"),
        "H3650": ("כִּמְרִיר", "H:N-M", "darkness", "kim.rir"),
        "H3651a": ("כֵּן", "H:Adv", "right", "ken"),
        "H3651b": ("[כִּי עַל] כֵּן", "H:Adv", "so", ""),
        "H3651c": ("כֵּן", "H:Adv", "so", "ken"),
        "H3652": ("כֵּן", "A:Adv", "thus", "ken"),
        "H3653": ("כֵּן", "H:N-M", "stand", "ken"),
        "H3654": ("כֵּן", "H:N-M", "gnat", "ken"),
        "H3655": ("כָּנָה", "H:V", "to flatter", "ka.nah"),
        "H3656": ("כַּנֶּה", "N:N--L", "Canneh", "kan.neh"),
        "H3657": ("כַּנָּה", "H:N-F", "shoot", "kan.nah"),
        "H3658": ("כִּנּוֹר", "H:N-M", "lyre", "kin.nor"),
        "H3659": ("כׇּנְיָהוּ", "N:N-M-P", "Coniah", "kon.ya.hu"),
        "H3660": ("כְּנֵ֫מָא", "A:Adv", "thus", "ke.ne.ma"),
        "H3661": ("כָּנַן", "H:N-F", "vineyard", "ka.nan"),
        "H3662": ("כְּנָנִי", "N:N-M-P", "Chenani", "ke.na.ni"),
        "H3663": ("כְּנַנְיָ֫הוּ", "N:N-M-P", "Chenaniah", "ke.nan.yah"),
        "H3664": ("כָּנַס", "H:V", "to gather", "ka.nas"),
        "H3665": ("כָּנַע", "H:V", "be humble", "ka.na"),
        "H3666": ("כִּנְעָה", "H:N-F", "bundle", "kin.ah"),
        "H3667a": ("כְּנַ֫עַן", "N:N--L / N:N--LG / N:N-M-P", "Canaan", "ke.na.an"),
        "H3667b": ("כְּנַ֫עַן", "N:N--L / N:N--LG / N:N-M-P", "merchant", "ke.na.an"),
        "H3668": ("כְּנַעֲנָה", "N:N-M-P", "Chenaanah", "ke.na.a.nah"),
        "H3669a": ("כְּנַעֲנִי", "N:N--LG", "Canaanite", "ke.na.a.ni"),
        "H3669b": ("כְּנַעֲנִי", "H:N-M", "merchant", "ke.na.a.ni"),
        "H3670": ("כָּנַף", "H:V", "to corner", "ka.naph"),
        "H3671": ("כָּנָף", "H:N-F", "wing", "ka.naph"),
        "H3672": ("כִּנֶּ֫רֶת", "N:N--L", "Chinnereth", "kin.n.rot"),
        "H3673": ("כְּנַשׁ", "A:V", "to gather", "ka.nash"),
        "H3674": ("כְּנָת", "H:N-M", "associate", "ke.nat"),
        "H3675": ("כְּנָת", "A:N-M", "associate", "ke.nat"),
        "H3676": ("כֵּס", "H:N-M", "throne", "kes"),
        "H3677": ("כֶּ֫סֶא", "H:N-M", "full moon", "ke.se"),
        "H3678": ("כִּסֵּא", "H:N-M", "throne", "kis.se"),
        "H3679": ("כַּשְׂדָּ֑י", "N:N--LG", "Chaldean", "kas.day"),
        "H3680": ("כָּסָה", "H:V", "to cover", "ka.sah"),
        "H3681": ("כָּסוּי", "H:N-M", "covering", "ka.suy"),
        "H3682": ("כְּסוּת", "H:N-F", "covering", "ke.sut"),
        "H3683": ("כָּסַח", "H:V", "to cut", "ka.sach"),
        "H3684": ("כְּסִיל", "H:N-M", "fool", "ke.sil"),
        "H3685": ("כְּסִיל", "H:N-M / N:N--T", "constellation", "ke.sil"),
        "H3686": ("כְּסִיל", "N:N--L", "Chesil", "ke.sil"),
        "H3687": ("כְּסִילוּת", "H:N-F", "stupidity", "ke.si.lut"),
        "H3688": ("כָּסַל", "H:V", "be stupid", "ka.sal"),
        "H3689": ("כֶּ֫סֶל", "H:N-M", "loin", "ke.sel"),
        "H3690": ("כִּסְלָה", "H:N-F", "confidence", "kis.lah"),
        "H3691": ("כִּסְלֵו", "N:N-M-T", "Chisleu", "kis.lev"),
        "H3692": ("כִּסְלוֹן", "N:N-M-P", "Chislon", "kis.lon"),
        "H3693": ("כְּסָלוֹן", "N:N--L", "Chesalon", "ke.sa.lon"),
        "H3694": ("כְּסֻלּוֹת", "N:N--L", "Chesulloth", "ke.sul.lot"),
        "H3695": ("כַּסְלֻחִים", "N:N--PG", "Casluhim", "kas.lu.chim"),
        "H3696": ("כִּסְלֹת תָּבֹר", "N:N--L", "Chisloth-tabor", "kis.lot ta.vor"),
        "H3697": ("כָּסַם", "H:V", "to shear", "ka.sam"),
        "H3698": ("כֻּסֶּ֫מֶת", "H:N-F", "rye", "kus.se.met"),
        "H3699": ("כָּסַס", "H:V", "to compute", "ka.sas"),
        "H3700": ("כָּסַף", "H:V", "to long", "ka.saph"),
        "H3701": ("כֶּ֫סֶף", "H:N-M", "silver", "ke.seph"),
        "H3702": ("כְּסַף", "A:N-M", "silver", "ke.saph"),
        "H3703": ("כָּסִפְיָא", "N:N--L", "Casiphia", "ka.siph.ya"),
        "H3704": ("כֶּ֫סֶת", "H:N-F", "amulet", "ke.set"),
        "H3705": ("כְּעַן", "A:Adv", "now", "ke.an"),
        "H3706": ("כְּעֶ֫נֶת", "A:Adv", "now", "ke.e.net"),
        "H3707": ("כָּעַס", "H:V", "to provoke", "ka.as"),
        "H3708a": ("כַּ֫עַס", "H:N-M", "vexation", "ka.as"),
        "H3708b": ("כַּ֫עַשׂ", "H:N-M", "vexation", ""),
        "H3709": ("כַּף", "H:N-F", "palm", "kaph"),
        "H3710": ("כֵּף", "H:N-M", "rock", "keph"),
        "H3711": ("כָּפָה", "H:V", "to subdue", "ka.phah"),
        "H3712": ("כִּפָּה", "H:N-F", "branch", "kip.pah"),
        "H3713a": ("כְּפוֹר", "H:N-M", "bowl", "ke.phor"),
        "H3713b": ("כְּפוֹר", "H:N-M", "frost", "ke.phor"),
        "H3714": ("כָּפִיס", "H:N-M", "rafter", "ka.phis"),
        "H3715a": ("כְּפִיר", "H:N-M", "lion", "ke.phir"),
        "H3715b": ("כְּפִירִים", "H:N-M", "villages", ""),
        "H3716": ("כְּפִירָה", "N:N--L", "Chephirah", "ke.phi.rah"),
        "H3717": ("כָּפַל", "H:V", "to double", "ka.phal"),
        "H3718": ("כֶּ֫פֶל", "H:N-M", "double", "ke.phel"),
        "H3719": ("כָּפַן", "H:V", "to hunger", "ka.phan"),
        "H3720": ("כָּפָן", "H:N-M", "famine", "ka.phan"),
        "H3721": ("כָּפַף", "H:V", "to bend", "ka.phaph"),
        "H3722a": ("כִּפֶּר", "H:V", "to atone", "ki.pher"),
        "H3722b": ("כָּפַר", "H:V", "to cover", "ka.phar"),
        "H3723": ("כָּפָר", "H:N-M", "village", "ka.phar"),
        "H3724a": ("כֹּפֶר", "H:N-M", "ransom", "ko.pher"),
        "H3724b": ("כֹּ֫פֶר", "H:N-M", "pitch", "ko.pher"),
        "H3724c": ("כֹּ֫פֶר", "H:N-M", "henna", "ko.pher"),
        "H3724d": ("כֹּפֶר", "H:N-M", "village", "ko.pher"),
        "H3725": ("כִּפֻּרִים", "H:N-M", "atonement", "kip.pur"),
        "H3726": ("כְּפַר הָעַמּוֹנִי", "N:N--L", "Chefar-haamonai", "ke.phar ha.am.mo.ni"),
        "H3727": ("כַּפֹּרֶת", "H:N-F", "mercy seat", "kap.po.ret"),
        "H3728": ("כָּפַשׁ", "H:V", "to cower", "ka.phash"),
        "H3729": ("כְּפַת", "A:V", "to bind", "ke.phat"),
        "H3730": ("כַּפְתּוֹר", "H:N-M", "capital", "kaph.tor"),
        "H3731": ("כַּפְתּוֹר", "N:N--L", "Caphtor", "kaph.tor"),
        "H3732": ("כַּפְתֹּרִי", "N:N--LG", "Caphthorite", "kaph.to.ri"),
        "H3733a": ("כָּר", "H:N-M", "saddle", "kar"),
        "H3733b": ("כַּר", "H:N-M", "pasture", "kar"),
        "H3733c": ("כַּר", "H:N-M", "ram", "kar"),
        "H3734a": ("כֹּר", "H:N-M", "kor", "kor"),
        "H3734b": ("כּוֹר", "A:N-M", "kor", ""),
        "H3735": ("כְּרָא", "A:V", "be distressed", "ka.ra"),
        "H3736": ("כרבל", "H:V", "be clothed", "kar.bel"),
        "H3737": ("כַּרְבְּלָא", "A:N-M", "helmet", "kar.be.la"),
        "H3738a": ("כוּר", "H:V", "to dig", "ka.rah"),
        "H3738b": ("כָּרָה", "H:V", "to pierce", "ka.rah"),
        "H3739a": ("כָּרָה", "H:V", "to trade", "ka.rah"),
        "H3739b": ("כָּרָה", "H:V", "to feed", "ka.rah"),
        "H3740": ("כֵּרָה", "H:N-F", "feast", "ke.rah"),
        "H3741": ("כָּרָה", "H:N-F", "cistern", "ka.rah"),
        "H3742": ("כְּרוּב", "H:N-M", "cherub", "ke.ruv"),
        "H3743": ("כְּרוּב", "N:N--L", "Cherub", "ke.ruv"),
        "H3744": ("כָּרוֹז", "A:N-M", "herald", "ka.roz"),
        "H3745": ("כְּרַז", "A:V", "to proclaim", "ke.raz"),
        "H3746": ("כָּרִי", "N:N--TG", "Carite", "ka.ri"),
        "H3747": ("כְּרִית", "N:N--L", "Cherith", "ke.rit"),
        "H3748": ("כְּרִיתוּת", "H:N-F", "divorce", "ke.ri.tut"),
        "H3749": ("כַּרְכֹּב", "H:N-M", "ledge", "kar.kov"),
        "H3750": ("כַּרְכֹּם", "H:N-M", "saffron", "kar.kom"),
        "H3751": ("כַּרְכְּמִישׁ", "N:N--L", "Carchemish", "kar.ke.mish"),
        "H3752": ("כַּרְכַּס", "N:N-M-P", "Carkas", "kar.kas"),
        "H3753": ("כִּרְכָּרָה", "H:N-F", "dromedary", "kar.ka.rah"),
        "H3754": ("כֶּרֶם", "H:N-M", "vineyard", "ke.rem"),
        "H3755": ("כָּרַם", "H:V", "to tend vineyards", "ko.rem"),
        "H3756": ("כַּרְמִי", "N:N-M-P", "Carmi", "kar.mi"),
        "H3757": ("כַּרְמִי", "N:N-M-P", "Carmite", "kar.mi"),
        "H3758": ("כַּרְמִיל", "H:N-M", "crimson", "kar.mil"),
        "H3759": ("כַּרְמֶל", "H:N-M", "plantation", "kar.mel"),
        "H3760": ("כַּרְמֶל", "N:N--L", "Carmel", "kar.mel"),
        "H3761": ("כַּרְמְלִי", "N:N--LG", "Carmelite", "kar.me.li"),
        "H3762": ("כַּרְמְלִית", "N:N-F-L", "Carmelitess", "kar.me.lit"),
        "H3763": ("כְּרָ֑ן", "N:N-M-P", "Cheran", "ke.ran"),
        "H3764": ("כׇּרְסֵא", "A:N-M", "throne", "kor.se"),
        "H3765": ("כִּרְסֵם", "H:V", "to tear off", "kir.sem"),
        "H3766": ("כָּרַע", "H:V", "to bow", "ka.ra"),
        "H3767": ("כֶּ֫רַע", "H:N-F", "leg", "ka.ra"),
        "H3768": ("כַּרְפַּס", "H:N-M", "fine linen", "kar.pas"),
        "H3769": ("כָּרַר", "H:V", "to dance", "ka.rar"),
        "H3770": ("כְּרֵשׂ", "H:N-M", "belly", "ke.re.s"),
        "H3771": ("כַּרְשְׁנָא", "N:N-M-P", "Carshena", "kar.she.na"),
        "H3772": ("כָּרַת", "H:V", "to cut", "ka.rat"),
        "H3773": ("כָּרֻתָה", "H:N-F", "beam", "ka.ru.tah"),
        "H3774": ("כְּרֵתִי", "N:N--TG", "Cherethite", "ke.re.ti"),
        "H3775": ("כֶּ֫שֶׂב", "H:N-M", "sheep", "ke.sev"),
        "H3776": ("כִּשְׂבָּה", "H:N-F", "lamb", "kis.bah"),
        "H3777": ("כֶּ֫שֶׂד", "N:N-M-P", "Chesed", "ke.sed"),
        "H3778": ("כַּשְׂדִּים", "N:N--LG", "Chaldea", "kas.di"),
        "H3779": ("כַּשְׂדַּי", "N:N--LG", "Chaldean", "kas.day"),
        "H3780": ("כָּשָׂה", "H:V", "be sated", "ka.sah"),
        "H3781": ("כַּשִּׁיל", "H:N-M", "axe", "kash.shil"),
        "H3782": ("כָּשַׁל", "H:V", "to stumble", "ka.shal"),
        "H3783": ("כִּשָּׁלוֹן", "H:N-M", "stumbling", "kish.sha.lon"),
        "H3784": ("כָּשַׁף", "H:V", "to practice sorcery", "ka.shaph"),
        "H3785": ("כֶּ֫שֶׁף", "H:N-M", "sorcery", "ke.sheph"),
        "H3786": ("כַּשָּׁף", "H:N-M", "sorcerer", "kash.shaph"),
        "H3787": ("כָּשֵׁר", "H:V", "to succeed", "ka.sher"),
        "H3788": ("כִּשְׁרוֹן", "H:N-M", "skill", "kish.ron"),
        "H3789": ("כָּתַב", "H:V", "to write", "ka.tav"),
        "H3790": ("כְּתַב", "A:V", "to write", "ke.tav"),
        "H3791": ("כְּתָב", "H:N-M", "writing", "ka.tav"),
        "H3792": ("כְּתָב", "A:N-M", "inscription", "ke.tav"),
        "H3793": ("כְּתֹ֫בֶת", "H:N-F", "imprint", "ke.to.vet"),
        "H3794": ("כִּתִּי", "N:N--LG", "Kittim", "kit.ti"),
        "H3795": ("כָּתִית", "H:A", "beaten", "ka.tit"),
        "H3796": ("כֹּ֫תֶל", "H:N-M", "wall", "ko.tel"),
        "H3797": ("כְּתַל", "A:N-M", "wall", "ke.tal"),
        "H3798": ("כִּתְלִישׁ", "N:N--L", "Chitlish", "kit.lish"),
        "H3799": ("כָּתַם", "H:V", "be stained", "ka.tam"),
        "H3800": ("כֶּ֫תֶם", "H:N-M", "gold", "ke.tem"),
        "H3801": ("כֻּתֹּ֫נֶת", "H:N-F", "tunic", "ke.to.net"),
        "H3802": ("כָּתֵף", "H:N-F", "shoulder", "ka.teph"),
        "H3803": ("כָּתַר", "H:V", "to surround", "ka.tar"),
        "H3804": ("כֶּ֫תֶר", "H:N-M", "crown", "ke.ter"),
        "H3805": ("כֹּתֶ֫רֶת", "H:N-F", "capital", "ko.te.ret"),
        "H3806": ("כָּתַשׁ", "H:V", "to pound", "ka.tash"),
        "H3807": ("כָּתַת", "H:V", "to crush", "ka.tat"),
        "H3808": ("לֹא", "H:Adv", "not", "lo"),
        "H3809": ("לָא", "A:Adv", "not", "la"),
        "H3810": ("לֹא דְבָר", "N:N--L", "Lo-debar", "lo de.var"),
        "H3811": ("לָאָה", "H:V", "be weary", "la.ah"),
        "H3812": ("לֵאָה", "N:N-F-P", "Leah", "le.ah"),
        "H3813": ("לָאַט", "H:V", "to cover", "la.at"),
        "H3814": ("לָאט", "H:N-M", "secret", "lat"),
        "H3815": ("לָאֵל", "N:N-M-P", "Lael", "la.el"),
        "H3816": ("לְאֹם", "H:N-M", "people", "le.om"),
        "H3817": ("לְאֻמִּ֑ים", "N:N-M-P", "Leummim", "le.um.mim"),
        "H3818": ("לֹא עַמִּי", "N:N-M-P", "Lo-ammi", "lo am.mi"),
        "H3819": ("לֹא רֻחָ֫מָה", "N:N-F-P", "Lo-ruhamah", "lo ru.cha.mah"),
        "H3820a": ("לֵב", "H:N-M", "heart", "lev"),
        "H3820b": ("לֵב", "H:N-M", "Leb-", ""),
        "H3821": ("לֵב", "A:N-M", "heart", "lev"),
        "H3822": ("לְבָאוֹת", "N:N--L", "Lebaoth", "le.va.ot"),
        "H3823a": ("לָבַב", "H:V", "to encourage", "la.vav"),
        "H3823b": ("לִבֵּב", "H:V", "to bake", ""),
        "H3824": ("לֵבָב", "H:N-M", "heart", "le.vav"),
        "H3825": ("לְבַב", "A:N-M", "heart", "le.vav"),
        "H3826": ("לִבָּה", "H:N-F", "heart", "lib.bah"),
        "H3827": ("לַבָּה", "H:N-F", "flame", "lab.bah"),
        "H3828": ("לְבֹנָה", "H:N-F", "frankincense", "le.vo.nah"),
        "H3829": ("לְבוֹנָה", "N:N--L", "Lebonah", "le.vo.nah"),
        "H3830": ("לְבוּשׁ", "H:N-M", "clothing", "le.vush"),
        "H3831": ("לְבוּשׁ", "A:N-M", "garment", "le.vush"),
        "H3832": ("לָבַט", "H:V", "to ruin", "la.vat"),
        "H3833a": ("לְבִי", "H:N", "lion", ""),
        "H3833b": ("לְבִיָּא", "H:N-F", "lioness", ""),
        "H3833c": ("לָבִיא", "H:N-M", "lion", "la.vi"),
        "H3834": ("לְבִבָה", "H:N-F", "cake", "la.vi.vah"),
        "H3835a": ("לָבֵן", "H:V", "to whiten", "la.van"),
        "H3835b": ("לָבַן", "H:V", "to make bricks", "la.van"),
        "H3836": ("לָבָן", "H:A", "white", "la.van"),
        "H3837a": ("לָבָן", "N:N-M-P", "Laban", "la.van"),
        "H3837b": ("לָבָן", "N:N--L", "Laban", "la.van"),
        "H3838": ("לְבָנָא", "N:N-M-P", "Lebana", "le.va.na"),
        "H3839": ("לִבְנֶה", "H:N-M", "poplar", "liv.neh"),
        "H3840": ("לִבְנָה", "H:N-F", "paving", "liv.nah"),
        "H3841": ("לִבְנָה", "N:N--L", "Libnah", "liv.nah"),
        "H3842": ("לְבָנָה", "H:N-F", "moon", "le.va.nah"),
        "H3843": ("לְבֵנָה", "H:N-F", "brick", "le.ve.nah"),
        "H3844": ("לְבָנוֹן", "N:N--L", "Lebanon", "le.va.non"),
        "H3845": ("לִבְנִי", "N:N-M-P", "Libni", "liv.ni"),
        "H3846": ("לִבְנִי", "N:N--PG", "Libnite", "liv.ni"),
        "H3847": ("לָבֵשׁ", "H:V", "to clothe", "la.vash"),
        "H3848": ("לְבֵשׁ", "A:V", "to clothe", "le.vash"),
        "H3849": ("לֹג", "H:N-M", "log", "log"),
        "H3850": ("לֹד", "N:N--L", "Lod", "lod"),
        "H3851": ("לַ֫הַב", "H:N-M", "flame", "la.hav"),
        "H3852": ("לֶהָבָה", "H:N-F", "flame", "le.ha.vah"),
        "H3853": ("לְהָבִים", "N:N--PG", "Lehabim", "le.ha.vim"),
        "H3854": ("לַ֫הַג", "H:N-M", "study", "la.hag"),
        "H3855": ("לָ֑הַד", "N:N-M-P", "Lahad", "la.had"),
        "H3856a": ("לָהַה", "H:V", "to languish", "la.hah"),
        "H3856b": ("לִהְלֵהַּ", "H:V", "to amaze", ""),
        "H3857": ("לָהַט", "H:V", "to kindle", "la.hat"),
        "H3858": ("לַ֫הַט", "H:N-M", "flame", "la.hat"),
        "H3859": ("לָהַם", "H:V", "to swallow", "la.ham"),
        "H3860": ("לָהֵן", "H:Conj", "therefore", "la.hen"),
        "H3861": ("לָהֵן", "A:Conj", "except", "la.hen"),
        "H3862": ("לַהֲקָה", "H:N-F", "company", "la.ha.qah"),
        "H3863": ("לוּ", "H:Intj", "if", "lu"),
        "H3864": ("לוּב", "N:N--LG", "Libyan", "lu.vi"),
        "H3865": ("לוּד", "N:N--L / N:N--LG / N:N-M-P", "Lud", "lud"),
        "H3866": ("לוּדִי", "N:N--LG", "Ludim", "lu.di"),
        "H3867a": ("לָוָה", "H:V", "to join", "la.vah"),
        "H3867b": ("לָוָה", "H:V", "to borrow", "la.vah"),
        "H3868": ("לוּז", "H:V", "be devious", "luz"),
        "H3869": ("לוּז", "H:N-M", "almond", "luz"),
        "H3870": ("לוּז", "N:N--L", "Luz", "luz"),
        "H3871": ("לוּחַ", "H:N-M", "tablet", "lu.ach"),
        "H3872": ("לוּחִית", "N:N--L", "Luhith", "lu.chit"),
        "H3873": ("הַלּוֹחֵשׁ", "N:N-M-P", "Hallohesh", "lo.chesh"),
        "H3874": ("לוּט", "H:V", "to wrap", "lut"),
        "H3875": ("לוֹט", "H:N-M", "covering", "lot"),
        "H3876": ("לוֹט", "N:N-M-P", "Lot", "lot"),
        "H3877": ("לוֹטָן", "N:N-M-P", "Lotan", "lo.tan"),
        "H3878": ("לֵוִי", "N:N-M-P", "Levi", "le.vi"),
        "H3879": ("לֵוָי", "A:N-M", "Levites", "le.vi"),
        "H3880": ("לִוְיָה", "H:N-F", "wreath", "liv.yah"),
        "H3881": ("לֵוִי", "N:N--PG", "Levite", "le.viy.yi"),
        "H3882": ("לִוְיָתָן", "N:N-M-T", "Leviathan", "liv.ya.tan"),
        "H3883": ("לוּל", "H:N-M", "stairs", "lul"),
        "H3884": ("לוּלֵא", "H:Neg", "unless", "lu.le"),
        "H3885a": ("לוּן", "H:V", "to lodge", "lun"),
        "H3885b": ("לוּן", "H:V", "to grumble", "lun"),
        "H3886a": ("לוּעַ", "H:V", "to swallow", "lu.a"),
        "H3886b": ("לוּעַ", "H:V", "to talk wildly", "lu.a"),
        "H3887": ("לִיץ", "H:V", "to mock", "luts"),
        "H3888": ("לוּשׁ", "H:V", "to knead", "lush"),
        "H3889": ("לוּשׁ", "N:N-M-P", "Laish", "lush"),
        "H3890": ("לְוָת", "A:Prep", "with", "le.vat"),
        "H3891": ("לָזוּת", "H:N-F", "perversity", "le.zut"),
        "H3892": ("לַח", "H:A", "fresh", "lach"),
        "H3893": ("לֵחַ", "H:N-M", "vigor", "le.ach"),
        "H3894": ("לְחוּם", "H:N-M", "intestine", "la.chum"),
        "H3895": ("לְחִי", "H:N-M", "jaw", "le.chi"),
        "H3896": ("לֶחִי", "N:N--L", "Lehi", "le.chi"),
        "H3897": ("לָחַךְ", "H:V", "to lick", "la.chakh"),
        "H3898a": ("לָחַם", "H:V", "to fight", "la.cham"),
        "H3898b": ("לָחַם", "H:V", "to feed on", "la.cham"),
        "H3899": ("לֶ֫חֶם", "H:N-M", "food", "le.chem"),
        "H3900": ("לְחֵם", "A:N-M", "feast", "le.chem"),
        "H3901": ("לָחֶם", "H:N-M", "war", "la.chem"),
        "H3902": ("לַחְמִי", "N:N-M-P", "Lahmi", "lach.mi"),
        "H3903": ("לַחְמָס", "N:N--L", "Lahmas", "lach.mas"),
        "H3904": ("לְחֵנָה", "A:N-F", "concubine", "le.che.nah"),
        "H3905": ("לָחַץ", "H:V", "to oppress", "la.chats"),
        "H3906": ("לַ֫חַץ", "H:N-M", "oppression", "la.chats"),
        "H3907": ("לָחַשׁ", "H:V", "to whisper", "la.chash"),
        "H3908": ("לַ֫חַשׁ", "H:N-M", "charm", "la.chash"),
        "H3909": ("לָט", "H:N-M", "secrecy", "lat"),
        "H3910": ("לֹט", "H:N-M", "myrrh", "lot"),
        "H3911": ("לְטָאָה", "H:N-F", "lizard", "le.ta.ah"),
        "H3912": ("לְטוּשִׁם", "N:N--LG", "Letushim", "le.tu.shim"),
        "H3913": ("לָטַשׁ", "H:V", "to sharpen", "la.tash"),
        "H3914": ("לֹיָה", "H:N-F", "wreath", "lo.yah"),
        "H3915": ("לַ֫יְלָה", "H:N-M", "night", "la.yil"),
        "H3916": ("לֵילָא", "A:N-M", "night", "le.le.ya"),
        "H3917": ("לִילִית", "H:N-F", "night-demon", "li.lit"),
        "H3918": ("לַ֫יִשׁ", "H:N-M", "lion", "la.yish"),
        "H3919a": ("לַ֫יִשׁ", "N:N--L", "Laish", "la.yish"),
        "H3919b": ("לַ֫יִשׁ", "N:N-M-P", "Laish", "la.yish"),
        "H3919c": ("לַ֫יְשָׁה", "N:N--L", "Laish", ""),
        "H3920": ("לָכַד", "H:V", "to capture", "la.khad"),
        "H3921": ("לֶ֫כֶד", "H:N-M", "capture", "le.khed"),
        "H3922": ("לֵכָה", "N:N--L", "Lecah", "le.khah"),
        "H3923": ("לָכִישׁ", "N:N--L", "Lachish", "la.khish"),
        "H3924": ("לוּלַי", "H:N-F", "loop", "lu.la.ah"),
        "H3925": ("לָמַד", "H:V", "to learn", "la.mad"),
        "H3926": ("לְמוֹ", "H:Prep", "upon", "le.mo"),
        "H3927": ("לְמוּאֵל", "N:N-M-P", "Lemuel", "le.mu.el"),
        "H3928": ("לִמֻּד", "H:A", "taught", "lim.mud"),
        "H3929": ("לֶ֫מֶךְ", "N:N-M-P", "Lamech", "le.mekh"),
        "H3930": ("לֹעַ", "H:N-M", "throat", "lo.a"),
        "H3931": ("לָעַב", "H:V", "to jest", "la.av"),
        "H3932": ("לָעַג", "H:V", "to mock", "la.ag"),
        "H3933": ("לַ֫עַג", "H:N-M", "derision", "la.ag"),
        "H3934": ("לָעֵג", "H:A", "mocking", "la.eg"),
        "H3935": ("לַעְדָּה", "N:N-M-P", "Laadah", "la.dah"),
        "H3936": ("לַעְדָּן", "N:N-M-P", "Ladan", "la.dan"),
        "H3937": ("לָעַז", "H:V", "to mumble", "la.az"),
        "H3938": ("לָעַט", "H:V", "to eat", "la.at"),
        "H3939": ("לַעֲנָה", "H:N-F", "wormwood", "la.a.nah"),
        "H3940": ("לַפִּיד", "H:N-M", "torch", "lap.pid"),
        "H3941": ("לַפִּידוֹת", "N:N-F-P", "Lappidoth", "lap.pi.dot"),
        "H3942": ("לִפְנָ֑י", "H:Prep", "before", "liph.nay"),
        "H3943": ("לָפַת", "H:V", "to twist", "la.phat"),
        "H3944": ("לָצוֹן", "H:N-M", "scorning", "la.tson"),
        "H3945": ("לָצַץ", "H:V", "to scorn", "la.tsats"),
        "H3946": ("לַקּוּם", "N:N--L", "Lakkum", "laq.qum"),
        "H3947": ("לָקַח", "H:V", "to take", "la.qach"),
        "H3948": ("לֶ֫קַח", "H:N-M", "teaching", "le.qach"),
        "H3949": ("לִקְחִי", "N:N-M-P", "Likhi", "liq.chi"),
        "H3950": ("לָקַט", "H:V", "to gather", "la.qat"),
        "H3951": ("לֶ֫קֶט", "H:N-M", "gleaning", "le.qet"),
        "H3952": ("לָקַק", "H:V", "to lick", "la.qaq"),
        "H3953": ("לָקַשׁ", "H:V", "to glean", "la.qash"),
        "H3954": ("לֶ֫קֶשׁ", "H:N-M", "spring crop", "le.qesh"),
        "H3955": ("לָשָׁד", "H:N-M", "juicy bit", "le.shad"),
        "H3956": ("לָשׁוֹן", "H:N-M", "tongue", "la.shon"),
        "H3957": ("לִשְׁכָּה", "H:N-F", "chamber", "lish.kah"),
        "H3958": ("לֶ֫שֶׁם", "H:N-M", "jacinth", "le.shem"),
        "H3959": ("לֶ֫שֶׁם", "N:N--L", "Leshem", "le.shem"),
        "H3960": ("לָשַׁן", "H:V", "to slander", "la.shan"),
        "H3961": ("לִשָּׁן", "A:N-M", "tongue", "lish.shan"),
        "H3962": ("לֶ֫שַׁע", "N:N--L", "Lasha", "le.sah"),
        "H3963": ("לֶ֫תֶךְ", "H:N-M", "lethek", "le.tekh"),
        "H3964": ("מָא", "A:ImpP / A:Intg", "what", "ma"),
        "H3965": ("מַאֲבוּס", "H:N-M", "granary", "ma.a.vus"),
        "H3966": ("מְאֹד", "H:Adv", "much", "me.od"),
        "H3967": ("מֵאָה", "H:N-F", "hundred", "me.ah"),
        "H3968": ("מֵאָה", "N:N--L", "Meah", "me.ah"),
        "H3969": ("מְאָה", "A:N-F", "hundred", "me.ah"),
        "H3970": ("מַאֲוַי", "H:N-M", "desire", "ma.a.vay"),
        "H3971a": ("מאוּם", "H:N-M", "blemish", "m.um"),
        "H3971b": ("מוּם", "H:N-M", "blemish", ""),
        "H3972": ("מְא֫וּמָה", "H:IndP", "anything", "me.u.mah"),
        "H3973": ("מָאוֹס", "H:N-M", "refuse", "ma.os"),
        "H3974": ("מָאוֹר", "H:N-M", "light", "ma.or"),
        "H3975": ("מְאוּרָה", "H:N-F", "hole", "me.u.rah"),
        "H3976": ("מֹאזֵן", "H:N-M", "balance", "mo.zen"),
        "H3977": ("מֹאזֵנְיָא", "A:N-M", "scale", "mo.zen"),
        "H3978": ("מַאֲכָל", "H:N-M", "food", "ma.a.khal"),
        "H3979": ("מַאֲכֶ֫לֶת", "H:N-F", "knife", "ma.a.khe.let"),
        "H3980": ("מַאֲכֹ֫לֶת", "H:N-F", "fuel", "ma.a.kho.let"),
        "H3981": ("מַאֲמָץ", "H:N-M", "power", "ma.a.mats"),
        "H3982": ("מַאֲמַר", "H:N-M", "command", "ma.a.mar"),
        "H3983": ("מֵאמַר", "A:N-M", "command", "me.mar"),
        "H3984": ("מָאן", "A:N-M", "utensil", "man"),
        "H3985": ("מָאֵן", "H:V", "to refuse", "ma.en"),
        "H3986": ("מָאֵן", "H:A", "refusing", "ma.en"),
        "H3987": ("מֵאֵן", "H:A", "refusing", "me.en"),
        "H3988a": ("מָאַס", "H:V", "to reject", "ma.as"),
        "H3988b": ("מָאַס", "H:V", "to flow", "ma.as"),
        "H3989": ("מַאֲפֶה", "H:N-M", "baked (good)", "ma.a.pheh"),
        "H3990": ("מַאֲפֵל", "H:N-M", "darkness", "ma.a.phel"),
        "H3991": ("מַאְפֵּלְיָה", "H:N-F", "darkness", "ma.phe.le.yah"),
        "H3992": ("מָאַר", "H:V", "to malign", "ma.ar"),
        "H3993": ("מַאֲרָב", "H:N-M", "ambush", "ma.a.rav"),
        "H3994": ("מְאֵרָה", "H:N-F", "curse", "me.e.rah"),
        "H3995": ("מִבְדָּלָה", "H:N-F", "separate place", "miv.da.lah"),
        "H3996": ("מָבוֹא", "H:N-M", "entrance", "ma.vo"),
        "H3997": ("מְבוֹאָה", "H:N-F", "entry", "me.vo.ah"),
        "H3998": ("מְבוּכָה", "H:N-F", "confusion", "me.vu.khah"),
        "H3999": ("מַבּוּל", "H:N-M", "flood", "mab.bul"),
        "H4000": ("מָבוֹן", "H:N-M", "teacher", "ma.von"),
        "H4001": ("מְבוּסָה", "H:N-F", "subjugation", "me.vu.sah"),
        "H4002": ("מַבּוּעַ", "H:N-M", "spring", "mab.bu.a"),
        "H4003": ("מְבוּקָה", "H:N-F", "emptiness", "me.vu.qah"),
        "H4004": ("מִבְחוֹר", "H:N-M", "choice", "miv.chor"),
        "H4005": ("מִבְחָר", "H:N-M", "best", "miv.char"),
        "H4006": ("מִבְחָר", "N:N-M-P", "Mibhar", "miv.char"),
        "H4007": ("מַבָּט", "H:N-M", "expectation", "mab.bat"),
        "H4008": ("מִבְטָא", "H:N-M", "rash word", "miv.ta"),
        "H4009": ("מִבְטָח", "H:N-M", "confidence", "miv.tach"),
        "H4010": ("מַבְלִיגִית", "H:N-F", "cheer", "mav.li.git"),
        "H4011": ("מִבְנֶה", "H:N-M", "structure", "miv.neh"),
        "H4012": ("מְבֻנַּי", "N:N-M-P", "Mebunnai", "me.vun.nay"),
        "H4013": ("מִבְצָר", "H:N-M", "fortification", "miv.tsar"),
        "H4014": ("מִבְצָר", "N:N-M-P", "Mibzar", "miv.tsar"),
        "H4015": ("מִבְרָח", "H:N-M", "fugitive", "miv.rach"),
        "H4016": ("מָבוֹשׁ", "H:N-M", "genitalia", "ma.vush"),
        "H4017": ("מִבְשָׂם", "N:N-M-P", "Mibsam", "miv.sam"),
        "H4018": ("מְבַשְּׁלוֹת", "H:N-F", "hearth", "me.vash.sh.lah"),
        "H4019": ("מַגְבִּישׁ", "N:N--L", "Magbish", "mag.bish"),
        "H4020": ("מִגְבָּלֹת", "H:N-F", "twisted", "mig.ba.lah"),
        "H4021": ("מִגְבָּעוֹת", "H:N-F", "headgear", "mig.ba.ah"),
        "H4022": ("מֶ֫גֶד", "H:N-M", "excellence", "me.ged"),
        "H4023": ("מְגִדּוֹ", "N:N--L", "Megiddo", "me.gid.don"),
        "H4024a": ("מִגְדּוֹל", "H:N-M", "tower", "mig.dol"),
        "H4024b": ("מִגְדֹּל", "N:N--L", "Migdol", ""),
        "H4025": ("מַגְדִּיאֵל", "N:N-M-P", "Magdiel", "mag.di.el"),
        "H4026": ("מִגְדָּל", "H:N-M", "tower", "mig.dal"),
        "H4027": ("מִגְדַּל־אֵל", "N:N--L", "Migdal-el", "mig.dal-el"),
        "H4028": ("מִגְדַּל־גַּד", "N:N--L", "Migdal-gad", "mig.dal-gad"),
        "H4029": ("מִגְדַּל־עֵ֫דֶר", "N:N--L", "Migdal-edar", "mig.dal-e.der"),
        "H4030": ("מִגְדָּנָה", "H:N-F", "precious thing", "mig.da.nah"),
        "H4031": ("מָגוֹג", "N:N-M-P / N:N--L", "Magog", "ma.gog"),
        "H4032": ("מָגוֹר", "H:N-M", "terror", "ma.gor"),
        "H4033": ("מָגוֹר", "H:N-M", "sojourning", "ma.gur"),
        "H4034": ("מְגוֹרָה", "H:N-F", "fear", "me.go.rah"),
        "H4035": ("מְגוּרָה", "H:N-F", "fear", "me.gu.rah"),
        "H4036": ("מָגוֹר מִסָּבִיב", "N:N-M-P", "Magor-missabib", "ma.gor mis.sa.viv"),
        "H4037": ("מַגְזֵרָה", "H:N-F", "axe", "mag.ze.rah"),
        "H4038": ("מַגָּל", "H:N-M", "sickle", "mag.gal"),
        "H4039": ("מְגִלָּה", "H:N-F", "scroll", "me.gil.lah"),
        "H4040": ("מְגִלָּה", "A:N-F", "scroll", "me.gil.lah"),
        "H4041": ("מְגַמָּה", "H:N-F", "horde", "me.gam.mah"),
        "H4042": ("מָגַן", "H:V", "to deliver", "ma.gan"),
        "H4043": ("מָגֵן", "H:N-M", "shield", "ma.gen"),
        "H4044": ("מְגִנָּה", "H:N-F", "covering", "me.gin.nah"),
        "H4045": ("מִגְעֶ֫רֶת", "H:N-F", "rebuke", "mig.e.ret"),
        "H4046": ("מַגֵּפָה", "H:N-F", "plague", "mag.ge.phah"),
        "H4047": ("מַגְפִּיעָשׁ", "N:N-M-P", "Magpiash", "mag.pi.ash"),
        "H4048": ("מָגַר", "H:V", "to cast", "ma.gar"),
        "H4049": ("מְגַר", "A:V", "to overthrow", "me.gar"),
        "H4050": ("מְגֵרָה", "H:N-F", "saw", "me.ge.rah"),
        "H4051": ("מִגְרוֹן", "N:N--L", "Migron", "mig.ron"),
        "H4052": ("מִגְרָעָה", "H:N-F", "recess", "mig.ra.ah"),
        "H4053": ("מֶגְרָפָה", "H:N-F", "clod", "mig.ra.phah"),
        "H4054": ("מִגְרָשׁ", "H:N-M", "pasture", "mig.rash"),
        "H4055": ("מַד", "H:N-M", "garment", "mad"),
        "H4056": ("מַדְבַּח", "A:N-M", "altar", "mad.bach"),
        "H4057a": ("מִדְבָּר", "H:N-M", "mouth", "mid.bar"),
        "H4057b": ("מִדְבָּר", "H:N-M", "wilderness", "mid.bar"),
        "H4058": ("מָדַד", "H:V", "to measure", "ma.dad"),
        "H4059": ("מָדַד", "H:V", "to extend", "mid.dad"),
        "H4060a": ("מִדָּה", "H:N-F", "measure", "mid.dah"),
        "H4060b": ("מִדָּה", "H:N-F", "tribute", "mid.dah"),
        "H4061": ("מִנְדָּה", "A:N-F", "tribute", "mid.dah"),
        "H4062": ("מַדְהֵבָה", "H:N-F", "insolence", "mad.he.vah"),
        "H4063": ("מָ֫דוּ", "H:N-M", "garment", "me.dev"),
        "H4064": ("מַדְוֶה", "H:N-M", "disease", "mad.veh"),
        "H4065": ("מַדּוּחַ", "H:N-M", "enticement", "mad.du.ach"),
        "H4066": ("מָדוֹן", "H:N-M", "strife", "ma.don"),
        "H4067": ("מָדוֹן", "H:N-M", "stature", "ma.don"),
        "H4068": ("מָדוֹן", "N:N--L", "Madon", "ma.don"),
        "H4069": ("מַדּוּעַ", "H:Adv", "why?", "mad.du.a"),
        "H4070a": ("מְדוֹר", "A:N-M", "dwelling", "me.dor"),
        "H4070b": ("מְדָר", "A:N-M", "dwelling", ""),
        "H4071": ("מְדוּרָה", "H:N-F", "pile", "me.du.rah"),
        "H4072": ("מִדְחֶה", "H:N-M", "ruin", "mid.cheh"),
        "H4073": ("מַדְחֵפָה", "H:N-F", "thrust", "me.dach.phah"),
        "H4074": ("מָדַי", "N:N--L / N:N--LG", "Media", "ma.day"),
        "H4075": ("מָדִי, מָדָאָה", "N:N--LG", "Mede", "ma.day"),
        "H4076": ("מָדַי", "N:N--L / N:N--LG", "Media", "ma.day"),
        "H4077": ("מָדָאָה", "N:N--LG", "Median", "ma.day"),
        "H4078": ("מַדַּי", "H:N", "sufficiency", "mad.day"),
        "H4079": ("מִדְיָן", "H:N-M", "contention", "mid.yan"),
        "H4080": ("מִדְיָן", "N:N--LG", "Midian", "mid.yan"),
        "H4081": ("מִדִּין", "N:N--L", "Middin", "mid.din"),
        "H4082": ("מְדִינָה", "H:N-F", "province", "me.di.nah"),
        "H4083": ("מְדִינָה", "A:N-F", "province", "me.di.nah"),
        "H4084": ("מִדְיָנִי", "N:N--PG", "Midianite", "mid.ya.ni"),
        "H4085": ("מְדֹכָה", "H:N-F", "mortar", "me.do.khah"),
        "H4086": ("מַדְמֵן", "N:N--L", "Madmen", "mad.men"),
        "H4087": ("מַדְמֵנָה", "H:N-F", "dunghill", "mad.me.nah"),
        "H4088": ("מַדְמֵנָה", "N:N--L", "Madmenah", "mad.me.nah"),
        "H4089": ("מַדְמַנָּה", "N:N-M-P / N:N--L", "Madmannah", "mad.man.nah"),
        "H4090": ("מְדָן", "H:N-M", "strife", "me.dan"),
        "H4091": ("מְדָן", "N:N-M-P", "Medan", "me.dan"),
        "H4092": ("מְדָנִי", "N:N--PG", "Midianite", "me.da.ni"),
        "H4093": ("מַדָּע", "H:N-M", "knowledge", "mad.da"),
        "H4094": ("מַדְקָרָה", "H:N-F", "thrust", "mad.qa.rah"),
        "H4095": ("מַדְרֵגָה", "H:N-F", "steep", "mad.re.gah"),
        "H4096": ("מִדְרָךְ", "H:N-M", "treading", "mid.rakh"),
        "H4097": ("מִדְרָשׁ", "H:N-M", "story", "mid.rash"),
        "H4098": ("מְדֻשָֿׁה", "H:N-F", "threshed", "me.dush.shah"),
        "H4099": ("הַמְּדָתָא", "N:N-M-P", "Hammedatha", "me.da.ta"),
        "H4100": ("מָה", "H:Intg", "what?", "mah"),
        "H4101": ("מָה", "A:Intg", "what?", "mah"),
        "H4102": ("מָהַהּ", "H:V", "to delay", "ma.hah"),
        "H4103": ("מְהוּמָה", "H:N-F", "tumult", "me.hu.mah"),
        "H4104": ("מְהוּמָן", "N:N-M-P", "Mehuman", "me.hu.man"),
        "H4105": ("מְהֵיטַבְאֵל", "N:N-M-P / N:N-F-P", "Mehetabel", "me.he.tav.el"),
        "H4106": ("מָהִיר", "H:A", "quick", "ma.hir"),
        "H4107": ("מָהַל", "H:V", "to weaken", "ma.hal"),
        "H4108": ("מַהְלֵךְ", "H:N-M", "path", "mah.lekh"),
        "H4109": ("מַהֲלָךְ", "H:N-M", "journey", "ma.ha.lakh"),
        "H4110": ("מַהֲלָל", "H:N-M", "praise", "ma.ha.lal"),
        "H4111": ("מַהֲלַלְאֵל", "N:N-M-P", "Mahalalel", "ma.ha.lal.el"),
        "H4112": ("מַהֲלֻמֿוֹת", "H:N-F", "blow", "ma.ha.lum.mah"),
        "H4113": ("מַהֲמֹרָה", "H:N-F", "flood", "ma.ha.mo.rah"),
        "H4114": ("מַהְפֵּכָה", "H:N-F", "overthrow", "mah.pe.khah"),
        "H4115": ("מַהְפֶּ֫כֶת", "H:N-F", "stocks", "mah.pe.khet"),
        "H4116a": ("מָהַר", "H:V", "to hasten", "ma.har"),
        "H4116b": ("מָהַר", "H:V", "to buy", "ma.har"),
        "H4117": ("מָהַר", "H:V", "to pay brideprice", "ma.har"),
        "H4118a": ("מַהֵר", "H:A", "quick", "ma.her"),
        "H4118b": ("מַהֵר", "H:Adv", "quickly", "ma.her"),
        "H4119": ("מֹ֫הַר", "H:N-M", "brideprice", "mo.har"),
        "H4120": ("מְהֵרָה", "H:N-F", "haste", "me.he.rah"),
        "H4121": ("מַהֲרַי", "N:N-M-P", "Maharai", "ma.ha.ray"),
        "H4122": ("מַהֵר שָׁלָל חָשׁ בַּז", "N:N-M-P", "Maher-shalal-hash-baz", "ma.her sha.lal chash baz"),
        "H4123": ("מַֽהֲתַלּוֹת", "H:N-F", "deception", "ma.ha.tal.lah"),
        "H4124": ("מוֹאָב", "N:N-M-P / N:N-M-PG", "Moab", "mo.av"),
        "H4125": ("מוֹאָבִי", "N:N--PG", "Moabitess", "mo.a.vi"),
        "H4126": ("מוֹבָא", "H:N-M", "entrance", "mo.va"),
        "H4127": ("מוּג", "H:V", "to melt", "mug"),
        "H4128": ("מוד", "H:V", "to shake", "mud"),
        "H4129": ("מוֹדַע", "H:N-M", "kinsman", "mo.da"),
        "H4130": ("מֹדַ֫עַת", "H:N-F", "kinsman", "mo.da.at"),
        "H4131": ("מוֹט", "H:V", "to shake", "mot"),
        "H4132": ("מוֹט", "H:N-M", "yoke", "mot"),
        "H4133": ("מוֹטָה", "H:N-F", "yoke", "mo.tah"),
        "H4134": ("מוּךְ", "H:V", "be poor", "mukh"),
        "H4135a": ("מוּל", "H:V", "to circumcise", "mul"),
        "H4135b": ("מָלַל", "H:V", "to circumcise", ""),
        "H4136": ("מוּל", "H:N-M", "opposite", "mul"),
        "H4137": ("מוֹלָדָה", "N:N--L", "Moladah", "mo.la.dah"),
        "H4138": ("מוֹלֶ֫דֶת", "H:N-F", "relatives", "mo.le.det"),
        "H4139": ("מוּלָה", "H:N-F", "circumcision", "mu.lah"),
        "H4140": ("מוֹלִיד", "N:N-M-P", "Molid", "mo.lid"),
        "H4141": ("מוּסָב", "H:N-M", "surrounding", "mu.sav"),
        "H4142": ("מוּסַבָּה", "H:A-F", "surrounded", "mu.sab.bah"),
        "H4143": ("מוּסָד", "H:N-M", "foundation", "mu.sad"),
        "H4144": ("מוֹסָד", "H:N-M", "foundation", "mo.sad"),
        "H4145": ("מוּסָדָה", "H:N-F", "appointment", "mu.sa.dah"),
        "H4146": ("מוֹסָדָה", "H:N-F", "foundation", "mo.sa.dah"),
        "H4147": ("מוֹסֵר", "H:N-M", "bond", "mo.ser"),
        "H4148": ("מוּסָר", "H:N-M", "discipline", "mu.sar"),
        "H4149a": ("מוֹסֵרָה", "N:N--L", "Moserah", "mo.se.rah"),
        "H4149b": ("מֹסֵרוֹת", "N:N--L", "Moseroth", ""),
        "H4150": ("מוֹעֵד", "H:N-M", "meeting", "mo.ed"),
        "H4151": ("מוֹעָד", "H:N-M", "ranks", "mo.ad"),
        "H4152": ("מוּעָדָה", "H:N-F", "designated", "mu.a.dah"),
        "H4153": ("מוֹעַדְיָה", "N:N-M-P", "Moadiah", "mo.ad.yah"),
        "H4154": ("מוּעֶדֶת", "H:N-F", "disjointed", "mu.e.det"),
        "H4155": ("מוּעָף", "H:N-M", "gloom", "mu.aph"),
        "H4156": ("מוֹעֵצָה", "H:N-F", "counsel", "mo.e.tsah"),
        "H4157": ("מוּעָקָה", "H:N-F", "distress", "mu.a.qah"),
        "H4158": ("מֵיפַ֫עַת", "N:N--L", "Mephaath", "mo.pha.at"),
        "H4159": ("מוֹפֵת", "H:N-M", "wonder", "mo.phet"),
        "H4160": ("מֵץ", "H:N-M", "extortioner", "muts"),
        "H4161": ("מוֹצָא", "H:N-M", "exit", "mo.tsa"),
        "H4162": ("מוֹצָא", "N:N-M-P", "Moza", "mo.tsa"),
        "H4163": ("מוֹצָאָה", "H:N-F", "going forth", "mo.tsa.ah"),
        "H4164": ("מוּצָק", "H:N-M", "constraint", "mu.tsaq"),
        "H4165": ("מוּצָק", "H:N-M", "casting", "mu.tsaq"),
        "H4166": ("מוּצֶ֫קֶת", "H:N-F", "casting", "mu.tsa.qah"),
        "H4167": ("מוּק", "H:V", "to mock", "muq"),
        "H4168": ("מוֹקֵד", "H:N-M", "burning", "mo.qed"),
        "H4169": ("מוֹקְדָה", "H:N-F", "hearth", "mo.qe.dah"),
        "H4170": ("מוֹקֵשׁ", "H:N-M", "snare", "mo.qesh"),
        "H4171": ("מוּר", "H:V", "to change", "mur"),
        "H4172a": ("מוֹרָא", "H:N-M", "fear", "mo.ra"),
        "H4172b": ("מוֹרָה", "H:N-M", "fear", ""),
        "H4173": ("מוֹרַג", "H:N-M", "threshing sled", "mo.rag"),
        "H4174": ("מוֹרָד", "H:N-M", "descent", "mo.rad"),
        "H4175": ("מוֹרֶה", "H:N-M", "rain/teacher", "mo.reh"),
        "H4176": ("מוֹרֶה", "N:N--L", "Moreh", "mo.reh"),
        "H4177": ("מוֹרָה", "H:N-M", "razor", "mo.rah"),
        "H4178": ("מוֹרָט", "H:V", "to peel", "mo.rat"),
        "H4179": ("מֹרִיָּה", "N:N--L", "Moriah", "mo.riy.yah"),
        "H4180": ("מוֹרָשׁ", "H:N-M", "possession", "mo.rash"),
        "H4181": ("מוֹרָשָׁה", "H:N-F", "possession", "mo.ra.shah"),
        "H4182": ("מוֹרֶ֫שֶׁת גַּת", "N:N--L", "Moresheth-gath", "mo.re.shet gat"),
        "H4183": ("מוֹרַשְׁתִּי", "N:N--LG", "Moreshethite", "mo.rash.ti"),
        "H4184": ("מוּשׁ", "H:V", "to feel", "mush"),
        "H4185": ("מוּשׁ", "H:V", "to remove", "mush"),
        "H4186": ("מוֹשָׁב", "H:N-M", "seat", "mo.shav"),
        "H4187": ("מוּשִׁי", "N:N-M-P", "Mushi", "mu.shi"),
        "H4188": ("מוּשִׁי", "N:N--PG", "Mushite", "mu.shi"),
        "H4189": ("מֹשֶׁכֶת", "H:N-F", "cord", "mo.she.khah"),
        "H4190": ("מוֹשָׁעָה", "H:N-F", "salvation", "mo.sha.ah"),
        "H4191": ("מוּת", "H:V", "to die", "mut"),
        "H4192": ("לַבֵּן", "H:N-M", "Of a son", "mut"),
        "H4193": ("מוֹת", "A:N-M", "death", "mot"),
        "H4194": ("מָ֫וֶת", "H:N-M", "death", "ma.vet"),
        "H4195": ("מוֹתָר", "H:N-M", "advantage", "mo.tar"),
        "H4196": ("מִזְבֵּחַ", "H:N-M", "altar", "miz.be.ach"),
        "H4197": ("מֶ֫זֶג", "H:N-M", "mixture", "me.zeg"),
        "H4198": ("מָזֶה", "H:A", "empty", "ma.zeh"),
        "H4199": ("מִזָּה", "N:N-M-P", "Mizzah", "miz.zah"),
        "H4200": ("מָ֫זוּ", "H:N-M", "granary", "me.zev"),
        "H4201": ("מְזוּזָה", "H:N-F", "doorpost", "me.zu.zah"),
        "H4202": ("מָזוֹן", "H:N-M", "food", "ma.zon"),
        "H4203": ("מָזוֹן", "A:N-M", "food", "ma.zon"),
        "H4204": ("מָזוֹר", "H:N-M", "net", "ma.zor"),
        "H4205": ("מָזוֹר", "H:N-M", "wound", "ma.zor"),
        "H4206a": ("מֵ֫זַח", "H:N-M", "belt", "ma.zi.ach"),
        "H4206b": ("מְזִיחַ", "H:N-M", "belt", "ma.zi.ach"),
        "H4207a": ("מַזְלֵג", "H:N-M", "fork", "maz.leg"),
        "H4207b": ("מִזְלָגָה", "H:N-M", "fork", ""),
        "H4208": ("מַזָּלוֹת", "H:N-F", "constellation", "maz.za.lah"),
        "H4209": ("מְזִמָּה", "H:N-F", "plot", "me.zim.mah"),
        "H4210": ("מִזְמוֹר", "H:N-M", "melody", "miz.mor"),
        "H4211": ("מַזְמֵרָה", "H:N-F", "pruner", "maz.me.rah"),
        "H4212": ("מְזַמֶּרֶת", "H:N-F", "snuffer", "me.zam.m.rah"),
        "H4213": ("מִזְעָר", "H:N-M", "little", "miz.ar"),
        "H4214": ("מִזְרֶה", "H:N-M", "pitchfork", "miz.reh"),
        "H4215": ("מְזָרֶה", "H:N-M", "scattering wind", "me.za.reh"),
        "H4216": ("מַזָּרוֹת", "H:N-F", "constellation", "maz.za.rah"),
        "H4217": ("מִזְרָח", "H:N-M", "east", "miz.rach"),
        "H4218": ("מִזְרָע", "H:N-M", "sowing", "miz.ra"),
        "H4219": ("מִזְרָק", "H:N-M", "bowl", "miz.raq"),
        "H4220": ("מֵחַ", "H:N-M", "fatling", "me.ach"),
        "H4221": ("מֹחַ", "H:N-M", "marrow", "mo.ach"),
        "H4222": ("מָחָא", "H:V", "to clap", "ma.ach"),
        "H4223": ("מְחָא", "A:V", "to smite", "me.ach"),
        "H4224a": ("מַחֲבֵא", "H:N-M", "hiding-place", "ma.cha.ve"),
        "H4224b": ("מַחֲבֹא", "H:N-M", "refuge", ""),
        "H4225": ("מַחְבֶּ֫רֶת", "H:N-F", "joining", "mach.be.ret"),
        "H4226": ("מְחַבְּרָה", "H:N-F", "clamp", "me.chab.b.rah"),
        "H4227": ("מַחֲבַת", "H:N-F", "griddle", "ma.cha.vat"),
        "H4228": ("מַחֲגֹ֫רֶת", "H:N-F", "girding", "ma.cha.go.ret"),
        "H4229a": ("מָחָה", "H:V", "to wipe", "ma.chah"),
        "H4229b": ("מָחָה", "H:V", "to strike", "ma.chah"),
        "H4229c": ("מָחָה", "H:V", "be marrow", "ma.chah"),
        "H4230": ("מְחוּגָה", "H:N-F", "compass", "me.chu.gah"),
        "H4231": ("מָחוֹז", "H:N-M", "haven", "ma.choz"),
        "H4232": ("מְחוּיָאֵל", "N:N-M-P", "Mehujael", "me.chu.ya.el"),
        "H4233": ("מַחֲוִים", "N:N--LG", "Mahavite", "ma.cha.vim"),
        "H4234": ("מָחוֹל", "H:N-M", "dance", "ma.chol"),
        "H4235": ("מָחוֹל", "N:N-M-P", "Mahol", "ma.chol"),
        "H4236": ("מַחֲזֶה", "H:N-M", "vision", "ma.cha.zeh"),
        "H4237": ("מֶחֱזָה", "H:N-F", "window", "me.che.zah"),
        "H4238": ("מַחֲזִיאֹת", "N:N-M-P", "Mahazioth", "ma.cha.zi.ot"),
        "H4239": ("מְחִי", "H:N-M", "blow", "me.chi"),
        "H4240": ("מְחִידָא", "N:N-M-P", "Mehida", "me.chi.da"),
        "H4241": ("מִחְיָה", "H:N-F", "recovery", "mich.yah"),
        "H4242": ("מְחִיר", "H:N-M", "price", "me.chir"),
        "H4243": ("מְחִיר", "N:N-M-P", "Mehir", "me.chir"),
        "H4244": ("מַחְלָה", "N:N-F-P", "Mahlah", "mach.lah"),
        "H4245a": ("מַחֲלֶה", "H:N-M", "sickness", "ma.cha.leh"),
        "H4245b": ("מַחֲלָה", "H:N-M", "sickness", ""),
        "H4246": ("מְחוֹלָה", "H:N-F", "dance", "me.cho.lah"),
        "H4247": ("מְחִלָּה", "H:N-F", "hole", "me.chil.lah"),
        "H4248": ("מַחְלוֹן", "N:N-M-P", "Mahlon", "mach.lon"),
        "H4249": ("מַחְלִי", "N:N-M-P", "Mahli", "mach.li"),
        "H4250": ("מַחְלִי", "N:N--PG", "Mahlite", "mach.li"),
        "H4251": ("מַחֲלוּי", "H:N-M", "suffering", "mach.luy"),
        "H4252": ("מַחֲלָף", "H:N-M", "knife", "ma.cha.laph"),
        "H4253": ("מַחֲלָפָה", "H:N-F", "lock", "mach.la.phah"),
        "H4254": ("מַחֲלָצָה", "H:N-F", "robe", "ma.cha.la.tsah"),
        "H4255": ("מַחְלְקָה", "A:N-F", "class", "mach.le.qah"),
        "H4256": ("מַחֲלֹ֫קֶת", "H:N-F", "division", "ma.cha.lo.qet"),
        "H4257": ("מַחֲלַת", "H:N-F", "Mahalath", "ma.cha.lat"),
        "H4258": ("מַחֲלַת", "N:N-F-P", "Mahalath", "ma.cha.lat"),
        "H4259": ("מְחֹלָתִי", "N:N--LG", "Meholathite", "me.cho.la.ti"),
        "H4260": ("מַחֲמָאֹת", "H:N-F", "butter", "ma.cha.ma.ah"),
        "H4261": ("מַחְמָד", "H:N-M", "desire", "mach.mad"),
        "H4262": ("מַחְמֹד", "H:N-M", "desirable", "mach.mud"),
        "H4263": ("מַחְמָל", "H:N-M", "compassion", "mach.mal"),
        "H4264": ("מַחֲנֶה", "H:N-M", "camp", "ma.cha.neh"),
        "H4265": ("מַחֲנֵה־דָ֑ן", "N:N--L", "Mahaneh-dan", "ma.cha.neh-dan"),
        "H4266": ("מַחֲנַיִם", "N:N--L", "Mahanaim", "ma.cha.na.yim"),
        "H4267": ("מַחֲנָק", "H:N-M", "strangling", "ma.cha.naq"),
        "H4268": ("מַחְסֶּה", "H:N-M", "refuge", "ma.cha.seh"),
        "H4269": ("מַחְסוֹם", "H:N-M", "muzzle", "mach.som"),
        "H4270": ("מַחְסוֹר", "H:N-M", "need", "mach.sor"),
        "H4271": ("מַחְסֵיָה", "N:N-M-P", "Mahseiah", "mach.se.yah"),
        "H4272": ("מָחַץ", "H:V", "to wound", "ma.chats"),
        "H4273": ("מַ֫חַץ", "H:N-M", "wound", "ma.chats"),
        "H4274": ("מַחְצֵב", "H:N-M", "hewing", "mach.tsev"),
        "H4275": ("מֶחֱצָה", "H:N-F", "half", "me.che.tsah"),
        "H4276": ("מַחֲצִית", "H:N-F", "half", "ma.cha.tsit"),
        "H4277": ("מָחַק", "H:V", "to destroy", "ma.chaq"),
        "H4278": ("מֶחְקָר", "H:N-M", "range", "mech.qar"),
        "H4279": ("מָחָר", "H:N-M", "tomorrow", "ma.char"),
        "H4280": ("מַחֲרָאָה", "H:N-F", "sewer", "ma.cha.ra.ah"),
        "H4281": ("מַחֲרֵשָׁה", "H:N-F", "plowshare", "ma.cha.re.shah"),
        "H4282": ("מַחֲרֶ֫שֶׁת", "H:N-F", "share", "ma.cha.re.shet"),
        "H4283": ("מׇחֳרָת", "H:N-F", "morrow", "mo.cho.rat"),
        "H4284": ("מַחֲשָׁבָה", "H:N-F", "plot", "ma.cha.sha.vah"),
        "H4285": ("מַחְשָׁךְ", "H:N-M", "dark", "mach.shakh"),
        "H4286": ("מַחְשֹׂף", "H:N-M", "exposing", "mach.soph"),
        "H4287": ("מַ֫חַת", "N:N-M-P", "Mahath", "ma.chat"),
        "H4288": ("מְחִתָּה", "H:N-F", "terror", "me.chit.tah"),
        "H4289": ("מַחְתָּה", "H:N-F", "censer", "mach.tah"),
        "H4290": ("מַחְתֶּ֫רֶת", "H:N-M", "breaking", "mach.te.ret"),
        "H4291": ("מְטָא", "A:V", "to reach", "me.ta"),
        "H4292": ("מַטְאֲטֵא", "H:N-M", "broom", "mat.a.te"),
        "H4293": ("מַטְבֵּחַ", "H:N-M", "slaughter", "mat.be.ach"),
        "H4294": ("מַטֶּה", "H:N-M", "tribe", "mat.teh"),
        "H4295": ("מַ֫טָּה", "H:Adv", "beneath", "mat.tah"),
        "H4296": ("מִטָּה", "H:N-F", "bed", "mit.tah"),
        "H4297": ("מֻטֶּה", "H:N-M", "perversion", "mut.teh"),
        "H4298": ("מֻטָּה", "H:N-F", "spread", "mut.tah"),
        "H4299": ("מַטְוֶה", "H:N-M", "yarn", "mat.veh"),
        "H4300": ("מְטִיל", "H:N-M", "rod", "me.til"),
        "H4301": ("מַטְמוֹן", "H:N-M", "treasure", "mat.mon"),
        "H4302": ("מַטָּע", "H:N-M", "plantation", "mat.ta"),
        "H4303": ("מַטְעָם", "H:N-M", "delicacy", "mat.am"),
        "H4304": ("מִטְפַּ֫חַת", "H:N-F", "cloak", "mit.pa.chat"),
        "H4305": ("מָטַר", "H:V", "to rain", "ma.tar"),
        "H4306": ("מָטַר", "H:N-M", "rain", "ma.tar"),
        "H4307": ("מַטָּרָה", "H:N-F", "guardhouse", "mat.ta.ra"),
        "H4308": ("מַטְרֵד", "N:N-F-P", "Matred", "mat.red"),
        "H4309": ("מַטְרִי", "N:N-M-P / N:N-M-PG", "Matrite", "mat.ri"),
        "H4310": ("מִי", "H:Intg", "who?", "mi"),
        "H4311": ("מֵידְבָא", "N:N--L", "Medeba", "me.de.va"),
        "H4312": ("מֵידָד", "N:N-M-P", "Medad", "me.dad"),
        "H4313": ("מֵי הַיַּרְקוֹן", "N:N--L", "Me-jarkon", "me hay.yar.qon"),
        "H4314": ("מֵי זָהָב", "N:N-M-P", "Mezahab", "me za.hav"),
        "H4315": ("מֵיטָב", "H:N-M", "best", "me.tav"),
        "H4316": ("מִיכָא", "N:N-M-P", "Mica", "mi.kah"),
        "H4317": ("מִיכָאֵל", "N:N-M-P", "Michael", "mi.kha.el"),
        "H4318": ("מִיכָה", "N:N-M-P", "Micah", "mi.khah"),
        "H4319": ("מִיכָהוּ", "N:N-M-P", "Micahu", "mi.kha.hu"),
        "H4320": ("מִיכָיָה", "N:N-M-P", "Micaiah", "mi.kha.yah"),
        "H4321": ("מִיכָֽיְהוּ", "N:N-M-P", "Micaiah", "mi.kha.ye.hu"),
        "H4322": ("מִיכָיָ֫הוּ", "N:N-M-P", "Micaiah", "mi.kha.ya.hu"),
        "H4323": ("מִיכָל", "H:N-M", "brook", "mi.khal"),
        "H4324": ("מִיכַל", "N:N-F-P", "Michal", "mi.khal"),
        "H4325": ("מַי", "H:N-M", "water", "ma.yim"),
        "H4326": ("מִיָּמִן", "N:N-M-P", "Mijamin", "miy.ya.min"),
        "H4327": ("מִין", "H:N-M", "kind", "min"),
        "H4328": ("מְיֻסָּדָה", "H:N-F", "foundation", "me.yus.sa.dah"),
        "H4329": ("מוּסָךְ", "H:N-M", "portico", "me.sakh"),
        "H4330": ("מִיץ", "H:N-M", "pressing", "mits"),
        "H4331": ("מֵישָׁא", "N:N-M-P", "Mesha", "me.sah"),
        "H4332": ("מִישָׁאֵל", "N:N-M-P", "Mishael", "mi.sha.el"),
        "H4333": ("מִישָׁאֵל", "N:N-M-P", "Mishael", "mi.sha.el"),
        "H4334": ("מִישׁוֹר", "H:N-M", "plain", "mi.shor"),
        "H4335": ("מֵישַׁךְ", "N:N-M-P", "Meshach", "me.shakh"),
        "H4336": ("מֵישַׁךְ", "N:N-M-P", "Meshach", "me.shakh"),
        "H4337": ("מֵישָׁע", "N:N-M-P", "Mesha", "me.sah"),
        "H4338": ("מֵישַׁע", "N:N-M-P", "Mesha", "me.sah"),
        "H4339": ("מֵישָׁר", "H:N-M", "uprightness", "me.shar"),
        "H4340": ("מֵיתָר", "H:N-M", "cord", "me.tar"),
        "H4341": ("מַכְאוֹב", "H:N-M", "pain", "makh.ov"),
        "H4342": ("מַכְבִּיר", "H:V", "to increase", "makh.bir"),
        "H4343": ("מַכְבֵּנָה", "N:N-M-P", "Machbena", "makh.be.na"),
        "H4344": ("מַכְבַּנַּי", "N:N-M-P", "Machbanai", "makh.ban.nay"),
        "H4345": ("מִכְבָּר", "H:N-M", "grate", "makh.ber"),
        "H4346": ("מַכְבֵּר", "H:N-M", "cloth", "makh.bar"),
        "H4347": ("מַכָּה", "H:N-F", "wound", "mak.kah"),
        "H4348": ("מִכְוָה", "H:N-F", "burn", "mikh.vah"),
        "H4349": ("מָכוֹן", "H:N-M", "foundation", "ma.khon"),
        "H4350": ("מְכוֹנָה", "H:N-F", "base", "me.kho.nah"),
        "H4351": ("מְכֹרָה", "H:N-F", "origin", "me.khu.rah"),
        "H4352": ("מָכִי", "N:N-M-P", "Machi", "ma.khi"),
        "H4353": ("מָכִיר", "N:N-M-P", "Machir", "ma.khir"),
        "H4354": ("מָכִירִי", "N:N--PG", "Machirite", "ma.khi.ri"),
        "H4355": ("מָכַךְ", "H:V", "to sink", "ma.khakh"),
        "H4356": ("מִכְלָה", "H:N-M", "fold", "mikh.la.ah"),
        "H4357": ("מִכְלָה", "H:N-F", "perfection", "mikh.lah"),
        "H4358": ("מִכְלוֹל", "H:N-M", "perfection", "mikh.lol"),
        "H4359": ("מִכְלָל", "H:N-M", "perfection", "mikh.lal"),
        "H4360": ("מַכְלֻל", "H:N-M", "perfection", "mikh.lul"),
        "H4361": ("מַכֹּ֫לֶת", "H:N-F", "food", "mak.ko.let"),
        "H4362": ("מִכְמָן", "H:N-M", "hidden", "mikh.man"),
        "H4363": ("מִכְמָס, מִכְמָשׂ", "N:N--L", "Michmash", "mikh.mas"),
        "H4364a": ("מִכְמָר", "H:N-M", "net", "makh.mar"),
        "H4364b": ("מַכְמֹר", "H:N-M", "net", ""),
        "H4365a": ("מִכְמֶ֫רֶת", "H:N-F", "net", "mikh.me.ret"),
        "H4365b": ("מִכְמֹ֫רֶת", "H:N-F", "net", ""),
        "H4366": ("מִכְמְתָת", "N:N--L", "Michmethath", "mikh.me.tat"),
        "H4367": ("מַכְנַדְבַי", "N:N-M-P", "Machnadebai", "makh.nad.vay"),
        "H4368": ("מְכֹנָה", "N:N--L", "Meconah", "me.kho.nah"),
        "H4369": ("מְכֻנָה", "H:N-F", "base", "me.khu.nah"),
        "H4370": ("מִכְנָס", "H:N-M", "undergarment", "mikh.nas"),
        "H4371": ("מֶ֫כֶס", "H:N-M", "tribute", "me.khes"),
        "H4372": ("מִכְסֶה", "H:N-M", "covering", "mikh.seh"),
        "H4373": ("מִכְסָה", "H:N-F", "number", "mikh.sah"),
        "H4374": ("מְכַסֶּה", "H:N-M", "covering", "me.khas.seh"),
        "H4375": ("מַכְפֵּלָה", "N:N--L", "Machpelah", "makh.pe.lah"),
        "H4376": ("מָכַר", "H:V", "to sell", "ma.khar"),
        "H4377": ("מֶ֫כֶר", "H:N-M", "merchandise", "me.kher"),
        "H4378": ("מַכָּר", "H:N-M", "acquaintance", "mak.kar"),
        "H4379": ("מִכְרֶה", "H:N-M", "pit", "mikh.reh"),
        "H4380": ("מְכֵרָה", "H:N-F", "sword", "me.khe.rah"),
        "H4381": ("מִכְרִי", "N:N-M-P", "Michri", "mikh.ri"),
        "H4382": ("מְכֵרָתִי", "N:N--LG", "Mecherathite", "me.khe.ra.ti"),
        "H4383": ("מִכְשׁוֹל", "H:N-M", "stumbling", "mikh.shol"),
        "H4384": ("מַכְשֵׁלָה", "H:N-F", "ruins", "makh.she.lah"),
        "H4385": ("מִכְתָּב", "H:N-M", "writing", "mikh.tav"),
        "H4386": ("מְכִתָּה", "H:N-M", "fragment", "me.khit.tah"),
        "H4387": ("מִכְתָּם", "H:N-M", "Michtam", "mikh.tam"),
        "H4388": ("מַכְתֵּשׁ", "H:N-M", "hollow", "makh.tesh"),
        "H4389": ("מַכְתֵּשׁ", "N:N--L", "Maktesh", "makh.tesh"),
        "H4390": ("מָלֵא", "H:V", "to fill", "ma.le"),
        "H4391": ("מְלָא", "A:V", "to fill", "me.la"),
        "H4392": ("מָלֵא", "H:A", "full", "ma.le"),
        "H4393": ("מְלֹא", "H:N-M", "fullness", "me.lo"),
        "H4394": ("מִלֻּא", "H:N-M", "setting", "mil.lu"),
        "H4395": ("מְלֵאָה", "H:N-F", "fruit", "me.le.ah"),
        "H4396": ("מִלֻּאָה", "H:N-F", "setting", "mil.lu.ah"),
        "H4397": ("מַלְאָךְ", "H:N-M", "messenger", "ma.le.akh"),
        "H4398": ("מַלְאַךְ", "A:N-M", "angel", "mal.akh"),
        "H4399": ("מְלָאכָה", "H:N-F", "work", "me.la.khah"),
        "H4400": ("מַלְאָכוּת", "H:N-F", "message", "mal.a.khut"),
        "H4401": ("מַלְאָכִי", "N:N-M-P", "Malachi", "mal.a.khi"),
        "H4402": ("מִלֵּאת", "H:N-F", "setting", "mil.let"),
        "H4403": ("מַלְבּוּשׁ", "H:N-M", "garment", "mal.bush"),
        "H4404": ("מַלְבֵּן", "H:N-M", "brick", "mal.ben"),
        "H4405": ("מִלָּה", "H:N-F", "speech", "mil.lah"),
        "H4406": ("מִלָּה", "A:N-F", "word", "mil.lah"),
        "H4407": ("מִלּוֹא", "N:N--L", "Millo", "mil.lo"),
        "H4408": ("מַלּוּחַ", "H:N-M", "mallow", "mal.lu.ach"),
        "H4409a": ("מַלּוּךְ", "N:N-M-P", "Malluch", "mal.lukh"),
        "H4409b": ("מְלִיכוּ", "N:N-M-P", "Melicu", ""),
        "H4410": ("מְלוּכָה", "H:N-F", "kingship", "me.lu.khah"),
        "H4411": ("מָלוֹן", "H:N-M", "lodging", "ma.lon"),
        "H4412": ("מְלוּנָה", "H:N-F", "lodge", "me.lu.nah"),
        "H4413": ("מַלּוֹתִי", "N:N-M-P", "Mallothi", "mal.lo.ti"),
        "H4414a": ("מָלַח", "H:V", "to dissipate", "ma.lach"),
        "H4414b": ("מָלַח", "H:V", "to salt", "ma.lach"),
        "H4415": ("מְלַח", "A:V", "to eat salt", "me.lach"),
        "H4416": ("מְלַח", "A:N-M", "salt", "me.lach"),
        "H4417": ("מֶ֫לַח", "H:N-M", "salt", "me.lach"),
        "H4418": ("מֶ֫לַח", "H:N-M", "rag", "ma.lach"),
        "H4419": ("מַלָּח", "H:N-M", "mariner", "mal.lach"),
        "H4420": ("מְלֵחָה", "H:N-F", "saltiness", "me.le.chah"),
        "H4421": ("מִלְחָמָה", "H:N-F", "battle", "mil.cha.mah"),
        "H4422": ("מָלַט", "H:V", "to escape", "ma.lat"),
        "H4423": ("מֶ֫לֶט", "H:N-M", "mortar", "me.let"),
        "H4424": ("מְלַטְיָה", "N:N-M-P", "Melatiah", "me.lat.yah"),
        "H4425": ("מְלִילָה", "H:N-F", "ear", "me.li.lah"),
        "H4426": ("מְלִיצָה", "H:N-F", "mockery", "me.li.tsah"),
        "H4427a": ("מָלַךְ", "H:V", "to reign", "ma.lakh"),
        "H4427b": ("מָלַךְ", "H:V", "to advise", "ma.lakh"),
        "H4428": ("מֶ֫לֶךְ", "H:N-M", "king", "me.lekh"),
        "H4429": ("מֶ֫לֶךְ", "N:N-M-P", "Melech", "me.lekh"),
        "H4430": ("מֶ֫לֶךְ", "A:N-M", "king", "me.lekh"),
        "H4431": ("מְלֵךְ", "A:N-M", "counsel", "me.lakh"),
        "H4432": ("מֹ֫לֶךְ", "N:N-M-T", "Molech", "mo.lekh"),
        "H4433": ("מַלְכָּה", "A:N-F", "queen", "mal.ka"),
        "H4434": ("מַלְכֹּ֫דֶת", "H:N-F", "snare", "mal.ko.det"),
        "H4435": ("מִלְכָּה", "N:N-F-P", "Milcah", "mil.kah"),
        "H4436": ("מַלְכָּה", "H:N-F", "queen", "mal.kah"),
        "H4437": ("מַלְכוּ", "A:N-F", "kingdom", "mal.khu"),
        "H4438": ("מַלְכוּת", "H:N-F", "royalty", "mal.khut"),
        "H4439": ("מַלְכִּיאֵל", "N:N-M-P", "Malchiel", "mal.ki.el"),
        "H4440": ("מַלְכִּיאֵלִי", "N:N--PG", "Malchielite", "mal.ki.e.li"),
        "H4441": ("מַלְכִּיָּהוּ", "N:N-M-P", "Malchijah", "mal.kiy.yah"),
        "H4442": ("מַלְכִּי־צֶ֫דֶק", "N:N-M-P", "Melchizedek", "mal.ki-tse.deq"),
        "H4443": ("מַלְכִּירָם", "N:N-M-P", "Malchiram", "mal.ki.ram"),
        "H4444": ("מַלְכִּישׁוּעַ", "N:N-M-P", "Malchi-Shua", "mal.ki.shu.a"),
        "H4445a": ("מַלְכָּם", "N:N-M-P", "Malcham", "mal.kam"),
        "H4445b": ("מִלכֹּם", "N:N-M-T", "Milcom", ""),
        "H4446": ("מְלֶ֫כֶת", "H:N-F", "queen", "me.le.khet"),
        "H4447": ("הַמֹּלֶכֶת", "N:N-F-P", "Hammolecheth", "mo.le.khet"),
        "H4448a": ("מָלַל", "H:V", "to speak", "ma.lal"),
        "H4448b": ("מָלַל", "H:V", "to rub", "ma.lal"),
        "H4449": ("מְלַל", "A:V", "to speak", "me.lal"),
        "H4450": ("מִֽלֲלַי", "N:N-M-P", "Milalai", "mi.la.lay"),
        "H4451": ("מַלְמֵד", "H:N-M", "oxgoad", "mal.mad"),
        "H4452": ("מָלַץ", "H:V", "to smooth", "ma.lats"),
        "H4453": ("מֶלְצָר", "H:N-M", "guardian", "mel.tsar"),
        "H4454": ("מָלַק", "H:V", "to nip", "ma.laq"),
        "H4455a": ("מַלְקוֹחַ", "H:N-M", "prey", "mal.qo.ach"),
        "H4455b": ("מַלְקוֹחַ", "H:N-M", "jaw", "mal.qo.ach"),
        "H4456": ("מַלְקוֹשׁ", "H:N-M", "spring rain", "mal.qosh"),
        "H4457": ("מֶלְקָחַ֫יִם", "H:N-M", "tong", "mel.qach"),
        "H4458": ("מֶלְתָּחָה", "H:N-F", "apparel", "mel.ta.chah"),
        "H4459": ("מַלְתָּעָה", "H:N-F", "tooth", "mal.ta.ah"),
        "H4460": ("מַמְּגֻרוֹת", "H:N-F", "granary", "mam.m.gu.rah"),
        "H4461": ("מֵמַד", "H:N-M", "measurement", "me.mad"),
        "H4462": ("מְמוּכָן", "N:N-M-P", "Memucan", "me.mu.khan"),
        "H4463": ("מָמוֹת", "H:N-M", "death", "ma.mot"),
        "H4464": ("מַמְזֵר", "H:N-M", "bastard", "mam.zer"),
        "H4465": ("מִמְכָּר", "H:N-M", "sale", "mim.kar"),
        "H4466": ("מִמְכֶּ֫רֶת", "H:N-F", "sale", "mim.ke.ret"),
        "H4467": ("מַמְלָכָה", "H:N-F", "kingdom", "mam.la.khah"),
        "H4468": ("מַמְלָכוּת", "H:N-M", "kingdom", "mam.la.khut"),
        "H4469": ("מִמְסָךְ", "H:N-M", "mixed drink", "mam.sakh"),
        "H4470": ("מֶ֫מֶר", "H:N-M", "bitterness", "me.mer"),
        "H4471": ("מַמְרֵא", "N:N-M-P / N:N--L", "Mamre", "mam.re"),
        "H4472": ("מַמְרוֹר", "H:N-M", "bitterness", "mam.ror"),
        "H4473": ("מִמְשַׁח", "H:N-M", "expanded", "mim.shach"),
        "H4474": ("מִמְשָׁל", "H:N-M", "dominion", "mim.shal"),
        "H4475": ("מֶמְשָׁלָה", "H:N-F", "dominion", "mem.sha.lah"),
        "H4476": ("מִמְשָׁק", "H:N-M", "possession", "mim.shaq"),
        "H4477": ("מַמְתַֿקִּים", "H:N-M", "sweetness", "mam.taq"),
        "H4478a": ("מָן", "H:N-M", "manna", "man"),
        "H4478b": ("מָן", "H:Intg", "What?", "man"),
        "H4479": ("מָן", "A:Intg", "who?", "man"),
        "H4480a": ("מִן־", "H:Prep", "from", "min"),
        "H4480b": ("מֵן", "H:N-M", "portion", ""),
        "H4481": ("מִן־", "A:Prep", "from", "min"),
        "H4482": ("מֵן", "H:N-M", "string", "men"),
        "H4483": ("מְנָה", "A:V", "to reckon/appoint", "me.na"),
        "H4484": ("מְנֵא", "A:N-M", "mina", "me.ne"),
        "H4485": ("מַנְגִּינָה", "H:N-F", "mocking", "man.gi.nah"),
        "H4486": ("מַנְדַּע", "A:N-M", "knowledge", "man.da"),
        "H4487": ("מָנָה", "H:V", "to count", "ma.nah"),
        "H4488": ("מָנֶה", "H:N-M", "mina", "ma.neh"),
        "H4489": ("מֹנֶה", "H:N-M", "time", "mo.neh"),
        "H4490": ("מָנָה", "H:N-F", "portion", "ma.nah"),
        "H4491": ("מִנְהָג", "H:N-M", "driving", "min.hag"),
        "H4492": ("מִנְהָרָה", "H:N-F", "cave", "min.ha.rah"),
        "H4493": ("מָנוֹד", "H:N-M", "shaking", "ma.nod"),
        "H4494": ("מָנוֹחַ", "H:N-M", "resting", "ma.no.ach"),
        "H4495": ("מָנוֹחַ", "N:N-M-P", "Manoah", "ma.no.ach"),
        "H4496": ("מְנוּחָה", "H:N-F", "resting", "me.nu.chah"),
        "H4497": ("מָנוֹן", "H:N-M", "progeny", "ma.non"),
        "H4498": ("מָנוֹס", "H:N-M", "refuge", "ma.nos"),
        "H4499": ("מְנוּסָה", "H:N-F", "fugitive", "me.nu.sah"),
        "H4500": ("מָנוֹר", "H:N-M", "loom-beam", "ma.nor"),
        "H4501": ("מְנוֹרָה", "H:N-F", "lampstand", "me.no.rah"),
        "H4502": ("מִנְזָר", "H:N-M", "prince", "min.n.zar"),
        "H4503": ("מִנְחָה", "H:N-F", "offering", "min.chah"),
        "H4504": ("מִנְחָה", "A:N-F", "offering", "min.chah"),
        "H4505": ("מְנַחֵם", "N:N-M-P", "Menahem", "me.na.chem"),
        "H4506a": ("מָנַ֫חַת", "N:N--L", "Manahath", "ma.na.chat"),
        "H4506b": ("מָנַ֫חַת", "N:N-M-P", "Manahath", "ma.na.chat"),
        "H4507": ("מְנִי", "N:N-M-T", "Destiny", "me.ni"),
        "H4508": ("מִנִּי", "N:N--L", "Minni", "min.ni"),
        "H4509": ("מִנְיָמִין", "N:N-M-P", "Miniamin", "min.ya.min"),
        "H4510": ("מִנְיָן", "A:N-M", "number", "min.yan"),
        "H4511": ("מִנִּית", "N:N--L", "Minnith", "min.nit"),
        "H4512": ("מִנְלֶה", "H:N-M", "gain", "min.leh"),
        "H4513": ("מָנַע", "H:V", "to withhold", "ma.na"),
        "H4514": ("מַנְעוּל", "H:N-M", "bolt", "man.ul"),
        "H4515": ("מִנְעָל", "H:N-M", "bolt", "min.al"),
        "H4516": ("מַנְעַמִּים", "H:N-M", "delicacy", "man.am"),
        "H4517": ("מְנַעֲנֵעַ", "H:N-M", "castanets", "me.na.na"),
        "H4518": ("מְנַקִּיָּה", "H:N-F", "bowl", "me.naq.qit"),
        "H4519": ("מְנַשֶּׁה", "N:N-M-P", "Manasseh", "me.nash.sheh"),
        "H4520": ("מְנַשִּׁי", "N:N--PG", "Manassite", "me.nash.shi"),
        "H4521": ("מְנָת", "H:N-F", "portion", "me.nat"),
        "H4522": ("מַס", "H:N-M", "taskworker", "mas"),
        "H4523": ("מָס", "H:A", "despairing", "mas"),
        "H4524": ("מֵסַב", "H:N-M", "surrounds", "me.sav"),
        "H4525": ("מַסְגֵּר", "H:N-M", "locksmith", "mas.ger"),
        "H4526": ("מִסְגֶּ֫רֶת", "H:N-F", "perimeter", "mis.ge.ret"),
        "H4527": ("מַסַּד", "H:N-M", "foundation", "mas.sad"),
        "H4528": ("מִסְדְּרוֹן", "H:N-M", "porch", "mis.de.ron"),
        "H4529": ("מָסָה", "H:V", "to liquefy", "ma.sah"),
        "H4530": ("מִסַּת", "H:N", "payment", "mis.sah"),
        "H4531a": ("מַסָּה", "H:N-F", "despair", "mas.sah"),
        "H4531b": ("מַסָּה", "H:N-F", "trial", "mas.sah"),
        "H4532": ("מַסָּה", "N:N--L", "Massah", "mas.sah"),
        "H4533": ("מַסְוֶה", "H:N-M", "veil", "mas.veh"),
        "H4534": ("מְסוּכָה", "H:N-F", "hedge", "me.su.khah"),
        "H4535": ("מַסָּח", "H:N-M", "defense", "mas.sach"),
        "H4536": ("מִסְחָר", "H:N-M", "merchandise", "mis.char"),
        "H4537": ("מָסַךְ", "H:V", "to mix", "ma.sakh"),
        "H4538": ("מֶ֫סֶךְ", "H:N-M", "mixture", "me.sekh"),
        "H4539": ("מָסָךְ", "H:N-M", "covering", "ma.sakh"),
        "H4540": ("מְסֻכָֿה", "H:N-F", "covering", "me.suk.kah"),
        "H4541a": ("מַסֵּכָה", "H:N-F", "liquid", "mas.se.khah"),
        "H4541b": ("מַסֵּכָה", "H:N-F", "veil", "mas.se.khah"),
        "H4542": ("מִסְכֵּן", "H:A", "poor", "mis.ken"),
        "H4543": ("מִסְכְּנוֹת", "H:N-F", "storage", "mis.ke.nah"),
        "H4544": ("מִסְכֵּנֻת", "H:N-F", "poverty", "mis.ke.nut"),
        "H4545": ("מַסֶּ֫כֶת", "H:N-F", "weave", "mas.se.khet"),
        "H4546": ("מְסִלָּה", "H:N-F", "highway", "me.sil.lah"),
        "H4547": ("מַסְלוּל", "H:N-M", "highway", "mas.lul"),
        "H4548": ("מַסְמֵר", "H:N-F", "nail", "mas.mer"),
        "H4549": ("מָסַס", "H:V", "to melt", "ma.sas"),
        "H4550": ("מַסַּע", "H:N-M", "journey", "mas.sa"),
        "H4551a": ("מַסָּע", "H:N-M", "quarry", "mas.sa"),
        "H4551b": ("מַסָּע", "H:N-M", "missile", "mas.sa"),
        "H4552": ("מִסְעָד", "H:N-M", "support", "mis.ad"),
        "H4553": ("מִסְפֵּד", "H:N-M", "mourning", "mis.ped"),
        "H4554": ("מִסְפּוֹא", "H:N-M", "fodder", "mis.po"),
        "H4555": ("מִסְפָּחָה", "H:N-F", "veil", "mis.pa.chah"),
        "H4556": ("מִסְפַּ֫חַת", "H:N-F", "scab", "mis.pa.chat"),
        "H4557": ("מִסְפָּר", "H:N-M", "number", "mis.par"),
        "H4558": ("מִסְפָּר", "N:N-M-P", "Mispar", "mis.par"),
        "H4559": ("מִסְפֶּרֶת", "N:N-M-P", "Mispereth", "mis.pe.ret"),
        "H4560": ("מָסַר", "H:V", "to commit", "ma.sar"),
        "H4561": ("מֹסָר", "H:N-M", "discipline", "mo.sar"),
        "H4562": ("מָסֹ֫רֶת", "H:N-F", "pledge", "ma.so.ret"),
        "H4563": ("מִסְתּוֹר", "H:N-M", "shelter", "mis.tor"),
        "H4564": ("מַסְתֵּר", "H:N-M", "hiding", "mas.ter"),
        "H4565": ("מִסְתָּר", "H:N-M", "hiding", "mis.tar"),
        "H4566": ("מַעְבָּד", "H:N-M", "work", "ma.bad"),
        "H4567": ("מַעֲבַד", "A:N-M", "work", "ma.vad"),
        "H4568": ("מַעֲבֶה", "H:N-M", "thickness", "ma.a.veh"),
        "H4569a": ("מַעֲבָר", "H:N-M", "ford", "ma.a.var"),
        "H4569b": ("מַעֲבָרָה", "H:N-M", "ford", ""),
        "H4570": ("מַעְגָּל", "H:N-M", "track", "ma.gal"),
        "H4571": ("מָעַד", "H:V", "to slip", "ma.ad"),
        "H4572": ("מַעֲדַי", "N:N-M-P", "Maadai", "ma.a.day"),
        "H4573": ("מַעֲדְיָה", "N:N-M-P", "Maadiah", "ma.a.de.yah"),
        "H4574": ("מַעֲדָן", "H:N-M", "delicacy", "ma.a.dan"),
        "H4575": ("מַעֲדַנּוֹת", "H:N-F", "bond", "ma.a.dan.nah"),
        "H4576": ("מַעְדֵּר", "H:N-M", "hoe", "ma.der"),
        "H4577": ("מְעָא", "A:N-M", "belly", "me.ah"),
        "H4578": ("מֵעֶה", "H:N-M", "belly", "me.eh"),
        "H4579": ("מָעָה", "H:N-F", "grain", "me.ah"),
        "H4580": ("מָעוֹג", "H:N-M", "bun", "ma.og"),
        "H4581": ("מָעוֹז", "H:N-M", "security", "ma.oz"),
        "H4582": ("מָעוֹךְ", "N:N-M-P", "Maoch", "ma.okh"),
        "H4583": ("מָעוֹן", "H:N-M", "habitation", "ma.on"),
        "H4584": ("מָעוֹן", "N:N--L / N:N--LG / N:N-M-P", "Maon", "ma.on"),
        "H4585": ("מְעֹנָה", "H:N-F", "habitation", "me.o.nah"),
        "H4586": ("מְעוּנִים", "N:N--LG", "Meunite", "me.u.ni"),
        "H4587": ("מְעוֹנֹתַי", "N:N-M-P", "Meonothai", "me.o.no.tay"),
        "H4588": ("מָעוּף", "H:N-M", "gloom", "ma.uph"),
        "H4589": ("מָעוֹר", "H:N-M", "nakedness", "ma.or"),
        "H4590": ("מַעַזְיָהוּ", "N:N-M-P", "Maaziah", "ma.az.yah"),
        "H4591": ("מָעַט", "H:V", "to diminish", "ma.at"),
        "H4592": ("מְעַט", "H:N-M", "little", "me.at"),
        "H4593": ("מָעֹט", "H:N", "sharp", "ma.ot"),
        "H4594": ("מַעֲטֶה", "H:N-M", "mantle", "ma.a.teh"),
        "H4595": ("מַעֲטָפָה", "H:N-F", "overtunic", "ma.a.ta.phah"),
        "H4596": ("מְעִי", "H:N-M", "heap", "me.i"),
        "H4597": ("מָעַי", "N:N-M-P", "Maai", "ma.ay"),
        "H4598": ("מְעִיל", "H:N-M", "robe", "me.il"),
        "H4599": ("מַעְיָן", "H:N-M", "spring", "ma.yan"),
        "H4600": ("מָעַךְ", "H:V", "to bruise", "ma.akh"),
        "H4601": ("מַעֲכָה", "N:N-M-P / N:N-F-P / N:N--L", "Maacah", "ma.a.khah"),
        "H4602": ("מַעֲכָתִי", "N:N--LG", "Maacathite", "ma.a.kha.ti"),
        "H4603": ("מָעַל", "H:V", "be unfaithful", "ma.al"),
        "H4604": ("מַ֫עַל", "H:N-M", "unfaithfulness", "ma.al"),
        "H4605": ("מַ֫עַל", "H:Adv", "above", "ma.al"),
        "H4606": ("מֶעָל", "A:N-M", "going down", "me.al"),
        "H4607": ("מֹ֫עַל", "H:N-M", "lifting", "mo.al"),
        "H4608": ("מַעֲלֶה", "H:N-M", "ascent", "ma.a.leh"),
        "H4609a": ("מַעֲלָה", "H:N-F", "thought", "ma.a.lah"),
        "H4609b": ("מַעֲלָה", "H:N-F", "step", "ma.a.lah"),
        "H4610": ("עַקְרַבִּים", "N:N--L", "Akrabbin", "ma.a.leh aq.rab.bim"),
        "H4611": ("מַעֲלָל", "H:N-M", "deed", "ma.a.lal"),
        "H4612": ("מַעֲמָד", "H:N-M", "office", "ma.a.mad"),
        "H4613": ("מׇעֳמָ֑ד", "H:N-M", "foothold", "mo.o.mad"),
        "H4614": ("מַעֲמָסָה", "H:N-F", "burden", "ma.a.ma.sah"),
        "H4615": ("מַעֲמַקִּים", "H:N-M", "deep", "ma.a.maq"),
        "H4616": ("מַ֫עַן", "H:Prep / H:Conj", "because", "ma.an"),
        "H4617": ("מַעֲנֶה", "H:N-M", "answer", "ma.a.neh"),
        "H4618": ("מַעֲנָה", "H:N-F", "furrow", "ma.a.nah"),
        "H4619": ("מַ֫עַץ", "N:N-M-P", "Maaz", "ma.ats"),
        "H4620": ("מַעֲצֵבָה", "H:N-F", "torment", "ma.a.tse.vah"),
        "H4621": ("מַעֲצָד", "H:N-M", "axe", "ma.a.tsad"),
        "H4622": ("מַעְצוֹר", "H:N-M", "restraint", "ma.tsor"),
        "H4623": ("מַעְצָר", "H:N-M", "restraint", "ma.tsar"),
        "H4624": ("מַעֲקֶה", "H:N-M", "parapet", "ma.a.qeh"),
        "H4625": ("מַעֲקָשׁ", "H:N-M", "crooked place", "ma.a.qash"),
        "H4626": ("מַעַר", "H:N-M", "nakedness", "ma.ar"),
        "H4627": ("מַעֲרָב", "H:N-M", "merchandise", "ma.a.rav"),
        "H4628": ("מַעֲרָב", "H:N-M", "west", "ma.a.rav"),
        "H4629": ("מַעֲרֶה", "H:N-M", "gap", "ma.a.reh"),
        "H4630": ("מַעֲרָה", "H:N-F", "army", "ma.a.rah"),
        "H4631": ("מְעָרָה", "H:N-F", "cave", "me.a.rah"),
        "H4632": ("מְעָרָה", "N:N--L", "Mearah", "me.a.rah"),
        "H4633": ("מַעֲרָךְ", "H:N-M", "plan", "ma.a.rakh"),
        "H4634": ("מַעֲרָכָה", "H:N-F", "rank", "ma.a.ra.khah"),
        "H4635": ("מַעֲרֶ֫כֶת", "H:N-F", "row", "ma.a.re.khet"),
        "H4636": ("מַעֲרֹם", "H:N-M", "naked", "ma.a.rom"),
        "H4637": ("מַעֲרָצָה", "H:N-F", "terror", "ma.a.ra.tsah"),
        "H4638": ("מַעֲרָת", "N:N--L", "Maarath", "ma.a.rat"),
        "H4639": ("מַעֲשֶׂה", "H:N-M", "deed", "ma.a.seh"),
        "H4640": ("מַעֲשַׂי", "N:N-M-P", "Maasai", "ma.a.say"),
        "H4641": ("מַעֲשֵׂיָהוּ", "N:N-M-P", "Maaseiah", "ma.a.se.yah"),
        "H4642": ("מַעֲשַׁקָּה", "H:N-F", "oppression", "ma.a.shaq.qah"),
        "H4643": ("מַעֲשֵׂר", "H:N-M", "tithe", "ma.a.ser"),
        "H4644": ("מֹף", "N:N--L", "Memphis", "moph"),
        "H4645": ("מִפְגָּע", "H:N-M", "target", "miph.ga"),
        "H4646": ("מַפָּח", "H:N-M", "exhalation", "map.pach"),
        "H4647": ("מַפֻּחַ", "H:N-M", "bellow", "map.pu.ach"),
        "H4648": ("מְפִיבֹשֶׁת", "N:N-M-P", "Mephibosheth", "me.phi.vo.shet"),
        "H4649": ("מֻפִּים", "N:N-M-P", "Muppim", "mup.pim"),
        "H4650": ("מֵפִיץ", "H:N-M", "war-club", "me.phits"),
        "H4651": ("מַפָּל", "H:N-M", "refuse", "map.pal"),
        "H4652": ("מִפְלָאָה", "H:N-F", "wonder", "miph.la.ah"),
        "H4653": ("מִפְלַגָּה", "H:N-F", "division", "miph.lag.gah"),
        "H4654a": ("מַפָּלָה", "H:N-F", "ruin", "map.pa.lah"),
        "H4654b": ("מַפֵּלָה", "H:N-F", "ruin", ""),
        "H4655": ("מִפְלָט", "H:N-M", "escape", "miph.lat"),
        "H4656": ("מִפְלֶ֫צֶת", "H:N-F", "horror", "miph.le.tset"),
        "H4657": ("מִפְלָשׂ", "H:N-M", "swaying", "miph.la.s"),
        "H4658": ("מַפֶּ֫לֶת", "H:N-F", "carcass", "map.pe.let"),
        "H4659a": ("מִפְעָל", "H:N-M", "work", "miph.al"),
        "H4659b": ("מִפְעָלָה", "H:N-M", "deed", ""),
        "H4660": ("מַפָּץ", "H:N-M", "shattering", "map.pats"),
        "H4661": ("מַפֵּץ", "H:N-M", "war-club", "map.pets"),
        "H4662": ("מִפְקָד", "H:N-M", "appointment", "miph.qad"),
        "H4663": ("מִפְקָד", "N:N--L", "Miphkad", "miph.qad"),
        "H4664": ("מִפְרָץ", "H:N-M", "landing", "miph.rats"),
        "H4665": ("מַפְרֶקֶת", "H:N-F", "neck", "miph.re.qet"),
        "H4666": ("מִפְרָשׂ", "H:N-M", "spreading", "miph.ra.s"),
        "H4667": ("מִפְשָׂעָה", "H:N-F", "hip", "miph.sa.ah"),
        "H4668": ("מַפְתֵּחַ", "H:N-M", "key", "maph.te.ach"),
        "H4669": ("מִפְתָּח", "H:N-M", "opening", "miph.tach"),
        "H4670": ("מִפְתָּן", "H:N-M", "threshold", "miph.tan"),
        "H4671": ("מוֹץ", "H:N-M", "chaff", "mots"),
        "H4672": ("מָצָא", "H:V", "to find", "ma.tsa"),
        "H4673": ("מַצָּב", "H:N-M", "station", "mats.tsav"),
        "H4674": ("מֻצָּב", "H:N-M", "entrenchment", "muts.tsav"),
        "H4675": ("מִצָּבָה", "H:N-F", "guard", "mats.tsa.vah"),
        "H4676": ("מַצֵּבָה", "H:N-F", "pillar", "mats.tse.vah"),
        "H4677": ("מְצֹבָיָה", "N:N--LG", "Mezobaite", "me.tso.va.yah"),
        "H4678": ("מַצֶּבֶת", "H:N-F", "pillar", "mats.tse.vet"),
        "H4679": ("מְצָד", "H:N-F", "stronghold", "me.tsad"),
        "H4680": ("מָצָה", "H:V", "to drain", "ma.tsah"),
        "H4681": ("מֹצָה", "N:N--L", "Mozah", "mo.tsah"),
        "H4682": ("מַצָּה", "H:N-F", "unleavened bread", "mats.tsah"),
        "H4683": ("מַצָּה", "H:N-F", "strife", "mats.tsah"),
        "H4684": ("מַצְהָלָה", "H:N-F", "neighing", "mats.ha.lah"),
        "H4685a": ("מָצוֹד", "H:N-M", "siegework", "ma.tsod"),
        "H4685b": ("מָצוֹד", "H:N-M", "net", "ma.tsod"),
        "H4685c": ("מְצוֹדָה", "H:N-F", "net", ""),
        "H4685d": ("מְצוֹדָה", "H:N-F", "stronghold", ""),
        "H4686a": ("מְצוּדָה", "H:N-F", "net", "ma.tsud"),
        "H4686b": ("מְצוּדָה", "H:N-F", "fortress", "ma.tsud"),
        "H4687": ("מִצְוָה", "H:N-F", "commandment", "mits.vah"),
        "H4688": ("מְצוֹלָה", "H:N-F", "depth", "me.tso.lah"),
        "H4689": ("מָצוֹק", "H:N-M", "distress", "ma.tsoq"),
        "H4690": ("מָצוּק", "H:N-M", "pillar", "ma.tsuq"),
        "H4691": ("מְצוּקָה", "H:N-F", "distress", "me.tsu.qah"),
        "H4692": ("מָצוֹר", "H:N-M", "siege", "ma.tsor"),
        "H4693": ("מָצוֹר", "N:N--L", "Egypt", "ma.tsor"),
        "H4694": ("מְצוּרָה", "H:N-F", "fortress", "me.tsu.rah"),
        "H4695": ("מַצּוּת", "H:N-F", "strife", "mats.tsut"),
        "H4696": ("מֵ֫צַח", "H:N-M", "forehead", "me.tsach"),
        "H4697": ("מִצְחָה", "H:N-F", "greave", "mits.chah"),
        "H4698": ("מְצִלָּה", "H:N-F", "bell", "me.tsil.lah"),
        "H4699": ("מְצֻלָה", "H:N-M", "ravine", "me.tsul.lah"),
        "H4700": ("מְצִלְתַּיִם", "H:N-F", "cymbal", "me.tse.let"),
        "H4701": ("מִצְנֶ֫פֶת", "H:N-F", "turban", "mits.ne.phet"),
        "H4702": ("מַצָּע", "H:N-M", "bed", "mats.tsa"),
        "H4703": ("מִצְעָד", "H:N-M", "step", "mits.ad"),
        "H4704": ("מִצְּעִירָה", "H:N-F", "little", "mits.ts.i.rah"),
        "H4705": ("מִצְעָר", "H:N-M", "little", "mits.ar"),
        "H4706": ("מִצְעָר", "N:N--L", "Mizar", "mits.ar"),
        "H4707": ("מִצְפֶּה", "H:N-M", "lookout", "mits.peh"),
        "H4708": ("מִצְפֶּה", "N:N--L", "Mizpeh", "mits.peh"),
        "H4709": ("מִצְפָּה", "N:N--L", "Mizpeh", "mits.pah"),
        "H4710": ("מַצְפּוֹן", "H:N-M", "treasure", "mits.pun"),
        "H4711": ("מָצַץ", "H:V", "to drink deeply", "ma.tsats"),
        "H4712": ("מֵצַר", "H:N-M", "terror", "me.tsar"),
        "H4713": ("מִצְרִי", "N:N--LG", "Egyptian", "mits.ri"),
        "H4714": ("מִצְרַ֫יִם", "N:N--L", "Egypt", "mits.ra.yim"),
        "H4715": ("מַצְרֵף", "H:N-M", "crucible", "mits.reph"),
        "H4716": ("מַק", "H:N-M", "decay", "maq"),
        "H4717": ("מַקָּבָה", "H:N-F", "hammer", "maq.qa.vah"),
        "H4718a": ("מַקֶּ֫בֶת", "H:N-F", "hammer", "maq.qe.vet"),
        "H4718b": ("מַקֶּבֶת", "H:N-F", "hole", "maq.qe.vet"),
        "H4719": ("מַקֵּדָה", "N:N--L", "Makkedah", "maq.qe.dah"),
        "H4720": ("מִקְדָּשׁ", "H:N-M", "sanctuary", "miq.dash"),
        "H4721": ("מַקְהֵל", "H:N-M", "assembly", "maq.hel"),
        "H4722": ("מַקְהֵלֹת", "N:N--L", "Makheloth", "maq.he.lot"),
        "H4723a": ("קֹוֶה", "N:N--L", "Kue", "miq.veh"),
        "H4723b": ("מִקְוֶה", "H:N-M", "hope", "miq.veh"),
        "H4723c": ("מִקְוֶה", "H:N-M", "collection", "miq.veh"),
        "H4724": ("מִקְוָה", "H:N-F", "reservoir", "miq.vah"),
        "H4725": ("מָקוֹם", "H:N-M", "standing place", "ma.qom"),
        "H4726": ("מָקוֹר", "H:N-M", "fountain", "ma.qor"),
        "H4727": ("מִקָּח", "H:N-M", "taking", "miq.qach"),
        "H4728": ("מַקָּחָה", "H:N-F", "ware", "maq.qa.chah"),
        "H4729": ("מִקְטָר", "H:N-M", "fumigation", "miq.tar"),
        "H4730": ("מִקְטֶ֫רֶת", "H:N-F", "censer", "miq.te.ret"),
        "H4731": ("מַקֵּל", "H:N-M", "rod", "maq.qel"),
        "H4732": ("מִקְלוֹת", "N:N-M-P", "Mikloth", "miq.lot"),
        "H4733": ("מִקְלָט", "H:N-M", "refuge", "miq.lat"),
        "H4734": ("מִקְלַ֫עַת", "H:N-F", "engraving", "miq.la.at"),
        "H4735": ("מִקְנֶה", "H:N-M", "livestock", "miq.neh"),
        "H4736": ("מִקְנָה", "H:N-F", "purchase", "miq.nah"),
        "H4737": ("מִקְנֵיָ֫הוּ", "N:N-M-P", "Mikneiah", "miq.ne.ya.hu"),
        "H4738": ("מִקְסָם", "H:N-M", "divination", "miq.sam"),
        "H4739": ("מָקַץ", "N:N--L", "Makaz", "ma.qats"),
        "H4740": ("מִקְצֹעַ", "H:N-M", "corner", "maq.tso.a"),
        "H4741": ("מַקְצֻעָה", "H:N-F", "plane", "maq.tsu.ah"),
        "H4742": ("מְקֻצְעָה", "H:N-F", "corner", "me.quts.ah"),
        "H4743": ("מָקַק", "H:V", "to rot", "ma.qaq"),
        "H4744": ("מִקְרָא", "H:N-M", "assembly", "miq.ra"),
        "H4745": ("מִקְרֶה", "H:N-M", "accident", "miq.reh"),
        "H4746": ("מְקָרֶה", "H:N-M", "rafter", "me.qa.reh"),
        "H4747": ("מְקֵרָה", "H:N-F", "cool", "me.qe.rah"),
        "H4748": ("מִקְשֶׁה", "H:N-M", "hairstyle", "miq.sheh"),
        "H4749": ("מִקְשָׁה", "H:N-F", "beating", "miq.shah"),
        "H4750": ("מִקְשָׁה", "H:N-F", "cucumber field", "miq.shah"),
        "H4751": ("מַר", "H:A-M", "bitter", "mar"),
        "H4752": ("מַר", "H:N-M", "drop", "mar"),
        "H4753": ("מֹר", "H:N-M", "myrrh", "mor"),
        "H4754": ("מָרָא", "H:V", "to flap", "ma.ra"),
        "H4755": ("מָרָא", "N:N-F-P", "Mara", "ma.ra"),
        "H4756": ("מָרֵא", "A:N-M", "lord", "ma.re"),
        "H4757": ("מְרֹדַךְ בַּלְאֲדָן", "N:N-M-P", "Merodach-baladan", "me.ro.dakh bal.a.dan"),
        "H4758": ("מַרְאֶה", "H:N-M", "appearance", "mar.eh"),
        "H4759a": ("מַרְאָה", "H:N-F", "vision", "mar.ah"),
        "H4759b": ("מַרְאָה", "H:N-F", "mirror", "mar.ah"),
        "H4760": ("מֻרְאָה", "H:N-F", "crop", "mur.ah"),
        "H4761": ("מְרַאֲשָׁוֹת", "H:N-F", "head", "mar.a.shah"),
        "H4762": ("מָרֵשָׁה", "N:N--L", "Mareshah", "mar.e.shah"),
        "H4763": ("מְרַאֲשָׁוֹת", "H:N-F", "head", "me.ra.a.shah"),
        "H4764": ("מֵרָב", "N:N-F-P", "Merab", "me.rav"),
        "H4765": ("מַרְבָד", "H:N-M", "covering", "mar.vad"),
        "H4766": ("מַרְבֶה", "H:N-M", "abundance", "mar.veh"),
        "H4767": ("מִרְבָּה", "H:N-F", "much", "mir.bah"),
        "H4768": ("מַרְבִּית", "H:N-F", "greatness", "mar.bit"),
        "H4769": ("מַרְבֵּץ", "H:N-M", "resting", "mar.bets"),
        "H4770": ("מַרְבֵּק", "H:N-M", "stall", "mar.beq"),
        "H4771": ("מַרְגּוֹעַ", "H:N-M", "rest", "mar.go.a"),
        "H4772": ("מַרְגְּלוֹת", "H:N-F", "feet", "mar.ge.lah"),
        "H4773": ("מַרְגֵּמָה", "H:N-F", "sling", "mar.ge.mah"),
        "H4774": ("מַרְגֵּעָה", "H:N-F", "repose", "mar.ge.ah"),
        "H4775": ("מָרַד", "H:V", "to rebel", "ma.rad"),
        "H4776": ("מְרַד", "A:N-M", "rebellion", "me.rad"),
        "H4777": ("מֶ֫רֶד", "H:N-M", "rebellion", "me.red"),
        "H4778": ("מֶ֫רֶד", "N:N-M-P", "Mered", "me.red"),
        "H4779": ("מָרָד", "A:A", "rebellious", "ma.rad"),
        "H4780": ("מַרְדּוּת", "H:N-F", "rebellious", "mar.dut"),
        "H4781": ("מְרֹדָךְ", "N:N-M-T", "Marduk", "me.ro.dakh"),
        "H4782": ("מׇרְדֳּכַי", "N:N-M-P", "Mordecai", "mor.de.khay"),
        "H4783": ("מֻרְדָּף", "H:N-M", "persecution", "mur.daph"),
        "H4784": ("מָרָה", "H:V", "to rebel", "ma.rah"),
        "H4785": ("מָרָה", "N:N--L", "Marah", "ma.rah"),
        "H4786": ("מֹרָה", "H:N-F", "bitterness", "mo.rah"),
        "H4787": ("מׇרָּה", "H:N-F", "bitterness", "mor.rah"),
        "H4788": ("מָרוֹד", "H:N-M", "wandering", "ma.rud"),
        "H4789": ("מֵרוֹז", "N:N--L", "Meroz", "me.roz"),
        "H4790": ("מְרוֹחַ", "H:N-M", "crushed", "me.ro.ach"),
        "H4791": ("מָרוֹם", "H:N-M", "height", "ma.rom"),
        "H4792": ("מֵרוֹם", "N:N--L", "Merom", "me.rom"),
        "H4793": ("מֵרוֹץ", "H:N-M", "race", "me.rots"),
        "H4794": ("מְרוּצָה", "H:N-F", "running", "me.ru.tsah"),
        "H4795": ("מָרוּק", "H:N-M", "rubbing", "ma.ruq"),
        "H4796": ("מָרוֹת", "N:N--L", "Maroth", "ma.rot"),
        "H4797": ("מִרְזַח", "H:N-M", "banquet", "mir.zach"),
        "H4798": ("מַרְזֵחַ", "H:N-M", "mourning", "mar.ze.ach"),
        "H4799": ("מָרַח", "H:V", "to rub", "ma.rach"),
        "H4800": ("מֶרְחָב", "H:N-M", "broad", "mer.chav"),
        "H4801": ("מֶרְחָק", "H:N-M", "distance", "mer.chaq"),
        "H4802": ("מַרְחֶ֫שֶׁת", "H:N-F", "pan", "mar.che.shet"),
        "H4803": ("מָרַט", "H:V", "to smooth", "ma.rat"),
        "H4804": ("מְרַט", "A:V", "to pluck", "me.rat"),
        "H4805": ("מְרִי", "H:N-M", "rebellion", "me.ri"),
        "H4806": ("מְרִיא", "H:N-M", "fatling", "me.ri"),
        "H4807": ("מְרִיב בַּ֫עַל", "N:N-M-P", "Merib-baal", "me.riv ba.al"),
        "H4808": ("מְרִיבָה", "H:N-F", "provocation", "me.ri.vah"),
        "H4809": ("מְרִיבָה", "N:N--L", "Meribah", "me.ri.vah"),
        "H4810": ("מְרִי בַעַל", "N:N-M-P", "Merib-baal", "me.ri va.al"),
        "H4811": ("מְרָיָה", "N:N-M-P", "Meraiah", "me.ra.yah"),
        "H4812": ("מְרָיוֹת", "N:N-M-P", "Meraioth", "me.ra.vot"),
        "H4813": ("מִרְיָם", "N:N-F-P", "Miriam", "mir.yam"),
        "H4814": ("מְרִירוּת", "H:N-F", "bitterness", "me.ri.rut"),
        "H4815": ("מְרִירִי", "H:A", "bitter", "me.ri.ri"),
        "H4816": ("מֹ֫רֶךְ", "H:N-M", "weakness", "mo.rekh"),
        "H4817": ("מֶרְכָּב", "H:N-M", "chariot", "mer.kav"),
        "H4818": ("מֶרְכָּבָה", "H:N-F", "chariot", "mer.ka.vah"),
        "H4819": ("מַרְכֹּ֫לֶת", "H:N-F", "market", "mar.ko.let"),
        "H4820": ("מִרְמָה", "H:N-F", "deceit", "mir.mah"),
        "H4821": ("מִרְמָה", "N:N-M-P", "Mirmah", "mir.mah"),
        "H4822": ("מְרֵמוֹת", "N:N-M-P", "Meremoth", "me.re.mot"),
        "H4823": ("מִרְמָס", "H:N-M", "trampling", "mir.mas"),
        "H4824": ("מֵרֹנֹתִי", "N:N--LG", "Meronothite", "me.ro.no.ti"),
        "H4825": ("מֶ֫רֶס", "N:N-M-P", "Meres", "me.res"),
        "H4826": ("מַרְסְנָא", "N:N-M-P", "Marsena", "mar.se.na"),
        "H4827": ("מֵרַע", "H:N-M", "mischief", "me.ra"),
        "H4828": ("מֵרֵעַ", "H:N-M", "companion", "me.re.a"),
        "H4829": ("מִרְעֶה", "H:N-M", "pasture", "mir.eh"),
        "H4830": ("מַרְעִית", "H:N-F", "pasturing", "mir.it"),
        "H4831": ("מַרְעֲלָה", "N:N--L", "Maralah", "mar.a.lah"),
        "H4832": ("מַרְפֵּא", "H:N-M", "healing", "mar.pe"),
        "H4833": ("מַרְפֵּשׂ", "H:N-M", "mud", "mir.pa.s"),
        "H4834": ("מָרַץ", "H:V", "be sick", "ma.rats"),
        "H4835": ("מְרוּצָה", "H:N-F", "oppression", "me.ru.tsah"),
        "H4836": ("מַרְצֵעַ", "H:N-M", "awl", "mar.tse.a"),
        "H4837": ("מַרְצֶ֫פֶת", "H:N-F", "pavement", "mar.tse.phet"),
        "H4838": ("מָרַק", "H:V", "to polish", "ma.raq"),
        "H4839": ("מָרָק", "H:N-M", "broth", "ma.raq"),
        "H4840": ("מֶרְקָח", "H:N-M", "spice", "mer.qach"),
        "H4841": ("מֶרְקָחָה", "H:N-F", "ointment pot/seasoning", "mer.qa.chah"),
        "H4842": ("מִרְקַ֫חַת", "H:N-F", "ointment", "mir.qa.chat"),
        "H4843": ("מָרַר", "H:V", "to provoke", "ma.rar"),
        "H4844": ("מָרֹר", "H:N-M", "bitterness", "me.ror"),
        "H4845": ("מְרֵרָה", "H:N-F", "gall", "me.re.rah"),
        "H4846": ("מְרֹרָה", "H:N-F", "gall", "me.ro.rah"),
        "H4847": ("מְרָרִי", "N:N-M-P", "Merari", "me.ra.ri"),
        "H4848": ("מְרָרִי", "N:N--PG", "Merarite", "me.ra.ri"),
        "H4849": ("מִרְשַׁ֫עַת", "H:N-F", "wickedness", "mir.sha.at"),
        "H4850": ("מְרָתַיִם", "N:N--L", "Merathaim", "me.ra.ta.yim"),
        "H4851": ("מַשׁ", "N:N-M-P", "Mash", "mash"),
        "H4852": ("מֵשָׁא", "N:N--L", "Mesha", "me.sah"),
        "H4853a": ("מַשָּׂא", "H:N-M", "burden", "mas.sa"),
        "H4853b": ("מַשָּׂא", "H:N-M", "oracle", "mas.sa"),
        "H4854": ("מַשָּׂא", "N:N-M-P", "Massa", "mas.sa"),
        "H4855": ("מַשָּׁא", "H:N-M", "interest", "mash.sah"),
        "H4856": ("מַשֹּׂא", "H:N-M", "respect", "mas.so"),
        "H4857": ("מַשְׁאָב", "H:N-M", "well", "mash.av"),
        "H4858": ("מַשָּׂאָה", "H:N-F", "smoke", "mas.sa.ah"),
        "H4859": ("מַשָּׁאָה", "H:N-F", "loan", "mash.sha.ah"),
        "H4860": ("מַשָּׁאוֹן", "H:N-M", "guile", "mash.sha.on"),
        "H4861": ("מִשְׁאָל", "N:N--L", "Mishal", "mish.al"),
        "H4862": ("מִשְׁאָלָה", "H:N-F", "petition", "mish.a.lah"),
        "H4863": ("מִשְׁאֶ֫רֶת", "H:N-F", "kneading", "mish.e.ret"),
        "H4864": ("מַשְׂאֵת", "H:N-F", "uprising", "mas.et"),
        "H4865": ("מִשְׁבְּצוֹת", "H:N-F", "filigree", "mish.be.tsah"),
        "H4866": ("מַשְׁבֵּר", "H:N-M", "birth", "mish.ber"),
        "H4867": ("מִשְׁבָּר", "H:N-M", "wave", "mish.bar"),
        "H4868": ("מִשְׁבָּת", "H:N-M", "annihilation", "mish.bat"),
        "H4869a": ("מִשְׂגָּב", "H:N-M", "high refuge", "mis.gav"),
        "H4869b": ("מִשְׂגָּב", "N:N--L", "Misgab", "mis.gav"),
        "H4870": ("מִשְׁגֶּה", "H:N-M", "mistake", "mish.geh"),
        "H4871": ("מָשָׁה", "H:V", "to draw", "ma.shah"),
        "H4872": ("מֹשֶׁה", "N:N-M-P", "Moses", "mo.sheh"),
        "H4873": ("מֹשֶׁה", "N:N-M-P", "Moses", "mo.sheh"),
        "H4874": ("מַשֶּׁה", "H:N-M", "loan", "mash.sheh"),
        "H4875": ("מְשׁוֹאָה", "H:N-F", "desolation", "me.sho.ah"),
        "H4876": ("מַשּׁוּאָה", "H:N-F", "desolation", "mash.shu.ah"),
        "H4877": ("מְשׁוֹבָב", "N:N-M-P", "Meshobab", "me.sho.vav"),
        "H4878": ("מְשׁוּבָה", "H:N-F", "faithlessness", "me.shu.vah"),
        "H4879": ("מְשׁוּגָה", "H:N-F", "error", "me.shu.gah"),
        "H4880a": ("מָשׁוֹט", "H:N-M", "oar", "ma.shot"),
        "H4880b": ("מִשּׁוֹט", "H:N-M", "oar", ""),
        "H4881a": ("מְשֻכָֿה", "H:N-F", "hedge", "me.su.khah"),
        "H4881b": ("מְשֻׂכָּה", "H:N-F", "hedge", "me.su.khah"),
        "H4882": ("מְשׁוּסָה", "H:N-F", "spoil", "me.shu.sah"),
        "H4883": ("מַשּׂוֹר", "H:N-M", "saw", "mas.s.vor"),
        "H4884": ("מְשׂוּרָה", "H:N-F", "capacity", "me.su.rah"),
        "H4885": ("מָשׂוֹשׂ", "H:N-M", "rejoicing", "ma.s.vo.s"),
        "H4886": ("מָשַׁח", "H:V", "to anoint", "ma.shach"),
        "H4887": ("מְשַׁח", "A:N-M", "oil", "me.shach"),
        "H4888a": ("מִשְׁחָה", "H:N-F", "anointing", "mish.chah"),
        "H4888b": ("מָשְׁחָה", "H:N-F", "consecrated portion", ""),
        "H4889": ("מַשְׁחִית", "H:N-M", "destruction", "mash.chit"),
        "H4890": ("מִשְׂחַק", "H:N-M", "laughing", "mis.chaq"),
        "H4891": ("מִשְׁחָר", "H:N-M", "dawn", "mish.char"),
        "H4892": ("מַשְׁחֵת", "H:N-M", "destruction", "mash.chet"),
        "H4893a": ("מִשְׁחַת", "H:N-M", "mutilation", "mish.chat"),
        "H4893b": ("מָשְׁחָת", "H:N-M", "corruption", "mish.chat"),
        "H4894a": ("מִשְׁטָח", "H:N-M", "spreading-place", "mish.to.ach"),
        "H4894b": ("מִשְׁטוֹחַ", "H:N-M", "spreading-place", "mish.to.ach"),
        "H4895": ("מַשְׂטֵמָה", "H:N-F", "hatred", "mas.te.mah"),
        "H4896": ("מִשְׁטָר", "H:N-M", "rule", "mish.tar"),
        "H4897": ("מֶ֫שִׁי", "H:N-M", "silk", "me.shi"),
        "H4898": ("מְשֵׁיזַבְאֵל", "N:N-M-P", "Meshezabel", "me.she.zav.el"),
        "H4899": ("מָשִׁיחַ", "H:N-M", "anointed", "ma.shi.ach"),
        "H4900": ("מָשַׁךְ", "H:V", "to draw", "ma.shakh"),
        "H4901": ("מֶ֫שֶׁךְ", "H:N-M", "bag/price", "me.shekh"),
        "H4902": ("מֶ֫שֶׁךְ", "N:N-M-P", "Meshech", "me.shekh"),
        "H4903": ("מִשְׁכַּב", "A:N-M", "bed", "mish.kav"),
        "H4904": ("מִשְׁכָּב", "H:N-M", "bed", "mish.kav"),
        "H4905": ("מַשְׂכִּיל", "H:N-M", "Maschil", "mas.kil"),
        "H4906": ("מַשְׂכִּית", "H:N-F", "figure", "mas.kit"),
        "H4907": ("מִשְׁכַּן", "A:N-M", "habitation", "mish.kan"),
        "H4908": ("מִשְׁכָּן", "H:N-M", "tabernacle", "mish.kan"),
        "H4909": ("מַשְׂכֹּ֫רֶת", "H:N-F", "wage", "mas.ko.ret"),
        "H4910": ("מָשַׁל", "H:V", "to rule", "ma.shal"),
        "H4911a": ("מָשַׁל", "H:V", "to liken", "ma.shal"),
        "H4911b": ("מָשַׁל", "H:V", "to use a proverb", "ma.shal"),
        "H4912": ("מָשָׁל", "H:N-M", "proverb", "ma.shal"),
        "H4913": ("מָשָׁל", "N:N--L", "Mashal", "ma.shal"),
        "H4914": ("מְשֹׁל", "H:N-M", "byword", "me.shol"),
        "H4915a": ("מֹ֫שֶׁל", "H:N-M", "likeness", "mo.shel"),
        "H4915b": ("מֹ֫שֶׁל", "H:N-M", "dominion", "mo.shel"),
        "H4916a": ("מִשְׁלַח", "H:N-M", "sending", "mish.lo.ach"),
        "H4916b": ("מִשְׁלוֹחַ", "H:N-M", "sending", "mish.lo.ach"),
        "H4917": ("מִשְׁלַ֫חַת", "H:N-F", "deputation", "mish.la.chat"),
        "H4918": ("מְשֻׁלָּם", "N:N-M-P", "Meshullam", "me.shul.lam"),
        "H4919": ("מְשִׁלֵּמוֹת", "N:N-M-P", "Meshillemoth", "me.shil.le.mot"),
        "H4920": ("מְשֶׁלֶמְיָהוּ", "N:N-M-P", "Meshelemiah", "me.she.lem.yah"),
        "H4921": ("מְשִׁלֵּמִית", "N:N-M-P", "Meshillemith", "me.shil.le.mit"),
        "H4922": ("מְשֻׁלֶּ֫מֶת", "N:N-F-P", "Meshullemeth", "me.shul.le.met"),
        "H4923": ("מְשַׁמָּה", "H:N-F", "devastation", "me.sham.mah"),
        "H4924a": ("שָׁמָן", "H:N-M", "fat", ""),
        "H4924b": ("מִשְׁמָן", "H:N-M", "fatness", ""),
        "H4924c": ("מַשְׁמָן", "H:N-M", "fat piece", "mash.man"),
        "H4925": ("מִשְׁמַנָּה", "N:N-M-P", "Mishmannah", "mish.man.nah"),
        "H4926": ("מִשְׁמָע", "H:N-M", "hearing", "mish.ma"),
        "H4927": ("מִשְׁמָע", "N:N-M-P", "Mishma", "mish.ma"),
        "H4928": ("מִשְׁמַ֫עַת", "H:N-F", "guard", "mish.ma.at"),
        "H4929": ("מִשְׁמָר", "H:N-M", "custody", "mish.mar"),
        "H4930": ("מַשְׂמְרָה", "H:N-F", "nail", "mas.m.rah"),
        "H4931": ("מִשְׁמֶ֫רֶת", "H:N-F", "charge", "mish.me.ret"),
        "H4932": ("מִשְׁנֶה", "H:N-M", "second", "mish.neh"),
        "H4933": ("מְשִׁסָּה", "H:N-F", "plunder", "me.shis.sah"),
        "H4934": ("מִשְׁעוֹל", "H:N-M", "passage", "mish.ol"),
        "H4935": ("מִשְׁעִי", "H:N-F", "cleansing", "mish.i"),
        "H4936": ("מִשְׁעָם", "N:N-M-P", "Misham", "mish.am"),
        "H4937a": ("מִשְׁעָן", "H:N-M", "support", "mish.en"),
        "H4937b": ("מִשְׁעֵן", "H:N-M", "support", "mish.en"),
        "H4938a": ("מִשְׁעֵנָה", "H:N-F", "support", "mish.e.nah"),
        "H4938b": ("מִשְׁעֶ֫נֶת", "H:N-F", "staff", ""),
        "H4939": ("מִשְׂפָּח", "H:N-M", "bloodshed", "mis.pach"),
        "H4940": ("מִשְׁפָּחָה", "H:N-F", "family", "mish.pa.chah"),
        "H4941": ("מִשְׁפָּט", "H:N-M", "justice", "mish.pat"),
        "H4942": ("מִשְׁפְּתַ֫יִם", "H:N-M", "sheepfold", "mish.pat"),
        "H4943": ("מֶ֫שֶׁק", "H:N-M", "acquisition", "me.sheq"),
        "H4944": ("מַשָּׁק", "H:N-M", "rushing", "mash.shaq"),
        "H4945a": ("מַשְׁקֶה", "H:N-M", "cupbearer", "mash.qeh"),
        "H4945b": ("מַשְׁקֶה", "H:N-M", "irrigation", "mash.qeh"),
        "H4946": ("מִשְׁקוֹל", "H:N-M", "weight", "mish.qol"),
        "H4947": ("מַשְׁקוֹף", "H:N-M", "lintel", "mash.qoph"),
        "H4948": ("מִשְׁקָל", "H:N-M", "weight", "mish.qal"),
        "H4949": ("מִשְׁקֶ֫לֶת", "H:N-F", "level", "mish.qe.let"),
        "H4950": ("מִשְׁקָע", "H:N-M", "clarified", "mish.qa"),
        "H4951": ("מִשְׂרָה", "H:N-F", "dominion", "mis.rah"),
        "H4952": ("מִשְׁרָה", "H:N-F", "juice", "mish.rah"),
        "H4953": ("מַשְׁרוֹקִי", "A:N-F", "flute", "mash.ro.qi"),
        "H4954": ("מִשְׁרָעִי", "N:N--LG", "Mishraite", "mish.ra.i"),
        "H4955": ("מִשְׂרָפָה", "H:N-F", "burning", "mis.ra.phah"),
        "H4956": ("מִשְׂרְפוֹת מַיִם", "N:N--L", "Misrephoth-maim", "mis.r.phot ma.yim"),
        "H4957": ("מַשְׂרֵקָה", "N:N--L", "Masrekah", "mas.re.qah"),
        "H4958": ("מַשְׂרֵת", "H:N-M", "pan", "mas.ret"),
        "H4959": ("מָשַׁשׁ", "H:V", "to feel", "ma.shash"),
        "H4960": ("מִשְׁתֶּה", "H:N-M", "feast", "mish.teh"),
        "H4961": ("מִשְׁתֶּה", "A:N-M", "feast", "mish.teh"),
        "H4962": ("מַת", "H:N-M", "man", "mat"),
        "H4963": ("מַתְבֵּן", "H:N-M", "straw", "mat.ben"),
        "H4964": ("מֶ֫תֶג", "H:N-M", "bridle", "me.teg"),
        "H4965": ("מֶתֶג הָאַמָּה", "N:N--L", "Metheg-ammah", "me.teg ha.am.mah"),
        "H4966": ("מָתוֹק", "H:A", "sweet", "ma.toq"),
        "H4967": ("מְתוּשָׁאֵל", "N:N-M-P", "Methushael", "me.tu.sha.el"),
        "H4968": ("מְתוּשֶׁ֫לַח", "N:N-M-P", "Methuselah", "me.tu.she.lach"),
        "H4969": ("מָתַח", "H:V", "to spread", "ma.tach"),
        "H4970": ("מָתַי", "H:Adv", "how", "ma.tay"),
        "H4971": ("מַתְכֹ֫נֶת", "H:N-F", "tally", "mat.kho.net"),
        "H4972": ("מַתְּלָאָה", "H:N-F", "weariness", "mat.t.la.ah"),
        "H4973": ("מְתַלְּעוֹת", "H:N-F", "jaw", "me.tal.l.ah"),
        "H4974": ("מְתֹם", "H:N-M", "soundness", "me.tom"),
        "H4975": ("מׇתְנַ֫יִם", "H:N-M", "loin", "mo.ten"),
        "H4976": ("מַתָּן", "H:N-M", "gift", "mat.tan"),
        "H4977": ("מַתָּן", "N:N-M-P", "Mattan", "mat.tan"),
        "H4978": ("מַתְּנָא", "A:N-F", "gift", "mat.t.na"),
        "H4979": ("מַתָּנָה", "H:N-F", "gift", "mat.ta.nah"),
        "H4980": ("מַתָּנָה", "N:N--L", "Mattanah", "mat.ta.nah"),
        "H4981": ("מִתְנִי", "N:N--LG", "Mithnite", "mit.ni"),
        "H4982": ("מַתְּנַי", "N:N-M-P", "Mattenai", "mat.t.nay"),
        "H4983": ("מַתַּנְיָ֫הוּ", "N:N-M-P", "Mattaniah", "mat.tan.yah"),
        "H4984": ("מִתְנַשֵּׂא", "H:V", "to_exalt", "mit.nas.se"),
        "H4985": ("מָתֹק", "H:V", "be sweet", "ma.taq"),
        "H4986": ("מֶ֫תֶק", "H:N-M", "sweetness", "me.teq"),
        "H4987": ("מֹ֫תֶק", "H:N-M", "sweetness", "mo.teq"),
        "H4988": ("מָתָק", "H:V", "to sweeten", "ma.taq"),
        "H4989": ("מִתְקָה", "N:N--L", "Mithkah", "mit.qah"),
        "H4990": ("מִתְרְדָת", "N:N-M-P", "Mithredath", "mit.re.dat"),
        "H4991": ("מַתַּת", "H:N-F", "gift", "mat.tat"),
        "H4992": ("מַתַּתָּה", "N:N-M-P", "Mattathah", "mat.tat.tah"),
        "H4993": ("מַתִּתְיָ֫הוּ", "N:N-M-P", "Mattithiah", "mat.tit.yah"),
        "H4994": ("נָא", "H:Intj", "please", "na"),
        "H4995": ("נָא", "H:A", "raw", "na"),
        "H4996": ("נֹא", "N:N--L", "No", "no"),
        "H4997": ("נֹאד", "H:N-M", "wineskin", "nod"),
        "H4998": ("נָאָה", "H:V", "be lovely", "na.ah"),
        "H4999": ("נָוָה", "H:N-F", "habitation", "na.ah"),
        "H5000": ("נָאוֶה", "H:A", "lovely", "na.eh"),
        "H5001": ("נָאַם", "H:V", "to prophesy", "na.am"),
        "H5002": ("נְאֻם", "H:N-M", "utterance", "ne.um"),
        "H5003": ("נָאַף", "H:V", "to commit adultery", "na.aph"),
        "H5004": ("נִאֻף", "H:N-M", "adultery", "ni.uph"),
        "H5005": ("נַאֲפוּף", "H:N-M", "adultery", "na.a.phuph"),
        "H5006": ("נָאַץ", "H:V", "to spurn", "na.ats"),
        "H5007a": ("נְאָצָה", "H:N-F", "contempt", "ne.a.tsah"),
        "H5007b": ("נֶאָצָה", "H:N-F", "contempt", ""),
        "H5008": ("נָאַק", "H:V", "to groan", "na.aq"),
        "H5009": ("נְאָקָה", "H:N-F", "groan", "ne.a.qah"),
        "H5010": ("נָאַר", "H:V", "to disown", "na.ar"),
        "H5011": ("נֹב", "N:N--L", "Nob", "nov"),
        "H5012": ("נָבָא", "H:V", "to prophesy", "na.va"),
        "H5013": ("נבא", "A:V", "to prophesy", "ne.va"),
        "H5014": ("נָבַב", "H:V", "be hollow", "na.vav"),
        "H5015a": ("נְבוֹ", "N:N-M-P / N:N--L", "Nebo", "ne.vo"),
        "H5015b": ("נְבוֹ", "N:N-M-T", "Nebo", "ne.vo"),
        "H5016": ("נְבוּאָה", "H:N-F", "prophecy", "ne.vu.ah"),
        "H5017": ("נְבוּאָה", "A:N-F", "prophesying", "ne.vu.ah"),
        "H5018": ("נְבוּזַרְאֲדָן", "N:N-M-P", "Nebuzaradan", "ne.vu.zar.a.dan"),
        "H5019": ("נְבוּכַדְרֶאצַּר, נְבוּכַדְנֶצַּר", "N:N-M-P", "Nebuchadrezzar", "ne.vu.khad.nets.tsar"),
        "H5020": ("נְבוּכַדְנֶצַּר", "N:N-M-P", "Nebuchadnezzar", "ne.vu.khad.nets.tsar"),
        "H5021": ("נְבוּשַׁזְבָּן", "N:N-M-P", "Nebushazban", "ne.vu.shaz.ban"),
        "H5022": ("נָבוֹת", "N:N-M-P", "Naboth", "na.vot"),
        "H5023": ("נְבִזְבָּה", "A:N-F", "reward", "ne.viz.bah"),
        "H5024": ("נָבַח", "H:V", "to bark", "na.vach"),
        "H5025": ("נֹ֫בַח", "N:N-M-P / N:N--L", "Nobah", "no.vach"),
        "H5026": ("נִבְחַז", "N:N-M-T", "Nibhaz", "niv.chaz"),
        "H5027": ("נָבַט", "H:V", "to look", "na.vat"),
        "H5028": ("נְבָט", "N:N-M-P", "Nebat", "ne.vat"),
        "H5029": ("נְבִיא", "A:N-M", "prophet", "ne.vi"),
        "H5030": ("נָבִיא", "H:N-M", "prophet", "na.vi"),
        "H5031": ("נְבִיאָה", "H:N-F", "prophetess", "ne.vi.ah"),
        "H5032": ("נְבָיוֹת", "N:N-M-P", "Nebaioth", "ne.va.vot"),
        "H5033": ("נֵ֫בֶךְ", "H:N-M", "spring", "ne.vekh"),
        "H5034a": ("נָבַל", "H:V", "be senseless", "na.val"),
        "H5034b": ("נָבֵל", "H:V", "to wither", "na.vel"),
        "H5035a": ("נֵ֫בֶל", "H:N-M", "bag", "ne.vel"),
        "H5035b": ("נֵ֫בֶל", "H:N-M", "harp", "ne.vel"),
        "H5036": ("נָבָל", "H:A", "foolish", "na.val"),
        "H5037": ("נָבָל", "N:N-M-P", "Nabal", "na.val"),
        "H5038": ("נְבֵלָה", "H:N-F", "carcass", "ne.ve.lah"),
        "H5039": ("נְבָלָה", "H:N-F", "folly", "ne.va.lah"),
        "H5040": ("נַבְלוּת", "H:N-F", "lewdness", "nav.lut"),
        "H5041": ("נְבַלָּ֑ט", "N:N--L", "Neballat", "ne.val.lat"),
        "H5042": ("נָבַע", "H:V", "to bubble", "na.va"),
        "H5043": ("נֶבְרַשְׁתָּא", "A:N-F", "lampstand", "nev.re.sah"),
        "H5044": ("נִבְשָׁן", "N:N--L", "Nibshan", "niv.shan"),
        "H5045": ("נֶ֫גֶב", "N:N--L", "Negeb", "ne.gev"),
        "H5046": ("נָגַד", "H:V", "to tell", "na.gad"),
        "H5047": ("נְגַד", "A:V", "to flow", "ne.gad"),
        "H5048": ("נֶ֫גֶד", "H:Prep", "before", "ne.ged"),
        "H5049": ("נֶ֫גֶד", "A:Prep", "before", "ne.ged"),
        "H5050": ("נָגַהּ", "H:V", "to shine", "na.gah"),
        "H5051": ("נֹ֫גַהּ", "H:N-F", "brightness", "no.gah"),
        "H5052": ("נֹ֫גַהּ", "N:N-M-P", "Nogah", "no.gah"),
        "H5053": ("נֹ֫גַהּ", "A:N-F", "daylight", "no.gah"),
        "H5054": ("נְגֹהָה", "H:N-F", "brightness", "ne.go.hah"),
        "H5055": ("נָגַח", "H:V", "to gore", "na.gach"),
        "H5056": ("נַגָּח", "H:A", "goring", "nag.gach"),
        "H5057": ("נָגִיד", "H:N-M", "leader", "na.gid"),
        "H5058": ("נְגִינָה", "H:N-F", "music", "ne.gi.nah"),
        "H5059": ("נָגַן", "H:V", "to play", "na.gan"),
        "H5060": ("נָגַע", "H:V", "to touch", "na.ga"),
        "H5061": ("נֶ֫גַע", "H:N-M", "plague", "ne.ga"),
        "H5062": ("נָגַף", "H:V", "to strike", "na.gaph"),
        "H5063": ("נֶ֫גֶף", "H:N-M", "plague", "ne.geph"),
        "H5064": ("נָגַר", "H:V", "to pour", "na.gar"),
        "H5065": ("נָגַשׂ", "H:V", "to oppress", "na.ga.s"),
        "H5066": ("נָגַשׁ", "H:V", "to approach", "na.gash"),
        "H5067": ("נֵד", "H:N-M", "heap", "ned"),
        "H5068": ("נָדַב", "H:V", "be willing", "na.dav"),
        "H5069": ("נְדַב", "A:V", "be willing", "ne.dav"),
        "H5070": ("נָדָב", "N:N-M-P", "Nadab", "na.dav"),
        "H5071": ("נְדָבָה", "H:N-F", "voluntariness", "ne.da.vah"),
        "H5072": ("נְדַבְיָה", "N:N-M-P", "Nedabiah", "ne.dav.yah"),
        "H5073": ("נִדְבָּךְ", "A:N-M", "layer", "nid.bakh"),
        "H5074": ("נָדַד", "H:V", "to wander", "na.dad"),
        "H5075": ("נְדַד", "A:V", "to flee", "ne.dad"),
        "H5076": ("נְדוּד", "H:N-M", "tossing", "na.dud"),
        "H5077a": ("נָדָא", "H:V", "to drive away", "na.dah"),
        "H5077b": ("נָדָה", "H:V", "to put away", "na.dah"),
        "H5078": ("נֶ֑דֶה", "H:N-M", "gift", "ne.deh"),
        "H5079": ("נִדָּה", "H:N-F", "impurity", "nid.dah"),
        "H5080": ("נָדַח", "H:V", "to banish", "na.dach"),
        "H5081": ("נָדִיב", "H:A", "noble", "na.div"),
        "H5082": ("נְדִיבָה", "H:N-F", "honor", "ne.di.vah"),
        "H5083": ("נָדָן", "H:N-M", "gift", "na.dan"),
        "H5084": ("נָדָן", "H:N-M", "sheath", "na.dan"),
        "H5085": ("נִדְנֵ֑ה", "A:N-M", "sheath", "nid.neh"),
        "H5086": ("נָדַף", "H:V", "to drive", "na.daph"),
        "H5087": ("נָדַר", "H:V", "to vow", "na.dar"),
        "H5088": ("נֶ֫דֶר", "H:N-M", "vow", "ne.der"),
        "H5089": ("נֹהַּ", "H:N-M", "eminency", "no.ah"),
        "H5090a": ("נָהַג", "H:V", "to lead", "na.hag"),
        "H5090b": ("נָהַג", "H:V", "to moan", "na.hag"),
        "H5091": ("נָהָה", "H:V", "to wail", "na.hah"),
        "H5092": ("נְהִי", "H:N-M", "wailing", "ne.hi"),
        "H5093": ("נִהְיָה", "H:N-F", "lament", "nih.yah"),
        "H5094a": ("נְהִיר", "A:N-M", "light", "ne.hir"),
        "H5094b": ("נְהִירוּ", "A:N-M", "illumination", ""),
        "H5095": ("נָהַל", "H:V", "to guide", "na.hal"),
        "H5096": ("נַהֲלֹל", "N:N--L", "Nahalal", "na.ha.lal"),
        "H5097": ("נַהֲלֹל", "H:N-M", "watering", "na.ha.lol"),
        "H5098": ("נָהַם", "H:V", "to groan", "na.ham"),
        "H5099": ("נַ֫הַם", "H:N-M", "roaring", "na.ham"),
        "H5100": ("נְהָמָה", "H:N-F", "groaning", "ne.ha.mah"),
        "H5101": ("נָהַק", "H:V", "to bray", "na.haq"),
        "H5102a": ("נָהַר", "H:V", "to flow", "na.har"),
        "H5102b": ("נָהַר", "H:V", "to shine", "na.har"),
        "H5103": ("נְהַר", "A:N-M", "river", "ne.har"),
        "H5104": ("נָהָר", "H:N-M", "river", "na.har"),
        "H5105": ("נְהָרָה", "H:N-F", "light", "ne.ha.rah"),
        "H5106": ("נוּא", "H:V", "to forbid", "nu"),
        "H5107": ("נוּב", "H:V", "to bear fruit", "nuv"),
        "H5108a": ("נוֹב", "H:N-M", "fruit", "nov"),
        "H5108b": ("נִיב", "H:N-M", "fruit", ""),
        "H5109": ("נוֹבָ֑י", "N:N-M-P", "Nebai", "no.vay"),
        "H5110": ("נוּד", "H:V", "to wander", "nud"),
        "H5111": ("נוּד", "A:V", "to flee", "nud"),
        "H5112": ("נוֹד", "H:N-M", "wandering", "nod"),
        "H5113": ("נוֹד", "N:N--L", "Nod", "nod"),
        "H5114": ("נוֹדָ֑ב", "N:N--LG", "Nodab", "no.dav"),
        "H5115a": ("נָוָה", "H:V", "to beautify", "na.vah"),
        "H5115b": ("נָוָה", "H:V", "to dwell", "na.vah"),
        "H5116a": ("נָוֶה", "H:N-M", "pasture", "na.veh"),
        "H5116b": ("נָוֶה", "H:A", "dwelling", "na.veh"),
        "H5117": ("נוּחַ", "H:V", "to rest", "nu.ach"),
        "H5118": ("נוּחַ", "H:N-M", "rest", "nu.ach"),
        "H5119": ("נוֹחָה", "N:N-M-P", "Nohah", "no.chah"),
        "H5120": ("נוּט", "H:V", "to shake", "nut"),
        "H5121": ("נָוִית", "N:N--L", "Naioth", "na.vit"),
        "H5122": ("נְוָלוּ", "A:N-F", "dunghill", "ne.va.lu"),
        "H5123": ("נוּם", "H:V", "to slumber", "num"),
        "H5124": ("נוּמָה", "H:N-F", "drowsiness", "nu.mah"),
        "H5125": ("נוּן", "H:V", "to propagate", "nun"),
        "H5126": ("נוּן", "N:N-M-P", "Non", "nun"),
        "H5127": ("נוּס", "H:V", "to flee", "nus"),
        "H5128": ("נוּעַ", "H:V", "to shake", "nu.a"),
        "H5129": ("נוֹעַדְיָה", "N:N-M-P / N:N-F-P", "Noadiah", "no.ad.yah"),
        "H5130a": ("נוּף", "H:V", "to sprinkle", "nuph"),
        "H5130b": ("נוּף", "H:V", "to wave", "nuph"),
        "H5131": ("נוֹף", "H:N-M", "elevation", "noph"),
        "H5132a": ("נָצָה", "H:V", "to flee", ""),
        "H5132b": ("נָצַץ", "H:V", "to bud", ""),
        "H5133": ("נוֹצָה", "H:N-F", "plumage", "no.tsah"),
        "H5134": ("נוּק", "H:V", "to suckle", "nuq"),
        "H5135": ("נוּר", "A:N", "fire", "nur"),
        "H5136": ("נוּשׁ", "H:V", "be sick", "nush"),
        "H5137a": ("נָזָה", "H:V", "to sprinkle", "na.zah"),
        "H5137b": ("נָזָה", "H:V", "to spring", "na.zah"),
        "H5138": ("נָזִיד", "H:N-M", "stew", "na.zid"),
        "H5139": ("נָזִיר", "N:N--TG", "Nazirite", "na.zir"),
        "H5140": ("נָזַל", "H:V", "to flow", "na.zal"),
        "H5141": ("נֶ֫זֶם", "H:N-M", "ring", "ne.zem"),
        "H5142": ("נְזַק", "A:V", "to suffer injury", "ne.zaq"),
        "H5143": ("נֶ֫זֶק", "H:N-M", "injury", "ne.zeq"),
        "H5144a": ("נָזַר", "H:V", "to dedicate", "na.zar"),
        "H5144b": ("נָזַר", "H:V", "be a Nazarite", "na.zar"),
        "H5145": ("נֵ֫זֶר", "H:N-M", "consecration", "ne.zer"),
        "H5146": ("נֹחַ", "N:N-M-P", "Noah", "no.ach"),
        "H5147": ("נַחְבִּי", "N:N-M-P", "Nahbi", "nach.bi"),
        "H5148": ("נָחָה", "H:V", "to lead", "na.chah"),
        "H5149": ("נְחוּם", "N:N-M-P", "Nehum", "ne.chum"),
        "H5150": ("נִחוּם", "H:N-M", "comfort", "ni.chum"),
        "H5151": ("נַחוּם", "N:N-M-P", "Nahum", "na.chum"),
        "H5152": ("נָחוֹר", "N:N-M-P", "Nahor", "na.chor"),
        "H5153": ("נָחוּשׁ", "H:A", "bronze", "na.chush"),
        "H5154": ("נְחוּשָׁה", "H:N-F", "bronze", "ne.chu.shah"),
        "H5155": ("נְחִילָה", "H:N-F", "flute", "ne.chi.lah"),
        "H5156": ("נָחִיר", "H:N-M", "nostril", "ne.chir"),
        "H5157": ("נָחַל", "H:V", "to inherit", "na.chal"),
        "H5158a": ("נַ֫חַל", "H:N-M", "torrent", "na.chal"),
        "H5158b": ("נַ֫חַל", "H:N-M", "palm-tree", "na.chal"),
        "H5159": ("נַחֲלָה", "H:N-F", "inheritance", "na.cha.lah"),
        "H5160": ("נַחֲלִיאֵל", "N:N--L", "Nahaliel", "na.cha.li.el"),
        "H5161": ("נֶחֱלָמִי", "N:N--LG", "Nehelamite", "ne.che.la.mi"),
        "H5162": ("נָחַם", "H:V", "to comfort", "na.cham"),
        "H5163": ("נַ֫חַם", "N:N-M-P", "Naham", "na.cham"),
        "H5164": ("נֹ֫חַם", "H:N-M", "repentance", "no.cham"),
        "H5165": ("נֶחָמָה", "H:N-F", "comfort", "ne.cha.mah"),
        "H5166": ("נְחֶמְיָה", "N:N-M-P", "Nehemiah", "ne.chem.yah"),
        "H5167": ("נַחֲמָנִי", "N:N-M-P", "Nahamani", "na.cha.ma.ni"),
        "H5168": ("נַ֫חְנוּ", "H:PerP-CP", "we", "nach.nu"),
        "H5169": ("נָחַץ", "H:V", "to urge", "na.chats"),
        "H5170a": ("נַ֫חַר", "H:N-M", "snorting", "na.char"),
        "H5170b": ("נַחֲרָה", "H:N-M", "snorting", ""),
        "H5171": ("נַחֲרַי", "N:N-M-P", "Naharai", "na.cha.ray"),
        "H5172": ("נָחַשׁ", "H:V", "to divine", "na.chash"),
        "H5173": ("נַ֫חַשׁ", "H:N-M", "divination", "na.chash"),
        "H5174": ("נְחָשׁ", "A:N-M", "bronze", "ne.chash"),
        "H5175": ("נָחָשׁ", "H:N-M", "serpent", "na.chash"),
        "H5176": ("נָחָשׁ", "N:N-M-P", "Nahash", "na.chash"),
        "H5177": ("נַחְשׁוֹן", "N:N-M-P", "Nahshon", "nach.shon"),
        "H5178a": ("נְחֹ֫שֶׁת", "H:N-M", "bronze", "ne.cho.shet"),
        "H5178b": ("נְחֹ֫שֶׁת", "H:N-M", "lust", "ne.cho.shet"),
        "H5179": ("נְחֻשְׁתָּא", "N:N-F-P", "Nehushta", "ne.chush.ta"),
        "H5180": ("נְחֻשְׁתָּן", "N:N-M-T", "Nehushtan", "ne.chush.tan"),
        "H5181": ("נָחֵת", "H:V", "to descend", "na.chat"),
        "H5182": ("נְחֵת", "A:V", "to descend", "ne.chat"),
        "H5183a": ("נַ֫חַת", "H:N", "quietness", "na.chat"),
        "H5183b": ("נַ֫חַת", "H:N", "descent", "na.chat"),
        "H5184": ("נַ֫חַת", "N:N-M-P", "Nahath", "na.chat"),
        "H5185": ("נָחֵת", "H:A", "descended", "na.chet"),
        "H5186": ("נָטָה", "H:V", "to stretch", "na.tah"),
        "H5187": ("נָטִיל", "H:A", "laden", "ne.til"),
        "H5188": ("נְטִיפָה", "H:N-F", "pendant", "ne.ti.phah"),
        "H5189": ("נְטִישָׁה", "H:N-F", "tendril", "ne.ti.shah"),
        "H5190": ("נָטַל", "H:V", "to lift", "na.tal"),
        "H5191": ("נְטַל", "A:V", "to lift", "ne.tal"),
        "H5192": ("נֵ֫טֶל", "H:N-M", "weight", "ne.tel"),
        "H5193": ("נָטַע", "H:V", "to plant", "na.ta"),
        "H5194": ("נֶ֫טַע", "H:N-M", "plantation", "ne.ta"),
        "H5195": ("נָטִיעַ", "H:N-M", "plant", "na.ti.a"),
        "H5196": ("נְטָעִים", "N:N--L", "Netaim", "ne.ta.im"),
        "H5197": ("נָטַף", "H:V", "to drip/prophesy", "na.taph"),
        "H5198a": ("נָטָף", "H:N-M", "drop", "na.taph"),
        "H5198b": ("נָטָף", "H:N-M", "gum", "na.taph"),
        "H5199": ("נְטֹפָה", "N:N--L", "Netophah", "ne.to.phah"),
        "H5200": ("נְטֹפָתִי", "N:N--LG", "Netophathite", "ne.to.pha.ti"),
        "H5201": ("נָטַר", "H:V", "to keep", "na.tar"),
        "H5202": ("נְטַר", "A:V", "to keep", "ne.tar"),
        "H5203": ("נָטַשׁ", "H:V", "to leave", "na.tash"),
        "H5204": ("נִי", "H:N-M", "wailing", "ni"),
        "H5205": ("נִיד", "H:N-M", "solace", "nid"),
        "H5206": ("נִידָה", "H:N-F", "filth", "ni.dah"),
        "H5207": ("נִיחֹחַ", "H:N-M", "soothing", "ni.cho.ach"),
        "H5208": ("נִיחוֹחַ", "A:N-M", "soothing", "ni.cho.ach"),
        "H5209": ("נִין", "H:N-M", "offspring", "nin"),
        "H5210": ("נִינְוֵה", "N:N--L", "Nineveh", "nin.veh"),
        "H5211": ("נִיס", "H:N-M", "fugitive", "nis"),
        "H5212": ("נִיסָן", "N:N--T", "Nisan", "ni.san"),
        "H5213": ("נִיצוֹץ", "H:N-M", "spark", "ni.tsots"),
        "H5214": ("נִיר", "H:V", "to break", "nir"),
        "H5215": ("נִיר", "H:N-M", "fallow ground", "nir"),
        "H5216a": ("נֵר", "H:N-M", "lamp", "nir"),
        "H5216b": ("נִיר", "H:N-M", "lamp", "nir"),
        "H5217": ("נָכָא", "H:V", "to whip", "na.kah"),
        "H5218a": ("נָכָא", "H:A", "stricken", "na.khe"),
        "H5218b": ("נָכֵא", "H:A", "stricken", "na.khe"),
        "H5219": ("נְכֹאת", "H:N-F", "tragacanth gum", "ne.khot"),
        "H5220": ("נֶ֫כֶד", "H:N-M", "progeny", "ne.khed"),
        "H5221": ("נָכָה", "H:V", "to smite", "na.khah"),
        "H5222": ("נֵכֶה", "H:A", "smitten", "ne.kheh"),
        "H5223": ("נָכֵה", "H:A", "crippled", "na.kheh"),
        "H5224": ("נְכוֹ", "N:N-M-P", "Neco", "ne.kho"),
        "H5225": ("נָכוֹן", "N:N-M-P", "Nacon", "na.khon"),
        "H5226": ("נֵכַח", "H:Adv", "before", "ne.khach"),
        "H5227": ("נֹ֫כַח", "H:Adv", "before", "no.khach"),
        "H5228": ("נָכֹחַ", "H:A", "straightforward", "na.kho.ach"),
        "H5229": ("נְכֹחָה", "H:A-F", "upright", "ne.kho.chah"),
        "H5230": ("נָכַל", "H:V", "to plot", "na.khal"),
        "H5231": ("נֵ֫כֶל", "H:N-M", "trick", "ne.khel"),
        "H5232": ("נְכַס", "A:N-M", "wealth", "ne.khas"),
        "H5233": ("נֶ֫כֶס", "H:N-M", "wealth", "ne.khes"),
        "H5234a": ("נָכַר", "H:V", "to recognize", "na.khar"),
        "H5234b": ("נָכַר", "H:V", "to alienate", "na.khar"),
        "H5235": ("נֶ֫כֶר", "H:N-M", "misfortune", "ne.kher"),
        "H5236": ("נֵכָר", "H:N-M", "foreign", "ne.khar"),
        "H5237": ("נׇכְרִי", "H:A", "foreign", "nokh.ri"),
        "H5238": ("נְכֹת", "H:N-F", "treasure", "ne.khot"),
        "H5239": ("נָלָה", "H:V", "to cease", "na.lah"),
        "H5240": ("נְמִבְזֶה", "H:A", "vile", "ne.miv.zeh"),
        "H5241": ("נְמוּאֵל", "N:N-M-P", "Nemuel", "ne.mu.el"),
        "H5242": ("נְמוּאֵלִי", "N:N--PG", "Nemuelite", "ne.mu.e.li"),
        "H5243a": ("מָלַל", "H:V", "to languish", "na.mal"),
        "H5243b": ("נָמַל", "H:V", "to cut", "na.mal"),
        "H5244": ("נְמָלָה", "H:N-F", "ant", "ne.ma.lah"),
        "H5245": ("נְמַר", "A:N-M", "leopard", "ne.mar"),
        "H5246": ("נָמֵר", "H:N-M", "leopard", "na.mer"),
        "H5247": ("נִמְרָה", "N:N--L", "Nimrah", "nim.rah"),
        "H5248": ("נִמְרֹד", "N:N-M-P", "Nimrod", "nim.rod"),
        "H5249": ("נִמְרִים", "N:N--L", "Nimrim", "nim.rim"),
        "H5250": ("נִמְשִׁי", "N:N-M-P", "Nimshi", "nim.shi"),
        "H5251": ("נֵס", "H:N-M", "ensign", "nes"),
        "H5252": ("נְסִבָּה", "H:N-F", "turn", "ne.sib.bah"),
        "H5253": ("נָסַג", "H:V", "to remove", "na.sag"),
        "H5254": ("נָסָה", "H:V", "to test", "na.sah"),
        "H5255": ("נָסַח", "H:V", "to pull", "na.sach"),
        "H5256": ("נְסַח", "A:V", "to pull away", "ne.sach"),
        "H5257a": ("נָסִיךְ", "H:N-M", "libation", "ne.sikh"),
        "H5257b": ("נָסִיךְ", "H:N-M", "prince", "ne.sikh"),
        "H5258a": ("נָסַךְ", "H:V", "to pour", "na.sakh"),
        "H5258b": ("נָסַךְ", "H:V", "to install", "na.sakh"),
        "H5259": ("נָסַךְ", "H:V", "to weave", "na.sakh"),
        "H5260": ("נְסַךְ", "A:V", "to pour", "ne.sakh"),
        "H5261": ("נְסַךְ", "A:N-M", "drink offering", "ne.sakh"),
        "H5262": ("נֶ֫סֶךְ", "H:N-M", "drink offering", "ne.sekh"),
        "H5263": ("נָסַס", "H:V", "to faint", "na.sas"),
        "H5264": ("נָסַס", "H:V", "to shine", "na.sas"),
        "H5265": ("נָסַע", "H:V", "to set out", "na.sa"),
        "H5266": ("סָלֵק", "H:V", "to ascend", "na.saq"),
        "H5267": ("נְסַק", "A:V", "to take up", "ne.saq"),
        "H5268": ("נִסְרֹךְ", "N:N-M-T", "Nisroch", "nis.rokh"),
        "H5269": ("נֵעָה", "N:N--L", "Neah", "ne.ah"),
        "H5270": ("נֹעָה", "N:N-F-P", "Noah", "no.ah"),
        "H5271a": ("נְעוּרִים", "H:N-M/F", "youth", "na.ur"),
        "H5271b": ("נְעוּרוֹת", "H:N-M/F", "youth", "na.ur"),
        "H5272": ("נְעִיאֵל", "N:N--L", "Neiel", "ne.i.el"),
        "H5273a": ("נָעִים", "H:A", "pleasant", "na.im"),
        "H5273b": ("נָעִים", "H:A", "musical", "na.im"),
        "H5274a": ("נָעַל", "H:V", "to lock", "na.al"),
        "H5274b": ("נָעַל", "H:V", "to shoe", "na.al"),
        "H5275": ("נַ֫עַל", "H:N-F", "sandal", "na.al"),
        "H5276": ("נָעֵם", "H:V", "be pleasant", "na.em"),
        "H5277": ("נָ֑עַם", "N:N-M-P", "Naam", "na.am"),
        "H5278": ("נֹ֫עַם", "H:N-M", "pleasantness", "no.am"),
        "H5279a": ("נַעֲמָה", "N:N-F-P", "Naamah", "na.a.mah"),
        "H5279b": ("נַעֲמָה", "N:N--L", "Naamah", "na.a.mah"),
        "H5280": ("נַעֲמִי", "N:N--PG", "Naamite", "na.a.mi"),
        "H5281": ("נׇעֳמִי", "N:N-F-P", "Naomi", "no.o.mi"),
        "H5282": ("נַעֲמָן", "H:N-M", "pleasantness", "na.a.man"),
        "H5283": ("נַעֲמָן", "N:N-M-P", "Naaman", "na.a.man"),
        "H5284": ("נַעֲמָתִי", "N:N--LG", "Naamathite", "na.a.ma.ti"),
        "H5285": ("נַעֲצוּץ", "H:N-M", "thorn bush", "na.a.tsuts"),
        "H5286": ("נָעַר", "H:V", "to growl", "na.ar"),
        "H5287": ("נָעַר", "H:V", "to shake", "na.ar"),
        "H5288": ("נַ֫עַר", "H:N-M", "youth", "na.ar"),
        "H5289": ("נַ֫עַר", "H:N-M", "newborn", "na.ar"),
        "H5290": ("נֹ֫עַר", "H:N-M", "youth", "no.ar"),
        "H5291": ("נַעֲרָה", "H:N-F", "maiden", "na.a.rah"),
        "H5292a": ("נַעֲרָה", "N:N--L", "Naarah", "na.a.rah"),
        "H5292b": ("נַעֲרָה", "N:N-F-P", "Naarah", "na.a.rah"),
        "H5293": ("נַעֲרַי", "N:N-M-P", "Naarai", "na.a.ray"),
        "H5294": ("נְעַרְיָה", "N:N-M-P", "Neariah", "ne.ar.yah"),
        "H5295": ("נַעֲרָן", "N:N--L", "Naaran", "na.a.ran"),
        "H5296": ("נְעֹ֫רֶת", "H:N-F", "tow", "ne.o.ret"),
        "H5297": ("נֹף", "N:N--L", "Memphis", "noph"),
        "H5298": ("נֶ֫פֶג", "N:N-M-P", "Nepheg", "ne.pheg"),
        "H5299a": ("נָפָה", "H:N-F", "sieve", "na.phah"),
        "H5299b": ("נָפָה", "H:N-F", "height", "na.phah"),
        "H5300": ("נְפִישְׁסִים, נפיסים", "N:N--T", "Nephisesim", "ne.phush.sim"),
        "H5301": ("נָפַח", "H:V", "to breathe", "na.phach"),
        "H5302": ("נֹ֫פַח", "N:N--L", "Nophah", "no.phach"),
        "H5303": ("נְפִלִים", "N:N--TG", "Nephilim", "ne.phil"),
        "H5304": ("נְפִיסִים", "N:N--TG", "Nephusim", "ne.phi.sim"),
        "H5305": ("נָפִישׁ", "N:N-M-P", "Naphish", "na.phish"),
        "H5306": ("נֹ֫פֶךְ", "H:N-M", "emerald", "no.phekh"),
        "H5307": ("נָפַל", "H:V", "to fall", "na.phal"),
        "H5308": ("נְפַל", "A:V", "to fall", "ne.phal"),
        "H5309": ("נֵ֫פֶל", "H:N-M", "miscarriage", "ne.phel"),
        "H5310a": ("נָפַץ", "H:V", "to shatter", "na.phats"),
        "H5310b": ("נָפַץ", "H:V", "to disperse", "na.phats"),
        "H5311": ("נֶ֫פֶץ", "H:N-M", "storm", "ne.phets"),
        "H5312": ("נְפַק", "A:V", "to go out", "ne.phaq"),
        "H5313": ("נִפְקָא", "A:N-F", "cost", "niph.qa"),
        "H5314": ("נָפַשׁ", "H:V", "be refreshed", "na.phash"),
        "H5315": ("נֶ֫פֶשׁ", "H:N-F", "soul", "ne.phesh"),
        "H5316": ("נֶ֫פֶת", "N:N--L", "Naphath", "ne.phet"),
        "H5317": ("נֹ֫פֶת", "H:N-M", "honey", "no.phet"),
        "H5318": ("נֶפְתּוֹחַ", "N:N--L", "Nephtoah", "neph.to.ach"),
        "H5319": ("נַפְתּוּלִים", "H:N-M", "wrestling", "naph.tul"),
        "H5320": ("נַפְתֻּחִים", "N:N--LG", "Naphtuhites", "naph.tu.chim"),
        "H5321": ("נַפְתָּלִי", "N:N-M-P", "Naphtali", "naph.ta.li"),
        "H5322a": ("נֵץ", "H:N-M", "flower", "nets"),
        "H5322b": ("נֵץ", "H:N-M", "hawk", "nets"),
        "H5323": ("נָצָא", "H:V", "to flee", "na.tsa"),
        "H5324": ("נָצַב", "H:V", "to stand", "na.tsav"),
        "H5325": ("נִצָּב", "H:N-M", "hilt", "nits.tsav"),
        "H5326": ("נִצְבָּה", "A:N-F", "firmness", "nits.bah"),
        "H5327a": ("נָצָה", "H:V", "to struggle", "na.tsah"),
        "H5327b": ("נָצָה", "H:V", "to desolate", "na.tsah"),
        "H5328": ("נִצָּה", "H:N-F", "flower", "nits.tsah"),
        "H5329": ("נָצַח", "H:V", "to conduct", "na.tsach"),
        "H5330": ("נְצַח", "A:V", "to distinguish oneself", "ne.tsach"),
        "H5331": ("נֵ֫צַח", "H:N-M", "perpetuity", "ne.tsach"),
        "H5332": ("נֵ֫צַח", "H:N-M", "lifeblood", "ne.tsach"),
        "H5333": ("נְצִיב", "H:N-M", "garrison", "ne.tsiv"),
        "H5334": ("נְצִיב", "N:N--L", "Nezib", "ne.tsiv"),
        "H5335": ("נְצִיחַ", "N:N-M-P", "Neziah", "ne.tsi.ach"),
        "H5336": ("נציר", "H:A", "preserved", "na.tsir"),
        "H5337": ("נָצַל", "H:V", "to rescue", "na.tsal"),
        "H5338": ("נְצַל", "A:V", "to rescue", "ne.tsal"),
        "H5339": ("נִצָּן", "H:N-M", "flower", "nits.tsan"),
        "H5340": ("נָצַץ", "H:V", "to shine", "na.tsats"),
        "H5341": ("נָצַר", "H:V", "to watch", "na.tsar"),
        "H5342": ("נֵ֫צֶר", "H:N-M", "branch", "ne.tser"),
        "H5343": ("נְקֵא", "A:A", "pure", "ne.qe"),
        "H5344a": ("נָקַב", "H:V", "to pierce", "na.qav"),
        "H5344b": ("נָקַב", "H:V", "to curse", "na.qav"),
        "H5345": ("נֶ֫קֶב", "H:N-M", "socket", "ne.qev"),
        "H5346": ("נֶ֫קֶב", "N:N--L", "Nekeb", "ne.qev"),
        "H5347": ("נְקֵבָה", "H:N-F", "female", "ne.qe.vah"),
        "H5348": ("נָקֹד", "H:A", "speckled", "na.qod"),
        "H5349": ("נֹקֵד", "H:N-M", "shepherd", "no.qed"),
        "H5350": ("נִקּוּד", "H:N-M", "crumb", "niq.qud"),
        "H5351": ("נְקֻדָּה", "H:N-F", "bead", "ne.qud.dah"),
        "H5352": ("נָקָה", "H:V", "to clear", "na.qah"),
        "H5353": ("נְקוֹדָא", "N:N-M-P", "Nekoda", "ne.qo.da"),
        "H5354": ("נָקַט", "H:V", "to tire", "na.qat"),
        "H5355a": ("נָקִי", "H:A", "innocent", "na.qi"),
        "H5355b": ("נָקִיא", "H:A", "innocent", ""),
        "H5356a": ("נִקָּיוֹן", "H:N-M", "innocence", "niq.qa.von"),
        "H5356b": ("קֵהָיוֹן", "H:N-M", "bluntness", ""),
        "H5357": ("נָקִיק", "H:N-M", "cleft", "na.qiq"),
        "H5358": ("נָקַם", "H:V", "to avenge", "na.qam"),
        "H5359": ("נָקָם", "H:N-M", "vengeance", "na.qam"),
        "H5360": ("נְקָמָה", "H:N-F", "vengeance", "ne.qa.mah"),
        "H5361": ("נָקַע", "H:V", "be alienated", "na.qa"),
        "H5362a": ("נָקַף", "H:V", "to strike", "na.qaph"),
        "H5362b": ("נָקַף", "H:V", "to surround", "na.qaph"),
        "H5363": ("נֹ֫קֶף", "H:N-M", "shaking", "no.qeph"),
        "H5364": ("נִקְפָּה", "H:N-F", "rope", "niq.pah"),
        "H5365": ("נָקַר", "H:V", "to dig", "na.qar"),
        "H5366": ("נְקָרָה", "H:N-F", "crevice", "ne.qa.rah"),
        "H5367": ("נָקַשׁ", "H:V", "to snare", "na.qash"),
        "H5368": ("נְקַשׁ", "A:V", "to knock", "ne.qash"),
        "H5369": ("נֵר", "N:N-M-P", "Ner", "ner"),
        "H5370": ("נֵרְגָּ֑ל", "N:N-M-T", "Nergal", "ne.re.gal"),
        "H5371": ("נֵרְגַּל שַׁרְאֶ֫צֶר", "N:N-M-P", "Nergal-sar-ezer", "ne.re.gal shar.e.tser"),
        "H5372": ("נִרְגָּן", "H:V", "whisperer", "nir.gan"),
        "H5373": ("נֵרְדְּ", "H:N-M", "nard", "ne.red"),
        "H5374": ("נֵרִיָהוּ", "N:N-M-P", "Neriah", "ne.riy.yah"),
        "H5375": ("נָשָׂא", "H:V", "to lift", "na.sa"),
        "H5376": ("נְשָׂא", "A:V", "to carry", "ne.sa"),
        "H5377": ("נָשָׁא", "H:V", "to deceive", "na.sah"),
        "H5378": ("נָשָׁא", "H:V", "to exact", "na.sah"),
        "H5379": ("נִשֵּׂאת", "H:N-F", "gift", "nis.set"),
        "H5380": ("נָשַׁב", "H:V", "to blow", "na.shav"),
        "H5381": ("נָשַׂג", "H:V", "to overtake", "na.sag"),
        "H5382": ("נָשָׁה", "H:V", "to forget", "na.shah"),
        "H5383": ("נָשָׁה", "H:V", "to lend", "na.shah"),
        "H5384": ("נָשֶׁה", "H:N-M", "hamstring", "na.sheh"),
        "H5385": ("נְשׂוּאָה", "H:N-F", "burden", "ne.su.ah"),
        "H5386": ("נְשִׁי", "H:N-M", "debt", "ne.shi"),
        "H5387a": ("נָשִׂיא", "H:N-M", "leader", "na.si"),
        "H5387b": ("נָשִׂיא", "H:N-M", "mist", "na.si"),
        "H5388": ("נְשִׁיָּה", "H:N-F", "forgetfulness", "ne.shiy.yah"),
        "H5389": ("נְשִׁין", "A:N-F", "wife", "na.shin"),
        "H5390": ("נְשִׁיקָה", "H:N-F", "kiss", "ne.shi.qah"),
        "H5391a": ("נָשַׁךְ", "H:V", "to bite", "na.shakh"),
        "H5391b": ("נָשַׁךְ", "H:V", "to pay interest", "na.shakh"),
        "H5392": ("נֶ֫שֶׁךְ", "H:N-M", "interest", "ne.shekh"),
        "H5393": ("נִשְׁכָּה", "H:N-F", "chamber", "nish.kah"),
        "H5394": ("נָשַׁל", "H:V", "to slip", "na.shal"),
        "H5395": ("נָשַׁם", "H:V", "to pant", "na.sham"),
        "H5396": ("נִשְׁמָא", "A:N-F", "breath", "nish.ma"),
        "H5397": ("נְשָׁמָה", "H:N-F", "breath", "ne.sha.mah"),
        "H5398": ("נָשַׁף", "H:V", "to blow", "na.shaph"),
        "H5399": ("נֶ֫שֶׁף", "H:N-M", "twilight", "ne.sheph"),
        "H5400": ("שָׂלַק", "H:V", "to kindle", "na.saq"),
        "H5401a": ("נָשַׁק", "H:V", "to kiss", "na.shaq"),
        "H5401b": ("נָשַׁק", "H:V", "to handle", "na.shaq"),
        "H5402": ("נֶ֫שֶׁק", "H:N-M", "weapon", "ne.sheq"),
        "H5403": ("נְשַׁר", "A:N-M", "eagle", "ne.shar"),
        "H5404": ("נֶשֶׁר", "H:N-M", "eagle", "ne.sher"),
        "H5405": ("נָשַׁת", "H:V", "be dry", "na.shat"),
        "H5406": ("נִשְׁתְּוָן", "H:N-M", "letter", "nish.te.van"),
        "H5407": ("נִשְׁתְּוָן", "A:N-M", "document", "nish.te.van"),
        "H5408": ("נָתַח", "H:V", "to cut", "na.tach"),
        "H5409": ("נֵ֫תַח", "H:N-M", "piece", "ne.tach"),
        "H5410a": ("נָתִיב", "H:N-M", "path", "na.tiv"),
        "H5410b": ("נְתִיבָה", "H:N-M", "path", ""),
        "H5411": ("נְתִינִים", "N:N--T", "temple servant", "na.tin"),
        "H5412": ("נְתִין", "A:N-M", "temple servant", "ne.tin"),
        "H5413": ("נָתַךְ", "H:V", "to pour", "na.takh"),
        "H5414": ("נָתַן", "H:V", "to give", "na.tan"),
        "H5415": ("נְתַן", "A:V", "to give", "ne.tan"),
        "H5416": ("נָתָן", "N:N-M-P", "Nathan", "na.tan"),
        "H5417": ("נְתַנְאֵל", "N:N-M-P", "Nethanel", "ne.tan.el"),
        "H5418": ("נְתַנְיָ֫הוּ", "N:N-M-P", "Nethaniah", "ne.tan.yah"),
        "H5419": ("נְתַן־מֶ֫לֶךְ", "N:N-M-P", "Nathan-melech", "ne.tan-me.lekh"),
        "H5420": ("נָתַס", "H:V", "to break", "na.tas"),
        "H5421": ("נָתַע", "H:V", "to break", "na.ta"),
        "H5422": ("נָתַץ", "H:V", "to tear", "na.tats"),
        "H5423": ("נָתַק", "H:V", "to tear", "na.taq"),
        "H5424": ("נֶ֫תֶק", "H:N-M", "scab", "ne.teq"),
        "H5425a": ("נָתַר", "H:V", "to start", "na.tar"),
        "H5425b": ("נָתַר", "H:V", "to free", "na.tar"),
        "H5426": ("נְתַר", "A:V", "to strip", "ne.tar"),
        "H5427": ("נֶ֫תֶר", "H:N-M", "lye", "ne.ter"),
        "H5428": ("נָתַשׁ", "H:V", "to uproot", "na.tash"),
        "H5429": ("סְאָה", "H:N-F", "seah", "se.ah"),
        "H5430": ("סְאוֹן", "H:N-M", "boot", "se.on"),
        "H5431": ("סָאַן", "H:V", "to tread", "sa.an"),
        "H5432": ("סֵאסֵא", "H:V", "to drive away", "sas.se.ah"),
        "H5433a": ("סָבָא", "H:V", "to imbibe", "sa.va"),
        "H5433b": ("סָבָא", "H:V", "be drunk", "sa.va"),
        "H5434": ("סְבָא", "N:N-M-P / N:N--L", "Seba", "se.va"),
        "H5435": ("סֹ֫בֶא", "H:N-M", "liquor", "so.ve"),
        "H5436": ("סְבָאִים", "N:N--LG", "Sabean", "se.va.i"),
        "H5437": ("סָבַב", "H:V", "to turn", "sa.vav"),
        "H5438": ("סִבָּה", "H:N-F", "turn", "sib.bah"),
        "H5439": ("סָבִיב", "H:Adv", "around", "sa.viv"),
        "H5440": ("סָבַךְ", "H:V", "to interweave", "sa.vakh"),
        "H5441": ("סְבֹךְ", "H:N-M", "thicket", "so.vekh"),
        "H5442": ("סְבַךְ", "H:N-M", "thicket", "se.vakh"),
        "H5443": ("שַׂבְּכָא", "A:N-F", "trigon", "sab.b.kah"),
        "H5444": ("סִבְּכַי", "N:N-M-P", "Sibbecai", "sib.b.khay"),
        "H5445": ("סָבַל", "H:V", "to bear", "sa.val"),
        "H5446": ("סְבַל", "A:V", "to bear", "se.val"),
        "H5447": ("סֵ֫בֶל", "H:N-M", "burden", "se.vel"),
        "H5448": ("סֹ֫בֶל", "H:N-M", "burden", "so.vel"),
        "H5449": ("סַבָּל", "H:N-M", "burden", "sab.bal"),
        "H5450": ("סִבְלָה", "H:N-F", "burden", "se.va.lah"),
        "H5451": ("סִבֹּלֶת", "H:N-F", "ear", "sib.bo.let"),
        "H5452": ("סְבַר", "A:V", "to intend", "se.var"),
        "H5453": ("סִבְרַ֫יִם", "N:N--L", "Sibraim", "siv.ra.yim"),
        "H5454": ("סַבְתָּה", "N:N-M-P", "Sabtah", "sav.ta"),
        "H5455": ("סַבְתְּכָא", "N:N-M-P", "Sabteca", "sav.te.kah"),
        "H5456": ("סָגַד", "H:V", "to prostrate", "sa.gad"),
        "H5457": ("סְגִ֑ד", "A:V", "to do homage", "se.gid"),
        "H5458": ("סְגוֹר", "H:N-M", "enclosure", "se.gor"),
        "H5459": ("סְגֻלָּה", "H:N-F", "possession", "se.gul.lah"),
        "H5460": ("סְגַן", "A:N-M", "prefect", "se.gan"),
        "H5461": ("סָגָן", "H:N-M", "ruler", "sa.gan"),
        "H5462": ("סָגַר", "H:V", "to shut", "sa.gar"),
        "H5463": ("סְגַר", "A:V", "to shut", "se.gar"),
        "H5464": ("סַגְרִיר", "H:N-M", "rain", "sag.rid"),
        "H5465": ("סַד", "H:N-M", "stock", "sad"),
        "H5466": ("סָדִין", "H:N-M", "linen", "sa.din"),
        "H5467": ("סְדֹם", "N:N--L", "Sodom", "se.dom"),
        "H5468": ("סֵ֫דֶר", "H:N-M", "order", "se.der"),
        "H5469": ("סַ֫הַר", "H:N-M", "roundness", "sa.har"),
        "H5470": ("סֹ֫הַר", "H:N-M", "prison", "so.har"),
        "H5471": ("סוֹא", "N:N-M-P", "So", "so"),
        "H5472": ("סוּג", "H:V", "to turn", "sug"),
        "H5473": ("סוּג", "H:V", "to fence", "sug"),
        "H5474": ("סוּגַר", "H:N-M", "cage", "su.gar"),
        "H5475": ("סוֹד", "H:N-M", "counsel", "sod"),
        "H5476": ("סוֹדִי", "N:N-M-P", "Sodi", "so.di"),
        "H5477": ("סוּחַ", "N:N-M-P", "Suah", "su.ach"),
        "H5478": ("סוּחָה", "H:N-F", "offal", "su.chah"),
        "H5479": ("סוֹטַי", "N:N-M-P", "Sotai", "so.tay"),
        "H5480": ("סוּךְ", "H:V", "to anoint", "sukh"),
        "H5481": ("סוּמְפּוֹנְיָה", "A:N-F", "bagpipe", "sum.po.ne.yah"),
        "H5482": ("סְוֵנֵה", "N:N--L", "Syene", "se.ve.neh"),
        "H5483a": ("סוּס", "H:N-M", "swallow", "sus"),
        "H5483b": ("סוּס", "H:N-M", "horse", "sus"),
        "H5484": ("סוּסָה", "H:N-F", "mare", "su.sah"),
        "H5485": ("סוּסִי", "N:N-M-P", "Susi", "su.si"),
        "H5486": ("סוּף", "H:V", "to cease", "suph"),
        "H5487": ("סוּף", "A:V", "be fulfilled", "suph"),
        "H5488": ("סוּף", "H:N-M / N:N--L", "reed", "suph"),
        "H5489": ("סוּף", "N:N--L", "Suph", "suph"),
        "H5490": ("סוֹף", "H:N-M", "end", "soph"),
        "H5491": ("סוֹף", "A:N-M", "end", "soph"),
        "H5492a": ("סוּפָה", "H:N-F", "whirlwind", "su.phah"),
        "H5492b": ("סוּפָה", "N:N--L", "Suphah", "su.phah"),
        "H5493": ("סוּר", "H:V", "to turn aside", "sur"),
        "H5494": ("סוּר", "H:N-M", "degenerate", "sur"),
        "H5495": ("סוּר", "N:N--T", "Sur", "sur"),
        "H5496": ("סוּת", "H:V", "to incite", "sut"),
        "H5497": ("סוּת", "H:N-M", "vesture", "sut"),
        "H5498": ("סָחַב", "H:V", "to drag", "sa.chav"),
        "H5499": ("סְחָבָה", "H:N-F", "rag", "se.cha.vah"),
        "H5500": ("סָחָה", "H:V", "to scrape", "sa.chah"),
        "H5501": ("סְחִי", "H:N-M", "offscouring", "se.chi"),
        "H5502": ("סָחַף", "H:V", "to prostatrate", "sa.chaph"),
        "H5503": ("סָחַר", "H:V", "to trade", "sa.char"),
        "H5504": ("סַחַר", "H:N-M", "profit", "sa.char"),
        "H5505": ("סָ֫חַר", "H:N-M", "merchandise", "sa.char"),
        "H5506": ("סְחֹרָה", "H:N-F", "merchandise", "se.cho.rah"),
        "H5507": ("סֹחֵרָה", "H:N-F", "buckler", "so.che.rah"),
        "H5508": ("סֹחֶ֫רֶת", "H:N-F", "stone", "so.che.ret"),
        "H5509": ("סִיג", "H:N-M", "dross", "sig"),
        "H5510": ("סִיוָן", "N:N--T", "Sivan", "si.van"),
        "H5511": ("סִיחוֹן", "N:N-M-P", "Sihon", "si.chon"),
        "H5512a": ("סִין", "N:N--L", "Sin", "sin"),
        "H5512b": ("סִין", "N:N--L", "Sin", "sin"),
        "H5513": ("סִינִי", "N:N--PG", "Sinite", "si.ni"),
        "H5514": ("סִינַי", "N:N--L", "Sinai", "si.nay"),
        "H5515": ("סִינִים", "N:N--LG", "Sinim", "si.nim"),
        "H5516": ("סִיסְרָא", "N:N-M-P", "Sisera", "sis.ra"),
        "H5517": ("סִיעָא", "N:N-M-P", "Sia", "si.a"),
        "H5518a": ("סִיר", "H:N-M", "pot", "sir"),
        "H5518b": ("סִיר", "H:N-M", "thorn", "sir"),
        "H5519": ("סַךְ", "H:N-M", "throng", "sakh"),
        "H5520": ("סֹךְ", "H:N-M", "lair", "sokh"),
        "H5521": ("סֻכָּה", "H:N-F", "booth", "suk.kah"),
        "H5522": ("סִכּוּת", "H:N-F", "tent", "sik.kut"),
        "H5523": ("סֻכּוֹת", "N:N--L", "Succoth", "suk.kot"),
        "H5524": ("סֻכּוֹת בְּנוֹת", "N:N-M-T", "Succoth-benoth", "suk.kot be.not"),
        "H5525": ("סֻכִּיִּים", "N:N--LG", "Sukkiim", "suk.ki"),
        "H5526a": ("סוּךְ", "H:V", "to fence", "sa.khakh"),
        "H5526b": ("סָכַךְ", "H:V", "to cover", "sa.khakh"),
        "H5526c": ("סָכַךְ", "H:V", "to weave", "sa.khakh"),
        "H5526d": ("סֹכֵךְ", "H:N-M", "protector", ""),
        "H5526e": ("שָׂכַךְ", "H:V", "to cover", ""),
        "H5526f": ("שָׂכַךְ", "H:V", "to weave", ""),
        "H5527": ("סְכָכָה", "N:N--L", "Secacah", "se.kha.khah"),
        "H5528": ("סָכַל", "H:V", "be foolish", "sa.khal"),
        "H5529": ("סֶ֫כֶל", "H:N-M", "folly", "se.khel"),
        "H5530": ("סָכָל", "H:N-M", "fool", "sa.khal"),
        "H5531a": ("שִׂכְלוּת", "H:N-F", "folly", "sikh.lut"),
        "H5531b": ("סִכְלוּת", "H:N-F", "folly", "sikh.lut"),
        "H5532": ("סָכַן", "H:V", "be useful", "sa.khan"),
        "H5533a": ("סָכַן", "H:V", "to endanger", "sa.khan"),
        "H5533b": ("סָכַן", "H:V", "to impoverish", "sa.khan"),
        "H5534": ("סָכַר", "H:V", "to stop", "sa.khar"),
        "H5535": ("סָכַת", "H:V", "be silent", "sa.khat"),
        "H5536": ("סַל", "H:N-M", "basket", "sal"),
        "H5537": ("סָלָא", "H:V", "to weigh", "sa.la"),
        "H5538": ("סִלָּא", "N:N--L", "Silla", "sil.la"),
        "H5539": ("סָלַד", "H:V", "to rejoice", "sa.lad"),
        "H5540": ("סֶ֫לֶד", "N:N-M-P", "Seled", "se.led"),
        "H5541a": ("סָלָה", "H:V", "to reject", "sa.lah"),
        "H5541b": ("סָלָה", "H:V", "to weigh", "sa.lah"),
        "H5542": ("סֶ֫לָה", "H:V", "Selah", "se.lah"),
        "H5543a": ("סַלָּ֑י", "N:N-M-P", "Sallai", "sal.lu"),
        "H5543b": ("סַלּוּ", "N:N-M-P", "Sallu", "sal.lu"),
        "H5543c": ("סַלוּא", "N:N-M-P", "Sallai", ""),
        "H5543d": ("סַלּוּא", "N:N-M-P", "Sallu", ""),
        "H5543e": ("סַלַּי", "N:N-M-P", "Sallai", ""),
        "H5544": ("סַלּוֹן", "H:N-M", "briar", "sil.lon"),
        "H5545": ("סָלַח", "H:V", "to forgive", "sa.lach"),
        "H5546": ("סַלָּח", "H:A", "forgiving", "sal.lach"),
        "H5547": ("סְלִיחָה", "H:N-F", "forgiveness", "se.li.chah"),
        "H5548": ("סַלְכָה", "N:N--L", "Salecah", "sal.khah"),
        "H5549": ("סָלַל", "H:V", "to build", "sa.lal"),
        "H5550": ("סֹלְלָה", "H:N-F", "mound", "so.le.lah"),
        "H5551": ("סֻלָּם", "H:N-M", "ladder", "sul.lam"),
        "H5552": ("סַלְסִלָּה", "H:N-F", "branch", "sal.sil.lah"),
        "H5553": ("סֶ֫לַע", "H:N-M", "crag", "se.la"),
        "H5554": ("סֶ֫לַע", "N:N--L", "Sela", "se.la"),
        "H5555": ("סֶלַע הַמַּחְלְקוֹת", "N:N--L", "Sela-hammahlekoth", "se.la ham.mach.le.qot"),
        "H5556": ("סׇלְעָם", "H:N-M", "locust", "sol.am"),
        "H5557": ("סָלַף", "H:V", "to pervert", "sa.laph"),
        "H5558": ("סֶ֫לֶף", "H:N-M", "crookedness", "se.leph"),
        "H5559": ("סְלֵק", "A:V", "to ascend", "se.liq"),
        "H5560": ("סֹ֫לֶת", "H:N-F", "fine flour", "so.let"),
        "H5561": ("סַם", "H:N-M", "spice", "sam"),
        "H5562": ("סַמְגַּר נְבוֹ", "N:N-M-P", "Samgar-nebo", "sam.gar ne.vo"),
        "H5563": ("סְמַדֵר", "H:N-M", "blossom", "se.ma.dar"),
        "H5564": ("סָמַךְ", "H:V", "to support", "sa.makh"),
        "H5565": ("סְמַכְיָהוּ", "N:N-M-P", "Semachiah", "se.makh.ya.hu"),
        "H5566": ("סֶ֫מֶל", "H:N-M", "idol", "se.mel"),
        "H5567": ("סָמַן", "H:V", "to appoint", "sa.man"),
        "H5568": ("סָמַר", "H:V", "to bristle up", "sa.mar"),
        "H5569": ("סָמָר", "H:A", "rough", "sa.mar"),
        "H5570": ("סְנָאָה", "N:N-M-P", "Senaah", "se.na.ah"),
        "H5571": ("סַנְבַלַּט", "N:N-M-P", "Sanballat", "san.val.lat"),
        "H5572": ("סְנֶה", "H:N-M", "bush", "se.neh"),
        "H5573": ("סֶ֑נֶּה", "N:N--L", "Seneh", "se.neh"),
        "H5574": ("סְנוּאָה", "N:N-M-P", "Hassenuah", "se.nu.ah"),
        "H5575": ("סַנְוֵרִים", "H:N-M", "blindness", "san.ver"),
        "H5576": ("סַנְחֵרִיב", "N:N-M-P", "Sennacherib", "san.che.riv"),
        "H5577": ("סַנְסִנִּים", "H:N-M", "fruit-stalk", "san.sin"),
        "H5578": ("סַנְסַנָּה", "N:N--L", "Sansannah", "san.san.nah"),
        "H5579": ("סְנַפִּיר", "H:N-M", "fin", "se.nap.pir"),
        "H5580": ("סָ֑ס", "H:N-M", "moth", "sas"),
        "H5581": ("סִסְמַי", "N:N-M-P", "Sismai", "sis.may"),
        "H5582": ("סָעַד", "H:V", "to support", "sa.ad"),
        "H5583": ("סְעַד", "A:V", "to support", "se.ad"),
        "H5584": ("סָעָה", "H:V", "to rush", "sa.ah"),
        "H5585": ("סָעִיף", "H:N-M", "cleft", "sa.iph"),
        "H5586": ("סָעַף", "H:V", "to lop", "sa.aph"),
        "H5587a": ("סְעִפָּה", "H:N-F", "division", "sa.iph"),
        "H5587b": ("שְׂעִפִּים", "H:N-F", "disquietings", ""),
        "H5588": ("סֵעֵף", "H:A", "divided", "se.eph"),
        "H5589": ("סְעַפָּה", "H:N-F", "bough", "se.ap.pah"),
        "H5590": ("סָעַר", "H:V", "to rage", "sa.ar"),
        "H5591a": ("סַ֫עַר", "H:N-M", "tempest", "sa.ar"),
        "H5591b": ("סְעָרָה", "H:N-M", "tempest", ""),
        "H5592a": ("סַף", "H:N-M", "basin", "saph"),
        "H5592b": ("סַף", "H:N-M", "threshold", "saph"),
        "H5593": ("סַף", "N:N-M-P", "Saph", "saph"),
        "H5594": ("סָפַד", "H:V", "to mourn", "sa.phad"),
        "H5595": ("סָפָה", "H:V", "to snatch", "sa.phah"),
        "H5596a": ("סָפַח", "H:V", "to attach", "sa.phach"),
        "H5596b": ("שִׂפַּח", "H:V", "to scar", ""),
        "H5597": ("סַפַּ֫חַת", "H:N-F", "scab", "sap.pa.chat"),
        "H5598": ("סִפַּי", "N:N-M-P", "Sippai", "sip.pay"),
        "H5599a": ("סָפִיחַ", "H:N-M", "outpouring", "sa.phi.ach"),
        "H5599b": ("סָפִיחַ", "H:N-M", "aftergrowth", "sa.phi.ach"),
        "H5600": ("סְפִינָה", "H:N-F", "ship", "se.phi.nah"),
        "H5601": ("סַפִּיר", "H:N-M", "sapphire", "sap.pir"),
        "H5602": ("סֵ֫פֶל", "H:N-M", "bowl", "se.phel"),
        "H5603": ("סָפַן", "H:V", "to cover", "sa.phan"),
        "H5604": ("סִפֻּן", "H:N-M", "ceiling", "sip.pun"),
        "H5605": ("סָפַף", "H:V", "to stand", "sa.phaph"),
        "H5606a": ("סָפַק", "H:V", "to slap", "sa.phaq"),
        "H5606b": ("שָׂפַק", "H:V", "to suffice", ""),
        "H5607a": ("שֶׂ֫פֶק", "H:N-M", "mockery", "se.pheq"),
        "H5607b": ("שֵׂ֫פֶק", "H:N-M", "sufficiency", "se.pheq"),
        "H5608a": ("סָפַר", "H:V", "to recount", "sa.phar"),
        "H5608b": ("סֹפֵר", "H:N-M", "secretary", ""),
        "H5609": ("סְפַר", "A:N-M", "book", "se.phar"),
        "H5610": ("סְפָר", "H:N-M", "numbering", "se.phar"),
        "H5611": ("סְפָר", "N:N--L", "Sephar", "se.phar"),
        "H5612a": ("סֵ֫פֶר", "H:N-M", "scroll", "se.pher"),
        "H5612b": ("סִפְרָה", "H:N-F", "book", ""),
        "H5613": ("סָפִר", "A:N-M", "scribe", "sa.pher"),
        "H5614": ("סְפָרַ֑ד", "N:N--L", "Sepharad", "se.pha.rad"),
        "H5615": ("סְפֹרָה", "H:N-F", "number", "se.pho.rah"),
        "H5616": ("סְפַרְוִי", "N:N--LG", "Sepharvite", "se.phar.vi"),
        "H5617": ("סְפַרְוַ֫יִם", "N:N--L", "Sepharvaim", "se.phar.va.yim"),
        "H5618": ("סֹפֶ֫רֶת", "N:N-M-P", "Hassophereth", "so.phe.ret"),
        "H5619": ("סָקַל", "H:V", "to stone", "sa.qal"),
        "H5620": ("סַר", "H:A", "stubborn", "sar"),
        "H5621": ("סָרָב", "H:N-M", "thorn", "sa.rav"),
        "H5622": ("סַרְבָּל", "A:N-M", "mantle", "sar.bal"),
        "H5623": ("סַרְגוֹן", "N:N-M-P", "Sargon", "sar.gon"),
        "H5624": ("סֶ֫רֶד", "N:N-M-P", "Sered", "se.red"),
        "H5625": ("סַרְדִּי", "N:N--PG", "Seredite", "sar.di"),
        "H5626": ("סִּרָה", "N:N--L", "Sirah", "si.rah"),
        "H5627": ("סָרָה", "H:N-F", "revolt", "sa.rah"),
        "H5628": ("סָרַח", "H:V", "to overrun", "sa.rach"),
        "H5629": ("סֶ֫רַח", "H:N-M", "excess", "se.rach"),
        "H5630": ("סִרְיֹן", "H:N-M", "armor", "sir.yon"),
        "H5631": ("סָרִיס", "H:N-M", "eunuch", "sa.ris"),
        "H5632": ("סָרֵךְ", "A:N-M", "chief", "sa.rekh"),
        "H5633a": ("סֶ֫רֶן", "H:N-M", "lord", "se.ren"),
        "H5633b": ("סֶ֫רֶן", "H:N-M", "axle", "se.ren"),
        "H5634": ("סַרְעַפָּה", "H:N-F", "bough", "sar.ap.pah"),
        "H5635": ("סָרַף", "H:V", "to burn", "sa.raph"),
        "H5636": ("סִרְפָּד", "H:N-M", "nettle", "sar.pad"),
        "H5637": ("סָרַר", "H:V", "to rebel", "sa.rar"),
        "H5638": ("סְתָיו", "H:N-M", "winter", "se.tav"),
        "H5639": ("סְתוּר", "N:N-M-P", "Sethur", "se.tur"),
        "H5640a": ("סָתַם", "H:V", "to close", "sa.tam"),
        "H5640b": ("שָׂתַם", "H:V", "to stopper", ""),
        "H5641": ("סָתַר", "H:V", "to hide", "sa.tar"),
        "H5642a": ("סְתַר", "A:V", "to hide", "se.tar"),
        "H5642b": ("סְתַר", "A:V", "to destroy", "se.tar"),
        "H5643a": ("סֵ֫תֶר", "H:N-M", "secrecy", "se.ter"),
        "H5643b": ("סִתְרָה", "H:N-F", "shelter", ""),
        "H5644": ("סִתְרִי", "N:N-M-P", "Sithri", "sit.ri"),
        "H5645": ("עָב", "H:N-M", "cloud", "av"),
        "H5646": ("עָב", "H:N-M", "threshold", "av"),
        "H5647": ("עָבַד", "H:V", "to serve", "a.vad"),
        "H5648": ("עֲבַד", "A:V", "to make", "a.vad"),
        "H5649": ("עֲבַד", "A:N-M", "servant/slave", "a.vad"),
        "H5650": ("עֶ֫בֶד", "H:N-M", "servant/slave", "e.ved"),
        "H5651": ("עֶ֫בֶד", "N:N-M-P", "Ebed", "e.ved"),
        "H5652": ("עֲבָד", "H:N-M", "work", "a.vad"),
        "H5653": ("עַבְדָּא", "N:N-M-P", "Abda", "av.da"),
        "H5654": ("עֹבֵד אֱדוֹם", "N:N-M-P", "Obed-edom", "o.ved e.dom"),
        "H5655": ("עַבְדְּאֵל", "N:N-M-P", "Abdeel", "av.de.el"),
        "H5656": ("עֲבֹדָה", "H:N-F", "service", "a.vo.dah"),
        "H5657": ("עֲבֻדָּה", "H:N-F", "service", "a.vud.dah"),
        "H5658": ("עַבְדּוֹן", "N:N-M-P / N:N--L", "Abdon", "av.don"),
        "H5659": ("עַבְדוּת", "H:N-F", "slavery", "av.dut"),
        "H5660": ("עַבְדִּי", "N:N-M-P", "Abdi", "av.di"),
        "H5661": ("עַבְדִּיאֵל", "N:N-M-P", "Abdiel", "av.di.el"),
        "H5662": ("עֹבַדְיָ֫הוּ", "N:N-M-P", "Obadiah", "o.vad.yah"),
        "H5663": ("עֶ֫בֶד מֶ֫לֶךְ", "N:N-M-P", "Ebed-melech", "e.ved me.lekh"),
        "H5664": ("עֲבֵד נְגוֹ", "N:N-M-P", "Abed-nego", "a.ved ne.go"),
        "H5665": ("עֲבֵד נְגוֹ", "N:N-M-P", "Abed-nego", "a.ved ne.go"),
        "H5666": ("עָבָה", "H:V", "to thicken", "a.vah"),
        "H5667": ("עֲבוֹט", "H:N-M", "pledge", "a.vot"),
        "H5668": ("עָבוּר", "H:Prep", "for the sake of", "a.vur"),
        "H5669": ("עָבוּר", "H:N-M", "produce", "a.vur"),
        "H5670": ("עָבַט", "H:V", "to lend", "a.vat"),
        "H5671": ("עַבְטִיט", "H:N-M", "debt", "av.tit"),
        "H5672": ("עֲבִי", "H:N-M", "thickness", "a.vi"),
        "H5673": ("עֲבִידָה", "A:N-F", "work", "a.vi.dah"),
        "H5674a": ("עָבַר", "H:V", "to pass", "a.var"),
        "H5674b": ("עָבַר", "H:V", "be arrogant", "a.var"),
        "H5675": ("עֲבַר", "A:N-M", "beyond", "a.var"),
        "H5676": ("עֵ֫בֶר", "H:N-M", "side", "e.ver"),
        "H5677": ("עֵ֫בֶר", "N:N-M-P", "Eber", "e.ver"),
        "H5678": ("עֶבְרָה", "H:N-F", "fury", "ev.rah"),
        "H5679": ("עֲבָרָה", "H:N-F", "ford", "a.va.rah"),
        "H5680": ("עִבְרִי", "N:N--PG", "Hebrew", "iv.ri"),
        "H5681": ("עִבְרִי", "N:N-M-P", "Ibri", "iv.ri"),
        "H5682": ("עֲבָרִים", "N:N--L", "Abarim", "a.va.rim"),
        "H5683": ("עֶבְרֹן", "N:N--L", "Ebron", "ev.ron"),
        "H5684": ("עַבְרֹנָה", "N:N--L", "Abronah", "ev.ro.nah"),
        "H5685": ("עָבַשׁ", "H:V", "to shrivel", "a.vash"),
        "H5686": ("עָבַת", "H:V", "to weave", "a.vat"),
        "H5687": ("עָבוֹת", "H:A", "leafy", "a.vot"),
        "H5688": ("עֲבֹת", "H:N", "cord", "a.vot"),
        "H5689": ("עָגַב", "H:V", "to lust", "a.gav"),
        "H5690": ("עָגָב", "H:N-M", "lust", "e.gev"),
        "H5691": ("עֲגָבָה", "H:N-F", "lust", "a.ga.vah"),
        "H5692": ("עֻגָה", "H:N-F", "bun", "ug.gah"),
        "H5693": ("עָגוּר", "H:N-M", "crane", "a.gur"),
        "H5694": ("עָגִיל", "H:N-M", "ring", "a.gil"),
        "H5695": ("עֵ֫גֶל", "H:N-M", "calf", "e.gel"),
        "H5696": ("עָגֹל", "H:A", "round", "a.gol"),
        "H5697a": ("עֶגְלָה", "H:N-F", "heifer", "eg.lah"),
        "H5697b": ("עֶגְלַת", "H:N-F", "Eglath", ""),
        "H5698": ("עֶגְלָה", "N:N-F-P", "Eglah", "eg.lah"),
        "H5699": ("עֲגָלָה", "H:N-F", "cart", "a.ga.lah"),
        "H5700": ("עֶגְלוֹן", "N:N-M-P / N:N--L", "Eglon", "eg.lon"),
        "H5701": ("עָגַם", "H:V", "be grieved", "a.gam"),
        "H5702": ("עָגַן", "H:V", "to refrain", "a.gan"),
        "H5703": ("עַד", "H:N-M", "perpetuity", "ad"),
        "H5704": ("עַד", "H:Prep", "till", "ad"),
        "H5705": ("עַד", "A:Part", "till", "ad"),
        "H5706": ("עַד", "H:N-M", "prey", "ad"),
        "H5707": ("עֵד", "H:N-M", "witness", "ed"),
        "H5708": ("עִדָּה", "H:N-F", "filth", "ed"),
        "H5709": ("עֲדָה", "A:V", "to pass on/over/away", "a.da"),
        "H5710a": ("עָדָה", "H:V", "to advance", "a.dah"),
        "H5710b": ("עָדָה", "H:V", "to adorn", "a.dah"),
        "H5711": ("עָדָה", "N:N-F-P", "Adah", "a.dah"),
        "H5712": ("עֵדָה", "H:N-F", "congregation", "e.dah"),
        "H5713a": ("עֵדָה", "H:N-F", "witness", "e.dah"),
        "H5713b": ("עֵדָה", "H:N-F", "testimony", "e.dah"),
        "H5714": ("עִדּוֹ, עִדּוֹא", "N:N-M-P", "Iddo", "id.do"),
        "H5715": ("עֵדוּת", "H:N-F", "testimony", "e.dut"),
        "H5716": ("עֲדִי", "H:N-M", "ornament", "a.di"),
        "H5717": ("עֲדִיאֵל", "N:N-M-P", "Adiel", "a.di.el"),
        "H5718": ("עֲדָיָ֫הוּ", "N:N-M-P", "Adaiah", "a.da.yah"),
        "H5719": ("עָדִין", "H:A", "voluptuous", "a.din"),
        "H5720": ("עָדִין", "N:N-M-P", "Adin", "a.din"),
        "H5721": ("עֲדִינָא", "N:N-M-P", "Adina", "a.di.na"),
        "H5722": ("עֲדִינוֹ", "N:N-M-P", "Adino", "a.di.no"),
        "H5723": ("עֲדִיתַ֫יִם", "N:N--L", "Adithaim", "a.di.ta.yim"),
        "H5724": ("עַדְלָ֑י", "N:N-M-P", "Adlai", "ad.lay"),
        "H5725": ("עֲדֻלָּם", "N:N--L", "Adullam", "a.dul.lam"),
        "H5726": ("עֲדֻלָּמִי", "N:N--PG", "Adullamite", "a.dul.la.mi"),
        "H5727": ("עָדַן", "H:V", "to luxuriate", "a.dan"),
        "H5728": ("עֲדֶן", "H:Adv", "till", "a.den"),
        "H5729": ("עֶ֫דֶן", "N:N--L", "Eden", "e.den"),
        "H5730a": ("עֵ֫דֶן", "H:N-M", "delicacy", "e.den"),
        "H5730b": ("עֶדְנָה", "H:N-M", "delight", ""),
        "H5731a": ("עֵ֫דֶן", "N:N-M-P", "Eden", "e.den"),
        "H5731b": ("עֵ֫דֶן", "N:N--L", "Eden", "e.den"),
        "H5732": ("עִדָּן", "A:N-M", "time", "id.dan"),
        "H5733": ("עַדְנָא", "N:N-M-P", "Adna", "ad.na"),
        "H5734": ("עַדְנָה", "N:N-M-P", "Adnah", "ad.nah"),
        "H5735": ("עֲדְעָדָה", "N:N--L", "Adadah", "a.de.a.dah"),
        "H5736": ("עָדַף", "H:V", "to remain", "a.daph"),
        "H5737a": ("עָדַר", "H:V", "to help", "a.dar"),
        "H5737b": ("עָדַר", "H:V", "to hoe", "a.dar"),
        "H5737c": ("עָדַר", "H:V", "to lack", "a.dar"),
        "H5738": ("עֶ֫דֶר", "N:N-M-P", "Eder", "e.der"),
        "H5739": ("עֵ֫דֶר", "H:N-M", "flock", "e.der"),
        "H5740a": ("עֵ֫דֶר", "N:N-M-P", "Eder", "e.der"),
        "H5740b": ("עֵ֫דֶר", "N:N--L", "Eder", "e.der"),
        "H5741": ("עַדְרִיאֵל", "N:N-M-P", "Adriel", "ad.ri.el"),
        "H5742": ("עֲדָשָׁה", "H:N-F", "lentil", "a.dash"),
        "H5743": ("עוּב", "H:V", "to becloud", "uv"),
        "H5744": ("עוֹבֵד", "N:N-M-P", "Obed", "o.ved"),
        "H5745": ("עוֹבָל", "N:N-M-P", "Obal", "o.val"),
        "H5746": ("עוּג", "H:V", "to bake", "ug"),
        "H5747": ("עוֹג", "N:N-M-P", "Og", "og"),
        "H5748": ("עוּגָב", "H:N-M", "pipe", "u.gav"),
        "H5749a": ("עוּד", "H:V", "to uphold", "ud"),
        "H5749b": ("עוּד", "H:V", "to testify", "ud"),
        "H5750": ("עוֹד", "H:Adv", "still", "od"),
        "H5751": ("עוֹד", "A:Adv", "still", "od"),
        "H5752": ("עֹדֵד", "N:N-M-P", "Oded", "o.ded"),
        "H5753a": ("עָוָה", "H:V", "to twist", "a.vah"),
        "H5753b": ("עָוָה", "H:V", "to pervert", "a.vah"),
        "H5754": ("עַוָּה", "H:N-F", "ruin", "av.vah"),
        "H5755": ("עַוָּה", "N:N--L", "Ava", "iv.vah"),
        "H5756": ("עוּז", "H:V", "to seek refuge", "uz"),
        "H5757": ("עַוִּי", "N:N--PG", "Avites", "av.vi"),
        "H5758": ("עֲוָיָה", "A:N-F", "iniquity", "iv.ya"),
        "H5759": ("עֲוִיל", "H:N-M", "boy", "a.vil"),
        "H5760": ("עֲוִיל", "H:N-M", "unjust one", "a.vil"),
        "H5761": ("עַוִּים", "N:N--L / N:N--LG", "Avvim", "av.vim"),
        "H5762": ("עֲוִית", "N:N--L", "Avith", "a.vit"),
        "H5763": ("עוּל", "H:V", "to nurse", "ul"),
        "H5764": ("עוּל", "H:N-M", "infant", "ul"),
        "H5765": ("עָוַל", "H:V", "to act unjustly", "a.val"),
        "H5766a": ("עָ֫וֶל", "H:N-M", "injustice", "e.vel"),
        "H5766b": ("עַוְלָה", "H:N-M", "injustice", ""),
        "H5767": ("עַוָּל", "H:N-M", "unjust", "av.val"),
        "H5768": ("עוֹלֵל", "H:N-M", "infant", "o.lel"),
        "H5769": ("עוֹלָם", "H:N-M", "forever", "o.lam"),
        "H5770": ("עִין", "H:V", "to watch", "a.van"),
        "H5771": ("עָוֺן", "H:N-M", "iniquity", "a.van"),
        "H5772": ("עֹנָה", "H:N-F", "cohabitation", "o.nah"),
        "H5773": ("עִוְעִים", "H:N-M", "distortion", "av.eh"),
        "H5774a": ("עוּף", "H:V", "to fly", "uph"),
        "H5774b": ("תְּעֻפָה", "H:N-F", "gloom", ""),
        "H5775": ("עוֹף", "H:N-M", "bird", "oph"),
        "H5776": ("עוֹף", "A:N-M", "bird", "oph"),
        "H5777": ("עֹפֶרֶת", "H:N-M", "lead", "o.phe.ret"),
        "H5778": ("עוֹפַי", "N:N-M-P", "Ephai", "o.phay"),
        "H5779": ("עוּץ", "H:V", "to plan", "uts"),
        "H5780": ("עוּץ", "N:N-M-P / N:N--L", "Uz", "uts"),
        "H5781": ("עוּק", "H:V", "to totter", "uq"),
        "H5782": ("עוּר", "H:V", "to rouse", "ur"),
        "H5783": ("עוּר", "H:V", "be exposed", "ur"),
        "H5784": ("עוּר", "A:N-M", "chaff", "ur"),
        "H5785": ("עוֹר", "H:N-M", "skin", "or"),
        "H5786": ("עָוַר", "H:V", "to blind", "a.var"),
        "H5787": ("עִוֵּר", "H:A", "blind", "iv.ver"),
        "H5788a": ("עִוָּרוֹן", "H:N-M", "blindness", "iv.va.ron"),
        "H5788b": ("עַוֶּ֫רֶת", "H:N-M", "blindness", ""),
        "H5789": ("עוּשׁ", "H:V", "to help", "ush"),
        "H5790": ("עוּת", "H:V", "to help", "ut"),
        "H5791": ("עָוַת", "H:V", "to pervert", "a.vat"),
        "H5792": ("עַוָּתָה", "H:N-F", "oppression", "av.va.tah"),
        "H5793": ("עוּתַי", "N:N-M-P", "Uthai", "u.tay"),
        "H5794": ("עַז", "H:A", "strong", "az"),
        "H5795": ("עֵז", "H:N-F", "goat", "ez"),
        "H5796": ("עֵז", "A:N-F", "goat", "ez"),
        "H5797": ("עֹז", "H:N-M", "strength", "oz"),
        "H5798a": ("עֻזָּא", "N:N-M-P / N:N--L", "Uzzah", "uz.za"),
        "H5798b": ("עֻזָּה", "N:N-M-P / N:N--L", "Uzza", ""),
        "H5799": ("עֲזָאזֵל", "H:N-M", "scapegoat", "a.za.zel"),
        "H5800a": ("עָזַב", "H:V", "to leave", "a.zav"),
        "H5800b": ("עָזַב", "H:V", "to restore", "a.zav"),
        "H5801": ("עִזָּבוֹן", "H:N-M", "ware", "iz.za.von"),
        "H5802": ("עַזְבּוּק", "N:N-M-P", "Azbuk", "az.buq"),
        "H5803": ("עַזְגָּד", "N:N-M-P", "Azgad", "az.gad"),
        "H5804": ("עַזָּה", "N:N--L", "Gaza", "az.zah"),
        "H5805": ("עֲזוּבָה", "H:N-F", "desolation", "a.zu.vah"),
        "H5806": ("עֲזוּבָה", "N:N-F-P", "Azubah", "a.zu.vah"),
        "H5807": ("עֱזוּז", "H:N-M", "strength", "e.zuz"),
        "H5808": ("עִזּוּז", "H:A", "mighty", "iz.zuz"),
        "H5809": ("עַזּוּר", "N:N-M-P", "Azzur", "az.zur"),
        "H5810": ("עָזַז", "H:V", "be strong", "a.zaz"),
        "H5811": ("עָזָז", "N:N-M-P", "Azaz", "a.zaz"),
        "H5812": ("עֲזַזְיָ֫הוּ", "N:N-M-P", "Azaziah", "a.zaz.ya.hu"),
        "H5813": ("עֻזִּי", "N:N-M-P", "Uzzi", "uz.zi"),
        "H5814": ("עֻזִּיָּא", "N:N-M-P", "Uzzia", "uz.ziy.ya"),
        "H5815": ("עֲזִיאֵל", "N:N-M-P", "Aziel", "a.zi.el"),
        "H5816": ("עֻזִּיאֵל", "N:N-M-P", "Uzziel", "uz.zi.el"),
        "H5817": ("עׇזִּיאֵלִי", "N:N--PG", "Uzzielite", "oz.zi.e.li"),
        "H5818": ("עֻזִּיָּהוּ", "N:N-M-P", "Uzziah", "uz.ziy.yah"),
        "H5819": ("עֲזִיזָא", "N:N-M-P", "Aziza", "a.zi.za"),
        "H5820": ("עַזְמָ֫וֶת", "N:N-M-P / N:N--L", "Azmaveth", "az.ma.vet"),
        "H5821": ("עַזָּן", "N:N-M-P", "Azzan", "az.zan"),
        "H5822": ("עׇזְנִיָּה", "H:N-F", "vulture", "oz.niy.yah"),
        "H5823": ("עָזַק", "H:V", "to dig about", "a.zaq"),
        "H5824": ("עִזְקָה", "A:N-F", "signet ring", "iz.qa"),
        "H5825": ("עֲזֵקָה", "N:N--L", "Azekah", "a.ze.qah"),
        "H5826": ("עָזַר", "H:V", "to help", "a.zar"),
        "H5827": ("עֶ֫זֶר", "N:N-M-P", "Ezer", "e.zer"),
        "H5828": ("עֵ֫זֶר", "H:N-M", "helper", "e.zer"),
        "H5829": ("עֵ֫זֶר", "N:N-M-P", "Ezer", "e.zer"),
        "H5830": ("עֶזְרָא", "N:N-M-P", "Ezra", "ez.ra"),
        "H5831": ("עֶזְרָא", "N:N-M-P", "Ezra", "ez.ra"),
        "H5832": ("עֲזַרְאֵל", "N:N-M-P", "Azarael", "a.zar.el"),
        "H5833": ("עֶזְרָה", "H:N-F", "help", "ez.rah"),
        "H5834": ("עֶזְרָה", "N:N-M-P", "Ezrah", "ez.rah"),
        "H5835": ("עֲזָרָה", "H:N-F", "enclosure", "a.za.rah"),
        "H5836": ("עֶזְרִי", "N:N-M-P", "Ezri", "ez.ri"),
        "H5837": ("עַזְרִיאֵל", "N:N-M-P", "Azriel", "az.ri.el"),
        "H5838": ("עֲזַרְיָ֫הוּ, עֲזַרְיָה", "N:N-M-P", "Azariah", "a.zar.yah"),
        "H5839": ("עֲזַרְיָה", "N:N-M-P", "Azariah", "a.zar.yah"),
        "H5840": ("עַזְרִיקָם", "N:N-M-P", "Azrikam", "az.ri.qam"),
        "H5841": ("עַזָּתִי", "N:N--LG", "Gazite", "az.za.ti"),
        "H5842": ("עֵט", "H:N-M", "stylus", "et"),
        "H5843": ("עֵטָא", "A:N-F", "counsel", "e.ta"),
        "H5844a": ("עָטָה", "H:V", "to enwrap", "a.tah"),
        "H5844b": ("עָטָה", "H:V", "to grasp", "a.tah"),
        "H5845": ("עֲטִין", "H:N-M", "bucket", "a.tin"),
        "H5846": ("עֲטִישָׁה", "H:N-F", "sneezing", "a.ti.shah"),
        "H5847": ("עֲטַלֵּף", "H:N-M", "bat", "a.tal.leph"),
        "H5848a": ("עָטַף", "H:V", "to turn aside", "a.taph"),
        "H5848b": ("עָטַף", "H:V", "to envelope", "a.taph"),
        "H5848c": ("עָטַף", "H:V", "to enfeeble", "a.taph"),
        "H5849a": ("עָטַר", "H:V", "to surround", "a.tar"),
        "H5849b": ("עָטַר", "H:V", "to crown", "a.tar"),
        "H5850": ("עֲטָרָה", "H:N-F", "crown", "a.ta.rah"),
        "H5851": ("עֲטָרָה", "N:N-F-P", "Atarah", "a.ta.rah"),
        "H5852": ("עֲטָרוֹת", "N:N--L", "Ataroth", "a.ta.rot"),
        "H5853": ("עַטְרוֹת אַדָּר", "N:N--L", "Ataroth-addar", "at.rot ad.dar"),
        "H5854": ("עַטְרוֹת בֵּית יוֹאָב", "N:N--L", "Atroth-beth-joab", "at.rot bet yo.av"),
        "H5855": ("עַטְרוֹת שׁוֹפָן", "N:N--L", "Atroth-shophan", "at.rot sho.phan"),
        "H5856": ("עִי", "H:N-M", "ruin", "i"),
        "H5857": ("עַי", "N:N--L", "Ai", "ay"),
        "H5858a": ("עֵיבָל", "N:N--L / N:N--LG", "Ebal", "e.val"),
        "H5858b": ("עֵיבָל", "N:N-M-P", "Ebal", "e.val"),
        "H5858c": ("עֵיבָל", "N:N--L", "Ebal", "e.val"),
        "H5859": ("עִיּוֹן", "N:N--L", "Ijon", "iy.yon"),
        "H5860a": ("עִיט", "H:V", "to scream", "it"),
        "H5860b": ("עִיט", "H:V", "to pounce", "it"),
        "H5861": ("עַ֫יִט", "H:N-M", "bird of prey", "a.yit"),
        "H5862": ("עֵיטָם", "N:N--L", "Etam", "e.tam"),
        "H5863": ("עִיֵּי הָעֲבָרִים", "N:N--L", "Iye-abarim", "iy.ye ha.a.va.rim"),
        "H5864": ("עִיִּים", "N:N--L", "Iyim", "iy.yim"),
        "H5865": ("עֵילוֹם", "H:N-M", "ever", "e.lom"),
        "H5866": ("עִילַי", "N:N-M-P", "Ilai", "i.lay"),
        "H5867a": ("עֵילָם", "N:N--L", "Elam", "e.lam"),
        "H5867b": ("עֵילָם", "N:N-M-P", "Elam", "e.lam"),
        "H5867c": ("עֵילָם", "N:N-M-P", "Elam", "e.lam"),
        "H5868a": ("עוֹנָה", "H:N-M/F", "strength", "a.yam"),
        "H5868b": ("עֲיָם", "H:N-M/F", "heat", "a.yam"),
        "H5869a": ("עַ֫יִן", "H:N-F", "eye", "a.yin"),
        "H5869b": ("עַ֫יִן", "H:N-F", "spring", "a.yin"),
        "H5870": ("עַ֫יִן", "A:N-F", "eye", "a.yin"),
        "H5871": ("עַ֫יִן", "N:N--L", "Ain", "a.yin"),
        "H5872": ("עֵין גֶּ֫דִי", "N:N--L", "En-gedi", "en ge.di"),
        "H5873": ("עֵין גַּנִּים", "N:N--L", "En-gannim", "en gan.nim"),
        "H5874": ("עֵין־דֹּר", "N:N--L", "En-dor", "en-dor"),
        "H5875": ("עֵין הַקּוֹרֵא", "N:N--L", "En-hakkore", "en haq.qo.re"),
        "H5876": ("עֵין חַדָּה", "N:N--L", "En-haddah", "en chad.dah"),
        "H5877": ("עֵין חָצוֹר", "N:N--L", "En-hazor", "en cha.tsor"),
        "H5878": ("עֵין חֲרֹד", "N:N--L", "En-harod", "en cha.rod"),
        "H5879": ("עֵינָם", "N:N--L", "Enam", "e.na.yim"),
        "H5880": ("עֵין מִשְׁפָּט", "N:N--L", "En-mishpat", "en mish.pat"),
        "H5881": ("עֵינָן", "N:N-M-P", "Enan", "e.nan"),
        "H5882": ("עֵין עֶגְלַ֫יִם", "N:N--L", "En-eglaim", "en eg.la.yim"),
        "H5883": ("עֵין רֹגֵל", "N:N--L", "En-rogel", "en ro.gel"),
        "H5884": ("עֵין רִמּוֹן", "N:N--L", "En-rimmon", "en rim.mon"),
        "H5885": ("עֵין שֶׁ֫מֶשׁ", "N:N--L", "En-shemesh", "en she.mesh"),
        "H5886": ("עֵין תַּנִּים", "N:N--L", "En-tannim", "en tan.nim"),
        "H5887": ("עֵין תַּפּוּחַ", "N:N--L", "En-tappuah", "en tap.pu.ach"),
        "H5888": ("עִיף", "H:V", "be faint", "a.yeph"),
        "H5889": ("עָיֵף", "H:A", "faint", "a.yeph"),
        "H5890": ("עֵיפָה", "H:N-F", "darkness", "e.phah"),
        "H5891": ("עֵיפָה", "N:N-M-P / N:N-F-P / N:N--L", "Ephah", "e.phah"),
        "H5892a": ("עִיר", "H:N-M", "excitement", "ir"),
        "H5892b": ("עִיר", "H:N-M", "city", "ir"),
        "H5893": ("עִיר", "N:N-M-P", "Ir", "ir"),
        "H5894": ("עִיר", "A:N-M", "watcher", "ir"),
        "H5895": ("עַ֫יִר", "H:N-M", "colt", "a.yir"),
        "H5896": ("עִירָא", "N:N-M-P", "Ira", "i.ra"),
        "H5897": ("עִירָד", "N:N-M-P", "Irad", "i.rad"),
        "H5898": ("עִיר־הַמֶּ֫לַח", "N:N--L", "City of Salt", "ir ham.me.lach"),
        "H5899": ("עִיר הַתְּמָרִים", "N:N--L", "Ir-hatmarim", "ir hat.t.ma.rim"),
        "H5900": ("עִירוּ", "N:N-M-P", "Iru", "i.ru"),
        "H5901": ("עִירִי", "N:N-M-P", "Iri", "i.ri"),
        "H5902": ("עִירָם", "N:N-M-P", "Iram", "i.ram"),
        "H5903": ("עֵירֹם", "H:A", "naked", "e.rom"),
        "H5904": ("עִיר נָחָשׁ", "N:N--L", "Ir-nahash", "ir na.chash"),
        "H5905": ("עִיר שָׁ֑מֶשׁ", "N:N--L", "Ir-shemesh", "ir she.mesh"),
        "H5906": ("עַ֫יִשׁ", "N:N-F-T", "Bear", "a.yish"),
        "H5907": ("עַכְבּוֹר", "N:N-M-P", "Achbor", "akh.bor"),
        "H5908": ("עַכָּבִישׁ", "H:N-M", "spider", "ak.ka.vish"),
        "H5909": ("עַכְבָּר", "H:N-M", "mouse", "akh.bar"),
        "H5910": ("עַכּוֹ", "N:N--L", "Acco", "ak.ko"),
        "H5911": ("עָכוֹר", "N:N--L", "Achor", "a.khor"),
        "H5912": ("עָכָן", "N:N-M-P", "Achan", "a.khan"),
        "H5913": ("עָכַס", "H:V", "to tinkle", "a.khas"),
        "H5914": ("עֶ֫כֶס", "H:N-M", "anklet", "e.khes"),
        "H5915": ("עַכְסָה", "N:N-F-P", "Achsah", "akh.sah"),
        "H5916": ("עָכַר", "H:V", "to trouble", "a.khar"),
        "H5917": ("עָכָר", "N:N-M-P", "Achar", "a.khar"),
        "H5918": ("עׇכְרָן", "N:N-M-P", "Ochran", "okh.ran"),
        "H5919": ("עַכְשׁוּב", "H:N-M", "viper", "akh.shuv"),
        "H5920": ("עַל", "H:N-M / H:Adv", "height", "al"),
        "H5921a": ("עַל", "H:Prep", "upon", "al"),
        "H5921b": ("[כִּי] עַל [כֵּן]", "H:Prep", "upon", ""),
        "H5922": ("עַל", "A:Prep", "since", "al"),
        "H5923": ("עֹל", "H:N-M", "yoke", "ol"),
        "H5924": ("עֵ֫לָּא", "A:Adv", "above", "el.la"),
        "H5925": ("עֻלָּא", "N:N-M-P", "Ulla", "ul.la"),
        "H5926": ("עִלֵּג", "H:A", "stammerer", "il.leg"),
        "H5927": ("עָלָה", "H:V", "to ascend", "a.lah"),
        "H5928": ("עֲלָת", "A:N-F", "burnt offering", "a.lah"),
        "H5929": ("עָלֶה", "H:N-M", "leaf", "a.leh"),
        "H5930a": ("עֹלָה", "H:N-F", "burnt offering", "o.lah"),
        "H5930b": ("עֹלָה", "H:N-F", "ascent", "o.lah"),
        "H5931": ("עִלָּה", "A:N-F", "pretext", "il.lah"),
        "H5932": ("עַלְוָה", "H:N-F", "iniquity", "al.vah"),
        "H5933": ("עַלְוָה", "N:N-M-P", "Alvah", "al.vah"),
        "H5934": ("עָלוּמִים", "H:N-M", "youth", "a.lum"),
        "H5935": ("עַלְיָן, עַלְוָן", "N:N-M-P", "Alian", "al.van"),
        "H5936": ("עֲלוּקָה", "H:N-F", "leech", "a.lu.qah"),
        "H5937": ("עָלַז", "H:V", "to exult", "a.laz"),
        "H5938": ("עָלֵז", "H:A", "exultant", "a.lez"),
        "H5939": ("עֲלָטָה", "H:N-F", "darkness", "a.la.tah"),
        "H5940": ("עֱלִי", "H:N-M", "pestle", "e.li"),
        "H5941": ("עֵלִי", "N:N-M-P", "Eli", "e.li"),
        "H5942": ("עִלִּי", "H:A", "upper", "il.li"),
        "H5943": ("עִלַּי", "A:A", "highest", "il.lay"),
        "H5944": ("עֲלִיָּה", "H:N-F", "upper room", "a.liy.yah"),
        "H5945a": ("עֶלְיוֹן", "H:A", "high", "el.yon"),
        "H5945b": ("עֶלְיוֹן", "H:N-M", "Highest", "el.yon"),
        "H5946": ("עֶלְיוֹן", "A:A", "Highest", "el.yon"),
        "H5947": ("עַלִּיז", "H:A", "jubilant", "al.liz"),
        "H5948": ("עֲלִיל", "H:N-M", "furnace", "a.lil"),
        "H5949": ("עֲלִילָה", "H:N-F", "wantonness", "a.li.lah"),
        "H5950": ("עֲלִילִיָּה", "H:N-F", "deed", "a.li.liy.yah"),
        "H5951": ("עֲלִיצוּת", "H:N-F", "exultation", "a.li.tsut"),
        "H5952": ("עֲלִי", "A:N-F", "roof", "al.lit"),
        "H5953a": ("עָלַל", "H:V", "to abuse", "a.lal"),
        "H5953b": ("עָלַל", "H:V", "to glean", "a.lal"),
        "H5953c": ("עָלַל", "H:V", "to mock", "a.lal"),
        "H5953d": ("עָלַל", "H:V", "to thrust", "a.lal"),
        "H5954": ("עֲלַל", "A:V", "to come", "a.lal"),
        "H5955": ("עֹלֵלוֹת", "H:N-F", "gleaning", "o.le.lah"),
        "H5956": ("עָלַם", "H:V", "to conceal", "a.lam"),
        "H5957": ("עָלַם", "A:N-M", "perpetuity", "a.lam"),
        "H5958": ("עֶ֫לֶם", "H:N-M", "youth", "e.lem"),
        "H5959": ("עַלְמָה", "H:N-F", "maiden", "al.mah"),
        "H5960": ("עַלְמוֹן", "N:N--L", "Almon", "al.mon"),
        "H5961": ("עֲלָמוֹת", "H:N-F", "Alamoth", "a.la.mot"),
        "H5962": ("עֵלְמָיֵא", "N:N--LG", "Elamite", "al.mi"),
        "H5963": ("עַלְמֹן דִּבְלָתָיְמָה", "N:N--L", "Almon-diblathaim", "al.mon div.la.ta.ye.mah"),
        "H5964": ("עָלֶ֫מֶת", "N:N-M-P / N:N--L", "Alemeth", "a.le.met"),
        "H5965": ("עָלַס", "H:V", "to rejoice", "a.las"),
        "H5966": ("עָלַע", "H:V", "to suck", "a.la"),
        "H5967": ("עֲלַע", "A:N-F", "rib", "a.la"),
        "H5968": ("עָלַף", "H:V", "to enwrap", "a.laph"),
        "H5969": ("עֻלְפֶּה", "H:N-M", "fainting", "ul.peh"),
        "H5970": ("עָלַץ", "H:V", "to rejoice", "a.lats"),
        "H5971a": ("עַם", "H:N-M", "people", "am"),
        "H5971b": ("עַם", "H:N-M", "kinsman", "am"),
        "H5972": ("עַם", "A:N-M", "people", "am"),
        "H5973a": ("עִם", "H:Prep", "with", "im"),
        "H5973b": ("מֵעִם", "H:Prep", "from with", ""),
        "H5974": ("עִם", "A:Prep", "with", "im"),
        "H5975": ("עָמַד", "H:V", "to stand", "a.mad"),
        "H5976": ("עָמַד", "H:V", "to shake", "a.mad"),
        "H5977": ("עֹ֫מֶד", "H:N-M", "post", "o.med"),
        "H5978": ("עִמָּדִי", "H:Prep", "with me", "im.mad"),
        "H5979": ("עֶמְדָּה", "H:N-F", "support", "em.dah"),
        "H5980": ("עֻמָּה", "H:N-F", "close", "um.mah"),
        "H5981": ("עֻמָּה", "N:N--L", "Ummah", "um.mah"),
        "H5982": ("עַמּוּד", "H:N-M", "pillar", "am.mud"),
        "H5983": ("עַמּוֹן", "N:N-M-P", "Ammon", "am.mon"),
        "H5984": ("עַמּוֹנִי", "N:N--PG", "Ammonite", "am.mo.ni"),
        "H5985": ("עַמּוֹנִית", "N:N-F-PG", "Ammonitess", "am.mo.nit"),
        "H5986": ("עָמוֹס", "N:N-M-P", "Amos", "a.mos"),
        "H5987": ("עָמוֹק", "N:N-M-P", "Amok", "a.moq"),
        "H5988": ("עַמִּיאֵל", "N:N-M-P", "Ammiel", "am.mi.el"),
        "H5989": ("עַמִּיהוּד", "N:N-M-P", "Ammihud", "am.mi.hud"),
        "H5990": ("עַמִּיזָבָד", "N:N-M-P", "Ammizabad", "am.mi.za.vad"),
        "H5991": ("עַמִּיחוּר", "N:N-M-P", "Ammihur", "am.mi.chur"),
        "H5992": ("עַמִּינָדָב", "N:N-M-P", "Amminadab", "am.mi.na.dav"),
        "H5993": ("עַמִּי נָדִיב", "N:N-M-P", "Amminadib", "am.mi na.div"),
        "H5994": ("עֲמִיק", "A:A", "deep", "a.miq"),
        "H5995": ("עָמִיר", "H:N-M", "sheaf", "a.mir"),
        "H5996": ("עַמִּישַׁדָּ֑י", "N:N-M-P", "Ammishaddai", "am.mi.shad.day"),
        "H5997": ("עָמִית", "H:N-M", "neighbor", "a.mit"),
        "H5998": ("עָמַל", "H:V", "to toil", "a.mal"),
        "H5999": ("עָמָל", "H:N", "trouble", "a.mal"),
        "H6000": ("עָמָל", "N:N-M-P", "Amal", "a.mal"),
        "H6001a": ("עָמֵל", "H:N-M", "labour(er)", "a.mel"),
        "H6001b": ("עָמֵל", "H:A", "laborious", "a.mel"),
        "H6002": ("עֲמָלֵק", "N:N-M-P", "Amalek", "a.ma.leq"),
        "H6003": ("עֲמָלֵקִי", "N:N--PG", "Amalekite", "a.ma.le.qi"),
        "H6004": ("עָמַם", "H:V", "to darken", "a.mam"),
        "H6005": ("עִמָּנוּאֵל", "N:N-M-P", "Immanuel", "im.ma.nu.el"),
        "H6006": ("עָמַס", "H:V", "to lift", "a.mas"),
        "H6007": ("עֲמַסְיָה", "N:N-M-P", "Amasiah", "a.mas.yah"),
        "H6008": ("עַמְעָד", "N:N--L", "Amad", "am.ad"),
        "H6009": ("עָמֹק", "H:V", "be deep", "a.maq"),
        "H6010": ("עֵ֫מֶק", "H:N-M", "valley", "e.meq"),
        "H6011": ("עֹ֫מֶק", "H:N-M", "depth", "o.meq"),
        "H6012": ("עָמֵק", "H:A", "unfathomable", "a.meq"),
        "H6013": ("עָמֹק", "H:A", "deep", "a.moq"),
        "H6014a": ("עִמֵּר", "H:V", "to bind", "a.mar"),
        "H6014b": ("עָמַר", "H:V", "to tyranise", "a.mar"),
        "H6015": ("עֲמַר", "A:N-M", "wool", "a.mar"),
        "H6016a": ("עֹ֫מֶר", "H:N-M", "sheaf", "o.mer"),
        "H6016b": ("עֹ֫מֶר", "H:N-M", "omer", "o.mer"),
        "H6017": ("עֲמֹרָה", "N:N--L", "Gomorrah", "a.mo.rah"),
        "H6018": ("עׇמְרִי", "N:N-M-P", "Omri", "om.ri"),
        "H6019": ("עַמְרָם", "N:N-M-P", "Amram", "am.ram"),
        "H6020": ("עַמְרָמִי", "N:N--PG", "Amramite", "am.ra.mi"),
        "H6021": ("עֲמָשָׂא", "N:N-M-P", "Amasa", "a.ma.sa"),
        "H6022": ("עֲמָשַׂי", "N:N-M-P", "Amasai", "a.ma.say"),
        "H6023": ("עֲמַשְׁסַי", "N:N-M-P", "Amashsai", "a.mash.say"),
        "H6024": ("עֲנָב", "N:N--L", "Anab", "a.nav"),
        "H6025": ("עֵנָב", "H:N-M", "grape", "e.nav"),
        "H6026": ("עָנֹג", "H:V", "to delight", "a.nag"),
        "H6027": ("עֹ֫נֶג", "H:N-M", "delight", "o.neg"),
        "H6028": ("עָנֹג", "H:A", "dainty", "a.nog"),
        "H6029": ("עָנַד", "H:V", "to bind", "a.nad"),
        "H6030a": ("עוּן", "H:V", "to dwell", "a.nah"),
        "H6030b": ("עָנָה", "H:V", "to answer", "a.nah"),
        "H6030c": ("עָנָה", "H:V", "to sing", "a.nah"),
        "H6031a": ("עָנָה", "H:V", "be occupied", "a.nah"),
        "H6031b": ("עָנָה", "H:V", "to afflict", "a.nah"),
        "H6032": ("עֲנָה", "A:V", "to answer", "a.nah"),
        "H6033": ("עֲנָי", "A:A", "poor", "a.nah"),
        "H6034": ("עֲנָה", "N:N-M-P / N:N-F-P", "Anah", "a.nah"),
        "H6035": ("עָנָו", "H:N-M", "poor", "a.nav"),
        "H6036": ("עָנוּב", "N:N-M-P", "Anub", "a.nuv"),
        "H6037": ("עַנְוָה", "H:N-F", "gentleness", "an.vah"),
        "H6038": ("עֲנָוָה", "H:N-F", "humility", "a.na.vah"),
        "H6039": ("עֱנוּת", "H:N-F", "affliction", "e.nut"),
        "H6040": ("עֳנִי", "H:N-M", "affliction", "o.ni"),
        "H6041": ("עָנִי", "H:A", "afflicted", "a.ni"),
        "H6042": ("עֻנִּי", "N:N-M-P", "Unni", "un.ni"),
        "H6043": ("עֲנָיָה", "N:N-M-P", "Anaiah", "a.na.yah"),
        "H6044": ("עָנִים", "N:N--L", "Anim", "a.nim"),
        "H6045": ("עִנְיָן", "H:N-M", "task", "in.yan"),
        "H6046": ("עָנֵם", "N:N--L", "Anem", "a.nem"),
        "H6047": ("עֲנָמִים", "N:N-M-P / N:N-M-PG", "Anamites", "a.na.mim"),
        "H6048": ("עֲנַמֶּ֫לֶךְ", "N:N-M-T", "Anammelech", "a.nam.me.lekh"),
        "H6049a": ("עָנַן", "H:V", "to cloud", "a.nan"),
        "H6049b": ("עָנַן", "H:V", "to divine", "a.nan"),
        "H6050": ("עֲנָן", "A:N-M", "cloud", "a.nan"),
        "H6051": ("עָנָן", "H:N-M", "cloud", "a.nan"),
        "H6052": ("עָנָן", "N:N-M-P", "Anan", "a.nan"),
        "H6053": ("עֲנָנָה", "H:N-F", "cloud", "a.na.nah"),
        "H6054": ("עֲנָנִי", "N:N-M-P", "Anani", "a.na.ni"),
        "H6055": ("עֲנַנְיָה", "N:N-M-P / N:N--L", "Ananiah", "a.nan.yah"),
        "H6056": ("עֲנַף", "A:N-M", "bough", "a.naph"),
        "H6057": ("עָנָף", "H:N-M", "branch", "a.naph"),
        "H6058": ("עָנֵף", "H:A", "thickly branched", "a.neph"),
        "H6059": ("עָנַק", "H:V", "to ornament", "a.naq"),
        "H6060a": ("עֲנָק", "H:N-M", "neck", "a.naq"),
        "H6060b": ("עֲנָק", "H:N-M", "necklace", "a.naq"),
        "H6061": ("עֲנָק", "N:N-M-P", "Anak", "a.naq"),
        "H6062": ("עֲנָקִי", "N:N--PG", "Anakim", "a.na.qi"),
        "H6063": ("עָנֵר", "N:N-M-P / N:N--L", "Aner", "a.ner"),
        "H6064": ("עָנַשׁ", "H:V", "to fine", "a.nash"),
        "H6065": ("עֲנָשׁ", "A:N-M", "confiscation", "a.nash"),
        "H6066": ("עֹ֫נֶשׁ", "H:N-M", "fine", "o.nesh"),
        "H6067": ("עֲנָת", "N:N-M-P", "Anath", "a.nat"),
        "H6068": ("עֲנָתוֹת", "N:N-M-P / N:N--L", "Anathoth", "a.na.tot"),
        "H6069": ("עַנְּתֹתִי", "N:N--LG", "Anathothite", "an.to.ti"),
        "H6070": ("עַנְתֹתִיָּה", "N:N-M-P", "Anthothijah", "an.to.tiy.yah"),
        "H6071": ("עָסִיס", "H:N-M", "sweet", "a.sis"),
        "H6072": ("עָסַס", "H:V", "to press", "a.sas"),
        "H6073": ("עֳפָיִם", "H:N-M", "branch", "o.phe"),
        "H6074": ("עֳפִי", "A:N-M", "foliage", "o.phi"),
        "H6075a": ("עָפַל", "H:V", "to swell", "a.phal"),
        "H6075b": ("עָפַל", "H:V", "to presume", "a.phal"),
        "H6076a": ("עֹ֫פֶל", "H:N-M", "hill", "o.phel"),
        "H6076b": ("עֹ֫פֶל", "H:N-M", "tumor", "o.phel"),
        "H6077": ("עֹ֫פֶל", "N:N--L", "Ophel", "o.phel"),
        "H6078": ("עׇפְנִי", "N:N--L", "Ophni", "oph.ni"),
        "H6079": ("עַפְעַף", "H:N-M", "eyelid", "aph.aph"),
        "H6080": ("עָפַר", "H:V", "to throw", "a.phar"),
        "H6081": ("עֵ֫פֶר", "N:N-M-P", "Epher", "e.pher"),
        "H6082": ("עֹ֫פֶר", "H:N-M", "fawn", "o.pher"),
        "H6083": ("עָפָר", "H:N-M", "dust", "a.phar"),
        "H6084": ("עׇפְרָה", "N:N-M-P / N:N--L", "Ophrah", "oph.rah"),
        "H6085": ("עֶפְרוֹן", "N:N-M-P / N:N--L", "Ephron", "eph.ron"),
        "H6086": ("עֵץ", "H:N-M", "tree", "ets"),
        "H6087a": ("עָצַב", "H:V", "to hurt", "a.tsav"),
        "H6087b": ("עָצַב", "H:V", "to shape", "a.tsav"),
        "H6088": ("עֲצַב", "A:V", "to pain", "a.tsav"),
        "H6089a": ("עֶ֫צֶב", "H:N-M", "toil", "e.tsev"),
        "H6089b": ("עֶ֫צֶב", "H:N-M", "vessel", "e.tsev"),
        "H6090a": ("עֹ֫צֶב", "H:N-M", "pain", "o.tsev"),
        "H6090b": ("עֹ֫צֶב", "H:N-M", "idol", "o.tsev"),
        "H6091": ("עָצָב", "H:N-M", "idol", "a.tsav"),
        "H6092": ("עָצֵב", "H:N-M", "worker", "a.tsev"),
        "H6093": ("עִצָּבוֹן", "H:N-M", "toil", "its.tsa.von"),
        "H6094": ("עַצֶּ֫בֶת", "H:N-F", "injury", "ats.tse.vet"),
        "H6095": ("עָצָה", "H:V", "to shut", "a.tsah"),
        "H6096": ("עָצֶה", "H:N-M", "spine", "a.tseh"),
        "H6097": ("עֵצָה", "H:N-F", "tree", "e.tsah"),
        "H6098": ("עֵצָה", "H:N-F", "counsel", "e.tsah"),
        "H6099": ("עָצוּם", "H:A", "mighty", "a.tsum"),
        "H6100": ("עֶצְיוֹן גֶּ֫בֶר", "N:N--L", "Ezion-geber", "ets.yon ge.ver"),
        "H6101": ("עָצֵל", "H:V", "be sluggish", "a.tsal"),
        "H6102": ("עָצֵל", "H:A", "sluggish", "a.tsel"),
        "H6103": ("עַצְלָה", "H:N-F", "sluggishness", "ats.lah"),
        "H6104": ("עַצְלוּת", "H:N-F", "sluggishness", "ats.lut"),
        "H6105a": ("עָצֹם", "H:V", "be vast", "a.tsam"),
        "H6105b": ("עָצַם", "H:V", "to shut eyes", "a.tsam"),
        "H6106": ("עֶ֫צֶם", "H:N-F", "bone", "e.tsem"),
        "H6107": ("עֶ֫צֶם", "N:N--L", "Ezem", "e.tsem"),
        "H6108": ("עֹ֫צֶם", "H:N-M", "strength", "o.tsem"),
        "H6109": ("עׇצְמָה", "H:N-F", "strength", "ots.mah"),
        "H6110": ("עַצֻמָה", "H:N-F", "defense", "ats.tsu.mah"),
        "H6111": ("עַצְמוֹן", "N:N--L", "Azmon", "ats.mon"),
        "H6112": ("עֵצֶן", "H:N-M", "spear", "e.tsen"),
        "H6113": ("עָצַר", "H:V", "to restrain", "a.tsar"),
        "H6114": ("עֶ֫צֶר", "H:N-M", "magistrate", "e.tser"),
        "H6115": ("עֹ֫צֶר", "H:N-M", "coercion", "o.tser"),
        "H6116": ("עֲצָרָה", "H:N-F", "assembly", "a.tsa.rah"),
        "H6117": ("עָקַב", "H:V", "to assail", "a.qav"),
        "H6118": ("עֵ֫קֶב", "H:N-M", "consequence", "e.qev"),
        "H6119": ("עָקֵב", "H:N-M", "heel", "a.qev"),
        "H6120": ("עָקֵב", "H:A", "supplanting", "a.qev"),
        "H6121a": ("עָקֹב", "H:A", "insidious", "a.qov"),
        "H6121b": ("עָקֹב", "H:A", "steep", "a.qov"),
        "H6122": ("עׇקְבָה", "H:N-F", "cunning", "oq.vah"),
        "H6123": ("עָקַד", "H:V", "to bind", "a.qad"),
        "H6124": ("עָקֹד", "H:A", "striped", "a.qod"),
        "H6125": ("עָקָה", "H:N-F", "pressure", "a.qah"),
        "H6126": ("עַקּוּב", "N:N-M-P", "Akkub", "aq.quv"),
        "H6127": ("עָקַל", "H:V", "to twist", "a.qal"),
        "H6128": ("עֲקַלְקַל", "H:A", "crooked", "a.qal.qal"),
        "H6129": ("עֲקַלָּתוֹן", "H:A", "crooked", "a.qal.la.ton"),
        "H6130": ("עֲקָן", "N:N-M-P", "Akan", "a.qan"),
        "H6131a": ("עָקַר", "H:V", "to uproot", "a.qar"),
        "H6131b": ("עִקֵּר", "H:V", "to hamstring", ""),
        "H6132": ("עֲקַר", "A:V", "be uprooted", "a.qar"),
        "H6133": ("עֵ֫קֶר", "H:N-M", "descendant", "e.qer"),
        "H6134": ("עֵקֶר", "N:N-M-P", "Eker", "e.qer"),
        "H6135": ("עָקָר", "H:A", "barren", "a.qar"),
        "H6136": ("עִקַּר", "A:N-M", "root", "iq.qar"),
        "H6137": ("עַקְרָב", "H:N-M / N:N--L", "scorpion", "aq.rav"),
        "H6138": ("עֶקְרוֹן", "N:N--L", "Ekron", "eq.ron"),
        "H6139": ("עֶקְרוֹנִי", "N:N--LG", "Ekronite", "eq.ro.ni"),
        "H6140": ("עָקַשׁ", "H:V", "to twist", "a.qash"),
        "H6141": ("עִקֵּשׁ", "H:A", "twisted", "iq.qesh"),
        "H6142": ("עִקֵּשׁ", "N:N-M-P", "Ikkesh", "iq.qesh"),
        "H6143": ("עִקְּשׁוּת", "H:N-F", "crookedness", "iq.q.shut"),
        "H6144": ("עָר", "N:N--L", "Ar", "ar"),
        "H6145": ("עָר", "H:N-M", "enemy", "ar"),
        "H6146": ("עָר", "A:N-M", "foe", "ar"),
        "H6147": ("עֵר", "N:N-M-P", "Er", "er"),
        "H6148": ("עָרַב", "H:V", "to pledge", "a.rav"),
        "H6149": ("עָרַב", "H:V", "to please", "a.rev"),
        "H6150": ("עָרַב", "H:V", "to grow dark", "a.rav"),
        "H6151": ("עֲרַב", "A:V", "to mix", "a.rav"),
        "H6152a": ("עֲרַב", "N:N--L", "Arabia", "a.rav"),
        "H6152b": ("עֲרָב", "N:N--LG", "Arabia", "a.rav"),
        "H6153": ("עֶ֫רֶב", "H:N-M", "evening", "e.rev"),
        "H6154a": ("עֵ֫רֶב", "H:N-M", "racial-mix", "e.rev"),
        "H6154b": ("עֵ֫רֶב", "H:N-M", "mixture", "e.rev"),
        "H6155": ("עֲרָבָה", "H:N-F", "willow", "a.rav"),
        "H6156": ("עָרֵב", "H:A", "sweet", "a.rev"),
        "H6157": ("עָרֹב", "H:N-M", "swarm", "a.rov"),
        "H6158": ("עֹרֵב", "H:N-M", "raven", "o.rev"),
        "H6159": ("עֹרֵב", "N:N-M-P / N:N--L", "Oreb", "o.rev"),
        "H6160": ("עֲרָבָה", "N:N--L", "Arabah", "a.ra.vah"),
        "H6161": ("עֲרֻבָּה", "H:N-F", "pledge", "a.rub.bah"),
        "H6162": ("עֵרָבוֹן", "H:N-M", "pledge", "a.ra.von"),
        "H6163a": ("עֲרָבִי", "N:N--LG", "Arab", "a.ra.vi"),
        "H6163b": ("עַרְבִי", "N:N--LG", "Arab", ""),
        "H6164": ("עַרְבָתִי", "N:N--LG", "Arbathite", "ar.va.ti"),
        "H6165": ("עָרַג", "H:V", "to long for", "a.rag"),
        "H6166a": ("עֲרָד", "N:N--L", "Arad", "a.rad"),
        "H6166b": ("עֲרָד", "N:N-M-P", "Arad", "a.rad"),
        "H6167": ("עֲרָד", "A:N-M", "wild donkey", "a.rad"),
        "H6168": ("עָרָה", "H:V", "to uncover", "a.rah"),
        "H6169": ("עָרָה", "H:N-F", "bulrush", "a.rah"),
        "H6170": ("עֲרוּגָה", "H:N-F", "bed", "a.ru.gah"),
        "H6171": ("עָרוֹד", "H:N-M", "wild donkey", "a.rod"),
        "H6172": ("עֶרְוָה", "H:N-F", "nakedness", "er.vah"),
        "H6173": ("עַרְוָה", "A:N-F", "dishonor", "ar.vah"),
        "H6174": ("עָרוֹם", "H:A", "naked", "a.rom"),
        "H6175": ("עָרוּם", "H:A", "prudent", "a.rum"),
        "H6176": ("עֲרוֹעֵר", "H:N-M", "juniper", "a.ro.er"),
        "H6177": ("עֲרֹעֵר, עֲדְעָדָה", "N:N--L", "Aroer", "a.ro.er"),
        "H6178": ("עָרוּץ", "H:A", "dreadful", "a.ruts"),
        "H6179": ("עֵרִי", "N:N-M-P", "Eri", "e.ri"),
        "H6180": ("עֵרִי", "N:N--PG", "Erite", "e.ri"),
        "H6181": ("עֶרְיָה", "H:N-F", "nakedness", "er.yah"),
        "H6182": ("עֲרִיסָה", "H:N-F", "dough", "a.ri.sah"),
        "H6183": ("עָרִיף", "H:N-M", "cloud", "a.riph"),
        "H6184": ("עָרִיץ", "H:A", "ruthless", "a.rits"),
        "H6185": ("עֲרִירִי", "H:A", "childless", "a.ri.ri"),
        "H6186a": ("עָרַךְ", "H:V", "to arrange", "a.rakh"),
        "H6186b": ("עָרַךְ", "H:V", "to value", "a.rakh"),
        "H6187": ("עֵ֫רֶךְ", "H:N-M", "valuation", "e.rekh"),
        "H6188": ("עָרַל", "H:V", "be uncircumcised", "a.rel"),
        "H6189": ("עָרֵל", "H:A", "uncircumcised", "a.rel"),
        "H6190": ("עׇרְלָה", "H:N-F", "foreskin", "or.lah"),
        "H6191": ("עָרֹם", "H:V", "be shrewd", "a.ram"),
        "H6192": ("עָרַם", "H:V", "to pile up", "a.ram"),
        "H6193": ("עֹ֫רֶם", "H:N-M", "craftiness", "o.rem"),
        "H6194": ("עֲרֵמָה", "H:N-F", "heap", "a.rem"),
        "H6195": ("עׇרְמָה", "H:N-F", "craftiness", "or.mah"),
        "H6196": ("עַרְמוֹן", "H:N-M", "plane tree", "ar.mon"),
        "H6197": ("עֵרָן", "N:N-M-P", "Eran", "e.ran"),
        "H6198": ("עֵרָנִי", "N:N--PG", "Eranite", "e.ra.ni"),
        "H6199": ("עַרְעָר", "H:A", "destitute", "a.r.ar"),
        "H6200": ("עֲרֹעֵרִי", "N:N--LG", "Aroerite", "a.ro.e.ri"),
        "H6201": ("עָרַף", "H:V", "to drop", "a.raph"),
        "H6202": ("עָרַף", "H:V", "to break the neck", "a.raph"),
        "H6203": ("עֹ֫רֶף", "H:N-M", "neck", "o.reph"),
        "H6204": ("עׇרְפָּה", "N:N-F-P", "Orpah", "or.pah"),
        "H6205": ("עֲרָפֶל", "H:N-M", "cloud", "a.ra.phel"),
        "H6206": ("עָרַץ", "H:V", "to tremble", "a.rats"),
        "H6207": ("עָרַק", "H:V", "to gnaw", "a.raq"),
        "H6208": ("עַרְקִי", "N:N--LG", "Arkite", "ar.qi"),
        "H6209": ("עָרַר", "H:V", "to strip", "a.rar"),
        "H6210": ("עֶ֫רֶשׂ", "H:N-F", "bed", "e.re.s"),
        "H6211a": ("עָשׁ", "H:N", "moth", "ash"),
        "H6211b": ("עֲשַׂב", "A:N", "grass", ""),
        "H6212": ("עֵ֫שֶׂב", "H:N-M", "vegetation", "e.sev"),
        "H6213a": ("עָשָׂה", "H:V", "to make", "a.sah"),
        "H6213b": ("עָשָׂה", "H:V", "to press", "a.sah"),
        "H6214": ("עֲשָׂהאֵל", "N:N-M-P", "Asahel", "a.sa.h.el"),
        "H6215": ("עֵשָׂו", "N:N-M-P", "Esau", "e.sav"),
        "H6216": ("עָשׁוֹק", "H:N-M", "oppressor", "a.shoq"),
        "H6217": ("עָשׁוּקִים", "H:N-M", "oppression", "a.shuq"),
        "H6218": ("עָשׂוֹר", "H:N-M", "ten", "a.s.vor"),
        "H6219": ("עָשׁוֹת", "H:A", "smooth", "a.shot"),
        "H6220": ("עַשְׁוָ֑ת", "N:N-M-P", "Ashvath", "ash.vat"),
        "H6221": ("עֲשִׂיאֵל", "N:N-M-P", "Asiel", "a.si.el"),
        "H6222": ("עֲשָׂיָה", "N:N-M-P", "Asaiah", "a.sa.yah"),
        "H6223": ("עָשִׁיר", "H:N-M", "rich", "a.shir"),
        "H6224": ("עֲשִׂירִי", "H:A", "tenth", "a.si.ri"),
        "H6225": ("עָשַׁן", "H:V", "be angry", "a.shan"),
        "H6226": ("עָשֵׁן", "H:A", "smoking", "a.shen"),
        "H6227": ("עָשָׁן", "H:N-M", "smoke", "a.shan"),
        "H6228": ("עָשָׁן", "N:N--L", "Ashan", "a.shan"),
        "H6229": ("עָשַׂק", "H:V", "to contend", "a.saq"),
        "H6230": ("עֵ֫שֶׂק", "N:N--L", "Esek", "e.seq"),
        "H6231": ("עָשַׁק", "H:V", "to oppress", "a.shaq"),
        "H6232": ("עֵ֫שֶׁק", "N:N-M-P", "Eshek", "e.sheq"),
        "H6233": ("עֹ֫שֶׁק", "H:N-M", "oppression", "o.sheq"),
        "H6234": ("עׇשְׁקָה", "H:N-F", "oppression", "osh.qah"),
        "H6235": ("עֶשֶׂר", "H:N", "ten", "e.ser"),
        "H6236": ("עֲשַׂר", "A:N", "ten", "a.sar"),
        "H6237": ("עָשַׂר", "H:V", "to tithe", "a.sar"),
        "H6238": ("עָשַׁר", "H:V", "to enrich", "a.shar"),
        "H6239": ("עֹ֫שֶׁר", "H:N-M", "riches", "o.sher"),
        "H6240": ("עָשָׂר", "H:N", "ten", "a.sar"),
        "H6241": ("עִשָּׂרוֹן", "H:N-M", "tenth", "is.sa.ron"),
        "H6242": ("עֶשְׂרִים", "H:N", "twenty", "es.rim"),
        "H6243": ("עֶשְׂרִין", "A:N", "twenty", "es.rin"),
        "H6244": ("עָשֵׁשׁ", "H:V", "to waste", "a.shesh"),
        "H6245a": ("עָשַׁת", "H:V", "to gleam", "a.shat"),
        "H6245b": ("עָשַׁת", "H:V", "to think", "a.shat"),
        "H6246": ("עֲשִׁת", "A:V", "to plan", "a.shit"),
        "H6247": ("עֶ֫שֶׁת", "H:N-M", "plate", "e.shet"),
        "H6248": ("עַשְׁתּוּת", "H:N-F", "thought", "ash.tut"),
        "H6249": ("עַשְׁתֵּי", "H:N", "eleven", "ash.te"),
        "H6250": ("עֶשְׁתֹּנָה", "H:N-F", "thought", "esh.to.nah"),
        "H6251": ("עַשְׁתָּרוֹת", "H:N-F", "young", "ash.te.rah"),
        "H6252": ("עַשְׁתָּרוֹת", "N:N-F-T / N:N--L", "Ashtaroth", "ash.ta.rot"),
        "H6253": ("עַשְׁתֹּ֫רֶת", "N:N-F-T", "Ashtoreth", "ash.to.ret"),
        "H6254": ("עַשְׁתְּרָתִי", "N:N--LG", "Ashterathite", "ash.te.ra.ti"),
        "H6255": ("עַשְׁתְּרֹת קַרְנַיִם", "N:N--L", "Ashteroth-karnaim", "ash.te.rot qar.na.yim"),
        "H6256": ("עֵת", "H:N-F", "time", "et"),
        "H6257": ("עָתֹד", "H:V", "be ready", "a.tad"),
        "H6258": ("עַתָּ֫ה", "H:Adv", "now", "at.tah"),
        "H6259": ("עָתוּד", "H:A", "ready", "a.tud"),
        "H6260": ("עַתּוּד", "H:N-M", "goat", "at.tud"),
        "H6261": ("עִתִּי", "H:A", "timely", "it.ti"),
        "H6262": ("עַתַּי", "N:N-M-P", "Attai", "at.tay"),
        "H6263": ("עֲתִיד", "A:A", "ready", "a.tid"),
        "H6264": ("עָתִיד", "H:A", "ready", "a.tid"),
        "H6265": ("עֲתָיָה", "N:N-M-P", "Athaiah", "a.ta.yah"),
        "H6266": ("עָתִיק", "H:A", "surpassing", "a.tiq"),
        "H6267": ("עַתִּיק", "H:A", "ancient/taken", "at.tiq"),
        "H6268": ("עַתִּיק", "A:A", "ancient", "at.tiq"),
        "H6269": ("עֲתָ֑ךְ", "N:N--L", "Athach", "a.takh"),
        "H6270": ("עַתְלַי", "N:N-M-P", "Athlai", "at.lay"),
        "H6271": ("עֲתַלְיָ֫הוּ", "N:N-M-P / N:N-F-P", "Athaliah", "a.tal.yah"),
        "H6272": ("עָתַם", "H:V", "to burn", "a.tam"),
        "H6273": ("עׇתְנִי", "N:N-M-P", "Othni", "ot.ni"),
        "H6274": ("עׇתְנִיאֵל", "N:N-M-P", "Othniel", "ot.ni.el"),
        "H6275": ("עָתַק", "H:V", "to proceed", "a.taq"),
        "H6276": ("עָתֵק", "H:A", "surpassing", "a.teq"),
        "H6277": ("עָתָק", "H:A", "arrogant", "a.taq"),
        "H6278": ("עֵת קָצִין", "N:N--L", "Eth-kazin", "et qa.tsin"),
        "H6279": ("עָתַר", "H:V", "to pray", "a.tar"),
        "H6280": ("עָתַר", "H:V", "be abundant", "a.tar"),
        "H6281": ("עֶ֫תֶר", "N:N--L", "Ether", "e.ter"),
        "H6282a": ("עָתָר", "H:N-M", "worshiper", "a.tar"),
        "H6282b": ("עָתָר", "H:N-M", "odour", "a.tar"),
        "H6283": ("עֲתֶ֫רֶת", "H:N-F", "abundance", "a.te.ret"),
        "H6284": ("פָּאָה", "H:V", "to cleave", "pa.ah"),
        "H6285": ("פֵּאָה", "H:N-F", "side", "pe.ah"),
        "H6286a": ("פָּאַר", "H:V", "to beautify", "pa.ar"),
        "H6286b": ("פִּאֵר", "H:V", "to re-harvest", ""),
        "H6287": ("פְּאֵר", "H:N-M", "headdress", "pe.er"),
        "H6288a": ("פֹּארָה", "H:N-F", "bough", "pe.o.rah"),
        "H6288b": ("פּוּרָה", "H:N-F", "bough", ""),
        "H6289": ("פָּארוּר", "H:N-M", "pale", "pa.rur"),
        "H6290": ("פָּארָן", "N:N--L", "Paran", "pa.ran"),
        "H6291": ("פַּגָּה", "H:N-M", "early fig", "pag"),
        "H6292": ("פִּגּוּל", "H:N-M", "refuse", "pig.gul"),
        "H6293": ("פָּגַע", "H:V", "to fall on", "pa.ga"),
        "H6294": ("פֶּ֫גַע", "H:N-M", "chance", "pe.ga"),
        "H6295": ("פַּגְעִיאֵל", "N:N-M-P", "Pagiel", "pag.i.el"),
        "H6296": ("פָּגַר", "H:V", "to faint", "pa.gar"),
        "H6297": ("פֶּ֫גֶר", "H:N-M", "corpse", "pe.ger"),
        "H6298": ("פָּגַשׁ", "H:V", "to meet", "pa.gash"),
        "H6299": ("פָּדָה", "H:V", "to ransom", "pa.dah"),
        "H6300": ("פְּדַהְאֵל", "N:N-M-P", "Pedahel", "pe.dah.el"),
        "H6301": ("פְּדָהצוּר", "N:N-M-P", "Pedahzur", "pe.da.h.tsur"),
        "H6302": ("פְּדוּיִם", "H:N-M", "ransomed", "pa.duy"),
        "H6303": ("פָּדוֹן", "N:N-M-P", "Padon", "pa.don"),
        "H6304": ("פְּדוּת", "H:N-F", "redemption", "pe.dut"),
        "H6305": ("פְּדָיָהוּ", "N:N-M-P", "Pedaiah", "pe.da.yah"),
        "H6306a": ("פִּדְיוֹם", "H:N-M", "redemption", "pid.yom"),
        "H6306b": ("פִּדְיוֹן", "H:N-M", "redemption", ""),
        "H6307": ("פַּדָּן", "N:N--L", "Paddan", "pad.dan"),
        "H6308": ("פָּדַע", "H:V", "to deliver", "pa.da"),
        "H6309": ("פֶּ֫דֶר", "H:N-M", "suet", "pe.der"),
        "H6310": ("פֶּה", "H:N-M", "lip", "peh"),
        "H6311": ("פֹּה", "H:Adv", "here", "poh"),
        "H6312": ("פּוּאָה", "N:N-M-P", "Puah", "pu.ah"),
        "H6313": ("פּוּג", "H:V", "be numb", "pug"),
        "H6314": ("פּוּגַת", "H:N-F", "cessation", "pu.gah"),
        "H6315": ("פּוּחַ", "H:V", "to breathe", "pu.ach"),
        "H6316": ("פּוּט", "N:N-M-P / N:N-M-PG", "Put", "put"),
        "H6317": ("פּוּטִיאֵל", "N:N-M-P", "Putiel", "pu.ti.el"),
        "H6318": ("פּוֹטִיפַר", "N:N-M-P", "Potiphar", "po.ti.phar"),
        "H6319": ("פּוֹטִי פֶ֫רַע", "N:N-M-P", "Potiphera", "po.ti phe.ra"),
        "H6320": ("פּוּךְ", "H:N-M", "color", "pukh"),
        "H6321": ("פּוֹל", "H:N-M", "bean", "pol"),
        "H6322": ("פּוּל", "N:N-M-P / N:N--L", "Pul", "pul"),
        "H6323": ("פּוּן", "H:V", "to distract", "pun"),
        "H6324": ("פּוּנִי", "N:N--PG", "Punite", "pu.ni"),
        "H6325": ("פּוּנֹן", "N:N--L", "Punon", "pu.non"),
        "H6326": ("פּוּעָה", "N:N-F-P", "Puah", "pu.ah"),
        "H6327a": ("פּוּץ", "H:V", "to scatter", "puts"),
        "H6327b": ("מֵפִיץ", "H:N-M", "scatterer", ""),
        "H6327c": ("פּוּץ", "H:V", "to flow", "puts"),
        "H6327d": ("פָּצַץ", "H:V", "to shatter", ""),
        "H6328": ("פּוּק", "H:V", "to reel", "puq"),
        "H6329": ("פּוּק", "H:V", "to promote", "puq"),
        "H6330": ("פּוּקָה", "H:N-F", "staggering", "pu.qah"),
        "H6331": ("פּוּר", "H:V", "to break", "pur"),
        "H6332": ("פּוּר", "H:N-M", "Purim", "pur"),
        "H6333": ("פּוּרָה", "H:N-F", "winepress", "pu.rah"),
        "H6334": ("פּוֹרָתָא", "N:N-M-P", "Poratha", "po.ra.ta"),
        "H6335a": ("פּוּשׁ", "H:V", "to leap", "push"),
        "H6335b": ("פּוּשׁ", "H:V", "to scatter", "push"),
        "H6336": ("פּוּתִי", "N:N--PG", "Puthite", "pu.ti"),
        "H6337": ("פַּז", "H:N-M", "pure gold", "paz"),
        "H6338": ("פָּזַז", "H:V", "be refined", "pa.zaz"),
        "H6339": ("פָּזַז", "H:V", "be agile", "pa.zaz"),
        "H6340": ("פָּזַר", "H:V", "to scatter", "pa.zar"),
        "H6341a": ("פַּח", "H:N-M", "snare", "pach"),
        "H6341b": ("פַּח", "H:N-M", "plate", "pach"),
        "H6342": ("פָּחַד", "H:V", "to dread", "pa.chad"),
        "H6343": ("פַּ֫חַד", "H:N-M", "dread", "pa.chad"),
        "H6344": ("פַּ֫חַד", "H:N-M", "thigh", "pa.chad"),
        "H6345": ("פַּחְדָּה", "H:N-F", "dread", "pach.dah"),
        "H6346": ("פֶּחָה", "H:N-M", "governor", "pe.chah"),
        "H6347": ("פֶּחָה", "A:N-M", "governor", "pe.chah"),
        "H6348": ("פָּחַז", "H:V", "be reckless", "pa.chaz"),
        "H6349": ("פַּ֫חַז", "H:N-M", "recklessness", "pa.chaz"),
        "H6350": ("פַּחֲזוּת", "H:N-F", "recklessness", "pa.cha.zut"),
        "H6351": ("פָּחַח", "H:V", "to ensnare", "pa.chach"),
        "H6352": ("פֶּחָם", "H:N-M", "coal", "pe.cham"),
        "H6353": ("פֶּחָר", "A:N-M", "potter", "pe.char"),
        "H6354": ("פַּ֫חַת", "H:N-M", "pit", "pa.chat"),
        "H6355": ("פַּ֫חַת מוֹאָב", "N:N-M-P", "Pahath-moab", "pa.chat mo.av"),
        "H6356": ("פְּחֶ֫תֶת", "H:N-F", "eating", "pe.che.tet"),
        "H6357": ("פִּטְדָה", "H:N-F", "topaz", "pit.dah"),
        "H6358": ("פָּטוּר", "H:V", "to_open", "pa.tur"),
        "H6359": ("פָּטִיר", "H:A", "free", "pa.tir"),
        "H6360": ("פַּטִּישׁ", "H:N-M", "hammer", "pat.tish"),
        "H6361": ("פְּטַשׁ", "A:N-M", "legging", "pat.tish"),
        "H6362": ("פָּטַר", "H:V", "to separate", "pa.tar"),
        "H6363a": ("פֶּ֫טֶר", "H:N-M", "firstborn", "pe.ter"),
        "H6363b": ("פִּטְרָה", "H:N-M", "firstborn", ""),
        "H6364": ("פִּי־בֶ֫סֶת", "N:N--L", "Pi-beseth", "pi-ve.set"),
        "H6365": ("פִּיד", "H:N-M", "disaster", "pid"),
        "H6366": ("פֵּיָה", "H:N-F", "two-edged", "pe.yah"),
        "H6367": ("פִּי הַחִירֹת", "N:N--L", "Pi-hahiroth", "pi ha.chi.rot"),
        "H6368": ("פִּיחַ", "H:N-M", "soot", "pi.ach"),
        "H6369": ("פִּיכֹל", "N:N-M-P", "Phicol", "pi.khol"),
        "H6370": ("פִּלֶ֫גֶשׁ", "H:N-F", "concubine", "pi.le.gesh"),
        "H6371": ("פִּימָה", "H:N-F", "excess fat", "pi.mah"),
        "H6372": ("פִּינְחָס", "N:N-M-P", "Phinehas", "pin.chas"),
        "H6373": ("פִּינֹן", "N:N-M-P", "Pinon", "pi.non"),
        "H6374": ("פִּיפִיָּה", "H:N-F", "tooth", "pi.phiy.yah"),
        "H6375": ("פִּיק", "H:N-M", "tottering", "piq"),
        "H6376": ("פִּישׁוֹן", "N:N--L", "Pishon", "pi.shon"),
        "H6377": ("פִּיתוֹן", "N:N-M-P", "Pithon", "pi.ton"),
        "H6378": ("פַּךְ", "H:N-M", "vial", "pakh"),
        "H6379": ("פָּכָה", "H:V", "to trickle", "pa.khah"),
        "H6380": ("פֹּכֶ֫רֶת הַצְבָיִם", "N:N-M-P", "Pochereth-hazzebaim", "po.khe.ret tse.va.yim"),
        "H6381": ("פָּלָא", "H:V", "to wonder", "pa.la"),
        "H6382": ("פֶּ֫לֶא", "H:N-M", "wonder", "pe.le"),
        "H6383": ("פִּלְאִי", "H:A", "incomprehensible", "pil.i"),
        "H6384": ("פַּלֻּאִי", "N:N--PG", "Palluite", "pal.lu.i"),
        "H6385": ("פָּלַג", "H:V", "to divide", "pa.lag"),
        "H6386": ("פְּלַג", "A:V", "to divide", "pe.lag"),
        "H6387": ("פְּלַג", "A:N-M", "half", "pe.lag"),
        "H6388": ("פֶּ֫לֶג", "H:N-M", "stream", "pe.leg"),
        "H6389": ("פֶּ֫לֶג", "N:N-M-P", "Peleg", "pe.leg"),
        "H6390": ("פְּלַגָּה", "H:N-F", "stream", "pe.lag.gah"),
        "H6391": ("פְּלֻגָּה", "H:N-F", "division", "pe.lug.gah"),
        "H6392": ("פְּלֻגָּה", "A:N-F", "division", "pe.lug.gah"),
        "H6393": ("פְּלָדָה", "H:N-F", "iron", "pe.la.dah"),
        "H6394": ("פִּלְדָּשׁ", "N:N-M-P", "Pildash", "pil.dash"),
        "H6395": ("פָּלָה", "H:V", "be distinguished", "pa.lah"),
        "H6396": ("פַּלּוּא", "N:N-M-P", "Pallu", "pal.lu"),
        "H6397": ("פְּלוֹנִי", "N:N--LG", "Pelonite", "pe.lo.ni"),
        "H6398": ("פָּלַח", "H:V", "to cleave", "pa.lach"),
        "H6399": ("פְּלַח", "A:V", "to serve", "pe.lach"),
        "H6400": ("פֶּ֫לַח", "H:N-F", "millstone", "pe.lach"),
        "H6401": ("פִּלְחָא", "N:N-M-P", "Pilha", "pil.ach"),
        "H6402": ("פׇּלְחָן", "A:N-M", "service", "pol.chan"),
        "H6403": ("פָּלַט", "H:V", "to escape", "pa.lat"),
        "H6404": ("פֶּ֫לֶט", "N:N-M-P", "Pelet", "pe.let"),
        "H6405": ("פַּלֵּט", "H:N-M", "deliverance", "pal.let"),
        "H6406": ("פַּלְטִי", "N:N-M-P", "Palti", "pal.ti"),
        "H6407": ("פַּלְטִי", "N:N--PG", "Paltite", "pal.ti"),
        "H6408": ("פִּלְטָ֑י", "N:N-M-P", "Piltai", "pil.tay"),
        "H6409": ("פַּלְטִיאֵל", "N:N-M-P", "Paltiel", "pal.ti.el"),
        "H6410": ("פְּלַטְיָ֫הוּ", "N:N-M-P", "Pelatiah", "pe.lat.yah"),
        "H6411a": ("פְּלָאיָה", "N:N-M-P", "Pelaiah", "pe.la.yah"),
        "H6411b": ("פְּלָיָה", "N:N-M-P", "Pelaiah", "pe.la.yah"),
        "H6412a": ("פָּלִיט", "H:N-M", "survivor", "pa.lit"),
        "H6412b": ("פָּלֵיט", "H:N-M", "survivor", ""),
        "H6413": ("פְּלֵיטָה", "H:N-F", "survivor", "pe.le.tah"),
        "H6414": ("פָּלִיל", "H:N-M", "judge", "pa.lil"),
        "H6415": ("פְּלִילָה", "H:N-F", "judge", "pe.li.lah"),
        "H6416": ("פְּלִילִי", "H:A", "judge", "pe.li.li"),
        "H6417": ("פְּלִילִיָּה", "H:N-F", "judgment", "pe.li.liy.yah"),
        "H6418": ("פֶּ֫לֶךְ", "H:N-M", "district", "pe.lekh"),
        "H6419": ("פָּלַל", "H:V", "to pray", "pa.lal"),
        "H6420": ("פָּלָל", "N:N-M-P", "Palal", "pa.lal"),
        "H6421": ("פְּלַלְיָה", "N:N-M-P", "Pelaliah", "pe.lal.yah"),
        "H6422": ("פַּלְמוֹנִי", "H:IndP", "certain", "pal.mo.ni"),
        "H6423": ("פְּלֹנִי", "H:IndP", "certain", "pe.lo.ni"),
        "H6424": ("פָּלַס", "H:V", "to envy", "pa.las"),
        "H6425": ("פֶּ֫לֶס", "H:N-M", "balance", "pe.les"),
        "H6426": ("פָּלַץ", "H:V", "to shudder", "pa.lats"),
        "H6427": ("פַּלָּצוּת", "H:N-F", "shuddering", "pal.la.tsut"),
        "H6428": ("פָּלַשׁ", "H:V", "to wallow", "pa.lash"),
        "H6429": ("פְּלֶ֫שֶׁת", "N:N--L", "Palestina", "pe.le.shet"),
        "H6430": ("פְּלִשְׁתִּי", "N:N--LG", "Philistine", "pe.lish.ti"),
        "H6431": ("פֶּ֫לֶת", "N:N-M-P", "Peleth", "pe.let"),
        "H6432": ("פְּלֵתִי", "N:N--TG", "Pelethite", "pe.le.ti"),
        "H6433": ("פֻּם", "A:N-M", "mouth", "pum"),
        "H6434": ("פֵּן", "H:N-M", "corner", "pen"),
        "H6435": ("פֶּן־", "H:Conj", "lest", "pen"),
        "H6436": ("פַּנַּג", "H:N-M", "cake", "pan.nag"),
        "H6437": ("פָּנָה", "H:V", "to turn", "pa.nah"),
        "H6438": ("פִּנָּה", "H:N-F", "corner", "pin.nah"),
        "H6439": ("פְּנוּאֵל", "N:N-M-P / N:N--L", "Peniel", "pe.nu.el"),
        "H6440": ("פָּנֶה", "H:N-M", "face", "pa.nim"),
        "H6441": ("פְּנִ֫ימָה", "H:Adv", "within", "pe.ni.mah"),
        "H6442": ("פְּנִימִי", "H:A", "inner", "pe.ni.mi"),
        "H6443": ("פְּנִינִים", "H:N-M", "jewel", "pa.nin"),
        "H6444": ("פְּנִנָּה", "N:N-F-P", "Peninnah", "pe.nin.nah"),
        "H6445": ("פָּנַק", "H:V", "to pamper", "pa.naq"),
        "H6446": ("פַּס", "H:N-M", "long-sleeved", "pas"),
        "H6447": ("פַּס", "A:N-M", "palm", "pas"),
        "H6448": ("פָּסַג", "H:V", "to go through", "pa.sag"),
        "H6449": ("פִּסְגָּה", "N:N--L", "Pisgah", "pis.gah"),
        "H6450": ("פַּס דַּמִּים", "N:N--L", "Pas-dammim", "pas dam.mim"),
        "H6451": ("פִּסָּה", "H:N-F", "abundance", "pis.sah"),
        "H6452a": ("פָּסַח", "H:V", "to pass", "pa.sach"),
        "H6452b": ("פָּסַח", "H:V", "to limp", "pa.sach"),
        "H6453": ("פֶּ֫סַח", "H:N-M", "Passover", "pe.sach"),
        "H6454": ("פָּסֵחַ", "N:N-M-P", "Paseah", "pa.se.ach"),
        "H6455": ("פִּסֵּחַ", "H:A", "lame", "pis.se.ach"),
        "H6456": ("פָּסִיל", "H:N-M", "idol", "pe.sil"),
        "H6457": ("פָּסַךְ", "N:N-M-P", "Pasach", "pa.sakh"),
        "H6458": ("פָּסַל", "H:V", "to hew", "pa.sal"),
        "H6459": ("פֶּ֫סֶל", "H:N-M", "idol", "pe.sel"),
        "H6460": ("פְּסַנְטֵרִין", "A:N-M", "psaltery", "pe.san.te.rin"),
        "H6461": ("פָּסַס", "H:V", "to disappear", "pa.sas"),
        "H6462": ("פִּסְפָּה", "N:N-M-P", "Pispa", "pis.pah"),
        "H6463": ("פָּעָה", "H:V", "to groan", "pa.ah"),
        "H6464": ("פָּ֑עוּ", "N:N--L", "Pai", "pa.u"),
        "H6465": ("פְּעוֹר", "N:N--L / N:N-M-T", "Peor", "pe.or"),
        "H6466": ("פָּעַל", "H:V", "to work", "pa.al"),
        "H6467": ("פֹּ֫עַל", "H:N-M", "work", "po.al"),
        "H6468": ("פְּעֻלָּה", "H:N-F", "wages", "pe.ul.lah"),
        "H6469": ("פְּעֻלְּתַי", "N:N-M-P", "Peullethai", "pe.ul.l.tay"),
        "H6470": ("פָּעַם", "H:V", "to trouble", "pa.am"),
        "H6471": ("פַּ֫עַם", "H:N-F", "beat", "pa.am"),
        "H6472": ("פַּעֲמֹן", "H:N-M", "bell", "pa.a.mon"),
        "H6473": ("פָּעַר", "H:V", "to open", "pa.ar"),
        "H6474": ("פַּעֲרַי", "N:N-M-P", "Paarai", "pa.a.ray"),
        "H6475": ("פָּצָה", "H:V", "to open", "pa.tsah"),
        "H6476": ("פָּצַח", "H:V", "to break out", "pa.tsach"),
        "H6477": ("פְּצִירָה", "H:N-F", "bluntness", "pe.tsi.rah"),
        "H6478": ("פָּצַל", "H:V", "to peel", "pa.tsal"),
        "H6479": ("פִּצְלָה", "H:N-F", "stripe", "pe.tsa.lah"),
        "H6480": ("פָּצַם", "H:V", "to split open", "pa.tsam"),
        "H6481": ("פָּצַע", "H:V", "to wound", "pa.tsa"),
        "H6482": ("פֶּ֫צַע", "H:N-M", "wound", "pe.tsa"),
        "H6483": ("פִּצֵּץ", "N:N-M-P", "Happizzez", "pits.tsets"),
        "H6484": ("פָּצַר", "H:V", "to press", "pa.tsar"),
        "H6485a": ("פָּקַד", "H:V", "to reckon", "pa.qad"),
        "H6485b": ("פְּקוּדִים", "H:N-M", "reckoning", ""),
        "H6486": ("פְּקֻדָּה", "H:N-F", "punishment", "pe.qud.dah"),
        "H6487": ("פִּקָּדוֹן", "H:N-M", "deposit", "piq.qa.don"),
        "H6488": ("פְּקִדֻת", "H:N-F", "oversight", "pe.qi.dut"),
        "H6489": ("פְּקוֹד", "N:N--L", "Pekod", "pe.qod"),
        "H6490": ("פִּקּוּד", "H:N-M", "precept", "piq.qud"),
        "H6491": ("פָּקַח", "H:V", "to open", "pa.qach"),
        "H6492": ("פֶּ֫קַח", "N:N-M-P", "Pekah", "pe.qach"),
        "H6493": ("פִּקֵּחַ", "H:A", "seeing", "piq.qe.ach"),
        "H6494": ("פְּקַחְיָה", "N:N-M-P", "Pekahiah", "pe.qach.yah"),
        "H6495": ("פְּקַח־קוֹחַ", "H:N-M", "opening", "pe.qach-qo.ach"),
        "H6496": ("פָּקִיד", "H:N-M", "overseer", "pa.qid"),
        "H6497": ("פֶּקָעִים", "H:N-M", "gourd", "pe.qa"),
        "H6498": ("פַּקֻּעֹת", "H:N-F", "gourd", "paq.qu.ah"),
        "H6499": ("פַּר", "H:N-M", "bullock", "par"),
        "H6500": ("פָּרָא", "H:V", "be fruitful", "pa.ra"),
        "H6501": ("פֶּ֫רֶא", "H:N-M", "wild donkey", "pe.re"),
        "H6502": ("פִּרְאָם", "N:N-M-P", "Piram", "pir.am"),
        "H6503": ("פַּרְוָר", "N:N--L", "Parbar", "par.bar"),
        "H6504": ("פָּרַד", "H:V", "to separate", "pa.rad"),
        "H6505": ("פֶּ֫רֶד", "H:N-M", "mule", "pe.red"),
        "H6506": ("פִּרְדָּה", "H:N-F", "female mule", "pir.dah"),
        "H6507": ("פְּרֻדָה", "H:N-F", "grain", "pe.ru.dah"),
        "H6508": ("פַּרְדֵּס", "H:N-M", "park", "par.des"),
        "H6509": ("פָּרָה", "H:V", "be fruitful", "pa.rah"),
        "H6510": ("פָּרָה", "H:N-F", "heifer", "pa.rah"),
        "H6511": ("פָּרָה", "N:N--L", "Parah", "pa.rah"),
        "H6512": ("פֵּרָה", "H:N-F", "mole", "pe.rah"),
        "H6513": ("פֻּרָה", "N:N-M-P", "Purah", "pu.rah"),
        "H6514": ("פְּרוּדָא", "N:N-M-P", "Perida", "pe.ru.da"),
        "H6515": ("פָּרוּחַ", "N:N-M-P", "Paruah", "pa.ru.ach"),
        "H6516": ("פַּרְוָ֑יִם", "N:N--L", "Parvaim", "par.va.yim"),
        "H6517": ("פָּרוּר", "H:N-M", "pot", "pa.rur"),
        "H6518": ("פֶּ֫רֶז", "H:N-M", "warrior", "pa.raz"),
        "H6519": ("פְּרָזָה", "H:N-F", "village", "pe.ra.zah"),
        "H6520": ("פְּרָזוֹן", "H:N-M", "villager", "pe.ra.zon"),
        "H6521": ("פְּרָזִי", "H:N-M", "villager", "pe.ra.zi"),
        "H6522": ("פְּרִזִּי", "N:N--LG", "Perizzite", "pe.riz.zi"),
        "H6523": ("פַּרְזֶל", "A:N-M", "iron", "par.zel"),
        "H6524a": ("פָּרַח", "H:V", "to sprout", "pa.rach"),
        "H6524b": ("פָּרַח", "H:V", "to break out", "pa.rach"),
        "H6524c": ("פָּרַח", "H:V", "to fly", "pa.rach"),
        "H6525": ("פֶּ֫רַח", "H:N-M", "flower", "pe.rach"),
        "H6526": ("פִּרְחָח", "H:N-M", "brood", "pir.chach"),
        "H6527": ("פָּרַט", "H:V", "to divide", "pa.rat"),
        "H6528": ("פֶּ֫רֶט", "H:N-M", "broken", "pe.ret"),
        "H6529": ("פְּרִי", "H:N-M", "fruit", "pe.ri"),
        "H6530": ("פָּרִיץ", "H:N-M", "violent", "pe.rits"),
        "H6531": ("פֶּ֫רֶךְ", "H:N-M", "severity", "pe.rekh"),
        "H6532": ("פָּרֹ֫כֶת", "H:N-F", "curtain", "po.re.khet"),
        "H6533": ("פָּרַם", "H:V", "to tear", "pa.ram"),
        "H6534": ("פַּרְמַשְׁתָּא", "N:N-M-P", "Parmashta", "par.mash.ta"),
        "H6535": ("פַּרְנָךְ", "N:N-M-P", "Parnach", "par.nakh"),
        "H6536": ("פָּרַס", "H:V", "to divide", "pa.ras"),
        "H6537a": ("פְּרַס", "A:V", "to divide", "pe.ras"),
        "H6537b": ("פְּרֵס", "A:N-M", "half", ""),
        "H6538": ("פֶּ֫רֶס", "H:N-M", "vulture", "pe.res"),
        "H6539": ("פָּרַס", "N:N--L / N:N--LG", "Persia", "pa.ras"),
        "H6540": ("פָּרַס", "N:N--L / N:N--LG", "Persian", "pa.ras"),
        "H6541": ("פַּרְסָה", "H:N-F", "hoof", "par.sah"),
        "H6542": ("פַּרְסִי, פָּרַס", "N:N--LG", "Persian", "par.si"),
        "H6543": ("פָּרְסָאָה", "N:N--LG", "Persian", "par.si"),
        "H6544a": ("פָּרַע", "H:V", "to lead", "pa.ra"),
        "H6544b": ("פָּרַע", "H:V", "to neglect", "pa.ra"),
        "H6545": ("פֶּ֫רַע", "H:N-M", "lock", "pe.ra"),
        "H6546": ("פֶּ֫רַע", "H:N-M", "leader", "par.ah"),
        "H6547": ("פַּרְעֹה", "N:N--T", "Pharaoh", "par.oh"),
        "H6548": ("חָפְרַע", "N:N-M-P", "Hophra", "par.oh choph.ra"),
        "H6549": ("פַּרְעֹה נְכֹה", "N:N-M-P", "Pharaoh-neco", "par.oh ne.khoh"),
        "H6550": ("פַּרְעֹשׁ", "H:N-M", "flea", "par.osh"),
        "H6551": ("פַּרְעֹשׁ", "N:N-M-P", "Parosh", "par.osh"),
        "H6552": ("פִּרְעָתוֹן", "N:N--L", "Pirathon", "pir.a.ton"),
        "H6553": ("פִּרְעָתוֹנִי", "N:N--LG", "Pirathonite", "pir.a.to.ni"),
        "H6554": ("פַּרְפַּר", "N:N--L", "Pharpar", "par.par"),
        "H6555": ("פָּרַץ", "H:V", "to break through", "pa.rats"),
        "H6556": ("פֶּ֫רֶץ", "H:N-M", "breach", "pe.rets"),
        "H6557": ("פֶּ֫רֶץ", "N:N-M-P", "Perez", "pe.rets"),
        "H6558": ("פַּרְצִי", "N:N--PG", "Perezite", "par.tsi"),
        "H6559": ("פְּרָצִים", "N:N--L", "Perazim", "pe.ra.tsim"),
        "H6560": ("פֶּרֶץ עֻזָּא", "N:N--L", "Perez-uzza", "pe.rets uz.za"),
        "H6561": ("פָּרַק", "H:V", "to tear", "pa.raq"),
        "H6562": ("פְּרַק", "A:V", "to break", "pe.raq"),
        "H6563": ("פֶּ֫רֶק", "H:N-M", "plunder", "pe.req"),
        "H6564": ("פָּרָק", "H:N-M", "broth", "pa.raq"),
        "H6565a": ("פָּרַר", "H:V", "to break", "pa.rar"),
        "H6565b": ("פָּרַר", "H:V", "to split", "pa.rar"),
        "H6566": ("פָּרַשׂ", "H:V", "to spread", "pa.ra.s"),
        "H6567a": ("פָּרַשׁ", "H:V", "to declare", ""),
        "H6567b": ("פָּרַשׁ", "H:V", "to pierce", ""),
        "H6568": ("פְּרַשׁ", "A:V", "to make clear", "pe.rash"),
        "H6569": ("פֶּ֫רֶשׁ", "H:N-M", "refuse", "pe.resh"),
        "H6570": ("פֶּ֫רֶשׁ", "N:N-M-P", "Peresh", "pe.resh"),
        "H6571a": ("פָּרָשׁ", "H:N-M", "horse", "pa.rash"),
        "H6571b": ("פָּרָשׁ", "H:N-M", "horseman", "pa.rash"),
        "H6572a": ("פַּרְשֶׁ֫גֶן", "H:N-M", "copy", "par.she.gen"),
        "H6572b": ("פַּתְשֶׁ֫גֶן", "H:N-M", "copy", ""),
        "H6573": ("פַּרְשֶׁ֫גֶן", "A:N-M", "copy", "par.she.gen"),
        "H6574": ("פַּרְשְׁדֹ֑נָה", "H:N-M", "refuse", "par.she.don"),
        "H6575": ("פָּרָשָׁה", "H:N-F", "declaration", "pa.ra.shah"),
        "H6576": ("פַּרְשֵׁז", "H:V", "to spread", "par.shez"),
        "H6577": ("פַּרְשַׁנְדָּתָא", "N:N-M-P", "Parshandatha", "par.shan.da.ta"),
        "H6578": ("פְּרָת", "N:N--L", "Euphrates", "pe.rat"),
        "H6579": ("פַּרְתְּמִים", "H:N-M", "noble", "par.tam"),
        "H6580": ("פַּשׁ", "H:N-M", "folly", "pash"),
        "H6581": ("פָּשָׂה", "H:V", "to spread", "pa.sah"),
        "H6582": ("פָּשַׁח", "H:V", "to tear", "pa.shach"),
        "H6583": ("פַּשְׁחוּר", "N:N-M-P", "Pashhur", "pash.chur"),
        "H6584": ("פָּשַׁט", "H:V", "to strip", "pa.shat"),
        "H6585": ("פָּשַׂע", "H:V", "to step", "pa.sa"),
        "H6586": ("פָּשַׁע", "H:V", "to transgress", "pa.sah"),
        "H6587": ("פֶּ֫שַׂע", "H:N-M", "step", "pe.sa"),
        "H6588": ("פֶּ֫שַׁע", "H:N-M", "transgression", "pe.sah"),
        "H6589": ("פָּשַׂק", "H:V", "to open", "pa.saq"),
        "H6590": ("פְּשַׁר", "A:V", "to interpret", "pe.shar"),
        "H6591": ("פְּשַׁר", "A:N-M", "interpretation", "pe.shar"),
        "H6592": ("פֵּ֫שֶׁר", "H:N-M", "interpretation", "pe.sher"),
        "H6593": ("פֵּ֫שֶׁת", "H:N-F", "flax", "pish.teh"),
        "H6594": ("פִּשְׁתָּה", "H:N-F", "flax", "pish.tah"),
        "H6595a": ("פַּת", "H:N-F", "morsel", "pat"),
        "H6595b": ("פָּתוֹת", "H:N-F", "morsel", ""),
        "H6596": ("פֹּת", "H:N-F", "hinge", "pot"),
        "H6597": ("פִּתְאֹם", "H:Adv", "suddenly", "pit.om"),
        "H6598": ("פַּת־בָּג", "H:N-M", "choice", "pat.bag"),
        "H6599": ("פִּתְגָּם", "H:N-M", "edict", "pit.gam"),
        "H6600": ("פִּתְגָם", "A:N-M", "edict", "pit.gam"),
        "H6601a": ("פָּתָה", "H:V", "to open wide", "pa.tah"),
        "H6601b": ("פָּתָה", "H:V", "to entice", "pa.tah"),
        "H6602": ("פְּתוּאֵל", "N:N-M-P", "Pethuel", "pe.tu.el"),
        "H6603": ("פִּתּוּחַ", "H:N-M", "engraving", "pit.tu.ach"),
        "H6604": ("פְּתוֹר", "N:N--L", "Pethor", "pe.tor"),
        "H6605a": ("פָּתַח", "H:V", "to open", "pa.tach"),
        "H6605b": ("פָּתַח", "H:V", "to engrave", "pa.tach"),
        "H6606": ("פְּתַח", "A:V", "to open", "pe.tach"),
        "H6607": ("פֶּ֫תַח", "H:N-M", "entrance", "pe.tach"),
        "H6608": ("פֵּ֫תַח", "H:N-M", "opening", "pe.tach"),
        "H6609": ("פְּתִיחָה", "H:N-F", "drawn sword", "pe.ti.chah"),
        "H6610": ("פִּתְחוֹן", "H:N-M", "opening", "pit.chon"),
        "H6611": ("פְּתַחְיָה", "N:N-M-P", "Pethahiah", "pe.tach.yah"),
        "H6612a": ("פֶּ֫תִי", "H:A", "simple", "pe.ti"),
        "H6612b": ("פְּתִי", "H:N-F", "simplicity", "pe.ti"),
        "H6613": ("פְּתָי", "A:N-M", "breadth", "pe.tay"),
        "H6614": ("פְּתִיגִיל", "H:N-M", "robe", "pe.ti.gil"),
        "H6615": ("פְּתַיּוּת", "H:N-F", "naivite", "pe.tay.yut"),
        "H6616": ("פָּתִיל", "H:N-M", "cord", "pa.til"),
        "H6617": ("פָּתַל", "H:V", "to twist", "pa.tal"),
        "H6618": ("פְּתַלְתֹּל", "H:A", "twisted", "pe.tal.tol"),
        "H6619": ("פִּתֹם", "N:N--L", "Pithom", "pi.tom"),
        "H6620": ("פֶּ֫תֶן", "H:N-M", "cobra", "pe.ten"),
        "H6621": ("פֶּתַע", "H:N-M", "suddenness", "pe.ta"),
        "H6622": ("פָּתַר", "H:V", "to interpret", "pa.tar"),
        "H6623": ("פִּתְרוֹן", "H:N-M", "interpretation", "pit.ron"),
        "H6624": ("פַּתְרוֹס", "N:N--L", "Pathros", "pat.ros"),
        "H6625": ("פַּתְרֻסִים", "N:N--LG", "Pathrusim", "pat.ru.si"),
        "H6626": ("פָּתַת", "H:V", "to break", "pa.tat"),
        "H6627": ("צֵאָה", "H:N-F", "excrement", "tsa.ah"),
        "H6628": ("צֶאֱלִים", "H:N-M", "lotus", "tse.el"),
        "H6629": ("צֹאן", "H:N-F", "flock", "tson"),
        "H6630": ("צַאֲנָן", "N:N--L", "Zaanan", "tsa.a.nan"),
        "H6631": ("צֶאֱצָא", "H:N-M", "offspring", "tse.e.tsa"),
        "H6632a": ("צָב", "H:N-M", "litter", "tsav"),
        "H6632b": ("צָב", "H:N-M", "lizard", "tsav"),
        "H6633": ("צָבָא", "H:V", "to serve", "tsa.va"),
        "H6634": ("צְבָא", "A:V", "to will", "tse.va"),
        "H6635a": ("צָבָא", "H:N-M", "army", "tsa.va"),
        "H6635b": ("צָבָא", "N:N-M-T", "Hosts", "tsa.va"),
        "H6636": ("צְבֹיִים", "N:N--L", "Zeboiim", "tse.vo.im"),
        "H6637": ("צֹבֵבָה", "N:N-M-P", "Zobebah", "tso.ve.vah"),
        "H6638": ("צָבָה", "H:V", "to swell", "tsa.vah"),
        "H6639": ("צָבֶה", "H:A", "swollen", "tsa.veh"),
        "H6640": ("צְבוּ", "A:N-M", "thing", "tse.vu"),
        "H6641": ("צָבוּעַ", "H:A", "variegated", "tsa.vu.a"),
        "H6642": ("צָבַט", "H:V", "to serve", "tsa.vat"),
        "H6643a": ("צְבִי", "H:N-M", "beauty", "tse.vi"),
        "H6643b": ("צְבִי", "H:N-M", "gazelle", "tse.vi"),
        "H6644": ("צִבְיָא", "N:N-M-P", "Zibia", "tsiv.ya"),
        "H6645": ("צִבְיָה", "N:N-F-P", "Zibiah", "tsiv.y.yah"),
        "H6646": ("צְבִיָּה", "H:N-F", "gazelle", "tse.viy.yah"),
        "H6647": ("צְבַע", "A:V", "to drench", "tse.va"),
        "H6648": ("צֶ֫בַע", "H:N-M", "dye", "tse.va"),
        "H6649": ("צִבְעוֹן", "N:N-M-P", "Zibeon", "tsiv.on"),
        "H6650": ("צְבֹעִים", "N:N--L", "Zeboim", "tse.vo.im"),
        "H6651": ("צָבַר", "H:V", "to heap", "tsa.var"),
        "H6652": ("צִבּוּר", "H:N-M", "heap", "tsib.bur"),
        "H6653": ("צְבָתִים", "H:N-M", "bundle", "tse.vet"),
        "H6654": ("צַד", "H:N-M", "side", "tsad"),
        "H6655": ("צַד", "A:N-M", "side", "tsad"),
        "H6656": ("צְדָא", "A:N-M", "intended", "tse.da"),
        "H6657": ("צְדָד", "N:N--L", "Zedad", "tse.dad"),
        "H6658a": ("צָדָה", "H:V", "to ambush", "tsa.dah"),
        "H6658b": ("צָדָה", "H:V", "to waste", "tsa.dah"),
        "H6659": ("צָדוֹק", "N:N-M-P", "Zadok", "tsa.doq"),
        "H6660": ("צְדִיָּה", "H:N-F", "ambush", "tse.diy.yah"),
        "H6661": ("צִדִּים", "N:N--L", "Ziddim", "tsid.dim"),
        "H6662": ("צַדִּיק", "H:A", "righteous", "tsad.diq"),
        "H6663": ("צָדֵק", "H:V", "to justify", "tsa.daq"),
        "H6664": ("צֶ֫דֶק", "H:N-M", "righteousness", "tse.deq"),
        "H6665": ("צִדְקָה", "A:N-F", "righteousness", "tsid.qah"),
        "H6666": ("צְדָקָה", "H:N-F", "righteousness", "tse.da.qah"),
        "H6667": ("צִדְקִיָּ֫הוּ", "N:N-M-P", "Zedekiah", "tsid.qiy.yah"),
        "H6668": ("צָהֵב", "H:V", "to gleam", "tsa.hav"),
        "H6669": ("צָהֹב", "H:A", "yellow", "tsa.hov"),
        "H6670a": ("צָהַל", "H:V", "to cry out", "tsa.hal"),
        "H6670b": ("צָהַל", "H:V", "to shine", "tsa.hal"),
        "H6671": ("צָהַר", "H:V", "to press", "tsa.har"),
        "H6672a": ("צֹ֫הַר", "H:N-M", "midday", "tso.har"),
        "H6672b": ("צֹ֫הַר", "H:N-F", "roof", "tso.har"),
        "H6673": ("צַו", "H:N-M", "command", "tsav"),
        "H6674": ("צוֹא", "H:A", "filthy", "tso"),
        "H6675": ("צֹאָה", "H:N-F", "filth", "tso.ah"),
        "H6676": ("צַוַּאר", "A:N-M", "neck", "tsav.var"),
        "H6677a": ("צַוָּאר", "H:N-M", "neck", "tsav.var"),
        "H6677b": ("צַוָּרוֹן", "H:N-M", "necklace", ""),
        "H6678": ("צוֹבָא", "N:N--L", "Zobah", "tso.va"),
        "H6679a": ("צוּד", "H:V", "to hunt", "tsud"),
        "H6679b": ("צוּד", "H:V", "to provision", "tsud"),
        "H6680": ("צָוָה", "H:V", "to command", "tsa.vah"),
        "H6681": ("צָוַח", "H:V", "to cry aloud", "tsa.vach"),
        "H6682": ("צְוָחָה", "H:N-F", "outcry", "tse.va.chah"),
        "H6683": ("צוּלָה", "H:N-F", "deep", "tsu.lah"),
        "H6684": ("צוּם", "H:V", "to fast", "tsum"),
        "H6685": ("צוֹם", "H:N-M", "fast", "tsom"),
        "H6686": ("צוּעָר", "N:N-M-P", "Zuar", "tsu.ar"),
        "H6687": ("צוּף", "H:V", "to flow", "tsuph"),
        "H6688": ("צוּף", "H:N-M", "honeycomb", "tsuph"),
        "H6689": ("צוּף", "N:N-M-P / N:N--L", "Zuph", "tsuph"),
        "H6690": ("צוֹפַח", "N:N-M-P", "Zophah", "tso.phach"),
        "H6691": ("צוֹפַר", "N:N-M-P", "Zophar", "tso.phar"),
        "H6692a": ("צוּץ", "H:V", "to blossom", "tsuts"),
        "H6692b": ("צוּץ", "H:V", "to gaze", "tsuts"),
        "H6693": ("צוּק", "H:V", "to press", "tsuq"),
        "H6694": ("צוּק", "H:V", "to pour", "tsuq"),
        "H6695a": ("צוֹק", "H:N-M", "distress", "tsoq"),
        "H6695b": ("צוּקָה", "H:N-F", "anguish", ""),
        "H6696a": ("צוּר", "H:V", "to confine", "tsur"),
        "H6696b": ("צוּר", "H:V", "to provoke", "tsur"),
        "H6696c": ("צוּר", "H:V", "to form", "tsur"),
        "H6697": ("צוּר", "H:N-M", "rock", "tsur"),
        "H6698": ("צוּר", "N:N-M-P", "Zur", "tsur"),
        "H6699": ("צוּרָה", "H:N-F", "design", "tsu.rah"),
        "H6700": ("צוּרִיאֵל", "N:N-M-P", "Zuriel", "tsu.ri.el"),
        "H6701": ("צוּרִישַׁדָּ֑י", "N:N-M-P", "Zurishaddai", "tsu.ri.shad.day"),
        "H6702": ("צוּת", "H:V", "to burn", "tsut"),
        "H6703": ("צַח", "H:A", "dazzling", "tsach"),
        "H6704": ("צִחֶה", "H:A", "parched", "tsi.cheh"),
        "H6705": ("צָחַח", "H:V", "be dazzling", "tsa.chach"),
        "H6706": ("צְחִיחַ", "H:N-M", "bare", "tse.chi.ach"),
        "H6707": ("צְחִיחָה", "H:N-F", "parched", "tse.chi.chah"),
        "H6708": ("צְחִיחִי", "H:N-M", "higher", "tse.chi.chi"),
        "H6709": ("צַחֲנָה", "H:N-F", "stench", "tsa.cha.nah"),
        "H6710": ("צַחְצָחָה", "H:N-F", "scorched", "tsach.tsa.chah"),
        "H6711": ("צָחַק", "H:V", "to laugh", "tsa.chaq"),
        "H6712": ("צְחֹק", "H:N-M", "laughter", "tse.choq"),
        "H6713": ("צַ֫חַר", "H:N-M", "gray", "tsa.char"),
        "H6714": ("צֹ֫חַר", "N:N-M-P", "Zohar", "tso.char"),
        "H6715": ("צָחֹר", "H:A", "tawny", "tsa.chor"),
        "H6716": ("צִי", "H:N-M", "ship", "tsi"),
        "H6717": ("צִיבָא", "N:N-M-P", "Ziba", "tsi.va"),
        "H6718a": ("צַ֫יִד", "H:N-M", "wild game", "tsa.yid"),
        "H6718b": ("צַ֫יִד", "H:N-M", "food", "tsa.yid"),
        "H6719": ("צַיָּד", "H:N-M", "hunter", "tsay.yad"),
        "H6720": ("צֵידָה", "H:N-F", "provision", "tse.dah"),
        "H6721": ("צִידוֹן", "N:N--L / N:N-M-P", "Sidon", "tsi.don"),
        "H6722": ("צִידֹנִי", "N:N--LG", "Sidonian", "tsi.do.ni"),
        "H6723": ("צִיָּה", "H:N-F", "dryness", "tsiy.yah"),
        "H6724": ("צָיוֹן", "H:N-M", "dryness", "tsi.von"),
        "H6725": ("צִיּוּן", "H:N-M", "signpost", "tsiy.yun"),
        "H6726": ("צִיּוֹן", "N:N--L", "Zion", "tsiy.yon"),
        "H6727": ("צִיחָא", "N:N-M-P", "Ziha", "tsi.ach"),
        "H6728": ("צִי", "H:N-M", "wild beast", "tsiy.yi"),
        "H6729": ("צִינֹק", "H:N-M", "stocks", "tsi.noq"),
        "H6730": ("צִיעֹר", "N:N--L", "Zior", "tsi.or"),
        "H6731a": ("צִיץ", "H:N-M", "flower", "tsits"),
        "H6731b": ("צִיץ", "H:N-M", "feather", "tsits"),
        "H6732": ("צִיץ", "N:N--L", "Ziz", "tsits"),
        "H6733": ("צִיצָה", "H:N-M", "flower", "tsi.tsah"),
        "H6734": ("צִיצִת", "H:N-F", "tassel", "tsi.tsit"),
        "H6735a": ("צִיר", "H:N-M", "envoy", "tsir"),
        "H6735b": ("צִיר", "H:N-M", "hinge", "tsir"),
        "H6735c": ("צִיר", "H:N-M", "pang", "tsir"),
        "H6736": ("צִיר", "H:N-M", "image", "tsir"),
        "H6737": ("צָיַר", "H:V", "to take provision", "tsa.yar"),
        "H6738": ("צֵל", "H:N-M", "shadow", "tsel"),
        "H6739": ("צְלָא", "A:V", "to pray", "tse.la"),
        "H6740": ("צָלָה", "H:V", "to roast", "tsa.lah"),
        "H6741": ("צִלָּה", "N:N-F-P", "Zillah", "tsil.lah"),
        "H6742": ("צְלִיל", "H:N-M", "bun", "tse.lul"),
        "H6743a": ("צָלַח", "H:V", "to rush", "tsa.lach"),
        "H6743b": ("צָלֵחַ", "H:V", "to prosper", ""),
        "H6744": ("צְלַח", "A:V", "to prosper", "tse.lach"),
        "H6745": ("צֵלָחָה", "H:N-F", "pot", "tse.la.chah"),
        "H6746": ("צְלֹחִית", "H:N-F", "jar", "tse.lo.chit"),
        "H6747": ("צַלַּ֫חַת", "H:N-F", "dish", "tsal.la.chat"),
        "H6748": ("צָלִי", "H:N-M / H:A", "roasted", "tsa.li"),
        "H6749": ("צָלַל", "H:V", "to sink", "tsa.lal"),
        "H6750": ("צָלַל", "H:V", "to tingle", "tsa.lal"),
        "H6751": ("צָלַל", "H:V", "to shade", "tsa.lal"),
        "H6752": ("צֵלֶל", "H:N-M", "shadow", "tse.lel"),
        "H6753": ("הַצְּלֶלְפּוֹנִי", "N:N-F-P", "Hazzelelponi", "tse.lel.po.ni"),
        "H6754": ("צֶ֫לֶם", "H:N-M", "image", "tse.lem"),
        "H6755": ("צְלֵם", "A:N-M", "image", "tse.lem"),
        "H6756a": ("צַלְמוֹן", "N:N--L", "Zalmon", "tsal.mon"),
        "H6756b": ("צַלְמוֹן", "N:N-M-P", "Zalmon", "tsal.mon"),
        "H6757": ("צַלְמָ֫וֶת", "H:N-M", "shadow", "tsal.ma.vet"),
        "H6758": ("צַלְמֹנָה", "N:N--L", "Zalmonah", "tsal.mo.nah"),
        "H6759": ("צַלְמֻנָּע", "N:N-M-P", "Zalmunna", "tsal.mun.na"),
        "H6760": ("צָלַע", "H:V", "to limp", "tsa.la"),
        "H6761": ("צֶ֫לַע", "H:N-M", "stumbling", "tse.la"),
        "H6762": ("צֵלַע", "N:N--L", "Zela", "tse.la"),
        "H6763": ("צֵלָע", "H:N-F", "side", "tse.la"),
        "H6764": ("צָלָף", "N:N-M-P", "Zalaph", "tsa.laph"),
        "H6765": ("צְלׇפְחָד", "N:N-M-P", "Zelophehad", "tse.loph.chad"),
        "H6766": ("צֶלְצַח", "N:N--L", "Zelzah", "tsel.tsach"),
        "H6767a": ("צִלְצָל", "H:N-M", "buzzing", ""),
        "H6767b": ("צִלִצָל", "H:N-M", "spear", ""),
        "H6767c": ("צְלָצַ֑ל", "H:N-M", "whirring", "tse.la.tsal"),
        "H6767d": ("צֶלְצְלִים", "H:N-M", "banging", ""),
        "H6768": ("צֶ֫לֶק", "N:N-M-P", "Zelek", "tse.leq"),
        "H6769": ("צִלְּתַי", "N:N-M-P", "Zillethai", "tsil.l.tay"),
        "H6770": ("צָמֵא", "H:V", "to thirst", "tsa.me"),
        "H6771": ("צָמֵא", "H:A", "thirsty", "tsa.me"),
        "H6772": ("צָמָא", "H:N-M", "thirst", "tsa.ma"),
        "H6773": ("צִמְאָה", "H:N-F", "parched", "tsim.ah"),
        "H6774": ("צִמָּאוֹן", "H:N-M", "parched", "tsim.ma.on"),
        "H6775": ("צָמַד", "H:V", "to join", "tsa.mad"),
        "H6776": ("צֶ֫מֶד", "H:N-M", "pair", "tse.med"),
        "H6777": ("צַמָּה", "H:N-F", "veil", "tsam.mah"),
        "H6778": ("צַמּוּק", "H:N-M", "cluster", "tsam.muq"),
        "H6779": ("צָמַח", "H:V", "to spring", "tsa.mach"),
        "H6780": ("צֶ֫מַח", "H:N-M", "branch", "tse.mach"),
        "H6781a": ("צָמִיד", "H:N-M", "bracelet", "tsa.mid"),
        "H6781b": ("צָמִיד", "H:N-M", "cover", "tsa.mid"),
        "H6782": ("צַמִּים", "H:N-M", "snare", "tsam.mim"),
        "H6783": ("צְמִיתֻת", "H:N-F", "finality", "tse.mi.tut"),
        "H6784": ("צָמַק", "H:V", "to shrivel", "tsa.maq"),
        "H6785": ("צֶ֫מֶר", "H:N-M", "wool", "tse.mer"),
        "H6786": ("צְמָרִי", "N:N--LG", "Zemarite", "tse.ma.ri"),
        "H6787": ("צְמָרַיִם", "N:N--L", "Zemaraim", "tse.ma.ra.yim"),
        "H6788": ("צַמֶּ֫רֶת", "H:N-F", "treetop", "tsam.me.ret"),
        "H6789": ("צָמַת", "H:V", "to destroy", "tsa.mat"),
        "H6790": ("צִן", "N:N--L", "Zin", "tsin"),
        "H6791": ("צֵן", "H:N-M", "thorn", "tsen"),
        "H6792": ("צֹנֶה", "H:N-M", "sheep", "tso.ne"),
        "H6793a": ("צִנָּה", "H:N-F", "hook", "tsin.nah"),
        "H6793b": ("צִנָּה", "H:N-F", "coolness", "tsin.nah"),
        "H6793c": ("צִנָּה", "H:N-F", "shield", "tsin.nah"),
        "H6794": ("צִנּוֹר", "H:N-M", "water", "tsin.nur"),
        "H6795": ("צָנַח", "H:V", "to descend", "tsa.nach"),
        "H6796": ("צָנִין", "H:N-M", "thorn", "tsa.nin"),
        "H6797": ("צָנִיף", "H:N-M", "turban", "tsa.niph"),
        "H6798": ("צָנַם", "H:V", "to dry", "tsa.nam"),
        "H6799": ("צְנָן", "N:N--L", "Zenan", "tse.nan"),
        "H6800a": ("צָנַע", "H:V", "be humble", "tsa.na"),
        "H6800b": ("צָנוּעַ", "H:A", "humble", ""),
        "H6801": ("צָנַף", "H:V", "to wrap", "tsa.naph"),
        "H6802": ("צְנֵפָה", "H:N-F", "toss", "tse.ne.phah"),
        "H6803": ("צִנְצֶ֫נֶת", "H:N-F", "jar", "tsin.tse.net"),
        "H6804": ("צַנְתְּרוֹת", "H:N-F", "pipe", "tsan.ta.rah"),
        "H6805": ("צָעַד", "H:V", "to march", "tsa.ad"),
        "H6806": ("צַ֫עַד", "H:N-M", "step", "tsa.ad"),
        "H6807a": ("צְעָדָה", "H:N-F", "marching", "tse.a.dah"),
        "H6807b": ("צְעָדָה", "H:N-F", "bracelette", "tse.a.dah"),
        "H6808": ("צָעָה", "H:V", "to march", "tsa.ah"),
        "H6809": ("צָעִיף", "H:N-M", "shawl", "tsa.iph"),
        "H6810": ("צָעִיר", "H:A", "little", "tsa.ir"),
        "H6811": ("צָעִיר", "N:N--L", "Zair", "tsa.ir"),
        "H6812": ("צְעִירָה", "H:N-F", "youth", "tse.i.rah"),
        "H6813": ("צָעַן", "H:V", "to move", "tsa.an"),
        "H6814": ("צֹ֫עַן", "N:N--L", "Zoan", "tso.an"),
        "H6815": ("צַעֲנַנִּים", "N:N--L", "Zaanannim", "tsa.a.nan.nim"),
        "H6816": ("צַעֲצֻעִים", "H:N-M", "image", "tsa.tsu.a"),
        "H6817": ("צָעַק", "H:V", "to cry", "tsa.aq"),
        "H6818": ("צְעָקָה", "H:N-F", "cry", "tsa.a.qah"),
        "H6819": ("צָעַר", "H:V", "be little", "tsa.ar"),
        "H6820": ("צֹ֫עַר", "N:N--L", "Zoar", "tso.ar"),
        "H6821": ("צָפַד", "H:V", "to shrivel", "tsa.phad"),
        "H6822": ("צָפָה", "H:V", "to watch", "tsa.phah"),
        "H6823": ("צָפָה", "H:V", "to overlay", "tsa.phah"),
        "H6824": ("צָפָה", "H:N-F", "discharge", "tsa.phah"),
        "H6825": ("צְפוֹ, צְפִי", "N:N-M-P", "Zepho", "tse.pho"),
        "H6826": ("צִפּוּי", "H:N-M", "plating", "tsip.puy"),
        "H6827": ("צְפוֹן", "N:N-M-P", "Zephon", "tse.phon"),
        "H6828": ("צָפוֹן", "H:N-F", "north", "tsa.phon"),
        "H6829": ("צָפוֹן", "N:N--L", "Zaphon", "tsa.phon"),
        "H6830": ("צְפוֹנִי", "H:A", "northern", "tse.pho.ni"),
        "H6831": ("צְפוֹנִי", "N:N--PG", "Zephonite", "tse.pho.ni"),
        "H6832": ("צָפִיעַ", "H:N-M", "dung", "tse.phu.a"),
        "H6833": ("צִפּוֹר", "H:N-F", "bird", "tsip.por"),
        "H6834": ("צִפּוֹר", "N:N-M-P", "Zippor", "tsip.por"),
        "H6835": ("צַפַּ֫חַת", "H:N-F", "jar", "tsap.pa.chat"),
        "H6836": ("צְפִּיָּה", "H:N-F", "watch", "tse.phiy.yah"),
        "H6837": ("צִפְיוֹן", "N:N-M-P", "Ziphion", "tsiph.yon"),
        "H6838": ("צְפִיחִת", "H:N-F", "flatbread", "tsap.pi.chit"),
        "H6839": ("צֹפִים", "N:N--L", "Zophim", "tso.phim"),
        "H6840": ("צָפִין", "H:N-M", "treasure", "tsa.phin"),
        "H6841": ("צְפִיר", "A:N-M", "male goat", "tse.phir"),
        "H6842": ("צָפִיר", "H:N-M", "male goat", "tsa.phir"),
        "H6843": ("צְפִירָה", "H:N-F", "diadem/doom", "tse.phi.rah"),
        "H6844": ("צָפִית", "H:N-F", "rug", "tsa.phit"),
        "H6845": ("צָפַן", "H:V", "to treasure", "tsa.phan"),
        "H6846": ("צְפַנְיָהוּ", "N:N-M-P", "Zephaniah", "tse.phan.yah"),
        "H6847": ("צָֽפְנַת פַּעְנֵחַ", "N:N-M-P", "Zaphenath-paneah", "tsoph.nat pa.ne.ach"),
        "H6848a": ("צֶ֫פַע", "H:N-M", "serpent", "tse.pah"),
        "H6848b": ("צִפְעוֹנִי", "H:N-M", "serpent", ""),
        "H6849": ("צְפִיעָה", "H:N-F", "offshoot", "tse.phi.ah"),
        "H6850": ("צָפַף", "H:V", "to whisper", "tsa.phaph"),
        "H6851": ("צַפְצָפָה", "H:N-F", "willow", "tsaph.tsa.phah"),
        "H6852": ("צָפַר", "H:V", "to depart", "tsa.phar"),
        "H6853": ("צְפַר", "A:N-M", "bird", "tse.phar"),
        "H6854": ("צְפַרְדֵּעַ", "H:N-F", "frog", "tse.phar.de.a"),
        "H6855": ("צִפֹּרָה", "N:N-F-P", "Zipporah", "tsip.po.rah"),
        "H6856": ("צִפֹּ֫רֶן", "H:N-M", "fingernail", "tsip.po.ren"),
        "H6857": ("צְפַ֑ת", "N:N--L", "Zephath", "tse.phat"),
        "H6858": ("צֶ֫פֶת", "H:N-F", "capital", "tse.phet"),
        "H6859": ("צְפַ֫תָה", "N:N--L", "Zephathah", "tse.pha.tah"),
        "H6860": ("צִקְלַג", "N:N--L", "Ziklag", "tsiq.lag"),
        "H6861": ("צִקָּלֹן", "H:N-M", "sack", "tsiq.lon"),
        "H6862a": ("צַר", "H:A", "narrow", "tsar"),
        "H6862b": ("צַר", "H:N-M", "distress", "tsar"),
        "H6862c": ("צַר", "H:N-M", "enemy", "tsar"),
        "H6862d": ("צַר", "H:N-M", "hard", "tsar"),
        "H6863": ("צֵר", "N:N--L", "Zer", "tser"),
        "H6864": ("צֹר", "H:N-M", "flint", "tsor"),
        "H6865": ("צֹר", "N:N--L", "Tyre", "tsor"),
        "H6866": ("צָרַב", "H:V", "to burn", "tsa.rav"),
        "H6867a": ("צָרָב", "H:A", "burning", "tsa.re.vet"),
        "H6867b": ("צָרֶ֫בֶת", "H:N-F", "scar", "tsa.re.vet"),
        "H6868": ("צְרֵדָה", "N:N--L", "Zeredah", "tse.re.dah"),
        "H6869a": ("בַּצָּרָה", "H:N-F", "dearth", "tsa.rah"),
        "H6869b": ("צָרָה", "H:N-F", "distress", "tsa.rah"),
        "H6869c": ("צָרָה", "H:N-F", "vexer", "tsa.rah"),
        "H6870": ("צְרוּיָה", "N:N-F-P", "Zeruiah", "tse.ru.yah"),
        "H6871": ("צְרוּעָה", "N:N-F-P", "Zeruah", "tse.ru.ah"),
        "H6872a": ("צְרוֹר", "H:N-M", "bundle", "tse.ror"),
        "H6872b": ("צְרוֹר", "H:N-M", "pebble", "tse.ror"),
        "H6872c": ("צְרוֹר", "N:N-M-P", "Zeror", "tse.ror"),
        "H6873": ("צָרַח", "H:V", "to cry", "tsa.rach"),
        "H6874": ("צְרִי", "N:N-M-P", "Zeri", "tse.ri"),
        "H6875": ("צֳרִי", "H:N-M", "balsam", "tse.ri"),
        "H6876": ("צֹרִי", "N:N--LG", "Tyrian", "tso.ri"),
        "H6877": ("צְרִיחַ", "H:N-M", "stronghold", "tse.ri.ach"),
        "H6878": ("צֹ֫רֶךְ", "H:N-M", "need", "tso.rekh"),
        "H6879": ("צָרַע", "H:V", "be leprous", "tsa.ra"),
        "H6880": ("צִרְעָה", "H:N-F", "hornet", "tsir.ah"),
        "H6881": ("צׇרְעָה", "N:N--L", "Zorah", "tsor.ah"),
        "H6882": ("צׇרְעָתִי", "N:N--LG", "Zorathite", "tsor.i"),
        "H6883": ("צָרַ֫עַת", "H:N-F", "leprosy", "tsa.ra.at"),
        "H6884": ("צָרַף", "H:V", "to refine", "tsa.raph"),
        "H6885": ("צֹרְפִי", "H:N-M", "goldsmith", "tso.re.phi"),
        "H6886": ("צָֽרְפַת", "N:N--L", "Zarephath", "tsa.re.phat"),
        "H6887a": ("צָרַר", "H:V", "to confine", "tsa.rar"),
        "H6887b": ("צָרַר", "H:V", "to constrain", "tsa.rar"),
        "H6887c": ("צָרַר", "H:V", "to distress", "tsa.rar"),
        "H6887d": ("צָרַר", "H:V", "to vex", "tsa.rar"),
        "H6887e": ("צָרַר", "H:V", "to rival", "tsa.rar"),
        "H6888": ("צְרֵרָה", "N:N--L", "Zererah", "tse.re.rah"),
        "H6889": ("צֶ֫רֶת", "N:N-M-P", "Zereth", "tse.ret"),
        "H6890": ("צֶרֶת הַשַּׁ֫חַר", "N:N--L", "Zereth-shahar", "tse.ret hash.sha.char"),
        "H6891": ("צָֽרְתָן", "N:N--L", "Zarethan", "tsa.re.tan"),
        "H6892a": ("קֵא", "H:N-M", "vomit", "qe"),
        "H6892b": ("קִיא", "H:N-M", "vomit", ""),
        "H6893": ("קָאַת", "H:N-F", "pelican", "qa.at"),
        "H6894": ("קַב", "H:N-M", "kab", "qav"),
        "H6895": ("קָבַב", "H:V", "to curse", "qa.vav"),
        "H6896": ("קֵבָה", "H:N-F", "stomach", "qe.vah"),
        "H6897": ("קֹבָה", "H:N-F", "belly", "qo.vah"),
        "H6898": ("קֻבָּה", "H:N-F", "tent", "qub.bah"),
        "H6899": ("קִבּוּץ", "H:N-M", "collection", "qib.buts"),
        "H6900": ("קְבֻרָה", "H:N-F", "tomb", "qe.vu.rah"),
        "H6901": ("קָבַל", "H:V", "to receive", "qa.val"),
        "H6902": ("קַבֵּל", "A:V", "to receive", "qe.val"),
        "H6903": ("קֳבֵל", "A:Conj", "before", "qe.vel"),
        "H6904": ("קְבֹל", "H:N-M", "battering-ram", "qo.vel"),
        "H6905": ("קָבָל", "H:Adv", "before", "qa.val"),
        "H6906": ("קָבַע", "H:V", "to rob", "qa.va"),
        "H6907": ("קֻבַּעַת", "H:N-F", "dreg", "qub.ba.at"),
        "H6908": ("קָבַץ", "H:V", "to gather", "qa.vats"),
        "H6909": ("קַבְצְאֵל", "N:N--L", "Kabzeel", "qav.tse.el"),
        "H6910": ("קְבֻצָה", "H:N-F", "gathering", "qe.vu.tsah"),
        "H6911": ("קִבְצַיִם", "N:N--L", "Kibzaim", "qiv.tsa.yim"),
        "H6912": ("קָבַר", "H:V", "to bury", "qa.var"),
        "H6913": ("קֶ֫בֶר", "H:N-M", "grave", "qe.ver"),
        "H6914": ("קִבְרוֹת הַתַּאֲוָה", "N:N--L", "Kibroth-hattaavah", "qiv.rot hat.ta.a.vah"),
        "H6915": ("קָדַד", "H:V", "to bow", "qa.dad"),
        "H6916": ("קִדָּה", "H:N-F", "cassia", "qid.dah"),
        "H6917": ("קָדוּם", "H:N-M", "ancient", "qa.dum"),
        "H6918": ("קָדוֹשׁ", "H:A", "holy", "qa.dosh"),
        "H6919": ("קָדַח", "H:V", "to kindle", "qa.dach"),
        "H6920": ("קַדַּ֫חַת", "H:N-F", "fever", "qad.da.chat"),
        "H6921": ("קָדִים", "H:N-M", "east", "qa.dim"),
        "H6922": ("קַדִּישׁ", "A:A", "holy", "qad.dish"),
        "H6923": ("קָדַם", "H:V", "to meet", "qa.dam"),
        "H6924a": ("קֶ֫דֶם", "H:N-M", "front", "qe.dem"),
        "H6924b": ("קֵ֫דֶם", "H:Adv", "east", ""),
        "H6925": ("קֳדָם", "A:Prep", "before", "qo.dam"),
        "H6926": ("קִדְמָה", "H:N-F", "east", "qid.mah"),
        "H6927": ("קַדְמָה", "H:N-F", "former", "qad.mah"),
        "H6928": ("קַדְמָה", "A:N-F", "previously", "qad.mah"),
        "H6929": ("קֵ֑דְמָה", "N:N-M-P", "Kedemah", "qe.de.mah"),
        "H6930": ("קַדְמוֹן", "H:A", "eastern", "qad.mon"),
        "H6931": ("קַדְמֹנִי", "H:A", "eastern", "qad.mo.ni"),
        "H6932": ("קְדֵמוֹת", "N:N--L", "Kedemoth", "qe.de.mot"),
        "H6933": ("קַדְמַי", "A:A", "first", "qad.may"),
        "H6934": ("קַדְמִיאֵל", "N:N-M-P", "Kadmiel", "qad.mi.el"),
        "H6935": ("קַדְמֹנִי", "N:N--LG", "Kadmonite", "qad.mo.ni"),
        "H6936": ("קׇדְקֹד", "H:N-M", "crown", "qod.qod"),
        "H6937": ("קָדַר", "H:V", "be dark", "qa.dar"),
        "H6938": ("קֵדָר", "N:N-M-P / N:N-M-PG", "Kedar", "qe.dar"),
        "H6939": ("קִדְרוֹן", "N:N--L", "Kidron", "qid.ron"),
        "H6940": ("קַדְרוּת", "H:N-F", "darkness", "qad.rut"),
        "H6941": ("קְדֹרַנִּית", "H:Adv", "mournfully", "qe.do.ran.nit"),
        "H6942": ("קָדַשׁ", "H:V", "to consecrate", "qa.dash"),
        "H6943": ("קֶ֫דֶשׁ", "N:N--L", "Kedesh", "qe.desh"),
        "H6944": ("קֹ֫דֶשׁ", "H:N-M", "holiness", "qo.desh"),
        "H6945": ("קָדֵשׁ", "H:N-M", "male cult prostitute", "qa.desh"),
        "H6946": ("קָדֵשׁ", "N:N--L", "Kadesh", "qa.desh"),
        "H6947": ("קָדֵשׁ בַּרְנֵעַ", "N:N--L", "Kadesh-barnea", "qa.desh bar.ne.a"),
        "H6948": ("קְדֵשָׁה", "H:N-F", "cult prostitute", "qe.de.shah"),
        "H6949": ("קָהָה", "H:V", "be blunt", "qa.hah"),
        "H6950": ("קָהַל", "H:V", "to gather", "qa.hal"),
        "H6951": ("קָהָל", "H:N-M", "assembly", "qa.hal"),
        "H6952": ("קְהִלָּה", "H:N-F", "assembly", "qe.hil.lah"),
        "H6953": ("קֹהֶ֫לֶת", "H:N-M", "preacher", "qo.he.let"),
        "H6954": ("קְהֵלָ֫תָה", "N:N--L", "Kehelathah", "qe.he.la.tah"),
        "H6955": ("קְהָת", "N:N-M-P", "Kohath", "qe.hat"),
        "H6956": ("קְהָתִי", "N:N--PG", "Kohathite", "qo.ha.ti"),
        "H6957a": ("קַו", "H:N-M", "cord", "qav"),
        "H6957b": ("קַו", "H:N-M", "line", "qav"),
        "H6958": ("קִיא", "H:V", "to vomit", "qo"),
        "H6959": ("קוֹבַע", "H:N-M", "helmet", "qo.va"),
        "H6960a": ("קָוָה", "H:V", "to await", "qa.vah"),
        "H6960b": ("קָוָה", "H:V", "to collect", "qa.vah"),
        "H6961": ("קוה", "H:N-M", "line", "qa.veh"),
        "H6962": ("קוּט", "H:V", "to loath", "qut"),
        "H6963a": ("קוֹל", "H:N-M", "voice", "qol"),
        "H6963b": ("קֹל", "H:N-M", "frivolity", ""),
        "H6964": ("קוֹלָיָה", "N:N-M-P", "Kolaiah", "qo.la.yah"),
        "H6965a": ("קָמָי", "H:V", "-Kemai", "qum"),
        "H6965b": ("קוּם", "H:V", "to arise", "qum"),
        "H6966": ("קוּם", "A:V", "to stand", "qum"),
        "H6967": ("קוֹמָה", "H:N-F", "height", "qo.mah"),
        "H6968": ("קוֹמְמִיּוּת", "H:N-F", "uprightness", "qo.me.miy.yut"),
        "H6969": ("קִינֵן", "H:V", "to chant", "qun"),
        "H6970": ("קוֹעַ", "N:N--L", "Koa", "qo.a"),
        "H6971": ("קוֹף", "H:N-M", "ape", "qoph"),
        "H6972": ("קיץ", "H:V", "to summer", "quts"),
        "H6973": ("קוּץ", "H:V", "to loathe", "quts"),
        "H6974": ("קִיץ", "H:V", "to awake", "quts"),
        "H6975": ("קוֹץ", "H:N-M", "thorn", "qots"),
        "H6976": ("הַקוֹץ", "N:N-M-P", "Hakkoz", "qots"),
        "H6977": ("קְוֻצּוֹת", "H:N-F", "lock", "qe.vuts.tsah"),
        "H6978": ("קַוְקָו", "H:N-M", "might", "qav-qav"),
        "H6979a": ("קוּר", "H:V", "to dig", "qur"),
        "H6979b": ("קָרַר", "H:V", "to cool", ""),
        "H6979c": ("קָרַר", "H:V", "to destroy", ""),
        "H6980": ("קוּר", "H:N-M", "web", "qur"),
        "H6981": ("קוֹרֵא", "N:N-M-P", "Kore", "qo.re"),
        "H6982": ("קוֹרָה", "H:N-F", "beam", "qo.rah"),
        "H6983": ("קוּשׁ", "H:V", "to lure", "qosh"),
        "H6984": ("קוּשָׁיָ֫הוּ", "N:N-M-P", "Kushaiah", "qu.sha.ya.hu"),
        "H6985": ("קָ֔ט", "H:A", "only", "qat"),
        "H6986": ("קֶ֫טֶב", "H:N-M", "destruction", "qe.tev"),
        "H6987": ("קֹטֶב", "H:N-M", "destruction", "qo.tev"),
        "H6988": ("קְטוֹרָה", "H:N-M", "incense", "qe.to.rah"),
        "H6989": ("קְטוּרָה", "N:N-F-P", "Keturah", "qe.tu.rah"),
        "H6990": ("קוֹט", "H:V", "to cut off", "qa.tat"),
        "H6991": ("קָטַל", "H:V", "to slay", "qa.tal"),
        "H6992": ("קְטַל", "A:V", "to slay", "qe.tal"),
        "H6993": ("קֶ֫טֶל", "H:N-M", "slaughter", "qe.tel"),
        "H6994": ("קָטֹן", "H:V", "be small", "qa.ton"),
        "H6995": ("קְטֹן", "H:N-M", "little finger", "qo.ten"),
        "H6996a": ("קָטָן", "H:A", "small", "qa.tan"),
        "H6996b": ("קָטֹן", "H:A", "small", ""),
        "H6997": ("קָטָן", "N:N-M-P", "Hakkatan", "qa.tan"),
        "H6998": ("קָטַף", "H:V", "to pluck", "qa.taph"),
        "H6999a": ("קָטַר", "H:V", "to offer", "qa.tar"),
        "H6999b": ("מֻקְטָר", "H:N-M", "incense", ""),
        "H6999c": ("מְקַטֵרָה", "H:N-F", "incense-altar", ""),
        "H7000": ("קָטַר", "H:V", "to enclose", "qa.tar"),
        "H7001": ("קְטַר", "A:N-M", "joint", "qe.tar"),
        "H7002": ("קִטֵּר", "H:N-F", "incense", "qit.ter"),
        "H7003": ("קִטְרוֹן", "N:N--L", "Kitron", "qit.ron"),
        "H7004": ("קְטֹ֫רֶת", "H:N-F", "incense", "qe.to.ret"),
        "H7005": ("קַטָּת", "N:N--L", "Kattath", "qat.tat"),
        "H7006": ("קָיָה", "H:V", "to vomit", "qa.yah"),
        "H7007": ("קַ֫יִט", "A:N-M", "summer", "qa.yit"),
        "H7008": ("קִיטוֹר", "H:N-M", "smoke", "qi.tor"),
        "H7009": ("קִים", "H:N-M", "adversary", "qim"),
        "H7010": ("קְיָם", "A:N-M", "statute", "qe.yam"),
        "H7011": ("קַיָּם", "A:A", "enduring", "qay.yam"),
        "H7012": ("קִימָה", "H:N-F", "rising", "qi.mah"),
        "H7013": ("קַ֫יִן", "H:N-M", "spear", "qa.yin"),
        "H7014a": ("קַ֫יִן", "N:N-M-PG", "Kenite", "qa.yin"),
        "H7014b": ("קַ֫יִן", "N:N-M-P", "Cain", "qa.yin"),
        "H7015": ("קִינָה", "H:N-F", "dirge", "qi.nah"),
        "H7016": ("קִינָה", "N:N--L", "Kinah", "qi.nah"),
        "H7017": ("קֵינִי", "N:N-M-PG", "Kenite", "qe.ni"),
        "H7018": ("קֵינָן", "N:N-M-P", "Kenan", "qe.nan"),
        "H7019": ("קַ֫יִץ", "H:N-M", "summer", "qa.yits"),
        "H7020": ("קִיצוֹן", "H:A", "outermost", "qi.tson"),
        "H7021": ("קִיקָיוֹן", "H:N-M", "plant", "qi.qa.von"),
        "H7022": ("קִיקָלוֹן", "H:N-M", "disgrace", "qi.qa.lon"),
        "H7023": ("קִיר", "H:N-M", "wall", "qir"),
        "H7024a": ("קִיר", "N:N--L", "Kir", "qir"),
        "H7024b": ("קִיר", "N:N--L", "Kir", "qir"),
        "H7025": ("קִיר חֶרֶשׂ", "N:N--L", "Kir-hareseth", "qir che.re.s"),
        "H7026": ("קֵרֹס", "N:N-M-P", "Keros", "qe.ros"),
        "H7027": ("קִישׁ", "N:N-M-P", "Kish", "qish"),
        "H7028": ("קִישׁוֹן", "N:N--L", "Kishon", "qi.shon"),
        "H7029": ("קִישִׁי", "N:N-M-P", "Kishi", "qi.shi"),
        "H7030": ("קַתָרֹס", "A:N-M", "lyre", "qi.ta.ros"),
        "H7031": ("קַל", "H:A", "swift", "qal"),
        "H7032": ("קָל", "A:N-M", "voice", "qal"),
        "H7033": ("קָלָה", "H:V", "to roast", "qa.lah"),
        "H7034": ("קָלָה", "H:V", "to dishonor", "qa.lah"),
        "H7035": ("קָלַהּ", "H:V", "to gather", "qa.lah"),
        "H7036": ("קָלוֹן", "H:N-M", "dishonor", "qa.lon"),
        "H7037": ("קַלַּ֫חַת", "H:N-F", "caldron", "qal.la.chat"),
        "H7038": ("קָלַט", "H:V", "be stunted", "qa.lat"),
        "H7039": ("קָלִי", "H:N-M", "roasted", "qa.li"),
        "H7040": ("קַלַּי", "N:N-M-P", "Kallai", "qal.lay"),
        "H7041": ("קֵלָיָה", "N:N-M-P", "Kelaiah", "qe.la.yah"),
        "H7042": ("קְלִיטָא", "N:N-M-P", "Kelita", "qe.li.ta"),
        "H7043": ("קָלַל", "H:V", "to lighten", "qa.lal"),
        "H7044": ("קָלָל", "H:A", "burnished", "qa.lal"),
        "H7045": ("קְלָלָה", "H:N-F", "curse", "qe.la.lah"),
        "H7046": ("קָלַס", "H:V", "to mock", "qa.las"),
        "H7047": ("קֶ֫לֶס", "H:N-M", "derision", "qe.les"),
        "H7048": ("קַלָּסָה", "H:N-F", "derision", "qal.la.sah"),
        "H7049a": ("קָלַע", "H:V", "to sling", "qa.la"),
        "H7049b": ("קָלַע", "H:V", "to carve", "qa.la"),
        "H7050a": ("קֶ֫לַע", "H:N-M", "sling", "qe.la"),
        "H7050b": ("קֶ֫לַע", "H:N-M", "curtain", "qe.la"),
        "H7051": ("קַלָּע", "H:N-M", "slinger", "qal.la"),
        "H7052": ("קְלֹקֵל", "H:A", "worthless", "qe.lo.qel"),
        "H7053": ("קִלְּשׁוֹן", "H:N-M", "fork", "qil.l.shon"),
        "H7054": ("קָמָה", "H:N-F", "standing grain", "qa.mah"),
        "H7055": ("קְמוּאֵל", "N:N-M-P", "Kemuel", "qe.mu.el"),
        "H7056": ("קָמוֹן", "N:N--L", "Kamon", "qa.mon"),
        "H7057": ("קִמּוֹשׂ", "H:N-M", "nettle", "qim.mosh"),
        "H7058": ("קֶ֫מַח", "H:N-M", "flour", "qe.mach"),
        "H7059": ("קָמַט", "H:V", "to seize", "qa.mat"),
        "H7060": ("קָמֵל", "H:V", "to decay", "qa.mal"),
        "H7061": ("קָמַץ", "H:V", "to grasp", "qa.mats"),
        "H7062": ("קֹ֫מֶץ", "H:N-M", "handful", "qo.mets"),
        "H7063": ("קִמָּשׂוֹן", "H:N-M", "thorn", "qim.ma.shon"),
        "H7064": ("קֵן", "H:N-M", "nest", "qen"),
        "H7065": ("קָנָא", "H:V", "be jealous", "qa.na"),
        "H7066": ("קְנָא", "A:V", "to buy", "qe.na"),
        "H7067": ("קַנָּא", "H:A", "jealous", "qan.na"),
        "H7068": ("קִנְאָה", "H:N-F", "jealousy", "qin.ah"),
        "H7069": ("קָנָה", "H:V", "to buy", "qa.nah"),
        "H7070": ("קָנֶה", "H:N-M", "branch", "qa.neh"),
        "H7071": ("קָנָה", "N:N--L", "Kanah", "qa.nah"),
        "H7072": ("קַנּוֹא", "H:A", "jealous", "qan.no"),
        "H7073": ("קְנַז", "N:N-M-P", "Kenaz", "qe.naz"),
        "H7074": ("קְנִזִּי", "N:N--PG", "Kenizzite", "qe.niz.zi"),
        "H7075": ("קִנְיָן", "H:N-M", "acquisition", "qin.yan"),
        "H7076": ("קִנָּמוֹן", "H:N-M", "cinnamon", "qin.na.mon"),
        "H7077": ("קִנֵּן", "H:V", "to make a nest", "qa.nan"),
        "H7078": ("קֵ֫נֶץ", "H:N-M", "snare", "qe.nets"),
        "H7079": ("קְנָת", "N:N--L", "Kenath", "qe.nat"),
        "H7080": ("קָסַם", "H:V", "to divine", "qa.sam"),
        "H7081": ("קֶ֫סֶם", "H:N-M", "divination", "qe.sem"),
        "H7082": ("קָסַס", "H:V", "to strip", "qa.sas"),
        "H7083": ("קֶ֫סֶת", "H:N-F", "pot", "qe.set"),
        "H7084": ("קְעִילָה", "N:N--L", "Keilah", "qe.i.lah"),
        "H7085": ("קַעֲקַע", "H:N-M", "incision", "qa.a.qa"),
        "H7086": ("קְעָרָה", "H:N-F", "dish", "qe.a.rah"),
        "H7087a": ("קָפָא", "H:V", "to congeal", "qa.pah"),
        "H7087b": ("קִפָּאוֹן", "H:N-M", "thickness", ""),
        "H7088": ("קָפַד", "H:V", "to roll", "qa.phad"),
        "H7089": ("קְפָדָה", "H:N-F", "anguish", "qe.pha.dah"),
        "H7090": ("קִפֹּד", "H:N-M", "porcupine", "qip.pod"),
        "H7091": ("קִפּוֹז", "H:N-M", "arrow snake", "qip.poz"),
        "H7092": ("קָפַץ", "H:V", "to gather", "qa.phats"),
        "H7093": ("קֵץ", "H:N-M", "end", "qets"),
        "H7094": ("קָצַב", "H:V", "to shear", "qa.tsav"),
        "H7095": ("קֶ֫צֶב", "H:N-M", "shape", "qe.tsev"),
        "H7096": ("קָצָה", "H:V", "to cut off", "qa.tsah"),
        "H7097a": ("קָצֶה", "H:N-M", "end", "qa.tseh"),
        "H7097b": ("קֵ֫צֶה", "H:N-M", "end", ""),
        "H7098": ("קָצָה", "H:N", "end", "qa.tsah"),
        "H7099": ("קָ֫צוּ", "H:N-M", "boundary", "qe.tsev"),
        "H7100": ("קֶ֫צַח", "H:N-M", "dill", "qe.tsach"),
        "H7101": ("קָצִין", "H:N-M", "chief", "qa.tsin"),
        "H7102": ("קְצִיעָה", "H:N-F", "cassia", "qe.tsi.ah"),
        "H7103": ("קְצִיעָה", "N:N-M-P", "Keziah", "qe.tsi.ah"),
        "H7104": ("קְצִיץ", "N:N--L", "Keziz", "qe.tsits"),
        "H7105a": ("קָצִיר", "H:N-M", "harvest", "qa.tsir"),
        "H7105b": ("קָצִיר", "H:N-M", "foliage", "qa.tsir"),
        "H7106a": ("קָצַע", "H:V", "to scrape", "qa.tsa"),
        "H7106b": ("קָצַע", "H:V", "to corner", "qa.tsa"),
        "H7107": ("קָצַף", "H:V", "be angry", "qa.tsaph"),
        "H7108": ("קְצַף", "A:V", "be angry", "qe.tsaph"),
        "H7109": ("קְצַף", "A:N-M", "wrath", "qe.tsaph"),
        "H7110a": ("קֶ֫צֶף", "H:N-M", "wrath", "qe.tseph"),
        "H7110b": ("קֶ֫צֶף", "H:N-M", "splinter", "qe.tseph"),
        "H7111": ("קְצָפָה", "H:N-F", "splinter", "qe.tsa.phah"),
        "H7112": ("קָצַץ", "H:V", "to cut", "qa.tsats"),
        "H7113": ("קְצַץ", "A:V", "to cut", "qe.tsats"),
        "H7114a": ("קָצֵר", "H:V", "be short", "qa.tsar"),
        "H7114b": ("קָצַר", "H:V", "to reap", "qa.tsar"),
        "H7115": ("קֹ֫צֶר", "H:N-M", "shortness", "qo.tser"),
        "H7116": ("קָצֵר", "H:A", "short", "qa.tser"),
        "H7117": ("קְצָת", "H:N-F", "end", "qe.tsat"),
        "H7118": ("קְצָת", "A:N-F", "end", "qe.tsat"),
        "H7119": ("קַר", "H:A", "cool", "qar"),
        "H7120": ("קֹר", "H:N-M", "cold", "qor"),
        "H7121": ("קָרָא", "H:V", "to call", "qa.ra"),
        "H7122": ("קָרָא", "H:V", "to encounter", "qa.ra"),
        "H7123": ("קְרָא", "A:V", "to read", "qe.ra"),
        "H7124": ("קֹרֵא", "H:N-M", "partridge", "qo.re"),
        "H7125": ("קִרְאָה", "H:V", "befall", "qir.ah"),
        "H7126": ("קָרַב", "H:V", "to present", "qa.rav"),
        "H7127": ("קְרֵב", "A:V", "to approach", "qe.rev"),
        "H7128": ("קְרָב", "H:N-M", "battle", "qe.rav"),
        "H7129": ("קְרָב", "A:N-M", "war", "qe.rav"),
        "H7130": ("קֶ֫רֶב", "H:N-M", "entrails", "qe.rev"),
        "H7131": ("קָרֵב", "H:A", "approaching", "qa.rev"),
        "H7132": ("קִרְבָה", "H:N-F", "nearness", "qe.ra.vah"),
        "H7133a": ("קׇרְבָּן", "H:N-M", "offering", "qor.ban"),
        "H7133b": ("קֻרְבָּן", "H:N-M", "offering", ""),
        "H7134": ("קַרְדֹּם", "H:N-M", "axe", "qar.dom"),
        "H7135": ("קָרָה", "H:N-F", "cold", "qa.rah"),
        "H7136a": ("קָרָה", "H:V", "to meet", "qa.rah"),
        "H7136b": ("קָרָה", "H:V", "to lay beams", "qa.rah"),
        "H7137": ("קָרֶה", "H:N-M", "accident", "qa.reh"),
        "H7138": ("קָרֹב", "H:A", "near", "qa.rov"),
        "H7139": ("קָרַח", "H:V", "to make bald", "qa.rach"),
        "H7140": ("קֶ֫רַח", "H:N-M", "ice", "qe.rach"),
        "H7141": ("קֹ֫רַח", "N:N-M-P", "Korah", "qo.rach"),
        "H7142": ("קֵרֵחַ", "H:A", "bald", "qe.re.ach"),
        "H7143": ("קָרֵחַ", "N:N-M-P", "Kareah", "qa.re.ach"),
        "H7144": ("קׇרְחָה", "H:N-F", "bald spot", "qor.chah"),
        "H7145": ("קׇרְחִי", "N:N--PG", "Korahite", "qor.chi"),
        "H7146": ("קָרַ֫חַת", "H:N-F", "baldness", "qa.ra.chat"),
        "H7147": ("קְרִי", "H:N-M", "hostility", "qe.ri"),
        "H7148": ("קָרִיא", "H:A", "chosen", "qa.ri"),
        "H7149": ("קִרְיָה", "A:N-F", "town", "qir.ya"),
        "H7150": ("קְרִיאָה", "H:N-F", "proclamation", "qe.ri.ah"),
        "H7151": ("קִרְיָה", "H:N-F", "town", "qir.yah"),
        "H7152": ("קְרִיּוֹת", "N:N--L", "Kerioth", "qe.riy.yot"),
        "H7153": ("קִרְיַת אַרְבַּע", "N:N--L", "Kiriath-arba", "qir.yat ar.ba"),
        "H7154": ("קִרְיַת בַּעַל", "N:N--L", "Kiriath-baal", "qir.yat ba.al"),
        "H7155": ("קִרְיַת חֻצוֹת", "N:N--L", "Kiriath-huzoth", "qir.yat chu.tsot"),
        "H7156": ("קִרְיָתַ֫יִם", "N:N--L", "Kiriathaim", "qir.ya.ta.yim"),
        "H7157": ("קִרְיַת יְעָרִים", "N:N--L", "Kiriath-jearim", "qir.yat ye.a.rim"),
        "H7158": ("קִרְיַת־סֵ֫פֶר", "N:N--L", "Kirjath-sepher", "qir.yat san.nah"),
        "H7159": ("קָרַם", "H:V", "to cover", "qa.ram"),
        "H7160": ("קָרַן", "H:V", "to shine", "qa.ran"),
        "H7161a": ("קֶ֫רֶן", "H:N-F", "horn", "qe.ren"),
        "H7161b": ("קַרְנַ֫יִם", "N:N--L", "Karnaim", ""),
        "H7162": ("קֶ֫רֶן", "A:N-F", "horn", "qe.ren"),
        "H7163": ("קֶרֶן הַפּוּךְ", "N:N-F-P", "Keren-happuch", "qe.ren hap.pukh"),
        "H7164": ("קָרַס", "H:V", "to stoop", "qa.ras"),
        "H7165": ("קֶ֫רֶס", "H:N-M", "clasp", "qe.res"),
        "H7166": ("קַרְסֹל", "H:N-F", "ankle", "qar.sol"),
        "H7167": ("קָרַע", "H:V", "to tear", "qa.ra"),
        "H7168": ("קֶ֫רַע", "H:N-M", "rags", "qe.ra"),
        "H7169": ("קָרַץ", "H:V", "to wink", "qa.rats"),
        "H7170": ("קְרַץ", "A:N-M", "charges", "qe.rats"),
        "H7171": ("קֶ֫רֶץ", "H:N-M", "stinging fly", "qe.rets"),
        "H7172": ("קַרְקַע", "H:N-M", "floor", "qar.qa"),
        "H7173": ("קַרְקַע", "N:N--L", "Karka", "qar.qa"),
        "H7174": ("קַרְקֹר", "N:N--L", "Karkor", "qar.qor"),
        "H7175": ("קֶ֫רֶשׁ", "H:N-M", "board", "qe.resh"),
        "H7176": ("קֶ֫רֶת", "H:N-F", "town", "qe.ret"),
        "H7177": ("קַרְתָּה", "N:N--L", "Kartah", "qar.tah"),
        "H7178": ("קַרְתָּן", "N:N--L", "Kartan", "qar.tan"),
        "H7179": ("קַשׁ", "H:N-M", "stubble", "qash"),
        "H7180": ("קִשֻּׁאָה", "H:N-F", "cucumber", "qish.shu"),
        "H7181": ("קָשַׁב", "H:V", "to listen", "qa.shav"),
        "H7182": ("קֶ֫שֶׁב", "H:N-M", "attentiveness", "qe.shev"),
        "H7183a": ("קַשָּׁב", "H:A", "attentive", "qash.shav"),
        "H7183b": ("קַשֻּׁב", "H:A", "attentive", ""),
        "H7184": ("קַשְׂוָה", "H:N-F", "jug", "qa.sah"),
        "H7185": ("קָשָׁה", "H:V", "to harden", "qa.shah"),
        "H7186": ("קָשֶׁה", "H:A", "severe", "qa.sheh"),
        "H7187": ("קְשֹׁט", "A:N-M", "truth", "qe.shot"),
        "H7188": ("קָשַׁח", "H:V", "to harden", "qa.shach"),
        "H7189a": ("קֹ֫שֶׁט", "H:N-M", "bow", "qo.shet"),
        "H7189b": ("קֹשְׁטְ", "H:N-M", "truth", ""),
        "H7190": ("קְשִׁי", "H:N-M", "stubbornness", "qe.shi"),
        "H7191": ("קִשְׁיוֹן", "N:N--L", "Kishion", "qish.yon"),
        "H7192": ("קְשִׂיטָה", "H:N-F", "coin", "qe.si.tah"),
        "H7193": ("קַשְׂקֶ֫שֶׂת", "H:N-F", "scale", "qas.qe.set"),
        "H7194": ("קָשַׁר", "H:V", "to conspire", "qa.shar"),
        "H7195": ("קֶ֫שֶׁר", "H:N-M", "conspiracy", "qe.sher"),
        "H7196": ("קִשֻּׁרִים", "H:N-M", "sash", "qish.shur"),
        "H7197a": ("קָשַׁשׁ", "H:V", "to gather", "qa.shash"),
        "H7197b": ("קָשַׁשׁ", "H:V", "to assemble", "qa.shash"),
        "H7198": ("קֶ֫שֶׁת", "H:N-F", "bow", "qe.shet"),
        "H7199": ("קַשָּׁת", "H:N-M", "bowman", "qash.shat"),
        "H7200": ("רָאָה", "H:V", "to see", "ra.ah"),
        "H7201": ("רָאָה", "H:N-F", "glede", "ra.ah"),
        "H7202": ("רָאֶה", "H:V", "to see", "ra.eh"),
        "H7203a": ("רֹאֶה", "H:N-M", "seer", "ro.eh"),
        "H7203b": ("רֹאֶה", "H:N-M", "vision", "ro.eh"),
        "H7204": ("רֹאֶה", "N:N-M-P", "Haroeh", "ro.eh"),
        "H7205": ("רְאוּבֵן", "N:N-M-P", "Reuben", "re.u.ven"),
        "H7206": ("רְאוּבֵנִי", "N:N--PG", "Reubenite", "re.u.ve.ni"),
        "H7207": ("רַאיָּה", "H:V", "to behold", "ra.a.vah"),
        "H7208": ("רְאוּמָה", "N:N-F-P", "Reumah", "re.u.mah"),
        "H7209": ("רְאִי", "H:N-M", "mirror", "re.i"),
        "H7210": ("רֳאִי", "H:N-M", "sight", "ro.i"),
        "H7211": ("רְאָיָה", "N:N-M-P", "Reaiah", "re.a.yah"),
        "H7212": ("רְאוּת", "H:N", "sight", "re.ut"),
        "H7213": ("רָאַם", "H:V", "to rise", "ra.am"),
        "H7214": ("רְאֵם", "H:N-M", "wild ox", "re.em"),
        "H7215": ("רָאמוֹת", "H:N-F", "coral", "ra.mah"),
        "H7216": ("רָאמוֹת", "N:N--L", "Ramoth", "ra.mot"),
        "H7217": ("רֵאשׁ", "A:N-M", "head", "resh"),
        "H7218a": ("רֹאשׁ", "H:N-M", "head", "rosh"),
        "H7218b": ("רֹאשׁ", "H:N-M / N:N--T", "leader", "rosh"),
        "H7219": ("רֹאשׁ", "H:N-M", "poison", "rosh"),
        "H7220": ("רֹאשׁ", "N:N-M-P", "Rosh", "rosh"),
        "H7221": ("רִאשָׁה", "H:N-F", "beginning", "ri.shah"),
        "H7222": ("רֹאשָׁה", "H:N-F", "top", "ro.shah"),
        "H7223": ("רִאשׁוֹן", "H:A", "first", "ri.shon"),
        "H7224": ("רִאשֹׁנִי", "H:A", "first", "ri.sho.ni"),
        "H7225": ("רֵאשִׁית", "H:N-F", "first", "re.shit"),
        "H7226": ("רַאֲשֹׁת", "H:N-F", "head-place", "ra.a.shot"),
        "H7227a": ("רַב", "H:A", "many", "rav"),
        "H7227b": ("רַב", "H:N-M", "chief", "rav"),
        "H7228": ("רַב", "H:N-M", "", "rav"),
        "H7229": ("רַב", "A:A", "great", "rav"),
        "H7230": ("רֹב", "H:N-M", "abundance", "rov"),
        "H7231": ("רָבַב", "H:V", "to multiply", "ra.vav"),
        "H7232": ("רָבַב", "H:V", "to multiply ten thousand", "ra.vav"),
        "H7233": ("רְבָבָה", "H:N-F", "myriad", "re.va.vah"),
        "H7234": ("רָבַד", "H:V", "to spread", "ra.vad"),
        "H7235a": ("רָבָה", "H:V", "to multiply", "ra.vah"),
        "H7235b": ("רָבָה", "H:V", "to shoot", "ra.vah"),
        "H7236": ("רְבָה", "A:V", "to grow great", "re.vah"),
        "H7237": ("רַבָּה", "N:N--L", "Rabbah", "rab.bah"),
        "H7238": ("רְבוּ", "A:N-F", "greatness", "re.vu"),
        "H7239": ("רִבּוֹ", "H:N-F", "ten thousand", "rib.bo"),
        "H7240": ("רִבּוֹ", "A:N-F", "myriad", "rib.bo"),
        "H7241": ("רְבִיבִים", "H:N-M", "shower", "ra.viv"),
        "H7242": ("רָבִיד", "H:N-M", "necklace", "ra.vid"),
        "H7243": ("רְבִיעִי", "H:A", "fourth", "re.vi.i"),
        "H7244": ("רְבִיעִי", "A:A", "fourth", "re.vi.ay"),
        "H7245": ("רַבִּית", "N:N--L", "Rabbith", "rab.bit"),
        "H7246": ("רָבַךְ", "H:V", "to stir", "ra.vakh"),
        "H7247": ("רִבְלָה", "N:N--L", "Riblah", "riv.lah"),
        "H7248": ("רַב־מָג", "N:N-M-P", "Rag-mag", "rav-mag"),
        "H7249": ("רַב־סָרִיס", "N:N-M-P", "Rab-Saris", "rav-sa.ris"),
        "H7250": ("רָבַע", "H:V", "to mate", "ra.va"),
        "H7251": ("רָבַע", "H:V", "to square", "ra.va"),
        "H7252": ("רֶבַע", "H:V", "to lie down", "re.va"),
        "H7253": ("רֶ֫בַע", "H:N-M", "fourth", "re.va"),
        "H7254": ("רֶ֫בַע", "N:N-M-P", "Reba", "re.va"),
        "H7255": ("רֹ֫בַע", "H:N-M", "fourth", "ro.va"),
        "H7256": ("רִבֵּעַ", "H:A", "fourth", "rib.be.a"),
        "H7257": ("רָבַץ", "H:V", "to stretch", "ra.vats"),
        "H7258": ("רֵ֫בֶץ", "H:N-M", "rest", "re.vets"),
        "H7259": ("רִבְקָה", "N:N-F-P", "Rebekah", "riv.qah"),
        "H7260": ("רַבְרַב", "A:A", "great", "rav.rav"),
        "H7261": ("רַבְרְבָן", "A:N-M", "noble", "rav.re.van"),
        "H7262": ("רַבְשָׁקֵה", "N:N-M-P", "Rabshakeh", "rav.sha.qeh"),
        "H7263": ("רֶ֫גֶב", "H:N-M", "clod", "re.gev"),
        "H7264": ("רָגַז", "H:V", "to tremble", "ra.gaz"),
        "H7265": ("רְגַז", "A:V", "to enrage", "re.gaz"),
        "H7266": ("רְגַז", "A:N-M", "rage", "re.gaz"),
        "H7267": ("רֹ֫גֶז", "H:N-M", "turmoil", "ro.gez"),
        "H7268": ("רַגָּז", "H:A", "quivering", "rag.gaz"),
        "H7269": ("רׇגְזָה", "H:N-F", "quivering", "rog.zah"),
        "H7270": ("רָגַל", "H:V", "to spy", "ra.gal"),
        "H7271": ("רְגַל", "A:N-F", "foot", "re.gal"),
        "H7272": ("רֶ֫גֶל", "H:N-F", "foot", "re.gel"),
        "H7273": ("רַגְלִי", "H:A", "on foot", "rag.li"),
        "H7274": ("רֹגְלִים", "N:N--L", "Rogelim", "ro.ge.lim"),
        "H7275": ("רָגַם", "H:V", "to stone", "ra.gam"),
        "H7276": ("רֶ֫גֶם", "N:N-M-P", "Regem", "re.gem"),
        "H7277": ("רִגְמָה", "H:N-F", "crowd", "rig.mah"),
        "H7278": ("רֶ֫גֶם מֶ֫לֶךְ", "N:N-M-P", "Regem-melech", "re.gem me.lekh"),
        "H7279": ("רָגַן", "H:V", "to grumble", "ra.gan"),
        "H7280a": ("רָגַע", "H:V", "to disturb", "ra.ga"),
        "H7280b": ("רָגַע", "H:V", "to rest", "ra.ga"),
        "H7280c": ("רָגַע", "H:V", "to harden", "ra.ga"),
        "H7281": ("רֶ֫גַע", "H:N-M", "moment", "re.ga"),
        "H7282": ("רָגֵעַ", "H:A", "restful", "ra.ge.a"),
        "H7283": ("רָגַשׁ", "H:V", "to throng", "ra.gash"),
        "H7284": ("רְגַשׁ", "A:V", "to throng", "re.gash"),
        "H7285a": ("רֶ֫גֶשׁ", "H:N-M", "throng", "re.gesh"),
        "H7285b": ("רִגְשָׁה", "H:N-F", "scheming", ""),
        "H7286": ("רָדַד", "H:V", "to subdue", "ra.dad"),
        "H7287a": ("רָדָה", "H:V", "to rule", "ra.dah"),
        "H7287b": ("רָדָה", "H:V", "to scrape", "ra.dah"),
        "H7288": ("רַדַּי", "N:N-M-P", "Raddai", "rad.day"),
        "H7289": ("רְדִיד", "H:N-M", "veil", "ra.did"),
        "H7290": ("רָדַם", "H:V", "to sleep", "ra.dam"),
        "H7291": ("רָדַף", "H:V", "to pursue", "ra.daph"),
        "H7292": ("רָהַב", "H:V", "to be assertive", "ra.hav"),
        "H7293": ("רַ֫הַב", "N:N--L", "RahabEgypt", "ra.hav"),
        "H7294": ("רַ֫הַב", "N:N-M-T", "pride/Rahab monster", "ra.hav"),
        "H7295": ("רָהָב", "H:A", "proud", "ra.hav"),
        "H7296": ("רֹ֫הָב", "H:N-M", "pride", "ro.hav"),
        "H7297": ("רָהָה", "H:V", "to fear", "ra.hah"),
        "H7298a": ("רַ֫הַט", "H:N-M", "trough", "ra.hat"),
        "H7298b": ("רַ֫הַט", "H:N-M", "lock", "ra.hat"),
        "H7299": ("רֵו", "A:N-M", "appearance", "rev"),
        "H7300": ("רוּד", "H:V", "to roam", "rud"),
        "H7301": ("רָוָה", "H:V", "to quench", "ra.vah"),
        "H7302": ("רָוֶה", "H:A", "watered", "ra.veh"),
        "H7303": ("רָהְגָּה", "N:N-M-P", "Rohgah", "ro.ha.gah"),
        "H7304": ("רָוַח", "H:V", "be wide", "ra.vach"),
        "H7305": ("רֶ֫וַח", "H:N-M", "space", "re.vach"),
        "H7306": ("רִיחַ", "H:V", "to smell", "ru.ach"),
        "H7307": ("רוּחַ", "H:N-F", "spirit", "ru.ach"),
        "H7308": ("רוּחַ", "A:N-F", "spirit", "ru.ach"),
        "H7309": ("רְוָחָה", "H:N-F", "relief", "re.va.chah"),
        "H7310": ("רְוָיָה", "H:N-F", "abundance", "re.va.yah"),
        "H7311a": ("רוּם", "H:V", "to exalt", "rum"),
        "H7311b": ("רָמַם", "H:V", "be rotten", ""),
        "H7312": ("רוּם", "H:N-M", "height", "rum"),
        "H7313": ("רוּם", "A:V", "to rise", "rum"),
        "H7314": ("רוּם", "A:N-M", "height", "rum"),
        "H7315": ("רוֹם", "H:Adv", "highly", "rom"),
        "H7316": ("רוּמָה", "N:N--L", "Rumah", "ru.mah"),
        "H7317": ("רוֹמָה", "H:A", "haughtily", "ro.mah"),
        "H7318": ("רוֹמָם", "H:N-M", "extolling", "ro.mam"),
        "H7319": ("רוֹמְמָה", "H:N-F", "high", "ro.me.mah"),
        "H7320": ("רֹמַ֫מְתִּי עֶ֫זֶר", "N:N-M-P", "Romamti-ezer", "ro.mam.ti e.zer"),
        "H7321": ("רוּעַ", "H:V", "to shout", "ru.a"),
        "H7322": ("רָפַף", "H:V", "to tremble", "ruph"),
        "H7323": ("רוּץ", "H:V", "to run", "ruts"),
        "H7324": ("רִיק", "H:V", "to empty", "ruq"),
        "H7325": ("רִיר", "H:V", "to run", "rur"),
        "H7326": ("רוּשׁ", "H:V", "be poor", "rush"),
        "H7327": ("רוּת", "N:N-F-P", "Ruth", "rut"),
        "H7328": ("רָז", "A:N-M", "mystery", "raz"),
        "H7329": ("רָזָה", "H:V", "to starve", "ra.zah"),
        "H7330": ("רָזֶה", "H:A", "lean", "ra.zeh"),
        "H7331": ("רְזוֹן", "N:N-M-P", "Rezon", "re.zon"),
        "H7332": ("רָזוֹן", "H:N-M", "leanness", "ra.zon"),
        "H7333": ("רָזוֹן", "H:N-M", "prince", "ra.zon"),
        "H7334": ("רָזִי", "H:N-M", "wasting", "ra.zi"),
        "H7335": ("רָזַם", "H:V", "to flash", "ra.zam"),
        "H7336": ("רָזַן", "H:V", "to rule", "ra.zan"),
        "H7337": ("רָחַב", "H:V", "to enlarge", "ra.chav"),
        "H7338": ("רַ֫חַב", "H:N-M", "breadth", "ra.chav"),
        "H7339": ("רְחוֹב", "H:N-F", "street/plaza", "re.chov"),
        "H7340": ("רְחוֹב", "N:N-M-P / N:N--L", "Rehob", "re.chov"),
        "H7341": ("רֹ֫חַב", "H:N-M", "width", "ro.chav"),
        "H7342": ("רָחָב", "H:A", "broad", "ra.chav"),
        "H7343": ("רָחָב", "N:N-F-P", "Rahab", "ra.chav"),
        "H7344": ("רְחֹבוֹת", "N:N--L", "Rehoboth", "re.cho.vot"),
        "H7345": ("רְחַבְיָהוּ", "N:N-M-P", "Rehabiah", "re.chav.yah"),
        "H7346": ("רְחַבְעָם", "N:N-M-P", "Rehoboam", "re.chav.am"),
        "H7347": ("רֵחֶה", "H:N-M", "millstone", "re.cheh"),
        "H7348a": ("רְחוּם", "N:N-M-P", "Rehum", "re.chum"),
        "H7348b": ("רְחוּם", "N:N-M-P", "Rehum", "re.chum"),
        "H7349": ("רַחוּם", "H:A", "compassionate", "ra.chum"),
        "H7350": ("רָחֹק", "H:A", "distant", "ra.choq"),
        "H7351": ("רָהִיט", "H:N-M", "rafter", "re.chit"),
        "H7352": ("רְחִיק", "A:A", "far", "ra.chiq"),
        "H7353": ("רָחֵל", "H:N-F", "ewe", "ra.chel"),
        "H7354": ("רָחֵל", "N:N-F-P", "Rachel", "ra.chel"),
        "H7355": ("רָחַם", "H:V", "to have compassion", "ra.cham"),
        "H7356a": ("רַחַם", "H:N-M", "womb", "ra.cham"),
        "H7356b": ("רַחֲמִים", "H:N-M", "compassion", ""),
        "H7357": ("רַ֫חַם", "N:N-M-P", "Raham", "ra.cham"),
        "H7358": ("רֶ֫חֶם", "H:N-M", "womb", "re.chem"),
        "H7359": ("רַחֲמִין", "A:N-M", "compassion", "re.chem"),
        "H7360": ("רָחָם", "H:N-M", "carrion", "ra.cham"),
        "H7361": ("רַחֲמָה", "H:N-F", "womb", "ra.cha.mah"),
        "H7362": ("רַחֲמָנִי", "H:A", "compassionate", "rach.ma.ni"),
        "H7363a": ("רָחַף", "H:V", "to relax", "ra.chaph"),
        "H7363b": ("רָחַף", "H:V", "to hover", "ra.chaph"),
        "H7364": ("רָחַץ", "H:V", "to wash", "ra.chats"),
        "H7365": ("רְחַץ", "A:V", "to trust", "re.chats"),
        "H7366": ("רַ֫חַץ", "H:N-M", "washing", "ra.chats"),
        "H7367": ("רַחְצָה", "H:N-F", "washing", "rach.tsah"),
        "H7368": ("רָחַק", "H:V", "to remove", "ra.chaq"),
        "H7369": ("רָחֵק", "H:A", "removed", "ra.cheq"),
        "H7370": ("רָחַשׁ", "H:V", "to overflow", "ra.chash"),
        "H7371": ("רַ֫חַת", "H:N-F", "shovel", "ra.chat"),
        "H7372": ("רָטֵב", "H:V", "be moist", "ra.tav"),
        "H7373": ("רָטֹב", "H:A", "fresh", "ra.tov"),
        "H7374": ("רֶ֫טֶט", "H:N-M", "panic", "re.tet"),
        "H7375": ("רֻטֲפַשׁ", "H:V", "be fresh", "ru.ta.phash"),
        "H7376": ("רָטַשׁ", "H:V", "to dash in pieces", "ra.tash"),
        "H7377": ("רִי", "H:N-M", "moisture", "ri"),
        "H7378": ("רִיב", "H:V", "to contend", "riv"),
        "H7379": ("רִיב", "H:N-M", "strife", "riv"),
        "H7380": ("רִיבַי", "N:N-M-P", "Ribai", "ri.vay"),
        "H7381": ("רֵיחַ", "H:N-M", "aroma", "re.ach"),
        "H7382": ("רֵיחַ", "A:N-F", "smell", "re.ach"),
        "H7383": ("רִיפָה", "H:N-F", "grain", "ri.phah"),
        "H7384a": ("דִּיפַת", "N:N-M-P", "Riphath", "ri.phat"),
        "H7384b": ("רִיפַת", "N:N-M-P", "Riphath", "ri.phat"),
        "H7385": ("רִיק", "H:N-M", "vain", "riq"),
        "H7386": ("רֵיק", "H:A", "worthless", "req"),
        "H7387": ("רֵיקָם", "H:Adv", "emptily", "re.qam"),
        "H7388": ("רִיר", "H:N-M", "spittle", "rir"),
        "H7389a": ("רֵישׁ", "H:N-M", "poverty", "resh"),
        "H7389b": ("רִישׁ", "H:N-M", "poverty", ""),
        "H7390": ("רַךְ", "H:A", "tender", "rakh"),
        "H7391": ("רֹךְ", "H:N-M", "tenderness", "rokh"),
        "H7392": ("רָכַב", "H:V", "to ride", "ra.khav"),
        "H7393": ("רֶ֫כֶב", "H:N-M", "chariot", "re.khev"),
        "H7394": ("רֵכָב", "N:N-M-P", "Rechab", "re.khav"),
        "H7395": ("רַכָּב", "H:N-M", "charioteer", "rak.kav"),
        "H7396": ("רִכְבָּה", "H:N-F", "riding", "rikh.bah"),
        "H7397a": ("רֵכָבִי", "N:N--L", "Rechabite", "re.khah"),
        "H7397b": ("רֵכָה", "N:N--L", "Rechah", "re.khah"),
        "H7398": ("רְכוּב", "H:N-M", "chariot", "re.khuv"),
        "H7399": ("רְכוּשׁ", "H:N-M", "property", "re.khush"),
        "H7400": ("רָכִיל", "H:N-M", "slander", "ra.khil"),
        "H7401": ("רָכַךְ", "H:V", "be tender", "ra.khakh"),
        "H7402": ("רָכַל", "H:V", "to trade", "ra.khal"),
        "H7403": ("רָכָל", "N:N--L", "Racal", "ra.khal"),
        "H7404": ("רְכֻלָּה", "H:N-F", "merchandise", "re.khul.lah"),
        "H7405": ("רָכַס", "H:V", "to bind", "ra.khas"),
        "H7406": ("רֶ֫כֶס", "H:N-M", "roughness", "re.khes"),
        "H7407": ("רֹכֶס", "H:N-M", "conspiracy", "ro.khes"),
        "H7408": ("רָכַשׁ", "H:V", "to gather", "ra.khash"),
        "H7409": ("רֶ֫כֶשׁ", "H:N-M", "steed", "re.khesh"),
        "H7410": ("רָם", "N:N-M-P", "Ram", "ram"),
        "H7411a": ("רָמָה", "H:V", "to shoot", "ra.mah"),
        "H7411b": ("רָמָה", "H:V", "to deceive", "ra.mah"),
        "H7412": ("רְמָא", "A:V", "to cast", "re.mah"),
        "H7413": ("רָמָה", "H:N-F", "high place", "ra.mah"),
        "H7414": ("רָמָה", "N:N--L", "Ramah", "ra.mah"),
        "H7415": ("רִמָּה", "H:N-F", "worm", "rim.mah"),
        "H7416": ("רִמּוֹן", "H:N-M", "pomegranate", "rim.mon"),
        "H7417a": ("רִמּוֹן", "N:N-M-T", "Rimmon", "rim.mon"),
        "H7417b": ("רִמּוֹן", "N:N-M-P", "Rimmon", "rim.mon"),
        "H7417c": ("רִמּוֹן, רִמּוֹנוֹ", "N:N--L", "Rimmon", "rim.mon"),
        "H7418": ("רָמוֹת", "N:N--L", "Ramoth", "ra.mot"),
        "H7419": ("רָמוּת", "H:N-F", "refuse", "ra.mut"),
        "H7420": ("רֹ֫מַח", "H:N-M", "spear", "ro.mach"),
        "H7421": ("רַמִּי", "N:N--LG", "Syrian", "ram.mi"),
        "H7422": ("רַמְיָה", "N:N-M-P", "Ramiah", "ram.yah"),
        "H7423a": ("רְמִיָּה", "H:N-F", "deceit", "re.miy.yah"),
        "H7423b": ("רְמִיָּה", "H:N-F", "slackness", "re.miy.yah"),
        "H7424": ("רַמָּךְ", "H:N-F", "mare", "ram.makh"),
        "H7425": ("רְמַלְיָ֜הוּ", "N:N-M-P", "Remaliah", "re.mal.ya.hu"),
        "H7426": ("רָמֹם", "H:V", "be exalted", "ra.mam"),
        "H7427": ("רוֹמֵמוּת", "H:N-F", "uplifting", "ro.me.mut"),
        "H7428": ("רִמֹּן פָּ֑רֶץ", "N:N--L", "Rimmon-perez", "rim.mon pe.rets"),
        "H7429": ("רָמַס", "H:V", "to trample", "ra.mas"),
        "H7430": ("רָמַשׂ", "H:V", "to creep", "ra.ma.s"),
        "H7431": ("רֶ֫מֶשׂ", "H:N-M", "creeping", "re.me.s"),
        "H7432": ("רֶמֶת", "N:N--L", "Remeth", "re.met"),
        "H7433": ("רָמֹת", "N:N-M-P / N:N--L", "Ramoth", "ra.mot gi.l.ad"),
        "H7434": ("רָמַת הַמִּצְפֶּה", "N:N--L", "Ramath-mizpeh", "ra.mat ham.mits.peh"),
        "H7435": ("רָמָתִי", "N:N--LG", "Ramathite", "ra.ma.ti"),
        "H7436": ("רָמָתַיִם צוֹפִים", "N:N--L", "Ramathaim-zophim", "ra.ma.ta.yim tso.phim"),
        "H7437": ("רָמַת לֶחִי", "N:N--L", "Ramath-lehi", "ra.mat le.chi"),
        "H7438": ("רֹן", "H:N-M", "song", "ron"),
        "H7439": ("רָנָה", "H:V", "to rattle", "ra.nah"),
        "H7440": ("רִנָּה", "H:N-F", "cry", "rin.nah"),
        "H7441": ("רִנָּה", "N:N-M-P", "Rinnah", "rin.nah"),
        "H7442a": ("רוּן", "H:V", "to overcome", "ra.nan"),
        "H7442b": ("רָנַן", "H:V", "to sing", "ra.nan"),
        "H7443": ("רְנָנִים", "H:N-M", "ostrich", "re.nen"),
        "H7444": ("רַנֵּן", "H:V", "to sing", "ran.nen"),
        "H7445": ("רְנָנָה", "H:N-F", "triumphing", "re.na.nah"),
        "H7446": ("רִסָּה", "N:N--L", "Rissah", "ris.sah"),
        "H7447a": ("רָסִיס", "H:N-M", "drop", "ra.sis"),
        "H7447b": ("רָסִיס", "H:N-M", "fragment", "ra.sis"),
        "H7448": ("רֶ֫סֶן", "H:N-M", "bridle", "re.sen"),
        "H7449": ("רֶ֫סֶן", "N:N--L", "Resen", "re.sen"),
        "H7450": ("רָסַס", "H:V", "to moisten", "ra.sas"),
        "H7451a": ("רַע", "H:A", "bad", "ra"),
        "H7451b": ("רַע", "H:N-M", "evil", "ra"),
        "H7451c": ("רָעָה", "H:N-F", "distress", ""),
        "H7452": ("רֵעַ", "H:N", "shouting", "re.a"),
        "H7453": ("רֵעַ", "H:N-M", "neighbor", "re.a"),
        "H7454": ("רֵעַ", "H:N-M", "thought", "re.a"),
        "H7455": ("רֹעַ", "H:N-M", "evil", "ro.a"),
        "H7456": ("רָעֵב", "H:V", "be hungry", "ra.ev"),
        "H7457": ("רָעֵב", "H:A", "hungry", "ra.ev"),
        "H7458": ("רָעָב", "H:N-M", "famine", "ra.av"),
        "H7459": ("רְעָבוֹן", "H:N-M", "famine", "re.a.von"),
        "H7460": ("רָעַד", "H:V", "to tremble", "ra.ad"),
        "H7461a": ("רַ֫עַד", "H:N-M", "trembling", "ra.ad"),
        "H7461b": ("רְעַדָה", "H:N-F", "trembling", ""),
        "H7462a": ("בֵּית־עֵ֫קֶד הָרֹעִים", "N:N--L", "of Shepherds", "ra.ah"),
        "H7462b": ("רָעָה", "H:V", "to pasture", "ra.ah"),
        "H7462c": ("רָעָה", "H:V", "to accompany", "ra.ah"),
        "H7462d": ("רָעָה", "H:V", "to befriend", "ra.ah"),
        "H7463": ("רֵעֶה", "H:N-M", "friend", "re.eh"),
        "H7464": ("רֵעָה", "H:N-F", "companion", "re.ah"),
        "H7465": ("רֹעָה", "H:V", "to shatter", "ro.ah"),
        "H7466": ("רְעוּ", "N:N-M-P", "Reu", "re.u"),
        "H7467": ("רְעוּאֵל", "N:N-M-P", "Reuel", "re.u.el"),
        "H7468": ("רְעוּת", "H:N-F", "neighbor", "re.ut"),
        "H7469": ("רְעוּת", "H:N-M", "longing", "re.ut"),
        "H7470": ("רְעוּ", "A:N-F", "pleasure", "re.ut"),
        "H7471": ("רְעִי", "H:N-M", "pasture", "re.i"),
        "H7472": ("רֵעִי", "N:N-M-P", "Rei", "re.i"),
        "H7473": ("רֹעִי", "H:V", "to shepherd", "ro.i"),
        "H7474": ("רַעְיָה", "H:N-F", "darling", "ra.yah"),
        "H7475": ("רַעְיוֹן", "H:N-M", "striving", "ra.yon"),
        "H7476": ("רַעְיוֹן", "A:N-M", "thought", "ra.yon"),
        "H7477": ("רָעַל", "H:V", "to quiver", "ra.al"),
        "H7478": ("רַ֫עַל", "H:N-M", "reeling", "ra.al"),
        "H7479": ("רְעָלָה", "H:N-F", "veil", "ra.a.lah"),
        "H7480": ("רְעֵלָיָה", "N:N-M-P", "Reelaiah", "re.e.la.yah"),
        "H7481": ("רָעַם", "H:V", "to thunder", "ra.am"),
        "H7482": ("רַ֫עַם", "H:N-M", "thunder", "ra.am"),
        "H7483": ("רַעְמָה", "H:N-F", "mane", "ra.mah"),
        "H7484": ("רַעְמָה", "N:N-M-P / N:N--L", "Raamah", "ra.mah"),
        "H7485": ("רַעַמְיָה", "N:N-M-P", "Raamiah", "ra.am.yah"),
        "H7486": ("רַֽעַמְסֵס", "N:N--L", "Rameses", "ra.me.ses"),
        "H7487": ("רַעֲנַן", "A:A", "luxuriant", "ra.a.nan"),
        "H7488a": ("רָעַן", "H:V", "be fresh", "ra.a.nan"),
        "H7488b": ("רַעֲנָן", "H:A", "fresh", "ra.a.nan"),
        "H7489a": ("רָעַע", "H:V", "be evil", "ra.a"),
        "H7489b": ("רָעַע", "H:V", "to shatter", "ra.a"),
        "H7490": ("רְעַע", "A:V", "to break", "re.a"),
        "H7491": ("רָעַף", "H:V", "to drip", "ra.aph"),
        "H7492": ("רָעַץ", "H:V", "to shatter", "ra.ats"),
        "H7493": ("רָעַשׁ", "H:V", "to shake", "ra.ash"),
        "H7494": ("רַ֫עַשׁ", "H:N-M", "quaking", "ra.ash"),
        "H7495": ("רָפָא", "H:V", "to heal", "ra.pah"),
        "H7496": ("רְפָאִים", "H:N-M", "shade", "ra.pah"),
        "H7497a": ("רָפָה", "N:N-M-T", "giant", "ra.pah"),
        "H7497b": ("רְפָאִים", "N:N--TG", "Rephaim", ""),
        "H7498": ("רָפָא", "N:N-M-P", "Rapha", "ra.pah"),
        "H7499": ("רְפֻאָה", "H:N-F", "remedy", "re.phu.ah"),
        "H7500": ("רִפְאוּת", "H:N-F", "healing", "riph.ut"),
        "H7501": ("רְפָאֵל", "N:N-M-P", "Rephael", "re.pha.el"),
        "H7502": ("רָפַד", "H:V", "to spread", "ra.phad"),
        "H7503": ("רָפָה", "H:V", "to slacken", "ra.phah"),
        "H7504": ("רָפֶה", "H:A", "weak", "ra.pheh"),
        "H7505": ("רָפוּא", "N:N-M-P", "Raphu", "ra.phu"),
        "H7506": ("רֶ֫פַח", "N:N-M-P", "Rephah", "re.phach"),
        "H7507": ("רְפִידָה", "H:N-F", "back", "re.phi.dah"),
        "H7508": ("רְפִידִים", "N:N--L", "Rephidim", "re.phi.dim"),
        "H7509": ("רְפָיָה", "N:N-M-P", "Rephaiah", "re.pha.yah"),
        "H7510": ("רִפְיוֹן", "H:N-M", "feebleness", "riph.yon"),
        "H7511": ("רָפַס", "H:V", "to stamp", "ra.phas"),
        "H7512": ("רְפַס", "A:V", "to tread", "re.phas"),
        "H7513": ("רַפְסֹדָה", "H:N-F", "raft", "raph.so.dah"),
        "H7514": ("רָפַק", "H:V", "to lean", "ra.phaq"),
        "H7515": ("רָפַשׂ", "H:V", "to foul", "ra.pha.s"),
        "H7516": ("רֶ֫פֶשׁ", "H:N-M", "mire", "re.phesh"),
        "H7517": ("רֶ֫פֶת", "H:N-M", "stall", "re.phet"),
        "H7518": ("רַץ", "H:N-M", "piece", "rats"),
        "H7519": ("רָצָא", "H:V", "to run", "ra.tsa"),
        "H7520": ("רָצַד", "H:V", "to watch with envy", "ra.tsad"),
        "H7521": ("רָצָה", "H:V", "to accept", "ra.tsah"),
        "H7522": ("רָצוֹן", "H:N-M", "acceptance", "ra.tson"),
        "H7523": ("רָצַח", "H:V", "to murder", "ra.tsach"),
        "H7524": ("רֶ֫צַח", "H:N-M", "shattering", "re.tsach"),
        "H7525": ("רִצְיָא", "N:N-M-P", "Rizia", "rits.ya"),
        "H7526": ("רְצִין", "N:N-M-P", "Rezin", "re.tsin"),
        "H7527": ("רָצַע", "H:V", "to bore", "ra.tsa"),
        "H7528": ("רָצַף", "H:V", "to fit", "ra.tsaph"),
        "H7529": ("רֶצֶף", "H:N-M", "coal", "re.tseph"),
        "H7530": ("רֶ֫צֶף", "N:N--L", "Rezeph", "re.tseph"),
        "H7531a": ("רִצְפָּה", "H:N-F", "pavement", "rits.pah"),
        "H7531b": ("רִצְפָּה", "H:N-F", "glowing stone", "rits.pah"),
        "H7532": ("רִצְפָּה", "N:N-F-P", "Rizpah", "rits.pah"),
        "H7533": ("רָצַץ", "H:V", "to crush", "ra.tsats"),
        "H7534": ("רַק", "H:A", "thin", "raq"),
        "H7535": ("רַק", "H:Adv", "except", "raq"),
        "H7536": ("רֹק", "H:N-M", "spittle", "roq"),
        "H7537": ("רָקֵב", "H:V", "to rot", "ra.qav"),
        "H7538": ("רָקָב", "H:N-M", "rottenness", "ra.qav"),
        "H7539": ("רִקָּבוֹן", "H:N-M", "rottenness", "riq.qa.von"),
        "H7540": ("רָקַד", "H:V", "to skip about", "ra.qad"),
        "H7541": ("רַקָּה", "H:N-F", "temple", "raq.qah"),
        "H7542": ("רַקּוֹן", "N:N--L", "Rakkon", "raq.qon"),
        "H7543": ("רָקַח", "H:V", "to mix", "ra.qach"),
        "H7544": ("רֶ֫קַח", "H:N-M", "spice", "re.qach"),
        "H7545": ("רֹ֫קֵחַ", "H:N-M", "spice", "ro.qach"),
        "H7546": ("רַקָּח", "H:N-M", "perfumer", "raq.qach"),
        "H7547": ("רַקּוּחַ", "H:N-M", "perfumery", "raq.qu.ach"),
        "H7548": ("רַקָּחָה", "H:N-F", "perfumer", "raq.qa.chah"),
        "H7549": ("רָקִיעַ", "H:N-M", "expanse", "ra.qi.a"),
        "H7550": ("רָקִיק", "H:N-M", "flatbread", "ra.qiq"),
        "H7551": ("רָקַם", "H:V", "to weave", "ra.qam"),
        "H7552": ("רֶ֫קֶם", "N:N-M-P / N:N--L", "Rekem", "re.qem"),
        "H7553": ("רִקְמָה", "H:N-F", "embroidery", "riq.mah"),
        "H7554": ("רָקַע", "H:V", "to beat", "ra.qa"),
        "H7555": ("רִקּוּעַ", "H:N-M", "hammered out", "riq.qu.a"),
        "H7556": ("רָקַק", "H:V", "to spit", "ra.qaq"),
        "H7557": ("רַקַּת", "N:N--L", "Rakkath", "raq.qat"),
        "H7558": ("רִשְׁיוֹן", "H:N-M", "permission", "rish.yon"),
        "H7559": ("רָשַׁם", "H:V", "to inscribe", "ra.sham"),
        "H7560": ("רְשַׁם", "A:V", "to sign", "re.sham"),
        "H7561": ("רָשַׁע", "H:V", "be wicked", "ra.sah"),
        "H7562": ("רֶ֫שַׁע", "H:N-M", "wickedness", "re.sah"),
        "H7563": ("רָשָׁע", "H:A", "wicked", "ra.sah"),
        "H7564": ("רִשְׁעָה", "H:N-F", "wickedness", "rish.ah"),
        "H7565": ("רֶ֫שֶׁף", "H:N-M", "flash", "re.sheph"),
        "H7566": ("רֶ֫שֶׁף", "N:N-M-P", "Resheph", "re.sheph"),
        "H7567": ("רָשַׁשׁ", "H:V", "to beat", "ra.shash"),
        "H7568": ("רֶ֫שֶׁת", "H:N-F", "net", "re.shet"),
        "H7569": ("רַתּוֹק", "H:N-M", "chain", "rat.toq"),
        "H7570": ("רָתַח", "H:V", "to boil", "ra.tach"),
        "H7571": ("רֶ֫תַח", "H:N-M", "boiling", "re.tach"),
        "H7572": ("רַתִּיקָה", "H:N-F", "chain", "rat.ti.qah"),
        "H7573": ("רָתַם", "H:V", "to bind", "ra.tam"),
        "H7574": ("רֹ֫תֶם", "H:N-M", "broom", "re.tem"),
        "H7575": ("רִתְמָה", "N:N--L", "Rithmah", "rit.mah"),
        "H7576": ("רָתַק", "H:V", "to bind", "ra.taq"),
        "H7577": ("רְתֻקָה", "H:N-F", "chain", "re.tu.qah"),
        "H7578": ("רְתֵת", "H:N-M", "trembling", "re.tet"),
        "H7579": ("שָׁאַב", "H:V", "to draw", "sha.av"),
        "H7580": ("שָׁאַג", "H:V", "to roar", "sha.ag"),
        "H7581": ("שְׁאָגָה", "H:N-F", "roaring", "she.a.gah"),
        "H7582": ("שָׁאָה", "H:V", "to crash", "sha.ah"),
        "H7583": ("שָׁאָה", "H:V", "to gaze", "sha.ah"),
        "H7584": ("שַׁאֲוָה", "H:N-F", "desolation", "sha.a.vah"),
        "H7585": ("שְׁאוֹל", "N:N--L", "Sheol", "she.ol"),
        "H7586": ("שָׁאוּל", "N:N-M-P", "Saul", "sha.ul"),
        "H7587": ("שָׁאוּלִי", "N:N--PG", "Shaulite", "sha.u.li"),
        "H7588": ("שָׁאוֹן", "H:N-M", "roar", "sha.on"),
        "H7589": ("שְׁאָט", "H:N-M", "scorn", "she.at"),
        "H7590": ("שׁוּט", "H:V", "to despise", "shat"),
        "H7591": ("שְׁאִיָּה", "H:N-F", "ruin", "she.iy.yah"),
        "H7592": ("שָׁאַל", "H:V", "to ask", "sha.al"),
        "H7593": ("שְׁאֵל", "A:V", "to ask", "she.el"),
        "H7594": ("שְׁאָל", "N:N-M-P", "Sheal", "she.al"),
        "H7595": ("שְׁאֵלָא", "A:N-F", "affair", "she.e.la"),
        "H7596": ("שְׁאֵלָה", "H:N-F", "petition", "she.e.lah"),
        "H7597a": ("שְׁאַלְתִּיאֵל", "N:N-M-P", "Shealtiel", "she.al.ti.el"),
        "H7597b": ("שַׁלְתִּיאֵל", "N:N-M-P", "Shealtiel", ""),
        "H7598": ("שְׁאַלְתִּיאֵל", "N:N-M-P", "Shealtiel", "she.al.ti.el"),
        "H7599": ("שָׁאַן", "H:V", "to rest", "sha.an"),
        "H7600": ("שַׁאֲנָן", "H:A", "secure", "sha.a.nan"),
        "H7601": ("שָׁאַס", "H:V", "to spoil", "sha.as"),
        "H7602a": ("שָׁאַף", "H:V", "to long for", "sha.aph"),
        "H7602b": ("שָׁאַף", "H:V", "to crush", "sha.aph"),
        "H7603": ("שְׂאֹר", "H:N-M", "leaven", "se.or"),
        "H7604": ("שָׁאַר", "H:V", "to remain", "sha.ar"),
        "H7605": ("שְׁאָר", "H:N-M", "remnant", "she.ar"),
        "H7606": ("שְׁאָר", "A:N-M", "remainder", "she.ar"),
        "H7607": ("שְׁאֵר", "H:N-M", "flesh", "she.er"),
        "H7608": ("שַׁאֲרָה", "H:N-F", "kinswomen", "sha.a.rah"),
        "H7609": ("שֶׁאֱרָה", "N:N-F-P", "Sheerah", "she.e.rah"),
        "H7610": ("שְׁאָר יָשׁוּב", "N:N-M-P", "Shear-jashub", "she.ar ya.shuv"),
        "H7611": ("שְׁאֵרִית", "H:N-F", "remnant", "she.e.rit"),
        "H7612": ("שֵׁאת", "H:N-F", "devastation", "shet"),
        "H7613": ("שְׂאֵת", "H:N-F", "elevation", "se.et"),
        "H7614": ("שְׁבָא", "N:N-M-P / N:N--L", "Sheba", "she.va"),
        "H7615": ("שְׁבָאִי", "N:N--PG", "Sabean", "she.va.i"),
        "H7616": ("שְׁבָבִים", "H:N-M", "fragment", "sha.vav"),
        "H7617": ("שָׁבָה", "H:V", "to take captive", "sha.vah"),
        "H7618": ("שְׁבוֹ", "H:N-F", "agate", "she.vu"),
        "H7619": ("שְׁבוּאֵל", "N:N-M-P", "Shebuel", "she.vu.el"),
        "H7620": ("שָׁבוּעַ", "H:N-M", "week", "sha.vu.a"),
        "H7621": ("שְׁבוּעָה", "H:N-F", "oath", "she.vu.ah"),
        "H7622": ("שְׁבִית", "H:N-F", "captivity", "she.vut"),
        "H7623a": ("שָׁבַח", "H:V", "to soothe", "sha.vach"),
        "H7623b": ("שָׁבַח", "H:V", "to praise", "sha.vach"),
        "H7624": ("שְׁבַח", "A:V", "to praise", "she.vach"),
        "H7625": ("שְׁבַט", "A:N-M", "tribe", "she.vat"),
        "H7626": ("שֵׁ֫בֶט", "H:N-M", "tribe", "she.vet"),
        "H7627": ("שְׁבָט", "N:N-M-T", "Shebat", "she.vat"),
        "H7628a": ("שְׁבִי", "H:N-M", "captivity", "she.vi"),
        "H7628b": ("שָׁבִי", "H:N-F", "captive", ""),
        "H7629": ("שֹׁבִי", "N:N-M-P", "Shobi", "sho.vi"),
        "H7630": ("שֹׁבָ֑י", "N:N-M-P", "Shobai", "sho.vay"),
        "H7631": ("שְׂבִיב", "A:N-M", "flame", "se.viv"),
        "H7632": ("שָׁבִיב", "H:N-M", "flame", "sha.viv"),
        "H7633": ("שִׁבְיָה", "H:N-F", "captive", "shiv.yah"),
        "H7634": ("שָֽׂכְיָה", "N:N-M-P", "Shachia", "shov.yah"),
        "H7635": ("שְׁבוּל", "H:N-M", "path", "sha.vil"),
        "H7636": ("שָׁבִיס", "H:N-M", "headband", "sha.vis"),
        "H7637": ("שְׁבִיעִי", "H:A", "seventh", "she.vi.i"),
        "H7638": ("שָׂבָךְ", "H:N-M", "net", "sa.vakh"),
        "H7639": ("שְׂבָכָה", "H:N-F", "latticework", "se.va.khah"),
        "H7640": ("שֹׁ֫בֶל", "H:N-M", "skirt", "sho.vel"),
        "H7641a": ("שִׁבֹּ֫לֶת", "H:N-F", "stream", "shib.bol"),
        "H7641b": ("שִׁבֹּ֫לֶת", "H:N-F", "ear", "shib.bol"),
        "H7642": ("שַׁבְּלוּל", "H:N-M", "snail", "shav.lul"),
        "H7643": ("שְׂבָם", "N:N--L", "Sebam", "se.vam"),
        "H7644": ("שֶׁבְנָא", "N:N-M-P", "Shebna", "shev.na"),
        "H7645": ("שְׁבַנְיָהוּ", "N:N-M-P", "Shebaniah", "she.van.yah"),
        "H7646": ("שָׂבֵעַ", "H:V", "to satisfy", "sa.va"),
        "H7647": ("שָׂבָע", "H:N-M", "abundance", "sa.va"),
        "H7648": ("שֹׂ֫בַע", "H:N-M", "satiety", "so.va"),
        "H7649": ("שָׂבֵעַ", "H:A", "sated", "sa.ve.a"),
        "H7650": ("שָׁבַע", "H:V", "to swear", "sha.va"),
        "H7651": ("שֶׁ֫בַע", "H:N", "seven", "she.va"),
        "H7652a": ("שֶׁ֫בַע", "N:N-M-P / N:N--L", "Sheba", "she.va"),
        "H7652b": ("שֶׁבַע", "N:N-M-P / N:N--L", "Sheba", "she.va"),
        "H7653": ("שִׂבְעָה", "H:N-F", "fullness", "siv.ah"),
        "H7654": ("שׇׂבְעָה", "H:N-F", "satiety", "sov.ah"),
        "H7655": ("שִׁבְעָה", "A:N-F", "seven", "shiv.ah"),
        "H7656": ("שִׁבְעָה", "N:N--L", "Shibah", "shiv.ah"),
        "H7657": ("שִׁבְעִים", "H:N", "seventy", "shiv.im"),
        "H7658": ("שִׁבְעָ֫נָה", "H:N", "seven", "shiv.a.nah"),
        "H7659": ("שִׁבְעָתַיִם", "H:N-F", "sevenfold", "shiv.a.ta.yim"),
        "H7660": ("שָׁבַץ", "H:V", "to weave", "sha.vats"),
        "H7661": ("שָׁבָץ", "H:N-M", "agony", "sha.vats"),
        "H7662": ("שְׁבַק", "A:V", "to leave", "she.vaq"),
        "H7663a": ("שָׂבַר", "H:V", "to inspect", "sa.var"),
        "H7663b": ("שָׂבַר", "H:V", "to await", "sa.var"),
        "H7664": ("שֵׂ֫בֶר", "H:N-M", "hope", "se.ver"),
        "H7665": ("שָׁבַר", "H:V", "to break", "sha.var"),
        "H7666": ("שָׁבַר", "H:V", "to buy grain", "sha.var"),
        "H7667": ("שֶׁ֫בֶר", "H:N-M", "breaking", "she.ver"),
        "H7668": ("שֶׁ֫בֶר", "H:N-M", "grain", "she.ver"),
        "H7669": ("שֶׁ֫בֶר", "N:N-M-P", "Sheber", "she.ver"),
        "H7670": ("שִׁבָּרוֹן", "H:N-M", "breaking", "shiv.ron"),
        "H7671": ("שְׁבָרִים", "N:N--L", "Shebarim", "she.va.rim"),
        "H7672": ("שְׁבַשׁ", "A:V", "be perplexed", "she.vash"),
        "H7673a": ("שָׁבַת", "H:V", "to cease", "sha.vat"),
        "H7673b": ("שָׁבַת", "H:V", "to keep", "sha.vat"),
        "H7674": ("שֶׁ֫בֶת", "H:N-F", "cessation", "she.vet"),
        "H7675": ("שֶׁ֫בֶת", "H:N-F", "seat", "she.vet"),
        "H7676": ("שַׁבָּת", "H:N", "sabbath", "shab.bat"),
        "H7677": ("שַׁבָּתוֹן", "H:N-M", "sabbath observance", "shab.ba.ton"),
        "H7678": ("שַׁבְּתַי", "N:N-M-P", "Shabbethai", "shab.b.tay"),
        "H7679": ("שָׂגָא", "H:V", "to grow great", "sa.ga"),
        "H7680": ("שְׂגָא", "A:V", "to grow great", "se.ga"),
        "H7681": ("שָׁגֵה", "N:N-M-P", "Shagee", "sha.ge"),
        "H7682": ("שָׂגַב", "H:V", "to exalt", "sa.gav"),
        "H7683": ("שָׁגַג", "H:V", "to go astray", "sha.gag"),
        "H7684": ("שְׁגָגָה", "H:N-F", "unintentionally", "she.ga.gah"),
        "H7685a": ("שָׂגָה", "H:V", "to grow", "sa.gah"),
        "H7685b": ("שָׂגָה", "H:V", "to increase", "sa.gah"),
        "H7686": ("שָׁגָה", "H:V", "to wander", "sha.gah"),
        "H7687": ("שְׂגוּב", "N:N-M-P", "Segub", "se.guv"),
        "H7688": ("שָׁגַח", "H:V", "to gaze", "sha.gach"),
        "H7689": ("שַׂגִּיא", "H:A", "great", "sag.gi"),
        "H7690": ("שַׂגִּיא", "A:A", "greatly", "sag.gi"),
        "H7691": ("שְׁגִיאָה", "H:N-F", "error", "she.gi.ah"),
        "H7692": ("שִׁגָּיוֹן", "N:N-M-T", "Shiggaion", "shig.ga.von"),
        "H7693": ("שָׁגֵל", "H:V", "to ravish", "sha.gal"),
        "H7694": ("שֵׁגָל", "H:N-F", "queen", "she.gal"),
        "H7695": ("שֵׁגָל", "A:N-F", "consort", "she.gal"),
        "H7696": ("שָׁגַע", "H:V", "be mad", "sha.ga"),
        "H7697": ("שִׁגָּעוֹן", "H:N-M", "madness", "shig.ga.on"),
        "H7698": ("שֶׁ֫גֶר", "H:N-F", "offspring", "she.ger"),
        "H7699a": ("שַׁד", "H:N-M", "breast", "shad"),
        "H7699b": ("שֹׁד", "H:N-M", "breast", ""),
        "H7700": ("שֵׁד", "H:N-M", "demon", "shed"),
        "H7701": ("שֹׁד", "H:N-M", "violence", "shod"),
        "H7702": ("שָׂדַד", "H:V", "to harrow", "sa.dad"),
        "H7703": ("שָׁדַד", "H:V", "to ruin", "sha.dad"),
        "H7704a": ("שָׂדַי", "H:N-M", "field", "sa.deh"),
        "H7704b": ("שָׂדֶה", "H:N-M", "land", "sa.deh"),
        "H7705": ("שִׁדָּה", "H:N-F", "concubine", "shid.dah"),
        "H7706": ("שַׁדַּי", "H:N-M", "Almighty", "shad.day"),
        "H7707": ("שְׁדֵיאוּר", "N:N-M-P", "Shedeur", "she.de.ur"),
        "H7708": ("שִׂדִּים", "N:N--L", "Siddim", "sid.dim"),
        "H7709": ("שְׁדֵמָה", "H:N-F", "field", "she.de.mah"),
        "H7710": ("שָׁדַף", "H:V", "to scorch", "sha.daph"),
        "H7711a": ("שְׁדֵפָה", "H:N-F", "blight", "she.de.phah"),
        "H7711b": ("שִׁדָּפוֹן", "H:N-M", "blight", ""),
        "H7712": ("שְׁדַר", "A:V", "to strive", "she.dar"),
        "H7713": ("שְׂדֵרָה", "H:N-F", "rank", "se.de.rah"),
        "H7714": ("שַׁדְרַךְ", "N:N-M-P", "Shadrach", "shad.rakh"),
        "H7715": ("שַׁדְרַךְ", "N:N-M-P", "Shadrach", "shad.rakh"),
        "H7716": ("שֶׂה", "H:N-M", "sheep", "seh"),
        "H7717": ("שָׂהֵד", "H:N-M", "advocate", "sa.hed"),
        "H7718": ("שֹׁ֫הַם", "H:N-M", "onyx", "sho.ham"),
        "H7719": ("שֹׁ֫הַם", "N:N-M-P", "Shoham", "sho.ham"),
        "H7720": ("שַׂהֲרוֹן", "H:N-M", "crescent", "sa.ha.ron"),
        "H7721": ("שׂוֹא", "H:V", "to arise", "s.vo"),
        "H7722a": ("שׁוֹא", "H:N-M", "ravage", "sho"),
        "H7722b": ("שׁוֹאָה", "H:N-F", "devastation", ""),
        "H7723": ("שָׁוְא", "H:N-M", "vanity", "shav"),
        "H7724": ("שְׁוָא", "N:N-M-P", "Sheva", "she.va"),
        "H7725": ("שׁוּב", "H:V", "to return", "shuv"),
        "H7726": ("שׁוֹבָב", "H:A", "turning back", "sho.vav"),
        "H7727": ("שׁוֹבָב", "N:N-M-P", "Shobab", "sho.vav"),
        "H7728": ("שׁוֹבֵב", "H:A", "backsliding", "sho.vev"),
        "H7729": ("שׁוּבָה", "H:N-F", "returning", "shu.vah"),
        "H7730": ("שׂוֹבֶךְ", "H:N-M", "thicket", "s.vo.vekh"),
        "H7731": ("שׁוֹבַךְ", "N:N-M-P", "Shobach", "sho.vakh"),
        "H7732": ("שׁוֹבָל", "N:N-M-P", "Shobal", "sho.val"),
        "H7733": ("שׁוֹבֵק", "N:N-M-P", "Shobek", "sho.veq"),
        "H7734": ("שׂוּג", "H:V", "to turn back", "sug"),
        "H7735": ("שׂוּג", "H:V", "to grow", "sug"),
        "H7736": ("שׁוּד", "H:V", "to waste", "shud"),
        "H7737a": ("שָׁוָה", "H:V", "be like", "sha.vah"),
        "H7737b": ("שָׁוָה", "H:V", "to set", "sha.vah"),
        "H7738": ("שָׁוָה", "H:N-F", "storm", "sha.vah"),
        "H7739a": ("שְׁוָה", "A:V", "be like", "she.vah"),
        "H7739b": ("שְׁוָה", "A:V", "be set", "she.vah"),
        "H7740": ("שָׁוֵה", "N:N--L", "Shaveh", "sha.veh"),
        "H7741": ("שָׁוֵה קִרְיָתַיִם", "N:N--L", "Shaveh-kiriathaim", "sha.veh qir.ya.ta.yim"),
        "H7742": ("שׂוּחַ", "H:V", "to meditate", "su.ach"),
        "H7743": ("שׁוּחַ", "H:V", "to sink", "shu.ach"),
        "H7744": ("שׁוּחַ", "N:N-M-P", "Shuah", "shu.ach"),
        "H7745": ("שׁוּחָה", "H:N-F", "pit", "shu.chah"),
        "H7746": ("שׁוּחָה", "N:N-M-P", "Shuhah", "shu.chah"),
        "H7747": ("שׁוּחִי", "N:N--PG", "Shuhite", "shu.chi"),
        "H7748": ("שׁוּחָם", "N:N-M-P", "Shuham", "shu.cham"),
        "H7749": ("שׁוּחָמִי", "N:N--PG", "Shuhamite", "shu.cha.mi"),
        "H7750": ("שׂוּט", "H:V", "to swerve", "sut"),
        "H7751a": ("שׁוּט", "H:V", "to rove", "shut"),
        "H7751b": ("שׁוּט", "H:V", "to row", "shut"),
        "H7752": ("שׁוֹט", "H:N-M", "whip", "shot"),
        "H7753": ("שׂוּךְ", "H:V", "to hedge", "sukh"),
        "H7754a": ("שׂוֹךְ", "H:N-M/F", "branch", "s.vokh"),
        "H7754b": ("שׂוֹכָה", "H:N-M/F", "branch", ""),
        "H7755": ("שׂוֹכֹה, שֹׂכֹה", "N:N--L", "Socoh", "s.vo.khoh"),
        "H7756": ("שׂוּכָתִים", "N:N--PG", "Sucathite", "su.kha.ti"),
        "H7757": ("שׁוּל", "H:N-M", "hem", "shul"),
        "H7758": ("שׁוֹלָל", "H:A", "barefoot", "sho.lal"),
        "H7759": ("שׁוּלַמִּית", "N:N-F-L", "Shulammite", "shu.lam.mit"),
        "H7760a": ("שׂוּם", "H:V", "to set", "sum"),
        "H7760b": ("שׂוּמָה", "H:N-F", "fate", ""),
        "H7761": ("שׂוּם", "A:V", "to issue", "sum"),
        "H7762": ("שׁוּם", "H:N-M", "garlic", "shum"),
        "H7763": ("שֹׁמֵר", "N:N-M-P", "Shomer", "sho.mer"),
        "H7764": ("שׁוּנִי", "N:N-M-P", "Shuni", "shu.ni"),
        "H7765": ("שׁוּנִי", "N:N--PG", "Shunite", "shu.ni"),
        "H7766": ("שׁוּנֵם", "N:N--L", "Shunem", "shu.nem"),
        "H7767": ("שׁוּנַמִּית", "N:N-F-L", "Shunammite", "shu.nam.mit"),
        "H7768": ("שָׁוַע", "H:V", "to cry", "sha.va"),
        "H7769": ("שׁוּעַ", "H:N-M", "cry", "shu.a"),
        "H7770": ("שׁוּעַ", "N:N-M-P", "Shua", "shu.a"),
        "H7771a": ("שׁוֹעַ", "H:A", "rich", "sho.a"),
        "H7771b": ("שׁוֹעַ", "H:N-M", "cry", "sho.a"),
        "H7772": ("שׁוֹעַ", "N:N--L", "Shoa", "sho.a"),
        "H7773": ("שֶׁוַע", "H:V", "to cry", "she.va"),
        "H7774": ("שׁוּעָא", "N:N-F-P", "Shua", "shu.a"),
        "H7775": ("שַׁוְעָה", "H:N-F", "cry", "shav.ah"),
        "H7776": ("שׁוּעָל", "H:N-M", "fox", "shu.al"),
        "H7777a": ("שׁוּעָל", "N:N-M-P", "Shual", "shu.al"),
        "H7777b": ("שׁוּעָל", "N:N--L", "Shual", "shu.al"),
        "H7778": ("שֹׁעֵר", "H:N-M", "gatekeeper", "sho.er"),
        "H7779": ("שׁוּף", "H:V", "to bruise", "shuph"),
        "H7780": ("שׁוֹפָךְ", "N:N-M-P", "Shophach", "sho.phakh"),
        "H7781": ("שׁוּפָמִי", "N:N--PG", "Shuphamite", "shu.pha.mi"),
        "H7782": ("שֹׁפָר", "H:N-M", "trumpet", "sho.phar"),
        "H7783": ("שׁוּק", "H:V", "to overflow", "shuq"),
        "H7784": ("שׁוּק", "H:N-M", "street", "shuq"),
        "H7785": ("שׁוֹק", "H:N-F", "leg", "shoq"),
        "H7786": ("שׂוּר", "H:V", "to reign", "sur"),
        "H7787": ("שׂוּר", "H:V", "to saw", "sur"),
        "H7788": ("שׁוּר", "H:V", "to travel", "shur"),
        "H7789": ("שׁוּר", "H:V", "to see", "shur"),
        "H7790": ("שׁוּר", "H:N-M", "enemy", "shur"),
        "H7791a": ("שׁוּר", "H:N-M", "wall", "shur"),
        "H7791b": ("שׁוּרָה", "H:N-F", "row", ""),
        "H7792": ("שׁוּר", "A:N-M", "wall", "shur"),
        "H7793": ("שׁוּר", "N:N--L", "Shur", "shur"),
        "H7794": ("שׁוֹר", "H:N-M", "cattle", "shor"),
        "H7795": ("שׂוֹרָה", "H:N-F", "row", "s.vo.rah"),
        "H7796": ("שֹׂרֵק", "N:N--L", "Sorek", "s.vo.req"),
        "H7797": ("שׂוּשׂ", "H:V", "to rejoice", "su.s"),
        "H7798": ("שַׁוְשָׁא", "N:N-M-P", "Shavsha", "shav.sah"),
        "H7799": ("שׁוּשַׁן", "H:N-M / N:N-M-T", "lily", "shu.shan"),
        "H7800": ("שׁוּשַׁן", "N:N--L", "Susa", "shu.shan"),
        "H7801": ("שׁוּשַׁנְכָיֵא", "N:N--L", "Susahite", "shu.shan.khi"),
        "H7802": ("שׁוּשַׁן עֵדוּת", "N:N-M-T", "Shushan-eduth", "shu.shan e.dut"),
        "H7803": ("שׁוּתֶ֫לַח", "N:N-M-P", "Shuthelah", "shu.te.lach"),
        "H7804": ("שֵׁיזִב", "A:V", "to rescue", "she.zav"),
        "H7805": ("שָׁזַף", "H:V", "to see", "sha.zaph"),
        "H7806": ("שָׁזַר", "H:V", "to twist", "sha.zar"),
        "H7807": ("שַׁח", "H:A", "lowly", "shach"),
        "H7808": ("שֵׂחַ", "H:N-M", "thought", "se.ach"),
        "H7809": ("שָׁחַד", "H:V", "to bribe", "sha.chad"),
        "H7810": ("שֹׁ֫חַד", "H:N-M", "bribe", "sha.chad"),
        "H7811": ("שָׂחָה", "H:V", "to swim", "sa.chah"),
        "H7812": ("שָׁחָה", "H:V", "to bow", "sha.chah"),
        "H7813": ("שָׂחוּ", "H:N-M", "swimming", "sa.chu"),
        "H7814": ("שְׂחֹק", "H:N-M", "laughter", "se.choq"),
        "H7815": ("שְׁחוֹר", "H:N-F", "blackness", "she.chor"),
        "H7816": ("שְׁחוּת", "H:N-F", "pit", "she.chut"),
        "H7817": ("שָׁחַח", "H:V", "to bow", "sha.chach"),
        "H7818": ("שָׂחַט", "H:V", "to squeeze", "sa.chat"),
        "H7819a": ("שָׁחַט", "H:V", "to slaughter", "sha.chat"),
        "H7819b": ("שַׁחֲטָה", "H:N-F", "slaughtering", ""),
        "H7820": ("שָׁחוּט", "H:V", "beaten", "sha.chat"),
        "H7821": ("שְׁחִיטָה", "H:N-F", "slaughtering", "she.chi.tah"),
        "H7822": ("שְׁחִין", "H:N-M", "boil", "she.chin"),
        "H7823": ("סָחִישׁ", "H:N-M", "offspring", "sha.chis"),
        "H7824": ("שָׁחִיף", "H:A", "paneled", "sha.chiph"),
        "H7825": ("שְׁחִית", "H:N-F", "pit", "she.chit"),
        "H7826": ("שַׁ֫חַל", "H:N-M", "lion", "sha.chal"),
        "H7827": ("שְׁחֵ֫לֶת", "H:N-F", "onycha", "she.che.let"),
        "H7828": ("שַׁ֫חַף", "H:N-M", "gull", "sha.chaph"),
        "H7829": ("שַׁחֶ֫פֶת", "H:N-F", "illness", "sha.che.phet"),
        "H7830": ("שַׁ֫חַץ", "H:N-M", "pride", "sha.chats"),
        "H7831": ("שַׁחֲצִ֫ימָה", "N:N--L", "Shahazimah", "sha.cha.tsom"),
        "H7832": ("שָׂחַק", "H:V", "to laugh", "sa.chaq"),
        "H7833": ("שָׁחַק", "H:V", "to beat", "sha.chaq"),
        "H7834": ("שַׁ֫חַק", "H:N-M", "cloud", "sha.chaq"),
        "H7835": ("שָׁחַר", "H:V", "be black", "sha.char"),
        "H7836": ("שָׁחַר", "H:V", "to seek", "sha.char"),
        "H7837": ("שַׁ֫חַר", "H:N-M", "dawn", "sha.char"),
        "H7838": ("שָׁחֹר", "H:A", "black", "sha.chor"),
        "H7839": ("שַׁחֲרוּת", "H:N-F", "dark hair", "sha.cha.rut"),
        "H7840": ("שְׁחַרְחֹר", "H:A", "blackish", "she.char.cho.ret"),
        "H7841": ("שְׁחַרְיָה", "N:N-M-P", "Shehariah", "she.char.yah"),
        "H7842": ("שַׁחֲרַ֫יִם", "N:N-M-P", "Shaharaim", "sha.cha.ra.yim"),
        "H7843": ("שָׁחַת", "H:V", "to ruin", "sha.chat"),
        "H7844": ("שְׁחַת", "A:V", "to corrupt", "she.chat"),
        "H7845": ("שַׁ֫חַת", "H:N-F", "pit", "sha.chat"),
        "H7846": ("שֵׂט", "H:N-M", "rebellion", "set"),
        "H7847": ("שָׂטָה", "H:V", "to turn aside", "sa.tah"),
        "H7848": ("שִׁטָּה", "H:N-F", "acacia", "shit.tah"),
        "H7849": ("שָׁטַח", "H:V", "to spread", "sha.tach"),
        "H7850": ("שֹׁטֵט", "H:N-M", "scourge", "sho.tet"),
        "H7851": ("שִׁטִּים", "N:N--L", "Shittim", "shit.tim"),
        "H7852": ("שָׂטַם", "H:V", "to hate", "sa.tam"),
        "H7853": ("שָׂטַן", "H:V", "to oppose", "sa.tan"),
        "H7854": ("שָׂטָן", "H:N-M / N:N--T", "Satan", "sa.tan"),
        "H7855": ("שִׂטְנָה", "H:N-F", "accusation", "sit.nah"),
        "H7856": ("שִׂטְנָה", "N:N--L", "Sitnah", "sit.nah"),
        "H7857": ("שָׁטַף", "H:V", "to overflow", "sha.taph"),
        "H7858": ("שֶׁ֫טֶף", "H:N-M", "flood", "she.teph"),
        "H7859": ("שְׂטַר", "A:N-M", "side", "she.tar"),
        "H7860": ("שֹׁטֵר", "H:N-M", "official", "sho.ter"),
        "H7861": ("שִׁטְרַי", "N:N-M-P", "Shitrai", "shit.ray"),
        "H7862": ("שַׁי", "H:N-M", "gift", "shay"),
        "H7863": ("שִׂיא", "H:N-M", "loftiness", "si"),
        "H7864": ("שְׁיָא", "N:N-M-P", "Sheya", "she.ya"),
        "H7865": ("שִׂיאֹן", "N:N--L", "Sion", "si.on"),
        "H7866": ("שִׁיאוֹן", "N:N--L", "Shion", "shi.on"),
        "H7867": ("שִׂיב", "H:V", "be gray", "siv"),
        "H7868": ("שִׂיב", "A:V", "be gray", "siv"),
        "H7869": ("שֵׂיב", "H:N-M", "age", "sev"),
        "H7870": ("שִׁיבָה", "H:N-M", "captivity", "shi.vah"),
        "H7871": ("שִׁיבָה", "H:N-F", "sojourn", "shi.vah"),
        "H7872": ("שֵׂיבָה", "H:N-F", "greyheaded", "se.vah"),
        "H7873": ("שִׂיג", "H:N-M", "pursuing", "sig"),
        "H7874": ("שִׂיד", "H:V", "to whitewash", "sid"),
        "H7875": ("שִׂיד", "H:N-M", "lime", "sid"),
        "H7876": ("שָׁיָה", "H:V", "to forget", "sha.yah"),
        "H7877": ("שִׁיזָא", "N:N-M-P", "Shiza", "shi.za"),
        "H7878": ("שִׂיחַ", "H:V", "to muse", "si.ach"),
        "H7879": ("שִׂיחַ", "H:N-M", "complaint", "si.ach"),
        "H7880": ("שִׂיחַ", "H:N-M", "bush", "si.ach"),
        "H7881": ("שִׂיחָה", "H:N-F", "meditation", "si.chah"),
        "H7882": ("שִׁיחָה", "H:N-F", "pit", "shi.chah"),
        "H7883": ("שִׁיחוֹר", "N:N--L", "Shihor", "shi.chor"),
        "H7884": ("שִׁיחוֹר לִבְנָת", "N:N--L", "Shihor-libnath", "shi.chor liv.nat"),
        "H7885": ("שַׁ֫יִט", "H:N-M", "oar", "sha.yit"),
        "H7886": ("שִׁילֹה", "N:N-M-P", "Shiloh", "shi.loh"),
        "H7887": ("שִׁלוֹ", "N:N--L", "Shiloh", "shi.loh"),
        "H7888": ("שִׁילֹנִי", "N:N--LG", "Shilonite", "shi.lo.ni"),
        "H7889": ("שִׁימוֹן", "N:N-M-P", "Shimon", "shi.mon"),
        "H7890": ("שַׁ֫יִן", "H:N-M", "urine", "sha.yin"),
        "H7891": ("שִׁיר", "H:V", "to sing", "shir"),
        "H7892a": ("שִׁיר", "H:N-M", "song", "shir"),
        "H7892b": ("שִׁירָה", "H:N-F", "song", ""),
        "H7893": ("שַׁ֫יִשׁ", "H:N-M", "alabaster", "sha.yish"),
        "H7894": ("שִׁישָׁא", "N:N-M-P", "Shisha", "shi.sah"),
        "H7895": ("שִׁישַׁק", "N:N-M-P", "Shishak", "shi.shaq"),
        "H7896": ("שִׁית", "H:V", "to set", "shit"),
        "H7897": ("שִׁית", "H:N-M", "garment", "shit"),
        "H7898": ("שַׁ֫יִת", "H:N-M", "thornbush", "sha.yit"),
        "H7899": ("שֵׂךְ", "H:N-M", "thorn", "sekh"),
        "H7900": ("שֹׂךְ", "H:N-M", "booth", "sokh"),
        "H7901": ("שָׁכַב", "H:V", "to lie down", "sha.khav"),
        "H7902": ("שְׁכָבָה", "H:N-F", "act/product of lying", "she.kha.vah"),
        "H7903": ("שְׁכֹ֫בֶת", "H:N-F", "copulation", "she.kho.vet"),
        "H7904": ("שָׁכָה", "H:V", "to lust", "sha.khah"),
        "H7905": ("שֻׂכָּה", "H:N-F", "spear", "suk.kah"),
        "H7906": ("שֵׂ֫כוּ", "N:N--L", "Secu", "se.khu"),
        "H7907": ("שֶׂכְוִי", "H:N-M", "heart", "sekh.vi"),
        "H7908": ("שְׁכוֹל", "H:N-M", "bereavement", "she.khol"),
        "H7909": ("שַׁכּוּל", "H:A", "childless", "shak.kul"),
        "H7910": ("שִׁכֹּר", "H:A", "drunken", "shik.kor"),
        "H7911": ("שָׁכַח", "H:V", "to forget", "sha.khach"),
        "H7912": ("שְׁכַח", "A:V", "to find", "she.khach"),
        "H7913": ("שָׁכֵחַ", "H:A", "forgetful", "sha.khe.ach"),
        "H7914": ("שְׂכִיָּה", "H:N-F", "image", "se.khiy.yah"),
        "H7915": ("שַׂכִּין", "H:N-M", "knife", "sak.kin"),
        "H7916": ("שָׂכִיר", "H:A", "hired", "sa.khir"),
        "H7917": ("שְׂכִירָה", "H:A", "hired", "se.khi.rah"),
        "H7918": ("שָׁכַךְ", "H:V", "to subside", "sha.khakh"),
        "H7919a": ("שָׂכַל", "H:V", "be prudent", "sa.khal"),
        "H7919b": ("שָׂכַל", "H:V", "to cross hands", "sa.khal"),
        "H7920": ("שְׂכַל", "A:V", "to contemplate", "se.khal"),
        "H7921a": ("שָׁכֹל", "H:V", "be bereaved", "sha.khol"),
        "H7921b": ("שָׁכוּל", "H:A", "barren", ""),
        "H7921c": ("מְשַׁכֶּ֫לֶת", "H:N-F", "barenness", ""),
        "H7922": ("שֵׂ֫כֶל", "H:N-M", "understanding", "se.khel"),
        "H7923": ("שִׁכֻּלִים", "H:N-M", "bereavement", "shik.ku.lim"),
        "H7924": ("שׇׂכְלְתָנוּ", "A:N-F", "insight", "sokh.le.ta.nu"),
        "H7925": ("שָׁכַם", "H:V", "to rise", "sha.kham"),
        "H7926": ("שְׁכֶם", "H:N-M", "shoulder", "she.khem"),
        "H7927": ("שְׁכֶם", "N:N-M-P / N:N--L", "Shechem", "she.khem"),
        "H7928": ("שֶׁ֫כֶם", "N:N-M-P", "Shechem", "she.khem"),
        "H7929": ("שִׁכְמָה", "H:N-F", "shoulder", "shikh.mah"),
        "H7930": ("שִׁכְמִי", "N:N--PG", "Shechemite", "shikh.mi"),
        "H7931": ("שָׁכַן", "H:V", "to dwell", "sha.khan"),
        "H7932": ("שְׁכֵן", "A:V", "to dwell", "she.khan"),
        "H7933": ("שֵׁ֫כֶן", "H:N-M", "habitation", "she.khen"),
        "H7934": ("שָׁכֵן", "H:A", "neighboring", "sha.khen"),
        "H7935": ("שְׁכַנְיָהוּ", "N:N-M-P", "Shecaniah", "she.khan.yah"),
        "H7936a": ("סָכַר", "H:V", "to hire", "sa.khar"),
        "H7936b": ("שָׂכַר", "H:V", "to hire", "sa.khar"),
        "H7937": ("שָׁכַר", "H:V", "be drunk", "sha.khar"),
        "H7938": ("שֶׂ֫כֶר", "H:N-M", "hire", "se.kher"),
        "H7939": ("שָׂכָר", "H:N-M", "wages", "sa.khar"),
        "H7940": ("שָׂכָר", "N:N-M-P", "Sacar", "sa.khar"),
        "H7941": ("שֵׁכָר", "H:N-M", "strong drink", "she.khar"),
        "H7942": ("שִׁכְּרוֹן", "N:N--L", "Shikkeron", "shik.k.ron"),
        "H7943": ("שִׁכָּרוֹן", "H:N-M", "drunkenness", "shik.ka.ron"),
        "H7944": ("שַׁל", "H:N-M", "irreverence", "shal"),
        "H7945": ("שֶׁל", "H:RelP", "which", "shel"),
        "H7946": ("שַׁלְאֲנָן", "H:A", "at ease", "shal.a.nan"),
        "H7947": ("שָׁלַב", "H:V", "to fit", "sha.lav"),
        "H7948": ("שַׁלָבִּים", "H:N-M", "frame", "sha.lav"),
        "H7949": ("שָׁלַג", "H:V", "to snow", "sha.lag"),
        "H7950": ("שֶׁ֫לֶג", "H:N-M", "snow", "she.leg"),
        "H7951": ("שָׁלָה", "H:V", "to prosper", "sha.lah"),
        "H7952": ("שָׁלָה", "H:V", "to neglect", "sha.lah"),
        "H7953": ("שָׁלָה", "H:V", "to extract", "sha.lah"),
        "H7954": ("שְׁלֵה", "A:V", "safe", "she.lah"),
        "H7955": ("שָׁלָה", "A:N-F", "error", "sha.lah"),
        "H7956": ("שֵׁלָה", "N:N-M-P", "Shelah", "she.lah"),
        "H7957": ("שַׁלְהֶ֫בֶת", "H:N-F", "flame", "shal.he.vet"),
        "H7958": ("שְׂלָו", "H:N-F", "quail", "se.lav"),
        "H7959": ("שָׁ֫לוּ", "H:N-M", "prosperity", "she.lev"),
        "H7960": ("שָׁלוּ", "A:N-F", "neglect", "sha.lu"),
        "H7961": ("שָׁלֵו", "H:A", "at ease", "sha.lev"),
        "H7962": ("שַׁלְוָה", "H:N-F", "ease", "shal.vah"),
        "H7963": ("שְׁלֵוָה", "A:N-F", "ease", "she.le.vah"),
        "H7964": ("שִׁלּוּחִים", "H:N-M", "parting gift", "shil.lu.ach"),
        "H7965": ("שָׁלוֹם", "H:N-M", "peace", "sha.lom"),
        "H7966": ("שִׁלּוּם", "H:N-M", "recompense", "shil.lum"),
        "H7967": ("שַׁלּוּם", "N:N-M-P", "Shallum", "shal.lum"),
        "H7968": ("שַׁלּוּן", "N:N-M-P", "Shallum", "shal.lun"),
        "H7969": ("שָׁלֹשׁ", "H:N", "three", "sha.losh"),
        "H7970": ("שְׁלֹשִׁים", "H:N-M", "thirty", "she.lo.shim"),
        "H7971": ("שָׁלַח", "H:V", "to send", "sha.lach"),
        "H7972": ("שְׁלַח", "A:V", "to send", "she.lach"),
        "H7973": ("שֶׁ֫לַח", "H:N-M", "missile", "she.lach"),
        "H7974": ("שֶׁ֫לַח", "N:N-M-P", "Shelah", "she.lach"),
        "H7975a": ("שֶׁ֫לַח", "N:N--L", "Shelah", "shi.lo.ach"),
        "H7975b": ("שִׁלֹחַ", "N:N--L", "Shiloah", "shi.lo.ach"),
        "H7976": ("שְׁלּוּחָה", "H:N-F", "shoot", "shil.lu.chah"),
        "H7977": ("שִׁלְחִי", "N:N-M-P", "Shilhi", "shil.chi"),
        "H7978": ("שִׁלְחִים", "N:N--L", "Shilhim", "shil.chim"),
        "H7979": ("שֻׁלְחָן", "H:N-M", "table", "shul.chan"),
        "H7980": ("שָׁלַט", "H:V", "to domineer", "sha.lat"),
        "H7981": ("שְׁלֵט", "A:V", "to rule", "she.let"),
        "H7982": ("שֶׁ֫לֶט", "H:N-M", "shield", "she.let"),
        "H7983": ("שִׁלְטוֹן", "H:N-M", "authority", "shil.ton"),
        "H7984": ("שִׁלְטוֹן", "A:N-M", "governor", "shil.ton"),
        "H7985": ("שׇׁלְטָן", "A:N-M", "dominion", "shol.tan"),
        "H7986": ("שַׁלֶּטֶת", "H:A", "imperious", "shal.le.tet"),
        "H7987": ("שְׁלִי", "H:N-M", "quietness", "she.li"),
        "H7988": ("שִׁלְיָה", "H:N-F", "afterbirth", "shil.yah"),
        "H7989": ("שַׁלִּיט", "H:A", "domineering", "shal.lit"),
        "H7990": ("שַׁלִּיט", "A:A", "ruling", "shal.lit"),
        "H7991a": ("שָׁלִישׁ", "H:N-M", "third", "sha.lish"),
        "H7991b": ("שָׁלִישׁ", "H:N-M", "triangle", "sha.lish"),
        "H7991c": ("שָׁלִישׁ", "H:N-M", "officer", "sha.lish"),
        "H7992": ("שְׁלִישִׁי", "H:A", "third", "she.li.shi"),
        "H7993": ("שָׁלַךְ", "H:V", "to throw", "sha.lakh"),
        "H7994": ("שָׁלָךְ", "H:N-M", "cormorant", "sha.lakh"),
        "H7995": ("שַׁלֶּ֫כֶת", "H:N-F", "felling", "shal.le.khet"),
        "H7996": ("שַׁלֶּ֫כֶת", "N:N--L", "Shallecheth", "shal.le.khet"),
        "H7997a": ("שָׁלַל", "H:V", "to take", "sha.lal"),
        "H7997b": ("שָׁלַל", "H:V", "to loot", "sha.lal"),
        "H7998": ("שָׁלָל", "H:N-M", "spoil", "sha.lal"),
        "H7999a": ("שָׁלֵם", "H:V", "to complete", "sha.lam"),
        "H7999b": ("שָׁלַם", "H:V", "to ally", "sha.lam"),
        "H8000": ("שְׁלֵם", "A:V", "be complete", "she.lam"),
        "H8001": ("שְׁלָם", "A:N-M", "peace", "she.lam"),
        "H8002": ("שֶׁ֫לֶם", "H:N-M", "peace offering", "she.lem"),
        "H8003": ("שָׁלֵם", "H:A", "complete", "sha.lem"),
        "H8004": ("שָׁלֵם", "N:N--L", "Salem", "sha.lem"),
        "H8005": ("שִׁלֵּם", "H:N-M", "recompense", "shil.lem"),
        "H8006": ("שִׁלֵּם", "N:N-M-P", "Shillem", "shil.lem"),
        "H8007": ("שַׂלְמָא", "N:N-M-P", "Salma", "sal.ma"),
        "H8008": ("שַׂלְמָה", "H:N-F", "garment", "sal.mah"),
        "H8009": ("שַׂלְמָה", "N:N-M-P", "Salmon", "sal.mah"),
        "H8010": ("שְׁלֹמֹה", "N:N-M-P", "Solomon", "she.lo.moh"),
        "H8011": ("שִׁלֻּמָה", "H:N-F", "recompense", "shil.lu.mah"),
        "H8012": ("שַׂלְמוֹן", "N:N-M-P", "Salmon", "sal.mon"),
        "H8013": ("שְׁלֹמוֹת", "N:N-M-P", "Shelomoth", "she.lo.mot"),
        "H8014": ("שַׂלְמַי", "N:N-M-P", "Shalmai", "sal.may"),
        "H8015": ("שְׁלֹמִי", "N:N-M-P", "Shelomi", "she.lo.mi"),
        "H8016": ("שִׁלֵּמִי", "N:N--PG", "Shillemite", "shil.le.mi"),
        "H8017": ("שְׁלֻמִיאֵל", "N:N-M-P", "Shelumiel", "she.lu.mi.el"),
        "H8018": ("שֶׁלֶמְיָהוּ", "N:N-M-P", "Shelemiah", "she.lem.yah"),
        "H8019a": ("שְׁלֹמִית", "N:N-M-P", "Shelomith", "she.lo.mit"),
        "H8019b": ("שְׁלֹמִית", "N:N-F-P", "Shelomith", "she.lo.mit"),
        "H8020": ("שַׁלְמַן", "N:N-M-P", "Shalman", "shal.man"),
        "H8021": ("שַׁלְמֹן", "H:N-M", "reward", "shal.mon"),
        "H8022": ("שַׁלְמַנְאֶ֫סֶר", "N:N-M-P", "Shalmaneser", "shal.man.e.ser"),
        "H8023": ("שִׁלֹנִי", "N:N-M-P", "Shiloni", "shi.lo.ni"),
        "H8024": ("שֵׁלָנִי", "N:N--PG", "Shelanite", "she.la.ni"),
        "H8025": ("שָׁלַף", "H:V", "to draw", "sha.laph"),
        "H8026": ("שֶׁ֫לֶף", "N:N-M-P", "Sheleph", "she.leph"),
        "H8027": ("שָׁלַשׁ", "H:V", "to do three", "sha.lash"),
        "H8028": ("שֶׁ֫לֶשׁ", "N:N-M-P", "Shelesh", "she.lesh"),
        "H8029": ("שִׁלֵּשׁ", "H:A", "third", "shil.lesh"),
        "H8030": ("שִׁלְשָׁה", "N:N-M-P", "Shilshah", "shil.shah"),
        "H8031": ("שָׁלִ֫שָׁה", "N:N--L", "Shalishah", "sha.li.shah"),
        "H8032": ("שִׁלְשֹׁם", "H:Adv", "three days ago", "shil.shom"),
        "H8033": ("שָׁם", "H:Adv", "there", "sham"),
        "H8034": ("שֵׁם", "H:N-M", "name", "shem"),
        "H8035": ("שֵׁם", "N:N-M-P", "Shem", "shem"),
        "H8036": ("שֻׁם", "A:N-M", "name", "shum"),
        "H8037": ("שַׁמָּא", "N:N-M-P", "Shamma", "sham.ma"),
        "H8038": ("שֶׁמְאֵ֫בֶר", "N:N-M-P", "Shemeber", "shem.e.ver"),
        "H8039": ("שִׁמְאָה", "N:N-M-P", "Shimeah", "shim.ah"),
        "H8040": ("שְׂמֹאל", "H:N-M", "left", "se.mo.l"),
        "H8041": ("שִׂמְאֵל", "H:V", "to go left", "sa.mal"),
        "H8042": ("שְׂמָאלִי", "H:A", "left", "se.ma.li"),
        "H8043": ("שִׁמְאָם", "N:N-M-P", "Shimeam", "shim.am"),
        "H8044": ("שַׁמְגַּר", "N:N-M-P", "Shamgar", "sham.gar"),
        "H8045": ("שָׁמַד", "H:V", "to destroy", "sha.mad"),
        "H8046": ("שְׁמַד", "A:V", "to destroy", "she.mad"),
        "H8047": ("שַׁמָּה", "H:N-F", "horror", "sham.mah"),
        "H8048": ("שַׁמָּה", "N:N-M-P", "Shammah", "sham.mah"),
        "H8049": ("שַׁמְהוּת", "N:N-M-P", "Shamhuth", "sham.hut"),
        "H8050": ("שְׁמוּאֵל", "N:N-M-P", "Samuel", "she.mu.el"),
        "H8051": ("שַׁמּוּעַ", "N:N-M-P", "Shammua", "sham.mu.a"),
        "H8052": ("שְׁמוּעָה", "H:N-F", "tidings", "she.mu.ah"),
        "H8053": ("שָׁמוּר", "N:N-M-P", "Shamir", "sha.mur"),
        "H8054": ("שַׁמּוֹת", "N:N-M-P", "Shammoth", "sham.mot"),
        "H8055": ("שָׂמַח", "H:V", "to rejoice", "sa.mach"),
        "H8056": ("שָׂמֵחַ", "H:A", "glad", "sa.me.ach"),
        "H8057": ("שִׂמְחָה", "H:N-F", "joy", "sim.chah"),
        "H8058": ("שָׁמַט", "H:V", "to release", "sha.mat"),
        "H8059": ("שְׁמִטָּה", "H:N-F", "remission", "she.mit.tah"),
        "H8060": ("שַׁמַּי", "N:N-M-P", "Shammai", "sham.may"),
        "H8061": ("שְׁמִידָ֑ע", "N:N-M-P", "Shemida", "she.mi.da"),
        "H8062": ("שְׁמִידָעִי", "N:N--PG", "Shemidaite", "she.mi.da.i"),
        "H8063": ("שְׂמִיכָה", "H:N-F", "rug", "se.mi.khah"),
        "H8064": ("שָׁמַיִם", "H:N-M", "heaven", "sha.ma.yim"),
        "H8065": ("שָׁמַ֫יִן", "A:N-M", "heaven", "sha.ma.yin"),
        "H8066": ("שְׁמִינִי", "H:A", "eighth", "she.mi.ni"),
        "H8067": ("שְׁמִינִית", "H:N-F", "Sheminith", "she.mi.nit"),
        "H8068": ("שָׁמִיר", "H:N-M", "thorn", "sha.mir"),
        "H8069": ("שָׁמִיר", "N:N--L", "Shamir", "sha.mir"),
        "H8070": ("שְׁמִירָמוֹת", "N:N-M-P", "Shemiramoth", "she.mi.ra.mot"),
        "H8071": ("שִׂמְלָה", "H:N-F", "mantle", "sim.lah"),
        "H8072": ("שַׂמְלָה", "N:N-M-P", "Samlah", "sam.lah"),
        "H8073": ("שַׁמְלַי", "N:N-M-P", "Shalmai", "sham.lay"),
        "H8074": ("שָׁמֵם", "H:V", "be desolate", "sha.mem"),
        "H8075": ("שְׁמַם", "A:V", "be appalled", "she.mam"),
        "H8076": ("שָׁמֵם", "H:A", "devastated", "sha.mem"),
        "H8077a": ("שְׁמָמָה", "H:N-F", "devastation", "she.ma.mah"),
        "H8077b": ("שִֽׁמְמָה", "H:N-F", "desolation", ""),
        "H8078": ("שִׁמָּמוֹן", "H:N-M", "horror", "shim.ma.mon"),
        "H8079": ("שְׂמָמִית", "H:N-F", "lizard", "se.ma.mit"),
        "H8080": ("שָׁמֵן", "H:V", "to grow fat", "sha.man"),
        "H8081": ("שֶׁ֫מֶן", "H:N-M", "oil", "she.men"),
        "H8082": ("שָׁמֵן", "H:A", "rich", "sha.men"),
        "H8083": ("שְׁמֹנֶה", "H:N", "eight", "she.mo.neh"),
        "H8084": ("שְׁמֹנִים", "H:A", "eighty", "she.mo.nim"),
        "H8085": ("שָׁמַע", "H:V", "to hear", "sha.ma"),
        "H8086": ("שְׁמַע", "A:V", "to hear", "she.ma"),
        "H8087": ("שֶׁ֫מַע", "N:N-M-P", "Shema", "she.ma"),
        "H8088a": ("שֶׁ֫מַע", "H:N-M", "sound", "she.ma"),
        "H8088b": ("שֵׁ֫מַע", "H:N-M", "report", "she.ma"),
        "H8089": ("שֹׁ֫מַע", "H:N-M", "report", "sho.ma"),
        "H8090": ("שְׁמַע", "N:N--L", "Shema", "she.ma"),
        "H8091": ("שָׁמָע", "N:N-M-P", "Shama", "sha.ma"),
        "H8092": ("שִׁמְעָא", "N:N-M-P", "Shimea", "shim.a"),
        "H8093": ("שִׁמְעָה", "N:N-M-P", "Shimeah", "shim.ah"),
        "H8094": ("שְׁמָעָה", "N:N-M-P", "Shemaah", "she.ma.ah"),
        "H8095": ("שִׁמְעוֹן", "N:N-M-P", "Simeon", "shim.on"),
        "H8096": ("שִׁמְעִי", "N:N-M-P", "Shimeah", "shim.i"),
        "H8097": ("שִׁמְעִי", "N:N--PG", "Shimeite", "shim.i"),
        "H8098": ("שְׁמַעְיָ֫הוּ", "N:N-M-P", "Shemaiah", "she.ma.yah"),
        "H8099": ("שִׁמְעֹנִי", "N:N--PG", "Simeonite", "shim.o.ni"),
        "H8100": ("שִׁמְעָת", "N:N-F-P", "Shimeath", "shim.at"),
        "H8101": ("שִׁמְעָתִים", "N:N--PG", "Shimeathite", "shim.a.ti"),
        "H8102": ("שֵׁ֫מֶץ", "H:N-M", "whisper", "she.mets"),
        "H8103": ("שִׁמְצָה", "H:N-F", "derision", "shim.tsah"),
        "H8104": ("שָׁמַר", "H:V", "to keep", "sha.mar"),
        "H8105": ("שֶׁ֫מֶר", "H:N-M", "dreg", "she.mer"),
        "H8106a": ("שֶׁ֫מֶר", "N:N-M-P", "Shemer", "she.mer"),
        "H8106b": ("שָׁ֫מֶד", "N:N-M-P", "Shemed", "she.mer"),
        "H8107": ("שִׁמֻּר", "H:N-M", "vigil", "shim.mur"),
        "H8108": ("שׇׁמְרָה", "H:N-F", "guard", "shom.rah"),
        "H8109": ("שְׁמֻרָה", "H:N-F", "waking", "she.mu.rah"),
        "H8110a": ("שִׁמְרוֹן", "N:N--L", "Shimron", "shim.ron"),
        "H8110b": ("שִׁמְרוֹן", "N:N-M-P", "Shimron", "shim.ron"),
        "H8111": ("שֹׁמְרוֹן", "N:N--L", "Samaria", "sho.me.ron"),
        "H8112": ("שִׁמְרוֹן מְראוֹן", "N:N--L", "Shimron-meron", "shim.ron me.r.on"),
        "H8113": ("שִׁמְרִי", "N:N-M-P", "Shimri", "shim.ri"),
        "H8114": ("שְׁמַרְיָ֫הוּ", "N:N-M-P", "Shamariah", "she.mar.yah"),
        "H8115": ("שׇֽׁמְרָ֑יִן", "N:N--L", "Samaria", "shom.ra.yin"),
        "H8116": ("שִׁמְרִית", "N:N-F-P", "Shimrith", "shim.rit"),
        "H8117": ("שִׁמְרֹנִי", "N:N--PG", "Shimronite", "shim.ro.ni"),
        "H8118": ("שֹׁמרֹנִי", "N:N-M-P", "Samaritan", "sho.m.ro.ni"),
        "H8119": ("שִׁמְרָת", "N:N-M-P", "Shimrath", "shim.rat"),
        "H8120": ("שְׁמַשׁ", "A:V", "to minister", "she.mash"),
        "H8121": ("שֶׁ֫מֶשׁ", "H:N", "sun", "she.mesh"),
        "H8122": ("שְׁמַשׁ", "A:N-M", "sun", "she.mesh"),
        "H8123": ("שִׁמְשׁוֹן", "N:N-M-P", "Samson", "shim.shon"),
        "H8124": ("שִׁמְשַׁי", "N:N-M-P", "Shimshai", "shim.shay"),
        "H8125": ("שַׁמְשְׁרַי", "N:N-M-P", "Shamsherai", "sham.she.ray"),
        "H8126": ("שֻׁמָתִי", "N:N--LG", "Shumathite", "shu.ma.ti"),
        "H8127": ("שֵׁן", "H:N-F", "tooth", "shen"),
        "H8128": ("שֵׁן", "A:N", "tooth", "shen"),
        "H8129": ("שֵׁן", "N:N--L", "Shen", "shen"),
        "H8130": ("שָׂנֵא", "H:V", "to hate", "sa.ne"),
        "H8131": ("שְׂנָא", "A:V", "to hate", "se.ne"),
        "H8132": ("שָׁנָא", "H:V", "to change", "sha.na"),
        "H8133": ("שְׁנָא", "A:V", "to change", "she.na"),
        "H8134": ("שִׁנְאָב", "N:N-M-P", "Shinab", "shin.av"),
        "H8135": ("שִׂנְאָה", "H:N-F", "hating", "sin.ah"),
        "H8136": ("שִׁנְאָן", "H:N-M", "thousands", "shin.an"),
        "H8137": ("שֶׁנְאַצַּ֑ר", "N:N-M-P", "Shenazzar", "shen.ats.tsar"),
        "H8138a": ("שָׁנָה", "H:V", "to change", "sha.nah"),
        "H8138b": ("שָׁנָה", "H:V", "to repeat", "sha.nah"),
        "H8139": ("שְׁנָה", "A:N-F", "sleep", "she.nah"),
        "H8140": ("שְׁנָה", "A:N-F", "year", "she.nah"),
        "H8141": ("שָׁנָה", "H:N-F", "year", "sha.neh"),
        "H8142": ("שֵׁנָה", "H:N-F", "sleep", "she.nah"),
        "H8143": ("שֶׁנְהַבִּים", "H:N-M", "ivory", "shen.hab.bim"),
        "H8144": ("שָׁנִי", "H:N-M", "scarlet", "sha.ni"),
        "H8145": ("שֵׁנִי", "H:N", "second", "she.ni"),
        "H8146": ("שָׂנִיא", "H:A", "hated", "sa.ni"),
        "H8147": ("שְׁנַ֫יִם", "H:N", "two", "she.na.yim"),
        "H8148": ("שְׁנִינָה", "H:N-F", "taunt", "she.ni.nah"),
        "H8149": ("שְׂנִיר", "N:N--L", "Senir", "she.nir"),
        "H8150": ("שָׁנַן", "H:V", "to sharpen", "sha.nan"),
        "H8151": ("שָׁנַס", "H:V", "to gird", "sha.nas"),
        "H8152": ("שִׁנְעָר", "N:N--L", "Shinar", "shin.ar"),
        "H8153": ("שְׁנָת", "H:N-F", "sleep", "she.nat"),
        "H8154": ("שָׁסָה", "H:V", "to plunder", "sha.sah"),
        "H8155": ("שָׁסַס", "H:V", "to plunder", "sha.sas"),
        "H8156": ("שָׁסַע", "H:V", "to cleave", "sha.sa"),
        "H8157": ("שֶׁ֫סַע", "H:N-M", "cleft", "she.sa"),
        "H8158": ("שָׁסַף", "H:V", "to hew", "sha.saph"),
        "H8159": ("שָׁעָה", "H:V", "to gaze", "sha.ah"),
        "H8160": ("שָׁעָה", "A:N-F", "moment", "sha.ah"),
        "H8161": ("שַׁעֲטָה", "H:N-F", "stamping", "sha.a.tah"),
        "H8162": ("שַׁעַטְנֵז", "H:N-M", "mixed stuff", "sha.at.nez"),
        "H8163a": ("שָׂעִר", "H:A", "hairy", "sa.ir"),
        "H8163b": ("שָׂעִיר", "H:N-M", "he-goat", "sa.ir"),
        "H8163c": ("שָׂעִיר", "H:N-M", "satyr", "sa.ir"),
        "H8164": ("שְׂעִירִים", "H:N-M", "rain", "sa.ir"),
        "H8165a": ("שֵׂעִיר", "N:N--L", "Seir", "se.ir"),
        "H8165b": ("שֵׂעִיר", "N:N-M-P", "Seir", "se.ir"),
        "H8166": ("שְׂעִירָה", "H:N-F", "female goat", "se.i.rah"),
        "H8167": ("שְׂעִירָה", "N:N--L", "Seirah", "se.i.rah"),
        "H8168": ("שֹׁ֫עַל", "H:N-M", "handful", "sho.al"),
        "H8169": ("שַׁעַלְבִים", "N:N--L", "Shaalbim", "sha.al.vim"),
        "H8170": ("שַׁעַלְבֹנִי", "N:N--LG", "Shaalbonite", "sha.al.vo.ni"),
        "H8171": ("שַׁעֲלִים", "N:N--L", "Shaalim", "sha.a.lim"),
        "H8172": ("שָׁעַן", "H:V", "to lean", "sha.an"),
        "H8173a": ("שָׁעַע", "H:V", "to smear", "sha.a"),
        "H8173b": ("שָׁעַע", "H:V", "to delight", "sha.a"),
        "H8174": ("שַׁ֫עַף", "N:N-M-P", "Shaaph", "sha.aph"),
        "H8175a": ("שָׂעַר", "H:V", "to shudder", "sa.ar"),
        "H8175b": ("שָׂעַר", "H:V", "to storm", "sa.ar"),
        "H8175c": ("שָׂעַר", "H:V", "to know", "sa.ar"),
        "H8176": ("שָׁעַר", "H:V", "to calculate", "sha.ar"),
        "H8177": ("שְׂעַר", "A:N-M", "hair", "se.ar"),
        "H8178a": ("שַׂ֫עַר", "H:N-M", "shuddering", "sa.ar"),
        "H8178b": ("שַׂ֫עַר", "H:N-M", "storm", "sa.ar"),
        "H8179": ("שַׁ֫עַר", "H:N-M", "gate", "sha.ar"),
        "H8180": ("שַׁ֫עַר", "H:N-M", "hundredfold", "sha.ar"),
        "H8181": ("שֵׂעָר", "H:N-M", "hair", "se.ar"),
        "H8182": ("שֹׁעָר", "H:A", "disgusting", "sho.ar"),
        "H8183": ("שְׂעָרָה", "H:N-F", "storm", "se.a.rah"),
        "H8184": ("שְׂעֹרָה", "H:N-F", "barley", "se.o.rah"),
        "H8185": ("שַׂעֲרָה", "H:N-F", "hair", "sa.a.rah"),
        "H8186a": ("שַׁעֲרוּרָה", "H:N-F", "horror", "sha.a.ru.rah"),
        "H8186b": ("שַׁעֲרִירִיָּה", "H:N-F", "horror", ""),
        "H8187": ("שְׁעַרְיָה", "N:N-M-P", "Sheariah", "she.ar.yah"),
        "H8188": ("שְׂעֹרִים", "N:N-M-P", "Seorim", "se.o.rim"),
        "H8189": ("שַׁעֲרַ֫יִם", "N:N--L", "Shaaraim", "sha.a.ra.yim"),
        "H8190": ("שַׁעַשְׁגַּז", "N:N-M-P", "Shaashgaz", "sha.ash.gaz"),
        "H8191": ("שַׁעֲשֻׁעִים", "H:N-M", "delight", "sha.shu.a"),
        "H8192": ("שָׁפָה", "H:V", "be bare", "sha.phah"),
        "H8193": ("שָׂפָה", "H:N-F", "lip", "sa.phah"),
        "H8194": ("שְׁפוֹת", "H:N-F", "cheese", "sha.phah"),
        "H8195": ("שְׁפוֹ, שְׁפִי", "N:N-M-P", "Shephi", "she.pho"),
        "H8196": ("שְׁפוֹט", "H:N-M", "judgment", "she.phot"),
        "H8197a": ("שְׁפוּפָם", "N:N-M-P", "Shephupham", "she.phu.pham"),
        "H8197b": ("שְׁפוּפָן", "N:N-M-P", "Shephuphan", ""),
        "H8198": ("שִׁפְחָה", "H:N-F", "maidservant", "shiph.chah"),
        "H8199": ("שָׁפַט", "H:V", "to judge", "sha.phat"),
        "H8200": ("שְׁפַט", "A:V", "to judge", "she.phat"),
        "H8201": ("שֶׁ֫פֶט", "H:N-M", "judgment", "she.phet"),
        "H8202": ("שָׁפָט", "N:N-M-P", "Shaphat", "sha.phat"),
        "H8203": ("שְׁפַטְיָהוּ", "N:N-M-P", "Shephatiah", "she.phat.yah"),
        "H8204": ("שִׁפְטָן", "N:N-M-P", "Shiphtan", "shiph.tan"),
        "H8205": ("שְׁפִי", "H:N-M", "bareness", "she.phi"),
        "H8206": ("שֻׁפִּים", "N:N-M-P", "Shuppim", "shup.pim"),
        "H8207": ("שְׁפִיפֹן", "H:N-M", "horned viper", "she.phi.phon"),
        "H8208": ("שָׁפִיר", "N:N--L", "Shaphir", "sha.phir"),
        "H8209": ("שַׁפִּיר", "A:A", "fair", "shap.pir"),
        "H8210": ("שָׁפַךְ", "H:V", "to pour", "sha.phakh"),
        "H8211": ("שֶׁ֫פֶךְ", "H:N-M", "pouring", "she.phekh"),
        "H8212": ("שׇׁפְכָה", "H:N-F", "penis", "shoph.khah"),
        "H8213": ("שָׁפֵל", "H:V", "to abase", "sha.phel"),
        "H8214": ("שְׁפֵל", "A:V", "be low", "she.phal"),
        "H8215": ("שְׁפַל", "A:A", "low", "she.phal"),
        "H8216": ("שֵׁ֫פֶל", "H:N-M", "poverty", "she.phel"),
        "H8217": ("שָׁפָל", "H:A", "low", "sha.phal"),
        "H8218": ("שִׁפְלָה", "H:N-F", "humiliation", "shiph.lah"),
        "H8219": ("שְׁפֵלָה", "N:N--L", "lowland", "she.phe.lah"),
        "H8220": ("שִׁפְלוּת", "H:N-F", "idleness", "shiph.lut"),
        "H8221": ("שְׁפָם", "N:N--L", "Shepham", "she.pham"),
        "H8222": ("שָׂפָם", "H:N-M", "mustache", "sa.pham"),
        "H8223": ("שָׁפָם", "N:N-M-P", "Shapham", "sha.pham"),
        "H8224": ("שִׂפְמוֹת", "N:N--L", "Siphmoth", "siph.mot"),
        "H8225": ("שִׁפְמִי", "N:N--LG", "Shiphmite", "shiph.mi"),
        "H8226": ("שָׂפַן", "H:V", "to treasure", "sa.phan"),
        "H8227a": ("שָׁפָן", "H:N-M", "rock badger", "sha.phan"),
        "H8227b": ("שָׁפָן", "N:N-M-P", "Shaphan", "sha.phan"),
        "H8228": ("שֶׁ֫פַע", "H:N-M", "abundance", "she.pah"),
        "H8229": ("שִׁפְעָה", "H:N-F", "abundance", "shiph.ah"),
        "H8230": ("שִׁפְעִי", "N:N-M-P", "Shiphi", "shiph.i"),
        "H8231": ("שָׁפַר", "H:V", "to polish", "sha.phar"),
        "H8232": ("שְׁפַר", "A:V", "to acceptable", "she.phar"),
        "H8233": ("שֶׁ֫פֶר", "H:N-M", "beauty", "she.pher"),
        "H8234": ("שֶׁ֫פֶר", "N:N--L", "Shepher", "she.pher"),
        "H8235": ("שִׁפְרָה", "H:N-F", "clearness", "shiph.rah"),
        "H8236": ("שִׁפְרָה", "N:N-F-P", "Shiphrah", "shiph.rah"),
        "H8237": ("שַׁפְרִיר", "H:N-M", "pavilion", "shaph.rur"),
        "H8238": ("שְׁפַרְפָּר", "A:N-M", "dawn", "she.phar.phar"),
        "H8239": ("שָׁפַת", "H:V", "to set", "sha.phat"),
        "H8240a": ("שְׁפַתָּ֑יִם", "H:N-M", "ash heep", "sha.phat"),
        "H8240b": ("שְׁפַתַּ֫יִם", "H:N-M", "hook", "sha.phat"),
        "H8241": ("שֶׁצֶף", "H:N-M", "overflowing", "she.tseph"),
        "H8242": ("שַׂק", "H:N-M", "sackcloth", "saq"),
        "H8243": ("שָׁק", "A:N-M", "leg", "shaq"),
        "H8244": ("שָׂקַד", "H:V", "to bind", "sa.qad"),
        "H8245": ("שָׁקַד", "H:V", "to watch", "sha.qad"),
        "H8246": ("שָׁקַד", "H:V", "be almond shaped", "sha.qad"),
        "H8247": ("שָׁקֵד", "H:N-M", "almond", "sha.qed"),
        "H8248": ("שָׁקָה", "H:V", "to water", "sha.qah"),
        "H8249": ("שִׁקֻּו", "H:N-M", "drink", "shiq.quv"),
        "H8250": ("שִׁקּוּי", "H:N-M", "drink", "shiq.quy"),
        "H8251": ("שִׁקּוּץ", "H:N-M", "abomination", "shiq.quts"),
        "H8252": ("שָׁקַט", "H:V", "to quiet", "sha.qat"),
        "H8253": ("שֶׁ֫קֶט", "H:N-M", "quietness", "she.qet"),
        "H8254": ("שָׁקַל", "H:V", "to weigh", "sha.qal"),
        "H8255": ("שֶׁ֫קֶל", "H:N-M", "shekel", "she.qel"),
        "H8256": ("שִׁקְמָה", "H:N-F", "sycamore", "shiq.mah"),
        "H8257": ("שָׁקַע", "H:V", "to sink", "sha.qa"),
        "H8258": ("שְׁקַעֲרוּרָה", "H:N-F", "hollow", "she.qa.ru.rah"),
        "H8259": ("שָׁקַף", "H:V", "to look", "sha.qaph"),
        "H8260": ("שֶׁ֫קֶף", "H:N-M", "frame", "she.qeph"),
        "H8261": ("שָׁקוּף", "H:N-M", "frame", "sha.quph"),
        "H8262": ("שִׁקֵּץ", "H:V", "to detest", "sha.qats"),
        "H8263": ("שֶׁ֫קֶץ", "H:N-M", "detestation", "she.qets"),
        "H8264": ("שָׁקַק", "H:V", "to rush", "sha.qaq"),
        "H8265": ("שָׂקַר", "H:V", "to ogle", "sa.qar"),
        "H8266": ("שָׁקַר", "H:V", "to deal", "sha.qar"),
        "H8267": ("שֶׁ֫קֶר", "H:N-M", "deception", "she.qer"),
        "H8268": ("שֹׁ֫קֶת", "H:N-F", "trough", "sho.qet"),
        "H8269": ("שַׂר", "H:N-M", "ruler", "sar"),
        "H8270": ("שֹׁר", "H:N-M", "umbilical cord", "shor"),
        "H8271": ("שְׁרָא", "A:V", "to loose", "she.re"),
        "H8272": ("שַׁרְאֶ֫צֶר", "N:N-M-P", "Sharezer", "shar.e.tser"),
        "H8273": ("שָׁרָב", "H:N-M", "scorching", "sha.rav"),
        "H8274": ("שֵׁרֵֽבְיָה", "N:N-M-P", "Sherebiah", "she.re.ve.yah"),
        "H8275": ("שַׁרְבִֿיט", "H:N-M", "scepter", "shar.vit"),
        "H8276": ("שָׂרַג", "H:V", "to intertwine", "sa.rag"),
        "H8277": ("שָׂרַד", "H:V", "to escape", "sa.rad"),
        "H8278": ("שְׂרָד", "H:N-M", "braiding", "se.rad"),
        "H8279": ("שֶׂ֫רֶד", "H:N-M", "stylus", "se.red"),
        "H8280": ("שָׂרָה", "H:V", "to strive", "sa.rah"),
        "H8281": ("שָׁרָה", "H:V", "to free", "sha.rah"),
        "H8282": ("שָׂרָה", "H:N-F", "princess", "sa.rah"),
        "H8283": ("שָׂרָה", "N:N-F-P", "Sarah", "sa.rah"),
        "H8284": ("שָׁרָה", "H:N-F", "vine-row", "sha.rah"),
        "H8285": ("שֵׁרָה", "H:N-F", "bracelet", "she.rah"),
        "H8286": ("שְׂרוּג", "N:N-M-P", "Serug", "se.rug"),
        "H8287": ("שָׁרוּחֶ֑ן", "N:N--L", "Sharuhen", "sha.ru.chen"),
        "H8288": ("שְׂרוֹךְ", "H:N-M", "thong", "se.rokh"),
        "H8289": ("שָׁרוֹן", "N:N--L", "Sharon", "sha.ron"),
        "H8290": ("שָׁרוֹנִי", "N:N--LG", "Sharonite", "sha.ro.ni"),
        "H8291": ("שָׂרֹק", "H:N-M", "tendril", "sa.ruq"),
        "H8292": ("שְׁרִיקָה", "H:N-F", "piping", "she.ru.qah"),
        "H8293": ("שֵׁרוּת", "H:N-F", "remnant", "she.rut"),
        "H8294": ("שֶׂ֫רַח, שָׂ֫רַח", "N:N-F-P", "Serah", "se.rach"),
        "H8295": ("שָׂרַט", "H:V", "to incise", "sa.rat"),
        "H8296a": ("שֶׂ֫רֶט", "H:N-M/F", "incision", "se.ret"),
        "H8296b": ("שָׂרֶ֫טֶת", "H:N-M/F", "incision", ""),
        "H8297": ("שָׂרַי", "N:N-F-P", "Sarai", "sa.ray"),
        "H8298": ("שָׁרָ֑י", "N:N-M-P", "Sharai", "sha.ray"),
        "H8299": ("שָׂרִיג", "H:N-M", "tendril", "sa.rig"),
        "H8300": ("שָׂרִיד", "H:N-M", "survivor", "sa.rid"),
        "H8301": ("שָׂרִיד", "N:N--L", "Sarid", "sa.rid"),
        "H8302a": ("שִׁרְיָה", "H:N-M/F", "lance", "shir.yon"),
        "H8302b": ("שִׁרְיוֹן", "H:N-M/F", "armor", "shir.yon"),
        "H8303": ("שִׂרְיֹן", "N:N--L", "Sirion", "shir.yon"),
        "H8304": ("שְׂרָיָהוּ", "N:N-M-P", "Seraiah", "se.ra.yah"),
        "H8305": ("שָׂרִיק", "H:A", "combed", "se.ri.qah"),
        "H8306": ("שָׁרִיר", "H:N-M", "muscle", "sha.rir"),
        "H8307": ("שְׁרִירוּת", "H:N-F", "stubbornness", "she.ri.rut"),
        "H8308": ("שָׂרַךְ", "H:V", "to twist", "sa.rakh"),
        "H8309": ("שְׁרֵמָה", "H:N-F", "field", "she.re.mah"),
        "H8310": ("שַׂרְסְכִים", "N:N-M-P", "Sarsechim", "sar.se.khim"),
        "H8311": ("שָׂרַע", "H:V", "to extend", "sa.ra"),
        "H8312": ("שַׂרְעַפִּים", "H:N-M", "anxiety", "sar.aph"),
        "H8313": ("שָׂרַף", "H:V", "to burn", "sa.raph"),
        "H8314a": ("שָׂרָף", "H:N-M", "serpent", "sa.raph"),
        "H8314b": ("שָׂרָף", "H:N-M", "Seraph", "sa.raph"),
        "H8315": ("שָׂרָף", "N:N-M-P", "Saraph", "sa.raph"),
        "H8316": ("שְׂרֵפָה", "H:N-F", "fire", "se.re.phah"),
        "H8317": ("שָׁרַץ", "H:V", "to swarm", "sha.rats"),
        "H8318": ("שֶׁ֫רֶץ", "H:N-M", "swarm", "she.rets"),
        "H8319": ("שָׁרַק", "H:V", "to whistle", "sha.raq"),
        "H8320": ("שָׂרֹק", "H:N-M", "sorrel", "sa.ruq"),
        "H8321a": ("שֹׂרֵק", "H:N-M", "vine", "so.req"),
        "H8321b": ("שֹׂרֵקָה", "H:N-F", "vine", ""),
        "H8322": ("שְׁרֵקָה", "H:N-F", "hissing", "she.re.qah"),
        "H8323a": ("שָׂרַר", "H:V", "to rule", "sa.rar"),
        "H8323b": ("שָׁרַר", "H:V", "to strengthen", "sha.rar"),
        "H8324": ("שׁוֹרֵר", "H:N-M", "enemy", "sha.rar"),
        "H8325": ("שָׁרָר", "N:N-M-P", "Sharar", "sha.rar"),
        "H8326": ("שֹׁרֶר", "H:N-M", "navel", "sho.rer"),
        "H8327": ("שָׁרַשׁ", "H:V", "to uproot", "sha.rash"),
        "H8328": ("שֹׁ֫רֶשׁ", "H:N-M", "root", "she.resh"),
        "H8329": ("שָׁ֑רֶשׁ", "N:N-M-P", "Sheresh", "she.resh"),
        "H8330": ("שֹׁ֫רֶשׁ", "A:N-M", "root", "sho.resh"),
        "H8331": ("שַׁרְשָׁה", "H:N-F", "chain", "shar.shah"),
        "H8332": ("שְׁרֹשִׁי", "A:N-F", "banishment", "she.ro.shu"),
        "H8333": ("שַׁרְשְׁרָה", "H:N-F", "chain", "shar.she.rah"),
        "H8334": ("שָׁרַת", "H:V", "to minister", "sha.rat"),
        "H8335": ("שָׁרֵת", "H:N-M", "ministry", "sha.ret"),
        "H8336a": ("שֵׁשׁ", "H:N-M", "alabaster", "shesh"),
        "H8336b": ("שֵׁשׁ", "H:N-M", "linen", "shesh"),
        "H8337": ("שֵׁשׁ", "H:N", "six", "shesh"),
        "H8338": ("שִׁשֵּׁא", "H:V", "to lead", "sha.sah"),
        "H8339": ("שֵׁשְׁבַּצַּר", "N:N-M-P", "Sheshbazzar", "she.she.bats.tsar"),
        "H8340": ("שֵׁשְׁבַּצַּר", "N:N-M-P", "Sheshbazzar", "she.she.bats.tsar"),
        "H8341": ("שִׁשָּׁה", "H:V", "to give a sixth", "sha.shah"),
        "H8342": ("שָׂשׂוֹן", "H:N-M", "rejoicing", "sa.s.von"),
        "H8343": ("שָׁשַׁי", "N:N-M-P", "Shashai", "sha.shay"),
        "H8344": ("שֵׁשַׁי", "N:N-M-P", "Sheshai", "she.shay"),
        "H8345": ("שִׁשִּׁי", "H:N", "sixth", "shish.shi"),
        "H8346": ("שִׁשִּׁים", "H:N", "sixty", "shish.shim"),
        "H8347": ("שֵׁשַׁךְ", "N:N--L", "Sheshak", "she.shakh"),
        "H8348": ("שֵׁשָׁן", "N:N-M-P", "Sheshan", "she.shan"),
        "H8349": ("שָׁשָׁק", "N:N-M-P", "Shashak", "sha.shaq"),
        "H8350": ("שָׁשֵׁר", "H:N-M", "vermilion", "sha.shar"),
        "H8351": ("שֵׁת", "H:N-M", "buttock", "shet"),
        "H8352": ("שֵׁת", "N:N-M-P", "Seth", "shet"),
        "H8353": ("שֵׁת", "A:A", "six", "shet"),
        "H8354": ("שָׁתָה", "H:V", "to drink", "sha.tah"),
        "H8355": ("שְׁתָה", "A:V", "to drink", "she.tah"),
        "H8356": ("שָׁת", "H:N-M", "foundation", "sha.tah"),
        "H8357": ("שֵׁת", "H:N-M", "buttock", "she.tah"),
        "H8358": ("שְׁתִי", "H:N-M", "drunkenness", "she.ti"),
        "H8359": ("שְׁתִי", "H:N-M", "warp", "she.ti"),
        "H8360": ("שְׁתִיָּה", "H:N-F", "drinking", "she.tiy.yah"),
        "H8361": ("שִׁתִּין", "A:N", "sixty", "shit.tin"),
        "H8362": ("שָׁתַל", "H:V", "to transplant", "sha.tal"),
        "H8363": ("שָׁתִיל", "H:N-M", "shoot", "she.til"),
        "H8364": ("שֻׁתַלְחִי", "N:N--PG", "Shuthelahite", "shu.tal.chi"),
        "H8365": ("שָׁתַם", "H:V", "to open", "sha.tam"),
        "H8366": ("שָׁתַן", "H:V", "to urinate", "sha.tan"),
        "H8367": ("שָׁתַק", "H:V", "be quiet", "sha.taq"),
        "H8368": ("שָׂתַר", "H:V", "to burst", "sa.tar"),
        "H8369": ("שֵׁתָר", "N:N-M-P", "Shethar", "she.tar"),
        "H8370": ("שְׁתַר בּוֹזְנַי", "N:N-M-P", "Shethar-bozenai", "she.tar bo.ze.nay"),
        "H8371": ("שָׁתַת", "H:V", "to appoint", "sha.tat"),
        "H8372": ("תָּא", "H:N-M", "chamber", "ta"),
        "H8373": ("תָּאַב", "H:V", "to long for", "ta.av"),
        "H8374": ("תָּאַב", "H:V", "to loathe", "ta.av"),
        "H8375": ("תַּאֲבָה", "H:N-F", "longing", "ta.a.vah"),
        "H8376": ("תָּאָה", "H:V", "to mark", "ta.ah"),
        "H8377": ("תְּאוֹ", "H:N-M", "antelope", "te.o"),
        "H8378": ("תַּאֲוָה", "H:N-F", "desire", "ta.a.vah"),
        "H8379": ("תַּאֲוָה", "H:N-F", "boundary", "ta.a.vah"),
        "H8380": ("תּוֹאָם", "H:N-M", "twin", "ta.om"),
        "H8381": ("תַּאֲלָה", "H:N-F", "curse", "ta.a.lah"),
        "H8382": ("תָּאַם", "H:V", "be double", "ta.am"),
        "H8383": ("תְּאֻנִים", "H:N-M", "toil", "te.un"),
        "H8384": ("תְּאֵנָה", "H:N-F", "fig", "te.en"),
        "H8385a": ("תַּאֲנָה", "H:N-F", "estrous", "ta.a.nah"),
        "H8385b": ("תֹּאֲנָה", "H:N-F", "opportunity", ""),
        "H8386": ("תַּאֲנִיָּה", "H:N-F", "mourning", "ta.a.niy.yah"),
        "H8387": ("תַּאֲנַת שִׁלֹ֑ה", "N:N--L", "Taanath-shiloh", "ta.a.nat shi.loh"),
        "H8388a": ("תָּאַר", "H:V", "to border", "ta.ar"),
        "H8388b": ("תָּאַר", "H:V", "to delimit", "ta.ar"),
        "H8389": ("תֹּ֫אַר", "H:N-M", "appearance", "to.ar"),
        "H8390": ("תַּאֲרֵעַ", "N:N-M-P", "Tarea", "ta.a.re.a"),
        "H8391": ("תְּאַשּׁוּר", "H:N-F", "boxtree", "te.ash.shur"),
        "H8392": ("תֵּבָה", "H:N-F", "ark", "te.vah"),
        "H8393": ("תְּבוּאָה", "H:N-F", "produce", "te.vu.ah"),
        "H8394": ("תְּבוּנָה", "H:N-M", "understanding", "ta.vun"),
        "H8395": ("תְּבוּסָה", "H:N-F", "ruin", "te.vu.sah"),
        "H8396": ("תָּבוֹר", "N:N--L", "Tabor", "ta.vor"),
        "H8397": ("תֶּ֫בֶל", "H:N-M", "perversion", "te.vel"),
        "H8398": ("תֵּבֵל", "H:N-F", "world", "te.vel"),
        "H8399": ("תַּבְלִית", "H:N-F", "destruction", "tav.lit"),
        "H8400": ("תְּבַלֻּל", "H:N-M", "defect", "te.bal.lul"),
        "H8401": ("תֶּ֫בֶן", "H:N-M", "straw", "te.ven"),
        "H8402": ("תִּבְנִי", "N:N-M-P", "Tibni", "tiv.ni"),
        "H8403": ("תַּבְנִית", "H:N-F", "pattern", "tav.nit"),
        "H8404": ("תַּבְעֵרָה", "N:N--L", "Taberah", "tav.e.rah"),
        "H8405": ("תֵּבֵץ", "N:N--L", "Thebez", "te.vets"),
        "H8406": ("תְּבַר", "A:V", "to break", "te.var"),
        "H8407": ("תִּגְלַת פִּלְאֶ֫סֶר", "N:N-M-P", "Tiglath-pileser", "tig.lat pil.e.ser"),
        "H8408": ("תַּגְמוּל", "H:N-M", "benefit", "tag.mul"),
        "H8409": ("תִּגְרָה", "H:N-F", "strife", "tig.rah"),
        "H8410": ("תִּדְהָר", "H:N-M", "elm", "tid.har"),
        "H8411": ("תְּדִירָא", "A:N-F", "continuance", "te.di.ra"),
        "H8412": ("תַּדְמֹר", "N:N--L", "Tadmor", "tad.mor"),
        "H8413": ("תִּדְעָל", "N:N-M-P", "Tidal", "tid.al"),
        "H8414": ("תֹּ֫הוּ", "H:N-M", "formlessness", "to.hu"),
        "H8415": ("תְּהוֹם", "H:N", "abyss", "te.hom"),
        "H8416": ("תְּהִלָּה", "H:N-F", "praise", "te.hil.lah"),
        "H8417": ("תׇּהֳלָה", "H:N-F", "error", "to.ho.lah"),
        "H8418": ("תַּהֲלוּכָה", "H:N-F", "procession", "ta.ha.lu.khah"),
        "H8419": ("תַּהְפֻּכָה", "H:N-F", "perversity", "tah.pu.khah"),
        "H8420": ("תָּו", "H:N-M", "mark", "tav"),
        "H8421": ("תּוּב", "A:V", "to return", "tuv"),
        "H8422": ("תּוּבַל", "N:N-M-P / N:N--L", "Tubal", "tu.val"),
        "H8423": ("תּוּבַל־קַ֫יִן", "N:N-M-P", "Tubal-cain", "tu.val qa.yin"),
        "H8424": ("תּוּגָה", "H:N-F", "grief", "tu.gah"),
        "H8425": ("תֹּגַרְמָה", "N:N-M-P / N:N--L", "Togarmah", "to.gar.mah"),
        "H8426": ("תּוֹדָה", "H:N-F", "thanksgiving", "to.dah"),
        "H8427": ("תָּוָה", "H:V", "to mark", "ta.vah"),
        "H8428": ("תָּוָה", "H:V", "to wound", "ta.vah"),
        "H8429": ("תְּוַהּ", "A:V", "be startled", "te.vah"),
        "H8430": ("תּוֹחַ", "N:N-M-P", "Toah", "to.ach"),
        "H8431": ("תּוֹחֶ֫לֶת", "H:N-F", "hope", "to.che.let"),
        "H8432": ("תָּ֫וֶךְ", "H:N-M", "midst", "ta.vekh"),
        "H8433a": ("תּוֹכֵחָה", "H:N-F", "rebuke", "to.khe.chah"),
        "H8433b": ("תּוֹכַ֫חַת", "H:N-F", "argument", ""),
        "H8434": ("תּוֹלָ֑ד", "N:N--L", "Tolad", "to.lad"),
        "H8435": ("תּוֹלֵדוֹת", "H:N-F", "generation", "to.le.dah"),
        "H8436": ("תִּילוֹן", "N:N-M-P", "Tilon", "tu.lon"),
        "H8437": ("תּוֹלָל", "H:N-M", "tormentor", "to.lal"),
        "H8438a": ("תּוֹלָע", "H:N-M/F", "worm", "to.la"),
        "H8438b": ("תּוֹלֵעָה", "H:N-M/F", "worm", ""),
        "H8439": ("תּוֹלָע", "N:N-M-P", "Tola", "to.la"),
        "H8440": ("תּוֹלָעִי", "N:N--PG", "Tolaite", "to.la.i"),
        "H8441": ("תּוֹעֵבַה", "H:N-F", "abomination", "to.e.vah"),
        "H8442": ("תּוֹעָה", "H:N-F", "error", "to.ah"),
        "H8443": ("תּוֹעָפָה", "H:N-F", "peak", "to.a.phah"),
        "H8444": ("תּוֹצָאָה", "H:N-F", "outgoing", "to.tsa.ah"),
        "H8445": ("תּוֹקַהַת", "N:N-M-P", "Tokhath", "to.qa.hat"),
        "H8446": ("תּוּר", "H:V", "to spy", "tur"),
        "H8447": ("תּוֹר", "H:N-M", "plait", "tor"),
        "H8448": ("תּוֹר", "H:N-F", "border", "tor"),
        "H8449": ("תֹּר", "H:N-F", "turtledove", "tor"),
        "H8450": ("תּוֹר", "A:N-M", "bullock", "tor"),
        "H8451": ("תּוֹרָה", "H:N-F", "instruction", "to.rah"),
        "H8452": ("תּוֹרָה", "H:N-F", "custom", "to.rah"),
        "H8453a": ("תּוֹשָׁב", "H:N-M", "sojourner", "to.shav"),
        "H8453b": ("תִּשְׁבֶּה", "H:N-M", "Tishbe", ""),
        "H8454": ("תּוּשִׁיָּה", "H:N-F", "wisdom", "tu.shiy.yah"),
        "H8455": ("תּוֹתָ֑ח", "H:N-M", "club", "to.tach"),
        "H8456": ("תָּזַז", "H:V", "to strike away", "ta.zaz"),
        "H8457": ("תַּזְנוּת", "H:N-F", "fornication", "taz.nut"),
        "H8458": ("תַּחְבֻּלָה", "H:N-F", "counsel", "tach.bu.lah"),
        "H8459": ("תֹּחוּ", "N:N-M-P", "Tohu", "to.chu"),
        "H8460": ("תְּחוֹת", "A:Prep", "under", "te.chot"),
        "H8461": ("תַּחְכְּמֹנִי", "N:N--LG", "Tahchemonite", "tach.ke.mo.ni"),
        "H8462": ("תְּחִלָּה", "H:N-F", "beginning", "te.chil.lah"),
        "H8463": ("תַּחֲלֻאִים", "H:N-M", "disease", "ta.cha.lu"),
        "H8464": ("תַּחְמָס", "H:N-M", "ostrich", "tach.mas"),
        "H8465": ("תַּ֫חַן", "N:N-M-P", "Tahan", "ta.chan"),
        "H8466": ("תַּחֲנָה", "H:N-F", "encampment", "ta.cha.nah"),
        "H8467": ("תְּחִנָּה", "H:N-F", "supplication", "te.chin.nah"),
        "H8468": ("תְּחִנָּה", "N:N-M-P", "Tehinnah", "te.chin.nah"),
        "H8469": ("תַּחֲנוּן", "H:N-M", "supplication", "ta.cha.nun"),
        "H8470": ("תַּחֲנִי", "N:N--PG", "Tahanite", "ta.cha.ni"),
        "H8471": ("תַּחְפַּנְחֵס", "N:N--L", "Tahapanes", "tach.pan.ches"),
        "H8472": ("תַּחְפְּנֵיס", "N:N-F-P", "Tahpenes", "tach.pe.nes"),
        "H8473": ("תַּחְרָא", "H:N-M", "breastplate", "ta.cha.ra"),
        "H8474": ("תַּחָרָה", "H:V", "to contend", "ta.cha.rah"),
        "H8475": ("תַּחֲרֵעַ", "N:N-M-P", "Tahrea", "tach.re.a"),
        "H8476": ("תַּ֫חַשׁ", "H:N-M", "leather", "ta.chash"),
        "H8477": ("תַּ֫חַשׁ", "N:N-M-P", "Tahash", "ta.chash"),
        "H8478": ("תַּחַת", "H:N-M", "underneath", "ta.chat"),
        "H8479": ("תַּחַת", "A:Prep", "under", "ta.chat"),
        "H8480": ("תַּ֫חַת", "N:N-M-P / N:N--L", "Tahath", "ta.chat"),
        "H8481": ("תַּחְתּוֹן", "H:A", "lower", "tach.ton"),
        "H8482": ("תַּחְתִּי", "H:A", "lower", "tach.ti"),
        "H8483": ("תַּחְתִּים חׇדְשִׁי", "N:N--L", "Tahtim-hodshi", "tach.tim chod.shi"),
        "H8484": ("תִּיכוֹן", "H:A", "middle", "ti.khon"),
        "H8485": ("תֵּימָא", "N:N-M-P / N:N--L", "Tema", "te.ma"),
        "H8486": ("תֵּימָן", "H:N-F", "south", "te.man"),
        "H8487": ("תֵּימָן", "N:N-M-P / N:N-M-PG", "Teman", "te.man"),
        "H8488": ("תֵּימְנִי", "N:N-M-P", "Temeni", "te.me.ni"),
        "H8489": ("תֵּימָנִי", "N:N--PG", "Temanite", "te.ma.ni"),
        "H8490": ("תִּימָרָה", "H:N-F", "column", "ti.ma.rah"),
        "H8491": ("תִּיצִי", "N:N--LG", "Tizite", "ti.tsi"),
        "H8492": ("תִּירוֹשׁ", "H:N-M", "new wine", "ti.rosh"),
        "H8493": ("תִּירְיָא", "N:N-M-P", "Tiria", "tir.ya"),
        "H8494": ("תִּירָ֑ס", "N:N-M-P", "Tiras", "ti.ras"),
        "H8495": ("תַּ֫יִשׁ", "H:N-M", "male goat", "ta.yish"),
        "H8496": ("תֹּךְ", "H:N-M", "oppression", "tokh"),
        "H8497": ("תָּכָה", "H:V", "to follow", "ta.khah"),
        "H8498": ("תְּכוּנָה", "H:N-F", "fitting", "te.khu.nah"),
        "H8499": ("תְּכוּנָה", "H:N-F", "place", "te.khu.nah"),
        "H8500": ("תֻּכִּיִּים", "H:N-M", "peacock", "tuk.ki"),
        "H8501": ("תָּכָךְ", "H:N-M", "deceitful", "ta.khakh"),
        "H8502": ("תִּכְלָה", "H:N-F", "perfection", "tikh.lah"),
        "H8503": ("תַּכְלִית", "H:N-F", "limit", "takh.lit"),
        "H8504": ("תְּכֵ֫לֶת", "H:N-F", "blue", "te.khe.let"),
        "H8505": ("תָּכַן", "H:V", "to measure", "ta.khan"),
        "H8506": ("תֹּ֫כֶן", "H:N-M", "quantity", "to.khen"),
        "H8507": ("תֹּ֫כֶן", "N:N--L", "Tochen", "to.khen"),
        "H8508": ("תׇּכְנִית", "H:N-F", "proportion", "tokh.nit"),
        "H8509": ("תַּכְרִיךְ", "H:N-M", "robe", "takh.rikh"),
        "H8510": ("תֵּל", "H:N-M", "mound", "tel"),
        "H8511": ("תָּלָא", "H:V", "to hang", "ta.la"),
        "H8512": ("תֵּל אָבִיב", "N:N--L", "Tel-abib", "tel a.viv"),
        "H8513": ("תְּלָאָה", "H:N-F", "hardship", "te.la.ah"),
        "H8514": ("תַּלְאֻבָה", "H:N-F", "drought", "tal.u.vah"),
        "H8515": ("תְּלַאשָּׂ֑ר", "N:N--L", "Telassar", "te.las.sar"),
        "H8516": ("תִּלְבֹּשֶׁת", "H:N-F", "clothing", "tal.bo.shet"),
        "H8517": ("תְּלַג", "A:N-M", "snow", "te.lag"),
        "H8518": ("תָּלָה", "H:V", "to hang", "ta.lah"),
        "H8519": ("תְּלֻנָּה", "H:N-F", "murmuring", "te.lu.nah"),
        "H8520": ("תֶּ֫לַח", "N:N-M-P", "Telah", "te.lach"),
        "H8521": ("תֵּל חַרְשָׁא", "N:N--L", "Tel-harsha", "tel char.sah"),
        "H8522": ("תְּלִי", "H:N-M", "quiver", "te.li"),
        "H8523a": ("תַּלְתִּי", "A:A", "third", "te.li.tay"),
        "H8523b": ("תְּלִיתַי", "A:A", "third", "te.li.tay"),
        "H8524": ("תָּלוּל", "H:A", "eminent", "ta.lal"),
        "H8525": ("תֶּ֫לֶם", "H:N-M", "furrow", "te.lem"),
        "H8526": ("תַּלְמַי", "N:N-M-P", "Talmai", "tal.may"),
        "H8527": ("תַּלְמִיד", "H:N-M", "pupil", "tal.mid"),
        "H8528": ("תֵּל מֶ֫לַח", "N:N--L", "Tel-melah", "tel me.lach"),
        "H8529": ("תָּלַע", "H:V", "to clothe in scarlet", "ta.la"),
        "H8530": ("תַּלְפִּיָּוֹת", "H:N-F", "weapon", "tal.piy.yah"),
        "H8531": ("תְּלַת", "A:N-M", "third", "te.lat"),
        "H8532": ("תְּלָת", "A:N", "three", "te.lat"),
        "H8533": ("תְּלָתִין", "A:N", "thirty", "te.la.tin"),
        "H8534": ("תַּלְתַּלִּים", "H:N-F", "lock", "tal.tal"),
        "H8535": ("תָּם", "H:A", "complete", "tam"),
        "H8536": ("תַּמָּה", "A:Adv", "there", "tam"),
        "H8537": ("תֹּם", "H:N-M", "integrity", "tom"),
        "H8538": ("תֻּמָּה", "H:N-F", "integrity", "tum.mah"),
        "H8539": ("תָּמַהּ", "H:V", "to astounded", "ta.mah"),
        "H8540": ("תְּמַהּ", "A:N-M", "wonder", "te.mah"),
        "H8541": ("תִּמָּהוֹן", "H:N-M", "bewilderment", "tim.ma.hon"),
        "H8542": ("תַּמּוּז", "N:N-M-T", "Tammuz", "tam.muz"),
        "H8543": ("תְּמוֹל", "H:Adv", "yesterday", "te.mol"),
        "H8544": ("תְּמוּנָה", "H:N-F", "likeness", "te.mu.nah"),
        "H8545": ("תְּמוּרָה", "H:N-F", "exchange", "te.mu.rah"),
        "H8546": ("תְּמוּתָה", "H:N-F", "death", "te.mu.tah"),
        "H8547": ("תָּ֑מַח", "N:N-M-P", "Temah", "te.mach"),
        "H8548": ("תָּמִיד", "H:N-M", "continually", "ta.mid"),
        "H8549": ("תָּמִים", "H:A", "unblemished", "ta.mim"),
        "H8550": ("תֻּמִּים", "H:N-M", "Thummim", "tum.mim"),
        "H8551": ("תָּמַךְ", "H:V", "to grasp", "ta.makh"),
        "H8552": ("תָּמַם", "H:V", "to finish", "ta.mam"),
        "H8553": ("תִּמְנָה", "N:N--L", "Timnah", "tim.nah"),
        "H8554": ("תִּמְנִי", "N:N--LG", "Timnite", "tim.ni"),
        "H8555": ("תִּמְנָע", "N:N-M-P / N:N-F-P", "Timna", "tim.na"),
        "H8556a": ("תִּמְנַת חֶרֶס", "N:N--L", "Timnath-heres", "tim.nat che.res"),
        "H8556b": ("תִּמְנַת סֶ֫רַח", "N:N--L", "Timnath-serah", ""),
        "H8557": ("תֶּ֫מֶס", "H:N-M", "melting", "te.mes"),
        "H8558": ("תָּמָר", "H:N-M", "palm", "ta.mar"),
        "H8559": ("תָּמָר", "N:N-F-P / N:N--L", "Tamar", "ta.mar"),
        "H8560": ("תֹּ֫מֶר", "H:N-M", "palm", "to.mer"),
        "H8561": ("תִּמֹרָה", "H:N-F", "palm", "tim.mor"),
        "H8562": ("תַּמְרוּק", "H:N-M", "cosmetic", "tam.ruq"),
        "H8563": ("תַּמְרוּר", "H:N-M", "bitterness", "tam.rur"),
        "H8564": ("תַּמְרוּר", "H:N-M", "signpost", "tam.rur"),
        "H8565": ("תַּן", "H:N-M", "dragon", "tan"),
        "H8566": ("תָּנָה", "H:V", "to hire", "ta.nah"),
        "H8567": ("תָּנָה", "H:V", "to recount", "ta.nah"),
        "H8568": ("תַּנָּה", "H:N-M", "dragon", "tan.nah"),
        "H8569": ("תְּנוּאָה", "H:N-F", "opposition", "te.nu.ah"),
        "H8570": ("תְּנוּבָה", "H:N-F", "fruit", "te.nu.vah"),
        "H8571": ("תְּנוּךְ", "H:N-M", "lobe", "te.nukh"),
        "H8572": ("תְּנוּמָה", "H:N-F", "slumber", "te.nu.mah"),
        "H8573": ("תְּנוּפָה", "H:N-F", "wave offering", "te.nu.phah"),
        "H8574": ("תַּנּוּר", "H:N-M", "oven", "tan.nur"),
        "H8575": ("תַּנְחוּם", "H:N", "consolation", "tan.chum"),
        "H8576": ("תַּנְחֻ֫מֶת", "N:N-M-P", "Tanhumeth", "tan.chu.met"),
        "H8577a": ("תַּן", "H:N-M", "jackal", "tan.nin"),
        "H8577b": ("תַּנִּין", "H:N-M", "serpent", "tan.nin"),
        "H8578": ("תִּנְיָן", "A:A", "second", "tin.yan"),
        "H8579": ("תִּנְיָנוּת", "A:Adv", "second time", "tin.ya.nut"),
        "H8580": ("תִּנְשֶׁ֫מֶת", "H:N-F", "chameleon", "tan.she.met"),
        "H8581": ("תַּעָב", "H:V", "to abhor", "ta.av"),
        "H8582": ("תָּעָה", "H:V", "to go astray", "ta.ah"),
        "H8583": ("תֹּ֫עִי, תֹּעוּ", "N:N-M-P", "Toi", "to.u"),
        "H8584": ("תְּעוּדָה", "H:N-F", "testimony", "te.u.dah"),
        "H8585a": ("תְּעָלָה", "H:N-F", "conduit", "te.a.lah"),
        "H8585b": ("תְּעָלָה", "H:N-F", "covering", "te.a.lah"),
        "H8586": ("תַּעֲלוּלִים", "H:N-M", "caprice", "ta.a.lul"),
        "H8587": ("תַּעֲלֻמָּהֿ", "H:N-F", "secret", "ta.a.lum.mah"),
        "H8588": ("תַּעֲנוּג", "H:N-M", "luxury", "ta.a.nug"),
        "H8589": ("תַּעֲנִית", "H:N-F", "fasting", "ta.a.nit"),
        "H8590": ("תַּעֲנָךְ", "N:N--L", "Taanach", "ta.a.nakh"),
        "H8591": ("תָּעַע", "H:V", "to mock", "ta.a"),
        "H8592": ("תַּעֲצֻמָה", "H:N-F", "power", "ta.a.tsu.mah"),
        "H8593": ("תַּ֫עַר", "H:N", "razor", "ta.ar"),
        "H8594": ("תַּעֲרֻבָה", "H:N-F", "security", "ta.a.ru.vah"),
        "H8595": ("תֵּעְתֻּעִים", "H:N-M", "delusion", "ta.tu.a"),
        "H8596": ("תֹּף", "H:N-M", "tambourine", "toph"),
        "H8597": ("תִּפְאָרָה", "H:N-F", "beauty", "tiph.a.rah"),
        "H8598": ("תַּפּוּחַ", "H:N-M", "apple", "tap.pu.ach"),
        "H8599a": ("תַּפֻּחַ", "N:N-M-P", "Tappuah", "tap.pu.ach"),
        "H8599b": ("תַּפּוּחַ", "N:N--L", "Tappuah", "tap.pu.ach"),
        "H8600": ("תְּפוֹצָה", "H:N-F", "dispersion", "te.pho.tsah"),
        "H8601": ("תֻּפִינִים", "H:N-M", "baked", "tu.phin"),
        "H8602a": ("תָּפֵל", "H:A", "insipid", "ta.phel"),
        "H8602b": ("תָּפֵל", "H:N-M", "whitewash", "ta.phel"),
        "H8603": ("תֹּ֫פֶל", "N:N--L", "Tophel", "to.phel"),
        "H8604": ("תִּפְלָה", "H:N-F", "folly", "tiph.lah"),
        "H8605": ("תְּפִלָּה", "H:N-F", "prayer", "te.phil.lah"),
        "H8606": ("תִּפְלֶ֫צֶת", "H:N-F", "terror", "tiph.le.tset"),
        "H8607": ("תִּפְסַח", "N:N--L", "Tiphsah", "tiph.sach"),
        "H8608": ("תָּפַף", "H:V", "to beat", "ta.phaph"),
        "H8609": ("תָּפַר", "H:V", "to sew", "ta.phar"),
        "H8610": ("תָּפַשׂ", "H:V", "to capture", "ta.pha.s"),
        "H8611": ("תֹּ֫פֶת", "H:N-F", "spit", "to.phet"),
        "H8612": ("תֹּ֫פֶת", "N:N--L", "Topheth", "to.phet"),
        "H8613": ("תׇּפְתֶּה", "N:N--L", "Topheth", "toph.teh"),
        "H8614": ("תִּפְתָּיֵא", "A:N-M", "magistrate", "tiph.tay"),
        "H8615a": ("תִּקְוָה", "H:N-F", "cord", "tiq.vah"),
        "H8615b": ("תִּקְוָה", "H:N-F", "hope", "tiq.vah"),
        "H8616": ("תִּקְוָה", "N:N-M-P", "Tikvah", "tiq.vah"),
        "H8617": ("תְּקוּמָה", "H:N-F", "standing", "te.qu.mah"),
        "H8618": ("תְּקוֹמֵם", "H:V", "to confront", "te.qo.mem"),
        "H8619": ("תָּקוֹעַ", "H:N-M", "trumpet", "ta.qo.a"),
        "H8620": ("תְּקוֹעַ", "N:N--L", "Tekoa", "te.qo.a"),
        "H8621": ("תְּקוֹעִי", "N:N--LG", "Tekoite", "te.qo.i"),
        "H8622": ("תְּקוּפָה", "H:N-F", "circuit", "te.qu.phah"),
        "H8623": ("תַּקִּיף", "H:A", "mighty", "taq.qiph"),
        "H8624": ("תַּקִּיף", "A:A", "strong", "taq.qiph"),
        "H8625a": ("תְּקַל", "A:V", "to weigh", "te.qal"),
        "H8625b": ("תְּקֵל", "A:N-M", "shekel", ""),
        "H8626": ("תָּקַן", "H:V", "be straight", "ta.qan"),
        "H8627": ("תְּקַן", "A:V", "to confirm", "te.qan"),
        "H8628": ("תָּקַע", "H:V", "to blow", "ta.qa"),
        "H8629": ("תֵּ֫קַע", "H:N-M", "blast", "te.qa"),
        "H8630": ("תָּקֵף", "H:V", "to prevail", "ta.qaph"),
        "H8631": ("תְּקֵף", "A:V", "to grow strong", "te.qeph"),
        "H8632a": ("תְּקָף", "A:N-M", "might", "te.qoph"),
        "H8632b": ("תְּקֹף", "A:N-M", "might", "te.qoph"),
        "H8633": ("תֹּ֫קֶף", "H:N-M", "power", "to.qeph"),
        "H8634": ("תַּרְאֲלָה", "N:N--L", "Taralah", "tar.a.lah"),
        "H8635": ("תַּרְבּוּת", "H:N-F", "increase", "tar.but"),
        "H8636": ("תַּרְבִּית", "H:N-F", "increment", "tar.bit"),
        "H8637": ("תִּרְגַּל", "H:V", "to teach", "tir.gal"),
        "H8638": ("תִּרְגֵּם", "H:V", "to translate", "tir.gam"),
        "H8639": ("תַּרְדֵּמָה", "H:N-F", "deep sleep", "tar.de.mah"),
        "H8640": ("תִּרְהָקָה", "N:N-M-P", "Tirhakah", "tir.ha.qah"),
        "H8641": ("תְּרוּמָה", "H:N-F", "contribution", "te.ru.mah"),
        "H8642": ("תְּרוּמִיָּה", "H:N-F", "contribution", "te.ru.miy.yah"),
        "H8643": ("תְּרוּעָה", "H:N-F", "shout", "te.ru.ah"),
        "H8644": ("תְּרוּפָה", "H:N-F", "healing", "te.ru.phah"),
        "H8645": ("תִּרְזָה", "H:N-F", "cypress", "tir.zah"),
        "H8646": ("תֶּ֫רַח", "N:N-M-P / N:N--L", "Terah", "te.rach"),
        "H8647": ("תִּרְחֲנָה", "N:N-M-P", "Tirhanah", "tir.cha.nah"),
        "H8648": ("תְּרֵין", "A:N", "two", "te.ren"),
        "H8649a": ("תׇּרְמָה", "H:N-F", "treachery", "tor.mah"),
        "H8649b": ("תַּרְמִית", "H:N-F", "deceitfulness", ""),
        "H8650": ("תֹּ֫רֶן", "H:N-M", "mast", "to.ren"),
        "H8651": ("תְּרַע", "A:N-M", "door", "te.ra"),
        "H8652": ("תָּרָע", "A:N-M", "doorkeeper", "ta.ra"),
        "H8653": ("תַּרְעֵלָה", "H:N-F", "reeling", "tar.e.lah"),
        "H8654": ("תִּרְעָתִים", "N:N--LG", "Tirathite", "tir.a.ti"),
        "H8655": ("תְּרָפִים", "H:N-M", "Teraphim", "te.ra.phim"),
        "H8656": ("תִּרְצָה", "N:N-F-P / N:N--L", "Tirzah", "tir.tsah"),
        "H8657": ("תֶּ֫רֶשׁ", "N:N-M-P", "Teresh", "te.resh"),
        "H8658": ("תַּרְשִׁישׁ", "H:N-M", "jasper", "tar.shish"),
        "H8659": ("תַּרְשִׁישׁ", "N:N-M-P / N:N--L", "Tarshish", "tar.shish"),
        "H8660": ("תִּרְשָׁתָא", "N:N-M-T", "governor", "tir.sha.ta"),
        "H8661": ("תַּרְתָּן", "N:N-M-T", "commander", "tar.tan"),
        "H8662": ("תַּרְתָּ֑ק", "N:N-M-T", "Tartak", "tar.taq"),
        "H8663": ("תְּשֻׁאָה", "H:N-F", "shout", "te.shu.ah"),
        "H8664": ("תִּשְׁבִּי", "N:N--LG", "Tishbite", "tish.bi"),
        "H8665": ("תַּשְׁבֵּץ", "H:N-M", "checkered", "tash.bets"),
        "H8666": ("תְּשׁוּבָה", "H:N-F", "turn", "te.shu.vah"),
        "H8667": ("תְּשׂ֫וּמֶת", "H:N-F", "security", "te.su.met"),
        "H8668": ("תְּשׁוּעָה", "H:N-F", "deliverance", "te.shu.ah"),
        "H8669": ("תְּשׁוּקָה", "H:N-F", "desire", "te.shu.qah"),
        "H8670": ("תְּשׁוּרָה", "H:N-F", "present", "te.shu.rah"),
        "H8671": ("תְּשִׁיעִי", "H:A", "ninth", "te.shi.i"),
        "H8672": ("תֵּ֫שַׁע", "H:N", "nine", "te.sah"),
        "H8673": ("תִּשְׁעִים", "H:N", "ninety", "tish.im"),
        "H8674": ("תַּתְּנַי", "N:N-M-P", "Tattenai", "ta.te.nay"),
        "H9000": ("ו", "H:Conj", "and", "v"),
        "H9001": ("וַ", "H:Conj", "and", "v"),
        "H9002": ("וְ", "H:Conj", "and", "v"),
        "H9003": ("ב", "", "in/on/with", "b"),
        "H9004": ("כ", "", "like/as", "k"),
        "H9005": ("ל", "", "to/for", "l"),
        "H9006": ("מ", "", "from", "m"),
        "H9007": ("שׁ", "", "which/that", "s"),
        "H9008": ("ה", "", "[?]", "i"),
        "H9009": ("ה", "", "[the]", "hé"),
        "H9010": ("א", "", "[the]", "a"),
        "H9011": ("ה", "", "[to]", "hé"),
        "H9012": ("ה", "", "[emph.?]", "p"),
        "H9013": ("ן", "", "[emph.?]", "n"),
        "H9014": ("-", "", "[-]", "-"),
        "H9015": ("|", "", "[ ]", "¦"),
        "H9016": (":", "", "[.]", ":"),
        "H9017": ("פ", "", "[¶]", "P"),
        "H9018": ("ס", "", "[¶]", "S"),
        "H9019": ("׆", "", "[¶]", "N"),
        "H9020": ("י", "Ps1c", "my", ""),
        "H9021": ("ךָ", "Ps2m", "your", ""),
        "H9022": ("ךְ", "Ps2f", "your", ""),
        "H9023": ("הוּ", "Ps3m", "his", ""),
        "H9024": ("הּ", "Ps3f", "her", ""),
        "H9025": ("נוּ", "Pp1c", "our", ""),
        "H9026": ("כֶם", "Pp2m", "your", ""),
        "H9027": ("כֶן", "Pp2f", "your", ""),
        "H9028": ("הֶם", "Pp3m", "their", ""),
        "H9029": ("הֶן", "Pp3f", "their", ""),
        "H9030": ("נִי", "Os1c", "me", ""),
        "H9031": ("ךָּ", "Os2m", "you", ""),
        "H9032": ("ךְ", "Os2f", "you", ""),
        "H9033": ("וֹ", "Os3m", "him", ""),
        "H9034": ("הָ", "Os3f", "her", ""),
        "H9035": ("נוּ", "Op1c", "us", ""),
        "H9036": ("כֶם", "Op2m", "you", ""),
        "H9037": ("כֶן", "Op2f", "you", ""),
        "H9038": ("הֶם", "Op3m", "them", ""),
        "H9039": ("הֶן", "Op3f", "them", ""),
        "H9040": ("נִי", "Ss1c", "I", ""),
        "H9041": ("ךָ", "Ss2m", "you", ""),
        "H9042": ("ךְ", "Ss2f", "you", ""),
        "H9043": ("נּוּ", "Ss3m", "he", ""),
        "H9044": ("נָּה", "Ss3f", "she", ""),
        "H9045": ("נוּ", "Sp1c", "we", ""),
        "H9046": ("כוֹן", "Sp2m", "you", ""),
        "H9047": ("", "Sp2f", "you", ""),
        "H9048": ("ם", "Sp3m", "they", ""),
        "H9049": ("ינה", "Sp3f", "they", ""),
        "G1": ("Ἀλφα", "G:N", "alpha", "Alpha"),
        "G2": ("Ἀαρών", "N:N-M-P", "Aaron", "Aarōn"),
        "G3": ("Ἀβαδδών", "N:N--T", "Abaddon", "Abaddōn"),
        "G4": ("ἀβαρής", "G:A", "not burdensome", "abarēs"),
        "G5": ("ἀββά", "G:N-M", "Father", "abba"),
        "G6": ("Ἄβελ", "N:N-M-P", "Abel", "Abel"),
        "G7": ("Ἀβιά", "N:N-M-P", "Abijah", "Abia"),
        "G8": ("Ἀβιαθάρ", "N:N-M-P", "Abiathar", "Abiathar"),
        "G9": ("Ἀβιληνή", "N:N-F-L", "Abilene", "Abilēnē"),
        "G10": ("Ἀβιούδ", "N:N-M-P", "Abiud", "Abioud"),
        "G11": ("Ἀβραάμ", "N:N-M-P", "Abraham", "Abraam"),
        "G12": ("ἄβυσσος", "G:N-F", "abyss", "abussos"),
        "G13": ("Ἄγαβος", "N:N-M-P", "Agabus", "Agabos"),
        "G14": ("ἀγαθοεργέω", "G:V", "to do good", "agathoergeō"),
        "G15": ("ἀγαθοποιέω", "G:V", "to do good", "agathopoieō"),
        "G16": ("ἀγαθοποιΐα", "G:N-F", "doing good", "agathopoiia"),
        "G17": ("ἀγαθοποιός", "G:N-M", "doing good", "agathopoios"),
        "G18": ("ἀγαθός", "G:A", "good-doer", "agathos"),
        "G19": ("ἀγαθωσύνη", "G:N-F", "goodness", "agathōsunē"),
        "G20": ("ἀγαλλίασις", "G:N-F", "joy", "agalliasis"),
        "G21": ("ἀγαλλιάω", "G:V", "to rejoice", "agalliaō"),
        "G22": ("ἄγαμος", "G:N-M/F", "unmarried", "agamos"),
        "G23": ("ἀγανακτέω", "G:V", "be indignant", "aganakteō"),
        "G24": ("ἀγανάκτησις", "G:N-F", "indignation", "aganaktēsis"),
        "G25": ("ἀγαπάω", "G:V", "to love", "agapaō"),
        "G26": ("ἀγάπη", "G:N-F", "love", "agapē"),
        "G27": ("ἀγαπητός", "G:A", "beloved", "agapētos"),
        "G28": ("Ἄγαρ", "N:N-F-P", "Hagar", "Agar"),
        "G29": ("ἀγγαρεύω", "G:V", "to force", "aggareuō"),
        "G30": ("ἀγγεῖον", "G:N-N", "jar", "aggeion"),
        "G31": ("ἀγγελία", "G:N-F", "message", "aggelia"),
        "G32": ("ἄγγελος", "G:N-M", "angel", "aggelos"),
        "G33": ("ἄγε", "G:V", "to lead", "age"),
        "G34": ("ἀγέλη", "G:N-F", "herd", "agelē"),
        "G35": ("ἀγενεαλόγητος", "G:A", "without genealogy", "agenealogētos"),
        "G36": ("ἀγενής", "G:A", "lowly", "agenēs"),
        "G37": ("ἁγιάζω", "G:V", "to sanctify", "hagiazō"),
        "G38": ("ἁγιασμός", "G:N-M", "holiness", "hagiasmos"),
        "G39": ("ἅγιον", "G:A", "holy", "hagion"),
        "G40": ("ἅγιος", "G:A", "holy", "hagios"),
        "G41": ("ἁγιότης", "G:N-F", "holiness", "hagiotēs"),
        "G42": ("ἁγιωσύνη", "G:N-F", "holiness", "hagiōsunē"),
        "G43": ("ἀγκάλη", "G:N-F", "arm", "agkalē"),
        "G44": ("ἄγκιστρον", "G:N-N", "fishhook", "agkistron"),
        "G45": ("ἄγκυρα", "G:N-F", "anchor", "agkura"),
        "G46": ("ἄγναφος", "G:A", "unshrunk", "agnaphos"),
        "G47": ("ἁγνεία", "G:N-F", "purity", "hagneia"),
        "G48": ("ἁγνίζω", "G:V", "to purify", "hagnizō"),
        "G49": ("ἁγνισμός", "G:N-M", "purification", "hagnismos"),
        "G50": ("ἀγνοέω", "G:V", "be ignorant", "agnoeō"),
        "G51": ("ἀγνόημα", "G:N-N", "error", "agnoēma"),
        "G52": ("ἄγνοια", "G:N-F", "ignorance", "agnoia"),
        "G53": ("ἁγνός", "G:A", "pure", "hagnos"),
        "G54": ("ἁγνότης", "G:N-F", "purity", "hagnotēs"),
        "G55": ("ἁγνῶς", "G:Adv", "purely", "hagnōs"),
        "G56": ("ἀγνωσία", "G:N-F", "ignorance", "agnōsia"),
        "G57": ("ἄγνωστος", "G:A", "unknown", "agnōstos"),
        "G58": ("ἀγορά", "G:N-F", "marketplace", "agora"),
        "G59": ("ἀγοράζω", "G:V", "to buy", "agorazō"),
        "G60": ("ἀγοραῖος", "G:A", "of the marketplace", "agoraios"),
        "G61": ("ἄγρα", "G:N-F", "catch", "agra"),
        "G62": ("ἀγράμματος", "G:A", "unschooled", "agrammatos"),
        "G63": ("ἀγραυλέω", "G:V", "to live outside", "agrauleō"),
        "G64": ("ἀγρεύω", "G:V", "to catch", "agreuō"),
        "G65": ("ἀγριέλαιος", "G:N-F", "wild olive tree", "agrielaios"),
        "G66": ("ἄγριος", "G:A", "wild", "agrios"),
        "G67": ("Ἀγρίππας", "N:N-M-P", "Agrippa", "Agrippas"),
        "G68": ("ἀγρός", "G:N-M", "field", "agros"),
        "G69": ("ἀγρυπνέω", "G:V", "be watchful", "agrupneō"),
        "G70": ("ἀγρυπνία", "G:N-F", "sleeplessness", "agrupnia"),
        "G71": ("ἄγω", "G:V", "to bring", "agō"),
        "G72": ("ἀγωγή", "G:N-F", "self-conduct", "agōgē"),
        "G73": ("ἀγών", "G:N-M", "fight", "agōn"),
        "G74": ("ἀγωνία", "G:N-F", "a struggle", "agōnia"),
        "G75": ("ἀγωνίζομαι", "G:V", "to struggle", "agōnizomai"),
        "G76": ("Ἀδάμ", "N:N-M-P", "Adam", "Adam"),
        "G77": ("ἀδάπανος", "G:A", "free", "adapanos"),
        "G78": ("Ἀδδί", "N:N-M-P", "Addi", "Addi"),
        "G79": ("ἀδελφή", "G:N-F", "sister", "adelphē"),
        "G80": ("ἀδελφός", "G:N-M", "brother", "adelphos"),
        "G81": ("ἀδελφότης", "G:N-F", "brotherhood", "adelphotēs"),
        "G82": ("ἄδηλος", "G:A", "unclear", "adēlos"),
        "G83": ("ἀδηλότης", "G:N-F", "uncertainty", "adēlotēs"),
        "G84": ("ἀδήλως", "G:Adv", "uncertainly", "adēlōs"),
        "G85": ("ἀδημονέω", "G:V", "be distressed", "adēmoneō"),
        "G86": ("ᾍδης", "G:N-M", "Hades", "Hadēs"),
        "G87": ("ἀδιάκριτος", "G:A", "impartial", "adiakritos"),
        "G88": ("ἀδιάλειπτος", "G:A", "continuous", "adialeiptos"),
        "G89": ("ἀδιαλείπτως", "G:Adv", "unceasingly", "adialeiptōs"),
        "G90": ("ἀδιαφθορία", "G:N-F", "unspoilable", "adiaphthoria"),
        "G91": ("ἀδικέω", "G:V", "to harm", "adikeō"),
        "G92": ("ἀδίκημα", "G:N-N", "crime", "adikēma"),
        "G93": ("ἀδικία", "G:N-F", "unrighteousness", "adikia"),
        "G94": ("ἄδικος", "G:A", "unjust", "adikos"),
        "G95": ("ἀδίκως", "G:Adv", "unjustly", "adikōs"),
        "G96": ("ἀδόκιμος", "G:A", "failing", "adokimos"),
        "G97": ("ἄδολος", "G:A", "pure", "adolos"),
        "G98": ("Ἀδραμυττηνός", "N:N--LG", "Adramyttene", "Adramuttēnos"),
        "G99": ("Ἀδρίας", "N:N-M-L", "Adriatic Sea", "Adrias"),
        "G100": ("ἁδρότης", "G:N-F", "abundance", "hadrotēs"),
        "G101": ("ἀδυνατέω", "G:V", "be impossible", "adunateō"),
        "G102": ("ἀδύνατος", "G:A", "unable", "adunatos"),
        "G103": ("ᾄδω", "G:V", "to sing", "adō"),
        "G104": ("ἀεί", "G:Adv", "always", "aei"),
        "G105": ("ἀετός", "G:N-M", "eagle", "aetos"),
        "G106": ("ἄζυμος", "G:A", "unleavened", "azumos"),
        "G107": ("Ἀζώρ", "N:N-M-P", "Azor", "Azōr"),
        "G108": ("Ἄζωτος", "N:N-F-L", "Azotus", "Azōtos"),
        "G109": ("ἀήρ", "G:N-M", "air", "aēr"),
        "G110": ("ἀθανασία", "G:N-F", "immortality", "athanasia"),
        "G111": ("ἀθέμιτος", "G:A", "unlawful", "athemitos"),
        "G112": ("ἄθεος", "G:A", "without God", "atheos"),
        "G113": ("ἄθεσμος", "G:A", "lawless", "athesmos"),
        "G114": ("ἀθετέω", "G:V", "to reject", "atheteō"),
        "G115": ("ἀθέτησις", "G:N-F", "nullification", "athetēsis"),
        "G116": ("Ἀθῆναι", "N:N-F-L", "Athens", "Athēnai"),
        "G117": ("Ἀθηναῖος", "G:A", "Athenian", "Athēnaios"),
        "G118": ("ἀθλέω", "G:V", "to compete", "athleō"),
        "G119": ("ἄθλησις", "G:N-F", "struggle", "athlēsis"),
        "G120": ("ἀθυμέω", "G:V", "be discouraged", "athumeō"),
        "G121": ("ἀθῷος", "G:A", "innocent", "athōos"),
        "G122": ("αἴγειος", "G:A", "goat's", "aigeios"),
        "G123": ("αἰγιαλός", "G:N-M", "shore", "aigialos"),
        "G124": ("Αἰγύπτιος", "G:A", "Egyptian", "Aiguptios"),
        "G125": ("Αἴγυπτος", "N:N-F-L", "Egypt", "Aiguptos"),
        "G126": ("ἀΐδιος", "G:A", "eternal", "aidios"),
        "G127": ("αἰδώς", "G:N-F", "modesty", "aidōs"),
        "G128": ("Αἰθίοψ", "N:N--LG", "Ethiopian", "Aithiops"),
        "G129": ("αἷμα", "G:N-N", "blood", "ahima"),
        "G130": ("αἱματεκχυσία", "G:N-F", "bloodshed", "ahimatekchusia"),
        "G131": ("αἱμορροέω", "G:V", "to bleed", "ahimorroeō"),
        "G132": ("Αἰνέας", "N:N-M-P", "Aeneas", "Aineas"),
        "G133": ("αἴνεσις", "G:N-F", "praise", "ainesis"),
        "G134": ("αἰνέω", "G:V", "to praise", "aineō"),
        "G135": ("αἴνιγμα", "G:N-N", "obscure thing", "ainigma"),
        "G136": ("αἶνος", "G:N-M", "praise", "ainos"),
        "G137": ("Αἰνών", "N:N--L", "Aenon", "Ainōn"),
        "G138": ("αἱρέω", "G:V", "to choose", "ahireō"),
        "G139": ("αἵρεσις", "G:N-F", "sect", "ahiresis"),
        "G140": ("αἱρετίζω", "G:V", "to choose", "ahiretizō"),
        "G141": ("αἱρετικός", "G:A", "schismatic", "ahiretikos"),
        "G142": ("αἴρω", "G:V", "to take up", "airō"),
        "G143": ("αἰσθάνομαι", "G:V", "to perceive", "aisthanomai"),
        "G144": ("αἴσθησις", "G:N-F", "insight", "aisthēsis"),
        "G145": ("αἰσθητήριον", "G:N-N", "sense", "aisthētērion"),
        "G146": ("αἰσχροκερδής", "G:A", "greedy", "aischrokerdēs"),
        "G147": ("αἰσχροκερδῶς", "G:Adv", "greedily", "aischrokerdōs"),
        "G148": ("αἰσχρολογία", "G:N-F", "obscenity", "aischrologia"),
        "G149": ("αἰσχρόν", "G:A", "shameful", "aischron"),
        "G150": ("αἰσχρός", "G:A", "shameful", "aischros"),
        "G151": ("αἰσχρότης", "G:N-F", "obscenity", "aischrotēs"),
        "G152": ("αἰσχύνη", "G:N-F", "shame", "aischunē"),
        "G153": ("αἰσχύνω", "G:V", "be ashamed", "aischunō"),
        "G154": ("αἰτέω", "G:V", "to ask", "aiteō"),
        "G155": ("αἴτημα", "G:N-N", "request", "aitēma"),
        "G156": ("αἰτία", "G:N-F", "cause/charge", "aitia"),
        "G157": ("αἰτίαμα", "G:N-N", "charge", "aitiama"),
        "G158": ("αἴτιον", "G:N-N", "reason for charge", "aition"),
        "G159": ("αἴτιος", "G:A", "causer", "aitios"),
        "G160": ("αἰφνίδιος", "G:A", "sudden", "aiphnidios"),
        "G161": ("αἰχμαλωσία", "G:N-F", "captivity", "aichmalōsia"),
        "G162": ("αἰχμαλωτεύω", "G:V", "to take captive", "aichmalōteuō"),
        "G163": ("αἰχμαλωτίζω", "G:V", "to capture", "aichmalōtizō"),
        "G164": ("αἰχμάλωτος", "G:N-M", "captive", "aichmalōtos"),
        "G165": ("αἰών", "G:N-M", "an age", "aiōn"),
        "G166": ("αἰώνιος", "G:A", "eternal", "aiōnios"),
        "G167": ("ἀκαθαρσία", "G:N-F", "impurity", "akatharsia"),
        "G168": ("ἀκαθάρτης", "G:N-F", "uncleanness", "akathartēs"),
        "G169": ("ἀκάθαρτος", "G:A", "unclean", "akathartos"),
        "G170": ("ἀκαιρέω", "G:V", "to lack opportunity", "akaireō"),
        "G171": ("ἀκαίρως", "G:Adv", "unseasonably", "akairōs"),
        "G172": ("ἄκακος", "G:A", "innocent", "akakos"),
        "G173": ("ἄκανθα", "G:N-F", "a thorn", "akantha"),
        "G174": ("ἀκάνθινος", "G:A", "thorny", "akanthinos"),
        "G175": ("ἄκαρπος", "G:A", "unfruitful", "akarpos"),
        "G176": ("ἀκατάγνωστος", "G:A", "beyond reproach", "akatagnōstos"),
        "G177": ("ἀκατακάλυπτος", "G:A", "uncovered", "akatakaluptos"),
        "G178": ("ἀκατάκριτος", "G:A", "uncondemned", "akatakritos"),
        "G179": ("ἀκατάλυτος", "G:A", "indestructible", "akatalutos"),
        "G180": ("ἀκατάπαυστος", "G:A", "unceasing", "akatapaustos"),
        "G181": ("ἀκαταστασία", "G:N-F", "disorder", "akatastasia"),
        "G182": ("ἀκατάστατος", "G:A", "restless", "akatastatos"),
        "G183": ("ἀκατάσχετος", "G:A", "uncontrollable", "akataschetos"),
        "G184": ("Ἀκελδαμά", "N:N--L", "Akeldama", "Akeldama"),
        "G185": ("ἀκέραιος", "G:A", "innocent", "akeraios"),
        "G186": ("ἀκλινής", "G:A", "unwavering", "aklinēs"),
        "G187": ("ἀκμάζω", "G:V", "to ripen", "akmazō"),
        "G188": ("ἀκμήν", "G:Adv", "still", "akmēn"),
        "G189": ("ἀκοή", "G:N-F", "hearing", "akoē"),
        "G190": ("ἀκολουθέω", "G:V", "to follow", "akoloutheō"),
        "G191": ("ἀκούω", "G:V", "to hear", "akouō"),
        "G192": ("ἀκρασία", "G:N-F", "self-indulgence", "akrasia"),
        "G193": ("ἀκρατής", "G:A", "intemperate", "akratēs"),
        "G194": ("ἄκρατος", "G:A", "undiluted", "akratos"),
        "G195": ("ἀκρίβεια", "G:N-F", "strictness", "akribeia"),
        "G196": ("ἀκριβέστατος", "G:A", "strictest", "akribestatos"),
        "G197": ("ἀκριβέστερον", "G:Adv", "stricter", "akribesteron"),
        "G198": ("ἀκριβόω", "G:V", "be exactly", "akriboō"),
        "G199": ("ἀκριβῶς", "G:Adv", "exactly", "akribōs"),
        "G200": ("ἀκρίς", "G:N-F", "locust", "akris"),
        "G201": ("ἀκροατήριον", "G:N-N", "hall", "akroatērion"),
        "G202": ("ἀκροατής", "G:N-M", "hearer", "akroatēs"),
        "G203": ("ἀκροβυστία", "G:N-F", "uncircumcision", "akrobustia"),
        "G204": ("ἀκρογωνιαῖος", "G:A", "cornerstone", "akrogōniaios"),
        "G205": ("ἀκροθίνιον", "G:N-N", "fine spoils", "akrothinion"),
        "G206": ("ἄκρον", "G:N-N", "end", "akron"),
        "G207": ("Ἀκύλας", "N:N-M-P", "Aquila", "Akulas"),
        "G208": ("ἀκυρόω", "G:V", "to nullify", "akuroō"),
        "G209": ("ἀκωλύτως", "G:Adv", "freely", "akōlutōs"),
        "G210": ("ἄκων", "G:A", "unwilling", "akōn"),
        "G211": ("ἀλάβαστρον", "G:N-N", "jar", "alabastron"),
        "G212": ("ἀλαζονεία", "G:N-F", "boasting", "alazoneia"),
        "G213": ("ἀλαζών", "G:N-M", "braggart", "alazōn"),
        "G214": ("ἀλαλάζω", "G:V", "to wail", "alalazō"),
        "G215": ("ἀλάλητος", "G:A", "inexpressible", "alalētos"),
        "G216": ("ἄλαλος", "G:A", "mute", "alalos"),
        "G217": ("ἅλας", "G:N-N", "salt", "halas"),
        "G218": ("ἀλείφω", "G:V", "to anoint", "aleiphō"),
        "G219": ("ἀλεκτοροφωνία", "G:N-F", "crowing", "alektorophōnia"),
        "G220": ("ἀλέκτωρ", "G:N-M", "rooster", "alektōr"),
        "G221": ("Ἀλεξανδρεύς", "N:N--LG", "Alexandrian", "Alexandreus"),
        "G222": ("Ἀλεξανδρῖνος", "N:N--LG", "Alexandrian", "Alexandrinos"),
        "G223": ("Ἀλέξανδρος", "N:N-M-P", "Alexander", "Alexandros"),
        "G224": ("ἄλευρον", "G:N-N", "flour", "aleuron"),
        "G225": ("ἀλήθεια", "G:N-F", "truth", "alētheia"),
        "G226": ("ἀληθεύω", "G:V", "be truthful", "alētheuō"),
        "G227": ("ἀληθής", "G:A", "true", "alēthēs"),
        "G228": ("ἀληθινός", "G:A", "true", "alēthinos"),
        "G229": ("ἀλήθω", "G:V", "to grind", "alēthō"),
        "G230": ("ἀληθῶς", "G:Adv", "truly", "alēthōs"),
        "G231": ("ἁλιεύς", "G:N-M", "fisherman", "halieus"),
        "G232": ("ἁλιεύω", "G:V", "to fish", "halieuō"),
        "G233": ("ἁλίζω", "G:V", "to salt", "halizō"),
        "G234": ("ἀλίσγημα", "G:N-N", "defilement", "alisgēma"),
        "G235": ("ἀλλά", "G:Conj", "but", "alla"),
        "G236": ("ἀλλάσσω", "G:V", "to change", "allassō"),
        "G237": ("ἀλλαχόθεν", "G:Adv", "from elsewhere", "allachothen"),
        "G238": ("ἀλληγορέω", "G:V", "to use an analogy", "allēgoreō"),
        "G239": ("ἀλληλουϊα", "H:Intj", "hallelujah", "allēlouia"),
        "G240": ("ἀλλήλων", "G:RefP", "one another", "allēlōn"),
        "G241": ("ἀλλογενής", "G:A", "foreign", "allogenēs"),
        "G242": ("ἅλλομαι", "G:V", "to spring", "hallomai"),
        "G243": ("ἄλλος", "G:A", "another", "allos"),
        "G244": ("ἀλλοτριεπίσκοπος", "G:N-M", "meddler", "allotriepiskopos"),
        "G245": ("ἀλλότριος", "G:A", "another’s", "allotrios"),
        "G246": ("ἀλλόφυλος", "G:A", "foreigner", "allophulos"),
        "G247": ("ἄλλως", "G:Adv", "otherwise", "allōs"),
        "G248": ("ἀλοάω", "G:V", "to thresh", "aloaō"),
        "G249": ("ἄλογος", "G:A", "unreasonable", "alogos"),
        "G250": ("ἀλόη", "G:N-F", "aloes", "aloē"),
        "G251": ("ἅλς", "G:N-M", "salt", "hals"),
        "G252": ("ἁλυκός", "G:A", "salty", "halukos"),
        "G253": ("ἄλυπος", "G:A", "without anxiety", "alupos"),
        "G254": ("ἅλυσις", "G:N-F", "chain", "halusis"),
        "G255": ("ἀλυσιτελής", "G:A", "unprofitable", "alusitelēs"),
        "G256": ("Ἀλφαῖος", "N:N-M-P", "Alphaeus", "Alphaios"),
        "G257": ("ἅλων", "G:N-F", "threshing-floor", "halōn"),
        "G258": ("ἀλώπηξ", "G:N-F", "fox", "alōpēx"),
        "G259": ("ἅλωσις", "G:N-F", "capture", "halōsis"),
        "G260": ("ἅμα", "G:Prep", "together", "hama"),
        "G261": ("ἀμαθής", "G:A", "ignorant", "amathēs"),
        "G262": ("ἀμαράντινος", "G:A", "unfading", "amarantinos"),
        "G263": ("ἀμάραντος", "G:A", "unfading", "amarantos"),
        "G264": ("ἁμαρτάνω", "G:V", "to sin", "hamartanō"),
        "G265": ("ἁμάρτημα", "G:N-N", "sin", "hamartēma"),
        "G266": ("ἁμαρτία", "G:N-F", "sin", "hamartia"),
        "G267": ("ἀμάρτυρος", "G:A", "without witness", "amarturos"),
        "G268": ("ἁμαρτωλός", "G:A", "sinful", "hamartōlos"),
        "G269": ("ἄμαχος", "G:A", "peacable", "amachos"),
        "G270": ("ἀμάω", "G:V", "to mow", "amaō"),
        "G271": ("ἀμέθυστος", "G:N-F", "amethyst", "amethustos"),
        "G272": ("ἀμελέω", "G:V", "to neglect", "ameleō"),
        "G273": ("ἄμεμπτος", "G:A", "blameless", "amemptos"),
        "G274": ("ἀμέμπτως", "G:Adv", "blamelessly", "amemptōs"),
        "G275": ("ἀμέριμνος", "G:A", "untroubled", "amerimnos"),
        "G276": ("ἀμετάθετος", "G:A", "unchangeable", "ametathetos"),
        "G277": ("ἀμετακίνητος", "G:A", "immovable", "ametakinētos"),
        "G278": ("ἀμεταμέλητος", "G:A", "irrevocable", "ametamelētos"),
        "G279": ("ἀμετανόητος", "G:A", "unrepentant", "ametanoētos"),
        "G280": ("ἄμετρος", "G:A", "immoderate", "ametros"),
        "G281": ("ἀμήν", "H:A", "amen", "amēn"),
        "G282": ("ἀμήτωρ", "G:A", "motherless", "amētōr"),
        "G283": ("ἀμίαντος", "G:A", "pure", "amiantos"),
        "G284": ("Ἀμιναδάβ", "N:N-M-P", "Amminadab", "Aminadab"),
        "G285": ("ἄμμος", "G:N-F", "sand", "ammos"),
        "G286": ("ἀμνός", "G:N-M", "lamb", "amnos"),
        "G287": ("ἀμοιβή", "G:N-F", "repayment", "amoibē"),
        "G288": ("ἄμπελος", "G:N-F", "vine", "ampelos"),
        "G289": ("ἀμπελουργός", "G:N-M", "a vine-worker", "ampelourgos"),
        "G290": ("ἀμπελών", "G:N-M", "vineyard", "ampelōn"),
        "G291": ("Ἀμπλίας", "N:N-M-P", "Ampliatus", "Amplias"),
        "G292": ("ἀμύνω", "G:V", "to defend", "amunō"),
        "G293": ("ἀμφίβληστρον", "G:N-N", "net", "amphiblēstron"),
        "G294": ("ἀμφιέννυμι", "G:V", "to clothe", "amphiennumi"),
        "G295": ("Ἀμφίπολις", "N:N--L", "Amphipolis", "Amphipolis"),
        "G296": ("ἄμφοδον", "G:N-N", "street", "amphodon"),
        "G297": ("ἀμφότερος", "G:A", "both", "amphoteros"),
        "G298": ("ἀμώμητος", "G:A", "blameless", "amōmētos"),
        "G299": ("ἄμωμος", "G:A", "blameless", "amōmos"),
        "G300": ("Ἀμών", "N:N-M-P", "Amon", "Amōn"),
        "G301": ("Ἀμώς", "N:N-M-P", "Amos", "Amōs"),
        "G302": ("ἄν", "G:Cond", "if", "an"),
        "G303": ("ἀνά", "G:Prep", "each", "ana"),
        "G304": ("ἀναβαθμός", "G:N-M", "stairs", "anabathmos"),
        "G305": ("ἀναβαίνω", "G:V", "to ascend", "anabainō"),
        "G306": ("ἀναβάλλω", "G:V", "to defer", "anaballō"),
        "G307": ("ἀναβιβάζω", "G:V", "to pull up", "anabibazō"),
        "G308": ("ἀναβλέπω", "G:V", "to look up/again", "anablepō"),
        "G309": ("ἀνάβλεψις", "G:N-F", "recovery of sight", "anablepsis"),
        "G310": ("ἀναβοάω", "G:V", "to cry out", "anaboaō"),
        "G311": ("ἀναβολή", "G:N-F", "delay", "anabolē"),
        "G312": ("ἀναγγέλλω", "G:V", "to report", "anaggellō"),
        "G313": ("ἀναγεννάω", "G:V", "to beget", "anagennaō"),
        "G314": ("ἀναγινώσκω", "G:V", "to read", "anaginōskō"),
        "G315": ("ἀναγκάζω", "G:V", "to compel", "anagkazō"),
        "G316": ("ἀναγκαῖος", "G:A", "necessary", "anagkaios"),
        "G317": ("ἀναγκαστῶς", "G:Adv", "necessarily", "anagkastōs"),
        "G318": ("ἀνάγκη", "G:N-F", "necessity", "anagkē"),
        "G319": ("ἀναγνωρίζω", "G:V", "to recognize", "anagnōrizō"),
        "G320": ("ἀνάγνωσις", "G:N-F", "reading", "anagnōsis"),
        "G321": ("ἀνάγω", "G:V", "to lead", "anagō"),
        "G322": ("ἀναδείκνυμι", "G:V", "to appoint", "anadeiknumi"),
        "G323": ("ἀνάδειξις", "G:N-F", "public appearance", "anadeixis"),
        "G324": ("ἀναδέχομαι", "G:V", "to receive", "anadechomai"),
        "G325": ("ἀναδίδωμι", "G:V", "to deliver", "anadidōmi"),
        "G326": ("ἀναζάω", "G:V", "to revive", "anazaō"),
        "G327": ("ἀναζητέω", "G:V", "to search", "anazēteō"),
        "G328": ("ἀναζώννυμι", "G:V", "to gird", "anazōnnumi"),
        "G329": ("ἀναζωπυρέω", "G:V", "to rekindle", "anazōpureō"),
        "G330": ("ἀναθάλλω", "G:V", "to renew", "anathallō"),
        "G331": ("ἀνάθεμα", "G:N-N", "devoted", "anathema"),
        "G332": ("ἀναθεματίζω", "G:V", "to take an oath", "anathematizō"),
        "G333": ("ἀναθεωρέω", "G:V", "to contemplate", "anatheōreō"),
        "G334": ("ἀνάθημα", "G:N-N", "a vow offering", "anathēma"),
        "G335": ("ἀναίδεια", "G:N-F", "shamelessnes", "anaideia"),
        "G336": ("ἀναίρεσις", "G:N-F", "murder", "anairesis"),
        "G337": ("ἀναιρέω", "G:V", "to kill", "anaireō"),
        "G338": ("ἀναίτιος", "G:A", "innocent", "anaitios"),
        "G339": ("ἀνακαθίζω", "G:V", "to sit up", "anakathizō"),
        "G340": ("ἀνακαινίζω", "G:V", "to restore", "anakainizō"),
        "G341": ("ἀνακαινόω", "G:V", "to renew", "anakainoō"),
        "G342": ("ἀνακαίνωσις", "G:N-F", "renewal", "anakainōsis"),
        "G343": ("ἀνακαλύπτω", "G:V", "to unveil", "anakaluptō"),
        "G344": ("ἀνακάμπτω", "G:V", "to return", "anakamptō"),
        "G345": ("ἀνάκειμαι", "G:V", "to recline", "anakeimai"),
        "G346": ("ἀνακεφαλαιόομαι", "G:V", "to summarise", "anakephalaioomai"),
        "G347": ("ἀνακλίνω", "G:V", "to recline", "anaklinō"),
        "G348": ("ἀνακόπτω", "G:V", "to restrain", "anakoptō"),
        "G349": ("ἀνακράζω", "G:V", "to yell", "anakrazō"),
        "G350": ("ἀνακρίνω", "G:V", "to investigate", "anakrinō"),
        "G351": ("ἀνάκρισις", "G:N-F", "investigation", "anakrisis"),
        "G352": ("ἀνακύπτω", "G:V", "to straighten up", "anakuptō"),
        "G353": ("ἀναλαμβάνω", "G:V", "to take up", "analambanō"),
        "G354": ("ἀνάληψις", "G:N-F", "ascension", "analēpsis"),
        "G355": ("ἀναλίσκω", "G:V", "to consume", "analiskō"),
        "G356": ("ἀναλογία", "G:N-F", "proportion", "analogia"),
        "G357": ("ἀναλογίζομαι", "G:V", "to consider", "analogizomai"),
        "G358": ("ἄναλος", "G:A", "unsalty", "analos"),
        "G359": ("ἀνάλυσις", "G:N-F", "departure", "analusis"),
        "G360": ("ἀναλύω", "G:V", "to depart", "analuō"),
        "G361": ("ἀναμάρτητος", "G:A", "sinless", "anamartētos"),
        "G362": ("ἀναμένω", "G:V", "to await", "anamenō"),
        "G363": ("ἀναμιμνήσκω", "G:V", "to remind", "anamimnēskō"),
        "G364": ("ἀνάμνησις", "G:N-F", "remembrance", "anamnēsis"),
        "G365": ("ἀνανεόομαι", "G:V", "to renew", "ananeoomai"),
        "G366": ("ἀνανήφω", "G:V", "to regain senses", "ananēphō"),
        "G367": ("Ἀνανίας", "N:N-M-P", "Ananias", "Ananias"),
        "G368": ("ἀναντίρρητος", "G:A", "indisputable", "anantirrētos"),
        "G369": ("ἀναντιρρήτως", "G:Adv", "without objection", "anantirrētōs"),
        "G370": ("ἀνάξιος", "G:A", "unworthy", "anaxios"),
        "G371": ("ἀναξίως", "G:Adv", "unworthily", "anaxiōs"),
        "G372": ("ἀνάπαυσις", "G:N-F", "rest", "anapausis"),
        "G373": ("ἀναπαύω", "G:V", "to give rest", "anapauō"),
        "G374": ("ἀναπείθω", "G:V", "to persuade", "anapeithō"),
        "G375": ("ἀναπέμπω", "G:V", "to send back", "anapempō"),
        "G376": ("ἀνάπηρος", "G:A", "crippled", "anapēros"),
        "G377": ("ἀναπίπτω", "G:V", "to recline", "anapiptō"),
        "G378": ("ἀναπληρόω", "G:V", "to fulfil", "anaplēroō"),
        "G379": ("ἀναπολόγητος", "G:A", "inexcusable", "anapologētos"),
        "G380": ("ἀναπτύσσω", "G:V", "to unroll", "anaptussō"),
        "G381": ("ἀνάπτω", "G:V", "to set fire", "anaptō"),
        "G382": ("ἀναρίθμητος", "G:A", "countless", "anarithmētos"),
        "G383": ("ἀνασείω", "G:V", "to incite", "anaseiō"),
        "G384": ("ἀνασκευάζω", "G:V", "to upset", "anaskeuazō"),
        "G385": ("ἀνασπάω", "G:V", "to pull up", "anaspaō"),
        "G386": ("ἀνάστασις", "G:N-F", "resurrection", "anastasis"),
        "G387": ("ἀναστατόω", "G:V", "to cause trouble", "anastatoō"),
        "G388": ("ἀνασταυρόω", "G:V", "to recrucify", "anastauroō"),
        "G389": ("ἀναστενάζω", "G:V", "to sigh deeply", "anastenazō"),
        "G390": ("ἀναστρέφω", "G:V", "to live/return", "anastrephō"),
        "G391": ("ἀναστροφή", "G:N-F", "behaviour", "anastrophē"),
        "G392": ("ἀνατάσσω", "G:V", "to compile", "anatassō"),
        "G393": ("ἀνατέλλω", "G:V", "to rise", "anatellō"),
        "G394": ("ἀνατίθημι", "G:V", "to set before", "anatithēmi"),
        "G395": ("ἀνατολή", "G:N-F", "east", "anatolē"),
        "G396": ("ἀνατρέπω", "G:V", "to overturn", "anatrepō"),
        "G397": ("ἀνατρέφω", "G:V", "to bring up", "anatrephō"),
        "G398": ("ἀναφαίνω", "G:V", "to appear", "anaphainō"),
        "G399": ("ἀναφέρω", "G:V", "to carry up", "anapherō"),
        "G400": ("ἀναφωνέω", "G:V", "to exclaim", "anaphōneō"),
        "G401": ("ἀνάχυσις", "G:N-F", "outpouring", "anachusis"),
        "G402": ("ἀναχωρέω", "G:V", "to leave", "anachōreō"),
        "G403": ("ἀνάψυξις", "G:N-F", "refreshment", "anapsuxis"),
        "G404": ("ἀναψύχω", "G:V", "to refresh", "anapsuchō"),
        "G405": ("ἀνδραποδιστής", "G:N-M", "slave-trader", "andrapodistēs"),
        "G406": ("Ἀνδρέας", "N:N-M-P", "Andrew", "Andreas"),
        "G407": ("ἀνδρίζω", "G:V", "to act like a man", "andrizō"),
        "G408": ("Ἀνδρόνικος", "N:N-M-P", "Andronicus", "Andronikos"),
        "G409": ("ἀνδροφόνος", "G:N-M", "murderer", "androphonos"),
        "G410": ("ἀνέγκλητος", "G:A", "irreproachable", "anegklētos"),
        "G411": ("ἀνεκδιήγητος", "G:A", "indescribable", "anekdiēgētos"),
        "G412": ("ἀνεκλάλητος", "G:A", "inexpressible", "aneklalētos"),
        "G413": ("ἀνέκλειπτος", "G:A", "inexhaustible", "anekleiptos"),
        "G414": ("ἀνεκτός", "G:A", "bearable", "anektos"),
        "G415": ("ἀνελεήμων", "G:A", "merciless", "aneleēmōn"),
        "G416": ("ἀνεμίζομαι", "G:V", "be wind-blown", "anemizomai"),
        "G417": ("ἄνεμος", "G:N-M", "wind", "anemos"),
        "G418": ("ἀνένδεκτος", "G:A", "impossible", "anendektos"),
        "G419": ("ἀνεξερεύνητος", "G:A", "unsearchable", "anexereunētos"),
        "G420": ("ἀνεξίκακος", "G:A", "not resentful", "anexikakos"),
        "G421": ("ἀνεξιχνίαστος", "G:A", "unsearchable", "anexichniastos"),
        "G422": ("ἀνεπαίσχυντος", "G:A", "unashamed", "anepaischuntos"),
        "G423": ("ἀνεπίληπτος", "G:A", "irreproachable", "anepilēptos"),
        "G424": ("ἀνέρχομαι", "G:V", "to go up", "anerchomai"),
        "G425": ("ἄνεσις", "G:N-F", "rest", "anesis"),
        "G426": ("ἀνετάζω", "G:V", "to question", "anetazō"),
        "G427": ("ἄνευ", "G:Prep", "without", "aneu"),
        "G428": ("ἀνεύθετος", "G:A", "unsuitable", "aneuthetos"),
        "G429": ("ἀνευρίσκω", "G:V", "to find", "aneuriskō"),
        "G430": ("ἀνέχω", "G:V", "to endure", "anechō"),
        "G431": ("ἀνεψιός", "G:N-M", "cousin", "anepsios"),
        "G432": ("ἄνηθον", "G:N-N", "dill", "anēthon"),
        "G433": ("ἀνήκω", "G:V", "be fitting", "anēkō"),
        "G434": ("ἀνήμερος", "G:A", "brutal", "anēmeros"),
        "G435": ("ἀνήρ", "G:N-M", "man", "anēr"),
        "G436": ("ἀνθίστημι", "G:V", "to oppose", "anthistēmi"),
        "G437": ("ἀνθομολογέομαι", "G:V", "to praise", "anthomologeomai"),
        "G438": ("ἄνθος", "G:N-N", "flower", "anthos"),
        "G439": ("ἀνθρακιά", "G:N-F", "charcoal fire", "anthrakia"),
        "G440": ("ἄνθραξ", "G:N-M", "charcoal", "anthrax"),
        "G441": ("ἀνθρωπάρεσκος", "G:A", "people-pleaser", "anthrōpareskos"),
        "G442": ("ἀνθρώπινος", "G:A", "human", "anthrōpinos"),
        "G443": ("ἀνθρωποκτόνος", "G:N-M", "murderer", "anthrōpoktonos"),
        "G444": ("ἄνθρωπος", "G:N-M", "a human", "anthrōpos"),
        "G445": ("ἀνθυπατεύω", "G:V", "to be proconsul", "anthupateuō"),
        "G446": ("ἀνθύπατος", "G:N-M", "proconsul", "anthupatos"),
        "G447": ("ἀνίημι", "G:V", "to loosen/leave", "aniēmi"),
        "G448": ("ἀνίλεως", "G:A", "merciless", "anileōs"),
        "G449": ("ἄνιπτος", "G:A", "unwashed", "aniptos"),
        "G450": ("ἀνίστημι", "G:V", "to arise", "anistēmi"),
        "G451": ("Ἄννα", "N:N-F-P", "Anna", "Anna"),
        "G452": ("Ἄννας", "N:N-M-P", "Annas", "Annas"),
        "G453": ("ἀνόητος", "G:A", "foolish", "anoētos"),
        "G454": ("ἄνοια", "G:N-F", "folly", "anoia"),
        "G455": ("ἀνοίγω", "G:V", "to open", "anoigō"),
        "G456": ("ἀνοικοδομέω", "G:V", "to rebuild", "anoikodomeō"),
        "G457": ("ἄνοιξις", "G:N-F", "opening", "anoixis"),
        "G458": ("ἀνομία", "G:N-F", "lawlessness", "anomia"),
        "G459": ("ἄνομος", "G:A", "lawless", "anomos"),
        "G460": ("ἀνόμως", "G:Adv", "without law", "anomōs"),
        "G461": ("ἀνορθόω", "G:V", "to restore", "anorthoō"),
        "G462": ("ἀνόσιος", "G:A", "unholy", "anosios"),
        "G463": ("ἀνοχή", "G:N-F", "tolerance", "anochē"),
        "G464": ("ἀνταγωνίζομαι", "G:V", "to struggle", "antagōnizomai"),
        "G465": ("ἀντάλλαγμα", "G:N-N", "in exchange", "antallagma"),
        "G466": ("ἀνταναπληρόω", "G:V", "to fill again", "antanaplēroō"),
        "G467": ("ἀνταποδίδωμι", "G:V", "to repay", "antapodidōmi"),
        "G468": ("ἀνταπόδομα", "G:N-N", "repayment", "antapodoma"),
        "G469": ("ἀνταπόδοσις", "G:N-F", "reward", "antapodosis"),
        "G470": ("ἀνταποκρίνομαι", "G:V", "to contradict", "antapokrinomai"),
        "G471": ("ἀντεῖπον", "G:V", "to contradict", "anteipon"),
        "G472": ("ἀντέχω", "G:V", "to cling to", "antechō"),
        "G473": ("ἀντί", "G:Prep", "for", "anti"),
        "G474": ("ἀντιβάλλω", "G:V", "to discuss", "antiballō"),
        "G475": ("ἀντιδιατίθημι", "G:V", "to oppose", "antidiatithēmi"),
        "G476": ("ἀντίδικος", "G:N-M", "opponent", "antidikos"),
        "G477": ("ἀντίθεσις", "G:N-F", "opposition", "antithesis"),
        "G478": ("ἀντικαθίστημι", "G:V", "to resist", "antikathistēmi"),
        "G479": ("ἀντικαλέω", "G:V", "to invite in return", "antikaleō"),
        "G480": ("ἀντίκειμαι", "G:V", "be an opponent", "antikeimai"),
        "G481": ("ἀντικρύ", "G:Adv", "opposite", "antikru"),
        "G482": ("ἀντιλαμβάνω", "G:V", "to help", "antilambanō"),
        "G483": ("ἀντιλέγω", "G:V", "to dispute", "antilegō"),
        "G484": ("ἀντίληψις", "G:N-F", "help", "antilēpsis"),
        "G485": ("ἀντιλογία", "G:N-F", "dispute", "antilogia"),
        "G486": ("ἀντιλοιδορέω", "G:V", "to retaliate", "antiloidoreō"),
        "G487": ("ἀντίλυτρον", "G:N-N", "ransom", "antilutron"),
        "G488": ("ἀντιμετρέω", "G:V", "to return", "antimetreō"),
        "G489": ("ἀντιμισθία", "G:N-F", "recompense", "antimisthia"),
        "G490": ("Ἀντιόχεια", "N:N-F-L", "Antioch", "Antiocheia"),
        "G491": ("Ἀντιοχεύς", "N:N--LG", "Antiochian", "Antiocheus"),
        "G492": ("ἀντιπαρέρχομαι", "G:V", "to pass", "antiparerchomai"),
        "G493": ("Ἀντίπας", "N:N-M-P", "Antipas", "Antipas"),
        "G494": ("Ἀντιπατρίς", "N:N--L", "Antipatris", "Antipatris"),
        "G495": ("ἀντιπέραν", "G:Prep", "opposite", "antiperan"),
        "G496": ("ἀντιπίπτω", "G:V", "to resist", "antipiptō"),
        "G497": ("ἀντιστρατεύομαι", "G:V", "to wage war against", "antistrateuomai"),
        "G498": ("ἀντιτάσσω", "G:V", "to resist", "antitassō"),
        "G499": ("ἀντίτυπος", "G:A", "representation", "antitupos"),
        "G500": ("ἀντίχριστος", "G:N-M", "antichrist", "antichristos"),
        "G501": ("ἀντλέω", "G:V", "to draw", "antleō"),
        "G502": ("ἄντλημα", "G:N-N", "bucket", "antlēma"),
        "G503": ("ἀντοφθαλμέω", "G:V", "to face", "antophthalmeō"),
        "G504": ("ἄνυδρος", "G:A", "waterless", "anudros"),
        "G505": ("ἀνυπόκριτος", "G:A", "genuine", "anupokritos"),
        "G506": ("ἀνυπότακτος", "G:A", "insubordinate", "anupotaktos"),
        "G507": ("ἄνω", "G:Adv", "above", "anō"),
        "G508": ("ἀνώγεον", "G:N-N", "an upper room", "anōgeon"),
        "G509": ("ἄνωθεν", "G:Adv", "from above/again", "anōthen"),
        "G510": ("ἀνωτερικός", "G:A", "interior", "anōterikos"),
        "G511": ("ἀνώτερος", "G:A", "higher", "anōteros"),
        "G512": ("ἀνωφελής", "G:A", "useless", "anōphelēs"),
        "G513": ("ἀξίνη", "G:N-F", "axe", "axinē"),
        "G514": ("ἄξιος", "G:A", "worthy", "axios"),
        "G515": ("ἀξιόω", "G:V", "to deem worthy", "axioō"),
        "G516": ("ἀξίως", "G:Adv", "appropriately", "axiōs"),
        "G517": ("ἀόρατος", "G:A", "invisible", "aoratos"),
        "G518": ("ἀπαγγέλλω", "G:V", "to announce", "apaggellō"),
        "G519": ("ἀπάγχω", "G:V", "to strangle", "apagchō"),
        "G520": ("ἀπάγω", "G:V", "to lead away", "apagō"),
        "G521": ("ἀπαίδευτος", "G:A", "uninstructed", "apaideutos"),
        "G522": ("ἀπαίρω", "G:V", "to take away", "apairō"),
        "G523": ("ἀπαιτέω", "G:V", "to demand", "apaiteō"),
        "G524": ("ἀπαλγέω", "G:V", "to become callous", "apalgeō"),
        "G525": ("ἀπαλλάσσω", "G:V", "to release", "apallassō"),
        "G526": ("ἀπαλλοτριόω", "G:V", "to alienate", "apallotrioō"),
        "G527": ("ἁπαλός", "G:A", "tender", "hapalos"),
        "G528": ("ἀπαντάω", "G:V", "to meet", "apantaō"),
        "G529": ("ἀπάντησις", "G:N-F", "meeting", "apantēsis"),
        "G530": ("ἅπαξ", "G:Adv", "once", "hapax"),
        "G531": ("ἀπαράβατος", "G:A", "permanent", "aparabatos"),
        "G532": ("ἀπαρασκεύαστος", "G:A", "unprepared", "aparaskeuastos"),
        "G533": ("ἀπαρνέομαι", "G:V", "to deny", "aparneomai"),
        "G534": ("ἀπαρτί", "G:Adv", "henceforth", "aparti"),
        "G535": ("ἀπαρτισμός", "G:N-M", "completion", "apartismos"),
        "G536": ("ἀπαρχή", "G:N-F", "firstfruits", "aparchē"),
        "G537": ("ἅπας", "G:A", "all", "hapas"),
        "G538": ("ἀπατάω", "G:V", "to trick", "apataō"),
        "G539": ("ἀπάτη", "G:N-F", "deceit", "apatē"),
        "G540": ("ἀπάτωρ", "G:A", "fatherless", "apatōr"),
        "G541": ("ἀπαύγασμα", "G:N-N", "radiance", "apaugasma"),
        "G542": ("ἀπεῖδον", "G:V", "to fix one's eyes", "apeidon"),
        "G543": ("ἀπείθεια", "G:N-F", "disobedience", "apeitheia"),
        "G544": ("ἀπειθέω", "G:V", "to disobey", "apeitheō"),
        "G545": ("ἀπειθής", "G:A", "disobedient", "apeithēs"),
        "G546": ("ἀπειλέω", "G:V", "to threaten", "apeileō"),
        "G547": ("ἀπειλή", "G:N-F", "threat", "apeilē"),
        "G548": ("ἄπειμι", "G:V", "be away", "apeimi"),
        "G549": ("ἄπειμι", "G:V", "to go", "apeimi"),
        "G550": ("ἀπεῖπον", "G:V", "to renounce", "apeipon"),
        "G551": ("ἀπείραστος", "G:A", "untempted", "apeirastos"),
        "G552": ("ἄπειρος", "G:A", "unacquainted", "apeiros"),
        "G553": ("ἀπεκδέχομαι", "G:V", "to expect", "apekdechomai"),
        "G554": ("ἀπεκδύομαι", "G:V", "to take off", "apekduomai"),
        "G555": ("ἀπέκδυσις", "G:N-F", "removal", "apekdusis"),
        "G556": ("ἀπελαύνω", "G:V", "to eject", "apelaunō"),
        "G557": ("ἀπελεγμός", "G:N-M", "discredit", "apelegmos"),
        "G558": ("ἀπελεύθερος", "G:N-M", "freedman", "apeleutheros"),
        "G559": ("Ἀπελλῆς", "N:N-M-P", "Apelles", "Apellēs"),
        "G560": ("ἀπελπίζω", "G:V", "to despair", "apelpizō"),
        "G561": ("ἀπέναντι", "G:Prep", "opposite", "apenanti"),
        "G562": ("ἀπέραντος", "G:A", "endless", "aperantos"),
        "G563": ("ἀπερισπάστως", "G:Adv", "undistracted", "aperispastōs"),
        "G564": ("ἀπερίτμητος", "G:A", "uncircumcised", "aperitmētos"),
        "G565": ("ἀπέρχομαι", "G:V", "to go away", "aperchomai"),
        "G566": ("ἀπέχει", "G:V", "to receive/be far", "apechei"),
        "G567": ("ἀπέχομαι", "G:V", "to receive/be far", "apechomai"),
        "G568": ("ἀπέχω", "G:V", "to have in full", "apechō"),
        "G569": ("ἀπιστέω", "G:V", "to disbelieve", "apisteō"),
        "G570": ("ἀπιστία", "G:N-F", "unbelief", "apistia"),
        "G571": ("ἄπιστος", "G:A", "unbelieving", "apistos"),
        "G572": ("ἁπλότης", "G:N-F", "sincerity", "haplotēs"),
        "G573": ("ἁπλοῦς", "G:A", "sound", "haplous"),
        "G574": ("ἁπλῶς", "G:Adv", "without reserve", "haplōs"),
        "G575": ("ἀπό", "G:Prep", "away from", "apo"),
        "G576": ("ἀποβαίνω", "G:V", "to get out", "apobainō"),
        "G577": ("ἀποβάλλω", "G:V", "to throw away", "apoballō"),
        "G578": ("ἀποβλέπω", "G:V", "to look ahead", "apoblepō"),
        "G579": ("ἀπόβλητος", "G:A", "rejected", "apoblētos"),
        "G580": ("ἀποβολή", "G:N-F", "deprivation", "apobolē"),
        "G581": ("ἀπογενόμενος", "G:V", "to cease to be", "apogenomenos"),
        "G582": ("ἀπογραφή", "G:N-F", "census", "apographē"),
        "G583": ("ἀπογράφω", "G:V", "to register", "apographō"),
        "G584": ("ἀποδείκνυμι", "G:V", "to display", "apodeiknumi"),
        "G585": ("ἀπόδειξις", "G:N-F", "demonstration", "apodeixis"),
        "G586": ("ἀποδεκατόω", "G:V", "to tithe", "apodekatoō"),
        "G587": ("ἀπόδεκτος", "G:A", "pleasing", "apodektos"),
        "G588": ("ἀποδέχομαι", "G:V", "to welcome", "apodechomai"),
        "G589": ("ἀποδημέω", "G:V", "to go abroad", "apodēmeō"),
        "G590": ("ἀπόδημος", "G:A", "absent", "apodēmos"),
        "G591": ("ἀποδίδωμι", "G:V", "to pay", "apodidōmi"),
        "G592": ("ἀποδιορίζω", "G:V", "to divide", "apodiorizō"),
        "G593": ("ἀποδοκιμάζω", "G:V", "to reject", "apodokimazō"),
        "G594": ("ἀποδοχή", "G:N-F", "acceptance", "apodochē"),
        "G595": ("ἀπόθεσις", "G:N-F", "removal", "apothesis"),
        "G596": ("ἀποθήκη", "G:N-F", "storehouse", "apothēkē"),
        "G597": ("ἀποθησαυρίζω", "G:V", "to store up", "apothēsaurizō"),
        "G598": ("ἀποθλίβω", "G:V", "to crowd up to", "apothlibō"),
        "G599": ("ἀποθνήσκω", "G:V", "to die", "apothnēskō"),
        "G600": ("ἀποκαθίστημι", "G:V", "to restore", "apokathistēmi"),
        "G601": ("ἀποκαλύπτω", "G:V", "to reveal", "apokaluptō"),
        "G602": ("ἀποκάλυψις", "G:N-F", "revelation", "apokalupsis"),
        "G603": ("ἀποκαραδοκία", "G:N-F", "eager expectation", "apokaradokia"),
        "G604": ("ἀποκαταλλάσσω", "G:V", "to reconcile", "apokatallassō"),
        "G605": ("ἀποκατάστασις", "G:N-F", "restoration", "apokatastasis"),
        "G606": ("ἀπόκειμαι", "G:V", "to lay up", "apokeimai"),
        "G607": ("ἀποκεφαλίζω", "G:V", "to behead", "apokephalizō"),
        "G608": ("ἀποκλείω", "G:V", "to shut", "apokleiō"),
        "G609": ("ἀποκόπτω", "G:V", "to cut off", "apokoptō"),
        "G610": ("ἀπόκριμα", "G:N-N", "verdict", "apokrima"),
        "G611": ("ἀποκρίνω", "G:V", "to answer", "apokrinō"),
        "G612": ("ἀπόκρισις", "G:N-F", "answer", "apokrisis"),
        "G613": ("ἀποκρύπτω", "G:V", "to conceal", "apokruptō"),
        "G614": ("ἀπόκρυφος", "G:A", "concealed", "apokruphos"),
        "G615": ("ἀποκτείνω", "G:V", "to kill", "apokteinō"),
        "G616": ("ἀποκυέω", "G:V", "to generate", "apokueō"),
        "G617": ("ἀποκυλίω", "G:V", "to roll away", "apokuliō"),
        "G618": ("ἀπολαμβάνω", "G:V", "to get back", "apolambanō"),
        "G619": ("ἀπόλαυσις", "G:N-F", "enjoyment", "apolausis"),
        "G620": ("ἀπολείπω", "G:V", "to leave", "apoleipō"),
        "G621": ("ἀπολείχω", "G:V", "to lick", "apoleichō"),
        "G622": ("ἀπολλύω", "G:V", "to destroy", "apolluō"),
        "G623": ("Ἀπολλύων", "N:N--T", "Apollyon", "Apolluōn"),
        "G624": ("Ἀπολλωνία", "N:N-F-L", "Apollonia", "Apollōnia"),
        "G625": ("Ἀπολλῶς", "N:N-M-P", "Apollos", "Apollōs"),
        "G626": ("ἀπολογέομαι", "G:V", "to defend oneself", "apologeomai"),
        "G627": ("ἀπολογία", "G:N-F", "defence", "apologia"),
        "G628": ("ἀπολούω", "G:V", "to wash off", "apolouō"),
        "G629": ("ἀπολύτρωσις", "G:N-F", "redemption", "apolutrōsis"),
        "G630": ("ἀπολύω", "G:V", "to release", "apoluō"),
        "G631": ("ἀπομάσσω", "G:V", "to wipe off", "apomassō"),
        "G632": ("ἀπονέμω", "G:V", "to render as due", "aponemō"),
        "G633": ("ἀπονίπτω", "G:V", "to wash off", "aponiptō"),
        "G634": ("ἀποπίπτω", "G:V", "to fall from", "apopiptō"),
        "G635": ("ἀποπλανάω", "G:V", "to mislead", "apoplanaō"),
        "G636": ("ἀποπλέω", "G:V", "to set sail", "apopleō"),
        "G637": ("ἀποπλύνω", "G:V", "to wash off", "apoplunō"),
        "G638": ("ἀποπνίγω", "G:V", "to choke", "apopnigō"),
        "G639": ("ἀπορέω", "G:V", "be perplexed", "aporeō"),
        "G640": ("ἀπορία", "G:N-F", "perplexity", "aporia"),
        "G641": ("ἀπορρίπτω", "G:V", "to throw off", "aporriptō"),
        "G642": ("ἀπορφανίζω", "G:V", "to orphan", "aporphanizō"),
        "G643": ("ἀποσκευάζω", "G:V", "to pack up", "aposkeuazō"),
        "G644": ("ἀποσκίασμα", "G:N-N", "shadow", "aposkiasma"),
        "G645": ("ἀποσπάω", "G:V", "to draw away", "apospaō"),
        "G646": ("ἀποστασία", "G:N-F", "apostasy", "apostasia"),
        "G647": ("ἀποστάσιον", "G:N-N", "divorce", "apostasion"),
        "G648": ("ἀποστεγάζω", "G:V", "to unroof", "apostegazō"),
        "G649": ("ἀποστέλλω", "G:V", "to send", "apostellō"),
        "G650": ("ἀποστερέω", "G:V", "to defraud", "apostereō"),
        "G651": ("ἀποστολή", "G:N-F", "apostleship", "apostolē"),
        "G652": ("ἀπόστολος", "G:N-M", "apostle", "apostolos"),
        "G653": ("ἀποστοματίζω", "G:V", "to interrogate", "apostomatizō"),
        "G654": ("ἀποστρέφω", "G:V", "to turn away", "apostrephō"),
        "G655": ("ἀποστυγέω", "G:V", "to abhor", "apostugeō"),
        "G656": ("ἀποσυνάγωγος", "G:A", "excommunicated", "aposunagōgos"),
        "G657": ("ἀποτάσσω", "G:V", "to leave", "apotassō"),
        "G658": ("ἀποτελέω", "G:V", "to complete", "apoteleō"),
        "G659": ("ἀποτίθημι", "G:V", "to put aside", "apotithēmi"),
        "G660": ("ἀποτινάσσω", "G:V", "to shake off", "apotinassō"),
        "G661": ("ἀποτίνω", "G:V", "to repay", "apotinō"),
        "G662": ("ἀποτολμάω", "G:V", "be bold", "apotolmaō"),
        "G663": ("ἀποτομία", "G:N-F", "severity", "apotomia"),
        "G664": ("ἀποτόμως", "G:Adv", "severely", "apotomōs"),
        "G665": ("ἀποτρέπω", "G:V", "to avoid", "apotrepō"),
        "G666": ("ἀπουσία", "G:N-F", "absence", "apousia"),
        "G667": ("ἀποφέρω", "G:V", "to carry off", "apopherō"),
        "G668": ("ἀποφεύγω", "G:V", "to escape", "apopheugō"),
        "G669": ("ἀποφθέγγομαι", "G:V", "to declare", "apophtheggomai"),
        "G670": ("ἀποφορτίζομαι", "G:V", "to unload", "apophortizomai"),
        "G671": ("ἀπόχρησις", "G:N-F", "using up", "apochrēsis"),
        "G672": ("ἀποχωρέω", "G:V", "to leave", "apochōreō"),
        "G673": ("ἀποχωρίζω", "G:V", "to separate from", "apochōrizō"),
        "G674": ("ἀποψύχω", "G:V", "to faint", "apopsuchō"),
        "G675": ("Ἄππιος", "N:N-M-L", "Appius", "Appios"),
        "G676": ("ἀπρόσιτος", "G:A", "unapproachable", "aprositos"),
        "G677": ("ἀπρόσκοπος", "G:A", "not giving offence", "aproskopos"),
        "G678": ("ἀπροσωπολήπτως", "G:Adv", "impartially", "aprosōpolēptōs"),
        "G679": ("ἄπταιστος", "G:A", "without falling", "aptaistos"),
        "G680": ("ἅπτω", "G:V", "to touch", "haptō"),
        "G681": ("ἅπτω", "G:V", "to kindle", "haptō"),
        "G682": ("Ἀπφία", "N:N-F-P", "Apphia", "Apphia"),
        "G683": ("ἀπωθέω", "G:V", "to reject", "apōtheō"),
        "G684": ("ἀπώλεια", "G:N-F", "destruction", "apōleia"),
        "G685": ("ἀρά", "G:N-F", "curse", "ara"),
        "G686": ("ἄρα", "G:Conj", "therefore", "ara"),
        "G687": ("ἆρα", "G:Intg+G:Neg", "no?", "ara"),
        "G688": ("Ἀραβία", "N:N-F-L", "Arabia", "Arabia"),
        "G689": ("Ἀράμ", "N:N-M-P", "Aram", "Aram"),
        "G690": ("Ἄραψ", "N:N--LG", "Arabian", "Araps"),
        "G691": ("ἀργέω", "G:V", "be idle", "argeō"),
        "G692": ("ἀργός", "G:A", "idle", "argos"),
        "G693": ("ἀργύρεος", "G:A", "silver", "argureos"),
        "G694": ("ἀργύριον", "G:N-N", "money", "argurion"),
        "G695": ("ἀργυροκόπος", "G:N-M", "silversmith", "argurokopos"),
        "G696": ("ἄργυρος", "G:N-M", "silver", "arguros"),
        "G697": ("Ἄρειος Πάγος", "N:N-M-L", "Mars' Hill", "Areios Pagos"),
        "G698": ("Ἀρεοπαγίτης", "N:N-M-LG", "Areopagite", "Areopagitēs"),
        "G699": ("ἀρεσκεία", "G:N-F", "pleasing", "areskeia"),
        "G700": ("ἀρέσκω", "G:V", "to please", "areskō"),
        "G701": ("ἀρεστός", "G:A", "pleasing", "arestos"),
        "G702": ("Ἀρέτας", "N:N-M-P", "Aretas", "Aretas"),
        "G703": ("ἀρετή", "G:N-F", "virtue", "aretē"),
        "G704": ("ἀρήν", "G:N-M", "lamb", "arēn"),
        "G705": ("ἀριθμέω", "G:V", "to number", "arithmeō"),
        "G706": ("ἀριθμός", "G:N-M", "number", "arithmos"),
        "G707": ("Ἀριμαθαία", "N:N-F-L", "Arimathea", "Arimathaia"),
        "G708": ("Ἀρίσταρχος", "N:N-M-P", "Aristarchus", "Aristarchos"),
        "G709": ("ἀριστάω", "G:V", "to eat early meal", "aristaō"),
        "G710": ("ἀριστερός", "G:A", "left", "aristeros"),
        "G711": ("Ἀριστόβουλος", "N:N-M-P", "Aristobulus", "Aristoboulos"),
        "G712": ("ἄριστον", "G:N-N", "early meal", "ariston"),
        "G713": ("ἀρκετός", "G:A", "sufficient", "arketos"),
        "G714": ("ἀρκέω", "G:V", "be sufficient", "arkeō"),
        "G715": ("ἄρκτος", "G:N-F", "bear", "arktos"),
        "G716": ("ἅρμα", "G:N-N", "chariot", "harma"),
        "G717": ("Ἀρμαγεδδών", "N:N--L", "Armageddon", "Armageddōn"),
        "G718": ("ἁρμόζω", "G:V", "to betroth", "harmozō"),
        "G719": ("ἁρμός", "G:N-M", "joint", "harmos"),
        "G720": ("ἀρνέομαι", "G:V", "to deny", "arneomai"),
        "G721": ("ἀρνίον", "G:N-N", "lamb", "arnion"),
        "G722": ("ἀροτριάω", "G:V", "to plow", "arotriaō"),
        "G723": ("ἄροτρον", "G:N-N", "plow", "arotron"),
        "G724": ("ἁρπαγή", "G:N-F", "plunder", "harpagē"),
        "G725": ("ἁρπαγμός", "G:N-M", "something to grasp", "harpagmos"),
        "G726": ("ἁρπάζω", "G:V", "to seize", "harpazō"),
        "G727": ("ἅρπαξ", "G:A", "rapacious", "harpax"),
        "G728": ("ἀρραβών", "G:N-M", "guarantee", "arrabōn"),
        "G729": ("ἄρραφος", "G:A", "seamless", "arraphos"),
        "G730": ("ἄρρην, ἄρσην", "G:A", "male", "arrēn, arsēn"),
        "G731": ("ἄρρητος", "G:A", "inexpressible", "arrētos"),
        "G732": ("ἄρρωστος", "G:A", "ill", "arrōstos"),
        "G733": ("ἀρσενοκοίτης", "G:N-M", "sodomy", "arsenokoitēs"),
        "G734": ("Ἀρτεμάς", "N:N-M-P", "Artemas", "Artemas"),
        "G735": ("Ἄρτεμις", "N:N--T", "Artemis", "Artemis"),
        "G736": ("ἀρτέμων", "G:N-M", "foresail", "artemōn"),
        "G737": ("ἄρτι", "G:Adv", "now", "arti"),
        "G738": ("ἀρτιγέννητος", "G:A", "newborn", "artigennētos"),
        "G739": ("ἄρτιος", "G:A", "competent", "artios"),
        "G740": ("ἄρτος", "G:N-M", "bread", "artos"),
        "G741": ("ἀρτύω", "G:V", "to season", "artuō"),
        "G742": ("Ἀρφαξάδ", "N:N-M-P", "Arphaxad", "Arphaxad"),
        "G743": ("ἀρχάγγελος", "G:N-M", "archangel", "archaggelos"),
        "G744": ("ἀρχαῖος", "G:A", "ancient", "archaios"),
        "G745": ("Ἀρχέλαος", "N:N-M-P", "Archelaus", "Archelaos"),
        "G746": ("ἀρχή", "G:N-F", "beginning", "archē"),
        "G747": ("ἀρχηγός", "G:N-M", "founder", "archēgos"),
        "G748": ("ἀρχιερατικός", "G:A", "high-priestly", "archieratikos"),
        "G749": ("ἀρχιερεύς", "G:N-M", "high-priest", "archiereus"),
        "G750": ("ἀρχιποίμην", "G:N-M", "chief shepherd", "archipoimēn"),
        "G751": ("Ἄρχιππος", "N:N-M-P", "Archippus", "Archippos"),
        "G752": ("ἀρχισυνάγωγος", "G:N-M", "synagogue leader", "archisunagōgos"),
        "G753": ("ἀρχιτέκτων", "G:N-M", "master-builder", "architektōn"),
        "G754": ("ἀρχιτελώνης", "G:N-M", "chief tax collector", "architelōnēs"),
        "G755": ("ἀρχιτρίκλινος", "G:N-M", "head waiter", "architriklinos"),
        "G756": ("ἄρχομαι", "G:V", "to rule/begin", "archomai"),
        "G757": ("ἄρχω", "G:V", "be first", "archō"),
        "G758": ("ἄρχων", "G:N-M", "ruler", "archōn"),
        "G759": ("ἄρωμα", "G:N-N", "spices", "arōma"),
        "G760": ("Ἀσά", "N:N-M-P", "Asa", "Asa"),
        "G761": ("ἀσάλευτος", "G:A", "unshakable", "asaleutos"),
        "G762": ("ἄσβεστος", "G:A", "unquenchable", "asbestos"),
        "G763": ("ἀσέβεια", "G:N-F", "ungodlinessness", "asebeia"),
        "G764": ("ἀσεβέω", "G:V", "be ungodly", "asebeō"),
        "G765": ("ἀσεβής", "G:A", "ungodly", "asebēs"),
        "G766": ("ἀσέλγεια", "G:N-F", "debauchery", "aselgeia"),
        "G767": ("ἄσημος", "G:A", "insignificant", "asēmos"),
        "G768": ("Ἀσήρ", "N:N-M-P", "Asher", "Asēr"),
        "G769": ("ἀσθένεια", "G:N-F", "weakness", "astheneia"),
        "G770": ("ἀσθενέω", "G:V", "be weak", "astheneō"),
        "G771": ("ἀσθένημα", "G:N-N", "weakness", "asthenēma"),
        "G772": ("ἀσθενής", "G:A", "weak", "asthenēs"),
        "G773": ("Ἀσία", "N:N-F-L", "Asia", "Asia"),
        "G774": ("Ἀσιανός", "N:N--LG", "Asian", "Asianos"),
        "G775": ("Ἀσιάρχης", "N:N--T", "Asiarch", "Asiarchēs"),
        "G776": ("ἀσιτία", "G:N-F", "fasting", "asitia"),
        "G777": ("ἄσιτος", "G:A", "fasting", "asitos"),
        "G778": ("ἀσκέω", "G:V", "to strive", "askeō"),
        "G779": ("ἀσκός", "G:N-M", "wineskin", "askos"),
        "G780": ("ἀσμένως", "G:Adv", "gladly", "asmenōs"),
        "G781": ("ἄσοφος", "G:A", "unwise", "asophos"),
        "G782": ("ἀσπάζομαι", "G:V", "to pay respects to", "aspazomai"),
        "G783": ("ἀσπασμός", "G:N-M", "salutation", "aspasmos"),
        "G784": ("ἄσπιλος", "G:A", "spotless", "aspilos"),
        "G785": ("ἀσπίς", "G:N-F", "asp", "aspis"),
        "G786": ("ἄσπονδος", "G:A", "irreconcilable", "aspondos"),
        "G787": ("ἀσσάριον", "G:N-N", "assarion", "assarion"),
        "G788": ("ἆσσον", "G:Adv", "nearer", "asson"),
        "G789": ("Ἄσσος", "N:N-F-L", "Assos", "Assos"),
        "G790": ("ἀστατέω", "G:V", "be unsettled", "astateō"),
        "G791": ("ἀστεῖος", "G:A", "beautiful", "asteios"),
        "G792": ("ἀστήρ", "G:N-M", "star", "astēr"),
        "G793": ("ἀστήρικτος", "G:A", "unstable", "astēriktos"),
        "G794": ("ἄστοργος", "G:A", "unfeeling", "astorgos"),
        "G795": ("ἀστοχέω", "G:V", "to deviate", "astocheō"),
        "G796": ("ἀστραπή", "G:N-F", "lightning", "astrapē"),
        "G797": ("ἀστράπτω", "G:V", "to flash", "astraptō"),
        "G798": ("ἄστρον", "G:N-N", "star", "astron"),
        "G799": ("Ἀσύγκριτος", "N:N-M-P", "Asyncritus", "Asugkritos"),
        "G800": ("ἀσύμφωνος", "G:A", "discordant", "asumphōnos"),
        "G801": ("ἀσύνετος", "G:A", "senseless", "asunetos"),
        "G802": ("ἀσύνθετος", "G:A", "untrustworthy", "asunthetos"),
        "G803": ("ἀσφάλεια", "G:N-F", "security", "asphaleia"),
        "G804": ("ἀσφαλής", "G:A", "secure", "asphalēs"),
        "G805": ("ἀσφαλίζω", "G:V", "to secure", "asphalizō"),
        "G806": ("ἀσφαλῶς", "G:Adv", "securely", "asphalōs"),
        "G807": ("ἀσχημονέω", "G:V", "to act improperly", "aschēmoneō"),
        "G808": ("ἀσχημοσύνη", "G:N-F", "indecency", "aschēmosunē"),
        "G809": ("ἀσχήμων", "G:A", "indecent", "aschēmōn"),
        "G810": ("ἀσωτία", "G:N-F", "debauchery", "asōtia"),
        "G811": ("ἀσώτως", "G:Adv", "in debauchery", "asōtōs"),
        "G812": ("ἀτακτέω", "G:V", "be idle", "atakteō"),
        "G813": ("ἄτακτος", "G:A", "disorderly", "ataktos"),
        "G814": ("ἀτάκτως", "G:Adv", "idly", "ataktōs"),
        "G815": ("ἄτεκνος", "G:A", "childless", "ateknos"),
        "G816": ("ἀτενίζω", "G:V", "to gaze", "atenizō"),
        "G817": ("ἄτερ", "G:Prep", "without", "ater"),
        "G818": ("ἀτιμάζω", "G:V", "to dishonor", "atimazō"),
        "G819": ("ἀτιμία", "G:N-F", "dishonour", "atimia"),
        "G820": ("ἄτιμος", "G:A", "dishonored", "atimos"),
        "G821": ("ἀτιμόω", "G:V", "to disgrace", "atimoō"),
        "G822": ("ἀτμίς", "G:N-F", "vapor", "atmis"),
        "G823": ("ἄτομος", "G:A", "instant", "atomos"),
        "G824": ("ἄτοπος", "G:A", "wrong", "atopos"),
        "G825": ("Ἀττάλεια", "N:N-F-L", "Attalia", "Attaleia"),
        "G826": ("αὐγάζω", "G:V", "to see", "augazō"),
        "G827": ("αὐγή", "G:N-F", "daybreak", "augē"),
        "G828": ("Αὔγουστος", "N:N-M-P", "Augustus", "Augoustos"),
        "G829": ("αὐθάδης", "G:A", "self-willed", "authadēs"),
        "G830": ("αὐθαίρετος", "G:A", "self-chosen", "authairetos"),
        "G831": ("αὐθεντέω", "G:V", "to domineer", "authenteō"),
        "G832": ("αὐλέω", "G:V", "to play the flute", "auleō"),
        "G833": ("αὐλή", "G:N-F", "palace/courtyard", "aulē"),
        "G834": ("αὐλητής", "G:N-M", "flute player", "aulētēs"),
        "G835": ("αὐλίζομαι", "G:V", "to spend the night", "aulizomai"),
        "G836": ("αὐλός", "G:N-M", "flute", "aulos"),
        "G837": ("αὐξάνω", "G:V", "to grow", "auxanō"),
        "G838": ("αὔξησις", "G:N-F", "growth", "auxēsis"),
        "G839": ("αὔριον", "G:Adv", "tomorrow", "aurion"),
        "G840": ("αὐστηρός", "G:A", "severe", "austēros"),
        "G841": ("αὐτάρκεια", "G:N-F", "self-sufficiency", "autarkeia"),
        "G842": ("αὐτάρκης", "G:A", "self-sufficient", "autarkēs"),
        "G843": ("αὐτοκατάκριτος", "G:A", "self-condemned", "autokatakritos"),
        "G844": ("αὐτόματος", "G:A", "by itself", "automatos"),
        "G845": ("αὐτόπτης", "G:N-M", "eyewitness", "autoptēs"),
        "G846": ("αὐτός", "G:PerP", "he/she/it/self", "autos"),
        "G847": ("αὐτοῦ", "G:Adv", "there", "autou"),
        "G848": ("αὑτοῦ", "G:RefP", "him/herself", "ahutou"),
        "G849": ("αὐτόχειρ", "G:A", "with one’s own hand", "autocheir"),
        "G850": ("αὐχμηρός", "G:A", "dingy", "auchmēros"),
        "G851": ("ἀφαιρέω", "G:V", "to remove", "aphaireō"),
        "G852": ("ἀφανής", "G:A", "hidden", "aphanēs"),
        "G853": ("ἀφανίζω", "G:V", "to destroy", "aphanizō"),
        "G854": ("ἀφανισμός", "G:N-M", "disappearance", "aphanismos"),
        "G855": ("ἄφαντος", "G:A", "disappearing", "aphantos"),
        "G856": ("ἀφεδρών", "G:N-M", "latrine", "aphedrōn"),
        "G857": ("ἀφειδία", "G:N-F", "unsparing", "apheidia"),
        "G858": ("ἀφελότης", "G:N-F", "sincerity", "aphelotēs"),
        "G859": ("ἄφεσις", "G:N-F", "forgiveness", "aphesis"),
        "G860": ("ἁφή", "G:N-F", "joint", "haphē"),
        "G861": ("ἀφθαρσία", "G:N-F", "incorruptibility", "aphtharsia"),
        "G862": ("ἄφθαρτος", "G:A", "incorruptible", "aphthartos"),
        "G863": ("ἀφίημι", "G:V", "to release", "aphiēmi"),
        "G864": ("ἀφικνέομαι", "G:V", "to reach", "aphikneomai"),
        "G865": ("ἀφιλάγαθος", "G:A", "hating good", "aphilagathos"),
        "G866": ("ἀφιλάργυρος", "G:A", "not greedy", "aphilarguros"),
        "G867": ("ἄφιξις", "G:N-F", "departure", "aphixis"),
        "G868": ("ἀφίστημι", "G:V", "to leave", "aphistēmi"),
        "G869": ("ἄφνω", "G:Adv", "suddenly", "aphnō"),
        "G870": ("ἀφόβως", "G:Adv", "fearlessly", "aphobōs"),
        "G871": ("ἀφομοιόω", "G:V", "to make like", "aphomoioō"),
        "G872": ("ἀφοράω", "G:V", "to consider", "aphoraō"),
        "G873": ("ἀφορίζω", "G:V", "to separate", "aphorizō"),
        "G874": ("ἀφορμή", "G:N-F", "opportunity", "aphormē"),
        "G875": ("ἀφρίζω", "G:V", "to foam", "aphrizō"),
        "G876": ("ἀφρός", "G:N-M", "foam", "aphros"),
        "G877": ("ἀφροσύνη", "G:N-F", "foolishness", "aphrosunē"),
        "G878": ("ἄφρων", "G:A", "foolish", "aphrōn"),
        "G879": ("ἀφυπνόω", "G:V", "to fall sleep", "aphupnoō"),
        "G880": ("ἄφωνος", "G:A", "mute", "aphōnos"),
        "G881": ("Ἀχάζ", "N:N-M-P", "Ahaz", "Achaz"),
        "G882": ("Ἀχαΐα", "N:N-F-L", "Achaia", "Achaia"),
        "G883": ("Ἀχαϊκός", "N:N-M-P", "Achaicus", "Achaikos"),
        "G884": ("ἀχάριστος", "G:A", "ungrateful", "acharistos"),
        "G885": ("Ἀχείμ", "N:N-M-P", "Akim", "Acheim"),
        "G886": ("ἀχειροποίητος", "G:A", "not man-made", "acheiropoiētos"),
        "G887": ("ἀχλύς", "G:N-F", "mist", "achlus"),
        "G888": ("ἀχρεῖος", "G:A", "worthless", "achreios"),
        "G889": ("ἀχρειόω", "G:V", "to make worthless", "achreioō"),
        "G890": ("ἄχρηστος", "G:A", "useless", "achrēstos"),
        "G891": ("ἄχρι", "G:Prep", "until", "achri"),
        "G892": ("ἄχυρον", "G:N-N", "chaff", "achuron"),
        "G893": ("ἀψευδής", "G:A", "not a liar", "apseudēs"),
        "G894": ("ἄψινθος", "G:N-F / N:N--T", "wormwood", "apsinthos"),
        "G895": ("ἄψυχος", "G:A", "lifeless", "apsuchos"),
        "G896": ("Βάαλ", "N:N--T", "Baal", "Baal"),
        "G897": ("Βαβυλών", "N:N-F-L", "Babylon", "Babulōn"),
        "G898": ("βαθμός", "G:N-M", "standing", "bathmos"),
        "G899": ("βάθος", "G:N-N", "depth", "bathos"),
        "G900": ("βαθύνω", "G:V", "to dig deep", "bathunō"),
        "G901": ("βαθύς", "G:A", "deep", "bathus"),
        "G902": ("βαΐον", "G:N-N", "palm branch", "baion"),
        "G903": ("Βαλαάμ", "N:N-M-P", "Balaam", "Balaam"),
        "G904": ("Βαλάκ", "N:N-M-P", "Balak", "Balak"),
        "G905": ("βαλλάντιον", "G:N-N", "purse", "ballantion"),
        "G906": ("βάλλω", "G:V", "to throw", "ballō"),
        "G907": ("βαπτίζω", "G:V", "to baptize", "baptizō"),
        "G908": ("βάπτισμα", "G:N-N", "baptism", "baptisma"),
        "G909": ("βαπτισμός", "G:N-M", "baptism", "baptismos"),
        "G910": ("Βαπτιστής", "G:N-M", "one who baptizes", "Baptistēs"),
        "G911": ("βάπτω", "G:V", "to dip", "baptō"),
        "G912": ("Βαραββᾶς", "N:N-M-P", "Barabbas", "Barabbas"),
        "G913": ("Βαράκ", "N:N-M-P", "Barak", "Barak"),
        "G914": ("Βαραχίας", "N:N-M-P", "Berekiah", "Barachias"),
        "G915": ("βάρβαρος", "G:A", "barbarian", "barbaros"),
        "G916": ("βαρέω", "G:V", "to burden", "bareō"),
        "G917": ("βαρέως", "G:Adv", "difficultly", "bareōs"),
        "G918": ("Βαρθολομαῖος", "N:N-M-P", "Bartholomew", "Bartholomaios"),
        "G919": ("Βαριησοῦς", "N:N-M-P", "Bar-Jesus", "Bariēsous"),
        "G920": ("Βαριωνᾶς", "N:N-M-P", "son of Jonas", "Bariōnas"),
        "G921": ("Βαρνάβας", "N:N-M-P", "Barnabas", "Barnabas"),
        "G922": ("βάρος", "G:N-N", "burden", "baros"),
        "G923": ("Βαρσαβᾶς", "N:N-M-P", "Barsabbas", "Barsabas"),
        "G924": ("Βαρτιμαῖος", "N:N-M-P", "Bartimaeus", "Bartimaios"),
        "G925": ("βαρύνω", "G:V", "to weigh down", "barunō"),
        "G926": ("βαρύς", "G:A", "weighty", "barus"),
        "G927": ("βαρύτιμος", "G:A", "expensive", "barutimos"),
        "G928": ("βασανίζω", "G:V", "to torture", "basanizō"),
        "G929": ("βασανισμός", "G:N-M", "torment", "basanismos"),
        "G930": ("βασανιστής", "G:N-M", "torturer", "basanistēs"),
        "G931": ("βάσανος", "G:N-F", "torment", "basanos"),
        "G932": ("βασιλεία", "G:N-F", "kingdom", "basileia"),
        "G933": ("βασίλειον", "G:A", "palace", "basileion"),
        "G934": ("βασίλειος", "G:A", "kingly", "basileios"),
        "G935": ("βασιλεύς", "G:N-M", "king", "basileus"),
        "G936": ("βασιλεύω", "G:V", "to reign", "basileuō"),
        "G937": ("βασιλικός", "G:A", "royal", "basilikos"),
        "G938": ("βασίλισσα", "G:N-F", "queen", "basilissa"),
        "G939": ("βάσις", "G:N-F", "foot", "basis"),
        "G940": ("βασκαίνω", "G:V", "to bewitch", "baskainō"),
        "G941": ("βαστάζω", "G:V", "to carry", "bastazō"),
        "G942": ("βάτος", "G:N-F", "thorn bush", "batos"),
        "G943": ("βάτος", "G:N-M", "bath", "batos"),
        "G944": ("βάτραχος", "G:N-M", "frog", "batrachos"),
        "G945": ("βαττολογέω", "G:V", "to babble", "battologeō"),
        "G946": ("βδέλυγμα", "G:N-N", "abomination", "bdelugma"),
        "G947": ("βδελυκτός", "G:A", "abominable", "bdeluktos"),
        "G948": ("βδελύσσομαι", "G:V", "to abhor", "bdelussomai"),
        "G949": ("βέβαιος", "G:A", "firm", "bebaios"),
        "G950": ("βεβαιόω", "G:V", "to confirm", "bebaioō"),
        "G951": ("βεβαίωσις", "G:N-F", "confirmation", "bebaiōsis"),
        "G952": ("βέβηλος", "G:A", "profane", "bebēlos"),
        "G953": ("βεβηλόω", "G:V", "to profane", "bebēloō"),
        "G954": ("Βεελζεβούλ", "N:N--T", "Beelzebub", "Beelzeboul"),
        "G955": ("Βελίαλ", "N:N--T", "Belial", "Belial"),
        "G956": ("βέλος", "G:N-N", "arrow", "belos"),
        "G957": ("βελτίων", "G:A", "better", "beltiōn"),
        "G958": ("Βενιαμίν", "N:N-M-P", "Benjamin", "Beniamin"),
        "G959": ("Βερνίκη", "N:N-F-P", "Bernice", "Bernikē"),
        "G960": ("Βέροια", "N:N--L", "Berea", "Beroia"),
        "G961": ("Βεροιαῖος", "G:A", "Berean", "Beroiaios"),
        "G962": ("Βηθαβαρά", "G:N-F", "Bethabara", "Bēthabara"),
        "G963": ("Βηθανία", "N:N-F-L", "Bethany", "Bēthania"),
        "G964": ("Βηθεσδά", "N:N--L", "Bethesda", "Bēthesda"),
        "G965": ("Βηθλεέμ", "N:N--L", "Bethlehem", "Bēthleem"),
        "G966": ("Βηθσαϊδά, Βηθζαθά", "N:N--L", "Bethsaida", "Bēthsaida"),
        "G967": ("Βηθφαγή", "N:N--L", "Bethphage", "Bēthphagē"),
        "G968": ("βῆμα", "G:N-N", "judgement seat", "bēma"),
        "G969": ("βήρυλλος", "G:N-M", "beryl", "bērullos"),
        "G970": ("βία", "G:N-F", "force", "bia"),
        "G971": ("βιάζω", "G:V", "to force", "biazō"),
        "G972": ("βίαιος", "G:A", "strong", "biaios"),
        "G973": ("βιαστής", "G:N-M", "forceful person", "biastēs"),
        "G974": ("βιβλαρίδιον", "G:N-N", "little scroll", "biblaridion"),
        "G975": ("βιβλίον", "G:N-N", "scroll", "biblion"),
        "G976": ("βίβλος", "G:N-F", "book", "biblos"),
        "G977": ("βιβρώσκω", "G:V", "to eat", "bibrōskō"),
        "G978": ("Βιθυνία", "N:N-F-L", "Bithynia", "Bithunia"),
        "G979": ("βίος", "G:N-M", "life", "bios"),
        "G980": ("βιόω", "G:V", "to live", "bioō"),
        "G981": ("βίωσις", "G:N-F", "lifestyle", "biōsis"),
        "G982": ("βιωτικός", "G:A", "of this life", "biōtikos"),
        "G983": ("βλαβερός", "G:A", "harmful", "blaberos"),
        "G984": ("βλάπτω", "G:V", "to hurt", "blaptō"),
        "G985": ("βλαστάνω", "G:V", "to sprout", "blastanō"),
        "G986": ("Βλάστος", "N:N-M-P", "Blastus", "Blastos"),
        "G987": ("βλασφημέω", "G:V", "to blaspheme", "blasphēmeō"),
        "G988": ("βλασφημία", "G:N-F", "blasphemy", "blasphēmia"),
        "G989": ("βλάσφημος", "G:A", "blasphemous", "blasphēmos"),
        "G990": ("βλέμμα", "G:N-N", "a look", "blemma"),
        "G991": ("βλέπω", "G:V", "to see", "blepō"),
        "G992": ("βλητέον", "G:A", "must be put", "blēteon"),
        "G993": ("Βοανεργές", "N:N-M-P", "Boanerges", "Boanerges"),
        "G994": ("βοάω", "G:V", "to cry out", "boaō"),
        "G995": ("βοή", "G:N-F", "outcry", "boē"),
        "G996": ("βοήθεια", "G:N-F", "help", "boētheia"),
        "G997": ("βοηθέω", "G:V", "to help", "boētheō"),
        "G998": ("βοηθός", "G:N-M", "a helper", "boēthos"),
        "G999": ("βόθυνος", "G:N-M", "pit", "bothunos"),
        "G1000": ("βολή", "G:N-F", "throwing", "bolē"),
        "G1001": ("βολίζω", "G:V", "to sound", "bolizō"),
        "G1002": ("βολίς", "G:N-F", "arrow", "bolis"),
        "G1003": ("Βοόζ", "N:N-M-P", "Boaz", "Booz"),
        "G1004": ("βόρβορος", "G:N-M", "mire", "borboros"),
        "G1005": ("βορρᾶς", "G:N-M", "the north", "borras"),
        "G1006": ("βόσκω", "G:V", "to feed", "boskō"),
        "G1007": ("Βοσόρ, Βεώρ", "N:N-M-P", "Beor", "Bosor, Beōr"),
        "G1008": ("βοτάνη", "G:N-F", "crop", "botanē"),
        "G1009": ("βότρυς", "G:N-M", "bunch of grapes", "botrus"),
        "G1010": ("βουλευτής", "G:N-M", "member of a council", "bouleutēs"),
        "G1011": ("βουλεύω", "G:V", "to plan", "bouleuō"),
        "G1012": ("βουλή", "G:N-F", "plan", "boulē"),
        "G1013": ("βούλημα", "G:N-N", "plan", "boulēma"),
        "G1014": ("βούλομαι", "G:V", "to plan", "boulomai"),
        "G1015": ("βουνός", "G:N-M", "hill", "bounos"),
        "G1016": ("βοῦς", "G:N-M", "ox", "bous"),
        "G1017": ("βραβεῖον", "G:N-N", "prize", "brabeion"),
        "G1018": ("βραβεύω", "G:V", "to rule", "brabeuō"),
        "G1019": ("βραδύνω", "G:V", "to delay", "bradunō"),
        "G1020": ("βραδυπλοέω", "G:V", "to sail slowly", "braduploeō"),
        "G1021": ("βραδύς", "G:A", "slow", "bradus"),
        "G1022": ("βραδύτης", "G:N-F", "slowness", "bradutēs"),
        "G1023": ("βραχίων", "G:N-M", "arm", "brachiōn"),
        "G1024": ("βραχύς", "G:A", "little", "brachus"),
        "G1025": ("βρέφος", "G:N-N", "infant", "brephos"),
        "G1026": ("βρέχω", "G:V", "to rain down", "brechō"),
        "G1027": ("βροντή", "G:N-F", "thunder", "brontē"),
        "G1028": ("βροχή", "G:N-F", "rain", "brochē"),
        "G1029": ("βρόχος", "G:N-M", "restraint", "brochos"),
        "G1030": ("βρυγμός", "G:N-M", "gnashing", "brugmos"),
        "G1031": ("βρύχω", "G:V", "to gnash", "bruchō"),
        "G1032": ("βρύω", "G:V", "to pour", "bruō"),
        "G1033": ("βρῶμα", "G:N-N", "food", "brōma"),
        "G1034": ("βρώσιμος", "G:A", "edible", "brōsimos"),
        "G1035": ("βρῶσις", "G:N-F", "eating", "brōsis"),
        "G1036": ("βυθίζω", "G:V", "to sink", "buthizō"),
        "G1037": ("βυθός", "G:N-M", "the deep", "buthos"),
        "G1038": ("βυρσεύς", "G:N-M", "tanner", "burseus"),
        "G1039": ("βύσσινος", "G:A", "fine linen", "bussinos"),
        "G1040": ("βύσσος", "G:N-F", "fine linen", "bussos"),
        "G1041": ("βωμός", "G:N-M", "altar", "bōmos"),
        "G1042": ("γαββαθά", "N:N--L", "Gabbatha", "gabbatha"),
        "G1043": ("Γαβριήλ", "N:N--T", "Gabriel", "Gabriēl"),
        "G1044": ("γάγγραινα", "G:N-F", "gangrene", "gaggraina"),
        "G1045": ("Γάδ", "N:N-M-P", "Gad", "Gad"),
        "G1046": ("Γαδαρηνός", "N:N--LG", "Gadarene", "Gadarēnos"),
        "G1047": ("γάζα", "G:N-F", "treasury", "gaza"),
        "G1048": ("Γάζα", "N:N-F-L", "Gazah", "Gaza"),
        "G1049": ("γαζοφυλάκιον", "G:N-N", "treasury", "gazophulakion"),
        "G1050": ("Γάϊος", "N:N-M-P", "Gaius", "Gaios"),
        "G1051": ("γάλα", "G:N-N", "milk", "gala"),
        "G1052": ("Γαλάτης", "N:N--LG", "Galatian", "Galatēs"),
        "G1053": ("Γαλατία", "N:N-F-L", "Galatia", "Galatia"),
        "G1054": ("Γαλατικός", "N:N--LG", "Galatian", "Galatikos"),
        "G1055": ("γαλήνη", "G:N-F", "calm", "galēnē"),
        "G1056": ("Γαλιλαία", "N:N-F-L", "Galilee", "Galilaia"),
        "G1057": ("Γαλιλαῖος", "N:N--LG", "Galilean", "Galilaios"),
        "G1058": ("Γαλλίων", "N:N-M-P", "Gallio", "Galliōn"),
        "G1059": ("Γαμαλιήλ", "N:N-M-P", "Gamaliel", "Gamaliēl"),
        "G1060": ("γαμέω", "G:V", "to marry", "gameō"),
        "G1061": ("γαμίσκω, γαμίζω", "G:V", "to give in marriage", "gamiskō, gamizō"),
        "G1062": ("γάμος", "G:N-M", "wedding", "gamos"),
        "G1063": ("γάρ", "G:Conj", "for", "gar"),
        "G1064": ("γαστήρ", "G:N-F", "belly", "gastēr"),
        "G1065": ("γέ", "G:Part", "indeed", "ge"),
        "G1066": ("Γεδεών", "N:N-M-P", "Gideon", "Gedeōn"),
        "G1067": ("γέεννα", "N:N--T", "Gehenna", "geenna"),
        "G1068": ("Γεθσημανῆ", "N:N--L", "Gethsemane", "Gethsēmanē"),
        "G1069": ("γείτων", "G:N-M/F", "neighbour", "geitōn"),
        "G1070": ("γελάω", "G:V", "to laugh", "gelaō"),
        "G1071": ("γέλως", "G:N-M", "laughter", "gelōs"),
        "G1072": ("γεμίζω", "G:V", "to fill", "gemizō"),
        "G1073": ("γέμω", "G:V", "be full", "gemō"),
        "G1074": ("γενεά", "G:N-F", "generation", "genea"),
        "G1075": ("γενεαλογέω", "G:V", "to trace genealogy", "genealogeō"),
        "G1076": ("γενεαλογία", "G:N-F", "genealogy", "genealogia"),
        "G1077": ("γενέσια, γενέθλιος", "G:N-N", "birthday", "genesia, genethlios"),
        "G1078": ("γένεσις", "G:N-F", "origin", "genesis"),
        "G1079": ("γενετή", "G:N-F", "birth", "genetē"),
        "G1080": ("γεννάω", "G:V", "to beget", "gennaō"),
        "G1081": ("γέννημα", "G:N-N", "offspring", "gennēma"),
        "G1082": ("Γεννησαρέτ", "N:N--L", "Gennesaret", "Gennēsaret"),
        "G1083": ("γέννησις", "G:N-F", "birth", "gennēsis"),
        "G1084": ("γεννητός", "G:A", "born", "gennētos"),
        "G1085": ("γένος", "G:N-N", "family", "genos"),
        "G1086": ("Γεργεσηνός, Γερασηνός", "N:N--LG", "Gerasene", "Gergesēnos, Gerasēnos"),
        "G1087": ("γερουσία", "G:N-F", "council of elders", "gerousia"),
        "G1088": ("γέρων", "G:N-M", "an old man", "gerōn"),
        "G1089": ("γεύω", "G:V", "to taste", "geuō"),
        "G1090": ("γεωργέω", "G:V", "to farm", "geōrgeō"),
        "G1091": ("γεώργιον", "G:N-N", "farm field", "geōrgion"),
        "G1092": ("γεωργός", "G:N-M", "farmer", "geōrgos"),
        "G1093": ("γῆ", "G:N-F", "earth", "gē"),
        "G1094": ("γῆρας", "G:N-N", "old age", "gēras"),
        "G1095": ("γηράσκω", "G:V", "to grow old", "gēraskō"),
        "G1096": ("γίνομαι", "G:V", "to be", "ginomai"),
        "G1097": ("γινώσκω", "G:V", "to know", "ginōskō"),
        "G1098": ("γλεῦκος", "G:N-N", "sweet wine", "gleukos"),
        "G1099": ("γλυκύς", "G:A", "sweet", "glukus"),
        "G1100": ("γλῶσσα", "G:N-F", "tongue", "glōssa"),
        "G1101": ("γλωσσόκομον", "G:N-N", "moneybag", "glōssokomon"),
        "G1102": ("γναφεύς", "G:N-M", "one who bleaches", "gnapheus"),
        "G1103": ("γνήσιος", "G:A", "genuine", "gnēsios"),
        "G1104": ("γνησίως", "G:Adv", "genuinely", "gnēsiōs"),
        "G1105": ("γνόφος", "G:N-M", "darkness", "gnophos"),
        "G1106": ("γνώμη", "G:N-F", "resolution", "gnōmē"),
        "G1107": ("γνωρίζω", "G:V", "to make known", "gnōrizō"),
        "G1108": ("γνῶσις", "G:N-F", "knowledge", "gnōsis"),
        "G1109": ("γνώστης", "G:N-M", "expert in", "gnōstēs"),
        "G1110": ("γνωστός, γνώριμος", "G:A", "acquainted with", "gnōstos, gnōrimos"),
        "G1111": ("γογγύζω", "G:V", "to murmur", "gogguzō"),
        "G1112": ("γογγυσμός", "G:N-M", "murmuring", "goggusmos"),
        "G1113": ("γογγυστής", "G:N-M", "a murmurer", "goggustēs"),
        "G1114": ("γόης", "G:N-M", "imposter", "goēs"),
        "G1115": ("Γολγοθᾶ", "N:N--L", "Golgotha", "Golgotha"),
        "G1116": ("Γόμορρα", "N:N--L", "Gomorrah", "Gomorra"),
        "G1117": ("γόμος", "G:N-M", "cargo", "gomos"),
        "G1118": ("γονεύς", "G:N-M", "parent", "goneus"),
        "G1119": ("γόνυ", "G:N-N", "a knee", "gonu"),
        "G1120": ("γονυπετέω", "G:V", "to kneel", "gonupeteō"),
        "G1121": ("γράμμα", "G:N-N", "something written", "gramma"),
        "G1122": ("γραμματεύς", "G:N-M", "scribe", "grammateus"),
        "G1123": ("γραπτός", "G:A", "written", "graptos"),
        "G1124": ("γραφή", "G:N-F", "a writing", "graphē"),
        "G1125": ("γράφω", "G:V", "to write", "graphō"),
        "G1126": ("γραώδης", "G:A", "old wives’ tale", "graōdēs"),
        "G1127": ("γρηγορέω", "G:V", "to keep watch", "grēgoreō"),
        "G1128": ("γυμνάζω", "G:V", "to train", "gumnazō"),
        "G1129": ("γυμνασία", "G:N-F", "training", "gumnasia"),
        "G1130": ("γυμνητεύω, γυμνιτεύω", "G:V", "be naked", "gumnēteuō, gumniteuō"),
        "G1131": ("γυμνός", "G:A", "naked", "gumnos"),
        "G1132": ("γυμνότης", "G:N-F", "nakedness", "gumnotēs"),
        "G1133": ("γυναικάριον", "G:N-N", "weak-willed woman", "gunaikarion"),
        "G1134": ("γυναικεῖος", "G:A", "feminine", "gunaikeios"),
        "G1135": ("γυνή", "G:N-F", "woman", "gunē"),
        "G1136": ("Γώγ", "N:N--L", "Gog", "Gōg"),
        "G1137": ("γωνία", "G:N-F", "corner", "gōnia"),
        "G1138": ("Δαυείδ, Δαυίδ, Δαβίδ", "N:N-M-P", "David", "Daueid, Dauid, Dabid"),
        "G1139": ("δαιμονίζομαι", "G:V", "be demonised", "daimonizomai"),
        "G1140": ("δαιμόνιον", "G:N-N", "demon", "daimonion"),
        "G1141": ("δαιμονιώδης", "G:A", "demonic", "daimoniōdēs"),
        "G1142": ("δαίμων", "G:N-M", "demon", "daimōn"),
        "G1143": ("δάκνω", "G:V", "to bite", "daknō"),
        "G1144": ("δάκρυ, δάκρυον", "G:N-N", "teardrop", "dakru, dakruon"),
        "G1145": ("δακρύω", "G:V", "to weep", "dakruō"),
        "G1146": ("δακτύλιος", "G:N-M", "ring", "daktulios"),
        "G1147": ("δάκτυλος", "G:N-M", "finger", "daktulos"),
        "G1148": ("Δαλμανουθά", "N:N--L", "Dalmanutha", "Dalmanoutha"),
        "G1149": ("Δαλματία", "N:N-F-L", "Dalmatia", "Dalmatia"),
        "G1150": ("δαμάζω", "G:V", "to tame", "damazō"),
        "G1151": ("δάμαλις", "G:N-F", "heifer", "damalis"),
        "G1152": ("Δάμαρις", "N:N-F-P", "Damaris", "Damaris"),
        "G1153": ("Δαμασκηνός", "G:A", "from Damascus", "Damaskēnos"),
        "G1154": ("Δαμασκός", "N:N-F-L", "Damascus", "Damaskos"),
        "G1155": ("δανείζω", "G:V", "to lend/borrow", "daneizō"),
        "G1156": ("δάνειον", "G:N-N", "debt", "daneion"),
        "G1157": ("δανειστής", "G:N-M", "moneylender", "daneistēs"),
        "G1158": ("Δανιήλ", "N:N-M-P", "Daniel", "Daniēl"),
        "G1159": ("δαπανάω", "G:V", "to spend", "dapanaō"),
        "G1160": ("δαπάνη", "G:N-F", "cost", "dapanē"),
        "G1161": ("δέ", "G:Conj", "then", "de"),
        "G1162": ("δέησις", "G:N-F", "prayer", "deēsis"),
        "G1163": ("δεῖ", "G:V", "be necessary", "dei"),
        "G1164": ("δεῖγμα", "G:N-N", "example", "deigma"),
        "G1165": ("δειγματίζω", "G:V", "to disgrace", "deigmatizō"),
        "G1166": ("δεικνύω", "G:V", "to show", "deiknuō"),
        "G1167": ("δειλία", "G:N-F", "timidity", "deilia"),
        "G1168": ("δειλιάω", "G:V", "be timid", "deiliaō"),
        "G1169": ("δειλός", "G:A", "timid", "deilos"),
        "G1170": ("δεῖνα", "G:N-M", "a certain one", "deina"),
        "G1171": ("δεινῶς", "G:Adv", "terribly", "deinōs"),
        "G1172": ("δειπνέω", "G:V", "to dine", "deipneō"),
        "G1173": ("δεῖπνον", "G:N-N", "dinner", "deipnon"),
        "G1174": ("δεισιδαίμων", "G:A", "religious", "deisidaimōn"),
        "G1175": ("δεισιδαιμονία", "G:N-F", "religion", "deisidaimonia"),
        "G1176": ("δέκα", "G:A", "ten", "deka"),
        "G1177": ("δεκαδύο", "G:A", "twelve", "dekaduo"),
        "G1178": ("δεκαπέντε", "G:A", "fifteen", "dekapente"),
        "G1179": ("Δεκάπολις", "N:N-F-L", "Decapolis", "Dekapolis"),
        "G1180": ("δεκατέσσαρες", "G:A", "fourteen", "dekatessares"),
        "G1181": ("δεκάτη", "G:A", "tenth", "dekatē"),
        "G1182": ("δέκατος", "G:A", "tenth", "dekatos"),
        "G1183": ("δεκατόω", "G:V", "to tithe", "dekatoō"),
        "G1184": ("δεκτός", "G:A", "acceptable", "dektos"),
        "G1185": ("δελεάζω", "G:V", "to entice", "deleazō"),
        "G1186": ("δένδρον", "G:N-N", "tree", "dendron"),
        "G1187": ("δεξιολάβος", "G:N-M", "slinger", "dexiolabos"),
        "G1188": ("δεξιός", "G:A", "right", "dexios"),
        "G1189": ("δέομαι", "G:V", "to pray", "deomai"),
        "G1190": ("Δερβαῖος", "G:A", "from Derbe", "Derbaios"),
        "G1191": ("Δέρβη", "N:N-F-L", "Derbe", "Derbē"),
        "G1192": ("δέρμα", "G:N-N", "leather", "derma"),
        "G1193": ("δερμάτινος", "G:A", "made of leather", "dermatinos"),
        "G1194": ("δέρω", "G:V", "to beat up", "derō"),
        "G1195": ("δεσμεύω", "G:V", "to bind", "desmeuō"),
        "G1196": ("δεσμέω", "G:V", "to shackle", "desmeō"),
        "G1197": ("δέσμη", "G:N-F", "bundle", "desmē"),
        "G1198": ("δέσμιος", "G:N-M", "prisoner", "desmios"),
        "G1199": ("δεσμός", "G:N-M", "chain", "desmos"),
        "G1200": ("δεσμοφύλαξ", "G:N-M", "jailer", "desmophulax"),
        "G1201": ("δεσμωτήριον", "G:N-N", "prison", "desmōtērion"),
        "G1202": ("δεσμώτης", "G:N-M", "prisoner", "desmōtēs"),
        "G1203": ("δεσπότης", "G:N-M", "master", "despotēs"),
        "G1204": ("δεῦρο", "G:Adv / G:Imperat", "come", "deuro"),
        "G1205": ("δεῦτε", "G:Adv / G:Imperat", "come", "deute"),
        "G1206": ("δευτεραῖος", "G:A", "the next day", "deuteraios"),
        "G1207": ("δευτερόπρωτος", "G:A", "second-first", "deuteroprōtos"),
        "G1208": ("δεύτερος", "G:A", "secondly", "deuteros"),
        "G1209": ("δέχομαι", "G:V", "to receive", "dechomai"),
        "G1210": ("δέω", "G:V", "to bind", "deō"),
        "G1211": ("δή", "G:Part", "so", "dē"),
        "G1212": ("δῆλος", "G:A", "clear", "dēlos"),
        "G1213": ("δηλόω", "G:V", "to make clear", "dēloō"),
        "G1214": ("Δημᾶς", "N:N-M-P", "Demas", "Dēmas"),
        "G1215": ("δημηγορέω", "G:V", "to give a speech", "dēmēgoreō"),
        "G1216": ("Δημήτριος", "N:N-M-P", "Demetrius", "Dēmētrios"),
        "G1217": ("δημιουργός", "G:N-M", "builder", "dēmiourgos"),
        "G1218": ("δῆμος", "G:N-M", "people", "dēmos"),
        "G1219": ("δημόσιος", "G:A", "public", "dēmosios"),
        "G1220": ("δηνάριον", "G:N-N", "denarius", "dēnarion"),
        "G1221": ("δήποτε", "G:Part", "whatever", "dēpote"),
        "G1222": ("δήπου", "G:Part", "surely", "dēpou"),
        "G1223": ("διά", "G:Prep", "through/because of", "dia"),
        "G1224": ("διαβαίνω", "G:V", "to cross", "diabainō"),
        "G1225": ("διαβάλλω", "G:V", "to accuse", "diaballō"),
        "G1226": ("διαβεβαιόω", "G:V", "to insist", "diabebaioō"),
        "G1227": ("διαβλέπω", "G:V", "to see clearly", "diablepō"),
        "G1228": ("διάβολος", "G:A", "devilish/the Devil", "diabolos"),
        "G1229": ("διαγγέλλω", "G:V", "to proclaim", "diaggellō"),
        "G1230": ("διαγίνομαι", "G:V", "to pass", "diaginomai"),
        "G1231": ("διαγινώσκω", "G:V", "to decide", "diaginōskō"),
        "G1232": ("διαγνωρίζω", "G:V", "to report", "diagnōrizō"),
        "G1233": ("διάγνωσις", "G:N-F", "decision", "diagnōsis"),
        "G1234": ("διαγογγύζω", "G:V", "to murmur", "diagogguzō"),
        "G1235": ("διαγρηγορέω", "G:V", "to wake", "diagrēgoreō"),
        "G1236": ("διάγω", "G:V", "to live", "diagō"),
        "G1237": ("διαδέχομαι", "G:V", "to receive in turn", "diadechomai"),
        "G1238": ("διάδημα", "G:N-N", "diadem", "diadēma"),
        "G1239": ("διαδίδωμι", "G:V", "to distribute", "diadidōmi"),
        "G1240": ("διάδοχος", "G:N-M", "successor", "diadochos"),
        "G1241": ("διαζώννυμι", "G:V", "to tie around", "diazōnnumi"),
        "G1242": ("διαθήκη", "G:N-F", "covenant", "diathēkē"),
        "G1243": ("διαίρεσις", "G:N-F", "variety", "diairesis"),
        "G1244": ("διαιρέω", "G:V", "to distribute", "diaireō"),
        "G1245": ("διακαθαρίζω", "G:V", "to clear out", "diakatharizō"),
        "G1246": ("διακατελέγχομαι", "G:V", "to refute", "diakatelegchomai"),
        "G1247": ("διακονέω", "G:V", "to serve", "diakoneō"),
        "G1248": ("διακονία", "G:N-F", "service", "diakonia"),
        "G1249": ("διάκονος", "G:N-M/F", "servant", "diakonos"),
        "G1250": ("διακόσιοι", "G:A", "two hundred", "diakosioi"),
        "G1251": ("διακούω", "G:V", "to give a hearing", "diakouō"),
        "G1252": ("διακρίνω", "G:V", "to judge/doubt", "diakrinō"),
        "G1253": ("διάκρισις", "G:N-F", "discernment", "diakrisis"),
        "G1254": ("διακωλύω", "G:V", "to hinder", "diakōluō"),
        "G1255": ("διαλαλέω", "G:V", "to discuss", "dialaleō"),
        "G1256": ("διαλέγω", "G:V", "to dispute", "dialegō"),
        "G1257": ("διαλείπω", "G:V", "to stop", "dialeipō"),
        "G1258": ("διάλεκτος", "G:N-F", "language", "dialektos"),
        "G1259": ("διαλλάσσω", "G:V", "be reconciled", "diallassō"),
        "G1260": ("διαλογίζομαι", "G:V", "to discuss", "dialogizomai"),
        "G1261": ("διαλογισμός", "G:N-M", "reasoning", "dialogismos"),
        "G1262": ("διαλύω", "G:V", "to disperse", "dialuō"),
        "G1263": ("διαμαρτύρομαι", "G:V", "to testify solemnly", "diamarturomai"),
        "G1264": ("διαμάχομαι", "G:V", "to contend sharply", "diamachomai"),
        "G1265": ("διαμένω", "G:V", "to remain", "diamenō"),
        "G1266": ("διαμερίζω", "G:V", "to divide", "diamerizō"),
        "G1267": ("διαμερισμός", "G:N-M", "division", "diamerismos"),
        "G1268": ("διανέμω", "G:V", "to spread", "dianemō"),
        "G1269": ("διανεύω", "G:V", "to signify", "dianeuō"),
        "G1270": ("διανόημα", "G:N-N", "thought", "dianoēma"),
        "G1271": ("διάνοια", "G:N-F", "mind", "dianoia"),
        "G1272": ("διανοίγω", "G:V", "to open up", "dianoigō"),
        "G1273": ("διανυκτερεύω", "G:V", "to spend the night", "dianuktereuō"),
        "G1274": ("διανύω", "G:V", "to continue", "dianuō"),
        "G1275": ("διαπαντός", "G:Adv", "always", "diapantos"),
        "G1276": ("διαπεράω", "G:V", "to cross", "diaperaō"),
        "G1277": ("διαπλέω", "G:V", "to sail across", "diapleō"),
        "G1278": ("διαπονέω", "G:V", "be vexed", "diaponeō"),
        "G1279": ("διαπορεύω", "G:V", "to go through", "diaporeuō"),
        "G1280": ("διαπορέω", "G:V", "be perplexed", "diaporeō"),
        "G1281": ("διαπραγματεύομαι", "G:V", "to gain in trade", "diapragmateuomai"),
        "G1282": ("διαπρίω", "G:V", "be furious", "diapriō"),
        "G1283": ("διαρπάζω", "G:V", "to rob", "diarpazō"),
        "G1284": ("διαρρήγνυμι", "G:V", "to tear", "diarrēgnumi"),
        "G1285": ("διασαφέω", "G:V", "to explain", "diasapheō"),
        "G1286": ("διασείω", "G:V", "to extort", "diaseiō"),
        "G1287": ("διασκορπίζω", "G:V", "to scatter", "diaskorpizō"),
        "G1288": ("διασπάω", "G:V", "to tear apart", "diaspaō"),
        "G1289": ("διασπείρω", "G:V", "to scatter", "diaspeirō"),
        "G1290": ("διασπορά", "G:N-F", "dispersion", "diaspora"),
        "G1291": ("διαστέλλω", "G:V", "to give orders", "diastellō"),
        "G1292": ("διάστημα", "G:N-N", "interval", "diastēma"),
        "G1293": ("διαστολή", "G:N-F", "distinction", "diastolē"),
        "G1294": ("διαστρέφω", "G:V", "to pervert", "diastrephō"),
        "G1295": ("διασῴζω", "G:V", "to save", "diasōzō"),
        "G1296": ("διαταγή", "G:N-F", "ordinance", "diatagē"),
        "G1297": ("διάταγμα", "G:N-N", "edict", "diatagma"),
        "G1298": ("διαταράσσω", "G:V", "to trouble", "diatarassō"),
        "G1299": ("διατάσσω", "G:V", "to direct", "diatassō"),
        "G1300": ("διατελέω", "G:V", "to continue", "diateleō"),
        "G1301": ("διατηρέω", "G:V", "to keep", "diatēreō"),
        "G1302": ("διατί", "G:Intg", "why?", "diati"),
        "G1303": ("διατίθημι", "G:V", "to make a covenant", "diatithēmi"),
        "G1304": ("διατρίβω", "G:V", "to remain", "diatribō"),
        "G1305": ("διατροφή", "G:N-F", "food", "diatrophē"),
        "G1306": ("διαυγάζω", "G:A / G:V", "to shine through", "diaugazō"),
        "G1307": ("διαφανής", "G:A", "transparent", "diaphanēs"),
        "G1308": ("διαφέρω", "G:V", "to spread/surpass", "diapherō"),
        "G1309": ("διαφεύγω", "G:V", "to escape", "diapheugō"),
        "G1310": ("διαφημίζω", "G:V", "to publish abroad", "diaphēmizō"),
        "G1311": ("διαφθείρω", "G:V", "to destroy", "diaphtheirō"),
        "G1312": ("διαφθορά", "G:N-F", "decay", "diaphthora"),
        "G1313": ("διάφορος", "G:A", "different", "diaphoros"),
        "G1314": ("διαφυλάσσω", "G:V", "to protect", "diaphulassō"),
        "G1315": ("διαχειρίζω", "G:V", "to kill", "diacheirizō"),
        "G1316": ("διαχωρίζω", "G:V", "be separated", "diachōrizō"),
        "G1317": ("διδακτικός", "G:A", "able to teach", "didaktikos"),
        "G1318": ("διδακτός", "G:A", "taught", "didaktos"),
        "G1319": ("διδασκαλία", "G:N-F", "teaching", "didaskalia"),
        "G1320": ("διδάσκαλος", "G:N-M", "teacher", "didaskalos"),
        "G1321": ("διδάσκω", "G:V", "to teach", "didaskō"),
        "G1322": ("διδαχή", "G:N-F", "teaching", "didachē"),
        "G1323": ("δίδραχμος", "G:N-N", "two-drachma", "didrachmos"),
        "G1324": ("Δίδυμος", "N:N-M-P", "Didymus", "Didumos"),
        "G1325": ("δίδωμι", "G:V", "to give", "didōmi"),
        "G1326": ("διεγείρω", "G:V", "to arouse", "diegeirō"),
        "G1327": ("διέξοδος", "G:N-F", "thoroughfare", "diexodos"),
        "G1328": ("διερμηνευτής", "G:N-M", "interpreter", "diermēneutēs"),
        "G1329": ("διερμηνεύω", "G:V", "to interpret", "diermēneuō"),
        "G1330": ("διέρχομαι", "G:V", "to pass through", "dierchomai"),
        "G1331": ("διερωτάω", "G:V", "to ascertain", "dierōtaō"),
        "G1332": ("διετής", "G:A", "two years old", "dietēs"),
        "G1333": ("διετία", "G:N-F", "two years", "dietia"),
        "G1334": ("διηγέομαι", "G:V", "to relate fully", "diēgeomai"),
        "G1335": ("διήγησις", "G:N-F", "narrative", "diēgēsis"),
        "G1336": ("διηνεκής", "G:A", "perpetual", "diēnekēs"),
        "G1337": ("διθάλασσος", "G:A", "sandbar", "dithalassos"),
        "G1338": ("διϊκνέομαι", "G:V", "to penetrate", "diikneomai"),
        "G1339": ("διΐστημι", "G:V", "to pass", "diistēmi"),
        "G1340": ("διϊσχυρίζομαι", "G:V", "to insist", "diischurizomai"),
        "G1341": ("δικαιοκρισία", "G:N-F", "justice", "dikaiokrisia"),
        "G1342": ("δίκαιος", "G:A", "just", "dikaios"),
        "G1343": ("δικαιοσύνη", "G:N-F", "righteousness", "dikaiosunē"),
        "G1344": ("δικαιόω", "G:V", "to justify", "dikaioō"),
        "G1345": ("δικαίωμα", "G:N-N", "righteous act", "dikaiōma"),
        "G1346": ("δικαίως", "G:Adv", "rightly", "dikaiōs"),
        "G1347": ("δικαίωσις", "G:N-F", "justification", "dikaiōsis"),
        "G1348": ("δικαστής", "G:N-M", "magistrate", "dikastēs"),
        "G1349": ("δίκη", "G:N-F", "condemnation", "dikē"),
        "G1350": ("δίκτυον", "G:N-N", "net", "diktuon"),
        "G1351": ("δίλογος", "G:A", "insincere", "dilogos"),
        "G1352": ("διό", "G:Conj", "therefore", "dio"),
        "G1353": ("διοδεύω", "G:V", "to go through", "diodeuō"),
        "G1354": ("Διονύσιος", "N:N-M-P", "Dionysius", "Dionusios"),
        "G1355": ("διόπερ", "G:Conj", "for", "dioper"),
        "G1356": ("διοπετής", "G:A", "fallen from heaven", "diopetēs"),
        "G1357": ("διόρθωσις", "G:N-F/N", "reformation", "diorthōsis"),
        "G1358": ("διορύσσω", "G:V", "to break in", "diorussō"),
        "G1359": ("Διόσκουροι", "N:N--T", "the twin gods", "Dioskouroi"),
        "G1360": ("διότι", "G:Conj", "because", "dioti"),
        "G1361": ("Διοτρεφής", "N:N-M-P", "Diotrephes", "Diotrephēs"),
        "G1362": ("διπλοῦς", "G:A", "double", "diplous"),
        "G1363": ("διπλόω", "G:V", "to double", "diploō"),
        "G1364": ("δίς", "G:Adv", "twice", "dis"),
        "G1365": ("διστάζω", "G:V", "to doubt", "distazō"),
        "G1366": ("δίστομος", "G:A", "double-edged", "distomos"),
        "G1367": ("δισχίλιοι", "G:A", "two thousand", "dischilioi"),
        "G1368": ("διϋλίζω", "G:V", "to strain out", "diulizō"),
        "G1369": ("διχάζω", "G:V", "to disunite", "dichazō"),
        "G1370": ("διχοστασία", "G:N-F", "dissension", "dichostasia"),
        "G1371": ("διχοτομέω", "G:V", "to cut in two", "dichotomeō"),
        "G1372": ("διψάω", "G:V", "to thirst", "dipsaō"),
        "G1373": ("δίψος", "G:N-N", "thirst", "dipsos"),
        "G1374": ("δίψυχος", "G:A", "double-minded", "dipsuchos"),
        "G1375": ("διωγμός", "G:N-M", "persecution", "diōgmos"),
        "G1376": ("διώκτης", "G:N-M", "persecutor", "diōktēs"),
        "G1377": ("διώκω", "G:V", "to pursue", "diōkō"),
        "G1378": ("δόγμα", "G:N-N", "decree", "dogma"),
        "G1379": ("δογματίζω", "G:V", "to decree", "dogmatizō"),
        "G1380": ("δοκέω", "G:V", "to think", "dokeō"),
        "G1381": ("δοκιμάζω", "G:V", "to test", "dokimazō"),
        "G1382": ("δοκιμή", "G:N-F", "test", "dokimē"),
        "G1383": ("δοκίμιον", "G:N-N", "testing", "dokimion"),
        "G1384": ("δόκιμος", "G:A", "tested", "dokimos"),
        "G1385": ("δοκός", "G:N-F", "plank", "dokos"),
        "G1386": ("δόλιος", "G:A", "deceitful", "dolios"),
        "G1387": ("δολιόω", "G:V", "to deceive", "dolioō"),
        "G1388": ("δόλος", "G:N-M", "deceit", "dolos"),
        "G1389": ("δολόω", "G:V", "to distort", "doloō"),
        "G1390": ("δόμα", "G:N-N", "gift", "doma"),
        "G1391": ("δόξα", "G:N-F", "glory", "doxa"),
        "G1392": ("δοξάζω", "G:V", "to glorify", "doxazō"),
        "G1393": ("Δορκάς", "N:N-F-P", "Dorcas", "Dorkas"),
        "G1394": ("δόσις", "G:N-F", "gift", "dosis"),
        "G1395": ("δότης", "G:N-M", "giver", "dotēs"),
        "G1396": ("δουλαγωγέω", "G:V", "to enslave", "doulagōgeō"),
        "G1397": ("δουλεία", "G:N-F", "slavery", "douleia"),
        "G1398": ("δουλεύω", "G:V", "be a slave", "douleuō"),
        "G1399": ("δούλη", "G:N-F", "female slave", "doulē"),
        "G1400": ("δοῦλον", "G:N-N", "slave", "doulon"),
        "G1401": ("δοῦλος", "G:N / G:A", "slave", "doulos"),
        "G1402": ("δουλόω", "G:V", "to enslave", "douloō"),
        "G1403": ("δοχή", "G:N-F", "banquet", "dochē"),
        "G1404": ("δράκων", "G:N-M", "dragon", "drakōn"),
        "G1405": ("δράσσομαι", "G:V", "to catch", "drassomai"),
        "G1406": ("δραχμή", "G:N-F", "drachma", "drachmē"),
        "G1407": ("δρέπανον", "G:N-N", "sickle", "drepanon"),
        "G1408": ("δρόμος", "G:N-M", "racecourse", "dromos"),
        "G1409": ("Δρούσιλλα", "N:N-F-P", "Drusilla", "Drousilla"),
        "G1410": ("δύναμαι", "G:V", "be able", "dunamai"),
        "G1411": ("δύναμις", "G:N-F", "power", "dunamis"),
        "G1412": ("δυναμόω", "G:V", "to empower", "dunamoō"),
        "G1413": ("δυνάστης", "G:N-M", "ruler", "dunastēs"),
        "G1414": ("δυνατέω", "G:V", "be able", "dunateō"),
        "G1415": ("δυνατός", "G:A", "able", "dunatos"),
        "G1416": ("δύνω, δῦμι", "G:V", "to set", "dunō, dumi"),
        "G1417": ("δύο", "G:A", "two", "duo"),
        "G1418": ("δυσ-", "G:A", "hard", "dus-"),
        "G1419": ("δυσβάστακτος", "G:A", "ponderous", "dusbastaktos"),
        "G1420": ("δυσεντερία", "G:N-F/N", "dysentery", "dusenteria"),
        "G1421": ("δυσερμήνευτος", "G:A", "hard to interpret", "dusermēneutos"),
        "G1422": ("δύσκολος", "G:A", "difficult", "duskolos"),
        "G1423": ("δυσκόλως", "G:Adv", "difficultly", "duskolōs"),
        "G1424": ("δυσμή", "G:N-F", "west", "dusmē"),
        "G1425": ("δυσνόητος", "G:A", "hard to understand", "dusnoētos"),
        "G1426": ("δυσφημία", "G:N-F", "slander", "dusphēmia"),
        "G1427": ("δώδεκα", "G:A", "twelve", "dōdeka"),
        "G1428": ("δωδέκατος", "G:A", "twelfth", "dōdekatos"),
        "G1429": ("δωδεκάφυλος", "G:N-N", "twelve tribes", "dōdekaphulos"),
        "G1430": ("δῶμα", "G:N-N", "housetop", "dōma"),
        "G1431": ("δωρεά", "G:N-F", "free gift", "dōrea"),
        "G1432": ("δωρεάν", "G:Adv", "freely", "dōrean"),
        "G1433": ("δωρέω", "G:V", "to give", "dōreō"),
        "G1434": ("δώρημα", "G:N-N", "free gift", "dōrēma"),
        "G1435": ("δῶρον", "G:N-N", "gift", "dōron"),
        "G1436": ("ἔα", "G:Intj", "ha!/aha!", "ea"),
        "G1437": ("ἐάν", "G:Cond", "if", "ean"),
        "G1438": ("ἑαυτοῦ", "G:RefP", "my/your/him-self", "heautou"),
        "G1439": ("ἐάω", "G:V", "to allow", "eaō"),
        "G1440": ("ἑβδομήκοντα", "G:A", "seventy", "hebdomēkonta"),
        "G1441": ("ἑβδομηκοντάκις", "G:Adv", "seventy times", "hebdomēkontakis"),
        "G1442": ("ἕβδομος", "G:A", "seventh", "hebdomos"),
        "G1443": ("Ἐβέρ", "N:N-M-P", "Eber", "Eber"),
        "G1444": ("Ἑβραϊκός", "N:N--LG", "Hebrew", "Hebraikos"),
        "G1445": ("Ἑβραῖος", "N:N--PG", "a Hebrew", "Hebraios"),
        "G1446": ("Ἑβραΐς", "G:A", "Aramaic", "Hebrais"),
        "G1447": ("Ἑβραϊστί", "G:Adv", "in Aramaic", "Hebraisti"),
        "G1448": ("ἐγγίζω", "G:V", "to come near", "eggizō"),
        "G1449": ("ἐγγράφω", "G:V", "to write in", "eggraphō"),
        "G1450": ("ἔγγυος", "G:A", "guarantor", "egguos"),
        "G1451": ("ἐγγύς", "G:Prep / G:Adv", "near", "eggus"),
        "G1452": ("ἐγγύτερος", "G:Adv", "nearer", "egguteros"),
        "G1453": ("ἐγείρω", "G:V", "to arise", "egeirō"),
        "G1454": ("ἔγερσις", "G:N-F", "resurrection", "egersis"),
        "G1455": ("ἐγκάθετος", "G:A", "a spy", "egkathetos"),
        "G1456": ("ἐγκαίνια", "G:N-N", "Feast of Dedication", "egkainia"),
        "G1457": ("ἐγκαινίζω", "G:V", "to inaugurate", "egkainizō"),
        "G1458": ("ἐγκαλέω", "G:V", "to accuse", "egkaleō"),
        "G1459": ("ἐγκαταλείπω", "G:V", "to leave behind", "egkataleipō"),
        "G1460": ("ἐγκατοικέω", "G:V", "to live among", "egkatoikeō"),
        "G1461": ("ἐγκεντρίζω", "G:V", "to ingraft", "egkentrizō"),
        "G1462": ("ἔγκλημα", "G:N-N", "accusation", "egklēma"),
        "G1463": ("ἐγκομβόομαι", "G:V", "to clothe oneself", "egkomboomai"),
        "G1464": ("ἐγκοπή", "G:N-F", "hindrance", "egkopē"),
        "G1465": ("ἐγκόπτω", "G:V", "to impede", "egkoptō"),
        "G1466": ("ἐγκράτεια", "G:N-F", "self-control", "egkrateia"),
        "G1467": ("ἐγκρατεύομαι", "G:V", "to self-control", "egkrateuomai"),
        "G1468": ("ἐγκρατής", "G:A", "self-controlled", "egkratēs"),
        "G1469": ("ἐγκρίνω", "G:V", "to classify", "egkrinō"),
        "G1470": ("ἐγκρύπτω", "G:V", "to mix", "egkruptō"),
        "G1471": ("ἔγκυος", "G:A", "pregnant", "egkuos"),
        "G1472": ("ἐγχρίω", "G:V", "to rub on", "egchriō"),
        "G1473": ("ἐγώ", "G:PerP", "I/we", "egō"),
        "G1474": ("ἐδαφίζω", "G:V", "to raze", "edaphizō"),
        "G1475": ("ἔδαφος", "G:N-N", "ground", "edaphos"),
        "G1476": ("ἑδραῖος", "G:A", "steadfast", "hedraios"),
        "G1477": ("ἑδραίωμα", "G:N-N", "foundation", "hedraiōma"),
        "G1478": ("Ἐζεκίας", "N:N-M-P", "Hezekiah", "Ezekias"),
        "G1479": ("ἐθελοθρησκεία", "G:N-F", "self-made religion", "ethelothrēskeia"),
        "G1480": ("ἐθίζω", "G:V", "be accustomed", "ethizō"),
        "G1481": ("ἐθνάρχης", "G:N-M", "governor", "ethnarchēs"),
        "G1482": ("ἐθνικός", "G:A", "Gentile-like", "ethnikos"),
        "G1483": ("ἐθνικῶς", "G:Adv", "Gentile-way", "ethnikōs"),
        "G1484": ("ἔθνος", "G:N-N", "Gentile nation", "ethnos"),
        "G1485": ("ἔθος", "G:N-N", "custom", "ethos"),
        "G1486": ("ἔθω", "G:V", "to have a custom", "ethō"),
        "G1487": ("εἰ", "G:Cond", "if", "ei"),
        "G1488": ("εἶ", "G:V", "you are", "ei"),
        "G1489": ("εἴγε", "G:Cond", "if indeed/otherwise", "eige"),
        "G1490": ("εἰ δὲ μή(γε)", "G:Part", "but if not", "ei de mē(ge)"),
        "G1491": ("εἶδος", "G:N-N", "appearance", "eidos"),
        "G1492": ("εἴδω", "G:V", "to perceive", "eidō"),
        "G1493": ("εἰδωλεῖον", "G:N-N", "idol's temple", "eidōleion"),
        "G1494": ("εἰδωλόθυτος", "G:A", "sacrificed to idols", "eidōlothutos"),
        "G1495": ("εἰδωλολατρεία", "G:N-F", "idolatry", "eidōlolatreia"),
        "G1496": ("εἰδωλολάτρης", "G:N-M", "idolater", "eidōlolatrēs"),
        "G1497": ("εἴδωλον", "G:N-N", "idol", "eidōlon"),
        "G1498": ("εἴην", "G:V", "may be", "eiēn"),
        "G1499": ("εἰ καί", "G:Conj", "if also", "ei kai"),
        "G1500": ("εἰκῇ", "G:Adv", "in vain", "eikē"),
        "G1501": ("εἴκοσι", "G:A", "twenty", "eikosi"),
        "G1502": ("εἴκω", "G:V", "to yield", "eikō"),
        "G1503": ("εἴκω", "G:V", "to resemble", "eikō"),
        "G1504": ("εἰκών", "G:N-F", "image", "eikōn"),
        "G1505": ("εἰλικρίνεια", "G:N-F", "sincerity", "eilikrineia"),
        "G1506": ("εἰλικρινής", "G:A", "pure", "eilikrinēs"),
        "G1507": ("εἱλίσσω", "G:V", "to roll up", "ehilissō"),
        "G1508": ("εἰ μή", "G:Cond + G:Neg", "except", "ei mē"),
        "G1509": ("εἰ μή τι", "G:Cond + G:Neg + G:Conj", "except what", "ei mē ti"),
        "G1510": ("εἰμί", "G:V", "to be", "eimi"),
        "G1511": ("εἶναι", "G:V", "to exist", "einai"),
        "G1512": ("εἴπερ", "G:Cond / G:Conj", "if so/since", "eiper"),
        "G1513": ("εἴπως", "G:Cond", "if somehow", "eipōs"),
        "G1514": ("εἰρηνεύω", "G:V", "be at peace", "eirēneuō"),
        "G1515": ("εἰρήνη", "G:N-F", "peace", "eirēnē"),
        "G1516": ("εἰρηνικός", "G:A", "peaceful", "eirēnikos"),
        "G1517": ("εἰρηνοποιέω", "G:V", "to make peace", "eirēnopoieō"),
        "G1518": ("εἰρηνοποιός", "G:A", "peacemaker", "eirēnopoios"),
        "G1519": ("εἰς", "G:Prep", "toward", "eis"),
        "G1520": ("εἷς", "G:A", "one", "ehis"),
        "G1521": ("εἰσάγω", "G:V", "to bring in", "eisagō"),
        "G1522": ("εἰσακούω", "G:V", "to listen to", "eisakouō"),
        "G1523": ("εἰσδέχομαι", "G:V", "to receive", "eisdechomai"),
        "G1524": ("εἴσειμι", "G:V", "to enter", "eiseimi"),
        "G1525": ("εἰσέρχομαι", "G:V", "to enter", "eiserchomai"),
        "G1526": ("εἰσί", "G:V", "they are", "eisi"),
        "G1527": ("εἷς καθ᾽ εἷς", "G:Adv", "one by one", "ehis kath᾽ ehis"),
        "G1528": ("εἰσκαλέω", "G:V", "to invite", "eiskaleō"),
        "G1529": ("εἴσοδος", "G:N-F", "entry", "eisodos"),
        "G1530": ("εἰσπηδάω", "G:V", "to rush into", "eispēdaō"),
        "G1531": ("εἰσπορεύω", "G:V", "to enter", "eisporeuō"),
        "G1532": ("εἰστρέχω", "G:V", "to run in", "eistrechō"),
        "G1533": ("εἰσφέρω", "G:V", "to bring in", "eispherō"),
        "G1534": ("εἶτα", "G:Adv", "then", "eita"),
        "G1535": ("εἴτε", "G:Conj", "whether", "eite"),
        "G1536": ("εἴ τις", "G:Cond", "if any", "ei tis"),
        "G1537": ("ἐκ", "G:Prep", "out from", "ek"),
        "G1538": ("ἕκαστος", "G:A", "each", "hekastos"),
        "G1539": ("ἑκάστοτε", "G:Adv", "always", "hekastote"),
        "G1540": ("ἑκατόν", "G:A", "hundred", "hekaton"),
        "G1541": ("ἑκατονταέτης", "G:A", "a hundred years old", "hekatontaetēs"),
        "G1542": ("ἑκατονταπλασίων", "G:A", "a hundred times", "hekatontaplasiōn"),
        "G1543": ("ἑκατόνταρχος", "G:N-M", "centurion", "hekatontarchos"),
        "G1544": ("ἐκβάλλω", "G:V", "to expel", "ekballō"),
        "G1545": ("ἔκβασις", "G:N-F", "way out", "ekbasis"),
        "G1546": ("ἐκβολή", "G:N-F", "jettisoning", "ekbolē"),
        "G1547": ("ἐκγαμίζω", "G:V", "to marry", "ekgamizō"),
        "G1548": ("ἐκγαμίσκω", "G:V", "to give in marriage", "ekgamiskō"),
        "G1549": ("ἔκγονος", "G:A", "descendant", "ekgonos"),
        "G1550": ("ἐκδαπανάω", "G:V", "to expend", "ekdapanaō"),
        "G1551": ("ἐκδέχομαι", "G:V", "to wait for", "ekdechomai"),
        "G1552": ("ἔκδηλος", "G:A", "clearly evident", "ekdēlos"),
        "G1553": ("ἐκδημέω", "G:V", "be away", "ekdēmeō"),
        "G1554": ("ἐκδίδωμι", "G:V", "to lease", "ekdidōmi"),
        "G1555": ("ἐκδιηγέομαι", "G:V", "to tell in detail", "ekdiēgeomai"),
        "G1556": ("ἐκδικέω", "G:V", "to avenge", "ekdikeō"),
        "G1557": ("ἐκδίκησις", "G:N-F", "vengeance", "ekdikēsis"),
        "G1558": ("ἔκδικος", "G:A", "avenging", "ekdikos"),
        "G1559": ("ἐκδιώκω", "G:V", "to persecute", "ekdiōkō"),
        "G1560": ("ἔκδοτος", "G:A", "handed over", "ekdotos"),
        "G1561": ("ἐκδοχή", "G:N-F", "expectation", "ekdochē"),
        "G1562": ("ἐκδύω", "G:V", "to strip", "ekduō"),
        "G1563": ("ἐκεῖ", "G:Adv", "there", "ekei"),
        "G1564": ("ἐκεῖθεν", "G:Adv", "from there", "ekeithen"),
        "G1565": ("ἐκεῖνος", "G:DemP", "that", "ekeinos"),
        "G1566": ("ἐκεῖσε", "G:Adv", "there", "ekeise"),
        "G1567": ("ἐκζητέω", "G:V", "to seek out", "ekzēteō"),
        "G1568": ("ἐκθαμβέω", "G:V", "be awe-struck", "ekthambeō"),
        "G1569": ("ἔκθαμβος", "G:A", "astonished", "ekthambos"),
        "G1570": ("ἔκθετος", "G:A", "exposed", "ekthetos"),
        "G1571": ("ἐκκαθαίρω", "G:V", "to cleanse", "ekkathairō"),
        "G1572": ("ἐκκαίω", "G:V", "be enflamed", "ekkaiō"),
        "G1573": ("ἐκκακέω", "G:V", "to lose heart", "ekkakeō"),
        "G1574": ("ἐκκεντέω", "G:V", "to pierce", "ekkenteō"),
        "G1575": ("ἐκκλάω", "G:V", "to break off", "ekklaō"),
        "G1576": ("ἐκκλείω", "G:V", "to exclude", "ekkleiō"),
        "G1577": ("ἐκκλησία", "G:N-F", "assembly", "ekklēsia"),
        "G1578": ("ἐκκλίνω", "G:V", "to turn from", "ekklinō"),
        "G1579": ("ἐκκολυμβάω", "G:V", "to swim out", "ekkolumbaō"),
        "G1580": ("ἐκκομίζω", "G:V", "to carry out", "ekkomizō"),
        "G1581": ("ἐκκόπτω", "G:V", "to prevent", "ekkoptō"),
        "G1582": ("ἐκκρεμάννυμι", "G:V", "to hang upon", "ekkremannumi"),
        "G1583": ("ἐκλαλέω", "G:V", "to speak out", "eklaleō"),
        "G1584": ("ἐκλάμπω", "G:V", "to shine out", "eklampō"),
        "G1585": ("ἐκλανθάνω", "G:V", "to forget", "eklanthanō"),
        "G1586": ("ἐκλέγω", "G:V", "to select", "eklegō"),
        "G1587": ("ἐκλείπω", "G:V", "to fail", "ekleipō"),
        "G1588": ("ἐκλεκτός", "G:A", "select", "eklektos"),
        "G1589": ("ἐκλογή", "G:N-F", "selecting", "eklogē"),
        "G1590": ("ἐκλύω", "G:V", "to faint", "ekluō"),
        "G1591": ("ἐκμάσσω", "G:V", "to wipe off", "ekmassō"),
        "G1592": ("ἐκμυκτηρίζω", "G:V", "to ridicule", "ekmuktērizō"),
        "G1593": ("ἐκνεύω", "G:V", "to withdraw", "ekneuō"),
        "G1594": ("ἐκνήφω", "G:V", "to wake up", "eknēphō"),
        "G1595": ("ἑκούσιος", "G:A", "voluntary", "hekousios"),
        "G1596": ("ἑκουσίως", "G:Adv", "voluntarily", "hekousiōs"),
        "G1597": ("ἔκπαλαι", "G:Adv", "of old", "ekpalai"),
        "G1598": ("ἐκπειράζω", "G:V", "to test/tempt", "ekpeirazō"),
        "G1599": ("ἐκπέμπω", "G:V", "to send away/out", "ekpempō"),
        "G1600": ("ἐκπετάζω", "G:V", "to extend", "ekpetazō"),
        "G1601": ("ἐκπίπτω", "G:V", "to fall out", "ekpiptō"),
        "G1602": ("ἐκπλέω", "G:V", "to sail out/away", "ekpleō"),
        "G1603": ("ἐκπληρόω", "G:V", "to fulfill", "ekplēroō"),
        "G1604": ("ἐκπλήρωσις", "G:N-F", "fulfillment", "ekplērōsis"),
        "G1605": ("ἐκπλήσσω", "G:V", "be astonished", "ekplēssō"),
        "G1606": ("ἐκπνέω", "G:V", "to expire", "ekpneō"),
        "G1607": ("ἐκπορεύω", "G:V", "to depart", "ekporeuō"),
        "G1608": ("ἐκπορνεύω", "G:V", "to fornicate", "ekporneuō"),
        "G1609": ("ἐκπτύω", "G:V", "to spit out", "ekptuō"),
        "G1610": ("ἐκριζόω", "G:V", "to uproot", "ekrizoō"),
        "G1611": ("ἔκστασις", "G:N-F", "amazement", "ekstasis"),
        "G1612": ("ἐκστρέφω", "G:V", "be warped", "ekstrephō"),
        "G1613": ("ἐκταράσσω", "G:V", "to disturb greatly", "ektarassō"),
        "G1614": ("ἐκτείνω", "G:V", "to stretch out", "ekteinō"),
        "G1615": ("ἐκτελέω", "G:V", "to finish up", "ekteleō"),
        "G1616": ("ἐκτένεια", "G:N-F", "earnestness", "ekteneia"),
        "G1617": ("ἐκτενέστερον", "G:A", "earnestly", "ektenesteron"),
        "G1618": ("ἐκτενής", "G:A", "fervent", "ektenēs"),
        "G1619": ("ἐκτενῶς", "G:Adv", "fervently", "ektenōs"),
        "G1620": ("ἐκτίθημι", "G:V", "to explain/expose", "ektithēmi"),
        "G1621": ("ἐκτινάσσω", "G:V", "to shake out/off", "ektinassō"),
        "G1622": ("ἐκτός", "G:Prep", "outside/except", "ektos"),
        "G1623": ("ἕκτος", "G:A", "sixth", "hektos"),
        "G1624": ("ἐκτρέπω", "G:V", "to turn/wander away", "ektrepō"),
        "G1625": ("ἐκτρέφω", "G:V", "to nourish/rear", "ektrephō"),
        "G1626": ("ἔκτρωμα", "G:N-N", "untimely birth", "ektrōma"),
        "G1627": ("ἐκφέρω", "G:V", "to bring/carry out", "ekpherō"),
        "G1628": ("ἐκφεύγω", "G:V", "to escape", "ekpheugō"),
        "G1629": ("ἐκφοβέω", "G:V", "to terrify", "ekphobeō"),
        "G1630": ("ἔκφοβος", "G:A", "terrified", "ekphobos"),
        "G1631": ("ἐκφύω", "G:V", "to put out", "ekphuō"),
        "G1632": ("ἐκχέω", "G:V", "to pour out", "ekcheō"),
        "G1633": ("ἐκχωρέω", "G:V", "to go out", "ekchōreō"),
        "G1634": ("ἐκψύχω", "G:V", "to expire", "ekpsuchō"),
        "G1635": ("ἑκών", "G:A", "voluntarily", "hekōn"),
        "G1636": ("ἐλαία", "G:N-F", "olive tree", "elaia"),
        "G1637": ("ἔλαιον", "G:N-N", "olive oil", "elaion"),
        "G1638": ("ἐλαιών", "N:N-M-L", "Olivet", "elaiōn"),
        "G1639": ("Ἐλαμίτης", "N:N--LG", "Elamite", "Elamitēs"),
        "G1640": ("ἐλάσσων", "G:A", "lesser", "elassōn"),
        "G1641": ("ἐλαττονέω", "G:V", "to have little", "elattoneō"),
        "G1642": ("ἐλαττόω", "G:V", "to make less", "elattoō"),
        "G1643": ("ἐλαύνω", "G:V", "to drive", "elaunō"),
        "G1644": ("ἐλαφρία", "G:N-F", "lightness", "elaphria"),
        "G1645": ("ἐλαφρός", "G:A", "light", "elaphros"),
        "G1646": ("ἐλάχιστος", "G:A", "least", "elachistos"),
        "G1647": ("ἐλαχιστότερος", "G:A", "least", "elachistoteros"),
        "G1648": ("Ἐλεάζαρ", "N:N-M-P", "Eleazar", "Eleazar"),
        "G1649": ("ἔλεγξις", "G:N-F", "rebuke", "elegxis"),
        "G1650": ("ἔλεγχος, ἐλεγμός", "G:N-M", "rebuke", "elegchos, elegmos"),
        "G1651": ("ἐλέγχω", "G:V", "to rebuke", "elegchō"),
        "G1652": ("ἐλεεινός", "G:A", "pitiful", "eleeinos"),
        "G1653": ("ἐλεέω, ἐλεάω", "G:V", "to have mercy", "eleeō, eleaō"),
        "G1654": ("ἐλεημοσύνη", "G:N-F", "charity", "eleēmosunē"),
        "G1655": ("ἐλεήμων", "G:A", "merciful", "eleēmōn"),
        "G1656": ("ἔλεος", "G:N-N", "mercy", "eleos"),
        "G1657": ("ἐλευθερία", "G:N-F", "freedom", "eleutheria"),
        "G1658": ("ἐλεύθερος", "G:A", "free/freedom", "eleutheros"),
        "G1659": ("ἐλευθερόω", "G:V", "to set free", "eleutheroō"),
        "G1660": ("ἔλευσις", "G:N-F", "coming", "eleusis"),
        "G1661": ("ἐλεφάντινος", "G:A", "made of ivory", "elephantinos"),
        "G1662": ("Ἐλιακείμ", "N:N-M-P", "Eliakim", "Eliakeim"),
        "G1663": ("Ἐλιέζερ", "N:N-M-P", "Eliezer", "Eliezer"),
        "G1664": ("Ἐλιούδ", "N:N-M-P", "Eliud", "Elioud"),
        "G1665": ("Ἐλισάβετ", "N:N-F-P", "Elizabeth", "Elisabet"),
        "G1666": ("Ἐλισσαῖος", "N:N-M-P", "Elisha", "Elissaios"),
        "G1667": ("ἑλίσσω", "G:N-N / G:V", "to roll up", "helissō"),
        "G1668": ("ἕλκος", "G:N-N", "sore", "helkos"),
        "G1669": ("ἑλκόω", "G:V", "to have/cause sores", "helkoō"),
        "G1670": ("ἕλκω", "G:V", "to draw/persuade", "helkuō, helkō"),
        "G1671": ("Ἑλλάς", "N:N-F-L", "Greece", "Hellas"),
        "G1672": ("Ἕλλην", "N:N--LG", "a Greek", "Hellēn"),
        "G1673": ("Ἑλληνικός", "N:N--LG", "Greek", "Hellēnikos"),
        "G1674": ("Ἑλληνίς", "N:N-F-LG", "Greek woman", "Hellēnis"),
        "G1675": ("Ἑλληνιστής", "N:N--LG", "Hellenist", "Hellēnistēs"),
        "G1676": ("Ἑλληνιστί", "G:Adv", "in Greek", "Hellēnisti"),
        "G1677": ("ἐλλογέω", "G:V", "to charge", "ellogeō"),
        "G1678": ("Ἐλμωδάμ", "N:N-M-P", "Elmadam", "Elmōdam"),
        "G1679": ("ἐλπίζω", "G:V", "to hope/expect", "elpizō"),
        "G1680": ("ἐλπίς", "G:N-F", "hope", "elpis"),
        "G1681": ("Ἐλύμας", "N:N-M-P", "Elymas", "Elumas"),
        "G1682": ("ἐλωΐ", "A:N", "my God", "elōi"),
        "G1683": ("ἐμαυτοῦ", "G:RefP", "myself", "emautou"),
        "G1684": ("ἐμβαίνω", "G:V", "to climb", "embainō"),
        "G1685": ("ἐμβάλλω", "G:V", "to throw in", "emballō"),
        "G1686": ("ἐμβάπτω", "G:V", "to dip in", "embaptō"),
        "G1687": ("ἐμβατεύω", "G:V", "to investigate", "embateuō"),
        "G1688": ("ἐμβιβάζω", "G:V", "to put on board", "embibazō"),
        "G1689": ("ἐμβλέπω", "G:V", "to look into/upon", "emblepō"),
        "G1690": ("ἐμβριμάομαι", "G:V", "be agitated", "embrimaomai"),
        "G1691": ("ἐμέ", "G:PerP", "me", "eme"),
        "G1692": ("ἐμέω", "G:V", "to vomit", "emeō"),
        "G1693": ("ἐμμαίνομαι", "G:V", "be enraged", "emmainomai"),
        "G1694": ("Ἐμμανουήλ", "N:N--T", "Immanuel", "Emmanouēl"),
        "G1695": ("Ἐμμαούς", "N:N-F-L", "Emmaus", "Emmaous"),
        "G1696": ("ἐμμένω", "G:V", "to abide in/by", "emmenō"),
        "G1697": ("Ἐμμόρ", "N:N-M-P", "Hamor", "Emmor"),
        "G1698": ("ἐμοί", "G:PerP", "I", "emoi"),
        "G1699": ("ἐμός", "G:PosP", "mine", "emos"),
        "G1700": ("ἐμοῦ", "G:PerP", "of me", "emou"),
        "G1701": ("ἐμπαιγμός", "G:N-M", "jeering", "empaigmos"),
        "G1702": ("ἐμπαίζω", "G:V", "to mock", "empaizō"),
        "G1703": ("ἐμπαίκτης", "G:N-M", "a mocker", "empaiktēs"),
        "G1704": ("ἐμπεριπατέω", "G:V", "to walk in/among", "emperipateō"),
        "G1705": ("ἐμπίμπλημι", "G:V", "to fill up", "empimplēmi"),
        "G1706": ("ἐμπίπτω", "G:V", "to fall into", "empiptō"),
        "G1707": ("ἐμπλέκω", "G:V", "to entangle/involve", "emplekō"),
        "G1708": ("ἐμπλοκή", "G:N-F", "braiding", "emplokē"),
        "G1709": ("ἐμπνέω", "G:V", "to breath into", "empneō"),
        "G1710": ("ἐμπορεύομαι", "G:V", "to traffic in", "emporeuomai"),
        "G1711": ("ἐμπορία", "G:N-F", "trade/business", "emporia"),
        "G1712": ("ἐμπόριον", "G:N-N", "marketplace/trade", "emporion"),
        "G1713": ("ἔμπορος", "G:N-M", "trader/merchant", "emporos"),
        "G1714": ("ἐμπρήθω", "G:V", "to burn", "emprēthō"),
        "G1715": ("ἔμπροσθεν", "G:Prep", "before", "emprosthen"),
        "G1716": ("ἐμπτύω", "G:V", "to spit on/at", "emptuō"),
        "G1717": ("ἐμφανής", "G:A", "revealed", "emphanēs"),
        "G1718": ("ἐμφανίζω", "G:V", "to show", "emphanizō"),
        "G1719": ("ἔμφοβος", "G:A", "afraid", "emphobos"),
        "G1720": ("ἐμφυσάω", "G:V", "to breathe into", "emphusaō"),
        "G1721": ("ἔμφυτος", "G:A", "implanted/ingrafted", "emphutos"),
        "G1722": ("ἐν", "G:Prep", "in/on/among", "en"),
        "G1723": ("ἐναγκαλίζομαι", "G:V", "to embrace", "enagkalizomai"),
        "G1724": ("ἐνάλιος", "G:A", "marine", "enalios"),
        "G1725": ("ἔναντι", "G:Prep", "before", "enanti"),
        "G1726": ("ἐναντίον", "G:Prep", "in front of", "enantion"),
        "G1727": ("ἐναντίος", "G:A", "against", "enantios"),
        "G1728": ("ἐνάρχομαι", "G:V", "to begin", "enarchomai"),
        "G1729": ("ἐνδεής", "G:A", "impoverished", "endeēs"),
        "G1730": ("ἔνδειγμα", "G:N-N", "evidence", "endeigma"),
        "G1731": ("ἐνδείκνυμι", "G:V", "to show", "endeiknumi"),
        "G1732": ("ἔνδειξις", "G:N-F", "demonstration", "endeixis"),
        "G1733": ("ἕνδεκα", "G:A", "eleven", "hendeka"),
        "G1734": ("ἑνδέκατος", "G:A", "eleventh", "hendekatos"),
        "G1735": ("ἐνδέχομαι", "G:V", "be possible", "endechomai"),
        "G1736": ("ἐνδημέω", "G:V", "be home", "endēmeō"),
        "G1737": ("ἐνδιδύσκω", "G:V", "to dress", "endiduskō"),
        "G1738": ("ἔνδικος", "G:A", "just", "endikos"),
        "G1739": ("ἐνδόμησις", "G:N-F", "material", "endomēsis"),
        "G1740": ("ἐνδοξάζομαι", "G:V", "be glorified", "endoxazomai"),
        "G1741": ("ἔνδοξος", "G:A", "honored", "endoxos"),
        "G1742": ("ἔνδυμα", "G:N-N", "clothing", "enduma"),
        "G1743": ("ἐνδυναμόω", "G:V", "to strengthen", "endunamoō"),
        "G1744": ("ἐνδύνω", "G:V", "to sneak", "endunō"),
        "G1745": ("ἔνδυσις", "G:N-F", "wearing", "endusis"),
        "G1746": ("ἐνδύω", "G:V", "to put on", "enduō"),
        "G1747": ("ἐνέδρα", "G:N-F", "plot/ambush", "enedra"),
        "G1748": ("ἐνεδρεύω", "G:V", "to ambush", "enedreuō"),
        "G1749": ("ἔνεδρον", "G:N-N", "hinderance", "enedron"),
        "G1750": ("ἐνειλέω", "G:V", "to enwrap", "eneileō"),
        "G1751": ("ἔνειμι", "G:V", "be in", "eneimi"),
        "G1752": ("ἕνεκα", "G:Prep", "because of", "heneka, ehineken"),
        "G1753": ("ἐνέργεια", "G:N-F", "active energy", "energeia"),
        "G1754": ("ἐνεργέω", "G:V", "be active", "energeō"),
        "G1755": ("ἐνέργημα", "G:N-N", "working", "energēma"),
        "G1756": ("ἐνεργής, ἐναργής", "G:A", "effective", "energēs, enargēs"),
        "G1757": ("ἐνευλογέομαι", "G:V", "be blessed", "eneulogeomai"),
        "G1758": ("ἐνέχω", "G:V", "to oppose", "enechō"),
        "G1759": ("ἐνθάδε", "G:Adv", "in/to this place", "enthade"),
        "G1760": ("ἐνθυμέομαι", "G:V", "to reflect on", "enthumeomai"),
        "G1761": ("ἐνθύμησις", "G:N-F", "reflection", "enthumēsis"),
        "G1762": ("ἔνι", "G:V", "there is", "eni"),
        "G1763": ("ἐνιαυτός", "G:N-M", "year", "eniautos"),
        "G1764": ("ἐνίστημι", "G:V", "be present", "enistēmi"),
        "G1765": ("ἐνισχύω", "G:V", "to strengthen", "enischuō"),
        "G1766": ("ἔννατος", "G:A", "ninth (hour)", "ennatos"),
        "G1767": ("ἐννέα", "G:A", "nine", "ennea"),
        "G1768": ("ἐνενήκοντα", "G:A", "ninety", "enenēkonta"),
        "G1769": ("ἐννεός", "G:A", "speechless", "enneos"),
        "G1770": ("ἐννεύω", "G:V", "to signify", "enneuō"),
        "G1771": ("ἔννοια", "G:N-F", "thought/purpose", "ennoia"),
        "G1772": ("ἔννομος", "G:A", "under law", "ennomos"),
        "G1773": ("ἔννυχος", "G:A / G:Adv", "at night", "ennuchos"),
        "G1774": ("ἐνοικέω", "G:V", "to dwell in/with", "enoikeō"),
        "G1775": ("ἑνότης", "G:N-F", "unity", "henotēs"),
        "G1776": ("ἐνοχλέω", "G:V", "to trouble", "enochleō"),
        "G1777": ("ἔνοχος", "G:A", "liable for", "enochos"),
        "G1778": ("ἔνταλμα", "G:N-N", "precept", "entalma"),
        "G1779": ("ἐνταφιάζω", "G:V", "to prepare burial", "entaphiazō"),
        "G1780": ("ἐνταφιασμός", "G:N-M", "burial", "entaphiasmos"),
        "G1781": ("ἐντέλλω", "G:V", "to order", "entellō"),
        "G1782": ("ἐντεῦθεν, ἔνθεν", "G:Adv", "from here", "enteuthen, enthen"),
        "G1783": ("ἔντευξις", "G:N-F", "intercession", "enteuxis"),
        "G1784": ("ἔντιμος", "G:A", "valued/honored", "entimos"),
        "G1785": ("ἐντολή", "G:N-F", "commandment", "entolē"),
        "G1786": ("ἐντόπιος", "G:A", "resident", "entopios"),
        "G1787": ("ἐντός", "G:Prep", "inside", "entos"),
        "G1788": ("ἐντρέπω", "G:V", "to cause shame", "entrepō"),
        "G1789": ("ἐντρέφω", "G:V", "be reared", "entrephō"),
        "G1790": ("ἔντρομος", "G:A", "trembling", "entromos"),
        "G1791": ("ἐντροπή", "G:N-F", "shame", "entropē"),
        "G1792": ("ἐντρυφάω", "G:V", "to revel", "entruphaō"),
        "G1793": ("ἐντυγχάνω", "G:V", "to call on", "entugchanō"),
        "G1794": ("ἐντυλίσσω", "G:V", "to wrap up", "entulissō"),
        "G1795": ("ἐντυπόω", "G:V", "to engrave", "entupoō"),
        "G1796": ("ἐνυβρίζω", "G:V", "to insult", "enubrizō"),
        "G1797": ("ἐνυπνιάζω", "G:V", "to dream", "enupniazō"),
        "G1798": ("ἐνύπνιον", "G:N-N", "dream", "enupnion"),
        "G1799": ("ἐνώπιον", "G:Prep", "before", "enōpion"),
        "G1800": ("Ἐνώς", "N:N-M-P", "Enosh", "Enōs"),
        "G1801": ("ἐνωτίζομαι", "G:V", "to listen to", "enōtizomai"),
        "G1802": ("Ἐνώχ", "N:N-M-P", "Enoch", "Enōch"),
        "G1803": ("ἕξ", "G:A", "six", "hex"),
        "G1804": ("ἐξαγγέλλω", "G:V", "to proclaim", "exaggellō"),
        "G1805": ("ἐξαγοράζω", "G:V", "to redeem", "exagorazō"),
        "G1806": ("ἐξάγω", "G:V", "to lead out", "exagō"),
        "G1807": ("ἐξαιρέω", "G:V", "to take out/select", "exaireō"),
        "G1808": ("ἐξαίρω", "G:V", "to expel", "exairō"),
        "G1809": ("ἐξαιτέω", "G:V", "to demand/ask for", "exaiteō"),
        "G1810": ("ἐξαίφνης", "G:Adv", "suddenly", "exaiphnēs"),
        "G1811": ("ἐξακολουθέω", "G:V", "to follow", "exakoloutheō"),
        "G1812": ("ἑξακόσιοι", "G:A", "six hundred", "hexakosioi"),
        "G1813": ("ἐξαλείφω", "G:V", "to blot out", "exaleiphō"),
        "G1814": ("ἐξάλλομαι", "G:V", "to leap up", "exallomai"),
        "G1815": ("ἐξανάστασις", "G:N-F", "resurrection", "exanastasis"),
        "G1816": ("ἐξανατέλλω", "G:V", "to sprout up", "exanatellō"),
        "G1817": ("ἐξανίστημι", "G:V", "to raise up", "exanistēmi"),
        "G1818": ("ἐξαπατάω", "G:V", "to deceive", "exapataō"),
        "G1819": ("ἐξάπινα", "G:Adv", "suddenly", "exapina"),
        "G1820": ("ἐξαπορέω", "G:V", "to despair", "exaporeō"),
        "G1821": ("ἐξαποστέλλω", "G:V", "to send out/away", "exapostellō"),
        "G1822": ("ἐξαρτίζω", "G:V", "to finish/furnish", "exartizō"),
        "G1823": ("ἐξαστράπτω", "G:V", "to flash forth", "exastraptō"),
        "G1824": ("ἐξαυτῆς", "G:Adv", "immediately", "exautēs"),
        "G1825": ("ἐξεγείρω", "G:V", "to raise", "exegeirō"),
        "G1826": ("ἔξειμι", "G:V", "to go out/away", "exeimi"),
        "G1827": ("ἐξελέγχω", "G:V", "to convict", "exelegchō"),
        "G1828": ("ἐξέλκω", "G:V", "to drag out/away", "exelkō"),
        "G1829": ("ἐξέραμα", "G:N-N", "vomit", "exerama"),
        "G1830": ("ἐξερευνάω", "G:V", "to search out", "exereunaō"),
        "G1831": ("ἐξέρχομαι", "G:V", "to go out", "exerchomai"),
        "G1832": ("ἔξεστι, ἐξόν", "G:V", "be permitted", "exesti, exon"),
        "G1833": ("ἐξετάζω", "G:V", "to find out", "exetazō"),
        "G1834": ("ἐξηγέομαι", "G:V", "to tell", "exēgeomai"),
        "G1835": ("ἑξήκοντα", "G:A", "sixty", "hexēkonta"),
        "G1836": ("ἑξῆς", "G:Adv", "next/afterward", "hexēs"),
        "G1837": ("ἐξηχέω", "G:V", "to ring/sound out", "exēcheō"),
        "G1838": ("ἕξις", "G:N-F", "habit/practice", "hexis"),
        "G1839": ("ἐξίστημι", "G:V", "to amaze", "existēmi"),
        "G1840": ("ἐξισχύω", "G:V", "to have power", "exischuō"),
        "G1841": ("ἔξοδος", "G:N-F", "departure", "exodos"),
        "G1842": ("ἐξολοθρεύω", "G:V", "to root out", "exolothreuō"),
        "G1843": ("ἐξομολογέομαι", "G:V", "to agree", "exomologeomai"),
        "G1844": ("ἐξορκίζω", "G:V", "to adjure", "exorkizō"),
        "G1845": ("ἐξορκιστής", "G:N-M", "exorcist", "exorkistēs"),
        "G1846": ("ἐξορύσσω", "G:V", "to dig through/out", "exorussō"),
        "G1847": ("ἐξουδενόω", "G:V", "be rejected", "exoudenoō"),
        "G1848": ("ἐξουθενέω", "G:V", "to reject", "exoutheneō"),
        "G1849": ("ἐξουσία", "G:N-F", "authority", "exousia"),
        "G1850": ("ἐξουσιάζω", "G:V", "to have authority", "exousiazō"),
        "G1851": ("ἐξοχή", "G:N-F", "prominent", "exochē"),
        "G1852": ("ἐξυπνίζω", "G:V", "to awake", "exupnizō"),
        "G1853": ("ἔξυπνος", "G:A", "awake", "exupnos"),
        "G1854": ("ἔξω", "G:Prep", "out/outside(r)", "exō"),
        "G1855": ("ἔξωθεν", "G:Prep", "outside", "exōthen"),
        "G1856": ("ἐξωθέω", "G:V", "to expel", "exōtheō"),
        "G1857": ("ἐξώτερος", "G:A", "outer", "exōteros"),
        "G1858": ("ἑορτάζω", "G:V", "to keep a festival", "heortazō"),
        "G1859": ("ἑορτή", "G:N-F", "festival", "heortē"),
        "G1860": ("ἐπαγγελία", "G:N-F", "promise", "epaggelia"),
        "G1861": ("ἐπαγγέλλω", "G:V", "to profess", "epaggellō"),
        "G1862": ("ἐπάγγελμα", "G:N-N", "a promise", "epaggelma"),
        "G1863": ("ἐπάγω", "G:V", "to bring upon", "epagō"),
        "G1864": ("ἐπαγωνίζομαι", "G:V", "to contend", "epagōnizomai"),
        "G1865": ("ἐπαθροίζομαι", "G:V", "to collect/crowd", "epathroizomai"),
        "G1866": ("Ἐπαίνετος", "N:N-M-P", "Epaenetus", "Epainetos"),
        "G1867": ("ἐπαινέω", "G:V", "to praise", "epaineō"),
        "G1868": ("ἔπαινος", "G:N-M", "praise", "epainos"),
        "G1869": ("ἐπαίρω", "G:V", "to lift up", "epairō"),
        "G1870": ("ἐπαισχύνομαι", "G:V", "be ashamed of", "epaischunomai"),
        "G1871": ("ἐπαιτέω", "G:V", "to ask/beg", "epaiteō"),
        "G1872": ("ἐπακολουθέω", "G:V", "to follow after", "epakoloutheō"),
        "G1873": ("ἐπακούω", "G:V", "to listen to", "epakouō"),
        "G1874": ("ἐπακροάομαι", "G:V", "to listen ro", "epakroaomai"),
        "G1875": ("ἐπάν", "G:Conj", "when/as soon as", "epan"),
        "G1876": ("ἐπάναγκες", "G:Adv", "necessarily", "epanagkes"),
        "G1877": ("ἐπανάγω", "G:V", "to set off", "epanagō"),
        "G1878": ("ἐπαναμιμνῄσκω", "G:V", "to remind", "epanamimnēskō"),
        "G1879": ("ἐπαναπαύω", "G:V", "to rest/rely on", "epanapauō"),
        "G1880": ("ἐπανέρχομαι", "G:V", "to return", "epanerchomai"),
        "G1881": ("ἐπανίστημι", "G:V", "to rebel against", "epanistēmi"),
        "G1882": ("ἐπανόρθωσις", "G:N-F", "correcting", "epanorthōsis"),
        "G1883": ("ἐπάνω", "G:Prep", "above", "epanō"),
        "G1884": ("ἐπαρκέω", "G:V", "to help", "eparkeō"),
        "G1885": ("ἐπαρχία", "G:N-F", "province", "eparchia"),
        "G1886": ("ἔπαυλις", "G:N-F", "residence", "epaulis"),
        "G1887": ("ἐπαύριον", "G:Adv", "the next day", "epaurion"),
        "G1888": ("ἐπαυτόφωρος, αὐτόφωρος", "G:A / G:Adv", "in the very act", "epautophōros, autophōros"),
        "G1889": ("Ἐπαφρᾶς", "N:N-M-P", "Epaphras", "Epaphras"),
        "G1890": ("ἐπαφρίζω", "G:V", "to foam up", "epaphrizō"),
        "G1891": ("Ἐπαφρόδιτος", "N:N-M-P", "Epaphroditus", "Epaphroditos"),
        "G1892": ("ἐπεγείρω", "G:V", "to awaken/rouse", "epegeirō"),
        "G1893": ("ἐπεί", "G:Conj", "since", "epei"),
        "G1894": ("ἐπειδή", "G:Conj", "since", "epeidē"),
        "G1895": ("ἐπειδήπερ", "G:Conj", "since", "epeidēper"),
        "G1896": ("ἐπεῖδον", "G:V", "to look upon", "epeidon"),
        "G1897": ("ἐπείπερ", "G:Conj", "since", "epeiper"),
        "G1898": ("ἐπεισαγωγή", "G:N-F", "introduction", "epeisagōgē"),
        "G1899": ("ἔπειτα", "G:Adv", "then", "epeita"),
        "G1900": ("ἐπέκεινα", "G:Prep", "beyond", "epekeina"),
        "G1901": ("ἐπεκτείνομαι", "G:V", "to reach out to", "epekteinomai"),
        "G1902": ("ἐπενδύω", "G:V", "be clothed", "ependuō"),
        "G1903": ("ἐπενδύτης", "G:N-M", "coat", "ependutēs"),
        "G1904": ("ἐπέρχομαι", "G:V", "to arrive/invade", "eperchomai"),
        "G1905": ("ἐπερωτάω", "G:V", "to question", "eperōtaō"),
        "G1906": ("ἐπερώτημα", "G:N-N", "pledge", "eperōtēma"),
        "G1907": ("ἐπέχω", "G:V", "to hold fast/out", "epechō"),
        "G1908": ("ἐπηρεάζω", "G:V", "to mistreat", "epēreazō"),
        "G1909": ("ἐπί", "G:Prep", "upon/to/against", "epi"),
        "G1910": ("ἐπιβαίνω", "G:V", "to mount/board", "epibainō"),
        "G1911": ("ἐπιβάλλω", "G:V", "to put on/seize", "epiballō"),
        "G1912": ("ἐπιβαρέω", "G:V", "to burden", "epibareō"),
        "G1913": ("ἐπιβιβάζω", "G:V", "to mount", "epibibazō"),
        "G1914": ("ἐπιβλέπω", "G:V", "to look upon/at", "epiblepō"),
        "G1915": ("ἐπίβλημα", "G:N-N", "patch", "epiblēma"),
        "G1916": ("ἐπιβοάω", "G:V", "to cry out", "epiboaō"),
        "G1917": ("ἐπιβουλή", "G:N-F", "plot", "epiboulē"),
        "G1918": ("ἐπιγαμβρεύω", "G:V", "to marry", "epigambreuō"),
        "G1919": ("ἐπίγειος", "G:A", "earthly", "epigeios"),
        "G1920": ("ἐπιγίνομαι", "G:V", "to arise/spring up", "epiginomai"),
        "G1921": ("ἐπιγινώσκω", "G:V", "to come to know", "epiginōskō"),
        "G1922": ("ἐπίγνωσις", "G:N-F", "knowledge", "epignōsis"),
        "G1923": ("ἐπιγραφή", "G:N-F", "inscription", "epigraphē"),
        "G1924": ("ἐπιγράφω", "G:V", "to write on", "epigraphō"),
        "G1925": ("ἐπιδείκνυμι", "G:V", "to show/prove", "epideiknumi"),
        "G1926": ("ἐπιδέχομαι", "G:V", "to welcome", "epidechomai"),
        "G1927": ("ἐπιδημέω", "G:V", "to sojourn", "epidēmeō"),
        "G1928": ("ἐπιδιατάσσομαι", "G:V", "to add to", "epidiatassomai"),
        "G1929": ("ἐπιδίδωμι", "G:V", "to give/deliver", "epididōmi"),
        "G1930": ("ἐπιδιορθόω", "G:V", "to straighten out", "epidiorthoō"),
        "G1931": ("ἐπιδύω", "G:V", "to set", "epiduō"),
        "G1932": ("ἐπιείκεια", "G:N-F", "gentleness", "epieikeia"),
        "G1933": ("ἐπιεικής", "G:A", "gentle", "epieikēs"),
        "G1934": ("ἐπιζητέω", "G:V", "to seek after", "epizēteō"),
        "G1935": ("ἐπιθανάτιος", "G:A", "condemned to death", "epithanatios"),
        "G1936": ("ἐπίθεσις", "G:N-F", "laying on", "epithesis"),
        "G1937": ("ἐπιθυμέω", "G:V", "to long for", "epithumeō"),
        "G1938": ("ἐπιθυμητής", "G:N-M", "one who desires", "epithumētēs"),
        "G1939": ("ἐπιθυμία", "G:N-F", "desire", "epithumia"),
        "G1940": ("ἐπικαθίζω", "G:V", "to sit on", "epikathizō"),
        "G1941": ("ἐπικαλέω", "G:V", "to call (on)/name", "epikaleō"),
        "G1942": ("ἐπικάλυμμα", "G:N-N", "covering", "epikalumma"),
        "G1943": ("ἐπικαλύπτω", "G:V", "to cover", "epikaluptō"),
        "G1944": ("ἐπικατάρατος", "G:A", "cursed", "epikataratos"),
        "G1945": ("ἐπίκειμαι", "G:V", "to lay on", "epikeimai"),
        "G1946": ("Ἐπικούρειος", "N:N--T", "Epicurean", "Epikoureios"),
        "G1947": ("ἐπικουρία", "G:N-F", "help", "epikouria"),
        "G1948": ("ἐπικρίνω", "G:V", "to decide", "epikrinō"),
        "G1949": ("ἐπιλαμβάνω", "G:V", "to catch", "epilambanō"),
        "G1950": ("ἐπιλανθάνω", "G:V", "to forget", "epilanthanō"),
        "G1951": ("ἐπιλέγω", "G:V", "to call/choose", "epilegō"),
        "G1952": ("ἐπιλείπω", "G:V", "be insufficient", "epileipō"),
        "G1953": ("ἐπιλησμονή", "G:N-F", "forgetfulness", "epilēsmonē"),
        "G1954": ("ἐπίλοιπος", "G:A", "remaining", "epiloipos"),
        "G1955": ("ἐπίλυσις", "G:N-F", "explanation", "epilusis"),
        "G1956": ("ἐπιλύω", "G:V", "to explain", "epiluō"),
        "G1957": ("ἐπιμαρτυρέω", "G:V", "to testify to", "epimartureō"),
        "G1958": ("ἐπιμέλεια", "G:N-F", "care", "epimeleia"),
        "G1959": ("ἐπιμελέομαι", "G:V", "to care", "epimeleomai"),
        "G1960": ("ἐπιμελῶς", "G:Adv", "carefully", "epimelōs"),
        "G1961": ("ἐπιμένω", "G:V", "to remain/keep on", "epimenō"),
        "G1962": ("ἐπινεύω", "G:V", "to accept", "epineuō"),
        "G1963": ("ἐπίνοια", "G:N-F", "thought", "epinoia"),
        "G1964": ("ἐπιορκέω", "G:V", "to break an oath", "epiorkeō"),
        "G1965": ("ἐπίορκος", "G:A", "perjurer", "epiorkos"),
        "G1966": ("ἐπιοῦσα", "G:V", "to come later", "epiousa"),
        "G1967": ("ἐπιούσιος", "G:A", "daily", "epiousios"),
        "G1968": ("ἐπιπίπτω", "G:V", "to fall/press upon", "epipiptō"),
        "G1969": ("ἐπιπλήσσω", "G:V", "to rebuke", "epiplēssō"),
        "G1970": ("ἐπιπνίγω", "G:V", "to choke", "epipnigō"),
        "G1971": ("ἐπιποθέω", "G:V", "to long for", "epipotheō"),
        "G1972": ("ἐπιπόθησις", "G:N-F", "longing", "epipothēsis"),
        "G1973": ("ἐπιπόθητος", "G:A", "longed for", "epipothētos"),
        "G1974": ("ἐπιποθία", "G:N-F", "longing", "epipothia"),
        "G1975": ("ἐπιπορεύομαι", "G:V", "to come/go to", "epiporeuomai"),
        "G1976": ("ἐπιρράπτω", "G:V", "to sew on", "epirraptō"),
        "G1977": ("ἐπιρρίπτω", "G:V", "to throw on", "epirriptō"),
        "G1978": ("ἐπίσημος", "G:A", "notable", "episēmos"),
        "G1979": ("ἐπισιτισμός", "G:N-M", "food", "episitismos"),
        "G1980": ("ἐπισκέπτομαι", "G:V", "to visit/care for", "episkeptomai"),
        "G1981": ("ἐπισκηνόω", "G:V", "to rest upon/dwell", "episkēnoō"),
        "G1982": ("ἐπισκιάζω", "G:V", "to overshadow", "episkiazō"),
        "G1983": ("ἐπισκοπέω", "G:V", "to oversee/care for", "episkopeō"),
        "G1984": ("ἐπισκοπή", "G:N-F", "oversight", "episkopē"),
        "G1985": ("ἐπίσκοπος", "G:N-M", "overseer", "episkopos"),
        "G1986": ("ἐπισπάω", "G:V", "to uncircumcise", "epispaō"),
        "G1987": ("ἐπίσταμαι", "G:V", "to know/understand", "epistamai"),
        "G1988": ("ἐπιστάτης", "G:N-M", "master", "epistatēs"),
        "G1989": ("ἐπιστέλλω", "G:V", "to write to", "epistellō"),
        "G1990": ("ἐπιστήμων", "G:A", "knowing", "epistēmōn"),
        "G1991": ("ἐπιστηρίζω", "G:V", "to strengthen", "epistērizō"),
        "G1992": ("ἐπιστολή", "G:N-F", "epistle", "epistolē"),
        "G1993": ("ἐπιστομίζω", "G:V", "to silence", "epistomizō"),
        "G1994": ("ἐπιστρέφω", "G:V", "to turn", "epistrephō"),
        "G1995": ("ἐπιστροφή", "G:N-F", "conversion", "epistrophē"),
        "G1996": ("ἐπισυνάγω", "G:V", "to gather", "episunagō"),
        "G1997": ("ἐπισυναγωγή", "G:N-F", "gathering", "episunagōgē"),
        "G1998": ("ἐπισυντρέχω", "G:V", "to run together", "episuntrechō"),
        "G1999": ("ἐπισύστασις, ἐπίστασις", "G:N-F", "insurrection", "episustasis, epistasis"),
        "G2000": ("ἐπισφαλής", "G:A", "dangerous/unsteady", "episphalēs"),
        "G2001": ("ἐπισχύω", "G:V", "to insist", "epischuō"),
        "G2002": ("ἐπισωρεύω", "G:V", "to accumulate", "episōreuō"),
        "G2003": ("ἐπιταγή", "G:N-F", "command", "epitagē"),
        "G2004": ("ἐπιτάσσω", "G:V", "to command", "epitassō"),
        "G2005": ("ἐπιτελέω", "G:V", "to complete", "epiteleō"),
        "G2006": ("ἐπιτήδειος", "G:A", "necessary", "epitēdeios"),
        "G2007": ("ἐπιτίθημι", "G:V", "to put/lay on", "epitithēmi"),
        "G2008": ("ἐπιτιμάω", "G:V", "to rebuke", "epitimaō"),
        "G2009": ("ἐπιτιμία", "G:N-F", "punishment", "epitimia"),
        "G2010": ("ἐπιτρέπω", "G:V", "to permit", "epitrepō"),
        "G2011": ("ἐπιτροπή", "G:N-F", "commission", "epitropē"),
        "G2012": ("ἐπίτροπος", "G:N-M", "manager", "epitropos"),
        "G2013": ("ἐπιτυγχάνω", "G:V", "to obtain", "epitugchanō"),
        "G2014": ("ἐπιφαίνω", "G:V", "to appear", "epiphainō"),
        "G2015": ("ἐπιφάνεια", "G:N-F", "appearing", "epiphaneia"),
        "G2016": ("ἐπιφανής", "G:A", "glorious", "epiphanēs"),
        "G2017": ("ἐπιφαύσκω", "G:V", "to shine on", "epiphauskō"),
        "G2018": ("ἐπιφέρω", "G:V", "to inflict", "epipherō"),
        "G2019": ("ἐπιφωνέω", "G:V", "to shout", "epiphōneō"),
        "G2020": ("ἐπιφώσκω", "G:V", "to dawn", "epiphōskō"),
        "G2021": ("ἐπιχειρέω", "G:V", "to attempt", "epicheireō"),
        "G2022": ("ἐπιχέω", "G:V", "to pour on/over", "epicheō"),
        "G2023": ("ἐπιχορηγέω", "G:V", "to supply", "epichorēgeō"),
        "G2024": ("ἐπιχορηγία", "G:N-F", "supply", "epichorēgia"),
        "G2025": ("ἐπιχρίω", "G:V", "to rub on", "epichriō"),
        "G2026": ("ἐποικοδομέω", "G:V", "to build up/upon", "epoikodomeō"),
        "G2027": ("ἐποκέλλω", "G:V", "to run aground", "epokellō"),
        "G2028": ("ἐπονομάζω", "G:V", "to name", "eponomazō"),
        "G2029": ("ἐποπτεύω", "G:V", "to observe", "epopteuō"),
        "G2030": ("ἐπόπτης", "G:N-M", "eyewitness", "epoptēs"),
        "G2031": ("ἔπος", "G:N-N", "a word", "epos"),
        "G2032": ("ἐπουράνιος", "G:A", "heavenly", "epouranios"),
        "G2033": ("ἑπτά", "G:A", "seven", "hepta"),
        "G2034": ("ἑπτάκις", "G:Adv", "seven times", "heptakis"),
        "G2035": ("ἑπτακισχίλιοι", "G:A", "seven thousand", "heptakischilioi"),
        "G2036": ("ἔπω, ἐρῶ, εἶπον", "G:V", "to say", "epō, erō, eipon"),
        "G2037": ("Ἔραστος", "N:N-M-P", "Erastus", "erastos"),
        "G2038": ("ἐργάζομαι", "G:V", "to work", "ergazomai"),
        "G2039": ("ἐργασία", "G:N-F", "work", "ergasia"),
        "G2040": ("ἐργάτης", "G:N-M", "worker", "ergatēs"),
        "G2041": ("ἔργον", "G:N-N", "work", "ergon"),
        "G2042": ("ἐρεθίζω", "G:V", "to provoke/irritate", "erethizō"),
        "G2043": ("ἐρείδω", "G:V", "to immobilise", "ereidō"),
        "G2044": ("ἐρεύγομαι", "G:V", "to utter/proclaim", "ereugomai"),
        "G2045": ("ἐρευνάω", "G:V", "to look for/into", "ereunaō"),
        "G2046": ("ἐρέω, ἐρῶ", "G:V", "to speak", "ereō, erō"),
        "G2047": ("ἐρημία", "G:N-F", "desert", "erēmia"),
        "G2048": ("ἔρημος", "G:A", "deserted", "erēmos"),
        "G2049": ("ἐρημόω", "G:V", "to lay waste", "erēmoō"),
        "G2050": ("ἐρήμωσις", "G:N-F", "devastation", "erēmōsis"),
        "G2051": ("ἐρίζω", "G:V", "to quarrel", "erizō"),
        "G2052": ("ἐριθεία", "G:N-F", "rivalry", "eritheia"),
        "G2053": ("ἔριον", "G:N-N", "wool", "erion"),
        "G2054": ("ἔρις", "G:N-F", "quarrel", "eris"),
        "G2055": ("ἐρίφιον", "G:N-N", "a goat", "eriphion"),
        "G2056": ("ἔριφος", "G:N-M", "a goat", "eriphos"),
        "G2057": ("Ἑρμᾶς", "N:N-M-P", "Hermas", "Hermas"),
        "G2058": ("ἑρμηνεία", "G:N-F", "interpretation", "hermēneia"),
        "G2059": ("ἑρμηνεύω", "G:V", "to interpret", "hermēneuō"),
        "G2060": ("Ἑρμῆς", "N:N-M-T", "Hermes", "Hermēs"),
        "G2061": ("Ἑρμογένης", "N:N-M-P", "Hermogenes", "Hermogenēs"),
        "G2062": ("ἑρπετόν", "G:N-N", "reptile", "herpeton"),
        "G2063": ("ἐρυθρός", "G:A", "red", "eruthros"),
        "G2064": ("ἔρχομαι", "G:V", "to come/go", "erchomai"),
        "G2065": ("ἐρωτάω", "G:V", "to ask", "erōtaō"),
        "G2066": ("ἐσθής", "G:N-F", "clothing", "esthēs"),
        "G2067": ("ἔσθησις", "G:N-F", "clothing", "esthēsis"),
        "G2068": ("ἐσθίω", "G:V", "to eat", "esthiō"),
        "G2069": ("Ἐσλί", "N:N-M-P", "Esli", "Esli"),
        "G2070": ("ἐσμέν", "G:V", "we are", "esmen"),
        "G2071": ("ἔσομαι", "G:V", "will be", "esomai"),
        "G2072": ("ἔσοπτρον", "G:N-N", "mirror", "esoptron"),
        "G2073": ("ἑσπέρα", "G:N-F", "evening", "hespera"),
        "G2074": ("Ἐσρώμ", "N:N-M-P", "Hezron", "Esrōm"),
        "G2075": ("ἐστέ", "G:V", "you are", "este"),
        "G2076": ("ἐστί", "G:V", "he is", "esti"),
        "G2077": ("ἔστω", "G:V", "to be", "estō"),
        "G2078": ("ἔσχατος", "G:A", "last/least", "eschatos"),
        "G2079": ("ἐσχάτως", "G:Adv", "extremely", "eschatōs"),
        "G2080": ("ἔσω", "G:Adv", "in/inner/inwardly", "esō"),
        "G2081": ("ἔσωθεν", "G:Adv", "inwardly", "esōthen"),
        "G2082": ("ἐσωτέρω", "G:A", "inner", "esōterō"),
        "G2083": ("ἑταῖρος", "G:N-M", "friend", "hetairos"),
        "G2084": ("ἑτερόγλωσσος", "G:A", "in another language", "heteroglōssos"),
        "G2085": ("ἑτεροδιδασκαλέω", "G:V", "to teach heresy", "heterodidaskaleō"),
        "G2086": ("ἑτεροζυγέω", "G:V", "to yoke a mismatch", "heterozugeō"),
        "G2087": ("ἕτερος", "G:A", "other", "heteros"),
        "G2088": ("ἑτέρως", "G:Adv", "differently", "heterōs"),
        "G2089": ("ἔτι", "G:Adv", "still", "eti"),
        "G2090": ("ἑτοιμάζω", "G:V", "to make ready", "hetoimazō"),
        "G2091": ("ἑτοιμασία", "G:N-F", "readiness", "hetoimasia"),
        "G2092": ("ἕτοιμος", "G:A", "ready", "hetoimos"),
        "G2093": ("ἑτοίμως", "G:Adv", "readily", "hetoimōs"),
        "G2094": ("ἔτος", "G:N-N", "year", "etos"),
        "G2095": ("εὖ", "G:Adv", "well/well done!", "eu"),
        "G2096": ("Εὖα", "N:N-F-P", "Eve", "Eua"),
        "G2097": ("εὐαγγελίζομαι", "G:V", "to speak good news", "euaggelizomai"),
        "G2098": ("εὐαγγέλιον", "G:N-N", "gospel", "euaggelion"),
        "G2099": ("εὐαγγελιστής", "G:N-M", "evangelist", "euaggelistēs"),
        "G2100": ("εὐαρεστέω", "G:V", "to please", "euaresteō"),
        "G2101": ("εὐάρεστος", "G:A", "well-pleasing", "euarestos"),
        "G2102": ("εὐαρέστως", "G:Adv", "acceptably", "euarestōs"),
        "G2103": ("Εὔβουλος", "N:N-M-P", "Eubulus", "Euboulos"),
        "G2104": ("εὐγενής", "G:A", "of noble birth", "eugenēs"),
        "G2105": ("εὐδία", "G:N-F", "fair weather", "eudia"),
        "G2106": ("εὐδοκέω", "G:V", "to delight", "eudokeō"),
        "G2107": ("εὐδοκία", "G:N-F", "goodwill", "eudokia"),
        "G2108": ("εὐεργεσία", "G:N-F", "good deed", "euergesia"),
        "G2109": ("εὐεργετέω", "G:V", "to do good", "euergeteō"),
        "G2110": ("εὐεργέτης", "G:N-M", "benefactor", "euergetēs"),
        "G2111": ("εὔθετος", "G:A", "suitable", "euthetos"),
        "G2112": ("εὐθέως", "G:Adv", "immediately", "eutheōs"),
        "G2113": ("εὐθυδρομέω", "G:V", "to sail straight", "euthudromeō"),
        "G2114": ("εὐθυμέω", "G:V", "be cheerful", "euthumeō"),
        "G2115": ("εὔθυμος", "G:A / G:Adv", "cheerful", "euthumos"),
        "G2116": ("εὐθύνω", "G:V", "to straighten", "euthunō"),
        "G2117": ("εὐθύς", "G:A", "straight", "euthus"),
        "G2118": ("εὐθύτης", "G:N-F", "righteousness", "euthutēs"),
        "G2119": ("εὐκαιρέω", "G:V", "to have opportunity", "eukaireō"),
        "G2120": ("εὐκαιρία", "G:N-F", "opportunity", "eukairia"),
        "G2121": ("εὔκαιρος", "G:A", "opportune", "eukairos"),
        "G2122": ("εὐκαίρως", "G:Adv", "well timed", "eukairōs"),
        "G2123": ("εὐκοπώτερος", "G:A", "easy", "eukopōteros"),
        "G2124": ("εὐλάβεια", "G:N-F", "reverence", "eulabeia"),
        "G2125": ("εὐλαβέομαι", "G:V", "to revere", "eulabeomai"),
        "G2126": ("εὐλαβής", "G:A", "devout", "eulabēs"),
        "G2127": ("εὐλογέω", "G:V", "to praise/bless", "eulogeō"),
        "G2128": ("εὐλογητός", "G:A", "praiseworthy", "eulogētos"),
        "G2129": ("εὐλογία", "G:N-F", "praise", "eulogia"),
        "G2130": ("εὐμετάδοτος", "G:A", "generous", "eumetadotos"),
        "G2131": ("Εὐνίκη", "N:N-F-P", "Eunice", "Eunikē"),
        "G2132": ("εὐνοέω", "G:V", "to reconcile", "eunoeō"),
        "G2133": ("εὔνοια", "G:N-F", "enthusiasm", "eunoia"),
        "G2134": ("εὐνουχίζω", "G:V", "to become a eunuch", "eunouchizō"),
        "G2135": ("εὐνοῦχος", "G:N-M", "eunuch", "eunouchos"),
        "G2136": ("Εὐοδία", "N:N-F-P", "Euodia", "Euodia"),
        "G2137": ("εὐοδόω", "G:V", "to get along well", "euodoō"),
        "G2138": ("εὐπειθής", "G:A", "compliant", "eupeithēs"),
        "G2139": ("εὐπερίστατος", "G:A", "thwarting", "euperistatos"),
        "G2140": ("εὐποιΐα", "G:N-F", "doing good", "eupoiia"),
        "G2141": ("εὐπορέω", "G:V", "be prosperous", "euporeō"),
        "G2142": ("εὐπορία", "G:N-F", "prosperity", "euporia"),
        "G2143": ("εὐπρέπεια", "G:N-F", "beauty", "euprepeia"),
        "G2144": ("εὐπρόσδεκτος", "G:A", "acceptable", "euprosdektos"),
        "G2145": ("εὐπρόσεδρος, εὐπάρεδρος", "G:A", "devoted", "euprosedros, euparedros"),
        "G2146": ("εὐπροσωπέω", "G:V", "to impress", "euprosōpeō"),
        "G2147": ("εὑρίσκω", "G:V", "to find/meet", "ehuriskō"),
        "G2148": ("Εὐροκλύδων", "N:N--T", "a north wind", "Eurokludōn"),
        "G2149": ("εὐρύχωρος", "G:A", "broad", "euruchōros"),
        "G2150": ("εὐσέβεια", "G:N-F", "piety", "eusebeia"),
        "G2151": ("εὐσεβέω", "G:V", "to show piety", "eusebeō"),
        "G2152": ("εὐσεβής", "G:A", "pious", "eusebēs"),
        "G2153": ("εὐσεβῶς", "G:Adv", "piously", "eusebōs"),
        "G2154": ("εὔσημος", "G:A", "intelligible", "eusēmos"),
        "G2155": ("εὔσπλαγχνος", "G:A", "compassionate", "eusplagchnos"),
        "G2156": ("εὐσχημόνως", "G:Adv", "properly", "euschēmonōs"),
        "G2157": ("εὐσχημοσύνη", "G:N-F", "propriety", "euschēmosunē"),
        "G2158": ("εὐσχήμων", "G:A", "proper", "euschēmōn"),
        "G2159": ("εὐτόνως", "G:Adv", "vehemently", "eutonōs"),
        "G2160": ("εὐτραπελία", "G:N-F", "vulgar jesting", "eutrapelia"),
        "G2161": ("Εὔτυχος", "N:N-M-P", "Eutychus", "Eutuchos"),
        "G2162": ("εὐφημία", "G:N-F", "good report", "euphēmia"),
        "G2163": ("εὔφημος", "G:A", "commendable", "euphēmos"),
        "G2164": ("εὐφορέω", "G:V", "be fruitful", "euphoreō"),
        "G2165": ("εὐφραίνω", "G:V", "to celebrate", "euphrainō"),
        "G2166": ("Εὐφράτης", "N:N-M-L", "Euphrates", "Euphratēs"),
        "G2167": ("εὐφροσύνη", "G:N-F", "joy", "euphrosunē"),
        "G2168": ("εὐχαριστέω", "G:V", "to thank", "eucharisteō"),
        "G2169": ("εὐχαριστία", "G:N-F", "thankfulness", "eucharistia"),
        "G2170": ("εὐχάριστος", "G:A", "thankful", "eucharistos"),
        "G2171": ("εὐχή", "G:N-F", "a vow/prayer", "euchē"),
        "G2172": ("εὔχομαι", "G:V", "to pray/wish for", "euchomai"),
        "G2173": ("εὔχρηστος", "G:A", "helpful", "euchrēstos"),
        "G2174": ("εὐψυχέω", "G:V", "be glad", "eupsucheō"),
        "G2175": ("εὐωδία", "G:N-F", "aroma", "euōdia"),
        "G2176": ("εὐώνυμος", "G:A", "left/south", "euōnumos"),
        "G2177": ("ἐφάλλομαι", "G:V", "to jump on", "ephallomai"),
        "G2178": ("ἐφάπαξ", "G:Adv", "once/at once", "ephapax"),
        "G2179": ("Ἐφεσῖνος", "N:N--LG", "Ephesian", "Ephesinos"),
        "G2180": ("Ἐφέσιος", "G:A", "Ephesian", "Ephesios"),
        "G2181": ("Ἔφεσος", "N:N-F-L", "Ephesus", "ephesos"),
        "G2182": ("ἐφευρετής", "G:N-M", "inventor", "epheuretēs"),
        "G2183": ("ἐφημερία", "G:N-F", "division", "ephēmeria"),
        "G2184": ("ἐφήμερος", "G:A", "daily", "ephēmeros"),
        "G2185": ("ἐφικνέομαι", "G:V", "to reach", "ephikneomai"),
        "G2186": ("ἐφίστημι", "G:V", "to approach", "ephistēmi"),
        "G2187": ("Ἐφραίμ", "N:N--L", "Ephraim", "Ephraim"),
        "G2188": ("ἐφφαθά", "A:V", "open!", "ephphatha"),
        "G2189": ("ἔχθρα", "G:N-F", "hostility", "echthra"),
        "G2190": ("ἐχθρός", "G:N", "enemy", "echthros"),
        "G2191": ("ἔχιδνα", "G:N-F", "snake", "echidna"),
        "G2192": ("ἔχω", "G:V", "to have/be", "echō"),
        "G2193": ("ἕως", "G:Prep", "until", "heōs"),
        "G2194": ("Ζαβουλών", "N:N--L", "Zebulun", "Zaboulōn"),
        "G2195": ("Ζακχαῖος", "N:N-M-P", "Zacchaeus", "Zakchaios"),
        "G2196": ("Ζαρά", "N:N-M-P", "Zerah", "Zara"),
        "G2197": ("Ζαχαρίας", "N:N-M-P", "Zechariah", "Zacharias"),
        "G2198": ("ζάω", "G:V", "to live", "zaō"),
        "G2199": ("Ζεβεδαῖος", "N:N-M-P", "Zebedee", "Zebedaios"),
        "G2200": ("ζεστός", "G:A", "hot", "zestos"),
        "G2201": ("ζεῦγος", "G:N-N", "a yoke/pair", "zeugos"),
        "G2202": ("ζευκτηρία", "G:N-F", "rope", "zeuktēria"),
        "G2203": ("Ζεύς, Δίς", "N:N--T", "Zeus", "Zeus, Dis"),
        "G2204": ("ζέω", "G:V", "be fervent", "zeō"),
        "G2205": ("ζῆλος", "G:N-M/N", "zeal", "zēlos"),
        "G2206": ("ζηλόω", "G:V", "be eager", "zēloō"),
        "G2207": ("ζηλωτής", "N:N-M-T", "zealot", "zēlōtēs"),
        "G2208": ("ζηλωτής", "N:N--T", "Zealot", "zēlōtēs"),
        "G2209": ("ζημία", "G:N-F", "damage/loss", "zēmia"),
        "G2210": ("ζημιόω", "G:V", "to lose", "zēmioō"),
        "G2211": ("Ζηνᾶς", "N:N-M-P", "Zenas", "Zēnas"),
        "G2212": ("ζητέω", "G:V", "to seek", "zēteō"),
        "G2213": ("ζήτημα", "G:N-N", "a question/dispute", "zētēma"),
        "G2214": ("ζήτησις", "G:N-F", "controversy", "zētēsis"),
        "G2215": ("ζιζάνιον", "G:N-N", "weed", "zizanion"),
        "G2216": ("Ζοροβάβελ", "N:N-M-P", "Zerubbabel", "Zorobabel"),
        "G2217": ("ζόφος", "G:N-M", "darkness", "zophos"),
        "G2218": ("ζυγός", "G:N-M", "yoke/scales", "zugos"),
        "G2219": ("ζύμη", "G:N-F", "leaven", "zumē"),
        "G2220": ("ζυμόω", "G:V", "to leaven", "zumoō"),
        "G2221": ("ζωγρέω", "G:V", "to capture alive", "zōgreō"),
        "G2222": ("ζωή", "G:N-F", "life", "zōē"),
        "G2223": ("ζώνη", "G:N-F", "belt/sash/girdle", "zōnē"),
        "G2224": ("ζώννυμι", "G:V", "to dress", "zōnnumi"),
        "G2225": ("ζωογονέω", "G:V", "to give life", "zōogoneō"),
        "G2226": ("ζῷον", "G:N-N", "living thing", "zōon"),
        "G2227": ("ζωοποιέω", "G:V", "to make alive", "zōopoieō"),
        "G2228": ("ἤ", "G:Conj", "or", "ē"),
        "G2229": ("ἦ", "G:Adv", "surely", "ē"),
        "G2230": ("ἡγεμονεύω", "G:V", "to govern", "hēgemoneuō"),
        "G2231": ("ἡγεμονία", "G:N-F", "reign", "hēgemonia"),
        "G2232": ("ἡγεμών", "G:N-M", "ruler", "hēgemōn"),
        "G2233": ("ἡγέομαι", "G:V", "to govern", "hēgeomai"),
        "G2234": ("ἡδέως", "G:Adv", "gladly", "hēdeōs"),
        "G2235": ("ἤδη", "G:Adv", "already", "ēdē"),
        "G2236": ("ἥδιστα", "G:A", "most gladly", "hēdista"),
        "G2237": ("ἡδονή", "G:N-F", "pleasure", "hēdonē"),
        "G2238": ("ἡδύοσμος", "G:N-N", "mint", "hēduosmos"),
        "G2239": ("ἦθος", "G:N-N", "moral", "ēthos"),
        "G2240": ("ἥκω", "G:V", "to come/be present", "hēkō"),
        "G2241": ("ἠλί", "G:N-M", "Eli", "ēli"),
        "G2242": ("Ἡλί", "N:N-M-P", "Heli", "Hēli"),
        "G2243": ("Ἡλίας", "N:N-M-P", "Elijah", "Hēlias"),
        "G2244": ("ἡλικία", "G:N-F", "age/height", "hēlikia"),
        "G2245": ("ἡλίκος", "G:Cor+G:Intg", "how great", "hēlikos"),
        "G2246": ("ἥλιος", "G:N-M", "sun", "hēlios"),
        "G2247": ("ἧλος", "G:N-M", "nail", "hēlos"),
        "G2248": ("ἡμᾶς", "G:PerP", "us", "hēmas"),
        "G2249": ("ἡμεῖς", "G:PerP", "we", "hēmeis"),
        "G2250": ("ἡμέρα", "G:N-F", "day", "hēmera"),
        "G2251": ("ἡμέτερος", "G:PosP", "our", "hēmeteros"),
        "G2252": ("ἤμην", "G:V", "I was", "ēmēn"),
        "G2253": ("ἡμιθανής", "G:A", "half-dead", "hēmithanēs"),
        "G2254": ("ἡμῖν", "G:PerP", "to us", "hēmin"),
        "G2255": ("ἥμισυς", "G:A", "half", "hēmisus"),
        "G2256": ("ἡμιώριον", "G:N-N", "half an hour", "hēmiōrion"),
        "G2257": ("ἡμῶν", "G:PerP", "of us", "hēmōn"),
        "G2258": ("ἦν", "G:V", "was", "ēn"),
        "G2259": ("ἡνίκα", "G:Conj", "when", "hēnika"),
        "G2260": ("ἤπερ", "G:Conj", "than", "ēper"),
        "G2261": ("ἤπιος", "G:A", "gentle", "ēpios"),
        "G2262": ("Ἤρ", "N:N-M-P", "Er", "Ēr"),
        "G2263": ("ἤρεμος", "G:A", "quiet", "ēremos"),
        "G2264": ("Ἡρώδης", "N:N-M-P", "Herod", "Hērōdēs"),
        "G2265": ("Ἡρωδιανοί", "N:N-M-T", "Herodians", "Hērōdianoi"),
        "G2266": ("Ἡρωδιάς", "N:N-F-P", "Herodias", "Hērōdias"),
        "G2267": ("Ἡρωδίων", "N:N-M-P", "Herodion", "Hērōdiōn"),
        "G2268": ("Ἡσαΐας", "N:N-M-P", "Isaiah", "Hēsaias"),
        "G2269": ("Ἠσαῦ", "N:N-M-P", "Esau", "Ēsau"),
        "G2270": ("ἡσυχάζω", "G:V", "be quiet/give up", "hēsuchazō"),
        "G2271": ("ἡσυχία", "G:N-F", "quietness", "hēsuchia"),
        "G2272": ("ἡσύχιος", "G:A", "quiet", "hēsuchios"),
        "G2273": ("ἤτοι", "G:Conj", "whether", "ētoi"),
        "G2274": ("ἡσσάομαι", "G:V", "be lesser", "hēssaomai"),
        "G2275": ("ἥττημα", "G:N-N", "loss", "hēttēma"),
        "G2276": ("ἥσσων", "G:A", "less/worse", "hētton, hēssōn"),
        "G2277": ("ἤτω", "G:V", "let him/it be", "ētō"),
        "G2278": ("ἠχέω", "G:V", "to resound", "ēcheō"),
        "G2279": ("ἦχος", "G:N-M", "sound", "ēchos"),
        "G2280": ("Θαδδαῖος", "N:N-M-P", "Thaddaeus", "Thaddaios"),
        "G2281": ("θάλασσα", "G:N-F", "sea", "thalassa"),
        "G2282": ("θάλπω", "G:V", "to care for", "thalpō"),
        "G2283": ("Θάμαρ", "N:N-F-P", "Tamar", "Thamar"),
        "G2284": ("θαμβέω", "G:V", "to astonish", "thambeō"),
        "G2285": ("θάμβος", "G:N-N", "amazement", "thambos"),
        "G2286": ("θανάσιμος", "G:A", "deadly", "thanasimos"),
        "G2287": ("θανατήφορος", "G:A", "deadly", "thanatēphoros"),
        "G2288": ("θάνατος", "G:N-M", "death", "thanatos"),
        "G2289": ("θανατόω", "G:V", "to kill", "thanatoō"),
        "G2290": ("θάπτω", "G:V", "to bury", "thaptō"),
        "G2291": ("Θάρα", "N:N-M-P", "Terah", "Thara"),
        "G2292": ("θαρρέω", "G:V", "be confident", "tharreō"),
        "G2293": ("θαρσέω", "G:V", "take heart", "tharseō"),
        "G2294": ("θάρσος", "G:N-N", "courage/confidence", "tharsos"),
        "G2295": ("θαῦμα", "G:N-N", "marvel", "thauma"),
        "G2296": ("θαυμάζω", "G:V", "to marvel", "thaumazō"),
        "G2297": ("θαυμάσιος", "G:A", "marvelous", "thaumasios"),
        "G2298": ("θαυμαστός", "G:A", "marvellous", "thaumastos"),
        "G2299": ("θεά", "G:N-F", "goddess", "thea"),
        "G2300": ("θεάομαι", "G:V", "to look at", "theaomai"),
        "G2301": ("θεατρίζω", "G:V", "to expose", "theatrizō"),
        "G2302": ("θέατρον", "G:N-N", "theatre", "theatron"),
        "G2303": ("θεῖον", "G:N-N", "sulfur", "theion"),
        "G2304": ("θεῖος", "G:A", "divine", "theios"),
        "G2305": ("θειότης", "G:N-F", "divinity", "theiotēs"),
        "G2306": ("θειώδης", "G:A", "sulphurous", "theiōdēs"),
        "G2307": ("θέλημα", "G:N-N", "will/desire", "thelēma"),
        "G2308": ("θέλησις", "G:N-F", "will", "thelēsis"),
        "G2309": ("θέλω", "G:V", "to will/desire", "thelō"),
        "G2310": ("θεμέλιος", "G:N-M/N", "foundation", "themelios"),
        "G2311": ("θεμελιόω", "G:V", "to found", "themelioō"),
        "G2312": ("θεοδίδακτος", "G:A", "divinely instructed", "theodidaktos"),
        "G2313": ("θεομαχέω", "G:V", "to fight God", "theomacheō"),
        "G2314": ("θεομάχος", "G:A", "opposing God", "theomachos"),
        "G2315": ("θεόπνευστος", "G:A", "God-breathed", "theopneustos"),
        "G2316": ("θεός", "G:N-M/F", "God", "theos"),
        "G2317": ("θεοσέβεια", "G:N-F", "reverence for God", "theosebeia"),
        "G2318": ("θεοσεβής", "G:A", "godly", "theosebēs"),
        "G2319": ("θεοστυγής", "G:A", "God-hating", "theostugēs"),
        "G2320": ("θεότης", "G:N-F", "deity", "theotēs"),
        "G2321": ("Θεόφιλος", "N:N-M-P", "Theophilus", "Theophilos"),
        "G2322": ("θεραπεία", "G:N-F", "service", "therapeia"),
        "G2323": ("θεραπεύω", "G:V", "to serve/heal", "therapeuō"),
        "G2324": ("θεράπων", "G:N-M", "servant", "therapōn"),
        "G2325": ("θερίζω", "G:V", "to reap", "therizō"),
        "G2326": ("θερισμός", "G:N-M", "harvest", "therismos"),
        "G2327": ("θεριστής", "G:N-M", "reaper", "theristēs"),
        "G2328": ("θερμαίνω", "G:V", "to warm", "thermainō"),
        "G2329": ("θέρμη", "G:N-F", "heat", "thermē"),
        "G2330": ("θέρος", "G:N-N", "summer", "theros"),
        "G2331": ("Θεσσαλονικεύς", "N:N--LG", "Thessalonian", "Thessalonikeus"),
        "G2332": ("Θεσσαλονίκη", "N:N-F-L", "Thessalonica", "Thessalonikē"),
        "G2333": ("Θευδᾶς", "N:N-M-P", "Theudas", "Theudas"),
        "G2334": ("θεωρέω", "G:V", "to see/experience", "theōreō"),
        "G2335": ("θεωρία", "G:N-F", "sight", "theōria"),
        "G2336": ("θήκη", "G:N-F", "sheath", "thēkē"),
        "G2337": ("θηλάζω", "G:V", "to suckle", "thēlazō"),
        "G2338": ("θῆλυς", "G:A", "female", "thēlus"),
        "G2339": ("θήρα", "G:N-F", "trap", "thēra"),
        "G2340": ("θηρεύω", "G:V", "to hunt/catch", "thēreuō"),
        "G2341": ("θηριομαχέω", "G:V", "to fight beasts", "thēriomacheō"),
        "G2342": ("θηρίον", "G:N-N", "wild animal", "thērion"),
        "G2343": ("θησαυρίζω", "G:V", "to store up", "thēsaurizō"),
        "G2344": ("θησαυρός", "G:N-M", "treasure", "thēsauros"),
        "G2345": ("θιγγάνω", "G:V", "to touch", "thigganō"),
        "G2346": ("θλίβω", "G:V", "to press on", "thlibō"),
        "G2347": ("θλῖψις", "G:N-F", "pressure", "thlipsis"),
        "G2348": ("θνῄσκω", "G:V", "to die/be dead", "thnēskō"),
        "G2349": ("θνητός", "G:A", "mortal", "thnētos"),
        "G2350": ("θορυβέω", "G:V", "to make commotion", "thorubeō"),
        "G2351": ("θόρυβος", "G:N-M", "commotion", "thorubos"),
        "G2352": ("θραύω", "G:V", "to crush", "thrauō"),
        "G2353": ("θρέμμα", "G:N-N", "livestock", "thremma"),
        "G2354": ("θρηνέω", "G:V", "to lament", "thrēneō"),
        "G2355": ("θρῆνος", "G:N-M", "lamentation", "thrēnos"),
        "G2356": ("θρησκεία", "G:N-F", "religion", "thrēskeia"),
        "G2357": ("θρησκός", "G:A", "religious", "thrēskos"),
        "G2358": ("θριαμβεύω", "G:V", "to triumph", "thriambeuō"),
        "G2359": ("θρίξ", "G:N-F", "hair", "thrix"),
        "G2360": ("θροέω", "G:V", "to alarm", "throeō"),
        "G2361": ("θρόμβος", "G:N-M", "drop", "thrombos"),
        "G2362": ("θρόνος", "G:N-M", "throne", "thronos"),
        "G2363": ("Θυάτειρα", "N:N--L", "Thyatira", "Thuateira"),
        "G2364": ("θυγάτηρ", "G:N-F", "daughter", "thugatēr"),
        "G2365": ("θυγάτριον", "G:N-N", "little daughter", "thugatrion"),
        "G2366": ("θύελλα", "G:N-F", "storm", "thuella"),
        "G2367": ("θύϊνος", "G:A", "citron", "thuinos"),
        "G2368": ("θυμίαμα", "G:N-N", "incense", "thumiama"),
        "G2369": ("θυμιατήριον", "G:N-N", "incense altar", "thumiatērion"),
        "G2370": ("θυμιάω", "G:V", "to burn incense", "thumiaō"),
        "G2371": ("θυμομαχέω", "G:V", "to quarrel", "thumomacheō"),
        "G2372": ("θυμός", "G:N-M", "wrath", "thumos"),
        "G2373": ("θυμόω", "G:V", "to anger", "thumoō"),
        "G2374": ("θύρα", "G:N-F", "door", "thura"),
        "G2375": ("θυρεός", "G:N-M", "long shield", "thureos"),
        "G2376": ("θυρίς", "G:N-F", "window", "thuris"),
        "G2377": ("θυρωρός", "G:N-M/F", "gatekeeper", "thurōros"),
        "G2378": ("θυσία", "G:N-F", "sacrifice", "thusia"),
        "G2379": ("θυσιαστήριον", "G:N-N", "altar", "thusiastērion"),
        "G2380": ("θύω", "G:V", "to sacrifice", "thuō"),
        "G2381": ("Θωμᾶς", "N:N-M-P", "Thomas", "Thōmas"),
        "G2382": ("θώραξ", "G:N-M", "breastplate", "thōrax"),
        "G2383": ("Ἰάειρος", "N:N-M-P", "Jairus", "Iaeiros"),
        "G2384": ("Ἰακώβ", "N:N-M-P", "Jacob", "Iakōb"),
        "G2385": ("Ἰάκωβος", "N:N-M-P", "James", "Iakōbos"),
        "G2386": ("ἴαμα", "G:N-N", "healing", "iama"),
        "G2387": ("Ἰαμβρῆς", "N:N-M-P", "Jambres", "Iambrēs"),
        "G2388": ("Ἰαννά", "N:N-M-P", "Janna", "Ianna"),
        "G2389": ("Ἰαννῆς", "N:N-M-P", "Jannes", "Iannēs"),
        "G2390": ("ἰάομαι", "G:V", "to heal", "iaomai"),
        "G2391": ("Ἰάρεδ", "N:N-M-P", "Jared", "Iared"),
        "G2392": ("ἴασις", "G:N-F", "healing", "iasis"),
        "G2393": ("ἴασπις", "G:N-F", "jasper", "iaspis"),
        "G2394": ("Ἰάσων", "N:N-M-P", "Jason", "Iasōn"),
        "G2395": ("ἰατρός", "G:N-M", "physician", "iatros"),
        "G2396": ("ἴδε", "G:Intj", "look!", "ide"),
        "G2397": ("ἰδέα", "G:N-F", "appearance", "idea"),
        "G2398": ("ἴδιος", "G:A", "one's own/private", "idios"),
        "G2399": ("ἰδιώτης", "G:N-M", "unlearned", "idiōtēs"),
        "G2400": ("ἰδού", "G:Intj", "look!", "idou"),
        "G2401": ("Ἰδουμαία", "N:N-F-L", "Idumea", "Idoumaia"),
        "G2402": ("ἱδρώς", "G:N-M", "sweat", "hidrōs"),
        "G2403": ("Ἰεζαβήλ", "N:N-F-P", "Jezebel", "Iezabēl"),
        "G2404": ("Ἱεράπολις", "N:N-F-L", "Hierapolis", "Hierapolis"),
        "G2405": ("ἱερατεία", "G:N-F", "priesthood", "hierateia"),
        "G2406": ("ἱεράτευμα", "G:N-N", "priesthood", "hierateuma"),
        "G2407": ("ἱερατεύω", "G:V", "to serve as priest", "hierateuō"),
        "G2408": ("Ἱερεμίας", "N:N-M-P", "Jeremiah", "Hieremias"),
        "G2409": ("ἱερεύς", "G:N-M", "priest", "hiereus"),
        "G2410": ("Ἱεριχώ", "N:N--L", "Jericho", "Hierichō"),
        "G2411": ("ἱερόν", "G:N-N", "temple", "hieros"),
        "G2412": ("ἱεροπρεπής", "G:A", "reverent", "hieroprepēs"),
        "G2413": ("ἱερός", "G:A", "sacred", "hieros"),
        "G2414": ("Ἱεροσόλυμα", "N:N--L", "Jerusalem", "Hierosoluma"),
        "G2415": ("Ἱεροσολυμίτης", "N:N-M-L", "of Jerusalem", "Hierosolumitēs"),
        "G2416": ("ἱεροσυλέω", "G:V", "to despoil a temple", "hierosuleō"),
        "G2417": ("ἱερόσυλος", "G:A", "temple robber", "hierosulos"),
        "G2418": ("ἱερουργέω", "G:V", "to minister", "hierourgeō"),
        "G2419": ("Ἱερουσαλήμ", "N:N--L", "Jerusalem", "Hierousalēm"),
        "G2420": ("ἱερωσύνη", "G:N-F", "priesthood", "hierōsunē"),
        "G2421": ("Ἰεσσαί", "N:N-M-P", "Jesse", "Iessai"),
        "G2422": ("Ἰεφθάε", "N:N-M-P", "Jephthah", "Iephthae"),
        "G2423": ("Ἰεχονίας", "N:N-M-P", "Jeconiah", "Iechonias"),
        "G2424": ("Ἰησοῦς", "N:N-M-P", "Jesus/Joshua", "Iēsous"),
        "G2425": ("ἱκανός", "G:A", "sufficient", "hikanos"),
        "G2426": ("ἱκανότης", "G:N-F", "sufficiency", "hikanotēs"),
        "G2427": ("ἱκανόω", "G:V", "to qualify", "hikanoō"),
        "G2428": ("ἱκετηρία", "G:N-F", "supplication", "hiketēria"),
        "G2429": ("ἰκμάς", "G:N-F", "moisture", "ikmas"),
        "G2430": ("Ἰκόνιον", "N:N--L", "Iconium", "Ikonion"),
        "G2431": ("ἱλαρός", "G:A", "cheerful", "hilaros"),
        "G2432": ("ἱλαρότης", "G:N-F", "cheerfulness", "hilarotēs"),
        "G2433": ("ἱλάσκομαι", "G:V", "to propitiate", "hilaskomai"),
        "G2434": ("ἱλασμός", "G:N-M", "propitiation", "hilasmos"),
        "G2435": ("ἱλαστήριος", "G:N-N", "propitiation", "hilastērios"),
        "G2436": ("ἵλεως", "G:A", "propitious/gracious", "hileōs"),
        "G2437": ("Ἰλλυρικόν", "N:N--L", "Illyricum", "Illurikon"),
        "G2438": ("ἱμάς", "G:N-M", "leather strap", "himas"),
        "G2439": ("ἱματίζω", "G:V", "to dress", "himatizō"),
        "G2440": ("ἱμάτιον", "G:N-N", "clothing", "himation"),
        "G2441": ("ἱματισμός", "G:N-M", "clothing", "himatismos"),
        "G2442": ("ἱμείρω", "G:V", "to desire", "himeirō, himeiromai"),
        "G2443": ("ἵνα", "G:Conj", "in order that/to", "hina"),
        "G2444": ("ἱνατί", "G:Adv", "why?", "hinati"),
        "G2445": ("Ἰόππη", "N:N-F-L", "Joppa", "Ioppē"),
        "G2446": ("Ἰορδάνης", "N:N-M-L", "Jordan", "Iordanēs"),
        "G2447": ("ἰός", "G:N-M", "poison/rust", "ios"),
        "G2448": ("Ἰουδά", "N:N--L", "Judea", "Iouda"),
        "G2449": ("Ἰουδαία", "N:N-F-L", "Judea", "Ioudaia"),
        "G2450": ("Ἰουδαΐζω", "G:V", "be Jewish", "Ioudaizō"),
        "G2451": ("Ἰουδαϊκός", "N:N--PG", "Jewish", "Ioudaikos"),
        "G2452": ("Ἰουδαϊκῶς", "G:Adv", "Jewishly", "Ioudaikōs"),
        "G2453": ("Ἰουδαῖος", "N:N--PG", "Jewish", "Ioudaios"),
        "G2454": ("Ἰουδαϊσμός", "N:N--T", "Judaism", "Ioudaismos"),
        "G2455": ("Ἰούδας", "N:N-M-P", "Judah/Judas/Jude", "Ioudas"),
        "G2456": ("Ἰουλία", "N:N-F-P", "Julia", "Ioulia"),
        "G2457": ("Ἰούλιος", "N:N-M-P", "Julius", "Ioulios"),
        "G2458": ("Ἰουνιᾶς", "N:N-F-P", "Junias", "Iounias"),
        "G2459": ("Ἰοῦστος", "N:N-M-P", "Justus", "Ioustos"),
        "G2460": ("ἱππεύς", "G:N-M", "horseman", "hippeus"),
        "G2461": ("ἱππικός", "G:A", "horseman", "hippikos"),
        "G2462": ("ἵππος", "G:N-M", "horse", "hippos"),
        "G2463": ("ἶρις", "G:N-F", "rainbow", "iris"),
        "G2464": ("Ἰσαάκ", "N:N-M-P", "Isaac", "Isaak"),
        "G2465": ("ἰσάγγελος", "G:A", "like an angel", "isaggelos"),
        "G2466": ("Ἰσαχάρ", "N:N-M-P", "Issachar", "Isachar"),
        "G2467": ("ἴσημι", "G:V", "to know", "isēmi"),
        "G2468": ("ἴσθι", "G:V", "you should be", "isthi"),
        "G2469": ("Ἰσκαριώτης, Ἰσκαριώθ", "N:N-M-P", "Iscariot", "Iskariōtēs, Iskariōth"),
        "G2470": ("ἴσος", "G:A", "equal", "isos"),
        "G2471": ("ἰσότης", "G:N-F", "equality", "isotēs"),
        "G2472": ("ἰσότιμος", "G:A", "equally valuable", "isotimos"),
        "G2473": ("ἰσόψυχος", "G:A", "like-minded", "isopsuchos"),
        "G2474": ("Ἰσραήλ", "N:N--L", "Israel", "Israēl"),
        "G2475": ("Ἰσραηλίτης", "N:N--PG", "Israelite", "Israēlitēs"),
        "G2476": ("ἵστημι", "G:V", "to stand", "histēmi"),
        "G2477": ("ἱστορέω", "G:V", "to get acquainted", "historeō"),
        "G2478": ("ἰσχυρός", "G:A", "strong", "ischuros"),
        "G2479": ("ἰσχύς", "G:N-F", "strength", "ischus"),
        "G2480": ("ἰσχύω", "G:V", "be strong", "ischuō"),
        "G2481": ("ἴσως", "G:Adv", "perhaps", "isōs"),
        "G2482": ("Ἰταλία", "N:N-F-L", "Italy", "Italia"),
        "G2483": ("Ἰταλικός", "N:N--LG", "Italian", "Italikos"),
        "G2484": ("Ἰτουραΐα", "N:N--L", "Iturea", "Itouraia"),
        "G2485": ("ἰχθύδιον", "G:N-N", "little fish", "ichthudion"),
        "G2486": ("ἰχθύς", "G:N-M", "fish", "ichthus"),
        "G2487": ("ἴχνος", "G:N-N", "track", "ichnos"),
        "G2488": ("Ἰωάθαμ", "N:N-M-P", "Jotham", "Iōatham"),
        "G2489": ("Ἰωάννα, Ἰωανάν", "N:N-F-P", "Joanna", "Iōanna, Iōanan"),
        "G2490": ("Ἰωαννᾶς", "N:N-M-P", "Joannan", "Iōannas"),
        "G2491": ("Ἰωάννης", "N:N-M-P", "John", "Iōannēs"),
        "G2492": ("Ἰώβ", "N:N-M-P", "Job", "Iōb"),
        "G2493": ("Ἰωήλ", "N:N-M-P", "Joel", "Iōēl"),
        "G2494": ("Ἰωνάν", "N:N-M-P", "Jonam", "Iōnan"),
        "G2495": ("Ἰωνᾶς", "N:N-M-P", "Jonah", "Iōnas"),
        "G2496": ("Ἰωράμ", "N:N-M-P", "Jehoram", "Iōram"),
        "G2497": ("Ἰωρείμ", "N:N-M-P", "Jorim", "Iōreim"),
        "G2498": ("Ἰωσαφάτ", "N:N-M-P", "Jehoshaphat", "Iōsaphat"),
        "G2499": ("Ἰωσή", "N:N-M-P", "Jose", "Iōsē"),
        "G2500": ("Ἰωσῆς", "N:N-M-P", "Joses", "Iōsēs"),
        "G2501": ("Ἰωσήφ", "N:N-M-P", "Joseph", "Iōsēph"),
        "G2502": ("Ἰωσίας", "N:N-M-P", "Josiah", "Iōsias"),
        "G2503": ("ἰῶτα", "G:N-N", "iota", "iōta"),
        "G2504": ("κἀγώ", "G:Conj+G:PerP", "and I", "kagō"),
        "G2505": ("καθά", "G:Conj", "as/just as", "katha"),
        "G2506": ("καθαίρεσις", "G:N-F", "destruction", "kathairesis"),
        "G2507": ("καθαιρέω", "G:V", "to take down", "kathaireō"),
        "G2508": ("καθαίρω", "G:V", "to prune", "kathairō"),
        "G2509": ("καθάπερ", "G:Conj", "just as", "kathaper"),
        "G2510": ("καθάπτω", "G:V", "to attach", "kathaptō"),
        "G2511": ("καθαρίζω", "G:V", "to clean", "katharizō"),
        "G2512": ("καθαρισμός", "G:N-M", "cleansing", "katharismos"),
        "G2513": ("καθαρός", "G:A", "clean", "katharos"),
        "G2514": ("καθαρότης", "G:N-F", "cleanness", "katharotēs"),
        "G2515": ("καθέδρα", "G:N-F", "seat", "kathedra"),
        "G2516": ("καθέζομαι", "G:V", "to sit down", "kathezomai"),
        "G2517": ("καθεξῆς", "G:Adv", "in order", "kathexēs"),
        "G2518": ("καθεύδω", "G:V", "to sleep", "katheudō"),
        "G2519": ("καθηγητής", "G:N-M", "teacher", "kathēgētēs"),
        "G2520": ("καθήκω", "G:V", "be fit", "kathēkō"),
        "G2521": ("κάθημαι", "G:V", "to sit", "kathēmai"),
        "G2522": ("καθημερινός", "G:A", "daily", "kathēmerinos"),
        "G2523": ("καθίζω", "G:V", "to seat", "kathizō"),
        "G2524": ("καθίημι", "G:V", "to lower", "kathiēmi"),
        "G2525": ("καθίστημι", "G:V", "to appoint/conduct", "kathistēmi"),
        "G2526": ("καθό", "G:Conj", "insofar as", "katho"),
        "G2527": ("καθόλου", "G:Adv", "at all", "katholou"),
        "G2528": ("καθοπλίζω", "G:V", "to arm fully", "kathoplizō"),
        "G2529": ("καθοράω", "G:V", "to perceive", "kathoraō"),
        "G2530": ("καθότι", "G:Conj", "as/just as", "kathoti"),
        "G2531": ("καθώς", "G:Conj", "as/just as", "kathōs"),
        "G2532": ("καί", "G:Conj", "and", "kai"),
        "G2533": ("Καϊάφας", "N:N-M-P", "Caiaphas", "Kaiaphas"),
        "G2534": ("καίγε", "G:Conj", "even/even though", "kaige"),
        "G2535": ("Κάϊν", "N:N-M-P", "Cain", "Kain"),
        "G2536": ("Καϊνάν, Καϊνάμ", "N:N-M-P", "Cainan", "Kainan, Kainam"),
        "G2537": ("καινός", "G:A", "new", "kainos"),
        "G2538": ("καινότης", "G:N-F", "newness", "kainotēs"),
        "G2539": ("καίπερ", "G:Conj", "although", "kaiper"),
        "G2540": ("καιρός", "G:N-M", "time/right time", "kairos"),
        "G2541": ("Καῖσαρ", "N:N--T", "Caesar", "Kaisar"),
        "G2542": ("Καισάρεια", "N:N-F-L", "Caesarea", "Kaisareia"),
        "G2543": ("καίτοι", "G:Conj", "and yet/although", "kaitoi"),
        "G2544": ("καίτοιγε", "G:Conj", "and yet/although", "kaitoige"),
        "G2545": ("καίω", "G:V", "to kindle/burn", "kaiō"),
        "G2546": ("κἀκεῖ", "G:Adv", "and there", "kakei"),
        "G2547": ("κἀκεῖθεν", "G:Adv", "and from there", "kakeithen"),
        "G2548": ("κἀκεῖνος", "G:DemP", "and that one", "kakeinos"),
        "G2549": ("κακία", "G:N-F", "evil", "kakia"),
        "G2550": ("κακοήθεια", "G:N-F", "malice", "kakoētheia"),
        "G2551": ("κακολογέω", "G:V", "to curse/revile", "kakologeō"),
        "G2552": ("κακοπάθεια", "G:N-F", "suffering", "kakopatheia"),
        "G2553": ("κακοπαθέω", "G:V", "to endure", "kakopatheō"),
        "G2554": ("κακοποιέω", "G:V", "to do evil/harm", "kakopoieō"),
        "G2555": ("κακοποιός", "G:N-M", "wrongdoing", "kakopoios"),
        "G2556": ("κακός", "G:A", "evil/harm", "kakos"),
        "G2557": ("κακοῦργος", "G:A", "criminal", "kakourgos"),
        "G2558": ("κακουχέω", "G:V", "to torment", "kakoucheō"),
        "G2559": ("κακόω", "G:V", "to harm", "kakoō"),
        "G2560": ("κακῶς", "G:Adv", "badly", "kakōs"),
        "G2561": ("κάκωσις", "G:N-F", "oppression", "kakōsis"),
        "G2562": ("καλάμη", "G:N-F", "straw/stubble", "kalamē"),
        "G2563": ("κάλαμος", "G:N-M", "reed/stick/pen", "kalamos"),
        "G2564": ("καλέω", "G:V", "to call", "kaleō"),
        "G2565": ("καλλιέλαιος", "G:N-F", "olive tree", "kallielaios"),
        "G2566": ("καλλίον", "G:Adv", "good", "kallion"),
        "G2567": ("καλοδιδάσκαλος", "G:A", "teaching good", "kalodidaskalos"),
        "G2568": ("Καλοὶ Λιμένες", "N:N--L", "Fair Havens", "Kaloi Limenes"),
        "G2569": ("καλοποιέω", "G:V", "to do good", "kalopoieō"),
        "G2570": ("καλός", "G:A / G:Adv", "good", "kalos"),
        "G2571": ("κάλυμμα", "G:N-N", "veil", "kalumma"),
        "G2572": ("καλύπτω", "G:V", "to cover", "kaluptō"),
        "G2573": ("καλῶς", "G:Adv", "well", "kalōs"),
        "G2574": ("κάμηλος", "G:N-F", "camel", "kamēlos"),
        "G2575": ("κάμινος", "G:N-F", "furnace/oven", "kaminos"),
        "G2576": ("καμμύω", "G:V", "to close", "kammuō"),
        "G2577": ("κάμνω", "G:V", "be weary/sick", "kamnō"),
        "G2578": ("κάμπτω", "G:V", "to bend/bow", "kamptō"),
        "G2579": ("κἄν", "G:Conj", "and/even if", "kan"),
        "G2580": ("Κανᾶ", "N:N--L", "Cana", "Kana"),
        "G2581": ("Κανανίτης, Καναναῖος", "N:N-M-T", "Zealot", "Kananitēs, Kananaios"),
        "G2582": ("Κανδάκη", "N:N--T", "Candace", "Kandakē"),
        "G2583": ("κανών", "G:N-M", "rule", "kanōn"),
        "G2584": ("Καπερναούμ", "N:N--L", "Capernaum", "Kapernaoum"),
        "G2585": ("καπηλεύω", "G:V", "to peddle", "kapēleuō"),
        "G2586": ("καπνός", "G:N-M", "smoke", "kapnos"),
        "G2587": ("Καππαδοκία", "N:N-F-L", "Cappadocia", "Kappadokia"),
        "G2588": ("καρδία", "G:N-F", "heart", "kardia"),
        "G2589": ("καρδιογνώστης", "G:N-M", "heart-knower", "kardiognōstēs"),
        "G2590": ("καρπός", "G:N-M", "fruit", "karpos"),
        "G2591": ("Κάρπος", "N:N-M-P", "Carpus", "Karpos"),
        "G2592": ("καρποφορέω", "G:V", "to bear fruit", "karpophoreō"),
        "G2593": ("καρποφόρος", "G:A", "fruitbearing", "karpophoros"),
        "G2594": ("καρτερέω", "G:V", "to endure", "kartereō"),
        "G2595": ("κάρφος", "G:N-N", "speck", "karphos"),
        "G2596": ("κατά", "G:Prep", "according to", "kata"),
        "G2597": ("καταβαίνω", "G:V", "to come/go down", "katabainō"),
        "G2598": ("καταβάλλω", "G:V", "to lay/throw down", "kataballō"),
        "G2599": ("καταβαρέω", "G:V", "to burden", "katabareō"),
        "G2600": ("κατάβασις", "G:N-F", "descent", "katabasis"),
        "G2601": ("καταβιβάζω", "G:V", "to bring down", "katabibazō"),
        "G2602": ("καταβολή", "G:N-F", "beginning", "katabolē"),
        "G2603": ("καταβραβεύω", "G:V", "to disqualify", "katabrabeuō"),
        "G2604": ("καταγγελεύς", "G:N-M", "proclaimer", "kataggeleus"),
        "G2605": ("καταγγέλλω", "G:V", "to proclaim", "kataggellō"),
        "G2606": ("καταγελάω", "G:V", "to mock", "katagelaō"),
        "G2607": ("καταγινώσκω", "G:V", "to condemn", "kataginōskō"),
        "G2608": ("κατάγνυμι", "G:V", "to break", "katagnumi"),
        "G2609": ("κατάγω", "G:V", "to bring down", "katagō"),
        "G2610": ("καταγωνίζομαι", "G:V", "to conquer", "katagōnizomai"),
        "G2611": ("καταδέω", "G:V", "to bandage", "katadeō"),
        "G2612": ("κατάδηλος", "G:A", "evident", "katadēlos"),
        "G2613": ("καταδικάζω", "G:V", "to condemn", "katadikazō"),
        "G2614": ("καταδιώκω", "G:V", "to seek", "katadiōkō"),
        "G2615": ("καταδουλόω", "G:V", "to enslave", "katadouloō"),
        "G2616": ("καταδυναστεύω", "G:V", "to oppress", "katadunasteuō"),
        "G2617": ("καταισχύνω", "G:V", "to dishonor", "kataischunō"),
        "G2618": ("κατακαίω", "G:V", "to burn", "katakaiō"),
        "G2619": ("κατακαλύπτω", "G:V", "to cover", "katakaluptō"),
        "G2620": ("κατακαυχάομαι", "G:V", "to boast", "katakauchaomai"),
        "G2621": ("κατάκειμαι", "G:V", "to recline", "katakeimai"),
        "G2622": ("κατακλάω", "G:V", "to break", "kataklaō"),
        "G2623": ("κατακλείω", "G:V", "to lock up", "katakleiō"),
        "G2624": ("κατακληροδοτέω", "G:V", "to distribute", "kataklērodoteō"),
        "G2625": ("κατακλίνω", "G:V", "to sit", "kataklinō"),
        "G2626": ("κατακλύζω", "G:V", "to flood", "katakluzō"),
        "G2627": ("κατακλυσμός", "G:N-M", "flood", "kataklusmos"),
        "G2628": ("κατακολουθέω", "G:V", "to follow", "katakoloutheō"),
        "G2629": ("κατακόπτω", "G:V", "to cut", "katakoptō"),
        "G2630": ("κατακρημνίζω", "G:V", "to cast down", "katakrēmnizō"),
        "G2631": ("κατάκριμα", "G:N-N", "condemnation", "katakrima"),
        "G2632": ("κατακρίνω", "G:V", "to condemn", "katakrinō"),
        "G2633": ("κατάκρισις", "G:N-F", "condemnation", "katakrisis"),
        "G2634": ("κατακυριεύω", "G:V", "to master", "katakurieuō"),
        "G2635": ("καταλαλέω", "G:V", "to slander", "katalaleō"),
        "G2636": ("καταλαλιά", "G:N-F", "slander", "katalalia"),
        "G2637": ("κατάλαλος", "G:A", "slanderous", "katalalos"),
        "G2638": ("καταλαμβάνω", "G:V", "to take/realize", "katalambanō"),
        "G2639": ("καταλέγω", "G:V", "to enrol", "katalegō"),
        "G2640": ("κατάλειμμα", "G:N-N", "remnant", "kataleimma"),
        "G2641": ("καταλείπω", "G:V", "to leave behind", "kataleipō"),
        "G2642": ("καταλιθάζω", "G:V", "to stone", "katalithazō"),
        "G2643": ("καταλλαγή", "G:N-F", "reconciliation", "katallagē"),
        "G2644": ("καταλλάσσω", "G:V", "to reconcile", "katallassō"),
        "G2645": ("κατάλοιπος", "G:A", "remaining", "kataloipos"),
        "G2646": ("κατάλυμα", "G:N-N", "guest room/inn", "kataluma"),
        "G2647": ("καταλύω", "G:V", "to destroy/lodge", "kataluō"),
        "G2648": ("καταμανθάνω", "G:V", "to consider", "katamanthanō"),
        "G2649": ("καταμαρτυρέω", "G:V", "to testify against", "katamartureō"),
        "G2650": ("καταμένω", "G:V", "to stay", "katamenō"),
        "G2651": ("καταμόνας", "G:Adv", "alone", "katamonas"),
        "G2652": ("κατανάθεμα", "G:N-N", "curse", "katanathema"),
        "G2653": ("καταναθεματίζω", "G:V", "to curse", "katanathematizō"),
        "G2654": ("καταναλίσκω", "G:V", "to consume", "katanaliskō"),
        "G2655": ("καταναρκάομαι", "G:V", "to burden", "katanarkaomai"),
        "G2656": ("κατανεύω", "G:V", "to signal", "kataneuō"),
        "G2657": ("κατανοέω", "G:V", "to observe", "katanoeō"),
        "G2658": ("καταντάω", "G:V", "to come to", "katantaō"),
        "G2659": ("κατάνυξις", "G:N-F", "stupor", "katanuxis"),
        "G2660": ("κατανύσσω", "G:V", "to pierce", "katanussō"),
        "G2661": ("καταξιόω", "G:V", "to consider worthy", "kataxioō"),
        "G2662": ("καταπατέω", "G:V", "to trample", "katapateō"),
        "G2663": ("κατάπαυσις", "G:N-F", "rest", "katapausis"),
        "G2664": ("καταπαύω", "G:V", "to keep from", "katapauō"),
        "G2665": ("καταπέτασμα", "G:N-N", "curtain", "katapetasma"),
        "G2666": ("καταπίνω", "G:V", "to swallow", "katapinō"),
        "G2667": ("καταπίπτω", "G:V", "to fall down", "katapiptō"),
        "G2668": ("καταπλέω", "G:V", "to sail", "katapleō"),
        "G2669": ("καταπονέω", "G:V", "to oppress", "kataponeō"),
        "G2670": ("καταποντίζω", "G:V", "to sink", "katapontizō"),
        "G2671": ("κατάρα", "G:N-F", "curse", "katara"),
        "G2672": ("καταράομαι", "G:V", "to curse", "kataraomai"),
        "G2673": ("καταργέω", "G:V", "to end", "katargeō"),
        "G2674": ("καταριθμέω", "G:V", "to number", "katarithmeō"),
        "G2675": ("καταρτίζω", "G:V", "to complete", "katartizō"),
        "G2676": ("κατάρτισις", "G:N-F", "perfection", "katartisis"),
        "G2677": ("καταρτισμός", "G:N-M", "preparation", "katartismos"),
        "G2678": ("κατασείω", "G:V", "to signal", "kataseiō"),
        "G2679": ("κατασκάπτω", "G:V", "to ruin", "kataskaptō"),
        "G2680": ("κατασκευάζω", "G:V", "to prepare", "kataskeuazō"),
        "G2681": ("κατασκηνόω", "G:V", "to dwell", "kataskēnoō"),
        "G2682": ("κατασκήνωσις", "G:N-F", "dwelling place", "kataskēnōsis"),
        "G2683": ("κατασκιάζω", "G:V", "to overshadow", "kataskiazō"),
        "G2684": ("κατασκοπέω", "G:V", "to spy", "kataskopeō"),
        "G2685": ("κατάσκοπος", "G:N-M", "spy", "kataskopos"),
        "G2686": ("κατασοφίζομαι", "G:V", "be cunning", "katasophizomai"),
        "G2687": ("καταστέλλω", "G:V", "to quiet", "katastellō"),
        "G2688": ("κατάστημα", "G:N-N", "behavior", "katastēma"),
        "G2689": ("καταστολή", "G:N-F", "attire/behaviour", "katastolē"),
        "G2690": ("καταστρέφω", "G:V", "to overturn", "katastrephō"),
        "G2691": ("καταστρηνιάω", "G:V", "to desire", "katastrēniaō"),
        "G2692": ("καταστροφή", "G:N-F", "ruin", "katastrophē"),
        "G2693": ("καταστρώννυμι", "G:V", "be thrown down", "katastrōnnumi"),
        "G2694": ("κατασύρω", "G:V", "to drag away", "katasurō"),
        "G2695": ("κατασφάζω", "G:V", "to slaughter", "katasphazō"),
        "G2696": ("κατασφραγίζω", "G:V", "to seal", "katasphragizō"),
        "G2697": ("κατάσχεσις", "G:N-F", "possession", "kataschesis"),
        "G2698": ("κατατίθημι", "G:V", "to attain", "katatithēmi"),
        "G2699": ("κατατομή", "G:N-F", "mutilation", "katatomē"),
        "G2700": ("κατατοξεύω", "G:V", "to shoot down", "katatoxeuō"),
        "G2701": ("κατατρέχω", "G:V", "to rush down", "katatrechō"),
        "G2702": ("καταφέρω", "G:V", "to vote against", "katapherō"),
        "G2703": ("καταφεύγω", "G:V", "to flee", "katapheugō"),
        "G2704": ("καταφθείρω", "G:V", "to deprave", "kataphtheirō"),
        "G2705": ("καταφιλέω", "G:V", "to kiss", "kataphileō"),
        "G2706": ("καταφρονέω", "G:V", "to despise", "kataphroneō"),
        "G2707": ("καταφρονητής", "G:N-M", "despiser", "kataphronētēs"),
        "G2708": ("καταχέω", "G:V", "to pour", "katacheō"),
        "G2709": ("καταχθόνιος", "G:A", "subterranean", "katachthonios"),
        "G2710": ("καταχράομαι", "G:V", "to use up", "katachraomai"),
        "G2711": ("καταψύχω", "G:V", "to cool", "katapsuchō"),
        "G2712": ("κατείδωλος", "G:A", "idolatrous", "kateidōlos"),
        "G2713": ("κατέναντι", "G:Prep", "before", "katenanti"),
        "G2714": ("κατενώπιον", "G:Prep", "before", "katenōpion"),
        "G2715": ("κατεξουσιάζω", "G:V", "to rule", "katexousiazō"),
        "G2716": ("κατεργάζομαι", "G:V", "to workout/produce", "katergazomai"),
        "G2718": ("κατέρχομαι", "G:V", "to descend", "katerchomai"),
        "G2719": ("κατεσθίω", "G:V", "to devour", "katesthiō"),
        "G2720": ("κατευθύνω", "G:V", "to guide", "kateuthunō"),
        "G2721": ("κατεφίστημι", "G:V", "to attack", "katephistēmi"),
        "G2722": ("κατέχω", "G:V", "to hold back/fast", "katechō"),
        "G2723": ("κατηγορέω", "G:V", "to accuse", "katēgoreō"),
        "G2724": ("κατηγορία", "G:N-F", "accusation", "katēgoria"),
        "G2725": ("κατήγορος", "G:N-M", "accuser", "katēgoros"),
        "G2726": ("κατήφεια", "G:N-F", "gloominess", "katēpheia"),
        "G2727": ("κατηχέω", "G:V", "to instruct", "katēcheō"),
        "G2728": ("κατιόομαι", "G:V", "to corrode", "katioomai"),
        "G2729": ("κατισχύω", "G:V", "to prevail", "katischuō"),
        "G2730": ("κατοικέω", "G:V", "to dwell", "katoikeō"),
        "G2731": ("κατοίκησις", "G:N-F", "dwelling", "katoikēsis"),
        "G2732": ("κατοικητήριον", "G:N-N", "dwelling place", "katoikētērion"),
        "G2733": ("κατοικία", "G:N-F", "dwelling", "katoikia"),
        "G2734": ("κατοπτρίζω", "G:V", "to reflect", "katoptrizō"),
        "G2735": ("κατόρθωμα", "G:N-N", "accomplishments", "katorthōma"),
        "G2736": ("κάτω", "G:Adv", "under", "katō"),
        "G2737": ("κατώτερος", "G:A", "lower", "katōteros"),
        "G2738": ("καῦμα", "G:N-N", "heat", "kauma"),
        "G2739": ("καυματίζω", "G:V", "to scorch", "kaumatizō"),
        "G2740": ("καῦσις", "G:N-F", "burning", "kausis"),
        "G2741": ("καυσόω", "G:V", "to burn", "kausoō"),
        "G2742": ("καύσων", "G:N-M", "heat", "kausōn"),
        "G2743": ("καυτηριάζω", "G:V", "to brand", "kautēriazō"),
        "G2744": ("καυχάομαι", "G:V", "to boast", "kauchaomai"),
        "G2745": ("καύχημα", "G:N-N", "pride", "kauchēma"),
        "G2746": ("καύχησις", "G:N-F", "pride", "kauchēsis"),
        "G2747": ("Κεγχρεαί", "N:N-F-L", "Cenchrea", "Kegchreai"),
        "G2748": ("κέδρος", "N:N--L", "cedar", "kedros"),
        "G2749": ("κεῖμαι", "G:V", "to lay/be appointed", "keimai"),
        "G2750": ("κειρία", "G:N-F", "graveclothes", "keiria"),
        "G2751": ("κείρω", "G:V", "to shear", "keirō"),
        "G2752": ("κέλευσμα", "G:N-N", "command", "keleusma"),
        "G2753": ("κελεύω", "G:V", "to order", "keleuō"),
        "G2754": ("κενοδοξία", "G:N-F", "empty conceit", "kenodoxia"),
        "G2755": ("κενόδοξος", "G:A", "conceited", "kenodoxos"),
        "G2756": ("κενός", "G:A", "empty", "kenos"),
        "G2757": ("κενοφωνία", "G:N-F", "empty talk", "kenophōnia"),
        "G2758": ("κενόω", "G:V", "to empty", "kenoō"),
        "G2759": ("κέντρον", "G:N-N", "sting", "kentron"),
        "G2760": ("κεντυρίων", "G:N-M", "centurion", "kenturiōn"),
        "G2761": ("κενῶς", "G:Adv", "vainly", "kenōs"),
        "G2762": ("κεραία", "G:N-F", "tittle", "keraia"),
        "G2763": ("κεραμεύς", "G:N-M", "potter", "kerameus"),
        "G2764": ("κεραμικός", "G:A", "made of clay", "keramikos"),
        "G2765": ("κεράμιον", "G:N-N", "clay jar", "keramion"),
        "G2766": ("κέραμος", "G:N-M", "clay roof tile", "keramos"),
        "G2767": ("κεράννυμι", "G:V", "to mix", "kerannumi"),
        "G2768": ("κέρας", "G:N-N", "horn", "keras"),
        "G2769": ("κεράτιον", "G:N-N", "husk", "keration"),
        "G2770": ("κερδαίνω", "G:V", "to gain", "kerdainō"),
        "G2771": ("κέρδος", "G:N-N", "gain", "kerdos"),
        "G2772": ("κέρμα", "G:N-N", "coin", "kerma"),
        "G2773": ("κερματιστής", "G:N-M", "moneychanger", "kermatistēs"),
        "G2774": ("κεφάλαιος", "G:N-N", "sum", "kephalaios"),
        "G2775": ("κεφαλαιόω", "G:V", "to strike a head", "kephalaioō"),
        "G2776": ("κεφαλή", "G:N-F", "head", "kephalē"),
        "G2777": ("κεφαλίς", "G:N-F", "scroll", "kephalis"),
        "G2778": ("κῆνσος", "G:N-M", "tax", "kēnsos"),
        "G2779": ("κῆπος", "G:N-M", "garden", "kēpos"),
        "G2780": ("κηπουρός", "G:N-M", "gardener", "kēpouros"),
        "G2781": ("κηρίον", "G:N-N", "honeycomb", "kērion"),
        "G2782": ("κήρυγμα", "G:N-N", "preaching", "kērugma"),
        "G2783": ("κῆρυξ", "G:N-M", "preacher", "kērux"),
        "G2784": ("κηρύσσω", "G:V", "to preach", "kērussō"),
        "G2785": ("κῆτος", "G:N-N", "sea monster", "kētos"),
        "G2786": ("Κηφᾶς", "N:N-M-P", "Cephas", "Kēphas"),
        "G2787": ("κιβωτός", "G:N-F", "ark", "kibōtos"),
        "G2788": ("κιθάρα", "G:N-F", "harp", "kithara"),
        "G2789": ("κιθαρίζω", "G:V", "to play the harp", "kitharizō"),
        "G2790": ("κιθαρῳδός", "G:N-M", "harpist", "kitharōdos"),
        "G2791": ("Κιλικία, Κιλικία", "N:N-F-L", "Cilicia", "Kilikia, Kilikia"),
        "G2792": ("κινάμωμον", "G:N-N", "cinnamon", "kinamōmon"),
        "G2793": ("κινδυνεύω", "G:V", "be in danger", "kinduneuō"),
        "G2794": ("κίνδυνος", "G:N-M", "danger", "kindunos"),
        "G2795": ("κινέω", "G:V", "to move", "kineō"),
        "G2796": ("κίνησις", "G:N-F", "motion", "kinēsis"),
        "G2797": ("Κίς", "N:N-M-P", "Kish", "Kis"),
        "G2798": ("κλάδος", "G:N-M", "branch", "klados"),
        "G2799": ("κλαίω", "G:V", "to weep", "klaiō"),
        "G2800": ("κλάσις", "G:N-F", "breaking", "klasis"),
        "G2801": ("κλάσμα", "G:N-N", "fragment", "klasma"),
        "G2802": ("Κλαύδη, Καῦδα", "N:N--L", "Cauda", "Klaudē, Kauda"),
        "G2803": ("Κλαυδία", "N:N-F-P", "Claudia", "Klaudia"),
        "G2804": ("Κλαύδιος", "N:N-M-P", "Claudius", "Klaudios"),
        "G2805": ("κλαυθμός", "G:N-M", "weeping", "klauthmos"),
        "G2806": ("κλάω", "G:V", "to break", "klaō"),
        "G2807": ("κλείς", "G:N-F", "key", "kleis"),
        "G2808": ("κλείω", "G:V", "to shut", "kleiō"),
        "G2809": ("κλέμμα", "G:N-N", "theft", "klemma"),
        "G2810": ("Κλεόπας", "N:N-M-P", "Cleopas", "Kleopas"),
        "G2811": ("κλέος", "G:N-N", "credit", "kleos"),
        "G2812": ("κλέπτης", "G:N-M", "thief", "kleptēs"),
        "G2813": ("κλέπτω", "G:V", "to steal", "kleptō"),
        "G2814": ("κλῆμα", "G:N-N", "branch", "klēma"),
        "G2815": ("Κλήμης", "N:N-M-P", "Clement", "Klēmēs"),
        "G2816": ("κληρονομέω", "G:V", "to inherit", "klēronomeō"),
        "G2817": ("κληρονομία", "G:N-F", "inheritance", "klēronomia"),
        "G2818": ("κληρονόμος", "G:N-M", "heir", "klēronomos"),
        "G2819": ("κλῆρος", "G:N-M", "lot", "klēros"),
        "G2820": ("κληρόω", "G:V", "to obtain", "klēroō"),
        "G2821": ("κλῆσις", "G:N-F", "calling", "klēsis"),
        "G2822": ("κλητός", "G:A", "called", "klētos"),
        "G2823": ("κλίβανος", "G:N-M", "oven", "klibanos"),
        "G2824": ("κλίμα", "G:N-N", "region", "klima"),
        "G2825": ("κλίνη", "G:N-F", "bed", "klinē"),
        "G2826": ("κλινίδιον", "G:N-N", "bed", "klinidion"),
        "G2827": ("κλίνω", "G:V", "to bow/lay down", "klinō"),
        "G2828": ("κλισία", "G:N-F", "group", "klisia"),
        "G2829": ("κλοπή", "G:N-F", "theft", "klopē"),
        "G2830": ("κλύδων", "G:N-M", "waves", "kludōn"),
        "G2831": ("κλυδωνίζω", "G:V", "to toss", "kludōnizō"),
        "G2832": ("Κλωπᾶς", "N:N-M-P", "Clopas", "Klōpas"),
        "G2833": ("κνήθω", "G:V", "to itch", "knēthō"),
        "G2834": ("Κνίδος", "N:N-F-L", "Cnidus", "Knidos"),
        "G2835": ("κοδράντης", "G:N-M", "penny", "kodrantēs"),
        "G2836": ("κοιλία", "G:N-F", "belly/womb/stomach", "koilia"),
        "G2837": ("κοιμάω", "G:V", "to sleep", "koimaō"),
        "G2838": ("κοίμησις", "G:N-F", "sleep", "koimēsis"),
        "G2839": ("κοινός", "G:A", "common", "koinos"),
        "G2840": ("κοινόω", "G:V", "to profane", "koinoō"),
        "G2841": ("κοινωνέω", "G:V", "to participate", "koinōneō"),
        "G2842": ("κοινωνία", "G:N-F", "participation", "koinōnia"),
        "G2843": ("κοινωνικός", "G:A", "generous", "koinōnikos"),
        "G2844": ("κοινωνός", "G:N-M", "participant", "koinōnos"),
        "G2845": ("κοίτη", "G:N-F", "bed", "koitē"),
        "G2846": ("κοιτών", "G:N-M", "bedroom/chamberlain", "koitōn"),
        "G2847": ("κόκκινος", "G:A", "scarlet", "kokkinos"),
        "G2848": ("κόκκος", "G:N-M", "seed", "kokkos"),
        "G2849": ("κολάζω", "G:V", "to punish", "kolazō"),
        "G2850": ("κολακεία", "G:N-F", "flattery", "kolakeia"),
        "G2851": ("κόλασις", "G:N-F", "punishment", "kolasis"),
        "G2852": ("κολαφίζω", "G:V", "to beat", "kolaphizō"),
        "G2853": ("κολλάω", "G:V", "to join", "kollaō"),
        "G2854": ("κολλούριον", "G:N-N", "eye salve", "kollourion"),
        "G2855": ("κολλυβιστής", "G:N-M", "moneychanger", "kollubistēs"),
        "G2856": ("κολοβόω", "G:V", "to shorten", "koloboō"),
        "G2857": ("Κολοσσαί, Κολασσαί", "N:N-F-L", "Colosse", "Kolossai, Kolassai"),
        "G2858": ("Κολοσσαεύς", "G:N-M", "Colossian", "Kolossaeus"),
        "G2859": ("κόλπος", "G:N-M", "bosom", "kolpos"),
        "G2860": ("κολυμβάω", "G:V", "to swim", "kolumbaō"),
        "G2861": ("κολυμβήθρα", "G:N-F", "pool", "kolumbēthra"),
        "G2862": ("κολωνία", "G:N-F", "colony", "kolōnia"),
        "G2863": ("κομάω", "G:V", "be long-haired", "komaō"),
        "G2864": ("κόμη", "G:N-F", "hair", "komē"),
        "G2865": ("κομίζω", "G:V", "to bring/be repaid", "komizō"),
        "G2866": ("κομψότερον", "G:A", "better", "kompsoteron"),
        "G2867": ("κονιάω", "G:V", "to whitewash", "koniaō"),
        "G2868": ("κονιορτός", "G:N-M", "dust", "koniortos"),
        "G2869": ("κοπάζω", "G:V", "to abate", "kopazō"),
        "G2870": ("κοπετός", "G:N-M", "lamentation", "kopetos"),
        "G2871": ("κοπή", "G:N-F", "slaughter", "kopē"),
        "G2872": ("κοπιάω", "G:V", "to labor", "kopiaō"),
        "G2873": ("κόπος", "G:N-M", "labor", "kopos"),
        "G2874": ("κοπρία", "G:N-F/N", "manure", "kopria"),
        "G2875": ("κόπτω", "G:V", "to cut/mourn", "koptō"),
        "G2876": ("κόραξ", "G:N-M", "raven", "korax"),
        "G2877": ("κοράσιον", "G:N-N", "girl", "korasion"),
        "G2878": ("κορβᾶν", "H:N", "Corban", "korban"),
        "G2879": ("Κορέ", "N:N-M-P", "Korah", "Kore"),
        "G2880": ("κορέννυμι", "G:V", "to satisfy", "korennumi"),
        "G2881": ("Κορίνθιος", "N:N--LG", "Corinthian", "Korinthios"),
        "G2882": ("Κόρινθος", "N:N-F-L", "Corinth", "Korinthos"),
        "G2883": ("Κορνήλιος", "N:N-M-P", "Cornelius", "Kornēlios"),
        "G2884": ("κόρος", "G:N-M", "cor", "koros"),
        "G2885": ("κοσμέω", "G:V", "to arrange", "kosmeō"),
        "G2886": ("κοσμικός", "G:A", "earthly", "kosmikos"),
        "G2887": ("κόσμιος", "G:A", "respectable", "kosmios"),
        "G2888": ("κοσμοκράτωρ", "G:N-M", "world power", "kosmokratōr"),
        "G2889": ("κόσμος", "G:N-M", "world", "kosmos"),
        "G2890": ("Κούαρτος", "N:N-M-P", "Quartus", "Kouartos"),
        "G2891": ("κοῦμι", "A:V", "stand up!", "koumi"),
        "G2892": ("κουστωδία", "G:N-F", "guard", "koustōdia"),
        "G2893": ("κουφίζω", "G:V", "to lighten", "kouphizō"),
        "G2894": ("κόφινος", "G:N-M", "basket", "kophinos"),
        "G2895": ("κράββατος", "G:N-M", "bed", "krabbatos"),
        "G2896": ("κράζω", "G:V", "to cry", "krazō"),
        "G2897": ("κραιπάλη", "G:N-F", "dissipation", "kraipalē"),
        "G2898": ("κρανίον", "G:N-N", "skull", "kranion"),
        "G2899": ("κράσπεδον", "G:N-N", "edge", "kraspedon"),
        "G2900": ("κραταιός", "G:A", "mighty", "krataios"),
        "G2901": ("κραταιόω", "G:V", "to strengthen", "krataioō"),
        "G2902": ("κρατέω", "G:V", "to grasp/seize", "krateō"),
        "G2903": ("κράτιστος", "G:A", "excellent", "kratistos"),
        "G2904": ("κράτος", "G:N-N", "power", "kratos"),
        "G2905": ("κραυγάζω", "G:V", "to shout", "kraugazō"),
        "G2906": ("κραυγή", "G:N-F", "shouting", "kraugē"),
        "G2907": ("κρέας", "G:N-N", "meat", "kreas"),
        "G2908": ("κρείσσων", "G:A", "greater", "kreissōn"),
        "G2909": ("κρείττων", "G:A", "stronger", "kreittōn"),
        "G2910": ("κρεμάω", "G:V", "to hang", "kremaō"),
        "G2911": ("κρημνός", "G:N-M", "cliff", "krēmnos"),
        "G2912": ("Κρής", "N:N--LG", "Cretan", "Krēs"),
        "G2913": ("Κρήσκης", "N:N-M-P", "Crescens", "Krēskēs"),
        "G2914": ("Κρήτη", "N:N-F-L", "Crete", "Krētē"),
        "G2915": ("κριθή", "G:N-F", "barley", "krithē"),
        "G2916": ("κρίθινος", "G:A", "barley", "krithinos"),
        "G2917": ("κρίμα", "G:N-N", "judgment", "krima"),
        "G2918": ("κρίνον", "G:N-N", "lily", "krinon"),
        "G2919": ("κρίνω", "G:V", "to judge", "krinō"),
        "G2920": ("κρίσις", "G:N-F", "judgment", "krisis"),
        "G2921": ("Κρίσπος", "N:N-M-P", "Crispus", "Krispos"),
        "G2922": ("κριτήριον", "G:N-N", "court/lawsuit", "kritērion"),
        "G2923": ("κριτής", "G:N-M", "judge", "kritēs"),
        "G2924": ("κριτικός", "G:A", "discerning", "kritikos"),
        "G2925": ("κρούω", "G:V", "to knock", "krouō"),
        "G2926": ("κρύπτη", "G:N-F", "cellar", "kruptē"),
        "G2927": ("κρυπτός", "G:A", "hidden", "kruptos"),
        "G2928": ("κρύπτω", "G:V", "to hide", "kruptō"),
        "G2929": ("κρυσταλλίζω", "G:V", "be clear", "krustallizō"),
        "G2930": ("κρύσταλλος", "G:N-M", "crystal", "krustallos"),
        "G2931": ("κρυφῇ", "G:Adv", "in secret", "kruphē"),
        "G2932": ("κτάομαι", "G:V", "to posses", "ktaomai"),
        "G2933": ("κτῆμα", "G:N-N", "possession", "ktēma"),
        "G2934": ("κτῆνος", "G:N-N", "animal", "ktēnos"),
        "G2935": ("κτήτωρ", "G:N-M", "owner", "ktētōr"),
        "G2936": ("κτίζω", "G:V", "to create", "ktizō"),
        "G2937": ("κτίσις", "G:N-F", "creation", "ktisis"),
        "G2938": ("κτίσμα", "G:N-N", "creature", "ktisma"),
        "G2939": ("κτίστης", "G:N-M", "creator", "ktistēs"),
        "G2940": ("κυβεία", "G:N-F", "cunning", "kubeia"),
        "G2941": ("κυβέρνησις", "G:N-F", "administration", "kubernēsis"),
        "G2942": ("κυβερνήτης", "G:N-M", "captain", "kubernētēs"),
        "G2943": ("κυκλόθεν", "G:Prep", "around", "kuklothen"),
        "G2944": ("κυκλόω", "G:V", "to surround", "kukloō"),
        "G2945": ("κύκλῳ", "G:Adv", "surrounding", "kuklō"),
        "G2946": ("κύλισμα", "G:N-M/N", "wallowing", "kulisma"),
        "G2947": ("κυλίω", "G:V", "to wallow", "kuliō"),
        "G2948": ("κυλλός", "G:A", "crippled", "kullos"),
        "G2949": ("κῦμα", "G:N-N", "a wave", "kuma"),
        "G2950": ("κύμβαλον", "G:N-N", "cymbal", "kumbalon"),
        "G2951": ("κύμινον", "G:N-N", "cumin", "kuminon"),
        "G2952": ("κυνάριον", "G:N-N", "little dog", "kunarion"),
        "G2953": ("Κύπριος", "N:N--LG", "Cyprian", "Kuprios"),
        "G2954": ("Κύπρος", "N:N-F-L", "Cyprus", "Kupros"),
        "G2955": ("κύπτω", "G:V", "to stoop/bend down", "kuptō"),
        "G2956": ("Κυρηναῖος", "N:N--LG", "from Cyrene", "Kurēnaios"),
        "G2957": ("Κυρήνη", "N:N-F-L", "Cyrene", "Kurēnē"),
        "G2958": ("Κυρήνιος", "N:N-M-P", "Quirinius", "Kurēnios"),
        "G2959": ("Κυρία", "G:N-F", "lady", "Kuria"),
        "G2960": ("κυριακός", "G:A", "the Lord’s", "kuriakos"),
        "G2961": ("κυριεύω", "G:V", "to lord over", "kurieuō"),
        "G2962": ("κύριος", "G:N-M", "lord", "kurios"),
        "G2963": ("κυριότης", "G:N-F", "lordship", "kuriotēs"),
        "G2964": ("κυρόω", "G:V", "to affirm", "kuroō"),
        "G2965": ("κύων", "G:N-M", "dog", "kuōn"),
        "G2966": ("κῶλον", "G:N-N", "corpse", "kōlon"),
        "G2967": ("κωλύω", "G:V", "to prevent", "kōluō"),
        "G2968": ("κώμη", "G:N-F", "village", "kōmē"),
        "G2969": ("κωμόπολις", "G:N-F", "village", "kōmopolis"),
        "G2970": ("κῶμος", "G:N-M", "orgy", "kōmos"),
        "G2971": ("κώνωψ", "G:N-M", "gnat", "kōnōps"),
        "G2972": ("Κώς", "N:N-F-L", "Cos", "Kōs"),
        "G2973": ("Κωσάμ", "N:N-M-P", "Cosam", "Kōsam"),
        "G2974": ("κωφός", "G:A", "deaf/mute", "kōphos"),
        "G2975": ("λαγχάνω", "G:V", "to choose by lot", "lagchanō"),
        "G2976": ("Λάζαρος", "N:N-M-P", "Lazarus", "Lazaros"),
        "G2977": ("λάθρα", "G:Adv", "quietly", "lathra"),
        "G2978": ("λαῖλαψ", "G:N-F", "storm", "lailaps"),
        "G2979": ("λακτίζω", "G:V", "to kick", "laktizō"),
        "G2980": ("λαλέω", "G:V", "to speak", "laleō"),
        "G2981": ("λαλιά", "G:N-F", "speech", "lalia"),
        "G2982": ("λαμά", "A:Intg", "why?", "lama"),
        "G2983": ("λαμβάνω", "G:V", "to take", "lambanō"),
        "G2984": ("Λάμεχ", "N:N-M-P", "Lamech", "Lamech"),
        "G2985": ("λαμπάς", "G:N-F", "window", "lampas"),
        "G2986": ("λαμπρός", "G:A", "shining", "lampros"),
        "G2987": ("λαμπρότης", "G:N-F", "brightness", "lamprotēs"),
        "G2988": ("λαμπρῶς", "G:Adv", "magnificently", "lamprōs"),
        "G2989": ("λάμπω", "G:V", "to shine", "lampō"),
        "G2990": ("λανθάνω", "G:V", "be hidden", "lanthanō"),
        "G2991": ("λαξευτός", "G:A", "engraved", "laxeutos"),
        "G2992": ("λαός", "G:N-M", "a people", "laos"),
        "G2993": ("Λαοδίκεια", "N:N-F-L", "Laodicea", "Laodikeia"),
        "G2994": ("Λαοδικεύς", "N:N--LG", "Laodicean", "Laodikeus"),
        "G2995": ("λάρυγξ", "G:N-M", "throat", "larugx"),
        "G2996": ("Λασαία", "N:N-F-L", "Lasea", "Lasaia"),
        "G2997": ("λάσχω", "G:V", "to burst", "laschō"),
        "G2998": ("λατομέω", "G:V", "to hew", "latomeō"),
        "G2999": ("λατρεία", "G:N-F", "ministry", "latreia"),
        "G3000": ("λατρεύω", "G:V", "to minister", "latreuō"),
        "G3001": ("λάχανον", "G:N-N", "plant", "lachanon"),
        "G3002": ("Λεββαῖος", "N:N-M-P", "Lebbaeus", "Lebbaios"),
        "G3003": ("λεγεών", "G:N-M/F", "legion", "legeōn"),
        "G3004": ("λέγω", "G:V", "to speak", "legō"),
        "G3005": ("λεῖμμα", "G:N-N", "remnant", "leimma"),
        "G3006": ("λεῖος", "G:A", "smooth", "leios"),
        "G3007": ("λείπω", "G:V", "to lack", "leipō"),
        "G3008": ("λειτουργέω", "G:V", "to minister", "leitourgeō"),
        "G3009": ("λειτουργία", "G:N-F", "ministry", "leitourgia"),
        "G3010": ("λειτουργικός", "G:A", "ministering", "leitourgikos"),
        "G3011": ("λειτουργός", "G:N-M", "minister", "leitourgos"),
        "G3012": ("λέντιον", "G:N-N", "towel", "lention"),
        "G3013": ("λεπίς", "G:N-F", "scale", "lepis"),
        "G3014": ("λέπρα", "G:N-F", "leprosy", "lepra"),
        "G3015": ("λεπρός", "G:A", "leprous", "lepros"),
        "G3016": ("λεπτός", "G:N-N", "coin", "leptos"),
        "G3017": ("Λευΐ", "N:N-M-P", "Levi", "Leui"),
        "G3018": ("Λευΐς", "N:N-M-P", "Levi", "Leuis"),
        "G3019": ("Λευΐτης", "N:N--PG", "Levite", "Leuitēs"),
        "G3020": ("Λευϊτικός", "N:N--PG", "Levitical", "Leuitikos"),
        "G3021": ("λευκαίνω", "G:V", "to bleach", "leukainō"),
        "G3022": ("λευκός", "G:A", "white", "leukos"),
        "G3023": ("λέων", "G:N-M", "lion", "leōn"),
        "G3024": ("λήθη", "G:N-F", "forgetfulness", "lēthē"),
        "G3025": ("ληνός", "G:N-F", "winepress", "lēnos"),
        "G3026": ("λῆρος", "G:N-M", "nonsense", "lēros"),
        "G3027": ("λῃστής", "G:N-M", "robber/rebel", "lēstēs"),
        "G3028": ("λῆψις", "G:N-F", "receiving", "lēpsis"),
        "G3029": ("λίαν", "G:Adv", "greatly", "lian"),
        "G3030": ("λίβανος", "G:N-M", "frankincense", "libanos"),
        "G3031": ("λιβανωτός", "G:N-M", "censer", "libanōtos"),
        "G3032": ("Λιβερτῖνος", "N:N--T", "Freedman", "Libertinos"),
        "G3033": ("Λιβύη", "N:N-F-L", "Libya", "Libuē"),
        "G3034": ("λιθάζω", "G:V", "to stone", "lithazō"),
        "G3035": ("λίθινος", "G:A", "stone", "lithinos"),
        "G3036": ("λιθοβολέω", "G:V", "to stone", "lithoboleō"),
        "G3037": ("λίθος", "G:N-M", "stone", "lithos"),
        "G3038": ("λιθόστρωτος", "G:A", "pavement", "lithostrōtos"),
        "G3039": ("λικμάω", "G:V", "to crush", "likmaō"),
        "G3040": ("λιμήν", "G:N-M", "harbor", "limēn"),
        "G3041": ("λίμνη", "G:N-F", "lake", "limnē"),
        "G3042": ("λιμός", "G:N-M/F", "hunger", "limos"),
        "G3043": ("λίνον", "G:N-N", "linen/wick", "linon"),
        "G3044": ("Λίνος", "N:N-M-P", "Linus", "Linos"),
        "G3045": ("λιπαρός", "G:A", "rich", "liparos"),
        "G3046": ("λίτρα", "G:N-F", "pound", "litra"),
        "G3047": ("λίψ", "G:N-M", "southwest", "lips"),
        "G3048": ("λογία", "G:N-F", "collection", "logia"),
        "G3049": ("λογίζομαι", "G:V", "to count", "logizomai"),
        "G3050": ("λογικός", "G:A", "spiritual", "logikos"),
        "G3051": ("λόγιον", "G:N-N", "oracles", "logion"),
        "G3052": ("λόγιος", "G:A", "learned", "logios"),
        "G3053": ("λογισμός", "G:N-M", "thought", "logismos"),
        "G3054": ("λογομαχέω", "G:V", "to quarrel", "logomacheō"),
        "G3055": ("λογομαχία", "G:N-F", "quarrel", "logomachia"),
        "G3056": ("λόγος", "G:N-M", "word", "logos"),
        "G3057": ("λόγχη", "G:N-F", "spear", "logchē"),
        "G3058": ("λοιδορέω", "G:V", "to revile", "loidoreō"),
        "G3059": ("λοιδορία", "G:N-F", "reviling", "loidoria"),
        "G3060": ("λοίδορος", "G:N-M", "reviler", "loidoros"),
        "G3061": ("λοιμός", "G:N-M", "pestilence", "loimos"),
        "G3062": ("λοιπός", "G:A", "remaining", "loipos"),
        "G3063": ("λοιπόν", "G:A", "henceforth", "loipon"),
        "G3064": ("λοιποῦ", "G:A", "henceforth", "loipou"),
        "G3065": ("Λουκᾶς", "N:N-M-P", "Luke", "Loukas"),
        "G3066": ("Λούκιος", "N:N-M-P", "Lucius", "Loukios"),
        "G3067": ("λουτρόν", "G:N-N", "washing", "loutron"),
        "G3068": ("λούω", "G:V", "to wash", "louō"),
        "G3069": ("Λύδδα", "N:N-F-L", "Lydda", "Ludda"),
        "G3070": ("Λυδία", "N:N-F-P", "Lydia", "Ludia"),
        "G3071": ("Λυκαονία", "N:N-F-L", "Lycaonia", "Lukaonia"),
        "G3072": ("Λυκαονιστί", "G:Adv", "in Lycaonian", "Lukaonisti"),
        "G3073": ("Λυκία", "N:N-F-L", "Lycia", "Lukia"),
        "G3074": ("λύκος", "G:N-M", "wolf", "lukos"),
        "G3075": ("λυμαίνομαι", "G:V", "to ravage", "lumainomai"),
        "G3076": ("λυπέω", "G:V", "to grieve", "lupeō"),
        "G3077": ("λύπη", "G:N-F", "grief", "lupē"),
        "G3078": ("Λυσανίας", "N:N-M-P", "Lysanias", "Lusanias"),
        "G3079": ("Λυσίας", "N:N-M-P", "Lysias", "Lusias"),
        "G3080": ("λύσις", "G:N-F", "divorce", "lusis"),
        "G3081": ("λυσιτελεῖ", "G:V", "to profit", "lusitelei"),
        "G3082": ("Λύστρα", "N:N--L", "Lystra", "Lustra"),
        "G3083": ("λύτρον", "G:N-N", "ransom", "lutron"),
        "G3084": ("λυτρόω", "G:V", "to ransom", "lutroō"),
        "G3085": ("λύτρωσις", "G:N-F", "redemption", "lutrōsis"),
        "G3086": ("λυτρωτής", "G:N-M", "redeemer", "lutrōtēs"),
        "G3087": ("λυχνία", "G:N-F", "lampstand", "luchnia"),
        "G3088": ("λύχνος", "G:N-M", "lamp", "luchnos"),
        "G3089": ("λύω", "G:V", "to loose", "luō"),
        "G3090": ("Λωΐς", "N:N-F-P", "Lois", "Lōis"),
        "G3091": ("Λώτ", "N:N-M-P", "Lot", "Lōt"),
        "G3092": ("Μαάθ", "N:N-M-P", "Maath", "Maath"),
        "G3093": ("Μαγδαλά, Μαγαδάν", "N:N--L", "Magadan", "Magdala, Magadan"),
        "G3094": ("Μαγδαληνή", "N:N-F-LG", "Magdalene", "Magdalēnē"),
        "G3095": ("μαγεία", "G:N-F", "magic", "mageia"),
        "G3096": ("μαγεύω", "G:V", "to practice magic", "mageuō"),
        "G3097": ("μάγος", "G:N-M", "sage", "magos"),
        "G3098": ("Μαγώγ", "N:N--L", "Magog", "Magōg"),
        "G3099": ("Μαδιάν", "N:N--L", "Midian", "Madian"),
        "G3100": ("μαθητεύω", "G:V", "to disciple", "mathēteuō"),
        "G3101": ("μαθητής", "G:N-M", "disciple", "mathētēs"),
        "G3102": ("μαθήτρια", "G:N-F", "disciple", "mathētria"),
        "G3103": ("Μαθουσάλα", "N:N-M-P", "Methuselah", "Mathousala"),
        "G3104": ("Μαϊνάν", "N:N-M-P", "Menna", "Mainan"),
        "G3105": ("μαίνομαι", "G:V", "to rave", "mainomai"),
        "G3106": ("μακαρίζω", "G:V", "to bless", "makarizō"),
        "G3107": ("μακάριος", "G:A", "blessed", "makarios"),
        "G3108": ("μακαρισμός", "G:N-M", "blessedness", "makarismos"),
        "G3109": ("Μακεδονία", "N:N-F-L", "Macedonia", "Makedonia"),
        "G3110": ("Μακεδών", "N:N--LG", "Macedonian", "Makedōn"),
        "G3111": ("μάκελλον", "G:N-N", "meat market", "makellon"),
        "G3112": ("μακράν", "G:Adv", "far", "makran"),
        "G3113": ("μακρόθεν", "G:Adv", "from afar", "makrothen"),
        "G3114": ("μακροθυμέω", "G:V", "to have patience", "makrothumeō"),
        "G3115": ("μακροθυμία", "G:N-F", "patience", "makrothumia"),
        "G3116": ("μακροθυμώς", "G:Adv", "patiently", "makrothumōs"),
        "G3117": ("μακρός", "G:A", "long/distant", "makros"),
        "G3118": ("μακροχρόνιος", "G:A", "long-lived", "makrochronios"),
        "G3119": ("μαλακία", "G:N-F", "sickness", "malakia"),
        "G3120": ("μαλακός", "G:A", "soft/effeminate", "malakos"),
        "G3121": ("Μαλελεήλ", "N:N-M-P", "Mahalalel", "Maleleēl"),
        "G3122": ("μάλιστα", "G:Adv", "especially", "malista"),
        "G3123": ("μᾶλλον", "G:Adv", "more", "mallon"),
        "G3124": ("Μάλχος", "N:N-M-P", "Malchus", "Malchos"),
        "G3125": ("μάμμη", "G:N-F", "grandmother", "mammē"),
        "G3126": ("μαμμωνᾶς", "G:N-M", "wealth", "mammōnas"),
        "G3127": ("Μαναήν", "N:N-M-P", "Manaen", "Manaēn"),
        "G3128": ("Μανασσῆς", "N:N-M-P", "Manasseh", "Manassēs"),
        "G3129": ("μανθάνω", "G:V", "to learn", "manthanō"),
        "G3130": ("μανία", "G:N-F", "insanity", "mania"),
        "G3131": ("μάννα", "H:N", "manna", "manna"),
        "G3132": ("μαντεύομαι", "G:V", "to divine", "manteuomai"),
        "G3133": ("μαραίνω", "G:V", "to fade", "marainō"),
        "G3134": ("μαρὰν ἀθά", "A:V + A:N", "Come, Lord!", "maran atha"),
        "G3135": ("μαργαρίτης", "G:N-M", "pearl", "margaritēs"),
        "G3136": ("Μάρθα", "N:N-F-P", "Martha", "Martha"),
        "G3137": ("Μαρία", "N:N-F-P", "Mary", "Maria"),
        "G3138": ("Μάρκος", "N:N-M-P", "Mark", "Markos"),
        "G3139": ("μάρμαρος", "G:N-M", "marble", "marmaros"),
        "G3140": ("μαρτυρέω", "G:V", "to testify", "martureō"),
        "G3141": ("μαρτυρία", "G:N-F", "testimony", "marturia"),
        "G3142": ("μαρτύριον", "G:N-N", "testimony", "marturion"),
        "G3143": ("μαρτύρομαι", "G:V", "to testify", "marturomai"),
        "G3144": ("μάρτυς", "G:N-M", "witness", "martus"),
        "G3145": ("μασσάομαι", "G:V", "to gnaw", "massaomai"),
        "G3146": ("μαστιγόω", "G:V", "to whip", "mastigoō"),
        "G3147": ("μαστίζω", "G:V", "to whip", "mastizō"),
        "G3148": ("μάστιξ", "G:N-F", "whip", "mastix"),
        "G3149": ("μαστός, μαζός", "G:N-M", "breast", "mastos, mazos"),
        "G3150": ("ματαιολογία", "G:N-F", "babble", "mataiologia"),
        "G3151": ("ματαιολόγος", "G:A", "babbler", "mataiologos"),
        "G3152": ("μάταιος", "G:A", "futile", "mataios"),
        "G3153": ("ματαιότης", "G:N-F", "futility", "mataiotēs"),
        "G3154": ("ματαιόω", "G:V", "to make futile", "mataioō"),
        "G3155": ("μάτην", "G:Adv", "in vain", "matēn"),
        "G3156": ("Ματθαῖος", "N:N-M-P", "Matthew", "Matthaios"),
        "G3157": ("Ματθάν", "N:N-M-P", "Matthan", "Matthan"),
        "G3158": ("Ματθάτ, Μαθθάτ", "N:N-M-P", "Matthat", "Matthat, Maththat"),
        "G3159": ("Ματθίας, Μαθθίας", "N:N-M-P", "Matthias", "Matthias, Maththias"),
        "G3160": ("Ματταθά", "N:N-M-P", "Mattatha", "Mattatha"),
        "G3161": ("Ματταθίας", "N:N-M-P", "Mattathias", "Mattathias"),
        "G3162": ("μάχαιρα", "G:N-F", "sword", "machaira"),
        "G3163": ("μάχη", "G:N-F", "quarrel", "machē"),
        "G3164": ("μάχομαι", "G:V", "to quarrel", "machomai"),
        "G3165": ("μέ", "G:PerP", "me", "me"),
        "G3166": ("μεγαλαυχέω", "G:V", "to boast", "megalaucheō"),
        "G3167": ("μεγαλεῖος", "G:A", "mighty", "megaleios"),
        "G3168": ("μεγαλειότης", "G:N-F", "majesty", "megaleiotēs"),
        "G3169": ("μεγαλοπρεπής", "G:A", "majestic", "megaloprepēs"),
        "G3170": ("μεγαλύνω", "G:V", "to magnify", "megalunō"),
        "G3171": ("μεγάλως", "G:Adv", "greatly", "megalōs"),
        "G3172": ("μεγαλωσύνη", "G:N-F", "majesty", "megalōsunē"),
        "G3173": ("μέγας", "G:A", "great", "megas"),
        "G3174": ("μέγεθος", "G:N-N", "greatness", "megethos"),
        "G3175": ("μεγιστάν", "G:N-M", "great man", "megistan"),
        "G3176": ("μέγιστος", "G:A", "greatest", "megistos"),
        "G3177": ("μεθερμηνεύω", "G:V", "to mean", "methermēneuō"),
        "G3178": ("μέθη", "G:N-F", "drunkenness", "methē"),
        "G3179": ("μεθίστημι", "G:V", "to move", "methistēmi"),
        "G3180": ("μεθοδεία", "G:N-F", "scheme", "methodeia"),
        "G3181": ("μεθόριος", "G:N-N", "boundary", "methorios"),
        "G3182": ("μεθύσκω", "G:V", "to get drunk", "methuskō"),
        "G3183": ("μέθυσος", "G:N-M", "drunkard", "methusos"),
        "G3184": ("μεθύω", "G:V", "to get drunk", "methuō"),
        "G3185": ("μεῖζον", "G:Adv", "greater", "meizon"),
        "G3186": ("μειζότερος", "G:Adv", "greatest", "meizoteros"),
        "G3187": ("μείζων", "G:A", "greater", "meizōn"),
        "G3188": ("μέλαν", "G:N-N", "ink", "melan"),
        "G3189": ("μέλας", "G:A", "black", "melas"),
        "G3190": ("Μελεᾶς", "N:N-M-P", "Melea", "Meleas"),
        "G3191": ("μελετάω", "G:V", "to meditate/plot", "meletaō"),
        "G3192": ("μέλι", "G:N-N", "honey", "meli"),
        "G3193": ("μελίσσιος", "G:A", "honeycomb", "melissios"),
        "G3194": ("Μελίτη", "N:N-F-L", "Malta", "Melitē"),
        "G3195": ("μέλλω", "G:V", "to ensue", "mellō"),
        "G3196": ("μέλος", "G:N-N", "member", "melos"),
        "G3197": ("Μελχί", "N:N-M-P", "Melki", "Melchi"),
        "G3198": ("Μελχισεδέκ", "N:N-M-P", "Melchizedek", "Melchisedek"),
        "G3199": ("μέλω", "G:V", "to concern", "melō"),
        "G3200": ("μεμβράνα", "G:N-F", "parchment", "membrana"),
        "G3201": ("μέμφομαι", "G:V", "to blame", "memphomai"),
        "G3202": ("μεμψίμοιρος", "G:A", "malcontent", "mempsimoiros"),
        "G3303": ("μέν", "G:Part", "on the other hand", "men"),
        "G3304": ("μενοῦνγε", "G:Part", "rather", "menounge"),
        "G3305": ("μέντοι", "G:Conj", "yet", "mentoi"),
        "G3306": ("μένω", "G:V", "to stay", "menō"),
        "G3307": ("μερίζω", "G:V", "to divide", "merizō"),
        "G3308": ("μέριμνα", "G:N-F", "concern", "merimna"),
        "G3309": ("μεριμνάω", "G:V", "to worry", "merimnaō"),
        "G3310": ("μερίς", "G:N-F", "part", "meris"),
        "G3311": ("μερισμός", "G:N-M", "division", "merismos"),
        "G3312": ("μεριστής", "G:N-M", "arbiter", "meristēs"),
        "G3313": ("μέρος", "G:N-N", "part", "meros"),
        "G3314": ("μεσημβρία", "G:N-F", "noon/south", "mesēmbria"),
        "G3315": ("μεσιτεύω", "G:V", "to guarantee", "mesiteuō"),
        "G3316": ("μεσίτης", "G:N-M", "mediator", "mesitēs"),
        "G3317": ("μεσονύκτιος", "G:N-N", "midnight", "mesonuktios"),
        "G3318": ("Μεσοποταμία", "N:N-F-L", "Mesopotamia", "Mesopotamia"),
        "G3319": ("μέσος", "G:A", "midst", "mesos"),
        "G3320": ("μεσότοιχος", "G:N-N", "wall", "mesotoichos"),
        "G3321": ("μεσουράνημα", "G:N-N", "midair", "mesouranēma"),
        "G3322": ("μεσόω", "G:V", "be in the middle", "mesoō"),
        "G3323": ("Μεσσίας", "N:N--T", "Messiah", "Messias"),
        "G3324": ("μεστός", "G:A", "full", "mestos"),
        "G3325": ("μεστόω", "G:V", "to fill", "mestoō"),
        "G3326": ("μετά", "G:Prep", "with/after", "meta"),
        "G3327": ("μεταβαίνω", "G:V", "to depart", "metabainō"),
        "G3328": ("μεταβάλλω", "G:V", "to change mind", "metaballō"),
        "G3329": ("μετάγω", "G:V", "to turn", "metagō"),
        "G3330": ("μεταδίδωμι", "G:V", "to share", "metadidōmi"),
        "G3331": ("μετάθεσις", "G:N-F", "removal/change", "metathesis"),
        "G3332": ("μεταίρω", "G:V", "to leave", "metairō"),
        "G3333": ("μετακαλέω", "G:V", "to summon", "metakaleō"),
        "G3334": ("μετακινέω", "G:V", "to shift", "metakineō"),
        "G3335": ("μεταλαμβάνω", "G:V", "to partake", "metalambanō"),
        "G3336": ("μετάληψις", "G:N-F", "partaking of", "metalēpsis"),
        "G3337": ("μεταλλάσσω", "G:V", "to exchange", "metallassō"),
        "G3338": ("μεταμέλλομαι", "G:V", "to repent", "metamellomai"),
        "G3339": ("μεταμορφόω", "G:V", "to transform", "metamorphoō"),
        "G3340": ("μετανοέω", "G:V", "to repent", "metanoeō"),
        "G3341": ("μετάνοια", "G:N-F", "repentance", "metanoia"),
        "G3342": ("μεταξύ", "G:Prep", "between/meanwhile", "metaxu"),
        "G3343": ("μεταπέμπω", "G:V", "to summon", "metapempō"),
        "G3344": ("μεταστρέφω", "G:V", "to change", "metastrephō"),
        "G3345": ("μετασχηματίζω", "G:V", "to transform", "metaschēmatizō"),
        "G3346": ("μετατίθημι", "G:V", "to transport", "metatithēmi"),
        "G3347": ("μετέπειτα", "G:Adv", "afterward", "metepeita"),
        "G3348": ("μετέχω", "G:V", "to share", "metechō"),
        "G3349": ("μετεωρίζω", "G:V", "to worry", "meteōrizō"),
        "G3350": ("μετοικεσία", "G:N-F", "deportation", "metoikesia"),
        "G3351": ("μετοικίζω", "G:V", "to deport", "metoikizō"),
        "G3352": ("μετοχή", "G:N-F", "participation", "metochē"),
        "G3353": ("μέτοχος", "G:A", "partaker", "metochos"),
        "G3354": ("μετρέω", "G:V", "to measure", "metreō"),
        "G3355": ("μετρητής", "G:N-M", "measure", "metrētēs"),
        "G3356": ("μετριοπαθέω", "G:V", "be gentle", "metriopatheō"),
        "G3357": ("μετρίως", "G:Adv", "moderately", "metriōs"),
        "G3358": ("μέτρον", "G:N-N", "measure", "metron"),
        "G3359": ("μέτωπον", "G:N-N", "forehead", "metōpon"),
        "G3360": ("μέχρι", "G:Prep", "until", "mechri"),
        "G3361": ("μή", "G:Neg", "not", "mē"),
        "G3362": ("ἐὰν μή", "G:Cond + G:Neg", "unless", "ean mē"),
        "G3363": ("ἵνα μή", "G:Adv", "lest/so that... not", "hina mē"),
        "G3364": ("οὐ μή", "G:Neg + G:Neg", "never", "ou mē"),
        "G3365": ("μηδαμῶς", "G:Neg", "surely not", "mēdamōs"),
        "G3366": ("μηδέ", "G:Neg+G:Conj", "nor", "mēde"),
        "G3367": ("μηδείς", "G:Neg", "nothing", "mēdeis"),
        "G3368": ("μηδέποτε", "G:Adv", "never", "mēdepote"),
        "G3369": ("μηδέπω", "G:Adv", "not yet", "mēdepō"),
        "G3370": ("Μῆδος", "N:N--LG", "Mede", "Mēdos"),
        "G3371": ("μηκέτι", "G:Adv", "never again", "mēketi"),
        "G3372": ("μῆκος", "G:N-N", "length", "mēkos"),
        "G3373": ("μηκύνω", "G:V", "to grow", "mēkunō"),
        "G3374": ("μηλωτή", "G:N-F", "sheepskin", "mēlōtē"),
        "G3375": ("μήν", "G:Part", "certainly", "mēn"),
        "G3376": ("μήν", "G:N-M", "month", "mēn"),
        "G3377": ("μηνύω", "G:V", "to disclose", "mēnuō"),
        "G3378": ("μὴ οὐκ", "G:Neg + G:Neg", "isn't it?", "mē ouk"),
        "G3379": ("μήποτε", "G:Neg", "lest", "mēpote"),
        "G3380": ("μήπω", "G:Adv", "not yet", "mēpō"),
        "G3381": ("μήπως", "G:Neg", "so that", "mēpōs"),
        "G3382": ("μηρός", "G:N-M", "thigh", "mēros"),
        "G3383": ("μήτε", "G:Neg+G:Conj", "neither", "mēte"),
        "G3384": ("μήτηρ", "G:N-F", "mother", "mētēr"),
        "G3385": ("μήτι", "G:Neg+G:Conj", "surely not", "mēti"),
        "G3386": ("γε", "G:Part", "how much more", "mētige"),
        "G3387": ("μήτις", "G:Part", "unless/any", "mētis"),
        "G3388": ("μήτρα", "G:N-F", "womb", "mētra"),
        "G3389": ("μητραλῴας", "G:N-M", "matricide", "mētralōas"),
        "G3390": ("μητρόπολις", "G:N-F", "capital", "mētropolis"),
        "G3391": ("μία", "G:A", "first", "mia"),
        "G3392": ("μιαίνω", "G:V", "to stain", "miainō"),
        "G3393": ("μίασμα", "G:N-N", "defilement", "miasma"),
        "G3394": ("μιασμός", "G:N-M", "defilement", "miasmos"),
        "G3395": ("μίγμα", "G:N-N", "mixture", "migma"),
        "G3396": ("μίγνυμι", "G:V", "to mix", "mignumi"),
        "G3397": ("μικρόν", "G:N-N", "little", "mikron"),
        "G3398": ("μικρός", "G:A", "small", "mikros"),
        "G3399": ("Μίλητος", "N:N-F-L", "Miletus", "Milētos"),
        "G3400": ("μίλιον", "G:N-N", "mile", "milion"),
        "G3401": ("μιμέομαι", "G:V", "to imitate", "mimeomai"),
        "G3402": ("μιμητής", "G:N-M", "imitator", "mimētēs"),
        "G3403": ("μιμνήσκω", "G:V", "to remember", "mimnēskō"),
        "G3404": ("μισέω", "G:V", "to hate", "miseō"),
        "G3405": ("μισθαποδοσία", "G:N-F", "recompense", "misthapodosia"),
        "G3406": ("μισθαποδότης", "G:N-M", "rewarder", "misthapodotēs"),
        "G3407": ("μίσθιος", "G:A", "hired worker", "misthios"),
        "G3408": ("μισθός", "G:N-M", "wage", "misthos"),
        "G3409": ("μισθόω", "G:V", "to hire", "misthoō"),
        "G3410": ("μίσθωμα", "G:N-N", "rented home", "misthōma"),
        "G3411": ("μισθωτός", "G:N-M", "hired worker", "misthōtos"),
        "G3412": ("Μιτυλήνη", "N:N-F-L", "Mitylene", "Mitulēnē"),
        "G3413": ("Μιχαήλ", "N:N-M-P", "Michael", "Michaēl"),
        "G3414": ("μνᾶ", "G:N-F", "mina", "mna"),
        "G3415": ("μνάομαι", "G:V", "to remember", "mnaomai"),
        "G3416": ("Μνάσων", "N:N-M-P", "Mnason", "Mnasōn"),
        "G3417": ("μνεία", "G:N-F", "remembrance", "mneia"),
        "G3418": ("μνῆμα", "G:N-N", "tomb", "mnēma"),
        "G3419": ("μνημεῖον", "G:N-N", "grave", "mnēmeion"),
        "G3420": ("μνήμη", "G:N-F", "remembrance", "mnēmē"),
        "G3421": ("μνημονεύω", "G:V", "to remember", "mnēmoneuō"),
        "G3422": ("μνημόσυνον", "G:N-N", "memorial", "mnēmosunon"),
        "G3423": ("μνηστεύω", "G:V", "to betroth", "mnēsteuō"),
        "G3424": ("μογιλάλος", "G:A", "hardly talking", "mogilalos"),
        "G3425": ("μόγις", "G:Adv", "hardly", "mogis"),
        "G3426": ("μόδιος", "G:N-M", "bucket", "modios"),
        "G3427": ("μοί", "G:PerP", "to me", "moi"),
        "G3428": ("μοιχαλίς", "G:A", "adulterous", "moichalis"),
        "G3429": ("μοιχάω", "G:V", "to commit adultery", "moichaō"),
        "G3430": ("μοιχεία", "G:N-F", "adultery", "moicheia"),
        "G3431": ("μοιχεύω", "G:V", "to commit adultery", "moicheuō"),
        "G3432": ("μοιχός", "G:N-M", "adulterer", "moichos"),
        "G3433": ("μόλις", "G:Adv", "hardly", "molis"),
        "G3434": ("Μολόχ", "N:N--T", "Molech", "Moloch"),
        "G3435": ("μολύνω", "G:V", "to defile", "molunō"),
        "G3436": ("μολυσμός", "G:N-M", "defilement", "molusmos"),
        "G3437": ("μομφή, μέμψις", "G:N-F", "complaint", "momphē, mempsis"),
        "G3438": ("μονή", "G:N-F", "abode", "monē"),
        "G3439": ("μονογενής", "G:A", "unique", "monogenēs"),
        "G3440": ("μόνον", "G:Adv", "only", "monon"),
        "G3441": ("μόνος", "G:A", "alone", "monos"),
        "G3442": ("μονόφθαλμος", "G:A", "one-eyed", "monophthalmos"),
        "G3443": ("μονόω", "G:V", "to leave alone", "monoō"),
        "G3444": ("μορφή", "G:N-F", "form", "morphē"),
        "G3445": ("μορφόω", "G:V", "to form", "morphoō"),
        "G3446": ("μόρφωσις", "G:N-F", "form", "morphōsis"),
        "G3447": ("μοσχοποιέω", "G:V", "to make a calf", "moschopoieō"),
        "G3448": ("μόσχος", "G:N-M", "calf", "moschos"),
        "G3449": ("μόχθος", "G:N-M", "toil", "mochthos"),
        "G3450": ("μοῦ", "G:PerP", "of me", "mou"),
        "G3451": ("μουσικός", "G:A", "musician", "mousikos"),
        "G3452": ("μυελός", "G:N-M", "marrow", "muelos"),
        "G3453": ("μυέω", "G:V", "to initiate", "mueō"),
        "G3454": ("μῦθος", "G:N-M", "myth", "muthos"),
        "G3455": ("μυκάομαι", "G:V", "to roar", "mukaomai"),
        "G3456": ("μυκτηρίζω", "G:V", "to mock", "muktērizō"),
        "G3457": ("μυλικός", "G:A", "millstone", "mulikos"),
        "G3458": ("μύλος", "G:N-M", "millstone", "mulos"),
        "G3459": ("μυλών", "G:N-M", "millhouse", "mulōn"),
        "G3460": ("Μύρα", "N:N--L", "Myra", "Mura"),
        "G3461": ("μυριάς", "G:N-F", "myriad", "murias"),
        "G3462": ("μυρίζω", "G:V", "to anoint", "murizō"),
        "G3463": ("μύριοι", "G:A", "myriad", "murioi"),
        "G3464": ("μύρον", "G:N-N", "ointment", "muron"),
        "G3465": ("Μυσία", "N:N-F-L", "Mysia", "Musia"),
        "G3466": ("μυστήριον", "G:N-N", "mystery", "mustērion"),
        "G3467": ("μυωπάζω", "G:V", "be nearsighted", "muōpazō"),
        "G3468": ("μώλωψ", "G:N-M", "wound", "mōlōps"),
        "G3469": ("μωμάομαι", "G:V", "to criticize", "mōmaomai"),
        "G3470": ("μῶμος", "G:N-M", "blemish", "mōmos"),
        "G3471": ("μωραίνω", "G:V", "to make insipid", "mōrainō"),
        "G3472": ("μωρία", "G:N-F", "foolishness", "mōria"),
        "G3473": ("μωρολογία", "G:N-F", "foolish talk", "mōrologia"),
        "G3474": ("μωρός", "G:A", "foolish", "mōros"),
        "G3475": ("Μωϋσῆς, Μωσῆς", "N:N-M-P", "Moses", "Mōusēs, Mōsēs"),
        "G3476": ("Ναασσών", "N:N-M-P", "Nahshon", "Naassōn"),
        "G3477": ("Ναγγαί", "N:N-M-P", "Naggai", "Naggai"),
        "G3478": ("Ναζαρέθ, Ναζαρά", "N:N--L", "Nazareth", "Nazareth, Nazara"),
        "G3479": ("Ναζαρηνός", "G:A", "Nazarene", "Nazarēnos"),
        "G3480": ("Ναζωραῖος", "N:N--LG", "Nazarene", "Nazōraios"),
        "G3481": ("Ναθάν", "N:N-M-P", "Nathan", "Nathan"),
        "G3482": ("Ναθαναήλ", "N:N-M-P", "Nathanael", "Nathanaēl"),
        "G3483": ("ναί", "G:Part", "yes", "nai"),
        "G3484": ("Ναΐν", "N:N--L", "Nain", "Nain"),
        "G3485": ("ναός", "G:N-M", "temple", "naos"),
        "G3486": ("Ναούμ", "N:N-M-P", "Nahum", "Naoum"),
        "G3487": ("νάρδος", "G:N-F", "nard", "nardos"),
        "G3488": ("Νάρκισσος", "N:N-M-P", "Narcissus", "Narkissos"),
        "G3489": ("ναυαγέω", "G:V", "be shipwrecked", "nauageō"),
        "G3490": ("ναύκληρος", "G:N-M", "captain", "nauklēros"),
        "G3491": ("ναῦς", "G:N-F", "ship", "naus"),
        "G3492": ("ναύτης", "G:N-M", "sailor", "nautēs"),
        "G3493": ("Ναχώρ", "N:N-M-P", "Nahor", "Nachōr"),
        "G3494": ("νεανίας", "G:N-M", "young man", "neanias"),
        "G3495": ("νεανίσκος", "G:N-M", "young man", "neaniskos"),
        "G3496": ("Νεάπολις", "N:N-F-L", "Neapolis", "Neapolis"),
        "G3497": ("Νεεμάν, Ναιμάν", "N:N-M-P", "Naaman", "Neeman, Naiman"),
        "G3498": ("νεκρός", "G:A", "dead", "nekros"),
        "G3499": ("νεκρόω", "G:V", "to put to death", "nekroō"),
        "G3500": ("νέκρωσις", "G:N-F", "death", "nekrōsis"),
        "G3501": ("νέος", "G:A", "new", "neos"),
        "G3502": ("νεοσσός", "G:N-M", "nestling", "neossos"),
        "G3503": ("νεότης", "G:N-F", "youth", "neotēs"),
        "G3504": ("νεόφυτος", "G:A", "new convert", "neophutos"),
        "G3505": ("Νέρων", "N:N-M-P", "Nero", "Nerōn"),
        "G3506": ("νεύω", "G:V", "to motion", "neuō"),
        "G3507": ("νεφέλη", "G:N-F", "cloud", "nephelē"),
        "G3508": ("Νεφθαλείμ", "N:N--PG", "Naphtali", "Nephthaleim"),
        "G3509": ("νέφος", "G:N-N", "cloud", "nephos"),
        "G3510": ("νεφρός", "G:N-M", "mind", "nephros"),
        "G3511": ("νεωκόρος", "G:N-M", "temple guard", "neōkoros"),
        "G3512": ("νεωτερικός", "G:A", "youthful", "neōterikos"),
        "G3513": ("νή", "G:Part", "as surely as", "nē"),
        "G3514": ("νήθω", "G:V", "to spin", "nēthō"),
        "G3515": ("νηπιάζω", "G:V", "be childlike", "nēpiazō"),
        "G3516": ("νήπιος", "G:A", "child", "nēpios"),
        "G3517": ("Νηρεύς", "N:N-M-P", "Nereus", "Nēreus"),
        "G3518": ("Νηρί", "N:N-M-P", "Neri", "Nēri"),
        "G3519": ("νησίον", "G:N-N", "small island", "nēsion"),
        "G3520": ("νῆσος", "G:N-F", "island", "nēsos"),
        "G3521": ("νηστεία", "G:N-F", "fasting", "nēsteia"),
        "G3522": ("νηστεύω", "G:V", "to fast", "nēsteuō"),
        "G3523": ("νῆστις", "G:A", "fasting", "nēstis"),
        "G3524": ("νηφαλέος", "G:A", "sober", "nēphaleos"),
        "G3525": ("νήφω", "G:V", "be sober", "nēphō"),
        "G3526": ("Νίγερ", "N:N-M-P", "Niger", "Niger"),
        "G3527": ("Νικάνωρ", "N:N-M-P", "Nicanor", "Nikanōr"),
        "G3528": ("νικάω", "G:V", "to conquer", "nikaō"),
        "G3529": ("νίκη", "G:N-F", "victory", "nikē"),
        "G3530": ("Νικόδημος", "N:N-M-P", "Nicodemus", "Nikodēmos"),
        "G3531": ("Νικολαΐτης", "N:N--LG", "Nicolaitan", "Nikolaitēs"),
        "G3532": ("Νικόλαος", "N:N-M-P", "Nicolas", "Nikolaos"),
        "G3533": ("Νικόπολις", "N:N-F-L", "Nicopolis", "Nikopolis"),
        "G3534": ("νῖκος", "G:N-N", "victory", "nikos"),
        "G3535": ("Νινευΐ", "N:N--L", "Nineveh", "Nineui"),
        "G3536": ("Νινευΐτης", "N:N--LG", "Ninevite", "Nineuitēs"),
        "G3537": ("νιπτήρ", "G:N-M", "wash basin", "niptēr"),
        "G3538": ("νίπτω", "G:V", "to wash", "niptō"),
        "G3539": ("νοέω", "G:V", "to understand", "noeō"),
        "G3540": ("νόημα", "G:N-N", "mind/thought", "noēma"),
        "G3541": ("νόθος", "G:A", "illegitimate", "nothos"),
        "G3542": ("νομή", "G:N-F", "pasture", "nomē"),
        "G3543": ("νομίζω", "G:V", "to think", "nomizō"),
        "G3544": ("νομικός", "G:A", "lawyer", "nomikos"),
        "G3545": ("νομίμως", "G:Adv", "lawfully", "nomimōs"),
        "G3546": ("νόμισμα", "G:N-N", "coin", "nomisma"),
        "G3547": ("νομοδιδάσκαλος", "G:N-M", "teacher of the law", "nomodidaskalos"),
        "G3548": ("νομοθεσία", "G:N-F", "law", "nomothesia"),
        "G3549": ("νομοθετέω", "G:V", "to give laws", "nomotheteō"),
        "G3550": ("νομοθέτης", "G:N-M", "lawgiver", "nomothetēs"),
        "G3551": ("νόμος", "G:N-M", "law", "nomos"),
        "G3552": ("νοσέω", "G:V", "be sick", "noseō"),
        "G3553": ("νόσημα", "G:N-N", "disease", "nosēma"),
        "G3554": ("νόσος", "G:N-F", "illness", "nosos"),
        "G3555": ("νοσσιά", "G:N-F", "chick", "nossia"),
        "G3556": ("νοσσίον", "G:N-N", "chick", "nossion"),
        "G3557": ("νοσφίζω", "G:V", "to embezzle", "nosphizō"),
        "G3558": ("νότος", "G:N-M", "south", "notos"),
        "G3559": ("νουθεσία", "G:N-F", "admonition", "nouthesia"),
        "G3560": ("νουθετέω", "G:V", "to admonish", "noutheteō"),
        "G3561": ("νουμηνία", "G:N-F", "New Moon", "noumēnia"),
        "G3562": ("νουνεχῶς", "G:Adv", "wisely", "nounechōs"),
        "G3563": ("νοῦς", "G:N-M", "mind", "nous"),
        "G3564": ("Νυμφᾶς", "N:N-F-P", "Nympha", "Numphas"),
        "G3565": ("νύμφη", "G:N-F", "bride", "numphē"),
        "G3566": ("νυμφίος", "G:N-M", "bridegroom", "numphios"),
        "G3567": ("νυμφών", "G:N-M", "bridegroom", "numphōn"),
        "G3568": ("νῦν", "G:Adv", "now", "nun"),
        "G3569": ("τανῦν", "G:Adv", "now", "tanun"),
        "G3570": ("νυνί", "G:Adv", "now", "nuni"),
        "G3571": ("νύξ", "G:N-F", "night", "nux"),
        "G3572": ("νύσσω", "G:V", "to pierce", "nussō"),
        "G3573": ("νυστάζω", "G:V", "to doze", "nustazō"),
        "G3574": ("νυχθήμερον", "G:N-N", "a night and a day", "nuchthēmeron"),
        "G3575": ("Νῶε", "N:N-M-P", "Noah", "Nōe"),
        "G3576": ("νωθρός", "G:A", "dull", "nōthros"),
        "G3577": ("νῶτον", "G:N-M", "back", "nōton"),
        "G3578": ("ξενία", "G:N-F", "lodging", "xenia"),
        "G3579": ("ξενίζω", "G:V", "to host", "xenizō"),
        "G3580": ("ξενοδοχέω", "G:V", "to show hospitality", "xenodocheō"),
        "G3581": ("ξένος", "G:A", "foreign", "xenos"),
        "G3582": ("ξέστης", "G:N-M", "pitcher", "xestēs"),
        "G3583": ("ξηραίνω", "G:V", "to dry", "xērainō"),
        "G3584": ("ξηρός", "G:A", "dried up/withered", "xēros"),
        "G3585": ("ξύλινος", "G:A", "wooden", "xulinos"),
        "G3586": ("ξύλον", "G:N-N", "wood", "xulon"),
        "G3587": ("ξυράω", "G:V", "to shave", "xuraō"),
        "G3588": ("ὁ", "G:Art", "the/this/who", "ho"),
        "G3589": ("ὀγδοήκοντα", "G:A", "eighty", "ogdoēkonta"),
        "G3590": ("ὄγδοος", "G:A", "eighth", "ogdoos"),
        "G3591": ("ὄγκος", "G:N-M", "impediment", "ogkos"),
        "G3592": ("ὅδε", "G:DemP", "this", "hode, hēde"),
        "G3593": ("ὁδεύω", "G:V", "to journey", "hodeuō"),
        "G3594": ("ὁδηγέω", "G:V", "to guide", "hodēgeō"),
        "G3595": ("ὁδηγός", "G:N-M", "guide/leader", "hodēgos"),
        "G3596": ("ὁδοιπορέω", "G:V", "to journey", "hodoiporeō"),
        "G3597": ("ὁδοιπορία", "G:N-F", "journey", "hodoiporia"),
        "G3598": ("ὁδός", "G:N-F", "road", "hodos"),
        "G3599": ("ὀδούς", "G:N-M", "tooth", "odous"),
        "G3600": ("ὀδυνάω", "G:V", "be anguished", "odunaō"),
        "G3601": ("ὀδύνη", "G:N-F", "anguish", "odunē"),
        "G3602": ("ὀδυρμός", "G:N-M", "mourning", "odurmos"),
        "G3603": ("ὅ ἐστι", "G:V", "which is", "ho esti"),
        "G3604": ("Ὀζίας", "N:N-M-P", "Uzziah", "Ozias"),
        "G3605": ("ὄζω", "G:V", "to stink", "ozō"),
        "G3606": ("ὅθεν", "G:Conj", "whence", "hothen"),
        "G3607": ("ὀθόνη", "G:N-F", "sheet", "othonē"),
        "G3608": ("ὀθόνιον", "G:N-N", "bandages", "othonion"),
        "G3609": ("οἰκεῖος", "G:A", "of one’s household", "oikeios"),
        "G3610": ("οἰκέτης", "G:N-M", "slave", "oiketēs"),
        "G3611": ("οἰκέω", "G:V", "to dwell", "oikeō"),
        "G3612": ("οἴκημα", "G:N-N", "cell", "oikēma"),
        "G3613": ("οἰκητήριον", "G:N-N", "dwelling", "oikētērion"),
        "G3614": ("οἰκία", "G:N-F", "home", "oikia"),
        "G3615": ("οἰκιακός", "G:N-M", "member of a house", "oikiakos"),
        "G3616": ("οἰκοδεσποτέω", "G:V", "to manage a house", "oikodespoteō"),
        "G3617": ("οἰκοδεσπότης", "G:N-M", "householder", "oikodespotēs"),
        "G3618": ("οἰκοδομέω", "G:V", "to build", "oikodomeō"),
        "G3619": ("οἰκοδομή", "G:N-F", "building", "oikodomē"),
        "G3620": ("οἰκοδομία", "G:N-F", "edification", "oikodomia"),
        "G3621": ("οἰκονομέω", "G:V", "to manage", "oikonomeō"),
        "G3622": ("οἰκονομία", "G:N-F", "management", "oikonomia"),
        "G3623": ("οἰκονόμος", "G:N-M", "manager", "oikonomos"),
        "G3624": ("οἶκος", "G:N-M", "house", "oikos"),
        "G3625": ("οἰκουμένη", "G:N-F", "world", "oikoumenē"),
        "G3626": ("οἰκουρός", "G:A", "busy at home", "oikouros"),
        "G3627": ("οἰκτείρω", "G:V", "to have compassion", "oikteirō"),
        "G3628": ("οἰκτιρμός", "G:N-M", "compassion", "oiktirmos"),
        "G3629": ("οἰκτίρμων", "G:A", "compassionate", "oiktirmōn"),
        "G3630": ("οἰνοπότης", "G:N-M", "drunkard", "oinopotēs"),
        "G3631": ("οἶνος", "G:N-M", "wine", "oinos"),
        "G3632": ("οἰνοφλυγία", "G:N-F", "drunkenness", "oinophlugia"),
        "G3633": ("οἴομαι", "G:V", "to suppose", "oiomai"),
        "G3634": ("οἷος", "G:Cor", "such as", "ohios"),
        "G3635": ("ὀκνέω", "G:V", "to delay", "okneō"),
        "G3636": ("ὀκνηρός", "G:A", "lazy", "oknēros"),
        "G3637": ("ὀκταήμερος", "G:A", "eighth day", "oktaēmeros"),
        "G3638": ("ὀκτώ", "G:A", "eight", "oktō"),
        "G3639": ("ὄλεθρος", "G:N-M", "destructive", "olethros"),
        "G3640": ("ὀλιγόπιστος", "G:A", "of little faith", "oligopistos"),
        "G3641": ("ὀλίγος", "G:A / G:Adv", "little/few", "oligos"),
        "G3642": ("ὀλιγόψυχος", "G:A", "fainthearted", "oligopsuchos"),
        "G3643": ("ὀλιγωρέω", "G:V", "to despise", "oligōreō"),
        "G3644": ("ὀλοθρευτής", "G:N-M", "destroyer", "olothreutēs"),
        "G3645": ("ὀλοθρεύω", "G:V", "to destroy", "olothreuō"),
        "G3646": ("ὁλοκαύτωμα", "G:N-N", "burnt offering", "holokautōma"),
        "G3647": ("ὁλοκληρία", "G:N-F", "wholeness", "holoklēria"),
        "G3648": ("ὁλόκληρος", "G:A", "whole", "holoklēros"),
        "G3649": ("ὀλολύζω", "G:V", "to wail", "ololuzō"),
        "G3650": ("ὅλος", "G:A", "all", "holos"),
        "G3651": ("ὁλοτελής", "G:A", "wholly", "holotelēs"),
        "G3652": ("Ὀλυμπ��ς", "N:N-M-P", "Olympas", "Olumpas"),
        "G3653": ("ὄλυνθος", "G:N-M", "late fig", "olunthos"),
        "G3654": ("ὅλως", "G:Adv", "at all", "holōs"),
        "G3655": ("ὄμβρος", "G:N-M", "rainstorm", "ombros"),
        "G3656": ("ὁμιλέω", "G:V", "to talk", "homileō"),
        "G3657": ("ὁμιλία", "G:N-F", "association", "homilia"),
        "G3658": ("ὅμιλος", "G:N-M", "crowd", "homilos"),
        "G3659": ("ὄμμα", "G:N-N", "eye", "omma"),
        "G3660": ("ὄμνυμι", "G:V", "to swear", "omnumi"),
        "G3661": ("ὁμοθυμαδόν", "G:Adv", "united", "homothumadon"),
        "G3662": ("ὁμοιάζω", "G:V", "to resemble", "homoiazō"),
        "G3663": ("ὁμοιοπαθής", "G:A", "like", "homoiopathēs"),
        "G3664": ("ὅμοιος", "G:A", "like", "homoios"),
        "G3665": ("ὁμοιότης", "G:N-F", "likeness", "homoiotēs"),
        "G3666": ("ὁμοιόω", "G:V", "to liken", "homoioō"),
        "G3667": ("ὁμοίωμα", "G:N-N", "likeness", "homoiōma"),
        "G3668": ("ὁμοίως", "G:Adv", "similarly", "homoiōs"),
        "G3669": ("ὁμοίωσις", "G:N-F", "likeness", "homoiōsis"),
        "G3670": ("ὁμολογέω", "G:V", "to confess/profess", "homologeō"),
        "G3671": ("ὁμολογία", "G:N-F", "confession", "homologia"),
        "G3672": ("ὁμολογουμένως", "G:Adv", "undeniably", "homologoumenōs"),
        "G3673": ("ὁμότεχνος", "G:A", "of the same trade", "homotechnos"),
        "G3674": ("ὁμοῦ", "G:Adv", "together", "homou"),
        "G3675": ("ὁμόφρων", "G:A", "like-minded", "homophrōn"),
        "G3676": ("ὅμως", "G:Conj", "just as", "homōs"),
        "G3677": ("ὄναρ", "G:N-N", "dream", "onar"),
        "G3678": ("ὀνάριον", "G:N-N", "young donkey", "onarion"),
        "G3679": ("ὀνειδίζω", "G:V", "to revile", "oneidizō"),
        "G3680": ("ὀνειδισμός", "G:N-M", "reproach", "oneidismos"),
        "G3681": ("ὄνειδος", "G:N-N", "disgrace", "oneidos"),
        "G3682": ("Ὀνήσιμος", "N:N-M-P", "Onesimus", "Onēsimos"),
        "G3683": ("Ὀνησίφορος", "N:N-M-P", "Onesiphorus", "Onēsiphoros"),
        "G3684": ("ὀνικός", "G:A", "huge millstone", "onikos"),
        "G3685": ("ὀνίνημι", "G:V", "to have joy", "oninēmi"),
        "G3686": ("ὄνομα", "G:N-N", "name", "onoma"),
        "G3687": ("ὀνομάζω", "G:V", "to name", "onomazō"),
        "G3688": ("ὄνος", "G:N-M/F", "donkey", "onos"),
        "G3689": ("ὄντως", "G:Adv", "really", "ontōs"),
        "G3690": ("ὄξος", "G:N-N", "vinegar", "oxos"),
        "G3691": ("ὀξύς", "G:A", "sharp/swift", "oxus"),
        "G3692": ("ὀπή", "G:N-F", "hole", "opē"),
        "G3693": ("ὄπισθεν", "G:Prep", "after", "opisthen"),
        "G3694": ("ὀπίσω", "G:Prep", "after", "opisō"),
        "G3695": ("ὁπλίζω", "G:V", "to arm", "hoplizō"),
        "G3696": ("ὅπλον", "G:N-N", "weapon", "hoplon"),
        "G3697": ("ὁποῖος", "G:Cor+G:Intg", "what sort", "hopoios"),
        "G3698": ("ὁπότε", "G:Conj", "when(-ever)", "hopote"),
        "G3699": ("ὅπου", "G:Conj", "where(-ever)", "hopou"),
        "G3700": ("ὀπτάνομαι", "G:V", "to appear", "optanomai"),
        "G3701": ("ὀπτασία", "G:N-F", "vision", "optasia"),
        "G3702": ("ὀπτός", "G:A", "broiled", "optos"),
        "G3703": ("ὀπώρα", "G:N-F", "fruit", "opōra"),
        "G3704": ("ὅπως", "G:Conj", "that", "hopōs"),
        "G3705": ("ὅραμα", "G:N-N", "vision", "horama"),
        "G3706": ("ὅρασις", "G:N-F", "appearance/vision", "horasis"),
        "G3707": ("ὁρατός", "G:A", "visible", "horatos"),
        "G3708": ("ὁράω", "G:V", "to see", "horaō"),
        "G3709": ("ὀργή", "G:N-F", "wrath", "orgē"),
        "G3710": ("ὀργίζω", "G:V", "to anger", "orgizō"),
        "G3711": ("ὀργίλος", "G:A", "quick-tempered", "orgilos"),
        "G3712": ("ὀργυιά", "G:N-F", "a fathom", "orguia"),
        "G3713": ("ὀρέγω", "G:V", "to aspire", "oregō"),
        "G3714": ("ὀρεινός", "G:A", "hilly", "oreinos"),
        "G3715": ("ὄρεξις", "G:N-F", "lust", "orexis"),
        "G3716": ("ὀρθοποδέω", "G:V", "be upright", "orthopodeō"),
        "G3717": ("ὀρθός", "G:A", "upright", "orthos"),
        "G3718": ("ὀρθοτομέω", "G:V", "to cut straight", "orthotomeō"),
        "G3719": ("ὀρθρίζω", "G:V", "to rise at dawn", "orthrizō"),
        "G3720": ("ὀρθρινός", "G:A", "dawn", "orthrinos"),
        "G3721": ("ὄρθριος", "G:A", "morning", "orthrios"),
        "G3722": ("ὄρθρος", "G:N-M", "dawn", "orthros"),
        "G3723": ("ὀρθῶς", "G:Adv", "correctly", "orthōs"),
        "G3724": ("ὁρίζω", "G:V", "to determine", "horizō"),
        "G3725": ("ὅριον", "G:N-N", "region", "horion"),
        "G3726": ("ὁρκίζω", "G:V", "to adjure", "horkizō"),
        "G3727": ("ὅρκος", "G:N-M", "oath", "horkos"),
        "G3728": ("ὁρκωμοσία", "G:N-F", "oath", "horkōmosia"),
        "G3729": ("ὁρμάω", "G:V", "to stampede", "hormaō"),
        "G3730": ("ὁρμή", "G:N-F", "impulse", "hormē"),
        "G3731": ("ὅρμημα", "G:N-N", "sudden violence", "hormēma"),
        "G3732": ("ὄρνεον", "G:N-N", "bird", "orneon"),
        "G3733": ("ὄρνις", "G:N-F", "hen", "ornis"),
        "G3734": ("ὁροθεσία", "G:N-F", "fixed boundary", "horothesia"),
        "G3735": ("ὄρος", "G:N-N", "mountain", "oros"),
        "G3736": ("ὀρύσσω", "G:V", "to dig", "orussō"),
        "G3737": ("ὀρφανός", "G:A", "orphan", "orphanos"),
        "G3738": ("ὀρχέομαι", "G:V", "to dance", "orcheomai"),
        "G3739": ("ὅς", "G:RelP", "which", "hos, hē"),
        "G3740": ("ὁσάκις", "G:Conj", "whenever", "hosakis"),
        "G3741": ("ὅσιος", "G:A", "sacred", "hosios"),
        "G3742": ("ὁσιότης", "G:N-F", "holiness", "hosiotēs"),
        "G3743": ("ὁσίως", "G:Adv", "devoutly", "hosiōs"),
        "G3744": ("ὀσμή", "G:N-F", "aroma", "osmē"),
        "G3745": ("ὅσος", "G:Cor", "just as/how much", "hosos"),
        "G3746": ("ὅσπερ", "G:RelP", "who", "hosper"),
        "G3747": ("ὀστέον", "G:N-N", "bone", "osteon"),
        "G3748": ("ὅστις", "G:RelP", "who/which", "hostis, hētis"),
        "G3749": ("ὀστράκινος", "G:A", "clay", "ostrakinos"),
        "G3750": ("ὄσφρησις", "G:N-F", "sense of smell", "osphrēsis"),
        "G3751": ("ὀσφῦς", "G:N-F", "loins", "osphus"),
        "G3752": ("ὅταν", "G:Conj", "when(-ever)", "hotan"),
        "G3753": ("ὅτε", "G:Conj", "when", "hote"),
        "G3754": ("ὅτι", "G:Conj", "that/since", "hoti"),
        "G3755": ("ὅτου", "G:Prep", "until", "hotou"),
        "G3756": ("οὐ", "G:Neg", "no", "ou"),
        "G3757": ("οὗ", "G:Adv", "whither", "ohu"),
        "G3758": ("οὐά", "G:Intj", "aha!", "oua"),
        "G3759": ("οὐαί", "G:Intj", "woe!", "ouai"),
        "G3760": ("οὐδαμῶς", "G:Adv", "by no means", "oudamōs"),
        "G3761": ("οὐδέ", "G:Neg+G:Conj", "and not", "oude"),
        "G3762": ("οὐδείς", "G:Neg", "none", "oudeis"),
        "G3763": ("οὐδέποτε", "G:Adv", "never", "oudepote"),
        "G3764": ("οὐδέπω", "G:Adv", "never", "oudepō"),
        "G3765": ("οὐκέτι", "G:Neg", "not any more", "ouketi"),
        "G3766": ("οὐκοῦν", "G:Neg+G:Conj", "so", "oukoun"),
        "G3767": ("οὖν", "G:Conj", "therefore/then", "oun"),
        "G3768": ("οὔπω", "G:Neg", "not yet", "oupō"),
        "G3769": ("οὐρά", "G:N-F", "tail", "oura"),
        "G3770": ("οὐράνιος", "G:A", "heavenly", "ouranios"),
        "G3771": ("οὐρανόθεν", "G:Adv", "from heaven", "ouranothen"),
        "G3772": ("οὐρανός", "G:N-M", "heaven", "ouranos"),
        "G3773": ("Οὐρβανός", "N:N-M-P", "Urbanus", "Ourbanos"),
        "G3774": ("Οὐρίας", "N:N-M-P", "Uriah", "Ourias"),
        "G3775": ("οὖς", "G:N-N", "ear", "ous"),
        "G3776": ("οὐσία", "G:N-F", "estate", "ousia"),
        "G3777": ("οὔτε", "G:Neg+G:Conj", "neither", "oute"),
        "G3778": ("οὗτος", "G:DemP", "this/he/she/it", "ohutos"),
        "G3779": ("οὕτως", "G:Adv", "thus(-ly)", "ohutō, ohutōs"),
        "G3780": ("οὐχί", "G:Neg", "not", "ouchi"),
        "G3781": ("ὀφειλέτης", "G:N-M", "debtor", "opheiletēs"),
        "G3782": ("ὀφειλή", "G:N-F", "debt", "opheilē"),
        "G3783": ("ὀφείλημα", "G:N-N", "debt", "opheilēma"),
        "G3784": ("ὀφείλω", "G:V", "to owe", "opheilō"),
        "G3785": ("ὄφελον", "G:Intj", "I wish!", "ophelon"),
        "G3786": ("ὄφελος", "G:N-N", "gain", "ophelos"),
        "G3787": ("ὀφθαλμοδουλεία", "G:N-F", "eye-service", "ophthalmodouleia"),
        "G3788": ("ὀφθαλμός", "G:N-M", "eye", "ophthalmos"),
        "G3789": ("ὄφις", "G:N-M", "snake", "ophis"),
        "G3790": ("ὀφρῦς", "G:N-F", "brow", "ophrus"),
        "G3791": ("ὀχλέω", "G:V", "to disturb", "ochleō"),
        "G3792": ("ὀχλοποιέω", "G:V", "to riot", "ochlopoieō"),
        "G3793": ("ὄχλος", "G:N-M", "crowd", "ochlos"),
        "G3794": ("ὀχύρωμα", "G:N-N", "stronghold", "ochurōma"),
        "G3795": ("ὀψάριον", "G:N-N", "fish", "opsarion"),
        "G3796": ("ὀψέ", "G:Adv", "late", "opse"),
        "G3797": ("ὄψιμος", "G:A", "late (rain)", "opsimos"),
        "G3798": ("ὀψία", "G:A", "evening", "opsia"),
        "G3799": ("ὄψις", "G:N-F", "face", "opsis"),
        "G3800": ("ὀψώνιον", "G:N-N", "compensation", "opsōnion"),
        "G3801": ("ο ων και ο ην και ο ερχομενος", "G:Art + G:V + G:Conj + G:Art + G:V + G:Conj + G:Art + G:V", "was, is, will be", "o ōn kai o ēn kai o erchomenos"),
        "G3802": ("παγιδεύω", "G:V", "to trap", "pagideuō"),
        "G3803": ("παγίς", "G:N-F", "trap", "pagis"),
        "G3804": ("πάθημα", "G:N-N", "suffering", "pathēma"),
        "G3805": ("παθητός", "G:A", "suffering", "pathētos"),
        "G3806": ("πάθος", "G:N-N", "passion", "pathos"),
        "G3807": ("παιδαγωγός", "G:N-M", "guardian", "paidagōgos"),
        "G3808": ("παιδάριον", "G:N-N", "boy", "paidarion"),
        "G3809": ("παιδεία", "G:N-F", "discipline", "paideia"),
        "G3810": ("παιδευτής", "G:N-M", "instructor", "paideutēs"),
        "G3811": ("παιδεύω", "G:V", "to instruct", "paideuō"),
        "G3812": ("παιδιόθεν", "G:Adv", "from childhood", "paidiothen"),
        "G3813": ("παιδίον", "G:N-N", "child", "paidion"),
        "G3814": ("παιδίσκη", "G:N-F", "maidservant", "paidiskē"),
        "G3815": ("παίζω", "G:V", "to play", "paizō"),
        "G3816": ("παῖς", "G:N-M/F", "child", "pais"),
        "G3817": ("παίω", "G:V", "to strike", "paiō"),
        "G3818": ("Πακατιανή", "N:N--L", "Pacatian", "Pakatianē"),
        "G3819": ("πάλαι", "G:Adv", "of old", "palai"),
        "G3820": ("παλαιός", "G:A", "old", "palaios"),
        "G3821": ("παλαιότης", "G:N-F", "oldness", "palaiotēs"),
        "G3822": ("παλαιόω", "G:V", "to make old", "palaioō"),
        "G3823": ("πάλη", "G:N-F", "struggle", "palē"),
        "G3824": ("παλιγγενεσία", "G:N-F", "regeneration", "paliggenesia"),
        "G3825": ("πάλιν", "G:Adv", "again", "palin"),
        "G3826": ("παμπληθεί", "G:Adv", "together", "pamplēthei"),
        "G3827": ("πάμπολυς", "G:A", "great", "pampolus"),
        "G3828": ("Παμφυλία", "N:N-F-L", "Pamphylia", "Pamphulia"),
        "G3829": ("πανδοχεῖον", "G:N-N", "inn", "pandocheion"),
        "G3830": ("πανδοχεύς", "G:N-M", "innkeeper", "pandocheus"),
        "G3831": ("πανήγυρις", "G:N-F", "assembly", "panēguris"),
        "G3832": ("πανοικί", "G:Adv", "with all the house", "panoiki"),
        "G3833": ("πανοπλία", "G:N-F", "complete armor", "panoplia"),
        "G3834": ("πανουργία", "G:N-F", "craftiness", "panourgia"),
        "G3835": ("πανοῦργος", "G:A", "crafty", "panourgos"),
        "G3836": ("πανταχόθεν", "G:Adv", "from all sides", "pantachothen"),
        "G3837": ("πανταχοῦ", "G:Adv", "everywhere", "pantachou"),
        "G3838": ("παντελής", "G:A", "completely", "pantelēs"),
        "G3839": ("πάντῃ", "G:Adv", "always", "pantē"),
        "G3840": ("πάντοθεν", "G:Adv", "from all sides", "pantothen"),
        "G3841": ("παντοκράτωρ", "G:N-M", "almighty", "pantokratōr"),
        "G3842": ("πάντοτε", "G:Adv", "always", "pantote"),
        "G3843": ("πάντως", "G:Adv", "surely", "pantōs"),
        "G3844": ("παρά", "G:Prep", "from/with/beside", "para"),
        "G3845": ("παραβαίνω", "G:V", "to transgress", "parabainō"),
        "G3846": ("παραβάλλω", "G:V", "to arrive", "paraballō"),
        "G3847": ("παράβασις", "G:N-F", "transgression", "parabasis"),
        "G3848": ("παραβάτης", "G:N-M", "transgresor", "parabatēs"),
        "G3849": ("παραβιάζομαι", "G:V", "to urge", "parabiazomai"),
        "G3850": ("παραβολή", "G:N-F", "parable", "parabolē"),
        "G3851": ("παραβουλεύομαι, παραβολεύομαι", "G:V", "to risk", "parabouleuomai, paraboleuomai"),
        "G3852": ("παραγγελία", "G:N-F", "order", "paraggelia"),
        "G3853": ("παραγγέλλω", "G:V", "to order", "paraggellō"),
        "G3854": ("παραγίνομαι", "G:V", "to come", "paraginomai"),
        "G3855": ("παράγω", "G:V", "to pass", "paragō"),
        "G3856": ("παραδειγματίζω", "G:V", "to disgrace", "paradeigmatizō"),
        "G3857": ("παράδεισος", "G:N-M", "paradise", "paradeisos"),
        "G3858": ("παραδέχομαι", "G:V", "to receive", "paradechomai"),
        "G3859": ("παραδιατριβή", "G:N-F", "contention", "paradiatribē"),
        "G3860": ("παραδίδωμι", "G:V", "to deliver", "paradidōmi"),
        "G3861": ("παράδοξος", "G:A", "remarkable", "paradoxos"),
        "G3862": ("παράδοσις", "G:N-F", "tradition", "paradosis"),
        "G3863": ("παραζηλόω", "G:V", "to make envious", "parazēloō"),
        "G3864": ("παραθαλάσσιος", "G:A", "seaside", "parathalassios"),
        "G3865": ("παραθεωρέω", "G:V", "to neglect", "paratheōreō"),
        "G3866": ("παραθήκη", "G:N-F", "deposit", "parathēkē"),
        "G3867": ("παραινέω", "G:V", "to urge", "paraineō"),
        "G3868": ("παραιτέομαι", "G:V", "to refuse/excuse", "paraiteomai"),
        "G3869": ("παρακαθίζω", "G:V", "to sit by", "parakathizō"),
        "G3870": ("παρακαλέω", "G:V", "to plead/comfort", "parakaleō"),
        "G3871": ("παρακαλύπτω", "G:V", "to hide", "parakaluptō"),
        "G3872": ("παρακαταθήκη", "G:N-F", "deposit", "parakatathēkē"),
        "G3873": ("παράκειμαι", "G:V", "be present", "parakeimai"),
        "G3874": ("παράκλησις", "G:N-F", "encouragement", "paraklēsis"),
        "G3875": ("παράκλητος", "G:N-M", "counsellor", "paraklētos"),
        "G3876": ("παρακοή", "G:N-F", "disobedience", "parakoē"),
        "G3877": ("παρακολουθέω", "G:V", "to follow", "parakoloutheō"),
        "G3878": ("παρακούω", "G:V", "to ignore", "parakouō"),
        "G3879": ("παρακύπτω", "G:V", "to stoop", "parakuptō"),
        "G3880": ("παραλαμβάνω", "G:V", "to take", "paralambanō"),
        "G3881": ("παραλέγω", "G:V", "to sail past", "paralegō"),
        "G3882": ("παράλιος", "G:A", "seaside", "paralios"),
        "G3883": ("παραλλαγή", "G:N-F", "variation", "parallagē"),
        "G3884": ("παραλογίζομαι", "G:V", "to deceive", "paralogizomai"),
        "G3885": ("παραλυτικός", "G:A", "paralytic", "paralutikos"),
        "G3886": ("παραλύω", "G:V", "to paralyze", "paraluō"),
        "G3887": ("παραμένω", "G:V", "to continue", "paramenō"),
        "G3888": ("παραμυθέομαι", "G:V", "to encourage", "paramutheomai"),
        "G3889": ("παραμυθία", "G:N-F", "comfort", "paramuthia"),
        "G3890": ("παραμύθιον", "G:N-N", "comfort", "paramuthion"),
        "G3891": ("παρανομέω", "G:V", "to break the law", "paranomeō"),
        "G3892": ("παρανομία", "G:N-F", "lawlessness", "paranomia"),
        "G3893": ("παραπικραίνω", "G:V", "to rebel", "parapikrainō"),
        "G3894": ("παραπικρασμός", "G:N-M", "rebellion", "parapikrasmos"),
        "G3895": ("παραπίπτω", "G:V", "to defect", "parapiptō"),
        "G3896": ("παραπλέω", "G:V", "to sail past", "parapleō"),
        "G3897": ("παραπλήσιον", "G:A", "almost", "paraplēsion"),
        "G3898": ("παραπλησίως", "G:Adv", "similarly", "paraplēsiōs"),
        "G3899": ("παραπορεύομαι", "G:V", "to pass by/through", "paraporeuomai"),
        "G3900": ("παράπτωμα", "G:N-N", "trespass", "paraptōma"),
        "G3901": ("παραρρέω", "G:V", "to drift away", "pararreō"),
        "G3902": ("παράσημος", "G:N-N", "figurehead", "parasēmos"),
        "G3903": ("παρασκευάζω", "G:V", "to prepare", "paraskeuazō"),
        "G3904": ("παρασκευή", "G:N-F", "Preparation Day", "paraskeuē"),
        "G3905": ("παρατείνω", "G:V", "to prolong", "parateinō"),
        "G3906": ("παρατηρέω", "G:V", "to observe", "paratēreō"),
        "G3907": ("παρατήρησις", "G:N-F", "observation", "paratērēsis"),
        "G3908": ("παρατίθημι", "G:V", "to set before", "paratithēmi"),
        "G3909": ("παρατυγχάνω", "G:V", "be there", "paratugchanō"),
        "G3910": ("παραυτίκα", "G:Adv", "momentary", "parautika"),
        "G3911": ("παραφέρω", "G:V", "to take away", "parapherō"),
        "G3912": ("παραφρονέω", "G:V", "be insane", "paraphroneō"),
        "G3913": ("παραφρονία", "G:N-F", "insanity", "paraphronia"),
        "G3914": ("παραχειμάζω", "G:V", "to winter", "paracheimazō"),
        "G3915": ("παραχειμασία", "G:N-F", "wintering", "paracheimasia"),
        "G3916": ("παραχρῆμα", "G:Adv", "instantly", "parachrēma"),
        "G3917": ("πάρδαλις", "G:N-F", "leopard", "pardalis"),
        "G3918": ("πάρειμι", "G:V", "be present", "pareimi"),
        "G3919": ("παρεισάγω", "G:V", "to introduce", "pareisagō"),
        "G3920": ("παρείσακτος", "G:A", "infiltrated", "pareisaktos"),
        "G3921": ("παρεισδύνω", "G:V", "to infiltrate", "pareisdunō"),
        "G3922": ("παρεισέρχομαι", "G:V", "to come in", "pareiserchomai"),
        "G3923": ("παρεισφέρω", "G:V", "to supply", "pareispherō"),
        "G3924": ("παρεκτός", "G:Prep", "except", "parektos"),
        "G3925": ("παρεμβολή", "G:N-F", "barracks", "parembolē"),
        "G3926": ("παρενοχλέω", "G:V", "to trouble", "parenochleō"),
        "G3927": ("παρεπίδημος", "G:A", "stranger", "parepidēmos"),
        "G3928": ("παρέρχομαι", "G:V", "to pass by", "parerchomai"),
        "G3929": ("πάρεσις", "G:N-F", "passing over", "paresis"),
        "G3930": ("παρέχω", "G:V", "to furnish occasion", "parechō"),
        "G3931": ("παρηγορία", "G:N-F", "comfort", "parēgoria"),
        "G3932": ("παρθένεια", "G:N-F", "virginity", "partheneia"),
        "G3933": ("παρθένος", "G:N-M/F", "virgin", "parthenos"),
        "G3934": ("Πάρθος", "N:N--LG", "Parthian", "Parthos"),
        "G3935": ("παρίημι", "G:V", "to neglect", "pariēmi"),
        "G3936": ("παρίστημι", "G:V", "to stand by", "paristēmi"),
        "G3937": ("Παρμενᾶς", "N:N-M-P", "Parmenas", "Parmenas"),
        "G3938": ("πάροδος", "G:N-F", "passing by", "parodos"),
        "G3939": ("παροικέω", "G:V", "be a stranger", "paroikeō"),
        "G3940": ("παροικία", "G:N-F", "sojourning", "paroikia"),
        "G3941": ("πάροικος", "G:A", "foreigner", "paroikos"),
        "G3942": ("παροιμία", "G:N-F", "proverb", "paroimia"),
        "G3943": ("πάροινος", "G:A", "drunken", "paroinos"),
        "G3944": ("παροίχομαι", "G:V", "to pass by", "paroichomai"),
        "G3945": ("παρομοιάζω", "G:V", "to resemble", "paromoiazō"),
        "G3946": ("παρόμοιος", "G:A", "like", "paromoios"),
        "G3947": ("παροξύνω", "G:V", "to provoke", "paroxunō"),
        "G3948": ("παροξυσμός", "G:N-M", "stirring up", "paroxusmos"),
        "G3949": ("παροργίζω", "G:V", "to anger", "parorgizō"),
        "G3950": ("παροργισμός", "G:N-M", "anger", "parorgismos"),
        "G3951": ("παροτρύνω", "G:V", "to incite", "parotrunō"),
        "G3952": ("παρουσία", "G:N-F", "coming", "parousia"),
        "G3953": ("παροψίς", "G:N-F", "dish", "paropsis"),
        "G3954": ("παρρησία", "G:N-F", "boldness", "parrēsia"),
        "G3955": ("παρρησιάζομαι", "G:V", "to preach boldly", "parrēsiazomai"),
        "G3956": ("πᾶς", "G:A", "all", "pas"),
        "G3957": ("πάσχα", "A:N", "Passover", "pascha"),
        "G3958": ("πάσχω", "G:V", "to suffer", "paschō"),
        "G3959": ("Πάταρα", "N:N--L", "Patara", "Patara"),
        "G3960": ("πατάσσω", "G:V", "to strike", "patassō"),
        "G3961": ("πατέω", "G:V", "to trample", "pateō"),
        "G3962": ("πατήρ", "G:N-M", "father", "patēr"),
        "G3963": ("Πάτμος", "N:N-F-L", "Patmos", "Patmos"),
        "G3964": ("πατραλῴας", "G:N-M", "patricide", "patralōas"),
        "G3965": ("πατριά", "G:N-F", "family line", "patria"),
        "G3966": ("πατριάρχης", "G:N-M", "patriarch", "patriarchēs"),
        "G3967": ("πατρικός", "G:A", "paternal", "patrikos"),
        "G3968": ("πατρίς", "G:N-F", "fatherland", "patris"),
        "G3969": ("Πατροβᾶς", "N:N-M-P", "Patrobas", "Patrobas"),
        "G3970": ("πατροπαράδοτος", "G:A", "from forefathers", "patroparadotos"),
        "G3971": ("πατρῷος", "G:A", "ancestral", "patrōos"),
        "G3972": ("Παῦλος", "N:N-M-P", "Paul", "Paulos"),
        "G3973": ("παύω", "G:V", "to cease", "pauō"),
        "G3974": ("Πάφος", "N:N-F-L", "Paphos", "Paphos"),
        "G3975": ("παχύνω", "G:V", "to thicken", "pachunō"),
        "G3976": ("πέδη", "G:N-F", "fetter", "pedē"),
        "G3977": ("πεδινός", "G:A", "level", "pedinos"),
        "G3978": ("πεζεύω", "G:V", "to walk", "pezeuō"),
        "G3979": ("πεζῇ", "G:Adv", "on foot", "pezē"),
        "G3980": ("πειθαρχέω", "G:V", "to obey", "peitharcheō"),
        "G3981": ("πειθός", "G:A", "persuasive", "peithos"),
        "G3982": ("πείθω", "G:V", "to persuade", "peithō"),
        "G3983": ("πεινάω", "G:V", "to hunger", "peinaō"),
        "G3984": ("πεῖρα", "G:N-F", "test", "peira"),
        "G3985": ("πειράζω", "G:V", "to test/tempt", "peirazō"),
        "G3986": ("πειρασμός", "G:N-M", "temptation/testing", "peirasmos"),
        "G3987": ("πειράω", "G:V", "to try", "peiraō"),
        "G3988": ("πεισμονή", "G:N-F", "persuasion", "peismonē"),
        "G3989": ("πέλαγος", "G:N-N", "sea", "pelagos"),
        "G3990": ("πελεκίζω", "G:V", "to behead", "pelekizō"),
        "G3991": ("πέμπτος", "G:A", "fifth", "pemptos"),
        "G3992": ("πέμπω", "G:V", "to send", "pempō"),
        "G3993": ("πένης", "G:N-M", "poor", "penēs"),
        "G3994": ("πενθερά", "G:N-F", "mother-in-law", "penthera"),
        "G3995": ("πενθερός", "G:N-M", "father-in-law", "pentheros"),
        "G3996": ("πενθέω", "G:V", "to mourn", "pentheō"),
        "G3997": ("πένθος", "G:N-N", "grief", "penthos"),
        "G3998": ("πενιχρός", "G:A", "poor", "penichros"),
        "G3999": ("πεντάκις", "G:Adv", "five times", "pentakis"),
        "G4000": ("πεντακισχίλιοι", "G:A", "five thousand", "pentakischilioi"),
        "G4001": ("πεντακόσιοι", "G:A", "five hundred", "pentakosioi"),
        "G4002": ("πέντε", "G:A", "five", "pente"),
        "G4003": ("πεντεκαιδέκατος", "G:A", "fifteenth", "pentekaidekatos"),
        "G4004": ("πεντήκοντα", "G:A", "fifty", "pentēkonta"),
        "G4005": ("πεντηκοστή", "G:N-F", "Pentecost", "pentēkostē"),
        "G4006": ("πεποίθησις", "G:N-F", "confidence", "pepoithēsis"),
        "G4007": ("περ", "G:Part", "fully", "per"),
        "G4008": ("πέραν", "G:Prep", "other side", "peran"),
        "G4009": ("πέρας", "G:N-N", "end", "peras"),
        "G4010": ("Πέργαμος", "N:N-F-L", "Pergamum", "Pergamos"),
        "G4011": ("Πέργη", "N:N-F-L", "Perga", "Pergē"),
        "G4012": ("περί", "G:Prep", "about", "peri"),
        "G4013": ("περιάγω", "G:V", "to take/go around", "periagō"),
        "G4014": ("περιαιρέω", "G:V", "to take away", "periaireō"),
        "G4015": ("περιαστράπτω", "G:V", "to shine", "periastraptō"),
        "G4016": ("περιβάλλω", "G:V", "to clothe", "periballō"),
        "G4017": ("περιβλέπω", "G:V", "to look around", "periblepō"),
        "G4018": ("περιβόλαιον", "G:N-N", "covering", "peribolaion"),
        "G4019": ("περιδέω", "G:V", "to wrap around", "perideō"),
        "G4020": ("περιεργάζομαι", "G:V", "be a busybody", "periergazomai"),
        "G4021": ("περίεργος", "G:A", "meddlesome/magic", "periergos"),
        "G4022": ("περιέρχομαι", "G:V", "to go around", "perierchomai"),
        "G4023": ("περιέχω", "G:V", "to contain", "periechō"),
        "G4024": ("περιζώννυμι", "G:V", "to gird", "perizōnnumi"),
        "G4025": ("περίθεσις", "G:N-F", "wearing", "perithesis"),
        "G4026": ("περιΐστημι", "G:V", "to stand around", "periistēmi"),
        "G4027": ("περικάθαρμα", "G:N-N", "garbage", "perikatharma"),
        "G4028": ("περικαλύπτω", "G:V", "to cover", "perikaluptō"),
        "G4029": ("περίκειμαι", "G:V", "to surround", "perikeimai"),
        "G4030": ("περικεφαλαία", "G:N-F", "helmet", "perikephalaia"),
        "G4031": ("περικρατής", "G:A", "in control of", "perikratēs"),
        "G4032": ("περικρύπτω", "G:V", "to hide", "perikruptō"),
        "G4033": ("περικυκλόω", "G:V", "to surround", "perikukloō"),
        "G4034": ("περιλάμπω", "G:V", "to shine around", "perilampō"),
        "G4035": ("περιλείπομαι", "G:V", "to remain", "perileipomai"),
        "G4036": ("περίλυπος", "G:A", "sorrowful", "perilupos"),
        "G4037": ("περιμένω", "G:V", "to await", "perimenō"),
        "G4038": ("πέριξ", "G:Adv", "around", "perix"),
        "G4039": ("περιοικέω", "G:V", "to dwell around", "perioikeō"),
        "G4040": ("περίοικος", "G:A", "neighboring", "perioikos"),
        "G4041": ("περιούσιος", "G:A", "special", "periousios"),
        "G4042": ("περιοχή", "G:N-F", "passage", "periochē"),
        "G4043": ("περιπατέω", "G:V", "to walk", "peripateō"),
        "G4044": ("περιπείρω", "G:V", "to pierce", "peripeirō"),
        "G4045": ("περιπίπτω", "G:V", "to fall upon", "peripiptō"),
        "G4046": ("περιποιέω", "G:V", "to gain", "peripoieō"),
        "G4047": ("περιποίησις", "G:N-F", "acquiring", "peripoiēsis"),
        "G4048": ("περιρρήγνυμι", "G:V", "to strip off", "perirrēgnumi"),
        "G4049": ("περισπάω", "G:V", "to distract", "perispaō"),
        "G4050": ("περισσεία", "G:N-F", "abundance", "perisseia"),
        "G4051": ("περίσσευμα", "G:N-N", "overflow", "perisseuma"),
        "G4052": ("περισσεύω", "G:V", "to abound/exceed", "perisseuō"),
        "G4053": ("περισσός", "G:A / G:Adv", "excessive/abundant", "perissos"),
        "G4054": ("περισσότερον", "G:Adv", "more abundantly", "perissoteron"),
        "G4055": ("περισσότερος", "G:A", "superabundant", "perissoteros"),
        "G4056": ("περισσοτέρως", "G:Adv", "superabundantly", "perissoterōs"),
        "G4057": ("περισσῶς", "G:Adv", "superabundantly", "perissōs"),
        "G4058": ("περιστερά", "G:N-F", "dove", "peristera"),
        "G4059": ("περιτέμνω", "G:V", "to circumcise", "peritemnō"),
        "G4060": ("περιτίθημι", "G:V", "to put on", "peritithēmi"),
        "G4061": ("περιτομή", "G:N-F", "circumcision", "peritomē"),
        "G4062": ("περιτρέπω", "G:V", "to drive insane", "peritrepō"),
        "G4063": ("περιτρέχω", "G:V", "to run around", "peritrechō"),
        "G4064": ("περιφέρω", "G:V", "to carry (around)", "peripherō"),
        "G4065": ("περιφρονέω", "G:V", "to despise", "periphroneō"),
        "G4066": ("περίχωρος", "G:A", "surrounding region", "perichōros"),
        "G4067": ("περίψημα", "G:N-N", "scum", "peripsēma"),
        "G4068": ("περπερεύομαι", "G:V", "to boast", "perpereuomai"),
        "G4069": ("Περσίς", "N:N-F-P", "Persis", "Persis"),
        "G4070": ("πέρυσι", "G:Adv", "last year", "perusi"),
        "G4071": ("πετεινός", "G:N-N", "bird", "peteinos"),
        "G4072": ("πέτομαι", "G:V", "to fly", "petomai"),
        "G4073": ("πέτρα", "G:N-F", "rock", "petra"),
        "G4074": ("Πέτρος", "N:N-M-P", "Peter", "Petros"),
        "G4075": ("πετρώδης", "G:A", "rocky", "petrōdēs"),
        "G4076": ("πήγανον", "G:N-N", "rue", "pēganon"),
        "G4077": ("πηγή", "G:N-F", "flow", "pēgē"),
        "G4078": ("πήγνυμι", "G:V", "to set up", "pēgnumi"),
        "G4079": ("πηδάλιον", "G:N-N", "rudder", "pēdalion"),
        "G4080": ("πηλίκος", "G:A", "how great", "pēlikos"),
        "G4081": ("πηλός", "G:N-M", "clay", "pēlos"),
        "G4082": ("πήρα", "G:N-F", "bag", "pēra"),
        "G4083": ("πῆχυς", "G:N-M", "cubit/hour", "pēchus"),
        "G4084": ("πιάζω", "G:V", "to arrest/catch", "piazō"),
        "G4085": ("πιέζω", "G:V", "to press down", "piezō"),
        "G4086": ("πιθανολογία", "G:N-F", "persuasive speech", "pithanologia"),
        "G4087": ("πικραίνω", "G:V", "to embitter", "pikrainō"),
        "G4088": ("πικρία", "G:N-F", "bitterness", "pikria"),
        "G4089": ("πικρός", "G:A", "bitter", "pikros"),
        "G4090": ("πικρῶς", "G:Adv", "bitterly", "pikrōs"),
        "G4091": ("Πιλᾶτος", "N:N-M-P", "Pilate", "Pilatos"),
        "G4092": ("πίμπρημι", "G:V", "to swell up", "pimprēmi"),
        "G4093": ("πινακίδιον", "G:N-N", "little tablet", "pinakidion"),
        "G4094": ("πίναξ", "G:N-M", "platter", "pinax"),
        "G4095": ("πίνω", "G:V", "to drink", "pinō"),
        "G4096": ("πιότης", "G:N-F", "richness", "piotēs"),
        "G4097": ("πιπράσκω", "G:V", "to sell", "pipraskō"),
        "G4098": ("πίπτω", "G:V", "to collapse", "piptō"),
        "G4099": ("Πισιδία", "N:N-F-L", "Pisidia", "Pisidia"),
        "G4100": ("πιστεύω", "G:V", "to trust (in)", "pisteuō"),
        "G4101": ("πιστικός", "G:A", "pure", "pistikos"),
        "G4102": ("πίστις", "G:N-F", "faith/trust", "pistis"),
        "G4103": ("πιστός", "G:A", "faithful", "pistos"),
        "G4104": ("πιστόω", "G:V", "be convinced", "pistoō"),
        "G4105": ("πλανάω", "G:V", "to lead astray", "planaō"),
        "G4106": ("πλάνη", "G:N-F", "error", "planē"),
        "G4107": ("πλανήτης", "G:A", "wandering", "planētēs"),
        "G4108": ("πλάνος", "G:A", "deceiving", "planos"),
        "G4109": ("πλάξ", "G:N-F", "tablet", "plax"),
        "G4110": ("πλάσμα", "G:N-N", "formed thing", "plasma"),
        "G4111": ("πλάσσω", "G:V", "to mold", "plassō"),
        "G4112": ("πλαστός", "G:A", "made-up", "plastos"),
        "G4113": ("πλατεῖα", "G:N-F", "wide street/road", "plateia"),
        "G4114": ("πλάτος", "G:N-N", "width", "platos"),
        "G4115": ("πλατύνω", "G:V", "to widen", "platunō"),
        "G4116": ("πλατύς", "G:N-F", "wide", "platus"),
        "G4117": ("πλέγμα", "G:N-N", "braided", "plegma"),
        "G4118": ("πλεῖστος", "G:A", "most", "pleistos"),
        "G4119": ("πλείων, πλεῖον", "G:A", "greater", "pleiōn, pleion"),
        "G4120": ("πλέκω", "G:V", "to weave", "plekō"),
        "G4121": ("πλεονάζω", "G:V", "to increase", "pleonazō"),
        "G4122": ("πλεονεκτέω", "G:V", "to exploit", "pleonekteō"),
        "G4123": ("πλεονέκτης", "G:N-M", "greedy", "pleonektēs"),
        "G4124": ("πλεονεξία", "G:N-F", "greediness", "pleonexia"),
        "G4125": ("πλευρά", "G:N-F", "side", "pleura"),
        "G4126": ("πλέω", "G:V", "to sail", "pleō"),
        "G4127": ("πληγή", "G:N-F", "plague/blow/wound", "plēgē"),
        "G4128": ("πλῆθος", "G:N-N", "multitude", "plēthos"),
        "G4129": ("πληθύνω", "G:V", "to multiply", "plēthunō"),
        "G4130": ("πλήθω, πίμπλημι", "G:V", "to fill", "plēthō, pimplēmi"),
        "G4131": ("πλήκτης", "G:N-M", "bully", "plēktēs"),
        "G4132": ("πλήμμυρα", "G:N-F", "flood", "plēmmura"),
        "G4133": ("πλήν", "G:Conj", "but/however", "plēn"),
        "G4134": ("πλήρης", "G:A", "full", "plērēs"),
        "G4135": ("πληροφορέω", "G:V", "to fulfill", "plērophoreō"),
        "G4136": ("πληροφορία", "G:N-F", "assurance", "plērophoria"),
        "G4137": ("πληρόω", "G:V", "to fulfill", "plēroō"),
        "G4138": ("πλήρωμα", "G:N-N", "fulfillment", "plērōma"),
        "G4139": ("πλησίον", "G:Prep", "near/neighbor", "plēsion"),
        "G4140": ("πλησμονή", "G:N-F", "gratification", "plēsmonē"),
        "G4141": ("πλήσσω", "G:V", "to strike", "plēssō"),
        "G4142": ("πλοιάριον", "G:N-N", "small boat", "ploiarion"),
        "G4143": ("πλοῖον", "G:N-N", "boat", "ploion"),
        "G4144": ("πλόος", "G:N-M", "voyage", "ploos"),
        "G4145": ("πλούσιος", "G:A", "rich", "plousios"),
        "G4146": ("πλουσίως", "G:Adv", "richly", "plousiōs"),
        "G4147": ("πλουτέω", "G:V", "be rich", "plouteō"),
        "G4148": ("πλουτίζω", "G:V", "to enrich", "ploutizō"),
        "G4149": ("πλοῦτος", "G:N-M", "riches", "ploutos"),
        "G4150": ("πλύνω", "G:V", "to wash", "plunō"),
        "G4151": ("πνεῦμα", "G:N-N", "spirit/breath", "pneuma"),
        "G4152": ("πνευματικός", "G:A", "spiritual", "pneumatikos"),
        "G4153": ("πνευματικῶς", "G:Adv", "spiritually", "pneumatikōs"),
        "G4154": ("πνέω", "G:V", "to blow", "pneō"),
        "G4155": ("πνίγω", "G:V", "to choke", "pnigō"),
        "G4156": ("πνικτός", "G:A", "strangled", "pniktos"),
        "G4157": ("πνοή", "G:N-F", "wind/breath", "pnoē"),
        "G4158": ("ποδήρης", "G:A", "floorlength", "podērēs"),
        "G4159": ("πόθεν", "G:Adv", "whence", "pothen"),
        "G4160": ("ποιέω", "G:V", "to do/make", "poieō"),
        "G4161": ("ποίημα", "G:N-N", "workmanship", "poiēma"),
        "G4162": ("ποίησις", "G:N-F", "doing", "poiēsis"),
        "G4163": ("ποιητής", "G:N-M", "doer", "poiētēs"),
        "G4164": ("ποικίλος", "G:A", "various", "poikilos"),
        "G4165": ("ποιμαίνω", "G:V", "to shepherd", "poimainō"),
        "G4166": ("ποιμήν", "G:N-M", "shepherd", "poimēn"),
        "G4167": ("ποίμνη", "G:N-F", "flock", "poimnē"),
        "G4168": ("ποίμνιον", "G:N-N", "flock", "poimnion"),
        "G4169": ("ποῖος", "G:Intg", "what?", "poios"),
        "G4170": ("πολεμέω", "G:V", "to fight", "polemeō"),
        "G4171": ("πόλεμος", "G:N-M", "war", "polemos"),
        "G4172": ("πόλις", "G:N-F", "city", "polis"),
        "G4173": ("πολιτάρχης", "G:N-M", "city authority", "politarchēs"),
        "G4174": ("πολιτεία", "G:N-F", "citizenship", "politeia"),
        "G4175": ("πολίτευμα", "G:N-N", "citizenship", "politeuma"),
        "G4176": ("πολιτεύω", "G:V", "be a citizen", "politeuō"),
        "G4177": ("πολίτης", "G:N-M", "citizen", "politēs"),
        "G4178": ("πολλάκις", "G:Adv", "often", "pollakis"),
        "G4179": ("πολλαπλασίων", "G:A", "many times more", "pollaplasiōn"),
        "G4180": ("πολυλογία", "G:N-F", "wordiness", "polulogia"),
        "G4181": ("πολυμερῶς", "G:Adv", "little by little", "polumerōs"),
        "G4182": ("πολυποίκιλος", "G:A", "manifold", "polupoikilos"),
        "G4183": ("πολύς", "G:A", "much", "polus"),
        "G4184": ("πολύσπλαγχνος", "G:A", "very compassionate", "polusplagchnos"),
        "G4185": ("πολυτελής", "G:A", "valuable", "polutelēs"),
        "G4186": ("πολύτιμος", "G:A", "valuable", "polutimos"),
        "G4187": ("πολυτρόπως", "G:Adv", "in many ways", "polutropōs"),
        "G4188": ("πόμα", "G:N-N", "drink", "poma"),
        "G4189": ("πονηρία", "G:N-F", "evil", "ponēria"),
        "G4190": ("πονηρός", "G:A", "evil/bad", "ponēros"),
        "G4191": ("πονηρότερος", "G:A", "more evil", "ponēroteros"),
        "G4192": ("πόνος", "G:N-M", "travail", "ponos"),
        "G4193": ("Ποντικός", "G:A", "from Pontus", "Pontikos"),
        "G4194": ("Πόντιος", "N:N-M-P", "Pontius", "Pontios"),
        "G4195": ("Πόντος", "N:N-M-L", "Pontus", "Pontos"),
        "G4196": ("Πόπλιος", "N:N-M-P", "Publius", "Poplios"),
        "G4197": ("πορεία", "G:N-F", "journey", "poreia"),
        "G4198": ("πορεύω", "G:V", "to travel", "poreuō"),
        "G4199": ("πορθέω", "G:V", "to lay waste", "portheō"),
        "G4200": ("πορισμός", "G:N-M", "gain", "porismos"),
        "G4201": ("Πόρκιος", "N:N-M-P", "Porcius", "Porkios"),
        "G4202": ("πορνεία", "G:N-F", "sexual sin", "porneia"),
        "G4203": ("πορνεύω", "G:V", "to sin sexually", "porneuō"),
        "G4204": ("πόρνη", "G:N-F", "prostitute", "pornē"),
        "G4205": ("πόρνος", "G:N-M", "sexual sinner", "pornos"),
        "G4206": ("πόρρω", "G:Adv", "far (away)", "porrō"),
        "G4207": ("πόρρωθεν", "G:Adv", "afar off", "porrōthen"),
        "G4208": ("πορρωτέρω", "G:Adv", "farther", "porrōterō"),
        "G4209": ("πορφύρα", "G:N-F", "purple cloth", "porphura"),
        "G4210": ("πορφύρεος", "G:A", "purple", "porphureos"),
        "G4211": ("πορφυρόπωλις", "G:N-F", "dealer in purple", "porphuropōlis"),
        "G4212": ("ποσάκις", "G:Adv", "how often!", "posakis"),
        "G4213": ("πόσις", "G:N-F", "drink", "posis"),
        "G4214": ("πόσος", "G:Cor+G:Intg", "how much/many", "posos"),
        "G4215": ("ποταμός", "G:N-M", "river", "potamos"),
        "G4216": ("ποταμοφόρητος", "G:A", "flooded", "potamophorētos"),
        "G4217": ("ποταπός", "G:Intg", "of what kind?", "potapos"),
        "G4218": ("ποτέ", "G:Part", "once/when", "pote"),
        "G4219": ("πότε", "G:Intg", "when?", "pote"),
        "G4220": ("πότερος", "G:Adv", "whether", "poteros"),
        "G4221": ("ποτήριον", "G:N-N", "cup", "potērion"),
        "G4222": ("ποτίζω", "G:V", "to water", "potizō"),
        "G4223": ("Ποτίολοι", "N:N-M-L", "Puteoli", "Potioloi"),
        "G4224": ("πότος", "G:N-M", "carousing", "potos"),
        "G4225": ("πού", "G:Adv", "somewhere", "pou"),
        "G4226": ("ποῦ", "G:Adv", "where?", "pou"),
        "G4227": ("Πούδης", "N:N-M-P", "Pudens", "Poudēs"),
        "G4228": ("πούς", "G:N-M", "foot", "pous"),
        "G4229": ("πρᾶγμα", "G:N-N", "thing", "pragma"),
        "G4230": ("πραγματεία", "G:N-F", "affairs", "pragmateia"),
        "G4231": ("πραγματεύομαι", "G:V", "to trade", "pragmateuomai"),
        "G4232": ("πραιτώριον", "G:N-N", "praetorium", "praitōrion"),
        "G4233": ("πράκτωρ", "G:N-M", "bailiff", "praktōr"),
        "G4234": ("πρᾶξις", "G:N-F", "action", "praxis"),
        "G4235": ("πρᾶος", "G:A", "gentle", "praos"),
        "G4236": ("πραότης", "G:N-F", "gentleness", "praotēs"),
        "G4237": ("πρασιά", "G:N-F", "group", "prasia"),
        "G4238": ("πράσσω", "G:V", "to do/require", "prassō"),
        "G4239": ("πραΰς", "G:A", "gentle", "praus"),
        "G4240": ("πραΰτης", "G:N-F", "gentleness", "prautēs"),
        "G4241": ("πρέπω", "G:V", "be proper", "prepō"),
        "G4242": ("πρεσβεία", "G:N-F", "delegation", "presbeia"),
        "G4243": ("πρεσβεύω", "G:V", "be ambassador", "presbeuō"),
        "G4244": ("πρεσβυτέριον", "G:N-N", "council of elders", "presbuterion"),
        "G4245": ("πρεσβύτερος", "G:A", "elder", "presbuteros"),
        "G4246": ("πρεσβύτης", "G:N-M", "old man", "presbutēs"),
        "G4247": ("πρεσβῦτις", "G:N-F", "old woman", "presbutis"),
        "G4248": ("πρηνής", "G:A", "headlong", "prēnēs"),
        "G4249": ("πρίζω", "G:V", "to saw (in two)", "prizō"),
        "G4250": ("πρίν", "G:Prep", "before", "prin"),
        "G4251": ("Πρίσκα", "N:N-F-P", "Prisca", "Priska"),
        "G4252": ("Πρίσκιλλα", "N:N-F-P", "Priscilla", "Priskilla"),
        "G4253": ("πρό", "G:Prep", "before", "pro"),
        "G4254": ("προάγω", "G:V", "to go/bring before", "proagō"),
        "G4255": ("προαιρέω", "G:V", "to predetermine", "proaireō"),
        "G4256": ("προαιτιάομαι", "G:V", "to accuse", "proaitiaomai"),
        "G4257": ("προακούω", "G:V", "to hear beforehand", "proakouō"),
        "G4258": ("προαμαρτάνω", "G:V", "to sin beforehand", "proamartanō"),
        "G4259": ("προαύλιον", "G:N-N", "entryway", "proaulion"),
        "G4260": ("προβαίνω", "G:V", "to advance", "probainō"),
        "G4261": ("προβάλλω", "G:V", "to put forth", "proballō"),
        "G4262": ("προβατικός", "G:A", "Sheep Gate", "probatikos"),
        "G4263": ("πρόβατον", "G:N-N", "sheep", "probaton"),
        "G4264": ("προβιβάζω", "G:V", "to prompt", "probibazō"),
        "G4265": ("προβλέπω", "G:V", "to foresee/plan", "problepō"),
        "G4266": ("προγίνομαι", "G:V", "to do beforehand", "proginomai"),
        "G4267": ("προγινώσκω", "G:V", "to know/choose", "proginōskō"),
        "G4268": ("πρόγνωσις", "G:N-F", "foreknowledge", "prognōsis"),
        "G4269": ("πρόγονος", "G:N-M", "parent/ancestor", "progonos"),
        "G4270": ("προγράφω", "G:V", "to write/designate", "prographō"),
        "G4271": ("πρόδηλος", "G:A", "obvious", "prodēlos"),
        "G4272": ("προδίδωμι", "G:V", "to give in advance", "prodidōmi"),
        "G4273": ("προδότης", "G:N-M", "traitor", "prodotēs"),
        "G4274": ("πρόδρομος", "G:A", "forerunner", "prodromos"),
        "G4275": ("προεῖδον", "G:V", "to foresee", "proeidon"),
        "G4276": ("προελπίζω", "G:V", "to hope beforehand", "proelpizō"),
        "G4277": ("προέπω", "G:V", "to predict", "proepō"),
        "G4278": ("προενάρχομαι", "G:V", "to begin beforehand", "proenarchomai"),
        "G4279": ("προεπαγγέλλω", "G:V", "to promise", "proepaggellō"),
        "G4280": ("προερέω", "G:V", "to predict", "proereō"),
        "G4281": ("προέρχομαι", "G:V", "to go before", "proerchomai"),
        "G4282": ("προετοιμάζω", "G:V", "to prepare", "proetoimazō"),
        "G4283": ("προευαγγελίζομαι", "G:V", "to speak good news", "proeuaggelizomai"),
        "G4284": ("προέχω", "G:V", "to excel", "proechō"),
        "G4285": ("προηγέομαι", "G:V", "to prefer", "proēgeomai"),
        "G4286": ("πρόθεσις", "G:N-F", "purpose", "prothesis"),
        "G4287": ("προθεσμία", "G:N-F", "set time", "prothesmia"),
        "G4288": ("προθυμία", "G:N-F", "eagerness", "prothumia"),
        "G4289": ("πρόθυμος", "G:A", "eager", "prothumos"),
        "G4290": ("προθύμως", "G:Adv", "eagerly", "prothumōs"),
        "G4291": ("προΐστημι", "G:V", "to set before", "proistēmi"),
        "G4292": ("προκαλέω", "G:V", "to provoke", "prokaleō"),
        "G4293": ("προκαταγγέλλω", "G:V", "to foretell", "prokataggellō"),
        "G4294": ("προκαταρτίζω", "G:V", "to arrange", "prokatartizō"),
        "G4295": ("πρόκειμαι", "G:V", "to set before", "prokeimai"),
        "G4296": ("προκηρύσσω", "G:V", "to announce", "prokērussō"),
        "G4297": ("προκοπή", "G:N-F", "progress", "prokopē"),
        "G4298": ("προκόπτω", "G:V", "to advance", "prokoptō"),
        "G4299": ("πρόκριμα", "G:N-N", "prejudice", "prokrima"),
        "G4300": ("προκυρόομαι", "G:V", "to establish", "prokuroomai"),
        "G4301": ("προλαμβάνω", "G:V", "to take beforehand", "prolambanō"),
        "G4302": ("προλέγω", "G:V", "to foretell", "prolegō"),
        "G4303": ("προμαρτύρομαι", "G:V", "to predict", "promarturomai"),
        "G4304": ("προμελετάω", "G:V", "to premeditate", "promeletaō"),
        "G4305": ("προμεριμνάω", "G:V", "to worry beforehand", "promerimnaō"),
        "G4306": ("προνοέω", "G:V", "to care for", "pronoeō"),
        "G4307": ("πρόνοια", "G:N-F", "foresight", "pronoia"),
        "G4308": ("προοράω", "G:V", "to foresee", "prooraō"),
        "G4309": ("προορίζω", "G:V", "to predestine", "proorizō"),
        "G4310": ("προπάσχω", "G:V", "to suffer before", "propaschō"),
        "G4311": ("προπέμπω", "G:V", "to help on the way", "propempō"),
        "G4312": ("προπετής", "G:A", "reckless", "propetēs"),
        "G4313": ("προπορεύω", "G:V", "to go before", "proporeuō"),
        "G4314": ("πρός", "G:Prep", "to/with", "pros"),
        "G4315": ("προσάββατον", "G:N-N", "Friday", "prosabbaton"),
        "G4316": ("προσαγορεύω", "G:V", "to designate", "prosagoreuō"),
        "G4317": ("προσάγω", "G:V", "to bring near", "prosagō"),
        "G4318": ("προσαγωγή", "G:N-F", "access", "prosagōgē"),
        "G4319": ("προσαιτέω", "G:V", "to beg", "prosaiteō"),
        "G4320": ("προσαναβαίνω", "G:V", "to go up", "prosanabainō"),
        "G4321": ("προσαναλίσκω", "G:V", "to expend", "prosanaliskō"),
        "G4322": ("προσαναπληρόω", "G:V", "to supply", "prosanaplēroō"),
        "G4323": ("προσανατίθημι", "G:V", "to confer", "prosanatithēmi"),
        "G4324": ("προσαπειλέω", "G:V", "to threaten further", "prosapeileō"),
        "G4325": ("προσδαπανάω", "G:V", "to spend extra", "prosdapanaō"),
        "G4326": ("προσδέω", "G:V", "to need", "prosdeō"),
        "G4327": ("προσδέχομαι", "G:V", "to wait for/welcome", "prosdechomai"),
        "G4328": ("προσδοκάω", "G:V", "to look for", "prosdokaō"),
        "G4329": ("προσδοκία", "G:N-F", "expectation", "prosdokia"),
        "G4330": ("προσεάω", "G:V", "to permit", "proseaō"),
        "G4331": ("προσεγγίζω", "G:V", "to approach", "proseggizō"),
        "G4332": ("προσεδρεύω", "G:V", "to sit near/serve", "prosedreuō"),
        "G4333": ("προσεργάζομαι", "G:V", "to earn more", "prosergazomai"),
        "G4334": ("προσέρχομαι", "G:V", "to come near/agree", "proserchomai"),
        "G4335": ("προσευχή", "G:N-F", "prayer", "proseuchē"),
        "G4336": ("προσεύχομαι", "G:V", "to pray", "proseuchomai"),
        "G4337": ("προσέχω", "G:V", "to watch out", "prosechō"),
        "G4338": ("προσηλόω", "G:V", "to nail", "prosēloō"),
        "G4339": ("προσήλυτος", "G:N-M", "proselyte", "prosēlutos"),
        "G4340": ("πρόσκαιρος", "G:A", "temporary", "proskairos"),
        "G4341": ("προσκαλέω", "G:V", "to call to/summon", "proskaleō"),
        "G4342": ("προσκαρτερέω", "G:V", "to continue in/with", "proskartereō"),
        "G4343": ("προσκαρτέρησις", "G:N-F", "perseverance", "proskarterēsis"),
        "G4344": ("προσκεφάλαιον", "G:N-N", "cushion", "proskephalaion"),
        "G4345": ("προσκληρόω", "G:V", "to join", "prosklēroō"),
        "G4346": ("πρόσκλισις", "G:N-F", "partiality", "prosklisis"),
        "G4347": ("προσκολλάω", "G:V", "to join", "proskollaō"),
        "G4348": ("πρόσκομμα", "G:N-N", "stumbling block", "proskomma"),
        "G4349": ("προσκοπή", "G:N-F", "stumbling", "proskopē"),
        "G4350": ("προσκόπτω", "G:V", "to strike", "proskoptō"),
        "G4351": ("προσκυλίω", "G:V", "to roll before", "proskuliō"),
        "G4352": ("προσκυνέω", "G:V", "to worship", "proskuneō"),
        "G4353": ("προσκυνητής", "G:N-M", "worshiper", "proskunētēs"),
        "G4354": ("προσλαλέω", "G:V", "to talk to/with", "proslaleō"),
        "G4355": ("προσλαμβάνω", "G:V", "to take", "proslambanō"),
        "G4356": ("πρόσληψις", "G:N-F", "acceptance", "proslēpsis"),
        "G4357": ("προσμένω", "G:V", "to remain/persist", "prosmenō"),
        "G4358": ("προσορμίζω", "G:V", "to moor/anchor", "prosormizō"),
        "G4359": ("προσοφείλω", "G:V", "to owe besides", "prosopheilō"),
        "G4360": ("προσοχθίζω", "G:V", "be angry", "prosochthizō"),
        "G4361": ("πρόσπεινος", "G:A", "very hungry", "prospeinos"),
        "G4362": ("προσπήγνυμι", "G:V", "to nail to", "prospēgnumi"),
        "G4363": ("προσπίπτω", "G:V", "to fall/beat", "prospiptō"),
        "G4364": ("προσποιέω", "G:V", "to pretend", "prospoieō"),
        "G4365": ("προσπορεύομαι", "G:V", "to approach", "prosporeuomai"),
        "G4366": ("προσρήγνυμι", "G:V", "to strike", "prosrēgnumi"),
        "G4367": ("προστάσσω", "G:V", "to order", "prostassō"),
        "G4368": ("προστάτις", "G:N-F", "patroness", "prostatis"),
        "G4369": ("προστίθημι", "G:V", "to add (to)", "prostithēmi"),
        "G4370": ("προστρέχω", "G:V", "to hasten", "prostrechō"),
        "G4371": ("προσφάγιον", "G:N-N", "fish", "prosphagion"),
        "G4372": ("πρόσφατος", "G:A", "new", "prosphatos"),
        "G4373": ("προσφάτως", "G:Adv", "recently", "prosphatōs"),
        "G4374": ("προσφέρω", "G:V", "to bring to", "prospherō"),
        "G4375": ("προσφιλής", "G:A", "lovely", "prosphilēs"),
        "G4376": ("προσφορά", "G:N-F", "offering", "prosphora"),
        "G4377": ("προσφωνέω", "G:V", "to call to/summon", "prosphōneō"),
        "G4378": ("πρόσχυσις", "G:N-F", "sprinkling", "proschusis"),
        "G4379": ("προσψαύω", "G:V", "to touch", "prospsauō"),
        "G4380": ("προσωποληπτέω", "G:V", "to favor", "prosōpolēpteō"),
        "G4381": ("προσωπολήπτης", "G:N-M", "prejudiced person", "prosōpolēptēs"),
        "G4382": ("προσωποληψία", "G:N-F", "favoritism", "prosōpolēpsia"),
        "G4383": ("πρόσωπον", "G:N-N", "face", "prosōpon"),
        "G4384": ("προτάσσω", "G:V", "to predetermine", "protassō"),
        "G4385": ("προτείνω", "G:V", "to stretch out", "proteinō"),
        "G4386": ("πρότερον", "G:A / G:Adv", "before", "proteron"),
        "G4387": ("πρότερος", "G:A", "before", "proteros"),
        "G4388": ("προτίθημι", "G:V", "to plan/present", "protithēmi"),
        "G4389": ("προτρέπω", "G:V", "to encourage", "protrepō"),
        "G4390": ("προτρέχω", "G:V", "to outrun", "protrechō"),
        "G4391": ("προϋπάρχω", "G:V", "be formerly", "prouparchō"),
        "G4392": ("πρόφασις", "G:N-F", "pretense", "prophasis"),
        "G4393": ("προφέρω", "G:V", "to bring out", "propherō"),
        "G4394": ("προφητεία", "G:N-F", "prophecy", "prophēteia"),
        "G4395": ("προφητεύω", "G:V", "to prophesy", "prophēteuō"),
        "G4396": ("προφήτης", "G:N-M", "prophet", "prophētēs"),
        "G4397": ("προφητικός", "G:A", "prophetic", "prophētikos"),
        "G4398": ("προφῆτις", "G:N-F", "prophetess", "prophētis"),
        "G4399": ("προφθάνω", "G:V", "to come before", "prophthanō"),
        "G4400": ("προχειρίζω", "G:V", "to appoint", "procheirizō"),
        "G4401": ("προχειροτονέω", "G:V", "to choose", "procheirotoneō"),
        "G4402": ("Πρόχορος", "N:N-M-P", "Procorus", "Prochoros"),
        "G4403": ("πρύμνα", "G:N-F", "stern", "prumna"),
        "G4404": ("πρωΐ", "G:Adv", "early", "prōi"),
        "G4405": ("πρωΐα", "G:N-F", "early morning", "prōia"),
        "G4406": ("πρώϊμος", "G:A", "early rain", "prōimos"),
        "G4407": ("πρωϊνός", "G:A", "morning", "prōinos"),
        "G4408": ("πρῷρα", "G:N-F", "bow", "prōra"),
        "G4409": ("πρωτεύω", "G:V", "be first", "prōteuō"),
        "G4410": ("πρωτοκαθεδρία", "G:N-F", "seat of honor", "prōtokathedria"),
        "G4411": ("πρωτοκλισία", "G:N-F", "place of honor", "prōtoklisia"),
        "G4412": ("πρῶτον", "G:Adv", "first", "prōton"),
        "G4413": ("πρῶτος", "G:A", "first", "prōtos"),
        "G4414": ("πρωτοστάτης", "G:N-M", "ringleader", "prōtostatēs"),
        "G4415": ("πρωτοτόκια", "G:N-N", "birthright", "prōtotokia"),
        "G4416": ("πρωτότοκος", "G:A", "firstborn", "prōtotokos"),
        "G4417": ("πταίω", "G:V", "to stumble", "ptaiō"),
        "G4418": ("πτέρνα", "G:N-F", "heel", "pterna"),
        "G4419": ("πτερύγιον", "G:N-N", "pinnacle", "pterugion"),
        "G4420": ("πτέρυξ", "G:N-F", "wing", "pterux"),
        "G4421": ("πτηνός", "G:A", "bird", "ptēnos"),
        "G4422": ("πτοέω", "G:V", "to frighten", "ptoeō"),
        "G4423": ("πτόησις", "G:N-F", "fear", "ptoēsis"),
        "G4424": ("Πτολεμαΐς", "N:N-F-L", "Ptolemais", "Ptolemais"),
        "G4425": ("πτύον", "G:N-N", "winnowing fork", "ptuon"),
        "G4426": ("πτύρομαι", "G:V", "to frighten", "pturomai"),
        "G4427": ("πτύσμα", "G:N-N", "saliva", "ptusma"),
        "G4428": ("πτύσσω", "G:V", "to roll up", "ptussō"),
        "G4429": ("πτύω", "G:V", "to spit", "ptuō"),
        "G4430": ("πτῶμα", "G:N-N", "corpse", "ptōma"),
        "G4431": ("πτῶσις", "G:N-F", "fall", "ptōsis"),
        "G4432": ("πτωχεία", "G:N-F", "poverty", "ptōcheia"),
        "G4433": ("πτωχεύω", "G:V", "be poor", "ptōcheuō"),
        "G4434": ("πτωχός", "G:A", "poor", "ptōchos"),
        "G4435": ("πυγμή", "G:N-F", "fist", "pugmē"),
        "G4436": ("Πύθων", "G:N-M", "divining spirit", "Puthōn"),
        "G4437": ("πυκνός", "G:A", "often", "puknos"),
        "G4438": ("πυκτεύω", "G:V", "to box", "pukteuō"),
        "G4439": ("πύλη", "G:N-F", "gate", "pulē"),
        "G4440": ("πυλών", "G:N-M", "gate", "pulōn"),
        "G4441": ("πυνθάνομαι", "G:V", "to inquire", "punthanomai"),
        "G4442": ("πῦρ", "G:N-N", "fire", "pur"),
        "G4443": ("πυρά", "G:N-F", "fire", "pura"),
        "G4444": ("πύργος", "G:N-M", "tower", "purgos"),
        "G4445": ("πυρέσσω", "G:V", "be feverish", "puressō"),
        "G4446": ("πυρετός", "G:N-M", "fever", "puretos"),
        "G4447": ("πύρινος", "G:A", "fiery", "purinos"),
        "G4448": ("πυρόω", "G:V", "to burn", "puroō"),
        "G4449": ("πυρράζω", "G:V", "be (fiery) red", "purrazō"),
        "G4450": ("πυρρός", "G:A", "fiery red", "purros"),
        "G4451": ("πύρωσις", "G:N-F", "burning", "purōsis"),
        "G4452": ("πω", "G:Part", "somehow", "pō"),
        "G4453": ("πωλέω", "G:V", "to sell", "pōleō"),
        "G4454": ("πῶλος", "G:N-M", "colt", "pōlos"),
        "G4455": ("πώποτε", "G:Adv", "ever", "pōpote"),
        "G4456": ("πωρόω", "G:V", "to harden", "pōroō"),
        "G4457": ("πώρωσις", "G:N-F", "hardening", "pōrōsis"),
        "G4458": ("πως", "G:Part", "how", "pōs"),
        "G4459": ("πῶς", "G:Intg", "how?!", "pōs"),
        "G4460": ("Ῥαάβ", "N:N-F-P", "Rahab", "Rhaab"),
        "G4461": ("ῥαββί", "H:N", "Rabbi", "rhabbi"),
        "G4462": ("ῥαββονί", "A:N", "Rabboni", "rhabboni"),
        "G4463": ("ῥαβδίζω", "G:V", "to beat with a rod", "rhabdizō"),
        "G4464": ("ῥάβδος", "G:N-F", "rod", "rhabdos"),
        "G4465": ("ῥαβδοῦχος", "G:N-M", "police", "rhabdouchos"),
        "G4466": ("Ῥαγαῦ", "N:N-M-P", "Reu", "Rhagau"),
        "G4467": ("ῥᾳδιούργημα", "G:N-N", "crime", "rhadiourgēma"),
        "G4468": ("ῥᾳδιουργία", "G:N-F", "trickery", "rhadiourgia"),
        "G4469": ("ῥακά", "A:N", "Raca", "rhaka"),
        "G4470": ("ῥάκος", "G:N-N", "a cloth", "rhakos"),
        "G4471": ("Ῥαμᾶ", "N:N--L", "Ramah", "Rhama"),
        "G4472": ("ῥαντίζω", "G:V", "to sprinkle", "rhantizō"),
        "G4473": ("ῥαντισμός", "G:N-M", "sprinkling", "rhantismos"),
        "G4474": ("ῥαπίζω", "G:V", "to slap", "rhapizō"),
        "G4475": ("ῥάπισμα", "G:N-N", "slap", "rhapisma"),
        "G4476": ("ῥαφίς", "G:N-F", "needle", "rhaphis"),
        "G4477": ("Ῥαχάβ", "N:N-F-P", "Rahab", "Rhachab"),
        "G4478": ("Ῥαχήλ", "N:N-F-P", "Rachel", "Rhachēl"),
        "G4479": ("Ῥεβέκκα", "N:N-F-P", "Rebekah", "Rhebekka"),
        "G4480": ("ῥέδα", "G:N-F", "carriage", "rheda"),
        "G4481": ("Ῥεμφάν", "N:N--T", "Rephan", "Rhemphan, Rhaiphan"),
        "G4482": ("ῥέω", "G:V", "to flow", "rheō"),
        "G4483": ("ῥέω, ἐρέω", "G:V", "to utter", "rheō, ereō"),
        "G4484": ("Ῥήγιον", "N:N--L", "Rhegium", "Rhēgion"),
        "G4485": ("ῥῆγμα", "G:N-N", "destruction", "rhēgma"),
        "G4486": ("ῥήγνυμι", "G:V", "to throw violently", "rhēgnumi"),
        "G4487": ("ῥῆμα", "G:N-N", "declaration", "rhēma"),
        "G4488": ("Ῥησά", "N:N-M-P", "Rhesa", "Rhēsa"),
        "G4489": ("ῥήτωρ", "G:N-M", "spokesman", "rhētōr"),
        "G4490": ("ῥητῶς", "G:Adv", "expressly", "rhētōs"),
        "G4491": ("ῥίζα", "G:N-F", "root", "rhiza"),
        "G4492": ("ῥιζόω", "G:V", "to root", "rhizoō"),
        "G4493": ("ῥιπή", "G:N-F", "twinkling", "rhipē"),
        "G4494": ("ῥιπίζω", "G:V", "to toss about", "rhipizō"),
        "G4495": ("ῥιπτέω", "G:V", "to throw off", "rhipteō"),
        "G4496": ("ῥίπτω", "G:V", "to throw/lay down", "rhiptō"),
        "G4497": ("Ῥοβοάμ", "N:N-M-P", "Rehoboam", "Rhoboam"),
        "G4498": ("Ῥόδη", "N:N-F-P", "Rhoda", "Rhodē"),
        "G4499": ("Ῥόδος", "N:N-F-L", "Rhodes", "Rhodos"),
        "G4500": ("ῥοιζηδόν", "G:Adv", "with a roar", "rhoizēdon"),
        "G4501": ("ῥομφαία", "G:N-F", "sword", "rhomphaia"),
        "G4502": ("Ῥουβήν", "N:N-M-P", "Reuben", "Rhoubēn"),
        "G4503": ("Ῥούθ", "N:N-F-P", "Ruth", "Rhouth"),
        "G4504": ("Ῥοῦφος", "N:N-M-P", "Rufus", "Rhouphos"),
        "G4505": ("ῥύμη", "G:N-F", "lane", "rhumē"),
        "G4506": ("ῥύομαι", "G:V", "to rescue", "rhuomai"),
        "G4507": ("ῥυπαρία", "G:N-F", "filth", "rhuparia"),
        "G4508": ("ῥυπαρός", "G:A", "filthy", "rhuparos"),
        "G4509": ("ῥύπος", "G:N-M", "filth", "rhupos"),
        "G4510": ("ῥυπόω", "G:V", "to defile", "rhupoō"),
        "G4511": ("ῥύσις", "G:N-F", "discharge", "rhusis"),
        "G4512": ("ῥυτίς", "G:N-F", "wrinkle", "rhutis"),
        "G4513": ("Ῥωμαϊκός", "N:Adv--L", "Latin", "Rhōmaikos"),
        "G4514": ("Ῥωμαῖος", "N:N--LG", "Roman", "Rhōmaios"),
        "G4515": ("Ῥωμαϊστί", "G:Adv", "in Latin", "Rhōmaisti"),
        "G4516": ("Ῥώμη", "N:N-F-L", "Rome", "Rhōmē"),
        "G4517": ("ῥώννυμι", "G:V", "farewell", "rhōnnumi"),
        "G4518": ("σαβαχθάνι", "A:V", "sabachthani", "sabachthani"),
        "G4519": ("σαβαώθ", "H:N", "of Hosts", "sabaōth"),
        "G4520": ("σαββατισμός", "G:N-M", "Sabbath", "sabbatismos"),
        "G4521": ("σάββατον", "G:N-N", "Sabbath", "sabbaton"),
        "G4522": ("σαγήνη", "G:N-F", "dragnet", "sagēnē"),
        "G4523": ("Σαδδουκαῖος", "N:N--T", "Sadducee", "Saddoukaios"),
        "G4524": ("Σαδώκ", "N:N-M-P", "Zadok", "Sadōk"),
        "G4525": ("σαίνω", "G:V", "to shake", "sainō"),
        "G4526": ("σάκκος", "G:N-M", "sackcloth", "sakkos"),
        "G4527": ("Σαλά", "N:N-M-P", "Sala", "Sala"),
        "G4528": ("Σαλαθιήλ", "N:N-M-P", "Salathiel", "Salathiēl"),
        "G4529": ("Σαλαμίς", "N:N--L", "Salamis", "Salamis"),
        "G4530": ("Σαλείμ", "N:N--L", "Salim", "Saleim"),
        "G4531": ("σαλεύω", "G:V", "to shake", "saleuō"),
        "G4532": ("Σαλήμ", "N:N--L", "Salem", "Salēm"),
        "G4533": ("Σαλμών", "N:N-M-P", "Salmon", "Salmōn"),
        "G4534": ("Σαλμώνη", "N:N--L", "Salmone", "Salmōnē"),
        "G4535": ("σάλος", "G:N-M", "tossing", "salos"),
        "G4536": ("σάλπιγξ", "G:N-F", "trumpet", "salpigx"),
        "G4537": ("σαλπίζω", "G:V", "to sound a trumpet", "salpizō"),
        "G4538": ("σαλπιστής", "G:N-M", "trumpeter", "salpistēs"),
        "G4539": ("Σαλώμη", "N:N-F-P", "Salome", "Salōmē"),
        "G4540": ("Σαμάρεια", "N:N-F-L", "Samaria", "Samareia"),
        "G4541": ("Σαμαρείτης", "N:N--LG", "Samaritan", "Samareitēs"),
        "G4542": ("Σαμαρεῖτις", "N:N-F-LG", "Samaritan", "Samareitis"),
        "G4543": ("Σαμοθρᾴκη", "N:N-F-L", "Samothrace", "Samothrakē"),
        "G4544": ("Σάμος", "N:N-F-L", "Samos", "Samos"),
        "G4545": ("Σαμουήλ", "N:N-M-P", "Samuel", "Samouēl"),
        "G4546": ("Σαμψών", "N:N-M-P", "Samson", "Sampsōn"),
        "G4547": ("σανδάλιον", "G:N-N", "sandal", "sandalion"),
        "G4548": ("σανίς", "G:N-F", "plank", "sanis"),
        "G4549": ("Σαούλ", "N:N-M-P", "Saul", "Saoul"),
        "G4550": ("σαπρός", "G:A", "rotten", "sapros"),
        "G4551": ("Σαπφείρη", "N:N-F-P", "Sapphira", "Sappheirē"),
        "G4552": ("σάπφειρος", "G:N-F", "sapphire", "sappheiros"),
        "G4553": ("σαργάνη", "G:N-F", "basket", "sarganē"),
        "G4554": ("Σάρδεις", "N:N-F-L", "Sardis", "Sardeis"),
        "G4555": ("σάρδινος", "G:N-M", "gem", "sardinos"),
        "G4556": ("σάρδιον", "G:N-N", "gem", "sardion"),
        "G4557": ("σαρδόνυξ", "G:N-M", "sardonyx", "sardonux"),
        "G4558": ("Σάρεπτα", "N:N--L", "Zarephath", "Sarepta"),
        "G4559": ("σαρκικός", "G:A", "fleshly", "sarkikos"),
        "G4560": ("σάρκινος", "G:A", "fleshly", "sarkinos"),
        "G4561": ("σάρξ", "G:N-F", "flesh", "sarx"),
        "G4562": ("Σαρούχ", "N:N-M-P", "Serug", "Sarouch"),
        "G4563": ("σαρόω", "G:V", "to sweep", "saroō"),
        "G4564": ("Σάρρα", "N:N-F-P", "Sarah", "Sarra"),
        "G4565": ("Σάρων", "N:N--L", "Sharon", "Sarōn"),
        "G4566": ("Σατάν", "N:N--T", "Satan", "Satan"),
        "G4567": ("Σατανᾶς", "N:N--T", "Satan", "Satanas"),
        "G4568": ("σάτον", "G:N-N", "seah", "saton"),
        "G4569": ("Σαῦλος", "N:N-M-P", "Saul", "Saulos"),
        "G4570": ("σβέννυμι", "G:V", "to extinguish", "sbennumi"),
        "G4571": ("σέ", "G:PerP", "you", "se"),
        "G4572": ("σεαυτοῦ", "G:RefP", "yourself", "seautou"),
        "G4573": ("σεβάζομαι", "G:V", "to worship", "sebazomai"),
        "G4574": ("σέβασμα", "G:N-N", "object of worship", "sebasma"),
        "G4575": ("σεβαστός", "G:A", "emperor", "sebastos"),
        "G4576": ("σέβομαι", "G:V", "be devout", "sebomai"),
        "G4577": ("σειρά", "G:N-F", "chain", "seira"),
        "G4578": ("σεισμός", "G:N-M", "earthquake", "seismos"),
        "G4579": ("σείω", "G:V", "to shake", "seiō"),
        "G4580": ("Σεκοῦνδος", "N:N-M-P", "Secundus", "Sekoundos"),
        "G4581": ("Σελεύκεια", "N:N-F-L", "Seleucia", "Seleukeia"),
        "G4582": ("σελήνη", "G:N-F", "moon", "selēnē"),
        "G4583": ("σεληνιάζομαι", "G:V", "be epileptic", "selēniazomai"),
        "G4584": ("Σεμεΐ", "N:N-M-P", "Semein", "Semei"),
        "G4585": ("σεμίδαλις", "G:N-F", "fine flour", "semidalis"),
        "G4586": ("σεμνός", "G:A", "noble", "semnos"),
        "G4587": ("σεμνότης", "G:N-F", "dignity", "semnotēs"),
        "G4588": ("Σέργιος", "N:N-M-P", "Sergius", "Sergios"),
        "G4589": ("Σήθ", "N:N-M-P", "Seth", "Sēth"),
        "G4590": ("Σήμ", "N:N-M-P", "Shem", "Sēm"),
        "G4591": ("σημαίνω", "G:V", "to signify", "sēmainō"),
        "G4592": ("σημεῖον", "G:N-N", "sign", "sēmeion"),
        "G4593": ("σημειόω", "G:V", "to note", "sēmeioō"),
        "G4594": ("σήμερον", "G:Adv", "today", "sēmeron"),
        "G4595": ("σήπω", "G:V", "to rot", "sēpō"),
        "G4596": ("σηρικός", "G:A", "silk", "sērikos"),
        "G4597": ("σής", "G:N-M", "moth", "sēs"),
        "G4598": ("σητόβρωτος", "G:A", "moth-eaten", "sētobrōtos"),
        "G4599": ("σθενόω", "G:V", "to strengthen", "sthenoō"),
        "G4600": ("σιαγών", "G:N-F", "cheek", "siagōn"),
        "G4601": ("σιγάω", "G:V", "be silent", "sigaō"),
        "G4602": ("σιγή", "G:N-F", "silence", "sigē"),
        "G4603": ("σιδήρεος", "G:A", "iron", "sidēreos"),
        "G4604": ("σίδηρος", "G:N-M", "iron", "sidēros"),
        "G4605": ("Σιδών", "N:N-F-L", "Sidon", "Sidōn"),
        "G4606": ("Σιδώνιος", "N:N--LG", "Sidonian", "Sidōnios"),
        "G4607": ("σικάριος", "G:N-M", "assassin", "sikarios"),
        "G4608": ("σίκερα", "G:N-N", "alcoholic drink", "sikera"),
        "G4609": ("Σίλας", "N:N-M-P", "Silas", "Silas"),
        "G4610": ("Σιλουανός", "N:N-M-P", "Silas/Silvanus", "Silouanos"),
        "G4611": ("Σιλωάμ", "N:N--L", "Siloam", "Silōam"),
        "G4612": ("σιμικίνθιον", "G:N-N", "apron", "simikinthion"),
        "G4613": ("Σίμων", "N:N-M-P", "Simon", "Simōn"),
        "G4614": ("Σινᾶ", "N:N--L", "Sinai", "Sina"),
        "G4615": ("σίναπι", "G:N-N", "mustard", "sinapi"),
        "G4616": ("σινδών", "G:N-F", "linen", "sindōn"),
        "G4617": ("σινιάζω", "G:V", "to sift", "siniazō"),
        "G4618": ("σιτευτός", "G:A", "fattened", "siteutos"),
        "G4619": ("σιτιστός", "G:A", "fattened", "sitistos"),
        "G4620": ("σιτομέτριον", "G:N-N", "grain ration", "sitometrion"),
        "G4621": ("σῖτος", "G:N-M/N", "grain", "sitos"),
        "G4622": ("Σιών", "N:N--L", "Zion", "Siōn"),
        "G4623": ("σιωπάω", "G:V", "be quiet", "siōpaō"),
        "G4624": ("σκανδαλίζω", "G:V", "to cause to stumble", "skandalizō"),
        "G4625": ("σκάνδαλον", "G:N-N", "stumbling block", "skandalon"),
        "G4626": ("σκάπτω", "G:V", "to dig", "skaptō"),
        "G4627": ("σκάφη", "G:N-F", "lifeboat", "skaphē"),
        "G4628": ("σκέλος", "G:N-N", "leg", "skelos"),
        "G4629": ("σκέπασμα", "G:N-N", "clothing", "skepasma"),
        "G4630": ("Σκευᾶς", "N:N-M-P", "Sceva", "Skeuas"),
        "G4631": ("σκευή", "G:N-F", "gear", "skeuē"),
        "G4632": ("σκεῦος", "G:N-N", "vessel", "skeuos"),
        "G4633": ("σκηνή", "G:N-F", "tent", "skēnē"),
        "G4634": ("σκηνοπηγία", "G:N-F", "Feast of Booths", "skēnopēgia"),
        "G4635": ("σκηνοποιός", "G:N-M", "tentmaker", "skēnopoios"),
        "G4636": ("σκῆνος", "G:N-N", "tent", "skēnos"),
        "G4637": ("σκηνόω", "G:V", "to dwell", "skēnoō"),
        "G4638": ("σκήνωμα", "G:N-N", "tent", "skēnōma"),
        "G4639": ("σκιά", "G:N-F", "shadow", "skia"),
        "G4640": ("σκιρτάω", "G:V", "to leap", "skirtaō"),
        "G4641": ("σκληροκαρδία", "G:N-F", "hardness of heart", "sklērokardia"),
        "G4642": ("σκληρός", "G:A", "hard", "sklēros"),
        "G4643": ("σκληρότης", "G:N-F", "hardness", "sklērotēs"),
        "G4644": ("σκληροτράχηλος", "G:A", "stiff-necked", "sklērotrachēlos"),
        "G4645": ("σκληρύνω", "G:V", "to harden", "sklērunō"),
        "G4646": ("σκολιός", "G:A", "crooked", "skolios"),
        "G4647": ("σκόλοψ", "G:N-M", "thorn", "skolops"),
        "G4648": ("σκοπέω", "G:V", "to watch out", "skopeō"),
        "G4649": ("σκοπός", "G:N-M", "goal", "skopos"),
        "G4650": ("σκορπίζω", "G:V", "to scatter", "skorpizō"),
        "G4651": ("σκορπίος", "G:N-M", "scorpion", "skorpios"),
        "G4652": ("σκοτεινός", "G:A", "dark", "skoteinos"),
        "G4653": ("σκοτία", "G:N-F", "darkness", "skotia"),
        "G4654": ("σκοτίζω", "G:V", "to darken", "skotizō"),
        "G4655": ("σκότος", "G:N-M/N", "darkness", "skotos"),
        "G4656": ("σκοτόω", "G:V", "to darken", "skotoō"),
        "G4657": ("σκύβαλον", "G:N-N", "garbage", "skubalon"),
        "G4658": ("Σκύθης", "N:N--LG", "Scythian", "Skuthēs"),
        "G4659": ("σκυθρωπός", "G:A", "be sad", "skuthrōpos"),
        "G4660": ("σκύλλω", "G:V", "to trouble", "skullō"),
        "G4661": ("σκῦλον", "G:N-N", "plunder", "skulon"),
        "G4662": ("σκωληκόβρωτος", "G:A", "worm-eated", "skōlēkobrōtos"),
        "G4663": ("σκώληξ", "G:N-M", "worm", "skōlēx"),
        "G4664": ("σμαράγδινος", "G:A", "emerald", "smaragdinos"),
        "G4665": ("σμάραγδος", "G:N-M", "emerald", "smaragdos"),
        "G4666": ("σμύρνα", "G:N-F", "myrrh", "smurna"),
        "G4667": ("Σμύρνα", "N:N-F-L", "Smyrna", "Smurna"),
        "G4668": ("Σμυρναῖος", "G:N-M", "Smyrnaean", "Smurnaios"),
        "G4669": ("σμυρνίζω", "G:V", "to mix with myrrh", "smurnizō"),
        "G4670": ("Σόδομα", "N:N--L", "Sodom", "Sodoma"),
        "G4671": ("σοί", "G:PerP", "you", "soi"),
        "G4672": ("Σολομών, Σαλωμών", "N:N-M-P", "Solomon", "Solomōn, Salōmōn"),
        "G4673": ("σορός", "G:N-F", "bier", "soros"),
        "G4674": ("σός", "G:PosP", "your", "sos"),
        "G4675": ("σοῦ", "G:PerP", "you", "sou"),
        "G4676": ("σουδάριον", "G:N-N", "handkerchief", "soudarion"),
        "G4677": ("Σουσάννα", "N:N-F-P", "Susanna", "Sousanna"),
        "G4678": ("σοφία", "G:N-F", "wisdom", "sophia"),
        "G4679": ("σοφίζω", "G:V", "to make wise", "sophizō"),
        "G4680": ("σοφός", "G:A", "wise", "sophos"),
        "G4681": ("Σπανία", "N:N-F-L", "Spain", "Spania"),
        "G4682": ("σπαράσσω", "G:V", "to convulse", "sparassō"),
        "G4683": ("σπαργανόω", "G:V", "to wrap", "sparganoō"),
        "G4684": ("σπαταλάω", "G:V", "to indulge", "spatalaō"),
        "G4685": ("σπάω", "G:V", "to draw", "spaō"),
        "G4686": ("σπεῖρα", "G:N-F", "band", "speira"),
        "G4687": ("σπείρω", "G:V", "to sow", "speirō"),
        "G4688": ("σπεκουλάτωρ", "G:N-M", "executioner", "spekoulatōr"),
        "G4689": ("σπένδω", "G:V", "to pour a libation", "spendō"),
        "G4690": ("σπέρμα", "G:N-N", "seed", "sperma"),
        "G4691": ("σπερμολόγος", "G:A", "babbler", "spermologos"),
        "G4692": ("σπεύδω", "G:V", "to hasten", "speudō"),
        "G4693": ("σπήλαιον", "G:N-N", "cave", "spēlaion"),
        "G4694": ("σπιλάς", "G:N-F", "reef", "spilas"),
        "G4695": ("σπιλόω", "G:V", "to stain", "spiloō"),
        "G4696": ("σπίλος", "G:N-M", "stain", "spilos"),
        "G4697": ("σπλαγχνίζω", "G:V", "to pity", "splagchnizō"),
        "G4698": ("σπλάγχνον", "G:N-N", "affection/entrails", "splagchnon"),
        "G4699": ("σπόγγος", "G:N-M", "sponge", "spoggos"),
        "G4700": ("σποδός", "G:N-F", "ashes", "spodos"),
        "G4701": ("σπορά", "G:N-F", "seed", "spora"),
        "G4702": ("σπόριμος", "G:A", "grainfield", "sporimos"),
        "G4703": ("σπόρος", "G:N-M", "seed", "sporos"),
        "G4704": ("σπουδάζω", "G:V", "be eager", "spoudazō"),
        "G4705": ("σπουδαῖος", "G:A", "eager", "spoudaios"),
        "G4706": ("σπουδαιότερον", "G:Adv", "more diligently", "spoudaioteron"),
        "G4707": ("σπουδαιότερος", "G:A", "more diligent", "spoudaioteros"),
        "G4708": ("σπουδαιοτέρως", "G:Adv", "more eagerly", "spoudaioterōs"),
        "G4709": ("σπουδαίως", "G:Adv", "diligently", "spoudaiōs"),
        "G4710": ("σπουδή", "G:N-F", "diligence", "spoudē"),
        "G4711": ("σπυρίς", "G:N-F", "basket", "spuris"),
        "G4712": ("στάδιος", "G:N-M/N", "stadium", "stadios"),
        "G4713": ("στάμνος", "G:N-F", "jar", "stamnos"),
        "G4714": ("στάσις", "G:N-F", "uprising", "stasis"),
        "G4715": ("στατήρ", "G:N-M", "coin", "statēr"),
        "G4716": ("σταυρός", "G:N-M", "cross", "stauros"),
        "G4717": ("σταυρόω", "G:V", "to crucify", "stauroō"),
        "G4718": ("σταφυλή", "G:N-F", "grapes", "staphulē"),
        "G4719": ("στάχυς", "G:N-M", "head of grain", "stachus"),
        "G4720": ("Στάχυς", "N:N-M-P", "Stachys", "Stachus"),
        "G4721": ("στέγη", "G:N-F", "roof", "stegē"),
        "G4722": ("στέγω", "G:V", "to endure", "stegō"),
        "G4723": ("στεῖρα", "G:N-F", "infertility", "steira"),
        "G4724": ("στέλλω", "G:V", "to avoid", "stellō"),
        "G4725": ("στέμμα", "G:N-N", "wreath", "stemma"),
        "G4726": ("στεναγμός", "G:N-M", "groan", "stenagmos"),
        "G4727": ("στενάζω", "G:V", "to groan", "stenazō"),
        "G4728": ("στενός", "G:A", "narrow", "stenos"),
        "G4729": ("στενοχωρέω", "G:V", "to press upon", "stenochōreō"),
        "G4730": ("στενοχωρία", "G:N-F", "hardship", "stenochōria"),
        "G4731": ("στερεός", "G:A", "strong", "stereos"),
        "G4732": ("στερεόω", "G:V", "to strengthen", "stereoō"),
        "G4733": ("στερέωμα", "G:N-N", "firmness", "stereōma"),
        "G4734": ("Στεφανᾶς", "N:N-M-P", "Stephanas", "Stephanas"),
        "G4735": ("στέφανος", "G:N-M", "crown", "stephanos"),
        "G4736": ("Στέφανος", "N:N-M-P", "Stephen", "Stephanos"),
        "G4737": ("στεφανόω", "G:V", "to crown", "stephanoō"),
        "G4738": ("στῆθος", "G:N-N", "chest", "stēthos"),
        "G4739": ("στήκω", "G:V", "to stand", "stēkō"),
        "G4740": ("στηριγμός", "G:N-M", "security", "stērigmos"),
        "G4741": ("στηρίζω", "G:V", "to establish", "stērizō"),
        "G4742": ("στίγμα", "G:N-N", "mark", "stigma"),
        "G4743": ("στιγμή", "G:N-F", "instant", "stigmē"),
        "G4744": ("στίλβω", "G:V", "to dazzle", "stilbō"),
        "G4745": ("στοά", "G:N-F", "portico", "stoa"),
        "G4746": ("στοιβάς, στιβάς", "G:N-F", "leafy branch", "stoibas, stibas"),
        "G4747": ("στοιχεῖον", "G:N-N", "principle", "stoicheion"),
        "G4748": ("στοιχέω", "G:V", "to follow", "stoicheō"),
        "G4749": ("στολή", "G:N-F", "robe", "stolē"),
        "G4750": ("στόμα", "G:N-N", "mouth", "stoma"),
        "G4751": ("στόμαχος", "G:N-M", "stomach", "stomachos"),
        "G4752": ("στρατεία", "G:N-F", "warfare", "strateia"),
        "G4753": ("στράτευμα", "G:N-N", "troops", "strateuma"),
        "G4754": ("στρατεύω", "G:V", "to battle", "strateuō"),
        "G4755": ("στρατηγός", "G:N-M", "officer/magistrate", "stratēgos"),
        "G4756": ("στρατιά", "G:N-F", "army", "stratia"),
        "G4757": ("στρατιώτης", "G:N-M", "soldier", "stratiōtēs"),
        "G4758": ("στρατολογέω", "G:V", "to enlist", "stratologeō"),
        "G4759": ("στρατοπεδάρχης", "G:N-M", "commander", "stratopedarchēs"),
        "G4760": ("στρατόπεδον", "G:N-N", "army camp", "stratopedon"),
        "G4761": ("στρεβλόω", "G:V", "to distort", "strebloō"),
        "G4762": ("στρέφω", "G:V", "to turn", "strephō"),
        "G4763": ("στρηνιάω", "G:V", "to revel", "strēniaō"),
        "G4764": ("στρῆνος", "G:N-N", "luxury", "strēnos"),
        "G4765": ("στρουθίον", "G:N-N", "sparrow", "strouthion"),
        "G4766": ("στρώννυμι", "G:V", "to spread", "strōnnumi"),
        "G4767": ("στυγητός", "G:A", "hated", "stugētos"),
        "G4768": ("στυγνάζω", "G:V", "be gloomy", "stugnazō"),
        "G4769": ("στῦλος", "G:N-M", "pillar", "stulos"),
        "G4770": ("Στωϊκός", "G:A", "Stoic", "Stōikos"),
        "G4771": ("σύ", "G:PerP", "you", "su"),
        "G4772": ("συγγένεια", "G:N-F", "kindred", "suggeneia"),
        "G4773": ("συγγενής", "G:A", "kindred", "suggenēs"),
        "G4774": ("συγγνώμη", "G:N-F", "concession", "suggnōmē"),
        "G4775": ("συγκάθημαι", "G:V", "to sit with", "sugkathēmai"),
        "G4776": ("συγκαθίζω", "G:V", "to sit down with", "sugkathizō"),
        "G4777": ("συγκακοπαθέω", "G:V", "to suffer with", "sugkakopatheō"),
        "G4778": ("συγκακουχέομαι", "G:V", "to suffer with", "sugkakoucheomai"),
        "G4779": ("συγκαλέω", "G:V", "to call together", "sugkaleō"),
        "G4780": ("συγκαλύπτω", "G:V", "to conceal", "sugkaluptō"),
        "G4781": ("συγκάμπτω", "G:V", "to bend", "sugkamptō"),
        "G4782": ("συγκαταβαίνω", "G:V", "to descend with", "sugkatabainō"),
        "G4783": ("συγκατάθεσις", "G:N-F", "agreement", "sugkatathesis"),
        "G4784": ("συγκατατίθημι", "G:V", "to consent", "sugkatatithēmi"),
        "G4785": ("συγκαταψηφίζομαι", "G:V", "to numbered with", "sugkatapsēphizomai"),
        "G4786": ("συγκεράννυμι", "G:V", "to unite", "sugkerannumi"),
        "G4787": ("συγκινέω", "G:V", "to stir up", "sugkineō"),
        "G4788": ("συγκλείω", "G:V", "to confine", "sugkleiō"),
        "G4789": ("συγκληρονόμος", "G:A", "co-heir", "sugklēronomos"),
        "G4790": ("συγκοινωνέω", "G:V", "to share with", "sugkoinōneō"),
        "G4791": ("συγκοινωνός", "G:N-M", "sharer", "sugkoinōnos"),
        "G4792": ("συγκομίζω", "G:V", "to entomb", "sugkomizō"),
        "G4793": ("συγκρίνω", "G:V", "to compare", "sugkrinō"),
        "G4794": ("συγκύπτω", "G:V", "to bend", "sugkuptō"),
        "G4795": ("συγκυρία", "G:N-F", "coincidence", "sugkuria"),
        "G4796": ("συγχαίρω", "G:V", "to rejoice with", "sugchairō"),
        "G4797": ("συγχέω", "G:V", "to confound", "sugcheō"),
        "G4798": ("συγχράομαι", "G:V", "to associate with", "sugchraomai"),
        "G4799": ("σύγχυσις", "G:N-F", "confusion", "sugchusis"),
        "G4800": ("συζάω", "G:V", "to live together", "suzaō"),
        "G4801": ("συζεύγνυμι", "G:V", "to join", "suzeugnumi"),
        "G4802": ("συζητέω", "G:V", "to debate", "suzēteō"),
        "G4803": ("συζήτησις", "G:N-F", "discussion", "suzētēsis"),
        "G4804": ("συζητητής", "G:N-M", "debater", "suzētētēs"),
        "G4805": ("σύζυγος", "G:A", "yokefellow", "suzugos"),
        "G4806": ("συζωοποιέω", "G:V", "to make alive with", "suzōopoieō"),
        "G4807": ("συκάμινος", "G:N-F", "mulberry tree", "sukaminos"),
        "G4808": ("συκῆ", "G:N-F", "fig tree", "sukē"),
        "G4809": ("συκομωραία", "G:N-F", "sycamore tree", "sukomōraia"),
        "G4810": ("σῦκον", "G:N-N", "fig", "sukon"),
        "G4811": ("συκοφαντέω", "G:V", "to extort", "sukophanteō"),
        "G4812": ("συλαγωγέω", "G:V", "to capture", "sulagōgeō"),
        "G4813": ("συλάω", "G:V", "to rob", "sulaō"),
        "G4814": ("συλλαλέω", "G:V", "to talk with", "sullaleō"),
        "G4815": ("συλλαμβάνω", "G:V", "to seize/conceive/help", "sullambanō"),
        "G4816": ("συλλέγω", "G:V", "to collect", "sullegō"),
        "G4817": ("συλλογίζομαι", "G:V", "to discuss", "sullogizomai"),
        "G4818": ("συλλυπέω", "G:V", "be grieved", "sullupeō"),
        "G4819": ("συμβαίνω", "G:V", "to happen", "sumbainō"),
        "G4820": ("συμβάλλω", "G:V", "to ponder/confer", "sumballō"),
        "G4821": ("συμβασιλεύω", "G:V", "to reign with", "sumbasileuō"),
        "G4822": ("συμβιβάζω", "G:V", "to join with", "sumbibazō"),
        "G4823": ("συμβουλεύω", "G:V", "to consult", "sumbouleuō"),
        "G4824": ("συμβούλιον", "G:N-N", "counsel/council", "sumboulion"),
        "G4825": ("σύμβουλος", "G:N-M", "counselor", "sumboulos"),
        "G4826": ("Συμεών", "N:N-M-P", "Simeon", "Sumeōn"),
        "G4827": ("συμμαθητής", "G:N-M", "fellow disciple", "summathētēs"),
        "G4828": ("συμμαρτυρέω", "G:V", "to testify with", "summartureō"),
        "G4829": ("συμμερίζω", "G:V", "to share", "summerizō"),
        "G4830": ("συμμέτοχος", "G:A", "sharer", "summetochos"),
        "G4831": ("συμμιμητής", "G:N-M", "co-imitator", "summimētēs"),
        "G4832": ("σύμμορφος", "G:A", "conformed", "summorphos"),
        "G4833": ("συμμορφόομαι", "G:V", "to make like", "summorphoomai"),
        "G4834": ("συμπαθέω", "G:V", "to sympathize", "sumpatheō"),
        "G4835": ("συμπαθής", "G:A", "sympathetic", "sumpathēs"),
        "G4836": ("συμπαραγίνομαι", "G:V", "to assemble", "sumparaginomai"),
        "G4837": ("συμπαρακαλέω", "G:V", "to encourage", "sumparakaleō"),
        "G4838": ("συμπαραλαμβάνω", "G:V", "to take along with", "sumparalambanō"),
        "G4839": ("συμπαραμένω", "G:V", "to continue", "sumparamenō"),
        "G4840": ("συμπάρειμι", "G:V", "be present with", "sumpareimi"),
        "G4841": ("συμπάσχω", "G:V", "to suffer with", "sumpaschō"),
        "G4842": ("συμπέμπω", "G:V", "to send with", "sumpempō"),
        "G4843": ("συμπεριλαμβάνω", "G:V", "to embrace", "sumperilambanō"),
        "G4844": ("συμπίνω", "G:V", "to drink with", "sumpinō"),
        "G4845": ("συμπληρόω", "G:V", "to (ful)fill", "sumplēroō"),
        "G4846": ("συμπνίγω", "G:V", "to choke", "sumpnigō"),
        "G4847": ("συμπολίτης", "G:N-M", "fellow citizen", "sumpolitēs"),
        "G4848": ("συμπορεύομαι", "G:V", "to go with", "sumporeuomai"),
        "G4849": ("συμπόσιον", "G:N-N", "common meal", "sumposion"),
        "G4850": ("συμπρεσβύτερος", "G:N", "fellow elder", "sumpresbuteros"),
        "G4851": ("συμφέρω", "G:A / G:V", "be profitable", "sumpherō"),
        "G4852": ("σύμφημι", "G:V", "to agree", "sumphēmi"),
        "G4853": ("συμφυλέτης", "G:N-M", "fellow countryman", "sumphuletēs"),
        "G4854": ("σύμφυτος", "G:A", "united", "sumphutos"),
        "G4855": ("συμφύω", "G:V", "to grow with", "sumphuō"),
        "G4856": ("συμφωνέω", "G:V", "to agree with", "sumphōneō"),
        "G4857": ("συμφώνησις", "G:N-F", "harmony", "sumphōnēsis"),
        "G4858": ("συμφωνία", "G:N-F", "music", "sumphōnia"),
        "G4859": ("σύμφωνος", "G:A", "mutual consent", "sumphōnos"),
        "G4860": ("συμψηφίζω", "G:V", "to calculate", "sumpsēphizō"),
        "G4861": ("σύμψυχος", "G:A", "harmonious", "sumpsuchos"),
        "G4862": ("σύν", "G:Prep", "with", "sun"),
        "G4863": ("συνάγω", "G:V", "to assemble", "sunagō"),
        "G4864": ("συναγωγή", "G:N-F", "synagogue", "sunagōgē"),
        "G4865": ("συναγωνίζομαι", "G:V", "to struggle", "sunagōnizomai"),
        "G4866": ("συναθλέω", "G:V", "to contend", "sunathleō"),
        "G4867": ("συναθροίζω", "G:V", "to assemble", "sunathroizō"),
        "G4868": ("συναίρω", "G:V", "to settle accounts", "sunairō"),
        "G4869": ("συναιχμάλωτος", "G:N-M", "fellow prisoner", "sunaichmalōtos"),
        "G4870": ("συνακολουθέω", "G:V", "to accompany", "sunakoloutheō"),
        "G4871": ("συναλίζω", "G:V", "to assemble", "sunalizō"),
        "G4872": ("συναναβαίνω", "G:V", "to ascend with", "sunanabainō"),
        "G4873": ("συνανάκειμαι", "G:V", "to dine with", "sunanakeimai"),
        "G4874": ("συναναμίγνυμι", "G:V", "to associate with", "sunanamignumi"),
        "G4875": ("συναναπαύομαι", "G:V", "to rest with", "sunanapauomai"),
        "G4876": ("συναντάω", "G:V", "to meet", "sunantaō"),
        "G4877": ("συνάντησις", "G:N-F", "meeting", "sunantēsis"),
        "G4878": ("συναντιλαμβάνομαι", "G:V", "to help", "sunantilambanomai"),
        "G4879": ("συναπάγω", "G:V", "to lead away with", "sunapagō"),
        "G4880": ("συναποθνήσκω", "G:V", "to die with", "sunapothnēskō"),
        "G4881": ("συναπόλλυμι", "G:V", "to die/destroy with", "sunapollumi"),
        "G4882": ("συναποστέλλω", "G:V", "to send with", "sunapostellō"),
        "G4883": ("συναρμολογέω", "G:V", "to join", "sunarmologeō"),
        "G4884": ("συναρπάζω", "G:V", "to seize", "sunarpazō"),
        "G4885": ("συναυξάνω", "G:V", "to grow together", "sunauxanō"),
        "G4886": ("σύνδεσμος", "G:N-M", "bond(age)", "sundesmos"),
        "G4887": ("συνδέω", "G:V", "to bind with", "sundeō"),
        "G4888": ("συνδοξάζω", "G:V", "to glory with", "sundoxazō"),
        "G4889": ("σύνδουλος", "G:N-M", "fellow slave", "sundoulos"),
        "G4890": ("συνδρομή", "G:N-F", "swarming", "sundromē"),
        "G4891": ("συνεγείρω", "G:V", "to raise up with", "sunegeirō"),
        "G4892": ("συνέδριον", "G:N-N", "council", "sunedrion"),
        "G4893": ("συνείδησις", "G:N-F", "conscience", "suneidēsis"),
        "G4894": ("συνείδω, σύνοιδα", "G:V", "be aware", "suneidō, sunoida"),
        "G4895": ("σύνειμι", "G:V", "be with", "suneimi"),
        "G4896": ("σύνειμι", "G:V", "to gather", "suneimi"),
        "G4897": ("συνεισέρχομαι", "G:V", "to enter with", "suneiserchomai"),
        "G4898": ("συνέκδημος", "G:N-M", "companion", "sunekdēmos"),
        "G4899": ("συνεκλεκτός", "G:A", "chosen with", "suneklektos"),
        "G4900": ("συνελαύνω", "G:V", "to urge", "sunelaunō"),
        "G4901": ("συνεπιμαρτυρέω", "G:V", "to testify with", "sunepimartureō"),
        "G4902": ("συνέπομαι", "G:V", "to accompany", "sunepomai"),
        "G4903": ("συνεργέω", "G:V", "to work with", "sunergeō"),
        "G4904": ("συνεργός", "G:A", "co-worker", "sunergos"),
        "G4905": ("συνέρχομαι", "G:V", "to assemble", "sunerchomai"),
        "G4906": ("συνεσθίω", "G:V", "to eat with", "sunesthiō"),
        "G4907": ("σύνεσις", "G:N-F", "understanding", "sunesis"),
        "G4908": ("συνετός", "G:A", "intelligent", "sunetos"),
        "G4909": ("συνευδοκέω", "G:V", "to agree to", "suneudokeō"),
        "G4910": ("συνευωχέομαι", "G:V", "to feast with", "suneuōcheomai"),
        "G4911": ("συνεφίστημι", "G:V", "to attack", "sunephistēmi"),
        "G4912": ("συνέχω", "G:V", "to hold/oppress", "sunechō"),
        "G4913": ("συνήδομαι", "G:V", "to delight", "sunēdomai"),
        "G4914": ("συνήθεια", "G:N-F", "custom", "sunētheia"),
        "G4915": ("συνηλικιώτης", "G:N-M", "contemporary", "sunēlikiōtēs"),
        "G4916": ("συνθάπτω", "G:V", "be buried with", "sunthaptō"),
        "G4917": ("συνθλάω", "G:V", "to shatter", "sunthlaō"),
        "G4918": ("συνθλίβω", "G:V", "to push against", "sunthlibō"),
        "G4919": ("συνθρύπτω", "G:V", "to break", "sunthruptō"),
        "G4920": ("συνίημι", "G:V", "to understand", "suniēmi"),
        "G4921": ("συνίστημι, συνιστάω", "G:V", "to commend", "sunistēmi, sunistaō"),
        "G4922": ("συνοδεύω", "G:V", "to travel with", "sunodeuō"),
        "G4923": ("συνοδία", "G:N-F", "caravan", "sunodia"),
        "G4924": ("συνοικέω", "G:V", "to live with", "sunoikeō"),
        "G4925": ("συνοικοδομέω", "G:V", "to built up with", "sunoikodomeō"),
        "G4926": ("συνομιλέω", "G:V", "to converse", "sunomileō"),
        "G4927": ("συνομορέω", "G:V", "be next to", "sunomoreō"),
        "G4928": ("συνοχή", "G:N-F", "anguish", "sunochē"),
        "G4929": ("συντάσσω", "G:V", "to direct", "suntassō"),
        "G4930": ("συντέλεια", "G:N-F", "consummation", "sunteleia"),
        "G4931": ("συντελέω", "G:V", "to complete", "sunteleō"),
        "G4932": ("συντέμνω", "G:V", "to cut short", "suntemnō"),
        "G4933": ("συντηρέω", "G:V", "to preserve", "suntēreō"),
        "G4934": ("συντίθημι", "G:V", "to agree", "suntithēmi"),
        "G4935": ("συντόμως", "G:Adv", "concisely", "suntomōs"),
        "G4936": ("συντρέχω", "G:V", "to flock", "suntrechō"),
        "G4937": ("συντρίβω", "G:V", "to break", "suntribō"),
        "G4938": ("σύντριμμα", "G:N-N", "ruin", "suntrimma"),
        "G4939": ("σύντροφος", "G:A", "brought up with", "suntrophos"),
        "G4940": ("συντυγχάνω", "G:V", "to meet with", "suntugchanō"),
        "G4941": ("Συντύχη", "N:N-F-P", "Syntyche", "Suntuchē"),
        "G4942": ("συνυποκρίνομαι", "G:V", "to join hypocrisy", "sunupokrinomai"),
        "G4943": ("συνυπουργέω", "G:V", "to join in helping", "sunupourgeō"),
        "G4944": ("συνωδίνω", "G:V", "to labor together", "sunōdinō"),
        "G4945": ("συνωμοσία", "G:N-F", "plot", "sunōmosia"),
        "G4946": ("Συράκουσαι", "N:N-F-L", "Syracuse", "Surakousai"),
        "G4947": ("Συρία", "N:N-F-L", "Syria", "Suria"),
        "G4948": ("Σύρος", "N:N--LG", "Syrian", "Suros"),
        "G4949": ("Συροφοίνισσα", "N:N-F-LG", "Syrophoenician", "Surophoinissa"),
        "G4950": ("σύρτις", "N:N-F-L", "Syrtis", "surtis"),
        "G4951": ("σύρω", "G:V", "to drag", "surō"),
        "G4952": ("συσπαράσσω", "G:V", "to convulse", "susparassō"),
        "G4953": ("σύσσημον", "G:N-N", "an agreed signal", "sussēmon"),
        "G4954": ("σύσσωμος", "G:A", "of the same body", "sussōmos"),
        "G4955": ("συστασιαστής", "G:N-M", "insurrectionist", "sustasiastēs"),
        "G4956": ("συστατικός", "G:A", "recommended", "sustatikos"),
        "G4957": ("συσταυρόομαι", "G:V", "to crucify with", "sustauroomai"),
        "G4958": ("συστέλλω", "G:V", "to wrap up", "sustellō"),
        "G4959": ("συστενάζω", "G:V", "to groan with", "sustenazō"),
        "G4960": ("συστοιχέω", "G:V", "to correspond to", "sustoicheō"),
        "G4961": ("συστρατιώτης", "G:N-M", "fellow soldier", "sustratiōtēs"),
        "G4962": ("συστρέφω", "G:V", "to gather", "sustrephō"),
        "G4963": ("συστροφή", "G:N-F", "commotion/plot", "sustrophē"),
        "G4964": ("συσχηματίζω", "G:V", "to conform to", "suschēmatizō"),
        "G4965": ("Συχάρ", "N:N--L", "Sychar", "Suchar"),
        "G4966": ("Συχέμ", "N:N--L", "Shechem", "Suchem"),
        "G4967": ("σφαγή", "G:N-F", "slaughter", "sphagē"),
        "G4968": ("σφάγιον", "G:N-N", "sacrificial victim", "sphagion"),
        "G4969": ("σφάζω", "G:V", "to slaughter", "sphazō"),
        "G4970": ("σφόδρα", "G:Adv", "very", "sphodra"),
        "G4971": ("σφοδρῶς", "G:Adv", "violently", "sphodrōs"),
        "G4972": ("σφραγίζω", "G:V", "to seal", "sphragizō"),
        "G4973": ("σφραγίς", "G:N-F", "seal", "sphragis"),
        "G4974": ("σφυρόν", "G:N-N", "ankle", "sphuron"),
        "G4975": ("σχεδόν", "G:Adv", "nearly", "schedon"),
        "G4976": ("σχῆμα", "G:N-N", "form", "schēma"),
        "G4977": ("σχίζω", "G:V", "to split", "schizō"),
        "G4978": ("σχίσμα", "G:N-N", "split", "schisma"),
        "G4979": ("σχοινίον", "G:N-N", "rope", "schoinion"),
        "G4980": ("σχολάζω", "G:V", "be devoted/empty", "scholazō"),
        "G4981": ("σχολή", "G:N-F", "lecture hall", "scholē"),
        "G4982": ("σῴζω", "G:V", "to save", "sōzō"),
        "G4983": ("σῶμα", "G:N-N", "body", "sōma"),
        "G4984": ("σωματικός", "G:A", "bodily", "sōmatikos"),
        "G4985": ("σωματικῶς", "G:Adv", "bodily", "sōmatikōs"),
        "G4986": ("Σώπατρος", "N:N-M-P", "Sopater", "Sōpatros"),
        "G4987": ("σωρεύω", "G:V", "to pile up", "sōreuō"),
        "G4988": ("Σωσθένης", "N:N-M-P", "Sosthenes", "Sōsthenēs"),
        "G4989": ("Σωσίπατρος", "N:N-M-P", "Sosipater", "Sōsipatros"),
        "G4990": ("σωτήρ", "G:N-M", "savior", "sōtēr"),
        "G4991": ("σωτηρία", "G:N-F", "salvation", "sōtēria"),
        "G4992": ("σωτήριον", "G:A", "saving", "sōtērion"),
        "G4993": ("σωφρονέω", "G:V", "be of sound mind", "sōphroneō"),
        "G4994": ("σωφρονίζω", "G:V", "to train", "sōphronizō"),
        "G4995": ("σωφρονισμός", "G:N-M", "self-discipline", "sōphronismos"),
        "G4996": ("σωφρόνως", "G:Adv", "in self-control", "sōphronōs"),
        "G4997": ("σωφροσύνη", "G:N-F", "mental soundness", "sōphrosunē"),
        "G4998": ("σώφρων", "G:A", "self-controlled", "sōphrōn"),
        "G4999": ("Ταβέρναι", "N:N-F-L", "tavern", "Tabernai"),
        "G5000": ("Ταβιθά", "N:N-F-P", "Tabitha", "Tabitha"),
        "G5001": ("τάγμα", "G:N-N", "order", "tagma"),
        "G5002": ("τακτός", "G:A", "appointed", "taktos"),
        "G5003": ("ταλαιπωρέω", "G:V", "to grieve", "talaipōreō"),
        "G5004": ("ταλαιπωρία", "G:N-F", "misery", "talaipōria"),
        "G5005": ("ταλαίπωρος", "G:A", "wretched", "talaipōros"),
        "G5006": ("ταλαντιαῖος", "G:A", "weighing a talent", "talantiaios"),
        "G5007": ("τάλαντον", "G:N-N", "talent", "talanton"),
        "G5008": ("ταλιθά", "A:N", "girl", "talitha"),
        "G5009": ("ταμεῖον", "G:N-N", "inner room", "tameion"),
        "G5010": ("τάξις", "G:N-F", "order", "taxis"),
        "G5011": ("ταπεινός", "G:A", "lowly", "tapeinos"),
        "G5012": ("ταπεινοφροσύνη", "G:N-F / G:A", "humility", "tapeinophrosunē"),
        "G5013": ("ταπεινόω", "G:V", "to humble", "tapeinoō"),
        "G5014": ("ταπείνωσις", "G:N-F", "lowliness", "tapeinōsis"),
        "G5015": ("ταράσσω", "G:V", "to trouble", "tarassō"),
        "G5016": ("ταραχή", "G:N-F", "disturbance", "tarachē"),
        "G5017": ("τάραχος", "G:N-M", "disturbance", "tarachos"),
        "G5018": ("Ταρσεύς", "N:N-M-L", "of Tarsus", "Tarseus"),
        "G5019": ("Ταρσός", "N:N-F-L", "Tarsus", "Tarsos"),
        "G5020": ("ταρταρόω", "G:V", "to condemn", "tartaroō"),
        "G5021": ("τάσσω", "G:V", "to appoint", "tassō"),
        "G5022": ("ταῦρος", "G:N-M", "bull", "tauros"),
        "G5023": ("ταῦτα", "G:DemP", "this", "tauta"),
        "G5024": ("ταὐτά", "G:Adv", "thus", "tauta"),
        "G5025": ("ταύταις", "G:Part", "hence", "tautais"),
        "G5026": ("ταύτῃ", "G:RelP", "her", "tautē"),
        "G5027": ("ταφή", "G:N-F", "burial (place)", "taphē"),
        "G5028": ("τάφος", "G:N-M", "grave", "taphos"),
        "G5029": ("τάχα", "G:Adv", "perhaps", "tacha"),
        "G5030": ("ταχέως", "G:Adv", "quickly", "tacheōs"),
        "G5031": ("ταχινός", "G:A", "quick", "tachinos"),
        "G5032": ("τάχιον", "G:Adv", "more quickly", "tachion"),
        "G5033": ("τάχιστα", "G:Adv", "most quickly", "tachista"),
        "G5034": ("τάχος", "G:N-N", "quickness", "tachos"),
        "G5035": ("ταχύ", "G:Adv", "quickly", "tachu"),
        "G5036": ("ταχύς", "G:A", "quick", "tachus"),
        "G5037": ("τε", "G:Conj", "and/both", "te"),
        "G5038": ("τεῖχος", "G:N-N", "wall", "teichos"),
        "G5039": ("τεκμήριον", "G:N-N", "clear proof", "tekmērion"),
        "G5040": ("τεκνίον", "G:N-N", "children", "teknion"),
        "G5041": ("τεκνογονέω", "G:V", "to have children", "teknogoneō"),
        "G5042": ("τεκνογονία", "G:N-F", "childbearing", "teknogonia"),
        "G5043": ("τέκνον", "G:N-N", "child", "teknon"),
        "G5044": ("τεκνοτροφέω", "G:V", "to raise children", "teknotropheō"),
        "G5045": ("τέκτων", "G:N-M", "craftsman", "tektōn"),
        "G5046": ("τέλειος", "G:A", "perfect", "teleios"),
        "G5047": ("τελειότης", "G:N-F", "perfection", "teleiotēs"),
        "G5048": ("τελειόω", "G:V", "to perfect", "teleioō"),
        "G5049": ("τελείως", "G:Adv", "completely", "teleiōs"),
        "G5050": ("τελείωσις", "G:N-F", "perfection", "teleiōsis"),
        "G5051": ("τελειωτής", "G:N-M", "finisher", "teleiōtēs"),
        "G5052": ("τελεσφορέω", "G:V", "to mature", "telesphoreō"),
        "G5053": ("τελευτάω", "G:V", "to decease", "teleutaō"),
        "G5054": ("τελευτή", "G:N-F", "death", "teleutē"),
        "G5055": ("τελέω", "G:V", "to finish", "teleō"),
        "G5056": ("τέλος", "G:N-N", "goal/tax", "telos"),
        "G5057": ("τελώνης", "G:N-M", "tax collector", "telōnēs"),
        "G5058": ("τελώνιον", "G:N-N", "tax booth", "telōnion"),
        "G5059": ("τέρας", "G:N-N", "wonders", "teras"),
        "G5060": ("Τέρτιος", "N:N-M-P", "Tertius", "Tertios"),
        "G5061": ("Τέρτυλλος", "N:N-M-P", "Tertullus", "Tertullos"),
        "G5062": ("τεσσαράκοντα", "G:A", "forty", "tessarakonta"),
        "G5063": ("τεσσαρακονταετής", "G:A", "forty years", "tessarakontaetēs"),
        "G5064": ("τέσσαρες", "G:A", "four", "tessares"),
        "G5065": ("τεσσαρεσκαιδέκατος", "G:A", "fourteenth", "tessareskaidekatos"),
        "G5066": ("τεταρταῖος", "G:A", "fourth", "tetartaios"),
        "G5067": ("τέταρτος", "G:A", "fourth", "tetartos"),
        "G5068": ("τετράγωνος", "G:A", "square", "tetragōnos"),
        "G5069": ("τετράδιον", "G:N-N", "squad of four", "tetradion"),
        "G5070": ("τετρακισχίλιοι", "G:A", "four thousand", "tetrakischilioi"),
        "G5071": ("τετρακόσιοι", "G:A", "four hundred", "tetrakosioi"),
        "G5072": ("τετράμηνος", "G:A", "four months", "tetramēnos"),
        "G5073": ("τετραπλόος", "G:A", "fourfold", "tetraploos"),
        "G5074": ("τετράπους", "G:A", "four-footed", "tetrapous"),
        "G5075": ("τετραρχέω", "G:V", "be a tetrarch", "tetrarcheō"),
        "G5076": ("τετράρχης", "G:N-M", "tetrarch", "tetrarchēs"),
        "G5077": ("τεφρόω", "G:V", "to turn to ashes", "tephroō"),
        "G5078": ("τέχνη", "G:N-F", "skill", "technē"),
        "G5079": ("τεχνίτης", "G:N-M", "craftsman", "technitēs"),
        "G5080": ("τήκω", "G:V", "to melt", "tēkō"),
        "G5081": ("τηλαυγῶς", "G:Adv", "clearly", "tēlaugōs"),
        "G5082": ("τηλικοῦτος", "G:DemP", "so great", "tēlikoutos"),
        "G5083": ("τηρέω", "G:V", "to keep", "tēreō"),
        "G5084": ("τήρησις", "G:N-F", "jail/keeping", "tērēsis"),
        "G5085": ("Τιβεριάς", "N:N-F-L", "Tiberias", "Tiberias"),
        "G5086": ("Τιβέριος", "N:N-M-P", "Tiberius", "Tiberios"),
        "G5087": ("τίθημι", "G:V", "to place", "tithēmi"),
        "G5088": ("τίκτω", "G:V", "to give birth to", "tiktō"),
        "G5089": ("τίλλω", "G:V", "to pluck", "tillō"),
        "G5090": ("Τιμαῖος", "N:N-M-P", "Timaeus", "Timaios"),
        "G5091": ("τιμάω", "G:V", "to honor", "timaō"),
        "G5092": ("τιμή", "G:N-F", "honor", "timē"),
        "G5093": ("τίμιος", "G:A", "precious", "timios"),
        "G5094": ("τιμιότης", "G:N-F", "wealth", "timiotēs"),
        "G5095": ("Τιμόθεος", "N:N-M-P", "Timothy", "Timotheos"),
        "G5096": ("Τίμων", "N:N-M-P", "Timon", "Timōn"),
        "G5097": ("τιμωρέω", "G:V", "to punish", "timōreō"),
        "G5098": ("τιμωρία", "G:N-F", "punishment", "timōria"),
        "G5099": ("τίνω", "G:V", "to pay", "tinō"),
        "G5100": ("τις", "G:PerP", "one", "tis"),
        "G5101": ("τίς", "G:Intg", "which?", "tis"),
        "G5102": ("τίτλος", "G:N-M", "title", "titlos"),
        "G5103": ("Τίτος", "N:N-M-P", "Titus", "Titos"),
        "G5104": ("τοί", "G:Part", "certainly", "toi"),
        "G5105": ("τοιγαροῦν", "G:Conj", "therefore", "toigaroun"),
        "G5106": ("τοίνυν", "G:Conj", "then", "toinun"),
        "G5107": ("τοιόσδε", "G:DemP", "such as this", "toiosde"),
        "G5108": ("τοιοῦτος", "G:DemP", "such as this", "toioutos"),
        "G5109": ("τοῖχος", "G:N-M", "wall", "toichos"),
        "G5110": ("τόκος", "G:N-M", "interest", "tokos"),
        "G5111": ("τολμάω", "G:V", "be bold", "tolmaō"),
        "G5112": ("τολμηρότερον", "G:Adv", "more boldly", "tolmēroteron"),
        "G5113": ("τολμητής", "G:N-M", "bold man", "tolmētēs"),
        "G5114": ("τομώτερος", "G:A", "sharper", "tomōteros"),
        "G5115": ("τόξον", "G:N-N", "bow", "toxon"),
        "G5116": ("τοπάζιον", "G:N-N", "topaz", "topazion"),
        "G5117": ("τόπος", "G:N-M", "place", "topos"),
        "G5118": ("τοσοῦτος", "G:Cor", "so great", "tosoutos"),
        "G5119": ("τότε", "G:Adv", "then", "tote"),
        "G5120": ("τοῦ", "G:RelP", "his", "tou"),
        "G5121": ("τοὐναντίον", "G:Adv", "instead", "tounantion"),
        "G5122": ("τοὔνομα", "G:N-N", "by name", "tounoma"),
        "G5123": ("τουτέστι", "G:Part", "that is", "toutesti"),
        "G5124": ("τοῦτο", "G:DemP", "that", "touto"),
        "G5125": ("τούτοις", "G:DemP", "to these", "toutois"),
        "G5126": ("τοῦτον", "G:DemP", "this", "touton"),
        "G5127": ("τούτου", "G:DemP", "of this", "toutou"),
        "G5128": ("τούτους", "G:DemP", "these", "toutous"),
        "G5129": ("τούτῳ", "G:DemP", "to this", "toutō"),
        "G5130": ("τούτων", "G:DemP", "of these", "toutōn"),
        "G5131": ("τράγος", "G:N-M", "he-goat", "tragos"),
        "G5132": ("τράπεζα", "G:N-F", "table", "trapeza"),
        "G5133": ("τραπεζίτης", "G:N-M", "money lender", "trapezitēs"),
        "G5134": ("τραῦμα", "G:N-N", "wound", "trauma"),
        "G5135": ("τραυματίζω", "G:V", "to wound", "traumatizō"),
        "G5136": ("τραχηλίζω", "G:V", "to lay bare", "trachēlizō"),
        "G5137": ("τράχηλος", "G:N-M", "neck", "trachēlos"),
        "G5138": ("τραχύς", "G:A", "rough", "trachus"),
        "G5139": ("Τραχωνῖτις", "N:N-F-L", "Traconitis", "Trachōnitis"),
        "G5140": ("τρεῖς, τρία", "G:A", "three", "treis, tria"),
        "G5141": ("τρέμω", "G:V", "to tremble", "tremō"),
        "G5142": ("τρέφω", "G:V", "to feed", "trephō"),
        "G5143": ("τρέχω", "G:V", "to run", "trechō"),
        "G5144": ("τριάκοντα", "G:A", "thirty", "triakonta"),
        "G5145": ("τριακόσιοι", "G:A", "three hundred", "triakosioi"),
        "G5146": ("τρίβολος", "G:N-M", "thistle", "tribolos"),
        "G5147": ("τρίβος", "G:N-F", "path", "tribos"),
        "G5148": ("τριετία", "G:N-F", "three years", "trietia"),
        "G5149": ("τρίζω", "G:V", "to gnash", "trizō"),
        "G5150": ("τρίμηνος", "G:A", "three months", "trimēnos"),
        "G5151": ("τρίς", "G:Adv", "three times", "tris"),
        "G5152": ("τρίστεγος", "G:N-N", "third story", "tristegos"),
        "G5153": ("τρισχίλιοι", "G:A", "three thousand", "trischilioi"),
        "G5154": ("τρίτος", "G:A", "third", "tritos"),
        "G5155": ("τρίχινος", "G:A", "of hair", "trichinos"),
        "G5156": ("τρόμος", "G:N-M", "trembling", "tromos"),
        "G5157": ("τροπή", "G:N-F", "turning", "tropē"),
        "G5158": ("τρόπος", "G:N-M", "way", "tropos"),
        "G5159": ("τροποφορέω", "G:V", "to put up with", "tropophoreō"),
        "G5160": ("τροφή", "G:N-F", "food", "trophē"),
        "G5161": ("Τρόφιμος", "N:N-M-P", "Trophimus", "Trophimos"),
        "G5162": ("τροφός", "G:N-F", "nursing mother", "trophos"),
        "G5163": ("τροχιά", "G:N-F", "track", "trochia"),
        "G5164": ("τροχός", "G:N-M", "course/wheel", "trochos"),
        "G5165": ("τρύβλιον", "G:N-N", "bowl", "trublion"),
        "G5166": ("τρυγάω", "G:V", "to harvest", "trugaō"),
        "G5167": ("τρυγών", "G:N-F", "dove", "trugōn"),
        "G5168": ("τρυμαλιά", "G:N-F", "hole", "trumalia"),
        "G5169": ("τρύπημα", "G:N-N", "hole", "trupēma"),
        "G5170": ("Τρύφαινα", "N:N-F-P", "Tryphena", "Truphaina"),
        "G5171": ("τρυφάω", "G:V", "to self-indulge", "truphaō"),
        "G5172": ("τρυφή", "G:N-F", "self-indulgence", "truphē"),
        "G5173": ("Τρυφῶσα", "N:N-F-P", "Tryphosa", "Truphōsa"),
        "G5174": ("Τρωάς", "N:N-F-L", "Troas", "Trōas"),
        "G5175": ("Τρωγύλλιον", "N:N--L", "Trogyllium", "Trōgullion"),
        "G5176": ("τρώγω", "G:V", "to eat", "trōgō"),
        "G5177": ("τυγχάνω", "G:V", "to obtain/happen", "tugchanō"),
        "G5178": ("τυμπανίζω", "G:V", "to torture", "tumpanizō"),
        "G5179": ("τύπος", "G:N-M", "mark/example", "tupos"),
        "G5180": ("τύπτω", "G:V", "to strike", "tuptō"),
        "G5181": ("Τύραννος", "N:N-M-P", "Tyrannus", "Turannos"),
        "G5182": ("τυρβάζω", "G:V", "to upset", "turbazō"),
        "G5183": ("Τύριος", "N:N--LG", "Tyrian", "Turios"),
        "G5184": ("Τύρος", "N:N-F-L", "Tyre", "Turos"),
        "G5185": ("τυφλός", "G:A", "blind", "tuphlos"),
        "G5186": ("τυφλόω", "G:V", "to blind", "tuphloō"),
        "G5187": ("τυφόω", "G:V", "be conceited", "tuphoō"),
        "G5188": ("τύφω", "G:V", "to smoulder", "tuphō"),
        "G5189": ("τυφωνικός", "G:A", "tempestuous", "tuphōnikos"),
        "G5190": ("Τυχικός", "N:N-M-P", "Tychicus", "Tuchikos"),
        "G5191": ("ὑακίνθινος", "G:A", "dark blue", "huakinthinos"),
        "G5192": ("ὑάκινθος", "G:N-M", "hyacinth", "huakinthos"),
        "G5193": ("ὑάλινος", "G:A", "glass", "hualinos"),
        "G5194": ("ὕαλος", "G:N-M", "glass", "hualos"),
        "G5195": ("ὑβρίζω", "G:V", "to mistreat", "hubrizō"),
        "G5196": ("ὕβρις", "G:N-F", "mistreatment", "hubris"),
        "G5197": ("ὑβριστής", "G:N-M", "insolent man", "hubristēs"),
        "G5198": ("ὑγιαίνω", "G:V", "be healthy", "hugiainō"),
        "G5199": ("ὑγιής", "G:A", "healthy", "hugiēs"),
        "G5200": ("ὑγρός", "G:A", "moist/green", "hugros"),
        "G5201": ("ὑδρία", "G:N-F", "jar", "hudria"),
        "G5202": ("ὑδροποτέω", "G:V", "to drink water", "hudropoteō"),
        "G5203": ("ὑδρωπικός", "G:A", "with dropsy", "hudrōpikos"),
        "G5204": ("ὕδωρ", "G:N-N", "water", "hudōr, hudatos"),
        "G5205": ("ὑετός", "G:N-M", "rain", "huetos"),
        "G5206": ("υἱοθεσία", "G:N-F", "adoption (as son)", "uhiothesia"),
        "G5207": ("υἱός", "G:N-M", "son", "uhios"),
        "G5208": ("ὕλη", "G:N-F", "forest", "hulē"),
        "G5209": ("ὑμᾶς", "G:", "you", "humas"),
        "G5210": ("ὑμεῖς", "G:RelP", "you", "humeis"),
        "G5211": ("Ὑμεναῖος", "N:N-M-P", "Hymenaeus", "Humenaios"),
        "G5212": ("ὑμέτερος", "G:PosP", "your", "humeteros"),
        "G5213": ("ὑμῖν", "G:RelP", "to you", "humin"),
        "G5214": ("ὑμνέω", "G:V", "to praise", "humneō"),
        "G5215": ("ὕμνος", "G:N-M", "hymn", "humnos"),
        "G5216": ("ὑμῶν", "G:RelP", "of you", "humōn"),
        "G5217": ("ὑπάγω", "G:V", "to go", "hupagō"),
        "G5218": ("ὑπακοή", "G:N-F", "obedience", "hupakoē"),
        "G5219": ("ὑπακούω", "G:V", "to obey", "hupakouō"),
        "G5220": ("ὕπανδρος", "G:A", "married", "hupandros"),
        "G5221": ("ὑπαντάω", "G:V", "to go meet", "hupantaō"),
        "G5222": ("ὑπάντησις", "G:N-F", "meeting", "hupantēsis"),
        "G5223": ("ὕπαρξις", "G:N-F", "property", "huparxis"),
        "G5224": ("ὑπάρχοντα", "G:N-N", "possession", "huparchonta"),
        "G5225": ("ὑπάρχω", "G:V", "to be", "huparchō"),
        "G5226": ("ὑπείκω", "G:V", "to submit", "hupeikō"),
        "G5227": ("ὑπεναντίος", "G:A", "opposed", "hupenantios"),
        "G5228": ("ὑπέρ", "G:Prep", "above/for", "huper"),
        "G5229": ("ὑπεραίρω", "G:V", "be haughty", "huperairō"),
        "G5230": ("ὑπέρακμος", "G:A", "past one’s prime", "huperakmos"),
        "G5231": ("ὑπεράνω", "G:Prep", "above", "huperanō"),
        "G5232": ("ὑπεραυξάνω", "G:V", "to increase", "huperauxanō"),
        "G5233": ("ὑπερβαίνω", "G:V", "to wrong", "huperbainō"),
        "G5234": ("ὑπερβαλλόντως", "G:Adv", "severely", "huperballontōs"),
        "G5235": ("ὑπερβάλλω", "G:V", "to surpass", "huperballō"),
        "G5236": ("ὑπερβολή", "G:N-F", "surpassing", "huperbolē"),
        "G5237": ("ὑπερείδω", "G:V", "to overlook", "hupereidō"),
        "G5238": ("ὑπερέκεινα", "G:Prep", "beyond", "huperekeina"),
        "G5239": ("ὑπερεκτείνω", "G:V", "to overextend", "huperekteinō"),
        "G5240": ("ὑπερεκχύνω", "G:V", "to overflow", "huperekchunō"),
        "G5241": ("ὑπερεντυγχάνω", "G:V", "to intercede", "huperentugchanō"),
        "G5242": ("ὑπερέχω", "G:V", "be higher", "huperechō"),
        "G5243": ("ὑπερηφανία", "G:N-F", "pride", "huperēphania"),
        "G5244": ("ὑπερήφανος", "G:A", "arrogant", "huperēphanos"),
        "G5245": ("ὑπερνικάω", "G:V", "to conquer", "hupernikaō"),
        "G5246": ("ὑπέρογκος", "G:A", "boastful", "huperogkos"),
        "G5247": ("ὑπεροχή", "G:N-F", "authority", "huperochē"),
        "G5248": ("ὑπερπερισσεύω", "G:V", "to abound", "huperperisseuō"),
        "G5249": ("ὑπερπερισσῶς", "G:Adv", "exceedingly", "huperperissōs"),
        "G5250": ("ὑπερπλεονάζω", "G:V", "to abound", "huperpleonazō"),
        "G5251": ("ὑπερυψόω", "G:V", "to exalt", "huperupsoō"),
        "G5252": ("ὑπερφρονέω", "G:V", "be haughty", "huperphroneō"),
        "G5253": ("ὑπερῷον", "G:N-N", "upper room", "huperōon"),
        "G5254": ("ὑπέχω", "G:V", "to undergo", "hupechō"),
        "G5255": ("ὑπήκοος", "G:A", "obedient", "hupēkoos"),
        "G5256": ("ὑπηρετέω", "G:V", "to serve", "hupēreteō"),
        "G5257": ("ὑπηρέτης", "G:N-M", "servant", "hupēretēs"),
        "G5258": ("ὕπνος", "G:N-M", "sleep", "hupnos"),
        "G5259": ("ὑπό", "G:Prep", "by/under", "hupo"),
        "G5260": ("ὑποβάλλω", "G:V", "to instigate", "hupoballō"),
        "G5261": ("ὑπογραμμός", "G:N-M", "example", "hupogrammos"),
        "G5262": ("ὑπόδειγμα", "G:N-N", "example", "hupodeigma"),
        "G5263": ("ὑποδείκνυμι", "G:V", "to show", "hupodeiknumi"),
        "G5264": ("ὑποδέχομαι", "G:V", "to receive", "hupodechomai"),
        "G5265": ("ὑποδέω", "G:V", "to put on", "hupodeō"),
        "G5266": ("ὑπόδημα", "G:N-N", "sandal", "hupodēma"),
        "G5267": ("ὑπόδικος", "G:A", "accountable", "hupodikos"),
        "G5268": ("ὑποζύγιον", "G:N-N", "donkey", "hupozugion"),
        "G5269": ("ὑποζώννυμι", "G:V", "to undergird", "hupozōnnumi"),
        "G5270": ("ὑποκάτω", "G:Prep", "under", "hupokatō"),
        "G5271": ("ὑποκρίνω", "G:V", "to pretend", "hupokrinō"),
        "G5272": ("ὑπόκρισις", "G:N-F", "hypocrisy", "hupokrisis"),
        "G5273": ("ὑποκριτής", "G:N-M", "hypocrite", "hupokritēs"),
        "G5274": ("ὑπολαμβάνω", "G:V", "to take up/suppose", "hupolambanō"),
        "G5275": ("ὑπολείπω", "G:V", "to leave", "hupoleipō"),
        "G5276": ("ὑπολήνιον", "G:N-N", "trough", "hupolēnion"),
        "G5277": ("ὑπολιμπάνω", "G:V", "to leave behind", "hupolimpanō"),
        "G5278": ("ὑπομένω", "G:V", "to remain/endure", "hupomenō"),
        "G5279": ("ὑπομιμνήσκω", "G:V", "to remind", "hupomimnēskō"),
        "G5280": ("ὑπόμνησις", "G:N-F", "remembrance", "hupomnēsis"),
        "G5281": ("ὑπομονή", "G:N-F", "perseverance", "hupomonē"),
        "G5282": ("ὑπονοέω", "G:V", "to suppose", "huponoeō"),
        "G5283": ("ὑπόνοια", "G:N-F", "suspicion", "huponoia"),
        "G5284": ("ὑποπλέω", "G:V", "to sail the lee", "hupopleō"),
        "G5285": ("ὑποπνέω", "G:V", "to blow gently", "hupopneō"),
        "G5286": ("ὑποπόδιον", "G:N-N", "footstool", "hupopodion"),
        "G5287": ("ὑπόστασις", "G:N-F", "confidence/essence", "hupostasis"),
        "G5288": ("ὑποστέλλω", "G:V", "to withdraw", "hupostellō"),
        "G5289": ("ὑποστολή", "G:N-F", "shrinking", "hupostolē"),
        "G5290": ("ὑποστρέφω", "G:V", "to return", "hupostrephō"),
        "G5291": ("ὑποστρώννυμι", "G:V", "to spread", "hupostrōnnumi"),
        "G5292": ("ὑποταγή", "G:N-F", "submission", "hupotagē"),
        "G5293": ("ὑποτάσσω", "G:V", "to subject", "hupotassō"),
        "G5294": ("ὑποτίθημι", "G:V", "to lay", "hupotithēmi"),
        "G5295": ("ὑποτρέχω", "G:V", "to sail under", "hupotrechō"),
        "G5296": ("ὑποτύπωσις", "G:N-F", "example", "hupotupōsis"),
        "G5297": ("ὑποφέρω", "G:V", "to endure", "hupopherō"),
        "G5298": ("ὑποχωρέω", "G:V", "to withdraw", "hupochōreō"),
        "G5299": ("ὑπωπιάζω", "G:V", "to wear out", "hupōpiazō"),
        "G5300": ("ὗς", "G:N-F", "sow", "hus"),
        "G5301": ("ὕσσωπος", "G:N-M/F", "hyssop", "hussōpos"),
        "G5302": ("ὑστερέω", "G:V", "to lack", "hustereō"),
        "G5303": ("ὑστέρημα", "G:N-N", "deficiency", "husterēma"),
        "G5304": ("ὑστέρησις", "G:N-F", "poverty", "husterēsis"),
        "G5305": ("ὕστερον", "G:Adv", "later", "husteron"),
        "G5306": ("ὕστερος", "G:A", "later", "husteros"),
        "G5307": ("ὑφαντός", "G:A", "woven", "huphantos"),
        "G5308": ("ὑψηλός", "G:A", "high", "hupsēlos"),
        "G5309": ("ὑψηλοφρονέω", "G:V", "be arrogant", "hupsēlophroneō"),
        "G5310": ("ὕψιστος", "G:A", "highest", "hupsistos"),
        "G5311": ("ὕψος", "G:N-N", "height", "hupsos"),
        "G5312": ("ὑψόω", "G:V", "to lift up", "hupsoō"),
        "G5313": ("ὕψωμα", "G:N-N", "height", "hupsōma"),
        "G5314": ("φάγος", "G:N-M", "glutton", "phagos"),
        "G5315": ("φαγεῖν", "G:V", "to eat", "phagein"),
        "G5316": ("φαίνω", "G:V", "to shine/appear", "phainō"),
        "G5317": ("Φάλεκ", "N:N-M-P", "Peleg", "Phalek"),
        "G5318": ("φανερός", "G:A", "manifest", "phaneros"),
        "G5319": ("φανερόω", "G:V", "to manifest", "phaneroō"),
        "G5320": ("φανερῶς", "G:Adv", "plainly", "phanerōs"),
        "G5321": ("φανέρωσις", "G:N-F", "manifestation", "phanerōsis"),
        "G5322": ("φανός", "G:N-M", "torch", "phanos"),
        "G5323": ("Φανουήλ", "N:N-M-P", "Phanuel", "Phanouēl"),
        "G5324": ("φαντάζω", "G:V", "to make visible", "phantazō"),
        "G5325": ("φαντασία", "G:N-F", "pageantry", "phantasia"),
        "G5326": ("φάντασμα", "G:N-N", "ghost", "phantasma"),
        "G5327": ("φάραγξ", "G:N-F", "valley", "pharagx"),
        "G5328": ("Φαραώ", "N:N--T", "Pharaoh", "Pharaō"),
        "G5329": ("Φάρες", "N:N-M-P", "Perez", "Phares"),
        "G5330": ("Φαρισαῖος", "N:N--T", "Pharisee", "Pharisaios"),
        "G5331": ("φαρμακεία", "G:N-F", "sorcery", "pharmakeia"),
        "G5332": ("φαρμακεύς", "G:N-M", "sorcerer", "pharmakeus"),
        "G5333": ("φάρμακος", "G:N-M/N", "sorcerer", "pharmakos"),
        "G5334": ("φάσις", "G:N-F", "news", "phasis"),
        "G5335": ("φάσκω", "G:V", "to claim", "phaskō"),
        "G5336": ("φάτνη", "G:N-F", "manger", "phatnē"),
        "G5337": ("φαῦλος", "G:A", "evil", "phaulos"),
        "G5338": ("φέγγος", "G:N-N", "brightness", "pheggos"),
        "G5339": ("φείδομαι", "G:V", "to spare", "pheidomai"),
        "G5340": ("φειδομένως", "G:Adv", "sparingly", "pheidomenōs"),
        "G5341": ("φελόνης, φαιλόνης", "G:N-M", "cloak", "phelonēs, phailonēs"),
        "G5342": ("φέρω", "G:V", "to bear/lead", "pherō"),
        "G5343": ("φεύγω", "G:V", "to flee", "pheugō"),
        "G5344": ("Φῆλιξ", "N:N-M-P", "Felix", "Phēlix"),
        "G5345": ("φήμη", "G:N-F", "news", "phēmē"),
        "G5346": ("φημί", "G:V", "to say", "phēmi"),
        "G5347": ("Φῆστος", "N:N-M-P", "Festus", "Phēstos"),
        "G5348": ("φθάνω", "G:V", "to precede/arrive", "phthanō"),
        "G5349": ("φθαρτός", "G:A", "perishable", "phthartos"),
        "G5350": ("φθέγγομαι", "G:V", "to speak", "phtheggomai"),
        "G5351": ("φθείρω", "G:V", "to destroy", "phtheirō"),
        "G5352": ("φθινοπωρινός", "G:A", "autumnal", "phthinopōrinos"),
        "G5353": ("φθόγγος", "G:N-M", "sound", "phthoggos"),
        "G5354": ("φθονέω", "G:V", "to envy", "phthoneō"),
        "G5355": ("φθόνος", "G:N-M", "envy", "phthonos"),
        "G5356": ("φθορά", "G:N-F", "corruption", "phthora"),
        "G5357": ("φιάλη", "G:N-F", "bowl", "phialē"),
        "G5358": ("φιλάγαθος", "G:A", "lover of good", "philagathos"),
        "G5359": ("Φιλαδέλφεια", "N:N-F-L", "Philadelphia", "Philadelpheia"),
        "G5360": ("φιλαδελφία", "G:N-F", "brotherly love", "philadelphia"),
        "G5361": ("φιλάδελφος", "G:A", "loving the brothers", "philadelphos"),
        "G5362": ("φίλανδρος", "G:A", "husband-loving", "philandros"),
        "G5363": ("φιλανθρωπία", "G:N-F", "benevolence", "philanthrōpia"),
        "G5364": ("φιλανθρώπως", "G:Adv", "benevolently", "philanthrōpōs"),
        "G5365": ("φιλαργυρία", "G:N-F", "love of money", "philarguria"),
        "G5366": ("φιλάργυρος", "G:A", "money-loving", "philarguros"),
        "G5367": ("φίλαυτος", "G:A", "selfish", "philautos"),
        "G5368": ("φιλέω", "G:V", "to love", "phileō"),
        "G5369": ("φιλήδονος", "G:A", "pleasure-loving", "philēdonos"),
        "G5370": ("φίλημα", "G:N-N", "kiss", "philēma"),
        "G5371": ("Φιλήμων", "N:N-M-P", "Philemon", "Philēmōn"),
        "G5372": ("Φίλητος", "N:N-M-P", "Philetus", "Philētos"),
        "G5373": ("φιλία", "G:N-F", "friendship", "philia"),
        "G5374": ("Φιλιππήσιος", "N:N--LG", "Philippian", "Philippēsios"),
        "G5375": ("Φίλιπποι", "N:N-M-L", "Philippi", "Philippoi"),
        "G5376": ("Φίλιππος", "N:N-M-P", "Philip", "Philippos"),
        "G5377": ("φιλόθεος", "G:A", "God-loving", "philotheos"),
        "G5378": ("Φιλόλογος", "N:N-M-P", "Philologus", "Philologos"),
        "G5379": ("φιλονεικία", "G:N-F", "love of dispute", "philoneikia"),
        "G5380": ("φιλόνεικος", "G:A", "dispute-loving", "philoneikos"),
        "G5381": ("φιλοξενία", "G:N-F", "hospitality", "philoxenia"),
        "G5382": ("φιλόξενος", "G:A", "hospitable", "philoxenos"),
        "G5383": ("φιλοπρωτεύω", "G:V", "to love to be first", "philoprōteuō"),
        "G5384": ("φίλος", "G:A", "friendly/friend", "philos"),
        "G5385": ("φιλοσοφία", "G:N-F", "philosophy", "philosophia"),
        "G5386": ("φιλόσοφος", "G:N-M", "philosopher", "philosophos"),
        "G5387": ("φιλόστοργος", "G:A", "devoted", "philostorgos"),
        "G5388": ("φιλότεκνος", "G:A", "parental love", "philoteknos"),
        "G5389": ("φιλοτιμέομαι", "G:V", "to aspire", "philotimeomai"),
        "G5390": ("φιλοφρόνως", "G:Adv", "hospitably", "philophronōs"),
        "G5391": ("φιλόφρων", "G:A", "friendly", "philophrōn"),
        "G5392": ("φιμόω", "G:V", "to muzzle", "phimoō"),
        "G5393": ("Φλέγων", "N:N-M-P", "Phlegon", "Phlegōn"),
        "G5394": ("φλογίζω", "G:V", "to kindle", "phlogizō"),
        "G5395": ("φλόξ", "G:N-F", "flame", "phlox"),
        "G5396": ("φλυαρέω", "G:V", "to talk nonsense", "phluareō"),
        "G5397": ("φλύαρος", "G:A", "gossipy", "phluaros"),
        "G5398": ("φοβερός", "G:A", "fearful", "phoberos"),
        "G5399": ("φοβέω", "G:V", "to fear", "phobeō"),
        "G5400": ("φόβητρον", "G:N-N", "fearful thing", "phobētron"),
        "G5401": ("φόβος", "G:N-M", "fear", "phobos"),
        "G5402": ("Φοίβη", "N:N-F-P", "Phoebe", "Phoibē"),
        "G5403": ("Φοινίκη", "N:N-F-L", "Phoenicia", "Phoinikē"),
        "G5404": ("φοῖνιξ", "G:N-M", "palm", "phoinix"),
        "G5405": ("Φοῖνιξ", "N:N-M-L", "Phenice", "Phoinix"),
        "G5406": ("φονεύς", "G:N-M", "murderer", "phoneus"),
        "G5407": ("φονεύω", "G:V", "to murder", "phoneuō"),
        "G5408": ("φόνος", "G:N-M", "murder", "phonos"),
        "G5409": ("φορέω", "G:V", "to wear", "phoreō"),
        "G5410": ("Φόρον", "G:N-N", "forum", "Phoron"),
        "G5411": ("φόρος", "G:N-M", "tax", "phoros"),
        "G5412": ("φορτίζω", "G:V", "to burden", "phortizō"),
        "G5413": ("φορτίον", "G:N-N", "burden", "phortion"),
        "G5414": ("φόρτος", "G:N-M", "cargo", "phortos"),
        "G5415": ("Φορτουνᾶτος", "N:N-M-P", "Fortunatus", "Phortounatos"),
        "G5416": ("φραγέλλιον", "G:N-N", "whip", "phragellion"),
        "G5417": ("φραγελλόω", "G:V", "to whip", "phragelloō"),
        "G5418": ("φραγμός", "G:N-M", "fence", "phragmos"),
        "G5419": ("φράζω", "G:V", "to explain", "phrazō"),
        "G5420": ("φράσσω", "G:V", "to stop", "phrassō"),
        "G5421": ("φρέαρ", "G:N-N", "well/abyss", "phrear"),
        "G5422": ("φρεναπατάω", "G:V", "to deceive", "phrenapataō"),
        "G5423": ("φρεναπάτης", "G:N-M", "deceiver", "phrenapatēs"),
        "G5424": ("φρήν", "G:N-F", "thinking", "phrēn"),
        "G5425": ("φρίσσω", "G:V", "to shudder", "phrissō"),
        "G5426": ("φρονέω", "G:V", "to reason", "phroneō"),
        "G5427": ("φρόνημα", "G:N-N", "purpose", "phronēma"),
        "G5428": ("φρόνησις", "G:N-F", "understanding", "phronēsis"),
        "G5429": ("φρόνιμος", "G:A", "thoughtful", "phronimos"),
        "G5430": ("φρονίμως", "G:Adv", "shrewdly", "phronimōs"),
        "G5431": ("φροντίζω", "G:V", "be careful", "phrontizō"),
        "G5432": ("φρουρέω", "G:V", "to guard", "phroureō"),
        "G5433": ("φρυάσσομαι", "G:V", "to rage", "phruassomai"),
        "G5434": ("φρύγανον", "G:N-N", "brushwood", "phruganon"),
        "G5435": ("Φρυγία", "N:N-F-L", "Phrygia", "Phrugia"),
        "G5436": ("Φύγελλος", "N:N-M-P", "Phygelus", "Phugellos"),
        "G5437": ("φυγή", "G:N-F", "flight", "phugē"),
        "G5438": ("φυλακή", "G:N-F", "prison/watch", "phulakē"),
        "G5439": ("φυλακίζω", "G:V", "to imprison", "phulakizō"),
        "G5440": ("φυλακτήριον", "G:N-N", "phylactery", "phulaktērion"),
        "G5441": ("φυλακτήριος", "G:N-M", "guard", "phulaktērios"),
        "G5442": ("φυλάσσω", "G:V", "to keep/guard", "phulassō"),
        "G5443": ("φυλή", "G:N-F", "tribe", "phulē"),
        "G5444": ("φύλλον", "G:N-N", "leaf", "phullon"),
        "G5445": ("φύραμα", "G:N-N", "lump", "phurama"),
        "G5446": ("φυσικός", "G:A", "natural", "phusikos"),
        "G5447": ("φυσικῶς", "G:Adv", "physically", "phusikōs"),
        "G5448": ("φυσιόω", "G:V", "to inflate", "phusioō"),
        "G5449": ("φύσις", "G:N-F", "nature", "phusis"),
        "G5450": ("φυσίωσις", "G:N-F", "arrogance", "phusiōsis"),
        "G5451": ("φυτεία", "G:N-F", "plant", "phuteia"),
        "G5452": ("φυτεύω", "G:V", "to plant", "phuteuō"),
        "G5453": ("φύω", "G:V", "to grow", "phuō"),
        "G5454": ("φωλεός", "G:N-M", "den", "phōleos"),
        "G5455": ("φωνέω", "G:V", "to call", "phōneō"),
        "G5456": ("φωνή", "G:N-F", "voice/sound", "phōnē"),
        "G5457": ("φῶς", "G:N-N", "light", "phōs"),
        "G5458": ("φωστήρ", "G:N-M", "light", "phōstēr"),
        "G5459": ("φωσφόρος", "G:A", "light-bearing", "phōsphoros"),
        "G5460": ("φωτεινός", "G:A", "bright", "phōteinos"),
        "G5461": ("φωτίζω", "G:V", "to illuminate", "phōtizō"),
        "G5462": ("φωτισμός", "G:N-M", "light", "phōtismos"),
        "G5463": ("χαίρω", "G:V", "to rejoice", "chairō"),
        "G5464": ("χάλαζα", "G:N-F", "hail", "chalaza"),
        "G5465": ("χαλάω", "G:V", "to lower", "chalaō"),
        "G5466": ("Χαλδαῖος", "N:N--LG", "Chaldean", "Chaldaios"),
        "G5467": ("χαλεπός", "G:A", "harsh", "chalepos"),
        "G5468": ("χαλιναγωγέω", "G:V", "to bridle", "chalinagōgeō"),
        "G5469": ("χαλινός", "G:N-M", "bridle", "chalinos"),
        "G5470": ("χάλκεος", "G:A", "bronze", "chalkeos"),
        "G5471": ("χαλκεύς", "G:N-M", "coppersmith", "chalkeus"),
        "G5472": ("χαλκηδών", "G:N-M", "chalcedony", "chalkēdōn"),
        "G5473": ("χαλκίον", "G:N-N", "kettle", "chalkion"),
        "G5474": ("χαλκολίβανος", "G:N-N", "bronze", "chalkolibanos"),
        "G5475": ("χαλκός", "G:N-M", "copper/bronze/coin", "chalkos"),
        "G5476": ("χαμαί", "G:Adv", "on the ground", "chamai"),
        "G5477": ("Χαναάν", "N:N--L", "Canaan", "Chanaan"),
        "G5478": ("Χανααναῖος", "N:N--LG", "Canaanite", "Chanaanaios"),
        "G5479": ("χαρά", "G:N-F", "joy", "chara"),
        "G5480": ("χάραγμα", "G:N-N", "image/mark", "charagma"),
        "G5481": ("χαρακτήρ", "G:N-M", "reproduction", "charaktēr"),
        "G5482": ("χάραξ", "G:N-M", "barricade", "charax"),
        "G5483": ("χαρίζω", "G:V", "to give grace", "charizō"),
        "G5484": ("χάριν", "G:Prep", "because of", "charin"),
        "G5485": ("χάρις", "G:N-F", "grace", "charis"),
        "G5486": ("χάρισμα", "G:N-N", "gift", "charisma"),
        "G5487": ("χαριτόω", "G:V", "to favor", "charitoō"),
        "G5488": ("Χαρράν", "N:N--L", "Haran", "Charran"),
        "G5489": ("χάρτης", "G:N-M", "paper", "chartēs"),
        "G5490": ("χάσμα", "G:N-N", "chasm", "chasma"),
        "G5491": ("χεῖλος", "G:N-N", "lip", "cheilos"),
        "G5492": ("χειμάζω", "G:V", "be storm-tossed", "cheimazō"),
        "G5493": ("χείμαρρος", "G:N-M", "brook", "cheimarros"),
        "G5494": ("χειμών", "G:N-M", "winter", "cheimōn"),
        "G5495": ("χείρ", "G:N-F", "hand", "cheir"),
        "G5496": ("χειραγωγέω", "G:V", "to lead by the hand", "cheiragōgeō"),
        "G5497": ("χειραγωγός", "G:N-M", "hand-leader", "cheiragōgos"),
        "G5498": ("χειρόγραφος", "G:N-N", "writen record", "cheirographos"),
        "G5499": ("χειροποίητος", "G:A", "hand-made", "cheiropoiētos"),
        "G5500": ("χειροτονέω", "G:V", "to appoint", "cheirotoneō"),
        "G5501": ("χείρων", "G:A", "worse than", "cheirōn"),
        "G5502": ("χερουβίμ", "N:N--T", "Cherub", "cheroubim"),
        "G5503": ("χήρα", "G:N-F", "widow", "chēra"),
        "G5504": ("χθές", "G:Adv", "yesterday", "chthes"),
        "G5505": ("χιλιάς", "G:N-F", "thousand", "chilias"),
        "G5506": ("χιλίαρχος", "G:N-M", "military officer", "chiliarchos"),
        "G5507": ("χίλιοι", "G:A", "thousand", "chilioi"),
        "G5508": ("Χίος", "N:N-F-L", "Chios", "Chios"),
        "G5509": ("χιτών", "G:N-M", "tunic", "chitōn"),
        "G5510": ("χιών", "G:N-F", "snow", "chiōn"),
        "G5511": ("χλαμύς", "G:N-F", "robe", "chlamus"),
        "G5512": ("χλευάζω", "G:V", "to sneer", "chleuazō"),
        "G5513": ("χλιαρός", "G:A", "lukewarm", "chliaros"),
        "G5514": ("Χλόη", "N:N-F-P", "Chloe", "Chloē"),
        "G5515": ("χλωρός", "G:A", "green", "chlōros"),
        "G5516": ("χξϛ", "G:A", "666", "ch.x.s"),
        "G5517": ("χοϊκός", "G:A", "made of dust", "choikos"),
        "G5518": ("χοῖνιξ", "G:N-F", "quart", "choinix"),
        "G5519": ("χοῖρος", "G:N-M", "pig", "choiros"),
        "G5520": ("χολάω", "G:V", "be angry", "cholaō"),
        "G5521": ("χολή", "G:N-F", "gall", "cholē"),
        "G5522": ("χόος", "G:N-M", "dust", "choos"),
        "G5523": ("Χοραζίν", "N:N--L", "Chorazin", "Chorazin"),
        "G5524": ("χορηγέω", "G:V", "to provide", "chorēgeō"),
        "G5525": ("χορός", "G:N-M", "a dance", "choros"),
        "G5526": ("χορτάζω", "G:V", "to feed", "chortazō"),
        "G5527": ("χόρτασμα", "G:N-N", "food", "chortasma"),
        "G5528": ("χόρτος", "G:N-M", "grass", "chortos"),
        "G5529": ("Χουζᾶς", "N:N-M-P", "Chuza", "Chouzas"),
        "G5530": ("χράω", "G:V", "to use", "chraō"),
        "G5531": ("χράω", "G:V", "to lend", "chraō"),
        "G5532": ("χρεία", "G:N-F", "need", "chreia"),
        "G5533": ("χρεωφειλέτης", "G:N-M", "debtor", "chreōpheiletēs"),
        "G5534": ("χρή", "G:V", "to ought/need", "chrē"),
        "G5535": ("χρῄζω", "G:V", "to need", "chrēzō"),
        "G5536": ("χρῆμα", "G:N-N", "money", "chrēma"),
        "G5537": ("χρηματίζω", "G:V", "to announce", "chrēmatizō"),
        "G5538": ("χρηματισμός", "G:N-M", "proclamation", "chrēmatismos"),
        "G5539": ("χρήσιμος", "G:A", "profitable", "chrēsimos"),
        "G5540": ("χρῆσις", "G:N-F", "relations", "chrēsis"),
        "G5541": ("χρηστεύομαι", "G:V", "be kind", "chrēsteuomai"),
        "G5542": ("χρηστολογία", "G:N-F", "smooth talk", "chrēstologia"),
        "G5543": ("χρηστός", "G:A", "good/kind", "chrēstos"),
        "G5544": ("χρηστότης", "G:N-F", "kindness", "chrēstotēs"),
        "G5545": ("χρῖσμα", "G:N-N", "anointing", "chrisma"),
        "G5546": ("Χριστιανός", "N:N--T", "Christian", "Christianos"),
        "G5547": ("Χριστός", "N:N--T", "Christ", "Christos"),
        "G5548": ("χρίω", "G:V", "to anoint", "chriō"),
        "G5549": ("χρονίζω", "G:V", "to delay", "chronizō"),
        "G5550": ("χρόνος", "G:N-M", "time", "chronos"),
        "G5551": ("χρονοτριβέω", "G:V", "to spend time", "chronotribeō"),
        "G5552": ("χρύσεος", "G:A", "golden", "chruseos"),
        "G5553": ("χρυσίον", "G:N-N", "gold", "chrusion"),
        "G5554": ("χρυσοδακτύλιος", "G:A", "gold-ringed", "chrusodaktulios"),
        "G5555": ("χρυσόλιθος", "G:N-M", "chrysolite", "chrusolithos"),
        "G5556": ("χρυσόπρασος", "G:N-M", "chrysoprase", "chrusoprasos"),
        "G5557": ("χρυσός", "G:N-M", "gold", "chrusos"),
        "G5558": ("χρυσόω", "G:V", "to gild", "chrusoō"),
        "G5559": ("χρώς", "G:N-M", "skin", "chrōs"),
        "G5560": ("χωλός", "G:A", "lame", "chōlos"),
        "G5561": ("χώρα", "G:N-F", "country", "chōra"),
        "G5562": ("χωρέω", "G:V", "to make room for", "chōreō"),
        "G5563": ("χωρίζω", "G:V", "to separate/leave", "chōrizō"),
        "G5564": ("χωρίον", "G:N-N", "place", "chōrion"),
        "G5565": ("χωρίς", "G:Prep", "without", "chōris"),
        "G5566": ("χῶρος", "G:N-M", "northwest", "chōros"),
        "G5567": ("ψάλλω", "G:V", "to sing praise", "psallō"),
        "G5568": ("ψαλμός", "G:N-M", "psalm", "psalmos"),
        "G5569": ("ψευδάδελφος", "G:N-M", "false brother", "pseudadelphos"),
        "G5570": ("ψευδαπόστολος", "G:N-M", "false apostle", "pseudapostolos"),
        "G5571": ("ψευδής", "G:A", "false", "pseudēs"),
        "G5572": ("ψευδοδιδάσκαλος", "G:N-M", "false teacher", "pseudodidaskalos"),
        "G5573": ("ψευδολόγος", "G:A", "lying", "pseudologos"),
        "G5574": ("ψεύδομαι", "G:V", "to lie", "pseudomai"),
        "G5575": ("ψευδομάρτυρ", "G:N-M", "false witness", "pseudomartur"),
        "G5576": ("ψευδομαρτυρέω", "G:V", "to perjure", "pseudomartureō"),
        "G5577": ("ψευδομαρτυρία", "G:N-F", "false testimony", "pseudomarturia"),
        "G5578": ("ψευδοπροφήτης", "G:N-M", "false prophet", "pseudoprophētēs"),
        "G5579": ("ψεῦδος", "G:N-N", "lie", "pseudos"),
        "G5580": ("ψευδόχριστος", "G:N-M", "false Christ", "pseudochristos"),
        "G5581": ("ψευδώνυμος", "G:A", "falsely called", "pseudōnumos"),
        "G5582": ("ψεῦσμα", "G:N-N", "falsehood", "pseusma"),
        "G5583": ("ψεύστης", "G:N-M", "liar", "pseustēs"),
        "G5584": ("ψηλαφάω", "G:V", "to touch", "psēlaphaō"),
        "G5585": ("ψηφίζω", "G:V", "to calculate", "psēphizō"),
        "G5586": ("ψῆφος", "G:N-F", "stone/vote", "psēphos"),
        "G5587": ("ψιθυρισμός", "G:N-M", "gossip", "psithurismos"),
        "G5588": ("ψιθυριστής", "G:N-M", "gossip", "psithuristēs"),
        "G5589": ("ψιχίον, ψίξ", "G:N-N", "crumb", "psichion, psix"),
        "G5590": ("ψυχή", "G:N-F", "soul", "psuchē"),
        "G5591": ("ψυχικός", "G:A", "natural", "psuchikos"),
        "G5592": ("ψῦχος", "G:N-N", "cold", "psuchos"),
        "G5593": ("ψυχρός", "G:A", "cold", "psuchros"),
        "G5594": ("ψύχω", "G:V", "to cool", "psuchō"),
        "G5595": ("ψωμίζω", "G:V", "to feed/dole out", "psōmizō"),
        "G5596": ("ψωμίον", "G:N-N", "piece", "psōmion"),
        "G5597": ("ψώχω", "G:V", "to rub", "psōchō"),
        "G5598": ("Ὦ", "G:N", "Omega", "Ō"),
        "G5599": ("ὦ", "G:Intj", "oh!", "ō"),
        "G5600": ("ὦ", "G:V", "may be", "ō"),
        "G5601": ("Ὠβήδ", "N:N-M-P", "Obed", "Ōbēd"),
        "G5602": ("ὧδε", "G:Adv", "here", "hōde"),
        "G5603": ("ᾠδή", "G:N-F", "song", "ōdē"),
        "G5604": ("ὠδίν", "G:N-F", "labor", "ōdin"),
        "G5605": ("ὠδίνω", "G:V", "be in labor", "ōdinō"),
        "G5606": ("ὦμος", "G:N-M", "shoulder", "ōmos"),
        "G5607": ("ὤν", "G:V", "being", "ōn"),
        "G5608": ("ὠνέομαι", "G:V", "to buy", "ōneomai"),
        "G5609": ("ᾠόν", "G:N-N", "an egg", "ōon"),
        "G5610": ("ὥρα", "G:N-F", "hour", "hōra"),
        "G5611": ("ὡραῖος", "G:A", "beautiful/timely", "hōraios"),
        "G5612": ("ὠρύομαι", "G:V", "to roar", "ōruomai"),
        "G5613": ("ὡς", "G:Conj", "which/how", "hōs"),
        "G5614": ("ὡσαννά", "H:Intj", "Hosanna!", "hōsanna"),
        "G5615": ("ὡσαύτως", "G:Adv", "likewise", "hōsautōs"),
        "G5616": ("ὡσεί", "G:Conj", "like/as/about", "hōsei"),
        "G5617": ("Ὡσηέ", "N:N-M-P", "Hosea", "Hōsēe"),
        "G5618": ("ὥσπερ", "G:Conj", "just as", "hōsper"),
        "G5619": ("ὡσπερεί", "G:Conj", "just as if", "hōsperei"),
        "G5620": ("ὥστε", "G:Conj", "so", "hōste"),
        "G5621": ("ὠτίον", "G:N-N", "ear", "ōtion"),
        "G5622": ("ὠφέλεια", "G:N-F", "advantage", "ōpheleia"),
        "G5623": ("ὠφελέω", "G:V", "to help", "ōpheleō"),
        "G5624": ("ὠφέλιμος", "G:A", "valuable", "ōphelimos"),
        "G6000": ("ἀγγέλλω", "G:V", "to report", "aggellō"),
        "G6001": ("ἄγγος", "G:N-N", "vessel", "aggos"),
        "G6002": ("Ἀδμίν", "N:N-M-P", "Admin", "Admin"),
        "G6003": ("ἀθροίζω", "G:V", "to gather", "athroizō"),
        "G6004": ("ἀλλαχοῦ", "G:Adv", "elsewhere", "allachou"),
        "G6005": ("ἀμφιέζω", "G:V", "to dress", "amphiezō"),
        "G6006": ("ἀμφιβάλλω", "G:V", "to cast", "amphiballō"),
        "G6007": ("ἄμωμον", "G:N-N", "amomum", "amōmon"),
        "G6008": ("ἀναπηδάω", "G:V", "to jump up", "anapēdaō"),
        "G6011": ("βελόνη", "G:N-F", "needle", "belonē"),
        "G6013": ("γένημα", "G:N-N", "produce", "genēma"),
        "G6014": ("δεκαοκτώ", "G:A", "eighteen", "dekaoktō"),
        "G6015": ("δέος", "G:N-N", "fear", "deos"),
        "G6016": ("διακαθαίρω", "G:V", "to scour", "diakathairō"),
        "G6017": ("διαχλευάζω", "G:V", "to deride", "diachleuazō"),
        "G6018": ("διενθυμέομαι", "G:V", "to ponder", "dienthumeomai"),
        "G6019": ("δισμυριάς", "G:A", "two-myriad", "dismurias"),
        "G6020": ("δοκιμασία", "G:N-F", "examination", "dokimasia"),
        "G6022": ("δυσφημέω", "G:V", "to defame", "dusphēmeō"),
        "G6023": ("ἐγκακέω", "G:V", "be discouraged", "egkakeō"),
        "G6025": ("εἵνεκεν", "G:Prep", "because", "ehineken"),
        "G6027": ("ἐκζήτησις", "G:N-F", "speculation", "ekzētēsis"),
        "G6028": ("ἐκθαυμάζω", "G:V", "be amazed", "ekthaumazō"),
        "G6029": ("ἐκπερισσῶς", "G:Adv", "exceedingly", "ekperissōs"),
        "G6030": ("ἐκπηδάω", "G:V", "to rush out", "ekpēdaō"),
        "G6031": ("ἐλεγμός", "G:N-M", "rebuke", "elegmos"),
        "G6032": ("ἐμπαιγμονή", "G:N-F", "derision", "empaigmonē"),
        "G6033": ("ἔνθεν", "G:Adv", "hence", "enthen"),
        "G6034": ("ἐνορκίζομαι", "G:V", "to adjure", "enorkizomai"),
        "G6035": ("ἐπάρατος", "G:A", "accursed", "eparatos"),
        "G6036": ("ἐπεισέρχομαι", "G:V", "to enter", "epeiserchomai"),
        "G6037": ("ἐπισπείρω", "G:V", "to sow above", "epispeirō"),
        "G6039": ("εὐθύμως", "G:Adv", "cheerfully", "euthumōs"),
        "G6041": ("ζηλεύω", "G:V", "to envy", "zēleuō"),
        "G6043": ("Ἰωδά", "N:N-M-P", "Joda", "Iōda"),
        "G6044": ("Ἰωσήχ", "N:N-M-P", "Josech", "Iōsēch"),
        "G6045": ("καθώσπερ", "G:Conj", "as", "kathōsper"),
        "G6046": ("καταβαρύνω", "G:V", "to burden", "katabarunō"),
        "G6047": ("καταγράφω", "G:V", "to delineate", "katagraphō"),
        "G6048": ("καταδίκη", "G:N-F", "judgment", "katadikē"),
        "G6049": ("κατακύπτω", "G:V", "to stoop", "katakuptō"),
        "G6050": ("κατευλογέω", "G:V", "to bless", "kateulogeō"),
        "G6051": ("κατήγωρ", "G:N-M", "an accuser", "katēgōr"),
        "G6052": ("κατοικίζω", "G:V", "to settle", "katoikizō"),
        "G6053": ("κατωτέρω", "G:Adv", "lower", "katōterō"),
        "G6055": ("κλινάριον", "G:N-N", "a cot", "klinarion"),
        "G6058": ("κρυφαῖος", "G:A", "hidden", "kruphaios"),
        "G6059": ("κυκλεύω", "G:V", "to encompass", "kukleuō"),
        "G6060": ("μετατρέπω", "G:V", "to turn around", "metatrepō"),
        "G6061": ("μύλινος", "G:A", "mill", "mulinos"),
        "G6063": ("οἶδα", "G:V", "to know", "oida"),
        "G6064": ("οἰκετεία", "G:N-F", "household", "oiketeia"),
        "G6065": ("οἰκοδόμος", "G:N-M", "builder", "oikodomos"),
        "G6066": ("ὀλιγοπιστία", "G:N-F", "little faith", "oligopistia"),
        "G6067": ("ὀλίγως", "G:Adv", "scarcely", "oligōs"),
        "G6068": ("ὀμίχλη", "G:N-F", "fog", "omichlē"),
        "G6069": ("πανταχῆ", "G:Adv", "everywhere", "pantachē"),
        "G6070": ("παρεμβάλλω", "G:V", "to encamp", "paremballō"),
        "G6071": ("περαιτέρω", "G:Adv", "further", "peraiterō"),
        "G6072": ("περιάπτω", "G:V", "to touch", "periaptō"),
        "G6073": ("πραϋπαθία", "G:N-F", "gentleness", "praupathia"),
        "G6074": ("προπάτωρ", "G:N-M", "ancestor", "propatōr"),
        "G6075": ("προσαίτης", "G:N-M", "a beggar", "prosaitēs"),
        "G6076": ("προσκλίνω", "G:V", "to lean toward", "prosklinō"),
        "G6077": ("Πύρρος", "N:N-M-P", "Pyrrhus", "Purros"),
        "G6078": ("πρώτως", "G:Adv", "originally", "prōtōs"),
        "G6079": ("σιτίον", "G:N-N", "food", "sition"),
        "G6080": ("συγγενίς", "G:N-F", "relative", "suggenis"),
        "G6081": ("συμπίπτω", "G:V", "be downcast", "sumpiptō"),
        "G6083": ("σύνοιδα", "G:V", "be conscious of", "sunoida"),
        "G6084": ("Τιτίος", "N:N-M-P", "Titius", "Titios"),
        "G6085": ("τρῆμα", "G:N-N", "hole", "trēma"),
        "G6087": ("ὑπερεκπερισσοῦ", "G:Adv", "superabundantly", "huperekperissou"),
        "G6088": ("ὑπερλίαν", "G:Adv", "greatly", "huperlian"),
        "G6090": ("ὠτάριον", "G:N-N", "ear", "ōtarion"),
        "G6091": ("δύσις", "G:N-F", "sunset", "dusis"),
        "G6092": ("ἐκβαίνω", "G:V", "to to_step_out", "ekbainō"),
        "G6093": ("σιρός", "G:N-M", "pit", "siros"),
        "G6094": ("αὐχέω", "G:V", "to to_boast", "aucheō"),
        "G6095": ("Ἀρνὶ", "", "Arni", "Arni"),
        "G6096": ("ἀποπληρόω", "G:V", "to satisfy", "apoplēroō"),
        "G6097": ("θρύπτω", "G:V", "to break", "thruptō"),
        "G6098": ("καταπίμπρημι", "", "burnt", "katapimprēmi"),
        "G6099": ("ξαίνω", "", "to comb", "xainō"),
        "G6100": ("ἀβασίλευτος", "", "independent", "abasileutos"),
        "G6101": ("ἄβατος", "", "impassable", "abatos"),
        "G6102": ("ἀβατόω", "", "to make impassable", "abatoō"),
        "G6103": ("ἀβοήθητος", "", "incurable", "aboēthētos"),
        "G6104": ("ἀβουλία", "", "thoughtlessness", "aboulia"),
        "G6105": ("ἅβρα", "", "handmaiden", "habra"),
        "G6106": ("ἀβροχία", "", "drought", "abrochia"),
        "G6107": ("ἀγαθόω", "", "to do good", "agathoō"),
        "G6108": ("ἀγαθύνω", "G:V", "be better", "agathunō"),
        "G6109": ("ἀγαθῶς", "", "fine", "agathōs"),
        "G6110": ("ἀγαλλίαμα", "", "a leap for joy", "agalliama"),
        "G6111": ("ἄγαλμα", "", "a statue", "agalma"),
        "G6112": ("αγανώθ", "", "cups", "aganōth"),
        "G6113": ("ἀγάπησις", "", "affection", "agapēsis"),
        "G6114": ("ἁγίασμα", "", "sanctification", "hagiasma"),
        "G6115": ("ἁγιαστήριον", "", "sanctuary", "hagiastērion"),
        "G6116": ("ἀγκαλίς", "", "an arm", "agkalis"),
        "G6117": ("ἀγκύλη", "", "a loop", "agkulē"),
        "G6118": ("ἀγκών", "", "an angle", "agkōn"),
        "G6119": ("ἀγκωνίσκος", "", "a joint", "agkōniskos"),
        "G6120": ("ἅγνισμα", "", "purification", "hagnisma"),
        "G6121": ("ἄγνος", "", "a chaste tree", "agnos"),
        "G6122": ("ἄγονος", "", "barren", "agonos"),
        "G6123": ("ἀγορασμός", "", "a purchase", "agorasmos"),
        "G6124": ("ἀγριαίνω", "G:V", "be wild", "agriainō"),
        "G6125": ("ἀγριοβάλανος", "", "holm oak", "agriobalanos"),
        "G6126": ("ἀγριομυρίκη", "", "tamarisk", "agriomurikē"),
        "G6127": ("ἄγροικος", "", "rugged", "agroikos"),
        "G6128": ("ἄγρωστις", "", "wild grass", "agrōstis"),
        "G6129": ("ἀγχιστεία", "", "right of inheritance", "agchisteia"),
        "G6130": ("ἀγχιστεύς", "", "a relative", "agchisteus"),
        "G6131": ("ἀγχιστεύω", "", "to be kin", "agchisteuō"),
        "G6132": ("ἄγχω", "", "to squeeze", "agchō"),
        "G6133": ("ἀδαμάντινος", "", "adamantine", "adamantinos"),
        "G6134": ("ἀδάμας", "", "adamant", "adamas"),
        "G6135": ("Ἄδαρ", "", "Adar", "Adar"),
        "G6136": ("ἄδειπνος", "", "supperless", "adeipnos"),
        "G6137": ("ἀδελφιδός", "", "a kinsman", "adelphidos"),
        "G6138": ("ἀδιάλυτος", "", "reinforced", "adialutos"),
        "G6139": ("ἀδολεσχέω", "", "to meditate", "adolescheō"),
        "G6140": ("ἀδολεσχία", "", "meditation", "adoleschia"),
        "G6141": ("ἀδοξέω", "G:V", "be despised", "adoxeō"),
        "G6142": ("ἁδρός", "", "stout", "hadros"),
        "G6143": ("ἁδρύνω", "", "to mature", "hadrunō"),
        "G6144": ("ἀδυναμία", "", "powerlessness", "adunamia"),
        "G6145": ("ἀέναος", "", "everlasting", "aenaos"),
        "G6146": ("ἀεργός", "", "idle", "aergos"),
        "G6147": ("ἀηδία", "", "rancor", "aēdia"),
        "G6148": ("Αθανιν", "", "Athanim", "Athanin"),
        "G6149": ("ἀθεσία", "", "rebellion", "athesia"),
        "G6150": ("ἀθέτημα", "", "a disregarding", "athetēma"),
        "G6152": ("ἀθυμία", "", "depression", "athumia"),
        "G6153": ("ἄθυτος", "", "unfit", "athutos"),
        "G6154": ("ἀθῳόω", "", "to acquit", "athōoō"),
        "G6155": ("αιδέδ", "", "acclamation", "aided"),
        "G6156": ("αἰδέομαι", "", "to stand in awe", "aideomai"),
        "G6157": ("αἰδοῖος", "", "genitals", "aidoios"),
        "G6158": ("αἰθάλη", "", "soot", "aithalē"),
        "G6159": ("αἴθριος", "", "open air", "aithrios"),
        "G6160": ("αιλ", "", "a portal", "ail"),
        "G6161": ("αιλάμ", "", "porch", "ailam"),
        "G6162": ("αιλαμμών", "", "pediment", "ailammōn"),
        "G6163": ("αἱμωδιάω", "", "to have tooth-chills", "ahimōdiaō"),
        "G6164": ("αἰνετός", "", "praiseworthy", "ainetos"),
        "G6165": ("αἴνη", "", "praise", "ainē"),
        "G6166": ("αἰνιγματιστής", "", "enigma speaker", "ainigmatistēs"),
        "G6167": ("αἴξ", "", "a goat", "aix"),
        "G6168": ("αἰπόλιον", "", "a flock of goats", "aipolion"),
        "G6169": ("αἰπόλος", "", "a herder", "aipolos"),
        "G6170": ("αἱρετός", "", "preferred", "ahiretos"),
        "G6171": ("αἰσθητικός", "", "sensitive", "aisthētikos"),
        "G6172": ("αἰσχρῶς", "", "disgracefully", "aischrōs"),
        "G6173": ("αἴτησις", "", "requesting", "aitēsis"),
        "G6174": ("αἰτιάομαι", "", "to blame", "aitiaomai"),
        "G6175": ("αἰτίωμα", "", "accusation", "aitiōma"),
        "G6176": ("αἰχμαλωτίς", "", "a captive", "aichmalōtis"),
        "G6177": ("ἀκακία", "", "innocence", "akakia"),
        "G6178": ("ἄκαν", "", "a thorn", "akan"),
        "G6179": ("ἀκάρδιος", "", "heartless", "akardios"),
        "G6180": ("ἀκατάποτος", "", "impotable", "akatapotos"),
        "G6181": ("ἀκατασκεύαστος", "", "unready", "akataskeuastos"),
        "G6182": ("ἀκατέργαστος", "", "unfinished", "akatergastos"),
        "G6183": ("ἄκαυστος", "", "inextinguishable", "akaustos"),
        "G6184": ("ἀκηδία", "", "indifference", "akēdia"),
        "G6185": ("ἀκηδιάω", "G:V", "be discouraged", "akēdiaō"),
        "G6186": ("ἀκιδωτός", "", "pointed", "akidōtos"),
        "G6187": ("ἀκίνητος", "", "fixed", "akinētos"),
        "G6188": ("ἀκίς", "", "a point", "akis"),
        "G6189": ("ἄκλητος", "", "uncalled", "aklētos"),
        "G6190": ("ἀκμή", "", "at this moment", "akmē"),
        "G6191": ("ἄκμων", "", "anvil", "akmōn"),
        "G6192": ("ἀκόλαστος", "", "unrestrained", "akolastos"),
        "G6193": ("ἀκονάω", "", "to sharpen", "akonaō"),
        "G6194": ("ἀκοντίζω", "", "to shoot", "akontizō"),
        "G6195": ("ἀκοντιστής", "", "a shooter", "akontistēs"),
        "G6196": ("ἄκοσμος", "", "unbecoming", "akosmos"),
        "G6197": ("ἀκουσιάζομαι", "", "to act unintentionally", "akousiazomai"),
        "G6198": ("ἀκούσιος", "", "unintentional", "akousios"),
        "G6199": ("ἀκουσίως", "", "unintentionally", "akousiōs"),
        "G6200": ("ἀκουστός", "", "audible", "akoustos"),
        "G6201": ("ἀκουτίζω", "", "to cause to hear", "akoutizō"),
        "G6202": ("ἄκρα", "", "Akra", "akra"),
        "G6203": ("ἀκριβασμός", "", "restriction", "akribasmos"),
        "G6204": ("ἀκροάομαι", "", "to hear", "akroaomai"),
        "G6205": ("ἀκρόασις", "", "a hearing", "akroasis"),
        "G6206": ("ἀκρόβυστος", "", "uncircumcised", "akrobustos"),
        "G6207": ("ἀκρόδρυα", "", "a fruit tree", "akrodrua"),
        "G6208": ("ἄκρος", "", "a toe/thumb/tip", "akros"),
        "G6209": ("ἀκρότομος", "", "chiseled", "akrotomos"),
        "G6210": ("ἀκρωτήριον", "", "an extremity", "akrōtērion"),
        "G6211": ("ἄκυρος", "", "void", "akuros"),
        "G6212": ("ἀλαζονεύομαι", "", "to act ostentatiously", "alazoneuomai"),
        "G6213": ("ἀλάλαγμα", "", "a price", "alalagma"),
        "G6214": ("ἀλαλαγμός", "", "a shout", "alalagmos"),
        "G6215": ("ἀλάομαι", "", "be a runaway", "alaomai"),
        "G6216": ("ἀλγέω", "", "to ache", "algeō"),
        "G6217": ("ἀλγηδών", "", "suffering", "algēdōn"),
        "G6218": ("ἄλγημα", "", "pain", "algēma"),
        "G6219": ("ἀλγηρός", "", "painful", "algēros"),
        "G6220": ("ἄλγος", "", "pain", "algos"),
        "G6221": ("ἄλειμμα", "", "an anointing", "aleimma"),
        "G6222": ("ἀλέω", "", "to grind flour", "aleō"),
        "G6223": ("ἁλιαίετος", "", "an osprey", "haliaietos"),
        "G6224": ("ἅλιμος", "", "a salty land", "halimos"),
        "G6225": ("ἀλισγέω", "", "to pollute", "alisgeō"),
        "G6226": ("ἁλίσκομαι", "", "to capture", "haliskomai"),
        "G6227": ("ἀλλ᾽ ἢ", "", "but only", "all᾽ ē"),
        "G6228": ("ἄλλαγμα", "", "a price", "allagma"),
        "G6229": ("ἀλλόγλωσσος", "", "foreign tongue", "alloglōssos"),
        "G6230": ("ἀλλοιόω", "", "to change", "alloioō"),
        "G6231": ("ἀλλοίωσις", "", "change", "alloiōsis"),
        "G6232": ("ἀλλοτριόω", "", "to alienate", "allotrioō"),
        "G6233": ("ἀλλοτρίως", "", "strangely", "allotriōs"),
        "G6234": ("ἀλλοτρίωσις", "", "alienation", "allotriōsis"),
        "G6235": ("ἀλλόφωνος", "", "foreign language", "allophōnos"),
        "G6236": ("ἅλμα", "", "a leap", "halma"),
        "G6237": ("ἅλμη", "", "brine", "halmē"),
        "G6238": ("ἁλμυρίς", "", "salt-flats", "halmuris"),
        "G6239": ("ἁλμυρός", "", "brackish", "halmuros"),
        "G6240": ("ἀλοητός", "", "threshing", "aloētos"),
        "G6241": ("ἀλοιφή", "", "ointment", "aloiphē"),
        "G6242": ("ἄλσος", "", "a sacred grove", "alsos"),
        "G6243": ("ἀλσώδης", "", "a woods", "alsōdēs"),
        "G6244": ("ἁλυσιδωτός", "", "a chain-work", "halusidōtos"),
        "G6245": ("ἄλφιτον", "", "toasted grain", "alphiton"),
        "G6246": ("ἀλφός", "", "psoriasis", "alphos"),
        "G6247": ("ἀλώθ", "", "aloe", "alōth"),
        "G6248": ("ἄμαξα", "", "a wagon", "amaxa"),
        "G6249": ("αμασενίθ", "", "octaves", "amasenith"),
        "G6250": ("ἀμάσητος", "", "unchewable", "amasētos"),
        "G6251": ("αματταρί", "", "a mark", "amattari"),
        "G6252": ("ἀμαυρός", "", "faint", "amauros"),
        "G6253": ("ἀμαυρόω", "", "to darken", "amauroō"),
        "G6254": ("ἀμβλύνω", "", "to blunt", "amblunō"),
        "G6255": ("ἀμέλγω", "", "to extract", "amelgō"),
        "G6256": ("ἄμελξις", "", "a milking", "amelxis"),
        "G6257": ("ἀμελῶς", "", "indifferently", "amelōs"),
        "G6258": ("ἀμέτρητος", "", "unmeasured", "ametrētos"),
        "G6259": ("ἄμητος", "", "a harvest", "amētos"),
        "G6260": ("ἀμισθί", "", "without pay", "amisthi"),
        "G6261": ("ἀμνάς", "", "a ewe-lamb", "amnas"),
        "G6262": ("ἀμνήστευτος", "", "not betrothed", "amnēsteutos"),
        "G6263": ("ἀμορίτης", "", "honey-bread", "amoritēs"),
        "G6264": ("ἀμπλάκημα", "", "an error", "amplakēma"),
        "G6265": ("ἀμύγδαλον", "", "an almond", "amugdalon"),
        "G6266": ("ἀμύθητος", "", "untold", "amuthētos"),
        "G6267": ("ἀμφίασις", "", "clothing", "amphiasis"),
        "G6268": ("ἀμφιβολεύς", "", "a fisherman", "amphiboleus"),
        "G6269": ("αμφίταπος", "", "a spread", "amphitapos"),
        "G6270": ("ἀμφοτεροδέξιος", "", "ambidextrous", "amphoterodexios"),
        "G6271": ("ἀνάμεσος", "", "in between", "anamesos"),
        "G6272": ("ἀναβαθμίς", "", "stairs", "anabathmis"),
        "G6273": ("ἀνάβασις", "", "ascending", "anabasis"),
        "G6274": ("ἀναβαστάζω", "", "to carry", "anabastazō"),
        "G6275": ("ἀναβάτης", "", "a rider", "anabatēs"),
        "G6276": ("ἀναβλαστάνω", "", "to sprout up", "anablastanō"),
        "G6277": ("ἀναβράσσω", "", "to stir up", "anabrassō"),
        "G6278": ("ἀναδενδράς", "", "tendril", "anadendras"),
        "G6279": ("ἀναζεύγνυμι", "", "to break camp", "anazeugnumi"),
        "G6280": ("ἀναζέω", "", "to break out", "anazeō"),
        "G6281": ("ἀναζυγή", "", "marching", "anazugē"),
        "G6282": ("ἀναιδής", "", "impudent", "anaidēs"),
        "G6283": ("ἀναιδῶς", "", "impudently", "anaidōs"),
        "G6284": ("ἀνακαίω", "", "to kindle", "anakaiō"),
        "G6285": ("ἀνακαλέω", "", "to call by name", "anakaleō"),
        "G6286": ("ἀνάκλισις", "", "laying down", "anaklisis"),
        "G6287": ("ἀνάκλιτος", "", "a couch", "anaklitos"),
        "G6288": ("ἀνακρούω", "", "to play music", "anakrouō"),
        "G6289": ("ἀναλάμπω", "", "to blaze up", "analampō"),
        "G6290": ("ἀνάλγητος", "", "unfeeling", "analgētos"),
        "G6291": ("ἀναλέγω", "", "to gather up", "analegō"),
        "G6292": ("ἀνάλημμα", "", "an elevation", "analēmma"),
        "G6293": ("ἀνάλωσις", "", "consumption", "analōsis"),
        "G6294": ("ἀναμίγνυμι", "", "to intermingle", "anamignumi"),
        "G6295": ("ἀνάνευσις", "", "reluctance", "ananeusis"),
        "G6296": ("ἀνανεύω", "", "to disagree", "ananeuō"),
        "G6297": ("ἀναντλέω", "G:V", "be fatigued", "anantleō"),
        "G6298": ("ἀναξηραίνω", "", "to dry up", "anaxērainō"),
        "G6299": ("ἀνάπαυμα", "", "rest", "anapauma"),
        "G6300": ("ἀναπετάω", "G:V", "be effronted", "anapetaō"),
        "G6301": ("ἀναπλήρωσις", "", "satisfying", "anaplērōsis"),
        "G6302": ("ἀναπνέω", "", "to breathe", "anapneō"),
        "G6303": ("ἀναπνοή", "", "breathing", "anapnoē"),
        "G6304": ("ἐναποθνῄσκω", "", "to die in", "enapothnēskō"),
        "G6305": ("ἀναποιέω", "", "to prepare", "anapoieō"),
        "G6306": ("ἀναπόστρεπτος", "", "not be turned away", "anapostreptos"),
        "G6307": ("ἀναπτερόω", "", "to incite", "anapteroō"),
        "G6308": ("ἀναρπάζω", "", "to kidnap", "anarpazō"),
        "G6309": ("ἀναρρήγνυμι", "", "to tear asunder", "anarrēgnumi"),
        "G6310": ("ἀνασκάπτω", "", "to dig up", "anaskaptō"),
        "G6311": ("ἀναστέλλω", "", "to rise upwards", "anastellō"),
        "G6312": ("ἀνάστημα", "", "a height", "anastēma"),
        "G6313": ("ἀνασχίζω", "", "to rip open", "anaschizō"),
        "G6314": ("ἀνασῴζω", "", "to escape", "anasōzō"),
        "G6315": ("ἀνατιναγμός", "", "violent shaking", "anatinagmos"),
        "G6316": ("ἀνατροπή", "", "a diet", "anatropē"),
        "G6317": ("ἀναφάλαντος", "", "forehead bald", "anaphalantos"),
        "G6318": ("ἀναφαλάντωμα", "", "forehead bald", "anaphalantōma"),
        "G6319": ("ἀναφορά", "", "an offering", "anaphora"),
        "G6320": ("ἀναφορεύς", "", "a bearing pole", "anaphoreus"),
        "G6321": ("ἀναφράσσω", "", "to block up", "anaphrassō"),
        "G6322": ("ἀναφυράω", "", "to mix up", "anaphuraō"),
        "G6323": ("ἀναφύω", "", "to grow up", "anaphuō"),
        "G6324": ("ἀναψυχή", "", "respite", "anapsuchē"),
        "G6325": ("ἀνδραγαθία", "", "valor", "andragathia"),
        "G6326": ("ἀνδρεία", "", "courage", "andreia"),
        "G6327": ("ἀνδρεῖος", "", "vigorous", "andreios"),
        "G6328": ("ἀνδρόγυνος", "", "effeminate", "androgunos"),
        "G6329": ("ἀνδρόω", "G:V", "to become a man", "androō"),
        "G6330": ("ἀνειλέω", "", "to unroll", "aneileō"),
        "G6331": ("ἀνελεημόνως", "", "mercilessly", "aneleēmonōs"),
        "G6332": ("ἀνέλπιστος", "", "unhoped for", "anelpistos"),
        "G6333": ("ἀνεμοφθορία", "", "wind-blown", "anemophthoria"),
        "G6334": ("ἀνεμόφθορος", "", "destroyed by wind", "anemophthoros"),
        "G6335": ("ἀνεξέλεγκτος", "", "unascertained", "anexelegktos"),
        "G6336": ("ἀνήκοος", "", "unhearing", "anēkoos"),
        "G6337": ("ἀνήλατος", "", "not malleable", "anēlatos"),
        "G6338": ("ἀνθ᾽ οὗ", "", "because of", "anth᾽ ohu"),
        "G6339": ("ἀνθ᾽ ὧν", "", "because of", "anth᾽ hōn"),
        "G6340": ("ἀνθέμιον", "", "a flower ornament", "anthemion"),
        "G6341": ("ἀνθέω", "", "to bloom", "antheō"),
        "G6342": ("ἄνθινος", "", "a flowered work", "anthinos"),
        "G6343": ("ἀνθομολόγησις", "", "confession", "anthomologēsis"),
        "G6344": ("ἀνθυφαιρέω", "", "to deduct", "anthuphaireō"),
        "G6345": ("ἀνίατος", "", "incurable", "aniatos"),
        "G6346": ("ἀνίπταμαι", "", "to fly about", "aniptamai"),
        "G6347": ("ἄνισχυς", "", "without strength", "anischus"),
        "G6348": ("ἄνοιγμα", "", "a passage", "anoigma"),
        "G6349": ("ἀνομέω", "", "to act lawlessly", "anomeō"),
        "G6350": ("ἀνόμημα", "", "a violation of law", "anomēma"),
        "G6351": ("ἀνορύσσω", "", "to root up", "anorussō"),
        "G6352": ("ἄνους", "", "mindless", "anous"),
        "G6353": ("ἀνταίρω", "", "to rise against", "antairō"),
        "G6354": ("ἀντακούω", "", "to listen in turn", "antakouō"),
        "G6355": ("ἀνταλλάσσω", "", "to bargain", "antallassō"),
        "G6356": ("ἀντάμειψις", "", "remuneration", "antameipsis"),
        "G6357": ("ἀνταναιρέω", "", "to take away in return", "antanaireō"),
        "G6358": ("ἀνταποθνήσκω", "", "to  die for", "antapothnēskō"),
        "G6359": ("ἀνταπόκρισις", "", "an answer", "antapokrisis"),
        "G6360": ("ἀντίγραφος", "", "a copy", "antigraphos"),
        "G6361": ("ἀντιδίδωμι", "", "to repay", "antididōmi"),
        "G6362": ("ἀντιδικέω", "", "to plead", "antidikeō"),
        "G6363": ("ἀντίζηλος", "", "a rival", "antizēlos"),
        "G6364": ("ἀντίθετος", "", "contrary", "antithetos"),
        "G6365": ("ἀντικαθίζομαι", "", "to relocate", "antikathizomai"),
        "G6366": ("ἀντικρίνω", "", "to adjudicate", "antikrinō"),
        "G6367": ("ἀντιλήπτωρ", "", "a shielder", "antilēptōr"),
        "G6368": ("ἀντίον", "", "a beam of a loom", "antion"),
        "G6369": ("ἀντιποιέω", "", "to act against", "antipoieō"),
        "G6370": ("ἀντιπολεμέω", "", "to battle", "antipolemeō"),
        "G6371": ("ἀντιπρόσωπος", "", "facing", "antiprosōpos"),
        "G6372": ("ἀντίρρησις", "", "a contradiction", "antirrēsis"),
        "G6373": ("ἀντιστήριγμα", "", "a support", "antistērigma"),
        "G6374": ("ἀντιστηρίζω", "", "to support", "antistērizō"),
        "G6375": ("ἀντιτίθημι", "", "to substitute", "antitithēmi"),
        "G6376": ("ἄντρον", "", "an inner room", "antron"),
        "G6377": ("ἀνυπόδετος", "", "barefoot", "anupodetos"),
        "G6378": ("ἀνυπομόνητος", "", "unsufferable", "anupomonētos"),
        "G6379": ("ἀνυπόστατος", "", "unsubdued", "anupostatos"),
        "G6380": ("ἀνυψόω", "", "to elevate", "anupsoō"),
        "G6381": ("ἀξιόπιστος", "", "worthy of trust", "axiopistos"),
        "G6382": ("ἀξίωμα", "", "a petition", "axiōma"),
        "G6383": ("ἄξων", "", "an axle", "axōn"),
        "G6384": ("ἀοίκητος", "", "uninhabited", "aoikētos"),
        "G6385": ("ἀορασία", "", "inability to see", "aorasia"),
        "G6386": ("ἀπαγγελία", "", "a report", "apaggelia"),
        "G6387": ("ἀπαγωγή", "", "enslavement", "apagōgē"),
        "G6388": ("ἀπαιδευσία", "", "stupidity", "apaideusia"),
        "G6389": ("ἀπαίτησις", "", "exaction", "apaitēsis"),
        "G6390": ("ἀπαλείφω", "", "to wipe away", "apaleiphō"),
        "G6391": ("ἀπαλλοτρίωσις", "", "alienation", "apallotriōsis"),
        "G6392": ("ἁπαλότης", "", "tenderness", "hapalotēs"),
        "G6393": ("ἁπαλύνω", "", "to make tender", "hapalunō"),
        "G6394": ("ἀπαμαυρόω", "", "to blur", "apamauroō"),
        "G6395": ("ἀπαναίνομαι", "", "to refuse", "apanainomai"),
        "G6396": ("ἀπαναισχυντέω", "", "to cast shame", "apanaischunteō"),
        "G6397": ("ἀπαναλίσκω", "", "to consume", "apanaliskō"),
        "G6398": ("ἀπαντή", "", "meet", "apantē"),
        "G6399": ("ἀπάντημα", "", "a meeting", "apantēma"),
        "G6400": ("ἀπάνωθεν", "", "from on top", "apanōthen"),
        "G6401": ("ἀπαράλλακτος", "", "unalterable", "aparallaktos"),
        "G6402": ("ἄπαρσις", "", "a departure", "aparsis"),
        "G6403": ("ἀπαρτία", "", "chattel", "apartia"),
        "G6404": ("ἀπάρχομαι", "", "to dedicate", "aparchomai"),
        "G6405": ("ἀπέκτασις", "", "spreading out", "apektasis"),
        "G6406": ("ἀπελέκητος", "", "unhewn", "apelekētos"),
        "G6407": ("ἀπελευθερόω", "", "to set free", "apeleutheroō"),
        "G6408": ("ἀπενεόομαι", "", "to silence", "apeneoomai"),
        "G6409": ("ἀπερείδω", "", "to fasten", "apereidō"),
        "G6410": ("ἀπερικάθαρτος", "", "impure", "aperikathartos"),
        "G6411": ("ἀπηλιώτης", "", "east wind", "apēliōtēs"),
        "G6412": ("ἄπιος", "", "pear tree", "apios"),
        "G6413": ("ἄπλαστος", "", "simple", "aplastos"),
        "G6414": ("ἄπληστος", "", "insatiable", "aplēstos"),
        "G6415": ("ἁπλοσύνη", "", "singleness", "haplosunē"),
        "G6416": ("ἁπλόω", "", "to simplify", "haploō"),
        "G6417": ("ἀποβιάζομαι", "", "to repel", "apobiazomai"),
        "G6418": ("ἀπογαλακτίζω", "", "to wean", "apogalaktizō"),
        "G6419": ("ἀπόγονος", "", "a descendant", "apogonos"),
        "G6420": ("ἀποδεσμεύω", "", "to bind up", "apodesmeuō"),
        "G6421": ("ἀπόδεσμος", "", "a bundle", "apodesmos"),
        "G6422": ("ἀποδέω", "", "to tie up", "apodeō"),
        "G6423": ("ἀποδιαιρέω", "", "to divide up", "apodiaireō"),
        "G6424": ("ἀποδιδράσκω", "", "to run away", "apodidraskō"),
        "G6425": ("ἀποδιώκω", "", "to banish", "apodiōkō"),
        "G6426": ("ἀπόδομα", "", "a gift", "apodoma"),
        "G6427": ("ἀπόδοσις", "", "restitution", "apodosis"),
        "G6428": ("ἀποθερίζω", "", "to mow", "apotherizō"),
        "G6429": ("ἀποικεσία", "", "resettlement", "apoikesia"),
        "G6430": ("ἀποικία", "", "resettlement", "apoikia"),
        "G6432": ("ἀποικισμός", "", "a settlement", "apoikismos"),
        "G6433": ("ἀποίχομαι", "", "to move", "apoichomai"),
        "G6434": ("ἀποκαθαίρω", "", "to clear away", "apokathairō"),
        "G6435": ("ἀποκαθαρίζω", "", "to cleanse", "apokatharizō"),
        "G6436": ("ἀποκάθημαι", "", "to sit apart", "apokathēmai"),
        "G6437": ("ἀποκακέω", "", "to suffer", "apokakeō"),
        "G6438": ("ἀποκαλέω", "", "to invoke", "apokaleō"),
        "G6439": ("ἀποκεντέω", "", "to pierce", "apokenteō"),
        "G6440": ("ἀποκέντησις", "", "a piercing", "apokentēsis"),
        "G6441": ("ἀποκιδαρόω", "G:V", "be without a turban", "apokidaroō"),
        "G6442": ("ἀποικίζω", "", "to exile", "apoikizō"),
        "G6443": ("ἀποκλαίω", "", "to weep", "apoklaiō"),
        "G6444": ("ἀπόκλεισμα", "", "a prison", "apokleisma"),
        "G6445": ("ἀποκλίνω", "", "to turn off", "apoklinō"),
        "G6446": ("ἀποκλύζω", "", "to rinse off", "apokluzō"),
        "G6447": ("ἀποκνίζω", "", "to pluck off", "apoknizō"),
        "G6448": ("ἀποκομίζω", "", "to transmit", "apokomizō"),
        "G6449": ("ἀποκρυβή", "", "concealment", "apokrubē"),
        "G6450": ("ἀποκρυφή", "", "concealment", "apokruphē"),
        "G6451": ("ἀποκωλύω", "", "to detain", "apokōluō"),
        "G6452": ("ἀποκωφόομαι", "", "be mute", "apokōphoomai"),
        "G6453": ("ἀπολακτίζω", "", "to kick up", "apolaktizō"),
        "G6454": ("ἀπολαύω", "", "to enjoy", "apolauō"),
        "G6455": ("ἀπολέγω", "", "to resign", "apolegō"),
        "G6456": ("ἀπολιθόω", "", "to petrify", "apolithoō"),
        "G6457": ("ἀπολόγημα", "", "a plea", "apologēma"),
        "G6458": ("ἀπόλοιπος", "", "left over", "apoloipos"),
        "G6459": ("ἀπολυτρόω", "G:V", "to redeem", "apolutroō"),
        "G6460": ("ἀπομέμφομαι", "", "to blame", "apomemphomai"),
        "G6461": ("ἀποξενόω", "", "be estranged", "apoxenoō"),
        "G6462": ("ἀποξηραίνω", "", "to cause to dry up", "apoxērainō"),
        "G6463": ("ἀποξύω", "", "to scrape off", "apoxuō"),
        "G6464": ("ἀποπειράομαι", "", "to put to the test", "apopeiraomai"),
        "G6465": ("ἀποπεμπτόω", "", "to take a fifth part", "apopemptoō"),
        "G6466": ("ἀποπηδάω", "", "to leap back", "apopēdaō"),
        "G6467": ("αποπιάζω", "", "to squeeze out", "apopiazō"),
        "G6468": ("ἀποπλάνησις", "", "digression", "apoplanēsis"),
        "G6469": ("ἀποποιέω", "", "to undo", "apopoieō"),
        "G6470": ("ἀποπομπαῖος", "", "a scapegoat", "apopompaios"),
        "G6471": ("ἀποπομπή", "", "acting as scapegoat", "apopompē"),
        "G6472": ("ἀπορρέω", "", "to drop down", "aporreō"),
        "G6473": ("απορρήσω", "", "to rip off", "aporrēsō"),
        "G6474": ("ἀποσάττω", "", "to unharness", "aposattō"),
        "G6475": ("ἀποσβέννυμι", "", "to consume", "aposbennumi"),
        "G6476": ("ἀποσείω", "", "to shake off", "aposeiō"),
        "G6477": ("ἀποσιωπάω", "", "to keep silent", "aposiōpaō"),
        "G6478": ("ἀποσκευή", "", "accoutrements", "aposkeuē"),
        "G6479": ("ἀποσκηνόω", "", "to move a tent", "aposkēnoō"),
        "G6480": ("ἀποσκληρύνω", "", "to harden against", "aposklērunō"),
        "G6481": ("ἀποσκοπεύω", "", "to watch over", "aposkopeuō"),
        "G6482": ("ἀποσκορακίζω", "", "to send far away", "aposkorakizō"),
        "G6483": ("ἀποσκορακισμός", "", "dismissal", "aposkorakismos"),
        "G6484": ("ἀποσοβέω", "", "to frighten away", "aposobeō"),
        "G6485": ("ἀπόσπασμα", "", "a broken branch", "apospasma"),
        "G6486": ("ἀποστάζω", "", "to drop from", "apostazō"),
        "G6487": ("ἀποσταλάζω", "", "to trickle down", "apostalazō"),
        "G6488": ("ἀπόστασις", "", "defection", "apostasis"),
        "G6489": ("ἀποστατέω", "", "to defect", "apostateō"),
        "G6490": ("ἀποστάτης", "", "a defector", "apostatēs"),
        "G6491": ("ἀποστάτις", "", "a defecting", "apostatis"),
        "G6492": ("ἀποστέργω", "", "to disregard", "apostergō"),
        "G6493": ("ἀποστροφή", "", "rejection", "apostrophē"),
        "G6494": ("ἀποσυνάγω", "", "to cure", "aposunagō"),
        "G6495": ("ἀποσυρίζω", "", "to whistle", "aposurizō"),
        "G6496": ("ἀποσφραγίζω", "", "to seal up", "aposphragizō"),
        "G6497": ("ἀποσφράγισμα", "", "a seal", "aposphragisma"),
        "G6498": ("ἀποσχίζω", "", "to sever", "aposchizō"),
        "G6499": ("ἀποτείνω", "", "to extend", "apoteinō"),
        "G6500": ("ἀποτέμνω", "", "to shred", "apotemnō"),
        "G6501": ("ἀποτηγανίζω", "", "to fry", "apotēganizō"),
        "G6502": ("αποτιννύω", "", "to pay for", "apotinnuō"),
        "G6503": ("ἀποτρέχω", "", "to run from", "apotrechō"),
        "G6504": ("ἀποτροπιάζω", "", "to pacify", "apotropiazō"),
        "G6505": ("ἀποτρυγάω", "", "to harvest", "apotrugaō"),
        "G6506": ("ἀποτυγχάνω", "", "to miss", "apotugchanō"),
        "G6507": ("ἀποτύφλωσις", "", "blindness", "apotuphlōsis"),
        "G6508": ("ἀποφαίνω", "", "to sentence", "apophainō"),
        "G6509": ("ἀπόφθεγμα", "", "a maxim", "apophthegma"),
        "G6510": ("ἀποφορά", "", "impost", "apophora"),
        "G6511": ("ἀποφράσσω", "", "to shut out", "apophrassō"),
        "G6512": ("ἀποφυσάω", "", "to blow away", "apophusaō"),
        "G6513": ("ἀποχέω", "", "to pour out", "apocheō"),
        "G6514": ("ἀποχώρησις", "", "voidance", "apochōrēsis"),
        "G6515": ("ἄπυρος", "", "unfired", "apuros"),
        "G6516": ("απφουσώθ", "", "for the sick", "apphousōth"),
        "G6517": ("σαφφώθ", "", "silver things", "saphphōth"),
        "G6518": ("ἀπῶρυξ", "", "a layer of vine", "apōrux"),
        "G6519": ("ἄπωσις", "", "repulsion", "apōsis"),
        "G6521": ("Ἀραβώθ", "", "wilderness", "Arabōth"),
        "G6522": ("ἀράομαι", "", "to curse", "araomai"),
        "G6523": ("ἀράχνη", "", "of a spider", "arachnē"),
        "G6524": ("ἀργία", "", "idleness", "argia"),
        "G6525": ("ἀργυροκοπέω", "", "to work silver", "argurokopeō"),
        "G6526": ("ἀργυρώνητος", "", "bought with silver", "argurōnētos"),
        "G6527": ("ἄρδην", "", "entirely", "ardēn"),
        "G6528": ("ἄρθρον", "", "articulation", "arthron"),
        "G6529": ("αριήλ", "", "altar hearth", "ariēl"),
        "G6530": ("αριήλ", "", "a lion-like person", "ariēl"),
        "G6531": ("ἀριθμητός", "", "counted", "arithmētos"),
        "G6532": ("ἀριώθ", "", "herbs", "ariōth"),
        "G6533": ("ἀρκεύθινος", "", "of juniper", "arkeuthinos"),
        "G6534": ("ἄρκευθος", "", "juniper", "arkeuthos"),
        "G6535": ("ἁρμονία", "", "a joint", "harmonia"),
        "G6536": ("ἀροτήρ", "", "a plowman", "arotēr"),
        "G6537": ("ἀροτρίασις", "", "plowing", "arotriasis"),
        "G6538": ("ἀροτρόπους", "", "a plowshare", "arotropous"),
        "G6539": ("ἄρουρα", "", "plowed fields", "aroura"),
        "G6540": ("ἅρπαγμα", "", "a prey", "harpagma"),
        "G6541": ("ἄρριζος", "", "rootless", "arrizos"),
        "G6542": ("ἀρρωστέω", "G:V", "be ill", "arrōsteō"),
        "G6543": ("ἀρρωστία", "", "illness", "arrōstia"),
        "G6544": ("ἀρσενικός", "", "a male", "arsenikos"),
        "G6545": ("ἄρσις", "", "a load", "arsis"),
        "G6546": ("ἀρτάβη", "", "a measure", "artabē"),
        "G6547": ("ἀρτήρ", "", "a shoulder", "artēr"),
        "G6548": ("ἀρτίως", "", "just now", "artiōs"),
        "G6549": ("ἀρτοκοπικός", "", "of a baker", "artokopikos"),
        "G6550": ("ἀρχιδεσμοφύλαξ", "", "chief jailer", "archidesmophulax"),
        "G6551": ("ἀρχιδεσμώτης", "", "chief jailer", "archidesmōtēs"),
        "G6552": ("ἀρχιεταῖρος", "", "chief friend", "archietairos"),
        "G6553": ("ἀρχιευνοῦχος", "", "chief eunuch", "archieunouchos"),
        "G6554": ("ἀρχιμάγειρος", "", "chief guard", "archimageiros"),
        "G6555": ("ἀρχιοινοχοΐα", "", "chief wine taster", "archioinochoia"),
        "G6556": ("ἀρχιοινοχόος", "", "chief wine taster", "archioinochoos"),
        "G6557": ("ἀρχιπατριώτης", "", "chief patriarch", "archipatriōtēs"),
        "G6558": ("ἀρχισιτοποιός", "", "chief baker", "archisitopoios"),
        "G6559": ("ἀρχιστράτηγος", "", "commander-in-chief", "archistratēgos"),
        "G6560": ("ἀρχισωματοφύλαξ", "", "chief guard", "archisōmatophulax"),
        "G6561": ("ἀρχιτεκτονέω", "G:V", "construction chief", "architektoneō"),
        "G6562": ("ἀρχιτεκτονία", "", "construction", "architektonia"),
        "G6563": ("ἀρχίφυλος", "", "chief of a tribe", "archiphulos"),
        "G6564": ("ἀρχός", "", "chief", "archos"),
        "G6565": ("ασαφίμ", "", "collection", "asaphim"),
        "G6566": ("ἀσβόλη", "", "lamp black", "asbolē"),
        "G6567": ("ἀσέβημα", "", "an act of impiety", "asebēma"),
        "G6568": ("ἄσηπτος", "", "incorruptible", "asēptos"),
        "G6569": ("ἀσίδα", "", "a stork", "asida"),
        "G6570": ("ἀσιτέω", "G:V", "to abstain", "asiteō"),
        "G6571": ("ᾆσμα", "", "a song", "asma"),
        "G6572": ("ἀσπάλαξ", "", "a mole", "aspalax"),
        "G6573": ("ἀσπιδίσκη", "", "a bezel", "aspidiskē"),
        "G6574": ("ἄστεγος", "", "homeless", "astegos"),
        "G6575": ("ἀστράγαλος", "", "a vertebrae", "astragalos"),
        "G6576": ("ἀστρολόγος", "", "astrologer", "astrologos"),
        "G6577": ("ἄσυλος", "", "against reprisal", "asulos"),
        "G6578": ("ἀσύμφορος", "", "hurtful", "asumphoros"),
        "G6579": ("ἀσυνετέω", "", "to lack sense", "asuneteō"),
        "G6580": ("ἀσυνθεσία", "", "breach-of-contract", "asunthesia"),
        "G6581": ("ἀσυνθετέω", "", "to break a contract", "asuntheteō"),
        "G6582": ("ἀσφαλτόπισσα", "", "tar", "asphaltopissa"),
        "G6583": ("ἄσφαλτος", "", "tar", "asphaltos"),
        "G6584": ("ἀσφαλτόω", "", "to apply tar", "asphaltoō"),
        "G6585": ("ἄσωτος", "", "carnal", "asōtos"),
        "G6586": ("ἀτάρ", "", "nevertheless", "atar"),
        "G6587": ("ἀτείχιστος", "", "unwalled", "ateichistos"),
        "G6588": ("ἀτεκνία", "", "childlessness", "ateknia"),
        "G6589": ("ἀτεκνόω", "G:V", "be barren", "ateknoō"),
        "G6590": ("ἀτιμώρητος", "", "unpunished", "atimōrētos"),
        "G6591": ("ἄτρακτος", "", "a spindle", "atraktos"),
        "G6592": ("ἀτραπός", "", "a short cut", "atrapos"),
        "G6593": ("ἄτρυγος", "", "pure", "atrugos"),
        "G6594": ("ἀττάκης", "", "a hopping locust", "attakēs"),
        "G6595": ("ἀττέλαβος", "", "a small locust", "attelabos"),
        "G6596": ("ἀτυχέω", "G:V", "be unfortunate", "atucheō"),
        "G6597": ("αὔγασμα", "", "a shining", "augasma"),
        "G6598": ("αὐγέω", "", "to shine", "augeō"),
        "G6599": ("αὐθαδίᾳ", "", "self-sufficiency", "authadia"),
        "G6600": ("αὐθημερινός", "", "daily", "authēmerinos"),
        "G6601": ("αὐθήμερος", "", "daily", "authēmeros"),
        "G6602": ("αὐλαία", "", "a curtain", "aulaia"),
        "G6603": ("αὖλαξ", "", "a furrow", "aulax"),
        "G6604": ("αὐλάρχης", "", "chief of the palace", "aularchēs"),
        "G6605": ("αὐλών", "", "a canyon", "aulōn"),
        "G6606": ("αὔρα", "", "a breeze", "aura"),
        "G6607": ("αὐταρκέω", "", "to suffice", "autarkeō"),
        "G6608": ("αὐτόθι", "", "at that time", "autothi"),
        "G6609": ("αὐτομολέω", "", "to desert", "automoleō"),
        "G6611": ("αὐτόχθων", "", "native-born", "autochthōn"),
        "G6612": ("αὐχήν", "", "nape", "auchēn"),
        "G6613": ("αὐχμός", "", "squalid", "auchmos"),
        "G6614": ("αὐχμώδης", "", "austere", "auchmōdēs"),
        "G6615": ("ἀφαγνίζω", "", "to purify", "aphagnizō"),
        "G6616": ("ἀφαίρεμα", "", "choice portion", "aphairema"),
        "G6617": ("ἀφάλλομαι", "", "to hop", "aphallomai"),
        "G6618": ("ἀφάπτω", "", "to affix", "aphaptō"),
        "G6619": ("ἄφεδρος", "", "menstruation", "aphedros"),
        "G6620": ("ἀφειδῶς", "", "unsparingly", "apheidōs"),
        "G6621": ("ἀφελῶς", "", "merely", "aphelōs"),
        "G6622": ("ἀφηγέομαι", "", "to guide", "aphēgeomai"),
        "G6623": ("ἄφθορος", "", "uncorrupted", "aphthoros"),
        "G6624": ("ἀφοβία", "", "fearlessness", "aphobia"),
        "G6625": ("ἄφοβος", "", "without fear", "aphobos"),
        "G6626": ("ἀφορία", "", "non-productivity", "aphoria"),
        "G6627": ("ἀφόρισμα", "", "separation", "aphorisma"),
        "G6628": ("ἀφορισμός", "", "an offering", "aphorismos"),
        "G6629": ("ἀφρονέω", "G:V", "be unwise", "aphroneō"),
        "G6630": ("ἀφρόνως", "", "unwisely", "aphronōs"),
        "G6631": ("ἀφυλάκτως", "", "unguardedly", "aphulaktōs"),
        "G6632": ("ἀφυστερέω", "", "to withhold", "aphustereō"),
        "G6633": ("αφφουσώθ", "", "apart", "aphphousōth"),
        "G6634": ("ἀφφώ", "", "now", "aphphō"),
        "G6635": ("ἀχάτης", "", "agate", "achatēs"),
        "G6636": ("ἄχει", "", "reed-grass", "achei"),
        "G6637": ("αχούχ", "", "a thorn-bush", "achouch"),
        "G6638": ("ἀωρία", "", "midnight", "aōria"),
        "G6639": ("ἄωρος", "", "unseasonable", "aōros"),
        "G6640": ("βααλτάμ", "", "master", "baaltam"),
        "G6641": ("βαδδίν", "", "linen clothes", "baddin"),
        "G6642": ("βαδίζω", "", "to proceed", "badizō"),
        "G6643": ("βαθέως", "", "deeply", "batheōs"),
        "G6644": ("βαθύφωνος", "", "deep voiced", "bathuphōnos"),
        "G6645": ("βαθύχειλος", "", "having thick lips", "bathucheilos"),
        "G6646": ("βαίθ", "", "bath (measure)", "baith"),
        "G6647": ("βαίνω", "", "to go", "bainō"),
        "G6648": ("βακτηρία", "", "a staff", "baktēria"),
        "G6649": ("βακχούρια", "", "first-fruit", "bakchouria"),
        "G6650": ("βάλανος", "", "a pin", "balanos"),
        "G6651": ("βάμμα", "", "a dye", "bamma"),
        "G6652": ("βαρ", "", "linen", "bar"),
        "G6653": ("βάραθρον", "", "a pit", "barathron"),
        "G6654": ("βᾶρις", "", "a palace", "baris"),
        "G6655": ("βαρύγλωσσος", "", "an unknown language", "baruglōssos"),
        "G6656": ("βαρυθυμέω", "G:V", "be weighed down", "baruthumeō"),
        "G6657": ("βαρυκάρδιος", "", "heavy-hearted", "barukardios"),
        "G6658": ("βασιλίσκος", "", "cobra", "basiliskos"),
        "G6659": ("βάσκανος", "", "bewitching", "baskanos"),
        "G6660": ("βάσταγμα", "", "a burden", "bastagma"),
        "G6661": ("βδέλλα", "", "leech", "bdella"),
        "G6662": ("βδελυγμός", "", "abomination", "bdelugmos"),
        "G6663": ("βεβαίως", "", "firmly", "bebaiōs"),
        "G6664": ("βεβήλωσις", "", "profaning", "bebēlōsis"),
        "G6665": ("βεδέκ", "", "a breach", "bedek"),
        "G6666": ("βεζέκ", "", "lightning", "bezek"),
        "G6667": ("βελόστασις", "", "weaponry", "belostasis"),
        "G6668": ("βερσεχθάν", "", "a chest", "bersechthan"),
        "G6669": ("βηρύλλιος", "", "beryl", "bērullios"),
        "G6670": ("βιαίως", "", "forcibly", "biaiōs"),
        "G6671": ("βιβάζω", "", "to breed", "bibazō"),
        "G6672": ("βίβλινος", "", "papyrus", "biblinos"),
        "G6673": ("βιβλιοθήκη", "", "a library", "bibliothēkē"),
        "G6674": ("βιβλιοφόρος", "", "courier", "bibliophoros"),
        "G6675": ("βῖκος", "", "a pitcher", "bikos"),
        "G6676": ("βιότης", "", "sustenance", "biotēs"),
        "G6677": ("βιρά", "", "a palace", "bira"),
        "G6678": ("βλέφαρον", "", "eyelid", "blepharon"),
        "G6679": ("βόθρος", "", "a cesspool", "bothros"),
        "G6680": ("βοΐδιον", "", "bullock", "boidion"),
        "G6681": ("βόλβιτον", "", "dung", "bolbiton"),
        "G6682": ("βόλος", "", "a casting", "bolos"),
        "G6683": ("βομβέω", "", "to resonate", "bombeō"),
        "G6684": ("βορά", "", "carrion", "bora"),
        "G6685": ("βόσκημα", "", "pastured", "boskēma"),
        "G6686": ("βόστρυχος", "", "a curl", "bostruchos"),
        "G6687": ("βοτρύδιον", "", "a small cluster of grapes", "botrudion"),
        "G6688": ("βούβαλος", "", "roebuck", "boubalos"),
        "G6689": ("βούκεντρον", "", "an oxgoad", "boukentron"),
        "G6690": ("βουκόλιον", "", "a herd", "boukolion"),
        "G6691": ("βουλευτικός", "", "of a counselor", "bouleutikos"),
        "G6692": ("βουνίζω", "", "to heap up", "bounizō"),
        "G6693": ("βούτομον", "", "flowering rush", "boutomon"),
        "G6694": ("βούτυρον", "", "butter", "bouturon"),
        "G6695": ("βραγχιάω", "", "sore throat", "bragchiaō"),
        "G6696": ("βραδύγλωσσος", "", "slow of tongue", "braduglōssos"),
        "G6697": ("βρόμος", "", "groaning", "bromos"),
        "G6698": ("βροντάω", "", "to thunder", "brontaō"),
        "G6699": ("βροτός", "", "human", "brotos"),
        "G6700": ("βροῦχος", "", "grasshopper", "brouchos"),
        "G6701": ("βρύχημα", "", "a roaring", "bruchēma"),
        "G6702": ("βρωτός", "", "eatable", "brōtos"),
        "G6703": ("βύρσα", "", "a hide", "bursa"),
        "G6704": ("βύω", "", "to plug", "buō"),
        "G6705": ("βῶλαξ", "", "a clod", "bōlax"),
        "G6706": ("βῶλος", "", "a droplet", "bōlos"),
        "G6707": ("γαβίς", "", "crystal", "gabis"),
        "G6708": ("γαζαρηνός", "", "astrologer", "gazarēnos"),
        "G6709": ("γαζοφύλαξ", "", "treasurer", "gazophulax"),
        "G6710": ("γαῖα", "", "land", "gaia"),
        "G6711": ("γαῖσος", "", "a javelin", "gaisos"),
        "G6712": ("γαλαθηνός", "", "suckling", "galathēnos"),
        "G6713": ("γαλεάγρα", "", "a wicker cage", "galeagra"),
        "G6714": ("γαλῆ", "", "weasel", "galē"),
        "G6715": ("γαμβρεύω", "G:V", "to marry", "gambreuō"),
        "G6716": ("γαμβρός", "", "an in-law", "gambros"),
        "G6717": ("γαρέμ", "", "frame", "garem"),
        "G6718": ("γασβαρηνός", "", "a treasurer", "gasbarēnos"),
        "G6719": ("γαυρίαμα", "", "prancing", "gauriama"),
        "G6720": ("γαυριάω", "", "to prance", "gauriaō"),
        "G6721": ("γαυρόω", "", "to prance", "gauroō"),
        "G6722": ("γεδδούρ", "", "a troop", "geddour"),
        "G6723": ("γεῖσος", "", "molding", "geisos"),
        "G6724": ("γειώρας", "", "a foreigner", "geiōras"),
        "G6725": ("γελοιάζω", "", "to joke", "geloiazō"),
        "G6726": ("γελοιασμός", "", "a joke", "geloiasmos"),
        "G6727": ("γελοιαστής", "", "a jokester", "geloiastēs"),
        "G6728": ("γέρας", "", "an honor", "geras"),
        "G6729": ("γεῦμα", "", "taste", "geuma"),
        "G6730": ("γεῦσις", "", "tasting", "geusis"),
        "G6731": ("γέφυρα", "", "a dam", "gephura"),
        "G6732": ("γεωμετρία", "", "a surveying", "geōmetria"),
        "G6733": ("γεωμετρικός", "", "of a surveyor", "geōmetrikos"),
        "G6734": ("γηγενής", "", "earth-born", "gēgenēs"),
        "G6735": ("γίγαρτον", "", "a grape-stone", "gigarton"),
        "G6736": ("γίγας", "", "a giant", "gigas"),
        "G6737": ("γλαύξ", "", "owl", "glaux"),
        "G6738": ("γλυκάζω", "", "to sweeten", "glukazō"),
        "G6739": ("γλυκαίνω", "", "to sweeten", "glukainō"),
        "G6740": ("γλύκασμα", "", "sweetness", "glukasma"),
        "G6741": ("γλυκασμός", "", "sweetness", "glukasmos"),
        "G6742": ("γλυκερός", "", "sweet", "glukeros"),
        "G6743": ("γλυκύτης", "", "sweetness", "glukutēs"),
        "G6744": ("γλύμμα", "", "a carving", "glumma"),
        "G6745": ("γλυπτός", "", "a carving", "gluptos"),
        "G6746": ("γλυφή", "", "a carving", "gluphē"),
        "G6747": ("γλύφω", "", "to carve", "gluphō"),
        "G6748": ("γλωσσότμητος", "", "with a tongue cut out", "glōssotmētos"),
        "G6749": ("γλωσσοχαριτέω", "G:V", "to flatter", "glōssochariteō"),
        "G6750": ("γλωσσώδης", "", "talkative", "glōssōdēs"),
        "G6751": ("γνάθος", "", "a cheek", "gnathos"),
        "G6752": ("γνοφερός", "", "dim", "gnopheros"),
        "G6753": ("γνοφόω", "", "to darken", "gnophoō"),
        "G6754": ("γνοφώδης", "", "dim", "gnophōdēs"),
        "G6755": ("γνώριμος", "", "acquaintance", "gnōrimos"),
        "G6756": ("γνωστῶς", "", "knowingly", "gnōstōs"),
        "G6757": ("γόγγυσις", "", "grumbling", "goggusis"),
        "G6758": ("γομόρ", "", "a homer", "gomor"),
        "G6759": ("γομφιάζω", "", "to have a toothache", "gomphiazō"),
        "G6760": ("γομφιασμός", "", "a toothache", "gomphiasmos"),
        "G6761": ("γονορρυής", "", "a seminal emission", "gonorruēs"),
        "G6762": ("γόνος", "", "semen", "gonos"),
        "G6763": ("γραμματεία", "", "writing", "grammateia"),
        "G6764": ("γραμματεύω", "G:V", "to write", "grammateuō"),
        "G6765": ("γραμματικός", "", "an academic", "grammatikos"),
        "G6766": ("γραμματοεισαγωγεύς", "", "a judicial recorder", "grammatoeisagōgeus"),
        "G6767": ("γραφεῖον", "", "a stylus", "grapheion"),
        "G6768": ("γραφίς", "", "a pen", "graphis"),
        "G6769": ("γρηγόρησις", "", "vigilance", "grēgorēsis"),
        "G6770": ("γρύζω", "", "to growl", "gruzō"),
        "G6771": ("γρύψ", "", "a griffin", "grups"),
        "G6772": ("γυμνόω", "G:V", "be naked", "gumnoō"),
        "G6773": ("γύμνωσις", "", "nakedness", "gumnōsis"),
        "G6774": ("γυναικών", "", "chamber of  women", "gunaikōn"),
        "G6775": ("γύναιον", "", "a helpless woman", "gunaion"),
        "G6776": ("γῦρος", "", "a curve", "guros"),
        "G6777": ("γυρόω", "", "to make a curvature", "guroō"),
        "G6778": ("γύψ", "", "a vulture", "gups"),
        "G6779": ("γωνιαῖος", "", "angular", "gōniaios"),
        "G6780": ("δαβείρ", "", "dabir", "dabeir"),
        "G6781": ("δαλός", "", "a firebrand", "dalos"),
        "G6782": ("δάσος", "", "a thicket", "dasos"),
        "G6783": ("δασύπους", "", "a hare", "dasupous"),
        "G6784": ("δασύς", "", "bushy", "dasus"),
        "G6785": ("δαψιλεύομαι", "", "to worry", "dapsileuomai"),
        "G6786": ("δείδω", "G:V", "be in awe", "deidō"),
        "G6787": ("δείλαιος", "", "wretched", "deilaios"),
        "G6788": ("δείλη", "", "afternoon", "deilē"),
        "G6789": ("δειλιαίνω", "", "to make timid", "deiliainō"),
        "G6790": ("δειλινός", "", "dusk", "deilinos"),
        "G6791": ("δεινός", "", "awful", "deinos"),
        "G6792": ("δεκάδαρχος", "", "a commander of ten", "dekadarchos"),
        "G6793": ("δεκαέξ", "", "sixteen", "dekaex"),
        "G6794": ("δεκαεπτά", "", "seventeen", "dekaepta"),
        "G6795": ("δεκάπηχυς", "", "ten cubits", "dekapēchus"),
        "G6796": ("δεκαπλασίων", "", "ten-times", "dekaplasiōn"),
        "G6797": ("δέκαρχος", "", "a commander of ten", "dekarchos"),
        "G6798": ("δεκατρεῖς", "", "thirteen", "dekatreis"),
        "G6799": ("δεκάχορδος", "", "ten-stringed", "dekachordos"),
        "G6800": ("δεξαμενή", "", "a trough", "dexamenē"),
        "G6801": ("δέρρις", "", "a hide covering", "derris"),
        "G6802": ("δεσπόζω", "G:V", "be master", "despozō"),
        "G6803": ("δεσποτεία", "", "mastery", "despoteia"),
        "G6804": ("δευτερεύω", "G:V", "be second", "deutereuō"),
        "G6805": ("δευτερονόμιον", "", "a second book of the law", "deuteronomion"),
        "G6806": ("δευτερόω", "G:V", "to repeat", "deuteroō"),
        "G6807": ("δευτέρωσις", "", "second rank", "deuterōsis"),
        "G6808": ("δῆγμα", "", "a strike", "dēgma"),
        "G6809": ("δήλωσις", "", "manifestation", "dēlōsis"),
        "G6810": ("διάβασις", "", "a ford", "diabasis"),
        "G6811": ("διάβημα", "", "a footstep", "diabēma"),
        "G6812": ("διαβητίζομαι", "", "to force through", "diabētizomai"),
        "G6813": ("διαβιβάζω", "", "to cause to pass over", "diabibazō"),
        "G6814": ("διαβιόω", "", "to continue to live", "diabioō"),
        "G6815": ("διαβοάω", "", "to proclaim forth", "diaboaō"),
        "G6816": ("διαβολή", "", "distraction", "diabolē"),
        "G6817": ("διαβουλεύω", "", "to deliberate upon", "diabouleuō"),
        "G6818": ("διαβουλία", "", "deliberation", "diaboulia"),
        "G6819": ("διαβούλιον", "", "deliberation", "diaboulion"),
        "G6820": ("διάγγελμα", "", "declaration", "diaggelma"),
        "G6821": ("διαγλύφω", "", "to carve", "diagluphō"),
        "G6822": ("διαγραφή", "", "a diagram", "diagraphē"),
        "G6823": ("διαγράφω", "", "to circumscribe", "diagraphō"),
        "G6824": ("διάδηλος", "", "apparent", "diadēlos"),
        "G6825": ("διαδύνω", "", "to penetrate", "diadunō"),
        "G6826": ("διαθερμαίνω", "", "to warm through", "diathermainō"),
        "G6827": ("διάθεσις", "", "a disposition", "diathesis"),
        "G6828": ("διαθρύπτω", "", "to break through", "diathruptō"),
        "G6829": ("δίαιτα", "", "habitation", "diaita"),
        "G6830": ("διαιτάω", "", "to pass one's life", "diaitaō"),
        "G6831": ("διακαθιζάνω", "", "to sit separately", "diakathizanō"),
        "G6832": ("διακάμπτω", "", "to bend", "diakamptō"),
        "G6833": ("διακενῆς", "", "ineffectual", "diakenēs"),
        "G6834": ("διάκενος", "", "empty", "diakenos"),
        "G6835": ("διακλάω", "", "to snap", "diaklaō"),
        "G6836": ("διακλέπτω", "", "to steal away", "diakleptō"),
        "G6837": ("διακομίζω", "", "to carry across", "diakomizō"),
        "G6838": ("διακοπή", "", "a breach", "diakopē"),
        "G6839": ("διακόπτω", "", "to cut through", "diakoptō"),
        "G6840": ("διακύπτω", "", "to look through", "diakuptō"),
        "G6841": ("διαλανθάνω", "", "to escape notice", "dialanthanō"),
        "G6842": ("διάλευκος", "", "white-mixed", "dialeukos"),
        "G6843": ("διάλλομαι", "", "to leap across", "diallomai"),
        "G6844": ("διαλογή", "", "versification", "dialogē"),
        "G6845": ("διάλυσις", "", "parting", "dialusis"),
        "G6846": ("διαμαρτάνω", "", "to miss entirely", "diamartanō"),
        "G6847": ("διαμαρτυρία", "", "bearing witness", "diamarturia"),
        "G6848": ("διαμετρέω", "", "to measure out", "diametreō"),
        "G6849": ("διαμέτρησις", "", "a measurement", "diametrēsis"),
        "G6850": ("διαναπαύω", "", "to give rest", "dianapauō"),
        "G6851": ("διανήθω", "", "to spin", "dianēthō"),
        "G6852": ("διανθίζω", "", "to decorate", "dianthizō"),
        "G6853": ("διανίστημι", "", "to arise", "dianistēmi"),
        "G6854": ("διανοέω", "", "to consider", "dianoeō"),
        "G6855": ("διανόησις", "", "a device", "dianoēsis"),
        "G6856": ("διαπαρατριβή", "", "useless disputation", "diaparatribē"),
        "G6857": ("διαπαρθενεύω", "", "to lose virginity", "diapartheneuō"),
        "G6858": ("διαπαύω", "", "to discontinue", "diapauō"),
        "G6859": ("διαπειλέω", "", "to threaten", "diapeileō"),
        "G6860": ("διαπέμπω", "", "to spread", "diapempō"),
        "G6861": ("διαπετάννυμι", "", "to open and spread out", "diapetannumi"),
        "G6862": ("διάπηγα", "", "a small pillar", "diapēga"),
        "G6863": ("διαπίπτω", "", "to fail", "diapiptō"),
        "G6864": ("διαπλατύνω", "", "to flatten out", "diaplatunō"),
        "G6865": ("διαπληκτίζομαι", "", "to skirmish", "diaplēktizomai"),
        "G6866": ("διαπνέω", "", "to refresh", "diapneō"),
        "G6867": ("διάπρασις", "", "a sale", "diaprasis"),
        "G6868": ("διάπτωσις", "", "a downfall", "diaptōsis"),
        "G6869": ("διαρπαγή", "", "ravaging", "diarpagē"),
        "G6870": ("διαρραίνω", "", "to sprinkle", "diarrainō"),
        "G6871": ("διαρρίπτω", "", "to scatter", "diarriptō"),
        "G6872": ("διαρτάω", "", "to waver", "diartaō"),
        "G6873": ("διαρτίζω", "", "to mold", "diartizō"),
        "G6874": ("διασαλεύω", "", "to excite", "diasaleuō"),
        "G6875": ("διασάφησις", "", "an explanation", "diasaphēsis"),
        "G6876": ("διασκεδάννυμι", "", "to disperse", "diaskedannumi"),
        "G6877": ("διασκευάζω", "", "to equip", "diaskeuazō"),
        "G6878": ("διασκευή", "", "equipment", "diaskeuē"),
        "G6879": ("διασκορπισμός", "", "dispersing", "diaskorpismos"),
        "G6880": ("δίασμα", "", "a warp", "diasma"),
        "G6881": ("διασπασμός", "", "a tearing in pieces", "diaspasmos"),
        "G6882": ("διαστροφή", "", "perverseness", "diastrophē"),
        "G6883": ("διαστρώννυμι", "", "to spread with carpets", "diastrōnnumi"),
        "G6884": ("διασφαγή", "", "a gap", "diasphagē"),
        "G6885": ("διασχίζω", "", "to cut asunder", "diaschizō"),
        "G6886": ("διάταξις", "", "a disposition", "diataxis"),
        "G6887": ("διατείνω", "", "to extend", "diateinō"),
        "G6888": ("διατήκω", "", "to melt", "diatēkō"),
        "G6889": ("διατήρησις", "", "preservation", "diatērēsis"),
        "G6890": ("διατίλλω", "", "to pluck out", "diatillō"),
        "G6891": ("διατόνιον", "", "a clasp", "diatonion"),
        "G6892": ("διατόρευμα", "", "carvings", "diatoreuma"),
        "G6893": ("διατρέπω", "G:V", "be overawed", "diatrepō"),
        "G6894": ("διατρέφω", "", "to maintain", "diatrephō"),
        "G6895": ("διατρέχω", "", "to run across", "diatrechō"),
        "G6896": ("διατριβή", "", "pastime", "diatribē"),
        "G6897": ("διαυγής", "", "transparent", "diaugēs"),
        "G6898": ("διαφαύσκω", "", "to show light through", "diaphauskō"),
        "G6899": ("διαφλέγω", "", "to burn up", "diaphlegō"),
        "G6900": ("διαφορέω", "", "to disperse", "diaphoreō"),
        "G6901": ("διαφόρημα", "", "dispersion", "diaphorēma"),
        "G6902": ("διαφωνέω", "", "to dissent", "diaphōneō"),
        "G6903": ("διαφωτίζω", "", "to illuminate", "diaphōtizō"),
        "G6904": ("διαχέω", "", "to diffuse", "diacheō"),
        "G6905": ("διαχρίω", "", "to smear all over", "diachriō"),
        "G6906": ("διάχρυσος", "", "interwoven with gold", "diachrusos"),
        "G6907": ("διάχυσις", "", "diffusion", "diachusis"),
        "G6908": ("διάψαλμα", "", "a pause", "diapsalma"),
        "G6909": ("διαψεύδω", "", "to disappoint", "diapseudō"),
        "G6910": ("δίγλωσσος", "", "double-tongued", "diglōssos"),
        "G6911": ("διδυμεύω", "", "to bear twins", "didumeuō"),
        "G6912": ("διεγγυάω", "", "to mortgage", "diegguaō"),
        "G6913": ("διεκβάλλω", "", "to pass out through", "diekballō"),
        "G6914": ("διεκβολή", "", "a mountain pass", "diekbolē"),
        "G6915": ("διελαύνω", "G:V", "to thrust through", "dielaunō"),
        "G6916": ("διελέγχω", "", "to plead", "dielegchō"),
        "G6917": ("διεμβάλλω", "", "to insert", "diemballō"),
        "G6918": ("διεξάγω", "", "to administer", "diexagō"),
        "G6919": ("διεξέρχομαι", "", "to go through", "diexerchomai"),
        "G6920": ("διετηρίς", "", "two whole years", "dietēris"),
        "G6921": ("διευλαβέομαι", "", "to revere", "dieulabeomai"),
        "G6922": ("διήγημα", "", "a tale", "diēgēma"),
        "G6923": ("διηθέω", "", "to refine", "diētheō"),
        "G6924": ("διηλόω", "", "to nail", "diēloō"),
        "G6925": ("δίθυμος", "", "at variance", "dithumos"),
        "G6926": ("διΐημι", "", "to thrust out", "diiēmi"),
        "G6927": ("δικάζω", "", "to act as judge", "dikazō"),
        "G6928": ("δικαστήριον", "", "court of justice", "dikastērion"),
        "G6929": ("δικτυόομαι", "", "lattice work", "diktuoomai"),
        "G6930": ("δικτυωτός", "", "latticed", "diktuōtos"),
        "G6931": ("δίμετρος", "", "a double measure", "dimetros"),
        "G6932": ("δίοδος", "", "a corridor", "diodos"),
        "G6933": ("διοικητής", "", "an administrator", "dioikētēs"),
        "G6934": ("διοικοδομέω", "", "to build", "dioikodomeō"),
        "G6935": ("διόλου", "", "wholly", "diolou"),
        "G6936": ("διοράω", "", "to see clearly", "dioraō"),
        "G6937": ("διορθόω", "", "to set right", "diorthoō"),
        "G6938": ("διορίζω", "", "to determine", "diorizō"),
        "G6939": ("διόρυγμα", "", "a ditch", "diorugma"),
        "G6940": ("δίπηχυς", "", "two cubits", "dipēchus"),
        "G6941": ("διπλασιάζω", "", "to repeat", "diplasiazō"),
        "G6942": ("διπλασιασμός", "", "doubling", "diplasiasmos"),
        "G6943": ("διπλοΐς", "", "a doubled garment", "diplois"),
        "G6944": ("δισσός", "", "double", "dissos"),
        "G6945": ("διυφαίνω", "", "to interweave", "diuphainō"),
        "G6946": ("διφθέρα", "", "a leather covering", "diphthera"),
        "G6947": ("δίφρος", "", "a chair", "diphros"),
        "G6948": ("διχηλέω", "G:V", "be cloven", "dichēleō"),
        "G6949": ("διχοτόμημα", "", "a piece", "dichotomēma"),
        "G6950": ("δίψα", "", "thirst", "dipsa"),
        "G6951": ("διωθέω", "", "to push away", "diōtheō"),
        "G6952": ("διώροφος", "", "second story", "diōrophos"),
        "G6953": ("διῶρυξ", "", "an aqueduct", "diōrux"),
        "G6954": ("διωστήρ", "", "a pole", "diōstēr"),
        "G6955": ("δοκιμαστής", "", "an approver", "dokimastēs"),
        "G6956": ("δόκωσις", "", "a building", "dokōsis"),
        "G6957": ("δολιότης", "", "deceit", "doliotēs"),
        "G6958": ("δολίως", "", "deceitfully", "doliōs"),
        "G6959": ("δόμος", "", "a layer", "domos"),
        "G6960": ("δόξασμα", "", "glory", "doxasma"),
        "G6961": ("δοξαστός", "", "glorious", "doxastos"),
        "G6962": ("δορά", "", "skin", "dora"),
        "G6963": ("δορκάδιον", "", "a young doe", "dorkadion"),
        "G6964": ("δόρκων", "", "a buck", "dorkōn"),
        "G6965": ("δόρυ", "", "a wooden spear", "doru"),
        "G6966": ("δοτός", "", "dedicated", "dotos"),
        "G6967": ("δράγμα", "", "a sheaf", "dragma"),
        "G6968": ("δράξ", "", "a handful", "drax"),
        "G6969": ("δρομεύς", "", "a runner", "dromeus"),
        "G6970": ("δρόσος", "", "dew", "drosos"),
        "G6971": ("δρυμός", "", "a grove", "drumos"),
        "G6972": ("δρῦς", "", "an oak", "drus"),
        "G6973": ("δυναστεία", "", "command", "dunasteia"),
        "G6974": ("κυρεία", "", "command", "kureia"),
        "G6975": ("δυναστεύω", "G:V", "be in power", "dunasteuō"),
        "G6976": ("δυνατῶς", "", "mightily", "dunatōs"),
        "G6977": ("δὺο δύο", "", "two by two", "duo duo"),
        "G6979": ("δυσκολία", "", "discontent", "duskolia"),
        "G6980": ("δύσκωφος", "", "hard of hearing", "duskōphos"),
        "G6981": ("δυστοκέω", "", "to suffer birth pangs", "dustokeō"),
        "G6982": ("δύσχρηστος", "", "inconvenient", "duschrēstos"),
        "G6983": ("δωδεκάμηνος", "", "twelve months", "dōdekamēnos"),
        "G6984": ("δωροδέκτης", "", "one that takes bribes", "dōrodektēs"),
        "G6985": ("δωρολήπτης", "", "one that receives bribes", "dōrolēptēs"),
        "G6986": ("ἐάνπερ", "G:Cond", "if indeed", "eanper"),
        "G6987": ("ἔαρ", "", "the spring season", "ear"),
        "G6988": ("ἑβδομάς", "", "a period of seven", "hebdomas"),
        "G6989": ("εβδομηκονταδύο", "", "seventy-two", "ebdomēkontaduo"),
        "G6990": ("εβδομηκονταεπτά", "", "seventy-seven", "ebdomēkontaepta"),
        "G6991": ("εβδομηκονταπέντε", "", "seventy-five", "ebdomēkontapente"),
        "G6992": ("εβδομηκοντατέσσαρες", "", "seventy-four", "ebdomēkontatessares"),
        "G6993": ("εβδομηκοντατρεῖς", "", "seventy-three", "ebdomēkontatreis"),
        "G6994": ("ἑβδομηκοστός", "", "seventieth", "hebdomēkostos"),
        "G6995": ("ἐγγαστρίμυθος", "", "one who delivers oracles", "eggastrimuthos"),
        "G6996": ("εγγελά", "", "a heifer", "eggela"),
        "G6997": ("ἐγγλύφω", "", "to engrave upon", "eggluphō"),
        "G6998": ("ἔγγραπτος", "", "written", "eggraptos"),
        "G6999": ("ἐγγυάω", "", "to guarantee a loan", "egguaō"),
        "G7000": ("ἐγγύη", "", "surety", "egguē"),
        "G7001": ("ἐγγύθεν", "", "near to", "egguthen"),
        "G7002": ("ἐγκάθημαι", "", "to lie in wait", "egkathēmai"),
        "G7003": ("ἐγκαθίζω", "", "to sit in place", "egkathizō"),
        "G7004": ("ἐγκαινισμός", "", "dedication", "egkainismos"),
        "G7005": ("ἐγκαίνωσις", "", "dedication", "egkainōsis"),
        "G7006": ("ἐγκαλύπτω", "", "to cover up", "egkaluptō"),
        "G7007": ("ἔγκαρπος", "", "containing fruit", "egkarpos"),
        "G7008": ("ἔγκατα", "", "insides", "egkata"),
        "G7009": ("ἐγκατακρύπτω", "", "to hide in", "egkatakruptō"),
        "G7010": ("ἐγκατάλειμμα", "", "leftover, surplus", "egkataleimma"),
        "G7011": ("ἐγκαταλιμπάνω", "", "to abandon", "egkatalimpanō"),
        "G7012": ("ἐγκαταπαίζω", "", "to mock", "egkatapaizō"),
        "G7013": ("ἐγκαυχάομαι", "", "to boast of", "egkauchaomai"),
        "G7014": ("ἔγκειμαι", "", "to cling", "egkeimai"),
        "G7015": ("ἐγκισσάω", "", "to stimulate", "egkissaō"),
        "G7016": ("ἐγκλείω", "", "to lock up", "egkleiō"),
        "G7017": ("ἔγκληρος", "", "hereditary", "egklēros"),
        "G7018": ("ἐγκλοιόω", "", "to bind with a cord", "egkloioō"),
        "G7019": ("ἐγκοίλια", "", "the intestines", "egkoilia"),
        "G7020": ("ἔγκοιλος", "", "imbedded under", "egkoilos"),
        "G7021": ("ἐγκολαπτός", "", "sculptured", "egkolaptos"),
        "G7022": ("ἐγκολάπτω", "", "to sculpt", "egkolaptō"),
        "G7023": ("ἐγκολλάω", "", "to join", "egkollaō"),
        "G7024": ("ἔγκοπος", "", "wearied", "egkopos"),
        "G7025": ("ἐγκοτέω", "G:V", "be angry", "egkoteō"),
        "G7026": ("ἐγκότημα", "", "an object of anger", "egkotēma"),
        "G7027": ("ἐγκρατέω", "", "to have power over", "egkrateō"),
        "G7028": ("ἐγκρίς", "", "a pastry", "egkris"),
        "G7029": ("ἐγκρούω", "", "to hammer in", "egkrouō"),
        "G7030": ("ἐγκρυφίας", "", "cakes", "egkruphias"),
        "G7031": ("ἔγκτητος", "", "procured", "egktētos"),
        "G7032": ("ἐγκυλίω", "", "to wrap up in", "egkuliō"),
        "G7033": ("ἐγκωμιάζω", "", "to laud", "egkōmiazō"),
        "G7034": ("ἐγκώμιον", "", "a commendation", "egkōmion"),
        "G7035": ("ἐγρήγορος", "", "a watchman", "egrēgoros"),
        "G7036": ("ἐγχειρέω", "", "to take in hand", "egcheireō"),
        "G7037": ("ἐγχείρημα", "", "an enterprise", "egcheirēma"),
        "G7038": ("ἐγχειρίδιος", "", "a knife", "egcheiridios"),
        "G7039": ("ἐγχειρίζω", "", "to commit to", "egcheirizō"),
        "G7040": ("ἐγχέω", "", "to pour", "egcheō"),
        "G7041": ("ἐγχρονίζω", "", "to linger", "egchronizō"),
        "G7042": ("ἐγχώριος", "", "native inhabitant", "egchōrios"),
        "G7043": ("ἔδεσμα", "", "food", "edesma"),
        "G7044": ("ἔδνα", "", "a dowry", "edna"),
        "G7045": ("ἕδρα", "", "buttock", "hedra"),
        "G7046": ("ἑδράζω", "", "to seat", "hedrazō"),
        "G7047": ("ἐθισμός", "", "a custom", "ethismos"),
        "G7048": ("εἰ δὲ καί", "", "but if even", "ei de kai"),
        "G7049": ("εἰ μήν", "", "is it not?", "ei mēn"),
        "G7050": ("εἰκάζω", "", "to imagine", "eikazō"),
        "G7051": ("εἰκάς", "", "the twentieth day", "eikas"),
        "G7052": ("εἰκοσαετής", "", "of twenty years", "eikosaetēs"),
        "G7053": ("εικοσιδύο", "", "twenty-two", "eikosiduo"),
        "G7054": ("εἰκοσιείς", "", "twenty-one", "eikosieis"),
        "G7055": ("εἰκοσιεννέα", "", "twenty-nine", "eikosiennea"),
        "G7056": ("εἰκοσιέξ", "", "twenty-six", "eikosiex"),
        "G7057": ("εἰκοσιεπτά", "", "twenty-seven", "eikosiepta"),
        "G7058": ("εἰκοσιοκτώ", "", "twenty-eight", "eikosioktō"),
        "G7059": ("εἰκοσιπέντε", "", "twenty-five", "eikosipente"),
        "G7060": ("εἰκοσιτέσσαρες", "", "twenty-four", "eikositessares"),
        "G7061": ("εἰκοσιτρεῖς", "", "twenty-three", "eikositreis"),
        "G7062": ("εἱλέω", "", "to wrap", "ehileō"),
        "G7063": ("εἶμι", "", "to come", "eimi"),
        "G7064": ("εἰρ", "", "a sentinel", "eir"),
        "G7065": ("εἰσάπαξ", "", "at once", "eisapax"),
        "G7066": ("εἰσβλέπω", "", "to look in", "eisblepō"),
        "G7067": ("εἰσδεκτός", "", "acceptable", "eisdektos"),
        "G7068": ("εισδύω", "G:V", "to enter into", "eisduō"),
        "G7069": ("εἰσοδιάζω", "", "income", "eisodiazō"),
        "G7070": ("εἰσόδιος", "", "an entrance", "eisodios"),
        "G7071": ("εἰσσπάομαι", "", "to draw into", "eisspaomai"),
        "G7072": ("εἰσφορά", "", "a contribution", "eisphora"),
        "G7073": ("ἑκάτερος", "", "to each other", "hekateros"),
        "G7074": ("ἑκατονταπλασίως", "", "a hundred foldly", "hekatontaplasiōs"),
        "G7075": ("ἑκατοντάς", "", "a hundred", "hekatontas"),
        "G7076": ("ἑκατοστεύω", "", "to bear a hundred", "hekatosteuō"),
        "G7078": ("ἐκβιάζω", "", "to expel", "ekbiazō"),
        "G7079": ("ἐκβλαστάνω", "", "to sprout forth", "ekblastanō"),
        "G7080": ("ἐκβλύζω", "", "to gush out", "ekbluzō"),
        "G7081": ("ἐκβράζω", "", "to throw out", "ekbrazō"),
        "G7082": ("ἐκβρασμός", "", "an eruption", "ekbrasmos"),
        "G7083": ("ἐκγελάω", "", "to laugh out loud", "ekgelaō"),
        "G7084": ("ἐκγράφω", "", "to write out", "ekgraphō"),
        "G7085": ("ἐκδανείζω", "", "to lend with interest", "ekdaneizō"),
        "G7086": ("ἐκδέρω", "", "to flay", "ekderō"),
        "G7087": ("ἐκδέω", "", "to suspend", "ekdeō"),
        "G7088": ("ἐκδιδύσκω", "", "to strip", "ekdiduskō"),
        "G7089": ("ἐκδικάζω", "", "to avenge", "ekdikazō"),
        "G7090": ("ἐκδικητής", "", "avenger", "ekdikētēs"),
        "G7091": ("ἐκζέω", "", "to erupt", "ekzeō"),
        "G7092": ("ἔκθεμα", "", "public notice", "ekthema"),
        "G7093": ("ἐκθερίζω", "", "to reap up", "ektherizō"),
        "G7094": ("ἐκθηλάζω", "", "to suck out", "ekthēlazō"),
        "G7095": ("ἐκθλίβω", "", "to squeeze out", "ekthlibō"),
        "G7096": ("ἐκκαθαρίζω", "", "to clear out", "ekkatharizō"),
        "G7097": ("ἑκκαίδεκα", "", "sixteen", "hekkaideka"),
        "G7098": ("ἑκκαιδέκατος", "", "sixteenth", "hekkaidekatos"),
        "G7099": ("ἐκκαλέω", "", "to call forth", "ekkaleō"),
        "G7100": ("ἐκκενόω", "", "to empty out", "ekkenoō"),
        "G7101": ("ἐκκήρυκτος", "", "banished", "ekkēruktos"),
        "G7102": ("ἐκκλησιάζω", "", "to hold an assembly", "ekklēsiazō"),
        "G7103": ("ἐκκλησιαστής", "", "assemblyman", "ekklēsiastēs"),
        "G7104": ("ἐκκλύζω", "", "to wash out", "ekkluzō"),
        "G7105": ("ἐκκόλαμμα", "", "engravement", "ekkolamma"),
        "G7106": ("ἐκκρούω", "", "to knock back", "ekkrouō"),
        "G7107": ("ἐκκύπτω", "", "to look out of", "ekkuptō"),
        "G7108": ("ἐκλαμβάνω", "", "to take out of", "eklambanō"),
        "G7109": ("ἐκλατομέω", "", "to quarry", "eklatomeō"),
        "G7110": ("ἐκλείχω", "", "to lick up", "ekleichō"),
        "G7111": ("ἔκλειψις", "", "end", "ekleipsis"),
        "G7112": ("ἐκλευκαίνω", "", "to whiten", "ekleukainō"),
        "G7113": ("ἔκλευκος", "", "very white", "ekleukos"),
        "G7114": ("ἐκλιμία", "", "craving", "eklimia"),
        "G7115": ("ἐκλογίζομαι", "", "to call into account", "eklogizomai"),
        "G7116": ("ἐκλοχίζω", "", "to select", "eklochizō"),
        "G7117": ("ἔκλυσις", "", "feebleness", "eklusis"),
        "G7118": ("ἐκλύτρωσις", "", "ransoming", "eklutrōsis"),
        "G7119": ("ἐκμαίνω", "", "to drive mad", "ekmainō"),
        "G7120": ("ἐκμετρέω", "", "to measure out", "ekmetreō"),
        "G7121": ("ἐκμιαίνω", "", "to defile", "ekmiainō"),
        "G7122": ("ἐκμυελίζω", "", "to extract marrow", "ekmuelizō"),
        "G7123": ("ἔκνηψις", "", "soberness", "eknēpsis"),
        "G7124": ("ἑκουσιάζομαι", "", "be willing", "hekousiazomai"),
        "G7125": ("ἑκουσιασμός", "", "a voluntary offering", "hekousiasmos"),
        "G7126": ("ἐκπεράω", "", "to carry over", "ekperaō"),
        "G7127": ("ἐκπεριπορεύομαι", "", "to march around", "ekperiporeuomai"),
        "G7128": ("ἐκπιέζω", "", "to pressure", "ekpiezō"),
        "G7129": ("ἐκπικραίνομαι", "", "to embitter greatly", "ekpikrainomai"),
        "G7130": ("ἐκπίνω", "", "to drink up", "ekpinō"),
        "G7131": ("ἔκπληξις", "", "amazement", "ekplēxis"),
        "G7132": ("ἐκπλύνω", "", "to thoroughly wash", "ekplunō"),
        "G7133": ("ἐκποιέω", "", "to furnish", "ekpoieō"),
        "G7134": ("ἐκπολεμέω", "", "to wage war", "ekpolemeō"),
        "G7135": ("ἐκπολιορκέω", "", "to capture a city", "ekpoliorkeō"),
        "G7136": ("ἐκπορθέω", "", "to pillage", "ekportheō"),
        "G7137": ("εκπριόω", "", "to buy off", "ekprioō"),
        "G7138": ("εκρέω", "", "to flow away", "ekreō"),
        "G7139": ("ἔκρηγμα", "", "an inundation", "ekrēgma"),
        "G7140": ("ἐκρήγνυμι", "", "be torn from", "ekrēgnumi"),
        "G7141": ("ἐκρίπτω", "", "to cast forth", "ekriptō"),
        "G7142": ("ἔκρυσις", "", "an outflow", "ekrusis"),
        "G7143": ("ἐκσαρκίζομαι", "", "to tear off flesh", "eksarkizomai"),
        "G7144": ("ἐκσιφωνίζω", "", "to siphon off", "eksiphōnizō"),
        "G7145": ("ἐκσπάω", "", "to pull out", "ekspaō"),
        "G7146": ("ἐκσπερματίζω", "", "to produce offspring", "ekspermatizō"),
        "G7147": ("ἐκστραγγίζω", "", "to strain out", "ekstraggizō"),
        "G7148": ("ἐκστρατεύω", "", "to march", "ekstrateuō"),
        "G7149": ("ἔκτασις", "", "stretching out", "ektasis"),
        "G7150": ("ἐκτάσσω", "", "to array", "ektassō"),
        "G7151": ("ἐκτέμνω", "", "to cut off", "ektemnō"),
        "G7152": ("ἐκτήκω", "", "to waste away", "ektēkō"),
        "G7153": ("ἐκτίκτω", "", "to bring forth", "ektiktō"),
        "G7154": ("ἐκτίλλω", "", "to pluck out", "ektillō"),
        "G7155": ("ἐκτιναγμός", "", "a thrusting out", "ektinagmos"),
        "G7156": ("ἐκτοκίζω", "", "to lend with interest", "ektokizō"),
        "G7157": ("ἐκτομίας", "", "emasculated", "ektomias"),
        "G7158": ("ἐκτρέχω", "", "to run forth", "ektrechō"),
        "G7159": ("ἐκτριβή", "", "obliteration", "ektribē"),
        "G7160": ("ἐκτρίβω", "", "to obliterate", "ektribō"),
        "G7161": ("ἔκτριψις", "", "obliteration", "ektripsis"),
        "G7162": ("ἐκτρυγάω", "", "to gather in grapes", "ektrugaō"),
        "G7163": ("ἐκτρώγω", "", "to chew away", "ektrōgō"),
        "G7164": ("ἐκτυπόω", "", "to shape", "ektupoō"),
        "G7165": ("ἐκτύπωμα", "", "an impression", "ektupōma"),
        "G7166": ("ἐκτύπωσις", "", "an impression", "ektupōsis"),
        "G7167": ("ἐκτυφλόω", "", "to make blind", "ektuphloō"),
        "G7168": ("ἐκφορά", "", "a funeral", "ekphora"),
        "G7169": ("ἐκφόριον", "", "resource", "ekphorion"),
        "G7170": ("ἐκφύρω", "", "to defile", "ekphurō"),
        "G7171": ("ἐκφυσάω", "", "to blow away", "ekphusaō"),
        "G7172": ("ἔκχυσις", "", "an outpouring", "ekchusis"),
        "G7173": ("ἐλάϊνος", "", "of olive", "elainos"),
        "G7174": ("ἐλαιολογέω", "", "to pick olives", "elaiologeō"),
        "G7175": ("ἔλασμα", "", "a hammered piece", "elasma"),
        "G7176": ("ἐλασσόω", "", "to lack", "elassoō"),
        "G7177": ("ἐλάτη", "", "fir tree", "elatē"),
        "G7178": ("ἐλάτινος", "", "of fir", "elatinos"),
        "G7179": ("ἐλατός", "", "hammered out", "elatos"),
        "G7180": ("ἔλαφος", "", "hind", "elaphos"),
        "G7181": ("ἐλέφας", "", "ivory", "elephas"),
        "G7182": ("ἑλικτός", "", "kneaded", "heliktos"),
        "G7183": ("ἕλιξ", "", "a fetter", "helix"),
        "G7184": ("ἔλλειμμα", "", "remains", "elleimma"),
        "G7185": ("ελμωνί", "", "a concealed place", "elmōni"),
        "G7186": ("ἕλος", "", "a marsh", "helos"),
        "G7187": ("ἐμβαθύνω", "", "to deepen", "embathunō"),
        "G7188": ("ἐμβρίμημα", "", "threatening", "embrimēma"),
        "G7189": ("ἔμετος", "", "vomit", "emetos"),
        "G7190": ("ἐμμολύνω", "", "to contaminate", "emmolunō"),
        "G7191": ("ἔμμονος", "", "permanent", "emmonos"),
        "G7192": ("εμμωνείμ", "", "such and such place", "emmōneim"),
        "G7193": ("ἔμπαιγμα", "", "mockery", "empaigma"),
        "G7194": ("ἐμπαραγίνομαι", "", "to come upon", "emparaginomai"),
        "G7195": ("ἐμπήγνυμι", "", "to stick into", "empēgnumi"),
        "G7196": ("ἐμπίμπρημι", "", "to set on fire", "empimprēmi"),
        "G7197": ("ἐμπιστεύω", "", "to entrust", "empisteuō"),
        "G7198": ("ἐμπλατύνω", "", "to widen", "emplatunō"),
        "G7199": ("ἐμπλόκιον", "", "a wreath", "emplokion"),
        "G7200": ("ἔμπνευσις", "", "blasting", "empneusis"),
        "G7201": ("ἐμποδίζω", "", "to impede", "empodizō"),
        "G7202": ("ἐμποδοστατέω", "", "to impede", "empodostateō"),
        "G7203": ("ἐμποιέω", "", "to cause", "empoieō"),
        "G7204": ("ἐμπολάω", "", "to do business", "empolaō"),
        "G7205": ("ἐμπρόσθιος", "", "in front", "emprosthios"),
        "G7206": ("ἔμπτυσμα", "", "spit", "emptusma"),
        "G7207": ("ἐμπυρίζω", "", "to burn", "empurizō"),
        "G7208": ("ἐμπυρισμός", "", "a combustion", "empurismos"),
        "G7209": ("ἔμπυρος", "", "on a fire", "empuros"),
        "G7210": ("ἐμφαίνω", "", "to appear", "emphainō"),
        "G7211": ("ἐμφανῶς", "", "visibly", "emphanōs"),
        "G7212": ("ἐμφραγμός", "", "an obstruction", "emphragmos"),
        "G7213": ("ἐμφράσσω", "", "to obstruct", "emphrassō"),
        "G7214": ("ἐνακούω", "", "to listen to attentively", "enakouō"),
        "G7215": ("ἐναλλάξ", "", "crosswise", "enallax"),
        "G7216": ("ἐνάλλομαι", "", "to assail", "enallomai"),
        "G7217": ("ἐναντιόομαι", "", "to withstand", "enantioomai"),
        "G7218": ("ἔνδεια", "", "a lack", "endeia"),
        "G7219": ("ἐνδελεχισμός", "", "in perpetuity", "endelechismos"),
        "G7220": ("ἐνδελεχῶς", "", "continually", "endelechōs"),
        "G7221": ("ἔνδεσμος", "", "bonding, bundles", "endesmos"),
        "G7222": ("ἐνδέω", "", "to bind in", "endeō"),
        "G7223": ("ἐνδιαβάλλω", "", "to be adversary", "endiaballō"),
        "G7224": ("ἐνδιατρίβω", "", "to spend one's time", "endiatribō"),
        "G7225": ("ἐνδίδωμι", "", "to give forth", "endidōmi"),
        "G7226": ("ἐνδογενής", "", "natural", "endogenēs"),
        "G7227": ("ἔνδοθεν", "", "within", "endothen"),
        "G7228": ("ἔνδον", "", "inside", "endon"),
        "G7229": ("ἐνδόξως", "", "gloriously", "endoxōs"),
        "G7230": ("ἐνδόσθια", "", "entrails", "endosthia"),
        "G7231": ("ἐνεῖδον", "", "to see in", "eneidon"),
        "G7232": ("ἐνείρω", "", "to entwine", "eneirō"),
        "G7233": ("ενενηκονταοκτώ", "", "ninety-eight", "enenēkontaoktō"),
        "G7234": ("ἐνεργός", "", "operative", "energos"),
        "G7235": ("ἐνευφραίνομαι", "", "be pleased with", "eneuphrainomai"),
        "G7236": ("ἐνεχυράζω", "", "to take for security", "enechurazō"),
        "G7237": ("ἐνεχύρασμα", "", "collateral", "enechurasma"),
        "G7238": ("ἐνεχυρασμός", "", "collateral", "enechurasmos"),
        "G7239": ("ἐνέχυρον", "", "a security", "enechuron"),
        "G7240": ("ἒνθα καὶ ἔνθα", "", "here and there", "entha kai entha"),
        "G7241": ("ἔνθεμα", "", "a garland", "enthema"),
        "G7242": ("ἐνθρονίζω", "G:V", "to enthrone", "enthronizō"),
        "G7243": ("ἐνθύμημα", "", "a thought", "enthumēma"),
        "G7244": ("ἐνθύμιος", "", "inner thought", "enthumios"),
        "G7245": ("ἐνιαύσιος", "", "of a year", "eniausios"),
        "G7246": ("ἐννακόσιοι", "", "nine hundred", "ennakosioi"),
        "G7247": ("ἐννεακαίδεκα", "", "nineteen", "enneakaideka"),
        "G7248": ("ἐννεακαιδέκατος", "", "nineteenth", "enneakaidekatos"),
        "G7249": ("ἐννενήκοντα", "", "ninety", "ennenēkonta"),
        "G7250": ("εννενηκονταδύο", "", "ninety-two", "ennenēkontaduo"),
        "G7251": ("εννενηκονταέξ", "", "ninety-six", "ennenēkontaex"),
        "G7252": ("ενενηκονταπέντε", "", "ninety-five", "enenēkontapente"),
        "G7253": ("ἐννοέω", "", "to reflect on", "ennoeō"),
        "G7254": ("ἐννοσσεύω, ἐννεοσσεύω", "", "to nest", "ennosseuō, enneosseuō"),
        "G7255": ("ἐνοικειόω", "G:V", "be related", "enoikeioō"),
        "G7256": ("ἐνοπλίζω", "", "to arm oneself", "enoplizō"),
        "G7257": ("ἔνοπλος", "", "armed", "enoplos"),
        "G7258": ("ἐνόρκιος", "", "affirmation", "enorkios"),
        "G7259": ("ἔνορκος", "", "to bind by an oath", "enorkos"),
        "G7260": ("ἐνσιτέομαι", "", "to feed upon", "ensiteomai"),
        "G7261": ("ἐνσκολιεύομαι", "", "to catch in a snare", "enskolieuomai"),
        "G7262": ("ἐντάσσω", "", "to arrange in", "entassō"),
        "G7263": ("ἐνταῦθα", "", "here", "entautha"),
        "G7264": ("ἐνταφιαστής", "", "an embalmer", "entaphiastēs"),
        "G7265": ("ἐντείνω", "", "to stretch tight", "enteinō"),
        "G7266": ("ἔντερον", "", "innards", "enteron"),
        "G7267": ("ἐντήκω", "", "to melt away", "entēkō"),
        "G7268": ("ἐντίθημι", "", "to insert", "entithēmi"),
        "G7269": ("ἐντιμάω", "", "to value", "entimaō"),
        "G7270": ("ἐντίμως", "", "highly valued", "entimōs"),
        "G7271": ("ἐντομίς", "", "a cut", "entomis"),
        "G7272": ("ἔντριτος", "", "three-stranded", "entritos"),
        "G7273": ("ἐντρύφημα", "", "amusement", "entruphēma"),
        "G7274": ("ἐνυπνιαστής", "", "a dreamer", "enupniastēs"),
        "G7275": ("ἔνυστρον", "", "large intestine", "enustron"),
        "G7276": ("ἐνώπιος", "", "face to face", "enōpios"),
        "G7277": ("ἕνωσις", "", "uniting", "henōsis"),
        "G7278": ("ἐνώτιον", "", "a ring", "enōtion"),
        "G7279": ("ἐξαγορεύω", "", "to confess", "exagoreuō"),
        "G7280": ("ἐξαγριαίνω", "", "to rage furiously", "exagriainō"),
        "G7281": ("ἐξαιρετός", "", "chosen out of", "exairetos"),
        "G7282": ("ἐξαίσιος", "", "extraordinary", "exaisios"),
        "G7283": ("ἑξάκις", "", "six times", "hexakis"),
        "G7284": ("ἑξακισχίλιοι", "", "six thousand", "hexakischilioi"),
        "G7285": ("ἐξακονάω", "", "to make very sharp", "exakonaō"),
        "G7286": ("ἑξακοσιοστός", "", "six hundredth", "hexakosiostos"),
        "G7287": ("ἐξακριβάζω", "", "to determine", "exakribazō"),
        "G7288": ("ἐξάλειπτρον", "", "an ointment jar", "exaleiptron"),
        "G7289": ("ἐξάλειψις", "", "a wiping out", "exaleipsis"),
        "G7290": ("ἐξαλλάσσω", "", "to change", "exallassō"),
        "G7291": ("ἔξαλλος", "", "special", "exallos"),
        "G7292": ("ἐξαμαρτάνω", "", "to lead into sin", "examartanō"),
        "G7293": ("ἑξάμηνος", "", "six months", "hexamēnos"),
        "G7294": ("ἐξαναλίσκω", "", "to  consume", "exanaliskō"),
        "G7295": ("εξεναντίας", "", "right opposite", "exenantias"),
        "G7296": ("ἐξανθέω", "", "to blossom", "exantheō"),
        "G7297": ("ἐξαντλέω", "", "to draw out", "exantleō"),
        "G7298": ("ἐξαπίνης", "", "sudden", "exapinēs"),
        "G7299": ("ἐξάπτω", "", "to light up", "exaptō"),
        "G7300": ("ἐξαριθμέω", "", "to count out", "exarithmeō"),
        "G7301": ("ἐξαρκέω", "G:V", "be enough", "exarkeō"),
        "G7302": ("ἔξαρσις", "", "a departure", "exarsis"),
        "G7303": ("ἐξαρτάω", "G:V", "be attached to", "exartaō"),
        "G7304": ("ἐξάρχω", "", "to originate", "exarchō"),
        "G7305": ("ἐξατιμόομαι", "", "to despise", "exatimoomai"),
        "G7306": ("ἐξέδρα", "", "an inner chamber", "exedra"),
        "G7307": ("ἐξεικονίζω", "", "to completely form", "exeikonizō"),
        "G7308": ("ἐξεκκλησιάζω", "", "to assemble", "exekklēsiazō"),
        "G7309": ("ἐξελαύνω", "", "to launch an expedition", "exelaunō"),
        "G7310": ("ἐξελίσσω", "", "to expand", "exelissō"),
        "G7311": ("ἐξεμέω", "", "to vomit forth", "exemeō"),
        "G7312": ("ἐξεργάζομαι", "", "to complete", "exergazomai"),
        "G7313": ("ἐξερεύγομαι", "", "to discharge forth", "exereugomai"),
        "G7314": ("ἐξερεύνησις", "", "a search", "exereunēsis"),
        "G7315": ("ἐξερημόω", "", "to make desolate", "exerēmoō"),
        "G7316": ("ἐξέρπω", "", "to creep forth", "exerpō"),
        "G7317": ("ἐξετασμός", "", "an inquisition", "exetasmos"),
        "G7318": ("ἐξεύρεσις", "", "a finding out", "exeuresis"),
        "G7319": ("ἐξέχω", "", "to protrude", "exechō"),
        "G7320": ("ἐξήγησις", "", "description", "exēgēsis"),
        "G7321": ("ἐξηγητής", "", "an expositor", "exēgētēs"),
        "G7322": ("ἐξηγορία", "", "an utterance", "exēgoria"),
        "G7323": ("ἑξηκονταδύο", "", "sixty-two", "hexēkontaduo"),
        "G7324": ("εξηκονταέξ", "", "sixty-six", "exēkontaex"),
        "G7325": ("εξηκονταεπτά", "", "sixty-seven", "exēkontaepta"),
        "G7326": ("ἑξηκονταετής", "", "sixty years old", "hexēkontaetēs"),
        "G7327": ("εξηκονταοκτώ", "", "sixty-eight", "exēkontaoktō"),
        "G7328": ("εξηκονταπέντε", "", "sixty-five", "exēkontapente"),
        "G7329": ("ἐξηλιάζω", "", "to hang in the sun", "exēliazō"),
        "G7330": ("ἐξίλασις", "", "atonement", "exilasis"),
        "G7331": ("ἐξιλάσκομαι", "", "to appease", "exilaskomai"),
        "G7332": ("ἐξίλασμα", "", "an appeasement", "exilasma"),
        "G7333": ("ἐξιλασμός", "", "a making an atonement", "exilasmos"),
        "G7334": ("ἐξιππάζομαι", "", "to ride forth", "exippazomai"),
        "G7335": ("ἐξίπταμαι", "", "to flutter away", "exiptamai"),
        "G7336": ("ἐξισόω", "", "to make equal", "exisoō"),
        "G7337": ("ἐξιχνιάζω", "", "to trace", "exichniazō"),
        "G7338": ("ἐξιχνιασμός", "", "a tracking out", "exichniasmos"),
        "G7339": ("ἐξοδία", "", "a departure", "exodia"),
        "G7340": ("ἐξοδιάζω", "", "to spend", "exodiazō"),
        "G7341": ("ἐξόδιος", "", "a holiday recess", "exodios"),
        "G7342": ("ἐξοικοδομέω", "", "to build up", "exoikodomeō"),
        "G7343": ("ἔξοικος", "", "homeless", "exoikos"),
        "G7344": ("ἐξοκέλλω", "", "to lead aground", "exokellō"),
        "G7345": ("ἐξόλλυμι", "", "to totally ruin", "exollumi"),
        "G7346": ("εξολόθρευσις", "", "devastation", "exolothreusis"),
        "G7347": ("ἐξομολόγησις", "", "acknowledgment", "exomologēsis"),
        "G7348": ("ἐξόπισθεν", "", "following", "exopisthen"),
        "G7349": ("ἐξοπλίζω", "", "to completely arm", "exoplizō"),
        "G7350": ("ἐξορμάω", "", "to sally out", "exormaō"),
        "G7351": ("ἐξουδένωμα", "", "contempt", "exoudenōma"),
        "G7352": ("ἐξουδένωσις", "", "contempt", "exoudenōsis"),
        "G7353": ("ἐξουθένημα", "", "a contemptible thing", "exouthenēma"),
        "G7354": ("ἐξυβρίζω", "", "to overflow", "exubrizō"),
        "G7355": ("ἔξωσμα", "", "purgation", "exōsma"),
        "G7356": ("ἐξωτέρω", "", "outer", "exōterō"),
        "G7357": ("ἐπαγωγή", "", "an invasion", "epagōgē"),
        "G7358": ("ἐπάδω", "", "to charm", "epadō"),
        "G7359": ("ἐπακρόασις", "", "heeding", "epakroasis"),
        "G7360": ("ἔπαλξις", "", "a parapet", "epalxis"),
        "G7361": ("ἐπανακαινίζω", "", "to renew", "epanakainizō"),
        "G7362": ("ἐπαναστρέφω", "", "to return", "epanastrephō"),
        "G7363": ("ἐπανατρυγάω", "", "to glean the vintage", "epanatrugaō"),
        "G7364": ("ἐπανήκω", "", "to come back again", "epanēkō"),
        "G7365": ("ἐπανθέω", "", "to bloom again", "epantheō"),
        "G7366": ("ἐπάνωθεν", "", "about", "epanōthen"),
        "G7367": ("ἐπαοιδή", "", "enchantment", "epaoidē"),
        "G7368": ("ἐπαοιδός", "", "an enchanter", "epaoidos"),
        "G7369": ("ἐπαποστέλλω", "", "to send as successor", "epapostellō"),
        "G7370": ("ἔπαρμα", "", "raised elevation", "eparma"),
        "G7371": ("ἔπαρσις", "", "a raising", "eparsis"),
        "G7372": ("ἐπαρυστήρ", "", "a funnel", "eparustēr"),
        "G7373": ("ἐπαρυστρίς", "", "an oil funnel", "eparustris"),
        "G7374": ("ἔπαρχος", "", "a chief ruler", "eparchos"),
        "G7375": ("ἐπαφίημι", "", "to let loose on", "epaphiēmi"),
        "G7376": ("ἔπειμι", "", "be used", "epeimi"),
        "G7377": ("ἔπειμι", "", "coming", "epeimi"),
        "G7378": ("ἐπεισφέρω", "", "to bring in addition", "epeispherō"),
        "G7379": ("ἐπελπίζω", "", "to raise hope", "epelpizō"),
        "G7380": ("ἐπερείδω", "", "to lean upon", "epereidō"),
        "G7381": ("ἐπερώτησις", "", "questioning", "eperōtēsis"),
        "G7382": ("ἐπέτειος", "", "yearly", "epeteios"),
        "G7383": ("ἐπευκτός", "", "longed for", "epeuktos"),
        "G7384": ("ἐπεύχομαι", "", "to invoke", "epeuchomai"),
        "G7385": ("ἐπήκοος", "", "attentive", "epēkoos"),
        "G7386": ("ἐπήλυτος", "", "a stranger", "epēlutos"),
        "G7387": ("ἐπίβασις", "", "a step", "epibasis"),
        "G7388": ("ἐπιβάτης", "", "a crewman", "epibatēs"),
        "G7389": ("ἐπιβόλαιον", "", "a covering", "epibolaion"),
        "G7390": ("ἐπιβουλεύω", "", "to plot against", "epibouleuō"),
        "G7391": ("ἐπίβουλος", "", "a plotter", "epiboulos"),
        "G7392": ("ἐπιβρέχω", "", "to rain upon", "epibrechō"),
        "G7393": ("ἐπιγαμία", "", "related by marriage", "epigamia"),
        "G7394": ("ἐπιγελάω", "", "to laugh at", "epigelaō"),
        "G7395": ("ἐπιγεμίζω", "", "to load upon", "epigemizō"),
        "G7396": ("ἐπιγνωμοσύνη", "", "knowledge", "epignōmosunē"),
        "G7397": ("ἐπιγνώμων", "", "an arbitrator", "epignōmōn"),
        "G7398": ("ἐπίγνωστος", "", "well known", "epignōstos"),
        "G7399": ("ἐπιγονή", "", "a breed", "epigonē"),
        "G7400": ("ἐπιδέκατος", "", "a tithe", "epidekatos"),
        "G7401": ("ἐπιδέξιος", "", "fitting", "epidexios"),
        "G7402": ("ἐπιδέω", "", "to fasten", "epideō"),
        "G7403": ("ἐπιδιπλόω", "", "to double", "epidiploō"),
        "G7404": ("ἐπιδιώκω", "", "to pursue after", "epidiōkō"),
        "G7405": ("ἐπιεικεύομαι", "", "be lenient", "epieikeuomai"),
        "G7406": ("ἐπιεικῶς", "", "please", "epieikōs"),
        "G7407": ("ἐπιζάω", "", "to outlive", "epizaō"),
        "G7408": ("ἐπιζήμιος", "", "a fine", "epizēmios"),
        "G7409": ("ἐπίθεμα", "", "a lid", "epithema"),
        "G7410": ("ἐπιθύμημα", "", "desire", "epithumēma"),
        "G7411": ("ἐπιθυμητός", "", "desirable", "epithumētos"),
        "G7412": ("ἐπιθύω", "", "to sacrifice upon", "epithuō"),
        "G7413": ("ἐπικαταλαμβάνω", "", "to overtake", "epikatalambanō"),
        "G7414": ("ἐπικαταράομαι", "", "to accurse", "epikataraomai"),
        "G7415": ("ἐπίκλητος", "", "selected", "epiklētos"),
        "G7416": ("ἐπικλίνω", "", "to incline", "epiklinō"),
        "G7417": ("ἐπικλύζω", "", "to inundate", "epikluzō"),
        "G7418": ("ἐπικοιμάομαι", "", "to rest upon", "epikoimaomai"),
        "G7419": ("ἐπικοσμέω", "", "to embellish", "epikosmeō"),
        "G7420": ("ἐπικραταιόω", "", "to prevail against", "epikrataioō"),
        "G7421": ("ἐπικρατέω", "", "to prevail", "epikrateō"),
        "G7422": ("ἐπικρεμάννυμι", "", "to hang upon", "epikremannumi"),
        "G7423": ("ἐπικροτέω", "", "to clap", "epikroteō"),
        "G7424": ("ἐπικρούω", "", "to strike", "epikrouō"),
        "G7425": ("ἐπικυλίω", "", "to roll upon", "epikuliō"),
        "G7426": ("ἐπιλάμπω", "", "to shine forth", "epilampō"),
        "G7427": ("ἐπίλεκτος", "", "chosen", "epilektos"),
        "G7428": ("ἐπιληπτεύομαι", "", "be possessed", "epilēpteuomai"),
        "G7429": ("ἐπίληπτος", "", "convulsed", "epilēptos"),
        "G7430": ("ἐπιμίγνυμι", "", "to intermix", "epimignumi"),
        "G7431": ("ἐπίμικτος", "", "intermixed", "epimiktos"),
        "G7432": ("ἐπιμύλιος", "", "an upper millstone", "epimulios"),
        "G7433": ("ἐπινοέω", "", "to think about", "epinoeō"),
        "G7434": ("ἐπινυστάζω", "", "to slumber", "epinustazō"),
        "G7435": ("ἐπιξενόομαι", "", "be welcomed as a guest", "epixenoomai"),
        "G7436": ("ἐπίπεμπτος", "", "a fifth part", "epipemptos"),
        "G7437": ("ἐπιπέμπω", "", "to bring in addition", "epipempō"),
        "G7438": ("ἐπιπολάζω", "", "to float", "epipolazō"),
        "G7439": ("ἐπιρραντίζω", "", "to sprinkle upon", "epirrantizō"),
        "G7440": ("ἐπιρρέω", "", "to overflow", "epirreō"),
        "G7441": ("ἐπίσαγμα", "", "a saddle", "episagma"),
        "G7442": ("ἐπισάττω", "", "to saddle", "episattō"),
        "G7443": ("ἐπισείω", "", "to stir up", "episeiō"),
        "G7444": ("ἐπισημαίνω", "", "to  mark", "episēmainō"),
        "G7445": ("ἐπισκάζω", "", "to limp", "episkazō"),
        "G7446": ("ἐπισκεπάζω", "", "to envelop", "episkepazō"),
        "G7447": ("ἐπισκευάζω", "", "to repair", "episkeuazō"),
        "G7448": ("ἐπίσκεψις", "", "a review", "episkepsis"),
        "G7449": ("ἐπίσπαστρον", "", "a draw curtain", "epispastron"),
        "G7450": ("ἐπισπεύδω", "", "to hurry", "epispeudō"),
        "G7451": ("ἐπισπλαγχνίζομαι", "", "to show compassion", "episplagchnizomai"),
        "G7452": ("ἐπισπουδάζω", "", "to hasten", "epispoudazō"),
        "G7453": ("ἐπισπουδαστής", "", "a taskmaster", "epispoudastēs"),
        "G7454": ("ἐπιστήμη", "", "higher knowledge", "epistēmē"),
        "G7455": ("ἐπιστήριγμα", "", "a stay", "epistērigma"),
        "G7456": ("ἐπιστοιβάζω", "", "to pile up", "epistoibazō"),
        "G7457": ("ἐπιστρατεύω", "", "to march against", "epistrateuō"),
        "G7458": ("ἐπισυνίστημι", "", "to rise up together against", "episunistēmi"),
        "G7459": ("ἐπισυστρέφω", "", "to gather together against", "episustrephō"),
        "G7460": ("ἐπισφραγίζω", "", "to put a seal upon", "episphragizō"),
        "G7461": ("ἐπιτήδευμα", "", "a practice", "epitēdeuma"),
        "G7462": ("ἐπιτηδεύω", "", "to apply", "epitēdeuō"),
        "G7463": ("ἐπιτίμησις", "", "reproach", "epitimēsis"),
        "G7464": ("ἐπιτοαυτό", "", "together", "epitoauto"),
        "G7465": ("ἐπιτρέχω", "", "to run to", "epitrechō"),
        "G7466": ("ἐπιτρίβω", "", "to crush", "epitribō"),
        "G7467": ("ἐπίτριψις", "", "wearing away", "epitripsis"),
        "G7468": ("ἐπιφημίζω", "", "to portend", "epiphēmizō"),
        "G7469": ("ἐπιφυλλίζω", "", "to glean", "epiphullizō"),
        "G7470": ("ἐπιφυλλίς", "", "a gleaning", "epiphullis"),
        "G7471": ("ἐπιχαίρω", "", "to rejoice at", "epichairō"),
        "G7472": ("ἐπιχαρής", "", "gratifying", "epicharēs"),
        "G7473": ("ἐπίχαρμα", "", "derision", "epicharma"),
        "G7474": ("ἐπίχαρτος", "", "incurring ridicule", "epichartos"),
        "G7475": ("ἐπίχειρον", "", "forearm", "epicheiron"),
        "G7476": ("ἐπίχυσις", "", "a vessel", "epichusis"),
        "G7477": ("ἐπιχώρησις", "", "grant", "epichōrēsis"),
        "G7478": ("ἐπόζω", "", "to stink", "epozō"),
        "G7479": ("ἐποίκιον", "", "an estate", "epoikion"),
        "G7480": ("ἐπονείδιστος", "", "reviled", "eponeidistos"),
        "G7481": ("ἔποψ", "", "a hoopoe", "epops"),
        "G7482": ("ἑπταετής", "", "of seven years", "heptaetēs"),
        "G7483": ("ἑπτακαίδεκα", "", "seventeen", "heptakaideka"),
        "G7484": ("ἑπτακαιδέκατος", "", "seventeenth", "heptakaidekatos"),
        "G7485": ("ἑπτακόσιοι", "", "seven hundred", "heptakosioi"),
        "G7486": ("ἑπτάμηνος", "", "of seven months", "heptamēnos"),
        "G7487": ("ἑπταπλάσιος", "", "seven-fold", "heptaplasios, heptaplasiōn"),
        "G7488": ("ἑπταπλασίων", "", "seven-fold", "heptaplasiōn"),
        "G7489": ("ἑπταπλασίως", "", "seven-fold", "heptaplasiōs"),
        "G7490": ("ἐπωμίς", "", "a shoulder-piece", "epōmis"),
        "G7491": ("ἐπωρύω", "", "to roar against", "epōruō"),
        "G7492": ("ἐραστής", "", "a lover", "erastēs"),
        "G7493": ("ἐράω", "", "to love passionately", "eraō"),
        "G7494": ("ἐργάβ", "", "a chest", "ergab"),
        "G7495": ("ἐργαλεῖον", "", "a work tool", "ergaleion"),
        "G7496": ("ἐργάσιμος", "", "a deed", "ergasimos"),
        "G7497": ("ἐργοδιωκτέω", "", "to direct works", "ergodiōkteō"),
        "G7498": ("ἐργοδιώκτης", "", "a foreman", "ergodiōktēs"),
        "G7499": ("ἐρεθισμός", "", "aggravation", "erethismos"),
        "G7500": ("ἐρεθιστής", "", "an irritant", "erethistēs"),
        "G7501": ("ἔρεισμα", "", "a support", "ereisma"),
        "G7502": ("ἐρεοῦς", "", "woolen", "ereous"),
        "G7503": ("ἐρημικός", "", "solitary", "erēmikos"),
        "G7504": ("ἐρημίτης", "", "a recluse", "erēmitēs"),
        "G7505": ("ἔριθος", "", "wool worker", "erithos"),
        "G7506": ("ἐρικτός", "", "ground up", "eriktos"),
        "G7507": ("ἑρμηνευτής", "", "a translator", "hermēneutēs"),
        "G7508": ("ἕρπω", "", "to crawl", "herpō"),
        "G7509": ("ἐρύθημα", "", "red dye", "eruthēma"),
        "G7510": ("ἐρυθροδανόω", "", "to dye red", "eruthrodanoō"),
        "G7511": ("ἐρυσίβη", "", "blight", "erusibē"),
        "G7512": ("ἐρῳδιός", "", "a heron", "erōdios"),
        "G7513": ("ἔρως", "", "passion", "erōs"),
        "G7514": ("ἑσπερινός", "", "during evening", "hesperinos"),
        "G7515": ("ἑστιατορία", "", "feasting", "hestiatoria"),
        "G7516": ("ἐσχάρα", "", "a grate", "eschara"),
        "G7517": ("ἐσχαρίτης", "", "broiled", "escharitēs"),
        "G7518": ("ἐσχατίζω", "G:V", "be late", "eschatizō"),
        "G7519": ("ἐσώτατος", "", "innermost", "esōtatos"),
        "G7520": ("εσώτερον", "", "inside", "esōteron"),
        "G7521": ("ἐτάζω", "", "to chastise", "etazō"),
        "G7522": ("ἑταίρα", "", "a mistress", "hetaira"),
        "G7523": ("ἑταιρία", "", "friendship", "hetairia"),
        "G7524": ("ἔτασις", "", "chastisement", "etasis"),
        "G7525": ("ἐτασμός", "", "chastisement", "etasmos"),
        "G7526": ("ἑτερόζυγος", "", "unequally yoked", "heterozugos"),
        "G7527": ("εὐαγγελία", "", "good news", "euaggelia"),
        "G7528": ("εὐάλωτος", "", "easily caught", "eualōtos"),
        "G7529": ("εὐάρμοστος", "", "well-tuned", "euarmostos"),
        "G7530": ("εὖγε", "G:Part", "well done", "euge"),
        "G7531": ("εὐγένεια", "", "magnanimity", "eugeneia"),
        "G7532": ("εὔγνωστος", "", "well-known", "eugnōstos"),
        "G7533": ("εὐδοκιμέω", "G:V", "be approved", "eudokimeō"),
        "G7534": ("εὐεκτέω", "", "to enjoy good health", "euekteō"),
        "G7535": ("εὔελπις", "", "confident", "euelpis"),
        "G7536": ("εὔζωνος", "", "well-equipped", "euzōnos"),
        "G7537": ("εὐήκοος", "", "heedful", "euēkoos"),
        "G7538": ("εὔηχος", "", "distinct", "euēchos"),
        "G7539": ("εὐθαλέω", "", "to thrive", "euthaleō"),
        "G7540": ("εὐθαλής", "", "flourishing", "euthalēs"),
        "G7541": ("εὐθηνέω", "", "to prosper", "euthēneō"),
        "G7542": ("εὐθηνία", "", "prosperity", "euthēnia"),
        "G7543": ("εὐθής", "", "right", "euthēs"),
        "G7544": ("εὐιλατεύω", "", "to propitiate", "euilateuō"),
        "G7545": ("εὐίλατος", "", "propitious", "euilatos"),
        "G7546": ("εὐκαταφρόνητος", "", "despicable", "eukataphronētos"),
        "G7547": ("εὐκλεής", "", "renowned", "eukleēs"),
        "G7548": ("εὐκληματέω", "", "to have good vine branches", "euklēmateō"),
        "G7549": ("εὔκολος", "", "well-tempered", "eukolos"),
        "G7550": ("εὔλαλος", "", "well-spoken", "eulalos"),
        "G7551": ("εὐμεγέθης", "", "huge", "eumegethēs"),
        "G7552": ("εὐμετάβολος", "", "changeable", "eumetabolos"),
        "G7553": ("εὐμήκης", "", "tall", "eumēkēs"),
        "G7554": ("εὔοδος", "", "prosperous", "euodos"),
        "G7555": ("εὐόδως", "", "prosperously", "euodōs"),
        "G7556": ("εὐπαθέω", "", "to enjoy pleasure", "eupatheō"),
        "G7557": ("εὐπάρυφος", "", "elegantly", "euparuphos"),
        "G7558": ("εὐπρεπής", "", "beautiful", "euprepēs"),
        "G7559": ("εὐπρόσωπος", "", "good-looking", "euprosōpos"),
        "G7560": ("εὑρετής", "", "an inventor", "ehuretēs"),
        "G7561": ("εὕρημα", "", "unexpected gain", "ehurēma"),
        "G7562": ("εὔριζος", "", "well-rooted", "eurizos"),
        "G7563": ("εὖρος", "", "breadth", "euros"),
        "G7564": ("εὐρύς", "", "broad", "eurus"),
        "G7565": ("εὐρωτιάω", "G:V", "be moldy", "eurōtiaō"),
        "G7566": ("εὔσκιος", "", "well-shaded", "euskios"),
        "G7567": ("εὐσταθέω", "G:V", "be stable", "eustatheō"),
        "G7568": ("εὐστόχως", "", "skillfully", "eustochōs"),
        "G7569": ("εὐστροφία", "", "versatility", "eustrophia"),
        "G7570": ("εὐσυναλλάκτως", "", "conciliatory", "eusunallaktōs"),
        "G7571": ("εὐτάκτως", "", "orderly", "eutaktōs"),
        "G7572": ("εὐχερής", "", "easily managed", "eucherēs"),
        "G7573": ("εὐχερῶς", "", "easily", "eucherōs"),
        "G7574": ("εὐψύχως", "", "confidently", "eupsuchōs"),
        "G7575": ("εὐώδης", "", "sweet smelling", "euōdēs"),
        "G7576": ("εὐωδιάζω", "G:V", "be fragrant", "euōdiazō"),
        "G7577": ("Ἐφαδανώ", "", "a royal pavilion", "Ephadanō"),
        "G7578": ("ἐφαμαρτάνω", "", "to tempt", "ephamartanō"),
        "G7579": ("ἐφάπτω", "", "to attach to", "ephaptō"),
        "G7580": ("ἐφέλκω", "", "to drag upon", "ephelkō"),
        "G7581": ("ἔφηλος", "", "peeling skin", "ephēlos"),
        "G7582": ("ἑφθός", "", "cooked", "hephthos"),
        "G7583": ("ἐφοδεύω", "", "to explore", "ephodeuō"),
        "G7584": ("ἐφοδιάζω", "", "to provide", "ephodiazō"),
        "G7585": ("ἐφόδιος", "", "supplies", "ephodios"),
        "G7586": ("ἐφοράω", "", "to inspect", "ephoraō"),
        "G7587": ("ἐφούδ", "", "an ephod", "ephoud"),
        "G7588": ("ἐχθραίνω", "", "to hate", "echthrainō"),
        "G7589": ("ἐχθρεύω", "G:V", "be an enemy", "echthreuō"),
        "G7590": ("ἐχθρία", "", "hatred", "echthria"),
        "G7591": ("ἐχῖνος", "", "hedgehog", "echinos"),
        "G7592": ("ἐξουσιαστής", "", "a potentate", "exousiastēs"),
        "G7593": ("ἕψεμα", "", "a stew", "hepsema"),
        "G7594": ("ἕψω", "", "to bake", "hepsō"),
        "G7595": ("ἑωθινός", "", "early  morning", "heōthinos"),
        "G7596": ("ἑωσφόρος", "", "morning star", "heōsphoros"),
        "G7597": ("ζέα", "", "spelt", "zea"),
        "G7598": ("ζέμα", "", "boiling", "zema"),
        "G7599": ("ζεύγνυμι", "", "harness", "zeugnumi"),
        "G7600": ("ζηλοτυπία", "", "jealousy", "zēlotupia"),
        "G7601": ("ζήλωσις", "", "jealousy", "zēlōsis"),
        "G7602": ("ζηλωτός", "", "zealous", "zēlōtos"),
        "G7603": ("ζιβύνη", "", "a pike", "zibunē"),
        "G7604": ("ζίου", "", "Zif", "ziou"),
        "G7605": ("ζυγόω", "", "to join together", "zugoō"),
        "G7606": ("ζῦθος", "", "beer", "zuthos"),
        "G7607": ("ζυμίτης", "", "leavened", "zumitēs"),
        "G7608": ("ζυμωτός", "", "leavened", "zumōtos"),
        "G7609": ("ζωγραφέω", "", "to portray", "zōgrapheō"),
        "G7610": ("ζωγρεία", "", "taking alive", "zōgreia"),
        "G7611": ("ζωμός", "", "broth", "zōmos"),
        "G7612": ("ζωοποίησις", "", "restoring to life", "zōopoiēsis"),
        "G7613": ("ζωόω", "", "to enliven", "zōoō"),
        "G7614": ("ζωπυρέω", "", "to enliven", "zōpureō"),
        "G7615": ("ζῶσις", "", "girding up", "zōsis"),
        "G7616": ("ἡγεμονικός", "", "governing", "hēgemonikos"),
        "G7617": ("ἥγημα", "", "a leading", "hēgēma"),
        "G7618": ("ἡγητέον", "", "be esteemed", "hēgēteon"),
        "G7619": ("ἡδύνω", "G:V", "be delicious", "hēdunō"),
        "G7620": ("ἡδύς", "", "agreeable", "hēdus"),
        "G7621": ("ἥδυσμα", "", "luscious", "hēdusma"),
        "G7622": ("ἡδυσμός", "", "luscious", "hēdusmos"),
        "G7623": ("ἡδύφωνος", "", "sweet sounding", "hēduphōnos"),
        "G7624": ("ἥδω", "", "light; (in weight)", "hēdō"),
        "G7625": ("Ἠθάμ", "", "continuance", "Ētham"),
        "G7626": ("ἤλεκτρον", "", "molten bronze", "ēlektron"),
        "G7627": ("ἡλιάζομαι", "", "to expose to sun", "hēliazomai"),
        "G7628": ("ἡμίεφθος", "", "half-boiled", "hēmiephthos"),
        "G7629": ("ἡμίονος", "", "the mule", "hēmionos"),
        "G7630": ("ἡμίσευμα", "", "the half", "hēmiseuma"),
        "G7631": ("ἡμισεύω", "", "to halve", "hēmiseuō"),
        "G7632": ("ἡνία", "", "reins", "hēnia"),
        "G7633": ("ἡνίοχος", "", "a charioteer", "hēniochos"),
        "G7634": ("ἧπαρ", "", "the liver", "hēpar"),
        "G7635": ("ἠρεμάζω", "G:V", "be calm", "ēremazō"),
        "G7636": ("ἥρως", "", "a hero", "hērōs"),
        "G7637": ("ἡσυχῇ", "", "tranquilly", "hēsuchē"),
        "G7638": ("ἠχώ", "", "an echo", "ēchō"),
        "G7639": ("θάλλω", "", "to flourish", "thallō"),
        "G7640": ("θαλπιώθ", "", "an armory", "thalpiōth"),
        "G7641": ("θανουρίμ", "", "a furnace", "thanourim"),
        "G7642": ("θαρσείς", "", "tharsis", "tharseis"),
        "G7643": ("θαυμαστόω", "", "to cause wonder", "thaumastoō"),
        "G7644": ("θαυμαστῶς", "", "wonderfully", "thaumastōs"),
        "G7645": ("θεέ", "", "a vestibule", "thee"),
        "G7646": ("θεεβουλαθώθ", "", "rule", "theeboulathōth"),
        "G7647": ("θεηλάθ", "", "a porch", "theēlath"),
        "G7648": ("θεκέλ", "", "thekel", "thekel"),
        "G7649": ("θελητής", "", "a soothsayer", "thelētēs"),
        "G7650": ("θελητός", "", "wanted", "thelētos"),
        "G7651": ("θέμα", "", "in place", "thema"),
        "G7652": ("θεμελίωσις", "", "groundwork", "themeliōsis"),
        "G7653": ("θεννουρίμ", "", "a furnace", "thennourim"),
        "G7654": ("θεράπαινα", "", "a female attendant", "therapaina"),
        "G7655": ("θεραφείν", "", "teraphim", "theraphein"),
        "G7656": ("θηρευτής", "", "a hunter", "thēreutēs"),
        "G7657": ("θερινός", "", "a harvest", "therinos"),
        "G7658": ("θέριστρον", "", "a reaping hook", "theristron"),
        "G7659": ("θερμασία", "", "heat", "thermasia"),
        "G7660": ("θερμαστρίς", "", "a tong", "thermastris"),
        "G7661": ("θερμός", "", "hot", "thermos"),
        "G7662": ("θέσις", "", "designation", "thesis"),
        "G7663": ("θεσμός", "", "a rule", "thesmos"),
        "G7664": ("θεωρητός", "", "something visible", "theōrētos"),
        "G7665": ("θηλυκός", "", "female", "thēlukos"),
        "G7666": ("θηλυμανής", "", "sex-crazed", "thēlumanēs"),
        "G7667": ("θημωνιά", "", "a heap", "thēmōnia"),
        "G7668": ("θήρ", "", "a wild beast", "thēr"),
        "G7669": ("θήρευμα", "", "a hunt", "thēreuma"),
        "G7670": ("θηριάλωτος", "", "taken by wild beasts", "thērialōtos"),
        "G7671": ("θηριόβρωτος", "", "eaten by wild beasts", "thēriobrōtos"),
        "G7672": ("θησαύρισμα", "", "a treasury", "thēsaurisma"),
        "G7673": ("θησαυροφύλαξ", "", "a treasurer", "thēsaurophulax"),
        "G7674": ("θίασος", "", "revelry", "thiasos"),
        "G7675": ("θῖβις", "", "a wicker basket", "thibis"),
        "G7676": ("θίς", "", "a hill", "this"),
        "G7677": ("θλαδίας", "", "crushed testicles", "thladias"),
        "G7678": ("θλάσμα", "", "a fracture", "thlasma"),
        "G7679": ("θλάω", "", "to crush", "thlaō"),
        "G7680": ("θλιμμός", "", "affliction", "thlimmos"),
        "G7681": ("θνησιμαῖος", "", "decaying flesh", "thnēsimaios"),
        "G7682": ("θολερός", "", "clouded", "tholeros"),
        "G7683": ("θράσος", "", "boldness", "thrasos"),
        "G7684": ("θρασυκάρδιος", "", "bold-hearted", "thrasukardios"),
        "G7685": ("θρασύνω", "G:V", "to embolden", "thrasunō"),
        "G7686": ("θρασύς", "", "bold", "thrasus"),
        "G7687": ("θραῦσις", "", "a devastation", "thrausis"),
        "G7688": ("θραῦσμα", "", "outbreak", "thrausma"),
        "G7689": ("θραυσμός", "", "a breaking", "thrausmos"),
        "G7690": ("θρεπτός", "", "brought up", "threptos"),
        "G7691": ("θρήνημα", "", "wailing", "thrēnēma"),
        "G7692": ("θρυλλέω", "G:V", "to chatter", "thrulleō"),
        "G7693": ("θρύλλημα", "", "a common topic", "thrullēma"),
        "G7694": ("θυία", "", "a hand-mill", "thuia"),
        "G7695": ("θυΐσκη", "", "an incense pan", "thuiskē"),
        "G7696": ("θύλακος", "", "a purse", "thulakos"),
        "G7697": ("θῦμα", "", "an offering", "thuma"),
        "G7698": ("θυμώδης", "", "inclined to rage", "thumōdēs"),
        "G7699": ("θύρωμα", "", "a doorway", "thurōma"),
        "G7700": ("θυσιάζω", "", "to sacrifice", "thusiazō"),
        "G7701": ("θυσίασμα", "", "a sacrifice", "thusiasma"),
        "G7702": ("θωδαθά", "", "thanksgiving", "thōdatha"),
        "G7703": ("ιαμίν", "", "mules", "iamin"),
        "G7704": ("ἰατής", "", "a healer", "iatēs"),
        "G7705": ("ἰατρεία", "", "healing", "iatreia"),
        "G7706": ("ἰατρεῖον", "", "a physician's fee", "iatreion"),
        "G7707": ("ἰατρεύω", "", "to treat; medically", "iatreuō"),
        "G7708": ("ἶβις", "", "an ibis", "ibis"),
        "G7709": ("ἰγνύα", "", "ham", "ignua"),
        "G7710": ("ἰδιοποιέω", "", "to adopt", "idiopoieō"),
        "G7711": ("ἱδρύω", "", "to secure", "hidruō"),
        "G7712": ("ἱέραξ", "", "a hawk", "hierax"),
        "G7713": ("ἱκανῶς", "", "fittingly", "hikanōs"),
        "G7714": ("ἱκετεύω", "", "to entreat", "hiketeuō"),
        "G7715": ("ἱκέτης", "", "a supplicant", "hiketēs"),
        "G7716": ("ἴκτερος", "", "jaundice", "ikteros"),
        "G7717": ("ἰκτίν", "", "a kite", "iktin"),
        "G7718": ("ἱλαρύνω", "", "to make happy", "hilarunō"),
        "G7719": ("ἱλαρῶς", "", "happily", "hilarōs"),
        "G7720": ("ἰλύς", "", "slime", "ilus"),
        "G7721": ("ἱματιοφύλαξ", "", "keeper of cloaks", "himatiophulax"),
        "G7722": ("ἴνδαλμα", "", "an effigy", "indalma"),
        "G7723": ("ἰξευτής", "", "a fowler", "ixeutēs"),
        "G7724": ("Ιοὐδαϊστί", "", "in Hebrew", "Ioudaistí"),
        "G7725": ("ἱππάζομαι", "", "to ride a horse", "hippazomai"),
        "G7726": ("ἱππάρχης", "", "cavalry commander", "hipparchēs"),
        "G7727": ("ἱππασία", "", "riding", "hippasia"),
        "G7728": ("ἱππεύω", "", "to ride horseback", "hippeuō"),
        "G7729": ("ἱπποδρόμος", "", "course of horses", "hippodromos"),
        "G7730": ("ἰσόω", "", "to make equal", "isoō"),
        "G7731": ("ἱστάνω", "", "to establish", "histanō"),
        "G7732": ("ἱστίον", "", "a shroud", "histion"),
        "G7733": ("ἱστός", "", "a mast", "histos"),
        "G7734": ("ἰσχίον", "", "a hip", "ischion"),
        "G7735": ("ἰσχνόφωνος", "", "weak voiced", "ischnophōnos"),
        "G7736": ("ἰσχυρόω", "", "to strengthen", "ischuroō"),
        "G7737": ("ἰσχυρῶς", "", "strongly", "ischurōs"),
        "G7738": ("ἰταμία", "", "audacity", "itamia"),
        "G7739": ("ἰταμός", "", "audacious", "itamos"),
        "G7740": ("ἰτέα", "", "willow", "itea"),
        "G7741": ("ἰχθυηρός", "", "of fish", "ichthuēros"),
        "G7742": ("ἰχθυϊκός", "", "fishing", "ichthuikos"),
        "G7743": ("ἰχνεύω", "", "to prowl", "ichneuō"),
        "G7744": ("ἰχώρ", "", "pus", "ichōr"),
        "G7745": ("Ἰωβήλ", "", "a ram", "Iōbēl"),
        "G7746": ("κάβος", "", "a cab measure", "kabos"),
        "G7747": ("καδημείν", "", "stream of antiquity", "kadēmein"),
        "G7748": ("καδησίμ", "", "male prostitutes", "kadēsim"),
        "G7749": ("κάδιον", "", "a canteen", "kadion"),
        "G7750": ("κάδος", "", "a pail", "kados"),
        "G7751": ("καθαγιάζω", "", "to consecrate", "kathagiazō"),
        "G7752": ("καθαρεύω", "G:V", "be clean", "kathareuō"),
        "G7753": ("καθαριότης", "", "cleanliness", "kathariotēs"),
        "G7754": ("κάθαρσις", "", "a cleansing", "katharsis"),
        "G7755": ("κάθεμα", "", "a necklace", "kathema"),
        "G7756": ("καθηλόω", "", "to nail up", "kathēloō"),
        "G7757": ("κάθιδρος", "", "sweating", "kathidros"),
        "G7758": ("καθιζάνω", "", "to sit down", "kathizanō"),
        "G7759": ("κάθισις", "", "sitting", "kathisis"),
        "G7760": ("καθοδηγέω", "", "to direct", "kathodēgeō"),
        "G7761": ("κάθοδος", "", "a return", "kathodos"),
        "G7762": ("καθομολογέω", "", "to promise", "kathomologeō"),
        "G7763": ("καθόρμιον", "", "a hanging necklace", "kathormion"),
        "G7764": ("καθυβρίζω", "", "to insult", "kathubrizō"),
        "G7765": ("καθυμνέω", "", "to sing praise", "kathumneō"),
        "G7766": ("καθυπνόω", "", "to sleep", "kathupnoō"),
        "G7767": ("καθυστερέω", "G:V", "be late", "kathustereō"),
        "G7768": ("καθυφαίνω", "", "to interweave", "kathuphainō"),
        "G7769": ("καινίζω", "", "to revive", "kainizō"),
        "G7770": ("καίριος", "", "timely", "kairios"),
        "G7771": ("κακοποίησις", "", "an evil deed", "kakopoiēsis"),
        "G7772": ("κακουργία", "", "evil actions", "kakourgia"),
        "G7773": ("κακοφροσύνη", "", "evil thinking", "kakophrosunē"),
        "G7774": ("κακόφρων", "", "evil-minded", "kakophrōn"),
        "G7775": ("καλαβώτης", "", "a newt", "kalabōtēs"),
        "G7776": ("κάλαθος", "", "a reed basket", "kalathos"),
        "G7777": ("καλαμάομαι", "", "to glean", "kalamaomai"),
        "G7778": ("καλάμινος", "", "of reed", "kalaminos"),
        "G7779": ("καλαμίσκος", "", "a branch", "kalamiskos"),
        "G7780": ("καλλιόω", "", "to beautify", "kallioō"),
        "G7781": ("καλλονή", "", "beauty", "kallonē"),
        "G7782": ("κάλλος", "", "beauty", "kallos"),
        "G7783": ("κάλλυνθρον", "", "a palm branch", "kallunthron"),
        "G7784": ("καλλωπίζω", "", "to bedeck", "kallōpizō"),
        "G7785": ("κάλος", "", "rope", "kalos"),
        "G7786": ("καλυπτήρ", "", "a lid", "kaluptēr"),
        "G7787": ("καλῴδιον", "", "rope", "kalōdion"),
        "G7788": ("καμάρα", "", "a vault", "kamara"),
        "G7789": ("καμηλοπάρδαλις", "", "giraffe", "kamēlopardalis"),
        "G7790": ("καμιναῖος", "", "of a furnace", "kaminaios"),
        "G7791": ("κάμπη", "", "a curve", "kampē"),
        "G7792": ("καμπύλος", "", "curved", "kampulos"),
        "G7793": ("κάνθαρος", "", "beetle", "kantharos"),
        "G7794": ("κανοῦν", "", "a bin", "kanoun"),
        "G7795": ("κάπηλος", "", "a peddler", "kapēlos"),
        "G7796": ("καπνίζω", "", "to smoke", "kapnizō"),
        "G7797": ("κάππαρις", "", "caper", "kapparis"),
        "G7798": ("καρδιόω", "", "to take heart", "kardioō"),
        "G7799": ("καρόω", "", "to stupefy", "karoō"),
        "G7800": ("καρπάσινος", "", "of cotton", "karpasinos"),
        "G7801": ("καρπίζω", "", "to gather fruit", "karpizō"),
        "G7802": ("κάρπιμος", "", "fruitful", "karpimos"),
        "G7803": ("καρπόβρωτος", "", "fruit-bearing", "karpobrōtos"),
        "G7804": ("καρπός", "", "wrist", "karpos"),
        "G7805": ("καρπόω", "", "to yield", "karpoō"),
        "G7806": ("κάρπωμα", "", "a yield", "karpōma"),
        "G7807": ("κάρπωσις", "", "a yield", "karpōsis"),
        "G7808": ("καρπωτός", "", "with long sleeves", "karpōtos"),
        "G7809": ("κάρταλλος", "", "a narrow basket", "kartallos"),
        "G7810": ("καρτερόω", "", "to strengthen", "karteroō"),
        "G7811": ("καρύα", "", "a walnut", "karua"),
        "G7812": ("καρύϊνος", "", "of walnut", "karuinos"),
        "G7813": ("καρυΐσκος", "", "nut-like", "karuiskos"),
        "G7814": ("κάρυον", "", "a walnut", "karuon"),
        "G7815": ("καρυωτά", "", "a knob", "karuōta"),
        "G7816": ("κασσία", "", "cassia", "kassia"),
        "G7817": ("κασσιτέρινος", "", "of tin", "kassiterinos"),
        "G7818": ("κασσίτερος", "", "tin", "kassiteros"),
        "G7819": ("κατα πόδας", "", "by tracks", "kata podas"),
        "G7820": ("καταβιάζω", "", "to constrain", "katabiazō"),
        "G7821": ("καταβιόω", "", "to spend one's life", "katabioō"),
        "G7822": ("καταβλέπω", "", "to look down", "katablepō"),
        "G7823": ("καταβοάω", "", "to yell out against", "kataboaō"),
        "G7824": ("καταβόσκω", "", "to graze upon", "kataboskō"),
        "G7825": ("κατάβρωμα", "", "a thing devoured", "katabrōma"),
        "G7826": ("κατάβρωσις", "", "a thing devoured", "katabrōsis"),
        "G7827": ("καταβρώσκω", "", "to devour", "katabrōskō"),
        "G7828": ("κατάγαιος", "", "on the ground", "katagaios"),
        "G7829": ("κατάγελως", "", "laughter", "katagelōs"),
        "G7830": ("καταγηράσκω", "G:V", "to age", "katagēraskō"),
        "G7831": ("καταγίνομαι", "", "to occupy", "kataginomai"),
        "G7832": ("καταδείκνυμι", "", "to introduce", "katadeiknumi"),
        "G7833": ("καταδέομαι", "", "to beseech", "katadeomai"),
        "G7834": ("κατάδεσμος", "", "a bandage", "katadesmos"),
        "G7835": ("καταδέχομαι", "", "to appreciate", "katadechomai"),
        "G7836": ("καταδιαιρέω", "", "to divide", "katadiaireō"),
        "G7837": ("καταδολεσχέω", "", "to converse with", "katadolescheō"),
        "G7838": ("καταδυναστεία", "", "tyranny", "katadunasteia"),
        "G7839": ("κατάδυσις", "", "a retreat", "katadusis"),
        "G7840": ("καταδύω", "", "to descend into", "kataduō"),
        "G7841": ("καταθαρσέω", "", "to rely upon", "katatharseō"),
        "G7842": ("καταθλάω", "", "to break in pieces", "katathlaō"),
        "G7843": ("καταθύμιος", "", "wishes", "katathumios"),
        "G7844": ("καταιγίς", "", "a blast", "kataigis"),
        "G7845": ("κατακάλυμμα", "", "an overcovering", "katakalumma"),
        "G7846": ("κατακάμπτω", "", "to bend down", "katakamptō"),
        "G7847": ("κατάκαρπος", "", "fruitful", "katakarpos"),
        "G7848": ("κατακάρπως", "", "fruitfully", "katakarpōs"),
        "G7849": ("κατακάρπωσις", "", "incinerated remains", "katakarpōsis"),
        "G7850": ("κατάκαυμα", "", "a burning", "katakauma"),
        "G7851": ("κατακενόω", "", "to empty", "katakenoō"),
        "G7852": ("κατακεντέω", "", "to pierce through", "katakenteō"),
        "G7853": ("κατακληρονομέω", "", "to allot", "kataklēronomeō"),
        "G7854": ("κατακληρόω", "", "to choose by lot", "kataklēroō"),
        "G7855": ("κατάκλιτον", "", "a divan", "katakliton"),
        "G7856": ("κατακονδυλίζω", "", "to strike with a fist", "katakondulizō"),
        "G7857": ("κατακοντίζω", "", "to run through", "katakontizō"),
        "G7858": ("κατάκοπος", "", "exhausted", "katakopos"),
        "G7859": ("κατακοσμέω", "", "to adorn", "katakosmeō"),
        "G7860": ("κατακρατέω", "", "to hold firmly", "katakrateō"),
        "G7861": ("κατακρύπτω", "", "to hide", "katakruptō"),
        "G7862": ("κατακτάομαι", "", "to acquire", "kataktaomai"),
        "G7863": ("κατακυκλόω", "", "to encircle", "katakukloō"),
        "G7864": ("κατακυλίνδω", "", "to roll downward", "katakulindō"),
        "G7865": ("κατάλειψις", "", "a remnant", "kataleipsis"),
        "G7866": ("καταλέω", "", "to grind down", "kataleō"),
        "G7867": ("κατάληψις", "", "a taking", "katalēpsis"),
        "G7868": ("καταλιθοβολέω", "", "to cast a stone at", "katalithoboleō"),
        "G7869": ("κατάλιθος", "", "inlaid with precious stone", "katalithos"),
        "G7870": ("καταλιμπάνω", "", "to leave behind", "katalimpanō"),
        "G7871": ("καταλογίζομαι", "", "to reckon", "katalogizomai"),
        "G7872": ("καταλοχία", "", "classification", "katalochia"),
        "G7873": ("καταλοχισμός", "", "distribution", "katalochismos"),
        "G7874": ("κατάλυσις", "", "a resting-place", "katalusis"),
        "G7875": ("καταμερίζω", "", "to divide among", "katamerizō"),
        "G7876": ("καταμετρέω", "", "to measure out", "katametreō"),
        "G7877": ("καταμίγνυμι", "", "to mix with", "katamignumi"),
        "G7878": ("καταμωκάομαι", "", "to deride", "katamōkaomai"),
        "G7879": ("κατανέμω", "", "to feed on", "katanemō"),
        "G7880": ("κατανίσταμαι", "", "to stand against", "katanistamai"),
        "G7881": ("κατάντημα", "", "an arrival", "katantēma"),
        "G7882": ("καταξαίνω", "", "to rend in shreds", "kataxainō"),
        "G7883": ("καταξηραίνω", "", "to totally dry up", "kataxērainō"),
        "G7884": ("κατάξηρος", "", "dried up", "kataxēros"),
        "G7885": ("καταπαίζω", "", "to mock", "katapaizō"),
        "G7886": ("καταπανουργέυω", "", "to deal treacherously", "katapanourgeuō"),
        "G7887": ("καταπάσσω", "", "to strew", "katapassō"),
        "G7888": ("καταπάτημα", "", "trampling", "katapatēma"),
        "G7889": ("καταπάτησις", "", "trampling", "katapatēsis"),
        "G7890": ("καταπελματόομαι", "", "to mend", "katapelmatoomai"),
        "G7891": ("καταπενθέω", "", "to mourn", "katapentheō"),
        "G7892": ("καταπέτομαι", "", "to fly down", "katapetomai"),
        "G7893": ("καταπήγνυμι", "", "to fasten down", "katapēgnumi"),
        "G7894": ("καταπηδάω", "", "to leap down", "katapēdaō"),
        "G7895": ("κατάπικρος", "", "very bitter", "katapikros"),
        "G7896": ("καταπιστεύω", "", "to confide in", "katapisteuō"),
        "G7897": ("καταπλάσσω", "", "to plaster", "kataplassō"),
        "G7898": ("κατάπληξις", "", "a terror", "kataplēxis"),
        "G7899": ("καταπλήσσω", "", "to strike with terror", "kataplēssō"),
        "G7900": ("καταπολεμέω", "", "to make war against", "katapolemeō"),
        "G7901": ("καταποντισμός", "", "drowning", "katapontismos"),
        "G7902": ("καταπραΰνω", "", "to soothe", "katapraunō"),
        "G7903": ("καταπρονομεύω", "", "to carry away captive", "katapronomeuō"),
        "G7904": ("καταπτήσσω", "G:V", "to awe", "kataptēssō"),
        "G7905": ("κατάπτωμα", "", "a ruin", "kataptōma"),
        "G7906": ("κατάρασις", "", "a curse", "katarasis"),
        "G7907": ("καταράσσω", "", "to break down", "katarassō"),
        "G7908": ("καταργυρόω", "", "to silver plate", "katarguroō"),
        "G7909": ("καταρράκτης", "", "a dungeon", "katarraktēs"),
        "G7910": ("καταρρεμβεύω", "", "to stagger", "katarrembeuō"),
        "G7911": ("καταρρέω", "", "to flow down", "katarreō"),
        "G7912": ("καταρρήγνυμι", "", "to break down", "katarrēgnumi"),
        "G7913": ("καταρρίπτω", "", "to hurl down", "katarriptō"),
        "G7914": ("κατάρχω", "", "to rule", "katarchō"),
        "G7915": ("κατασβεννύω", "", "to extinguish", "katasbennuō"),
        "G7916": ("κατασιγάω", "", "to observe silence", "katasigaō"),
        "G7917": ("κατασιωπάω", "", "to observe silence", "katasiōpaō"),
        "G7918": ("κατασκεδάζω", "", "to scatter upon", "kataskedazō"),
        "G7919": ("κατασκέπτομαι", "", "to survey", "kataskeptomai"),
        "G7920": ("κατασκευή", "", "apparatus", "kataskeuē"),
        "G7921": ("κατάσκιος", "", "shady", "kataskios"),
        "G7922": ("κατασμικρύνω", "", "to reduce", "katasmikrunō"),
        "G7923": ("κατασπαταλάω", "", "to live wastefully", "kataspatalaō"),
        "G7924": ("κατασπάω", "", "to tear down", "kataspaō"),
        "G7925": ("κατασπείρω", "", "to scatter abroad", "kataspeirō"),
        "G7926": ("κατασπεύδω", "", "to hasten", "kataspeudō"),
        "G7927": ("κατασπουδάζομαι", "", "to take seriously", "kataspoudazomai"),
        "G7928": ("καταστασιάζω", "", "to revolt", "katastasiazō"),
        "G7929": ("καταστενάζω", "", "to groan", "katastenazō"),
        "G7930": ("καταστηρίζω", "", "to firmly fix", "katastērizō"),
        "G7931": ("καταστραγγίζω", "", "to make drop", "katastraggizō"),
        "G7932": ("καταστρατοπεδεύω", "", "to bivouac", "katastratopedeuō"),
        "G7933": ("κατατάσσω", "", "to delegate", "katatassō"),
        "G7934": ("κατατείνω", "", "to violently strain", "katateinō"),
        "G7935": ("κατατέμνω", "", "to mutilate", "katatemnō"),
        "G7936": ("κατατέρπω", "", "to delight", "kataterpō"),
        "G7937": ("κατατήκω", "", "to dissolve", "katatēkō"),
        "G7938": ("κατατίλλω", "", "to pull about", "katatillō"),
        "G7939": ("κατατρίβω", "", "to wear away", "katatribō"),
        "G7940": ("κατατρυφάω", "", "to revel", "katatruphaō"),
        "G7941": ("κατατρώγω", "", "to devour", "katatrōgō"),
        "G7942": ("κατατυγχάνω", "", "to attain", "katatugchanō"),
        "G7943": ("καταφαίνω", "", "to appear", "kataphainō"),
        "G7944": ("καταφερής", "", "inclined", "katapherēs"),
        "G7945": ("καταφθάνω", "", "to fall upon unawares", "kataphthanō"),
        "G7946": ("καταφθορά", "", "corruption", "kataphthora"),
        "G7947": ("καταφλέγω", "", "to burn up", "kataphlegō"),
        "G7948": ("καταφλογίζω", "", "to ignite", "kataphlogizō"),
        "G7949": ("κατάφοβος", "", "fear", "kataphobos"),
        "G7950": ("καταφυγή", "", "a refuge", "kataphugē"),
        "G7951": ("καταφύτευσις", "", "a planting", "kataphuteusis"),
        "G7952": ("καταφυτεύω", "", "to plant", "kataphuteuō"),
        "G7953": ("καταχαίρω", "", "to rejoice against", "katachairō"),
        "G7954": ("καταχαλάω", "", "to let down", "katachalaō"),
        "G7955": ("καταχαλκόω", "", "to braze", "katachalkoō"),
        "G7956": ("καταχρίω", "", "to besmear", "katachriō"),
        "G7957": ("καταχρύσεος", "", "abounding in gold", "katachruseos"),
        "G7958": ("καταχρυσόω", "", "to gild", "katachrusoō"),
        "G7959": ("κατάχυσις", "", "a throwing down", "katachusis"),
        "G7960": ("καταχώννυμι", "", "to heap upon", "katachōnnumi"),
        "G7961": ("καταχωρίζω", "", "to set down in writing", "katachōrizō"),
        "G7962": ("κατειδών", "", "to look down", "kateidōn"),
        "G7963": ("κατεντευκτής", "", "an accuser", "katenteuktēs"),
        "G7964": ("κατεπείγω", "", "to coerce", "katepeigō"),
        "G7965": ("κατεῖπω", "", "to denounce", "kateipō"),
        "G7966": ("κατεργασία", "", "working out", "katergasia"),
        "G7967": ("κάτεργος", "", "upkeep", "katergos"),
        "G7968": ("κατευθύ", "", "straightway", "kateuthu"),
        "G7969": ("κατευοδόω", "", "to greatly prosper", "kateuodoō"),
        "G7970": ("κατοικεσία", "", "habitation", "katoikesia"),
        "G7971": ("κατοικοδομέω", "", "to build", "katoikodomeō"),
        "G7972": ("κάτοικος", "", "an inhabitant", "katoikos"),
        "G7973": ("κατοίομαι", "", "be arrogant", "katoiomai"),
        "G7974": ("κατόπισθεν", "", "after", "katopisthen"),
        "G7975": ("κάτοπτρον", "", "a mirror", "katoptron"),
        "G7976": ("κατορθόω", "", "to keep straight", "katorthoō"),
        "G7977": ("κατόρθωσις", "", "a success", "katorthōsis"),
        "G7978": ("κατορύσσω", "", "to bury", "katorussō"),
        "G7979": ("κατορχέομαι", "", "to treat despitefully", "katorcheomai"),
        "G7980": ("κατοχεύω", "", "to mate", "katocheuō"),
        "G7981": ("κατόχιμος", "", "possession", "katochimos"),
        "G7982": ("κάτοχος", "", "a hold", "katochos"),
        "G7983": ("κατωδυνάω", "", "to afflict grievously", "katōdunaō"),
        "G7984": ("κατώδυνος", "", "in severe pain", "katōdunos"),
        "G7985": ("κάτωθεν", "", "from below", "katōthen"),
        "G7986": ("καυλός", "", "a stem", "kaulos"),
        "G7987": ("καψάκης", "", "a jar", "kapsakēs"),
        "G7988": ("κέγχρος", "", "millet", "kegchros"),
        "G7989": ("κέδρινος", "", "of cedar", "kedrinos"),
        "G7990": ("κεκρυμμένως", "", "secretly", "kekrummenōs"),
        "G7991": ("κενολογέω", "", "to use empty words", "kenologeō"),
        "G7992": ("κενοτάφιον", "", "a statue", "kenotaphion"),
        "G7993": ("κεντέω", "", "to sting", "kenteō"),
        "G7994": ("κεπφωθείς", "", "being easily led on", "kepphōtheis"),
        "G7995": ("κέρασμα", "", "a mixture", "kerasma"),
        "G7996": ("κεράστης", "", "a horned serpent", "kerastēs"),
        "G7997": ("κερατίζω", "", "to butt", "keratizō"),
        "G7998": ("κερατίνη", "", "a horn", "keratinē"),
        "G7999": ("κερατιστής", "", "goring", "keratistēs"),
        "G8000": ("κεραυνός", "", "a thunderbolt", "keraunos"),
        "G8001": ("κεραυνόω", "", "to strike with a thunderbolt", "keraunoō"),
        "G8002": ("κέρκος", "", "a tail", "kerkos"),
        "G8003": ("κέρκωψ", "", "mischievous", "kerkōps"),
        "G8004": ("κεφουρέ", "", "a basin", "kephoure"),
        "G8005": ("κεφουρῆς", "", "a basin", "kephourēs"),
        "G8006": ("κηλιδόω", "", "to spot", "kēlidoō"),
        "G8007": ("κημός", "", "a muzzle", "kēmos"),
        "G8008": ("κηρός", "", "beeswax", "kēros"),
        "G8009": ("κίβδηλος", "", "commingled", "kibdēlos"),
        "G8010": ("κίδαρις", "", "a turban", "kidaris"),
        "G8011": ("κινύρα", "", "a lute", "kinura"),
        "G8012": ("κιρνάω", "", "to dilute", "kirnaō"),
        "G8013": ("κισσάω", "", "to crave for strange food", "kissaō"),
        "G8014": ("κιχράω", "", "to lend", "kichraō"),
        "G8015": ("κίων", "", "a monumental pillar", "kiōn"),
        "G8016": ("κλαυθμών", "", "a place of weeping", "klauthmōn"),
        "G8017": ("κλεῖθρον", "", "a bolt", "kleithron"),
        "G8018": ("κληδονίζω", "", "to prognosticate", "klēdonizō"),
        "G8019": ("κληδονισμός", "", "prognosticating", "klēdonismos"),
        "G8020": ("κληματίς", "", "a small vine branch", "klēmatis"),
        "G8021": ("κληροδοσία", "", "an allotment", "klērodosia"),
        "G8022": ("κληροδοτέω", "", "to allot", "klērodoteō"),
        "G8023": ("κληρωτί", "", "by casting lots", "klērōti"),
        "G8024": ("κλιμακτήρ", "", "a step", "klimaktēr"),
        "G8025": ("κλῖμαξ", "", "a stairway", "klimax"),
        "G8026": ("κλίτος", "", "a corner", "klitos"),
        "G8027": ("κλοιός", "", "a neck yoke", "kloios"),
        "G8028": ("κλοποφορέω", "", "to ransack", "klopophoreō"),
        "G8029": ("κλώθω", "", "to twine", "klōthō"),
        "G8030": ("κλών", "", "a branch", "klōn"),
        "G8031": ("κλῶσμα", "", "yarn", "klōsma"),
        "G8032": ("κλωστός", "", "spun", "klōstos"),
        "G8033": ("κνήμη", "", "shank", "knēmē"),
        "G8034": ("κνημίς", "", "a leg covering", "knēmis"),
        "G8035": ("κνήφη", "", "itching", "knēphē"),
        "G8036": ("κνίδη", "", "nettle", "knidē"),
        "G8037": ("κνίζω", "", "to pick", "knizō"),
        "G8038": ("κοιλάς", "", "a cavity", "koilas"),
        "G8039": ("κοίλη", "", "a ship's hold", "koilē"),
        "G8040": ("κοῖλος", "", "hollow", "koilos"),
        "G8041": ("κοιλοσταθμέω", "", "to vault", "koilostathmeō"),
        "G8042": ("κοιλόσταθμος", "", "vaulted", "koilostathmos"),
        "G8043": ("κοίλωμα", "", "a hollow", "koilōma"),
        "G8044": ("κοιμίζω", "", "to rest", "koimizō"),
        "G8045": ("κοιτάζω", "", "to lay in bed", "koitazō"),
        "G8046": ("κοιτασία", "", "a laying with", "koitasia"),
        "G8047": ("κολαβρίζω", "", "to crush", "kolabrizō"),
        "G8048": ("κολακεύω", "", "to flatter", "kolakeuō"),
        "G8049": ("κολαπτός", "", "a gourd", "kolaptos"),
        "G8050": ("κολάπτω", "", "to chisel", "kolaptō"),
        "G8051": ("κολεός", "", "a sheath", "koleos"),
        "G8052": ("κόλλη", "", "glue", "kollē"),
        "G8053": ("κολλυρίζω", "", "to bake biscuits", "kollurizō"),
        "G8054": ("κολλυρίς", "", "a biscuit", "kolluris"),
        "G8055": ("κολοβόκερκος", "", "tailless", "kolobokerkos"),
        "G8056": ("κολοβόρριν", "", "splitmouth", "koloborrin"),
        "G8057": ("κολοκύνθη", "", "a gourd", "kolokunthē"),
        "G8058": ("κόλπωμα", "", "recess", "kolpōma"),
        "G8059": ("κόνδυ", "", "a drinking cup", "kondu"),
        "G8060": ("κονδυλίζω", "", "to smite", "kondulizō"),
        "G8061": ("κονδυλισμός", "", "ill-treatment", "kondulismos"),
        "G8062": ("κονία", "", "lime", "konia"),
        "G8063": ("κονίαμα", "", "whitewash", "koniama"),
        "G8064": ("κοντός", "", "a shaft", "kontos"),
        "G8065": ("κόνυζα", "", "a briar", "konuza"),
        "G8066": ("κοπανίζω", "", "to pound out", "kopanizō"),
        "G8067": ("κοπόω", "", "to trouble", "kopoō"),
        "G8068": ("κόπρος", "", "dung", "kopros"),
        "G8069": ("κόπωσις", "", "weariness", "kopōsis"),
        "G8070": ("κόρη", "", "pupil", "korē"),
        "G8071": ("κόριον", "", "coriander", "korion"),
        "G8072": ("κορύνη", "", "a truncheon", "korunē"),
        "G8073": ("κορυφή", "", "head", "koruphē"),
        "G8074": ("κορώνη", "", "a crowbar", "korōnē"),
        "G8075": ("κόσυμβος", "", "a fringe", "kosumbos"),
        "G8076": ("κοσυμβωτός", "", "fringed", "kosumbōtos"),
        "G8077": ("κοτύλη", "", "a small cup", "kotulē"),
        "G8078": ("κουρά", "", "shearing", "koura"),
        "G8079": ("κουρεύς", "", "a barber", "koureus"),
        "G8080": ("κοῦφος", "", "light (in weight)", "kouphos"),
        "G8081": ("κούφως", "", "nimbly", "kouphōs"),
        "G8082": ("κόχλαξ", "", "a pebble", "kochlax"),
        "G8083": ("κραιπαλάω", "G:V", "be dizzy from wine", "kraipalaō"),
        "G8084": ("κρᾶμα", "", "mixed wine", "krama"),
        "G8085": ("κραταιότης", "", "force", "krataiotēs"),
        "G8086": ("κραταίωμα", "", "fortification", "krataiōma"),
        "G8087": ("κραταιῶς", "", "forcefully", "krataiōs"),
        "G8088": ("κραταίωσις", "", "fortification", "krataiōsis"),
        "G8089": ("κρατήρ", "", "a basin", "kratēr"),
        "G8090": ("κρεάγρα", "", "a meat hook", "kreagra"),
        "G8091": ("κρεανομέω", "", "to dress meat", "kreanomeō"),
        "G8092": ("κρεμαστός", "", "hanging", "kremastos"),
        "G8093": ("κρήνη", "", "a fountain", "krēnē"),
        "G8094": ("κρηπίς", "", "a bank", "krēpis"),
        "G8095": ("κρίκος", "", "a hook", "krikos"),
        "G8096": ("κριός", "", "a ram", "krios"),
        "G8097": ("κρόκη", "", "woof", "krokē"),
        "G8098": ("κροκόδειλος", "", "crocodile", "krokodeilos"),
        "G8099": ("κρόκος", "", "saffron", "krokos"),
        "G8100": ("κρόμμυον", "", "onion", "krommuon"),
        "G8101": ("κροσσός", "", "a border", "krossos"),
        "G8102": ("κροσσωτός", "", "bordered fringes", "krossōtos"),
        "G8103": ("κρόταφος", "", "temple", "krotaphos"),
        "G8104": ("κροτέω", "", "to clap", "kroteō"),
        "G8105": ("κρυβῆ", "", "secretly", "krubē"),
        "G8106": ("κρυφαίως", "", "secretly", "kruphaiōs"),
        "G8107": ("κρύφιος", "", "private", "kruphios"),
        "G8108": ("κτείνω", "", "to slay", "kteinō"),
        "G8109": ("κτηνότροφος", "", "grazing cattle", "ktēnotrophos"),
        "G8110": ("κτηνώδης", "", "brutish", "ktēnōdēs"),
        "G8111": ("κτῆσις", "", "a possession", "ktēsis"),
        "G8112": ("κύαθος", "", "a cup", "kuathos"),
        "G8113": ("κύαμος", "", "a bean", "kuamos"),
        "G8114": ("κυβερνάω", "", "to devise", "kubernaō"),
        "G8115": ("κύβος", "", "a cube", "kubos"),
        "G8116": ("κυδοιμός", "", "an uproar", "kudoimos"),
        "G8117": ("κῦδος", "", "dignity", "kudos"),
        "G8118": ("κύησις", "", "conception", "kuēsis"),
        "G8119": ("κύκλωμα", "", "a swirl", "kuklōma"),
        "G8120": ("κύκνος", "", "swan", "kuknos"),
        "G8121": ("κυμαίνω", "", "to swell up", "kumainō"),
        "G8122": ("κυμάτιον", "", "a waved border", "kumation"),
        "G8123": ("κυμβαλίζω", "G:V", "to play cymbals", "kumbalizō"),
        "G8124": ("κυνηγέω", "", "to hunt with hounds", "kunēgeō"),
        "G8125": ("κυνηγός", "", "a hunter with hounds", "kunēgos"),
        "G8126": ("κυνικός", "", "churlish", "kunikos"),
        "G8127": ("κυνόμυια", "", "the dog-fly", "kunomuia"),
        "G8128": ("κυοφορέω", "G:V", "be with child", "kuophoreō"),
        "G8129": ("κυπαρίσσινος", "", "of cypress", "kuparissinos"),
        "G8130": ("κυπάρισσος", "", "cypress", "kuparissos"),
        "G8131": ("κυπρίζω", "", "to blossom", "kuprizō"),
        "G8132": ("κυπρισμός", "", "a blossom", "kuprismos"),
        "G8133": ("κυρτός", "", "humpback", "kurtos"),
        "G8134": ("κύτος", "", "extent", "kutos"),
        "G8135": ("κύφω", "", "to have downcast eyes", "kuphō"),
        "G8136": ("κυψέλη", "", "a granary", "kupselē"),
        "G8137": ("κύω", "", "to sire", "kuō"),
        "G8138": ("κώδων", "", "a bell", "kōdōn"),
        "G8139": ("κώθων", "", "toasting", "kōthōn"),
        "G8140": ("κωθωνίζω", "", "to toast", "kōthōnizō"),
        "G8141": ("κωλέα", "", "a hind quarter", "kōlea"),
        "G8142": ("κώλυμα", "", "a restraint", "kōluma"),
        "G8143": ("κωμάρχης", "", "a magistrate", "kōmarchēs"),
        "G8144": ("κώπη", "", "an oar", "kōpē"),
        "G8145": ("κωπηλάτης", "", "an oarsman", "kōpēlatēs"),
        "G8146": ("κωφεύω", "G:V", "be silent", "kōpheuō"),
        "G8147": ("κωφόω", "G:V", "be mute", "kōphoō"),
        "G8148": ("λαβή", "", "a handle", "labē"),
        "G8149": ("λαβίς", "", "a tong", "labis"),
        "G8150": ("λάβρος", "", "fierce", "labros"),
        "G8151": ("λάγανον", "", "a pancake", "laganon"),
        "G8152": ("λαθραίως", "", "clandestinely", "lathraiōs"),
        "G8153": ("λάθριος", "", "private", "lathrios"),
        "G8154": ("λάκκος", "", "a pit", "lakkos"),
        "G8155": ("λάλημα", "", "a discussion", "lalēma"),
        "G8156": ("λαλητός", "", "power of speech", "lalētos"),
        "G8157": ("λαμπάδιον", "", "an oil lamp bowl", "lampadion"),
        "G8158": ("λαμπήνη", "", "a covered royal chariot", "lampēnē"),
        "G8159": ("λαμπηνικός", "", "like a covered chariot", "lampēnikos"),
        "G8160": ("λαμπτήρ", "", "a torch", "lamptēr"),
        "G8161": ("λαξευτήριον", "", "a stone chisel", "laxeutērion"),
        "G8162": ("λαξεύω", "", "to dress stone", "laxeuō"),
        "G8163": ("λάπτω", "", "to lap", "laptō"),
        "G8164": ("λάρος", "", "gull", "laros"),
        "G8165": ("λατομητός", "", "quarried", "latomētos"),
        "G8166": ("λατόμος", "", "a quarrier", "latomos"),
        "G8167": ("λατρευτός", "", "servile", "latreutos"),
        "G8168": ("λάφυρα", "", "bounty", "laphura"),
        "G8169": ("λαχανεία", "", "vegetables", "lachaneia"),
        "G8170": ("λέαινα", "", "lioness", "leaina"),
        "G8171": ("λεαίνω", "", "to grind", "leainō"),
        "G8172": ("λέβης", "", "a kettle", "lebēs"),
        "G8173": ("λειτούργημα", "", "ministration", "leitourgēma"),
        "G8174": ("λειτουργήσιμος", "", "ministration", "leitourgēsimos"),
        "G8175": ("λειχήν", "", "scabbed", "leichēn"),
        "G8176": ("λείχω", "", "to lick", "leichō"),
        "G8177": ("λεκάνη", "", "a pan", "lekanē"),
        "G8178": ("λέξις", "", "form of speech", "lexis"),
        "G8179": ("λεπίζω", "", "to peel", "lepizō"),
        "G8180": ("λέπισμα", "", "a peel", "lepisma"),
        "G8181": ("λεπράω", "G:V", "to become leprous", "lepraō"),
        "G8182": ("λεπρόομαι", "", "become leprous", "leproomai"),
        "G8183": ("λεπτύνω", "", "to make fine", "leptunō"),
        "G8184": ("λεπυρός", "", "rind", "lepuros"),
        "G8185": ("λέσχη", "", "intrigue", "leschē"),
        "G8186": ("λευκανθίζω", "G:V", "be white", "leukanthizō"),
        "G8187": ("λεύκη", "", "white poplar", "leukē"),
        "G8188": ("λεωπετρία", "", "a smooth rock", "leōpetria"),
        "G8189": ("λῆμμα", "", "a concern", "lēmma"),
        "G8190": ("λῃστήριον", "", "a band of robbers", "lēstērion"),
        "G8191": ("λιγύριον", "", "amber", "ligurion"),
        "G8192": ("λιθουργέω", "", "to cut stones", "lithourgeō"),
        "G8193": ("λιθουργικός", "", "stonecutting", "lithourgikos"),
        "G8194": ("λικμήτωρ", "", "a winnower", "likmētōr"),
        "G8195": ("λικμός", "", "a winnowing shovel", "likmos"),
        "G8196": ("λιμά", "", "lima", "lima"),
        "G8197": ("λιμαγχονέω", "", "to cause to hunger", "limagchoneō"),
        "G8198": ("λιμοκτονέω", "", "to famish", "limoktoneō"),
        "G8199": ("λιμώσσω", "G:V", "be famished", "limōssō"),
        "G8200": ("λίνεος", "", "of flaxen linen", "lineos"),
        "G8201": ("λινοκαλάμη", "", "a stalk of flax", "linokalamē"),
        "G8202": ("λιπαίνω", "", "to anoint", "lipainō"),
        "G8203": ("λίπασμα", "", "fatness", "lipasma"),
        "G8204": ("λίσσομαι", "", "to implore", "lissomai"),
        "G8205": ("λιτανεύω", "", "to implore", "litaneuō"),
        "G8206": ("λιτός", "", "cheap", "litos"),
        "G8207": ("λοβός", "", "a lobe", "lobos"),
        "G8208": ("λοιδόρησις", "", "reviling", "loidorēsis"),
        "G8209": ("λοιμεύομαι", "", "to injure", "loimeuomai"),
        "G8210": ("λουτήρ", "", "a bathing tub", "loutēr"),
        "G8211": ("λουτρών", "", "a bathhouse", "loutrōn"),
        "G8212": ("λοφιά", "", "a ridge", "lophia"),
        "G8213": ("λοχεύω", "", "to give birth", "locheuō"),
        "G8214": ("λυπηρός", "", "distressing", "lupēros"),
        "G8215": ("λῶμα", "", "a hem", "lōma"),
        "G8216": ("μᾶ", "", "by!", "ma"),
        "G8217": ("μαασφάρ", "", "a number", "maasphar"),
        "G8218": ("μαγειρεῖον", "", "a cooking place", "mageireion"),
        "G8219": ("μαγειρεύω", "", "to cut up meat", "mageireuō"),
        "G8220": ("μαγείρισσα", "", "a female cook", "mageirissa"),
        "G8221": ("μάγειρος", "", "a male cook", "mageiros"),
        "G8222": ("μαγίς", "", "a loaf", "magis"),
        "G8223": ("μαδήση", "", "loose of hair", "madēsē"),
        "G8224": ("μαδαρόω", "", "to pluck hair", "madaroō"),
        "G8225": ("μαελέθ", "", "a harp", "maeleth"),
        "G8226": ("μαζουρώθ", "", "mazuroth", "mazourōth"),
        "G8227": ("μάθημα", "", "a lesson", "mathēma"),
        "G8228": ("μαῖα", "", "a midwife", "maia"),
        "G8229": ("μαιμάσσω", "G:V", "be irresistibly led", "maimassō"),
        "G8230": ("μαιόομαι", "", "to act as midwife", "maioomai"),
        "G8231": ("μακαριστός", "", "most blessed", "makaristos"),
        "G8232": ("μακρόβιος", "", "long-lived", "makrobios"),
        "G8233": ("μακροημερεύω", "", "to prolong one's days", "makroēmereuō"),
        "G8234": ("μακροήμερος", "", "many days", "makroēmeros"),
        "G8235": ("μακρόθυμος", "", "lenient", "makrothumos"),
        "G8236": ("μακρότης", "", "duration", "makrotēs"),
        "G8237": ("μακροχρονέω", "", "to live a long time", "makrochroneō"),
        "G8238": ("μάκρυμμα", "", "abomination", "makrumma"),
        "G8239": ("μακρύνω", "G:V", "to prolong", "makrunō"),
        "G8240": ("μάλα", "", "by all means", "mala"),
        "G8241": ("μάλαγμα", "", "a dressing", "malagma"),
        "G8242": ("μαλακίζομαι", "", "be infirm", "malakizomai"),
        "G8243": ("μαλακύνω", "", "to soften", "malakunō"),
        "G8244": ("μαλακῶς", "", "softly", "malakōs"),
        "G8245": ("μάν", "", "manna", "man"),
        "G8246": ("μαναά", "", "a gift", "manaa"),
        "G8247": ("μάνδρα", "", "haven", "mandra"),
        "G8248": ("μανδραγόρας", "", "mandrake", "mandragoras"),
        "G8249": ("μανδύας", "", "a uniform", "manduas"),
        "G8250": ("μανή", "", "mane", "manē"),
        "G8251": ("μανιάκης", "", "a golden necklace", "maniakēs"),
        "G8252": ("μαντεία", "", "divination", "manteia"),
        "G8253": ("μαντεῖον", "", "an oracle", "manteion"),
        "G8254": ("μάντις", "", "a clairvoyant", "mantis"),
        "G8255": ("μαρμάρινος", "", "of marble", "marmarinos"),
        "G8256": ("μαρσίππιον", "", "a money bag", "marsippion"),
        "G8257": ("μάρσιππος", "", "a bag", "marsippos"),
        "G8258": ("μασαναί", "", "second section", "masanai"),
        "G8259": ("μασμαρώθ", "", "a snuffer", "masmarōth"),
        "G8260": ("ματαίως", "", "acting in folly", "mataiōs"),
        "G8261": ("μαχβάρ", "", "a rag", "machbar"),
        "G8262": ("μαχεήλ", "", "food", "macheēl"),
        "G8263": ("μαχητής", "", "a warrior", "machētēs"),
        "G8264": ("μάχιμος", "", "combat", "machimos"),
        "G8265": ("μαωζείν", "", "a fortress", "maōzein"),
        "G8266": ("μεγαλοπρέπεια", "", "majesty", "megaloprepeia"),
        "G8267": ("μεγαλοπτέρυγος", "", "having large wings", "megalopterugos"),
        "G8268": ("μεγαλορρημονέω", "", "to speak great words", "megalorrēmoneō"),
        "G8269": ("μεγαλορρημοσύνη", "", "lofty language", "megalorrēmosunē"),
        "G8270": ("μεγαλορρήμων", "", "lofty speaking", "megalorrēmōn"),
        "G8271": ("μεγαλόσαρκος", "", "fleshy", "megalosarkos"),
        "G8272": ("μεγαλόφρων", "", "high-minded", "megalophrōn"),
        "G8273": ("μεγάλωμα", "", "magnificence", "megalōma"),
        "G8274": ("μεθοδεύω", "", "to use craft", "methodeuō"),
        "G8275": ("μέθυσμα", "", "strong drink", "methusma"),
        "G8276": ("μεθωεσίμ", "", "a genealogical register", "methōesim"),
        "G8277": ("μέλαθρον", "", "a ridge", "melathron"),
        "G8278": ("μελαθρόω", "", "to arch over", "melathroō"),
        "G8279": ("μελάνθιον", "", "pepperwort", "melanthion"),
        "G8280": ("μελανόομαι", "", "to blacken", "melanoomai"),
        "G8281": ("μελέτη", "", "meditation", "meletē"),
        "G8282": ("μελίζω", "", "to dismember", "melizō"),
        "G8283": ("μέλισσα", "", "bee", "melissa"),
        "G8284": ("μελισσών", "", "an apiary", "melissōn"),
        "G8285": ("μέμψις", "", "complaint", "mempsis"),
        "G8286": ("μέντοιγε", "", "however", "mentoige"),
        "G8287": ("μεριτεύομαι", "", "to partition", "meriteuomai"),
        "G8288": ("μεσαῖος", "", "middle", "mesaios"),
        "G8289": ("μέσαντιον", "", "a weaver's beam", "mesantion"),
        "G8290": ("μεσημβρινός", "", "midday", "mesēmbrinos"),
        "G8291": ("μεσθάαλ", "", "a wardrobe", "mesthaal"),
        "G8292": ("μεσοπόρφυρος", "", "purple ornamentation", "mesoporphuros"),
        "G8293": ("μεσσάβ", "", "a garrison", "messab"),
        "G8294": ("μεταβολή", "", "revolt", "metabolē"),
        "G8295": ("μετάβολος", "", "a trader", "metabolos"),
        "G8296": ("μετακίνησις", "", "removal", "metakinēsis"),
        "G8297": ("μεταλλεύω", "", "to mine", "metalleuō"),
        "G8298": ("μεταμέλεια", "", "repentance", "metameleia"),
        "G8299": ("μετάμελος", "", "regret", "metamelos"),
        "G8300": ("μεταναστεύω", "", "to migrate", "metanasteuō"),
        "G8301": ("μετανίστημι", "", "to change residence", "metanistēmi"),
        "G8302": ("μεταπίπτω", "", "to degenerate", "metapiptō"),
        "G8303": ("μετασκευάζω", "", "to fashion differently", "metaskeuazō"),
        "G8304": ("μεταστροφή", "", "converted", "metastrophē"),
        "G8305": ("μετάφρενον", "", "the upper back", "metaphrenon"),
        "G8306": ("μετέρχομαι", "", "to pass between the ranks", "meterchomai"),
        "G8307": ("μετεωρισμός", "", "a crest", "meteōrismos"),
        "G8308": ("μετέωρος", "", "a meteorite", "meteōros"),
        "G8309": ("μετοικέω", "", "to displace", "metoikeō"),
        "G8310": ("μετοικία", "", "displacement", "metoikia"),
        "G8311": ("μέτοικος", "", "a refugee", "metoikos"),
        "G8312": ("μέτρησις", "", "a measuring", "metrēsis"),
        "G8313": ("μετριάζω", "G:V", "be content", "metriazō"),
        "G8314": ("μεχωνώθ", "", "a base", "mechōnōth"),
        "G8315": ("μηθέτερος", "", "neither one", "mētheteros"),
        "G8316": ("μῆλον", "", "cheeks", "mēlon"),
        "G8317": ("μηνιαῖος", "", "of a month", "mēniaios"),
        "G8318": ("μῆνις", "", "vehement anger", "mēnis"),
        "G8319": ("μηνίσκος", "", "a crescent", "mēniskos"),
        "G8320": ("μηνίω", "G:V", "be infuriated", "mēniō"),
        "G8321": ("μηρία", "", "the thigh", "mēria"),
        "G8322": ("μηρυκάομαι", "", "to chew the cud", "mērukaomai"),
        "G8323": ("μηρυκισμός", "", "chewing the cud", "mērukismos"),
        "G8324": ("μηρύομαι", "", "to furl", "mēruomai"),
        "G8325": ("μηχανεύομαι", "", "to construct", "mēchaneuomai"),
        "G8326": ("μηχανή", "", "a machine", "mēchanē"),
        "G8327": ("μίανσις", "", "defilement", "miansis"),
        "G8328": ("μικρότης", "", "thinness", "mikrotēs"),
        "G8329": ("μίλτος", "", "vermilion", "miltos"),
        "G8330": ("μίσγω", "", "to mingle", "misgō"),
        "G8331": ("μισητός", "", "detested", "misētos"),
        "G8332": ("μῖσος", "", "hatred", "misos"),
        "G8333": ("μίτρα", "", "a mitre", "mitra"),
        "G8334": ("μνημόσυνος", "", "a memorandum", "mnēmosunos"),
        "G8335": ("μνησικακέω", "", "to resent", "mnēsikakeō"),
        "G8336": ("μνησίκακος", "", "resentful", "mnēsikakos"),
        "G8337": ("μόλιβδος", "", "lead", "molibdos"),
        "G8338": ("μολόχη", "", "a mallow (plant)", "molochē"),
        "G8339": ("μόλυνσις", "", "contamination", "molunsis"),
        "G8340": ("μονάζω", "", "to live alone", "monazō"),
        "G8341": ("μόνιμος", "", "a stable", "monimos"),
        "G8342": ("μονιός", "", "a wild boar", "monios"),
        "G8343": ("μονόζωνος", "", "lightly armed", "monozōnos"),
        "G8344": ("μονόκερως", "", "a unicorn", "monokerōs"),
        "G8345": ("μονομαχέω", "", "to fight one on one", "monomacheō"),
        "G8346": ("μόνορχις", "", "having one testicle", "monorchis"),
        "G8347": ("μονότροπος", "", "simple", "monotropos"),
        "G8348": ("μοσφαθαϊμ", "", "sheepfolds", "mosphathaim"),
        "G8349": ("μοσχάριον", "", "a young calf", "moscharion"),
        "G8350": ("μοτόω", "", "to dress a wound", "motoō"),
        "G8351": ("μοχθέω", "", "to make an effort", "mochtheō"),
        "G8352": ("μοχλός", "", "a bar", "mochlos"),
        "G8353": ("μυγαλῆ", "", "a field-mouse", "mugalē"),
        "G8354": ("μυελόομαι", "", "to fill with marrow", "mueloomai"),
        "G8355": ("μυῖα", "", "a fly", "muia"),
        "G8356": ("μυκτήρ", "", "nostril", "muktēr"),
        "G8357": ("μυκτηρισμός", "", "sneering", "muktērismos"),
        "G8358": ("μύλη", "", "a molar tooth", "mulē"),
        "G8359": ("μυξωτῆρες", "", "a tube", "muxōtēres"),
        "G8360": ("μυρεψικός", "", "scented", "murepsikos"),
        "G8361": ("μυρεψός", "", "a perfumer", "murepsos"),
        "G8362": ("μυριοπλάσιος", "", "ten thousand-fold", "murioplasios"),
        "G8363": ("μυρμηκιάω", "G:V", "be troubled with warts", "murmēkiaō"),
        "G8364": ("μυρμηκολέων", "", "a small lion", "murmēkoleōn"),
        "G8365": ("μύρμηξ", "", "an ant", "murmēx"),
        "G8366": ("μυρσίνη", "", "a myrtle tree", "mursinē"),
        "G8367": ("μῦς", "", "a mouse", "mus"),
        "G8368": ("μυσαρός", "", "detestable", "musaros"),
        "G8369": ("μύσταξ", "", "a mustache", "mustax"),
        "G8370": ("μωκάομαι", "", "to scorn", "mōkaomai"),
        "G8371": ("μωμητός", "", "scoffing", "mōmētos"),
        "G8372": ("νάβλα", "", "a stringed instrument", "nabla"),
        "G8373": ("ναίω", "", "to inhabit", "naiō"),
        "G8374": ("νᾶμα", "", "juice", "nama"),
        "G8375": ("νάπη", "", "a grove", "napē"),
        "G8376": ("ναρκάω", "G:V", "be paralyzed", "narkaō"),
        "G8377": ("ναῦλον", "", "fare", "naulon"),
        "G8378": ("ναυτικός", "", "mariner", "nautikos"),
        "G8379": ("νεᾶνις", "", "a young woman", "neanis"),
        "G8380": ("νέβελ", "", "a hide flask", "nebel"),
        "G8381": ("νεβρός", "", "a fawn", "nebros"),
        "G8382": ("νεέλασσα", "", "ostriches", "neelassa"),
        "G8383": ("νεζέρ", "", "a diadem", "nezer"),
        "G8384": ("νεῖκος", "", "altercation", "neikos"),
        "G8385": ("νέμω", "", "to feed", "nemō"),
        "G8386": ("νεόω", "", "to plow fallow land", "neoō"),
        "G8387": ("νέσσα", "", "feathers", "nessa"),
        "G8388": ("νεῦμα", "", "a beckon", "neuma"),
        "G8389": ("νευρά", "", "the bow string", "neura"),
        "G8390": ("νευροκοπέω", "", "to hamstring", "neurokopeō"),
        "G8391": ("νεῦρον", "", "a nerve", "neuron"),
        "G8392": ("νεχωθά", "", "spices", "nechōtha"),
        "G8393": ("νέωμα", "", "a field prepared for plowing", "neōma"),
        "G8394": ("νηπιότης", "", "infancy", "nēpiotēs"),
        "G8395": ("νηστός", "", "spun", "nēstos"),
        "G8396": ("νικοποιέω", "", "to bring success", "nikopoieō"),
        "G8397": ("Νισάν", "", "Nisan", "Nisan"),
        "G8398": ("νίτρον", "", "bleach", "nitron"),
        "G8399": ("νιφετός", "", "snowflakes", "niphetos"),
        "G8400": ("νοήμων", "", "intelligent", "noēmōn"),
        "G8401": ("νοητῶς", "", "intelligible", "noētōs"),
        "G8402": ("νομάς", "", "feeding", "nomas"),
        "G8403": ("νόμιμος", "", "laws", "nomimos"),
        "G8404": ("νομοθέσμως", "", "lawfully", "nomothesmōs"),
        "G8405": ("νομός", "", "abode", "nomos"),
        "G8406": ("νοσερός", "", "diseased", "noseros"),
        "G8407": ("νοσσεύω", "", "to nest", "nosseuō"),
        "G8408": ("νοσσοποιέω", "", "to build a nest", "nossopoieō"),
        "G8409": ("νωτόφορος", "", "a load carrier", "nōtophoros"),
        "G8410": ("νουθέτημα", "", "admonition", "nouthetēma"),
        "G8411": ("νουθέτησις", "", "admonition", "nouthetēsis"),
        "G8412": ("νυκτερινός", "", "nightly", "nukterinos"),
        "G8413": ("νυκτερίς", "", "a bat", "nukteris"),
        "G8414": ("νυκτικόραξ", "", "an owl", "nuktikorax"),
        "G8415": ("νυμφαγωγός", "", "groomsman", "numphagōgos"),
        "G8416": ("νύμφευσις", "", "a betrothing", "numpheusis"),
        "G8417": ("νύσταγμα", "", "a slumbering", "nustagma"),
        "G8418": ("νυσταγμός", "", "a slumber", "nustagmos"),
        "G8419": ("νωθροκάρδιος", "", "dull of heart", "nōthrokardios"),
        "G8420": ("νωκήδ", "", "a shepherd", "nōkēd"),
        "G8421": ("ξ", "", "60", "x"),
        "G8422": ("ξανθίζω", "G:V", "be yellowish", "xanthizō"),
        "G8423": ("ξένιος", "", "gifts", "xenios"),
        "G8424": ("ξενισμός", "", "hospitality", "xenismos"),
        "G8425": ("ξηρασία", "", "dryness", "xērasia"),
        "G8426": ("ξίφος", "", "a  knife-blade", "xiphos"),
        "G8427": ("ξυλάριον", "", "a stick", "xularion"),
        "G8428": ("ξυλοκόπος", "", "a woodcutter", "xulokopos"),
        "G8429": ("ξυλοφορία", "", "bearing wood", "xulophoria"),
        "G8430": ("ξυλοφόρος", "", "a wood bearer", "xulophoros"),
        "G8431": ("ξυλόω", "", "to board", "xuloō"),
        "G8432": ("ξυνωρίς", "", "a double-horse chariot", "xunōris"),
        "G8433": ("ξύρησις", "", "shaving", "xurēsis"),
        "G8434": ("ξυρόν", "", "a razor", "xuron"),
        "G8435": ("ξυστός", "", "planed", "xustos"),
        "G8436": ("ξύω", "", "to scrape", "xuō"),
        "G8437": ("ὀβελίσκος", "", "a point", "obeliskos"),
        "G8438": ("ὀβολός", "", "a obolus", "obolos"),
        "G8439": ("ογδοηκονταέξ", "", "eighty-six", "ogdoēkontaex"),
        "G8440": ("ογδοηκονταπέντε", "", "eighty-five", "ogdoēkontapente"),
        "G8441": ("ὀγδοηκοντατέσσαρες", "", "eighty-four", "ogdoēkontatessares"),
        "G8442": ("ογδοηκοντατρεῖς", "", "eighty-three", "ogdoēkontatreis"),
        "G8443": ("ὀγδοηκοστός", "", "eightieth", "ogdoēkostos"),
        "G8444": ("ὁδοιπόρος", "", "a traveler", "hodoiporos"),
        "G8445": ("ὁδοποιέω", "", "to open a way", "hodopoieō"),
        "G8446": ("ὀδυνηρός", "", "grievous", "odunēros"),
        "G8447": ("ὀδύρομαι", "", "to grieve", "oduromai"),
        "G8448": ("οἰακίζω", "", "to steer", "oiakizō"),
        "G8449": ("οἰκειότης", "", "intimacy", "oikeiotēs"),
        "G8450": ("οἰκέτις", "", "a female domestic servant", "oiketis"),
        "G8451": ("οἴκησις", "", "place of abode", "oikēsis"),
        "G8452": ("οἰκητός", "", "inhabited", "oikētos"),
        "G8453": ("οἰκήτωρ", "", "an inhabitant", "oikētōr"),
        "G8454": ("οἰκίζω", "", "to resettle", "oikizō"),
        "G8455": ("οἰκίσκος", "", "niche", "oikiskos"),
        "G8456": ("οἰκογενής", "", "a native-born servant", "oikogenēs"),
        "G8457": ("οἰκόπεδον", "", "an area", "oikopedon"),
        "G8458": ("οἰκτείρημα", "", "pitying", "oikteirēma"),
        "G8459": ("οἶκτος", "", "a lament", "oiktos"),
        "G8460": ("οἰκτρός", "", "pitiable", "oiktros"),
        "G8461": ("οἴμοι", "", "alas!", "oimoi"),
        "G8462": ("οἰνοποτέω", "", "to drink wine", "oinopoteō"),
        "G8463": ("οἰνοφλυγέω", "G:V", "be drunk with wine", "oinophlugeō"),
        "G8464": ("οἰνοχόη", "", "a female wine server", "oinochoē"),
        "G8465": ("οἰνοχόος", "", "a male wine server", "oinochoos"),
        "G8466": ("οἰνόω", "G:V", "be drunk with wine", "oinoō"),
        "G8467": ("οἰφί", "", "an ephah", "oiphi"),
        "G8468": ("οἴχομαι", "", "be undone", "oichomai"),
        "G8469": ("οἰωνίζομαι", "", "to foretell", "oiōnizomai"),
        "G8470": ("οἰώνισμα", "", "an omen", "oiōnisma"),
        "G8471": ("οἰωνισμός", "", "an omen", "oiōnismos"),
        "G8472": ("οἰωνός", "", "an omen", "oiōnos"),
        "G8473": ("ὀκλάζω", "", "to kneel", "oklazō"),
        "G8474": ("ὀκνηρία", "", "slothful", "oknēria"),
        "G8475": ("ὀκτακισχίλιοι", "", "eight thousand", "oktakischilioi"),
        "G8476": ("ὀκτακόσιοι", "", "eight hundred", "oktakosioi"),
        "G8477": ("ὀκτάπηχυς", "", "eight cubits", "oktapēchus"),
        "G8478": ("ὀκτωκαίδεκα", "", "eighteen", "oktōkaideka"),
        "G8479": ("ὀκτωκαιδέκατος", "", "eighteenth", "oktōkaidekatos"),
        "G8480": ("ὀλέθριος", "", "pernicious", "olethrios"),
        "G8481": ("ὀλέκω", "", "to destroy", "olekō"),
        "G8482": ("ὀλιγόβιος", "", "short-lived", "oligobios"),
        "G8483": ("ὀλιγοστός", "", "very few", "oligostos"),
        "G8484": ("ὀλιγότης", "", "fewness", "oligotēs"),
        "G8485": ("ὀλιγοψυχέω", "G:V", "be faint-hearted", "oligopsucheō"),
        "G8486": ("ὀλιγοψυχία", "", "faint-heartedness", "oligopsuchia"),
        "G8487": ("ὀλιγόω", "", "to lessen", "oligoō"),
        "G8488": ("ὀλισθάνω", "", "to slip", "olisthanō"),
        "G8489": ("ὀλίσθημα", "", "a slip", "olisthēma"),
        "G8490": ("ὁλκή", "", "a scale-weight", "holkē"),
        "G8491": ("ὄλλυμι", "", "to destroy", "ollumi"),
        "G8492": ("ὁλοκάρπωμα", "", "a whole offering", "holokarpōma"),
        "G8493": ("ὁλοκάρπωσις", "", "a whole offering", "holokarpōsis"),
        "G8494": ("ὁλόκαυτος", "", "wholly burnt", "holokautos"),
        "G8495": ("ὁλοκαύτωσις", "", "bringing of a whole burnt-offering", "holokautōsis"),
        "G8496": ("ὀλολυγμός", "", "shrieking of women", "ololugmos"),
        "G8497": ("ὁλοπόρφυρος", "", "entirely of purple", "holoporphuros"),
        "G8498": ("ὁλόρριζος", "", "entirely from a root", "holorrizos"),
        "G8499": ("ὁλοσχερῶς", "", "soundly", "holoscherōs"),
        "G8500": ("ὄλυρα", "", "a wild oat", "olura"),
        "G8501": ("ὀλυρίτης", "", "of oaten bread", "oluritēs"),
        "G8502": ("ὁμαλίζω", "", "to level", "homalizō"),
        "G8503": ("ὁμαλισμός", "", "leveling", "homalismos"),
        "G8504": ("ὄμβρημα", "", "a shower", "ombrēma"),
        "G8505": ("ὅμηρος", "", "a security treaty", "homēros"),
        "G8506": ("ὁμολόγως", "", "confessedly", "homologōs"),
        "G8507": ("ὁμομήτριος", "", "born of the same mother", "homomētrios"),
        "G8508": ("ὁμονοέω", "", "to consent", "homonoeō"),
        "G8509": ("ὁμόνοια", "", "concord", "homonoia"),
        "G8510": ("ὁμοπάτριος", "", "of the same father", "homopatrios"),
        "G8511": ("ὁμορέω", "", "to adjoin", "homoreō"),
        "G8512": ("ὅμορος", "", "adjoining", "homoros"),
        "G8513": ("ὀμφακίζω", "G:V", "be sour", "omphakizō"),
        "G8514": ("ὀμφαλός", "", "anavel", "omphalos"),
        "G8515": ("ὄμφαξ", "", "an unripe grape", "omphax"),
        "G8516": ("ὄναγρος", "", "a wild donkey", "onagros"),
        "G8517": ("ὀνείδισμα", "", "a scorning", "oneidisma"),
        "G8518": ("ὄνησις", "", "profitable", "onēsis"),
        "G8519": ("ὀνοκένταυρος", "", "a satyr", "onokentauros"),
        "G8520": ("ὀνομαστός", "", "famous", "onomastos"),
        "G8521": ("ὄνυξ", "", "a claw", "onux"),
        "G8522": ("ὀνυχίζω", "", "to claw", "onuchizō"),
        "G8523": ("ὀνύχιον", "", "onyx", "onuchion"),
        "G8524": ("ὀνυχιστήρ", "", "cloven-footed", "onuchistēr"),
        "G8525": ("ὀξέως", "", "swiftly", "oxeōs"),
        "G8526": ("ὀξυγράφος", "", "writing fast", "oxugraphos"),
        "G8527": ("ὀξύθυμος", "", "quick to rage", "oxuthumos"),
        "G8528": ("ὀξύνω", "", "to sharpen", "oxunō"),
        "G8529": ("ὀξύτης", "", "swift", "oxutēs"),
        "G8530": ("ὀπήτιον", "", "a shoemaker's awl", "opētion"),
        "G8531": ("ὀπίσθιος", "", "posterior", "opisthios"),
        "G8532": ("ὀπισθίως", "", "backwardly", "opisthiōs"),
        "G8533": ("ὀπισθότονος", "", "convulsion", "opisthotonos"),
        "G8534": ("ὀπισθοφανῶς", "", "backwards", "opisthophanōs"),
        "G8535": ("ὁπλή", "", "a hoof", "hoplē"),
        "G8536": ("ὁπλίτης", "", "armed with a large shield", "hoplitēs"),
        "G8537": ("ὁπλοθήκη", "", "an armory", "hoplothēkē"),
        "G8538": ("ὁπλομάχος", "", "an armed warrior", "hoplomachos"),
        "G8539": ("ὁπλοφόρος", "", "an armor-bearer", "hoplophoros"),
        "G8540": ("ὀπτάζομαι", "", "to behold", "optazomai"),
        "G8541": ("ὀπτάω", "", "to bake", "optaō"),
        "G8542": ("ὀπωροφυλάκιον", "", "a storehouse of fruits", "opōrophulakion"),
        "G8543": ("ὁρατής", "", "an observer", "horatēs"),
        "G8544": ("ὁρατικός", "", "observant", "horatikos"),
        "G8545": ("ὄργανον", "", "an instrument", "organon"),
        "G8546": ("ὀρθόω", "G:V", "be straight up", "orthoō"),
        "G8547": ("ὁρισμός", "", "enactment", "horismos"),
        "G8548": ("ὁρκισμός", "", "a conjuring", "horkismos"),
        "G8549": ("ὁρμίσκος", "", "pendant", "hormiskos"),
        "G8550": ("ὅρμος", "", "a mooring", "hormos"),
        "G8551": ("ὀρνίθιον", "", "a small bird", "ornithion"),
        "G8552": ("ὀρνιθοσκοπέομαι", "", "to use augury", "ornithoskopeomai"),
        "G8553": ("ὀρόδαμνος", "", "a scion", "orodamnos"),
        "G8554": ("ὅρος", "", "a confirmation", "horos"),
        "G8555": ("ὀρόφωμα", "", "roofing", "orophōma"),
        "G8556": ("ὀρτυγομήτρα", "", "a mother-quail", "ortugomētra"),
        "G8557": ("ὄρυξ", "", "a gazelle", "orux"),
        "G8558": ("ὀρφανία", "", "bereavement", "orphania"),
        "G8559": ("ὁσιόω", "G:V", "to purify", "hosioō"),
        "G8560": ("ὁστισοῦν", "", "whatsoever", "hostisoun"),
        "G8561": ("ὄστρακον", "", "a potsherd", "ostrakon"),
        "G8562": ("ὀσφραίνομαι", "", "to savor", "osphrainomai"),
        "G8563": ("ὀσφρασία", "", "a smell", "osphrasia"),
        "G8564": ("οὐδαμοῦ", "", "not at all", "oudamou"),
        "G8565": ("ουδὲ μή", "G:Neg+G:Conj + G:Neg", "neither", "oude mē"),
        "G8566": ("οὐλή", "", "a discoloration", "oulē"),
        "G8567": ("οὐραγία", "", "a rear guard", "ouragia"),
        "G8568": ("οὐρέω", "", "to urinate", "oureō"),
        "G8569": ("οὔριος", "", "a rotten egg", "ourios"),
        "G8570": ("οὖρον", "", "urine", "ouron"),
        "G8571": ("ὀφθαλμοφανῶς", "", "clear to the eyes", "ophthalmophanōs"),
        "G8572": ("ὀφιομάχης", "", "a crawling locust", "ophiomachēs"),
        "G8573": ("ὄχθη", "", "a riverbank", "ochthē"),
        "G8574": ("ὀχληρία", "", "rioting", "ochlēria"),
        "G8575": ("ὀχυρός", "", "fortified", "ochuros"),
        "G8576": ("ὀχυρόω", "", "to fortify", "ochuroō"),
        "G8577": ("ὀψίζω", "", "to arrive late", "opsizō"),
        "G8578": ("ὄψον", "", "fish", "opson"),
        "G8579": ("παγετός", "", "icy coldness", "pagetos"),
        "G8580": ("πάγος", "", "ice", "pagos"),
        "G8581": ("παθεινός", "", "a mourner", "patheinos"),
        "G8582": ("παιγνία", "", "play", "paignia"),
        "G8583": ("παίγνιον", "", "a plaything", "paignion"),
        "G8584": ("παλάθη", "", "a dried cluster", "palathē"),
        "G8585": ("παλαιστή", "", "a palm (plant)", "palaistē"),
        "G8586": ("παλαίω", "", "to wrestle", "palaiō"),
        "G8587": ("παλαίωμα", "", "that which has grown old", "palaiōma"),
        "G8588": ("παλαίωσις", "", "that which has grown old", "palaiōsis"),
        "G8589": ("παλλακή", "", "a concubine", "pallakē"),
        "G8590": ("παλλακίς", "", "a concubine", "pallakis"),
        "G8591": ("πάλλω", "G:V", "to agitate", "pallō"),
        "G8592": ("παμβότανον", "", "abundant herbage", "pambotanon"),
        "G8593": ("πανδημεί", "", "in full assembly", "pandēmei"),
        "G8594": ("πανηγυρίζω", "", "to assemble for a festival", "panēgurizō"),
        "G8595": ("πάνθηρ", "", "a panther", "panthēr"),
        "G8596": ("πανοικία", "", "whole family", "panoikia"),
        "G8597": ("πανουργεύω", "", "to trick", "panourgeuō"),
        "G8598": ("παντοδαπός", "", "of all kinds", "pantodapos"),
        "G8599": ("πάπυρος", "", "papyrus (plant)", "papuros"),
        "G8600": ("παραβαπτός", "", "deeply dyed", "parabaptos"),
        "G8601": ("παραβιβάζω", "", "to cast aside", "parabibazō"),
        "G8602": ("παραβλέπω", "", "to look over", "parablepō"),
        "G8603": ("παράγγελμα", "", "a mobilization order", "paraggelma"),
        "G8604": ("παραγραφίς", "", "a scraper", "paragraphis"),
        "G8605": ("παραγωνίσκος", "", "a carpenter's square", "paragōniskos"),
        "G8606": ("παράδειγμα", "", "a model", "paradeigma"),
        "G8607": ("παραδείκνυμι", "", "to hold forth as an example", "paradeiknumi"),
        "G8608": ("παραδοξάζω", "", "to do an incredible thing", "paradoxazō"),
        "G8609": ("παραδρομή", "", "a passing by", "paradromē"),
        "G8610": ("παραζώνη", "", "a sash", "parazōnē"),
        "G8611": ("παραθερμαίνω", "", "to overheat", "parathermainō"),
        "G8612": ("παράθεσις", "", "provision", "parathesis"),
        "G8613": ("παραθλίβω", "", "to press against", "parathlibō"),
        "G8614": ("παραιρέω", "", "to lift from", "paraireō"),
        "G8615": ("παρακάθημαι", "", "to sit near", "parakathēmai"),
        "G8616": ("παρακατατίθημι", "", "to deposit in care of", "parakatatithēmi"),
        "G8617": ("παρακελεύομαι", "", "to rouse", "parakeleuomai"),
        "G8618": ("παρακλητικός", "", "comforting", "paraklētikos"),
        "G8619": ("παρακλήτωρ", "", "a comforter", "paraklētōr"),
        "G8620": ("παράκοιτος", "", "a mistress", "parakoitos"),
        "G8621": ("παρακρούω", "", "to cheat", "parakrouō"),
        "G8622": ("παραλαλέω", "", "to speak improperly", "paralaleō"),
        "G8623": ("παράλλαξις", "", "alteration", "parallaxis"),
        "G8624": ("παραλλάσσω", "", "to alter", "parallassō"),
        "G8625": ("παράλυσις", "", "paralysis", "paralusis"),
        "G8626": ("παραναλίσκω", "", "to uselessly consume", "paranaliskō"),
        "G8627": ("παράνομος", "", "a lawbreaker", "paranomos"),
        "G8628": ("παρανόμως", "", "unlawfully", "paranomōs"),
        "G8629": ("παραξιφίς", "", "a dagger", "paraxiphis"),
        "G8630": ("παράπαν", "", "thoroughly", "parapan"),
        "G8631": ("παραπέτασμα", "", "a canopy", "parapetasma"),
        "G8632": ("παράπληκτος", "", "deranged", "paraplēktos"),
        "G8633": ("παραπληξία", "", "derangement", "paraplēxia"),
        "G8634": ("παράπτωσις", "", "a transgression", "paraptōsis"),
        "G8635": ("παραρρίπτω", "", "to throw aside", "pararriptō"),
        "G8636": ("παράρυμα", "", "a sheet of leather", "pararuma"),
        "G8637": ("παρασιωπάω", "", "to remain silent", "parasiōpaō"),
        "G8638": ("παράστασις", "", "position", "parastasis"),
        "G8639": ("παρασυμβάλλομαι", "", "to resemble", "parasumballomai"),
        "G8640": ("παράταξις", "", "battle array", "parataxis"),
        "G8641": ("παρατάσσω", "", "to deploy", "paratassō"),
        "G8642": ("παρατρέχω", "G:V", "be a bodyguard", "paratrechō"),
        "G8643": ("παραφρόνησις", "", "ranting", "paraphronēsis"),
        "G8644": ("παραφυάς", "", "a shoot", "paraphuas"),
        "G8645": ("παρεδρεύω", "", "to attend upon", "paredreuō"),
        "G8646": ("παρεῖδω", "", "to ignore", "pareidō"),
        "G8647": ("παρεκτείνω", "", "to reach forth", "parekteinō"),
        "G8648": ("παρέλκυσις", "", "a deferment", "parelkusis"),
        "G8649": ("πάρεξ", "", "besides", "parex"),
        "G8650": ("παρεξίστημι", "", "to move out of place", "parexistēmi"),
        "G8651": ("παρθένια", "", "marks of virginity", "parthenia"),
        "G8652": ("παρθενικός", "", "virgin", "parthenikos"),
        "G8653": ("παριστάνω", "", "to present", "paristanō"),
        "G8654": ("παροδεύω", "", "to travel by", "parodeuō"),
        "G8655": ("παροικεσία", "", "a sojourning", "paroikesia"),
        "G8656": ("παροίκησις", "", "a sojourning", "paroikēsis"),
        "G8657": ("παροινέω", "", "to insult drunkenly", "paroineō"),
        "G8658": ("παροιστράω", "G:V", "be in heat", "paroistraō"),
        "G8659": ("παροράω", "", "to overlook", "paroraō"),
        "G8660": ("παρόργισμα", "", "provocation", "parorgisma"),
        "G8661": ("παρωμίς", "", "a shoulder strap", "parōmis"),
        "G8662": ("πάσσαλος", "", "a peg", "passalos"),
        "G8663": ("πάσσω", "", "to strew", "passō"),
        "G8664": ("παστός", "", "nuptial chamber", "pastos"),
        "G8665": ("παστοφόριον", "", "a chamber", "pastophorion"),
        "G8666": ("πάτημα", "", "trampled", "patēma"),
        "G8667": ("πατητός", "", "trampled", "patētos"),
        "G8668": ("πατράδελφος", "", "an uncle", "patradelphos"),
        "G8669": ("πάτραρχος", "", "a patriarchal idol", "patrarchos"),
        "G8670": ("πάτριος", "", "of a father", "patrios"),
        "G8671": ("παῦσις", "", "cessation", "pausis"),
        "G8672": ("πάχνη", "", "frost", "pachnē"),
        "G8673": ("πάχος", "", "thickness", "pachos"),
        "G8674": ("παχύς", "", "robust", "pachus"),
        "G8675": ("πεδάω", "", "to shackle", "pedaō"),
        "G8676": ("πεδίον", "", "a plain", "pedion"),
        "G8677": ("πεζός", "", "a footman", "pezos"),
        "G8678": ("πειρατεύω", "", "to maraud", "peirateuō"),
        "G8679": ("πειρατήριον", "", "a band of marauders", "peiratērion"),
        "G8680": ("πειρατής", "", "a marauder", "peiratēs"),
        "G8681": ("πέλας", "", "a one near", "pelas"),
        "G8682": ("πελεκάν", "", "pelican", "pelekan"),
        "G8683": ("πελεκάω", "", "to hew", "pelekaō"),
        "G8684": ("πελεκητός", "", "hewn", "pelekētos"),
        "G8685": ("πέλεκυς", "", "a hewing axe", "pelekus"),
        "G8686": ("πελιδνός", "", "dark colored", "pelidnos"),
        "G8687": ("πελιόομαι", "", "be darkened", "pelioomai"),
        "G8688": ("πελταστής", "", "armed with a small shield", "peltastēs"),
        "G8689": ("πέλτη", "", "a small shield", "peltē"),
        "G8690": ("πέλυξ", "", "a hewing axe", "pelux"),
        "G8691": ("πέμμα", "", "a cake", "pemma"),
        "G8692": ("πενθικός", "", "mournful", "penthikos"),
        "G8693": ("πενία", "", "poverty", "penia"),
        "G8694": ("πένομαι", "", "be in need", "penomai"),
        "G8695": ("πενταετής", "", "of five years", "pentaetēs"),
        "G8696": ("πεντάπηχυς", "", "of five cubits", "pentapēchus"),
        "G8697": ("πενταπλασίως", "", "five-fold", "pentaplasiōs"),
        "G8698": ("πενταπλοῦς", "", "five-fold way", "pentaplous"),
        "G8699": ("πεντεκαίδεκα", "", "fifteen", "pentekaideka"),
        "G8700": ("πεντηκονταδύο", "", "fifty-two", "pentēkontaduo"),
        "G8701": ("πεντηκονταέξ", "", "fifty-six", "pentēkontaex"),
        "G8702": ("πεντηκονταεπτά", "", "fifty-seven", "pentēkontaepta"),
        "G8703": ("πεντηκονταπέντε", "", "fifty-five", "pentēkontapente"),
        "G8704": ("πεντηκονταπέτης", "", "fifty years", "pentēkontapetēs"),
        "G8705": ("πεντηκόνταρχος", "", "a commander of fifty", "pentēkontarchos"),
        "G8706": ("πεντηκοντατρεῖς", "", "fifty-three", "pentēkontatreis"),
        "G8707": ("πεντηκοντέσσαρες", "", "fifty-four", "pentēkontessares"),
        "G8708": ("πεντηκοστός", "", "fiftieth", "pentēkostos"),
        "G8709": ("πέπειρος", "", "mature", "pepeiros"),
        "G8710": ("πεποιθότως", "", "securely", "pepoithotōs"),
        "G8711": ("πέπων", "", "a melon", "pepōn"),
        "G8712": ("περαίνω", "", "to achieve", "perainō"),
        "G8713": ("περασμός", "", "a limit", "perasmos"),
        "G8714": ("περάτης", "", "a traveler", "peratēs"),
        "G8715": ("πέρδιξ", "", "partridge", "perdix"),
        "G8716": ("περιαργυρόω", "", "to silver plate", "periarguroō"),
        "G8717": ("περίβλεπτος", "", "admired", "peribleptos"),
        "G8718": ("περίβλημα", "", "a wrap-around", "periblēma"),
        "G8719": ("περιβολή", "", "a wrap-around", "peribolē"),
        "G8720": ("περίβολος", "", "an enclosure", "peribolos"),
        "G8721": ("περιγίνομαι", "", "be an advantage", "periginomai"),
        "G8722": ("περιδειπνέω", "", "to eat at a wake", "perideipneō"),
        "G8723": ("περιδέξιος", "", "a right armband", "peridexios"),
        "G8724": ("περιδράσσομαι", "", "to grasp in the hands", "peridrassomai"),
        "G8725": ("περίειμι", "", "to remain over", "perieimi"),
        "G8726": ("περίζωμα", "", "a loincloth", "perizōma"),
        "G8727": ("περίθεμα", "", "an adornment", "perithema"),
        "G8728": ("περικαθαίρω", "", "to purge", "perikathairō"),
        "G8729": ("περικαθαρίζω", "", "to purge", "perikatharizō"),
        "G8730": ("περικάθημαι", "", "to besiege", "perikathēmai"),
        "G8731": ("περικαθίζω", "", "to besiege", "perikathizō"),
        "G8732": ("περικείρω", "", "to shave around", "perikeirō"),
        "G8733": ("περικνημίς", "", "a legging", "periknēmis"),
        "G8734": ("περικοσμέω", "", "to adorn sumptuously", "perikosmeō"),
        "G8735": ("περικύκλῳ", "", "surrounding", "perikuklō"),
        "G8736": ("περιλαμβάνω", "", "to embrace", "perilambanō"),
        "G8737": ("περίληψις", "", "an embrace", "perilēpsis"),
        "G8738": ("περίλοιπος", "", "residue", "periloipos"),
        "G8739": ("περίμετρον", "", "perimeter", "perimetron"),
        "G8740": ("περιοδεύω", "", "to travel about", "periodeuō"),
        "G8741": ("περίοδος", "", "a circuit", "periodos"),
        "G8742": ("περιοικοδομέω", "", "to enclose", "perioikodomeō"),
        "G8743": ("περιονυχίζω", "", "to trim nails", "perionuchizō"),
        "G8744": ("περιουσιασμός", "", "prized possession", "periousiasmos"),
        "G8745": ("περίπατος", "", "a promenade", "peripatos"),
        "G8746": ("περιπλέκω", "", "to twist", "periplekō"),
        "G8747": ("περιπόλιον", "", "a township", "peripolion"),
        "G8748": ("περιπόρφυρος", "", "objects edged with purple", "periporphuros"),
        "G8749": ("περίπτερος", "", "sparks", "peripteros"),
        "G8750": ("περίπτωμα", "", "by chance", "periptōma"),
        "G8751": ("περιρραίνω", "", "to sprinkle about", "perirrainō"),
        "G8752": ("περισιαλόομαι", "", "to inlay", "perisialoomai"),
        "G8753": ("περισκελής", "", "pants", "periskelēs"),
        "G8754": ("περισκέπω", "", "to cover", "periskepō"),
        "G8755": ("περισπασμός", "", "distraction", "perispasmos"),
        "G8756": ("περισπόρια", "", "outskirts", "perisporia"),
        "G8757": ("περίστασις", "", "a surrounding space", "peristasis"),
        "G8758": ("περιστέλλω", "", "to screen", "peristellō"),
        "G8759": ("περιστήθιον", "", "a breast-plate", "peristēthion"),
        "G8760": ("περιστολή", "", "attire", "peristolē"),
        "G8761": ("περιστόμιος", "", "a cleft", "peristomios"),
        "G8762": ("περιστρέφω", "", "to move around", "peristrephō"),
        "G8763": ("περίστυλος", "", "a peristyle", "peristulos"),
        "G8764": ("περισύρω", "", "to tear away", "perisurō"),
        "G8765": ("περισχίζω", "", "to split", "perischizō"),
        "G8766": ("περιτειχίζω", "", "to wall around", "periteichizō"),
        "G8767": ("περίτειχος", "", "a rampart", "periteichos"),
        "G8768": ("περιφέρεια", "", "madness", "periphereia"),
        "G8769": ("περιφερής", "", "circumference", "peripherēs"),
        "G8770": ("περιφορά", "", "deviation", "periphora"),
        "G8771": ("περιφράσσω", "", "to enclose", "periphrassō"),
        "G8772": ("περιχαλκόω", "", "to brass plate", "perichalkoō"),
        "G8773": ("περιχαρακόω", "", "to surround with a rampart", "pericharakoō"),
        "G8774": ("περιχαρής", "", "overjoyed", "pericharēs"),
        "G8775": ("περιχέω", "", "to pour about", "pericheō"),
        "G8776": ("περιχρυσόω", "", "to gild", "perichrusoō"),
        "G8777": ("περκάζω", "", "to grow dark", "perkazō"),
        "G8778": ("πέσσω", "", "to bake", "pessō"),
        "G8779": ("πέταλον", "", "a leaf", "petalon"),
        "G8780": ("πετάννυμι", "", "to spread out", "petannumi"),
        "G8781": ("πέταυρον", "", "a perch", "petauron"),
        "G8782": ("πέτρινος", "", "of rock", "petrinos"),
        "G8783": ("πετροβόλος", "", "a slinger", "petrobolos"),
        "G8784": ("πεύκη", "", "pine tree", "peukē"),
        "G8785": ("πεύκινος", "", "of pine", "peukinos"),
        "G8786": ("πέψις", "", "baking", "pepsis"),
        "G8787": ("πῆγμα", "", "a bank of water", "pēgma"),
        "G8788": ("πηδάω", "", "to spring up", "pēdaō"),
        "G8789": ("πήλινος", "", "of clay", "pēlinos"),
        "G8790": ("πιαίνω", "", "to fatten", "piainō"),
        "G8791": ("πίθηκος", "", "ape", "pithēkos"),
        "G8792": ("πίθος", "", "a cask", "pithos"),
        "G8793": ("πικρασμός", "", "bitterness", "pikrasmos"),
        "G8794": ("πικρίς", "", "a bitter herb", "pikris"),
        "G8795": ("πίνινος", "", "of mother of pearl", "pininos"),
        "G8796": ("πίσσα", "", "pitch", "pissa"),
        "G8797": ("πίτυς", "", "pine tree", "pitus"),
        "G8798": ("πίων", "", "plentiful", "piōn"),
        "G8799": ("πλαγιάζω", "G:V", "to bend", "plagiazō"),
        "G8800": ("πλάγιος", "", "sideways", "plagios"),
        "G8801": ("πλάνησις", "", "a delusion", "planēsis"),
        "G8802": ("πλάτανος", "", "plane tree", "platanos"),
        "G8803": ("πλατυσμός", "", "an enlargement", "platusmos"),
        "G8804": ("πλειστάκις", "", "very often", "pleistakis"),
        "G8805": ("πλεονάκις", "", "many times", "pleonakis"),
        "G8806": ("πλεόνασμα", "", "surplus", "pleonasma"),
        "G8807": ("πλεονασμός", "", "usury", "pleonasmos"),
        "G8808": ("πλεοναστός", "", "superabundant", "pleonastos"),
        "G8809": ("πλεονέκτημα", "", "advantage", "pleonektēma"),
        "G8810": ("πλευρόν", "", "a side", "pleuron"),
        "G8811": ("πλημμέλεια", "", "a trespass", "plēmmeleia"),
        "G8812": ("πλημμελέω", "", "to offend", "plēmmeleō"),
        "G8813": ("πλημμέλημα", "", "a trespass", "plēmmelēma"),
        "G8814": ("πλημμέλησις", "", "a trespass", "plēmmelēsis"),
        "G8815": ("πλήρωσις", "", "a filling", "plērōsis"),
        "G8816": ("πλησίος", "", "dear one", "plēsios"),
        "G8817": ("πλινθεία", "", "making of bricks", "plintheia"),
        "G8818": ("πλινθεύω", "", "to make bricks", "plintheuō"),
        "G8819": ("πλινθίον", "", "a brick-kiln", "plinthion"),
        "G8820": ("πλίνθος", "", "a brick", "plinthos"),
        "G8821": ("πλινθουργία", "", "brickmaking", "plinthourgia"),
        "G8822": ("πλοκή", "", "wreathen", "plokē"),
        "G8823": ("πλόκιον", "", "a braid", "plokion"),
        "G8824": ("πλωτός", "", "afloat", "plōtos"),
        "G8825": ("πνευματοφορέομαι", "", "be carried by wind", "pneumatophoreomai"),
        "G8826": ("πνευματοφόρος", "", "carried by wind", "pneumatophoros"),
        "G8827": ("πνεύμων", "", "lungs", "pneumōn"),
        "G8828": ("πόα", "", "herbage", "poa"),
        "G8829": ("ποθέω", "", "to feel absence", "potheō"),
        "G8830": ("ποικιλία", "", "embroidery", "poikilia"),
        "G8831": ("ποικίλλω", "", "to embroidered", "poikillō"),
        "G8832": ("ποίκιλμα", "", "color", "poikilma"),
        "G8833": ("ποικιλτής", "", "an embroiderer", "poikiltēs"),
        "G8834": ("ποικιλτικός", "", "embroidery", "poikiltikos"),
        "G8835": ("ποικιλτός", "", "an embroidered work", "poikiltos"),
        "G8836": ("ποικίλως", "", "variously", "poikilōs"),
        "G8837": ("ποιμενικός", "", "of a shepherd", "poimenikos"),
        "G8838": ("πόκος", "", "fleece", "pokos"),
        "G8839": ("πολεμικός", "", "warfare", "polemikos"),
        "G8840": ("πολέμιος", "", "warlike", "polemios"),
        "G8841": ("πολεμιστής", "", "a warrior", "polemistēs"),
        "G8842": ("πολιά", "", "grayness of hair", "polia"),
        "G8843": ("πολιορκέω", "", "to assault", "poliorkeō"),
        "G8844": ("πολιορκία", "", "an assault", "poliorkia"),
        "G8845": ("πολιός", "", "gray hair", "polios"),
        "G8846": ("πολλαχῶς", "", "in many ways", "pollachōs"),
        "G8847": ("πολλοστός", "", "a great deal", "pollostos"),
        "G8848": ("πολυάνδριον", "", "a cemetery", "poluandrion"),
        "G8849": ("πολυέλεος", "", "full of mercy", "polueleos"),
        "G8850": ("πολυήμερος", "", "of many days", "poluēmeros"),
        "G8851": ("πολυοδία", "", "many ways", "poluodia"),
        "G8852": ("πολυοχλία", "", "a great multitude", "poluochlia"),
        "G8853": ("πολυπλασιάζω", "", "to manifoldly multiply", "poluplasiazō"),
        "G8854": ("πολυπληθέω", "G:V", "be numerous", "poluplētheō"),
        "G8855": ("πολυπληθύνω", "", "to greatly multiply", "poluplēthunō"),
        "G8856": ("πολύπλοκος", "", "crafty", "poluplokos"),
        "G8857": ("πολυρρήμων", "", "talkative", "polurrēmōn"),
        "G8858": ("πολυτόκος", "", "prolific", "polutokos"),
        "G8859": ("πολυχρονίζω", "", "to delay long", "poluchronizō"),
        "G8860": ("πολυχρόνιος", "", "long-lived", "poluchronios"),
        "G8861": ("πολυωρέω", "", "to take great care", "poluōreō"),
        "G8862": ("πονέω", "", "to suffer pain", "poneō"),
        "G8863": ("πονηρεύομαι", "", "to act wickedly", "ponēreuomai"),
        "G8864": ("ποντοπορέω", "", "to pass through a sea", "pontoporeō"),
        "G8865": ("πόρευσις", "", "a trek", "poreusis"),
        "G8866": ("πορνεῖον", "", "a place of harlotry", "porneion"),
        "G8867": ("πορνικός", "", "of a harlot", "pornikos"),
        "G8868": ("πορνοκόπος", "", "a whoremonger", "pornokopos"),
        "G8869": ("πορφυρίς", "", "of purple", "porphuris"),
        "G8870": ("πορφυρίων", "", "purple-legged stork", "porphuriōn"),
        "G8871": ("ποσαπλῶς", "", "how often", "posaplōs"),
        "G8872": ("πότημα", "", "a drink", "potēma"),
        "G8873": ("ποτιστήριον", "", "a channel", "potistērion"),
        "G8874": ("ποτός", "", "a beverage", "potos"),
        "G8875": ("πράσινος", "", "leek colored", "prasinos"),
        "G8876": ("πρᾶσις", "", "a sale", "prasis"),
        "G8877": ("πράσον", "", "a leek", "prason"),
        "G8878": ("πραΰθυμος", "", "gentle-minded", "prauthumos"),
        "G8879": ("πραΰνω", "", "to calm", "praunō"),
        "G8880": ("πρεσβεῖον", "", "senior", "presbeion"),
        "G8881": ("πρέσβυς", "", "an ambassador", "presbus"),
        "G8882": ("πρήθω", "", "to bloat", "prēthō"),
        "G8883": ("πρίαμαι", "", "to buy", "priamai"),
        "G8884": ("πριστηροειδής", "", "toothed", "pristēroeidēs"),
        "G8885": ("πρίων", "", "a saw", "priōn"),
        "G8886": ("προαίρεσις", "", "a resolve", "proairesis"),
        "G8887": ("προανατάσσω", "", "to prefer", "proanatassō"),
        "G8888": ("προανατέλλω", "", "to sprout up early", "proanatellō"),
        "G8889": ("προαπαγγέλλω", "", "to forewarn", "proapaggellō"),
        "G8890": ("προάστειον", "", "outskirts", "proasteion"),
        "G8891": ("πρόβασις", "", "an entrance approach", "probasis"),
        "G8892": ("πρόβλημα", "", "a riddle", "problēma"),
        "G8893": ("προεκφέρω", "", "to put forth first", "proekpherō"),
        "G8894": ("προθυμέομαι", "", "to feel eager", "prothumeomai"),
        "G8895": ("πρόθυρον", "", "a threshold", "prothuron"),
        "G8896": ("προΐημι", "", "to let go", "proiēmi"),
        "G8897": ("προκαθίστημι", "", "to post in front", "prokathistēmi"),
        "G8898": ("προκαταλαμβάνω", "G:V", "be first to take", "prokatalambanō"),
        "G8899": ("προλήνιον", "", "a wine press", "prolēnion"),
        "G8900": ("πρόλοβος", "", "a crop", "prolobos"),
        "G8901": ("προμαχών", "", "a battlement", "promachōn"),
        "G8902": ("προνομεύω", "", "to despoil", "pronomeuō"),
        "G8903": ("προνομή", "", "plunder", "pronomē"),
        "G8904": ("προοίμιον", "", "a retort", "prooimion"),
        "G8905": ("πρόπαππος", "", "a forefather", "propappos"),
        "G8906": ("προπέτεια", "", "rashness", "propeteia"),
        "G8907": ("προπίπτω", "", "to fall down", "propiptō"),
        "G8908": ("πρόπυλον", "", "a gateway", "propulon"),
        "G8909": ("προσανάβασις", "", "an ascent leading to", "prosanabasis"),
        "G8910": ("προσαποθνῄσκω", "", "to die in addition", "prosapothnēskō"),
        "G8911": ("προσβάλλω", "", "to strike up", "prosballō"),
        "G8912": ("προσβλητός", "", "amalgamated", "prosblētos"),
        "G8913": ("προσγίνομαι", "", "unite with", "prosginomai"),
        "G8914": ("προσδεκτός", "", "acceptable", "prosdektos"),
        "G8915": ("προσδίδωμι", "", "to give in addition", "prosdidōmi"),
        "G8916": ("προσοράω", "", "to look at", "prosoraō"),
        "G8917": ("προσεῖπον", "", "to say to", "proseipon"),
        "G8918": ("προσεκκαίω", "", "to burn besides", "prosekkaiō"),
        "G8919": ("προσεμπρήθω", "", "to set on fire", "prosemprēthō"),
        "G8920": ("προσέτι", "", "moreover", "proseti"),
        "G8921": ("προσηλυτεύω", "", "to convert", "prosēluteuō"),
        "G8922": ("προσηνής", "", "kind", "prosēnēs"),
        "G8923": ("πρόσθεμα", "", "an addition", "prosthema"),
        "G8924": ("πρόσθεσις", "", "an addition", "prosthesis"),
        "G8925": ("προσθλίβω", "", "to press to", "prosthlibō"),
        "G8926": ("προσκαίω", "G:V", "be burnt through", "proskaiō"),
        "G8927": ("προσκαταλείπω", "", "to leave surplus", "proskataleipō"),
        "G8928": ("πρόσκαυμα", "", "burnt", "proskauma"),
        "G8929": ("πρόσκειμαι", "", "to join up with", "proskeimai"),
        "G8930": ("προσκεφαλής", "", "a pillow", "proskephalēs"),
        "G8931": ("προσκρούω", "", "to rush upon", "proskrouō"),
        "G8932": ("προσλογίζομαι", "", "to count in addition to", "proslogizomai"),
        "G8933": ("προσμίγνυμι", "", "to mingle with", "prosmignumi"),
        "G8934": ("προσνοέω", "", "to pay attention to", "prosnoeō"),
        "G8935": ("πρόσοδος", "", "revenue", "prosodos"),
        "G8936": ("προσόζω", "", "to give out an odor", "prosozō"),
        "G8937": ("προσοίγω", "", "to shut", "prosoigō"),
        "G8938": ("προσόχθισμα", "", "a loathsome thing", "prosochthisma"),
        "G8939": ("πρόσοψις", "", "aspect", "prosopsis"),
        "G8940": ("προσπαίζω", "", "to play before", "prospaizō"),
        "G8941": ("προσραίνω", "", "to sprinkle on", "prosrainō"),
        "G8942": ("προσσιελίζω", "", "to salivate upon", "prossielizō"),
        "G8943": ("πρόσταγμα", "", "an order", "prostagma"),
        "G8944": ("προστάς", "", "an antechamber", "prostas"),
        "G8945": ("προστάτης", "", "a superintendent", "prostatēs"),
        "G8946": ("προσχαίρω", "", "to rejoice with", "proschairō"),
        "G8947": ("προσχέω", "", "to pour on", "proscheō"),
        "G8948": ("πρόσχωμα", "", "a mound", "proschōma"),
        "G8949": ("προσχωρέω", "", "to join with", "proschōreō"),
        "G8950": ("προτείχισμα", "", "area around a wall", "proteichisma"),
        "G8951": ("προτέρημα", "", "honor", "proterēma"),
        "G8952": ("προτομή", "", "an upper part", "protomē"),
        "G8953": ("προφασίζομαι", "", "to make an excuse", "prophasizomai"),
        "G8954": ("προφασιστικός", "", "offered as an excuse", "prophasistikos"),
        "G8955": ("προφυλακή", "", "an advance guard", "prophulakē"),
        "G8956": ("προφύλαξ", "", "an advance guard", "prophulax"),
        "G8957": ("προφυλάσσω", "", "to keep guard over", "prophulassō"),
        "G8958": ("πρόχειρος", "", "beforehand", "procheiros"),
        "G8959": ("προχώρημα", "", "excrement", "prochōrēma"),
        "G8960": ("πρώην", "", "a day before", "prōēn"),
        "G8961": ("πρωΐθεν", "", "from morning", "prōithen"),
        "G8962": ("πρῳρεύς", "", "a captain", "prōreus"),
        "G8963": ("πρωτοβαθρέω", "G:V", "be seated first", "prōtobathreō"),
        "G8964": ("πρωτοβολέω", "", "to put forth first", "prōtoboleō"),
        "G8965": ("πρωτογενής", "", "first-born", "prōtogenēs"),
        "G8966": ("πρωτογέννημα", "", "first produce", "prōtogennēma"),
        "G8967": ("πρωτόγονος", "", "first-ripe", "prōtogonos"),
        "G8968": ("πρωτολογία", "", "beginning of speaking", "prōtologia"),
        "G8969": ("πρωτοτοκεύω", "", "to give a right of first-born", "prōtotokeuō"),
        "G8970": ("πρωτοτοκέω", "", "to give birth first time", "prōtotokeō"),
        "G8971": ("πταῖσμα", "", "fault", "ptaisma"),
        "G8972": ("πταρμός", "", "a sneezing", "ptarmos"),
        "G8973": ("πτερνίζω", "", "to stomp", "pternizō"),
        "G8974": ("πτερνισμός", "", "trickery", "pternismos"),
        "G8975": ("πτερόν", "", "a feather", "pteron"),
        "G8976": ("πτεροφυέω", "", "to grow wings", "pterophueō"),
        "G8977": ("πτερύσσομαι", "", "to flap wings", "pterussomai"),
        "G8978": ("πτερωτός", "", "feathered", "pterōtos"),
        "G8979": ("πτήσσω", "", "to alarm", "ptēssō"),
        "G8980": ("πτίλλος", "", "hairless", "ptillos"),
        "G8981": ("πτύελος", "", "spittle", "ptuelos"),
        "G8982": ("πτύξις", "", "fold", "ptuxis"),
        "G8983": ("πτυχή", "", "fold", "ptuchē"),
        "G8984": ("πτωχίζω", "", "to make poor", "ptōchizō"),
        "G8985": ("πύγαργος", "", "white-tailed hart", "pugargos"),
        "G8986": ("πυθμήν", "", "a lower branch", "puthmēn"),
        "G8987": ("πυκάζω", "G:V", "to enclose", "pukazō"),
        "G8988": ("πυλωρός", "", "a gatekeeper", "pulōros"),
        "G8989": ("πυξίον", "", "a writing tablet", "puxion"),
        "G8990": ("πύξος", "", "boxwood", "puxos"),
        "G8991": ("πυργόβαρις", "", "a towered fortification", "purgobaris"),
        "G8992": ("πυρεῖον", "", "a censer", "pureion"),
        "G8993": ("πυρίκαυστος", "", "scorched", "purikaustos"),
        "G8994": ("πυρός", "", "wheat", "puros"),
        "G8995": ("πυροφόρος", "", "a wheat harvest", "purophoros"),
        "G8996": ("πυρράκης", "", "fiery red", "purrakēs"),
        "G8997": ("πυρρίζω", "G:V", "be reddish", "purrizō"),
        "G8998": ("πυρσεύω", "", "to light a fire", "purseuō"),
        "G8999": ("πυρσός", "", "a signal-fire", "pursos"),
        "G9000": ("πυρφόρος", "", "a fire-bearer", "purphoros"),
        "G9001": ("πυρωτής", "", "a refiner", "purōtēs"),
        "G9002": ("πώγων", "", "a beard", "pōgōn"),
        "G9003": ("ῥαγάς", "", "a breach", "rhagas"),
        "G9004": ("ράδαμνος", "", "a tender branch", "radamnos"),
        "G9005": ("Ῥαθαμά", "", "broom shrub", "Rhathama"),
        "G9006": ("ῥαθυμέω", "G:V", "be lazy", "rhathumeō"),
        "G9007": ("ῥαίνω", "", "to sprinkle", "rhainō"),
        "G9008": ("ῥακώδης", "", "ragged", "rhakōdēs"),
        "G9009": ("ῥάμμα", "", "thread", "rhamma"),
        "G9010": ("ῥάμνος", "", "white-thorn", "rhamnos"),
        "G9011": ("ῥαντός", "", "speckled", "rhantos"),
        "G9012": ("ῥαπτός", "", "strung together", "rhaptos"),
        "G9013": ("ῥάπτω", "", "to sew", "rhaptō"),
        "G9014": ("ρασίμ", "", "courier", "rasim"),
        "G9015": ("ῥάσσω", "", "to dash down", "rhassō"),
        "G9016": ("ῥαφιδευτής", "", "a stitcher", "rhaphideutēs"),
        "G9017": ("ῥαφιδευτός", "", "a stitched work", "rhaphideutos"),
        "G9018": ("ῥάχις", "", "a spine", "rhachis"),
        "G9019": ("ῥέγχω", "", "to snore", "rhegchō"),
        "G9020": ("ῥεμβεύω", "", "to stray", "rhembeuō"),
        "G9021": ("ῥέμβω", "", "to stray", "rhembō"),
        "G9022": ("ῥῆσις", "", "a saying", "rhēsis"),
        "G9023": ("ῥητίνη", "", "balm", "rhētinē"),
        "G9024": ("ῥητός", "", "in particular", "rhētos"),
        "G9025": ("ῥῖγος", "", "shivering", "rhigos"),
        "G9026": ("ῥίζωμα", "", "a root", "rhizōma"),
        "G9027": ("ῥίν", "", "nostril", "rhin"),
        "G9028": ("ῥιπιστός", "", "ventilated", "rhipistos"),
        "G9029": ("ῥόα", "", "a pomegranate", "rhoa"),
        "G9030": ("ῥοιζέω", "", "to gurgle", "rhoizeō"),
        "G9031": ("ῥοῖζος", "", "impetus", "rhoizos"),
        "G9032": ("ῥοΐσκος", "", "a pomegranate figure", "rhoiskos"),
        "G9033": ("ῥόπαλον", "", "a club", "rhopalon"),
        "G9034": ("ῥοπή", "", "crux", "rhopē"),
        "G9035": ("ῥοών", "", "a pomegranate grove", "rhoōn"),
        "G9036": ("ῥύαξ", "", "an overflow", "rhuax"),
        "G9037": ("ῥυθμίζω", "", "to compose", "rhuthmizō"),
        "G9038": ("ῥυθμός", "", "a proportion", "rhuthmos"),
        "G9039": ("ῥύστης", "", "a rescuer", "rhustēs"),
        "G9040": ("Ρωκεϊμ", "", "a perfumer", "Rōkeim"),
        "G9041": ("ῥωγαλέος", "", "stone", "rhōgaleos"),
        "G9042": ("ς", "", "6", "s"),
        "G9043": ("Σαββατίζω", "", "to observe sabbaths", "Sabbatizō"),
        "G9044": ("σαβεί", "", "glory", "sabei"),
        "G9045": ("σαβέκ", "", "a thicket", "sabek"),
        "G9046": ("σάγμα", "", "a packsaddle", "sagma"),
        "G9047": ("Σαδδαΐ", "", "Saddai", "Saddai"),
        "G9048": ("σαδημώθ", "", "a plain", "sadēmōth"),
        "G9049": ("σαδηρώθ", "", "ranks", "sadērōth"),
        "G9050": ("σαθρός", "", "rotten", "sathros"),
        "G9051": ("σαθρόω", "", "to disintegrate", "sathroō"),
        "G9052": ("σαμβύκη", "", "a sambuke", "sambukē"),
        "G9053": ("σανιδωτός", "", "planked", "sanidōtos"),
        "G9054": ("σαπρία", "", "rottenness", "sapria"),
        "G9055": ("σαπρίζω", "", "to rot", "saprizō"),
        "G9056": ("σαράβαρα", "", "pantaloons", "sarabara"),
        "G9057": ("σάρδιος", "", "a carnelian", "sardios"),
        "G9058": ("σατραπεία", "", "a satrapy", "satrapeia"),
        "G9059": ("σατράπης", "", "satrap", "satrapēs"),
        "G9060": ("σαύρα", "", "a lizard", "saura"),
        "G9061": ("σαφώθ", "", "cheese", "saphōth"),
        "G9062": ("σαφῶς", "", "clearly", "saphōs"),
        "G9063": ("Σειρήν", "", "a siren", "Seirēn"),
        "G9064": ("σειρομάστης", "", "a spear", "seiromastēs"),
        "G9065": ("σελίς", "", "column", "selis"),
        "G9066": ("σελισία", "", "of three years", "selisia"),
        "G9067": ("σεραφίμ", "", "seraphim", "seraphim"),
        "G9068": ("σευτλίον", "", "a beet", "seutlion"),
        "G9069": ("σημαία", "", "a flag", "sēmaia"),
        "G9070": ("σημασία", "", "a cheer", "sēmasia"),
        "G9071": ("σημείωσις", "", "a signal", "sēmeiōsis"),
        "G9072": ("σῆψις", "", "putrefaction", "sēpsis"),
        "G9073": ("σθένος", "", "strength (physical)", "sthenos"),
        "G9074": ("σιαγόνιον", "", "a jawbone", "siagonion"),
        "G9075": ("σῖγα", "", "quiet", "siga"),
        "G9076": ("σιδήριον", "", "an iron implement", "sidērion"),
        "G9077": ("σίελον", "", "saliva", "sielon"),
        "G9078": ("σίκιμα", "", "a portion", "sikima"),
        "G9079": ("σίκλος", "", "a shekel", "siklos"),
        "G9080": ("σικυήρατον", "", "a cucumber garden", "sikuēraton"),
        "G9081": ("σίκυος", "", "a cucumber", "sikuos"),
        "G9082": ("σιρώνων", "", "ornaments", "sirōnōn"),
        "G9083": ("σισόη", "", "a consecrated lock of hair", "sisoē"),
        "G9084": ("σιτέω", "G:V", "be well fed", "siteō"),
        "G9085": ("σιτοβολών", "", "a granary", "sitobolōn"),
        "G9086": ("σιτοδεία", "", "scarcity", "sitodeia"),
        "G9087": ("σιτοδοσία", "", "provision", "sitodosia"),
        "G9088": ("σιτομετρέω", "", "to measure out grain", "sitometreō"),
        "G9089": ("σιτοποιός", "", "a baker", "sitopoios"),
        "G9090": ("σιωπή", "", "silence", "siōpē"),
        "G9091": ("σιώπησις", "", "a veil", "siōpēsis"),
        "G9092": ("σκάλλω", "", "to till", "skallō"),
        "G9093": ("σκαμβός", "", "crooked", "skambos"),
        "G9094": ("σκελίζω", "", "to trip up", "skelizō"),
        "G9095": ("σκεπάζω", "", "to shelter", "skepazō"),
        "G9096": ("σκέπαρνον", "", "a hammer", "skeparnon"),
        "G9097": ("σκεπαστής", "", "a shelterer", "skepastēs"),
        "G9098": ("σκέπη", "", "a shelter", "skepē"),
        "G9099": ("σκεπηνός", "", "protected", "skepēnos"),
        "G9100": ("σκέπτομαι", "", "to look about", "skeptomai"),
        "G9101": ("σκευασία", "", "a concoction", "skeuasia"),
        "G9102": ("σκευαστός", "", "concocted", "skeuastos"),
        "G9103": ("σκευοφύλαξ", "", "provisions officer", "skeuophulax"),
        "G9104": ("σκῆπτρον", "", "a chiefdom", "skēptron"),
        "G9105": ("σκιάδιον", "", "an awning", "skiadion"),
        "G9106": ("σκιάζω", "", "to shade", "skiazō"),
        "G9107": ("σκληροκάρδιος", "", "hard-hearted", "sklērokardios"),
        "G9108": ("σκληροπρόσωπος", "", "harsh", "sklēroprosōpos"),
        "G9109": ("σκληρῶς", "", "harshly", "sklērōs"),
        "G9110": ("σκνίψ", "", "a midge", "sknips"),
        "G9111": ("σκολιάζω", "G:V", "to perverse", "skoliazō"),
        "G9112": ("σκολιότης", "", "deformity", "skoliotēs"),
        "G9113": ("σκολιῶς", "", "crookedly", "skoliōs"),
        "G9114": ("σκόπελον", "", "a high rock", "skopelon"),
        "G9115": ("σκοπιά", "", "a height", "skopia"),
        "G9116": ("σκόρδον", "", "garlic", "skordon"),
        "G9117": ("σκοτάζω", "", "to darken", "skotazō"),
        "G9118": ("σκοτομήνη", "", "moonlight", "skotomēnē"),
        "G9119": ("σκυθρωπάζω", "", "to look downcast", "skuthrōpazō"),
        "G9120": ("σκυλεύω", "", "to despoil", "skuleuō"),
        "G9121": ("σκύμνος", "", "a cub", "skumnos"),
        "G9122": ("σκυτάλη", "", "a stave", "skutalē"),
        "G9123": ("σκῶλον", "", "an impediment", "skōlon"),
        "G9124": ("σμαραγδίτης", "", "an emerald", "smaragditēs"),
        "G9125": ("σμῆγμα", "", "beauty treatment", "smēgma"),
        "G9126": ("σμικρύνω", "", "to diminish", "smikrunō"),
        "G9127": ("σμῖλαξ", "", "a yew tree", "smilax"),
        "G9128": ("σμυρίτης", "", "an emery stone", "smuritēs"),
        "G9129": ("σμύρνινος", "", "of myrrh", "smurninos"),
        "G9130": ("σοφιστής", "", "a wise man", "sophistēs"),
        "G9131": ("σοφόω", "", "to give wisdom", "sophoō"),
        "G9132": ("σοφῶς", "", "wisely", "sophōs"),
        "G9133": ("σπάδων", "", "a castrato", "spadōn"),
        "G9134": ("σπανίζω", "G:V", "be depleted", "spanizō"),
        "G9135": ("σπάνιος", "", "sparingly", "spanios"),
        "G9136": ("σπάργανον", "", "swaddling cloth", "sparganon"),
        "G9137": ("σπαρτίον", "", "a cord", "spartion"),
        "G9138": ("σπερματίζω", "", "to conceive", "spermatizō"),
        "G9139": ("σπερματισμός", "", "a discharge of semen", "spermatismos"),
        "G9140": ("σπιθαμή", "", "a span", "spithamē"),
        "G9141": ("σπινθήρ", "", "a spark", "spinthēr"),
        "G9142": ("σποδιά", "", "a pile of ashes", "spodia"),
        "G9143": ("σποδοειδής", "", "ashen", "spodoeidēs"),
        "G9144": ("σπονδεῖον", "", "a libation bowl", "spondeion"),
        "G9145": ("σπονδή", "", "a libation", "spondē"),
        "G9146": ("σταγών", "", "a drop (liquid)", "stagōn"),
        "G9147": ("στάζω", "", "to drip", "stazō"),
        "G9148": ("σταθμάω", "", "to measure by a rule", "stathmaō"),
        "G9149": ("στάθμιον", "", "a scale-weight", "stathmion"),
        "G9150": ("σταθμός", "", "a doorpost", "stathmos"),
        "G9151": ("σταῖς", "", "dough", "stais"),
        "G9152": ("στακτή", "", "balsam", "staktē"),
        "G9153": ("σταλάζω", "", "to trickle", "stalazō"),
        "G9154": ("σταφίς", "", "a dried grape", "staphis"),
        "G9155": ("στέαρ", "", "fat", "stear"),
        "G9156": ("στεατόομαι", "", "to grow fat", "steatoomai"),
        "G9157": ("στεγάζω", "", "to roof", "stegazō"),
        "G9158": ("στεγνός", "", "roofed", "stegnos"),
        "G9159": ("στέλεχος", "", "a stick", "stelechos"),
        "G9160": ("στέμφυλον", "", "dregs", "stemphulon"),
        "G9161": ("στενακτός", "", "bemoaned", "stenaktos"),
        "G9162": ("στένω", "", "to moan", "stenō"),
        "G9163": ("στενῶς", "", "in a strait", "stenōs"),
        "G9164": ("στένωσις", "", "straitness", "stenōsis"),
        "G9165": ("στερεοκάρδιος", "", "hard-hearted", "stereokardios"),
        "G9166": ("στερέω", "", "to deprive", "stereō"),
        "G9167": ("στερίσκω", "", "to deprive", "steriskō"),
        "G9168": ("στεφάνη", "", "a rim", "stephanē"),
        "G9169": ("στηθοδεσμίς", "", "a breastband", "stēthodesmis"),
        "G9170": ("στηθύνιον", "", "a breast", "stēthunion"),
        "G9171": ("στήλη", "", "a monument", "stēlē"),
        "G9172": ("στηλογραφία", "", "an inscription on a monument", "stēlographia"),
        "G9173": ("στηλόω", "", "to set up a monument", "stēloō"),
        "G9174": ("στήλωμα", "", "a pillar", "stēlōma"),
        "G9175": ("στήμων", "", "a warp", "stēmōn"),
        "G9176": ("στήριγμα", "", "a reliance", "stērigma"),
        "G9177": ("στιβαρός", "", "dense", "stibaros"),
        "G9178": ("στιβαρῶς", "", "densely", "stibarōs"),
        "G9179": ("στίβι", "", "antimony", "stibi"),
        "G9180": ("στιβίζομαι", "", "to tinge with antimony", "stibizomai"),
        "G9181": ("στικτός", "", "a mark", "stiktos"),
        "G9182": ("στιλβόω", "", "to brighten", "stilboō"),
        "G9183": ("στίλβωσις", "", "shine", "stilbōsis"),
        "G9184": ("στίχος", "", "a row", "stichos"),
        "G9185": ("στοιβάζω", "", "to pile", "stoibazō"),
        "G9186": ("στοιβή", "", "a pile", "stoibē"),
        "G9187": ("στοιχίζω", "", "to set in rows", "stoichizō"),
        "G9188": ("στολίζω", "", "to robe", "stolizō"),
        "G9189": ("στολισμός", "", "a uniform", "stolismos"),
        "G9190": ("στολιστής", "", "a keeper of wardrobe", "stolistēs"),
        "G9191": ("στοχάζομαι", "", "to take thought", "stochazomai"),
        "G9192": ("στοχαστής", "", "a thinker", "stochastēs"),
        "G9193": ("στραγγαλιά", "", "perverseness", "straggalia"),
        "G9194": ("στραγγαλιώδης", "", "insidious", "straggaliōdēs"),
        "G9195": ("στραγγίζω", "", "to wring out", "straggizō"),
        "G9196": ("στρατοκῆρυξ", "", "herald of an army", "stratokērux"),
        "G9197": ("στρατοπεδεία", "", "a  military encampment", "stratopedeia"),
        "G9198": ("στρατοπεδεύω", "", "to encamp", "stratopedeuō"),
        "G9199": ("στρεβλός", "", "crooked", "streblos"),
        "G9200": ("στρέμμα", "", "a strand", "stremma"),
        "G9201": ("στρίφνος", "", "tough", "striphnos"),
        "G9202": ("στροβέω", "", "to whirl", "strobeō"),
        "G9203": ("στρογγύλος", "", "globular", "stroggulos"),
        "G9204": ("στρογγύλωμα", "", "round", "stroggulōma"),
        "G9205": ("στρογγύλωσις", "", "a battleline", "stroggulōsis"),
        "G9206": ("στρουθίζω", "", "to chirp", "strouthizō"),
        "G9207": ("στρουθός", "", "an ostrich", "strouthos"),
        "G9208": ("στροφεύς", "", "a hinge", "stropheus"),
        "G9209": ("στροφή", "", "a shift and turn", "strophē"),
        "G9210": ("στρόφιγξ", "", "a hinge", "strophigx"),
        "G9211": ("στροφωτός", "", "turnings", "strophōtos"),
        "G9212": ("στρῶμα", "", "a bedding", "strōma"),
        "G9213": ("στρωμνή", "", "a strewn bed", "strōmnē"),
        "G9214": ("στυγνός", "", "gloomy", "stugnos"),
        "G9215": ("στυππέϊνος", "", "of hemp", "stuppeinos"),
        "G9216": ("στυππίον", "", "hemp", "stuppion"),
        "G9217": ("στυράκινος", "", "of a green poplar", "sturakinos"),
        "G9218": ("συγγίνομαι", "", "be intimate with", "sugginomai"),
        "G9219": ("συγγραφή", "", "a writ", "suggraphē"),
        "G9220": ("συγκαθυφαίνω", "", "to interweave", "sugkathuphainō"),
        "G9221": ("συγκαίω", "", "to burn with", "sugkaiō"),
        "G9222": ("συγκάλυμμα", "", "a marriage veil", "sugkalumma"),
        "G9223": ("συγκατακληρονομέομαι", "", "to inherit together", "sugkataklēronomeomai"),
        "G9224": ("συγκαταμίγνυμι", "", "to mix together with", "sugkatamignumi"),
        "G9225": ("συγκατεσθίω", "", "to devour together", "sugkatesthiō"),
        "G9226": ("συγκαταφέρω", "", "to carry together", "sugkatapherō"),
        "G9227": ("συγκατοικέω", "", "to dwell together", "sugkatoikeō"),
        "G9228": ("σύγκειμαι", "", "be situated together", "sugkeimai"),
        "G9229": ("συγκερατίζομαι", "", "to mix horns", "sugkeratizomai"),
        "G9230": ("συγκλασμός", "", "a splinter", "sugklasmos"),
        "G9231": ("συγκλάω", "", "to break in pieces together", "sugklaō"),
        "G9232": ("σύγκλεισμα", "", "a joinery", "sugkleisma"),
        "G9233": ("συγκλεισμός", "", "a confinement", "sugkleismos"),
        "G9234": ("συγκλειστός", "", "joined", "sugkleistos"),
        "G9235": ("σύγκλητος", "", "summoned", "sugklētos"),
        "G9236": ("συγκλύζω", "", "to engulf", "sugkluzō"),
        "G9237": ("σύγκοιτος", "", "a bed-mate", "sugkoitos"),
        "G9238": ("συγκόπτω", "", "to cut down", "sugkoptō"),
        "G9239": ("σύγκρασις", "", "an admixture", "sugkrasis"),
        "G9240": ("σύγκριμα", "", "interpretation", "sugkrima"),
        "G9241": ("σύγκρισις", "", "amalgamation", "sugkrisis"),
        "G9242": ("συγκροτέω", "", "to strike together", "sugkroteō"),
        "G9243": ("συγκυρέω", "", "to fall in with", "sugkureō"),
        "G9244": ("συζώννυμι", "", "to tie together", "suzōnnumi"),
        "G9245": ("συκάμινον", "", "fruit of sycamine trees", "sukaminon"),
        "G9246": ("συκεών", "", "a fig grove", "sukeōn"),
        "G9247": ("συκοφάντης", "", "an extortioner", "sukophantēs"),
        "G9248": ("συκοφαντία", "", "extortion", "sukophantia"),
        "G9249": ("σύλληψις", "", "a seizing", "sullēpsis"),
        "G9250": ("συλλογή", "", "collection", "sullogē"),
        "G9251": ("συλλογισμός", "", "assessment", "sullogismos"),
        "G9252": ("συλλοιδορέω", "", "to join in reviling", "sulloidoreō"),
        "G9253": ("συμβαστάζω", "G:V", "be compared with", "sumbastazō"),
        "G9254": ("σύμβλημα", "", "a coupling", "sumblēma"),
        "G9255": ("σύμβλησις", "", "a coupling", "sumblēsis"),
        "G9256": ("συμβοηθός", "", "an ally", "sumboēthos"),
        "G9257": ("συμβολή", "", "a compact", "sumbolē"),
        "G9258": ("συμβολοκοπέω", "G:V", "be fond of carousing", "sumbolokopeō"),
        "G9259": ("σύμβολον", "", "a symbol", "sumbolon"),
        "G9260": ("συμβόσκομαι", "", "to feed together", "sumboskomai"),
        "G9261": ("συμβουλία", "", "advice", "sumboulia"),
        "G9262": ("συμμαχέω", "", "to fight along with", "summacheō"),
        "G9263": ("συμμαχία", "", "an alliance in war", "summachia"),
        "G9264": ("σύμμετρος", "", "well-proportioned", "summetros"),
        "G9265": ("συμμιγής", "", "mixed together", "summigēs"),
        "G9266": ("συμμίγνυμι", "", "to mix together with", "summignumi"),
        "G9267": ("σύμμικτος", "", "consolidation", "summiktos"),
        "G9268": ("σύμμιξις", "", "an alliance", "summixis"),
        "G9269": ("συμπαρίστημι", "", "to stand up with", "sumparistēmi"),
        "G9270": ("σύμπας", "", "all things", "sumpas"),
        "G9271": ("συμπατέω", "", "to trample upon together", "sumpateō"),
        "G9272": ("συμπεραίνω", "", "to finish off together", "sumperainō"),
        "G9273": ("συμπεριφέρω", "", "to accommodate", "sumperipherō"),
        "G9274": ("σύμπλεκτος", "", "closely joined", "sumplektos"),
        "G9275": ("συμπλέκω", "G:V", "be closely joined", "sumplekō"),
        "G9276": ("συμπλήρωσις", "", "fulfillment", "sumplērōsis"),
        "G9277": ("συμποδίζω", "", "to bind hand and foot", "sumpodizō"),
        "G9278": ("συμπολεμέω", "", "to join in war with", "sumpolemeō"),
        "G9279": ("συμπορπάομαι", "", "to clasp together", "sumporpaomai"),
        "G9280": ("συμπροπέμπω", "", "to escort", "sumpropempō"),
        "G9281": ("συμπρόσειμι", "", "to adhere with", "sumproseimi"),
        "G9282": ("συμπροσπλέκομαι", "", "to grapple together", "sumprosplekomai"),
        "G9283": ("σύμπτωμα", "", "a coincidence", "sumptōma"),
        "G9284": ("συμφλέγω", "", "to burn together", "sumphlegō"),
        "G9285": ("συμφοράζω", "", "to lament together", "sumphorazō"),
        "G9286": ("συμφράσσω", "", "to shut up together", "sumphrassō"),
        "G9287": ("συμφρύγω", "", "to parch", "sumphrugō"),
        "G9288": ("συμφύρω", "", "to blend together", "sumphurō"),
        "G9289": ("συμφώνως", "", "harmoniously", "sumphōnōs"),
        "G9290": ("συμψάω", "", "to scrape away", "sumpsaō"),
        "G9291": ("συνᾴδω", "", "to sing together in concert", "sunadō"),
        "G9292": ("συνάλλαγμα", "", "exchange", "sunallagma"),
        "G9293": ("συνανάμιξις", "", "intermingling with", "sunanamixis"),
        "G9294": ("συναναφέρω", "", "to join in carrying off", "sunanapherō"),
        "G9295": ("συναντή", "", "a meeting with", "sunantē"),
        "G9296": ("συνάντημα", "", "an event", "sunantēma"),
        "G9297": ("συνάπτω", "", "to join together", "sunaptō"),
        "G9298": ("συναριθμέω", "", "to count with", "sunarithmeō"),
        "G9299": ("συναυλίζομαι", "", "to lodge with", "sunaulizomai"),
        "G9300": ("σύναψις", "", "an association", "sunapsis"),
        "G9301": ("συνδειπνέω", "", "to dine with", "sundeipneō"),
        "G9302": ("συνδυάζω", "", "to associate oneself", "sunduazō"),
        "G9303": ("σύνεγγυς", "", "in the vicinity", "suneggus"),
        "G9304": ("συνεδριάζω", "", "to  sit together with", "sunedriazō"),
        "G9305": ("συνεκπολεμέω", "", "to join in making war", "sunekpolemeō"),
        "G9306": ("συνεκπορεύομαι", "", "to go forth together", "sunekporeuomai"),
        "G9307": ("συνεκτρέφω", "", "to bring up together", "sunektrephō"),
        "G9308": ("συνέλκω", "", "to draw together", "sunelkō"),
        "G9309": ("συνεξαίρω", "", "to raise up together", "sunexairō"),
        "G9310": ("συνεξέρχομαι", "", "to go out together", "sunexerchomai"),
        "G9311": ("συνεπακολουθέω", "", "to follow after together", "sunepakoloutheō"),
        "G9312": ("συνεπισκέπω", "", "to consider together", "sunepiskepō"),
        "G9313": ("συνεπίσταμαι", "", "be conscious of", "sunepistamai"),
        "G9314": ("συνεπισχύω", "", "to vigorously assist", "sunepischuō"),
        "G9315": ("συνεπιτίθημι", "", "to join in making an attack", "sunepitithēmi"),
        "G9316": ("συνεταιρίς", "", "a female companion", "sunetairis"),
        "G9317": ("συνέταιρος", "", "a male companion", "sunetairos"),
        "G9318": ("συνετίζω", "", "to bring understanding to", "sunetizō"),
        "G9319": ("συνετῶς", "", "expertly", "sunetōs"),
        "G9320": ("συνευφραίνομαι", "", "be glad with", "suneuphrainomai"),
        "G9321": ("συνῆλιξ", "", "a contemporary", "sunēlix"),
        "G9322": ("συνθέλω", "", "to acquiesce", "sunthelō"),
        "G9323": ("σύνθεμα", "", "an agreement with", "sunthema"),
        "G9324": ("σύνθεσις", "", "a composition", "sunthesis"),
        "G9325": ("σύνθετος", "", "compounded", "sunthetos"),
        "G9326": ("συνθήκη", "", "a treaty", "sunthēkē"),
        "G9327": ("συνίστωρ", "", "a joint witness", "sunistōr"),
        "G9328": ("συννέφω", "", "to collect clouds together", "sunnephō"),
        "G9329": ("συννεφής", "", "covered with clouds", "sunnephēs"),
        "G9330": ("σύννυμφος", "", "a sister-in-law", "sunnumphos"),
        "G9331": ("σύνοδος", "", "a convocation", "sunodos"),
        "G9332": ("συνοικίζω", "", "to live together", "sunoikizō"),
        "G9333": ("συνούλωσις", "", "closing of a wound", "sunoulōsis"),
        "G9334": ("συνταγή", "", "an arranged order", "suntagē"),
        "G9335": ("σύνταγμα", "", "an arranged order", "suntagma"),
        "G9336": ("σύνταξις", "", "a rate", "suntaxis"),
        "G9337": ("συνταράσσω", "", "to disturb", "suntarassō"),
        "G9338": ("συντέλεσμα", "", "custom (tax)", "suntelesma"),
        "G9339": ("συντίμησις", "", "a price", "suntimēsis"),
        "G9340": ("συντριβή", "", "a breaking", "suntribē"),
        "G9341": ("συντριμμός", "", "a break", "suntrimmos"),
        "G9342": ("σύντριψις", "", "a defeat", "suntripsis"),
        "G9343": ("συντροχάζω", "", "to roll together", "suntrochazō"),
        "G9344": ("συνυφαίνω", "", "to weave together", "sunuphainō"),
        "G9345": ("συνυφή", "", "a woven part", "sunuphē"),
        "G9346": ("συνωμότης", "", "a confederate", "sunōmotēs"),
        "G9347": ("σύριγμα", "", "a hissing", "surigma"),
        "G9348": ("σῦριγξ", "", "a flute", "surigx"),
        "G9349": ("συρίζω", "", "to hiss", "surizō"),
        "G9350": ("συρισμός", "", "a hissing", "surismos"),
        "G9351": ("συρράπτω", "", "to sew together", "surraptō"),
        "G9352": ("σύσκηνος", "", "a fellow tent-dweller", "suskēnos"),
        "G9353": ("συσκιάζω", "", "to overshadow", "suskiazō"),
        "G9354": ("σύσκιος", "", "shady", "suskios"),
        "G9355": ("συσκοτάζω", "", "to darken", "suskotazō"),
        "G9356": ("συσπάω", "", "to shrivel", "suspaō"),
        "G9357": ("συσσεισμός", "", "a quaking", "susseismos"),
        "G9358": ("συσσείω", "", "to shake", "susseiō"),
        "G9359": ("σύστασις", "", "a joint-conspiracy", "sustasis"),
        "G9360": ("σύστεμα", "", "collection", "sustema"),
        "G9361": ("σύστρεμμα", "", "a confederation", "sustremma"),
        "G9362": ("συσφίγγω", "G:V", "to condense", "susphiggō"),
        "G9363": ("σφαιρωτήρ", "", "a knob", "sphairōtēr"),
        "G9364": ("σφακελίζω", "", "to have an inflammation", "sphakelizō"),
        "G9365": ("σφαλερός", "", "unsteady", "sphaleros"),
        "G9366": ("σφάλλω", "", "to trip", "sphallō"),
        "G9367": ("σφάλμα", "", "a trip", "sphalma"),
        "G9368": ("σφενδονάω", "", "to sling", "sphendonaō"),
        "G9369": ("σφενδόνη", "", "a sling", "sphendonē"),
        "G9370": ("σφενδονήτης", "", "a slinger", "sphendonētēs"),
        "G9371": ("σφηκιά", "", "a swarm of wasps", "sphēkia"),
        "G9372": ("σφηνόω", "", "to wedge", "sphēnoō"),
        "G9373": ("σφίγγω", "", "to grasp", "sphiggō"),
        "G9374": ("σφοδρός", "", "vehement", "sphodros"),
        "G9375": ("σφόνδυλος", "", "a neck vertebrae", "sphondulos"),
        "G9376": ("σφῦρα", "", "an adz", "sphura"),
        "G9377": ("σφυροκοπέω", "", "to strike with a hammer", "sphurokopeō"),
        "G9378": ("σφυροκόπος", "", "a hammer-smith", "sphurokopos"),
        "G9379": ("σχάζω", "", "to open up", "schazō"),
        "G9380": ("σχεδία", "", "a barge", "schedia"),
        "G9381": ("σχίδαξ", "", "kindling", "schidax"),
        "G9382": ("σχίζα", "", "a dart", "schiza"),
        "G9383": ("σχισμή", "", "a fissure", "schismē"),
        "G9384": ("σχιστός", "", "shredded", "schistos"),
        "G9385": ("σχοίνισμα", "", "a measured out line", "schoinisma"),
        "G9386": ("σχοινισμός", "", "a piece of measured out land", "schoinismos"),
        "G9387": ("σχοῖνος", "", "a rush", "schoinos"),
        "G9388": ("σχολαστής", "", "an idler", "scholastēs"),
        "G9389": ("σωματοποιέω", "", "to rejuvenate", "sōmatopoieō"),
        "G9390": ("σωρήκ", "", "a choice vine", "sōrēk"),
        "G9391": ("σωρός", "", "a heap", "sōros"),
        "G9392": ("σωφάρ", "", "ram's horn", "sōphar"),
        "G9394": ("ταινία", "", "fascia", "tainia"),
        "G9395": ("τακτικός", "", "tactician", "taktikos"),
        "G9396": ("ταλαιπώρως", "", "miserably", "talaipōrōs"),
        "G9397": ("τάλας", "", "miserable", "talas"),
        "G9398": ("ταμίας", "", "a storekeeper", "tamias"),
        "G9399": ("ταμιεύω", "", "to store up", "tamieuō"),
        "G9400": ("τανύω", "", "to stretch", "tanuō"),
        "G9401": ("ταπεινοφρονέω", "G:V", "be humble-minded", "tapeinophroneō"),
        "G9402": ("ταπεινόφρων", "", "humble-minded", "tapeinophrōn"),
        "G9403": ("ταραχώδης", "", "disturbing", "tarachōdēs"),
        "G9404": ("Τάρταρος", "", "infernal region", "Tartaros"),
        "G9405": ("τάφρος", "", "a trench", "taphros"),
        "G9406": ("ταχύνω", "", "to hasten", "tachunō"),
        "G9407": ("τείνω", "", "to stretch out", "teinō"),
        "G9408": ("τειχήρης", "", "walled", "teichērēs"),
        "G9409": ("τειχίζω", "", "to wall", "teichizō"),
        "G9410": ("τειχιστής", "", "a stonemason", "teichistēs"),
        "G9411": ("τεκνοποιέω", "", "to produce children", "teknopoieō"),
        "G9412": ("τεκταίνομαι", "", "to contrive", "tektainomai"),
        "G9413": ("τεκτονικός", "", "a woodworker", "tektonikos"),
        "G9414": ("τελαμών", "", "a ligature", "telamōn"),
        "G9415": ("τελεσιουργέω", "", "to perfect a work", "telesiourgeō"),
        "G9416": ("τελετή", "", "mystic rites", "teletē"),
        "G9417": ("τελευταῖος", "", "finality", "teleutaios"),
        "G9418": ("τέμενος", "", "a sacred precinct", "temenos"),
        "G9419": ("τέμνω", "", "to prune", "temnō"),
        "G9420": ("τερατοσκόπος", "", "an observer of signs", "teratoskopos"),
        "G9421": ("τερέβινθος", "", "terebinth resin", "terebinthos"),
        "G9422": ("τέρετρον", "", "a drill", "teretron"),
        "G9423": ("τερπνός", "", "delightful", "terpnos"),
        "G9424": ("τερπνότης", "", "delightfulness", "terpnotēs"),
        "G9425": ("τέρπω", "", "to delight", "terpō"),
        "G9426": ("τέρψις", "", "delight", "terpsis"),
        "G9427": ("τεσσαρακονταδύο", "", "forty-two", "tessarakontaduo"),
        "G9428": ("τεσσαρακονταεπτά", "", "forty-seven", "tessarakontaepta"),
        "G9429": ("τεσσαρακονταοκτώ", "", "forty-eight", "tessarakontaoktō"),
        "G9430": ("τεσσαρακονταπέντε", "", "forty-five", "tessarakontapente"),
        "G9431": ("τεσσαρακοντατρεῖς", "", "forty-three", "tessarakontatreis"),
        "G9432": ("τεσσαρακοστός", "", "fortieth", "tessarakostos"),
        "G9433": ("τεσσαρεσκαίδεκα", "", "fourteen", "tessareskaideka"),
        "G9434": ("τετράδραχμος", "", "four-drachma", "tetradrachmos"),
        "G9435": ("τετραίνω", "G:V", "to puncture", "tetrainō"),
        "G9436": ("τετρακισμύριοι", "", "forty thousand", "tetrakismurioi"),
        "G9437": ("τετρακοσιοστός", "", "four hundredth", "tetrakosiostos"),
        "G9438": ("τετράπεδος", "", "quadrangular", "tetrapedos"),
        "G9439": ("τετραπλῶς", "", "fourfold", "tetraplōs"),
        "G9440": ("τετράς", "", "fourth", "tetras"),
        "G9441": ("τετράστιχος", "", "arranged in four rows", "tetrastichos"),
        "G9442": ("τεχνάζω", "", "to cunningly contrive", "technazō"),
        "G9443": ("Τηβήθ", "", "Tebeth", "Tēbēth"),
        "G9444": ("τήγανον", "", "a frying pan", "tēganon"),
        "G9445": ("τηλαύγημα", "", "radiance", "tēlaugēma"),
        "G9446": ("τηλαυγής", "", "radiant", "tēlaugēs"),
        "G9447": ("τηλαύγησις", "", "radiance", "tēlaugēsis"),
        "G9448": ("τιάρα", "", "a tiara", "tiara"),
        "G9449": ("τιθηνέω", "", "to suckle", "tithēneō"),
        "G9450": ("τιθηνός", "", "a wet-nurse", "tithēnos"),
        "G9451": ("τίμημα", "", "valuation", "timēma"),
        "G9452": ("τιμογραφέω", "", "to assess", "timographeō"),
        "G9453": ("τίναγμα", "", "vibration", "tinagma"),
        "G9454": ("Τιτάν", "", "titan", "Titan"),
        "G9455": ("τιτρώσκω", "", "to pierce", "titrōskō"),
        "G9456": ("τμητός", "", "to cut into shape", "tmētos"),
        "G9457": ("τοκετός", "", "birth", "toketos"),
        "G9458": ("τόλμα", "", "daring", "tolma"),
        "G9459": ("τολύπη", "", "a wild gourd", "tolupē"),
        "G9460": ("τομή", "", "pruning", "tomē"),
        "G9461": ("τομίς", "", "a pruning knife", "tomis"),
        "G9462": ("τόμος", "", "a roll of papyrus", "tomos"),
        "G9463": ("τόξευμα", "", "a bow", "toxeuma"),
        "G9464": ("τοξεύω", "", "to shoot", "toxeuō"),
        "G9465": ("τοξότης", "", "a bowman", "toxotēs"),
        "G9466": ("τοπάρχης", "", "a toparch", "toparchēs"),
        "G9467": ("τορευτός", "", "turned", "toreutos"),
        "G9468": ("τραγέλαφος", "", "an antelope", "tragelaphos"),
        "G9469": ("τρανός", "", "plainly", "tranos"),
        "G9470": ("τραυματίας", "", "slain", "traumatias"),
        "G9471": ("τραχηλιάω", "", "to stiffen the neck", "trachēliaō"),
        "G9472": ("τρέπω", "", "to rout", "trepō"),
        "G9473": ("τριακονταδύο", "", "thirty-two", "triakontaduo"),
        "G9474": ("τριακονταεννέα", "", "thirty-nine", "triakontaennea"),
        "G9475": ("τριακονταέξ", "", "thirty-six", "triakontaex"),
        "G9476": ("τριακονταεπτά", "", "thirty-seven", "triakontaepta"),
        "G9477": ("τριακονταέτης", "", "thirty years old", "triakontaetēs"),
        "G9478": ("τριακονταοκτώ", "", "thirty-eight", "triakontaoktō"),
        "G9479": ("τριακονταπέντε", "", "thirty-five", "triakontapente"),
        "G9480": ("τριακοντατρεῖς", "", "thirty-three", "triakontatreis"),
        "G9481": ("τριακοντατέσσαρες", "", "thirty-four", "triakontatessares"),
        "G9482": ("τριακοστός", "", "thirtieth", "triakostos"),
        "G9483": ("τρίβω", "G:V", "be very busy", "tribō"),
        "G9484": ("τριέτης", "", "of three years", "trietēs"),
        "G9485": ("τριετίζω", "G:V", "be three years old", "trietizō"),
        "G9486": ("τριημερία", "", "third day", "triēmeria"),
        "G9487": ("τριμερίζω", "G:V", "be three parts", "trimerizō"),
        "G9488": ("τριόδους", "", "three-pronged", "triodous"),
        "G9489": ("τριπλοῦς", "", "triple", "triplous"),
        "G9490": ("τρισκαίδεκα", "", "thirteen", "triskaideka"),
        "G9491": ("τρισκαιδέκατος", "", "thirteenth", "triskaidekatos"),
        "G9492": ("τρισσεύω", "", "to do three times", "trisseuō"),
        "G9493": ("τρισσός", "", "third rank", "trissos"),
        "G9494": ("τρισσόω", "", "to do a third time", "trissoō"),
        "G9495": ("τρισσῶς", "", "third", "trissōs"),
        "G9496": ("τριστάτης", "", "a tribune", "tristatēs"),
        "G9497": ("τριταῖος", "", "three days", "tritaios"),
        "G9498": ("τρίχαπτος", "", "a braid", "trichaptos"),
        "G9499": ("τρίχωμα", "", "hair on a head", "trichōma"),
        "G9500": ("τριώροφος", "", "third story", "triōrophos"),
        "G9501": ("τροπόω", "", "to put to flight", "tropoō"),
        "G9502": ("τροφεύω", "", "to nourish", "tropheuō"),
        "G9503": ("τροφοφορέω", "", "to nurture", "trophophoreō"),
        "G9504": ("τροχίσκος", "", "a disc", "trochiskos"),
        "G9505": ("τρυγητής", "", "a grape gatherer", "trugētēs"),
        "G9506": ("τρυγητός", "", "a gathering of crops", "trugētos"),
        "G9507": ("τρυγίας", "", "wine with dregs", "trugias"),
        "G9508": ("τρυπάω", "", "to make a hole", "trupaō"),
        "G9509": ("τρυφαλίς", "", "cheese", "truphalis"),
        "G9510": ("τρυφερός", "", "delicate", "trupheros"),
        "G9511": ("τρυφερότης", "", "delicacy", "trupherotēs"),
        "G9512": ("τρώγλη", "", "a bore", "trōglē"),
        "G9513": ("τυλόω", "G:V", "be hardened", "tuloō"),
        "G9514": ("τυμπανίστρια", "", "performing on a tambourine", "tumpanistria"),
        "G9515": ("τύμπανον", "", "a tambourine", "tumpanon"),
        "G9516": ("τυραννέω", "G:V", "be sovereign", "turanneō"),
        "G9517": ("τυραννίς", "", "a wife of a sovereign", "turannis"),
        "G9518": ("τυρόω", "", "to curdle into cheese", "turoō"),
        "G9519": ("τύχη", "", "good luck", "tuchē"),
        "G9520": ("ὕαινα", "", "a hyena", "huaina"),
        "G9521": ("ὑβριστικός", "", "outrageous", "hubristikos"),
        "G9522": ("ὑγιάζω", "", "to heal", "hugiazō"),
        "G9523": ("ὑγίεια", "", "health", "hugieia"),
        "G9524": ("ὑγιῶς", "", "fairly", "hugiōs"),
        "G9525": ("ὑγραίνω", "", "to wet", "hugrainō"),
        "G9526": ("ὑγρασία", "", "wetness", "hugrasia"),
        "G9527": ("ὑδραγωγός", "", "an aqueduct", "hudragōgos"),
        "G9528": ("ὑδρεύω", "", "to draw water", "hudreuō"),
        "G9529": ("ὑδρίσκη", "", "a water-pot", "hudriskē"),
        "G9530": ("ὑδροφόρος", "", "a water-carrier", "hudrophoros"),
        "G9531": ("ὕειος", "", "like a pig", "hueios"),
        "G9532": ("ὑετίζω", "", "to rain", "huetizō"),
        "G9533": ("ὑλακτέω", "", "to bark", "hulakteō"),
        "G9534": ("ὑλώδης", "", "flush", "hulōdēs"),
        "G9535": ("ὕμνησις", "", "singing of praise", "humnēsis"),
        "G9536": ("ὑμνῳδέω", "", "to sing a hymn", "humnōdeō"),
        "G9537": ("ὕπαιθρος", "", "open air", "hupaithros"),
        "G9538": ("ὕπατος", "", "supreme", "hupatos"),
        "G9539": ("ὑπεξαίρω", "", "to remove secretly", "hupexairō"),
        "G9540": ("ὑπεράνωθεν", "", "far above", "huperanōthen"),
        "G9541": ("ὑπέραρσις", "", "overflowing", "huperarsis"),
        "G9542": ("ὑπερασπίζω", "", "to shield", "huperaspizō"),
        "G9543": ("ὑπερασπισμός", "", "shielding", "huperaspismos"),
        "G9544": ("ὑπερασπιστής", "", "a defender", "huperaspistēs"),
        "G9545": ("ὑπερδυναμόω", "", "to overpower", "huperdunamoō"),
        "G9547": ("ὑπερεκχέω", "", "to overflow", "huperekcheō"),
        "G9548": ("ὑπερηφανέω", "G:V", "be prideful", "huperēphaneō"),
        "G9549": ("ὑπέρθυρον", "", "a lintel", "huperthuron"),
        "G9550": ("ὑπερισχύω", "", "to excel in strength", "huperischuō"),
        "G9551": ("ὑπέρκειμαι", "", "to have precedence", "huperkeimai"),
        "G9552": ("ὑπερκρατέω", "", "to prevail against", "huperkrateō"),
        "G9553": ("ὑπερμεγέθης", "", "immense", "hupermegethēs"),
        "G9554": ("ὑπερμήκης", "", "exceedingly tall", "hupermēkēs"),
        "G9555": ("ὕπερον", "", "a pestle", "huperon"),
        "G9556": ("ὑπερόρασις", "", "neglect", "huperorasis"),
        "G9557": ("ὑπεροράω", "", "to neglect", "huperoraō"),
        "G9558": ("ὑπέροψις", "", "disdain", "huperopsis"),
        "G9559": ("ὑπερτίθημι", "", "to procrastinate", "hupertithēmi"),
        "G9560": ("ὑπερφερής", "", "overwhelming", "huperpherēs"),
        "G9561": ("ὑπερφέρω", "", "to overwhelm", "huperpherō"),
        "G9562": ("ὑπερχαρής", "", "overjoyed", "hupercharēs"),
        "G9563": ("ὑπερχέω", "", "to overflow", "hupercheō"),
        "G9564": ("ὑπερωμία", "", "head and shoulders above", "huperōmia"),
        "G9565": ("ὑπεύθυνος", "", "accountable", "hupeuthunos"),
        "G9566": ("ὑπηρεσία", "", "service", "hupēresia"),
        "G9567": ("ὑπνόω", "", "to sleep", "hupnoō"),
        "G9568": ("ὑπνώδης", "", "sleepy", "hupnōdēs"),
        "G9569": ("ὑποβλέπω", "", "to suspect", "hupoblepō"),
        "G9570": ("ὑπόγαιος", "", "subterranean", "hupogaios"),
        "G9571": ("ὑποδύτης", "", "an undergarment", "hupodutēs"),
        "G9572": ("ὑπόθεμα", "", "an underpart", "hupothema"),
        "G9573": ("ὑποκαίω", "", "to fire up", "hupokaiō"),
        "G9574": ("ὑποκαλύπτω", "", "to cover up", "hupokaluptō"),
        "G9575": ("ὑποκάτωθεν", "", "from beneath", "hupokatōthen"),
        "G9576": ("ὑπόκειμαι", "", "to lay in place of", "hupokeimai"),
        "G9577": ("ὑπόλειμμα", "", "a vestige", "hupoleimma"),
        "G9578": ("ὑπόλοιπος", "", "a vestige", "hupoloipos"),
        "G9579": ("ὑπόλυσις", "", "a loosening", "hupolusis"),
        "G9580": ("ὑπολύω", "", "to untie", "hupoluō"),
        "G9581": ("ὑπόμνημα", "", "a record", "hupomnēma"),
        "G9582": ("ὑπομνηματισμός", "", "a memoir", "hupomnēmatismos"),
        "G9583": ("ὑπομνηματογράφος", "", "a recorder", "hupomnēmatographos"),
        "G9584": ("ὑπονύσσω", "", "to goad", "huponussō"),
        "G9585": ("ὑποπίπτω", "G:V", "be penitent", "hupopiptō"),
        "G9586": ("ὑποπτεύω", "", "to dread", "hupopteuō"),
        "G9587": ("ὑποπυρρίζω", "G:V", "be reddish", "hupopurrizō"),
        "G9588": ("ὑποσκελίζω", "", "to trip up", "huposkelizō"),
        "G9589": ("ὑποσκέλισμα", "", "a fall", "huposkelisma"),
        "G9590": ("ὑπόστημα", "", "a garrison", "hupostēma"),
        "G9591": ("ὑποστήριγμα", "", "a support", "hupostērigma"),
        "G9592": ("ὑποστηρίζω", "", "to support", "hupostērizō"),
        "G9593": ("ὑποτίτθιος", "", "under a breast", "hupotitthios"),
        "G9594": ("ὑπουργός", "", "an aid", "hupourgos"),
        "G9595": ("ὑπόφαυσις", "", "a narrow opening", "hupophausis"),
        "G9596": ("ὑπόφορος", "", "subject to tribute", "hupophoros"),
        "G9597": ("ὑποχείριος", "", "under one's hand", "hupocheirios"),
        "G9598": ("ὑποχόνδριος", "", "a spleen", "hupochondrios"),
        "G9599": ("ὑπόχρεως", "", "a debtor", "hupochreōs"),
        "G9600": ("ὑποχυτήρ", "", "an oil flask", "hupochutēr"),
        "G9601": ("ὑπτιάζω", "", "to turn up and open hands", "huptiazō"),
        "G9602": ("ὕπτιος", "", "steep", "huptios"),
        "G9603": ("ὑπώπιος", "", "a bruise", "hupōpios"),
        "G9604": ("ὑστεροβουλία", "", "an afterthought", "husteroboulia"),
        "G9605": ("ὑφαίνω", "", "to weave", "huphainō"),
        "G9606": ("ὑφαιρέω", "", "to take up", "huphaireō"),
        "G9607": ("ὑφάντης", "", "a weaver", "huphantēs"),
        "G9608": ("ὕφασμα", "", "a woven work", "huphasma"),
        "G9609": ("ὑφίστημι", "", "to stand", "huphistēmi"),
        "G9610": ("ὑψηλοκάρδιος", "", "a proud heart", "hupsēlokardios"),
        "G9611": ("ὕψωσις", "", "act of exaltation", "hupsōsis"),
        "G9612": ("ὕω", "", "to rain", "huō"),
        "G9613": ("φαιός", "", "gray", "phaios"),
        "G9614": ("φακός", "", "a flask", "phakos"),
        "G9615": ("φαλακρός", "", "bald", "phalakros"),
        "G9616": ("φαλάκρωμα", "", "baldness", "phalakrōma"),
        "G9617": ("φαρέτρα", "", "a quiver", "pharetra"),
        "G9618": ("φαρμακεύω", "", "to administer potions", "pharmakeuō"),
        "G9619": ("φάρμακον", "", "a potion", "pharmakon"),
        "G9620": ("φαρουρίμ", "", "a compound", "pharourim"),
        "G9621": ("φάρυγξ", "", "a throat", "pharugx"),
        "G9622": ("φασέκ", "", "a passover", "phasek"),
        "G9623": ("φάσμα", "", "a visible manifestation", "phasma"),
        "G9624": ("φατνόω", "", "to decorate with fretwork", "phatnoō"),
        "G9625": ("φάτνωμα", "", "a fretwork", "phatnōma"),
        "G9626": ("φαυλίζω", "", "to treat as worthless", "phaulizō"),
        "G9627": ("φαύλισμα", "", "a careless attitude", "phaulisma"),
        "G9628": ("φαυλισμός", "", "disparagement", "phaulismos"),
        "G9629": ("φαυλίστρια", "", "heedless", "phaulistria"),
        "G9630": ("φαῦσις", "", "giving light", "phausis"),
        "G9631": ("φερνή", "", "a dowry", "phernē"),
        "G9632": ("φερνίζω", "", "to endow", "phernizō"),
        "G9633": ("φθάρμα", "", "corruption", "phtharma"),
        "G9634": ("φθέγμα", "", "an utterance", "phthegma"),
        "G9635": ("φθειρίζομαι", "", "to fumigate", "phtheirizomai"),
        "G9636": ("φθίνω", "", "to wane", "phthinō"),
        "G9637": ("φιλαμαρτήμων", "", "fond of sinning", "philamartēmōn"),
        "G9638": ("φιλεχθρέω", "G:V", "to enjoy quarreling", "philechthreō"),
        "G9639": ("φιλιάζω", "", "to befriend", "philiazō"),
        "G9640": ("φιλογέωργος", "", "fond of husbandry", "philogeōrgos"),
        "G9641": ("φιλογύναιος", "", "fond of women", "philogunaios"),
        "G9642": ("φιλονεικέω", "G:V", "be fond of altercations", "philoneikeō"),
        "G9643": ("φιμός", "", "a rein", "phimos"),
        "G9644": ("φλεγμαίνω", "G:V", "be inflamed", "phlegmainō"),
        "G9645": ("φλέγω", "", "to blaze", "phlegō"),
        "G9646": ("φλέψ", "", "an artery", "phleps"),
        "G9647": ("φλιά", "", "a doorway", "phlia"),
        "G9648": ("φλόγινος", "", "flaming", "phloginos"),
        "G9649": ("φλυκτίς", "", "a boil", "phluktis"),
        "G9650": ("φοβερίζω", "", "to throw into fear", "phoberizō"),
        "G9651": ("φοβερισμός", "", "fright", "phoberismos"),
        "G9652": ("φοβερῶς", "", "fearfully", "phoberōs"),
        "G9653": ("φοινίκεος", "", "crimson", "phoinikeos"),
        "G9654": ("φοινικών", "", "palm-grove", "phoinikōn"),
        "G9655": ("φονευτής", "", "a man-killer", "phoneutēs"),
        "G9656": ("φονοκτονέω", "", "to pollute with murder", "phonoktoneō"),
        "G9657": ("φορβεά", "", "a halter", "phorbea"),
        "G9658": ("φορεῖον", "", "a carriage", "phoreion"),
        "G9659": ("φορθομμίν", "", "nobles", "phorthommin"),
        "G9660": ("φορολόγητος", "", "tributary", "phorologētos"),
        "G9661": ("φορολόγος", "", "tribute-gatherer", "phorologos"),
        "G9662": ("φρίκη", "", "shuddering awe", "phrikē"),
        "G9663": ("φρικτός", "", "a cause for shuddering", "phriktos"),
        "G9664": ("φρικώδης", "", "a cause for shuddering", "phrikōdēs"),
        "G9665": ("φροντίς", "", "bewilderment", "phrontis"),
        "G9666": ("φρουρά", "", "a detachment of soldiers", "phroura"),
        "G9667": ("φρουραί", "", "lots", "phrourai"),
        "G9668": ("φρύαγμα", "", "neighing", "phruagma"),
        "G9669": ("φρύγιον", "", "dried sticks", "phrugion"),
        "G9670": ("φρύγω", "", "to parch", "phrugō"),
        "G9671": ("φυγαδεία", "", "flight", "phugadeia"),
        "G9672": ("φυγαδεῖον", "", "a place of refuge", "phugadeion"),
        "G9673": ("φυγαδευτήριον", "", "a place of refuge", "phugadeutērion"),
        "G9674": ("φυγαδεύω", "", "to drive into exile", "phugadeuō"),
        "G9675": ("φυγάς", "", "an exile", "phugas"),
        "G9676": ("φυή", "", "development", "phuē"),
        "G9677": ("φύλαγμα", "", "an injunction", "phulagma"),
        "G9678": ("φυλάκισσα", "", "a female keeper", "phulakissa"),
        "G9679": ("φύλαρχος", "", "a tribal chief", "phularchos"),
        "G9680": ("φύρασις", "", "a mixing together", "phurasis"),
        "G9681": ("φυράω", "", "to mix up", "phuraō"),
        "G9682": ("φυρμός", "", "a befouling", "phurmos"),
        "G9683": ("φύρω", "", "to befoul", "phurō"),
        "G9684": ("φυσάω", "", "to blow", "phusaō"),
        "G9685": ("φυσητήρ", "", "a pair of bellows", "phusētēr"),
        "G9686": ("φύτευμα", "", "a plant", "phuteuma"),
        "G9687": ("φυτόν", "", "a plant", "phuton"),
        "G9688": ("φωράω", "G:V", "be caught in the act", "phōraō"),
        "G9689": ("χ", "", "600", "ch"),
        "G9690": ("χαίνω", "", "to gape wide", "chainō"),
        "G9691": ("χαλαστόν", "", "a chain-work", "chalaston"),
        "G9692": ("χαλβάνη", "", "galbanum", "chalbanē"),
        "G9693": ("χάλιξ", "", "gravel", "chalix"),
        "G9694": ("χαλκεῖον", "", "a brass cauldron", "chalkeion"),
        "G9695": ("χάλκειος", "", "of brass", "chalkeios"),
        "G9696": ("χαλκεύω", "", "to forge", "chalkeuō"),
        "G9697": ("χαμαιλέων", "", "chameleon", "chamaileōn"),
        "G9698": ("χάος", "", "chaos", "chaos"),
        "G9699": ("χαραδριός", "", "a curlew", "charadrios"),
        "G9700": ("χαρακοβολία", "", "a palisade", "charakobolia"),
        "G9701": ("χαρακόω", "", "to build a palisade", "charakoō"),
        "G9702": ("χαράκωσις", "", "a siege mound", "charakōsis"),
        "G9703": ("χαράσσω", "", "to grave", "charassō"),
        "G9704": ("χαρμονή", "", "a cause for joy", "charmonē"),
        "G9705": ("χαρμοσύνη", "", "joyfulness", "charmosunē"),
        "G9706": ("χαροποιός", "", "a causing joy", "charopoios"),
        "G9707": ("χαρτίον", "", "papyrus paper", "chartion"),
        "G9708": ("χαυών", "", "a cake", "chauōn"),
        "G9709": ("χαφουρή", "", "bowls", "chaphourē"),
        "G9710": ("χειμερινός", "", "of winter", "cheimerinos"),
        "G9711": ("χειροπέδαι", "", "manacles", "cheiropedai"),
        "G9712": ("χειροτονία", "", "stretching forth hands", "cheirotonia"),
        "G9713": ("χειρόω", "", "to lay hands against", "cheiroō"),
        "G9714": ("χελιδών", "", "a swallow", "chelidōn"),
        "G9715": ("χελώνη", "", "a tortoise", "chelōnē"),
        "G9716": ("χελώνιον", "", "a lip", "chelōnion"),
        "G9717": ("χερούβ", "", "a cherub", "cheroub"),
        "G9718": ("χερσαῖος", "", "of dry land", "chersaios"),
        "G9719": ("χέρσος", "", "uncultivated", "chersos"),
        "G9720": ("χερσόω", "", "to make barren", "chersoō"),
        "G9721": ("χέω", "", "to pour", "cheō"),
        "G9722": ("χηλή", "", "a claw", "chēlē"),
        "G9723": ("χηρεία", "", "widowhood", "chēreia"),
        "G9724": ("χήρευσις", "", "widowhood", "chēreusis"),
        "G9725": ("χηρεύω", "G:V", "be a widow", "chēreuō"),
        "G9726": ("χθιζός", "", "of yesterday", "chthizos"),
        "G9727": ("χθών", "", "soil", "chthōn"),
        "G9728": ("χῖδρον", "", "green wheat", "chidron"),
        "G9729": ("χιλιαρχία", "", "a command of a thousand", "chiliarchia"),
        "G9730": ("χιλιοπλασίως", "", "a thousand times more", "chilioplasiōs"),
        "G9731": ("χίλιος", "", "a thousand", "chilios"),
        "G9732": ("χίμαιρα", "", "a yearling", "chimaira"),
        "G9733": ("χίμαρος", "", "1 winter yearling", "chimaros"),
        "G9734": ("χιονόομαι", "", "to snow", "chionoomai"),
        "G9735": ("χλαῖνα", "", "a goat's hair coat", "chlaina"),
        "G9736": ("χλεύασμα", "", "an object of taunts", "chleuasma"),
        "G9737": ("χλευασμός", "", "taunting", "chleuasmos"),
        "G9738": ("χλίδων", "", "an armlet", "chlidōn"),
        "G9739": ("χλωρίζω", "G:V", "be greenish", "chlōrizō"),
        "G9740": ("χλωρότης", "", "greenness", "chlōrotēs"),
        "G9741": ("χνόος", "", "dust", "chnoos"),
        "G9742": ("χοιρογρύλλιος", "", "hyrax", "choirogrullios"),
        "G9743": ("χολέρα", "", "cholera", "cholera"),
        "G9744": ("χόλος", "", "bitter anger", "cholos"),
        "G9745": ("χονδρίτης", "", "of groats", "chondritēs"),
        "G9746": ("χορδή", "", "string of a lyre", "chordē"),
        "G9747": ("χορεύω", "", "to join in a dance", "choreuō"),
        "G9748": ("χορηγία", "", "a bestowing of expenses", "chorēgia"),
        "G9749": ("χόριον", "", "afterbirth", "chorion"),
        "G9750": ("χορρί", "", "a patrol", "chorri"),
        "G9751": ("χορτασία", "", "a filling", "chortasia"),
        "G9752": ("χορτομανέω", "G:V", "be overgrown", "chortomaneō"),
        "G9753": ("χοῦς", "", "a coos", "chous"),
        "G9754": ("χρεμετίζω", "", "to snort", "chremetizō"),
        "G9755": ("χρεμετισμός", "", "snorting", "chremetismos"),
        "G9756": ("χρέος", "", "a debt", "chreos"),
        "G9757": ("χρησμολογέω", "", "to speak oracles", "chrēsmologeō"),
        "G9758": ("χρῖσις", "", "an anointing", "chrisis"),
        "G9759": ("χροιά", "", "complexion", "chroia"),
        "G9760": ("χρυσαυγέω", "", "to shine like gold", "chrusaugeō"),
        "G9761": ("χρυσοτόρευτος", "", "wrought in gold", "chrusotoreutos"),
        "G9762": ("χρυσοχόος", "", "a goldsmith", "chrusochoos"),
        "G9763": ("χυδαῖος", "", "extensive", "chudaios"),
        "G9764": ("χύμα", "", "an increase", "chuma"),
        "G9765": ("χυτός", "", "cast", "chutos"),
        "G9766": ("χύτρα", "", "an earthen pot", "chutra"),
        "G9767": ("χυτρόγαυλος", "", "a pot", "chutrogaulos"),
        "G9768": ("χυτρόπους", "", "a pot with feet", "chutropous"),
        "G9769": ("χωθαρώθ", "", "a capital", "chōtharōth"),
        "G9770": ("χωλαίνω", "G:V", "be lame", "chōlainō"),
        "G9771": ("χῶμα", "", "an embankment", "chōma"),
        "G9772": ("χωμαρείμ", "", "idolatrous priests", "chōmareim"),
        "G9773": ("χωματίζω", "G:V", "be fortified", "chōmatizō"),
        "G9774": ("χώνευμα", "", "a molten casting", "chōneuma"),
        "G9775": ("χώνευσις", "", "a molten casting", "chōneusis"),
        "G9776": ("χωνευτήριον", "", "a foundry furnace", "chōneutērion"),
        "G9777": ("χωνευτής", "", "a smelterer", "chōneutēs"),
        "G9778": ("χωνευτός", "", "casted", "chōneutos"),
        "G9779": ("χωνεύω", "", "to cast in a furnace", "chōneuō"),
        "G9780": ("χωρισμός", "", "separation", "chōrismos"),
        "G9781": ("χωροβατέω", "", "to explore", "chōrobateō"),
        "G9782": ("ψαλίς", "", "a clip", "psalis"),
        "G9783": ("ψαλτήριον", "", "a psaltery", "psaltērion"),
        "G9784": ("ψαλτός", "", "strummed chords of music", "psaltos"),
        "G9785": ("ψαλτῳδέω", "", "to sing along with stringed instruments", "psaltōdeō"),
        "G9786": ("ψαλτῳδός", "", "a psalm singer", "psaltōdos"),
        "G9787": ("ψαρός", "", "dapple-gray", "psaros"),
        "G9788": ("ψέκας", "", "mist", "psekas"),
        "G9789": ("ψελλίζω", "", "to stutter", "psellizō"),
        "G9790": ("ψέλλιον", "", "a bracelet", "psellion"),
        "G9791": ("ψευδῶς", "", "falsely", "pseudōs"),
        "G9792": ("ψηλαφητός", "", "tangible", "psēlaphētos"),
        "G9793": ("ψήφισμα", "", "a referendum", "psēphisma"),
        "G9794": ("ψιθυρίζω", "", "to whisper", "psithurizō"),
        "G9795": ("ψιλός", "", "bare", "psilos"),
        "G9796": ("ψιλόω", "", "to make bear", "psiloō"),
        "G9797": ("ψόα", "", "a flank", "psoa"),
        "G9798": ("ψόγος", "", "fault", "psogos"),
        "G9799": ("ψοφέω", "", "to stamp", "psopheō"),
        "G9800": ("ψόφος", "", "a noise", "psophos"),
        "G9801": ("ψυγμός", "", "a refreshing", "psugmos"),
        "G9802": ("ψυκτήρ", "", "a wine-cooler", "psuktēr"),
        "G9803": ("ψύλλος", "", "a flea", "psullos"),
        "G9804": ("ψωμός", "", "a morsel", "psōmos"),
        "G9805": ("ψώρα", "", "mange", "psōra"),
        "G9806": ("ψωραγριάω", "", "to have chronic mange", "psōragriaō"),
        "G9807": ("ᾤα", "", "an edge", "ōa"),
        "G9808": ("ᾠδός", "", "a singer", "ōdos"),
        "G9809": ("ὠθέω", "", "to thrust through", "ōtheō"),
        "G9810": ("ὠμία", "", "protrusion", "ōmia"),
        "G9811": ("ὠμός", "", "raw", "ōmos"),
        "G9812": ("ὠμοτοκέω", "", "to bring forth prematurely", "ōmotokeō"),
        "G9813": ("ὡραιόομαι", "", "be beautiful", "hōraioomai"),
        "G9814": ("ὡραιότης", "", "beauty", "hōraiotēs"),
        "G9815": ("ὡραϊσμός", "", "finery", "hōraismos"),
        "G9816": ("ὥριμος", "", "in season", "hōrimos"),
        "G9817": ("ὠρύωμα", "", "a roaring", "ōruōma"),
        "G9818": ("ὡς ἂν", "", "continually", "hōs an"),
        "G9819": ("ὠτότμητος", "", "mutilated ears", "ōtotmētos"),
        "G9820": ("ὠφέλημα", "", "benefit", "ōphelēma"),
        "G9821": ("ὤχρα", "", "paleness", "ōchra"),
        "G9822": ("εὐρυχωρία", "", "expanse", "euruchōria"),
        "G9823": ("στρεπτός", "", "twisted", "streptos"),
        "G9824": ("ἵν", "", "hin", "hin"),
        "G9825": ("Ἀσέν", "", "Asen", "Asen"),
        "G9826": ("Βουγαίου", "", "Bougean", "Bougaiou"),
        "G9827": ("Ἐδέμ", "", "Edem", "Edem"),
        "G9828": ("Γεργεσαίοι", "", "Gergashites", "Gergesaioi"),
        "G9829": ("Ἡλιουπόλεως", "", "Heliopolis", "Hēlioupoleōs"),
        "G9830": ("Ὡδουά", "", "Hodijah", "Hōdoua"),
        "G9831": ("Ιανήν", "", "Janen", "Ianēn"),
        "G9832": ("Ιεμνάθ", "", "Jemnath", "Iemnath"),
        "G9833": ("Ιωθώρ", "", "Jothor", "Iōthōr"),
        "G9834": ("Παρίνοις", "", "Parian", "Parinois"),
        "G9835": ("Σουταλαάμ", "", "Sutalaam", "Soutalaam"),
        "G9836": ("Ταάμ", "", "Tam", "Taam"),
        "G9837": ("φαρές", "", "peres", "phares"),
        "G9838": ("θραέλ", "", "gate", "thrael"),
        "G9839": ("εἰκοστός", "", "twenty", "eikostos"),
        "G9840": ("Σουρ", "", "Sur", "Sour"),
        "G9841": ("Αυσίτιδος", "", "Ausis", "Ausitidos"),
        "G9842": ("ἐνθέμιον", "", "socket", "enthemion"),
        "G9843": ("ακί", "", "unexcepted", "aki"),
        "G9844": ("βλαστός", "", "offspring", "blastos"),
        "G9845": ("τυρός", "", "cheese", "turos"),
        "G9980": ("πηρόω", "", "to disable", "pēroō"),
        "G9981": ("προβάτιον", "", "lamb", "probation"),
        "G9982": ("προσανέχω", "", "to rise up", "prosanechō"),
        "G9983": ("προσβιβάζω", "", "to persuade", "prosbibazō"),
        "G9984": ("προσλέγω", "", "to answer", "proslegō"),
        "G9985": ("Πύθιος", "", "Pythius", "Puthios"),
        "G9986": ("ὑσσός", "", "javelin", "hussos"),
        "G9987": ("περιραίνω", "G:V", "to sprinkle", "perirainō"),
        "G9989": ("ὅρος", "", "boundary", "horos"),
        "G9990": ("ιβ", "G:A", "12", "ib"),
        "G9991": ("ρμδ", "G:A", "144", "rmd"),
        "G9992": ("τυπικῶς", "G:Adv", "figuratively", "tupikōs"),
        "G9993": ("ἕλιγμα", "G:N", "wrapping", "heligma"),
        "G9994": ("οἰκουργός", "G:A", "home-working", "oikourgos"),
        "G9995": ("ἱερόθυτος", "G:A", "sacrificed to idols", "eirothuros"),
        "G9996": ("συναλλάσσω", "G:V", "to reconcile", "sunallassō"),
        "G20001": ("ἀβατόομαι", "", "be made desert", "abatoomai"),
        "G20002": ("ἀβοηθησία", "", "helplessness", "aboēthēsia"),
        "G20003": ("ἀβούλευτος", "", "ill-advised,.", "abouleutos"),
        "G20004": ("ἀγαθότης", "", "goodness", "agathotēs"),
        "G20005": ("ἀγαυρίαμα", "", "insolence", "agauriama"),
        "G20006": ("ἀγαυριάομαι", "", "be insolent", "agauriaomai"),
        "G20007": ("ἀγερωχία", "", "feats of mastery", "agerōchia"),
        "G20008": ("ἀγέρωχος", "", "arrogant", "agerōchos"),
        "G20009": ("ἁγιαστία", "", "service", "hagiastia"),
        "G20010": ("ἁγιοποιέω", "G:V", "to sanctify", "hagiopoieō"),
        "G20011": ("ἁγιστεία", "", "service", "hagisteia"),
        "G20013": ("ἁγνιασμός", "", "purification", "hagniasmos"),
        "G20014": ("ἄγυια", "", "city", "aguia"),
        "G20015": ("ἀγύναιξ", "", "wifeless", "agunaix"),
        "G20016": ("ἀδελφικός", "", "sisterly", "adelphikos"),
        "G20017": ("ἀδελφοπρεπῶς", "", "as befits a brother", "adelphoprepōs"),
        "G20018": ("ἀδιάστροφος", "", "without molestation", "adiastrophos"),
        "G20019": ("ἀδιάτρεπτος", "", "headstrong", "adiatreptos"),
        "G20020": ("ἀδιεξέταστος", "", "unworthy", "adiexetastos"),
        "G20021": ("ἀδρανής", "", "useless", "adranēs"),
        "G20022": ("ἀδυναμέω", "", "be incapable", "adunameō"),
        "G20023": ("ἀέρινος", "", "grey", "aerinos"),
        "G20024": ("ἀζημίωτος", "", "immune", "azēmiōtos"),
        "G20025": ("ἀθεΐα", "", "faithlessness", "atheia"),
        "G20026": ("ἀθλοθετέω", "G:V", "to direct", "athlotheteō"),
        "G20027": ("ἄθροισμα", "", "human organism", "athroisma"),
        "G20028": ("ἀθώητος", "", "impunity", "athōētos"),
        "G20029": ("αἴγεος", "", "a goat's skin", "aigeos"),
        "G20030": ("αἰκισμός", "", "wrench, shock", "aikismos"),
        "G20031": ("αἴλουρος", "", "cat", "ailouros"),
        "G20032": ("αἱμάσσω", "", "be bloody", "ahimassō"),
        "G20033": ("αἱμόρραντος", "", "blood-sprinkled", "ahimorrantos"),
        "G20034": ("αἱρετίς", "", "one who chooses", "ahiretis"),
        "G20035": ("ἀκάλυπτος", "", "uncovered, unveiled", "akaluptos"),
        "G20036": ("ἀκαταμάχητος", "", "unconquerable", "akatamachētos"),
        "G20037": ("ἀκαταστατέω", "", "be unstable", "akatastateō"),
        "G20038": ("ἀκηλίδωτος", "", "spotless", "akēlidōtos"),
        "G20039": ("ἀκοίμητος", "", "sleepless, unresting", "akoimētos"),
        "G20040": ("ἀκοινώνητος", "", "inhuman", "akoinōnētos"),
        "G20041": ("ἀκοπίατος", "", "never tires of", "akopiatos"),
        "G20042": ("ἀκουσείω", "G:V", "to long to hear", "akouseiō"),
        "G20043": ("ἀκριβάζω", "", "be proud", "akribazō"),
        "G20044": ("ἄκριτος", "", "undiscriminating", "akritos"),
        "G20045": ("ἀκρωτηριάζω", "G:V", "to jut out", "akrōtēriazō"),
        "G20046": ("ἀλλοεθνής", "", "with foreign foe", "alloethnēs"),
        "G20047": ("ἀλλοφυλέω", "G:V", "to become native", "allophuleō"),
        "G20048": ("ἀλλοφυλισμός", "", "becoming native", "allophulismos"),
        "G20049": ("ἀμβλυωπέω", "", "be dim-sighted", "ambluōpeō"),
        "G20050": ("ἀμειδής", "", "gloomy", "ameidēs"),
        "G20051": ("ἀμείδητος", "", "gloomy", "ameidētos"),
        "G20052": ("ἀμνησία", "", "decree of amnesty", "amnēsia"),
        "G20053": ("ἀμνησικακία", "", "forgivingness", "amnēsikakia"),
        "G20054": ("ἀμνηστία", "", "amnesty", "amnēstia"),
        "G20055": ("ἀμόλυντος", "", "stainless", "amoluntos"),
        "G20056": ("ἄμυνα", "", "vengeance, requital", "amuna"),
        "G20057": ("ἀμφιάζω", "G:V", "to put on", "amphiazō"),
        "G20058": ("ἀνάβαθρα", "", "going up, ascent", "anabathra"),
        "G20059": ("ἀναβίωσις", "", "return to life", "anabiōsis"),
        "G20060": ("ἀνάγλυφος", "", "low relief", "anagluphos"),
        "G20061": ("ἀναγνεία", "", "abominable wickedness", "anagneia"),
        "G20062": ("ἀνάγωγος", "", "inerudite et", "anagōgos"),
        "G20063": ("ἀνάδυσις", "", "emergence", "anadusis"),
        "G20064": ("ἀναληπτήρ", "", "bowl", "analēptēr"),
        "G20065": ("ἀνάλογος", "", "resembling", "analogos"),
        "G20066": ("ἀνανεάζω", "G:V", "to renew youth", "ananeazō"),
        "G20067": ("ἀνανέομαι", "", "mount up", "ananeomai"),
        "G20068": ("ἀναποδίζω", "G:V", "to correct oneself", "anapodizō"),
        "G20069": ("ἀναποδισμός", "", "recall", "anapodismos"),
        "G20070": ("ἀναπτέρωσις", "", "clamorous", "anapterōsis"),
        "G20071": ("ἀνατολάς", "", "eastwards", "anatolas"),
        "G20072": ("ἀνδραγάθησις", "", "bravery", "andragathēsis"),
        "G20073": ("ἀνδρειόω", "G:V", "to become a man", "andreioō"),
        "G20074": ("ἀνδρολογία", "", "census", "andrologia"),
        "G20075": ("ἀνείκαστος", "", "incapable of artistic representation", "aneikastos"),
        "G20076": ("ἀνεκλιπής", "", "incessant", "aneklipēs"),
        "G20077": ("ἀνεπιτρέπτως", "", "without permission", "anepitreptōs"),
        "G20078": ("ἀνθράκινος", "", "(woad)", "anthrakinos"),
        "G20079": ("ἀνίκητος", "", "plaster", "anikētos"),
        "G20080": ("ἀνοιγεύς", "", "opener", "anoigeus"),
        "G20081": ("ἀνομβρέω", "G:V", "to pour forth", "anombreō"),
        "G20082": ("ἀντανακλάω", "G:V", "to cause to revert", "antanaklaō"),
        "G20083": ("ἀντιδοκέω", "G:V", "to think oneself equal to", "antidokeō"),
        "G20084": ("ἀντικάθημαι", "", "lie over against", "antikathēmai"),
        "G20085": ("ἄντικρυς", "G:Adv", "facing", "antikrus"),
        "G20086": ("ἀντιπεριβάλλω", "", "be clothed about", "antiperiballō"),
        "G20087": ("ἀντίπτωμα", "", "accident", "antiptōma"),
        "G20088": ("ἀντισχύω", "G:V", "to repel by force", "antischuō"),
        "G20089": ("ἀντιφιλοσοφέω", "", "be contrary", "antiphilosopheō"),
        "G20090": ("ἀνυπέρθετος", "", "incomparable", "anuperthetos"),
        "G20091": ("ἀπαδικέω", "G:V", "to wrong", "apadikeō"),
        "G20092": ("ἀπαρασήμαντος", "", "unnoticed", "aparasēmantos"),
        "G20093": ("ἀπασπάζομαι", "", "take leave of", "apaspazomai"),
        "G20094": ("ἀπάτησις", "", "beguiling", "apatēsis"),
        "G20095": ("ἀπειράγαθος", "", "foolish", "apeiragathos"),
        "G20096": ("ἀπεναντίον", "", "opposite", "apenantion"),
        "G20097": ("ἀπένθητος", "", "unlamented", "apenthētos"),
        "G20098": ("ἀπερίσπαστος", "", "continually", "aperispastos"),
        "G20099": ("ἀπευθανατίζω", "", "happily", "apeuthanatizō"),
        "G20100": ("ἀπλάνητος", "", "err", "aplanētos"),
        "G20101": ("ἀπληστεύομαι", "", "insatiable", "aplēsteuomai"),
        "G20102": ("ἀποβάπτω", "G:V", "to draw", "apobaptō"),
        "G20103": ("ἀπόβλημα", "", "anything cast away", "apoblēma"),
        "G20104": ("ἀποδεκατίζω", "G:V", "to tithe", "apodekatizō"),
        "G20105": ("ἀποδιαστέλλω", "", "forbidden", "apodiastellō"),
        "G20106": ("ἀποδοχεῖον", "", "cistern", "apodocheion"),
        "G20107": ("ἀποκάλυμμα", "", "arevelation", "apokalumma"),
        "G20108": ("ἀποκενόω", "G:V", "to evacuate", "apokenoō"),
        "G20109": ("ἀπόκλειστος", "", "shut off, enclosed", "apokleistos"),
        "G20110": ("ἀποκοσμέω", "G:V", "to remove", "apokosmeō"),
        "G20111": ("ἀποκτέννω", "G:V", "to kill", "apoktennō"),
        "G20112": ("ἀπομιτρόω", "G:V", "to remove the mitre", "apomitroō"),
        "G20113": ("ἀποξαίνω", "G:V", "to scarify", "apoxainō"),
        "G20114": ("ἀποπαρθενεύομαι", "", "deflower", "apopartheneuomai"),
        "G20115": ("ἀποπρατίζομαι", "", "sell", "apopratizomai"),
        "G20116": ("ἀποργίζομαι", "", "be angry", "aporgizomai"),
        "G20117": ("ἀπόρρηξις", "", "breaking", "aporrēxis"),
        "G20118": ("ἀπορρωγάς", "", "breakup", "aporrōgas"),
        "G20120": ("ἀποσκοπέω", "G:V", "to regard", "aposkopeō"),
        "G20121": ("ἀποσκορπίζω", "", "to scatter widely", "aposkorpizō"),
        "G20122": ("ἀποσκυθίζω", "", "be shaved bare", "aposkuthizō"),
        "G20123": ("ἀποστρεβαλόομαι", "", "be tortured", "apostrebaloomai"),
        "G20124": ("ἀποσφενδονάω", "G:V", "to hurl", "aposphendonaō"),
        "G20125": ("ἀποτεκνόομαι", "", "be childless", "apoteknoomai"),
        "G20126": ("ἀποτίναγμα", "", "tow", "apotinagma"),
        "G20127": ("ἀποτίνυμι", "", "to repay", "apotinumi"),
        "G20128": ("ἀπότομος", "", "precisely", "apotomos"),
        "G20129": ("ἀποχύνω", "", "to pour out", "apochunō"),
        "G20130": ("ἀπροσδεής", "", "self-sufficient", "aprosdeēs"),
        "G20131": ("ἀπτόητος", "", "undaunted", "aptoētos"),
        "G20132": ("ἄπωθεν", "", "far from", "apōthen"),
        "G20133": ("ἀπωσμός", "", "replusion", "apōsmos"),
        "G20134": ("ἀργυρολόγητος", "", "levy", "argurologētos"),
        "G20135": ("ἀργυροχόος", "", "silver worker", "argurochoos"),
        "G20136": ("ἀρδαλόω", "G:V", "to spread", "ardaloō"),
        "G20137": ("ἀρεταλογία", "", "recitation of", "aretalogia"),
        "G20138": ("ἀρετηφόρος", "", "virtuous", "aretēphoros"),
        "G20139": ("ἀρθρέμβολον", "", "instruments of torture", "arthrembolon"),
        "G20140": ("ἄρκος", "G:N-F", "bear", "arkos"),
        "G20141": ("ἁρμόνιος", "", "fitting, harmonious", "harmonios"),
        "G20142": ("ἀροτριασμός", "", "ploughing", "arotriasmos"),
        "G20143": ("ἀροτριόω", "", "be ploughed", "arotrioō"),
        "G20144": ("ἀρραβωνίζεται", "", "deposit paid", "arrabōnizetai"),
        "G20145": ("ἀρρενώδης", "", "brave.", "arrenōdēs"),
        "G20146": ("ἄρσην", "", "aršan-", "arsēn"),
        "G20147": ("ἀρχιεράομαι", "", "priestess", "archieraomai"),
        "G20148": ("ἀρχιερατεύω", "", "be high-priest", "archierateuō"),
        "G20149": ("ἀρχιερωσύνη", "", "high-priesthood", "archierōsunē"),
        "G20150": ("ἀρῳδιός", "", "heron", "arōdios"),
        "G20151": ("ἀσθενόψυχος", "", "weak-minded", "asthenopsuchos"),
        "G20152": ("ἀσκοπυτίνη", "", "leathern canteen", "askoputinē"),
        "G20153": ("ἀσταθής", "", "unsteady, unstable", "astathēs"),
        "G20154": ("ἀσυρής", "", "lewd, filthy", "asurēs"),
        "G20155": ("ἀσχολέω", "", "to do business", "ascholeō"),
        "G20156": ("ἀτεκνέω", "G:V", "to have no children", "atekneō"),
        "G20157": ("ἀτιμασμός", "", "dishonour", "atimasmos"),
        "G20158": ("ἀτίμητος", "", "invaluable", "atimētos"),
        "G20159": ("ἀτοπία", "", "wickedness, misdeed", "atopia"),
        "G20160": ("ἀτράπελος", "", "difficult", "atrapelos"),
        "G20161": ("αὐθεντία", "", "restriction", "authentia"),
        "G20162": ("αὐθωρός", "", "taking immediate effect", "authōros"),
        "G20163": ("αὐλαῖος", "", "doorkeeper", "aulaios"),
        "G20164": ("αὐτεξουσιότης", "", "free will", "autexousiotēs"),
        "G20165": ("αὐτοδέσποτος", "", "absolute master", "autodespotos"),
        "G20166": ("αὐτοσχέδιος", "", "wild, natural", "autoschedios"),
        "G20167": ("ἄφεμα", "", "taxation", "aphema"),
        "G20168": ("ἀφέστιος", "", "far from home", "aphestios"),
        "G20169": ("ἀφήγημα", "", "guiding, leading", "aphēgēma"),
        "G20170": ("ἀφόμοιος", "", "copy", "aphomoios"),
        "G20171": ("ἀφορολόγητος", "", "untaxed", "aphorologētos"),
        "G20172": ("ἀφρονεύομαι", "", "to act foolishly", "aphroneuomai"),
        "G20173": ("ἄφυκτος", "", "unable to escape", "aphuktos"),
        "G20174": ("ἀχανής", "", "vast, immense", "achanēs"),
        "G20175": ("ἀχρειότης", "", "unprofitableness, worthlessness", "achreiotēs"),
        "G20176": ("βαθύγλωσσος", "", "unintelligible", "bathuglōssos"),
        "G20177": ("βάϊς", "", "bai", "bais"),
        "G20178": ("βαρυηχής", "", "deep-roaring", "baruēchēs"),
        "G20179": ("βαρυωπέω", "", "be dim-sighted", "baruōpeō"),
        "G20180": ("βασκανία", "", "jealousy", "baskania"),
        "G20181": ("βιβλιοφυλάκιον", "", "archives", "bibliophulakion"),
        "G20182": ("βλαστάω", "G:V", "to bring forth", "blastaō"),
        "G20183": ("βόμβησις", "", "buzzing", "bombēsis"),
        "G20184": ("βοοζύγιον", "", "ox-yoke", "boozugion"),
        "G20185": ("βραχυτελής", "", "brief", "brachutelēs"),
        "G20186": ("βρῶμος", "", "stink", "brōmos"),
        "G20187": ("βύβλινος", "", "papyrus", "bublinos"),
        "G20188": ("βύβλος", "", "papyrus", "bublos"),
        "G20189": ("βυθοτρεφής", "", "living in the deep", "buthotrephēs"),
        "G20190": ("γαλακτοτροφέω", "G:V", "to give milk", "galaktotropheō"),
        "G20191": ("γαλακτοτροφία", "", "to suckle", "galaktotrophia"),
        "G20192": ("γενεσιάρχης", "", "creator", "genesiarchēs"),
        "G20193": ("γενεσιουργός", "", "fashioner, creator", "genesiourgos"),
        "G20194": ("γένημα", "", "produce", "genēma"),
        "G20195": ("γῆρος", "", "old age", "gēros"),
        "G20196": ("γίγνομαι", "", "gigno, gnatus.", "gignomai"),
        "G20197": ("γλωσσοκομεῖον", "", "coffin", "glōssokomeion"),
        "G20198": ("γνωριστής", "", "diviner", "gnōristēs"),
        "G20199": ("γονορρυέω", "", "be subject to spermatorrhoea", "gonorrueō"),
        "G20200": ("γόος", "", "for", "goos"),
        "G20201": ("δᾳδουχία", "", "torch-bearing", "dadouchia"),
        "G20202": ("δακτυλήθρα", "", "thumb-screw", "daktulēthra"),
        "G20203": ("δανίζω", "G:V", "to lend/borrow", "danizō"),
        "G20204": ("δειλανδρέω", "", "be cowardly", "deilandreō"),
        "G20205": ("δειλόομαι", "", "be afraid", "deiloomai"),
        "G20206": ("δειλόψυχος", "", "fainthearted", "deilopsuchos"),
        "G20207": ("δεινάζω", "", "be in straits", "deinazō"),
        "G20208": ("δεκαπλασιάζω", "G:V", "to multiply by ten", "dekaplasiazō"),
        "G20209": ("δεκάταρχος", "", "head of a", "dekatarchos"),
        "G20210": ("δεξιάζω", "G:V", "to approve", "dexiazō"),
        "G20211": ("δέσις", "", "joint", "desis"),
        "G20212": ("δεσποτεύω", "G:V", "to enjoy ownership of . .", "despoteuō"),
        "G20213": ("δευτερολογέω", "G:V", "to speak a second time", "deuterologeō"),
        "G20214": ("δηλαϊστός", "", "wretched", "dēlaistos"),
        "G20215": ("δημαγωγέω", "G:V", "be popular", "dēmagōgeō"),
        "G20216": ("διαβιάζομαι", "", "penetrate", "diabiazomai"),
        "G20217": ("διαγορεύω", "G:V", "to declare", "diagoreuō"),
        "G20218": ("διαγωνιάω", "G:V", "to stand in dread of", "diagōniaō"),
        "G20219": ("διαδιδράσκω", "G:V", "to fly in all directions", "diadidraskō"),
        "G20220": ("διαιτέω", "G:V", "to turn by entreaty", "diaiteō"),
        "G20221": ("διακαθίζω", "G:V", "to cause to sit apart", "diakathizō"),
        "G20222": ("διακατέχω", "G:V", "to keep on foot", "diakatechō"),
        "G20223": ("διακολυμβάω", "G:V", "to dive and swim across", "diakolumbaō"),
        "G20224": ("διακριβεία", "", "minute observance", "diakribeia"),
        "G20225": ("διάληψις", "", "digression", "dialēpsis"),
        "G20226": ("διάλιθος", "", "set with precious stones", "dialithos"),
        "G20227": ("διαλοιδόρησις", "", "railing, abuse", "dialoidorēsis"),
        "G20228": ("διαμασάομαι", "", "carp at", "diamasaomai"),
        "G20229": ("διαμελίζω", "G:V", "to dismember", "diamelizō"),
        "G20230": ("διαπαρατηρέομαι", "", "lie in wait for continually", "diaparatēreomai"),
        "G20231": ("διαπειράζω", "G:V", "to attempt", "diapeirazō"),
        "G20232": ("διαπέφλοιδεν", "", "interceed", "diapephloiden"),
        "G20233": ("διαπυρόομαι", "", "be consumed with thirst", "diapuroomai"),
        "G20234": ("διαρρυθμίζω", "G:V", "to arrange in order", "diarruthmizō"),
        "G20235": ("διασαφίζω", "", "to declare", "diasaphizō"),
        "G20236": ("διασκεδάζω", "G:V", "to disperse", "diaskedazō"),
        "G20237": ("διασκιρτάω", "G:V", "to leap about", "diaskirtaō"),
        "G20238": ("διάσταλσις", "", "arrangement, compact", "diastalsis"),
        "G20239": ("διαστράπτω", "G:V", "to glance like lightning", "diastraptō"),
        "G20240": ("διασυρίζω", "G:V", "to whistle idly", "diasurizō"),
        "G20241": ("διασφραγίζομαι", "", "seal up", "diasphragizomai"),
        "G20242": ("διατυπόω", "G:V / G:A", "be arranged, regulated by agreement", "diatupoō"),
        "G20243": ("διαφαιρέω", "G:V", "to take quite away", "diaphaireō"),
        "G20244": ("διαφθονέω", "", "be envied", "diaphthoneō"),
        "G20245": ("διαψιθυρίζω", "G:V", "to whisper", "diapsithurizō"),
        "G20246": ("διγομία", "", "double burden, load", "digomia"),
        "G20247": ("διεκκύπτω", "G:V", "to peep out", "diekkuptō"),
        "G20248": ("διεμπίμπλημι", "", "fill completely", "diempimplēmi"),
        "G20249": ("διεστραμμένως", "", "distortedly", "diestrammenōs"),
        "G20250": ("δικαιοκρίτης", "", "righteous judge", "dikaiokritēs"),
        "G20251": ("δικαιολογία", "", "forensic speeches", "dikaiologia"),
        "G20252": ("δίκησις", "", "vengeance", "dikēsis"),
        "G20253": ("διοργίζομαι", "", "be very angry", "diorgizomai"),
        "G20254": ("διορθρίζω", "G:V", "to rise early", "diorthrizō"),
        "G20255": ("διορθωτής", "", "of books, editor, reviser", "diorthōtēs"),
        "G20256": ("διορυγή", "", "canal", "diorugē"),
        "G20257": ("δισχιλιάς", "", "number of twenty thousand", "dischilias"),
        "G20258": ("διτάλαντος", "", "worth two talents", "ditalantos"),
        "G20259": ("διχομηνία", "", "mid-menstrual period", "dichomēnia"),
        "G20260": ("δορατοφόρος", "", "spear-carrying", "doratophoros"),
        "G20261": ("δορυφορία", "", "body-guard", "doruphoria"),
        "G20262": ("δράκος", "", "eye", "drakos"),
        "G20263": ("δυνάστευμα", "", "natural resources", "dunasteuma"),
        "G20264": ("δυσάθλιος", "", "most miserable", "dusathlios"),
        "G20265": ("δυσάλυκτος", "", "hard to escape", "dusaluktos"),
        "G20266": ("δυσδιήγητος", "", "hard to narrate", "dusdiēgētos"),
        "G20267": ("δυσκατάπαυστος", "", "restless", "duskatapaustos"),
        "G20268": ("δυσκλεής", "", "infamous, shameful", "duskleēs"),
        "G20269": ("δυσνοέω", "", "be ill-affected", "dusnoeō"),
        "G20270": ("δυσπέτημα", "", "misfortune", "duspetēma"),
        "G20271": ("δυσπόλεμος", "", "unlucky in war", "duspolemos"),
        "G20272": ("δυσσέβημα", "", "impious act", "dussebēma"),
        "G20273": ("ἑβδομάζω", "G:V", "to keep the Sabbath", "hebdomazō"),
        "G20274": ("ἐγγιάω", "", "be near akin", "eggiaō"),
        "G20275": ("ἐγγυιόω", "G:V", "to stretch the limbs upon", "egguioō"),
        "G20276": ("ἐγκαθυφαίνω", "G:V", "to set-firm", "egkathuphainō"),
        "G20277": ("ἐγκαίω", "G:V", "to offer sacrifice", "egkaiō"),
        "G20278": ("ἐγκαταλοχίζω", "G:V", "to divide into teams", "egkatalochizō"),
        "G20279": ("ἐγκηδεύω", "G:V", "to bury in", "egkēdeuō"),
        "G20280": ("ἐγκόλαμμα", "", "engraved inscription", "egkolamma"),
        "G20281": ("ἐγκοσμέω", "", "be adorned", "egkosmeō"),
        "G20282": ("ἔγκτησις", "", "acquisition of territory", "egktēsis"),
        "G20283": ("ἐθέλω", "G:V", "be willing", "ethelō"),
        "G20284": ("ἐθνηδόν", "", "by nations, as a whole nation", "ethnēdon"),
        "G20285": ("εἷ", "", "where", "ehi"),
        "G20286": ("εἰδέα", "", "likeness", "eidea"),
        "G20287": ("εἰδέχθεια", "", "odious, ugly look", "eidechtheia"),
        "G20288": ("εἴδησις", "", "forms of knowledge", "eidēsis"),
        "G20289": ("εἴλω", "G:V", "to constrain", "eilō"),
        "G20290": ("εἰσκολάπτω", "G:V", "to carve upon", "eiskolaptō"),
        "G20291": ("εἰσκυκλέω", "G:V", "to plunge into", "eiskukleō"),
        "G20292": ("εἰσκύπτω", "G:V", "to overhang", "eiskuptō"),
        "G20293": ("ἐκβεβηλόω", "G:V", "to profane", "ekbebēloō"),
        "G20294": ("ἐκβιάομαι", "", "to expell", "ekbiaomai"),
        "G20295": ("ἔκβολος", "", "has broken in upon", "ekbolos"),
        "G20296": ("ἐκβραγμός", "", "erruption", "ekbragmos"),
        "G20297": ("ἐκγεννάω", "G:V", "to bringforth", "ekgennaō"),
        "G20298": ("ἐκδειματόω", "", "to scare", "ekdeimatoō"),
        "G20299": ("ἐκδιαιτάω", "G:V", "to make to change one's habits", "ekdiaitaō"),
        "G20300": ("ἐκδικησία", "", "to avenge", "ekdikēsia"),
        "G20301": ("ἐκζητητής", "", "searcher out", "ekzētētēs"),
        "G20303": ("ἐκθλιβή", "", "oppression", "ekthlibē"),
        "G20304": ("ἔκθλιψις", "", "elision", "ekthlipsis"),
        "G20305": ("ἐκκοσμέω", "G:V", "to adorn", "ekkosmeō"),
        "G20306": ("ἔκλαμπρος", "", "very bright", "eklampros"),
        "G20307": ("ἔκλαμψις", "", "sudden development", "eklampsis"),
        "G20308": ("ἐκλεκτόω", "G:V", "to purify", "eklektoō"),
        "G20309": ("ἐκλεπίζω", "G:V", "to peel", "eklepizō"),
        "G20310": ("ἐκλικμάω", "G:V", "to sift", "eklikmaō"),
        "G20311": ("ἐκλογιστής", "", "accountant", "eklogistēs"),
        "G20312": ("ἐκλογιστία", "", "reckoning: accounts", "eklogistia"),
        "G20313": ("ἐκμελίζω", "G:V", "to dismember", "ekmelizō"),
        "G20314": ("ἐκπαίζω", "G:V", "to mock", "ekpaizō"),
        "G20315": ("ἐκπετάννυμι", "", "gaping", "ekpetannumi"),
        "G20316": ("ἐκπιάζω", "G:V", "to squeeze", "ekpiazō"),
        "G20317": ("ἐκπολιτεύω", "G:V", "to corrupt", "ekpoliteuō"),
        "G20318": ("ἐκριζωτής", "", "rooter out, destroyer", "ekrizōtēs"),
        "G20319": ("ἐκσαόω", "G:V", "to escape", "eksaoō"),
        "G20320": ("ἐκσπονδυλίζω", "G:V", "to break the vertebrae", "ekspondulizō"),
        "G20321": ("ἐκσυρίζω", "G:V", "to hiss loudly", "eksurizō"),
        "G20322": ("ἐκτοπίζω", "G:V", "to shun", "ektopizō"),
        "G20323": ("ἐκφλέγω", "G:V", "to warm", "ekphlegō"),
        "G20324": ("ἐκφόρτιον", "", "produce", "ekphortion"),
        "G20325": ("ἐκφυγή", "", "escape", "ekphugē"),
        "G20326": ("ἐκχάσκω", "G:V", "to gaze", "ekchaskō"),
        "G20327": ("ἐκχολάω", "", "be angry", "ekcholaō"),
        "G20328": ("ἐλαττονόω", "G:V", "to diminish", "elattonoō"),
        "G20329": ("ἐλεάω", "", "to have pity", "eleaō"),
        "G20330": ("ἐλεημοποιός", "", "giving alms", "eleēmopoios"),
        "G20331": ("ἐλευστέον", "", "one must come", "eleusteon"),
        "G20332": ("ἐλεφαντάρχης", "", "commander of a squadron of sixteen elephants", "elephantarchēs"),
        "G20333": ("ἐμβίωσις", "", "taking root", "embiōsis"),
        "G20334": ("ἐμμανής", "", "maddened by", "emmanēs"),
        "G20335": ("ἐμμελέτημα", "", "instrument for practice", "emmeletēma"),
        "G20336": ("ἐμπαρρησιάζομαι", "", "to speak openly", "emparrēsiazomai"),
        "G20337": ("ἐμπειρέω", "", "be experienced in, have knowledge of", "empeireō"),
        "G20338": ("ἐμπληθύνομαι", "", "be filled with", "emplēthunomai"),
        "G20339": ("ἐμποδιστικός", "", "trammelling", "empodistikos"),
        "G20340": ("ἐμπολιορκέω", "G:V", "to besiege", "empoliorkeō"),
        "G20341": ("ἔμπονος", "", "vehement", "emponos"),
        "G20342": ("ἐμπορπάω", "G:V", "to fastened with a brooch upon the shoulder", "emporpaō"),
        "G20343": ("ἐμπορπόομαι", "", "to wear", "emporpoomai"),
        "G20344": ("ἐμφανισμός", "", "explanation", "emphanismos"),
        "G20345": ("ἐμφύρω", "G:V", "to confuse", "emphurō"),
        "G20346": ("ἐμφυσιόω", "G:V", "to instil", "emphusioō"),
        "G20347": ("ἐναποσφραγίζω", "G:V", "to impress", "enaposphragizō"),
        "G20348": ("ἐνάρμοστος", "", "concordant", "enarmostos"),
        "G20349": ("ἐνατενίξω", "G:V", "to stare", "enatenixō"),
        "G20350": ("ἐνδείκτης", "", "informer, complainant", "endeiktēs"),
        "G20351": ("ἐνδελεχέω", "", "be persistently afflicted", "endelecheō"),
        "G20352": ("ἐνδελεχίζω", "G:V", "to persevere", "endelechizō"),
        "G20353": ("ἐνδεχομένως", "", "to the best of his ability", "endechomenōs"),
        "G20354": ("ἐνδιαλλάσσω", "G:V", "to sodomize", "endiallassō"),
        "G20355": ("ἐνδιασπείρω", "G:V", "to impregnate", "endiaspeirō"),
        "G20356": ("ἐνδοσθίδια", "", "stomach", "endosthidia"),
        "G20357": ("ἐνεμπορεύομαι", "", "trade with one in", "enemporeuomai"),
        "G20358": ("ἐνεξουσιάζω", "", "be brought into subjection", "enexousiazō"),
        "G20359": ("ἐνευκαιρέω", "G:V", "to idle", "eneukaireō"),
        "G20360": ("ἐνῆλιξ", "", "of age, in the prime of manhood", "enēlix"),
        "G20361": ("ἔνηχος", "", "sonorous", "enēchos"),
        "G20362": ("ἔνθεσμος", "", "valid", "enthesmos"),
        "G20363": ("ἐνθρύπτω", "G:V", "to crumble into liquid", "enthruptō"),
        "G20364": ("ἐνθυσιάζω", "G:V", "to sacrifice", "enthusiazō"),
        "G20365": ("ἐννέμω", "G:V", "to live amongst", "ennemō"),
        "G20367": ("ἔννευμα", "", "signal, wave of the hand", "enneuma"),
        "G20368": ("ἐννοσσοποιέομαι", "", "make oneself a nest on", "ennossopoieomai"),
        "G20369": ("ἐνσείω", "G:V", "to jostle", "enseiō"),
        "G20370": ("ἐνσκηνόω", "G:V", "to encamp", "enskēnoō"),
        "G20371": ("ἐντιμόομαι", "", "be held in honour", "entimoomai"),
        "G20372": ("ἐντιναγμός", "", "shaking", "entinagmos"),
        "G20373": ("ἐντινάσσω", "G:V", "to collide with", "entinassō"),
        "G20374": ("ἐντόσθιος", "", "inwards, entrails", "entosthios"),
        "G20375": ("ἐντότερος", "", "inner", "entoteros"),
        "G20376": ("ἐντυχία", "", "records", "entuchia"),
        "G20377": ("ἐνυποκρίνομαι", "", "play the hypocrite", "enupokrinomai"),
        "G20378": ("ἐνυποτάσσω", "", "be made subject", "enupotassō"),
        "G20379": ("ἐνυψόω", "G:V", "to exalt, excite", "enupsoō"),
        "G20380": ("ἐξάδελφος", "", "nephew", "exadelphos"),
        "G20381": ("ἐξαλλοιόω", "G:V", "to change utterly", "exalloioō"),
        "G20382": ("ἐξαλλοτριόω", "", "be estranged", "exallotrioō"),
        "G20383": ("ἐξαποστολή", "", "discharge", "exapostolē"),
        "G20384": ("ἔξαρθρος", "", "loose-jointed", "exarthros"),
        "G20385": ("ἐξαρθρόω", "G:V", "to dislocate", "exarthroō"),
        "G20386": ("ἐξατιμάζω", "G:V", "to dishonour utterly", "exatimazō"),
        "G20387": ("ἐξαφίημι", "", "squander", "exaphiēmi"),
        "G20388": ("ἐξαφίστημι", "", "grow out of", "exaphistēmi"),
        "G20389": ("ἐξέλευσις", "", "to leave", "exeleusis"),
        "G20390": ("ἐξευμενίζω", "G:V", "to propitiate", "exeumenizō"),
        "G20391": ("ἐξευφραίνομαι", "", "show rejoicing", "exeuphrainomai"),
        "G20392": ("ἐξημερόω", "G:V", "to civilize", "exēmeroō"),
        "G20393": ("ἐξηχευη", "", "stupes", "exēcheuē"),
        "G20394": ("ἐξισάζω", "G:V", "to equate", "exisazō"),
        "G20395": ("ἐξιστάνω", "", "to divert", "existanō"),
        "G20396": ("ἐξοδεύω", "G:V", "to depart", "exodeuō"),
        "G20397": ("ἐξολέθρευμα", "", "destruction", "exolethreuma"),
        "G20398": ("ἐξομβρέω", "G:V", "to pour like rain", "exombreō"),
        "G20399": ("ἐξόμνυμι", "", "swear, make affidavit", "exomnumi"),
        "G20400": ("ἐξουδενέω", "", "to dispise", "exoudeneō"),
        "G20401": ("ἔξοχος", "", "above all", "exochos"),
        "G20402": ("ἐξυπνόω", "G:V", "to wake", "exupnoō"),
        "G20403": ("ἐξυψόω", "G:V", "to elevate", "exupsoō"),
        "G20404": ("ἐξωτάτω", "", "outermost", "exōtatō"),
        "G20405": ("ἑόρτασμα", "", "festival, holiday", "heortasma"),
        "G20406": ("ἐπαίτης", "", "begging", "epaitēs"),
        "G20407": ("ἐπακουστός", "", "be obeyed", "epakoustos"),
        "G20408": ("ἐπαλγής", "", "painful", "epalgēs"),
        "G20409": ("ἐπανδρόω", "G:V", "to make manly", "epandroō"),
        "G20410": ("ἐπάνοδος", "", "fuller statement", "epanodos"),
        "G20411": ("ἐπαξονέω", "G:V", "to enroll", "epaxoneō"),
        "G20412": ("ἐπάρδω", "G:V", "to irrigate", "epardō"),
        "G20413": ("ἐπαρήγω", "G:V", "to help", "eparēgō"),
        "G20414": ("ἔπαρξις", "", "haughty", "eparxis"),
        "G20415": ("ἐπασθμαίνω", "G:V", "to pant", "epasthmainō"),
        "G20416": ("ἐπεισκυκλέω", "G:V", "to roll", "epeiskukleō"),
        "G20418": ("ἐπιθραύω", "G:V", "to break besides", "epithrauō"),
        "G20419": ("ἐπικαθιζάνω", "", "to lie down", "epikathizanō"),
        "G20420": ("ἐπικαινίζω", "G:V", "to renew", "epikainizō"),
        "G20421": ("ἐπικάλαμοι", "", "stalk", "epikalamoi"),
        "G20422": ("ἐπικαρπολογέομαι", "", "glean", "epikarpologeomai"),
        "G20423": ("ἐπικαταπνέω", "", "against", "epikatapneō"),
        "G20424": ("ἐπικινέω", "", "be moved", "epikineō"),
        "G20425": ("ἐπίκλησις", "", "appellatio, appeal", "epiklēsis"),
        "G20426": ("ἐπίκοιλος", "", "porous, spongy", "epikoilos"),
        "G20427": ("ἐπικουφίζω", "G:V", "to lighten", "epikouphizō"),
        "G20428": ("ἐπίκρασις", "", "tempering", "epikrasis"),
        "G20429": ("ἐπίκριμα", "", "decretum", "epikrima"),
        "G20430": ("ἐπίκυφος", "", "bent over, crooked", "epikuphos"),
        "G20431": ("ἐπιλαλέω", "", "be said of . .", "epilaleō"),
        "G20432": ("ἐπιλειόω", "G:V", "to shave", "epileioō"),
        "G20433": ("ἐπιλήθω", "G:V", "to forget", "epilēthō"),
        "G20435": ("ἐπιλυπέω", "", "be troubled at", "epilupeō"),
        "G20436": ("ἐπίμεικτος", "", "in combination", "epimeiktos"),
        "G20437": ("ἐπιμερίζω", "G:V", "to enumerate", "epimerizō"),
        "G20438": ("ἐπιμεσόω", "", "be at the middle", "epimesoō"),
        "G20439": ("ἐπιμήκης", "", "extending", "epimēkēs"),
        "G20440": ("ἐπιμίξ", "", "respect of persons", "epimix"),
        "G20441": ("ἐπίμοχθος", "", "with toil", "epimochthos"),
        "G20442": ("ἐπιπληθύνω", "G:V", "to superabound", "epiplēthunō"),
        "G20443": ("ἐπίπληξις", "", "plagues", "epiplēxis"),
        "G20444": ("ἐπίποκος", "", "covered with wool, woolly", "epipokos"),
        "G20445": ("ἐπιρρωγολογέομαι", "", "glean grapes off", "epirrōgologeomai"),
        "G20446": ("ἔπισος", "", "comparable", "episos"),
        "G20447": ("ἐπίστασις", "", "stoppage", "epistasis"),
        "G20448": ("ἐπισυνέχω", "G:V", "to appropriate", "episunechō"),
        "G20449": ("ἐπιτομή", "", "right of cutting", "epitomē"),
        "G20450": ("ἐπιφαυλίζω", "G:V", "to minimise", "epiphaulizō"),
        "G20451": ("ἐπιφυτεύω", "G:V", "to plant", "epiphuteuō"),
        "G20452": ("ἐπιψάλλω", "G:V", "to sing", "epipsallō"),
        "G20453": ("ἐποξύνω", "G:V", "to excite", "epoxunō"),
        "G20454": ("ἐποργίζομαι", "", "be wroth at", "eporgizomai"),
        "G20455": ("ἑπταμήτωρ", "", "mother of seven children", "heptamētōr"),
        "G20456": ("ἑπταπλόος", "", "sevenfold", "heptaploos"),
        "G20457": ("ἑπτάπυργος", "", "seven-towered", "heptapurgos"),
        "G20458": ("ἐριθεύομαι", "", "indulge in petty intrigue", "eritheuomai"),
        "G20459": ("ἐριστής", "", "to dispute", "eristēs"),
        "G20460": ("ἐρισύβη", "", "blight", "erisubē"),
        "G20461": ("ἐρυθαίνω", "G:V", "to blush", "eruthainō"),
        "G20462": ("ἐρύθρημα", "", "red", "eruthrēma"),
        "G20463": ("ἔσθω", "G:V", "to devour", "esthō"),
        "G20464": ("ἐσσήν", "", "[hudot ]ōšen", "essēn"),
        "G20465": ("ἐσχατόγηρως", "", "in extreme old age", "eschatogērōs"),
        "G20466": ("ἑτεροκωφέω", "", "be deaf of one ear", "heterokōpheō"),
        "G20467": ("εὐαγής", "", "rūgio", "euagēs"),
        "G20468": ("εὐαπάντητος", "", "affable, courteous", "euapantētos"),
        "G20469": ("εὐδράνεια", "", "bodily strength and health", "eudraneia"),
        "G20470": ("εὔθικτος", "", "witty", "euthiktos"),
        "G20471": ("εὐκατάλλακτος", "", "easily appeased, placable", "eukatallaktos"),
        "G20472": ("εὔκληρος", "", "fortunate", "euklēros"),
        "G20473": ("εὐκοπία", "", "ease, facility", "eukopia"),
        "G20474": ("εὐκοσμέω", "G:V", "to behave orderly", "eukosmeō"),
        "G20475": ("εὐλογίζω", "", "to bless", "eulogizō"),
        "G20476": ("εὔλογος", "", "honourably", "eulogos"),
        "G20477": ("εὔοπτος", "", "well-cooked", "euoptos"),
        "G20478": ("εὐπείθεια", "", "ready obedience", "eupeitheia"),
        "G20479": ("εὐπόρφυρος", "", "of bright purple colour", "euporphuros"),
        "G20480": ("εὔσταθμος", "", "in weight", "eustathmos"),
        "G20481": ("εὔτηκτος", "", "fusibility", "eutēktos"),
        "G20482": ("εὐχαρής", "", "pleasant", "eucharēs"),
        "G20483": ("εὐχαριστήριος", "", "mission of thanks", "eucharistērios"),
        "G20484": ("ἐφηβία", "", "gymnasium", "ephēbia"),
        "G20485": ("ἐφιορκία", "", "perjury", "ephiorkia"),
        "G20486": ("ἐφύβριστος", "", "contemptible", "ephubristos"),
        "G20487": ("ἐχθές", "", "yesterday", "echthes"),
        "G20488": ("ζευγίζω", "G:V", "to yoke together", "zeugizō"),
        "G20489": ("ζῆ", "", "life", "zē"),
        "G20490": ("ζῶ", "", "life", "zō"),
        "G20491": ("ζωγρίας", "", "one taken alive", "zōgrias"),
        "G20492": ("ζωνῖτις", "", "seams", "zōnitis"),
        "G20493": ("ζωτικός", "", "true to life", "zōtikos"),
        "G20494": ("ἡγεμόνη", "", "flagship", "hēgemonē"),
        "G20495": ("ἡγεμονίδης", "", "govenor", "hēgemonidēs"),
        "G20496": ("ἥγησις", "", "command", "hēgēsis"),
        "G20497": ("ἡδομένως", "", "with joy, gladly", "hēdomenōs"),
        "G20498": ("ἠθολογέω", "", "comedy of manners", "ēthologeō"),
        "G20499": ("ἡλιάζω", "G:V", "to ferment", "hēliazō"),
        "G20500": ("ἡμεράλωψ", "", "nictilopa", "hēmeralōps"),
        "G20501": ("ἡμιθαλής", "", "half-green", "hēmithalēs"),
        "G20502": ("ἡμίθνητος", "", "half-dead", "hēmithnētos"),
        "G20503": ("ἡμισεύελπις", "", "half-hopeful", "hēmiseuelpis"),
        "G20504": ("ἡμισευτής", "", "half time", "hēmiseutēs"),
        "G20505": ("ἡμισύνθεσις", "", "half-set", "hēmisunthesis"),
        "G20507": ("ἠνίδε", "", "look!", "ēnide"),
        "G20508": ("ἤνυστρον", "", "first stomach", "ēnustron"),
        "G20509": ("ἡπατοσκοπέω", "G:V", "to inspect the liver for soothsaying", "hēpatoskopeō"),
        "G20510": ("ἠπιότης", "", "gentleness", "ēpiotēs"),
        "G20511": ("ἠχέτης", "", "sounding", "ēchetēs"),
        "G20512": ("θεόκτιστος", "", "founded by God", "theoktistos"),
        "G20513": ("θεριστήριον", "", "reaping-hook", "theristērion"),
        "G20514": ("θοίνη", "", "feeding upon", "thoinē"),
        "G20515": ("θραῦμα", "", "destruction", "thrauma"),
        "G20516": ("θρησκεύω", "", "be a devotee", "thrēskeuō"),
        "G20517": ("θρονίζομαι", "", "be initiated, consecrated", "thronizomai"),
        "G20518": ("θυμαρής", "", "well-pleasing, delightful", "thumarēs"),
        "G20519": ("θυρεόφορος", "", "armed with such a shield", "thureophoros"),
        "G20521": ("θωρακισμός", "", "arming with breastplates", "thōrakismos"),
        "G20522": ("ἰδιόγραφος", "", "separately written", "idiographos"),
        "G20523": ("ἰδιότης", "", "relationship", "idiotēs"),
        "G20524": ("ἱερεία", "", "sanctuary", "hiereia"),
        "G20525": ("ἱερόδουλος", "", "templecourtesans", "hierodoulos"),
        "G20526": ("ἱεροσύλημα", "", "sacrilege", "hierosulēma"),
        "G20527": ("ἱεροψάλτης", "", "singer in the temple", "hieropsaltēs"),
        "G20528": ("ἱερόψυχος", "", "of holy, pious soul", "hieropsuchos"),
        "G20529": ("ἱέρωμα", "", "consecrated object, offering", "hierōma"),
        "G20530": ("ἴκτηρ", "", "suppliant", "iktēr"),
        "G20531": ("ἱλαροτραγῳδία", "", "burlesque tragedy", "hilarotragōdia"),
        "G20532": ("ἱλάσιμος", "", "placable", "hilasimos"),
        "G20533": ("ἱλαστής", "", "propitiator", "hilastēs"),
        "G20534": ("ἱλατεύω", "", "gracious", "hilateuō"),
        "G20535": ("ἱμάντωσις", "", "piece of timber used instead of a bond-stone", "himantōsis"),
        "G20536": ("ἰσάστερος", "", "like a star, bright as a star", "isasteros"),
        "G20537": ("ἰσηγορέομαι", "", "speak as an equal", "isēgoreomai"),
        "G20538": ("ἰσοδυναμέω", "", "be equivalent", "isodunameō"),
        "G20539": ("ἰσοδύναμος", "", "equal", "isodunamos"),
        "G20540": ("ἰσοπολίτης", "", "equitable citizen", "isopolitēs"),
        "G20541": ("ἴσχυσις", "", "strength", "ischusis"),
        "G20542": ("καθάπαν", "", "on the whole", "kathapan"),
        "G20543": ("καθαριόω", "G:V", "to purify", "katharioō"),
        "G20544": ("καθάρισις", "", "cleansing", "katharisis"),
        "G20545": ("καθεῖς", "", "one by one, one after antoher", "katheis"),
        "G20546": ("καθεύρεμα", "", "invention", "katheurema"),
        "G20547": ("καθηγεμών", "", "leader", "kathēgemōn"),
        "G20548": ("καθιδρύω", "G:V", "to settle", "kathidruō"),
        "G20549": ("καθιστάνω", "G:V", "to establish", "kathistanō"),
        "G20550": ("καινουργός", "", "novelty", "kainourgos"),
        "G20551": ("κακοεξία", "", "bad habit", "kakoexia"),
        "G20552": ("κακόμοχθος", "", "perversely", "kakomochthos"),
        "G20553": ("κακοπραγία", "", "misdeeds", "kakopragia"),
        "G20554": ("καμιναία", "", "furnace", "kaminaia"),
        "G20555": ("καπνοδόκη", "", "hole in the roof for the smoke to pass through", "kapnodokē"),
        "G20556": ("κάρρον", "", "coach-builder", "karron"),
        "G20557": ("καρτερός", "", "resolutely", "karteros"),
        "G20558": ("καρτεροψυχία", "", "strength of spirit", "karteropsuchia"),
        "G20559": ("καταβαρύνω", "G:V", "to overload", "katabarunō"),
        "G20560": ("καταβάσιος", "", "descent", "katabasios"),
        "G20561": ("καταγογγύζω", "G:V", "to murmur against", "katagogguzō"),
        "G20562": ("καταδαμάζω", "G:V", "to subdue", "katadamazō"),
        "G20563": ("καταδαπανάω", "G:V", "to consume", "katadapanaō"),
        "G20564": ("καταδεσμεύω", "G:V", "to repeat", "katadesmeuō"),
        "G20565": ("καταδεύω", "G:V", "to make wet", "katadeuō"),
        "G20566": ("καταδέω", "G:V", "to lack", "katadeō"),
        "G20567": ("καταδρομή", "", "cryptoporticus", "katadromē"),
        "G20568": ("κατάκλειστος", "", "precious", "katakleistos"),
        "G20569": ("κατάκλιτος", "", "flowing down", "kataklitos"),
        "G20570": ("καταλαλητέον", "", "one must talk against", "katalalēteon"),
        "G20571": ("καταλεαίνω", "G:V", "to placate", "kataleainō"),
        "G20572": ("καταμερισμός", "", "distribution", "katamerismos"),
        "G20573": ("καταφαγεῖν", "", "spend in eating, waste", "kataphagein"),
        "G20574": ("καταφράσσω", "G:V", "to protect", "kataphrassō"),
        "G20575": ("καταφρονητέον", "", "one must despise", "kataphronēteon"),
        "G20576": ("κατάχρεος", "", "involved in", "katachreos"),
        "G20577": ("καταψευσμός", "", "slander, calumny", "katapseusmos"),
        "G20578": ("κατεμβλέπω", "G:V", "to despise", "katemblepō"),
        "G20579": ("κατενισχύω", "G:V", "to establish", "katenischuō"),
        "G20580": ("κατένωπα", "", "over against, right opposite", "katenōpa"),
        "G20581": ("κατεπίθυμος", "", "very eager", "katepithumos"),
        "G20582": ("κατεπικύπτω", "G:V", "to bow down upon", "katepikuptō"),
        "G20583": ("κατερευνάω", "G:V", "to dīrīmat", "katereunaō"),
        "G20584": ("κατευθικτέω", "G:V", "to hit exactly", "kateuthikteō"),
        "G20586": ("κατευφημέω", "G:V", "to applaud", "kateuphēmeō"),
        "G20587": ("κατιμονεύω", "G:V", "to let down", "katimoneuō"),
        "G20588": ("κατοδυνάω", "G:V", "to afflict grievously", "katodunaō"),
        "G20590": ("κατοικτείρω", "G:V", "to feel compassion", "katoikteirō"),
        "G20591": ("κατοινύσαι", "", "hidden", "katoinusai"),
        "G20592": ("κατοπίσω", "G:V", "to overtake", "katopisō"),
        "G20593": ("κατώτατος", "", "lowest", "katōtatos"),
        "G20594": ("καυστικός", "", "inflammatory humours", "kaustikos"),
        "G20595": ("καυτήριον", "", "kiln", "kautērion"),
        "G20596": ("κειράς", "", "shorn", "keiras"),
        "G20597": ("κελευσματικῶς", "", "by way of command", "keleusmatikōs"),
        "G20598": ("κενεών", "", "vacant space", "keneōn"),
        "G20599": ("κενοδοξέω", "", "be vain-glorious", "kenodoxeō"),
        "G20600": ("κηρογονία", "", "combs", "kērogonia"),
        "G20601": ("κίνημα", "", "moving things", "kinēma"),
        "G20602": ("κισσόφυλλον", "", "part of a torsion-engine shaped like an ivy-leaf", "kissophullon"),
        "G20603": ("κλεψιμαῖος", "", "furtim", "klepsimaios"),
        "G20604": ("κληδών", "", "name, appellation", "klēdōn"),
        "G20605": ("κληρουχία", "", "inheritance", "klērouchia"),
        "G20606": ("κλητέος", "", "one must call", "klēteos"),
        "G20607": ("κμητός", "", "scratch", "kmētos"),
        "G20608": ("κοίλασμα", "", "interior", "koilasma"),
        "G20609": ("κοιλότης", "", "shortage of cash", "koilotēs"),
        "G20610": ("κολαβρισμός", "", "derided", "kolabrismos"),
        "G20611": ("κολλύρα", "", "bake", "kollura"),
        "G20612": ("κολλύριον", "", "PHolm., PLeid.X.", "kollurion"),
        "G20613": ("κολοβόρινος", "", "slit-nosed", "koloborinos"),
        "G20615": ("κοσμοπληθής", "", "filling the world", "kosmoplēthēs"),
        "G20616": ("κράτησις", "", "holding firm, steadying", "kratēsis"),
        "G20617": ("κρυφός", "", "lurking-place", "kruphos"),
        "G20618": ("κύκλος", "", "sphere, globe", "kuklos"),
        "G20619": ("κύκνειος", "", "of Cycnus", "kukneios"),
        "G20620": ("κυλίκιον", "", "small cup", "kulikion"),
        "G20621": ("κυνάμυια", "", "shameless fly", "kunamuia"),
        "G20622": ("κυνήγιον", "", "prey", "kunēgion"),
        "G20623": ("κυοφορία", "", "pregnancy", "kuophoria"),
        "G20624": ("κύπρος", "", "a measure of corn", "kupros"),
        "G20625": ("κυρία", "", "possession, control", "kuria"),
        "G20626": ("κωλῆ", "", "membrum virile", "kōlē"),
        "G20627": ("κωνώπιον", "", "cōnōpēum", "kōnōpion"),
        "G20628": ("Λακωνικός", "", "female garment", "Lakōnikos"),
        "G20629": ("λάμψις", "", "shining", "lampsis"),
        "G20630": ("λαογραφία", "", "poll-tax", "laographia"),
        "G20631": ("λαπιστής", "", "swaggerer", "lapistēs"),
        "G20632": ("λατόμημα", "", "stone hewn from a quarry", "latomēma"),
        "G20633": ("λαφυρεύω", "G:V", "to plunder", "laphureuō"),
        "G20634": ("λεληθότως", "", "secretly", "lelēthotōs"),
        "G20635": ("λεοντηδόν", "", "like a lion", "leontēdon"),
        "G20636": ("λέπυρον", "", "rind, shell, husk", "lepuron"),
        "G20637": ("λεχώς", "", "giving birth", "lechōs"),
        "G20638": ("λιβανόομαι", "", "be mixed with frankincense", "libanoomai"),
        "G20639": ("λιθοβόλος", "", "struck with stones, stoned", "lithobolos"),
        "G20640": ("λιποτακτέω", "G:V", "to desert", "lipotakteō"),
        "G20641": ("λιτανεία", "", "entreaty", "litaneia"),
        "G20642": ("λοιμότης", "", "pestilent condition", "loimotēs"),
        "G20643": ("λοχώ", "", "to give birth", "lochō"),
        "G20644": ("λυθρώδης", "", "defiled with gore", "luthrōdēs"),
        "G20645": ("λυσιτέλεια", "", "economy", "lusiteleia"),
        "G20646": ("λυτρωτός", "", "redeemable", "lutrōtos"),
        "G20647": ("λωποδυτέω", "G:V", "to rob", "lōpoduteō"),
        "G20648": ("μάγδωλος", "", "migdol", "magdōlos"),
        "G20649": ("μαγειρευτικός", "", "cooking", "mageireutikos"),
        "G20650": ("μαγικός", "", "skilled in magic", "magikos"),
        "G20651": ("μάγιν", "", "cooking", "magin"),
        "G20652": ("μαδαρότης", "", "falling off of the eyelashes", "madarotēs"),
        "G20653": ("μαδάω", "G:V", "be bald", "madaō"),
        "G20654": ("μαδών", "", "íš mādhón", "madōn"),
        "G20655": ("μᾶζα", "", "amalgam", "maza"),
        "G20656": ("μακροβίωσις", "", "long living", "makrobiōsis"),
        "G20657": ("μακροημέρευσις", "", "length of days", "makroēmereusis"),
        "G20658": ("μάκροθριξ", "", "long-haired", "makrothrix"),
        "G20659": ("μακροτέρω", "", "farther off", "makroterō"),
        "G20660": ("μακροτονέω", "G:V", "to persevere", "makrotoneō"),
        "G20661": ("μακροχρονίζω", "", "long-lasting", "makrochronizō"),
        "G20662": ("μαλακία", "", "calmness of the sea, malacia ac tranquillitas", "malakia"),
        "G20663": ("μαλακοψυχέω", "", "be cowardly", "malakopsucheō"),
        "G20664": ("μαλάκυνσις", "", "softening", "malakunsis"),
        "G20665": ("μαρτιχόρας", "", "mard-khwār", "martichoras"),
        "G20666": ("μαστιγοφόρος", "", "policeman", "mastigophoros"),
        "G20667": ("μαστιγωτικός", "", "demanding vengeance", "mastigōtikos"),
        "G20668": ("μαστιστής", "", "scourger", "mastistēs"),
        "G20669": ("ματαιοτεχνία", "", "dicta est", "mataiotechnia"),
        "G20670": ("ματαιόφρων", "", "weak-minded", "mataiophrōn"),
        "G20671": ("ματαιόφωνος", "", "talking idly", "mataiophōnos"),
        "G20672": ("μαχητέον", "", "one must fight", "machēteon"),
        "G20673": ("μεγάλαρτος", "", "large loaf", "megalartos"),
        "G20674": ("μεγαλαυχία", "", "exultation", "megalauchia"),
        "G20675": ("μεγαλόδοξος", "", "very glorious", "megalodoxos"),
        "G20676": ("μεγαλοημέρευσις", "", "long-lived", "megaloēmereusis"),
        "G20677": ("μεγαλοκράτωρ", "", "long-ruling", "megalokratōr"),
        "G20678": ("μεγαλομερής", "", "magnificent, sumptuous", "megalomerēs"),
        "G20679": ("μεγαλοποιέω", "G:V", "to magnify", "megalopoieō"),
        "G20680": ("μεγαλορρέκτης", "", "one who does great things", "megalorrektēs"),
        "G20681": ("μεγάλος", "", "great", "megalos"),
        "G20682": ("μεγαλοσθενής", "", "of great strength", "megalosthenēs"),
        "G20683": ("μεγαλοφρονέω", "", "be arrogant", "megalophroneō"),
        "G20684": ("μεγαλόψυχος", "", "romantic, Quixotic", "megalopsuchos"),
        "G20685": ("μεγαλώνυμος", "", "having a higher denominator", "megalōnumos"),
        "G20686": ("μεγαλωστί", "", "far and wide, over a vast space", "megalōsti"),
        "G20687": ("μεγαρτός", "", "envious", "megartos"),
        "G20688": ("μεγήριτος", "", "passing lovely", "megēritos"),
        "G20689": ("μεθαρμόζω", "G:V", "to re-order", "metharmozō"),
        "G20690": ("μεθερμηνευτικός", "", "fit for interpreting", "methermēneutikos"),
        "G20691": ("μεθήμων", "", "remiss, careless", "methēmōn"),
        "G20692": ("μεθοδευτικός", "", "regular", "methodeutikos"),
        "G20693": ("μέθοδος", "", "stratagem", "methodos"),
        "G20694": ("μεθορίζω", "G:V", "to determine", "methorizō"),
        "G20695": ("μέθυσις", "", "drunkenness", "methusis"),
        "G20696": ("μεθυσοκότταβος", "", "drunk with cottabus-playing", "methusokottabos"),
        "G20697": ("μελάθρα", "", "beam", "melathra"),
        "G20698": ("μελετητικός", "", "of meditations", "meletētikos"),
        "G20699": ("μελετητός", "", "be gained by practice", "meletētos"),
        "G20700": ("μελισσόφυτον", "", "balm", "melissophuton"),
        "G20701": ("μελοποιός", "", "tuneful", "melopoios"),
        "G20702": ("μεριδάρχης", "", "province", "meridarchēs"),
        "G20703": ("μεριδαρχία", "", "his office", "meridarchia"),
        "G20704": ("μεριζομένως", "", "suffering", "merizomenōs"),
        "G20705": ("μερινων", "", "portioned", "merinōn"),
        "G20706": ("μεριτεία", "", "division", "meriteia"),
        "G20707": ("μεροποσπόρος", "", "begetting men", "meroposporos"),
        "G20708": ("μεσάζω", "G:V", "to occupy a central position", "mesazō"),
        "G20709": ("μεσημβρίζω", "", "noon", "mesēmbrizō"),
        "G20710": ("μεσονεφής", "", "in the midst", "mesonephēs"),
        "G20711": ("μεσοπορέω", "", "be half-way", "mesoporeō"),
        "G20712": ("μεσοπόρος", "", "mid", "mesoporos"),
        "G20713": ("μεσορραγής", "", "rent in twain", "mesorragēs"),
        "G20714": ("μεταγίγνομαι", "", "be transferred, carried away", "metagignomai"),
        "G20715": ("μεταδιαιτάω", "G:V", "to change one's way of life", "metadiaitaō"),
        "G20716": ("μετακιρνάω", "G:V", "to intermix", "metakirnaō"),
        "G20717": ("μεταλλευτός", "", "be got by mining", "metalleutos"),
        "G20718": ("μετανάστατος", "", "having departed", "metanastatos"),
        "G20719": ("μεταπαιδεύω", "G:V", "to acquire a fresh tendency", "metapaideuō"),
        "G20720": ("μετατεύχω", "G:V", "to refashion", "metateuchō"),
        "G20721": ("μετεξέτεροι", "", "a certain amount of", "metexeteroi"),
        "G20722": ("μετεωροπορία", "", "travelling through air", "meteōroporia"),
        "G20723": ("μετοιακίζομαι", "", "have one's course changed", "metoiakizomai"),
        "G20724": ("μετοίκησις", "", "transmigration", "metoikēsis"),
        "G20725": ("μετοχετεύω", "G:V", "to divert", "metocheteuō"),
        "G20726": ("μετρητέον", "", "one must measure", "metrēteon"),
        "G20727": ("μὴ οὐ", "", "isn't it?", "mē ou"),
        "G20728": ("μήλωσις", "", "use of the probe", "mēlōsis"),
        "G20729": ("μηνίαμα", "", "wrath", "mēniama"),
        "G20730": ("μηνιάω", "", "be wrathful", "mēniaō"),
        "G20731": ("μηνιτής", "", "wrathful man", "mēnitēs"),
        "G20732": ("μὴπολλάκις", "", "lest perchance", "mēpollakis"),
        "G20733": ("μηροραφής", "", "thigh-burnt", "mēroraphēs"),
        "G20734": ("μηρυκίζω", "", "to chew the cud", "mērukizō"),
        "G20735": ("μηχανάριος", "", "engineer", "mēchanarios"),
        "G20736": ("μιαροφαγέω", "G:V", "to eat abominable food", "miarophageō"),
        "G20737": ("Μιμνέρμειον", "", "gymnasium named after Mimnermus", "Mimnermeion"),
        "G20738": ("μισάρετος", "", "hating virtue", "misaretos"),
        "G20739": ("μίσευμα", "", "half", "miseuma"),
        "G20740": ("μισθαρχίδης", "", "mercenary", "mistharchidēs"),
        "G20741": ("μισοξενία", "", "guests", "misoxenia"),
        "G20742": ("μισοπονηρέω", "G:V", "to hate wickedness", "misoponēreō"),
        "G20743": ("μισοπονηρία", "", "hatred because of wickedness", "misoponēria"),
        "G20744": ("μισοπόνηρος", "", "with hostile sentiments", "misoponēros"),
        "G20745": ("μίσυβρις", "", "hating insolence", "misubris"),
        "G20746": ("Μίτρα", "", "Mitra", "Mitra"),
        "G20747": ("μνημονευτός", "", "objects of memory", "mnēmoneutos"),
        "G20748": ("μνησίθεος", "", "remembering God", "mnēsitheos"),
        "G20749": ("μνησικακία", "", "remembrance of wrongs", "mnēsikakia"),
        "G20750": ("μοιχαίνω", "", "to commit adultery", "moichainō"),
        "G20751": ("μοιχεύτρια", "", "adulteress", "moicheutria"),
        "G20752": ("μόλυβος", "", "lead", "molubos"),
        "G20753": ("μολυνοπραγμονέομαι", "", "get into dirty quarrels", "molunopragmoneomai"),
        "G20754": ("μολυντός", "", "apt to make dirty", "moluntos"),
        "G20755": ("μόλυσμα", "", "spot, taint", "molusma"),
        "G20756": ("μονήμερος", "", "lasting one day", "monēmeros"),
        "G20757": ("μονικῶς", "", "uniquely", "monikōs"),
        "G20758": ("μονιμότης", "", "constancy, steadfastness", "monimotēs"),
        "G20759": ("μονόζυξ", "", "single, solitary", "monozux"),
        "G20760": ("μονοκένταυρος", "", "man with ox's head", "monokentauros"),
        "G20761": ("μονορύχης", "", "digging with one point", "monoruchēs"),
        "G20762": ("μονοφαγία", "", "eating alone", "monophagia"),
        "G20763": ("μονοφάγος", "", "eating alone", "monophagos"),
        "G20764": ("μορφήεις", "", "well-formed, shapely", "morphēeis"),
        "G20765": ("μόσχευμα", "", "PLond.ined.", "moscheuma"),
        "G20766": ("μοχθίζω", "G:V", "to suffer", "mochthizō"),
        "G20767": ("μυελοποιός", "", "strengthening", "muelopoios"),
        "G20768": ("μυθόλογος", "", "prating", "muthologos"),
        "G20769": ("μυκτηριάζω", "G:V", "to dry nose", "muktēriazō"),
        "G20770": ("μυλήφατος", "", "bruised in a mill", "mulēphatos"),
        "G20771": ("μυλόομαι", "", "be hardened, cicatrized", "muloomai"),
        "G20772": ("μυρεψία", "", "preparation of unguents", "murepsia"),
        "G20773": ("μυρέψιον", "", "prepared unguent", "murepsion"),
        "G20774": ("μυρίνινος", "", "of myrrh", "murininos"),
        "G20775": ("μυριοπάλαι", "", "time out of mind", "muriopalai"),
        "G20776": ("μυριότης", "", "myriad", "muriotēs"),
        "G20777": ("μυρισμός", "", "an anointing", "murismos"),
        "G20778": ("μυρμηκίασις", "", "irritation of the skin", "murmēkiasis"),
        "G20779": ("μυρμηκοειδής", "", "like an ant", "murmēkoeidēs"),
        "G20780": ("μυροβραχής", "", "wet with unguent", "murobrachēs"),
        "G20781": ("μυσαρία", "", "loathsomeness", "musaria"),
        "G20782": ("μυσάρχης", "", "the originator of a foul deed", "musarchēs"),
        "G20783": ("μυσερός", "", "defiled", "museros"),
        "G20784": ("μυστάδης", "", "teacher", "mustadēs"),
        "G20785": ("μυστηρικός", "", "for mysteries", "mustērikos"),
        "G20786": ("μυστικός", "", "scribere", "mustikos"),
        "G20787": ("μύστις", "", "initiator", "mustis"),
        "G20788": ("μώϊον", "", "jar", "mōion"),
        "G20789": ("μώκημα", "", "mockery", "mōkēma"),
        "G20790": ("μωκός", "", "mocker", "mōkos"),
        "G20791": ("μωλωπικός", "", "suitable for weals", "mōlōpikos"),
        "G20792": ("μώμημα", "", "blame", "mōmēma"),
        "G20793": ("μῶν", "G:Neg", "are you", "mōn"),
        "G20794": ("μωρεύω", "G:V", "be foolish", "mōreuō"),
        "G20795": ("μωροπόνηρος", "", "stupidly wicked", "mōroponēros"),
        "G20796": ("ναῦλος", "", "rent", "naulos"),
        "G20797": ("νάφθα", "", "naft", "naphtha"),
        "G20798": ("νεικέω", "G:V", "to quarrell", "neikeō"),
        "G20799": ("νεκριμαῖος", "", "carcas", "nekrimaios"),
        "G20800": ("νεόκτιστος", "", "newly created", "neoktistos"),
        "G20801": ("νεοσσιά", "", "beehive", "neossia"),
        "G20802": ("νεοσσοποιέω", "", "to build a nest", "neossopoieō"),
        "G20803": ("νηπιοκτόνος", "", "slaying children", "nēpioktonos"),
        "G20804": ("νίζω", "G:V", "to wash hands", "nizō"),
        "G20805": ("νοερός", "", "world", "noeros"),
        "G20806": ("νοθεύω", "G:V", "to consider spurious", "notheuō"),
        "G20807": ("νόμαρχος", "", "govenor", "nomarchos"),
        "G20808": ("νομιστέος", "", "one must account, deem", "nomisteos"),
        "G20809": ("νομοφύλαξ", "", "observer of the law", "nomophulax"),
        "G20810": ("νόος", "", "point", "noos"),
        "G20811": ("νοσφιδόν", "", "by stealth", "nosphidon"),
        "G20812": ("νυμφικός", "", "of the Nymphs", "numphikos"),
        "G20813": ("νωθρότης", "", "sluggishness", "nōthrotēs"),
        "G20814": ("Ξανδικός", "", "Month Eandikos", "Xandikos"),
        "G20815": ("ξενιτεία", "", "life of a mercenary in foreign service", "xeniteia"),
        "G20816": ("ξενολογέω", "G:V", "to enlist mercenaries", "xenologeō"),
        "G20817": ("ξενοτροφέω", "G:V", "to maintain mercenary troops", "xenotropheō"),
        "G20818": ("ξεστός", "", "smooth", "xestos"),
        "G20819": ("οἰκετικός", "", "slaves", "oiketikos"),
        "G20820": ("οἰκίον", "", "nest", "oikion"),
        "G20821": ("οἰνοδόχος", "", "cupbearer", "oinodochos"),
        "G20822": ("οἰστρηλασία", "", "mad passion", "oistrēlasia"),
        "G20823": ("Οἰτόσυρος", "", "Oitosupos (Apollo)", "Oitosuros"),
        "G20824": ("οἰωνόβρωτος", "", "be eaten of birds", "oiōnobrōtos"),
        "G20825": ("ὄλβος", "", "olg[uglide]ā", "olbos"),
        "G20826": ("ὀλεθρεία", "", "fated", "olethreia"),
        "G20827": ("ὀλεθροφόρος", "", "destruction-bringing", "olethrophoros"),
        "G20828": ("ὀλιγοποιέω", "G:V", "to diminish", "oligopoieō"),
        "G20829": ("ὁλοκαρπόομαι", "", "be offered as a whole burnt-offering", "holokarpoomai"),
        "G20830": ("ὄλονθος", "", "sterile male inflorescence of the caprifig", "olonthos"),
        "G20831": ("ὁλορριζί", "", "utterly", "holorrizi"),
        "G20832": ("ὁλοσφύρατος", "", "made of solid beaten metal", "holosphuratos"),
        "G20833": ("ὁμείρομαι", "", "yearn for", "homeiromai"),
        "G20834": ("ὀμναῖσαι", "", "remembered", "omnaisai"),
        "G20835": ("ὁμοζηλία", "", "common zeal", "homozēlia"),
        "G20836": ("ὁμόσπονδος", "", "bound by treaty to", "homospondos"),
        "G20837": ("ὁμόψυχος", "", "endowed with the same soul", "homopsuchos"),
        "G20838": ("ὀνοκάρδιον", "", "chamelion", "onokardion"),
        "G20839": ("ὀνοματογραφία", "", "writing of names", "onomatographia"),
        "G20840": ("ὁπηνίκα", "", "when once", "hopēnika"),
        "G20841": ("ὁπλοδοτέω", "G:V", "to arm", "hoplodoteō"),
        "G20842": ("ὁπλολογέω", "", "be disarmed", "hoplologeō"),
        "G20843": ("ὁπλοποιέω", "G:V", "to make weapons", "hoplopoieō"),
        "G20845": ("ὀργιάω", "G:V", "be fierce", "orgiaō"),
        "G20846": ("ὀρθρόω", "", "dawn", "orthroō"),
        "G20847": ("ὀρνιθοπώλης", "", "dealer in birds", "ornithopōlēs"),
        "G20848": ("ὀροφοιτάω", "", "frequent mountains", "orophoitaō"),
        "G20849": ("ὀροφόω", "", "be roofed", "orophoō"),
        "G20850": ("ὀστρακώδης", "", "full of potsherds", "ostrakōdēs"),
        "G20851": ("ὀσφράδιον", "", "nosegay", "osphradion"),
        "G20852": ("οὐ μή", "", "surely not", "ou mē"),
        "G20853": ("οὐθείς", "", "none", "outheis"),
        "G20854": ("οὐραγέω", "G:V", "to lag behind", "ourageō"),
        "G20855": ("ὀφιόδηκτος", "", "bitten by a serpent", "ophiodēktos"),
        "G20856": ("ὄφλησις", "", "penalty", "ophlēsis"),
        "G20857": ("ὀχλαγωγέω", "G:V", "to attract a crowd", "ochlagōgeō"),
        "G20858": ("ὀχυρωμάτιον", "", "small fort", "ochurōmation"),
        "G20859": ("ὀχύρωσις", "", "fortification", "ochurōsis"),
        "G20860": ("ὀψοποίημα", "", "food", "opsopoiēma"),
        "G20861": ("ὄψος", "", "fish", "opsos"),
        "G20862": ("παγγέωργος", "", "master-gardener", "paggeōrgos"),
        "G20863": ("παγκρατής", "", "victorious", "pagkratēs"),
        "G20864": ("πάθνη", "", "manger", "pathnē"),
        "G20865": ("παθοκράτεια", "", "government of the passions, self-restraint", "pathokrateia"),
        "G20866": ("παιωνισμός", "", "chanting of the paean", "paiōnismos"),
        "G20867": ("παλαστή", "", "*Geom.", "palastē"),
        "G20868": ("παμβασιλεύς", "", "absolute monarch", "pambasileus"),
        "G20869": ("παμμελής", "", "with all the limbs, entire", "pammelēs"),
        "G20870": ("παμμίαρος", "", "all-abominable", "pammiaros"),
        "G20871": ("παμμιγής", "", "mixed of all sorts, all-blended", "pammigēs"),
        "G20872": ("πανάγιος", "", "all-holy", "panagios"),
        "G20873": ("πανεθνεί", "", "with the whole nation", "panethnei"),
        "G20874": ("πανεπόπτης", "", "all-observing", "panepoptēs"),
        "G20875": ("πανέχινος", "", "shell/shield", "panechinos"),
        "G20876": ("πανηγυρισμός", "", "display, ostentation", "panēgurismos"),
        "G20877": ("πανόδυρτος", "", "most lamentable", "panodurtos"),
        "G20878": ("πανούργευμα", "", "wonderful feats", "panourgeuma"),
        "G20879": ("πανούργημα", "", "sophistry", "panourgēma"),
        "G20880": ("παντεπίσκοπος", "", "all-surveying", "pantepiskopos"),
        "G20881": ("παντευχία", "", "in full armour", "panteuchia"),
        "G20882": ("παντοδύναμος", "", "all-powerful", "pantodunamos"),
        "G20883": ("παντοκρατέω", "", "be almighty", "pantokrateō"),
        "G20884": ("παντοτρόφος", "", "all-nurturing", "pantotrophos"),
        "G20885": ("παντοφαγία", "", "indiscriminate eating", "pantophagia"),
        "G20886": ("παραβαπτιστής", "", "impostor", "parabaptistēs"),
        "G20887": ("παραβασιλεύω", "G:V", "to commit treason", "parabasileuō"),
        "G20888": ("παραγώνιος", "", "adjacent to an angle", "paragōnios"),
        "G20889": ("παραδειγματισμός", "", "military reprimand", "paradeigmatismos"),
        "G20890": ("παραδειπνίζω", "G:V", "to cause to dine", "paradeipnizō"),
        "G20891": ("παράθεμα", "", "appendage", "parathema"),
        "G20892": ("παρακαθεύδω", "G:V", "to keep watch by", "parakatheudō"),
        "G20893": ("παρακάλυμμα", "", "excuse, pretext", "parakalumma"),
        "G20894": ("παρακλείω", "G:V", "to shut up", "parakleiō"),
        "G20895": ("παρακλίνω", "G:V", "to avoid", "paraklinō"),
        "G20896": ("παραναγιγνώσκω", "G:V", "to read publicly", "paranagignōskō"),
        "G20897": ("παρανακλίνω", "G:V", "to lay beside", "paranaklinō"),
        "G20898": ("παραπλαγιάζω", "G:V", "to present oneself obliquely", "paraplagiazō"),
        "G20899": ("παραριθμέω", "G:V", "to cheat", "pararithmeō"),
        "G20900": ("παραρρέγχω", "", "near", "pararregchō"),
        "G20901": ("παραστήκω", "", "to preside", "parastēkō"),
        "G20902": ("παρασυλλογιστικός", "", "fallacious", "parasullogistikos"),
        "G20903": ("παρασφαλίζω", "G:V", "to fortify next in order", "parasphalizō"),
        "G20904": ("παραχωρέω", "G:V", "to flow", "parachōreō"),
        "G20905": ("παρδάλειος", "", "like a pard", "pardaleios"),
        "G20906": ("παρεγκλίνω", "G:V", "to incline", "paregklinō"),
        "G20907": ("παρεισπορεύομαι", "", "enter", "pareisporeuomai"),
        "G20908": ("παρεκλείπω", "G:V", "to fail", "parekleipō"),
        "G20909": ("παρελέγχω", "G:V", "to rebuke", "parelegchō"),
        "G20910": ("παρέλκω", "", "be derived", "parelkō"),
        "G20911": ("παρεμπίπτω", "", "be included in one form", "parempiptō"),
        "G20912": ("παρέξω", "", "outside", "parexō"),
        "G20913": ("παρεπιδείκνυμι", "", "exhibit out of season, make a display", "parepideiknumi"),
        "G20914": ("παρθενεία", "", "maiden song", "partheneia"),
        "G20915": ("παροδοποιέω", "G:V", "to make a side-road", "parodopoieō"),
        "G20916": ("πάροδος", "", "passing by", "parodos"),
        "G20917": ("παροιμιάζω", "G:V", "to make proverbial", "paroimiazō"),
        "G20918": ("παρόρασις", "", "overlooking", "parorasis"),
        "G20919": ("παρπεπιθών", "", "to beguile", "parpepithōn"),
        "G20920": ("παρρησιαστικός", "", "outspoken", "parrēsiastikos"),
        "G20921": ("παρωθέω", "G:V", "to deny", "parōtheō"),
        "G20922": ("πεδήτης", "", "one fettered, prisoner", "pedētēs"),
        "G20923": ("πέλμα", "", "stalk", "pelma"),
        "G20924": ("πελματόομαι", "", "become worn in the sole", "pelmatoomai"),
        "G20925": ("πενέω", "", "be poor", "peneō"),
        "G20926": ("πενταπέτηλον", "", "wild rose", "pentapetēlon"),
        "G20927": ("πεντάπλοκος", "", "five times twisted", "pentaplokos"),
        "G20928": ("πεντάπολις", "", "league of five cities", "pentapolis"),
        "G20930": ("πεντηκοντατέσσαρες", "", "fifty-four", "pentēkontatessares"),
        "G20931": ("περάτηθεν", "", "from the far side", "peratēthen"),
        "G20932": ("περιαγκωνίζω", "G:V", "to tie the hands behind the back", "periagkōnizō"),
        "G20933": ("περιαντλέω", "", "be completely drowned", "periantleō"),
        "G20934": ("περιάργυρος", "", "set in silver", "periarguros"),
        "G20935": ("περιβιόω", "G:V", "to keep alive", "peribioō"),
        "G20936": ("περιβόητος", "", "utter frantic cries", "periboētos"),
        "G20937": ("περιβώμιος", "", "space round a", "peribōmios"),
        "G20938": ("περίγλυφον", "", "carved figure", "perigluphon"),
        "G20939": ("περιδιπλόω", "G:V", "to wrap round", "peridiploō"),
        "G20940": ("περικαίω", "G:V", "to inflame", "perikaiō"),
        "G20941": ("περικατάληπτος", "", "detected", "perikatalēptos"),
        "G20942": ("περικλάω", "G:V", "to break", "periklaō"),
        "G20943": ("περικομπέω", "G:V", "to declare loudly", "perikompeō"),
        "G20944": ("περικρατέω", "", "be mastered, cured", "perikrateō"),
        "G20945": ("περιλακίζω", "G:V", "to rend all round", "perilakizō"),
        "G20946": ("περίλημμα", "", "embrace", "perilēmma"),
        "G20947": ("περιξύω", "G:V", "to scrape round", "perixuō"),
        "G20948": ("περιπαθής", "", "pathetic, heartrending", "peripathēs"),
        "G20949": ("περιπιλέω", "G:V", "to cover thick all round", "peripileō"),
        "G20950": ("περιπόθητος", "", "much-beloved", "peripothētos"),
        "G20951": ("περιρραντίζω", "G:V", "to sprinkle", "perirrantizō"),
        "G20952": ("περιρρέω", "G:V", "to surround", "perirreō"),
        "G20953": ("περισθενής", "", "exceeding strong", "peristhenēs"),
        "G20954": ("περισκέλια", "", "drawers", "periskelia"),
        "G20955": ("περισκυθίζω", "G:V", "to scalp in Scythian fashion", "periskuthizō"),
        "G20956": ("περιστερίδιον", "", "small pidgeon", "peristeridion"),
        "G20957": ("περιστέφω", "G:V", "to enwreathe", "peristephō"),
        "G20958": ("περιστροφή", "", "whorls", "peristrophē"),
        "G20959": ("περιτήκω", "G:V", "to cover with a coat", "peritēkō"),
        "G20960": ("περίφρων", "", "despising", "periphrōn"),
        "G20961": ("περιφυτεύω", "G:V", "to plant round about", "periphuteuō"),
        "G20962": ("περιχαλάω", "G:V", "to relax all round", "perichalaō"),
        "G20963": ("περιψύχω", "G:V", "to refresh", "peripsuchō"),
        "G20964": ("πέρνημι", "", "renaid", "pernēmi"),
        "G20965": ("πεταλόω", "", "plates", "petaloō"),
        "G20966": ("πέτασος", "", "baldacchino", "petasos"),
        "G20967": ("πηδαλιουχέω", "G:V", "to govern", "pēdalioucheō"),
        "G20968": ("πηλουργός", "", "working in clay", "pēlourgos"),
        "G20969": ("πήσσω", "", "to pitch (a tent)", "pēssō"),
        "G20970": ("πιμπλάω", "", "multitude", "pimplaō"),
        "G20971": ("πιστοποιέω", "G:V", "to confirm", "pistopoieō"),
        "G20972": ("πλεοναστέον", "", "one must use in abundance", "pleonasteon"),
        "G20973": ("πλεύμων", "", "lungs", "pleumōn"),
        "G20974": ("πληθύς", "", "plebs", "plēthus"),
        "G20975": ("πληθύω", "G:V", "to multiply", "plēthuō"),
        "G20976": ("πλινθεῖον", "", "-casing", "plintheion"),
        "G20977": ("πολεμοτροφέω", "G:V", "to maintain war", "polemotropheō"),
        "G20978": ("πολεμόω", "", "be made an enemy of", "polemoō"),
        "G20979": ("πολιτευτής", "", "statesman", "politeutēs"),
        "G20980": ("πολυετής", "", "keeping for many years", "poluetēs"),
        "G20981": ("πολυημερεύω", "G:V", "to attain length of days", "poluēmereuō"),
        "G20982": ("πολυκέφαλος", "", "the hissing of the serpents round the Gorgon's head", "polukephalos"),
        "G20983": ("πολύορκος", "", "swearing much", "poluorkos"),
        "G20984": ("πολύπαις", "", "with many children", "polupais"),
        "G20985": ("πολυπειρία", "", "great experience", "polupeiria"),
        "G20986": ("πολύπειρος", "", "much-experienced", "polupeiros"),
        "G20987": ("πολυπλάσιος", "", "many times", "poluplasios"),
        "G20988": ("πολυπλήθεια", "", "number", "poluplētheia"),
        "G20989": ("πολύφροντις", "", "full of thought", "poluphrontis"),
        "G20990": ("πονήρευμα", "", "condition", "ponēreuma"),
        "G20991": ("ποντόβροχος", "", "drowned in the sea", "pontobrochos"),
        "G20992": ("πορευτός", "", "going, travelling", "poreutos"),
        "G20993": ("πόρπη", "", "hair-clasp", "porpē"),
        "G20994": ("πραγματικός", "", "troublesome, formidable", "pragmatikos"),
        "G20995": ("προαλής", "", "rather rashly", "proalēs"),
        "G20996": ("προαναμέλπω", "G:V", "to sing before", "proanamelpō"),
        "G20997": ("προαναπληρόω", "G:V", "to fulfil before", "proanaplēroō"),
        "G20998": ("προβασανίζω", "G:V", "to torture before", "probasanizō"),
        "G20999": ("προβασκάνιον", "", "muttonium", "probaskanion"),
        "G21000": ("προβλής", "", "forelands, headlands", "problēs"),
        "G21001": ("πρόβλητος", "", "spread, beaten out", "problētos"),
        "G21002": ("προγονικός", "", "ancestral", "progonikos"),
        "G21003": ("προενέχομαι", "", "be involved in before", "proenechomai"),
        "G21004": ("προεξαποστέλλω", "G:V", "to send out before", "proexapostellō"),
        "G21005": ("προήγορος", "", "defender, advocate", "proēgoros"),
        "G21006": ("προήκω", "G:V", "to extend", "proēkō"),
        "G21007": ("προθερίζω", "G:V", "to reap first", "protherizō"),
        "G21008": ("πρόκαιρος", "", "prematurely", "prokairos"),
        "G21009": ("προκακόομαι", "", "be afflicted before", "prokakoomai"),
        "G21010": ("προκατασκευάζω", "G:V", "to prepare before", "prokataskeuazō"),
        "G21011": ("προκατασκιρρόομαι", "", "inveterate", "prokataskirroomai"),
        "G21012": ("προκαταχωρίζω", "G:V", "to set down first", "prokatachōrizō"),
        "G21013": ("πρόκρημνος", "", "overhanging, beetling", "prokrēmnos"),
        "G21014": ("πρόμαχος", "", "cake", "promachos"),
        "G21015": ("προμηνύω", "G:V", "to predict", "promēnuō"),
        "G21016": ("προνουμηνία", "", "the day before a new moon", "pronoumēnia"),
        "G21017": ("προοδηγός", "", "one who goes before to show the way", "proodēgos"),
        "G21018": ("προπορευτής", "", "vanguard", "proporeutēs"),
        "G21019": ("προπράσσω", "G:V", "to preempt", "proprassō"),
        "G21020": ("προπτύω", "G:V", "to spit out", "proptuō"),
        "G21021": ("πρόπτωσις", "", "degeneration", "proptōsis"),
        "G21022": ("προσαναλέγω", "G:V", "to recount", "prosanalegō"),
        "G21023": ("προσαναπαύω", "G:V", "to find rest in", "prosanapauō"),
        "G21024": ("προσανατρέπω", "G:V", "to overturn besides", "prosanatrepō"),
        "G21025": ("προσανοικοδομέομαι", "", "be added for edification", "prosanoikodomeomai"),
        "G21026": ("προσαξιόω", "G:V", "to demand", "prosaxioō"),
        "G21027": ("προσαποστέλλω", "G:V", "to dispatch besides", "prosapostellō"),
        "G21028": ("προσαπωθέω", "G:V", "to reject", "prosapōtheō"),
        "G21029": ("προσαρτίως", "", "lately", "prosartiōs"),
        "G21030": ("προσγελάω", "G:V", "to smile upon", "prosgelaō"),
        "G21031": ("προσγράφω", "G:V", "to write", "prosgraphō"),
        "G21033": ("προσεμβριμάομαι", "", "threaten besides", "prosembrimaomai"),
        "G21034": ("προσεμπυρίζω", "", "to burn through", "prosempurizō"),
        "G21035": ("προσενέχομαι", "", "be holden by, in the grip of", "prosenechomai"),
        "G21036": ("προσεξηγέομαι", "", "relate besides", "prosexēgeomai"),
        "G21037": ("προσεπικατατείνω", "G:V", "to strain", "prosepikatateinō"),
        "G21038": ("προσεπιτιμάω", "G:V", "to reproach", "prosepitimaō"),
        "G21039": ("προσημειόομαι", "", "be noted before", "prosēmeioomai"),
        "G21040": ("προσκαταβαίνω", "G:V", "to go down to meet", "proskatabainō"),
        "G21041": ("προσκατατείνω", "G:V", "to extend", "proskatateinō"),
        "G21042": ("προσκήνιον", "", "painted scenery at the back of the stage", "proskēnion"),
        "G21043": ("πρόσκλησις", "", "invocation", "prosklēsis"),
        "G21044": ("προσοδύρομαι", "", "lament beside", "prosoduromai"),
        "G21045": ("προσονομάζω", "", "be surnamed", "prosonomazō"),
        "G21046": ("προσοχή", "", "putting to land", "prosochē"),
        "G21047": ("προσοχυρόω", "G:V", "to strengthen", "prosochuroō"),
        "G21048": ("προσπάσσω", "G:V", "to sprinkle upon", "prospassō"),
        "G21049": ("προσπυρόω", "G:V", "to kindle incense", "prospuroō"),
        "G21050": ("προσταγή", "", "an order", "prostagē"),
        "G21051": ("προσταράσσω", "G:V", "to trouble further", "prostarassō"),
        "G21052": ("πρόστιμον", "", "licence-fee", "prostimon"),
        "G21053": ("προσυμνέω", "G:V", "to celebrate in song as well", "prosumneō"),
        "G21054": ("προσυστέλλομαι", "", "be drawn up beforehand", "prosustellomai"),
        "G21055": ("προσυφόω", "G:V", "to raise still higher", "prosuphoō"),
        "G21056": ("προσφύω", "G:V", "to cling fast", "prosphuō"),
        "G21057": ("πρόσχεσις", "", "obtutus)", "proschesis"),
        "G21058": ("προσχράομαι", "", "abuse", "proschraomai"),
        "G21059": ("προσωθέω", "G:V", "to push", "prosōtheō"),
        "G21060": ("προσωπεῖον", "", "person", "prosōpeion"),
        "G21061": ("προϋποτάσσω", "G:V", "to place before", "proupotassō"),
        "G21062": ("προϋφίσταμαι", "", "antecedents", "prouphistamai"),
        "G21063": ("προφανής", "", "conspicuous, extraordinary", "prophanēs"),
        "G21064": ("προχαλάω", "", "be loosed beforehand", "prochalaō"),
        "G21065": ("πρόχωμα", "", "dam", "prochōma"),
        "G21066": ("πρύτανις", "", "president", "prutanis"),
        "G21067": ("πρωϊζός", "", "yesterday", "prōizos"),
        "G21068": ("πρωτοκλήσια", "", "coronation festival", "prōtoklēsia"),
        "G21069": ("πρωτοκουρία", "", "first shearing", "prōtokouria"),
        "G21070": ("πρωτόπλαστος", "", "first-formed", "prōtoplastos"),
        "G21071": ("πτίλος", "", "lippus", "ptilos"),
        "G21072": ("πτοία", "", "excitement", "ptoia"),
        "G21073": ("πυροβόλος", "", "arrows tipped with fire", "purobolos"),
        "G21074": ("πυρροπτέρυξ", "", "red-winged", "purropterux"),
        "G21075": ("πυρώτερος", "", "redder", "purōteros"),
        "G21076": ("ῥαβδίον", "", "pointer", "rhabdion"),
        "G21077": ("ῥανίς", "", "drop, spot", "rhanis"),
        "G21078": ("ῥεμβάς", "", "roving", "rhembas"),
        "G21079": ("ῥοδοφόρος", "", "bearing roses", "rhodophoros"),
        "G21080": ("ῥῦσις", "", "deliverance", "rhusis"),
        "G21081": ("ῥώξ", "", "entrance", "rhōx"),
        "G21082": ("ῥωποπώλης", "", "vendor", "rhōpopōlēs"),
        "G21084": ("σαγή", "", "padding", "sagē"),
        "G21085": ("σανίδωμα", "", "table", "sanidōma"),
        "G21086": ("σατάνειος", "", "medlar", "sataneios"),
        "G21087": ("σβεστικός", "", "quenching", "sbestikos"),
        "G21088": ("σέβημα", "", "act of worship", "sebēma"),
        "G21089": ("Σειρήνειος", "", "bewitching", "Seirēneios"),
        "G21090": ("σεῖσμα", "", "extortions", "seisma"),
        "G21091": ("σηκός", "", "weight", "sēkos"),
        "G21092": ("σήπη", "", "decay", "sēpē"),
        "G21093": ("σθένω", "G:V", "be able", "sthenō"),
        "G21094": ("σιγηρός", "", "silence", "sigēros"),
        "G21095": ("σιδηροδέσμος", "", "with bonds of iron", "sidērodesmos"),
        "G21096": ("σιτοδοτεία", "", "provision", "sitodoteia"),
        "G21097": ("σκέπασις", "", "protection", "skepasis"),
        "G21098": ("σκεύασις", "", "food preparation", "skeuasis"),
        "G21099": ("σκεύασμα", "", "furniture", "skeuasma"),
        "G21100": ("σκευαστής", "", "preparer", "skeuastēs"),
        "G21101": ("σκήνωσις", "", "dwelling in one", "skēnōsis"),
        "G21102": ("σκιαγράφος", "", "perspective-painter, scene-painter", "skiagraphos"),
        "G21103": ("σκληρόπους", "", "hard-footed", "sklēropous"),
        "G21104": ("σκοπεύω", "G:V", "to watch out", "skopeuō"),
        "G21105": ("σκορακισμός", "", "contumely", "skorakismos"),
        "G21106": ("σκορπίδιον", "", "small scorpion", "skorpidion"),
        "G21107": ("σκοτεία", "", "darkness", "skoteia"),
        "G21108": ("σκυβαλίζω", "G:V", "to reject contemptuously", "skubalizō"),
        "G21109": ("σκυλεία", "", "despoiling, plundering", "skuleia"),
        "G21110": ("σκυλμός", "", "expenditure of effort, trouble", "skulmos"),
        "G21111": ("σκῶλος", "", "evil, ruin", "skōlos"),
        "G21112": ("σπάλαξ", "", "meadow-saffron, Colchicum parnassicum", "spalax"),
        "G21113": ("σπάνις", "", "craving", "spanis"),
        "G21114": ("σπασμός", "", "drawing", "spasmos"),
        "G21115": ("σπατάλη", "", "bracelet", "spatalē"),
        "G21116": ("σπειρηδόν", "", "in maniples, manipulatim", "speirēdon"),
        "G21117": ("σπλαγχνισμός", "", "a feeding on the inwards of a sacrifice", "splagchnismos"),
        "G21118": ("σπλαγχνοφάγος", "", "eating the", "splagchnophagos"),
        "G21119": ("σποδόομαι", "", "with ashes", "spodoomai"),
        "G21120": ("σταθμικός", "", "by weight", "stathmikos"),
        "G21121": ("στεατοκήλη", "", "sebaceous formation in the scrotum", "steatokēlē"),
        "G21122": ("στέγος", "", "brothel", "stegos"),
        "G21123": ("στειρόω", "G:V", "to make barren", "steiroō"),
        "G21124": ("στέργω", "G:V", "to desire", "stergō"),
        "G21125": ("στερεοειδής", "", "of solid nature", "stereoeidēs"),
        "G21126": ("στερέωσις", "", "obstinacy", "stereōsis"),
        "G21127": ("στεφανηφορέω", "", "be a", "stephanēphoreō"),
        "G21128": ("στηθιστήρ", "", "poitrel", "stēthistēr"),
        "G21129": ("στήλωσις", "", "recording on a tablet", "stēlōsis"),
        "G21130": ("στιμμίζω", "G:V", "to apply any eye-salve", "stimmizō"),
        "G21131": ("στιππύϊνος", "", "made of tow", "stippuinos"),
        "G21132": ("στιππύον", "", "hemp", "stippuon"),
        "G21133": ("στιχίζω", "G:V", "to arrange", "stichizō"),
        "G21134": ("στοιχιαῖος", "", "course", "stoichiaios"),
        "G21135": ("στομίς", "", "a bit (of a horse)", "stomis"),
        "G21136": ("στόμωμα", "", "a scale which flies from hammered iron", "stomōma"),
        "G21137": ("στόρνυμι", "", "strew.", "stornumi"),
        "G21138": ("στραγγαλάω", "G:V", "to strangle", "straggalaō"),
        "G21139": ("στραγγαλίς", "", "induration", "straggalis"),
        "G21140": ("στραγγεύομαι", "", "agino", "straggeuomai"),
        "G21141": ("στρατηγέω", "G:V", "be general", "stratēgeō"),
        "G21142": ("στρατηγία", "", "province governed by a", "stratēgia"),
        "G21143": ("στρέβλη", "", "torture", "streblē"),
        "G21144": ("στρεβλωτήριος", "", "rack", "streblōtērios"),
        "G21145": ("στρυφαλίς", "", "cheese", "struphalis"),
        "G21146": ("στρύχνον", "", "winter cherry", "struchnon"),
        "G21147": ("συγγελάω", "G:V", "to laugh with", "suggelaō"),
        "G21148": ("συγγενεύς", "", "relatives", "suggeneus"),
        "G21149": ("συγγυμνασία", "", "experience, training", "suggumnasia"),
        "G21150": ("συγκαταγηράσκω", "G:V", "to age together", "sugkatagēraskō"),
        "G21151": ("συγκατακλείω", "G:V", "to enclose together", "sugkatakleiō"),
        "G21152": ("συγκεντέω", "G:V", "to pierce", "sugkenteō"),
        "G21153": ("συγκεραστός", "", "a mixed drink, aqua calda", "sugkerastos"),
        "G21154": ("συγκεραυνόω", "G:V", "to thunder-strike", "sugkeraunoō"),
        "G21155": ("συγκληρονομέω", "", "be joint-heir", "sugklēronomeō"),
        "G21156": ("συγκρουσμός", "", "collision", "sugkrousmos"),
        "G21157": ("συγκτίζω", "", "be created along with", "sugktizō"),
        "G21158": ("συγχρονίζω", "", "be contemporaneous", "sugchronizō"),
        "G21159": ("συζυγής", "", "conjugate", "suzugēs"),
        "G21160": ("σύζωμα", "", "girdle", "suzōma"),
        "G21161": ("σύκειον", "", "useless", "sukeion"),
        "G21162": ("συκών", "", "fig-yard", "sukōn"),
        "G21163": ("συλλοχάω", "G:V", "to ambush", "sullochaō"),
        "G21164": ("συλλοχισμός", "", "musterroll, census", "sullochismos"),
        "G21165": ("συλλύω", "G:V", "to rest under the same roof.", "sulluō"),
        "G21166": ("συμβίωσις", "", "club, society", "sumbiōsis"),
        "G21167": ("συμβιωτής", "", "confidants", "sumbiōtēs"),
        "G21168": ("συμβολοφύλαξ", "", "keeper of receipts", "sumbolophulax"),
        "G21169": ("συμβουλευτής", "", "-senator", "sumbouleutēs"),
        "G21170": ("συμβραβεύω", "", "be assessor with", "sumbrabeuō"),
        "G21171": ("συμμαίνομαι", "", "be mad together, join in madness", "summainomai"),
        "G21172": ("συμμένω", "G:V", "to hold together", "summenō"),
        "G21173": ("συμμιαίνω", "G:V", "to defile together with", "summiainō"),
        "G21174": ("συμμισοπονηρέω", "G:V", "to feel common hatred of what is bad", "summisoponēreō"),
        "G21175": ("συμμολύνω", "G:V", "to disgrace together", "summolunō"),
        "G21176": ("σύμπορος", "", "accompanying", "sumporos"),
        "G21177": ("συμποσία", "", "drinking together", "sumposia"),
        "G21178": ("συμπροπορεύομαι", "", "travel forward with", "sumproporeuomai"),
        "G21179": ("συμφλογίζω", "G:V", "to burn together", "sumphlogizō"),
        "G21180": ("συμφρονέω", "G:V", "to bring together in thought", "sumphroneō"),
        "G21181": ("σύναγμα", "", "gravel in the kidneys", "sunagma"),
        "G21182": ("συναναστρέφω", "G:V", "to wrestle with", "sunanastrephō"),
        "G21183": ("συναναστροφή", "", "living with, intercourse", "sunanastrophē"),
        "G21184": ("συναποκλείω", "G:V", "to shut up altogether", "sunapokleiō"),
        "G21185": ("συναποκρύπτω", "G:V", "to conceal together", "sunapokruptō"),
        "G21186": ("συνδάκνω", "G:V", "to bite together", "sundaknō"),
        "G21187": ("σύνδειπνος", "", "members of a dining-club", "sundeipnos"),
        "G21188": ("συνεῖπον", "", "arranged", "suneipon"),
        "G21189": ("συνεκκεντέω", "G:V", "to stab at once", "sunekkenteō"),
        "G21190": ("συνεκτρίβω", "G:V", "to destroy utterly together", "sunektribō"),
        "G21191": ("συνέλευσις", "", "stronghold", "suneleusis"),
        "G21192": ("συνερίζω", "G:V", "to contend together", "sunerizō"),
        "G21193": ("συνευφράζομαι", "", "counsel well with", "suneuphrazomai"),
        "G21194": ("συνίσσεις", "", "recommendation", "sunisseis"),
        "G21195": ("συνοδυνάομαι", "", "suffer pain with", "sunodunaomai"),
        "G21196": ("συνοράω", "G:V", "to decide", "sunoraō"),
        "G21197": ("συνουσιασμός", "", "licentiousness", "sunousiasmos"),
        "G21198": ("συρίσδω", "", "to pipe", "surisdō"),
        "G21199": ("συρρέμβομαι", "", "roam together", "surrembomai"),
        "G21200": ("συσκήνιος", "", "tent-mate", "suskēnios"),
        "G21201": ("συσσύρω", "G:V", "to sweep along with one", "sussurō"),
        "G21202": ("σύστημα", "", "machine, apparatus", "sustēma"),
        "G21203": ("συστροφία", "", "familiarity", "sustrophia"),
        "G21204": ("σφηκίας", "", "beam", "sphēkias"),
        "G21205": ("σφήν", "", "an instrument of torture", "sphēn"),
        "G21206": ("σφιγγία", "", "greed", "sphiggia"),
        "G21207": ("σφυρωτήρ", "", "leather thong, shoe-latchet", "sphurōtēr"),
        "G21208": ("σχεδιάζω", "G:V", "to act with insufficient care", "schediazō"),
        "G21209": ("σχῖνος", "", "mastich, Pistacia Lentiscus", "schinos"),
        "G21210": ("σωματοφύλαξ", "", "protecting the body", "sōmatophulax"),
        "G21211": ("σωρηδόν", "", "in arithmetical progression", "sōrēdon"),
        "G21212": ("σῶς", "", "safe, sure, certain", "sōs"),
        "G21213": ("ταμιεῖον", "", "chamber, closet", "tamieion"),
        "G21214": ("ταπεινότης", "", "meanness", "tapeinotēs"),
        "G21215": ("Ταρταρόπαις", "", "child of Tartarus", "Tartaropais"),
        "G21216": ("Ταρτησσός", "", "Tarshish", "Tartēssos"),
        "G21217": ("ταχύνοια", "", "sagacity", "tachunoia"),
        "G21218": ("ταχύρρωστος", "", "swift-rushing, swift-flying", "tachurrōstos"),
        "G21219": ("ταώς", "", "tukkîyîm", "taōs"),
        "G21220": ("τεκνοφόνος", "", "child-murdering", "teknophonos"),
        "G21221": ("τέκταινα", "", "carpenter", "tektaina"),
        "G21222": ("τελεσφόρος", "", "sorceress", "telesphoros"),
        "G21223": ("τελίσκω", "G:V", "to dedicate", "teliskō"),
        "G21224": ("τελωνέω", "G:V", "to collect taxes", "telōneō"),
        "G21225": ("τερατοποιός", "", "wonders", "teratopoios"),
        "G21226": ("τετραμερής", "", "quadripartite", "tetramerēs"),
        "G21227": ("τετραπλῇ", "", "in a fourfold manner, fourfold", "tetraplē"),
        "G21228": ("τεῦχος", "", "masonry, fabric", "teuchos"),
        "G21229": ("τεχνῖτις", "", "courtesan", "technitis"),
        "G21230": ("τηγανίζω", "G:V", "to fry", "tēganizō"),
        "G21231": ("τηλαύγεια", "", "splendour", "tēlaugeia"),
        "G21232": ("τιθηνία", "", "nursing", "tithēnia"),
        "G21233": ("τιθηνόκομον", "", "nursing", "tithēnokomon"),
        "G21234": ("τιμωρητής", "", "avenger", "timōrētēs"),
        "G21235": ("τινάσσω", "G:V", "to quake", "tinassō"),
        "G21236": ("τοκοπράκτωρ", "", "one who exacts interest", "tokopraktōr"),
        "G21237": ("τολυπεύω", "G:V", "to endure", "tolupeuō"),
        "G21238": ("τομός", "", "sharpest, hottest", "tomos"),
        "G21239": ("τοξικός", "", "venenum", "toxikos"),
        "G21240": ("τοξόκλυτος", "", "famed for archery", "toxoklutos"),
        "G21241": ("τοπαρχία", "", "district governed by a", "toparchia"),
        "G21242": ("τοπομαχέω", "G:V", "to manoeuvre", "topomacheō"),
        "G21243": ("τοτέ", "", "now and then", "tote"),
        "G21245": ("τρικυμία", "", "swell", "trikumia"),
        "G21246": ("τρίοδος", "", "the 'trivium'", "triodos"),
        "G21247": ("τριπλάσιος", "", "thrice as much as", "triplasios"),
        "G21248": ("τρισαλιτήριος", "", "thrice-sinful", "trisalitērios"),
        "G21249": ("τρισσαχῆ", "", "in three places", "trissachē"),
        "G21250": ("τρισσοκέφαλος", "", "three-headed", "trissokephalos"),
        "G21251": ("τριώριον", "", "trihorium", "triōrion"),
        "G21252": ("τρομέω", "G:V", "to stand in awe of", "tromeō"),
        "G21253": ("τροφοποιός", "", "rearing, bringing up", "trophopoios"),
        "G21254": ("τροχαντήρ", "", "processes", "trochantēr"),
        "G21255": ("τροχιαῖος", "", "worked by a wheel", "trochiaios"),
        "G21256": ("τροχωτός", "", "wheel", "trochōtos"),
        "G21257": ("τρυφερεύομαι", "", "fastidious", "truphereuomai"),
        "G21258": ("τρυφεροστήμων", "", "texture", "trupherostēmōn"),
        "G21259": ("τρώγανα", "", "fruit eating", "trōgana"),
        "G21260": ("Τρωγοδύται", "", "Trogodutai", "Trōgodutai"),
        "G21261": ("τυμπανόεις", "", "drum-like", "tumpanoeis"),
        "G21262": ("τυπογράφος", "", "lines", "tupographos"),
        "G21263": ("τυπτητέος", "", "be beaten", "tuptēteos"),
        "G21264": ("τυροφόρος", "", "with cheese on it", "turophoros"),
        "G21265": ("τύφωσις", "", "crazy vanity", "tuphōsis"),
        "G21266": ("ὑβρίστρια", "", "insolent", "hubristria"),
        "G21267": ("ὗλις", "", "mud", "hulis"),
        "G21268": ("ὑλοτόμος", "", "worm", "hulotomos"),
        "G21269": ("ὑμνητός", "", "sung of, praised, lauded", "humnētos"),
        "G21270": ("ὑμνογράφος", "", "composer of hymns", "humnographos"),
        "G21271": ("ὑπαίρω", "G:V", "to self-exalt", "hupairō"),
        "G21272": ("ὑπεραγόντως", "", "exceedingly", "huperagontōs"),
        "G21273": ("ὑπεράγω", "G:V", "to exalt", "huperagō"),
        "G21274": ("ὑπεραινετός", "", "be praised exceedingly", "huperainetos"),
        "G21275": ("ὑπεράλλομαι", "", "leap to a high place", "huperallomai"),
        "G21276": ("ὑπερένδοξος", "", "exceedingly glorious", "huperendoxos"),
        "G21277": ("ὑπερηφανεύω", "G:V", "to behave arrogantly", "huperēphaneuō"),
        "G21278": ("ὑπέρμαχος", "", "champion, defender", "hupermachos"),
        "G21279": ("ὑπερτήκω", "G:V", "to melt exceedingly", "hupertēkō"),
        "G21280": ("ὑπερτιμάω", "G:V", "to honour too much", "hupertimaō"),
        "G21281": ("ὑπερύμνητος", "", "highly extolled", "huperumnētos"),
        "G21282": ("ὑπέρφοβος", "", "very terrible", "huperphobos"),
        "G21283": ("ὑπερφωνέω", "G:V", "to sing loudly", "huperphōneō"),
        "G21284": ("ὑπερῷος", "", "upper", "huperōos"),
        "G21285": ("ὑπευλαβέομαι", "", "be somewhat afraid", "hupeulabeomai"),
        "G21286": ("ὑπόθεσις", "", "thing placed under, base", "hupothesis"),
        "G21287": ("ὑποθραύω", "G:V", "to break", "hupothrauō"),
        "G21288": ("ὑπόληψις", "", "payment in advance", "hupolēpsis"),
        "G21289": ("ὑπολυπέομαι", "", "be grieved at heart", "hupolupeomai"),
        "G21290": ("ὑπομάσθιος", "", "below-breast", "hupomasthios"),
        "G21291": ("ὑπομνηματίζομαι", "", "explain, interpret", "hupomnēmatizomai"),
        "G21292": ("ὑπονόημα", "", "supposition", "huponoēma"),
        "G21293": ("ὑπονοθεύω", "G:V", "to procure by corruption", "huponotheuō"),
        "G21294": ("ὑπονομεύω", "G:V", "to intrigue", "huponomeuō"),
        "G21295": ("ὑποσχάζω", "G:V", "to trip up", "huposchazō"),
        "G21296": ("ὑποτομεύς", "", "a cutting instrument", "hupotomeus"),
        "G21297": ("ὑπόφρικος", "", "shuddering a little", "hupophrikos"),
        "G21298": ("ὑποφυλλίς", "", "a gleaning", "hupophullis"),
        "G21299": ("ὑψαυχενέω", "G:V", "to show off", "hupsaucheneō"),
        "G21300": ("φαλακρόομαι", "", "become bald", "phalakroomai"),
        "G21301": ("φαλάντωμα", "", "bald place", "phalantōma"),
        "G21302": ("φαντασιοκοπέω", "G:V", "to indulge vain fancies", "phantasiokopeō"),
        "G21303": ("φαρμακία", "", "witchcraft", "pharmakia"),
        "G21304": ("φάτνωσις", "", "ceiling in coffers", "phatnōsis"),
        "G21305": ("φειδώ", "", "thrift, sparing", "pheidō"),
        "G21306": ("φθειριάω", "G:V", "to have corrupt stock", "phtheiriaō"),
        "G21307": ("φιλανθρωπέω", "G:V", "to obtain redress", "philanthrōpeō"),
        "G21308": ("φιλαργυρέω", "G:V", "to love money", "philargureō"),
        "G21309": ("φιλαρχία", "", "efforts to gain power", "philarchia"),
        "G21310": ("φιλελεήμων", "", "compassionate", "phileleēmōn"),
        "G21311": ("φιλόγονος", "", "loving one's children", "philogonos"),
        "G21312": ("φιλογύνης", "", "womaniser", "philogunēs"),
        "G21313": ("φιλοπολίτης", "", "loving one's fellow-citizens", "philopolitēs"),
        "G21314": ("φιλοτεκνία", "", "love of one's children", "philoteknia"),
        "G21315": ("φιλόψυχος", "", "souls", "philopsuchos"),
        "G21316": ("φλεγμονή", "", "heat, passion, excess", "phlegmonē"),
        "G21317": ("φοβεροειδής", "", "terrible to behold", "phoberoeidēs"),
        "G21318": ("φοιβάω", "G:V", "to cleanse", "phoibaō"),
        "G21319": ("φονώδης", "", "deadly, malignant", "phonōdēs"),
        "G21320": ("φορβαία", "", "carry-chair", "phorbaia"),
        "G21321": ("φορολογία", "", "tribute", "phorologia"),
        "G21322": ("φρενόω", "", "be high-minded, elated", "phrenoō"),
        "G21323": ("φρικασμός", "", "shuddering, shivering", "phrikasmos"),
        "G21324": ("φρούρησις", "", "watching", "phrourēsis"),
        "G21325": ("φρυακτής", "", "arrogant", "phruaktēs"),
        "G21326": ("φυγαγωγός", "", "dragging along fugitives", "phugagōgos"),
        "G21327": ("φυλάρχης", "", "a chief", "phularchēs"),
        "G21328": ("φυτός", "", "fruitful", "phutos"),
        "G21329": ("φωταγωγέω", "G:V", "to draw down supernatural illumination", "phōtagōgeō"),
        "G21330": ("χαιρετίζω", "G:V", "to salute", "chairetizō"),
        "G21331": ("χαλκοπλάστης", "", "bronze-worker", "chalkoplastēs"),
        "G21332": ("χαροποίημα", "", "joy caused to any one", "charopoiēma"),
        "G21333": ("χαροπός", "", "grey", "charopos"),
        "G21334": ("χαρτηρία", "", "papyrus", "chartēria"),
        "G21335": ("χείμαρος", "", "plug in a ship's bottom", "cheimaros"),
        "G21336": ("χειρονομία", "", "pantomimic movement, gesticulation", "cheironomia"),
        "G21337": ("χελύνιον", "", "cranium", "chelunion"),
        "G21338": ("χελώνινος", "", "made of tortoise-shell", "chelōninos"),
        "G21339": ("χελωνίς", "", "threshold", "chelōnis"),
        "G21340": ("χεροκένως", "", "with empty hands", "cherokenōs"),
        "G21341": ("χθύπτης", "", "knock", "chthuptēs"),
        "G21342": ("χλωροβοτάνη", "", "green fodder", "chlōrobotanē"),
        "G21343": ("χοανεύω", "G:V", "to bring together", "choaneuō"),
        "G21344": ("χορεῖον", "", "dancing-place", "choreion"),
        "G21345": ("χορτώδης", "", "of grass, like grass", "chortōdēs"),
        "G21346": ("χοῦς", "", "dust", "chous"),
        "G21347": ("χρηματιστήριον", "", "oracle, sanctuary", "chrēmatistērion"),
        "G21348": ("χρησιμεύω", "G:V", "be useful", "chrēsimeuō"),
        "G21349": ("χρηστοήθεια", "", "goodness of heart", "chrēstoētheia"),
        "G21350": ("χρονίσκος", "", "a short time", "chroniskos"),
        "G21351": ("χρυσουργός", "", "goldsmith", "chrusourgos"),
        "G21352": ("χυλός", "", "flavour", "chulos"),
        "G21353": ("ψαλμῳδός", "", "psalmist", "psalmōdos"),
        "G21354": ("ψάλτης", "", "harper", "psaltēs"),
        "G21355": ("ψάμμος", "", "sabulum.", "psammos"),
        "G21356": ("ψαμμωτός", "", "stucco", "psammōtos"),
        "G21357": ("ψάω", "G:V", "to rub", "psaō"),
        "G21358": ("ψευδοθυρίς", "", "secret door", "pseudothuris"),
        "G21359": ("ψηλάφησις", "", "tickling", "psēlaphēsis"),
        "G21360": ("ψηφολογέω", "", "to make mosaic", "psēphologeō"),
        "G21361": ("ψίθυρος", "", "twittering", "psithuros"),
        "G21362": ("ψογέω", "", "to blame", "psogeō"),
        "G21363": ("ψυχαγωγία", "", "animi oblectamenta procurentur", "psuchagōgia"),
        "G21364": ("ψυχουλκέομαι", "", "be at the last gasp", "psuchoulkeomai"),
        "G21365": ("ὠδίς", "", "[hudot ]èbel", "ōdis"),
        "G21366": ("ὤμιον", "", "small shoulder", "ōmion"),
        "G21367": ("ὠμότης", "", "savagery, fierceness, cruelty", "ōmotēs"),
        "G21368": ("ὠμόφρων", "", "savage-minded", "ōmophrōn"),
        "G21369": ("ὡραΐζω", "G:V", "to make display", "hōraizō"),
    }

    @staticmethod
    def getLexicalData(lexicalEntry, singleLine=False):
        if lexicalEntry in LexicalData.data or "{0}a".format(lexicalEntry) in LexicalData.data:
            lexeme, pos, gloss, transliteration = LexicalData.data[lexicalEntry if lexicalEntry in LexicalData.data else "{0}a".format(lexicalEntry)]
            lexeme = "<heb>{0}</heb>".format(lexeme) if re.match("^[HE]", lexicalEntry) else "<grk>{0}</grk>".format(lexeme)
            data = "<transliteration>{2}</transliteration> | <e>{0}</e> | <esblu>{1}</esblu>".format(pos, gloss, transliteration)
            if singleLine:
                return "{0} {1}".format(lexeme, data)
            else:
                return "<h3>{0}</h3> <p>{1}</p>".format(lexeme, data)
        else:
            return ""

    @staticmethod
    def getLexicalDataRaw(lexicalEntry):
        if lexicalEntry in LexicalData.data or "{0}a".format(lexicalEntry) in LexicalData.data:
            lexeme, pos, gloss, transliteration = LexicalData.data[lexicalEntry if lexicalEntry in LexicalData.data else "{0}a".format(lexicalEntry)]
            return (gloss, lexeme, transliteration, pos)
        else:
            return

    @staticmethod
    def populatePronunciation():
        filename = "LexicalData.py"
        if os.path.isfile(filename):
            file = open(filename, "r", encoding="utf-8")
            data = ""
            lines = file.readlines()
            for i in range(len(lines)):
                line = lines[i]
                if re.search(r'^\s+"H', line):
                    (start, end) = LexicalData.getLastQuotes(line)
                    if start + 1 == end:
                        if re.search(r'a":', line):
                            nextLine = lines[i+1]
                        # else:
                        #     nextLine = lines[i-1]
                            (nextLineStart, nextLineEnd) = LexicalData.getLastQuotes(nextLine)
                            nextBlock = nextLine[nextLineStart+1:nextLineEnd]
                            line = LexicalData.insertText(line, start, end, nextBlock)
                data += line
            f = open("LexicalData2.py", "w", encoding="utf-8")
            f.write(data)
            f.close()
        else:
            print("Could not open {0}".format(filename))

    @staticmethod
    def getLastQuotes(line):
        count = 0
        start = 0
        end = 0
        for i in range(len(line)):
            x = line[i]
            if x == '"':
                count += 1
                if count == 9:
                    start = i
                if count == 10:
                    end = i
        return (start, end)

    #staticmethod
    def insertText(line, start, end, block):
        newLine = '{0}"{1}{2}'.format(line[:start], block, line[end:])
        return newLine

if __name__ == "__main__":

    # s = '"H27": ("אֲבִידָן", "N:N-M-P", "Abidan", ""),'
    # (start, end) = LexicalData.getLastQuotes(s)
    # print("{0}:{1}".format(start, end))
    # print(s[start+1:end])

    # s = '"H27": ("אֲבִידָן", "N:N-M-P", "Abidan", ""),'
    # (start, end) = LexicalData.getLastQuotes(s)
    # s = LexicalData.insertText(s, start, end, "hello")
    # print(s)

    LexicalData.populatePronunciation()
